
{**************************************************************************** }
{ Version 1.0 2021 }
{ Author: Oivind Muller }
{ }
{ This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
{ }
{ **************************************************************************** }
unit ILActionBar;

{ Contains the TILActionBar and TILMessageBar components }

interface
uses ImageLibrary, ILMessages, System.SysUtils, Vcl.Graphics, Windows, System.Classes, System.Types, System.UITypes,
  Vcl.Controls, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ComCtrls, System.Actions, Vcl.ActnList, VCL.Menus;

type

  TActionButton = (abNavEof, abNavNext, abNavPrev, abNavBof, abToggleViewMode, abFreeSearch, abFilter, abInsert,
  abDelete, abClear, abUndo, abSave, abSearch, abInsertImages, abUpdate, abColumns, abSort, abSQL, abExecute);
  TActionButtons = set of TActionButton;
  TControlButton = (cbClearValues, cbPopValues, cbClearSingleValue, cbShowAll, cbShowNone, cbToggleDual, cbMoveUp, cbMoveDown, cbLogOp);
  TControlButtons = set of TControlButton;

  TMessageTypes = set of TMessageType;
  TmessageOptions = set of TMessageOption;


  const
  C_DataActions : array[abNavEof .. abExecute] of string =
  ('NavEof', 'NavNext', 'NavPrior', 'NavBof', 'ToggleViewMode', 'FreeSearch', 'Filter', 'Add', 'Delete', 'Clear', 'Undo', 'Save',
  'Search', 'Images', 'Update', 'Columns', 'Sort', 'SQL', 'ExecCMD');
  { Equals the names of the bitmap ids in the resource file. Used as names on actions }

  C_ControlActions : array[cbClearValues .. cbLogOp] of string =
  ('ClearValues', 'PopValues', 'ClearSingleValue', 'ShowAll', 'ShowNone', 'ToggleDual', 'MoveUp', 'MoveDown', 'LogOp');

type

   TMessageInfo = record
    MessagetType : TMessageType;
    IsOwnText : Boolean;
    MessageKind : TMessageOption;
    DatasetIndex : integer;
    Text : string; {Display text}
    State : integer;
    Count : integer;
    SelectedCount : integer;
    SQLOperation : TSQLType; { Type of sql: select, update, etc }
    Glyph: TIcon;
    Visible : Boolean;
  end;

   TDeleteQueryEvent = procedure(Sender: TObject; var Accept : Boolean) of object;
   TSaveStateEvent = procedure(Sender: TObject; const ASaveState : integer) of object;
   TInsertStateEvent = procedure(Sender: TObject; const AnInsertState : integer) of object;
   TResultEvent = procedure(Sender : TObject; const Command : TCommandMode; const AResult : integer) of object;
   TExecSortEvent = procedure(Sender : TObject; var Ordering : TSortOrdering) of Object;
   TFilterTableEvent  = procedure(Sender : TObject; var FilterTable : TILDataTable) of Object;
   TDrawMessageEvent = procedure(Sender : TObject; MessageInfo : TMessageInfo;
    var Rect : TRect; Canvas : TCanvas) of Object;
   TShowMessageEvent  = procedure(Sender : TObject; AMessage : TMessageType;
    AState : Integer; MessageKind : TMessageOption; var Visible : Boolean) of Object;

  TILMessageBar = class;
  TILActionBar = class(TToolbar)
    private
      FFirstCmdBtn : TToolButton;
      FFirstCtrlBtn : TToolButton;
      FLastNavBtn : TToolButton;
      FActionList : TActionList;
      FActionButtons : TActionButtons;
      FControlButtons : TControlButtons;
      FCtrlCnt, FActionCnt, FSubActionCnt : integer;
      FDropMenu : TPopupMenu;
      FDropBtn : TToolButton;
      FEnableShortcuts : Boolean;
      FTable : TILTable;
      FOnDeleteQuery : TDeleteQueryEvent;
      FOnValidationResult : TResultEvent;
      FOnExecResult : TResultEvent;
      FOnExecSort : TExecSortEvent;
      FOnSetFilterTable : TFilterTableEvent;
      FOnSaveState : TSaveStateEvent;
      FOnInsertState : TInsertStateEvent;
      FOldCommandMode : Boolean;

      function GetImageMode : Boolean;
      procedure SetTable(Value : TILTable);
      function GetTable : TILTable;
      function GetVisibleButtons(Index : integer) : TToolButton;
      function GetVisibleButtonCount : integer;
      procedure CreateActionButtons;
      procedure CreateControlButtons;
      procedure SetActionButtons(const Value : TActionButtons);
      procedure SetControlButtons(const Value : TControlButtons);
      procedure SetImageMode (const Value : Boolean);
      procedure NavNextExec(Sender : TObject);
      procedure NavNextUpd(Sender : TObject);
      procedure NavPrevExec(Sender : TObject);
      procedure NavPrevUpd(Sender : TObject);
      procedure NavFirstExec(Sender : TObject);
      procedure NavFirstUpd(Sender : TObject);
      procedure NavLastExec(Sender : TObject);
      procedure NavLastUpd(Sender : TObject);
      procedure ExecExec(Sender : TObject);
      procedure ExecUpd(Sender : TObject);
      procedure InsEmptyExec(Sender : TObject);
      procedure InsEmptyUpd(Sender : TObject);
      procedure UpdExec(Sender : TObject);
      procedure UpdUpd(Sender : TObject);
      procedure SearchExec(Sender : TObject);
      procedure SearchUpd(Sender : TObject);
      procedure FreeSearchExec(Sender : TObject);
      procedure FreeSearchUpd(Sender : TObject);
      procedure FilterExec(Sender : TObject);
      procedure FilterUpd(Sender : TObject);
      procedure DelExec(Sender : TObject);
      procedure DelUpd(Sender : TObject);
      procedure UndoExec(Sender : TObject);
      procedure UndoUpd(Sender : TObject);
      procedure ClearExec(Sender : TObject);
      procedure ClearUpd(Sender : TObject);
      procedure InsImagesExec(Sender : TObject);
      procedure InsImagesUpd(Sender : TObject);
      procedure SaveExec(Sender : TObject);
      procedure SaveUpd(Sender : TObject);
      procedure ColumnsExec(Sender : TObject);
      procedure ColumnsUpd(Sender : TObject);
      procedure SortExec(Sender : TObject);
      procedure SortUpd(Sender : TObject);
      procedure SQLExec(Sender : TObject);
      procedure SQLUpd(Sender : TObject);
      procedure ToggleVMExec(Sender : TObject);
      procedure ToggleVMUpd(Sender : TObject);

      procedure ClearValsExec(Sender : TObject);
      procedure ClearValsUpd(Sender : TObject);
      procedure PopValsExec(Sender : TObject);
      procedure PopValsUpd(Sender : TObject);
      procedure ClearSingleValExec(Sender : TObject);
      procedure ClearSingleValUpd(Sender : TObject);
      procedure ShowAllExec(Sender : TObject);
      procedure ShowAllUpd(Sender : TObject);
      procedure ShowNoneExec(Sender : TObject);
      procedure ShowNoneUpd(Sender : TObject);
      procedure ToggleDualExec(Sender : TObject);
      procedure ToggleDualUpd(Sender : TObject);
      procedure MoveUpExec(Sender : TObject);
      procedure MoveUpUpd(Sender : TObject);
      procedure MoveDownExec(Sender : TObject);
      procedure MoveDownUpd(Sender : TObject);
      procedure LogOpAndExec(Sender : TObject);
      procedure LogOpAndUpd(Sender : TObject);
      procedure LogOpAndOrExclExec(Sender : TObject);
      procedure LogOpAndOrExclUpd(Sender : TObject);
      procedure LogOpAndOrMixedExec(Sender : TObject);
      procedure LogOpAndOrMixedUpd(Sender : TObject);
      procedure LogOpExec(Sender : TObject);
      procedure LogOpUpd(Sender : TObject);

      function CommandToTB(ACommand : TCommandMode) : TToolButton;
      procedure SetCtrlFocus(Usg : TdataUsage);
      procedure AdjustHints;
      procedure AssembleActionButtons;
      procedure AssembleControlButtons;
      procedure ChangeCmdState(const OldCmdMode : TCommandMode; CurrentAction : TAction);
      procedure SetFilterTable;
      function IsIdle(const NeedWorkList : Boolean = False) : Boolean;
      procedure ShowLogOpButtons(Model : TSearchModel);
      function WFFromActiveCtrl(const CheckDual : Boolean) : TILWorkField;
      function DataFieldFromActiveCtrl(var Usg : TDataUsage) : TILDataField;
      function UpdateWithImage(const ACmdMode : TCommandMode; Inserting : Boolean) : integer;
      function UseCommandModes : Boolean;
    protected
      procedure Loaded; override;
      procedure ValidationResult(const Command : TcommandMode; const AResult : integer); dynamic;
      procedure ExecResult(const Command : TCommandMode; const AResult : integer); dynamic;
      procedure DeleteRecord(var Accept : Boolean); dynamic;
      procedure ChangeCommand; dynamic;
      procedure SaveState(const ASaveState : integer); dynamic;
      procedure ExecSort(var Ordering : TSortOrdering); dynamic;
    public
      procedure GetChildren(Proc : TGetChildProc; Root : TComponent); override;
      property ActionList : TActionList read FActionList write FActionList;
      property VisibleButtons[Index : integer] : TToolButton read GetVisibleButtons;
      property VisibleButtonCount : integer read GetVisibleButtonCount;
      constructor Create(AComponent : TComponent); override;
      destructor Destroy; override;
      procedure ResolveMenu(AMenu : TMenu);
      procedure ShowButtons;
      function ButtonToAction(AButton : TActionButton) : TAction; overload;
      function ButtonToAction(AButton : TControlButton) : TAction; overload;
      function ActionToTB(AnAction : TAction) : TToolButton;

    published
      property ActionButtons : TActionButtons read FActionButtons write SetActionButtons
      default [abNavEof, abNavNext, abNavPrev, abNavBof, abToggleViewMode, abInsert,
         abDelete, abClear, abUndo, abSave, abSearch, abUpdate, abExecute];
      property ControlButtons : TControlButtons read FControlButtons write SetControlButtons
       default [cbClearValues, cbPopValues, cbClearSingleValue, cbShowAll, cbShowNone];
      property EnableShortCuts : Boolean read FEnableShortCuts write FEnableShortCuts default false;
      property ImageMode : Boolean read GetImageMode write SetImageMode default False;
      property OnDeleteQuery : TDeleteQueryEvent read FOnDeleteQuery write FOnDeleteQuery;
      property OnValidationResult : TResultEvent read FOnValidationResult write FOnValidationResult;
      property OnExecResult : TResultEvent read FOnExecResult write FOnExecResult;
      property OnInsertState : TInsertStateEvent read FOnInsertState write FOnInsertState;
      property OnSetFilterTable : TFilterTableEvent read FOnSetFilterTable write FOnSetFilterTable;
      property OnSaveState : TSaveStateEvent read FOnSaveState write FOnSaveState;
      property OnExecSort: TExecSortEvent read FOnExecSort write FOnExecSort;
      property Table : TILTable read GetTable write SetTable;
  end;

  TPanelInfo = record
    Width : integer;
    Text : string; {Display text}
    Progress : integer; { Progresser}
    Operation : TILOperation;{ Progress operation}
    Count : integer; { Progress counter}
    Visible : Boolean;
    SQLOperation : TSQLType; { Type of sql: select, update, etc }
    Glyph : VCL.Graphics.TIcon;
    { Ideally it should only be necessary with one Count. Should be optimized, but not a big case}
  end;

  TPanels = array [mtDataset..mtProgress] of TPanelInfo;

  {


  Message bar is completely automated when attatched to an action bar.
  If not attatched and work style is command modes, it is automated, except for sql messages.
  It is then necessary to call DBCommons.CanExecSQL which calls UpdateMessage.
  if "free style" mode, no SQL automation takes place. One must call CanExec.. and set SQL mode manually.}

  TILMessageBar = class(TCustomControl)
  private
    FErrorTextColor : TColor;
    FErrorTextFontStyle : TFontStyles;
    FReadyTextColor : TColor;
    FReadyTextFontStyle : TFontStyles;
    FMessageTypes : TMessageTypes; { Which messages should be shown}
    FMessageHandler : TILMessageHandler;
    FGlyphs : TImageList;
    FOnDrawMessage : TDrawMessageEvent;
    FOnShowMessage : TShowMessageEvent;
    FPanels : TPanels;
    FDatasetOptions : TMessageOptions;
    FSaveOptions : TmessageOptions;
    FUpdateOptions : TmessageOptions;
    FSearchOptions : TmessageOptions;
    FOwnText : string;
    FOwnTextAlignment : TAlignment;
    FOwnerDraw : Boolean;
    FSQLGlyph : VCL.Graphics.TIcon;
    FErrorGlyph : VCL.Graphics.TIcon;
    FReadyGlyph : VCL.Graphics.TIcon;
    FTrafficLights : Boolean;
    FProgressBar : Boolean;
    FProgressBarColor : TColor;
    FProgressActive : Boolean;
    FPrognosing : Boolean;
    FPrognoseStartTick : LongWord;
    FStartProgress : integer;
    FTable : TilTable;
    FOrigUpdateOptions : TMessageOptions;
    FSuppressMissingValue : Boolean;

    function GetMessages(Index : TMessageType) : TMessageInfo;
    function GetTable : TILTable;
    function GetSQLMode : Boolean;
    procedure SetSQLMode(Value : Boolean);
    procedure SetGlyphs(Value : TImageList);
    procedure SetUpdateOptions(Value : TMessageOptions);
    procedure SetSaveOptions(Value : TMessageOptions);
    procedure SetDatasetOptions(Value : TMessageOptions);
    procedure SetSearchOptions(Value : TMessageOptions);
    procedure SetSuppressMissingValue(Value : Boolean);
    procedure SetTrafficLights(Value : Boolean);
    procedure SetOwnText(Value : string);
    procedure SetTable(Value : TilTable);
    function MessageRect(AMessage : TMessageType) : TRect;
    procedure InitPanels;
    procedure SetMessageTypes(Value : TMessageTypes);
    procedure ShowProgress;
    procedure CreateMsgPanels;
    procedure LoadGlyphs;
  protected
    procedure Loaded; override;
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DrawMessage; dynamic;
  public
    constructor Create(AComponent : TComponent); override;
    destructor Destroy; override;
    function GlyphRect(AMessage : TmessageType) : TRect;
    function OwnTextRect : TRect;
    function TextRect(AMessage : TMessageType) : TRect;
    procedure Clear;
    procedure MsgEvent(Sender : TObject);
    procedure SetVisibles;
    procedure UpdateMessage(Operation : TILOperation; const Text : string; const Progress, Count : integer); overload;
    property Messages[Index : TMessageType] : TMessageInfo read GetMessages;
    property MessageHandler : TILMessageHandler read FMessageHandler write FMessageHandler;
  published
    property Align;
    property Anchors;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDock;
    property OnStartDrag;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property StyleElements;
    property Visible;

    property ErrorTextColor : TColor read FErrorTextColor write FErrorTextColor default clMaroon;
    property ErrorTextFontStyle : TFontStyles read FErrorTextFontStyle write FErrorTextFontStyle default [fsBold];
    property Glyphs : TImageList read FGlyphs write SetGlyphs;
    property ReadyTextColor : TColor read FReadyTextColor write FReadyTextColor default clGreen;
    property ReadyTextFontStyle : TFontStyles read FReadyTextFontStyle write FReadyTextFontStyle default [fsBold];
    property OwnText : string read FOwnText write SetOwnText;
    property OwnTextAlignment : TAlignment read FOwnTextAlignment write FOwnTextAlignment default taLeftJustify;
    property OnDrawMessage : TDrawMessageEvent read FOnDrawMessage write FOnDrawMessage;
    property OnShowMessage : TShowMessageEvent read FOnShowMessage write FOnShowMessage;
    property DatasetOptions : TMessageOptions read FDatasetOptions write SetDatasetOptions default [moConfirm, moSelCount];
    property SaveOptions : TmessageOptions read FSaveOptions write SetSaveOptions default [moError, moConfirm];
    property UpdateOptions : TmessageOptions read FUpdateOptions write SetUpdateOptions default [moError, moConfirm];
    property SearchOptions : TmessageOptions read FSearchOptions write SetSearchOptions default [moError, moConfirm];
    property OwnerDraw : Boolean read FOwnerDraw write FOwnerDraw default false;
    property SQLMode : Boolean read GetSQLMode write SetSQLMode;
    property SuppressMissingValue : Boolean read FSuppressMissingValue write SetSuppressMissingValue default false;
    property TrafficLights : Boolean read FTrafficLights write SetTrafficLights default false;
    property MessageTypes : TMessageTypes read FMessageTypes
      write SetMessageTypes default [mtDataset, mtSearch, mtEdit, mtSave];
    property ProgressBar : Boolean read FProgressBar write FProgressBar default true;
    property ProgressBarColor : TColor read FProgressBarColor write FProgressBarColor default clLime;
    property Table : TILTable read GetTable write SetTable;
  end;

implementation
uses VCL.Forms, VCL.ExtDlgs, VCL.Dialogs, VCL.ClipBrd, VCL.Styles, VCL.Themes;

function UseClipBrdImage : Boolean;
begin
  Result := false;
  if Clipboard.HasFormat(CF_BITMAP) then
  begin
    if MessageDlg('Use image on the clipboard?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
     Result := True;
    end;
  end;
end;

constructor TILActionBar.Create(AComponent: TComponent);
var
  Btn : TToolButton;
begin
  inherited;
  FActionList := TActionList.Create(Self);
  Parent := AComponent as TWinControl;
  Images := TImagelist.Create(Self);
  FActionList.Images := Images;
  ShowHint := True;
  FActionButtons := [abNavEof, abNavNext, abNavPrev, abNavBof, abToggleViewMode,
  abInsert, abDelete, abClear, abUndo, abSave, abSearch, abUpdate, abExecute];
  FControlButtons :=[cbClearValues, cbPopValues, cbClearSingleValue, cbShowAll, cbShowNone];
  CreateControlButtons;
  CreateActionButtons;
  Btn := TToolButton.Create(Self);
  Btn.Parent := Self;
  Btn.Name := 'Sep1';
  Btn.Tag := -1;
  Btn.Visible := false;
  Btn.Style := tbsSeparator;
  Btn.Width := 5;
  Btn := TToolButton.Create(Self);
  Btn.Parent := Self;
  Btn.Tag := -1;
  Btn.Name := 'Sep2';
  Btn.Visible := false;
  Btn.Style := tbsSeparator;
  Btn.Width := 5;
  Btn := TToolButton.Create(Self);
  Btn.Parent := Self;
  Btn.Tag := -1;
  Btn.Name := 'Sep3';
  Btn.Visible := false;
  Btn.Style := tbsSeparator;
  Btn.Width := 5;
  RegBars.Add(Self);
end;

destructor TILActionBar.Destroy;
var
  indx : integer;
begin
  indx := RegBars.IndexOf(Self);
  if Indx <> -1 then
    RegBars.Delete(Indx);
  inherited;

end;

procedure TILActionBar.ResolveMenu;
var
 i : integer;
 ImgCnt : integer;

 procedure AttatchAction(AMenuItem : TMenuItem);
  var
    s, s2 : string;
    i : integer;
    P : integer;
  begin
      s := AMenuItem.Caption;
      if (s <> '') and (s[1] = '_') then
      begin
        System.Delete(s, 1,1);
        P := Pos('&', s);
        if P <> 0 then
          System.Delete(s, P,1);
        for i := 0 to FActionList.ActionCount-1 do
        begin
          s2 := FActionList[i].Name;
          if SameText(S, S2) then
          begin
            AMenuItem.Action := FActionList[i];
          end;
        end;
      end
      else
      begin
        if AMenuItem.ImageIndex <> -1 then
          AMenuItem.ImageIndex := AMenuItem.ImageIndex  + ImgCnt;
      end;
  end;

  procedure DoResolveMenu(AMenu : TMenuItem);
  var
    i : integer;
  begin
    { Attatch actions to menues: }
    for i := 0 to AMenu.Count - 1 do
    begin
      AttatchAction(AMenu[i]);
      DoResolveMenu(AMenu[i]);
    end;

  end;

begin
   ImgCnt := Images.Count;
   if Assigned(AMenu.Images) then
     Images.AddImages(AMenu.Images);
   AMenu.Images := Images;
   for i := 0 to AMenu.Items.Count-1 do
   begin
      AttatchAction(AMenu.Items[i]);
      DoResolveMenu(AMenu.Items[i]);
   end;
end;

procedure TILActionBar.ShowButtons;
var
 Sep1, Sep2, Sep3 : TToolButton;
begin
  FFirstCmdBtn := nil;
  FFirstCtrlBtn := nil;
  FLastNavBtn := nil;
  AssembleControlButtons;
  AssembleActionButtons;
  Sep1 := FindComponent('Sep1') as TToolButton;
  Sep2 := FindComponent('Sep2') as TToolButton;
  Sep3 := FindComponent('Sep3') as TToolButton;
  Sep1.Visible := false;
  Sep2.Visible := false;
  Sep3.Visible := false;
  if ButtonCount < 2 then
    Exit;
  if (FFirstCmdBtn <> nil) and (FFirstCmdBtn <> Buttons[0]) then
  begin
    Sep1.Visible := True;
    Sep1.Left := FFirstCmdBtn.Left;
  end;
  if (FFirstCtrlBtn <> nil) and (FFirstCtrlBtn <> Buttons[0]) then
  begin
    Sep2.Visible := True;
    Sep2.Left := FFirstCtrlBtn.Left;
  end;

  if (FLastNavBtn <> nil) and (FLastNavBtn <> Buttons[1])
  and (ButtonCount > FLastNavBtn.Index+1)
  and (Buttons[FLastNavBtn.Index+1].Style <> tbsSeparator) then
  begin
    Sep3.Visible := True;
    Sep3.Left := Buttons[FLastNavBtn.Index+1].Left;
  end;

end;

procedure TILActionBar.AssembleActionButtons;
var
  A : TAction;
  I : TActionButton;
  T : TToolButton;
  Cmp : TComponent;

  function HasViewers : Boolean;
  var
   i : integer;
  begin
     Result := false;
     for I := 0 to TMan.TableCount-1 do
       if Tman.Tables[i].Viewer <> nil then
       begin
         Result := True;
         Break;
       end;
  end;

begin
  FActionCnt := 0;
  FSubActionCnt := 0;
  for I := abNavEof to abExecute do
  begin
     A := ButtonToAction(I);
     Cmp := FindComponent('tb_' + C_DataActions[i]);
     T := Cmp as TToolButton;
     A.Visible := (I in FActionButtons) or (csDesigning in ComponentState);
     if UseCommandModes then
     begin
       if A.Visible and (FFirstCmdBtn = nil) and (T.Tag <> -1) then
         FFirstCmdBtn := T
       else if A.Visible and (Pos('Nav', A.Name) = 1) then
         FLastNavBtn := T;

       case I of
         abExecute:
         begin
           A.Hint := Translator.GetTranslation(T_ABExecute, TRANSL_TERMS);
             if EnableShortCuts then
            A.ShortCut := vkE or scCtrl;
         end;
         abSearch:
           begin
            A.GroupIndex := 1;
            A.AutoCheck := True;
            A.Checked := True;
            T.Down := True;
            T.Style := tbsCheck;
            A.Hint := Translator.GetTranslation(T_ABEnterSearchMode, TRANSL_TERMS);
            if EnableShortCuts then
              A.ShortCut := vkF or scCtrl;
           end;
         abInsertImages:
           begin
             A.GroupIndex := 2;
             A.AutoCheck := True;
             T.Style := tbsCheck;
             T.AllowAllUp := True;
             A.Hint := Translator.GetTranslation(T_ABIncludeImages, TRANSL_TERMS);
           end;
         abUpdate:
           begin
             A.GroupIndex := 1;
             A.AutoCheck := True;
             T.Style := tbsCheck;
             A.Hint := Translator.GetTranslation(T_ABEnterUpdateMode, TRANSL_TERMS);
             if EnableShortCuts then
               A.ShortCut := vkU or scCtrl;
           end;
         abColumns:
           begin
             A.GroupIndex := 1;
             A.AutoCheck := True;
             T.Style := tbsCheck;
             A.Hint := Translator.GetTranslation(T_ABEnterColumnsMode, TRANSL_TERMS);
             if EnableShortCuts then
               A.ShortCut := vkL or scCtrl;
           end;
         abSort:
           begin
             A.GroupIndex := 1;
             A.AutoCheck := True;
             T.Style := tbsCheck;
             A.Hint := Translator.GetTranslation(T_ABEnterSortMode, TRANSL_TERMS);
             if EnableShortCuts then
              A.ShortCut := vkO or scCtrl;
           end;
         abSQL:
           begin
             A.GroupIndex := 1;
             A.AutoCheck := True;
             T.Style := tbsCheck;
             A.Hint := Translator.GetTranslation(T_ABEnterSQLMode, TRANSL_TERMS);
             if EnableShortCuts then
               A.ShortCut := vkQ or scCtrl;
           end;
       end;

       T.AllowAllUp := false;
     end
     else
     case I of
       abSearch:
       begin
         A.Hint := Translator.GetTranslation(T_Search, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkF or scCtrl;
         A.Checked := false;
         A.AutoCheck := false;
         A.GroupIndex := 0;
       end;
       abUpdate:
       begin
         A.Hint := Translator.GetTranslation(T_Update, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkU or scCtrl;
         A.Checked := false;
         A.AutoCheck := false;
         A.GroupIndex := 0;
       end;
     end;

     case I of
       abExecute: if not (csDesigning in ComponentState) then
         if A.Visible then A.Visible := UseCommandModes;
       abFreeSearch :
       begin
         if not (csDesigning in ComponentState) and not (csLoading in ComponentState) then
           if A.Visible then A.Visible := (UseCommandModes and Table.DataHandler.FreeSearch)
           or (not UseCommandModes and not (abSearch in ActionButtons));
         A.Hint := Translator.GetTranslation(T_Search, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkF or scShift or scCtrl;
       end;
       abFilter:
       begin
         A.Hint := Translator.GetTranslation(T_ABFilter, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkF or scCtrl or scAlt;
       end;
       abUndo:
       begin
         A.Hint := Translator.GetTranslation(T_ABUndo, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkZ or scCtrl;
       end;
       abToggleViewMode: A.Hint := Translator.GetTranslation(T_ABToggleViewMode, TRANSL_TERMS);
       abNavNext: A.Hint := Translator.GetTranslation(T_ABNextRecord, TRANSL_TERMS);
       abNavPrev: A.Hint := Translator.GetTranslation(T_ABPrevRecord, TRANSL_TERMS);
       abNavEof: A.Hint := Translator.GetTranslation(T_ABLastRecord, TRANSL_TERMS);
       abNavBof: A.Hint := Translator.GetTranslation(T_ABFirstRecord, TRANSL_TERMS);
       abInsert:
       begin
         A.Hint := Translator.GetTranslation(T_ABInsert, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkI or scCtrl;
       end;
       abDelete: A.Hint := Translator.GetTranslation(T_Delete, TRANSL_TERMS);
       abSave:
       begin
         A.Hint := Translator.GetTranslation(T_ABSave, TRANSL_TERMS);
         if EnableShortCuts then
            A.ShortCut := vkS or scCtrl;
       end;
       abClear: A.Hint := Translator.GetTranslation(T_ABClear, TRANSL_TERMS);
       abInsertImages: if not UseCommandModes then
        begin
           A.GroupIndex := 0;
           A.AutoCheck := false;
           T.Style := tbsButton;
           A.Hint := Translator.GetTranslation(T_ABIncludeImages, TRANSL_TERMS);
        end;
     end;
     A.Caption := A.Hint;
     if A.Visible then
     begin
       inc(FActionCnt);
       if I in [abNavEof .. abSave] then
         inc(FSubActionCnt);
     end;
  end;
  AdjustHints;
end;

procedure TILActionBar.AssembleControlButtons;
var
  A : TAction;
  I : TControlButton;
begin
   FCtrlCnt := 0;
   for I := cbClearValues to cbLogOp do
  begin
     A := ButtonToAction(I);
      if csDesigning in ComponentState then
       A.Visible := True
     else if UseCommandModes and (I in FControlButtons) then
       A.Visible := True
     else if (I in FControlButtons) and (I In [cbLogOp, cbToggleDual, cbClearValues, cbPopValues]) then
       A.Visible := True
      else
       A.Visible := false;
     if A.Visible then
     begin
       if FFirstCtrlBtn = nil then
         FFirstCtrlBtn := ActionToTB(A);
       inc(FCtrlCnt);
     end;
     case I of
       cbClearValues: A.Hint := Translator.GetTranslation(T_CBClearDacs, TRANSL_TERMS);
       cbPopValues: A.Hint := Translator.GetTranslation(T_CBPopDacs, TRANSL_TERMS);
       cbClearSingleValue: A.Hint := Translator.GetTranslation(T_CBClearSingleDac, TRANSL_TERMS);
       cbShowAll: A.Hint := Translator.GetTranslation(T_CBShowAllControls, TRANSL_TERMS);
       cbShowNone: A.Hint := Translator.GetTranslation(T_CBHideAllControls, TRANSL_TERMS);
       cbToggleDual: A.Hint := Translator.GetTranslation(T_CBToggleDual, TRANSL_TERMS);
       cbMoveUp: A.Hint := Translator.GetTranslation(T_CBMoveUp, TRANSL_TERMS);
       cbMoveDown: A.Hint := Translator.GetTranslation(T_CBMoveDown, TRANSL_TERMS);
       cbLogOp: A.Hint := Translator.GetTranslation(T_CBLogOp, TRANSL_TERMS);
     end;
     A.Caption := A.Hint;
  end;
end;

procedure TILActionBar.Loaded;
begin
    FOldCommandMode := TMan.UseCommandModes;
  ShowButtons;
  inherited;
end;

procedure TILActionBar.GetChildren(Proc : TGetChildProc; Root : TComponent);
begin
// Prevent the streaming of the interally created tool buttons
end;

procedure TILActionBar.ValidationResult(const Command : TcommandMode; const AResult : integer);
begin
   if Command = TMan.CommandMode then
   begin
    if Assigned(FOnValidationResult) then
     FOnValidationResult(Self, Command, AResult);
   end;
end;

procedure TILActionBar.ExecResult(const Command : TCommandMode; const AResult : integer);
begin
   if Assigned(FOnExecResult) then
    FOnExecResult(Self, Command, AResult);
end;

procedure TILActionBar.DeleteRecord(var Accept : Boolean);
begin
   if not Tman.AutoSave or not Assigned(FOnDeleteQuery) then
   begin
     Accept := True;
     Exit;
   end;
   Accept := false;
   FOnDeleteQuery(Self, Accept);
end;

procedure TILActionBar.ChangeCommand;
var
  A : TAction;
begin
   if TMan.CommandMode in [cmSearch, cmColumns, cmSort, cmSQL] then
   begin
     A := ButtonToAction(abInsertImages);
     A.Checked := False;
     ActionToTB(A).Down := false;
   end;
   if Assigned(TMan.OnChangeCommandMode) then
    Tman.OnChangeCommandMode(Self);
end;

procedure TILActionBar.SaveState(const ASaveState : integer);
begin
   if Assigned(FOnSaveState) then
    FOnSaveState(Self, ASaveState);
end;

procedure TILActionBar.ExecSort(var Ordering : TSortOrdering);
begin
  Ordering := soAscending;
  if Assigned(FOnExecSort) then
    FOnExecSort(Self, Ordering);
end;

function TILActionBar.GetVisibleButtons(Index: Integer) : TToolButton;
var
  ind : integer;
  I: Integer;
begin
   Ind := 0;
   Result := nil;
   for I := 0 to ButtonCount-1 do
   begin
     if Buttons[i].Visible then
     begin
       if Ind = Index then
       begin
         Result := Buttons[i];
         Break;
       end
       else
         inc(Ind);
     end;
   end;
end;

function TILActionBar.GetVisibleButtonCount : integer;
var
  I: Integer;
begin
   Result := 0;
   for I := 0 to ButtonCount-1 do
   begin
     if Buttons[i].Visible then
     begin
       inc(result);
     end;
   end;
end;

procedure TILActionBar.CreateActionButtons;
var
  Btn : TToolButton;
  BM : VCL.Graphics.TBitmap;
  A : TAction;
  Cl : TColor;
  i : TActionButton;
  U, E : TNotiFyEvent;
  Indx : integer;
begin
  for I := abExecute downto abNavEof do
  begin
   U := nil;
   E := nil;
   if i = abNavEof then
   begin
     U :=  NavLastUpd;
     E := NavLastExec;
   end
   else if i = abNavNext then
   begin
     U :=  NavNextUpd;
     E := NavNextExec;
   end
   else if i = abNavPrev then
   begin
     U :=  NavPrevUpd;
     E := NavPrevExec;
   end
   else if i = abNavBof then
   begin
     U :=  NavFirstUpd;
     E := NavFirstExec;
   end
   else if i = abToggleViewMode then
   begin
     U :=  ToggleVMUpd;
     E := ToggleVMExec;
   end
   else if i = abExecute then
   begin
     U :=  ExecUpd;
     E := ExecExec;
   end
   else if i = abSearch then
   begin
     U :=  SearchUpd;
     E := SearchExec;
   end
   else if i = abInsertImages then
   begin
     U :=  InsImagesUpd;
     E := InsImagesExec;
   end
   else if i = abDelete then
   begin
     U :=  DelUpd;
     E := DelExec;
   end
   else if i = abUpdate then
   begin
     U :=  UpdUpd;
     E := UpdExec;
   end
   else if i = abColumns then
   begin
     U :=  ColumnsUpd;
     E := ColumnsExec;
   end
   else if i = abSort then
   begin
     U :=  SortUpd;
     E := SortExec;
   end
   else if i = abSQL then
   begin
    U :=  SQLUpd;
    E := SQLExec;
   end
   else if i = abSave then
   begin
     U :=  SaveUpd;
     E := SaveExec;
   end
   else if i = abUndo then
   begin
     U :=  UndoUpd;
     E := UndoExec;
   end
   else if i = abClear then
   begin
     U :=  ClearUpd;
     E := ClearExec;
   end
   else if i = abInsert then
   begin
     U :=  InsEmptyUpd;
     E := InsEmptyExec;
   end
   else if i = abFreeSearch then
   begin
     U :=  FreeSearchUpd;
     E := FreeSearchExec;
   end
   else if i = abFilter then
   begin
     U :=  FilterUpd;
     E := FilterExec;
   end;

   BM := TMan.LoadResource(C_DataActions[i]);
   Cl := BM.Canvas.Pixels[0, BM.Height-1];
   Indx := Images.AddMasked(BM, Cl);
   BM.Free;
   A := TAction.Create(Self);
   A.Name := C_DataActions[i];
   A.ActionList := FActionList;
   A.ImageIndex := Indx;
   A.OnExecute := E;
   A.OnUpdate := U;
   Btn := TToolButton.Create(Self);
   Btn.Tag := -1;
   Btn.Parent := Self;
   Btn.Action := A;
   Btn.Name := 'tb_' + C_DataActions[i];
   case i of
     abSearch: Btn.Tag := Ord(cmSearch);
     abUpdate: Btn.Tag := Ord(cmUpdate);
     abColumns: Btn.Tag := Ord(cmColumns);
     abSort: Btn.Tag := Ord(cmSort);
     abSQL: Btn.Tag := Ord(cmSQL);
   end;
  end;
end;

procedure TILActionBar.CreateControlButtons;
var
  Btn : TToolButton;
  BM : VCL.Graphics.TBitmap;
  A : TAction;
  Cl : TColor;
  i : TControlButton;
  j : integer;
  U, E : TNotiFyEvent;
  Indx : integer;
  M : TMenuItem;
begin
  FDropBtn := TToolButton.Create(Self);
  FDropBtn.Tag := -2;
  FDropBtn.Parent := Self;
  FDropBtn.Name := 'tb_' + C_ControlActions[cbLogOp];
  FDropBtn.Style := tbsDropDown;
  FDropMenu := TPopupMenu.Create(Self);
  FDropMenu.Images := Images;
  FDropBtn.DropdownMenu := FDropMenu;
  A := TAction.Create(Self);
  A.OnExecute := LogopExec;
  A.Name := C_ControlActions[cbLogOp];
  A.ActionList := FActionList;
  FDropBtn.Action := A;

  for J := 0 to 2 do
  begin
    if J = 0 then
    begin
     BM := TMan.LoadResource('ANDLogOp');
    end
    else if J = 1 then
    begin
      BM := TMan.LoadResource('ANDORLogOp');
    end
    else
    begin
      BM := TMan.LoadResource('ANDORMixedLogOp');
    end;

    Cl := BM.Canvas.Pixels[0, BM.Height-1];
    Indx := Images.AddMasked(BM, Cl);
    BM.Free;

    M := TMenuItem.Create(FDropMenu);
    FDropMenu.Items.Add(M);
    A := TAction.Create(Self);
    A.ActionList := FActionList;
    A.ImageIndex := Indx;
    if J = 0 then
    begin
     A.Name := 'LogOpAndOnly';
     A.Caption := Translator.GetTranslation(T_CBAndOnly, TRANSL_TERMS);
     A.OnExecute := LogOpAndExec;
     A.OnUpdate := LogOpAndUpd;
    end
    else if J = 1 then
    begin
     A.Name := 'LogOpAndOrExclusively';
     A.Caption := Translator.GetTranslation(T_CBAndOrExclusively, TRANSL_TERMS);
     A.OnExecute := LogOpAndOrExclExec;
     A.OnUpdate := LogOpAndOrExclUpd;
    end
    else
    begin
     A.Name := 'LogOpAndOrMixed';
     A.Caption := Translator.GetTranslation(T_CBAndOrMixed, TRANSL_TERMS);
     A.OnExecute := LogOpAndOrMixedExec;
     A.OnUpdate := LogOpAndOrMixedUpd;
    end;
    M.Action := A;
  end;
  FDropBtn.ImageIndex := Fdropmenu.Items[0].ImageIndex;

  U := nil;
  E := nil;
  for I := cbMoveDown downto cbClearValues do
  begin
   if i = cbClearValues then
   begin
     U :=  ClearValsUpd;
     E := ClearValsExec;
   end
   else if i = cbPopValues then
   begin
     U :=  PopValsUpd;
     E := PopValsExec;
   end
   else if i = cbClearSingleValue then
   begin
     U :=  ClearSingleValUpd;
     E := ClearSingleValExec;
   end
   else if i = cbShowAll then
   begin
     U :=  ShowAllUpd;
     E := ShowAllExec;
   end
   else if i = cbShowNone then
   begin
     U :=  ShowNoneUpd;
     E := ShowNoneExec;
   end
   else if i = cbToggleDual then
   begin
     U := ToggleDualUpd;
     E := ToggleDualExec;
   end
   else if i = cbMoveUp then
   begin
     U := MoveUpUpd;
     E := MoveUpExec;
   end
   else if i = cbMoveDown then
   begin
     U := MoveDownUpd;
     E := MoveDownExec;
   end;

   BM := TMan.LoadResource(C_ControlActions[i]);
   Cl := BM.Canvas.Pixels[0, BM.Height-1];
   Indx := Images.AddMasked(BM, Cl);
   BM.Free;
   A := TAction.Create(Self);
   A.Name := C_ControlActions[i];
   A.ActionList := FActionList;
   A.ImageIndex := Indx;
   A.OnExecute := E;
   A.OnUpdate := U;
   Btn := TToolButton.Create(Self);
   Btn.Tag := -1;
   Btn.Parent := Self;
   Btn.Action := A;
   Btn.Name := 'tb_' + C_ControlActions[i];
  end;
end;

function TILActionBar.IsIdle(const NeedWorkList : Boolean = False) : Boolean;
var
  Bar : TILActionBar;
  i : integer;
  ActiveBar : TILActionBar;
begin
  Result := (TMan <> nil) and (Tman.Connected);
  if Result then
  begin
     ActiveBar := nil;
     for I := 0 to RegBars.Count-1 do
     begin
       Bar := TILActionBar(RegBars[i]);
       if Bar.FTable = TMan.ActiveTable then
       begin
         ActiveBar := Bar;
         Break;
       end;
     end;
     if ActiveBar <> nil then
       Result := (ActiveBar.FTable = FTable)
     else if FTable <> nil then
      Result := False;
  end;
  if Result and NeedWorkList then
    Result := (TMan.WorkList <> nil);
  if Result then
    if not Tman.Connected and ButtonToAction(abInsertImages).Checked  then
     ButtonToAction(abInsertImages).Checked := false;

end;

procedure TILActionBar.ShowLogOpButtons(Model : TSearchModel);
var
  i : integer;
  Show : Boolean;
  P : TILControlPanel;
begin
   Show := Model in [smAndOrAligned, smAndOrMixed];
   for I := 0 to TMan.PanelCount-1 do
   begin
     P := Tman.FindPanel(Table, cmSearch);
     if P = TMan.Panels[i] then
       TMan.Panels[i].ShowLogOpButtons := Show;
   end;
end;

function TILActionBar.CommandToTB(ACommand : TCommandMode) : TToolButton;
var
  i : integer;
begin
  Result := nil;
  for i := 0 to ButtonCount-1 do
  begin
    if TCommandMode(Buttons[i].Tag) = ACommand then
    begin
      Result := Buttons[i];
      Break;
    end;
  end;
end;

procedure TILActionBar.SetCtrlFocus(Usg: TDataUsage);
var
  Ctrl : TWinControl;
begin
  if (Table.DataHandler <> nil) and
  (Table.DataHandler.DataFields.Count > 0) then
  begin
     if Usg = duData then
       Ctrl := Table.DataHandler.DataFields.Items[0].DataControl
     else
       Ctrl := Table.DataHandler.DataFields.Items[0].SearchControl;
     if Ctrl <> nil then
      if Ctrl.CanFocus then
        Ctrl.SetFocus;
  end;
end;

procedure TILActionBar.AdjustHints;
var
 A0, A2, A3 : TAction;
begin
   A2 := ButtonToAction(abUpdate);
   A3 := ButtonToAction(abInsert);
   A0 := ButtonToAction(abInsertImages);
   if A0.Checked then
   begin
      if not UseCommandModes then
      begin
        A3.Hint := Translator.GetTranslation(T_ABInsertEmptyImg, TRANSL_TERMS);
        A2.Hint := Translator.GetTranslation(T_ABUpdateIMG, TRANSL_TERMS);
      end
      else
      begin
        A3.Hint := Translator.GetTranslation(T_ABInsert, TRANSL_TERMS);
        A2.Hint := Translator.GetTranslation(T_ABEnterUpdateMode, TRANSL_TERMS);
      end
   end
   else
   begin
      if not UseCommandModes then
      begin
        A3.Hint := Translator.GetTranslation(T_ABInsert, TRANSL_TERMS);
        A2.Hint := Translator.GetTranslation(T_Update, TRANSL_TERMS);
      end
      else
      begin
        A3.Hint := Translator.GetTranslation(T_ABInsert, TRANSL_TERMS);
        A2.Hint := Translator.GetTranslation(T_ABEnterUpdateMode, TRANSL_TERMS);
      end;
   end;
end;

function TILActionBar.WFFromActiveCtrl(const CheckDual : Boolean) : TILWorkField;
var
  Ctrl : TControl;
  i, j : integer;
begin
   Result := nil;
   Ctrl := Screen.ActiveForm.ActiveControl;
   if Ctrl = nil then
    Exit;
   for I := 0 to Tman.PanelCount-1 do
   begin
     for j := 0 to TMan.Panels[i].Count-1 do
       if TMan.Panels[i].Items[j].Control = Ctrl then
       begin
         if CheckDual then
         begin
          if (TMan.Panels[i].Items[j].Usage = duSearch)
          or ((TMan.Panels[i].Items[j].Usage = duData)
           and (TMan.Panels[i].Items[j].Table <> nil)
           and (TMan.Panels[i].Items[j].Table.DataHandler <> nil)
           and (TMan.Panels[i].Items[j].Table.DataHandler.MultiSelect))
          then
           Result := TMan.Panels[i].Items[j];
         end
         else
           Result := TMan.Panels[i].Items[j];
         Break;
       end;
       if Result <> nil then
         Break;
   end;
end;

function TILActionBar.DataFieldFromActiveCtrl(var Usg : TDataUsage) : TILDataField;
var
 i, j : integer;
 Ctrl : TControl;
begin
   Result := nil;
   Ctrl := Screen.ActiveForm.ActiveControl;
   if Ctrl = nil then
    Exit;
   for I := 0 to TMan.TableCount-1 do
   begin
     if TMan.Tables[i].DataHandler = nil then
       Continue;
     if TMan.Tables[i].DataHandler.DataFields = nil then
       Continue;
     for J := 0 to TMan.Tables[i].DataHandler.DataFields.Count-1 do
     begin
       if (Ctrl =  TMan.Tables[i].DataHandler.DataFields.Items[J].DataControl)
       or (Ctrl =  TMan.Tables[i].DataHandler.DataFields.Items[J].SearchControl) then
       begin
         Result := TMan.Tables[i].DataHandler.DataFields.Items[J];
         if Ctrl =  TMan.Tables[i].DataHandler.DataFields.Items[J].DataControl then
           Usg := duData
         else
           Usg := duSearch;
         Break;
       end;
     end;
     if Result <> nil then
       Break;

   end;
end;

procedure TILActionBar.ChangeCmdState(const OldCmdMode : TCommandMode; CurrentAction : TAction);
var
  TB : TToolButton;
begin
  if not IsIdle then
    Exit;
  if FOldCommandMode <> TMan.UseCommandModes then
  begin
    ShowButtons;
    FOldCommandMode := TMan.UseCommandModes;
  end;
  if not CurrentAction.Checked then
    Exit;
  if OldCmdMode = TMan.CommandMode then
    Exit;
  TB := ActionToTB(CurrentAction);
  TB.Down := false;
  CurrentAction.Checked := false;
  TB := CommandToTB(TMan.CommandMode);
  TB.Down := True;
  with TB.Action as TAction do Checked := True;
  ChangeCommand;
end;

procedure TILActionBar.SetActionButtons(const Value : TActionButtons);
begin
   FActionButtons := Value;
   if not (csLoading in ComponentState) and not (csDesigning in ComponentState) then
     ShowButtons;
end;

procedure TILActionBar.SetControlButtons(const Value : TControlButtons);
begin
   FControlButtons := Value;
   if not (csLoading in ComponentState) and not (csDesigning in ComponentState) then
     ShowButtons;
end;

procedure TILActionBar.SetImageMode (const Value : Boolean);
  procedure DoSet;
  var
  A : TAction;
  Btn : TToolButton;
  begin
     A := ButtonToAction(abInsertImages);
     A.Checked := Value;
     Btn := ActionToTB(A);
     Btn.Down := Value;
  end;

begin
  DoSet;
end;

function TILActionBar.GetImageMode : Boolean;
var
  A : TAction;
begin
     A := ButtonToAction(abInsertImages);
     Result := A.Checked;
end;

procedure TILActionBar.SetTable(Value : TILTable);
begin
   if FTable = Value then
     Exit;
   FTable := Value;
   if not (csDesigning in ComponentState) then
     ShowButtons;
end;

function TILActionBar.GetTable : TILTable;
begin
    if csDesigning in ComponentState then
      Result := FTable
    else if FTable = nil then
      Result := Tman.ActiveTable
    else
      Result := FTable;
end;

function TILActionBar.ActionToTB(AnAction : TAction) : TToolButton;
begin
  Result := FindComponent('tb_' + AnAction.Name) as TToolButton;
end;

function TILActionBar.ButtonToAction(AButton : TActionButton) : TAction;
var
  i : integer;
  S : string;
begin
   Result := nil;
   S := C_DataActions[AButton];
   for I := 0 to FActionList.ActionCount-1 do
   if SameText(FActionlist[i].Name, S) then
   begin
      Result := FActionlist[i] as TAction;
      Break;
   end;
end;

function TILActionBar.ButtonToAction(AButton : TControlButton) : TAction;
var
  i : integer;
  S : string;
begin
   Result := nil;
   S := C_ControlActions[AButton];
   for I := 0 to FActionList.ActionCount-1 do
   if SameText(FActionlist[i].Name, S) then
   begin
      Result := FActionlist[i] as TAction;
      Break;
   end;
end;

function TILActionBar.UseCommandModes : Boolean;
begin
   Result := TMan.UseCommandModes;
end;


function TILActionBar.UpdateWithImage(const ACmdMode: TCommandMode; Inserting : Boolean) : integer;
var
  TB : TToolButton;
  AC : TAction;
begin
  Result := Table.DBWork.UpdateImageFromSource(Inserting);
  ExecResult(cmUpdate, Result);
  if Result <> 0 then
  begin
    AC := ButtonToAction(abInsertImages);
    TB := ActionToTB(AC);
    AC.Checked := False;
    TB.Down := false;
  end;

end;

procedure TILActionBar.NavNextExec(Sender : TObject);
begin
    Table.Dataset.Next;
end;

procedure TILActionBar.NavNextUpd(Sender : TObject);
begin
    TAction(Sender).Enabled := IsIdle and
     (Table.Dataset.Position < Table.Dataset.Count- 1);
end;

procedure TILActionBar.NavPrevExec(Sender : TObject);
begin
   Table.Dataset.Previous;
end;

procedure TILActionBar.NavPrevUpd(Sender : TObject);
begin

    TAction(Sender).Enabled := IsIdle and (Table.Dataset.Position > 0);
end;

procedure TILActionBar.NavFirstExec(Sender : TObject);
begin
   Table.Dataset.First;
end;

procedure TILActionBar.NavFirstUpd(Sender : TObject);
begin

    TAction(Sender).Enabled := IsIdle and (Table.Dataset.Position > 0);
end;

procedure TILActionBar.NavLastExec(Sender : TObject);
begin
   Table.Dataset.Last;
end;

procedure TILActionBar.NavLastUpd(Sender : TObject);
begin
    TAction(Sender).Enabled := IsIdle and
     (Table.Dataset.Position < Table.Dataset.Count- 1);
end;

procedure TILActionBar.ExecExec(Sender : TObject);
var
 A : TAction;
 Ordering : TSortOrdering;
 Res : integer;
begin
  if ButtonToAction(abInsertImages).Checked then
  begin
    UpdateWithImage(TMan.CommandMode, false);
  end
  else
  begin
     A := ButtonToAction(abInsertImages);
     A.Checked := False;
     ActionToTB(A).Down := false;
     if TMan.CommandMode = cmSort then
       ExecSort(Ordering);
     if TMan.CommandMode = cmSort then
     begin
       if Ordering = soDescending then
         Res := TMan.ExecCommand(ovSortDesc)
       else
         Res := TMan.ExecCommand(ovDefault);
     end
     else
     begin
       Res := TMan.ExecCommand(ovDefault);
     end;
     if TMan.CommandMode in [cmSearch, cmUpdate, cmSQL] then
       ExecResult(TMan.CommandMode, Res);
  end;
end;

procedure TILActionBar.ExecUpd(Sender : TObject);
var
  A : TAction;
  Res : integer;
  Opt : TBaseCommandOption;
  Ordering : TSortOrdering;
begin
    A := Sender as TAction;
    if not IsIdle then
    begin
       A.Enabled := false;
       Exit;
    end;

    if UseCommandModes and (TMan.ActiveRecord <> nil) and (TMan.CommandMode = cmUpdate)
    and TMan.ActiveRecord.IsDeleted then
    begin
      A.Enabled := false;
      Exit;
    end;

    begin
      if ButtonToAction(abInsertImages).Checked and (TMan.CommandMode = cmUpdate) then
        Opt := ovUpdateImage
      else if TMan.CommandMode = cmSort then
      begin
        ExecSort(Ordering);
        if Ordering = soDescending then
          Opt := ovSortDesc
        else
          Opt := ovDefault;
      end
      else
        Opt := ovDefault;

      Res := TMan.CanExecCommand(TMan.CommandMode, Opt);
      A.Enabled := (Res = 0);
      ValidationResult(TMan.CommandMode, Res);
    end;
end;

procedure TILActionBar.InsEmptyExec(Sender : TObject);
var
  AC : TAction;
  TB : TToolButton;
begin
     if ButtonToAction(abInsertImages).Checked
       and (Table.Datahandler.ImageSource <> isImageTable) then
     begin
       if not (Table.Datahandler.ImageSource in [isFile, isFileOrClipboard]) then
        Table.DBWork.Insert;
       UpdateWithImage(cmUpdate, True);
     end
     else
     begin
       Table.DBWork.Insert;
       AC := ButtonToAction(abInsertImages);
       AC.Checked := false;
       TB := ActionToTB(AC);
       TB.Down := false;
       ExecResult(cmUpdate, 1);
     end;
end;

procedure TILActionBar.InsEmptyUpd(Sender : TObject);
var
  EI : integer;
  Res : integer;
  Rec : TILDataRecord;
begin
   if not IsIdle or Table.IsReadOnly then
   begin
     TAction(Sender).Enabled := false;
   end
   else
   begin
     Res := 0;
     if Table is TILTreeViewTable then
     with Table as TILTreeViewTable do
     begin
       if (FocusedRecord = nil)
       and (TreeViewParams.InsertMode <> timRoot)
       then
         Res := E_TVReference;
     end;
     if Res <> 0 then
     begin
       TAction(Sender).Enabled := false;
       if Assigned(FOnInsertState) then
         FOnInsertState(Self, Res);
       Exit;
     end;
     Rec := nil;
     if Table.Dataset.Count > 0 then
       Rec := Table.Dataset[Table.Dataset.Count-1];
     if Rec = nil then
       TAction(Sender).Enabled := True
     else if Rec.IsNew or Table.Dataset.Changed then
       TAction(Sender).Enabled := (Table.DBWork.CanSave(EI) = 0)
     else
       TAction(Sender).Enabled := True;
     if not TAction(Sender).Enabled then
     begin
       if Assigned(FOnInsertState) then
         FOnInsertState(Self, -1)
     end
     else
       if Assigned(FOnInsertState) then
         FOnInsertState(Self, 0)
   end;
end;

procedure TILActionBar.UpdExec(Sender : TObject);
var
  Res : integer;
begin
   if UseCommandModes then
   begin
      if TMan.CommandMode <> cmUpdate then
      begin
        TMan.CommandMode := cmUpdate;
        ChangeCommand;
      end;
      ActionToTB(Sender as TAction).Down := True;
      SetCtrlFocus(duData);
   end
   else
   begin
     if ImageMode then
      begin
        UpdateWithImage(cmUpdate, false);
      end
      else
      begin
        Res := Table.DBWork.Update(nil);
        ExecResult(cmUpdate, Res);
      end;
   end;
end;

procedure TILActionBar.UpdUpd(Sender : TObject);
var
  Res : integer;
  Btn : TToolButton;
begin
   if not IsIdle then
    TAction(Sender).Enabled := false
   else if Table.IsReadOnly then
   begin
     TAction(Sender).Enabled := false;
   end
   else if UseCommandModes then
   begin
      TAction(Sender).Enabled := True;
      if TMan.CommandMode = cmUpdate then
      begin
       Btn := ActionToTB(TAction(Sender));
       TAction(Sender).Checked := True;
       Btn.Down := True;
      end;
   end
   else
   begin
      if ImageMode then
      begin
        if Table.DataHandler.ImageSource = isClipBoard then
        begin
         if Clipboard.HasFormat(CF_BITMAP) then
           Res := 0
         else
           Res := -1
        end
        else
          Res := 0;
        if (Res = 0) and TMan.ActiveRecord.Changed then
         Res := TMan.CanExecCommand(cmUpdate, ovDefault);
      end
      else
        Res := TMan.CanExecCommand(cmUpdate, ovDefault);
      TAction(Sender).Enabled := (Res = 0);
   end;
   ChangeCmdState(cmUpdate, TAction(Sender));
end;

procedure TILActionBar.SearchExec(Sender : TObject);
var
  Res : integer;
begin
   if UseCommandModes then
   begin
      if TMan.CommandMode <> cmSearch then
      begin
        TMan.CommandMode := cmSearch;
        ChangeCommand;
      end;
      ActionToTB(Sender as TAction).Down := True;
      SetCtrlFocus(duSearch);
   end
   else
   begin
      Res := Table.DBView.Search();
      ExecResult(cmSearch, Res);
   end;
end;

procedure TILActionBar.SearchUpd(Sender : TObject);
var
  Btn : TToolButton;
begin
   if UseCommandModes then
   begin
      TAction(Sender).Enabled := IsIdle;
      if TMan.CommandMode = cmSearch then
      begin
       Btn := ActionToTB(TAction(Sender));
       TAction(Sender).Checked := True;
       Btn.Down := True;
      end;
   end
   else
   begin
      TAction(Sender).Enabled := IsIdle and (TMan.CanExecCommand(cmSearch, ovDefault) = 0);
   end;
   ChangeCmdState(cmSearch, TAction(Sender));
end;

procedure TILActionBar.FreeSearchExec(Sender : TObject);
begin
    ExecResult(cmSearch, Table.DBView.Search);
end;

procedure TILActionBar.FreeSearchUpd(Sender : TObject);
var
  Res : integer;
  Enabl : Boolean;
begin
   Enabl := IsIdle and Table.DataHandler.FreeSearch and (TMan.CommandMode <> cmSearch);
   if not Enabl then
   begin
     TAction(Sender).Enabled := False;
     Exit;
   end;
   Res := TMan.CanExecCommand(cmSearch, ovDefault);
   TAction(Sender).Enabled := (Res = 0);
   ValidationResult(cmSearch, Res);
end;

procedure TILActionBar.FilterExec(Sender : TObject);
begin
    SetFilterTable;
end;

procedure TILActionBar.FilterUpd(Sender : TObject);
var
  TBL : TILDataTable;
  AC : TAction;
  IsLinked : Boolean;
  i : integer;
  Enabl : Boolean;
begin
   AC := TAction(Sender);
   if not IsIdle then
   begin
     AC.Enabled := false;
     Exit;
   end;
   IsLinked := (Table is TILDataTable) and (TILDataTable(Table).MasterLinkCount > 0);
   if (Table is TILDataTable) and Table.IsFreeTable then
     if not IsLinked then
   begin
     AC.Enabled := false;
     Exit;
   end;
   if (TILDataTable(TILDataTable(Table).ParentTable) = nil) and not IsLinked then
   begin
     AC.Enabled := false;
     Exit;
   end;
   if not IsLinked then
   begin
    TBL := TILDataTable(TILDataTable(Table).ParentTable);
    AC.Enabled := TBL.Dataset.Count > 0;
   end
   else
   begin
     Enabl := False;
     for I := 0 to TILDataTable(Table).MasterLinkCount-1 do
     begin
        if TILDataTable(Table).MasterLinks[i].Dataset.Count > 0 then
        begin
          Enabl := True;
          Break;
        end;
     end;
     AC.Enabled := Enabl;
   end;
end;

procedure TILActionBar.DelExec(Sender : TObject);
var
  Accept : Boolean;
  Res : integer;
  i : integer;
begin
    if not Table.FocusedRecord.IsNew then
    begin
      for I := 0 to Table.SelCount-1 do
      begin
       Res := Table.DBWork.CanDelete(Table.Dataset.Selection[i].RecordIndex);
       if Res <> 0 then
       begin
         ShowILErrorText(Res);
         Exit;
       end;
      end;

    end;
    DeleteRecord(Accept);
    if Accept then
    begin
      Table.DBWork.Delete(-1);
    end;
end;

procedure TILActionBar.DelUpd(Sender : TObject);
begin
   TAction(Sender).Enabled := IsIdle and (Table.Dataset.Position <> -1)
   and not Table.IsReadOnly and not Table.FocusedRecord.IsDeleted;
end;

procedure TILActionBar.UndoExec(Sender : TObject);
begin
  Table.DBWork.RevertChanges(False);
end;

procedure TILActionBar.UndoUpd(Sender : TObject);
var
  Enabl : Boolean;
begin
  Enabl := IsIdle;
  if Enabl then
  begin
    if Table.MultiSelected then
      TAction(Sender).Enabled := Table.Dataset.CanRevert
    else
      TAction(Sender).Enabled := Table.Dataset.CanRevert(False);
  end
  else
    TAction(Sender).Enabled := false;
end;

procedure TILActionBar.ClearExec(Sender : TObject);
begin
  Table.DBWork.ClearDataset;
end;

procedure TILActionBar.ClearUpd(Sender : TObject);
begin
  TAction(Sender).Enabled := IsIdle and (Table.Dataset.Count > 0);
end;

procedure TILActionBar.InsImagesUpd(Sender : TObject);
begin
  if not IsIdle then
  begin
    TAction(Sender).Enabled := false;
    Exit;
  end;
  if UseCommandModes then
     TAction(Sender).Enabled := IsIdle
      and (Table.DataHandler.ImageSource <> isNoImages)
  else
  begin
     TAction(Sender).Enabled := IsIdle and (Table.FocusedRecord <> nil)
     and (Table.DataHandler.ImageSource <> isNoImages);
  end;
  if not TAction(Sender).Enabled then
   TAction(Sender).Checked := false;

end;

procedure TILActionBar.InsImagesExec(Sender : TObject);
var
  A : TAction;
  TB : TToolButton;
begin
   A := TAction(Sender);
   TB := ActionToTB(A);
   if not UseCommandModes then
     UpdateWithImage(cmUpdate, True)
   else
    A.Checked := TB.Down;
   AdjustHints;
end;

procedure TILActionBar.SaveExec(Sender : TObject);
begin
   TMan.SaveAll;
end;

procedure TILActionBar.SaveUpd(Sender : TObject);
var
  T : TILTable;
  ErrIndx : integer;
  Res : integer;
begin
   if IsIdle then
   begin
    Res := TMan.CanSaveAll(T, ErrIndx);
    TAction(Sender).Enabled := (Res = 0);
    SaveState(Res);
   end
   else
   begin
     TAction(Sender).Enabled := false;
     SaveState(-1);
   end;
end;

procedure TILActionBar.ColumnsExec(Sender : TObject);
begin
   if TMan.CommandMode <> cmColumns then
   begin
        TMan.CommandMode := cmColumns;
        ChangeCommand;
   end;
    ActionToTB(Sender as TAction).Down := True;
end;

procedure TILActionBar.ColumnsUpd(Sender : TObject);
var
  Btn : TToolButton;
begin
  TAction(Sender).Enabled := IsIdle and UseCommandModes;
  if UseCommandModes and (TMan.CommandMode = cmColumns) then
  begin
       Btn := ActionToTB(TAction(Sender));
       TAction(Sender).Checked := True;
       Btn.Down := True;
  end;
  ChangeCmdState(cmColumns, TAction(Sender));
end;

procedure TILActionBar.SortExec(Sender : TObject);
begin
   if TMan.CommandMode <> cmSort then
      begin
        TMan.CommandMode := cmSort;
        ChangeCommand;
      end;
   ActionToTB(Sender as TAction).Down := True;
end;

procedure TILActionBar.SortUpd(Sender : TObject);
var
  Btn : TToolButton;
begin
   TAction(Sender).Enabled := IsIdle and UseCommandModes;
   if UseCommandModes and( TMan.CommandMode = cmSort) then
   begin
       Btn := ActionToTB(TAction(Sender));
       TAction(Sender).Checked := True;
       Btn.Down := True;
   end;
   ChangeCmdState(cmSort, TAction(Sender));
end;

procedure TILActionBar.SQLExec(Sender : TObject);
var
  sl : TStringList;
  S : String;
begin
   if UseCommandModes then
   begin
     if TMan.CommandMode <> cmSQL then
      begin
        TMan.CommandMode := cmSQL;
        ChangeCommand;
      end;
     ActionToTB(Sender as TAction).Down := True;
   end
   else
   begin
     S := '';
     TMan.OnGetSQLtext(Self, S);
     if S = '' then
       Exit;
     sl := TStringList.Create;
     try
      sl.Text := S;
      ExecResult(cmSQL, Table.DBCommons.ExecSQL(sl))
     finally
       sl.Free;
     end;
   end;
end;

procedure TILActionBar.SQLUpd(Sender : TObject);
var
  Btn : TToolButton;
  sl : TStringList;
  S : string;
begin
    if not UseCommandModes and IsIdle and Assigned(TMan.OnGetSQLText) then
    begin
       S := '';
       TMan.OnGetSQLtext(Self, S);
       sl := TStringList.Create;
       sl.Text := S;
       try
        TAction(Sender).Enabled := (Table.DBCommons.CanExecSQL(Sl) = 0);
       finally
         sl.Free;
       end;
       Exit;
    end;
    TAction(Sender).Enabled := IsIdle and UseCommandModes;
    if UseCommandModes and (TMan.CommandMode = cmSQL) then
    begin
       Btn := ActionToTB(TAction(Sender));
       TAction(Sender).Checked := True;
       Btn.Down := True;
    end;
    ChangeCmdState(cmSQL, TAction(Sender));
end;

procedure TILActionBar.ToggleVMExec(Sender : TObject);
begin
   if Table.Viewer.ViewMode = vmReport then
     Table.Viewer.ViewMode := vmImages
   else
     Table.Viewer.ViewMode := vmReport;
end;

procedure TILActionBar.ToggleVMUpd(Sender : TObject);
begin
   TAction(Sender).Enabled := IsIdle and Assigned(Table.Viewer);
end;

{ Control buttons }

procedure TILActionBar.ClearSingleValExec(Sender : TObject);
var
  C : TWinControl;
  D : TILDataField;
  Usg : TDataUsage;
begin
  C := Screen.ActiveForm.ActiveControl;
  D := DataFieldFromActiveCtrl(Usg);
  if Usg = duData then
   D.Table.DBWork.ClearControls(C)
  else
   D.Table.DBView.ClearControls(C);

end;

procedure TILActionBar.ClearSingleValUpd(Sender : TObject);
 var
  C : TWinControl;
  D : TILDatafield;
  Usg : TDataUsage;
begin
  C := Screen.ActiveForm.ActiveControl;
  if C = nil then
  begin
    TAction(Sender).Enabled := false;
    Exit;
  end;
  D := DataFieldFromActiveCtrl(Usg);
  if D = nil then
  begin
    TAction(Sender).Enabled := false;
  end
  else
    TAction(Sender).Enabled := D.Table.IsControlActive(C);
end;

procedure TILActionBar.ClearValsExec(Sender : TObject);
var
  Usg : TDataUsage;
begin
   if not UseCommandModes then
   begin
     DataFieldFromActiveCtrl(Usg);
     if Usg = duData then
       Table.DBWork.ClearControls
     else
       Table.DBView.ClearControls;
   end
   else
   begin
     if Tman.CommandMode = cmUpdate then
       Table.DBWork.ClearControls
     else
       Table.DBView.ClearControls;
   end;
end;

procedure TILActionBar.ClearValsUpd(Sender : TObject);
var
 Usg : TDataUsage;
 DF : TILDataField;

  function AllInActive : Boolean;
  var
    i : integer;
    DF2 : TILDataField;
  begin
    Result := True;
    if not UseCommandModes then
    for I := 0 to DF.Collection.Count-1 do
    begin
       DF2 := DF.Collection.Items[i] as TILDataField;
       if (USG = duData) and (DF2.DataControl <> nil) then
       begin
         if Table.IsControlActive(DF2.DataControl) then
         begin
           Result := false;
           Break;
         end;
       end
       else if (USG = duSearch) and (DF2.SearchControl <> nil) then
       begin
         if Table.IsControlActive(DF2.SearchControl) then
         begin
           Result := false;
           Break;
         end;
       end
    end

    else for I := 0 to TMan.WorkFieldCount-1 do
      begin
        if TMan.WorkFields[i].Control <> nil then
        if Table.IsControlActive(TMan.WorkFields[i].Control) then
        begin
          Result := false;
          Break;
        end;
      end;
  end;

begin
   if IsIdle and not UseCommandModes then
    DF := DatafieldFromActiveCtrl(Usg);
   TAction(Sender).Enabled := (IsIdle and UseCommandModes
   and (Tman.CommandMode in [cmSearch, cmUpdate])
   and not AllInactive)
   or (IsIdle and not UseCommandModes and (DF <> nil) and not AllInactive);
   ;
end;

procedure TILActionBar.PopValsExec(Sender : TObject);
var
  Usg : TDataUsage;
begin
   if not UseCommandModes then
   begin
     DataFieldFromActiveCtrl(Usg);
     if Usg = duData then
       Table.DBWork.WriteControls(Table.FocusedRecord)
     else
       Table.DBView.WriteControls(Table.FocusedRecord);
   end
   else
   begin
    if Tman.CommandMode = cmUpdate then
     Table.DBWork.WriteControls(Table.FocusedRecord)
   else
     Table.DBView.WriteControls(Table.FocusedRecord);
   end;
end;

procedure TILActionBar.PopValsUpd(Sender : TObject);
var
  DF : TILDataField;
  Usg : TDataUsage;
  HasCtrls : Boolean;
  i : integer;
begin
  if IsIdle and not UseCommandModes then
  begin
    DF := DatafieldFromActiveCtrl(Usg);
    TAction(Sender).Enabled :=  (DF <> nil) and (Table.FocusedRecord <> nil);
  end
  else if IsIdle then
  begin
      HasCtrls := false;
      for I := 0 to TMan.WorkFieldCount-1 do
       if TMan.WorkFields[i].Visible then
       begin
         HasCtrls := True;
         Break;
       end;
       TAction(Sender).Enabled := (HasCtrls and (Tman.CommandMode in [cmSearch, cmUpdate])
       and (Table.FocusedRecord <> nil))
  end
  else TAction(Sender).Enabled := False;
end;

procedure TILActionBar.ShowAllExec(Sender : TObject);
begin
   TMan.WorkList.CheckAll(cbChecked);
   TMan.ResolveWorklist;
end;

procedure TILActionBar.ShowAllUpd(Sender : TObject);
 function AllVisible : Boolean;
 var
  i : integer;
 begin
    Result := True;
    for I := 0 to Tman.WorkFieldCount-1 do
      if not Tman.WorkFields[i].Visible then
      begin
        Result := false;
        Break;
      end;
 end;

begin
  TAction(Sender).Enabled := IsIdle(True) and UseCommandModes
   and (Tman.CommandMode in [cmSearch, cmUpdate, cmColumns, cmSort])
   and not AllVisible;
end;

procedure TILActionBar.ShowNoneExec(Sender : TObject);
begin
  TMan.WorkList.CheckAll(cbUnChecked);
   TMan.ResolveWorklist;
end;

procedure TILActionBar.ShowNoneUpd(Sender : TObject);
function AllHidden : Boolean;
 var
  i : integer;
 begin
    Result := True;
    for I := 0 to Tman.WorkFieldCount-1 do
      if Tman.WorkFields[i].Visible then
      begin
        Result := false;
        Break;
      end;
 end;
begin
  TAction(Sender).Enabled := IsIdle(True) and UseCommandModes
   and (Tman.CommandMode in [cmSearch, cmUpdate, cmColumns, cmSort])
   and not AllHidden;
end;

procedure TILActionBar.ToggleDualExec(Sender : TObject);
var
  Wf : TILWorkField;
begin
    Wf := WFFromActiveCtrl(True);
    if WF.Expanded then
       WF.Collapse
    else
       WF.Expand;
end;

procedure TILActionBar.ToggleDualUpd(Sender : TObject);
var
  Enabl : Boolean;
  Wf : TILWorkField;

begin
  if UseCommandModes and (TMan.WorkList <> nil) and (TMan.WorkList.ItemIndex <> -1) then
  begin
   Enabl := IsIdle(True) and (TMan.WorkList.ItemIndex <> -1)
   and (Tman.CommandMode in [cmSearch, cmUpdate]);
   if Enabl then
   begin
    Wf := TMan.WorkList.Items.Objects[TMan.WorkList.ItemIndex] as TILWorkField;
    Enabl := Wf.CanExpand or WF.CanCollapse;
   end;
  end
  else
  begin
    Wf := WfFromActiveCtrl(True);
    Enabl := (WF <> nil);
    if Enabl then
     Enabl := Wf.CanExpand or WF.CanCollapse;
  end;
  TAction(Sender).Enabled := Enabl;
end;

procedure TILActionBar.MoveUpExec(Sender : TObject);
begin
  TMan.MoveWorkField(TMan.WorkList.ItemIndex, TMan.WorkList.ItemIndex - 1);
end;

procedure TILActionBar.MoveUpUpd(Sender : TObject);
begin
  TAction(Sender).Enabled := IsIdle(True) and
   (TMan.CanMoveWorkField(TMan.WorkList.ItemIndex, TMan.WorkList.ItemIndex - 1));
end;

procedure TILActionBar.MoveDownExec(Sender : TObject);
begin
   TMan.MoveWorkField(TMan.WorkList.ItemIndex, TMan.WorkList.ItemIndex + 1);
end;

procedure TILActionBar.MoveDownUpd(Sender : TObject);
begin
  TAction(Sender).Enabled := IsIdle(True) and
  (TMan.CanMoveWorkField(TMan.WorkList.ItemIndex, TMan.WorkList.ItemIndex + 1));
end;

procedure TILActionBar.LogOpAndExec(Sender : TObject);
begin
    FDropBtn.ImageIndex := TAction(Sender).ImageIndex;
    ShowLogOpButtons(smAndAligned);
    TMan.SearchModel := smAndAligned;
end;

procedure TILActionBar.LogOpAndUpd(Sender : TObject);
begin
   TAction(Sender).Enabled := IsIdle;
end;

procedure TILActionBar.LogOpAndOrExclExec(Sender : TObject);
begin
   FDropBtn.ImageIndex := TAction(Sender).ImageIndex;
   ShowLogOpButtons(smAndOrAligned);
   TMan.SearchModel := smAndOrAligned;
end;

procedure TILActionBar.LogOpAndOrExclUpd(Sender : TObject);
begin
  TAction(Sender).Enabled := IsIdle;
end;

procedure TILActionBar.LogOpAndOrMixedExec(Sender : TObject);
begin
  FDropBtn.ImageIndex := TAction(Sender).ImageIndex;
  ShowLogOpButtons(smAndOrMixed);
  TMan.SearchModel := smAndOrMixed;
end;

procedure TILActionBar.LogOpAndOrMixedUpd(Sender : TObject);
begin
   TAction(Sender).Enabled := IsIdle and UseCommandModes;
end;

procedure TILActionBar.LogOpExec(Sender : TObject);
begin
  //
end;

procedure TILActionBar.LogOpUpd(Sender : TObject);
begin
  //
end;

procedure TILActionBar.SetFilterTable;
  var
    i : integer;
    Found : Boolean;
    Value : TILDataTable;
  begin
    if not Assigned(FOnSetFilterTable) then
    begin
      if TILDataTable(Table).MasterLinkCount > 0 then
        TILDataTable(Table).MasterLinks[0].FindLinkedRecords
      else if TILDataTable(Table).MasterSource <> nil then
        Table.DBView.FilterRecords(TILDataTable(Table));
      Exit;
    end;

    Value := nil;
    if TILDataTable(Table).MasterLinkCount = 0 then
    begin
      Table.DBView.FilterRecords(Value);
      Exit;
    end;

    FOnSetFilterTable(Self, Value);


    if Value = nil then
    begin
      Table.DBView.FilterRecords(Value);
      Exit;
    end;

    if not TMan.IsInSearchPath(Table, Value) then
    begin
        HandleILException(E_TableOutOfPath, Value.TableName);
    end;

    if Value.DetailLinkCount > 0 then
    begin
      Found := false;
      for I := 0 to Value.DetailLinkCount-1 do
        if Value.DetailLinks[i] = Table then
        begin
          Found := True;
          Break;
        end;
      if not Found then
      begin
       HandleILException(E_NotAParentTable, Value.TableName);
      end;
    end;

    if (Value <> TILDataTable(Table).MasterSource) and (Value.DetailLinkCount = 0)  then
      HandleILException(E_NotAParentTable, Value.TableName);

    Table.DBView.FilterRecords(Value);
  end;

{ TILMessageBar -------------------------------------------------------------------}

  constructor TILMessageBar.Create(AComponent : TComponent);
  begin
    inherited;
    ControlStyle := ControlStyle + [csReplicatable, csPannable, csOpaque];
    DoubleBuffered := True;
    Height := 23;
    Align := alBottom;
    FMessageTypes := [mtDataset, mtSearch, mtEdit, mtSave];
    FMessageHandler := TILMessageHandler.Create;
    FDatasetOptions := [moSelCount, moConfirm];
    FSaveOptions := [moError, moConfirm];
    FUpdateOptions := [moError, moConfirm];
    FSearchOptions := [moError, moConfirm];
    FOrigUpdateOptions := FUpdateOptions;
    FProgressBar := True;
    FprogressBarColor := clLime;
    FErrorTextFontStyle := [fsBold];
    FReadyTextColor := clGreen;
    FErrorTextColor := clMaroon;
    FReadyTextFontStyle := [fsBold];
    CreateMsgPanels;
    Action := TAction.Create(Self);
    LoadGlyphs;
    RegMbs.Add(Self);
  end;

  destructor TILMessageBar.Destroy;
  var
    I : TMessageType;
    Ind : integer;
  begin
    inherited;
    for I := mtDataset to mtSave do
    begin
      FPanels[i].Glyph.Free;
    end;
    FSQLGlyph.Free;
    FErrorGlyph.Free;
    FReadyGlyph.Free;
    FMessageHandler.Free;
    Ind := RegMbs.IndexOf(Self);
    if Ind <> -1 then
      RegMbs.Delete(Ind);
  end;

  procedure TILMessageBar.Loaded;
  begin
    inherited;
  end;

  procedure TILMessageBar.LoadGlyphs;
  var
    I : TMessageType;
    BM : VCL.Graphics.TBitmap;
  begin
    for I := mtDataset to mtSave do
    begin
       BM := nil;
       try
       case I of
         mtDataset:
         begin
          if Assigned(FGlyphs) and (FGlyphs.Count > 0) then
          begin
           FGlyphs.GetIcon(0, FPanels[i].Glyph);
          end
          else
           BM := TMan.LoadResource('MBDataset');
         end;
         mtSearch:
         begin
          if Assigned(FGlyphs) and (FGlyphs.Count > 1) then
          begin
           FGlyphs.GetIcon(1, FPanels[i].Glyph);
          end
          else
           BM := TMan.LoadResource('MBSearch');
         end;
          mtEdit:
         begin
          if Assigned(FGlyphs) and (FGlyphs.Count > 2) then
          begin
           FGlyphs.GetIcon(2, FPanels[i].Glyph);
          end
          else
           BM := TMan.LoadResource('MBUpdate');
         end;
         mtSave:
         begin
          if Assigned(FGlyphs) and (FGlyphs.Count > 3) then
          begin
           FGlyphs.GetIcon(3, FPanels[i].Glyph);
          end
          else
           BM := TMan.LoadResource('MBSave');
         end;
         mtProgress: ;
       end;
       if BM <> nil then
       begin
         if Assigned(FPanels[i].Glyph) then
           FPanels[i].Glyph.Free;
         FPanels[i].Glyph := BitmapToIcon(BM);
       end;
       finally
         if BM <> nil then
          BM.Free;
       end;
    end;

    if Assigned(FGlyphs) and (FGlyphs.Count > 4) then
    begin
      FGlyphs.GetIcon(4, FSQLGlyph);
    end
    else
     BM := TMan.LoadResource('MBSQL');
    if BM <> nil then
    begin
      if Assigned(FSQLGlyph) then
       FSQLGlyph.Free;
      FSQLGlyph := BitmapToIcon(BM);
      BM.Free;
    end;

    if Assigned(FGlyphs) and (FGlyphs.Count > 5) then
    begin
      FGlyphs.GetIcon(5, FErrorGlyph);
    end
    else
      BM := TMan.LoadResource('Error');
    if BM <> nil then
    begin
      if Assigned(FErrorGlyph) then
        FErrorGlyph.Free;
      FErrorGlyph := BitmapToIcon(BM);
      BM.Free;
    end;

    if Assigned(FGlyphs) and (FGlyphs.Count > 6) then
    begin
      FGlyphs.GetIcon(6, FReadyGlyph);
    end
    else
      BM := TMan.LoadResource('Ready');
    if BM <> nil then
    begin
      if Assigned(FReadyGlyph) then
        FReadyGlyph.Free;
      FReadyGlyph := BitmapToIcon(BM);
      BM.Free;
    end;
  end;

  procedure TILMessageBar.InitPanels;
  var
    i : TMessageType;
  begin
    for I := mtDataset to mtProgress do
    with FPanels[i] do
    begin
      Width := 100;
      Text := '';
      Count := 0;
      Progress := 0;
      Operation := iloFinished;
      SQLOperation := ctUnknown;
      Visible := false;
    end;
  end;

  procedure TILMessageBar.DrawMessage;
  var
    I : TMessageType;
    MI : TMessageInfo;
    R : TRect;
  begin
    R.Left := 0;
    R.Top := 0;
    R.Bottom := ClientHeight;
    R.Right := ClientWidth;
    for I := mtDataset to mtSave do
    if FPanels[i].Visible then
    begin
       if FPanels[i].Visible then
       begin
          MI := GetMessages(i);
       end;
       FOnDrawMessage(Self, MI, R, Canvas);
       R.Left := R.Right + 5;
       R.Right := ClientWidth;
    end;
    MI.Count := -1;
    MI.SelectedCount := -1;
    MI.IsownText := True;
    MI.DatasetIndex := -1;
    MI.MessagetType := mtEdit;
    MI.MessageKind := moNotAvailable;
    MI.Text := OwnText;
    MI.State := -1;
    MI.SQLOperation := ctUnknown;
    MI.Glyph := nil;
    FOnDrawMessage(Self, MI, R, Canvas);
  end;

  procedure TILMessageBar.Paint;
  var
   I : TMessageType;
   X : integer;

   procedure SetWidths;
   var
    I : TMessageType;

    procedure DoSetWidth(AState : integer);
    begin
      if (I = mtDataset) then
      begin
        FPanels[I].Width := Canvas.TextWidth(FPanels[i].Text) + 24; //Glyph + 3 + text + 5
        Exit;
      end;

      if TrafficLights and (AState >= 0) then
        FPanels[I].Width := 40; // Glyph + 3 + traffic light + 5
      if AState > 0 then
      begin
       Canvas.Font.Style := ErrorTextFontStyle;
       Canvas.Font.Color := ErrorTextColor;
      end
      else if (AState = 0) and not (MessageHandler.MessageKind(I) = moConfirm) then
      begin
         Canvas.Font.Style := ReadyTextFontStyle;
         Canvas.Font.Color := ReadyTextColor;
      end;
      if not TrafficLights or (MessageHandler.MessageKind(I) = moConfirm) then
       FPanels[I].Width := Canvas.TextWidth(FPanels[i].Text) + 24; //Glyph + 3 + text + 5
    end;

   begin
     for I := mtDataset to mtProgress do
     begin
       if Assigned(FOnShowMessage) then
       begin
         FOnShowMessage(Self, I, MessageHandler.ValidationState(i),
          MessageHandler.MessageKind(i), FPanels[i].Visible);
       end;
       if not FPanels[i].Visible then
         Continue;
       if I <> mtProgress then
       begin
           FPanels[i].Text := MessageHandler.MessageText(I);
           DoSetWidth(MessageHandler.ValidationState(I));
       end;
     end; {Loop}
   end;

   function WriteMessage(Msg : TMessageType) : integer;
   var
     P : TPanelInfo;
     R : TRect;
     X : integer;

     procedure SetTextProps(AState : integer);
     begin
       if (AState > 0) then
       begin
          R := TextRect(Msg);
          Canvas.Brush.Style := bsClear;
          Canvas.Font.Color := FErrorTextColor;
          Canvas.Font.Style := FErrorTextFontStyle;
       end
       else if (AState = 0) and not (MessageHandler.MessageKind(Msg) = moConfirm) then
       begin
          R := TextRect(Msg);
          Canvas.Brush.Style := bsClear;
          if Msg = mtSearch then
          begin
            Canvas.Font.Style := [];
            if MessageHandler.ConfirmCount(Msg) = 0 then
              Canvas.Font.Color := FErrorTextColor
            else if TStyleManager.IsCustomStyleActive then
            begin
               Canvas.Font.Color := StyleServices.GetStyleFontColor(sfStatusPaneltextNormal);
            end
           else
             Canvas.Font.Color := clWindowText;
          end
          else
          begin
            Canvas.Font.Color := FReadyTextColor;
            Canvas.Font.Style := FReadyTextFontStyle;
          end;
       end;
     end;

     procedure DoWriteMessage(AState : integer);
     begin
        if TrafficLights and not (MessageHandler.MessageKind(Msg) = moConfirm)
        and (Msg in [mtSearch, mtSave, mtEdit])
        and not((Msg = mtSearch) and (AState = 0)) then
        begin
          if AState > 0 then
          begin
           Canvas.Draw(R.Left, R.Top, FErrorGlyph);
           X := R.Right;
          end
          else if AState = 0 then
          begin
           Canvas.Draw(R.Left, R.Top, FReadyGlyph);
           X := R.Right;
          end;
        end
        else
        begin
          Canvas.Brush.Style := bsClear;
          Canvas.TextOut(R.Left, R.Top, FPanels[Msg].Text);
        end;
     end;

     procedure WriteGlyphs;
     begin
       R := MessageRect(Msg);
       if SQLMode and (Msg = mtEdit) then
         Canvas.Draw(R.Left, 1, FSQLGlyph)
       else if (Msg = mtEdit) then
         Canvas.Draw(R.Left, 1, FPanels[Msg].Glyph)
       else
         Canvas.Draw(R.Left, 1, FPanels[Msg].Glyph);
     end;

   begin
      Result := 0;
      P := FPanels[Msg];
      X := -1;
      if TStyleManager.IsCustomStyleActive then
      begin
       Canvas.Font.Color := StyleServices.GetStyleFontColor(sfStatusPaneltextNormal);
      end
      else
       Canvas.Font.Color := clWindowText;

      if Msg =  mtDataset then
      begin
          R := TextRect(Msg);
          DoWriteMessage(-1);
      end
      else if Msg = mtProgress then
      begin
         if FProgressActive then
           ShowProgress;
         Exit;
      end
      else
      begin
          SetTextProps(MessageHandler.ValidationState(Msg));
          R := TextRect(Msg);
          DoWriteMessage(MessageHandler.ValidationState(Msg));
      end;

      Canvas.Font.Style := [];
      if X = -1 then
        Result := Canvas.PenPos.X
      else
        Result := X;

      WriteGlyphs;
   end;

   procedure WriteText(StartPos : integer);
   var
     X, W : integer;
     R : TRect;
   begin
      if StartPos > 5 then
        X := StartPos + 5
      else
        X:= StartPos;
      Canvas.Pen.Color := clGray;
      if TStyleManager.IsCustomStyleActive then
      begin
       Canvas.Font.Color := StyleServices.GetStyleFontColor(sfStatusPaneltextNormal);
       Canvas.Pen.Color := Canvas.Font.Color;
      end
      else
       Canvas.Font.Color := clWindowText;
      R.Left := X;
      R.Top := 0;
      R.Right := ClientWidth;
      R.Bottom := ClientHeight;
      if (StartPos > 5) then
      begin
       Canvas.MoveTo(R.Left-1, R.Top);
       Canvas.LineTo(R.Left-1, R.Bottom);
      end;
      W := Canvas.TextWidth(OwnText);
      if OwnTextAlignment = taLeftJustify then
        X := R.Left + 3
      else if OwnTextAlignment = taCenter then
        X := ((R.Right - R.Left) div 2) - (W div 2)
      else
        X := R.Right - W - 5;
      Canvas.TextOut(X, R.Top + 2, OwnText);
   end;

  begin
     Inherited;
     X := 5;
     if TStyleManager.IsCustomStyleActive then
     begin
       Canvas.Brush.Color := StyleServices.GetSystemColor(Color);
       Canvas.Pen.Color := StyleServices.GetSystemColor(clGray);
     end
     else
     begin
       Canvas.Brush.Color := Color;
       Canvas.Pen.Color := clGray;
     end;
     Canvas.FillRect(ClientRect);
     if MessageHandler.Table = nil then
       Exit;

     SetWidths;
     if (csDesigning in ComponentState) then
       Exit;

     if OwnerDraw and not FProgressActive then
     begin
       if Assigned(FOnDrawMessage) then
        DrawMessage;
       Exit;
     end;

     Canvas.Font.Assign(Font);
     for I := mtDataset to mtProgress do
     begin
       if FPanels[i].Visible then
       begin
         X := WriteMessage(i);
       end;
     end;
     if (OwnText <> '') and not FProgressActive  then
     begin
         WriteText(X);
     end;
  end;

  procedure TILMessageBar.CreateParams(var Params: TCreateParams);
  begin
    inherited CreateParams(Params);
    Params.Style := Params.Style or ws_Border;
  end;

  procedure TILMessageBar.SetMessageTypes(Value : TMessageTypes);
  begin
    if Value = MessageTypes then
      Exit;
    FmessageTypes := Value;
    CreateMsgPanels;
  end;

  procedure TILMessageBar.CreateMsgPanels;
  begin
    InitPanels;
    if mtDataset in MessageTypes then
    begin
      FPanels[mtDataset].Width := 130;
    end;
    if mtSearch in MessageTypes then
    begin
      FPanels[mtSearch].Width := 130;
    end;
    if mtEdit in MessageTypes then
    begin
      FPanels[mtEdit].Width := 130;
    end;
    if mtSave in MessageTypes then
    begin
      FPanels[mtSave].Width := 130;
    end;
    if mtProgress in MessageTypes then
    begin
      FPanels[mtProgress].Width := 130;
    end;
    Invalidate;
  end;

  procedure TILMessageBar.ShowProgress;
  var
    R, R2 : TRect;
    Pst : integer;
    W : integer;
    S : string;
  begin
    if FPanels[mtProgress].Count = 0 then
      Exit;
    S :=  FPanels[mtProgress].Text;
    W := Canvas.TextWidth(S);
    R.Left := 5;
    R.Top := 2;
    R.Height := 15;

    Canvas.Font.Style := [];
    if TStyleManager.IsCustomStyleActive then
      begin
       Canvas.Font.Color := StyleServices.GetStyleFontColor(sfStatusPaneltextNormal);
      end
      else
       Canvas.Font.Color := clWindowText;
    Canvas.TextOut(R.Left, R.Top, S + ': ');
    Pst := Round((FPanels[mtProgress].Progress-FStartProgress) /
     (FPanels[mtProgress].Count-FStartProgress) * 100);
    if ProgressBar then
    begin
      R.Width := 125;
      R.Left := R.Left + W + 5;
      R.Width := 125;

      Canvas.Pen.Color := clGray;
      Canvas.Brush.Style := bsClear;
      Canvas.Rectangle(R);
      inc(R.Left);
      inc(R.Top);
      dec(R.Bottom);
      dec(R.Right);

      W := Round(Pst * R.Width / 100);
      Canvas.Brush.Color := FProgressbarColor;
      Canvas.Brush.Style := bsSolid;
      R2 := R;
      R2.Right := R.Left + W;
      Canvas.FillRect(R2);
    end
    else
    begin
      Canvas.TextOut(Canvas.PenPos.X+ 5, R.Top, IntToStr(Pst) + '%');
    end;
  end;

  procedure TILMessageBar.MsgEvent(Sender : TObject);
  begin
     SetVisibles;
  end;

  procedure TILMessageBar.UpdateMessage(Operation : TILOperation;
   const Text : string; const Progress, Count : integer);
  var
    P : TPanelInfo;
  begin
    if not (mtProgress in MessageTypes) then
      Exit;
    if Operation = iloFinished then
    begin
      FProgressActive := false;
      FPrognosing := False;
      FStartProgress := 0;
      SetVisibles;
      Invalidate;
      Exit;
    end;
    P := FPanels[mtProgress];
    P.Operation := Operation;
    P.Progress := Progress;
    P.Count := Count;
    P.Text := Text;
    FPanels[mtProgress] := P;
    if not FProgressActive and not FPrognosing then
    begin
       FPrognoseStartTick := GetTickCount;
       FPrognosing := True;
    end
    else if FPrognosing and not FProgressActive then
    begin
      if GetTickCount - FPrognoseStartTick > 500 then
      begin
         FPrognosing := false;
         if Progress / Count < 0.5 then
         begin
           FProgressActive := True;
           FStartProgress := Progress;
           Clear;
           FPanels[mtProgress].Visible := True;
           Repaint;
         end;
      end;

    end
    else if FProgressActive then
       Repaint
    else
      Invalidate;
  end;

  procedure TILMessageBar.Clear;
  var
    I : TMessageType;
  begin
    for I := mtDataset to mtSave do
    begin
      FPanels[i].Visible := false;
    end;
  end;

  procedure TILMessageBar.SetVisibles;
  begin
       if (TMan = nil) or (Table = nil) then
        Exit;

       FPanels[mtDataset].Visible := (mtDataset in Self.MessageTypes);
       FPanels[mtSave].Visible := (mtSave in Self.MessageTypes) and (SaveOptions <> []);
       FPanels[mtEdit].Visible := (mtEdit in Self.MessageTypes) and (UpdateOptions <> []);
       FPanels[mtSearch].Visible := (mtSearch in Self.MessageTypes) and (SearchOptions <> []);
       FPanels[mtProgress].Visible := (mtProgress in Self.MessageTypes);

       if MessageHandler.MessageKind(mtSave) = moNotAvailable then
         FPanels[mtSave].Visible := False
       else if (MessageHandler.ValidationState(mtSave) = 0) and not (moReady in SaveOptions) then
         FPanels[mtSave].Visible := False
       else if (MessageHandler.ValidationState(mtSave) > 0) and not (moError in SaveOptions) then
         FPanels[mtSave].Visible := False
       else if (MessageHandler.MessageKind(mtSave) = moConfirm) and not (moConfirm in SaveOptions) then
         FPanels[mtSave].Visible := False;
       if not FPanels[mtSave].Visible then
        if (MessageHandler.MessageKind(mtSave) = moConfirm) and (moConfirm in SaveOptions) then
         FPanels[mtSave].Visible := True;

       if MessageHandler.MessageKind(mtEdit) = moNotAvailable then
         FPanels[mtEdit].Visible := False
       else if (MessageHandler.ValidationState(mtEdit) = 0) and not (moReady in UpdateOptions) then
         FPanels[mtEdit].Visible := False
       else if (MessageHandler.ValidationState(mtEdit) > 0) and not (moError in UpdateOptions) then
         FPanels[mtEdit].Visible := False
       else if (MessageHandler.MessageKind(mtEdit) = moConfirm) and not (moConfirm in UpdateOptions) then
         FPanels[mtEdit].Visible := False;
       if not FPanels[mtEdit].Visible then
        if (MessageHandler.MessageKind(mtEdit) = moConfirm) and (moConfirm in UpdateOptions) then
         FPanels[mtEdit].Visible := True;

       if MessageHandler.MessageKind(mtSearch) = moNotAvailable then
         FPanels[mtSearch].Visible := False
       else if (MessageHandler.ValidationState(mtSearch) > 0) and not (moError in SearchOptions) then
         FPanels[mtSearch].Visible := False
       else if (MessageHandler.MessageKind(mtSearch) = moConfirm) and not (moConfirm in SearchOptions) then
         FPanels[mtSearch].Visible := False;

       if FPanels[mtProgress].Visible and (FPanels[mtProgress].Count > 0) and FProgressActive then
       begin
          FPanels[mtSave].Visible := False;
          FPanels[mtEdit].Visible := False;
          FPanels[mtDataset].Visible := False;
          Exit;
       end;
       if (MessageHandler.ValidationState(mtSave) >= 0) and FPanels[mtSave].Visible then
       begin
          FPanels[mtSearch].Visible := False;
       end
       else if (MessageHandler.ValidationState(mtEdit) >= 0) and FPanels[mtEdit].Visible then
       begin
          FPanels[mtSave].Visible := FPanels[mtSave].Visible
           and (MessageHandler.MessageKind(mtSave) = moConfirm);

          FPanels[mtSearch].Visible := FPanels[mtSearch].Visible
          and (MessageHandler.ValidationState(mtEdit) = 0) and (MessageHandler.MessageKind(mtSearch) = moConfirm);
       end
       else if (MessageHandler.ValidationState(mtSearch) > 0) and FPanels[mtSearch].Visible then
       begin
         if (moError in SearchOptions) and (MessageHandler.ValidationState(mtSearch) > 0)
          { Ready is ignored  for search}
         then
         begin
            FPanels[mtSave].Visible := False;
            FPanels[mtEdit].Visible := False;
         end;
       end;

       Repaint;
   end;

  function TILMessageBar.GetTable : TILTable;
  begin
    if csDesigning in ComponentState then
      Result := FTable
    else if FTable = nil then
      Result := Tman.ActiveTable
    else
      Result := FTable;
  end;

  function TILMessageBar.GetMessages(Index : TMessageType) : TMessageInfo;
  begin
    Result.IsOwnText := false;
    Result.DatasetIndex := Table.Dataset.Position;
    Result.MessagetType := Index;
    Result.MessageKind := MessageHandler.MessageKind(Index);
    Result.Text := FPanels[Index].Text;
    Result.State := MessageHandler.ValidationState(Index);
    Result.Glyph := FPanels[Index].Glyph;
    Result.SelectedCount := Table.Dataset.SelCount;
    Result.Count := MessageHandler.ConfirmCount(Index);
    Result.SQLOperation := FPanels[Index].SQLOperation;
    Result.Visible := FPanels[Index].Visible;
  end;

  function TILMessageBar.GetSQLMode : Boolean;
  begin
    Result := MessageHandler.SQLMode;
  end;

  procedure TILMessageBar.SetSQLMode(Value : Boolean);
  begin
    if Value <> MessageHandler.SQLMode then
    begin
     MessageHandler.SQLMode := Value;
    end;
  end;

  procedure TILMessageBar.SetGlyphs(Value : TImageList);
  begin
    FGlyphs := Value;
    LoadGlyphs;
  end;

  procedure TILMessageBar.SetUpdateOptions(Value : TMessageOptions);
  begin
     if Value = FUpdateOptions then
       Exit;
     if moSelCount in Value then
      Value := Value - [moSelCount];
     FUpdateOptions := Value;
  end;

  procedure TILMessageBar.SetSaveOptions(Value : TMessageOptions);
  begin
     if Value = FSaveOptions then
       Exit;
     if moSelCount in Value then
      Value := Value - [moSelCount];
     FSaveOptions := Value;
  end;

  procedure TILMessageBar.SetDatasetOptions(Value : TMessageOptions);
  begin
     if Value = FDatasetOptions then
       Exit;
     if moReady in Value then
      Value := Value - [moReady];
     if moError in Value then
      Value := Value - [moError];
     FDatasetOptions := Value;
  end;

  procedure TILMessageBar.SetSearchOptions(Value : TMessageOptions);
  begin
     if Value = FSearchOptions then
       Exit;
     if moReady in Value then
      Value := Value - [moReady];
     if moSelCount in Value then
      Value := Value - [moSelCount];
      FSearchOptions := Value;
  end;

  procedure TILMessageBar.SetSuppressMissingValue(Value : Boolean);
  begin
    if Value = FSuppressMissingValue then
      Exit;
    FSuppressMissingValue := Value;
    if Value then
    begin
     MessageHandler.AddToFilter(mtSave, moError, E_MandatoryValue);
     MessageHandler.AddToFilter(mtEdit, moError, E_MandatoryValue);
    end
    else
    begin
     MessageHandler.RemoveFromFilter(mtSave, moError, E_MandatoryValue);
     MessageHandler.RemoveFromFilter(mtEdit, moError, E_MandatoryValue);
    end;

  end;

  procedure TILMessageBar.SetOwnText(Value : string);
  begin
    FOwnText := Value;
    Invalidate;
  end;

  procedure TILMessageBar.SetTable(Value : TilTable);
  begin
    if Value = FTable then
      Exit;
    FTable := Value;
    MessageHandler.Table := Value;

  end;

  procedure TILMessageBar.SetTrafficLights(Value : Boolean);
  begin
     if FTrafficLights = Value then
       Exit;
     FTrafficLights := Value;
     Invalidate;
  end;

  function TILMessageBar.MessageRect(AMessage : TMessageType) : TRect;
  var
   i : TMessageType;
   x1, x2 : integer;
  begin
     X1 := 0;
     X2 := 0;
     Result := Rect(0,0,0,0);
     for I := mtDataset to mtProgress do
     begin
       if FPanels[i].Visible then
       begin
          x2 := x2 + FPanels[i].Width;
          if i = AMessage then
          begin
            Result := ClientRect;
            Result.Left := x1;
            Result.Right := x2;
          end
          else
            x1 := x2;
       end;
     end;
  end;

  function TILMessageBar.GlyphRect(AMessage : TmessageType) : TRect;
  begin
    Result := Rect(0,0,0,0);
    Result := MessageRect(AMessage);
     if not Result.IsEmpty then
     begin
       Result.Left := Result.Left + 3;
       Result.Top := 1;
       Result.Right := Result.Left + 16;
       Result.Bottom := Result.Top + 16;
     end;
  end;

  function TILMessageBar.OwnTextRect : TRect;
  var
    I : TmessageType;
    X : integer;
  begin
    Result := Rect(0,0,0,0);
    if OwnText = '' then
      Exit;
    X := 0;
    for I := mtDataset to MtSave do
    begin
      if Fpanels[i].Visible then
       inc(X, FPanels[i].Width);
    end;
    Result.Left := X +3;
    Result.Top := 0;
    Result.Bottom := ClientHeight;
    Result.Right := ClientWidth;
  end;

  function TILMessageBar.TextRect(AMessage : TMessageType) : TRect;
  begin
     Result := MessageRect(AMessage);
     if not Result.IsEmpty then
     begin
       inc(Result.Left, 19);
       inc(Result.Top, 2);
       Result.Right := result.Left + FPanels[AMessage].Width - 21;
       Result.Bottom := Result.Bottom-2;
     end;
  end;


end.

