{**************************************************************************** }
{ Version 1.0.1 2021 }
{ Author: Oivind Muller }
{ }
{ This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
{ }
{ **************************************************************************** }


{ Notes ***********************************************************************

  The ImageLibrary code is extensive and comments are sporadic. However,
  here we outline some essential methods and concepts that might be helpful in
  understanding the logic behind the component:

  TILTable
  ========
  The TILTable compontent and its ancestors TILDataTable, TILTreeViewTable,
  TILImageTable and TILLookupTable are the backbone of the system. All database
  operations are executed by calling methods in these compponents. To make the
  API more readale the table methods are subdivided into three namespaces:

  DBView: Contains methods for manipulating table data
  DBWork: Contains methods for searching tables
  DBCommons: Contains methods that are common for both DBView and DBWork.

  Methods in these namespaces, classes actually, are mainly wrappers around
  the core methods found in TILTable.

  There is also a DBAdmin name space. This handles administrative tasks
  and is mostly used the ILib designer app.

  Connecting and loading
  ======================
  Accessing the database requires the system to be an an idle state. That means
  that the metadata of all the tables must have been successfuly streamned
  from the database description file, <TableName>_Meta.XML.

  As metadata is not stored in dfm files, the system must wait for the application
  forms to be loaded, until it can start the streaming process.

  The streaming takes place in the DoLoaded method of the table manager. This is
  initialized by the overriden Loaded method of the table manager which posts a
  WM_Loaded message to itself.

  When executing, DOLoaded calls the method SyncTable for each table found in
  the DFM. SyncTable sets table metedata properties and creates TILFieldDef
  objects according to the field definitions in the meta file.

  The DoLoaded method also performs consitency checks and initializing procdures.

  At the end it sets an IsLoade flag and calls the OnLoaded event handler
  if present.

  The search and update engines
  =============================
  The work hores behind searching and manipulating is the TSearcher and TUpdater
  classes. On start up ILIB, for each table, creates the Searcher object which
  is available throghout a session. The Searcher object is responsible
  for populating the Dataset array. Besides mainitaining the global Searcher
  object ILib consecutively creates and disposes of local searcher objects
  for internal operations.

  The search and update engines relies on input data from SQL or data aware
  controls or by explicit commands from UI functions channeled throught the
  DB... methods.

  When data comes from the data aware controls, both enginges call a method
  named ReadControls that transfers the values to a TILData record. This is
  in turn analyzed by a method named RecordToExpression which translates the
  update / search specifications to a format that can be processed by
  the engines. The same transformation does the SQL parser method
  SQLToExpression.

  The search engine
  -----------------
  The search engine searches records by two methods: If one or more search fields
  are indexed it calls a method named GetFromIndexes. If there is a mix of
  indexed and none indexed search fields, GetFromIndexes grabs what it can find,
  narrowing the search space as much as possible, and leaves it up to the other
  method, the scanner, do do the rest.

  ILib loads index tables into memeory on start up, calling the
  TILTable.LoadIndexes method. They are stored in two TList objects:
  FLinkList and FKeyList.

  The update engine
  -----------------
  TUpdater objects are only created when needed. It handles all the DB operation
  types: update, insert and delete. The method to call is Update. The component
  decieds which operation to implement by parsing the input data (SQL, the values
  of the data controls or other parameters)

  Master detail processing
  =========================
  For each table ILIb creates an instance of the TILFocusHandler class. Besides
  executing the OnFocusRecord event handler, this class is responsible for
  building the master detail chain. The focus handlers cooperate in this matter;
  when one record gets the focus, the handler executes queries that
  bring up the child or master records relating to that record. The related
  records that now get the focus in their turn execute the same process,  etc.
  Fucus handlers are private fields of TILTable: FFocHandler. See also
  Master / detail processing in the help file.

  Saving and transaction handling
  ===============================
  The Save method of DBWork and the SaveAll method of the table manager (calls
  DBWork.Save for each changed table dataset) ultimately calls the Dataset.Save
  method. This may adress different other methods depending on the kind of
  operation, but all ends with the TILTable.SetRecords or the
  TILTable.AddToTable methods. These actually write the changed or inserted
  records to the transaction log by calling the TTableManager.AddToTransLog method.

  Writing as a concequence of SQL execution are handled by the Update method of
  the update engine, but this too finally leads to SetRecords or AddToTable.

  The update stack
  ----------------
  The table that initiates a transaction is given the state as Transaction owner.
  The owner is pushed to a log called an update stack (TTablemanager.FUpdateStack).
  Any additional updates that come from responding to the OnTrigger event is
  likewise added to the stack. When a table is updated the stack is popped.
  Only when it reaches zero the actual physical write takes place.
  AddToUpdateStack and RemoveFromUpdatestack are the methods that maintain the
  stack.

  If you have created a persistent transacion with BEGINTRANS the physical
  write does not take place until COMMITTRANS is called, independant of the state
  of the update stack.

  Physical write
  --------------
  When a transaction is secured, the actual update of the database file starts.
  The table manager uses the SetTransState method to keep track of transaction
  states, if they are pending, committed or rolled back. Calling SetTransState
  with the committed paramater, executes the RunTransactions method. This
  reads the transasction log and writes the transes to the database. At the end
  of the process, if changes involves indexed fields, the index tables for that table
  are also written to the file, calling the SaveIndexes method.

  Both LoadIndexes and SaveIndexes run in their own threads.

  You find a schematic overview of the transaction system in the topic
  Handling transactons in the file.

  The database file
  =================
  Operations on the database file are controlled by a class named
  TFileContainer. This class stores data in sub files (TSubFile) which
  in turn organizes the data in units called records.

  On start up the table manager craetes an instance of this class under the name
  FLibContainer.

  New records are added to the end of the sub files and can be identified by their
  numeric position in the file, the index. The Items property of TSubFile allows
  reading and writing records using this index.

  A TILTable maintains four sub files in the database file:
  One file for the record data: (<Tablename>_data)
  Two files for the field indexes: <TableName>_KeyLinks / _KeyList
  One file for the record index: <Tablename>_Data_IndexTable
  The last one is used to lookup the physical file position of a specific record index.

  When ILib loads the database it creates objects for these sub files:

  FSF_Data connects to the data sub file
  FSF_FKeyList connects to the KeyLinks sub file
  FSF_FLinkList connects to the KeyList sub file

  The data index table does not have a corresponding object as it is handleded
  interally by the file container.

  Alle reads and writes from and to the database are channelled through these pipelines.
}

{$Define UseIndy}
{$UnDef DLLAccess}

{Undefine UseIndy to exclude indy components from the package. This will deprive
ILib from downloading images from the internet

The DLLAccess directive must be on when compiling for ILib.DLL. Do not change.}

Unit ImageLibrary;
interface
uses
  System.SysUtils, Vcl.Graphics, Windows, System.Classes, System.Types,
  Vcl.Controls, Vcl.StdCtrls, Vcl.ExtCtrls, CommCtrl, System.UITypes,
  VCL.Checklst, Vcl.ComCtrls, Messages, Xml.XMLDoc, Xml.XMLIntf,
  Vcl.Imaging.JPEG, ActiveX, ShlObj, PropertyProvider,
  {$IfDef UseIndy}
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  {$EndIf}
  VCL.Grids, VCL.Forms, VCL.Buttons, IniFiles;

const
  WM_UpdateRptColumns = wm_user + 200;
  WM_Loaded = wm_user + 203;
  WM_RefreshTreeView = wm_user + 204;
  WM_UpdatePanel = wm_user + 205;
  WM_UpdateSupporter = wm_user + 206;
  WM_RunFocus = wm_user + 207;
  WM_SetResEditBtn = 208;
  WM_AlignLogOps = wm_user + 2 + 3;
  WM_Lookups = wm_user + 210;
  WM_ShowValueCombo = wm_user + 211;
  WM_ShowActionCombo = wm_User + 212;
  WM_UpdateWorkList = wm_user + 214;

  tag_Set = '[Set]';
  tag_Add = '[Add]';
  tag_Delete = '[Delete]';
  tag_Replace = '[Replace]';
  tag_Value = '[Value]';
  tag_Action = '[Action]';

{$EXTERNALSYM IID_IThumbnailProvider}
  IID_IThumbnailProvider: TGUID = '{e357fccd-a995-4576-b01f-234630154e96}';

  MED_NONE = 0;
  MED_IMAGE = 1;
  MED_VIDEO = 2;
  MED_AUDIO = 3;

  c_LogAnd = '&';
  c_LogOr = '|';
  c_ResourceSeparator = '@';
  c_ResourceStrSeparator = #1;
  NegSign = '!';
  MaxFloat = 9999999999;
  EmptyValue = 'EmptyValue';

  { XML tags: }
  XRec = 'Rc';
  XSystem = 'Sys';
  XRecIndex = 'In';
  XRecID = 'Id';
  XRegDate = 'RDt';
  XChangeDate = 'CDt';
  XRecState = 'St';
  XImgOffset = 'IOf';
  XImgSize = 'ISz';
  XImgFile = 'Ifl';
  XMedia = 'Med';
  XData = 'Dat';
  XValue = 'Vl';
  XUserID = 'UI';

  { Transaction states }
  TrPending = 0;
  TrCommitted = 2;
  TrCommittedActive = 3;
  TrRolledback = 4;
  TrPendingActive = 5;

  CLIPBOARD_IMAGE = '<CLIPBOARD_IMAGE>';
  IMAGE_TABLE = '<ImageTable>'; { Used as file name when image references an image table}
  c_PageBase = 65535; {Base pagesize for paged record layout }

type
{$EXTERNALSYM IThumbnailProvider}
  IThumbnailProvider = interface(IUnknown)
    ['{e357fccd-a995-4576-b01f-234630154e96}']
    function GetThumbnail(cx: uint; out hBitmap: hBitmap; out bitmapType: dword)
      : HRESULT; stdcall;
  end;

  ILException = class(Exception)
  private
    FErrorCode : integer;
  public
    property ErrorCode : integer read FErrorCode;
  end;

  { Used by translator: }
  TCaps = record
    Name : string;
    Translation : string;
  end;
  TTranslateCaps = array of TCaps;

  { Forward class declarations}
  TILColumns = class;
  TILComboBox = class;
  TILControlPanel = class;
  TILCustomGrid = class;
  TILData = class;
  TILDataField = class;
  TILDataRecord = class;
  TILDataset = class;
  TILDatasetViewer = class;
  TILDataTable = class;
  TILEdit = class;
  TILField = class;
  TILFieldDef = class;
  TILFieldDefs = class;
  TILImage = class;
  TILImageTable = class;
  TILLogOpButton = class;
  TILLookupTable = class;
  TILMessageHandler = class;
  TILPropertyList = class;
  TILPropertyValue = class;
  TILPropertyValues = class;
  TILRecord = class;
  TILRecordViewer = class;
  TILReplicator = class;
  TILReport = class;
  TILReportCell = class;
  TILReportRow = class;
  TSelectFields = class;
  TILSQL = class;
  TILSQLParam = class;
  TILSQLParams = class;
  TILMemTable = class;
  TILTable = class;
  TILTreeViewParams = class;
  TILTreeViewTable = class;
  TILView = class;
  TILWorkField = class;
  TSubFile = class;
  TTableManager = class;
  TUpdater = class;
  TChangeTracker = class;

  { Enum types }
  TAggFunc = (agCount, agMax, agMin, agAvg, agSum, agNone);
  TAggFuncs = set of TAggFunc;
  TAlert = (alReadOnly, alRollback, alLoading, alMulSelNoEdit, alMulSelNoSelect);
  TAlterFieldOption = (afIdentifier, afDecimals, afMaxLength, afMaxValues, afIndexed, afIndexPriority,
   afMandatory, afUnique, afReportField, afReportCaption, afColumnWidth, afImagetext,
   afInitialValue, afDateFormat, afAlignment);
  TCommandOption = (ovSearchAnd, ovSearchOr, ovUpdateData, ovUpdateImage, ovSortAsc, ovSortDesc,
   ovDefault, ovSearchRoot, ovInsertSiblingA, ovInsertChild, ovInsertSiblingB, ovSortSiblings, ovUpArrow, ovDownArrow);
  {Use by the wsCommandMode work style, defines options used with commands }
  TBaseCommandOption = ovSearchAnd .. ovDefault;
  TCommandMode = (cmSearch, cmUpdate, cmSort, cmColumns, cmSQL);
  TCommandModes = set of TCommandMode;
  TCommandOptions = set of TCommandOption;
  TControlType = (ctUndefined, ctSearchControls, ctDataControls);
   { Used by TILControlPanel to define what kind od controls to generate}
  TDatasetChange = (dsDataUpdated, dsImageUpdated, dsInserted, dsDeleted, dsReverted, dsCleared, dsPopulated, dsSaved);
  TDataType = (dtString, dtText, dtNumber, dtBoolean, dtDate, dtResource, dtLongText);
  TDataUsage = (duSearch, duData);
  TDateSearchFormat = (dsfDate, dsfDateTime);
  TDBOperation = (dboUpdate, dboInsert, dboDelete, dboSave, dboSearch, dboCancelUpdate, dboNone); { Defines a database operation.
   Used by numerous functions}
  TDualCtrlOption = (doCoPicklist, doLogPickList);
  TDualCtrlOptions = set of TDualCtrlOption;
  TFieldLogOp = (flAnd, flOr);
  TFieldType = (ftUserData, ftSystemData, ftCalculated, ftAutoIncrement, ftStoredEvaluation);
  TFileType = (iftBMP, iftJPG, iftPng, iftGif, iftTiff, iftWPD, iftAVI, iftWMV, iftMP4, iftMOV, iftMPG,
   iftOGV, iftMP3, {$IfDef UseIndy} iftURL, {$EndIf} iftUnKnown);
  TFilterState = (fsStandard, fsUseDACs, fsUseFilter); {Detail record filtering}
  TILOperation = (iloSearch, iloUpdate, iloDelete, iloRemoveDeleted, iloRecreateImages, iloProcessTransLog,
   iloRecreateTextLobs, iloAlterIndexSpace, iloAlterRecordSize, iloExtractIndexes, iloBuildIndexes, iloAlterTableSpace,
   iloAlterRecordFreeSpace, iloAlterRecordLayout, iloReadFile, iloChangeOperation, iloFinished);
  {TILOperatrion is used by the progress alert feature. Notifies the kind of operation going on.
  It is also used internally to define an alter db operation}
  TImageLayout = (ilStripHorz, ilStripVert, ilGrid);
  TImageLocalSize = (ils0, ils100, ils200, ils300, ils400, ils500, ilsFullSize); { Size of images stored in the database}
  TImageOption = (ioWrapPartialVisibles); { DatasetViewer Image mode. Only this option}
  TImageOptions = set of TImageOption;
  TImageSource = (isNoImages, isFile, isClipBoard, isFileOrClipBoard, isImageTable, isManual);
  {Defines from which source the system will look fore an image. In TILData}
  TIndexPriority = (ipLow, ipIndefinite, ipHigh);
  TIndexSearchState = (issComplete, issExclude, issInclude, issScan);
  {Used by the indexing system (TSearcher.GetFromIndexes) Tells the resulting state of an indexed search}
  TJPEGQuality = (jqHigh, jqGood, jqLow, jqDraft, jqAuto);
  TLogOp = (loAnd, loOr);
  TLogOpType = (ltFieldAligned, ltField, ltValue);
  TLongTextAction = (ltaUpload, ltaDownload, ltaClearEditors, ltaEnable, ltaDisable, ltaSampleText);
  TLongTextActions = set of TLongTextAction;
  TLookupType = (looNone, looHistory, looTableRef, looTableKey, looPickValues, looParentField);
  TLTPageSize = (ltp1KB, ltp2KB, ltp4KB, ltp8KB, ltp16KB, ltp32KB); {Defines LongText page size}
  TMDOption = (mdLookupChildren, mdLookupParents, mdDenyChild, mdDenyParent); {Master / detail processing options}
  TMDOptions = set of TMDOption;
  { QuickBuilder modes }
  TMessageOption = (moError, moReady, moConfirm, moSelCount, moNotAvailable);
  TMessageType = (mtDataset, mtSearch, mtEdit, mtSave, mtProgress);
  TQuickMode = (qbCreateDB, qbCreateChildTable, qbAlter);
  TRecordElement = (reData, reFilename, reMediaProps, reRegDate, reChangeDate, reRecordIndex, reRecordID,
   reLinkID, reState, reUserID);
  { Used by the TILRecordViewer to filter elemnts to display}
  TRecordElements = set of TRecordElement;
  TRecordLayout = (rlVariableSize, rlFixedSize, rlPaged); {Record storing options}
  TRectType = (rtItemRect, rtTextRect, rtImageRect, rtCheckRect, rtCaptionRect, rtExpandBtnRect, rtPlayerIconRect);
  { Defines rectangles of special elements in the dataset viewer}
  TRelations = (relMany, relOne); {Record count restrictions of a table}
  TReportGraphics = (rgRecordImage, rgFieldImage, rgCheckBox, rgUpdateMarker, rgNone);
  { Used by the InfoFromPos function that reports thh graphic element at a certain position}
  TReportImages = (riNone, riHover, riHoverCtrl, riEmbed); {Image display flags in DatasetViewer, report mode}
  TReportImageSize = (risSmall, risMedium, risLarge); {Size of images in DV report mode}
  TReportOption = (roAutoExpand, roColumnClickSort, roOutlineExpansion, roGridlines, roEditable,
   roShowExpandButtons, roShowFieldThumbnails, roAutoAppend, roSuppressRepeatingValues);
  TReportOptions = set of TReportOption;
  TSearchCtrlMode = (scmWhere, scmStartWith); {TILTreeViewParams options}
  TSearchModel = (smAndAligned, smAndOrAligned, smAndOrMixed);
  TSelectFieldType = (stField, stCalc, stFunction, stAlien); {Type of select fields. Used by the search engine}
  TSortOrdering = (soAscending, soDescending);
  TSQLType = (ctSelect, ctUpdate, ctInsert, ctDelete, ctUnknown);
  TSQLTypes = set of TSQLType;
  TStyleTarget = (stBackground, stFont, stFontHot, stFontCaption, stHotTrack,stFocused, stGridlines,
   stCheckbox, stCheckBoxBorder, stImageBorder,    stActiveColHeading, stInactiveColHeading,
   stCaption, stInactiveCaption,  stColPressed, stBorder, stSelCol, stHotCol);
  {Style colors}
  TTreeViewOption = ovSearchRoot .. ovSortSiblings;
  {Use by the wsCommandMode work style, defines options used with commands in connections with TILreeView tables}
  TTreeViewOptions = set of TTreeViewOption;
  TTreeViewSorting = (tsBySiblings, tsByRecords);
  TTVInsertMode = (timRoot, timChild, timSiblingBefore, timSiblingAfter);
  {Tells how new records are inserted in a TILTreeeView table}
  TUpdateGridMode = (gmSimple, gmUpdateActions);
  TUpdateMarkers = (upmNone, upmChanged, upmPreviewChange, upmBoth); {Update markers in DatasetViewer}
  TUpdateMethod = (umReplaceAll, umReplaceFields, umAddValue, umRemoveValue, umReplaceValue, umExpression, umNone);
  TUpdateTarget = (utrDataSet, utrTable);
  TUpdateType = dboUpdate .. dboInsert;
  TValueLogOp = (vlAnd, vlOr, vlAndAll);
  TViewMode = (vmImages, vmReport);
  TWorkFieldCreator = (wcPanel, wcManager);
  TWorkStyle = (wsCommandModes, wsOther);

  TReportInfo = record
    Row: TILReportRow;
    Cell: TILReportCell;
    DataRecord: TILDataRecord;
    Graphics: TReportGraphics;
  end;

  { Event types }
  TActivateTableEvent = procedure(Sender: TObject; Table : TILTable; var Cancel: Boolean)of object;
  TAfterDrawImageItemEvent = procedure(Sender: TObject; Canvas: TCanvas; DataRecord: TILDataRecord;
   const ItemRect, TextRect: TRect) of object;
  TAlertEvent = procedure(Sender: TObject; const Alert : TAlert) of object;
  TCalcFieldEvent = procedure(Sender: TObject; Data: TILDataRecord; Field: TILField) of object;
  TCellEvent = procedure(Sender: TObject; Cell: TILReportCell) of object;
  TChangeDACEvent = procedure(Sender : TObject; DataField : TILDataField; Usage : TDataUsage; Value : string) of object;
  TClickCellEvent = procedure(Sender: TObject; Cell: TILReportCell; const ImageClicked : boolean) of object;
  TClickImageEvent = procedure(Sender : TObject; ARecord : TILDataRecord; const X, Y : integer) of Object;
  TClickReportEvent = procedure(Sender: TObject; const Info: TReportInfo) of object;
  TDatasetChangeEvent  = procedure(Sender : TObject; const Change : TDatasetChange) of Object;
  TDrawImageItemEvent = procedure(Sender: TObject; Canvas: TCanvas; DataRecord: TILDataRecord; ItemRect, TextRect:
   TRect; var Accepted: Boolean) of object;
  TDrawLogOpButtonEvent = procedure(Sender : TILLogOpButton; Canvas : TCanvas; var Handled : Boolean) of object;
  TDrawReportCellEvent = procedure(Sender: TObject; Font: TFont; Cell: TILReportCell; var Value: string) of object;
  TDrawReportCellExEvent = procedure(Sender: TObject; Canvas : TCanvas; Cell: TILReportCell; var Value: string) of object;
  TDrawReportRowEvent = procedure(Sender: TObject; Canvas: TCanvas; Row: TILReportRow; var Accepted: Boolean) of object;
  TDrawUpdateMarkerEvent = procedure(Sender: TObject; const Marker: TUpdateMarkers; Canvas: TCanvas; ItemRect:
   TRect; var Handled: Boolean) of object;
  TEvaluateFieldEvent = procedure(Sender : TObject; ADataRecord : TILDataRecord; const AFieldName : string;
   const ASearchValue : string; var Accepted : Boolean) of object;
  TExecResourceEvent = procedure(Sender: TObject; const Resource: string; var Handled: Boolean) of object;
  TFieldEvent = procedure(Sender: TObject; Field: TILField) of object;
  TFilterEvent = procedure(Sender: TObject; Params : TILSQLParams) of object;
  TFocusEvent = procedure(Sender: TObject; const FromIndex, ToIndex: integer; var Allow: Boolean) of object;
  TImageTableEvent = procedure(Sender: TObject; var ImageRecordIndex : integer) of object;
  TIndexWordEvent = procedure(Sender: TObject; const WordToIndex: string; var Accept: Boolean) of object;
  TItemEvent = procedure(Sender: TObject; const Index: integer) of object;
  TLinkEvent = procedure(Sender: TObject; ParentTable : TILTable; var ParentRecordIndex : integer) of object;
  TLongTextEvent = procedure(Sender: TObject; Field: TILField; var SampleText : string; const Actions : TLongTextActions) of object;
  TMDLookupEvent = procedure(Sender: TObject; TargetTable : TILDataTable; var Accept : Boolean) of object;
  TMessageTextEvent = procedure(Sender: TObject; const AMessageType : TMessageType;
   var AMessageText : string; var Handled : Boolean) of object;
  TProgressEvent = procedure(Sender: TObject; const Operation: TILOperation; Count, Progress: integer) of object;
  TPropertyEvent = procedure(Sender: TObject; Prop: TILPropertyValue; var FormattedValue: string) of object;
  TRecordEvent = procedure(Sender: TObject; Operation : TDBOperation; Rec: TILDataRecord) of object;
  TSearchEvent = procedure(Sender: TObject; Data: TILDataRecord; var Accepted: Boolean) of object;
  TSearchLongTextEvent = procedure(Sender: TObject; Field: TILField; const SearchText : string; var Found : Boolean) of object;
  TSQLTextEvent = procedure(Sender : TObject; var SQLText : string) of object;
  TStateImageEvent = procedure(Sender : TObject; const Index : integer; var StateIndex : integer) of object;
  TUnsavedEvent = procedure(Sender: TObject; var Save, Cancel: Boolean) of object;
  TUpdateImageEvent = procedure(Sender: TObject; var Image : TPicture; var FileName : string) of object;
  TWorkFieldEvent =  procedure(Sender: TObject; Workfield : TILWorkField; var Accept : Boolean) of object;
  TWorkListEvent = procedure(Sender: TObject; ControlPanel : TILControlPanel; var Handled : Boolean) of object;

  {Defining the header fields in the FileContainer file:}
  TFileSpecEx = record
    FileName: shortstring;
    FileSize: int64;
    FileSpace: int64;
    RecordSpace: integer;
    FixedStringSize: Boolean;
    Sorted: Boolean;
    Duplicates: TDuplicates;
    Offset: int64;
    IndexTable: Boolean;
  end;
  { Used internnally }

  TFileSpec = record
    FileName: shortstring;
    FileSpace: int64;
    RecordSpace: integer;
    FixedStringSize: Boolean;
    Sorted: Boolean;
    Duplicates: TDuplicates;
  end;
  { The callers structure }

  TKey = class
    Key : string;
    ID : integer;
  end;

  { Main database file }
  TFileContainer = class
  private
    FFileName: string;
    FFiles: array of TSubFile;
    FMaxFiles: integer;
    FStream: TFileStream;
    FUnicode : Boolean;
    FBusy : Boolean;
    function GetCount: integer;
    function GetFiles(Index: integer): TSubFile;
    procedure Grow(const FromPosition: int64; const ByteCount: integer);
  public
    constructor Create(const FileName: string; const MaxFiles: integer; const ReadOnly: Boolean);
    destructor Destroy; override;
    function Add(FileSpec: TFileSpec): TSubFile;
    function IndexOf(const FileName: shortstring): integer;
    function IsEmpty : Boolean;
    function ReadHeader(const Index: integer): TFileSpecEx;
    procedure WriteHeader(const Index: integer; Info: TFileSpecEx);
    property Busy : Boolean read FBusy;
    property Count: integer read GetCount;
    property Files[Index: integer]: TSubFile read GetFiles;
    property Stream: TFileStream read FStream;
    property Unicode : Boolean read FUnicode write FUnicode;
  end;

  { File within the file container }
  TSubFile = class
  private
    FDeleteCount: integer;
    FFileName: shortstring;
    FFileSpec: TFileSpecEx;
    FIndexTable: TSubFile;
    FInsertCount: integer;
    FOwner: TFileContainer;
    FPaged : Boolean;
    FPageN : Boolean; { Signals a new page. See SetPagedRecord}
    FPosition: int64;
    FStream: TFileStream;
    function AddStream(Source: TStream) : int64;
    function GetAbsItems(Index : integer) : string;
    function GetCount: int64;
    function GetDuplicates: TDuplicates;
    function GetFixedStringSize: Boolean;
    function GetFromTransLog(Index : integer) : string;
    function GetIndex: integer;
    function GetIsIndexTable: Boolean;
    function GetItems(Index: integer): string;
    function GetOffset: int64; { Beginning of data }
    function GetPagedRecord(const PagePos : integer) : string;
    function GetPosition: int64;
    function GetRecordSpace: integer; { Space occupied by one record, migth be variable }
    function GetSize: int64; { Size occupied by data }
    function GetSorted: Boolean;
    function GetSpace: int64; { Total space available }
    function GetSS(const Value : string) : TStringStream;
    function PageSize : integer;
    function RealIndex(const SourceIndex : integer) : integer;
    function SetPagedRecord(const Value : string; PagePos : integer) : integer;
    procedure DeleteStream(const FromPosition: int64; const ByteCount: int64);
    procedure DoInsert(const Index: integer; const S: string);
    procedure InsertStream(Source: TStream; const AtPosition: int64);
    procedure MoveDown(const FromPosition: int64; const ByteCount: int64);
    procedure MoveUp(const FromPosition: int64; const ByteCount: int64);
    procedure RenumberIndex(AnIndexTable: TSubFile; const FromIndex: integer; const ByteCount: integer);
    procedure SetItems(Index: integer; Value: string);
    procedure SetPosition(const Value : int64);
    procedure SetSize(const Value: int64);
    procedure SetSorted(const Value : Boolean);
  public
    constructor Create(const FileName: shortstring; OwnerFile: TFileContainer);
    function Add(const S: string; FreeSpace : integer = 0): integer;
    function Find(const AString: string; var Index: integer): Boolean;
    function HasSpace(const Index : Integer; const Arecord : string) : integer;
    function PageCount(Index : integer) : integer;
    function PositionOfRecord(const Index: integer): int64;
    function SizeOfData(const Index: integer): integer;
    function SizeOfRecord(const Index: integer): integer;
    function StringToStream(const AString: string; FreeSpace : integer = 0): TMemoryStream;
    procedure Clear;
    procedure Delete(const Index: integer);
    procedure Insert(const Index: integer; const S: string);
    procedure ReplaceAll(AStream : TMemoryStream);
    procedure SaveToStream(const Index: integer; Stream: TMemoryStream);
    procedure Truncate(const NewFileSpace: integer);
    property AbsItems[Index : integer] : string read GetAbsItems;
    property Count: int64 read GetCount;
    property Duplicates: TDuplicates read GetDuplicates;
    property FixedStringSize: Boolean read GetFixedStringSize;
    property Index: integer read GetIndex;
    property IsIndexTable: Boolean read GetIsIndexTable;
    property Items[Index: integer]: string read GetItems  write SetItems; default;
    property Offset: int64 read GetOffset;
    property Paged : Boolean read FPaged;
    property Position: int64 read GetPosition write SetPosition;
    property RecordSpace: integer read GetRecordSpace;
    property Size: int64 read GetSize write SetSize;
    property Sorted: Boolean read GetSorted write SetSorted;
    property Space: int64 read GetSpace;
  end;

  { Node in table tree hierachy}
  TTableNode = class
  private
    FTable: TILTable;
    FTree: TTableManager;
    function GetAllChildren(Index: integer): TTableNode;
    function GetChildren(Index: integer): TTableNode;
    function GetCount: integer;
    function GetCountAll: integer;
    function GetLevel: integer;
    function GetParent: TTableNode;
  public
    constructor Create(ATable: TILDataTable; ATree: TTableManager);
    property AllChildren[Index: integer]: TTableNode read GetAllChildren;
    property Children[Index: integer]: TTableNode read GetChildren;
    property Count: integer read GetCount;
    property CountAll: integer read GetCountAll;
    property Level: integer read GetLevel;
    property Parent: TTableNode read GetParent; { Children and grand children etc }
    property Table: TILTable read FTable;
  end;

  TILTransLogItem = class
   private
    FOldData: string;
    FRecordIndex : integer;
    FTableName : string;
    FTrans : string;
    FTransID : Integer;
    FTransType : TDBOperation;
   public
    property OldData : string read FOldData write FOldData;
    property RecordIndex : integer read FRecordIndex write FRecordIndex;
    property TableName : string read FTableName write FTableName;
    property Trans : string read FTrans write FTrans;
    property TransID : integer read FTransID write FTransID;
    property TransType : TDBOperation read FTransType write FTransType;
  end;

  TTreeDataNode = record
    Dataset: TILDataset;
    Table: TILDataTable;
  end;

  TTransPos = class { Used by FtranslogIndex }
    Operation : TDBOperation;
    Position : integer;
    Size : integer;
    TableName : string;
  end;

  TTreeData = array of TTreeDataNode;
  TTreeTables = array of TILDataTable;
  TFreeTables = array of  TILTable;

  { The elements of Workfields history buffer }
  TWorkHistoryElement = class
  private
     FCmd : TCommandMode;
     FTbl : TILTable;
     FValues : TILDataRecord;
     FWorkFields : TStringList;
  public
     destructor Destroy; override;
  end;

  { The Focus handler is the key to master detail processing. Executes the OnFocusRecord event}
  TFocusHandler = class
  private
    FDataSet : TILDataset;
    FFocRecord : TILDataRecord;
    FHasExecuted : Boolean;
    FOriginator : TFocusHandler;
    FSender : TFocusHandler;
    FTable : TILTable;
    FUptracing, FDownTracing : Boolean;
    function AcceptTarget(UseParent : Boolean; ATarget : TILDataTable) : Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    function InDownList : Boolean;
    function InUplist(ARecord : TILDatarecord) : TILDatarecord;
    procedure ClearAll;
    procedure ClearDownList;
    procedure ClearUpList;
    procedure CreateDownList;
    procedure CreateUpList;
    procedure Execute(Adataset : TILDataset; APosition : integer);
    procedure RunHandler;
  end;

  TUpdateStackItem = record
    Table : TILTable;
    UpdateID : integer;
  end;

  TUpdateStack = array of TUpdateStackItem;

  { The update stack tracks if one and the same trigger is called with a different update id.
  Is so, the trigger chain forms a circular loop. The log is maintained by the different
  update procedurers and tested in AddToTransLog}

  TTableManager = class(TComponent)
  private
    {The FIsServer and FServer fields are used by ILIB to handle "clients" in a home network with shared folders.
    A client is a computer different from the one that stores the database, i.e. the "server". A client must
    keep a UNC refernece to the server database folder in the registry:

    HKEY_CURRENT_USER\Software\ImageLibrary\<DatabaseName>.Location

    For instance;
    Location = \\ThatComputer\ThatSharedDBFolder.

    When ILib loades the database, it reads the reference and decide if
    the machine is server or client by comparing computer name with the name in the registry.
    Sets FIsServer and copies the server name into FServer.

    The shared folders must be located on the C drive of the server machine.

    Running a client may generate a lot of net traffic and is particulary slow when handling large images.

    See the TOUnc, UNCFromFileName, IsUnc, and CollapseUNC procs and SetNames method of TableManager.
    }

    { Backup file names}
    _FLibPath : string;
    _FMetaName  : string;
    _FTableCfgName : string;
    _FTextLobsName : string;
    _FThumbName : string;
    _FTransLogName  : string;
    _FUserIniName  : string;

    {Temp file names, used with cleaning operaitons}
    _FTmpLibPath : string;
    _FTmpThumbName : string;
    _FTmpTextLobsName : string;
    _FTmpTransLogName : string;

    FActiveFocHandler : TFocusHandler;
    FAffectedTables : TList;
    FAliasFalse : string;
    FAliasNull : string;
    FAliasTrue : string;
    FAutoSave : Boolean;
    FAutoUpdate : Boolean;
    FCfgFile : TIniFile;
    FCheckmarkBM: Vcl.Graphics.TBitmap;
    FCommandMode : TCommandMode;
    FConnectSilent : Boolean;
    FCreatingTable : Boolean;
    FDatabase : string;
    FDisableTranslog : boolean;
    FDLLAccess : Boolean; {Caller is DLL}
    FExtDesigner : Boolean;  {Designer app running }
    FExternalMeta : Boolean;
    FFreeTables : TFreeTables;
    FGarbaging : Boolean; { True the RemoveGarbage is run }
    FHWnd : HWND;
    FIsDBEmpty : Boolean;
    FIsLoaded : Boolean;
    FIsReadOnly : Boolean;
    FIsServer: Boolean;
    FLibContainer : TFileContainer;
    FLibPath : string;
    FLocation: string;
    FMainThreadBusy : Boolean; { True when main thread accesses db file}
    FMetaFileStamp : TDateTime;
    FMetaName : string;
    FNodes: TList;
    FOnActivateTable: TNotifyEvent;
    FOnActivation : TActivateTableEvent;
    FOnAlert : TAlertEvent;
    FOnChangeCommandMode : TNotifyEvent;
    FOnChangeWorkFields : TNotifyEvent;
    FOnClickPlayerIcon : TClickImageEvent;
    FOnClickWorkList : TWorkListEvent;
    FOnGetSQLText : TSQLTextEvent;
    FOnLoaded: TNotifyEvent;
    FOnProgress : TProgressEvent;
    FOnUnsaved : TUnsavedEvent;
    FPlayerIcon : TPicture;
    FPanelBuild : Boolean;
    FPanels : TList;
    FIntPlayerBM: Vcl.Graphics.TBitmap;
    FIntPlayerIcon: Vcl.Graphics.TIcon;
    { The following two regulate how often the OnProgress event is fired }
    FProgressFreq : integer;
    FProgressTicker : longword;
    FRollbackName : string;
    FRootTable: TILDataTable;
    FSaveOnDelete : Boolean;
    FSavingAll : Boolean;
    FSearchFieldOrder: String;
    FSearchModel: TSearchModel;
    FServer : string;
    FSharedImages: TILImageTable;
    FSkipValidation : Boolean;
    FSystemBusy : Boolean; { True when TIL indexer threads run }
    FTableBM: Vcl.Graphics.TBitmap;
    FTableCfgName : string;
    FTableList: TListBox;
    FTableUnderMouse : TILTable;
    FTextLobs : TFileStream;
    FTextLobsName : string;
    FThumbName : string;
    FThumbs : TFileStream;
    FTransactionActive : Boolean;
    FTransHistory : Boolean;
    FTransID : integer;
    FTransLogAge : Integer;
    FTransLogBuffer : TMemoryStream;
    FTransLogFile : TFileStream;
    FTransLogIndex : TStringList;
    FTransLogName : string;
    FTransOwner : TILTable;
    FTreeTables: TTreeTables;
    FUnicode : Boolean;
    FUpdateIDCounter : integer;
    FUpdater : TUpdater;
    FUpdateStack : TUpdateStack;
    FUserID : string;
    FUserIniName : string;
    FWorkFields : TStringList;
    FWorkHistory : TList;
    FWorkList : TCheckListBox;
    FWorkStyle : TWorkStyle;

    function Buble : integer;
    function CheckUnsaved(ATable : TILTable) : Boolean;
    function EncodeTrans(TblName : string; RecIndex : integer; TrType : TDBOperation;
     OldData, TrData : string; TransID : integer) : string;
    function GetActiveRecord : TILDataRecord;
    function GetActiveTable: TILTable;
    function GetAutoSave : Boolean;
    function GetAutoUpdate : Boolean;
    function GetConnected: Boolean;
    function GetDatabase : string;
    function GetFreeTableCount : integer;
    function GetFreeTables(Index : integer) : TILTable;
    function GetFromHistory(Tbl : TILTable; const Cmd : TCommandMode) : TWorkHistoryElement;
    function GetItems(Index: integer): TTableNode;
    function GetLocation: string;
    function GetPanelCount : integer;
    function GetPanels(Index : integer) : TILControlPanel;
    function GetReportFalse : string;
    function GetReportNull : string;
    function GetReportTrue : string;
    function GetRoot: TTableNode;
    function GetSaveOnDelete : Boolean;
    function GetTableCount : integer;
    function GetTableNodeCount: integer;
    function GetTables(Index : integer) : TILTable; overload;
    function GetTables(TableName : string) : TILTable; overload;
    function GetTreeTableCount: integer;
    function GetWorkFieldCount : integer;
    function GetWorkFields(Index : integer) : TILWorkField;
    function HasUserFields(ATable : TILTable) : Boolean;
    function IsValueNull(AValue : string) : Boolean;
    function LogOpButtonOfControl(AControl : TWinControl; const LogOpType : TLogOpType) : TILLogOpButton;
    function TablesOfMeta : TStringList;
    procedure AddToTransLog(TblName : string; RecIndex : integer;
      TrType : TDBOperation; OldData, TrData : string; DataRec : TilDatarecord);
    procedure AddToUpdateStack(ATable : TILTable; ID : integer);
    procedure CheckDanglingHandlers;
    procedure ClearTranslog;
    procedure CloseFiles;
    procedure ConnectSilent;
    procedure CreateChildTable(AParentTable, AChildTable: TILDataTable);
    procedure CreateFiles;
    procedure CreateFreeTable(ATable : TIlTable);
    procedure CreateLookupTable(ALookupTable: TILLookupTable);
    procedure CreateSearchFieldOrder;
    procedure DoCreateTable(ATable : TILTable);
    procedure DoLoadIndexes;
    procedure DoShowWorkFields(APanel : TILControlPanel);
    procedure EnablePanel(APanel : TILControlPanel; Enabl : Boolean);
    procedure EnablePanels;
    procedure FindFreeTables;
    procedure HandleTransError(E : Exception);
    procedure InvalidateMemTables(ATable : TILMemTable = nil);
    procedure LoadIndexes;
    procedure MakeBackups(Option : integer);
    procedure OpenFiles;
    procedure RegisterFreeTable(ATable : TILTable);
    procedure RemoveFromUpdateStack;
    procedure RemoveWorkField(ADataField : TILDataField);
    procedure RestoreFromBackups(Option : integer);
    procedure RunTransactions(ChangedTables : TList);
    procedure SaveIndexes;
    procedure SaveToHistory(Tbl : TILTable; const Cmd : TCommandMode; WorkFields : TStringList);
    procedure SaveTranslog;
    procedure SetActiveTable(const Value : TILTable);
    procedure SetAliasFalse(const Value : string);
    procedure SetAliasNull(const Value : string);
    procedure SetAliasTrue(const Value : string);
    procedure SetCommandMode(const Value : TCommandMode);
    procedure SetConnected(const Value : Boolean);
    procedure SetDatabase(const Value : string);
    procedure SetIntPlayerIcon(AnIcon : TPicture);
    procedure SetLocation(const Value : string);
    procedure SetPlayerIcon(Value : TPicture);
    procedure SetSearchModel(const Value : TSearchModel);
    procedure SetTableList(const Value : TListBox);
    procedure SetTransLogAge(Value : integer);
    procedure SetUnicode(const Value : Boolean);
    procedure SetUserID(const Value : string);
    procedure SetWorkList(const Value : TCheckListBox);
    procedure SetWorkStyle(const Value : TWorkStyle);
    procedure TreeViewClick(Sender: TObject);
    procedure UnregisterFreeTable(ATable : TILTable);
    procedure UpdateWorkList;
    procedure WndMethod(var Msg : TMessage);
    procedure WorkListClick(Sender: TObject);
    procedure WriteBrowser(APanel : TILControlPanel);

    {Aliases. See AliasFalse, etc}
    property ReportFalse: string read GetReportFalse;
    property ReportTrue: string read GetReportTrue;
    property ReportNull : string read GetReportNull;
    {SaveOnDelete is always false. Saves updates may be unintentionally}
    property SaveOnDelete : Boolean read GetSaveOnDelete write FSaveOnDelete default false;

  protected
    procedure Add(ATable: TILDataTable);
    procedure ChangeWorkFields; dynamic;
    procedure Clear;
    procedure Delete(ATable: TILTable);
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Progress(Table : TObject; const Operation : TILOperation; const Count, Progress : integer); dynamic;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CanExecCommand(const ACommand : TCommandMode; const AnOption : TBaseCommandOption) : integer;
    function CanMoveWorkField(const FromIndex, ToIndex : integer) : Boolean;
    function CanSaveAll(var ErrorTable : TILTable; var ErrorIndex : integer) : integer;
    function CheckDupDB(ADBName : string) : Boolean;
    function CheckDupTable(ATableName : string) : Boolean;
    function CheckScript(const AFileName : string; var ErrorIndex : integer) : integer; overload;
    function CheckScript(Statements : TStrings; var ErrorIndex : integer) : integer; overload;
    function CheckStates : TILTable;
    function ChildrenChanged(ParentTable : TILDataTable) : integer;
    function CleanUp(ATable : TILTable) : integer;
    function CommandFromSQL(SQL: TStrings): TSQLType;
    function DefaultColWidth(FieldDef : TILFieldDef) : integer;
    function ExecCommand(const AnOption : TBaseCommandOption) : integer;
    function ExecScript(const AFileName : string; var ErrIndex : integer) : integer; overload;
    function ExecScript(Statements : TStrings; var ErrIndex : integer) : integer; overload;
    function ExecSQL(SQL : TStrings; ActivateTable : Boolean = false) : integer;
    function FindMissingTables : integer;
    function FindPanel(ATable : TILTable; ACommand : TCommandMode) : TILControlPanel;
    function FreeTableOf(const TableName: string): TILTable;
    function GetMBarFromTable(ATable : TILTable) : TObject;
    function GetMessageHandler(ATable : TILTable; var LastIndex : integer): TILMessageHandler;
    function GetProgressText(Operation : TILOperation) : string;
    function GetTransState(var TrID : integer) : integer;
    function IsChild(AParent, ATable: TILTable): Boolean;
    function IsInLinkedPath(Table1, Table2 : TilTable; var Master : TILTable) : Boolean;
    function IsInPath(Table1, Table2: TILTable): Boolean;
    function IsInSearchPath(Table1, Table2: TILTable): Boolean;
    function IsLinkedChild(AParentTable, ATestTable : TILDataTable) : Boolean; overload;
    function IsLinkedChild(ATable : TILDataTable) : Boolean; overload;
    function IsRelated(AParent, ATable: TILTable): Boolean;
    function IsTableRegistered(ATable : TILTable) : Boolean;
    function LoadResource(const ResName : string) : VCL.Graphics.TBitmap;
    function LoadResourceIcon(const ResName : string) : VCL.Graphics.TIcon;
    function LoadTransLog(FileName : string = '') : TStringList;
    function MoveWorkField(const FromIndex: Integer; const ToIndex: Integer) : Boolean;
    function NodeOf(ATable: TILTable): TTableNode;
    function OptionInCommand(const ACommand : TCommandMode; const Option : TBaseCommandOption) : Boolean;
    function RollbackTrans : integer;
    function SaveAll: integer;
    function SyncTable(ATable : TILTable) : string;
    function TableFromSQL(SQL : Tstrings) : TILTable;
    function TableIndex(ATable : TILTable) : integer;
    function TableIndexOf(ATable : TILTable) : integer;
    function TableOf(const TableName: string): TILTable;
    function TreeTableOf(const TableName: string): TILDataTable;
    function TruncateTranslog(ToDate : TDateTime) : integer;
    function UseCommandModes : Boolean;
    function WorkFieldOf(AControl : TWinControl) : TILWorkField;
    procedure AddInheritedFields(ATable : TILTable);
    procedure BeginTrans;
    procedure ClearData; { Clear all data components. Called by designmer app }
    procedure ClearDatabase;
    procedure ClearDatasets;
    procedure ClearTableLists;
    procedure ClearTableTree;
    procedure CommitTrans;
    procedure CreateDatabase;
    procedure CreateDBFromMeta(const Metafile : string);
    procedure CreateNodes;
    procedure CreateTable(AParentTable, AChildTable : TILDataTable); overload;
    procedure CreateTable(ATable : TILTable); overload;
    procedure CreateTableFromMeta(const TableName : string; const Metafile : string);
    procedure CreateWorkFields(const ATable : TILTable; const CommandMode : TCommandMode);
    procedure DoLoaded;
    procedure InitWorkFields;
    procedure LaunchQuickBuilder(const Operation : TQuickMode);
    procedure MoveDatabase(const OldDir, NewDir : string);
    procedure PopulateTableList(AList: TListBox);
    procedure RebuildAllIndexes;
    procedure RecreateDatbase;
    procedure RecreateWorkFields;
    procedure RemoveGarbage;
    procedure RenameDatabase(const OldName, NewName : string);
    procedure ResolveWorklist;
    procedure SetNames;
    procedure SetTransState(State : integer);
    procedure SyncDetailLinks(ATable : TILTable);
    procedure Synchronize;
    procedure SyncRelations(ATable : TILTable);
    procedure UpdateColumnOrder;
    procedure UpdateSortOrder;

    property ActiveRecord : TILDataRecord read GetActiveRecord;
    property ActiveTable: TILTable read GetActiveTable write SetActiveTable;
    property CommandMode : TCommandMode read FCommandMode write SetCommandMode;
    property Connected: Boolean read GetConnected write SetConnected;
    property DLLAccess : Boolean read FDLLAccess write FDLLAccess;
    property ExtDesigner : Boolean read FExtDesigner write FExtDesigner; { True when external designer app i running}
    property ExternalMeta : Boolean read FExternalMeta write FExternalMeta;
     { True when schema created by external designer app. Prevents saving metdata }
    property FreeTableCount : integer read GetFreeTableCount;
    property FreeTables[Index : integer]: TILTable read GetFreeTables;
    property Lib : TFileContainer read FLibContainer;
    property Location: string read GetLocation write SetLocation;
    property MetaFileStamp : TDateTime read FMetaFilestamp;
    property MetaName : string read FMetaName;
    property PanelCount : integer read GetPanelCount;
    property Panels[Index : integer] : TILControlPanel read GetPanels;
    property Root: TTableNode read GetRoot;
    property RootTable: TILDataTable read FRootTable;
    property SearchFieldOrder: string read FSearchFieldOrder write FSearchFieldOrder;
    property SharedImages: TILImageTable read FSharedImages;
    property TableCount : integer read GetTableCount;
    property TableNodeCount: integer read GetTableNodeCount;
    property TableNodes[Index: integer]: TTableNode read GetItems;
    property Tables[Index : integer] : TILTable read GetTables; default;
    property Tables[TableName : string] : TILTable read GetTables; default;
    property TransactionActive : Boolean read FTransactionActive;
    property TransHistory : Boolean read FTransHistory;
    property TransLogBuffer : TMemoryStream read FTransLogBuffer;
    property TransLogIndex : TStringlist read FTransLogIndex;
    property TranslogName : string read FTranslogName;
    property TreeTableCount: integer read GetTreeTableCount;
    property TreeTables: TTreeTables read FTreeTables write FTreeTables;
    property Unicode : Boolean read FUnicode write SetUnicode;
    property WorkFieldCount : integer read GetWorkFieldCount;
    property WorkFields[Index : integer] : TILWorkField read GetWorkFields;

  published
    property AliasFalse : string read FAliasFalse write SetAliasFalse;
    property AliasNull : string read FAliasNull write SetAliasNull;
    property AliasTrue : string read FAliasTrue write SetAliasTrue;
    property AutoSave : Boolean read GetAutoSave write FAutoSave default false;
    property AutoUpdate : Boolean read GetAutoUpdate write FAutoUpdate default false;
    property Database : string read GetDatabase write SetDatabase;
    property IsReadOnly : Boolean read FIsReadOnly write FIsReadOnly default false;
    property OnActivateTable: TNotifyEvent read FOnActivateTable write FOnActivateTable;
    property OnActivation : TActivateTableEvent read FOnActivation write FOnActivation;
    property OnAlert : TAlertEvent read FOnAlert write FOnAlert;
    property OnChangeCommandMode : TNotifyEvent read FOnChangeCommandMode write FOnChangeCommandMode;
    property OnChangeWorkFields : TNotifyEvent read FOnChangeWorkFields write FOnChangeWorkFields;
    property OnClickPlayerIcon : TClickImageEvent read FOnClickPlayerIcon write FOnClickPlayerIcon;
    property OnClickWorkList : TWorkListEvent read FOnClickWorkList write FOnClickWorkList;
    property OnGetSQLText : TSQLTextEvent read FOnGetSQLText write FOnGetSQLText;
    property OnLoaded: TNotifyEvent read FOnLoaded write FOnLoaded;
    property OnProgress : TProgressEvent read FOnProgress write FOnProgress;
    property OnUnsaved : TUnsavedEvent read FOnUnsaved write FOnUnsaved;
    property PlayerIcon : TPicture read FPlayerIcon write SetPlayerIcon;
    property ProgressFreq : integer read FProgressFreq write FProgressFreq default 0;
    property SearchModel: TSearchModel read FSearchModel write SetSearchModel default smAndAligned;
    property TableList: TListBox read FTableList write SetTableList;
    property TransLogAge : integer read FTranslogAge write FTransLogAge default 0;
    property UserID : string read FUserID write SetUserID;
    property WorkList: TChecklistBox read FWorkList write SetWorkList;
    property WorkStyle : TWorkStyle read FWorkStyle write SetWorkStyle default wsCommandModes;
  end;

  { Record for scan search. Stores info about evaluation results }
  TFieldHit = record
    CountAnd: integer; { Number of times the field is and-referenced }
    CountOr: integer; { Number of times the field is or-referenced }
    FName: string; { Field to hit }
    HitAll: Boolean; { True if the co == is applied }
    HitCountAnd: integer; { Number of and hits for partial ands }
    HitCountOr: integer; { Number of or hits }
  end;

  { Base class for TSearcher and TUpdater }
  TExpressionHandler = class
  private
    FChecking: Boolean;
    FDecomps: array [0 .. 29] of TStringList;
    FExpression: TStringList;
    FPrepared: Boolean;
    FPrivate : Boolean;
    FSections: array [0 .. 2] of TStringList;
    Ilib: TILTable;
    function GetSections: integer; virtual; abstract;
    procedure SetExpression(const Value : TStringList);
  public
    constructor Create(TableOwner: TILTable; AnExpression: TStringList); overload;
    constructor Create(TableOwner: TILTable; ARecord: TILRecord); overload;
    destructor Destroy; override;
    function DecomposeElement(AnElement: string; Decomp: TStringList) : integer; virtual;
    function Execute(ADataRecord: TILDataRecord): Boolean; virtual;
    function Prepare: integer; virtual;
    function RecordToExpression(ARecord: TILRecord): TStringList; virtual;
    function TrimOperators(AnExpression: string): string; virtual;
    function Validate: integer; virtual;
    property Expression: TStringList read FExpression write SetExpression;
  end;

  TTablePath = array [0 .. 20] of TILTable;  { Used the TSearcher to check search paths }

  { Search engine }
  TSearcher = class(TExpressionHandler)
  private
    FColumnSource : TILColumns;
    FFetchCount: integer;
    FFetchRecords: TStringList;
    FFieldSource: TILFieldDefs;
    FIndexSearchState: TIndexSearchState;
    FMaxCount: integer;
    FResumeIndex: integer; { The next item that should be fetched }
    FResumeSearchType: integer; { 0 for regular search, 1 for index search }
    FRootCondition: TStringList;
    FSelectFields: TSelectFields;
    FTableCount: integer;
    FTablePath: TTablePath; { Collects the tables involved. Checks for out of path }
    FTreeViewing: Boolean;
    FTVParamsSource : TILTreeViewParams;

    function GetFromIndexes(var State: TIndexSearchState; RootDataset: TILDataset = nil): TStringList;
    function GetRemainingRecords: integer;
    function GetSections: integer; override;
    function InPath(ParentID, ChildID: integer; IncludeRoot: Boolean): Boolean;
    function Join(var OutData : TILDataset) : integer;
    function Search(OutDataset: TILDataset; AMaxCount: integer;  SkipImages: Boolean = false): TILDataset; overload;
    function SplitTableField(FieldName: string; var TblName, FldName: string) : TILTable;
  public
    destructor Destroy; override;
    function CanResume: Boolean;
    function DecomposeElement(AnElement: string; Decomp: TStringList) : integer; override;
    function Execute(ADataRecord: TILDataRecord): Boolean; override;
    function Prepare: integer; override;
    function RecordToExpression(ARecord: TILRecord): TStringList; override;
    function Resume(Dataset: TILDataset): Boolean;
    function Search : TILDataset; overload;
    function TrimOperators(AnExpression: string): string; override;
    property RemainingRecords: integer read GetRemainingRecords;
    property TreeViewing: Boolean read FTreeViewing write FTreeViewing;
   property MaxCount: integer read FMaxCount write FMaxCount;
  end;

  { Update engine }
  TUpdater = class(TExpressionHandler)
  private
    FError: integer;
    FInsertIntoRecord: Boolean; { True if insertion should result in a stand alone record }
    FInsertRecord: TILDataRecord; { Gets a pointer when FInsertIntoRecord is true }
    FOperation: TDBOperation; { Update, insert delete }
    FUpdateMethod: TUpdateMethod;
    FUpdateTarget: TUpdateTarget; { Dataset or table }
    FRecordIndex : integer; { Used by DLL UpdateRecord, set ny DBWork.UpdateTable}

    function GetSections: integer; override;
  public
    constructor Create(TableOwner: TILTable; ARecord: TILRecord); overload;
    destructor Destroy; override;
    function DecomposeElement(AnElement: string; Decomp: TStringList) : integer; override;
    function Execute(ADataRecord: TILDataRecord): Boolean; override;
    function Prepare: integer; override;
    function RecordToExpression(ARecord: TILRecord): TStringList; override;
    function TrimOperators(AnExpression: string): string; override;
    function Update: integer;
    property InsertRecord: TILDataRecord read FInsertRecord;
    property UpdateTarget: TUpdateTarget read FUpdateTarget write FUpdateTarget;
  end;

  { TSelectField and TSelectFields: Classes for handling fields in select statements }
  TSelectField = class
  private
    FAggregateField: string;
    FAggregateFunc: TAggFunc;
    FError: integer;
    FFieldDef: TILFieldDef;
    FFieldName: string;
    FFieldSource: TILFieldDefs;
    FFieldType: TSelectFieldType;
    FFormula: TStringList;
    FOwner: TSelectFields;
    FTableName : string;
    Ilib: TILTable;

    function GetFieldsFromExpression(const Expression: string): TStringList;
    function HasFieldType(const FieldString: string; const DataType: TDataType): Boolean;
    procedure ParseField(FieldString: string);
  public
    constructor Create(Owner: TSelectFields; FieldString: string; FieldSource: TILFieldDefs);
    destructor Destroy; override;
    function CalcValue(Data: TILDataRecord): string;
    property Error: integer read FError;
    property FieldName: string read FFieldName;
    property FieldType: TSelectFieldType read FFieldType;
    property TableName : string read FTableName;
  end;

  TSelectFields = class
  private
    FAggFuncs: TAggFuncs;
    FAggVals: array [agCount .. agSum] of Extended;
    FBuilding: Boolean;
    FOwner: TSearcher;
    FSelectFields: TList;
    Ilib: TILTable;
    FError: integer;
    function GetAlienCount : integer;
    function GetAliens(Index : integer) : TSelectField;
    function GetCount: integer;
    function GetItems(Index: integer): TSelectField;
    function IsLiterals: Boolean;
  public
    constructor Create(Owner: TSearcher; Fields: TStringList; FieldSource: TILFieldDefs);
    destructor Destroy; override;
    function IndexOfCalcField(const FieldName: string): integer;
    procedure Clear;
    property AlienCount : integer read GetAlienCount;
    property Aliens[Index : integer] : TSelectfield read GetAliens;
    property Count: integer read GetCount;
    property Items[Index: integer]: TSelectField read GetItems;

  end;

  { Used by TestUpdate and TestUpdateDataset which tests the contenet of data controls to
  detect if one or more records in Dataset needs updating. See also NeedsUdate
  and NeedsUpdateDataset.}
  TILTestResult = class
  private
    FAdds: TStringList;
    FDeletes: TStringList;
    FRecordIndex: integer;
  public
    constructor Create;
    destructor Destroy; override;
    property RecordIndex: integer read FRecordIndex;
    property Adds: TStringList read FAdds;
    property Deletes: TStringList read FDeletes;
  end;

  TILUpdateTest = class
  private
   FResults: TList;
   function GetCount: integer;
   function GetItem(Index: integer): TILTestResult;
   procedure SetItem(Index: integer; Value: TILTestResult);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(ATestResult: TILTestResult);
    property Count: integer read GetCount;
    property Items[Index: integer]: TILTestResult read GetItem write SetItem;
  end;

  { The record fields }
  TILField = class
  private
    FExpression: TStringList;
    FFieldDefs: TILFieldDefs;
    FFieldName: string;
    FIsCalculated : Boolean;
    FLongText : string;
    FLTStream : TMemoryStream;
    FLookupRecordIndex : integer; { If field is lookups store the record index in the lookup table}
    FOldValues: TStringList;
    FOwnerRecord: TILRecord;
    FResFilePos : integer; { Position of resource string(s) in text lobs }
    FResourceStrings : TStringList; { Preserves resource strings for late restore }
    FSuppressOutput : Boolean; {Suppresses output in data set viewer columns}
    FTextSourceChanged : Boolean; { True when text in editor has changed. Set by user in the OnUpdateLongtext event }
    FTextUpdated : Boolean; { True when long text is uploaded from the editor. Set by user in the OnUpdateLongtext event}
    FUsage: TDataUsage;
    FValues: TStringList;

    Ilib: TILTable;
    function DecodeTableKey(SearchVal : string) : string;
    function GetAsBoolean: Boolean;
    function GetAsDate: TDate;
    function GetAsDateTime: TDateTime;
    function GetAsInteger: integer;
    function GetAsNumber: Real;
    function GetAsStream : TMemoryStream;
    function GetAsString: string;
    function GetAsText: string;
    function GetAsValues: TStringList;
    function GetCount: integer;
    function GetDisplayValue : string;
    function GetFieldDef: TILFieldDef;
    function GetFieldName : string;
    function GetIndex: integer;
    function GetItems(Index: integer): string;
    function GetLookupSourceRecord : TILDataRecord;
    function GetSampleLongText(const MaxLength : integer) : string;
    function GetTextLob(const Position : integer) : string;
    function SaveLongText(const Value : string ; const Position : Integer) : Integer; overload;
    function SaveLongTextStream(const Value : TStream ; const Position : Integer;
     AFile : TFileStream = nil) : Integer; overload;
    procedure GetStreamedTextLob(const Position : integer; TheStream : TMemoryStream);
    procedure ResetLTFlags;
    procedure SetAsBoolean(const Value : Boolean);
    procedure SetAsDate(const Value : TDate);
    procedure SetAsDateTime(const Value : TDateTime);
    procedure SetAsInteger(const Value : integer);
    procedure SetAsNumber(const Value : Real);
    procedure SetAsStream(const Value : TMemoryStream);
    procedure SetAsString(const Value : string);
    procedure SetAsText(const Value : string);
    procedure SetAsValues(const Value : TStringList);
    procedure SetOldState;
    procedure SetTextSourceChanged(Value : Boolean);
    procedure UpdateImageText;
  public
    constructor Create(TableOwner: TILTable; AFieldName: string; OwnerFields: TILFieldDefs);
    destructor Destroy; override;
    function IndexOf(const Value: string): integer;
    function IsEmpty: Boolean;
    function IsNull : Boolean;
    function LongTextSize : Integer;
    function SaveLongText : Integer; overload;
    function SaveLongTextStream : Integer; overload;
    procedure Change(Sender: TObject);
    procedure Clear;
    procedure SaveLT;
    procedure SetNullValue;

    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsDate: TDate read GetAsDate write SetAsDate;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsInteger: integer read GetAsInteger write SetAsInteger;
    property AsNumber: Real read GetAsNumber write SetAsNumber;
    property AsStream : TMemoryStream read GetAsStream write SetAsStream;
    property AsString: string read GetAsString write SetAsString;
    property AsText: string read GetAsText write SetAsText;
    property AsValues: TStringList read GetAsValues write SetAsValues;
    property Count: integer read GetCount;
    property DisplayValue : string read GetDisplayValue; {Gets the string value any data type. For long texts only sample text}
    property Expression: TStringList read FExpression write FExpression;
    property FieldDef: TILFieldDef read GetFieldDef;
    property FieldName: string read GetFieldName;
    property Index: integer read GetIndex;
    property Items[Index: integer]: string read GetItems; default;
    property LookupSourceRecord : TILDataRecord read GetLookupSourceRecord;
    property OwnerRecord: TILRecord read FOwnerRecord;
    property SuppressOutput : Boolean read FSuppressOutput write FSuppressOutput;
    property Table : TILTable read Ilib;
    property TextSourceChanged : Boolean read FTextSourceChanged write SetTextSourceChanged;
    property Usage: TDataUsage read FUsage;
  end;

  { Field meta data.
  The field definitions are stored in the meta file, generated by the app
  designer or the quick builder. When an application launches, the values are read from
  the meta descriptions, not from the dfm.}
  TILFieldDef = class(TCollectionItem)
  private
    FAlien : Boolean;
    FAlignment : TAlignment;
    FColumnWidth: integer;
    FDataControl: TWinControl;
    FDataLabel: TLabel;
    FDataType: TDataType;
    FDateFormat : string;
    FDCExit: TNotifyEvent;
    FDCKeyDown: TKeyEvent;
    FDecimals: integer;
    FFieldID : integer;
    FFieldLogOp: TFieldLogOp;
    FFieldName: string;
    FFields: TILFieldDefs;
    FFieldType: TFieldType;
    FFormula : string;
    FIdentifier: Boolean;
    FImageText: Boolean;
    FIndexed: Boolean;
    FIndexPriority: TIndexPriority;
    FInitialValue : string;
    FIsUpdated : Boolean;
    { Used to provide synchronization of lookup comboboxes.
    Prevents WriteControls to run GetLookups too many times. This flag is set in SetRecords
    and DoAddToTable and reset by WriteControls}
    FLookupField: string;
    FLookupSearchField : string;
    FLookupTable: TILTable;
    FLookupType : TLookupType;
    FLTPageSize : TLTPageSize;
    FMandatory: Boolean;
    FMaxLength: integer;
    FMaxValues: integer;
    FPickValues : TStringList;
    FReportCaption: string;
    FReportField: Boolean;
    FSCExit: TNotifyEvent;
    FSCKeyDown: TKeyEvent;
    FSCDataChange : TNotifyEvent;
    FSCSearchChange : TNotifyEvent;
    FSearchControl: TWinControl;
    FSearchLabel: TLabel;
    FTemporary: Boolean;
    FUnique : Boolean;
    FUserCalc : Boolean;
    FValueLogOp: TValueLogOp;
    Ilib: TILTable;

    function CalcUserCalc(DataRec : TILDataRecord) : string;
    function GetCaption: string;
    function GetFieldLogOp: TFieldLogOp;
    function GetInitialValue : string;
    function GetIsReadOnly : Boolean;
    function GetValueLogOp: TValueLogOp;
    procedure AssignPickValues(ACtrl : TWinControl);
    procedure PropagatePickValues(const Clear : Boolean);
    procedure SetDataControl(const Value: TWinControl);
    procedure SetDataLabel(const Value : TLabel);
    procedure SetDataType(Value : TDataType);
    procedure SetDecimals(Value : integer);
    procedure SetFieldLogOp(const Value : TFieldLogOp);
    procedure SetFieldName(const Value : string);
    procedure SetMaxValues(const Value : integer);
    procedure SetNextAutoValue(AValue : integer);
    procedure SetPickValues(const Value : TStringList);
    procedure SetSearchControl(const Value: TWinControl);
    procedure SetSearchLabel(const Value : TLabel);
  protected
    function GetDisplayName: string; override;
  public
   { For historical reasons the DACS are handled as field def properties. The published
   interface is to be found in the TILDataField class which mirrors and syncs the Dacs props.

   The same is true for the Alignment, ColumnWidth and ImageText properties, they are all
   mirrored by the TILDataField class }

    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    function GetNextAutoValue(Update : Boolean) : Integer;
    function IsLookup : Boolean;
    function IsMultiVal: Boolean;
    procedure Assign(Source: TPersistent); override;
    property Alignment : TAlignment read FAlignment;
    property Caption: string read GetCaption;
    property ColumnWidth: integer read FColumnWidth;
    property DataControl: TWinControl read FDataControl write SetDataControl;
    property DataLabel: TLabel read FDataLabel write SetDataLabel;
    property DateFormat : string read FDateFormat;
    property FieldID : integer read FFieldID;
    property FieldLogOp: TFieldLogOp read GetFieldLogOp write SetFieldLogOp;
    property FieldType: TFieldType read FFieldType;
    property Identifier: Boolean read FIdentifier;
    property ImageText: Boolean read FImageText;
    property Indexed: Boolean read FIndexed;
    property IndexPriority: TIndexPriority read FIndexPriority;
    property InitialValue : string read GetInitialValue;
    property IsReadOnly : Boolean read GetIsReadOnly;
    property LookupField: string read FLookupField;
    property LookupSearchField : string read FLookupSearchField;
    property LookupTable: TILTable read FLookupTable;
    property LookupType : TLookupType read FLookupType;
    property LTPageSize : TLTPageSize read FLTPageSize;
    property Mandatory: Boolean read FMandatory;
    property MaxLength: integer read FMaxLength;
    property PickValues : TStringList read FPickValues write SetPickValues;
    property ReportCaption: string read FReportCaption;
    property ReportField: Boolean read FReportField;
    property SearchControl: TWinControl read FSearchControl write SetSearchControl;
    property SearchLabel: TLabel read FSearchLabel write SetSearchLabel;
    property Table : TILTable read Ilib;
    property Unique : Boolean read FUnique;
    property ValueLogOp: TValueLogOp read GetValueLogOp write FValueLogOp;
  published
    property DataType: TDataType read FDataType write SetDataType default dtString;
    property Decimals: integer read FDecimals write SetDecimals default -1;
    property FieldName: string read FFieldName write SetFieldName;
    property MaxValues: integer read FMaxValues write SetMaxValues default 0;
  end;

  TILFieldDefs = class(TOwnedCollection)
  private
    FFieldLogOp: TFieldLogOp;
    function AddAlienField(Table : TilTable; SourceField: TILFieldDef): TILFieldDef;
    function AddCalcField(const FieldName: string): TILFieldDef;
    function CheckCtrlDataType(ACtrl: TWinControl; const DataType: TDataType; const Usage: TDataUsage): Boolean;
    function GetFieldLogOp: TFieldLogOp;
    function GetItem(AIndex: integer): TILFieldDef;
    function HasIndexes: Boolean;
    function HasLookups: Boolean;
    procedure ClearCalcFields(const SyncColumns: Boolean = True);
    procedure SetFieldLogOp(const Value : TFieldLogOp);
    procedure SetItem(AIndex: integer; const Value: TILFieldDef);

  protected
    procedure Notify(Item: TCollectionItem; Action: TCollectionNotification); override;
    procedure Update(Item: TCollectionItem); override;
  public
    function Add: TILFieldDef;
    function HasTempFields: Boolean;
    function IndexOf(const ItemName: string): integer;
    function IndexOfCaption(const ACaption: string): integer;
    function IndexOfControl(AControl: TWinControl): integer;
    function IndexOfID(const AnID : integer): integer;
    function IndexOfLabel(ALabel: Tlabel): integer;
    procedure Assign(Source: TPersistent); override;
    property FieldLogOp: TFieldLogOp read GetFieldLogOp write SetFieldLogOp;
    property Items[AIndex: integer]: TILFieldDef read GetItem write SetItem;
  end;

  TILDataFields = class(TOwnedCollection)
  private
    function GetItem(AIndex: integer): TILDataField;
    function GetTable : TILTable;
    procedure SetItem(AIndex: integer; const Value: TILDataField);
  protected
    procedure Notify(Item: TCollectionItem;
      Action: TCollectionNotification); override;
    procedure Update(Item: TCollectionItem); override;
  public
    function Add: TILDataField;
    function IndexOf(const FieldName: string): integer;
    procedure Assign(Source: TPersistent); override;
    property Items[AIndex: integer]: TILDataField read GetItem write SetItem;
    property Table : TILTable read GetTable;
  end;

  { The items of the Datafields list of TILData. TILData exposes table
  fields and methods for the user }
  TILDataField = class(TCollectionItem)
  private
    FAlignment : TAlignment;
    FCaption : string;
    FColumnVisible : Boolean;
    FColumnWidth : integer;
    FDacSearchVisible : Boolean;
    FDacUpdateVisible : Boolean;
    FDataControl : TWinControl;
    FDataFields: TILDataFields;
    FDataLabel : TLabel;
    FFieldName: string;
    FImageTag : Boolean;
    FSearchControl : TWinControl;
    FSearchLabel : TLabel;
    FUseAsImageText : Boolean;
    FOrigLabelFont : TFont;
    function GetAlignment : TAlignment;
    function GetColumnWidth : integer;
    function GetField : TILField;
    function GetFieldDef : TILFieldDef;
    function GetTable : TILTable;
    procedure CtrlChng(Sender : TObject);
    procedure SetAlignment(const Value : TAlignment);
    procedure SetCaption(const Value : string);
    procedure SetColumnVisible(const Value : Boolean);
    procedure SetColumnWidth(const Value : integer);
    procedure SetDacSearchVisible(Value : Boolean);
    procedure SetDacUpdateVisible(Value : Boolean);
    procedure SetDataControl(const Value: TWinControl);
    procedure SetDataLabel(const Value : TLabel);
    procedure SetFieldName(const Value: string);
    procedure SetImageTag(const Value : Boolean);
    procedure SetLookup(Ctrl : TWinControl);
    procedure SetSearchControl(const Value: TWinControl);
    procedure SetSearchLabel(const Value : TLabel);
    procedure SetUseAsImageText(const Value : Boolean);

  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Field : TILField read GetField;
    property FieldDef : TILFieldDef read GetFieldDef;
    property OrigLabelFont : TFont read FOrigLabelFont;
    property Table : TILTable read GetTable;
  published
    property Alignment : TAlignment read GetAlignment write SetAlignment;
    property Caption : string read FCaption write SetCaption;
    property ColumnVisible : Boolean read FColumnVisible write SetColumnVisible default true;
    property ColumnWidth : integer read GetColumnWidth write SetColumnWidth default 50;
    property DacSearchVisible : Boolean read FDacSearchVisible write SetDacSearchVisible default false;
    property DacUpdateVisible : Boolean read FDacUpdateVisible write SetDacUpdateVisible default true;
    property DataControl: TWinControl read FDataControl write SetDataControl;
    property DataLabel: TLabel read FDataLabel write SetDataLabel;
    property FieldName: string read FFieldName write SetFieldName;
    property ImageTag : Boolean read FImageTag write SetImageTag;
    property SearchControl: TWinControl read FSearchControl write SetSearchControl;
    property SearchLabel: TLabel read FSearchLabel write SetSearchLabel;
    property UseAsImageText: Boolean read FUseAsImageText write SetUseAsImageText default false;
  end;

  { The property list feature makes system fields and photo properties available for the user}

  TILPropertyItem = class(TCollectionItem)
  private
    FDataType: TDataType;
    FDisplayName: string;
    FPID: integer;
    FPIDName: string;
    FPropertyItems: TILPropertyList;
    procedure SetPIDName(const Value : string);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property DataType: TDataType read FDataType;
    property PID: integer read FPID;
  published
    property DisplayName: string read FDisplayName write FDisplayName;
    property PIDName: string read FPIDName write SetPIDName;
  end;

  TILPropertyList = class(TOwnedCollection)
  private
    function GetItem(AIndex: integer): TILPropertyItem;
    procedure SetItem(AIndex: integer; const Value: TILPropertyItem);
  protected
  public
    function Add: TILPropertyItem;
    function IndexOf(const ItemName: string): integer;
    function IndexOfPID(const PID: integer): integer;
    procedure Assign(Source: TPersistent); override;
    property Items[AIndex: integer]: TILPropertyItem read GetItem write SetItem;
  end;

  { Property values read property values as variants}

  TILPropertyValue = class
  private
    FDataType: TDataType;
    FOwner: TILPropertyValues;
    FPID: integer;
    FValue: Variant;
    function GetDisplayName: string;
    function GetDisplayValue: string;
  public
    property DataType: TDataType read FDataType;
    property DisplayName: string read GetDisplayName;
    property DisplayValue: string read GetDisplayValue;
    property PID: integer read FPID;
    property Value: Variant read FValue;
    { String representation of Value. If evnthandler OnFormat is not present, basicalley formatted }
  end;

  TILPropertyValues = class
  private
    FList: TList;
    Ilib: TILTable;
    function GetCount: integer;
    function GetProperty(Index: integer): TILPropertyValue;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(AProperty: TILPropertyValue);
    procedure Delete(const Index: integer);
    property Count: integer read GetCount;
    property Items[Index: integer]: TILPropertyValue read GetProperty;
  end;

  TILRecord = class
    { Base class for records }
  private
    FFields: TList;
    FIsDataControls : Boolean; {The record represents the values of the data controls. Used in validaiton}
    Ilib: TILTable;
    function GetActiveFieldSource : TILFieldDefs;
    function CalcQualified(const FieldIndex : integer) : Boolean;
    function GetCount: integer;
    function GetItems(FieldName: string): TILField; overload;
    function GetItems(Index: integer): TILField; overload;
    procedure SetItems(FieldName: string; Value: TILField); overload;
    procedure SetItems(Index: integer; Value: TILField); overload;
  public
    constructor Create(TableOwner: TILTable);
    destructor Destroy; override;
    function HasTempFields: Boolean;
    function IndexOf(const FieldName: string): integer;
    function IsDataCleared(All : Boolean): Boolean;
    function ValueOf(const AFieldName: string): TStringList;
    procedure Add(AValue: TILField);
    procedure Assign(Source: TILRecord);
    procedure Clear;
    procedure ClearData;
    procedure Delete(const Index: integer);
    property Count: integer read GetCount;
    property Items[FieldName: string]: TILField read GetItems write SetItems; default;
    property Items[Index: integer]: TILField read GetItems write SetItems; default;
    property Table : TILTable read Ilib;
  end;

  { Class for real life records, used in datasets}
  TILDataRecord = class(TILRecord)
  private
    FChangedDate: TDateTime;
    FDupRec : TILDataRecord;
    FDupRecID : integer;
    FEditingImageText: Boolean;
    FFileName: string;
    FImage: Vcl.Graphics.TBitmap;
    FImageChanged: Boolean;
    FImageFetched: Boolean;
    FImageText: string;
    FImgFilePos : integer;  { Position of file name in textlobs }
    FImgTableIndex: integer; { Image lookup table ref. }
    FIsDeleted : Boolean; {Unsaved record deleted }
    FMediaType: integer;
    FOldFileName: string;
    FOldImage: Vcl.Graphics.TBitmap;
    FOldImageText: string;
    FOldMediaType: integer;
    FOldState: string;
    FOldTableIndex: integer;
    FOwnerDataset: TILDataset;
    FOwnsImage: Boolean;
    FPropertyValues: TILPropertyValues;
    FRecordID : integer;
    FRecordIndex: integer;
    FRegDate: TDateTime;
    FSelected: Boolean;
    FSiblingList : TStringList; { Used with tree view tables to keep track of the
    sibling indexes when inserteing and deleting nodes. See also TILTreeViewTable FRootSiblings}
    FState: String;
    FThumbnailOffset: int64;
    FThumbnailSize: int64;
    FTVParentID: integer;  { Used with tree view tables}
    FTVParentRecord : TILDataRecord; { Used with tree view tables}
    FTVSiblingIndex: integer; { Used with tree view tables}
    FUndeleted : Boolean;
    FUserID : string;
    FValidState : integer;
    FValidText : string;
    function CheckDuplicates(Checking : Boolean) : integer;
    function GetChanged: Boolean;
    function GetColCount : integer;
    function GetColumns(Index : integer) : TILField;
    { Deletion flags:}
    function GetDeleted : Boolean;
    function GetDeletedMoved : Boolean; {Depricated }
    function GetDeletedTrans : Boolean;
    function GetDeletedUser : Boolean;
    function GetFileName : string;
    function GetImageText: string;
    function GetImageTextField: TILField;
    function GetIndex: integer;
    function GetTable : TILTable;
    function GetTextLob(const Position : integer) : string;
    function GetTVLevel : integer;
    function GetTVParentID : integer;
    function GetTVParentRecord : TILDataRecord;
    function GetTVSiblingIndex : integer;
    function MustStoreLocally(const AFileName: string): Boolean;
    function SaveFileResourceName(const Position : integer; AName : string; AFile : TFileStream = nil) : integer;
    procedure AssignSystemValues;
    procedure ConnectToMaster(ATVParentRecord: TILDataRecord); overload;
    procedure ConnectToMaster(const MasterID: integer); overload;
    procedure LoadImage(const AFileName: string);
    procedure LoadVideoFrame(const AFileName: string);
    procedure SetDeletedTrans(const Value : Boolean);
    procedure SetDeletedUser(const Value : Boolean);
    procedure SetImageText(const Value : string);
    procedure SetSelected(const Value : Boolean);
    procedure SetTVLevel(const Value : integer);
    procedure SetTVParentRecord(const Value : TILDataRecord);
    procedure SetTVSiblingIndex(const Value : integer);

  protected
    property ImageFetched: Boolean read FImageFetched;
  public
    constructor Create(TableOwner: TILTable; const CreateFields: Boolean);
    destructor Destroy; override;
    function CanPasteImage(var HasLink: Boolean): Boolean;
    function GetFromImageTable(const TagName: string) : Boolean;
    function HasLocalImage: Boolean;
    function IsNew: Boolean;
    function NeedsUpdate(Expression: TStrings): Boolean; overload;
    function NeedsUpdate: Boolean; overload;
    function Validate(const Checking: Boolean = false; const Tracking : integer = 0): integer; { Validates data before saving }
    function ValueOfPropField(const PID: integer): TStringList; overload;
    function ValueOfPropField(const PIDName: string): TStringList; overload;
    function ValueOfSysField(const ASysName: string): TStringList; overload;
    function ValueOfSysField(const ASysNameID: integer): TStringList; overload;
    procedure AddToImageTable(const TagName: string);
    procedure Assign(ASource: TILDataRecord);
    procedure AssignFields(ASource: TILRecord);
    procedure AssignMedia(ASource: TILDataRecord);
    procedure CalcFields;
    procedure CancelEditImageText;
    procedure CopyImage;
    procedure CopyImageLink;
    procedure CreateProperties(const AFileName: string);
    procedure EditImageText;
    procedure FetchImage; {Internal use }
    procedure InitFields(InitialValues: TILDataRecord; const IncludeTemps: Boolean = True);
    procedure LoadFileData(const AFileName: string);
    procedure LoadFromFile(const AFileName : string);
    procedure PasteImage;
    procedure PasteImageLink;
    procedure Refresh;
    procedure RemoveImage;
    procedure Save;
    procedure SaveToFile(const AFileName : string);
    procedure SetEditImageText;
    procedure UpdateImage(AnImage: TPicture); overload;
    procedure UpdateImage(AFileName: string); overload;
    property Changed: Boolean read GetChanged;
    property ChangedDate: TDateTime read FChangedDate write FChangedDate;
    property ColCount : integer read GetColCount;
    property Columns[Index : integer] : TILField read GetColumns;
    property Deleted : Boolean read GetDeleted;
    property DeletedMoved : Boolean read GetDeletedMoved;
    property DeletedTrans : Boolean read GetDeletedTrans write SetDeletedTrans;
    property DeletedUser : Boolean read GetDeletedUser write SetDeletedUser;
    property FileName: string read GetFileName write FFileName;
    { File names are stored as long texts. When records are saved, literal file names arising from inserts
    or updates are saved to textlobs file and stored in the record with a file pos ref.
    The GetFileName func looks up the file name from the textlobs file if it is pos refrenced,
    else grabs the literal name.}
    property Image: Vcl.Graphics.TBitmap read FImage write FImage;
    property ImageText: string read GetImageText write SetImageText;
    property ImageTextField: TILField read GetImageTextField;
    property ImgTableIndex : integer read FImgTableIndex;
    property Index: integer read GetIndex;
    property IsDeleted : Boolean read FIsDeleted;
    property MediaType: integer read FMediaType;
    property OwnerDataset: TILDataset read FOwnerDataset;
    property OwnsImage: Boolean read FOwnsImage write FOwnsImage;
    property PropertyValues: TILPropertyValues read FPropertyValues;
    property RecordID: integer read FRecordID;
    property RecordIndex: integer read FRecordIndex;
    property RegDate: TDateTime read FRegDate write FRegDate;
    property Selected: Boolean read FSelected write SetSelected;
    property Table : TILTable read GetTable;
    property ThumbnailOffset: int64 read FThumbnailOffset  write FThumbnailOffset;
    property ThumbnailSize: int64 read FThumbnailSize write FThumbnailSize;
    property TVLevel : integer read GetTVLevel write SetTVLevel;
    property TVParentID: integer read GetTVParentID write FTVParentID;
    property TVParentRecord : TILDataRecord read GetTVParentRecord write SetTVParentRecord;
    property TVSiblingIndex: integer read GetTVSiblingIndex write SetTVSiblingIndex;
    property UserID : string read FUserID;
  end;

  TILDataset = class
  private
    FChngTracker : TChangeTracker;
    FColumnSource : TILColumns;
    FDataset: TList;
    FIsJoined : Boolean;
    {True when dataset contains a joined recordset. Only occurs with SQL
     The following prevents repeatingly iterating over the the dataset to
    check for selected and changed records. Iterating only takes place when value is -1.
    The FSelstate cooperaties with the selection buffer. When -1 the buffer is refreshed }
    FIsReadOnly: Boolean;
    FOwnsRecords: Boolean;
    FPosition: integer;
    FPrivateFields : TILFieldDefs;
    FSelBuffer : TList; {In memory buffer keeping the selected records.
    Refreshed every time FSelstate dets -1. Keeping a dynamic count of selected records, is very slow}
    FSelState : integer; {-1 = unknown, 0 known (in selbuffer)}
    FTriggerChange: Boolean; { Set within a Begin / EndUpdate session when focus changes. When EndUpdate,
     triggers the focus change event. }
    FUpdater : Tupdater; { Points to a TUpdater when Save is called from an Updater}
    FUpdating: Boolean;  {Set after BeginUpdate, reset when EndUpdate}
    Ilib: TILTable;

    function GetChangeCount : integer;
    function GetChanged: Boolean;
    function GetColumnSource : TILColumns;
    function GetCount: integer;
    function GetFocusedRecord: TILDataRecord;
    function GetIsReadOnly: Boolean;
    function GetItems(Index: integer): TILDataRecord;
    function GetParentRecord : TILDataRecord;
    function GetSelCount: integer;
    function GetSelection(Index: integer): TILDataRecord;
    procedure DoSort(Compare: TListSortCompare);
    procedure RunFocus(Posit: integer; RecursiveClear : Boolean = True);
    procedure SetPosition(const Value : integer);
  protected

  public
    constructor Create(TableOwner: TILTable);
    destructor Destroy; override;
    function BOF : Boolean;
    function CanRevert(const SelectionOnly : Boolean = true) : Boolean;
    function CanSave(var ErrorIndex : integer) : integer;
    function Delete(const Index: integer) : integer;
    function EOF : Boolean;
    function IndexOf(ADataRecord: TILDataRecord): integer; overload;
    function IndexOf(const ARecordIndex: integer): integer; overload;
    function ReadXMLFile(const FileName: string): integer;
    function Save: integer;
    function Validate(var ErrorIndex: integer): integer; { Check if ok to save }
    procedure Add(ADataRecord: TILDataRecord);
    procedure BeginUpdate;
    procedure Clear(const Recursive : Boolean = True);
    procedure EndUpdate;
    procedure Exchange(const Index1, Index2: integer);
    procedure ExchangeMedia(const Index: integer; WithDataset: TILDataset; const WithIndex: integer);
    procedure First;
    procedure Last;
    procedure MoveTo(const Index: integer);
    procedure Next;
    procedure Previous;
    procedure RevertChanges(const SelectionOnly: Boolean = True);
    procedure SaveAsJSON(const AFileName : string; const SelectionOnly: Boolean);
    procedure SaveAsText(const AFileName : string; const ColumnSeparator, TextQualifier:
     Char; const IncludeTitles: Boolean;  const SelectionOnly: Boolean);
    procedure SaveAsXML(const FileName: string; const SelectionOnly: Boolean);
    procedure Sort(FieldNames: TStringList; const Ordering: TSortOrdering); overload;
    procedure Sort; overload;
    function ToJSON(SelectionOnly : Boolean) : string;
    function ToXML(SelectionOnly : Boolean) : string;
    property Changed: Boolean read GetChanged;
    property ChangeCount : integer read GetChangeCount;
    property ColumnSource : TILColumns read GetColumnSource;
    property Count: integer read GetCount;
    property FocusedRecord: TILDataRecord read GetFocusedRecord;
    property IsJoined : Boolean read FIsJoined;
    property IsReadOnly: Boolean read GetIsReadOnly write FIsReadOnly;
    property Items[Index: integer]: TILDataRecord read GetItems; default;
    property OwnsRecords: Boolean read FOwnsRecords write FOwnsRecords;
    property ParentRecord : TILDataRecord read GetParentRecord;
    property Position: integer read FPosition write SetPosition;
    property SelCount: integer read GetSelCount;
    property Selection[Index: integer]: TILDataRecord read GetSelection;
    property Source: TList read FDataset write FDataset;
  end;

  TILComboBox = class(TCombobox)
  private
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  end;

  TILEdit = class(TEdit)
  private
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  end;


  TILResourceList = class(TListBox)
   private
      FAddresses : TStringList;
      FImageIndex : TImageIndex;
      FImages : TImageList;
      FMouseDown : Boolean;
      FMouseInBtn : Boolean;
      FOnAddResource : TNotifyEvent;
      FOnUpdateResource : TNotifyEvent;
      FSingleLine : Boolean;

      function GetAddresses(Index : integer) : string;
      function GetBtnRect(const Index : integer) : TRect;
      function GetEmpty : Boolean;
      function GetFieldDef : TILFieldDef;
      function GetResourceStr(Index : integer) : string;
      procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
      procedure ComputeHeight;
      procedure DrawItm(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
      procedure MouseLeave(Sender : TObject);
      procedure SetAddresses(Index : integer; Value : string);
      procedure SetEmpty(const Value : Boolean);
      procedure SetImageIndex(const Value : TImageIndex);
      procedure SetImages(const Value : TImageList);
      procedure SetSingleLine(const Value : Boolean);

   protected
      procedure KeyDown(var Key: Word; Shift: TShiftState); override;
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
      procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState;  X, Y: integer); override;
      procedure Resize; override;

   public
      constructor Create(AnOwner : TComponent); override;
      destructor Destroy; override;
      procedure AddResource(const AName, AAddress : string);
      procedure DeleteResource(const Index : integer);
      procedure UpdateResource(const Index : integer; const AName, AAddress : string);
      property Addresses[Index : integer] : String read GetAddresses write SetAddresses;
      property Empty : Boolean read GetEmpty write SetEmpty;
      property FieldDef : TILFieldDef read GetFieldDef;
      property ResourceStr[Index : integer] : string read GetResourceStr;

   published
      property ImageIndex : TImageIndex read FImageIndex write SetImageIndex;
      property Images : TImageList read FImages write SetImages;
      property OnAddResource : TNotifyEvent read FOnAddResource write FOnAddResource;
      property OnUpdateResource : TNotifyEvent read FOnUpdateResource write FOnUpdateResource;
      property SingleLine : Boolean read FSingleLine write SetSingleLine default true;
  end;

  { Used to display records from a TILTreeview data table }
  TILTreeView = class(TTreeView)
  private
    FBuilding: Boolean;
    FTVTable: TILTreeViewTable;
    procedure WMREFRESHTREEVIEW(var Message: TMessage); message WM_RefreshTreeView;
  protected
    function CanChange(Node: TTreeNode): Boolean; override;
    function CanExpand(Node: TTreeNode): Boolean; override;
    procedure Change(Node: TTreeNode); override;
    procedure Edit(const Item: TTVItem); override;
    procedure Expand(Node: TTreeNode); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;

  public
    function NodeOfRecord(ARecord: TILRecord): TTreeNode;
    procedure BuildTree(AdataSet: TILDataset);
    property Building: Boolean read FBuilding write FBuilding;
    property TVTable: TILTreeViewTable read FTVTable;
  end;

  { Image viewer }
  TILImage = class(TCustomControl)
    private
       FBM : VCL.Graphics.TBitmap;
       FDrawBM : VCL.Graphics.Tbitmap;
       FFileName : string;
       FInternalLoad : Boolean;
       FInZoomWindow : Boolean;
       FJPEGQuality : TJPEGQuality;
       FMouseInNext, FMouseInPrev : Boolean;
       FShowNavButtons : Boolean;
       FSourceBM : VCL.Graphics.TBitmap;
       FSourceField : TILField;
       FSourceRecord : TILDataRecord;
       function GetDataset : TILDataset;
       function GetDrawBitmap : VCL.Graphics.TBitmap;
       function GetJPEGScale : TJPEGScale;
       function GetNextRect : TRect;
       function GetPrevRect : TRect;
       function GetSourceBitmap : VCL.Graphics.TBitmap;
       function GetSourcePicture : TPicture;
       function GetTable : TILDataTable;
       procedure InternalClear;
       procedure LoadFromClipBoard;
       procedure LoadFromField(AField : TILField);
       procedure LoadFromFile(FileName : string);
       procedure LoadFromPicture(APicture : TPicture);
       procedure LoadFromRecord(ARecord : TILDataRecord);
       procedure SetDrawBM;
       procedure SetSourceField(const Value : TILField);
       procedure SetSourceRecord(const Value : TILDataRecord);
       procedure UpdateImage;
       procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
       procedure WndOnShow(Sender : TObject);
    protected
       procedure CreateParams(var Params: TCreateParams); override;
       procedure KeyDown(var Key : word; Shift : TShiftState); override;
       procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
       procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
       procedure Paint; override;
       procedure Resize; override;
       procedure WMERASEBKGND(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;

    public
      constructor Create(AOwner : TComPonent); override;
      destructor Destroy; override;
      function EmptySource : Boolean;
      procedure Clear; { Removes the image, but not the resource record pointers }
      procedure ClearSource; { Removes the resource record poiners pointer and the image }
      procedure LoadImage(AnImage : TPicture); overload;
      procedure LoadImage(AFileName : string); overload;
      procedure RemoveSource; { Removes the resource record pointers, but not the image }
      procedure ShowRecordImage;
      property Dataset : TILDataset read GetDataSet;
      property DrawBitmap : VCL.Graphics.TBitmap read GetDrawBitmap;
      property FileName : string read FFileName;
      property InZoomWindow : Boolean read FInZoomWindow;
      property SourceBitmap : VCL.Graphics.TBitmap read GetSourceBitmap;
      property SourceField : TILField read FSourceField write SetSourceField;
      property SourcePicture : TPicture read GetSourcePicture;
      property SourceRecord : TILDataRecord read FSourceRecord write SetSourceRecord;
      property Table : TILDataTable read GetTable;

    published
      property Align;
      property Anchors;
      property Color;
      property Constraints;
      property DragCursor;
      property DragKind;
      property DragMode;
      property Enabled;
      property Font;
      property JPEGQuality : TJPEGQuality read FJPEGQuality write FJPEGQuality default jqAuto;
      property OnClick;
      property OnContextPopup;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDock;
      property OnEndDrag;
      property OnMouseDown;
      property OnMouseEnter;
      property OnMouseLeave;
      property OnMouseMove;
      property OnMouseUp;
      property OnMouseWheel;
      property OnMouseWheelDown;
      property OnMouseWheelUp;
      property OnStartDock;
      property OnStartDrag;
      property ParentColor;
      property ParentShowHint;
      property PopupMenu;
      property ShowHint;
      property ShowNavButtons : Boolean read FShowNavButtons write FShowNavButtons default false;
      property StyleElements;
      property Visible;
  end;

  TILCellEditor = class
    { Manages the record editors. This moves around with the selected cell }
  private
    FActiveCtrl: TWinControl;
    FCell: TILReportCell;
    FComboCtrl: TILComboBox; { Combo }
    FEditCtrl: TILEdit;
    FLookupVisible: Boolean;
    FMemoCtrl: TMemo;
    FMouseMoved: Boolean;
    FResourceCtrl : TILResourceList;
    FSemafor : Boolean;
    FViewer : TILDatasetViewer;
    Ilib: TILTable;

    procedure ComboChange(Sender: TObject);
    procedure ComboCloseUp(Sender : TObject);
    procedure ComboKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure EditChange(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure MemoChange(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ProcessKey(Key: Word; Shift: TShiftState);
    procedure ProcessTab(const Fwd: Boolean);
    procedure ResourceKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

  public
    constructor Create(AViewer: TILDatasetViewer);
    destructor Destroy; override;
    procedure Cancel;
    procedure Edit;
    procedure MoveTo(ACell: TILReportCell);
    procedure UpdateValue;
    property ActiveCtrl: TWinControl read FActiveCtrl write FActiveCtrl;
    property Cell: TILReportCell read FCell write FCell;
  end;

  {TILreport, TILReportRow and TILReportCell organize the textual part of
  the dataset viewer.}
  TILReportCell = class
  private
    FEditing: Boolean;
    FFieldDef: TILFieldDef;
    FOwner: TILReportRow;
    FSelected: Boolean;
    FViewer : TILDatasetViewer;
    FVisible: Boolean;
    Ilib: TILTable;
    function GetData: TILDataRecord;
    function GetField : TILField;
    function GetFieldIndex: integer;
    function GetImage: Vcl.Graphics.TBitmap;
    function GetImageFileName: string;
    function GetImageRect: TRect;
    function GetIndex: integer;
    function GetItemRect: TRect;
    function GetPosition : integer;
    function GetTable : TILTable;
    function GetValue: string;
    procedure SetPosition(const Value : integer);
    procedure SetSelected(const Value : Boolean);
    procedure SetValue(Val: string);
    procedure SetVisible(const Value : Boolean);
  public
    constructor Create(AnOwner: TILReportRow; AFieldDef: TILFieldDef);
    destructor Destroy; override;
    function CanEdit : integer;
    function IsEmpty : Boolean;
    procedure CancelEdit;
    procedure Edit;
    property Data: TILDataRecord read GetData;
    property Editing: Boolean read FEditing;
    property Field : TILField read GetField;
    property FieldDef: TILFieldDef read FFieldDef;
    property FieldIndex: integer read GetFieldIndex; { Index in data record }
    property Image: Vcl.Graphics.TBitmap read GetImage;
    property ImageFileName: string read GetImageFileName;
    property ImageRect: TRect read GetImageRect;
    property Index: integer read GetIndex;
    property ItemRect: TRect read GetItemRect;
    property Owner: TILReportRow read FOwner;
    property Position : integer read GetPosition write SetPosition;
    property Selected: Boolean read FSelected write SetSelected;
    property Table : TILTable read GetTable;
    property Value: string read GetValue write SetValue;
    property Visible: Boolean read FVisible write SetVisible;
  end;

  TILReportRow = class
  private
    FColumnList: TList;
    FData: TILDataRecord;
    FExpansionCount: integer;
    FExpansionIndex: integer;
    FIndex: integer;
    FOwnerReport: TILReport;
    Ilib: TILTable;
    function GetCheckRect: TRect;
    function GetCount: integer;
    function GetExpandBtnRect: TRect;
    function GetExpanded: Boolean;
    function GetImageRect: TRect;
    function GetItemRect: TRect;
    function GetItems(Index: integer): TILReportCell;
    function GetDatasetIndex: integer;
  public
    constructor Create(AnOwner: TILReport);
    destructor Destroy; override;
    function CanExpand: Boolean;
    function IsDeleted : Boolean;
    procedure Clear;
    property CheckRect: TRect read GetCheckRect;
    property Count: integer read GetCount;
    property Data: TILDataRecord read FData;
    property ExpandBtnRect: TRect read GetExpandBtnRect;
    property Expanded: Boolean read GetExpanded;
    property ExpansionCount: integer read FExpansionCount;
    property ExpansionIndex: integer read FExpansionIndex;
    property ImageRect: TRect read GetImageRect;
    property Index: integer read FIndex;
    property ItemRect: TRect read GetItemRect;
    property Items[Index: integer]: TILReportCell read GetItems;
    property OwnerReport: TILReport read FOwnerReport;
    property DatasetIndex: integer read GetDatasetIndex;
  end;

  TILReport = class
  private
    FColumnsList: TList;
    FSelectedCell: TILReportCell;
    Ilib: TILTable;
    function DoExpand(Index: integer; const MinCount: integer): integer;
    function GetCells(Row, Cell: integer): TILReportCell;
    function GetCount: integer;
    function GetExpansionRect(Index: integer): TRect;
    function GetFirstVisibleColumn: integer;
    function GetFirstVisibleItem: integer;
    function GetItems(Index: integer): TILReportRow;
    function GetLastVisibleColumn: integer;
    function GetLastVisibleItem: integer;
    procedure Populate;
    procedure SetSelectedCell(const Value : TILReportCell);
  public
    constructor Create(TableOwner: TILTable);
    destructor Destroy; override;
    function CanAddSelected : integer;
    function CanDeleteSelected : integer;
    function CellAtXY(const X, Y: integer): TILReportCell;
    function Collapse(Index: integer): integer;
    function Expand(const Index: integer): integer;
    function Expanded(Index: integer): Boolean;
    function InfoFromPos(const X, Y: integer): TReportInfo;
    function RecordIndexToRowIndex(const RecordIndex: integer): integer;
    function RowAtXY(const X, Y: integer): integer;
    procedure AddSelected;
    procedure Clear;
    procedure CollapseAll;
    procedure CollapseSelection;
    procedure DeleteSelected;
    procedure ExpandAll;
    procedure ExpandSelection;
    procedure Redraw;
    property Cells[Row, Cell: integer]: TILReportCell read GetCells;
    property Count: integer read GetCount;
    property ExpansionRect[Index: integer]: TRect read GetExpansionRect;
    property FirstVisibleColumn: integer read GetFirstVisibleColumn;
    property FirstVisibleItem: integer read GetFirstVisibleItem;
    property Items[Index: integer]: TILReportRow read GetItems;
    property LastVisibleColumn: integer read GetLastVisibleColumn;
    property LastVisibleItem: integer read GetLastVisibleItem;
    property SelectedCell: TILReportCell read FSelectedCell write SetSelectedCell;
  end;

  { Handles columns and sorting }
  TILColumns = class
  private
    FChanged: Boolean;
    FClickSortIndex: integer;
    FClickSortOrdering: TSortOrdering;
    FFieldSource: TILFieldDefs;
    FOutDefs: TStringList;
    FSortChanged : Boolean;
    FSortFields: TStringList;
    FSortOrdering: TSortOrdering;
    Ilib: TILTable;
    function GetColumnHeaderRect(Index: integer): TRect;
    function GetCount: integer;
    function GetExcludedCount: integer;
    function GetExcludedItems(Index: integer): TILFieldDef;
    function GetItems(Index: integer): TILFieldDef;
    function GetTotalWidth: integer;
    procedure AssignSource;
    procedure CheckColumns(Cols : TStrings);
    procedure Chng(Sender: TObject);
    procedure SetSortOrdering(const Value : TSortOrdering);
  public
    constructor Create(AnOwner: TILTable; FieldSource: TILFieldDefs);
    destructor Destroy; override;
    function ColumnFromPos(const XPos: integer): integer;
    function ColumnHeaderAtXY(X, Y: integer): integer;
    function ColumnPos(const ColumnIndex: integer; const FromLeftColumn: Boolean): integer;
    function ColumnSplitterAtXY(X, Y: integer): integer;
    function CompareColumns(Columns : TChecklistBox) : Boolean; overload;
    function CompareColumns(Columns : TStrings) : Boolean; overload;
    function CompareSort(SortKey : TCheckListBox; const Ordering : TSortOrdering) : Boolean; overload;
    function CompareSort(SortKey : TStrings; const Ordering : TSortOrdering) : Boolean; overload;
    function IndexOf(const FieldName: string): integer;
    function RectFromLeftColumn(const YPos: integer): TRect;
    procedure Add(const FieldName: string);
    procedure Apply;
    procedure Assign(Source: TILColumns);
    procedure Clear;
    procedure Delete(const Index: integer);
    procedure GetVisibleColumns(var First, Last: integer);
    procedure Insert(const Index: integer; const FieldName: string);
    procedure Move(const FromIndex, ToIndex: integer);
    procedure RestoreDefaults;
    procedure SetColumns(Columns : TChecklistBox); overload;
    procedure SetColumns(Columns : TStrings); overload;
    procedure SetSortKey(SortKey : TCheckListBox; const Ordering : TSortOrdering) overload;
    procedure SetSortKey(SortKey : TStrings; const Ordering : TSortOrdering); overload;
    property Changed: Boolean read FChanged write FChanged;
    property ColumnHeaderRect[Index: integer]: TRect read GetColumnHeaderRect;
    property Count: integer read GetCount;
    property ExcludedCount: integer read GetExcludedCount;
    property ExcludedItems[Index: integer]: TILFieldDef read GetExcludedItems;
    property Items[Index: integer]: TILFieldDef read GetItems; default;
    property SortChanged: Boolean read FSortChanged write FSortChanged;
    property SortFields: TStringList read FSortFields;
    property SortOrdering: TSortOrdering read FSortOrdering write SetSortOrdering;
    property TotalWidth: integer read GetTotalWidth;
  end;

  { Base class of the DB... namespaces}
  TILDB = class
  private
    Ilib: TILTable;
  public
    constructor Create(AnOwner: TILTable);
    destructor Destroy; override;
    function ControlsEmpty: Boolean;
    procedure CheckConn;
    function ExecSQL(AnExpression: TStrings): integer;
    function ReadControls(Fields: TILRecord): integer;
    procedure AddFieldSpec(Fields: TILRecord; const FieldName, Spec: string);
    procedure ClearControls(AControl : TWinControl = nil);
    procedure WriteControls(Fields: TILRecord);
  end;

  TILDBAdmin = class
  private
    Ilib: TILTable;
  public
    { DB Admin methods. This is nothing but grouping of the TIL private methods }
    procedure AlterField(FieldName: string; Alter: TAlterFieldOption; Value: Variant);
    procedure AlterIndexSpace;
    procedure AlterRecordFreeSpace;
    procedure AlterRecordLayout(const NewLayout: TRecordLayout);
    procedure AlterRecordSize;
    procedure AlterTable(const Option: TILOperation; const NewSpace: integer; NewIndexes, NewLookups: TStringList);
    procedure AlterTableSpace(const NewSpace: integer);
    procedure ClearTable;
    procedure RebuildIndexes;
    procedure RenameTable(const OldName, NewName : string);
  end;

  TILDBCommons = class
  private
    Ilib: TILTable;
  public
    function CanExecSQL(SQL : TStrings) : integer;
    function CompareIdents(Record1, Record2: TILDataRecord): TilField;
    function CompareIDS(OldData, NewData: TILDataRecord): Boolean;
    function CompareRecords(Record1, Record2: TILRecord; Delta: Boolean): Boolean;
    function CompareUniques(Record1, Record2: TILDataRecord): TilField;
    function ExecSQL(Expression: TStrings; SQLTypes: TSQLTypes = []): integer;
    function IndexOfID(ADataRecord: TILDataRecord; var FieldNames : string): integer;
    function IsControlActive(AControl: TControl): Boolean;
    function IsRecordEmpty(ARecord: TILRecord): Boolean;
    function IsRecordOrphan(const RecordIndex: integer): Boolean;
  end;

  TILDBView = class(TILDB)
  private
    Ilib: TILTable;
    function DoSearchAgg(const FieldName, Func: string; var AResult : Real;
      Condition : TStringList = nil): Integer;
    function GetSearchEngine: TSearcher;
  public
    constructor Create(AnOwner: TILTable);
    function CanSearch : integer;
    function CheckSource(Source: TILRecord): integer; overload;
    function CheckSource(Source: TStrings): integer; overload;
    function CheckSource: integer; overload;
    function ControlsToExpression(Expression: TStringList; const IncludeDetailTables: Boolean = false): integer;
    function ControlsToSQL(AllInPath : Boolean) : TStringList;
    function Exists(Expression: TStringList; var RecordIndex : integer): Boolean;
    function ExpressionToSQL(AnExpression : TStringList) : TStringList;
    function Search(const AnExpression : string) : integer; overload;
    function Search(const FieldOrder: string; Source : TObject): integer; overload;
    function Search(Source: TObject = nil): integer; overload;
    function SearchAvg(const FieldName: string; var AResult : Real; Condition : TSTringList = nil): Integer;
    function SearchCount(const FieldName: string; var AResult : integer; Condition : TStringList = nil): Integer;
    function SearchDeleted: TILDataset;
    function SearchMax(const FieldName: string; var AResult : Real; Condition : TSTringList = nil): Integer;
    function SearchMin(const FieldName: string; var AResult : Real; Condition : TSTringList = nil): Integer;
    function SearchPrivate(AnExpression : string; const PerformCalc : Boolean = false): TILDataset; overload;
    function SearchPrivate(AnExpression: TStringList; const PerformCalc : Boolean = false): TILDataset; overload;
    function SearchPrivate(ARecord: TILRecord; const PerformCalc : Boolean = false): TILDataset; overload;
    function SearchSum(const FieldName: string; var AResult : Real; Condition : TSTringList = nil): Integer;
    procedure FilterRecords(AFilterTable : TILDataTable);
    procedure SetLogOp(const FieldName: string; const LogOp: TFieldLogOp); overload;
    procedure SetLogOp(const FieldName: string; const LogOp: TValueLogOp); overload;
    procedure SetLogOp(LogOp: TFieldLogOp); overload;
    procedure SetLogOp(SearchControl: TWinControl; const LogOp: TFieldLogOp); overload;
    procedure SetLogOp(SearchControl: TWinControl; const LogOp: TValueLogOp); overload;
    property SearchEngine: TSearcher read GetSearchEngine;
  end;

  TILDBWork = class(TILDB)
  private
    Ilib: TILTable;
    FRecordIndex : integer;
    function DoInsert(Target: TUpdateTarget; Source: TILRecord; FileName: string; Image: TPicture): integer;
    function GetDisableIndexUpdate: Boolean;
    function GetUpdateTarget : TUpdateTarget;
    procedure SetDisableIndexUpdate(const Value : Boolean);
    procedure SetUpdateTarget(const Value : TUpdateTarget);
  public
    constructor Create(AnOwner: TILTable);
    function CanDelete(Index : integer) : integer;
    function CanInsert : Integer;
    function CanRevert : Boolean;
    function CanSave(var ErrorIndex : integer) : integer;
    function CanUpdateData : integer;
    function CanUpdateImage : integer;
    function CheckSource(const UpdateType: TUpdateType): integer; overload;
    function CheckSource(Source: TILRecord): integer; overload;
    function CheckSource(Source: TStrings): integer; overload;
    function ControlsToExpression: TStringList;
    function CreateDataRecord: TILDataRecord;
    function Delete(const AnIndex: integer): integer;
    function DeleteFromTable(const ARecordIndex : integer) : Boolean;
    function Insert(const MediaFile: string; Source: TILRecord): integer; overload;
    function Insert(Image: TPicture; Source: TILRecord): integer; overload;
    function Insert(MediaFiles: TStrings; const ReplaceDataset: Boolean) : integer; overload;
    function Insert(Source: TILRecord) : integer; overload;
    function Insert: integer; overload;
    function InsertEmptyRecord : integer;
    function LinkImage(const AnIndex : integer; const TagName : string) : Boolean;
    function NeedsUpdate(Target: TILDataRecord; Source: TObject = nil): Boolean;
    function NeedsUpdateDataset(Source: TObject = nil): Boolean;
    function ReadXMLFile(const AFileName: string): integer;
    function Save: integer;
    function TestUpdate(Target: TILDataRecord; var Updated: Boolean; Source: TObject = nil): TILTestResult;
    function TestUpdateDataset(Source: TObject = nil): TILUpdateTest;
    function Undelete(const RecordIndex: integer): Boolean;
    function Update(Source: TILRecord): integer;
    function UpdateFileNames(const OldPattern, NewPattern : string; const Occurence : integer;
     Query : TStringList = nil) : integer;
    function UpdateImageFromSource(Inserting : Boolean) : integer;
    function UpdateRecord(TargetRecord: TILDataRecord; Source: TILRecord = nil): integer;
    {$IfDef DLLAccess}
    function UpdateRecordImage(AStream : Pointer; Size : Integer; ARecordIndex : integer) : integer; overload;
    function UpdateRecordImage(AFileName : string; ARecordIndex : integer) : integer; overload;
    function UpdateTable(ASource : TStrings; ARecordIndex : integer) : integer;
    {$EndIf}
    function ValidateDataset(var ErrorIndex: integer): integer;
    procedure CalcControls;
    procedure CalcRecord(ARecord : TILDataRecord);
    procedure ClearDataset;
    procedure ExchangeMedia(Dataset1, Dataset2: TILDataset; const Index1, Index2: integer);
    procedure RevertChanges(const SelectionOnly : Boolean);
    procedure SaveAsJSON(const AFileName: string);
    procedure SaveAsXML(const AFileName: string);
    procedure UpdateImage(const AnIndex: integer; AnImage: TPicture); overload;
    procedure UpdateImage(const AnIndex: integer; const AFileName: string); overload;

    property DisableIndexUpdate: Boolean read GetDisableIndexUpdate write SetDisableIndexUpdate;
    property UpdateTarget : TUpdateTarget read GetUpdateTarget write SetUpdateTarget;
  end;

  TLookupTbls = record
    FieldName : string;
    TableName : string;
  end;

  TILReplicatorItem = class
  private
    FAlias : string;
    FDateSearchFormat : string;
    FErrorState : string;
    FFirstRecordID : integer;
    FFreeTable : Boolean;
    FIsRoot : Boolean;
    FLocalImageSize : string;
    FRelations : string;
    FLookupTbls : array of TLookUpTbls;
    FMasterSource : string;
    FMetaString: string;
    FOwner : TILReplicator;
    FParentFields : Boolean;
    FRecordFreeSpace : string;
    FRecordLayout : string;
    FRecordSize : string;
    FShared : Boolean;
    FTable: TILTable;
    FTableClass : string;
    FTableName: string;
    FTableSpace: string;
    procedure CreateFields;
    procedure Link;
    procedure Load;
    property MetaString: string read FMetaString;
    property TableName: string read FTableName;
  public
    property Table: TILTable read FTable;
  end;

  TILReplicator = class
  private
    FDBName : string;
    FExtDesigner : Boolean;
    FMetaFileName: string;
    FMetaString: string;
    FOwner: TTableManager;
    FRootTable : string;
    FTableList: TList;
    FTableName : string;
    FTransHistory : Boolean;
    FUnicode : string;
    FXMLDoc: TXMLDocument;
    function GetCount: integer;
    function GetItems(Index: integer): TILReplicatorItem;
    procedure Add(ATable: TILReplicatorItem);
  public
    constructor Create(const AMetaFileName: string; TableManager : TTableManager); overload;
    constructor Create(const AMetaFileName: string; TableManager : TTableManager; const ExtDesigner : Boolean); overload;
    destructor Destroy; override;
    function TableOf(const TableName : string) : TILTable;
    procedure Replicate(const TableName : string); overload;
    procedure Replicate; overload;
    property Count: integer read GetCount;
    property Items[Index: integer]: TILReplicatorItem read GetItems;
  end;

  TILDatasetViewer = class(TCustomControl)
  private
    FAfterDrawImageItem: TAfterDrawImageItemEvent;
    FCaption: string;
    FCaptionAlignment: TAlignment;
    FCellEditor: TILCellEditor;
    FCellUnderMouse: TILReportCell;
    FColumnClicked: integer;
    FColumnSpacing: integer; { Report item spacing }
    FDrawBM : VCL.Graphics.TBitmap;
    FFirstItemIndex: integer;{ Index in dataset that represents the item at topleft of the canvas }
    FFocusRectangle: Boolean;
    FGridLineColor : TColor;
    FHighLightColumn: integer;
    FHotTrack: Boolean;
    FHoverCtrl : Boolean; {True when only ctrl pressed when mouse moves and ReportImages is riHoverCtrl }
    FImageHeight: integer;
    FImageItemSpacing: integer;
    FImageLayout: TImageLayout;
    FImageMem : integer;
    FImageOptions: TImageOptions;
    FImageRowSpacing: integer;
    FImageTextSpace: integer;
    FImageWidth: integer;
    FImgHScrollPos: integer;{ The position of horz scrollbar when ioHidePartialVisible = false
    Toggles between 0 and 1 }
    FItemUnderMouse: integer;
    FLeftColumn: integer; { Index of the first visible column in report mode }
    FLineSpacing: integer; { Report line spacing }
    FOnClickCaption : TNotifyEvent;
    FOnClickCell : TClickCellEvent;
    FOnClickImage : TClickImageEvent;
    FOnDrawImageItem: TDrawImageItemEvent;
    FOnDrawReportCell: TDrawReportCellEvent;
    FOnDrawReportCellEx: TDrawReportCellExEvent;
    FOnDrawReportRow: TDrawReportRowEvent;
    FOnDrawUpdateMarker: TDrawUpdateMarkerEvent;
    FOnEditCell : TCellEvent;
    FOnExecResource: TExecResourceEvent;
    FOnMouseEnterCell: TCellEvent;
    FOnMouseEnterItem: TItemEvent;
    FOnMouseLeaveCell: TCellEvent;
    FOnMouseLeaveItem: TItemEvent;
    FOnSetValue : TCellEvent;
    FReport: TILReport;
    FReportImages: TReportImages;
    FReportImageSize: TReportImageSize;
    FReportOptions: TReportOptions;
    FScrolling: Boolean;
    FSemaphor : Boolean;
    FShowCheckBoxes: Boolean;
    FSplitterClicked: integer;
    FTable : TILTable;
    FTLReport, FTLImages: integer; { Holds the top left index for each view mode.
     When viewMode is set, the FFirstItemIndex is asigned one of theese values }
    FUpdateMarkers: TUpdateMarkers;
    FViewMode: TViewMode;
    FZoomImage : TILImage;
    FZoomWindow : TForm;
    function CanScroll(const Forwrd: Boolean; const ScrollDir: integer): Boolean;
    function DoZoom(const Width, Height : integer) : TILImage;
    function GetColCount: integer;
    function GetImageHeight: integer;
    function GetImageRowSpacing: integer;
    function GetImageWidth: integer;
    function GetImgVertSpace: integer;
    function GetLeftMargin: integer;
    function GetOutputCount: integer;
    function GetReportLineHeight: integer;
    function GetRowCount: integer;
    function GetRptImageSize: integer;
    function GetRptLeftMargin: integer;
    function GetRptVertSpace: integer;
    function GetStyleColor(const Element: TStyleElements; const StyleTarget: TStyleTarget): TColor;
    function GetTable : TILTable;
    function GetTopMargin: integer;
    function GetViewMode: TViewMode;
    function IsColumnVisible(const ColumnIndex: integer; var AtRight: Boolean): Boolean;
    function IsImageFullyVisible(const ItemIndex: integer): Boolean; overload;
    function IsImageFullyVisible(const ItemIndex: integer; const AtRight: Boolean) : Boolean; overload;
    function ProcessAlignment(const A : TAlignment; const AString: string; const LeftPos : integer; const MaxWidth: integer): integer;
    function ProcessString(const AString: string; const MaxWidth: integer): string;
    procedure CMDESIGNHitTest(var Message: TCMDesignHitTest); message CM_DesignHittest;
    procedure ColumnClick(Index: integer);
    procedure CreateReport;
    procedure DrawCaption(Cnv: TCanvas);
    procedure DrawImages;
    procedure DrawReport;
    procedure GetVisibles(var First, Last: integer);
    procedure ImgTxtEditorKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ReleaseImageMem(ExcludeStart, ExcludeEnd: integer);
    procedure SetCaption(const Value : string);
    procedure SetCaptionAlignment(const Value : TAlignment);
    procedure SetFirstItemIndex(const Value : integer);
    procedure SetGridlineColor(Value : TColor);
    procedure SetImageHeight(const Value : integer);
    procedure SetImageLayout(const Value : TImageLayout);
    procedure SetImageRowSpacing(const Value : integer);
    procedure SetImageWidth(const Value : integer);
    procedure SetLeftColumn(const Value : integer);
    procedure SetReportImages(const Value : TReportImages);
    procedure SetReportImageSize(const Value : TReportImageSize);
    procedure SetReportOptions(const Value : TReportOptions);
    procedure SetShowCheckBoxes(const Value : Boolean);
    procedure SetTable(const Value : TILTable);
    procedure SetUpdateMarkers(const Value : TUpdateMarkers);
    procedure SetViewMode(const Value : TViewMode);
    procedure UpdateScrollPos;
    procedure UpdateScrollRange;
    procedure WMERASEBKGND(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMMouseLeave(var Message: TWMMouse); message WM_MOUSELEAVE;
    procedure WMSETFOCUS(var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    property LeftColumn: integer read FLeftColumn write SetLeftColumn;
    property OutputCount: integer read GetOutputCount;
    property ReportLineHeight: integer read GetReportLineHeight;
    property ReportLineSpacing: integer read FLineSpacing;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure Resize; override;
    procedure Loaded; override;
  public
    function CanReportDisplayImages : Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetItemRect(const ARectType: TRectType; const ItemIndex: integer): TRect;
    function ImageTextFits(const TheText: string): Boolean;
    function ItemAtXY(const X, Y: integer): integer;
    procedure ScrollBy(const Delta: integer);
    procedure ScrollImages(const ScrollCode: integer; Delta: integer);
    procedure ScrollIntoView(Index: integer);
    procedure ScrollReport(const ScrollCode: integer; Delta: integer);
    procedure ScrollTo(const Index: integer);
    procedure SetBounds(ALeft: integer; ATop: integer; AWidth: integer;  AHeight: integer); override;
    procedure ShowZoomWindow(AField : TILField; const Width, Height : integer); overload;
    procedure ShowZoomWindow(ARecord : TILDataRecord; const Width, Height : integer); overload;
    procedure UpdateScrollers;

    property CellEditor: TILCellEditor read FCellEditor;
    property ColCount: integer read GetColCount;
    property FirstItemIndex: integer read FFirstItemIndex  write SetFirstItemIndex;
    property ItemUnderMouse: integer read FItemUnderMouse;
    property Report: TILReport read FReport;
    property RowCount: integer read GetRowCount;
    property ZoomImage : TILImage read FZoomImage;
    property ZoomWindow : TForm read FZoomWindow;

  published
    property AfterDrawImageItem: TAfterDrawImageItemEvent read FAfterDrawImageItem write FAfterDrawImageItem;
    property Align;
    property Anchors;
    property Caption: string read FCaption write SetCaption;
    property CaptionAlignment: TAlignment read FCaptionAlignment write SetCaptionAlignment default taCenter;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusRectangle: Boolean read FFocusRectangle write FFocusRectangle default True;
    property Font;
    property GridLineColor : TColor read FGridLineColor write SetGridlineColor  default clActiveBorder;
    property HotTrack: Boolean read FHotTrack write FHotTrack default false;
    property ImageHeight: integer read GetImageHeight write SetImageHeight default 120;
    property ImageItemSpacing: integer read FImageItemSpacing write FImageItemSpacing default 10;
    property ImageLayout: TImageLayout read FImageLayout write SetImageLayout default ilGrid;
    property ImageOptions: TImageOptions read FImageOptions write FImageOptions default [ioWrapPartialVisibles];
    property ImageRowSpacing: integer read FImageRowSpacing write SetImageRowSpacing default 10;
    property ImageTextSpace: integer read FImageTextSpace write FImageTextSpace default 0;
    property ImageWidth: integer read GetImageWidth write SetImageWidth default 120;
    property OnClick;
    property OnClickCaption : TNotifyEvent read FOnClickCaption write FOnClickCaption;
    property OnClickCell : TClickCellEvent read FOnClickCell write FOnClickCell;
    property OnClickImage : TClickImageEvent read FOnClickImage write FOnClickImage;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawImageItem: TDrawImageItemEvent read FOnDrawImageItem write FOnDrawImageItem;
    property OnDrawReportCell: TDrawReportCellEvent read FOnDrawReportCell write FOnDrawReportCell;
    property OnDrawReportCellEx: TDrawReportCellExEvent read FOnDrawReportCellEx write FOnDrawReportCellEx;
    property OnDrawReportRow: TDrawReportRowEvent read FOnDrawReportRow write FOnDrawReportRow;
    property OnDrawUpdateMarker : TDrawUpdateMarkerEvent read FOnDrawUpdateMarker write FOnDrawUpdateMarker;
    property OnEditCell : TCellEvent read FOnEditCell write FOnEditCell;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExecResource : TExecResourceEvent read FOnExecResource write FOnExecResource;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseEnterCell : TCellEvent read FOnMouseEnterCell write FOnMouseEnterCell;
    property OnMouseEnterItem : TItemEvent read FOnMouseEnterItem write FOnMouseEnterItem;
    property OnMouseLeave;
    property OnMouseLeaveCell : TCellEvent read FOnMouseLeaveCell write FOnMouseLeaveCell;
    property OnMouseLeaveItem : TItemEvent read FOnMouseLeaveItem write FOnMouseLeaveItem;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnSetValue : TCellEvent read FOnSetValue write FOnSetValue;
    property OnStartDock;
    property OnStartDrag;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ReportImages: TReportImages read FReportImages write SetReportImages default riNone;
    property ReportImageSize: TReportImageSize read FReportImageSize write SetReportImageSize default risMedium;
    property ReportOptions: TReportOptions read FReportOptions write SetReportOptions default [];
    property ShowCheckBoxes: Boolean read FShowCheckBoxes write SetShowCheckBoxes default false;
    property ShowHint;
    property StyleElements;
    property Table : TILTable read GetTable write SetTable;
    property TabStop;
    property UpdateMarkers: TUpdateMarkers read FUpdateMarkers write SetUpdateMarkers default upmNone;
    property ViewMode: TViewMode read GetViewMode write SetViewMode default vmReport;
    property Visible;
  end;

  TILData = class(TComponent)
  private
    FDataSetMaxCount : integer;
    FTable : TILTable;
    FMultiSelect : Boolean;
    FFilterState : TFilterState;
    FFilter : TStringList;
    FAfterResume: TNotifyEvent;
    FAutoCreateFields : Boolean;
    FBeforeFocusRecord: TFocusEvent;
    FBeforeResume: TNotifyEvent;
    FDataFields : TILDataFields;
    FDisableCommandModes : Boolean;
    FFreeSearch : Boolean;
    FHideInTableList : Boolean;
    FImageControl : TILImage;
    FImageSource : TImageSource;
    FIsReadOnly : Boolean;
    FMDOptions : TMDOptions;
    FOnDatasetChange : TDatasetChangeEvent;
    FOnCalcField: TCalcFieldEvent;
    FOnChangeDAC : TChangeDACEvent;
    FOnCreateWorkField : TWorkFieldEvent;
    FOnEvaluateField : TEvaluateFieldEvent;
    FOnFetchRecord: TSearchEvent;
    FOnFilter : TFilterEvent;
    FOnFocusRecord: TNotifyEvent;
    FOnFormatProperty: TPropertyEvent;
    FOnImageTableRequest : TImageTableEvent;
    FOnIndexWord: TIndexWordEvent;
    FOnInitRecord : TRecordEvent;
    FOnMissingLink : TLinkEvent;
    FOnSearchLongText : TSearchLongTextEvent;
    FOnSelect: TItemEvent;
    FOnTrigger : TRecordEvent;
    FOnUpdateImage : TUpdateImageEvent;
    FOnUpdateLongText : TLongTextEvent;
    FPrecheckConstraints : Boolean;
    FPropagateSearchFields : Boolean;
    FPropertyList : TILPropertyList;
    FSaveColumns : Boolean;
    FSortKey : TStringList;
    FSortOrdering : TSortOrdering;
    FSummaryControl : TILRecordViewer;
    FTreeViewControl : TILTreeView;
    FTreeViewLabelField : string;
    function GetDataset : TILDataset;
    function GetDBCommons : TILDBCommons;
    function GetDBView : TILDBView;
    function GetDBWork : TILDBWork;
    function GetImageTagField : TILDataField;
    function GetMultiSelected : Boolean;
    function GetReadOnly : Boolean;
    procedure AssignColumns;
    procedure CreateSummary(DataRecord: TILDataRecord);
    procedure FillDataFields;
    procedure InitLookups;
    procedure SetAutoCreateFields(const Value : Boolean);
    procedure SetDataFields(const Value : TILDataFields);
    procedure SetDataset(const Value : TILDataset);
    procedure SetDatasetMaxCount(const Value : integer);
    procedure SetFilter(const Value : TStringList);
    procedure SetHideInTableList(Value : Boolean);
    procedure SetMultiSelect(const Value : Boolean);
    procedure SetPrecheckConstraints(const Value : Boolean);
    procedure SetPropagateSearchFields(Value : Boolean);
    procedure SetPropertyList(const Value : TILPropertyList);
    procedure SetSaveColumns(const Value : Boolean);
    procedure SetSortKey(const Value : TStringList);
    procedure SetSortOrdering(const Value : TSortOrdering);
    procedure SetSummaryControl(const Value : TILRecordViewer);
    procedure SetTable(const Value : TILTable);
    procedure SetTreeViewControl(const Value : TILTreeView);
    procedure SetTreeViewLabelField(const Value : string);
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function GetImageTagValue(Data : TILDataRecord) : string;
    procedure BeforeDestruction; override;
    property Dataset : TILDataset read GetDataset write SetDataset;
    property DBCommons : TILDBCommons read GetDBCommons;
    property DBView : TILDBView read GetDBView;
    property DBWork : TILDBWork read GetDBWork;
    property ImageTagField : TILDataField read GetImageTagField;
    property MultiSelected : Boolean read GetMultiselected;
  published
    property AfterResume: TNotifyEvent read FAfterResume write FAfterResume;
    property AutoCreateFields : Boolean read FAutoCreateFields write SetAutoCreateFields default True;
    property BeforeFocusRecord: TFocusEvent read FBeforeFocusRecord write FBeforeFocusRecord;
    property BeforeResume: TNotifyEvent read FBeforeResume write FBeforeResume;
    property DataFields : TILDataFields read FDataFields write SetDataFields;
    property DatasetMaxCount : integer read FDatasetMaxCount write SetDatasetMaxCount default 10000;
    property DisableCommandModes : Boolean read FDisableCommandModes write FDisableCommandModes default false;
    property Filter : TStringList read FFilter write SetFilter;
    property FilterState : TFilterState read FFilterState write FFilterState default fsStandard;
    property FreeSearch : Boolean read FFreeSearch write FFreeSearch default false;
    property HideInTableList : Boolean read FHideInTableList write SetHideInTableList default false;
    property ImageControl : TILImage read FImageControl write FImageControl;
    property ImageSource : TImageSource read FImageSource write FImageSource default isNoImages;
    property IsReadOnly: Boolean read GetReadOnly write FIsReadOnly default false;
    property MDOptions : TMDOptions read FMDOptions write FMDOptions default [mdLookupParents];
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect default false;
    property OnCalcField : TCalcFieldEvent read FOnCalcField write FOnCalcField;
    property OnChangeDAC : TChangeDACEvent read FOnChangeDAC write FOnChangeDAC;
    property OnCreateWorkField : TWorkFieldEvent read FOnCreateWorkField write FOnCreateWorkField;
    property OnDatasetChange : TDatasetChangeEvent read FOnDatasetChange write FOnDatasetChange;
    property OnEvaluateField : TEvaluateFieldEvent read FOnEvaluateField write FOnEvaluateField;
    property OnFetchRecord: TSearchEvent read FOnFetchRecord  write FOnFetchRecord;
    property OnFilter : TFilterEvent read FOnFilter write FOnFilter;
    property OnFocusRecord: TNotifyEvent read FOnFocusRecord write FOnFocusRecord;
    property OnFormatProperty : TPropertyEvent read FOnFormatProperty write FOnFormatProperty;
    property OnImageTableRequest : TImageTableEvent read FOnImageTableRequest write FOnImageTableRequest;
    property OnIndexWord : TIndexWordEvent read FOnIndexWord write FOnIndexWord;
    property OnInitRecord : TRecordEvent read FOnInitRecord write FOnInitRecord;
    property OnMissingLink : TLinkEvent read FOnMissingLink write FOnMissingLink;
    property OnSearchLongText : TSearchLongTextEvent read FOnSearchLongText write FOnSearchLongText;
    property OnSelect: TItemEvent read FOnSelect write FOnSelect;
    property OnTrigger : TRecordEvent read FOnTrigger write FOnTrigger;
    property OnUpdateImage : TUpdateImageEvent read FOnUpdateImage write FOnUpdateImage;
    property OnUpdateLongText : TLongTextEvent read FOnUpdateLongText write FOnUpdateLongtext;
    property PrecheckConstraints : Boolean read FPrecheckConstraints write SetPrecheckConstraints default True;
    property PropagateSearchFields : Boolean read FPropagateSearchFields write SetPropagateSearchFields default true;
    property PropertyList : TilPropertyList read FPropertyList write SetPropertyList;
    property SaveColumns : Boolean read FSaveColumns write SetSaveColumns default false;
    property SortKey : TStringList read FSortKey write SetSortKey;
    property SortOrdering : TSortOrdering read FSortOrdering write SetSortOrdering default soAscending;
    property SummaryControl : TILRecordViewer read FSummaryControl write SetSummaryControl;
    property Table : TILTable read FTable write SetTable;
    property TreeViewControl : TILTreeView read FTreeViewControl write SetTreeViewControl;
    property TreeViewLabelField : string read FTreeViewLabelField write SetTreeViewLabelField;
  end;

  TChangeTracker = class
  private
    FChngRecords : Tlist;
    function GetCount : integer;
    function GetChngRecord(Index : integer) : TILDataRecord;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(Index : integer); overload;
    procedure Delete(ARecord : TILDataRecord); overload;
    procedure Add(ARecord : TILDataRecord);
    property Count : integer read GetCount;
    property ChngRecords[Index : integer] : TIlDataRecord read GetChngRecord;
  end;

  TILTable = class(TComponent)
  private
    FActive: Boolean;
    FAlias : string;
    FColumns: TILColumns;
    FCommanded: Boolean;
    FComputerName: string;
    FConnected: Boolean;
    FDataHandler : TILData;
    FDataHandlers : TList;
    FDataName: shortstring;
    FDataset: TILDataset;
    FDatasetMaxCount: integer;
    FDatasetSearch: Boolean; { Flags that output from a search method is sent to Dataset }
    FDateSearchFormat: TDateSearchFormat;
    FDBAdmin: TILDBAdmin;
    FDBCommons: TILDBCommons;
    FDBView: TILDBView;
    FDBWork: TILDBWork;
    FDetailLinks : TList;
    FDisableIndexUpdate: Boolean;
    FDontLookUp: Boolean;{ Sets true when writecontrols runs, to prevent lookups }
    FErrorState : integer;
    FFields: TILFieldDefs;
    FFirstRecordID : integer;
    FFocHandler : TFocusHandler;
    FHwnd : HWnd; { Pointer to invisible window. Used for messaging }
    FImageLocalSize: TImageLocalSize;
    FImageMem: int64; { Stores the memory concumed by the images. Max 10 meg before releasing }
    FImgHScrollPos: integer; { The position of horz scrollbar when ioHidePartialVisible = false
     Toggles between 0 and 1 }
    FImgTxtEditor: TMemo;
    FIndexing : Boolean; { True when UpdateIndexes is running }
    FIndexUpdated : Boolean; { True if index is updated. See RunTransactions}
    FIsReadOnly: Boolean;
    FKeyIDsName: shortstring;
    FKeyList : TList;
    FKeyListName: shortstring;
    FLinkList : TList;
    FLinkListName: shortstring;
    FLookupControl: TListBox;
    FLookupOwner: TWinControl;
    FLookupsAssigned: Boolean; { Prevents reassigning of lookup controls after table is closed and reopened.
     Reassigning resulted in a stack overflow! }
    FMasterLinks : TList;
    FMasterSource: TILDataTable;
    FMultiSelect: Boolean;
    FNamesMaxLength: integer;
    FNextKeyValId: integer; { Index key id generator }
    FNumberMaxLength: integer;
    FParentFields : Boolean;
    FPrecheckConstraints : Boolean;
    FPrivateCalc : Boolean;
    FReadSearchPathControls : Boolean; {True (default) if all the conrols in the path should be read,
     false if only the controls of the table.    Used by the filter system. See ControlsToSQL}
    FRecordFreeSpace : integer;
    FRecordLayout : TRecordLayout;
    FRecordSize : integer;
    FRelations : TRelations;
    FSearcher: TSearcher;
    FSearching : Boolean;
    FSearchOriginator: TILTable; { Set on Mastersource to tell which table initiated the search }
    FSF_Data: TSubFile;     { FSFs: Pointers to DB file }
    FSF_KeyList: TSubFile;
    FSF_LinkList: TSubFile;
    FTableName: string;
    FTableSpace: integer;
    FTextMaxLength: integer;
    FTickStart, FTickEnd: dword;
    FTranslateLookups: Boolean; { Set by RebuildIndexes to prevent GetRecords to translate lookup refs to literals }
    FUpdateTarget: TUpdateTarget;
    FUpmCtrlRecord : TILRecord; {Ensures that ReadControls only invalidates a viewer (to print an update marker)
    when the data conrols are actually changed}
    FViewer : TILDatasetViewer;
    FWordMaxLength: integer;
    FWritingControls : Boolean;
    FXMLDoc: TXMLDocument;

    function AddKey(KeyType : integer; AKey : string; AKeyID : integer; Index : integer) : integer;
    function AddToDataset(AnImage: TPicture): integer; overload;
    function AddToDataset(MediaFile: string): integer; overload;
    function AddToDataset(Data: TILDataRecord): integer; overload;
    function AddToDataset(Data: TILDataRecord; AnImage: TPicture) : integer; overload;
    function AddToDataset(Data: TILDataRecord; MediaFile: string) : integer; overload;
    function AddToDataset(MediaFiles: TStrings; const ReplaceDataset: Boolean = false) : integer; overload;
    function AddToDataset: integer; overload;
    function AddToTable(Data: TILDataRecord): integer; overload;
    function AddToTable(Data: TILDataRecord; const MediaFile: string) : integer; overload;
    function CheckMandatories(Data: TILDataRecord; var Missing: string) : Boolean;
    function CheckSearchExpression(ARecord: TILRecord): integer; overload;
    function CheckSearchExpression(Expression: TStrings): integer; overload;
    function CheckSearchExpression: integer; overload;
    function CheckUpdateExpression(ARecord: TILRecord): integer; overload;
    function CheckUpdateExpression(Expression: TStrings): integer; overload;
    function CheckUpdateExpression: integer; overload;
    function CompareIdents(Record1, Record2: TILDataRecord): TILField;
    function CompareIDS(OldData, NewData: TILDataRecord): Boolean;
    function CompareRecords(Record1, Record2: TILRecord; const Delta: Boolean): Boolean;
    function CompareUniques(Record1, Record2: TILDataRecord): TILField;
    function ComputeFieldSpace(AField : TILFieldDef; const RecSize : Boolean = false) : integer;
    function Connect: integer;
    function CreateDataRecord: TILDataRecord;
    function CreateSearcher(ATable : TILTable; Expr : TStringList) : TSearcher; overload;
    function CreateSearcher(ATable : TILTable; ASearchRecord : TILRecord) : TSearcher; overload;
    function DataCtrlsToExpression: TStringList;
    function DeleteFromTable(const Index: integer): integer; overload;
    function DeleteFromTable(const Index: integer; const Deletions : Boolean): integer; overload;
    function DoReadImages(ImageFiles: TStrings; const ReplaceDataset: Boolean): integer;
    function DoReadXMLFile(const FileName: string; const Target: integer): integer;{ Reads an xml file into dataset }
    function DoTestUpdate(ASourceRecord: TILRecord; ATargetRecord: TILDataRecord; AnExpression: TStringList; var Updated: Boolean): TILTestResult;
    function ExecSQL(Expression: TStrings; const SQLTypes: TSQLTypes = []): integer;
    function Exists(Expression: TStringList; var RecordIndex : integer): Boolean;
    function FindKey(KeyType : integer; AKey : string; var Index : integer) : Boolean;
    function GetActiveSearchCtrlCount: integer;
    function GetActiveSearchCtrls(Index: integer): TILFieldDef;
    function GetAlias : string;
    function GetAlignOperators: Boolean;
    function GetAllChildCount: integer;
    function GetAllChildren(Index: integer): TILTable;
    function GetChildCount: integer;
    function GetChildrn(Index: integer): TILTable;
    function GetColumns : TILColumns;
    function GetConnected: Boolean;
    function GetDatahandler : TILData;
    function GetDataHandlers(Index : integer) : TILData;
    function GetDataHandlerCount : integer;
    function GetDataset : TILDataset;
    function GetDetailLinkCount : integer;
    function GetDetailLinks(Index : integer) : TILDataTable;
    function GetFields : TILFieldDefs;
    function GetFocusedRecord : TILDataRecord;
    function GetIndexSpace: integer;
    function GetJPEGQuality: TJPEGScale;
    function GetKeyWordID(Save : Boolean) : integer;
    function GetLookupClients(const ALookupType : TLookupType; AField : TILFieldDef) : TList;
    function GetLookupKeyField(ALookupTable : TILTable) : TILFieldDef; {Gets the lookup field looTableKey}
    function GetLookupParentValue(LookupTable : TILTable; const LookupField : string; LRec : TILDataRecord): TStrings;
    function GetLookupTableID(LookupTable: TILTable; const LookupField : string; const Value: string): string; { Internal by TILImage }
    function GetLookupTableValue(LookupTable: TILTable; const LookupField : string; const RecordIndex: integer): string;
    function GetMasterLinkCount : integer;
    function GetMasterLinks(Index : integer) : TILDataTable;
    function GetMasterSource: TILDataTable;
    function GetMultiSelected : Boolean;
    function GetNextRecordID : integer;
    function GetParentTable : TILTable;
    function GetReadOnly : Boolean;
    function GetRecordCount: integer;
    function GetRecords(Index: integer): TILDataRecord;
    function GetRecordsAbs(Index: integer): TILDataRecord;
    function GetRecordsX(Index: integer): TILDataRecord;
    function GetResNames(Vals: TStringList): TStringList; { Get the resource names from the resource strings in vals (FValues) }
    function GetRoot: TILTable;
    function GetSearcher(ATable : TILTable) : TSearcher;
    function GetSelCount: integer;
    function GetSelection(Index: integer): TILDataRecord;
    function GetStoreImagesLocally : Boolean;
    function GetThumbnail(const Offset, Count: int64; const Scale : Boolean = True): Vcl.Graphics.TBitmap;
    function GetTVControl : TILTreeView;
    function GetTVTable : TILTreeViewTable;
    function HasDatasetTemps: Boolean;
    function HasHandler : Boolean;
    function IndexOfID(ADataRecord: TILDataRecord; var FieldNames : string): integer;
    function IndexOfUnique(ADataRecord : TILDataRecord; var FieldName : string) : integer;
    function IsControlsEmpty(const Usage: TDataUsage): Boolean;
    function IsDetail: Boolean;
    function IsFieldLookupSource(const ALookupType : TLookupType; AField : TILFieldDef) : Boolean;
    function IsMaster: Boolean;
    function IsRecordOrphan(const RecordIndex: integer): Boolean;
    function IsRoot: Boolean;
    function IsRunning: Boolean;
    function IsStringType(const DataType: TDataType): Boolean;
    function KeyCount(const AValue: string): integer;{ Counts how many records using this value }
    function NoOutput: Boolean; { True if dataset viewer does not exist or has no columns }
    function ReadControls(ARecord: TILRecord; const Usage: TDataUsage): integer;
    function Search(ASearchRecord: TILRecord): integer; overload;
    function Search(Expression: TStringList): integer; overload;
    function SearchCtrlsToExpression(Expression: TStringList; const IncludeDetailTables: Boolean = false): integer;
    function SearchDeleted: TILDataset;
    function SearchDownTree(StartTable: TILTable; const StartID: integer; EndTable: TILTable): TTreeData;
    function SearchUpLink(Expression: TStringList; ParentTable: TILTable; AChildRecord: TILDataRecord): TILDataRecord;
    function SearchUpTree(Expression: TStringList; ParentTable: TILTable; AChildRecord: TILDataRecord): TILDataRecord;
    function SetDateNumberFormat(const AValue: string; const ADataType: TDataType; const DecimalCount: integer; const UseLocale : Boolean = false): string;
    function TestUpdate(ASourceRecord: TILRecord; ATargetRecord: TILDataRecord; var Updated: Boolean): TILTestResult; overload;
    function TestUpdate(ATargetRecord: TILDataRecord; Expression: TStrings; var Updated: Boolean): TILTestResult; overload;
    function TestUpdateDataset(Expression: TStringList): TILUpdateTest; overload;
    function TestUpdateDataset: TILUpdateTest; overload;
    function Undelete(const Index: integer): Boolean;
    function UpdateDataset(AnUpdateRecord: TILRecord): integer; overload;
    function UpdateDataset: integer; overload;
    function UpdateRecord(ASourceRecord: TILRecord; ATargetRecord: TILDataRecord): integer; overload;
    function UpdateRecord(ATargetRecord: TILDataRecord; Expression: TStringList): integer; overload;
    procedure AlterField(const FieldName: string; const Alter: TAlterFieldOption; const Value: Variant);
    procedure AlterIndexSpace;
    procedure AlterRecordFreeSpace;
    procedure AlterRecordLayout(const NewLayout : TRecordLayout);
    procedure AlterRecordSize;
    procedure AlterTable(const Option: TILOperation; NewSpace: integer; NewIndexes, NewLookups: TStringList);
    procedure AlterTableSpace(const NewSpace: integer);
    procedure BuildTextIndex(DataRecord: TILDataRecord; const FieldName: string; const RecordIndex: integer);
    procedure CalcControls;
    procedure ClearControls(const Usage: TDataUsage; AControl : TWinControl = nil);
    procedure ClearEdits; { Stups cell editing in dataset viewer if actrive}
    procedure ClearIndex;
    procedure ClearTable;
    procedure ClearTextIndex(const RecordIndex: integer; const FieldName: string);
    procedure CloseTable;
    procedure ControlsExit(Sender: TObject);
    procedure ControlsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure DeleteImage(const AnIndex: integer);
    procedure Disconnect;
    procedure DoAddToDataset(ADataRecord: TILDataRecord; const AnImageFile: string; AnImage: TPicture);
    procedure DoAddToTable(ADataRecord: TILDataRecord);
    procedure DoDeselectAll;
    procedure GetDetails; { fetches detail records }
    procedure GetImgDimensions(var W, H: integer); { Gets image store dimenisons }
    procedure IncludeParentFields(DataRec : TILDataRecord);
    procedure Inval;
    procedure LoadKeyList;
    procedure LoadLinkList;
    procedure LookUpClick(Sender: TObject);
    procedure OpenTable(const AFileName: string);
    procedure RebuildIndexes;
    procedure RenameTable(OldName, NewName : string);
    procedure ResetDataControls(SelCnt : integer = -1); { See code }
    procedure ResetDataset;
    procedure ResetOldValues(ADataRecord: TILDataRecord); { Clears Old values }
    procedure RestoreResourceStrings(ARecord : TILDatarecord);
    procedure SaveColumns;
    procedure SaveImage(AThumbnail: Vcl.Graphics.TBitmap; var Offset, Count: int64);
    procedure SaveIndex;
    procedure SaveKeyList;
    procedure SaveLinkList;
    procedure SaveResourceStrings(ARecord : TILDataRecord);
    procedure SetConnected(const Value : Boolean);
    procedure SetData(const Value: TILFieldDefs);
    procedure SetDataset(const Value : TILDataset);
    procedure SetDatasetMaxCount(const Value : integer);
    procedure SetInitials(ADataRecord : TILDataRecord);
    procedure SetLogOp(const FieldName: string; const LogOp: TFieldLogOp); overload;
    procedure SetLogOp(const FieldName: string; const LogOp: TValueLogOp); overload;
    procedure SetLogOp(const LogOp: TFieldLogOp); overload;
    procedure SetLogOp(SearchControl: TWinControl; const LogOp: TFieldLogOp); overload;
    procedure SetLogOp(SearchControl: TWinControl; const LogOp: TValueLogOp); overload;
    procedure SetMultiSelect(const Value : Boolean);
    procedure SetNames(ATable: TILTable); { Sets names to essential paths and table sub file names }
    procedure SetRecords(Index: integer; Value: TILDataRecord);
    procedure SetRecordStates(ARecord : TILDataRecord);{ Sets the Old values of ARecord
     (OldImage, OldFileName, etc) to the actual Values of ARecord, Usede by Records, RecordsX and RecordsAbs getters}
    procedure SetTableName(const Value : string);
    procedure TextDifference(OldText, NewText, Adds, Deletes: TStringList);
    procedure UpdateImage(const AnIndex: integer; const AFileName: string);
    procedure UpdateIndexes(ADataRecord: TILDataRecord; const ATableIndex: integer; const AMovedRecord : integer);
    procedure UpdateTextIndex(DataRecord: TILDataRecord; const FieldName: string; RecordIndex: integer);
    procedure WndMethod(var Msg : TMessage);
    procedure WriteControls(ARecord: TILRecord; const Usage: TDataUsage; const SkipInvisibles : Boolean = false);
    property ActiveSearchCtrlCount: integer read GetActiveSearchCtrlCount;
    property ActiveSearchCtrls[Index: integer]: TILFieldDef read GetActiveSearchCtrls;
    property AlignOperators: Boolean read GetAlignOperators;
    property Connected: Boolean read GetConnected write SetConnected;
    property DatasetMaxCount: integer read FDatasetMaxCount write SetDatasetMaxCount;
    property DateSearchFormat: TDateSearchFormat read FDateSearchFormat write FDateSearchFormat default dsfDate;
    property DisableIndexUpdate: Boolean read FDisableIndexUpdate write FDisableIndexUpdate;
    property FirstRecordID : integer read FFirstRecordID;
    property ImageLocalSize: TImageLocalSize read FImageLocalSize;
    property LookupControl: TListBox read FLookupControl;
    property ParentFields : Boolean read FParentFields;{ True when parent fields is available for viewing.
     Created as looParentTable }
    property PrecheckConstraints : Boolean read FPrecheckConstraints write FPrecheckConstraints;
    property RecordFreeSpace : integer read FRecordFreeSpace;
    property RecordLayout : TRecordLayout read FRecordLayout;
    property RecordSize : integer read FRecordSize;
    property Searcher: TSearcher read FSearcher;
    property StoreImagesLocally : Boolean read GetStoreImagesLocally;
    property TableSpace: integer read FTableSpace;
    property TVControl : TILTreeView read GetTVControl; { Short hand ref to tiltable.TreeViewControl instance }
    property TVTable : TILTreeViewTable read GetTVTable; { Short hand ref to tiltable instance }
    property UpdateTarget: TUpdateTarget read FUpdateTarget write FUpdateTarget;
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property AllChildCount: integer read GetAllChildCount;
    property AllChildren[Index: integer]: TILTable read GetAllChildren;
    property ChildCount: integer read GetChildCount;
    property Children[Index: integer]: TILTable read GetChildrn;
    property DetailLinkCount : integer read GetDetailLinkCount;
    property DetailLinks[Index : integer] : TILDataTable read GetDetailLinks;
    property MasterLinkCount : integer read GetMasterLinkCount;
    property MasterLinks[Index : integer] : TILDataTable read GetMasterLinks;
    property MasterSource: TILDataTable read GetMasterSource;
    property ParentTable : TILTable read GetParentTable;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ComputeAllFieldsSpace : integer; { Used by designer app }
    function ComputeRecordSize : integer;  { Used by designer app }
    function Count : integer; { Counts number of active recors in table }
    function DeletedCount : integer;
    function GetImage(const DatasetIndex: integer): Vcl.Graphics.TBitmap; overload;
    function GetImage(DataRecord: TILDataRecord): Vcl.Graphics.TBitmap; overload;
    function GetLookups(const AFieldName : string; AnExpression: string; const Usage : TDataUsage): TStringList;
    function HasViewer : Boolean;
    function IsControlActive(AControl: TControl): Boolean; { Used by acton bar }
    function IsFreeTable : Boolean;
    function IsTableEmpty : Boolean;
    function ReadXMLFile(const FileName: string): integer; { Reads records from a xml file }
    function RecordToJSON(ARecord : TILDatarecord) : string;
    function RecordToXML(ARecord: TILDataRecord; const KeepFieldNames : Boolean; const ConvertLookups : Boolean = false): string;
    function XMLToRecord(const ALibstring: string; const SkipImage: Boolean = false; FieldSource : TILFieldDefs = nil) : TILDataRecord; { Used by deginer app}
    procedure DeselectAll;
    procedure SaveAsJSON(const AFileName: string); { Saves records to json file }
    procedure SaveAsXML(const AFileName: string); { Saves records to xml file }
    procedure SelectAll;
    procedure SetLookup(AControl: TWinControl; AField: TILFieldDef);
    procedure SetLookups(const Reset : Boolean = false);
    procedure ShowLookups(AParentControl: TWinControl); overload;
    procedure ShowLookups(AParentControl: TWinControl; FieldName: string); overload;

    property Alias : string read GetAlias write FAlias;
    property Columns: TILColumns read GetColumns;
    property ComputerName: string read FComputerName;
    property DataHandler : TILData read GetDataHandler;
    property DataHandlerCount : integer read GetDataHandlerCount;
    property DataHandlers[Index : integer] : TILData read GetDataHandlers;
    property Dataset: TILDataset read GetDataset write SetDataset;
    property DBAdmin: TILDBAdmin read FDBAdmin; { Admin namespace }
    property DBCommons: TILDBCommons read FDBCommons; { Common db functions name space }
    property DBView: TILDBView read FDBView; { Search namespace }
    property DBWork: TILDBWork read FDBWork; { Update namespace }
    property ErrorState : integer read FErrorState write FErrorState; { Used by designer app }
    property Fields: TILFieldDefs read GetFields write SetData;
    property FocusedRecord : TILDataRecord read GetFocusedRecord;
    property IndexSpace: integer read GetIndexSpace; { Used by designer app }
    property IsReadOnly: Boolean read GetReadOnly write FIsReadOnly default false;
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect;
    property MultiSelected: Boolean read GetMultiSelected;
    property RecordCount: integer read GetRecordCount; { Number of records in table, deleted included } { Multiselect is also a TILData prop. Keeping it here allows multi select without using TILData }
    property RecordsAbs[Index : Integer] : TILDataRecord read GetRecordsAbs; { Records fetched with aboslute index, ignoring moved references }
    property Records[Index: integer]: TILDataRecord read GetRecords  write SetRecords;
    property RecordsX[Index : Integer] : TILDataRecord read GetRecordsX; { Records without image }
    property Relations : TRelations read FRelations;
    property Root: TILTable read GetRoot;
    property SelCount: integer read GetSelCount;
    property Selection[Index: integer]: TILDataRecord read GetSelection;
    property TickStart: dword read FTickStart;
    property TickEnd: dword read FTickEnd;
    property Viewer : TILDatasetViewer read FViewer write FViewer;
  published
    property TableName: string read FTableName write SetTableName;
  end;

  TILDataTable = class(TILTable)
  private
    FLocatingLinkedParent : Boolean;
    { True when running GetLinkedParent, to avoid GetLinkedRecords to execute }
    procedure FetchParentFields(ARecord : TILDatarecord);
    procedure UpdateParentField(ARecord : TILDataRecord);
    procedure UpdateTableKey(const AKeyField : string; const ANewValue, AnOldValue : string);
  public
    function CopyTable : TILDataTable;
    function FindLinkedRecords : Boolean;
    function GetFromImageTable(const TagName: string; TargetRecord: TILDataRecord) : Boolean;
    function GetLinkedParentRecord(AChildRecord : TILDataRecord;
     const ALinkField : string; AParentTable : TILDataTable) : Boolean;
    function GetLinkedRecords(AParentRecord : TILDataRecord; const ALinkField : string) : integer;
    procedure AddToImageTable(const TagName: string; SourceRecord: TILDataRecord);
    procedure AddUserCalcField(FieldName : string; const DataType : TDataType;
     ColWidth : integer; const Formula : string; const CreateSearchControl : Boolean;  const Save : Boolean);
    procedure DeleteUserCalcField(const FieldName : string; const AMessage : string);
    procedure EditUserCalcField(const FieldName : string; const DataType : TDataType;
     const ColWidth : integer; const Formula : string; const Save : Boolean);
    procedure GetChildRecords(AChildTable : TILDataTable);
    procedure GetUserCalcProps(const FieldName : string; var DataType : TDataType;
     var ColWidth : integer; var Formula : string; var Save : Boolean);
    property AllChildCount;
    property AllChildren;
    property ChildCount;
    property Children;
    property DateSearchFormat;
    property DetailLinkCount;
    property DetailLinks;
    property Fields;
    property MasterLinkCount;
    property MasterLinks;
    property MasterSource;
    property ParentTable;
  end;

  TILView = class(TILTable)
  private
    FSourceTable : TILTable;
    procedure SetSourceTable(Value : TILTable);
  public
    constructor Create(AComponent : TComponent); override;
    procedure AssignSource;
  published
    property SourceTable : TILTable read FSourceTable write SetSourceTable;
  end;

  TILMemTable = class(TILTable)
  private
    FBaseTables : TStrings;
    FOnInvalidateTable : TNotifyEvent;
    FTableRecords : TILDataset;
    procedure SetBaseTables(Value : TStrings);
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function RecordOf(const ARecordIndex : integer) : TILDataRecord;
    procedure RecreateRecords;
    property TableRecords : TILDataset read FTableRecords;
  published
    property BaseTables : Tstrings read FBaseTables write SetBaseTables;
    property Fields;
    property OnInvalidateTable : TNotifyEvent read FOnInvalidateTable write FOnInvalidateTable;
  end;

  TILSaveIndex = class(TThread)
  private
  protected
    procedure Execute; override;
  end;

  TILLoadIndex = class(TThread)
  protected
    procedure Execute; override;
  end;

  TILTreeViewParams = class
  private
    FFocusReferenced : Boolean; {Always true}
    FIlib: TILTreeViewTable;
    FInsertMode : TTVInsertMode;
    FInsertParentID: integer;
    FInsertParentRecord: TILDataRecord;
    FInsertSiblingIndex: integer;
    FRecursive: Boolean;
    FRootCondition: TILRecord;
    FSearchCtrlMode: TSearchCtrlMode;
    FSortingMethod: TTreeViewSorting;
    FWhereCondition: TILRecord;
    function GetInsertParentRecord : TILDataRecord;
    procedure SetInsertMode(const Value : TTVInsertMode);
    procedure SetSearchCtrlMode(const Value : TSearchCtrlMode);
    procedure SetSortingMethod(const Value : TTreeViewSorting);
  public
    constructor Create(AnOwner: TILTreeViewTable);
    destructor Destroy; override;
    procedure Assign(Source : TILTreeViewParams);
    procedure ChangeCtrlMode(const AMode : TSearchCtrlMode);
    procedure SetInsertAfter(ASiblingRecord : TILDataRecord);
    procedure SetInsertBefore(ASiblingRecord : TILDataRecord);
    procedure SetInsertChild(AParentRecord : TILDataRecord);
    property Ilib: TILTreeViewTable read FIlib;
    property InsertMode : TTVInsertMode read FInsertMode write SetInsertMode;
    property InsertParentID: integer read FInsertParentID write FInsertParentID;  {Not used}
    property InsertParentRecord : TILDataRecord read GetInsertParentRecord write FInsertParentRecord;
    property InsertSiblingIndex: integer read FInsertSiblingIndex write FInsertSiblingIndex;
     { Not to be changed. Maintained by ILib}
    property Recursive: Boolean read FRecursive write FRecursive;
    property RootCondition: TILRecord read FRootCondition write FRootCondition;
    property SearchCtrlMode: TSearchCtrlMode read FSearchCtrlMode write SetSearchCtrlMode;
    property SortingMethod: TTreeViewSorting read FSortingMethod write SetSortingMethod;
    property WhereCondition: TILRecord read FWhereCondition write FWhereCondition;
  end;

  TILTreeViewTable = class(TILDataTable)
  private
    FRootSiblings : TStringList;
    FTreeViewControl: TILTreeView;
    FTreeViewLabelField: string;
    FTreeViewParams: TILTreeViewParams;
    function GetSiblings(ParentID, Index: integer): TILDataRecord;
    function Circular(const ParentID, ChildID: integer): Boolean;
    function GetParentID(ChildID: integer): integer;
    procedure SetTreeViewControl(const Value : TILTreeView);
    procedure SetTreeViewLabelField(const Value : string);
    function GetLevel : integer;
    function GetSiblingIndex : integer;
    procedure RenumberSiblings(AParentRecord : TILDataRecord; const NewSiblingIndex : integer);
    procedure FillSiblings(ParentRecord : TILDataRecord);
    procedure AddToSiblingList(Index : integer; ParentRecord, Sibling : TILDataRecord);
    procedure DeleteSiblingList(ParentRecord, Sibling : TILDataRecord);
    procedure MoveSibling(NewIndex : integer; Sibling : TILDataRecord);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CollapseNode(ParentRecord: TilDataRecord): integer;
    function ExpandNode(ParentRecord: TilDataRecord; Recursive: Boolean): integer;
    function GetParentRecord(ChildID: integer): TILDataRecord; overload;
    function GetParentRecord(ChildRecord: TILDataRecord): TILDataRecord; overload;
    function GetPathNodes(ParentID: integer): TILDataset; overload;
    function GetPathNodes(ParentRecord: TILDataRecord): TILDataset; overload;
    function GetTVChildCount(ParentID: integer): integer; overload;
    function GetTVChildCount(ParentRecord: TILDataRecord): integer; overload;
    function GetTVChildren(ParentID: integer): TILDataset; overload;
    function GetTVChildren(ParentRecord: TILDataRecord): TILDataset; overload;
    function InPath(ParentRecord, ChildRecord : TILDataRecord; IncludeRoot : Boolean) : Boolean;
    function MoveNode(ARecordIndex: integer; NewParentID: integer; NewSiblingIndex: integer ): integer; overload;  { Depricated}
    function MoveNode(SourceNode : TILDataRecord; NewParent: TILDataRecord; SiblingIndex: integer ): integer; overload;
    procedure InsertAfter(ASiblingRecord : TILDataRecord; ReadCtrls : Boolean);
    procedure InsertBefore(ASiblingRecord : TILDataRecord; ReadCtrls : Boolean);
    procedure InsertChild(AParentRecord : TILDataRecord; ReadCtrls : Boolean);
    procedure InsertRoot(ReadCtrls : Boolean);
    property ParentID[ChildID: integer]: integer read GetParentID;
    property Siblings[ParentID, Index: integer]: TILDataRecord read GetSiblings;
    property TreeViewControl: TILTreeView read FTreeViewControl write SetTreeViewControl;
    property TreeViewLabelField: string read FTreeViewLabelField write SetTreeViewLabelField;
    property TreeViewParams: TILTreeViewParams read FTreeViewParams write FTreeViewParams;
  end;

  TILLookupTable = class(TILTable)
  private
    FFieldName: string;
    function GetLookupFieldName: string;
    procedure CreateLookupField(const AFieldName: string; const ColumnWidth: integer = 50);
    procedure SetLookupFieldName(const Value : string);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    property FieldName: string read GetLookupFieldName write SetLookupFieldName;
  end;

  TILImageTable = class(TILLookupTable)
  public
    constructor Create(AOwner: TComponent); override;
    function GetBitmap(const RecordIndex: integer; const Size: integer) : Vcl.Graphics.TBitmap; overload;
    function GetBitmap(const TagName : string; const Size: integer) : Vcl.Graphics.TBitmap; overload;
    function IndexOfTagName(const TagName : string) : integer;
    function ShareWith(const TagName: string; TargetRecord: TILDataRecord) : Boolean; overload;
    function ShareWith(TargetRecord: TILDataRecord) : Boolean; overload;
    procedure Add(const TagName: string; const FileName: string); overload;
    procedure Add(const TagName: string; Image: TPicture); overload;
    procedure Add(const TagName: string; SourceRecord: TILDataRecord); overload;
    procedure Add(Files: TStringList); overload;
    procedure ChangeImage(const FileName : string); overload;
    procedure ChangeImage(Image : TPicture); overload;
  end;

  TILWorkField = class
  private
      FCommand : TCommandMode;
      FControl : TWinControl;
      FCreator : TWorkFieldCreator;
      FExpandControl : TILCustomGrid;
      FExpanded : Boolean;
      FField : TILFieldDef;
      FFieldLogOpButton : TILLogOpButton;
      FIsVisible : Boolean;
      FLabel : TLabel;
      FOrigControl : TWinControl;
      FPanel : TILControlPanel;
      FTable : TILTable;
      FUsage : TDataUsage;
      FValueLogOpButton : TILLogOpButton;
      function GetEnabled : Boolean;
      function GetVisible : Boolean;
      procedure SetEnabled(const Value : Boolean);
      procedure SetVisible(const Value : Boolean);
      property IsVisible : Boolean read FIsVisible write FIsVisible;
  public
      function CanCollapse : Boolean;
      function CanExpand : Boolean;
      procedure Collapse;
      procedure Expand(RowCount : integer = 3);
      property Command : TCommandMode read FCommand write FCommand;
      property Control : TWinControl read FControl write FControl;
      property ControlLabel : TLabel read FLabel write FLabel;
      property Creator : TWorkFieldCreator read FCreator;
      property Enabled : Boolean read GetEnabled write SetEnabled;
      property Expanded : Boolean read FExpanded;
      property Field : TILFieldDef read FField write FField;
      property FieldLogOpButton : TILLogOpButton read FFieldLogOpButton write FFieldLogOpButton;
      property Panel : TILControlPanel read FPanel;
      property Table : TILTable read FTable write FTable;
      property Usage : TDataUsage read FUsage write FUsage;
      property ValueLogOpButton : TILLogOpButton read FValueLogOpButton write FValueLogOpButton;
      property Visible : Boolean read GetVisible write SetVisible;
  end;

  TILControlPanel = class(TPanel)
    private
      FControlList : TList;
      FControlSpacing : integer;
      FControlType : TControlType;
      FCtrlsLeft : integer;
      FCtrlsTop : integer;
      FDataSource : TILData;
      FDeletionUSG : TDatausage; { Flag used by TILTable.Notification to signal what kind of label must be
       deleted when a dac is deleted }
      FDualCtrlOptions : TDualCtrlOptions;
      FEnableScroller : Boolean;
      FItems : TList;
      FLabelSpace : integer;
      FLogOpButton : TILLogOpButton;
      FOnDrawLogOpButton : TDrawLogOpButtonEvent;
      FShowLogopButtons : Boolean;
      FSourceBuild : Boolean;

      function GetAbsoluteHeight : integer;
      function GetAbsoluteWidth : integer;
      function GetCount : integer;
      function GetEnableScroller : Boolean;
      function GetILWorkField(Index : integer) : TILWorkField;
      function GetTVParams : TILTreeViewParams;
      function GetVisibleCount : integer;
      function GetVisibles(Index : integer) : TILWorkField;
      procedure CheckCPScroller;
      procedure ComputeLabelSpace;
      procedure CreateLogOpButtons;
      procedure DoShowCtrl(const Index : integer; AnItem : TILWorkField; X, Y : integer);
      procedure RemoveLogopButtons;
      procedure SetControlSpacing(const Value : integer);
      procedure SetCtrlsLeft(const Value : integer);
      procedure SetCtrlsTop(const Value : integer);
      procedure SetDataSource(const Value : TILData);
      procedure SetEnableScroller(const Value : Boolean);
      procedure SetShowLogOpButtons(const Value : Boolean);
      procedure ShowCtrls(Ctrls : TList); overload;
      procedure ShowCtrls(Ctrls : TStrings); overload;
    protected
      procedure AlignControls(AControl : TControl; var Rect : TRect); override;
      procedure Loaded; override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    public
      constructor Create(AOwner : TComponent); override;
      destructor Destroy; override;
      procedure Clear;
      procedure CreateItems(Init : Boolean = False);
      procedure EnableCtrls(Enable : Boolean);
      procedure GenerateControls(const Value : TControlType);
      procedure ShowCtrls(ATable : TILTable; AUsage : TDataUsage); overload;
      property AbsoluteHeight : integer read GetAbsoluteHeight;
      property AbsoluteWidth : integer read GetAbsoluteWidth;
      property Count : integer read GetCount;
      property Items[Index : integer] : TILWorkField read GeTILWorkField;
      property LogOpButton : TILLogOpButton read FLogOpButton write FLogOpButton; { Field aligned log op}
      property TVParams : TILTreeViewParams read GetTVParams;
      property VisibleCount : integer read GetVisibleCount;
      property Visibles[Index : integer] : TILWorkField read GetVisibles;
    published
      property ControlSpacing : integer read FControlSpacing write SetControlSpacing default 3;
      property CtrlsLeft : integer read FCtrlsLeft write SetCtrlsLeft default 5;
      property CtrlsTop : integer read FCtrlsTop write SetCtrlsTop default 5;
      property DataSource : TILData read FDataSource write SetDataSource;
      property DualCtrlOptions : TDualCtrlOptions read FDualCtrlOptions write FDualCtrlOptions default [doLogPicklist];
      property EnableScroller : Boolean read GetEnableScroller write SetEnableScroller default false;
      property OnDrawLogOpButton : TDrawLogOpButtonEvent read FOnDrawLogOpButton write FOnDrawLogOpButton;
      property ShowLogOpButtons : Boolean read FShowLogOpButtons write SetShowLogopButtons default false;
  end;

  TILLogOpButton = class(TGraphicControl)
  private
    FBordered : Boolean;
    FLogOpType : TLogOpType;
    FOnDrawButton : TDrawLogOpButtonEvent;
    FSearchControl : TWinControl;
    FState : TLogOp;
    FTable : TILTable;
    function GetFieldDef : TILFieldDef;
    procedure SetBordered(const Value : Boolean);
    procedure SetLogopType(const Value : TlogOpType);
    procedure SetSearchControl(const Value : TWinControl);
    procedure SetState(const Value : TLogOp);
    procedure SetTable(const Value : TILTable);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner : TComponent); override;
    property FieldDef : TILFieldDef  read GetFieldDef;
  published
    property Bordered : Boolean read FBordered write SetBordered default true;
    property Color;
    property Enabled;
    property Font;
    property LogOpType : TLogOpType read FLogOpType write SetLogOpType default ltFieldAligned ;
    property OnDrawButton : TDrawLogOpButtonEvent read FOnDrawButton write FOnDrawButton;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property SearchControl : TWinControl read FSearchControl write SetSearchControl;
    property ShowHint;
    property State : TLogOp read FState write SetState default loAnd;
    property StyleElements;
    property Table : TILTable read FTable write SetTable;
    property Visible;
  end;

  TILCustomGrid = class(TStringGrid)
  private
    { Private declarations }
     FAlignValueCol : Boolean;
     FAutoSizeHeight : Boolean;
     FCBKeyDown : Boolean;
     FComboBox: TILComboBox;
     FComboCol : integer;
     FComboRow : integer;
     FDropStyleChanged : Boolean;
     FField : TILFieldDef;
     FOldCol, FOldRow : integer;
     FOnChange : TNotifyEvent;
     FPickValues : TStringList;
     FSemafor : Boolean;
     function GetInPlaceText : string;
     function GetPickValues : TStringList;
     function GetTable : TILTable;
     function GetValue(Index : integer) : string;
     function GetValueRowCount : integer;
     function HasCellCombo(const ACol, ARow : integer) : Boolean;
     procedure ComboChanged(Sender  : TObject);
     procedure ComboKeyDown(Sender : TObject; var Key : Word; Shift : TShiftstate);
     procedure DoAlignValueCol; virtual; abstract;
     procedure DoAutoSizeHeight;
     procedure HideCombobox;
     procedure SetAlignValueCol(const Value : Boolean);
     procedure SetAutoSizeHeight(const Value : Boolean);
     procedure SetField(const Value : TILFieldDef);
     procedure SetValue(Index : integer; Value : string);
     procedure SetValueRowCount(const Value : integer);
     procedure SupplyOperators; virtual;
     procedure WMCommand( var msg: TWMCommand ); message WM_COMMAND;
     procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
     procedure WMHScroll(var Message : TWMHScroll); message WM_HScroll;
     procedure WMLOOKUPS(var Message: TMessage); message WM_Lookups;
     procedure WMVScroll(var Message : TWMHScroll); message WM_VScroll;
  protected
    { Protected declarations }
    procedure Resize; override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public declarations }
     Constructor Create(AOWner:TComponent);override;
     Destructor Destroy;override;
     function ValueColIndex : integer; virtual; abstract;
     procedure ClearValues; virtual; abstract;
     procedure RelocateComboBox; virtual; abstract;
     procedure ResetColumns; virtual; abstract;
     property Combo : TILComboBox read FComboBox;
     property Field : TILFieldDef read FField write SetField;
     property InPlaceText : string read GetInplaceText;
     property PickValues : TStringList read GetPickValues;
     property Table : TILTable read GetTable;
     property Values[Index : integer] : string read GetValue write SetValue;

  published
    { Published declarations }
     property AlignValueCol : Boolean read FAlignValueCol write SetAlignValueCol default false;
     property AutoSizeHeight : Boolean read FAutoSizeHeight write SetAutoSizeHeight default false;
     property OnChange : TNotifyEvent read FOnChange write FOnChange;
     property ValueRowCount : integer read GetValueRowCount write SetValueRowCount;
  end;

  TILSearchGrid = class(TILCustomGrid)
  private
    { Private declarations }
     FCoPicklist : Boolean;
     FLogPicklist : Boolean;
     FShowFieldLogOp : Boolean;
     function GetLogOp(Index : integer) : string;
     function GetCoOp(Index : integer) : string;
     function GetValueColWidth : integer;
     procedure DoAlignValueCol; override;
     procedure SetValueColWidth(const Value : integer);
     procedure SupplyOperators; override;
     procedure WMALIGNLOGOPS(var Message: TMessage); message WM_AlignLogOps;
     procedure WMSHOWVALUECOMBO(var Message: TMessage); message WM_ShowValueCombo;
     function GetLogColWidth : integer;
     function GetCoColWidth : integer;
     procedure SetLogColWidth(const Value : integer);
     procedure SetCoColWidth(const Value : integer);
     procedure SetCoPickList(const Value : Boolean);
     procedure SetLogPickList(const Value : Boolean);
     procedure SetShowFieldLogOp(const Value : Boolean);
  protected
    { Protected declarations }
    function SelectCell(ACol, ARow : Longint) : Boolean; override;
    procedure Click; override;
    procedure KeyDown(var Key: Word; Shift : TShiftState); override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;  X, Y: integer); override;

  public
    { Public declarations }
     constructor Create(AOWner:TComponent);override;
     function CoOpColIndex : integer;
     function LogOpColIndex : integer;
     function ValueColIndex : integer; override;
     procedure ClearValues; override;
     procedure RelocateComboBox; override;
     procedure ResetColumns; override;
     property CoOp[Index : integer] : string read GetCoOp;
     property LogOp[Index : integer] : string read GetLogOp;

  published
    { Published declarations }
     property CoOpColWidth : integer read GetCoColWidth write SetCoColWidth;
     property CoPickList : Boolean read FCoPickList write SetCoPickList default true;
     property LogOpColWidth : integer read GetLogColWidth write SetLogColWidth;
     property LogPickList : Boolean read FLogPickList write SetLogPickList default True;
     property ShowFieldLogOp : Boolean read FShowFieldLogOp write SetShowFieldLogOp default true;
     property ValueColWidth : integer read GetValueColWidth write SetValueColWidth;
  end;

  TILUpdateGrid = class(TILCustomGrid)
  private
    FMode : TUpdateGridMode;
    FTranslations : TStringList;
    function GetTagValue(Index : integer) : string;
    procedure DoAlignValueCol; override;
    procedure SetMode(const Value : TUpdateGridMode);
    procedure SupplyOperators; override;
    procedure WMSHOWACTIONCOMBO(var Message: TMessage); message WM_ShowActionCombo;
  protected
    procedure Click; override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: integer); override;
  public
    Constructor Create(AOWner:TComponent);override;
    destructor Destroy; override;
    function GetActionColWidth(Cnvs : TCanvas = nil) : integer;
    function GetTranslation(Tagname : string) : string;
    function GetTranslTag(Transl : string) : string;
    function StartRow : integer;
    function ValueColIndex : integer; override;
    procedure ClearValues; override;
    procedure RelocateComboBox; override;
    procedure ResetColumns; override;
    property TagValue[Index : integer] : string read GetTagValue;
  published
    property Mode : TUpdateGridMode read FMode write SetMode default gmSimple;
  end;

  TILRecordViewer = class(TCustomListView)
  private
     FFieldCaption : string;
     FImageControl : TILImage;
     FOnStateImage : TStateImageEvent;
     FShowEmptyFields : Boolean;
     FValueCaption : string;
     FViewElements : TRecordElements;
     procedure SetFieldCaption(const Value : string);
     procedure SetImageControl(const Value : TILImage);
     procedure SetValueCaption(const Value : string);
  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure ShowRecord(ARecord : TILDataRecord);
  published
    property Align;
    property FieldCaption : string read FFieldCaption write SetFieldCaption;
    property FlatScrollBars;
    property Font;
    property GridLines;
    property HideSelection;
    property HotTrack;
    property ImageControl : TILImage read FImageControl write SetImageControl;
    property Items;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnInfoTip;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnSelectItem;
    property OnStartDock;
    property OnStartDrag;
    property OnStateImage : TStateImageEvent read FOnStateImage write FOnStateImage;
    property ParentColor;
    property ParentShowHint;
    property ShowEmptyFields : Boolean read FShowEmptyFields write FShowEmptyFields default false;
    property ShowHint;
    property StateImages;
    property ValueCaption : string read FValueCaption write SetValueCaption;
    property ViewElements : TRecordElements read FViewElements write FViewElements default [reData];
  end;

  TILSQLParams = class
  private
    FItems : TList;
    FOwner : TILSQL;
    function Add : TILSQLParam;
    function GetCount : integer;
    function GetItems(Index : integer) : TILSQLParam;
  public
    constructor Create;
    destructor Destroy; override;
    function ParamByName(const AName : string) : TILSQLParam;
    procedure Clear;
    procedure ClearValues;
    property Count : integer read GetCount;
    property Items[Index : integer] : TILSqlParam read GetItems; default;
    property Owner : TILSQL read FOwner;
  end;

  TILSQLParam = class
  private
    FCndOperator : string;
    FFieldName : string;
    FParamName : string;
    FParams : TILSQLParams;
    FParamValue : Variant;
    FTableName : string;
    FValueField : TILFieldDef;
    FValueTable : TILTable;
    function GetFieldDef : TILFieldDef;
    procedure SetCndOperator(const Value : string);
    procedure SetParamValue(const Value : Variant);
  public
    property CndOperator : string read FCndOperator write SetCndOperator;
    property FieldDef : TILFieldDef read GetFieldDef;
    property FieldName : string read FFieldName;
    property ParamName : string read FParamName;
    property Params : TILSQLParams read FParams;
    property ParamValue : Variant read FParamValue write SetParamValue;
    property TableName : string read FTableName;
  end;

  TILSQL = class(TComponent)
  private
   FIsFilter : Boolean; { True if used by the filter system}
   FNoCheck : Boolean;
   FParams : TILSQLParams;
   FSQLStatement : TStringList;
   FTargetTable : TILTable;
   FValuedSQL : TStringList;
   function GetSQLType : TSQLType;
   function ParseStatement : integer;
   procedure InitParams;
   procedure SetSQLStatement(const Value : TStringlist);
  protected
  public
   constructor Create(AnOwner : TComponent); override;
   destructor Destroy; override;
   function CheckStatement : integer;
   function Execute : integer;
   function StringsToVarArray(Str : TStrings) : Variant;
   procedure SetValues;
   property Params : TILSQLParams read FParams;
   property SQLType : TSQLType read GetSQLType;
  published
    property SQLStatement : TStringList read FSQLStatement write SetSQLStatement;
    property TargetTable : TILTable read FTargetTable write FTargetTable;
  end;

  TTranslator = class
  private
    FEnFormatSettings : TFormatSettings;
    FFormatSettings : TFormatSettings;
    FTranslateFieldCaps : TTranslateCaps;
    FTranslateTableCaps : TTranslateCaps;
    function GetLanguage : string;
    procedure SetLanguage(Language : string);
  public
    constructor Create;
    destructor Destroy; override;
    function FormatDT(const ADate : TDateTime; const AFormat : string) : string;
    function FormatNum(const Anumber : Extended; Decimals : integer) : string; overload;
    function FormatNum(const Anumber : integer) : string; overload;
    function FormatStr(const AStr, A1, A2, A3 : string) : string;
    function GetTranslation(const FieldNames : Boolean; const TheName: string): string; overload;
    function GetTranslation(const ID: integer; const TranslType: integer): string; overload;
    procedure AddFieldName(const FieldName: string; const Translation: string);
    procedure AddTableName(const TableName: string; const Translation: string);
    procedure ClearCaps;
    property ENFormatSettings : TFormatSettings read FENFormatSettings;
    property Language : string read GetLanguage write SetLanguage;
    property TranslateFieldCaps : TTranslateCaps read FTranslateFieldCaps write FTranslateFieldCaps;
    property TranslateTableCaps : TTranslateCaps read FTranslateTableCaps write FTranslateFieldCaps;
  end;


  { TilMessageHandler -----------------------------------------------------------}

  TMessageFilter = class
    MessageType : TMessageType;
    MessageKind : TMessageOption;
    MesageID : integer;
  end;

  TILMessageHandler = class
  private
    FTarget : TUpdateTarget;
    FInsertCount : integer;
    FDeleteCount : integer;
    FUpdateCount : integer;
    FSearchCount : integer;
    FInsertState : integer;
    FSearchState : integer;
    FUpdateState : integer;
    FSaveState : integer;
    FSaveCount : integer;
    FUpdateComment : string;
    FSearchComment : string;
    FSaveComment : string;
    FLastOperation : TDBOperation;
    FSQLMode : Boolean;
    FTable : TILTable;
    FFilters : TList;

    FOnChangeMessage : TNotifyEvent;
    FOnComposeMessage : TMessageTextEvent;

    function GetTable : TILTable;
    function GetEditCount : integer;
    function GetAffectedTables : TList;

    function GetSearchMessage : string;
    function GetUpdateMessage : string;
    function GetSaveMessage : string;

    function GetInsertCountMessage : string;
    function GetSearchCountMessage : string;
    function GetUpdateCountMessage : string;
    function GetDeleteCountMessage : string;

    function GetSearchStateMessage : string;
    function GetUpdateStateMessage : string;
    function GetSaveStateMessage : string;

    property UpdateState : integer read FUpdateState;
    property SearchState : integer read FSearchState;
    property SaveState : integer read FSaveState;

    property UpdateStateMessage : string read GetUpdateStateMessage;
    property SearchStateMessage : string read GetSearchStateMessage;
    property SaveStateMessage : string read GetSaveStateMessage;

    property SearchCountMessage : String read GetSearchCountMessage;
    property UpdateCountMessage : String read GetUpdateCountMessage;
    property InsertCountMessage : String read GetInsertCountMessage;
    property DeleteCountMessage : String read GetDeleteCountMessage;

    property SaveMessage : string read GetSavemessage;
    property SearchMessage : string read GetSearchMessage;
    property UpdateMessage : String read GetUpdateMessage;

    procedure SetSQLMode(Value : Boolean);
    procedure SetLastOperation(Value : TDBOperation);
    function ExcludeMessage(AMessagetype : TMessageType; AMessage : integer) : Boolean;
  public
     constructor Create;
     destructor Destroy; override;
     function MessageKind(AMessageType : TMessageType) : TMessageOption;
     function MessageText(AMessageType : TMessageType) : string;
     function ValidationState(AMessageType : TMessageType) : integer;
     function ConfirmCount(AMessageType : TMessageType) : integer;
     function ConfirmMessage(AMessageType : TMessageType) : string;
     procedure Reset;
     procedure AddCount(AnOperation : TDBOperation; ACount : integer; Increment : Boolean = True);
     procedure ControlsChanged(Usage : TDataUsage);
     procedure SQLChanged(SQLType : TSQLType; AState : integer);
     procedure DatasetChanged(Operation : TDBOperation);
     procedure AddToFilter(AMessageType : TMessageType; AMessageKind : TMessageOption; AMessage : integer);
     procedure RemoveFromFilter(AMessageType : TMessageType; AMessageKind : TMessageOption; AMessage : integer);
     function InFilter(AMessageType : TMessageType; AMessageKind : TMessageOption; AMessage : integer) : Boolean;
     property Table : TILTable read GetTable write FTable;
     property Target : TUpdateTarget read FTarget write FTarget;
     property LastOperation : TDBOperation read FLastOperation write SetLastOperation;
     property AffectedTables : TList read GetAffectedTables;
     property SQLMode : Boolean read FSQLMode write SetSQLMode;
     property InsertCount : integer read FInsertCount;
     property DeleteCount : integer read FDeleteCount;
     property UpdateCount : integer read FUpdateCount;
     property SearchCount : integer read FSearchCount;
     property SaveCount : integer read FSaveCount;
     property EditCount : integer read GetEditCount;

     property OnChangeMessage : TNotifyEvent read FOnChangeMessage write FOnChangeMessage;
     property OnComposeMessage : TMessageTextEvent read FOnComposeMessage write FOnComposeMessage;
  end;

  TILResourceDialog = class(TComponent)
  private
   { Private declarations }
   FAddressControl : TEdit;
   FCancel: TButton;
   FNameControl : TEdit;
   FOK: TButton;
   FResDialog : TForm;
   FTextField : TILField;
   procedure ResOnClose(Sender : TObject; var CanClose : Boolean);
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); overload; override;
    constructor Create(AOwner : TComponent; ATextField : TILField); reintroduce; overload;
    function Execute: Boolean;
    property AddressControl : TEdit read FAddressControl write FAddressControl;
    property NameControl : TEdit read FNameControl;
  end;

  {Used by Quickbuilder}
  TFieldDef = record
     Caption : string;
     DataType : TDataType;
     Decimals : integer;
     FieldID : integer;
     FieldName : string;
     Identifier : Boolean;
     Indexed : Boolean;
     IsDeleted : Boolean;
     Mandatory : Boolean;
     MultiVal : Boolean;
     Unique : Boolean;
  end;

  TTableDef = record
      Alias : string;
    DatabaseName : string;
    FieldCount : integer;
    Fields : array[1..20] of TFieldDef;
    Location : string;
    ParentTable : string;
    TableName : string;
    constructor Create(const ALocation, ADatabaseName, ATableName : string);
    procedure Add(const FieldName, Caption : string; const DataType : TDatatype;
      const Decimals : integer; const Indexed, Unique, Mandatory : Boolean); overload;
    procedure Add(FieldDef : TFieldDef); overload;
  end;

  const
   ALTER_FIELDNAME = 1;
   ALTER_CAPTION = 2;
   ALTER_DATATYPE = 3;
   ALTER_DECIMALS = 4;
   ALTER_INDEXED = 5;
   ALTER_UNIQUE = 6;
   ALTER_MANDATORY = 7;
   ALTER_IDENTIFIER = 8;
   ALTER_MULVAL = 9;
  type

  TQuickBuilder = class
  private
    FCreating : Boolean;
    FDBName : string;
    FMode : TQuickMode;
    function CreateTable(DBNode : iXMLNode; TableDef : TTableDef) : iXmlNode;
    function GetFieldNode(XDoc : TXMLDocument; TableName, FieldName : string) : iXmlNode;
    procedure CheckDBNames(Def : TTableDef);
    procedure CreateFields(TableNode : iXMLNode; TableDef : TTableDef; Index : integer = -1);
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddField(const TableName : string; FieldDef : TFieldDef);
    procedure AlterField(const TableName, FieldName : string; const AlterElement : integer; FieldDef : TFieldDef);
    procedure CreateChildTable(TableDef : TTableDef);
    procedure CreateSimpleDB(TableDef : TTableDef);
    procedure DropField(const TableName : string; const FieldName : string);
    property Mode : TQuickMode read FMode write FMode;
  end;

function CenterText(const Txt: string; Cnv: TCanvas; R: TRect): TPoint;
function CheckUrl(const URL: string): Boolean;
function CreateResourceStr(const Name, Address: string): string;
function CreateScaledImage(const AFileName: string; const AWidth, AHeight: integer): Vcl.Graphics.TBitmap;
{$IfDef UseIndy}
function DownLoadImage(const URL: string): TWICImage;
{$Endif}
function ExecuteResource(const Resource: string): integer;
function FillZeroes(const Source: string; const Count: integer): string;
function GetBitmapFromFile(const AFileName: string; const Quality: TJPEGScale) : Vcl.Graphics.TBitmap;
function GetFileLastModDate(const Filename: string): TDateTime;
function GetFileType(const AFileName: string): TFileType;
function GetILErrorText(const ErrorCode: integer): string;
function GetMediaThumb(const AFilePath: string; const ASize: integer; Bmp: Vcl.Graphics.TBitmap): HRESULT;
function GetResourceDisplayElement(const ResourceStr: string): string;
function GetSysDataType(const AFieldName: string): TDataType;
function IsVideoFile(const FileName: string): Boolean;
function NameCheck(const AName : string) : Boolean;
function Quoted(const S : string) : string;
procedure CheckOrientation(const AFileName: string; ABitmap: Vcl.Graphics.TBitmap);
procedure CreateListFromDelimiter(const ADelimiter: Char; AText: string; AList: TStringList);
procedure GetResourceElements(const ResourceStr: string; var Name, Address: string);
procedure HandleILException(const ErrorCode: integer; const Comment: string = '');
procedure ScaleBitmap(ABitmap: Vcl.Graphics.TBitmap; Width, Height: integer; DownOnly: Boolean = True);
procedure TicksToTime(const Ticks: Longint; var Secs, MilSecs: integer);
procedure TrimList(AList: TStringList; TrimQuoted: Boolean = True);
function BitmapToIcon(ABitmap : VCL.Graphics.TBitmap) : TIcon;
procedure ShowILErrorText(const ErrorCode : integer; Comment : string = '');

var
  Translator: TTranslator;
  TMan : TTableManager;
  {The Reg... variables are arrays of essensial components. The are populated by the
  constructors and accesed when needed }
  RegBars : TList;
  RegMbs : TList;
  RegMbh : TList;


implementation

{$R 'ImgLibResource.res'}

uses URLMon, Masks, Math, System.Variants, System.Character,
  VCL.Imaging.GifImg, VCL.Imaging.PngImage, Vcl.Dialogs, VCL.ClipBrd, StrUtils, ComObj, ShellAPI, Registry, Propsys,
  WiniNet, VCL.Themes, VCL.Styles, DateUtils, VCL.ExtDlgs,
  ILMessages, VCL.GraphUtil, ILActionBar, form_QuickBuilder;

var
  HotTrackClr: TColor;
  SelectedClr: TColor;
  Importing : Boolean;
  RegTables : TList;
  RegHandlers : TList;
  RegSQLs : TList;
  RegPanels : TList;
  TMan2 : TTableManager;

const
  MaxLibFiles = 30;
  c_ValueSeparator = ',';
  c_LineBreakSeparator = '|';
  c_FieldNameStart = '<';
  c_FieldNameEnd = '>';
  c_NotOperator = '^';
  c_IntervalDelimiter = '';
  c_OrSeparator = '/';
  c_ValueSepSubst = '~v~';
  c_FieldStartSubst = '~s~';
  c_FieldEndSubst = '~e~';
  c_LineBreakSubst = '~l~';
  c_HeadValSeparator = '~';
  c_HeadItemSeparator = '|';
  c_SysFieldCount = 11;
  c_EmptyIndexName = '';

  c_SubstAdd = #1;
  c_SubstSub = #2;
  c_SubstMul = #3;
  c_SubstDiv = #4;
  c_SubstParStart = #6;
  c_SubstParEnd = #7;

  SB_SCROLLBY = -1;
  SB_SCROLLTO = -2;

  c_SysFieldNames: array [1 .. c_SysFieldCount] of string =
    ('SYS_ThumbnailOffset', 'SYS_ThumbnailSize', 'SYS_FileName', 'SYS_State',
    'SYS_LinkID', 'SYS_RegDate', 'SYS_ChangedDate', 'SYS_MediaType',
    'SYS_Level', 'SYS_SiblingIndex', 'SYS_RecordID');

   c_TblAddNote = 'To make the new table available for your project you must, if it is open in the IDE, ' +
   'reload the form containing the TableManager.';

  SYS_ThumbnailOffset = 1;
  SYS_ThumbnailSize = 2;
  SYS_FileName = 3;
  SYS_RecordState = 4;
  SYS_LinkID = 5;
  SYS_RegDate = 6;
  SYS_ChangedDate = 7;
  SYS_MediaType = 8;
  SYS_Level = 9;
  SYS_SiblingIndex = 10;
  SYS_RecordID = 11;

  c_CheckBoxWidth = 14;

var
  SortFieldnames: TStringList;
  SortAscending: Boolean;
  SortLib: TILTable;
  SortDataset : TILDataset;

type
   TNewVal = class
      NV : string;
   end;


{ TTableDef record -----------------------------------------------------
Used with Quickbuilder
}

constructor TTableDef.Create(const ALocation: string; const ADatabaseName, ATableName: string);
var
  i : integer;
begin
    if not DirectoryExists(ALocation) then
      HandleILException(E_InvalidDir, ALocation);

    if (ADatabaseName = '')
    or not NameCheck(ADatabaseName) then
     HandleILException(E_InvalidChar);

    if (ATableName = '')
    or not NameCheck(ATableName) then
     HandleILException(E_InvalidChar);

    Location := ALocation;
    DatabaseName := ADatabaseName;
    TableName := ATableName;
    ParentTable := '';
    FieldCount := 0;
    for I := 1 to 20 do
    begin
      Fields[i].FieldName := '';
      Fields[i].Caption := '';
      Fields[i].DataType := dtString;
      Fields[i].Indexed := false;
      Fields[i].Unique := false;
      Fields[i].Mandatory := false;
      Fields[i].Decimals := -1;
    end;
end;

procedure TTableDef.Add(FieldDef : TFieldDef);
var
  i : integer;
begin
  for I := 1 to FieldCount do
  begin
    if SameText(Fields[i].FieldName, FieldDef.FieldName) then
    begin
      Comment1 := FieldDef.Caption;
      HandleILException(E_DupFieldName)
    end;
  end;
  if not NameCheck(FieldDef.FieldName) then
      HandleIlException(E_InvalidChar);
  inc(FieldCount);
  Fields[FieldCount] := FieldDef;
end;

procedure TTableDef.Add(const FieldName, Caption : string; const DataType : TDatatype; const Decimals : integer;
 const Indexed, Unique, Mandatory : Boolean);
var
  i : integer;
begin
  for I := 1 to FieldCount do
  begin
    if SameText(Fields[i].FieldName, FieldName) then
    begin
      Comment1 := FieldName;
      HandleILException(E_DupFieldName)
    end;
  end;
  if not NameCheck(FieldName) then
      HandleIlException(E_InvalidChar);
  inc(FieldCount);
  Fields[FieldCount].FieldName := FieldName;
  Fields[FieldCount].Caption := Caption;
  Fields[FieldCount].DataType := DataType;
  Fields[FieldCount].Decimals := Decimals;
  Fields[FieldCount].Indexed := Indexed;
  Fields[FieldCount].Unique := Unique;
  Fields[FieldCount].Mandatory := Mandatory;
end;

{ TQuickBuilder ---------------------------------------------------------}

function TQuickBuilder.CreateTable(DBNode : iXMLNode; TableDef : TTableDef) : iXmlNode;
begin
   Result := DBNode.AddChild('Table');
   Result.AddChild('TableName');
   Result.AddChild('Alias');
   Result.AddChild('TableClass');
   Result.AddChild('TableSpace');
   Result.AddChild('Relations');
   Result.AddChild('RecordFreeSpace');
   Result.AddChild('RecordLayout');
   Result.AddChild('RecordSize');
   Result.AddChild('LocalImageSize');
   Result.AddChild('IsRoot');
   Result.AddChild('FreeTable');
   Result.AddChild('MasterSource');
   Result.AddChild('ErrorState');

   Result.ChildNodes['TableName'].Text := TableDef.TableName;
   Result.ChildNodes['Alias'].Text := '';
   Result.ChildNodes['TableClass'].Text := 'TILDataTable';
   Result.ChildNodes['TableSpace'].Text := '100000';
   Result.ChildNodes['Relations'].Text := '0';
   Result.ChildNodes['RecordFreeSpace'].Text := '0';
   Result.ChildNodes['RecordLayout'].Text := '2';
   Result.ChildNodes['RecordSize'].Text := '0'; {Compact}
   Result.ChildNodes['LocalImageSize'].Text := '3';
   if Mode = qbCreateDB then
     Result.ChildNodes['IsRoot'].Text := '1'
   else
     Result.ChildNodes['IsRoot'].Text := '0';
   Result.ChildNodes['FreeTable'].Text := '0';
   Result.ChildNodes['MasterSource'].Text := TableDef.ParentTable;
   Result.ChildNodes['ErrorState'].Text := '0';
end;

procedure TQuickBuilder.CreateFields(TableNode : iXMLNode; TableDef : TTableDef; Index : integer = -1);
var
 i : integer;
 Prop : iXmlNode;
 Fld : iXmlNode;
 Def : TFieldDef;
 Start : integer;
 Ind : integer;

 function GetNext : Integer;
 var
   i : integer;
   T : TILTable;
 begin
   T := TMan.TableOf(TableDef.TableName);
   Result := 1;
   for i := 0 to T.Fields.Count-1 do
   begin
      if T.Fields.Items[i].FieldID > Result then
        Result := T.Fields.Items[i].FieldID + 1;
   end;
 end;

begin
   if (Index <> -1) then
     Start := 1
   else
     Start := 0;

   Ind := Start;
   for I := Start to TableDef.FieldCount do
   begin
      if I = 0 then
        Ind := 1;
      if SameText(TableDef.Fields[Ind].FieldName, 'SYS_LinkID') then
       Ind := 0
     else if SameText(TableDef.Fields[Ind].FieldName, 'SYS_Level') then
       Ind := 1
     else if SameText(TableDef.Fields[Ind].FieldName, 'SYS_SiblingIndex') then
       Ind := 2
     else
     begin
       if (Start = 1) and (Tman.Database = '') then
         Ind := i-1
       else if Index <> -1 then
         Ind := Index
       else
         Ind := i;
     end;

    Fld := TableNode.AddChild('Field', Ind);
    Fld.AddChild('FieldName');
    Fld.AddChild('FieldID');
    Fld.AddChild('FieldType');
    Fld.AddChild('DataType');
    Fld.AddChild('Decimals');
    Fld.AddChild('Identifier');
    Fld.AddChild('Unique');
    Fld.AddChild('ImageText');
    Fld.AddChild('Indexed');
    Fld.AddChild('IndexPriority');
    Fld.AddChild('LookupType');
    Fld.AddChild('InitialValue');
    Fld.AddChild('LookupTable');
    Fld.AddChild('LookupField');
    Fld.AddChild('LookupSearchField');
    Fld.AddChild('Mandatory');
    Fld.AddChild('MaxLength');
    Fld.AddChild('MaxValues');
    Fld.AddChild('ReportField');
    Fld.AddChild('ReportCaption');
    Fld.AddChild('ColumnWidth');
    Fld.AddChild('Alignment');
    Fld.AddChild('PickValues');
    Fld.AddChild('DateFormat');
    Fld.AddChild('ErrorState');
    Prop := Fld;

    if i > 0 then
    begin
     Def := TableDef.Fields[i];
     Prop.ChildNodes['FieldName'].Text := Def.FieldName;
     if TMan.TableOf(TableDef.TableName) = nil then { New table }
       Prop.ChildNodes['FieldID'].Text := IntToStr(i+1)
     else
       Prop.ChildNodes['FieldID'].Text := IntToStr(GetNext);
     Prop.ChildNodes['FieldType'].Text := '0';
     Prop.ChildNodes['DataType'].Text := IntToStr(Ord(Def.DataType));
     Prop.ChildNodes['Decimals'].Text := IntToStr(Def.Decimals);
     Prop.ChildNodes['Identifier'].Text := IntToStr(Integer(Def.Identifier));
     Prop.ChildNodes['Unique'].Text := IntToStr(Integer(Def.Unique));
     Prop.ChildNodes['ImageText'].Text := '0';
     Prop.ChildNodes['Indexed'].Text := IntToStr(Integer(Def.Indexed));
     Prop.ChildNodes['IndexPriority'].Text := '1';
     Prop.ChildNodes['LookupType'].Text := '0';
     Prop.ChildNodes['InitialValue'].Text := '';
     Prop.ChildNodes['LookupTable'].Text := '';
     Prop.ChildNodes['LookupField'].Text := '';
     Prop.ChildNodes['LookupSearchField'].Text := '';
     Prop.ChildNodes['Mandatory'].Text := IntToStr(Integer(Def.Mandatory));
     Prop.ChildNodes['MaxLength'].Text := '0';
     if Def.MultiVal then
       Prop.ChildNodes['MaxValues'].Text := '0'
     else
       Prop.ChildNodes['MaxValues'].Text := '1';
     Prop.ChildNodes['ReportField'].Text := '1';
     Prop.ChildNodes['ReportCaption'].Text := Def.Caption;
     Prop.ChildNodes['ColumnWidth'].Text := '100';
     Prop.ChildNodes['DateFormat'].Text := '';
     Prop.ChildNodes['Alignment'].Text := '0';
    end
    else begin
     Prop.ChildNodes['FieldName'].Text := 'SYS_LinkID';
     Prop.ChildNodes['FieldID'].Text := '1';
     Prop.ChildNodes['FieldType'].Text := '1';
     Prop.ChildNodes['DataType'].Text := '2';
     Prop.ChildNodes['Decimals'].Text := '-1';
     Prop.ChildNodes['Identifier'].Text := '0';
     Prop.ChildNodes['Unique'].Text := '0';
     Prop.ChildNodes['ImageText'].Text := '0';
     Prop.ChildNodes['Indexed'].Text := '1';
     Prop.ChildNodes['IndexPriority'].Text := '1';
     Prop.ChildNodes['LookupType'].Text := '0';
     Prop.ChildNodes['InitialValue'].Text := '';
     Prop.ChildNodes['LookupTable'].Text := '';
     Prop.ChildNodes['LookupField'].Text := '';
     Prop.ChildNodes['LookupSearchField'].Text := '';
     Prop.ChildNodes['Mandatory'].Text := '1';
     Prop.ChildNodes['MaxLength'].Text := '0';
     Prop.ChildNodes['MaxValues'].Text := '1';
     Prop.ChildNodes['ReportField'].Text := '0';
     Prop.ChildNodes['ReportCaption'].Text := '';
     Prop.ChildNodes['ColumnWidth'].Text := '50';
     Prop.ChildNodes['DateFormat'].Text := '';
     Prop.ChildNodes['Alignment'].Text := '0';
    end;
   end;
end;

procedure TQuickBuilder.CreateChildTable(TableDef : TTableDef);
var
  XDoc : TXMLDocument;
  Meta : iXmlNode;
  Tbl : iXMLNode;

begin
   XDoc := TXMLDocument.Create(Application.MainForm);
   try
     XDoc.Active := True;
     XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
     XDoc.LoadFromFile(TMan.FMetaName);
     Meta := XDoc.DocumentElement.ChildNodes.FindNode('Meta');
     Tbl := CreateTable(Meta, TableDef);
     CreateFields(Tbl, TableDef);
     XDoc.SaveToFile(TMan.FMetaName);
     Tman.CreateTableFromMeta(TableDef.TableName, TMan.FMetaName);
     ShowMessage('Table ' + TableDef.TableName + ' created!');
   finally
    XDoc.Free;
   end;

end;

procedure TQuickBuilder.CreateSimpleDB(TableDef : TTableDef);
var
  XDoc : TXMLDocument;
  N : IXmlNode;
  Meta : iXmlNode;
  Tbl : iXMLNode;
  Reg : TRegistry;
begin
   if not FCreating then
     HandleILException(E_DBLoaded);
   CheckDBNames(TableDef);

   FDBName := TableDef.DatabaseName;
   TMan.ExtDesigner := True;
   XDoc := TXMLDocument.Create(Application.MainForm);
   try
   XDoc.Active := True;
   N := Xdoc.AddChild('Ilb');
   Meta := N.AddChild('Meta');
   N := Meta.AddChild('ExtDesigner');
   N.Text := '0';
   N := Meta.AddChild('DBName');
   N.Text := FDBName;
   N := Meta.AddChild('State');
   N.Text := '2';

   Tbl := CreateTable(Meta, TableDef);
   CreateFields(Tbl, TableDef, 1);

   Reg := TRegistry.Create(KEY_WRITE);
   try
   Reg.RootKey := HKEY_CURRENT_USER;
   if not Reg.KeyExists('\Software\ImageLibrary') then
   begin
      if not Reg.OpenKey('\Software\ImageLibrary', true) then
        HandleILException(0, 'Could not create registry key ImageLibrary');
   end;
   if not Reg.OpenKey('\Software\ImageLibrary\' + FDBName, True) then
      HandleILException(E_RegKey, FDBName);

   Reg.WriteString('Location', TableDef.Location);
   finally
    Reg.Free;
   end;

   XDoc.SaveToFile(TableDef.Location + '\' + FDBName + '_Meta.xml');
   TMan.Location := TableDef.Location;
   TMan.CreateDBFromMeta(TableDef.Location + '\' + FDBName + '_Meta.xml');
   TMan.SetNames;
   TMan.DoLoaded;
   TMan.ExtDesigner := false;
   finally
     XDoc.Free;
   end;
   ShowMessage('Database ' + FDBName + ' created!');
end;

procedure TQuickBuilder.CheckDBNames(Def : TTableDef);
begin
    if not DirectoryExists(Def.Location) then
      HandleILException(E_InvalidDir, Def.Location);

    if (Def.DatabaseName = '')
    or not NameCheck(Def.DatabaseName) then
     HandleILException(E_InvalidChar);

    if (Def.TableName = '')
    or not NameCheck(Def.TableName) then
     HandleILException(E_InvalidChar);

    if TMan.CheckDupDB(Def.DatabaseName) then
      HandleILException(E_DupDBName);

    if TMan.CheckDupTable(Def.TableName) then
      HandleILException(E_DupTableName);
end;

function TQuickBuilder.GetFieldNode(XDoc : TXMLDocument; TableName, FieldName : string) : iXmlNode;
var
  ANode, Meta, Tbl, Fld : ixmlNode;
  i, j : integer;
  begin
    Result := nil;
    ANode := Xdoc.DocumentElement;
    Meta := ANode.ChildNodes['Meta'];
    for I := 0 to Meta.ChildNodes.Count-1 do
    begin
      if Meta.ChildNodes[i].NodeName = 'Table' then
      begin
         Tbl := Meta.ChildNodes[i];
         ANode := Tbl.Childnodes.FindNode('TableName');
         if (ANode <> nil) and SameText(ANode.Text, TableName) then
         begin
           for j := 0 to Tbl.ChildNodes.Count-1 do
           begin
              Fld := Tbl.ChildNodes[j];
              if Fld.NodeName = 'Field' then
              begin
                ANode := Fld.Childnodes.FindNode('FieldName');
                if (ANode <> nil) and Sametext(ANode.Text, FieldName) then
                begin
                  Result := ANode;
                  Break;
                end;
              end;
           end;
           if Result <> nil then
             Break;
         end;
      end;
    end;
end;

procedure TQuickBuilder.DropField(const TableName : string; const FieldName : string);
var
  XDoc : TXMLDocument;
  Tbl, Fld : ixmlNode;
  i : integer;
  T : TILTable;
  Found : Boolean;
begin
   if FCreating then
     HandleILException(E_DBNotLoaded);
   T := TMan.TableOf(TableName);
   if T = nil then
     HandleILException(E_InvalidTableName, TableName);
   Found := false;
   for I := 0 to T.Fields.Count-1 do
     if SameText(T.Fields.Items[i].FieldName, FieldName) then
     begin
       Found := True;
     end;
   if not Found then
     HandleILException(E_InvalidFieldName, FieldName);
   TMan.ExtDesigner := True;
   XDoc := TXMLDocument.Create(Application.MainForm);
   XDoc.Active := True;
   XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
   XDoc.LoadFromFile(TMan.FMetaName);
   try
    Fld := GetFieldNode(Xdoc, TableName, FieldName);
    { Gets the FieldName node }
    if Fld <> nil then
    begin
      Tbl := Fld.ParentNode.ParentNode;
      Tbl.ChildNodes.Remove(Fld.ParentNode);
    end
    else
      HandleILException(E_MissingMetaField, FieldName);
    XDoc.SaveToFile(Tman.FMetaName);
   finally
     XDoc.Free;
   end;
   TMan.SyncTable(T);
   TMan.CheckDanglingHandlers;
end;

procedure TQuickBuilder.AlterField(const TableName, FieldName : string; const AlterElement : Integer; FieldDef : TFieldDef);
var
  XDoc : TXMLDocument;
  ANode : ixmlNode;
  i : integer;
  T : TILTable;
  Found : Boolean;
begin
   if FCreating then
     HandleILException(E_DBNotLoaded);
   T := TMan.TableOf(TableName);
   if T = nil then
     HandleILException(E_InvalidTableName, TableName);
   Found := false;
   for I := 0 to T.Fields.Count-1 do
     if SameText(T.Fields.Items[i].FieldName, FieldName) then
     begin
       Found := True;
       Break;
     end;
   if not Found then
     HandleILException(E_InvalidFieldName, FieldName);
   if AlterElement = ALTER_FIELDNAME then
   begin
     if not T.IsTableEmpty then
       HandleILException(E_TableNotEmpty);
     for I := 0 to T.Fields.Count-1 do
     if SameText(T.Fields.Items[i].FieldName, FieldDef.FieldName) then
       HandleILException(E_DupFieldName, FieldDef.FieldName);
     if not NameCheck(FieldDef.FieldName) then
       HandleILException(E_InvalidChar);
   end;

   TMan.ExtDesigner := True;
   XDoc := TXMLDocument.Create(Application.MainForm);
   XDoc.Active := True;
   XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
   XDoc.LoadFromFile(TMan.FMetaName);
   try
   ANode := GetFieldNode(XDoc, TableName, FieldName);
   if ANode = nil then
     HandleILException(E_MissingMetaField, FieldName);
   if AlterElement = ALTER_FIELDNAME then
   begin
      if SameText(ANode.Text, FieldDef.FieldName) then
      begin
        Exit;
      end;
      ANode.Text := FieldDef.FieldName;
   end
   else if AlterElement = ALTER_CAPTION then
   begin
      ANode := ANode.ParentNode;
      ANode := ANode.ChildNodes.FindNode('ReportCaption');
      if SameText(ANode.Text, FieldDef.Caption) then
      begin
        Exit;
      end;
      ANode.Text := FieldDef.Caption;
   end
   else if AlterElement = ALTER_INDEXED then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('Indexed');
     if ((ANode.Text = '1') and FieldDef.Indexed)
     or ((ANode.Text = '0') and not FieldDef.Indexed) then
     begin
        Exit;
     end;
     if FieldDef.Indexed then
       ANode.Text := '1'
     else
       ANode.Text := '0';
   end
   else if AlterElement = ALTER_DATATYPE then
   begin
     if not T.IsTableEmpty then
       HandleILException(E_TableNotEmpty);
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('DataType');
     if StrToInt(ANode.Text) = Ord(FieldDef.DataType) then
     begin
        Exit;
     end;
     ANode.Text := IntToStr(Ord(FieldDef.DataType));
   end
   else if AlterElement = ALTER_DECIMALS then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('Decimals');
     if StrToInt(ANode.Text) = FieldDef.Decimals then
     begin
        Exit;
     end;
     ANode.Text := IntToStr(FieldDef.Decimals);
   end
   else if AlterElement = ALTER_UNIQUE then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('Unique');
     if ((ANode.Text = '1') and FieldDef.Unique)
     or ((ANode.Text = '0') and not FieldDef.Unique) then
     begin
        Exit;
     end;
     if FieldDef.Unique then
       ANode.Text := '1'
     else
       ANode.Text := '0';
   end
   else if AlterElement = ALTER_MANDATORY then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('Mandatory');
     if ((ANode.Text = '1') and FieldDef.Mandatory)
     or ((ANode.Text = '0') and not FieldDef.Mandatory) then
     begin
        Exit;
     end;
     if FieldDef.Mandatory then
       ANode.Text := '1'
     else
       ANode.Text := '0';
   end
   else if AlterElement = ALTER_IDENTIFIER then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('Identifier');
     if ((ANode.Text = '1') and FieldDef.Identifier)
     or ((ANode.Text = '0') and not FieldDef.Identifier) then
     begin
        Exit;
     end;
     if FieldDef.Identifier then
       ANode.Text := '1'
     else
       ANode.Text := '0';
   end
   else if AlterElement = ALTER_MULVAL then
   begin
     ANode := ANode.ParentNode;
     ANode := ANode.ChildNodes.FindNode('MaxValues');
     if ((ANode.Text = '1') and not FieldDef.MultiVal)
     or ((ANode.Text <> '1') and FieldDef.Multival) then
     begin
        Exit;
     end;
     if FieldDef.Multival then
       ANode.Text := '0'
     else
       ANode.Text := '1';
   end;

   XDoc.SaveToFile(Tman.FMetaName);
   finally
     XDoc.Free;
   end;
   TMan.SyncTable(T);
end;

procedure TQuickBuilder.AddField(const TableName : string; FieldDef : TFieldDef);
var
  XDoc : TXMLDocument;
  ANode, Meta : ixmlNode;
  i : integer;
  Def : TTableDef;
  T : TILTable;
  Ind : integer;
begin
   if FCreating then
     HandleILException(E_DBNotLoaded);

   T := TMan.TableOf(TableName);
   if T = nil then
     HandleILException(E_InvalidTableName, TableName);
   for I := 0 to T.Fields.Count-1 do
     if SameText(T.Fields.Items[i].FieldName, FieldDef.FieldName) then
       HandleILException(E_DupFieldName, FieldDef.FieldName);
   if not NameCheck(FieldDef.FieldName) then
       HandleILException(E_InvalidChar);
   Ind := T.Fields.Count;
   TMan.ExtDesigner := True;
   XDoc := TXMLDocument.Create(Application.MainForm);
   XDoc.Active := True;
   XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
   XDoc.LoadFromFile(TMan.FMetaName);
   try
    ANode := Xdoc.DocumentElement;
    Meta := ANode.ChildNodes['Meta'];
    for I := 0 to Meta.ChildNodes.Count-1 do
    begin
      if Meta.ChildNodes[i].NodeName = 'Table' then
      begin
         ANode := Meta.ChildNodes[i];
         ANode := Anode.Childnodes.FindNode('TableName');
         if (ANode <> nil) and SameText(ANode.Text, TableName) then
         begin
           Def := TTableDef.Create(TMan.Location, Tman.Database, TableName);
           Def.FieldCount := 1;
           Def.Fields[1] := FieldDef;
           CreateFields(Meta.ChildNodes[i], Def, Ind);
           Break;
         end;
      end;
    end;
    XDoc.SaveToFile(Tman.FMetaName);
   finally
     XDoc.Free;
   end;
   TMan.SyncTable(T);
end;

constructor TQuickBuilder.Create;
begin
   inherited Create;
   if TMan = nil then
   begin
      HandleILException(E_ManagerNotFound);
   end;
   FDBName := TMan.Database;
   FCreating := (FDBName = '');
   if FCreating then
   begin
     TMan.ExtDesigner := false;
   end;
end;

destructor TQuickBuilder.Destroy;
begin
   inherited;
end;

{ Helper functions -------------------------------------------------------------}

procedure HandleILException(const ErrorCode: integer; const Comment: string = '');
var
  X : ILException;
  s : string;
begin
  if Comment <> '' then
  begin
   Comment1 := Comment;
   Comment2 := '';
   Comment3 := '';
  end
  else
  begin
    if ErrorCode >= 200 then
      S := E_Internals[ErrorCode]
    else
    begin
      begin
        S := E_ErrorText[ErrorCode];
      end;
    end;
    if Pos('%s', S) = 0 then
    begin
      Comment1 := '';
      Comment2 := '';
      Comment3 := '';
    end;
  end;
  if (ErrorCode = 0) and (Comment <> '')then
    X := ILException.Create(Comment)
  else
    X := ILException.Create(GetILErrorText(ErrorCode));
  X.FErrorCode := ErrorCode;
  try
    raise X;
  finally
    Comment1 := '';
    Comment2 := '';
    Comment3 := '';
  end;
end;

procedure ShowILErrorText(const ErrorCode : integer; Comment : string = '');
begin
  MessageDlg(GetILErrorText(ErrorCode) + '. ' + Comment, mtError, [mbOk], 0);
end;

function GetAppdataFolderPath: string;
var
  FilePath: array [0 .. MAX_PATH] of Char;
begin
  SHGetFolderPath(0, CSIDL_APPDATA, 0, 0, FilePath);
  Result := FilePath;
end;

function CenterText(const Txt: string; Cnv: TCanvas; R: TRect): TPoint;
var
  W, H, WSpace, HSpace: integer;
begin
  W := Cnv.TextWidth(Txt);
  H := Cnv.TextHeight(Txt);
  WSpace := R.Width - W;
  if WSpace < 0 then
  begin
    Dec(R.Left, Abs(WSpace div 2));
    Inc(R.Right, Abs(WSpace div 2));
    WSpace := 0;
  end;
  HSpace := R.Height - H;
  if HSpace < 0 then
  begin
    Dec(R.Top, Abs(HSpace div 2));
    Inc(R.Bottom, Abs(HSpace div 2));
    HSpace := 0;
  end;
  Result.X := R.Left + WSpace div 2;
  Result.Y := R.Top + (HSpace div 2) - 1;
end;

function IsWholeWord(const S: string): Boolean;
var
  i: integer;
begin
  Result := True;
  for i := 1 to Length(S) do
    if CharInSet(S[i], [' ', '.', ',', ';', '?', '!', '/', '\', '%', '&', '(',
      ')', '=', '+', '-', '"', '.', ':', '|', '{', '}', '[', ']', '>', '<'])
    then
    begin
      Result := false;
      Break;
    end;
end;

function VKeytoWideString (const Key : Word) : WideString;
var
  WBuff         : array [0..255] of WideChar;
  KeyboardState : TKeyboardState;
  UResult       : Integer;
begin
  Result := '';
  GetKeyBoardState (KeyboardState);
  ZeroMemory(@WBuff[0], SizeOf(WBuff));
  UResult := ToUnicode(key, MapVirtualKey(key, 0), KeyboardState, WBuff, Length(WBuff), 0);
  if UResult > 0 then
    SetString(Result, WBuff, UResult)
  else if UResult = -1 then
    Result := WBuff;
end;


function GetWords(Txt: TStringList): TStringList;
var
  S: string;
begin
  S := Txt.Text;
  S := StringReplace(S, '.', ' ', [rfReplaceAll]);
  S := StringReplace(S, ',', ' ', [rfReplaceAll]);
  S := StringReplace(S, ';', ' ', [rfReplaceAll]);
  S := StringReplace(S, ' - ', ' ', [rfReplaceAll]);
  S := StringReplace(S, '?', ' ', [rfReplaceAll]);
  S := StringReplace(S, '!', ' ', [rfReplaceAll]);
  S := StringReplace(S, '/', ' ', [rfReplaceAll]);
  S := StringReplace(S, '\', ' ', [rfReplaceAll]);
  S := StringReplace(S, '%', ' ', [rfReplaceAll]);
  S := StringReplace(S, '&', ' ', [rfReplaceAll]);
  S := StringReplace(S, '(', ' ', [rfReplaceAll]);
  S := StringReplace(S, ')', ' ', [rfReplaceAll]);
  S := StringReplace(S, '=', ' ', [rfReplaceAll]);
  S := StringReplace(S, '+', ' ', [rfReplaceAll]);
  S := StringReplace(S, '-', ' ', [rfReplaceAll]);
  S := StringReplace(S, '"', ' ', [rfReplaceAll]);
  S := StringReplace(S, '.', ' ', [rfReplaceAll]);
  S := StringReplace(S, ':', ' ', [rfReplaceAll]);
  S := StringReplace(S, '|', ' ', [rfReplaceAll]);
  S := StringReplace(S, '{', ' ', [rfReplaceAll]);
  S := StringReplace(S, '}', ' ', [rfReplaceAll]);
  S := StringReplace(S, '[', ' ', [rfReplaceAll]);
  S := StringReplace(S, ']', ' ', [rfReplaceAll]);
  S := StringReplace(S, '>', ' ', [rfReplaceAll]);
  S := StringReplace(S, '<', ' ', [rfReplaceAll]);
  S := StringReplace(S, '''', ' ', [rfReplaceAll]);
  S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);

  while Pos('  ', S) <> 0 do
    S := StringReplace(S, '  ', ' ', [rfReplaceAll]);
  Result := TStringList.Create;
  Result.Sorted := True;
  Result.Duplicates := DupIgnore;
  CreateListFromDelimiter(' ', S, Result);
  TrimList(Result);
end;

procedure TicksToTime(const Ticks: Longint; var Secs, MilSecs: integer);
begin
  Secs := Ticks div 1000;
  MilSecs := Ticks mod 1000;
end;

function GetLastErrorText: string;
var
  C: array [Byte] of Char;
begin
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nil, GetLastError,
    LOCALE_USER_DEFAULT, C, SizeOf(C), nil);
  Result := C;
end;

function IsUrl(const URL: String): Boolean;
begin
  Result := false;
  if Pos('://', URL) = 0 then
    Exit;
  Result := (IsValidURL(nil, PWideChar(URL), 0) = S_OK)
end;

function CheckUrl(const URL: string): Boolean;
var
  hSession, hfile: hInternet;
  dwindex, dwcodelen: dword;
  dwcode: array [1 .. 20] of Char;
  res: pchar;
begin
  Result := false;
  hSession := InternetOpen('InetURL:/1.0', INTERNET_OPEN_TYPE_PRECONFIG,
    nil, nil, 0);
  if Assigned(hSession) then
  begin
    hfile := InternetOpenUrl(hSession, pchar(URL), nil, 0,
      INTERNET_FLAG_RELOAD, 0);
    dwindex := 0;
    dwcodelen := 10;
    HttpQueryInfo(hfile, HTTP_QUERY_STATUS_CODE, @dwcode, dwcodelen, dwindex);
    res := pchar(@dwcode);
    Result := (res = '200') or (res = '302');
    if Assigned(hfile) then
      InternetCloseHandle(hfile);
    InternetCloseHandle(hSession);
  end;
end;

{$IfDef UseIndy}
function DownLoadImage(const URL: string): TWICImage;
var
  IdHTTP1: TIdHTTP;
  M: TMemoryStream;
  Img: TWICImage;
begin
  Result := nil;
  if not CheckUrl(URL) then
    Exit;
  M := TMemoryStream.Create;
  IdHTTP1 := TIdHTTP.Create(nil);
  try
    IdHTTP1.HandleRedirects := True;
    IdHTTP1.Request.UserAgent := 'Embarcadero URI Client/1.0';
    try
      IdHTTP1.Get(URL, M);
      M.Position := 0;
    except
      HandleILException(E_InternetAccess);
    end;

    Img := TWICImage.Create;
    try
      Img.LoadFromStream(M);
    except
      FreeAndNil(Img); { Not a supported image }
    end;
    Result := Img;

  finally
    IdHTTP1.Free;
    M.Free;
  end;
end;
{$EndIF}

function SortProc(Item1, Item2: Pointer): integer;
var
  Indx: integer;
  DR1: TILDataRecord;
  DR2: TILDataRecord;
  s1, s2: string;
  i: integer;
  F : TStrings;


  function SetDateFormat(const S: string): string;
  var
    DT: TDateTime;
    fmt: TFormatSettings;
  begin
    if S = '' then
    begin
      Result := '';
      Exit;
    end;
    DT := StrToDateTime(S, Translator.FFormatSettings);
    fmt := TFormatSettings.Create(Translator.Language);
    fmt.ShortDateFormat := 'yyyymmddhhnnss';
    Result := DateTimeToStr(DT, fmt);
  end;

  function SetPathString(ARecord: TILDataRecord; const SortField: string): string;
  var
    S, V: string;
    SID: integer;
    DR: TILDataRecord;
    Indx: integer;
  begin
    S := '';
    if ARecord[c_SysFieldNames[SYS_LinkID]].IsEmpty then
    begin
      if ARecord['sys_Level'].AsInteger = 0 then
      begin
       Indx := ARecord.IndexOf(SortField);
       V := ARecord[Indx].AsString;
      if ARecord[Indx].FieldDef.DataType = dtDate then
        V := SetDateFormat(V)
      else if not SameText(SortField, c_SysFieldNames[SYS_SiblingIndex]) then
        V := SortLib.SetDateNumberFormat(V, ARecord[Indx].FieldDef.DataType,
          ARecord[Indx].FieldDef.Decimals);
       Result := IntToStr(ARecord['sys_Level'].AsInteger)  + V;
       Exit;
      end;
    end;
    SID := ARecord.RecordIndex;
    while SID <> -1 do
    begin
      DR := SortLib.Dataset[SortLib.Dataset.IndexOf(SID)];
      Indx := DR.IndexOf(SortField);
      V := DR[Indx].AsString;
      if DR[Indx].FieldDef.DataType = dtDate then
        V := SetDateFormat(V)
      else if not SameText(SortField, c_SysFieldNames[SYS_SiblingIndex]) then
        V := SortLib.SetDateNumberFormat(V, DR[Indx].FieldDef.DataType,
          DR[Indx].FieldDef.Decimals);
      S := IntToStr(DR['sys_Level'].AsInteger) + V + S;

      if not DR[c_SysFieldNames[SYS_LinkID]].IsEmpty then
      begin
        SID := DR[c_SysFieldNames[SYS_LinkID]].AsInteger
      end
      else
        SID := -1;
    end;
    Result := S;
  end;

begin
  while TMan.FSystemBusy do
    Application.ProcessMessages;
  DR1 := TILDataRecord(Item1);
  DR2 := TILDataRecord(Item2);
  Result := 0;
  for i := 0 to SortFieldnames.Count - 1 do
  begin
    Indx := DR1.IndexOf(SortFieldnames[i]);
    if Indx = -1 then
    begin
      Comment1 := SortFieldnames[i];
      HandleILException(E_InvalidFieldName, '');
    end;

    if (DR1[Indx].FieldDef.LookupType = looParentField) then
    begin
     F :=  SortLib.GetLookupParentValue
      (DR1[Indx].FieldDef.LookupTable, DR1[Indx].FieldDef.LookupField, DR1);
     s1 := '';
     if (F <> nil) and (F.Count > 0) then
      s1 := F[0];
     F.Free;
    end
    else
    begin
      s1 := DR1[Indx].AsString;
    end;
    if (DR2[Indx].FieldDef.LookupType = looParentField) then
    begin
     s2 := '';
     F := SortLib.GetLookupParentValue
      (DR2[Indx].FieldDef.LookupTable, DR2[Indx].FieldDef.LookupField, DR2);
     if (F <> nil) and (F.Count > 0) then
      s2 := F[0];
     F.Free;
    end
    else
      s2 := DR2[Indx].AsString;
    if s1 <> '' then
    begin
      if (DR1[Indx].FieldDef.LookupType = looTableKey)
      and (DR1[Indx].FieldDef.DataType = dtNumber) then
        s1 := DR1[Indx].DisplayValue
      else
      if DR1[Indx].FieldDef.DataType = dtDate then
        s1 := SetDateFormat(s1)
      else if not SameText(SortFieldnames[i], c_SysFieldNames[SYS_SiblingIndex])
      then
        s1 := SortLib.SetDateNumberFormat(s1, DR1[Indx].FieldDef.DataType,
          DR1[Indx].FieldDef.Decimals);
    end;
    if s2 <> '' then
    begin
      if (DR2[Indx].FieldDef.LookupType = looTableKey)
      and (DR2[Indx].FieldDef.DataType = dtNumber) then
        s2 := DR2[Indx].DisplayValue
      else
      if DR2[Indx].FieldDef.DataType = dtDate then
        s2 := SetDateFormat(s2)
      else if not SameText(SortFieldnames[i], c_SysFieldNames[SYS_SiblingIndex])
      then
        s2 := SortLib.SetDateNumberFormat(s2, DR2[Indx].FieldDef.DataType,
          DR2[Indx].FieldDef.Decimals);
    end;

    if (SortLib is TILTreeViewTable) and (SortDataset = SortLib.Dataset) and
      (TILTreeViewTable(SortLib).TreeViewParams.SortingMethod = tsBySiblings)
    then
    begin
      s1 := SetPathString(DR1, SortFieldnames[i]);
      s2 := SetPathString(DR2, SortFieldnames[i]);
    end;

    if SortAscending then
    begin
       Result := AnsiCompareText(s1, s2);
    end
    else
    begin
       Result := -AnsiCompareText(s1, s2);
    end;
    if Result <> 0 then
      Break;
  end;
end;

function GetILErrorText(const ErrorCode: integer): string;
var
  S: string;
begin
  if ErrorCode = -1 then
  begin
    Comment1 := '';
    Comment2 := '';
    Comment3 := '';
    S := Translator.GetTranslation(T_NoAction, TRANSL_TERMS);
    if S <> '' then
      Result := S
    else
      Result := UE_NoActionAvailable;
  end
  else if ErrorCode = 0 then
    Result := ''
  else
  begin
    if ErrorCode >= 200 then
      Result := E_Internals[ErrorCode]
    else
    begin
      begin
        Result := Translator.GetTranslation(ErrorCode, TRANSL_USER_ERRORS);
      end;
    end;
  end;

  { Check if Comments might be a fieldname or table name }
  if Comment1 <> '' then
  begin
    S := Translator.GetTranslation(True, Comment1);
    if S = '' then
      S := Translator.GetTranslation(False, Comment1);
    if S <> '' then
     Comment1 := S;
  end;

  if Comment2 <> '' then
  begin
    S := Translator.GetTranslation(True, Comment2);
    if S = '' then
      S := Translator.GetTranslation(False, Comment2);
    if S <> '' then
     Comment2 := S;
  end;

  if Comment3 <> '' then
  begin
    S := Translator.GetTranslation(True, Comment3);
    if S = '' then
      S := Translator.GetTranslation(False, Comment3);
    if S <> '' then
     Comment3 := S;
  end;

  if Pos('%s', Result) = 0 then
  begin
     if Comment1 <> '' then
       Result := Result + '( ' + Comment1 + ')';
  end
  else
  begin
    Result := Translator.FormatStr(Result, Comment1, Comment2, Comment3);
  end;
end;

function GetFileLastModDate(const Filename: string): TDateTime;
begin
  Fileage(Filename, Result);
end;

function FillZeroes(const Source: string; const Count: integer): string;
begin
  Result := Source;
  while (Length(Result) < Count) do
    Result := '0' + Result;
end;

function SetESCCodes(const Source : string) : string;
{ Sets esq codes for wildcards characters as literals. The user marks a
literal wildcatd character with quotes
$ + A = asterisk
$ + Q = quesation mark}
var
  W1, W2 : string;
begin
   W1 := '"*"';
   W2 := '"?"';
   Result := StringReplace(Source, W1, '$A', [rfReplaceAll]);
   Result := StringReplace(Result, W2, '$Q', [rfReplaceAll]);
end;

function RevertESQCodes(const Source : string) : string;
{ Revets esq codes for wildcards to their literal values }
begin
  Result := StringReplace(Source, '$A', '*', [rfReplaceAll]);
  Result := StringReplace(Result, '$Q', '?', [rfReplaceAll]);
end;

{ Extract thumbnails from file based images }
function GetMediaThumb(const AFilePath: string; const ASize: integer;
  Bmp: Vcl.Graphics.TBitmap): HRESULT;
var
  fileShellItem: IShellItem;
  thumbProvider: IThumbnailProvider;
  bmpType: dword;
  hBmp: hBitmap;
begin
  Result := CoInitializeEx(nil, COINIT_APARTMENTTHREADED or
    COINIT_DISABLE_OLE1DDE);
  if Succeeded(Result) then
  begin
    Result := SHCreateItemFromParsingName(PWideChar(AFilePath), nil,
      IID_IShellItem, fileShellItem);
    if Succeeded(Result) then
    begin
      Result := fileShellItem.BindToHandler(nil, BHID_ThumbnailHandler,
        IID_IThumbnailProvider, thumbProvider);
      if Succeeded(Result) then
      begin
        Result := thumbProvider.GetThumbnail(ASize, hBmp, bmpType);
        if Succeeded(Result) then
          Bmp.Handle := hBmp;
      end;
      thumbProvider := nil;
    end;
    fileShellItem := nil;
  end;
  CoUninitialize;
end;

function GetGradEndColor(StartClr : TColor) : TColor; { Used by datasetviewer]}
   var
   Clr : Longint;
   R, G , B : Integer;
  begin
     Clr := ColorToRGB(StartClr);
     R := GetRValue(Clr);
     G := GetGvalue(Clr);
     B := GetBValue(Clr);
     R := R-50;
     if R < 0 then
      R := 0;
     G := G-50;
     if G < 0 then
       G := 0;
     B := B-50;
     if B < 0 then
      B := 0;
     Result := RGB(R, G, B);
  end;

function BitmapToIcon(ABitmap : VCL.Graphics.TBitmap) : TIcon;
var
   ImageList: TImageList;
   Clr : TColor;
begin
    Result := TIcon.Create;
    ImageList := TImageList.CreateSize(ABitmap.Width, ABitmap.Height);
    try
      Clr := Abitmap.Canvas.Pixels[0,ABitmap.Height-1];
      ImageList.AddMasked(ABitmap, Clr);
      ImageList.GetIcon(0, Result);
    finally
        ImageList.Free;
    end;
end;

procedure DrawAlphaBlend(DestBM: HDC; const Rect: TRect; const Blend: Byte;
  const SourceColor: TColor = clBlack);
var
  Ahdc: HDC; { handle of the DC we will create }
  bf: BLENDFUNCTION; { structure for alpha blending }
  Ahbitmap: hBitmap; { bitmap handle }
  bmi: BITMAPINFO; { bitmap header }
  pvBits: Pointer; { pointer to DIB section }
  BMWidth, BMHeight: ULONG; { window width/height }
  BM : VCL.Graphics.TBitmap;
const
  AC_SRC_ALPHA = $1;

begin
  BMWidth := Rect.Right - Rect.Left;
  BMHeight := Rect.Bottom - Rect.Top;

  if ((BMWidth = 0) and (BMHeight = 0)) then
    Exit;

  { create a DC for our bitmap -- the source DC for AlphaBlend }
  Ahdc := CreateCompatibleDC(DestBM);

  try
    { zero the memory for the bitmap info }
    ZeroMemory(@bmi, SizeOf(BITMAPINFO));

    { setup bitmap info }
    bmi.bmiHeader.biSize := SizeOf(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth := BMWidth;
    bmi.bmiHeader.biHeight := BMHeight;
    bmi.bmiHeader.biPlanes := 1;
    bmi.bmiHeader.biBitCount := 32; { four 8-bit components }
    bmi.bmiHeader.biCompression := BI_RGB;
    bmi.bmiHeader.biSizeImage := BMWidth * BMHeight * 4;

    { create our DIB section and select the bitmap into the dc }
    Ahbitmap := CreateDIBSection(Ahdc, bmi, DIB_RGB_COLORS, pvBits, 0, 0);
    BM := nil;
    try
      SelectObject(Ahdc, Ahbitmap);
      bf.BlendOp := AC_SRC_OVER;
      bf.BlendFlags := 0;
      bf.SourceConstantAlpha := Blend;
      bf.AlphaFormat := 0; { ignore source alpha channel }

      BM := VCL.Graphics.TBitmap.Create;
      BM.Canvas.Handle := AhDc;
      BM.Canvas.Brush.Color := SourceColor;
      BM.Canvas.FillRect(Rect);

      AlphaBlend(DestBM, Rect.Left, Rect.Top, BMWidth, BMHeight, Ahdc, 0, 0,
        BMWidth, BMHeight, bf);

    finally
      DeleteObject(Ahbitmap);
      BM.Free;
    end;
  finally
    DeleteDC(Ahdc);
  end;
end;

procedure RotateBitmapPLG(Bmp: Vcl.Graphics.TBitmap; Rads: Single;
  AdjustSize: Boolean; BkColor: TColor = clNone);
var
  C: Single;
  S: Single;
  Tmp: Vcl.Graphics.TBitmap;
  OffsetX: Single;
  OffsetY: Single;
  Points: array [0 .. 2] of TPoint;

  procedure SetTmp;
  begin
    Tmp.TransparentColor := Bmp.TransparentColor;
    Tmp.TransparentMode := Bmp.TransparentMode;
    Tmp.Transparent := Bmp.Transparent;
    Tmp.Canvas.Brush.Color := BkColor;
  end;

begin
  C := Cos(Rads);
  S := Sin(Rads);
  Tmp := Vcl.Graphics.TBitmap.Create;
  try
    Tmp.PixelFormat := Bmp.PixelFormat;
{$IFDEF D2010AndHigher}
    if Tmp.PixelFormat <> pf32Bit then
    begin
      SetTmp;
    end
    else
    begin
      Tmp.AlphaFormat := Bmp.AlphaFormat;
      Tmp.Canvas.Brush.Color := BkColor;
    end;
{$ELSE}
    SetTmp;
{$ENDIF}
    if AdjustSize then
    begin
      Tmp.Width := Round(Bmp.Width * Abs(C) + Bmp.Height * Abs(S));
      Tmp.Height := Round(Bmp.Width * Abs(S) + Bmp.Height * Abs(C));
      OffsetX := (Tmp.Width - Bmp.Width * C + Bmp.Height * S) / 2;
      OffsetY := (Tmp.Height - Bmp.Width * S - Bmp.Height * C) / 2;
    end
    else
    begin
      Tmp.Width := Bmp.Width;
      Tmp.Height := Bmp.Height;
      OffsetX := (Bmp.Width - Bmp.Width * C + Bmp.Height * S) / 2;
      OffsetY := (Bmp.Height - Bmp.Width * S - Bmp.Height * C) / 2;
    end;
    Points[0].X := Round(OffsetX);
    Points[0].Y := Round(OffsetY);
    Points[1].X := Round(OffsetX + Bmp.Width * C);
    Points[1].Y := Round(OffsetY + Bmp.Width * S);
    Points[2].X := Round(OffsetX - Bmp.Height * S);
    Points[2].Y := Round(OffsetY + Bmp.Height * C);
    PlgBlt(Tmp.Canvas.Handle, Points, Bmp.Canvas.Handle, 0, 0, Bmp.Width,
      Bmp.Height, 0, 0, 0);
    Bmp.Assign(Tmp);
  finally
    Tmp.Free;
  end;
end;

function RectWidth(Rect: TRect): integer;
begin
  Result := Rect.Right - Rect.Left;
end;

function RectHeight(Rect: TRect): integer;
begin
  Result := Rect.Bottom - Rect.Top;
end;

function RectCenter(Rect: TRect): TPoint;
begin
  Result.X := Rect.Left + (RectWidth(Rect) div 2);
  Result.Y := Rect.Top + (RectHeight(Rect) div 2);
end;

procedure FitToRect(AHostRect, AClientRect: TRect; Centered: Boolean;
  var FitRect: TRect);
{ Computes size and position of AClientRect so that it fills the maximum possible space in
  AHostRect without loosing proportions }
var
  W, H: integer;
  wprop, hprop: Double;
begin
  W := RectWidth(AHostRect);
  H := RectHeight(AHostRect);
  wprop := W / RectWidth(AClientRect);
  hprop := H / RectHeight(AClientRect);
  if wprop <= hprop then
  begin
    H := Round(wprop * RectHeight(AClientRect));
    FitRect.Left := AHostRect.Left;
    FitRect.Right := AHostRect.Right;
    if Centered then
    begin
      FitRect.Top := RectCenter(AHostRect).Y - (H div 2);
      FitRect.Bottom := FitRect.Top + H;
    end
    else
    begin
      FitRect.Top := AHostRect.Top;
      FitRect.Bottom := FitRect.Top + H;
    end;
  end
  else
  begin
    W := Round(hprop * RectWidth(AClientRect));
    FitRect.Top := AHostRect.Top;
    FitRect.Bottom := AHostRect.Bottom;
    if Centered then
    begin
      FitRect.Left := RectCenter(AHostRect).X - (W div 2);
      FitRect.Right := FitRect.Left + W;
    end
    else
    begin
      FitRect.Left := AHostRect.Left;
      FitRect.Right := FitRect.Left + W;
    end;
  end;
end;

procedure ScaleBitmap(ABitmap: Vcl.Graphics.TBitmap; Width, Height: integer;
  DownOnly: Boolean = True);
var
  R1, R2, R3: TRect;
  OrigSize: TSize;
  NewBM: Vcl.Graphics.TBitmap;
begin
  if (Width <= 0) or (Height <= 0) then
    Exit;
  NewBM := Vcl.Graphics.TBitmap.Create;
  try

    OrigSize.cx := ABitmap.Width;
    OrigSize.cy := ABitmap.Height;
    if (OrigSize.cx > Width) or (OrigSize.cy > Height) or not DownOnly then
    begin
      R1.Left := 0;
      R1.Top := 0;
      R1.Right := Width;
      R1.Bottom := Height;
      R2.Left := 0;
      R2.Top := 0;
      R2.Bottom := OrigSize.cy;
      R2.Right := OrigSize.cx;
      FitToRect(R1, R2, false, R3);
      NewBM.Width := RectWidth(R3);
      NewBM.Height := RectHeight(R3);

      SetStretchBltMode(NewBM.Canvas.Handle, STRETCH_HALFTONE);
      SetBrushOrgEx(NewBM.Canvas.Handle, 0, 0, nil);
      StretchBlt(NewBM.Canvas.Handle, R3.Left, R3.Top, RectWidth(R3),
        RectHeight(R3), ABitmap.Canvas.Handle, R2.Left, R2.Top, RectWidth(R2),
        RectHeight(R2), SRCCOPY);
    end
    else
    begin
      NewBM.Width := OrigSize.cx;
      NewBM.Height := OrigSize.cy;
      NewBM.Canvas.Draw(0, 0, ABitmap);
    end;
    ABitmap.Assign(NewBM);
  finally
    NewBM.Free;
  end;
end;

function GetCompName(FName: string): string;
var
  iLen: cardinal;
  P: integer;
begin
  if FName = '' then
  begin
    iLen := MAX_COMPUTERNAME_LENGTH + 1;
    Result := StringOfChar(#0, iLen);
    GetComputerName(pchar(Result), iLen);
    SetLength(Result, iLen);
    Exit;
  end;
  Result := FName;
  Delete(Result, 1, 2);
  P := Pos('\', Result);
  if P > 0 then
    Result := Copy(Result, 1, P - 1);
end;

function HasWildCards(AVal: string): Boolean;
begin
  Result := (Pos('*', AVal) <> 0) or (Pos('?', AVal) <> 0);
end;

procedure CreateListFromDelimiter(const ADelimiter: Char; AText: string;
  AList: TStringList);
var
  P: integer;
  S: string;
begin
  AList.Clear;
  P := Pos(ADelimiter, AText);
  while P <> 0 do
  begin
    S := Copy(AText, 1, P - 1);
    Delete(AText, 1, P);
    AList.Add(S);
    P := Pos(ADelimiter, AText);
  end;
  if AText <> '' then
    AList.Add(AText);
end;

function CharInQuotes(S: string; Index: integer; InParenthesis : Boolean): Boolean;
var
  P1, P2: integer;
const
  qt = '''';

  procedure CheckParenthesis;
  var
    LCnt, RCnt : integer;
  begin
    P1 := Index-1;
    RCnt := 0;
    LCnt := 0;
    while P1 > 0 do
    begin
      if S[P1] = '(' then
      begin
        inc(LCnt)
      end
      else if S[P1] = ')' then
        inc(RCnt);
      Dec(P1);
    end;
    if (LCnt > 0) or (RCNt > 0) then
      Result := (LCnt <> RCnt);

  end;

begin
  Result := false;
  P1 := Pos(qt, S);
  if P1 = 0 then
  begin
    if InParenthesis then
      CheckParenthesis;
    Exit;
  end;
  repeat
    P2 := Pos(qt, S, P1 + 1);
    if P2 = 0 then
      Break;
    if (Index > P1) and (Index < P2) then
    begin
      Result := True;
      Break;
    end;
    P1 := Pos(qt, S, P2 + 1);
  until P1 = 0;
  if not Result and InParenthesis then
    CheckParenthesis;
end;

function PosRestrict(Substring, SearchString: string; ExcludeParenthesis : Boolean;
  StartPos: integer = 1): integer;
var
  P : integer;
begin
  Substring := Substring.ToUpper;
  SearchString := SearchString.ToUpper;
  Result := Pos(Substring, SearchString, StartPos);
  P := Result;
  while CharInQuotes(SearchString, Result, ExcludeParenthesis) and (Result <> 0) do
    Result := Pos(Substring, SearchString, Result + 1);
  if (Result = 0) and (P <> 0) then
    Result := PosRestrict(SubString, SearchString, ExcludeParenthesis, P + 1);
end;

function TrimBlanks(const AStr: string; TrimQuoted: Boolean = True): string;
var
  HasStQuotes, HasNdQuotes: Boolean;
  P: integer;
begin
  Result := AStr;
  if AStr = '' then
    Exit;
  while Pos(' ', Result) = 1 do
    Delete(Result, 1, 1);
  while Pos(#$D#$A, Result) = 1 do
    Delete(Result, 1, 2);
  HasStQuotes := (Pos('''', AStr) = 1);
  HasNdQuotes := (Pos('''', AStr, 2) = Length(AStr));
  if HasStQuotes and HasNdQuotes and not TrimQuoted then
    Exit;
  if HasStQuotes then
    Delete(Result, 1, 1);
  if HasNdQuotes then
    Delete(Result, Length(Result), 1);
  P := PosRestrict('  ', Result, False);
  while P <> 0 do
  begin
    Delete(Result, P, 1);
    P := PosRestrict('  ', Result, False);
  end;
  while (Result <> '') and (Result[1] = ' ') do
    Delete(Result, 1, 1);
  while (Result <> '') and (Result[Length(Result)] = ' ') do
    Delete(Result, Length(Result), 1);
  if HasStQuotes then
    Result := '''' + Result;
  if HasNdQuotes then
    Result := Result + '''';

end;

procedure TrimList(AList: TStringList; TrimQuoted: Boolean = True);
var
  i: integer;
  S: string;
  NewList: TStringList;
begin
  NewList := TStringList.Create;
  try
    for i := 0 to AList.Count - 1 do
    begin
      S := AList[i];
      S := TrimBlanks(S, TrimQuoted);
      if S <> '' then
        NewList.Add(S);
    end;
    AList.Assign(NewList);
  finally
    NewList.Free;
  end;
end;

function Quoted(const S : string) : string;
begin
  Result := '''' + S + '''';
end;

procedure GetResourceElements(const ResourceStr: string;
  var Name, Address: string);
var
  P: integer;
begin
  Address := '';
  Name := '';
  P := Pos(c_ResourceSeparator, ResourceStr);
  if P = 0 then
    Address := ResourceStr
  else
  begin
    Name := TrimBlanks(Copy(ResourceStr, 1, P - 1));
    Address := TrimBlanks(Copy(ResourceStr, P + Length(c_ResourceSeparator),
      Length(ResourceStr)));
  end;
end;

function CreateResourceStr(const Name, Address: string): string;
begin
    Result := Name + c_ResourceSeparator + Address;
end;

function GetResourceDisplayElement(const ResourceStr: string): string;
var
  A, N: string;
begin
  GetResourceElements(ResourceStr, N, A);
  if N <> '' then
    Result := N
  else
    Result := A;
end;

function ExecuteResource(const Resource: string): integer;
var
  N, A: string;
begin
  GetResourceElements(Resource, N, A);
  if Trim(A) = '' then
  begin
    Result := -1;
    Exit;
  end;
  Result := ShellExecute(0, 'Open', pchar(A), nil, nil, SW_SHOW);
  if Result > 32 then
    Result := 0;
end;

function IsMediaFile(AFileName: string): Boolean;
{ Any types of graphic or audio }
begin
  if IsUrl(AFileName) then
    Result := True
  else
    Result := (GetFileType(AFileName) <> iftUnKnown) and FileExists(AFileName);
end;


function GetAddressFromClipBrd(var IsFileUrl : Boolean) : string;
begin
     IsFileUrl := false;
     if ClipBoard.HasFormat(cf_Text) then
     begin
        Result := ClipBoard.AsText;
        if not FileExists(Result) then
        begin
          if not IsUrl(Result) then
            Result := ''
          else
          begin
            if (GetFileType(Result) = iftUnKnown) then
              Result := ''
            else
              IsFileUrl := True;
          end;
        end;
     end
     else
       Result := '';
end;

procedure RemoveComments(S: TStrings);
var
  i: integer;
  s1: string;
  sl: TStringList;
  Indx: integer;
  Found: Boolean;
begin
  if S.Count = 0 then
    Exit;
  Found := false;
  sl := TStringList.Create;
  try
    for i := 0 to S.Count - 1 do
    begin
      s1 := S[i];
      Indx := PosRestrict('//', s1, False);
      if Indx <> 0 then
      begin
        Found := True;
        if Indx = 1 then
          Continue;
        Delete(s1, Indx, Length(s1));
        sl.Add(s1);
      end
      else
        sl.Add(s1);
    end;
    if Found then
      S.Assign(sl);

    s1 := S.Text;
    if PosRestrict('&', s1, False) = 0 then
      Exit;
    while Pos(' ' + #13#10, s1) <> 0 do
      s1 := StringReplace(s1, ' ' + #13#10, #13#10, [rfReplaceAll]);
    s1 := StringReplace(s1, '&' + #13#10, ' ', [rfReplaceAll]);
    s1 := TrimBlanks(s1, false);
    sl.Text := s1;
    TrimList(sl, false);
    S.Assign(sl);

  finally
    sl.Free;
  end;
end;

function PosCaseless(Substring, SearchString: string;
  StartPos: integer = 1): integer;
begin
  Substring := Substring.ToUpper;
  SearchString := SearchString.ToUpper;
  Result := Pos(Substring, SearchString, StartPos);
end;

function GetInnerParenthesis(var PExpression: string;
  var Removed: string): string;
{ Retrieves the content of the first innermost parentheseis. Returns the
  string that was processed in Removed }
var
  P: integer;
  P2: integer;
begin
  Result := '';
  P := PosRestrict('(', PExpression, false);
  if P = 0 then
    Exit;
  P2 := 0;
  while P <> 0 do
  begin
    P2 := P;
    P := PosRestrict('(', PExpression, False, P + 1);
  end;
  P := PosRestrict(')', PExpression, False, P2);
  if P = 0 then
  begin
    Result := '#Error';
    Exit;
  end;
  Inc(P2);
  Result := Copy(PExpression, P2, P - P2);
  Dec(P2);
  Removed := Copy(PExpression, P2, P - P2 + 1);
end;

function ToUNC(FName: string; ServerName : string = ''): string;
var
  P: integer;
  iLen: cardinal;
begin
  Result := FName;
  if IsUrl(FName) then
    Exit;
  if Pos('\\', FName) = 1 then
  begin
    Exit;
  end;

  if ServerName = '' then
  begin
   iLen := MAX_COMPUTERNAME_LENGTH + 1;
   Result := StringOfChar(#0, iLen);
   GetComputerName(pchar(Result), iLen);
   SetLength(Result, iLen);
  end
  else
    Result := ServerName;
  P := Pos(':\', FName);
  if P <> 0 then
    Delete(FName, 1, 3);
  if ServerName <> '' then
    Result := Result + '\' + FName
  else
    Result := '\\' + Result + '\' + FName;

end;

function IsUNC(const AFileName : string) : Boolean;
begin
  Result := false;
  if IsUrl(AFileName) then
    Exit;
  if Pos('\\', AFileName) = 1 then
    Result := True;
end;

function UNCFromFileName(const AFileName : string) : string;
var
  P : integer;
begin
  Result := '';
  if IsUrl(AFileName) then
    Exit;
  P := Pos('\\', AFileName);
  if P <> 1 then
    Exit;
  P := Pos('\', AFileName, P+2);
  if P = 0 then
    Exit;
  Result := Copy(AFileName, 1, P-1);
end;

function CollapseUNC(const AFileName : string) : string;
var
 P : integer;
begin
   Result := AFileName;
   if IsURL(AFileName) then
     Exit;
   P := Pos('\\', AFileName);
  if P <> 1 then
    Exit;
  P := Pos('\', AFileName, P+2);
  if P = 0 then
    Exit;
  Delete(Result, 1, P);
  Insert('C:\', Result, 1);
end;

function NameCheck(const AName : string) : Boolean;
var
  i : integer;
begin
  Result := True;
  if Length(AName) > 0 then
    begin
      if not CharInSet(AName[1], ['a' .. 'z', 'A' .. 'Z', '_']) then
      begin
        Result := false;
        Exit;
      end;
    end;
    for i := 2 to Length(AName) do
    begin
      if not CharInSet(AName[i], ['a' .. 'z', 'A' .. 'Z', '_', '0' .. '9']) then
      begin
        Result := false;
        Break;
      end;
    end;
end;

function SysIdOf(const SysName: string): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 1 to c_SysFieldCount do
    if SameText(c_SysFieldNames[i], SysName) then
    begin
      Result := i;
      Exit;
    end;
end;

function GetSysDataType(const AFieldName: string): TDataType;
var
  Indx: integer;
  D: integer;
begin
  Result := dtString;
  if PosCaseless('PID_', AFieldName) <> 0 then
  begin
    Indx := PIDOf(AFieldName);
    if Indx > 0 then
    begin
      D := DataTypeOfPID(Indx);
      if D in [DT_FLOAT, DT_INTEGER] then
        Result := dtNumber
      else if D = DT_STRING then
        Result := dtString
      else if D = DT_DATE then
        Result := dtDate
      else if D = DT_BOOLEAN then
        Result := dtBoolean
    end
    else
      HandleILException(0, 'Not found');
    Exit;
  end;
  if (SameText(AFieldName, c_SysFieldNames[SYS_FileName])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_RecordState])) then
    Result := dtString
  else if (AnsiSameText(AFieldName, c_SysFieldNames[SYS_ThumbnailSize])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_ThumbnailOffset])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_Level])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_SiblingIndex])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_RecordID])) or
    (SameText(AFieldName, c_SysFieldNames[SYS_LinkID])) then
    Result := dtNumber
  else
    Result := dtDate;
end;

procedure CheckOrientation(const AFileName: string; ABitmap: Vcl.Graphics.TBitmap);
var
  lRadians: Real;
  Angl: integer;
  Prop: TPropertyProvider;
  Orient: integer;
  O: Variant;
begin
  Prop := TPropertyProvider.Create(AFileName);
  try
    Angl := 0;
    O := Prop.GetPropValueAsVariant(PID_PHOTO_ORIENTATION);
    if VarIsNull(O) then
      Orient := 1
    else
      Orient := O;
    if Orient = 6 then
      Angl := 90
    else if Orient = 3 then
      Angl := 180
    else if Orient = 8 then
      Angl := 270;
    if Angl <> 0 then
    begin
      lRadians := PI * Angl / 180;
      RotateBitmapPLG(ABitmap, lRadians, True, clWhite);
    end;

  finally
    Prop.Free;
  end;
end;

function GetFileType(const AFileName: string): TFileType;
var
  Ext: string;
begin
  {$IfDef UseIndy}
  if IsUrl(AFileName) then
  begin
    Result := iftURL;
    Exit;
  end;
  {$Else}
  if IsUrl(AFileName) then
  begin
    Result := iftUnKnown;
    Exit;
  end;
  {$EndIf}

  Ext := ExtractFileExt(AFileName).ToUpper;
  if (Ext = '.JPG') or (Ext = '.JPEG') then
    Result := iftJPG
  else if (Ext = '.TIF') or (Ext = '.TIFF') then
    Result := iftTiff
  else if (Ext = '.WDP') then
    Result := iftWPD
  else if (Ext = '.PNG') then
    Result := iftPng
  else if (Ext = '.GIF') then
    Result := iftGif
  else if (Ext = '.BMP') then
    Result := iftBMP
  else if Ext = '.AVI' then
    Result := iftAVI
  else if Ext = '.WMV' then
    Result := iftWMV
  else if (Ext = '.MP4') or (Ext = '.M4P') or (Ext = '.M4V') then
    Result := iftMP4
  else if Ext = '.MOV' then
    Result := iftMOV
  else if Ext = '.MPG' then
    Result := iftMPG
  else if Ext = '.OGV' then
    Result := iftOGV
  else if Ext = '.MP3' then
    Result := iftMp3
  else
    Result := iftUnKnown;
end;

function IsVideoFile(const FileName: string): Boolean;
var
  FT: TFileType;
begin
  FT := GetFileType(FileName);
  Result := (FT in [iftAVI .. iftOGV]);
end;

function IsAudioFile(FileName: string): Boolean;
var
  FT: TFileType;
begin
  FT := GetFileType(FileName);
  Result := (FT in [iftMP3]);
end;

function IsImageFile(FileName: string): Boolean;
var
  FT: TFileType;
begin
  FT := GetFileType(FileName);
  Result := (FT in [iftBMP .. iftWPD {$IfDef UseIny} , iftURL{$Endif}]);
end;

function GetBitmapFromFile(const AFileName: string; const Quality: TJPEGScale)
  : Vcl.Graphics.TBitmap;
var
  FT: TFileType;
  Img: TWICImage;
  JPG: TJPEGImage;
begin
  Result := nil;
  FT := GetFileType(AFileName);
  if FT = iftUnKnown then
    Exit;
  Result := Vcl.Graphics.TBitmap.Create;
  {$IfDef UseIndy}
  if FT = iftURL then
  begin
    try
      Img := DownLoadImage(AFileName);
      if Img = nil then
      begin
        FreeAndNil(Result);
        Exit;
      end;
    except
      Result.Free;
      raise;
    end;
    if not Img.Empty then
      Result.Assign(Img);
    Img.Free;
  end
  else
  {$EndIf}
  if FT = iftJPG then
  begin
    JPG := TJPEGImage.Create;
    JPG.Scale := Quality;
    try
      JPG.LoadFromFile(AFileName);
      Result.Assign(JPG);
    finally
      JPG.Free;
    end;
  end
  else if FT in [iftTiff, iftWPD, iftPng, iftGif] then
  begin
    Img := TWICImage.Create;
    try
      Img.LoadFromFile(AFileName);
      Result.Assign(Img);
    finally
      Img.Free;
    end;
  end
  else
  begin
    Result.LoadFromFile(AFileName);
  end;
end;

function IsFileInUse(FileName: TFileName): Boolean;
var
  HFileRes: hfile;
begin
  Result := false;
  if not FileExists(FileName) then
    Exit;
  HFileRes := CreateFile(pchar(FileName), GENERIC_READ or GENERIC_WRITE, 0, nil,
    OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  Result := (HFileRes = INVALID_HANDLE_VALUE);
  if not Result then
    CloseHandle(HFileRes);
end;

function CreateScaledImage(const AFileName: String; const AWidth, AHeight: integer)
  : Vcl.Graphics.TBitmap;
var
  JP: TJPEGImage;
  Info: TWin32FileAttributeData;
  Sz: int64;
  AStr: String;
  FT: TFileType;
begin
  FT := GetFileType(AFileName);
  if FT = iftUnKnown then
  begin
    Comment1 := AFileName;
    HandleILException(E_UnsupportedFileFormat, '');
  end;
  Sz := 0;
  {$IfDef UseIndy}
  if not(FT = iftURL) then
  {$EndIf}
    if GetFileAttributesEx(PWideChar(AFileName), GetFileExInfoStandard, @Info)
    then
      Sz := int64(Info.nFileSizeLow) or int64(Info.nFileSizeHigh shl 32);
  if FT = iftJPG then
  begin
    JP := TJPEGImage.Create;
    try
      if Sz > 3000000 then
        JP.Scale := jsHalf
      else if Sz > 1000000 then
        JP.Scale := jsQuarter
      else if Sz > 500000 then
        JP.Scale := jsHalf;
      JP.LoadFromFile(AFileName);
      Result := Vcl.Graphics.TBitmap.Create;
      Result.Assign(JP);
      AStr := AFileName;
      CheckOrientation(AStr, Result);
      ScaleBitmap(Result, AWidth, AHeight);
    finally
      JP.Free;
    end
  end
  else
  begin
    Result := GetBitmapFromFile(AFileName, jsHalf);
    { Quality is indifferent here }
    if Result <> nil then
      ScaleBitmap(Result, AWidth, AHeight);
  end;
end;

{ TTableNode -------------------------------------------------------------- }

constructor TTableNode.Create(ATable: TILDataTable; ATree: TTableManager);
begin
  inherited Create;
  FTree := ATree;
  FTable := ATable;
  if Length(FTree.FTreeTables) = 0 then
    ATable.FActive := True;
  SetLength(FTree.FTreeTables, Length(ATree.FTreeTables) + 1);
  FTree.FTreeTables[High(FTree.FTreeTables)] := ATable;
  FTree.FNodes.Add(Self);
end;

function TTableNode.GetCount: integer;
var
  i: integer;
begin
  Result := 0;
  for i := 0 to High(FTree.FTreeTables) do
  begin
    if FTree.FTreeTables[i].MasterSource = FTable then
      Inc(Result);
  end;
end;

function TTableNode.GetChildren(Index: integer): TTableNode;
var
  i: integer;
  Ind: integer;
begin
  Result := nil;
  Ind := -1;
  for i := 0 to High(FTree.FTreeTables) do
  begin
    if (FTree.FTreeTables[i].MasterSource = FTable) then
    begin
      Inc(Ind);
      if Ind = Index then
      begin
        Result := FTree.NodeOf(FTree.FTreeTables[i]);
        Break;
      end;
    end;
  end;
end;

function TTableNode.GetCountAll: integer;

  procedure DoCount(ANode: TTableNode);
  var
    N: TTableNode;
    i: integer;
  begin
    for i := 0 to ANode.Count - 1 do
    begin
      N := ANode.Children[i];
      Inc(Result);
      DoCount(N);
    end;
  end;

begin
  Result := 0;
  DoCount(Self);
end;

function TTableNode.GetLevel: integer;
var
  N: TTableNode;
  procedure DoSearch(ANode: TTableNode; Level: integer);
  var
    N: TTableNode;
    i: integer;
    Lvl: integer;
  begin
    if Result <> 0 then
      Exit;
    Lvl := Level;
    for i := 0 to ANode.Count - 1 do
    begin
      N := ANode.Children[i];
      if N = Self then
      begin
        Result := Lvl;
        Break;
      end;

      DoSearch(N, Level + 1);
    end;
  end;

begin
  Result := 0;
  N := FTree.Root;
  DoSearch(N, 1);
end;

function TTableNode.GetParent: TTableNode;
var
  N: TTableNode;
  procedure DoSearch(ANode: TTableNode);
  var
    N: TTableNode;
    i: integer;
  begin
    if Result <> nil then
      Exit;
    for i := 0 to ANode.Count - 1 do
    begin
      N := ANode.Children[i];
      if N = Self then
      begin
        Result := ANode;
        Break;
      end;

      DoSearch(N);
    end;
  end;

begin
  Result := nil;
  N := FTree.Root;
  if N = Self then
    Exit;
  DoSearch(N);
end;

function TTableNode.GetAllChildren(Index: integer): TTableNode;
var
  Ind: integer;

  procedure DoGet(ANode: TTableNode);
  var
    N: TTableNode;
    i: integer;
  begin
    for i := 0 to ANode.Count - 1 do
    begin
      N := ANode.Children[i];
      Inc(Ind);
      if Index = Ind then
      begin
        Result := N;
        Break;
      end;
      DoGet(N);
      if Result <> nil then
        Break;
    end;
  end;

begin
  Ind := -1;
  Result := nil;
  DoGet(Self);
end;

{ TWorkHistoryElement -------------------------------------------------------}

destructor TWorkHistoryElement.Destroy;
var
  i : integer;
begin
  FValues.Free;
  for i := 0 to FWorkFields.Count-1 do
  begin
     TILWorkField(FWorkFields.Objects[i]).Free;
  end;
  FWorkFields.Free;
  inherited;
end;

{ TFocusHandler -------------------------------------------------------------

There is one focus handler per table. These cooperate in creating the table chains.
The focus handler is resposible for populating the parent(s) and displaying the
children. Its like a handling over a baton. The handler that receives the baton,
does its part of the job, then handling it to the parent and / or the children in
a cascading movement}

constructor TFocusHandler.Create;
begin
   inherited;
end;

destructor TFocusHandler.Destroy;
begin
   inherited;
end;

function TFocusHandler.AcceptTarget(UseParent : Boolean; ATarget: TILDataTable) : Boolean;
begin
     if UseParent then { Uptrace}
     begin
        Result := (mdLookupParents in FTable.DataHandler.MDOptions)
        and not (mdDenyChild in ATarget.DataHandler.MDOptions);
     end
     else
     begin
        Result := (mdLookupChildren in FTable.DataHandler.MDOptions)
        and not (mdDenyParent in ATarget.DataHandler.MDOptions);
     end;
end;

procedure TFocusHandler.Execute(Adataset : TILDataset; APosition : integer);
var
  OldPos : integer;
  TN : TTreeNode;

  procedure WriteCtrls(Tbl : TILTable; Recrd : TILDataRecord);
  var
      T : TTableManager;
      i : integer;
      s : string;
    begin
      T := TMan;
      if T = nil then
        Exit;
      if (FDataset.SelCount = 0) or not Tbl.MultiSelected then
      begin
        begin
          if Recrd = nil then
          begin
           Tbl.ClearControls(duData);
          end
          else
          begin
           TBL.FPrivateCalc := True;
           { This influences on the TILRecord.ValueOf function that is used
           by WriteControls}
           Tbl.WriteControls(Recrd, duData);
           TBL.FPrivateCalc := False;
          end;
        end;
      end;

      S := '';
      if (Recrd = nil) and Assigned(FTable.DataHandler.FOnUpdateLongText)  then
      begin
        for I := 0 to FTable.Fields.Count-1 do
        begin
            if (FTable.Fields.Items[i].DataType = dtLongText) then
             FTable.DataHandler.FOnUpdateLongText(FTable.Datahandler, nil, S, [ltaClearEditors])
        end;
      end

      else if Assigned(FTable.DataHandler.FOnUpdateLongText) then
      for I := 0 to Recrd.Count-1 do
      begin
         if (Recrd[i].FieldDef.DataType = dtLongText) then
         begin
            if Recrd.IsNew then
             FTable.DataHandler.FOnUpdateLongText(FTable.Datahandler, nil, S, [ltaClearEditors])
            else
            begin
              if FTable.IsReadOnly then
               FTable.DataHandler.FOnUpdateLongText(FTable.Datahandler, Recrd[i], S, [ltaDownload, ltaDisable])
              else
               FTable.DataHandler.FOnUpdateLongText(FTable.Datahandler, Recrd[i], S, [ltaDownload]);
            end;
            Recrd[i].ResetLTFlags;
         end;
      end;
    end;

     procedure Reset;
     begin
         FUptracing := False;
         FDownTracing := false;
         FOriginator := nil;
         FHasExecuted := false;
         TMan.FActiveFocHandler := FSender;
     end;

     procedure CheckUnsaved;
     var
      Res : integer;
     begin
       if (APosition <> OldPos) and not (OldPos >= FTable.Dataset.Count)
        and (FTable.FocusedRecord <> nil) then
       begin
         if FTable.FocusedRecord.Changed then
         begin
           Res := FTable.FocusedRecord.Validate(True);
         end
         else
           Res := -1;
         if FTable.FocusedRecord.IsNew then
         begin
           if FTable.FocusedRecord.IsDataCleared(False) then
           begin
             Reset;
             FTable.Dataset.Delete(FTable.FocusedRecord.Index);
             if Assigned(FTable.Datahandler.FOnDatasetChange) then
               FTable.Datahandler.FOnDatasetChange(FTable, dsDeleted);
             Exit;
           end;
           if (Res = 0) and Tman.AutoSave then
            FTable.DBWork.Save
           else if (Res > 0) and Tman.AutoSave then
           begin
            Reset;
            if FTable.HasViewer and (roEditable in FTable.FViewer.ReportOptions) then
             FTable.FViewer.Report.SelectedCell := nil;
            HandleILException(Res);
           end;
         end
         else if (Res = 0) and Tman.AutoSave then
           FTable.DBWork.Save
         else if (Res > 0) and Tman.AutoSave then
         begin
           Reset;
           if FTable.HasViewer and (roEditable in FTable.FViewer.ReportOptions) then
             FTable.FViewer.Report.SelectedCell := nil;
           HandleILException(Res);
         end;
       end;
     end;

     procedure CheckUpdates;
     begin
       if not Tman.UseCommandModes or (Tman.UseCommandModes and (TMan.CommandMode = cmUpdate)) then
        if (APosition <> OldPos) and not (OldPos >= FTable.Dataset.Count)
        and TMan.AutoUpdate and (FTable.FocusedRecord <> nil) then
        begin
          if FTable.DBWork.NeedsUpdate(FTable.FocusedRecord) then
          begin
            FTable.DBWork.Update(nil);
          end;
        end;
     end;

begin
   if FHasExecuted then
   begin
     HandleILException(E_Circular);
   end;

   FSender := TMan.FActiveFocHandler;
   TMan.FActiveFocHandler := Self;
   FTable := ADataset.Ilib;
   FDataset := ADataset;
   FHasExecuted := True;
   OldPos := FDataset.Position;
   If FSender = nil then
     FOriginator := self;

   {if (APosition = OldPos) and (APosition <> -1) then
   begin
     Reset;
     RunHandler;
     Exit;
   end;}

   CheckUpdates;
   CheckUnsaved;

   if (APosition = -1) or (ADataset.Count = 0) then
   begin
     if FUpTracing then
     begin
       Reset;
       Exit;
     end;
     FTable.Datahandler.CreateSummary(nil);
     WriteCtrls(FTable, nil);
     ADataset.FPosition := -1;
     FFocRecord := nil;
     if mdLookupChildren in FTable.DataHandler.MDOptions then
       ClearDownList;
     Reset;
     RunHandler;
     Exit;
   end
   else
   begin
    if ADataset.Position < ADataset.Count then
    begin
     if ADataset.Position <> -1 then
      if ADataset.Items[ADataset.Position].FEditingImageText then
       ADataset.Items[ADataset.Position].CancelEditImageText;
    end;

    FFocRecord := FDataset[APosition];

    if FTable.TVControl <> nil then
    with FTable.TVControl do
    begin
         if Items.Count = 0 then
           BuildTree(FTable.Dataset);
         Building := True;
         try
            TN := NodeOfRecord(FFocRecord);
            Selected := TN;
          finally
            Building := false;
          end;
    end;

    { Set position and selection }
    if (FDataset.Selcount = 1) and (FDataset.FPosition <> -1) and (FDataset.FPosition < FDataset.Count)
    then
    begin
        FDataset.Items[FDataset.FPosition].FSelected := false;
        FDataset.FSelState := -1;
    end;

    FDataset.FPosition := APosition;

    if FTable is TILTreeViewTable then
        TILTreeViewTable(FTable).FillSiblings(FDataset.Items[FDataset.FPosition]);
    if not FTable.MultiSelected or ((FDataset.Selcount = 1) and FTable.FMultiSelect) then
    begin
       FDataset.Items[APosition].FSelected := True;
       FDataset.FSelState := -1;
    end;
   end;

   { Handle record properties }
   if not Assigned(FDataset.Items[FDataset.FPosition].PropertyValues) then
     FDataset.Items[FDataset.FPosition].CreateProperties(FDataset.Items[FDataset.FPosition].FileName);

  { Write data of fucused record to DACS }
   WriteCtrls(FTable, FDataset.FocusedRecord);
   FTable.Datahandler.CreateSummary(nil);

   { Update cell selection }
   if FTable.HasViewer and (FTable.Viewer.FReport.FSelectedCell <> nil) then
   begin
     if FDataset.IsReadOnly then
       FTable.Viewer.Report.FSelectedCell := nil
     else
     begin
       if FTable.Viewer.Report.SelectedCell.Data.Index <> APosition then
        FTable.Viewer.Report.SelectedCell := FTable.Viewer.Report.Cells[
         FTable.Viewer.Report.RecordIndexToRowIndex(FTable.Dataset.Position), 0];
     end;
   end;

   { Handle multi value xpansion }
   if FTable.HasViewer and (roAutoExpand in FTable.FViewer.ReportOptions)
      and (FTable.FViewer.ViewMode = vmReport) and
        not FTable.FViewer.CanReportDisplayImages then
   begin
     if OldPos < FTable.Dataset.Count then
       FTable.Viewer.FReport.Collapse(OldPos);
     FTable.Viewer.FReport.Expand(FDataset.Position);
   end;

   if FDataset.FIsJoined or (FTable is TILView) then
   begin
     Reset;
     RunHandler;
     Exit;
   end;

   { Track parents and children }
   if ADataset.Ilib.IsFreeTable and (ADataset.Ilib.MasterLinkCount = 0) then
   begin
   end
   else
   try
      CreateUpList;
      CreateDownList;
   except
     Reset;
     raise;
   end;

   try
    FTable.Inval;
   except
    Reset;
    raise;
   end;
   Reset;
   RunHandler;
end;

procedure TFocusHandler.CreateUpList;
var
  R : TILDataRecord;
  P : TILTable;
  i : integer;
  InListIndex : integer;

  procedure CheckLinkList;
  var
    I, J, K : integer;
    T : TILDataTable;
    S : string;
    SQ : TStringList;
    Found : Boolean;
  begin
   for I := 0 to FFocRecord.Ilib.MasterLinkCount-1 do
   { Move through the master tables of the focused record }
   begin
     if (FSender = FFocRecord.Ilib.MasterLinks[i].FFocHandler) and FDownTracing then
       Continue;
     if not AcceptTarget(True, FFocRecord.Ilib.MasterLinks[i]) then
       Continue;

     Found := false;
     T := FFocRecord.Ilib.MasterLinks[i];
     for j := 0 to FFocRecord.Count-1 do
     { Move through the fields of the focused record to find the master table that links to it}
     begin
       S := '';
       if (FFocRecord[j].FieldDef.LookupType = looTableKey) and
       SameText(FFocRecord[j].FieldDef.LookupTable.TableName, T.TableName)
       then
       begin
           S := FFocRecord[j].Asstring;
           for k := 0 to T.Dataset.Count-1 do
           begin
             { Move through the master table dataset, to identify the record with a lookup field that
             matches the value of the corresponding field in focused record}
             if T.Dataset[k][FFocRecord[j].FieldDef.LookupField].AsString = S then
             begin
               T.FFocHandler.FUptracing := True;
               try
                T.Dataset.Position := k;
               finally
                T.FFocHandler.FUptracing := False;
               end;
               Found := True;
               Break;
             end;
           end;
           if Found then
            Break;
       end
       else
         Continue;
       if (S = '') then
         Continue;
       if FFocRecord[j].FieldDef.DataType = dtString then
         S := '''' + S + '''';
       SQ := TStringList.Create;
       try
         SQ.Add('select * where ' +  FFocRecord[j].FieldDef.LookupField + ' = ' + S);

         T.FFocHandler.FUptracing := True;
         try
           T.DBView.Search(SQ);
         finally
           T.FFocHandler.FUptracing := False;
         end;
       finally
         SQ.Free;
       end;
     end;
     if not Found then
      T.Dataset.Position := 0;
   end;
  end;

begin
    if FTable.IsRoot then
     Exit;

   if not (mdLookupParents in FTable.DataHandler.MDOptions) then
     Exit;

   if FDownTracing then
   begin
     { We trace one up with tables conating table key links }
     if FTable.MasterLinkCount > 0 then
       CheckLinkList;
     Exit;
   end;

   if FTable.MasterSource = nil then {Standalone lnklist}
   begin
     CheckLinkList;
     Exit;
   end;

   if FFocRecord = nil then
    Exit;

   R := InUpList(FFocRecord);
   if R <> nil then
   begin
     if R.Ilib.Dataset.Position <> R.Index then
     begin
       R.Ilib.Dataset.Position := R.Index;
     end;
     CheckLinkList;
     Exit;
   end;
   InListindex := -1;
   if FTable.MasterSource.Dataset.Count > 0 then
   begin
     for I := 0 to FTable.MasterSource.Dataset.Count-1 do
     begin
       if (FTable.MasterSource.Dataset[i].RecordIndex = FFocRecord['SYS_LinkID'].AsInteger) then
         InListIndex := I;
     end;
   end;
   if InListIndex <> -1 then
   begin
     FTable.MasterSource.Dataset.Position := InListIndex;
     if FTable.MasterSource.HasViewer then
       FTable.MasterSource.Viewer.Invalidate;
     CheckLinkList;
     Exit;
   end;
   P := FTable.MasterSource;
   if not AcceptTarget(True, TILDataTable(P)) then
   begin
     CheckLinkList;
     Exit;
   end;
   R := P.RecordsX[FFocRecord['SYS_LinkID'].AsInteger];
   R.Ilib := P;
   P.FFocHandler.FUptracing := True;
   try
    P.Dataset.Clear(False);
    P.Dataset.Add(R); { Also creates the report }
    P.FFocHandler.FUptracing := True;
    P.Dataset.Position := 0;
   finally
     P.FFocHandler.FUptracing := False;
   end;
   if P.TVTable <> nil then
    begin
       { Make focused rec the root:}
       P.Dataset[0]['SYS_LinkId'].Clear;
       P.FocusedRecord.TVParentRecord := nil;
       if P.TVControl <> nil then
         P.TVControl.BuildTree(P.Dataset);
    end;

   CheckLinkList;

end;

procedure TFocusHandler.CreateDownList;
var
  i : integer;

procedure CheckLinkList;
var
  i, j, k : integer;
  R : TILDataRecord;
  T : TILDataTable;
  Indx : integer;
  Found : Boolean;
begin
     for I := 0 to FTable.DetailLinkCount-1 do
     begin
       if (FSender = FFocRecord.Ilib.DetailLinks[i].FFocHandler) then
         Continue;
        if not AcceptTarget(False, FTable.DetailLinks[i]) then
           Continue;

        Found := false;
        T := FTable.DetailLinks[i];
        for j := 0 to T.Dataset.Count-1 do
        { Move through det detail table records to find a match with the parent}
        begin
           R := T.Dataset[j];
           for k := 0 to R.Count-1 do { Check fields }
           begin
              if (R[k].FieldDef.LookupType = looTableKey)
              and (R[k].FieldDef.LookupTable = FTable) then
              begin
                Indx := FFocRecord.IndexOf(R[k].FieldDef.LookupField);
                if Indx <> -1 then
                begin
                   Found := SameText(FFocRecord[Indx].AsString, R[k].AsString);
                   if Found then
                   begin
                     T.FFocHandler.FDownTracing := True;
                     try
                       T.Dataset.Position := j;
                     finally
                      T.FFocHandler.FDownTracing := False;
                     end;
                     Break;
                   end;
                end;
              end;
           end;
           if Found then
             Break;
        end;

        if not Found then
        begin
            FTable.DetailLinks[i].FFocHandler.FDownTracing := True;
            try
              TILDataTable(FTable).FindLinkedRecords;
            finally
              FTable.DetailLinks[i].FFocHandler.FDownTracing := False;
            end;
        end
        else if FTable.Datahandler.FilterState <> fsStandard then
           TILDataTable(FTable).FindLinkedRecords;
     end;
end;

begin
  if FUpTracing then
  begin
     { We trace down with tables conating table key links }
     if (FTable.DetailLinkCount = 0) and (FTable.ChildCount = 0) then
      Exit;
  end;

  if not (mdLookupChildren in FTable.DataHandler.MDOptions) then
     Exit;

  if InDownList and (FTable.Datahandler.FilterState = fsStandard)  then
  { If filtering force lookups }
  begin
    CheckLinkList;
    Exit;
  end;

  for I := 0 to FTable.ChildCount-1 do
  begin
      if FTable.Children[i].MasterLinkCount > 0 then
        Continue;

      if (FSender = TILDataTable(FTable.Children[i]).FFocHandler) then
        Continue;

      if not AcceptTarget(False, TILDataTable(FTable.Children[i])) then
        Continue;

      FTable.Children[i].FFocHandler.FDownTracing := True;
      try
       FTable.Children[i].GetDetails;
       if FTable.Children[i].Dataset.Count > 0 then
        FTable.Children[i].Dataset.Position := 0;
      finally
       FTable.Children[i].FFocHandler.FDownTracing := False;
      end;
  end;
  CheckLinkList;

end;

procedure TFocusHandler.ClearAll;
begin
   ClearUpList;
   ClearDownList;
end;

procedure TFocusHandler.ClearDownList;
var
  i : integer;
begin
  for I := 0 to FTable.ChildCount -1 do
  begin
    if not AcceptTarget(False, TILDataTable(FTable.Children[i])) then
      Continue;
    FTable.Children[i].Dataset.Clear(False);
  end;
  for I := 0 to FTable.DetailLinkCount-1 do
  begin
    if not AcceptTarget(False, FTable.DetailLinks[i]) then
      Continue;
    FTable.DetailLinks[i].Dataset.Clear(False);
  end;
end;

procedure TFocusHandler.ClearUpList;
var
  i : integer;
  R : TILDataRecord;
  TBL : TILTable;
begin
  TBL := nil;
  for I := 0 to FTable.MasterSource.Dataset.Count-1 do
  begin
    R := FTable.MasterSource.Dataset[i];
    R.Ilib.Dataset[0].Free;
    R.Ilib.Dataset.FDataset.Delete(0);
    TBL := R.Ilib;
  end;
  if TBL <> nil then
   TBL.Dataset.Position := -1;
end;

function TFocusHandler.InUplist(ARecord : TILDatarecord) : TILDataRecord;
var
  i : integer;
  R : TILDataRecord;
  P : TILTable;
begin
   Result := nil;
   P := FTable.MasterSource;
   if P <> nil then
   for I := 0 to P.Dataset.Count-1 do
     begin
       R := P.Dataset[i];
       if R.RecordIndex = ARecord['SYS_LinkID'].AsInteger then
       begin
         Result := R;
         Break;
       end;
     end;
end;

function TFocusHandler.InDownList : Boolean;
var
  i, j : integer;
  T : TILDatatable;
begin
   Result := False;
   if FTable.ChildCount = 0 then
     Exit;
   for I := 0 to FTable.ChildCount-1 do
   if FTable.Children[i].DataHandler.FilterState <> fsStandard then
     Exit;
   for I := 0 to FTable.ChildCount-1 do
   begin
     T := FTable.Children[I] as TILDataTable;
     if T.Dataset.Count > 0 then
       if T.FocusedRecord['SYS_LinkId'].AsInteger = FFocRecord.RecordIndex  then
       begin
         Result := True;
         Break;
       end;
     if not Result then
     begin
       for j := 0 to T.Dataset.Count-1 do
       if T.Dataset[j]['SYS_LinkId'].AsInteger = FFocRecord.RecordIndex then
       begin
         T.Dataset.Position := J;
         Result:= True;
         Break;
       end;
       if Result then
        Break;
     end;
   end;
end;

procedure TFocusHandler.RunHandler;
var
  MH : TILMessageHandler;
  Ind : integer;
begin
  if FTable.HasHandler then
  begin
     Ind := -1;
     MH := TMan.GetMessageHandler(FTable, Ind);
     while MH <> nil do
     begin
       MH.DatasetChanged(dboNone);
       MH := TMan.GetMessageHandler(FTable, Ind);
     end;
   if Assigned(FTable.DataHandler.FOnFocusRecord) then
    PostMessage(FTable.FHwnd, WM_RunFocus, 0, 0);
  end;
end;

{ TTableManager ------------------------------------------------------------ }

constructor TTableManager.Create(AOwner: TComponent);
begin
  inherited;
  FHWnd := AllocateHWnd(WndMethod);
  FNodes := TList.Create;
  FWorkFields := TStringList.Create;
  FPanels := TList.Create;
  FWorkHistory := TList.Create;
  FWorkStyle := wsCommandModes;
  FAliasTrue := 'Yes';
  FAliasFalse := 'No';
  FAliasNull := 'Null';
  FTranslogBuffer := TMemoryStream.Create;
  FTransLogIndex := TStringList.Create;
  FTransLogIndex.Duplicates := dupIgnore;
  FTransLogIndex.OwnsObjects := True;
  FTransLogIndex.Sorted := True;
  FPlayerIcon := TPicture.Create;
  FAffectedTables := TList.Create;
  FIsServer := True;
  TMan := Self;
end;

function TTableManager.TruncateTranslog(ToDate : TDateTime) : integer;
var
    Log : TStringList;
    LogItm : TILTranslogItem;
    i : integer;
    DR : TILDataRecord;
    T : TILTable;
    Cnt, DelCnt : integer;
    S : string;
    NewLog : TStringList;

  begin
     Result := 0;
     if not Connected then
       HandleILException(E_NoConnection);
     if IsReadOnly then
       HandleILException(E_ReadOnly);
     {if Assigned(FTransLogFile) then
     begin
      FreeAndNil(FTranslogFile);
     end;}
     DelCnt := 0;
     Log := LoadTransLog(TranslogName);
     Cnt := Log.Count;
     try
      if Cnt = 0 then
       Exit;
      for i  := 0 to Cnt-1 do
      begin
        LogItm := TILTransLogItem(Log.Objects[i]);
        T := TableOf(LogItm.TableName);
        DR := T.XMLToRecord(LogItm.Trans, True);
        try
          if DateOf(DR.ChangedDate) <= DateOf(ToDate) then
          begin
             inc(DelCnt);
             Continue;
          end
          else
          begin
            DelCnt := i;
            Break;
          end;
        finally
          DR.Free;
        end;
      end;

      while Log.Count > Cnt - DelCnt do
       Log.Delete(0);

      FreeAndNil(FTranslogFile);
      NewLog := TStringList.Create;
      if Unicode then
       NewLog.DefaultEncoding := TEncoding.Utf8;
      try
       for I := 0 to Log.Count-1 do
       begin
        LogItm := TILTransLogItem(Log.Objects[i]);
        S := TMan.EncodeTrans(LogItm.TableName, LogItm.FRecordIndex,
         LogItm.FTransType, LogItm.OldData, LogItm.Trans, LogItm.TransID);
        System.Delete(S, Length(S)-1, 2);
        NewLog.Add(S);
       end;

      NewLog.SaveToFile(TranslogName);
      finally
        NewLog.Free;
      end;

      FTransLogFile := TFileStream.Create(FTranslogName, fmOpenReadWrite);
      Result := DelCnt;


     finally
       Log.Free;
     end;
  end;

destructor TTableManager.Destroy;
var
  i: integer;
begin
  inherited;
  Database := '';
  if Assigned(FThumbs) then
    FThumbs.Free;
  if Assigned(FTextLobs) then
    FTextLobs.Free;
  if Assigned(FLibContainer)  then
    FLibContainer.Free;
  for i := 0 to TableNodeCount - 1 do
    TTableNode(FNodes[i]).Free;
  FNodes.Free;

  for i := 0 to FWorkFields.Count-1 do
    TILWorkField(FWorkFields.Objects[i]).Free;
  FWorkFields.Free;
  FPanels.Free;
  for I := 0 to FWorkHistory.Count-1 do
    TWorkHistoryElement(FWorkHistory[i]).Free;
  FWorkHistory.Free;
  if Assigned(FCfgFile) then
  begin
    FCfgFile.Free;
  end;
  if Assigned(FTransLogFile) then
  begin
     FreeAndNil(FTranslogFile);
  end;
  Finalize(FTreeTables);
  FTransLogBuffer.Free;
  FTransLogIndex.Free;
  if Assigned(FCheckmarkBM) then
    FCheckmarkBM.Free;
  if Assigned(FIntPlayerIcon) then
    FIntPlayerIcon.Free;
  if Assigned(FIntPlayerBM) then
    FIntPlayerBM.Free;
  FPlayerIcon.Free;
  if Assigned(FTableBM) then
    FTableBM.Free;
  FAffectedTables.Free;
  RegTables.Clear;
  RegPanels.Clear;
  RegHandlers.Clear;
  DeallocateHWnd(FHWnd);
  TMan := nil;
end;

procedure TTableManager.WndMethod(var Msg : TMessage);
  var
  Handled: Boolean;
begin
  Handled := True;
  case Msg.Msg of
    WM_Loaded: DoLoaded;
    WM_UpdateWorklist : UpdateWorkList;
    else
      Handled := False;
  end;
  if Handled  then
    Msg.Result := 0
  else
    Msg.Result := DefWindowProc(FHWnd, Msg.Msg,
      Msg.WParam, Msg.LParam);
end;

procedure TTableManager.Notification(AComponent: TComponent; Operation: TOperation);
var
  Indx, PanIndx : integer;
  Tbl : TILTable;
begin
  inherited Notification(AComponent, Operation);
  if Tman = nil then
    Exit;
  if csDestroying in ComponentState then
    Exit;
  if Operation = opRemove then
  begin
    Indx := -1;
    if AComponent is TILTable then
    begin
      Indx := RegTables.IndexOf(AComponent);
    end;
    if AComponent is TILDataTable then
    begin
      TBL := TILTable(AComponent);
      if TBL = RootTable then
      begin
        Clear;
        FRootTable := nil;
      end
      else
      begin
        Delete(TILDataTable(AComponent));
      end;
      if Assigned(FTableList) then
        PopulateTableList(FTableList);
      CreateWorkFields(ActiveTable, CommandMode);
      PostMessage(FHWnd, WM_UpdateWorklist, 0, 0);

    end
    else if AComponent is TILControlPanel then
    begin
       PanIndx := RegPanels.IndexOf(AComponent);
       if PanIndx <> -1 then
       begin
         RegPanels.Delete(PanIndx);
         PanIndx := FPanels.IndexOf(AComponent);
         if PanIndx <> -1 then
           FPanels.Delete(PanIndx);
       end;
       CreateWorkFields(ActiveTable, CommandMode);
       PostMessage(FHWnd, WM_UpdateWorklist, 0, 0);

    end
    else if AComponent = FTableList then
      FTableList := nil
    else if AComponent = FWorkList then
      FWorkList := nil
    else if AComponent = FSharedImages then
      FSharedImages := nil;
    if Indx <> -1 then
    begin
      RegTables.Delete(Indx);
    end;
  end;
end;

function TTableManager.LoadResource(const ResName : string) : VCL.Graphics.TBitmap;
 var
   HM : HModule;
 begin
  Result := VCL.Graphics.TBitmap.Create;
  HM := FindClassHInstance(TTableManager);
  try
    Result.LoadFromResourceName(HM, Resname);
  except
    FreeAndNil(Result);
  end;
 end;

 function TTableManager.LoadResourceIcon(const ResName : string) : VCL.Graphics.TIcon;
 var
   HM : HModule;
 begin
  Result := VCL.Graphics.TIcon.Create;
  HM := FindClassHInstance(TTableManager);
  try
    Result.LoadFromResourceName(HM, Resname);
  except
    FreeAndNil(Result);
  end;
 end;

 procedure TTableManager.CheckDanglingHandlers;
 var
   Dels : TList;
   I, J : integer;
   D : TILData;
   DF : TILDataField;
   Indx : integer;
 begin
    Dels := TList.Create;
    D := nil;
    try
    for I := 0 to RegHandlers.Count-1 do
    begin
      D := TILData(RegHandlers[i]);
      for J := 0 to D.DataFields.Count-1 do
      begin
        DF := D.DataFields.Items[j];
        if D.Table <> nil then
          Indx := D.Table.Fields.IndexOf(DF.FieldName)
        else
          Indx := 0;
        if Indx = -1 then
        begin
          Dels.Add(DF);
          if D.SortKey.IndexOf(DF.FFieldName) <> -1 then
          begin
              D.SortKey.Clear;
              D.SortOrdering := soAscending;
          end;
          if (D.Table <> nil) and (D.Table.Columns <> nil) then
          begin
            D.Table.Columns.AssignSource;
          end;
        end;
      end;
    end;

    if (Dels.Count > 0) and (D <> nil) and D.SaveColumns then
    { Saved columns not likely to be valid when deleted or changed names}
    begin
        FCfgFile.EraseSection(D.Table.TableName + '_ColumnSequence');
        FCfgFile.EraseSection(D.Table.TableName + '_ColumnSort');
    end;

    for I := 0 to Dels.Count-1 do
    begin
      DF := TIlDataField(Dels[i]);
      if DF.DataControl <> nil then
        FreeAndNil(DF.FDataControl);
      if DF.DataLabel <> nil then
        FreeAndNil(DF.FDataLabel);
      if DF.SearchControl <> nil then
        FreeAndNil(DF.FSearchControl);
      if DF.SearchLabel <> nil then
        FreeAndNil(DF.FSearchLabel);
      Indx := DF.Index;
      DF.FDataFields.Delete(Indx);
    end;
    finally
      Dels.Free;
    end;
 end;

 procedure TTableManager.ClearData;
 var
   i : integer;
 begin
   for I := 0 to WorkFieldCount-1 do
     WorkFields[i].Free;
   FWorkFields.Clear;
   for I := 0 to FWorkHistory.Count-1 do
     TWorkHistoryElement(FWorkHistory[i]).Free;
   FWorkHistory.Clear;

   while RegHandlers.Count > 0 do
    TILData(RegHandlers[0]).Free;
   RegHandlers.Clear;
 end;

 procedure TTableManager.DoLoaded;
 var
    i, j : integer;
    M: TILDataTable;
    ErrTbl : TILTable;
    Indx : integer;
    Tbl : TILTable;
    D : TILData;
    DF : TILDataField;
    FD : TOpenDialog;
    S, SName : string;
    Dels : TList;

    procedure RefreshControls(AControl : TControl);
    var
        iControl: Integer;
    begin
        if AControl = nil then
            Exit;
        if AControl is TWinControl then
        begin
            for iControl := 0 to TWinControl(AControl).ControlCount - 1 do
                RefreshControls(TWinControl(AControl).Controls[iControl]);
        end;

        TWinControl(AControl).Invalidate;
    end;

    procedure SyncFields(Tbl : TILTable);
    var
      J : integer;
    begin
      if Tbl.DataHandler <> nil then
      { Update field defs from datafields }
      for J := 0 to Tbl.Fields.Count-1 do
      begin
          Indx := TBl.DataHandler.DataFields.IndexOf(Tbl.Fields.Items[j].FieldName);
          if Indx = -1 then
            Continue;
          Tbl.Fields.Items[j].FDataControl := Tbl.DataHandler.DataFields.Items[Indx].DataControl;
          Tbl.Fields.Items[j].FSearchControl := Tbl.DataHandler.DataFields.Items[Indx].SearchControl;
          Tbl.Fields.Items[j].FDataLabel := Tbl.DataHandler.DataFields.Items[Indx].DataLabel;
          Tbl.Fields.Items[j].FSearchLabel := Tbl.DataHandler.DataFields.Items[Indx].SearchLabel;
          Tbl.Fields.Items[j].FAlignment := Tbl.DataHandler.DataFields.Items[Indx].FAlignment;
          Tbl.Fields.Items[j].FImageText := Tbl.DataHandler.DataFields.Items[Indx].FUseAsImageText;
          Tbl.Fields.Items[j].FColumnWidth := Tbl.DataHandler.DataFields.Items[Indx].FColumnWidth;
          if (Tbl.DataHandler.DataFields.Items[Indx].Caption = '') then
            Tbl.DataHandler.DataFields.Items[Indx].FCaption := Tbl.Fields.Items[j].Caption;
      end;
    end;

  begin
    Application.ProcessMessages;
    if Database = '' then
    begin
      if Assigned(FOnLoaded) then
      FOnLoaded(Self);
      Exit;
    end;
    if not FileExists(FLibPath) then
    begin
       ShowMessage('Could not find database file ' + FLibPath + '.');
       FD := TOpenDialog.Create(Self);
       try
        FD.Filter := 'Database files (*.dat)|*.Dat';
        S := ExtractFileDir(FLibPath);
        if DirectoryExists(S) then
         FD.InitialDir := S;
        if FD.Execute then
        begin
          S := ExtractFileName(FD.FileName);
          i := Pos(S.ToUpper, '.DAT');
          if i = 0 then
            HandleILException(E_InvalidDBFileName);
          S := Copy(S, 1, i-1);
          Database := S; {This clears all existing
                                      tables and load the new ones }
        end
        else
        begin
          Database := ''; {This clears all existing tables }
           HandleILException(E_LoadDatabase);
        end;
       finally
         FD.Free;
       end;
       Exit;
    end;

    if not FileExists(FMetaname) then
    begin
      Comment1 := FMetaname;
      HandleILException(E_MissingMeta);
    end;

    try
      FindMissingTables;
    except on E:Exception do
      begin
       if ILException(E).ErrorCode = E_DBConstruct then
       begin
          MessageDlg('Warning! Database is under reconstruction. ' +
          'Please finish construction and reload your form before proceeding.', mtWarning, [mbOk], 0);
       end;
      end;
    end;

    { Ensure that all tables have a valid datahandler }
    for I := 0 to RegTables.Count-1 do
    begin
      if TILTable(RegTables[i]).DataHandler = nil then
      begin
        TILTable(RegTables[i]).FDataHandler := TILData.Create(Self);
        TILTable(RegTables[i]).FDataHandler.Name := '_Internal' + TILTable(RegTables[i]).Name;
        TILTable(RegTables[i]).FDataHandler.AutoCreateFields := false;
        TILTable(RegTables[i]).DataHandler.FHideInTableList := True;
        TILTable(RegTables[i]).FDataHandler.Table := TILTable(RegTables[i]);
      end;
    end;

    { Sync tables with metadata and data controls from dfm}
    Dels := TList.Create;
    try
    repeat
    for I := 0 to RegTables.Count-1 do
    begin
      if TILTable(RegTables[i]) is TILMemTable then
      begin
         SyncFields(TILTable(RegTables[i]));
         Continue;
      end;
      S := SyncTable(TILTable(RegTables[i]));
      if (S = '')  then
      begin
         Dels.Add(RegTables[i]);
         Continue;
      end
      else if not (TILTable(RegTables[i]) is TILView)
       and not SameText(TILTable(RegTables[i]).ClassName, S) then
      { Table has changed class }
      begin
       TBL := TILTable(RegTables[i]);
       D := Tbl.DataHandler;
       if D <> nil then
        begin
         D.FTreeViewControl := nil;
         D.FTreeViewLabelField := '';
         D.SortKey.Clear;
         D.SortOrdering := soAscending;
         for j  := 0 to D.DataFields.Count-1 do
         begin
           DF := D.DataFields.Items[j];
           if (DF.DataControl <> nil)
           and (DF.DataControl.Parent is TILControlPanel) then
             DF.DataControl.Free;
           if (DF.DataLabel <> nil)
           and (DF.DataLabel.Parent is TILControlPanel) then
             DF.DataLabel.Free;
           if (DF.SearchControl <> nil)
           and (DF.SearchControl.Parent is TILControlPanel) then
             DF.SearchControl.Free;
           if (DF.SearchLabel <> nil)
           and (DF.SearchLabel.Parent is TILControlPanel) then
             DF.SearchLabel.Free;
         end;
        end;
        SName := Tbl.TableName;
        FreeAndNil(TBL);
        if SameText(S, 'TILTreeViewTable') then
          TBL := TILTreeViewTable.Create(Owner)
        else if SameText(S, 'TILDataTable') then
          TBL := TILDataTable.Create(Owner)
        else if SameText(S, 'TILImageTable') then
          TBL := TILImageTable.Create(Owner)
        else
          TBL := TILLookupTable.Create(Owner);
        Tbl.FTableName := SName;
        Tbl.Name := 'Tbl_' + Tbl.TableName;
        Tbl.FDataHandler := D;
        Tbl.FDataHandler.Table := TBL;
        RegTables[i] := TBL;

        S := 'Rerun';
        Break;
      end;
      Tbl := TILTable(RegTables[i]);
      SyncFields(Tbl);
    end;
    until (S <> 'Rerun');

    for I := 0 to Dels.Count-1 do
    begin
      TILTable(Dels[i]).Free;
    end;

    finally
      Dels.Free;
    end;

    for I := 0 to RegTables.Count-1 do
    begin
      if TILTable(RegTables[i]) is TILMemTable then
        Continue;
      SyncRelations(TILTable(RegTables[i]));
      if Assigned(TILTable(RegTables[i]).Columns) then
        TILTable(RegTables[i]).Columns.AssignSource
    end;

    for I := 0 to RegTables.Count-1 do
    begin
      if TILTable(RegTables[i]) is TILMemTable then
        Continue;
      SyncDetailLinks(TILTable(RegTables[i]));
    end;

    CreateNodes;

    for I := 0 to RegTables.Count-1 do
    begin
      if TreeTableOf(TILTable(RegTables[i]).TableName) = nil then
        RegisterFreeTable(TILTable(RegTables[i]));
    end;

    { Check lookup table consistensy }
    for i := 0 to TableNodeCount - 1 do
      begin
        M := TreeTables[i];
        for j := 0 to M.Fields.Count - 1 do
        begin
          if M.Fields.Items[j].LookupType in [looTableRef, looTableKey] then
          begin
            Comment1 := M.TableName + '.' + M.Fields.Items[j].Caption;
            if (M.Fields.Items[j].LookupTable = nil) or (M.Fields.Items[j].LookupField = '') then
            begin
              if csDesigning in ComponentState then
               ShowMessage(GetILErrorText(E_looTableNotDefined1))
              else
               HandleILException(E_looTableNotDefined2);
            end
            else if M.Fields.Items[j].LookupType = looTableKey then
            if M.Fields.Items[j].LookupSearchField = '' then
            begin
              if csDesigning in ComponentState then
               ShowMessage(GetILErrorText(E_looTableNotDefined1))
              else
               HandleILException(E_looTableNotDefined2);
            end;
            Comment1 := '';
          end;
        end;
    end;

    { Check if there are data handlers with dangling field refs }
    CheckDanglingHandlers;

    { Connect data handlers for lookup fields, init checklists and alignment / columnwidth overrules}
    for I := 0 to RegHandlers.Count-1 do
    begin
      D := TILData(RegHandlers[i]);
      if D.Table <> nil then
      begin
        if D.Table.Columns = nil then
          D.Table.FColumns := TILColumns.Create(D.Table, D.Table.Fields);
        if D.DataFields.Count > 0 then
         D.Table.Columns.Clear;

      end;
      for J := 0 to D.DataFields.Count-1 do
      begin
        DF := D.DataFields.Items[j];
        if DF.SearchControl is TChecklistBox then
        with DF.SearchControl as TChecklistBox do
          CheckAll(cbGrayed, True);
      end;
      D.InitLookups;
    end;

    { Check if ImageTableRef is Valid }
    if SharedImages <> nil then
    begin
      if TableOf(SharedImages.TableName) = nil then
        FSharedImages := nil;
    end;


    ErrTbl := CheckStates;
    if (ErrTbl <> nil) and not ExtDesigner then
    begin
      MessageDlg('Table ' + ErrTbl.TableName +
       ' is in an erroneous state. Please correct the error in your administration tool.', mtError, [mbOk], 0);
    end;

    { Init SQL handlers }

    for I := 0 to RegSQLs.Count-1 do
    begin
      TILSQL(RegSQLs[i]).FNoCheck := True;
      TILSQL(RegSQLs[i]).InitParams;
      TILSQL(RegSQLs[i]).FNoCheck := False;
    end;

    if Assigned(FTableList) then
    begin
      PopulateTableList(FTableList);
    end;

    for I := 0 to RegPanels.Count-1 do
    begin
      TILControlPanel(RegPanels[i]).CreateItems(True);
      if TILControlPanel(RegPanels[i]).ShowLogOpButtons then
      begin
        TILControlPanel(RegPanels[i]).CreateLogOpButtons;
        TILControlPanel(RegPanels[i]).ShowCtrls(TILControlPanel(RegPanels[i]).FItems);
      end;
    end;

    for I := 0 to RegPanels.Count-1 do
      FPanels.Add(RegPanels[i]);
    InitWorkFields;
    UpdateWorkList;
    if not (csDesigning in ComponentState) then
      EnablePanels;

    if FileExists(FMetaName) then
     FMetaFileStamp := GetFileLastModDate(FMetaName);

    RefreshControls(Application.MainForm);

    for I := 0 to TableCount-1 do
    begin
      if Tables[i].Columns.Count = 0 then
        Tables[i].Columns.AssignSource;
      if Tables[i].HasViewer then
      begin
        Tables[i].Viewer.UpdateScrollRange;
        Tables[i].Viewer.UpdateScrollPos;
      end;
    end;

    if not (csDesigning in ComponentState) then
    for I := 0 to RegMbs.Count-1 do
    begin
      TILMessageBar(RegMbs[i]).MessageHandler.OnChangeMessage := TILMessageBar(RegMbs[i]).MsgEvent;
      TILMessageBar(RegMbs[i]).SetVisibles;
    end;

    FIsLoaded := True;

    if Assigned(FOnLoaded) then
      FOnLoaded(Self);
  end;

function TTableManager.GetTreeTableCount: integer;
begin
  Result := FNodes.Count;
end;


function TTableManager.GetTableNodeCount: integer;
begin
  Result := FNodes.Count;
end;

function TTableManager.GetRoot: TTableNode;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to High(FTreeTables) do
    if FTreeTables[i].MasterSource = nil then
    begin
      Result := NodeOf(FTreeTables[i]);
      Break;
    end;
end;

function TTableManager.GetItems(Index: integer): TTableNode;
begin
  Result := FNodes[Index];
end;

function TTableManager.GetActiveTable: TILTable;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to High(FTreeTables) do
    if FTreeTables[i].FActive then
    begin
      Result := FTreeTables[i];
      Break;
    end;

  if Result = nil then
  for i := 0 to FreeTableCount-1 do
    if FreeTables[i].FActive then
    begin
      Result := FreeTables[i];
      Break;
    end;

  if (Result = nil) and (Length(FTreeTables) > 0) then
  begin
    Result := FTreeTables[0];
  end;
end;

function TTableManager.GetActiveRecord : TILDataRecord;
begin
   if Assigned(ActiveTable) then
     Result := ActiveTable.FocusedRecord
   else
     Result := nil;
end;


procedure TTableManager.SetActiveTable(const Value : TILTable);
var
  i: integer;
  Cancel : Boolean;
  MH : TILMessageHandler;
begin

  if Value = ActiveTable then
    Exit;
  if RootTable = nil then
    Exit;

  if (Value = nil) then
    Exit;

  if ActiveTable.HasViewer then
  begin
   if ActiveTable.Viewer.Report.SelectedCell <> nil then
   begin
      ActiveTable.Viewer.Report.SelectedCell.CancelEdit;
   end
   else if ActiveTable.Dataset.Position <> -1 then
      if ActiveTable.Dataset.Items[ActiveTable.Dataset.Position].FEditingImageText then
       ActiveTable.Dataset.Items[ActiveTable.Dataset.Position].CancelEditImageText;
  end;

 { if not Assigned(FOnUnsaved) and not AutoSave then
  begin
    ActiveTable.Dataset.RevertChanges(False);
  end
  else if not TMan.CheckUnsaved(ActiveTable) then
  begin
    if Assigned(TableList) then
      TableList.ItemIndex := TableList.Items.IndexOf(ActiveTable.Alias);
    Exit;
  end
  else if ActiveTable.Dataset.Changed then
    ActiveTable.Dataset.RevertChanges(False);}


  Cancel := false;
  if Assigned(FOnActivation) then
  begin
    FOnActivation(Self, Value, Cancel);
     if Cancel then
    begin
     if Assigned(TableList) then
      TableList.ItemIndex := TableList.Items.IndexOf(ActiveTable.Alias);
     Exit;
    end;
  end;


  SaveToHistory(ActiveTable, CommandMode, FWorkFields);

 if ActiveTable.MultiSelected then
    ActiveTable.DeselectAll;

  for i := 0 to TableCount - 1 do
  begin
    Tables[i].FActive := false;
  end;

  Value.FActive := True;

  if Value.HasHandler then
  begin
    Value.DataHandler.CreateSummary(nil);
    if Assigned(Value.DataHandler.FImageControl)
    and not Value.HasViewer
    then
      Value.DataHandler.FImageControl.LoadFromRecord(Value.FocusedRecord);
  end;
  if Assigned(Value.Viewer) then
  begin
   if Assigned(Value.Viewer.FZoomImage) and Value.Viewer.FZoomWindow.Visible then
     Value.Viewer.FZoomImage.LoadFromRecord(Value.FocusedRecord)
   else if Assigned(Value.Viewer.FZoomImage) then
   begin
     Value.Viewer.FZoomImage.ClearSource;
     Value.Viewer.FZoomImage.FSourceRecord := Value.FocusedRecord;
   end;
  end;

  if (Value is TILTreeViewTable)
  and (TILTreeViewTable(Value).TreeViewControl <> nil)
  then
   TILTreeViewTable(Value).TreeViewControl.BuildTree(Value.Dataset);

  if Assigned(FTableList) then
  begin
    if Assigned(Value.DataHandler) and Value.Datahandler.HideInTableList then
     FTableList.ItemIndex := -1
    else
    for i := 0 to FTableList.Items.Count - 1 do
    begin
      if SameText(FTableList.Items[i], Value.Alias) then
      begin
        FTableList.ItemIndex := i;
        Break;
      end;
    end;
  end;

  if UseCommandModes then
  begin
    CreateWorkFields(ActiveTable, CommandMode);
    UpdateWorkList;
    if (ActiveRecord <> nil) then
    begin
      if ((CommandMode = cmUpdate) and not ActiveTable.MultiSelected or (ActiveTable.SelCount = 0))
      or (CommandMode = cmSearch)
      then
        ActiveTable.DBWork.WriteControls(ActiveRecord);
    end;
    if not (CommandMode in [cmSort, cmColumns, cmSQL]) then
    begin
      DoShowWorkFields(FindPanel(ActiveTable, CommandMode));
      if CommandMode = cmSearch then
       WriteBrowser(FindPanel(ActiveTable, cmUpdate));
    end;
  end
  else
  begin
   if (Workstyle = wsCommandModes) then {Disabled}
   begin
     for I := 0 to FWorkfields.Count-1 do
      FworkFields.Objects[i].Free;
     FWorkFields.Clear;
     if Assigned(FWorkList) then
       FWorkList.Clear;
   end
   else
   begin
      for I := 0 to RegPanels.Count-1 do
      begin
        TILControlPanel(RegPanels[i]).ShowCtrls(TILControlPanel(RegPanels[i]).FItems);
      end;
   end;
  end;

  if not (csDesigning in ComponentState) then
    EnablePanels;
  for I := 0 to TableCount-1 do
    if Tables[i].HasViewer then
      Tables[i].Inval;

  for I := 0 to RegMbh.Count-1 do
  begin
     MH  := TILMessageHandler(RegMbh[i]);
     MH.Reset;
  end;

  ActiveTable.Dataset.RunFocus(ActiveTable.Dataset.Position);
  if Assigned(FOnActivateTable) then
    FOnActivateTable(Value);
end;

procedure TTableManager.SetCommandMode(const Value : TCommandMode);
var
  SampleText : string;
begin
  if FCommandMode = Value then
    Exit;
  if not UseCommandModes then
  begin
    FCommandMode := value;
    Exit;
  end;
  if (Value in [cmSort, cmColumns]) and ActiveTable.Fields.HasTempFields then
  begin
    HandleILException(E_TempColumns);
    Exit;
  end;
  SaveToHistory(ActiveTable, CommandMode, FWorkFields);

  if ActiveTable.HasViewer then
  begin
   if ActiveTable.Viewer.Report.SelectedCell <> nil then
   begin
      ActiveTable.Viewer.Report.SelectedCell.CancelEdit;
   end
   else if ActiveTable.Dataset.Position <> -1 then
      if ActiveTable.Dataset.Items[ActiveTable.Dataset.Position].FEditingImageText then
       ActiveTable.Dataset.Items[ActiveTable.Dataset.Position].CancelEditImageText;
  end;

  FCommandMode := Value;

  CreateWorkFields(ActiveTable, CommandMode);
  UpdateWorkList;
  if not (CommandMode in [cmSort, cmColumns, cmSQL]) then
  begin
    DoShowWorkFields(FindPanel(ActiveTable, CommandMode));
    if CommandMode = cmSearch then
      WriteBrowser(FindPanel(ActiveTable, cmUpdate));
  end;
  if not (csDesigning in ComponentState) then
  begin
   if (ActiveTable <> nil) and ActiveTable.HasHandler
   and (ActiveTable.DataHandler.ImageControl <> nil) then
   begin
     if TMan.ActiveRecord <> nil then
      ActiveTable.DataHandler.ImageControl.SourceRecord := TMan.ActiveRecord;
  end;

  if ActiveRecord <> nil then
  begin
      if ((CommandMode = cmUpdate) and not ActiveTable.MultiSelected or (ActiveTable.SelCount = 0))
      or (CommandMode = cmSearch) then
        ActiveTable.DBWork.WriteControls(ActiveRecord)
  end;
  EnablePanels;
  if ActiveTable.HasHandler and Assigned(ActiveTable.DataHandler.FOnUpdateLongText)
  then
  begin
     SampleText := '';
     if not (FCommandMode = cmUpdate) then
       ActiveTable.DataHandler.FOnUpdateLongText(ActiveTable.DataHandler, nil, SampleText, [ltaDisable])
     else
       ActiveTable.DataHandler.FOnUpdateLongText(ActiveTable.DataHandler, nil, SampleText, [ltaEnable])
  end;
end;

if Assigned(FOnChangeCommandMode) then
  FOnChangeCommandMode(Self);
 if ActiveTable.HasViewer then
  ActiveTable.Viewer.Invalidate;
end;

procedure TTableManager.SetIntPlayerIcon(AnIcon : TPicture);
var
  BM : VCL.Graphics.TBitmap;
begin
  if (AnIcon.Graphic is TIcon) then
  begin
    if Assigned(FIntPlayerBM) then
      FreeAndNil(FIntPlayerBM);
    if not Assigned(FIntPlayerIcon) then
      FIntPlayerIcon := TIcon.Create;
    FIntPlayerIcon.Assign(AnIcon);
  end
  else if Assigned(AnIcon.Graphic) then
  begin
    if Assigned(FIntPlayerIcon) then
      FreeAndNil(FIntPlayerIcon);
    if Assigned(FIntPlayerBM) then
      FreeAndNil(FIntPlayerBM);

    BM := VCL.Graphics.TBitmap.Create;
    try
    if AnIcon.Graphic is VCL.Graphics.TBitmap then
    begin
     BM.Assign(AnIcon);
    end
    else
    begin
      BM.Assign(AnIcon.Bitmap);
    end;
    if BM.PixelFormat = pf32Bit then
    begin
       BM.AlphaFormat := afDefined;
       FIntPlayerBM := VCL.Graphics.TBitmap.Create;
       FIntPlayerBM.Assign(BM);
    end
    else
     FIntPlayerIcon := BitmapToIcon(BM);
    finally
      BM.Free;
    end;
  end
  else
  begin
   if Assigned(FIntPlayerIcon) then
      FreeAndNil(FIntPlayerIcon);
    if Assigned(FIntPlayerBM) then
      FreeAndNil(FIntPlayerBM);
  end;
end;

procedure TTableManager.SetPlayerIcon(Value : TPicture);
begin
   if Value = FPlayerIcon then
   begin
     Exit;
   end;
  FPlayerIcon.Assign(Value);
  SetIntPlayerIcon(Value);
end;

procedure TTableManager.SetSearchModel(const Value : TSearchModel);
var
  M: TILDataTable;
  i, j: integer;
  P : TILControlPanel;

begin
  if Value = FSearchModel then
    Exit;
  FSearchModel := Value;
  M := RootTable;
  if M = nil then
    Exit;
  ActiveTable.SetLogOp(flAnd);
  for I := 0 to ActiveTable.Fields.Count-1 do
  begin
     ActiveTable.SetLogOp(ActiveTable.Fields.Items[i].FieldName, flAnd);
     ActiveTable.SetLogOp(ActiveTable.Fields.Items[i].FieldName, vlAnd);
  end;

  if not (csLoading in ComponentState) and UseCommandModes then
  begin
    for I := 0 to PanelCount-1 do
    begin
      if Panels[i].ShowLogOpButtons then
      begin
        Panels[i].RemoveLogopButtons;
        Panels[i].CreateLogopButtons;
      end;
    end;
    while FWorkHistory.Count > 0 do
    { Clear history so the new model can be recreated }
    begin
        TWorkHistoryElement(FWorkHistory[0]).Free;
        FWorkHistory.Delete(0);
    end;
    P := FindPanel(ActiveTable, cmSearch);
    if P <> nil then
     if (CommandMode = cmSearch) then
     begin
      CreateWorkFields(ActiveTable, cmSearch);
      UpdateWorkList;
      P.ShowCtrls(FWorkFields);
     end;
  end;

  if Value = smAndOrMixed then
    CreateSearchFieldOrder;

  { Notify search grids }
  for i := 0 to High(TreeTables) do
    for j := 0 to TreeTables[i].Fields.Count - 1 do
    begin
      if TreeTables[i].Fields.Items[j].SearchControl is TILSearchGrid then
        SendMessage(TreeTables[i].Fields.Items[j].SearchControl.Handle,
          WM_AlignLogOps, 0, 0);
    end;
end;

procedure TTableManager.SetAliasTrue(const Value : string);
begin
   if Trim(Value) = '' then
     Exit;
   FAliasTrue := Value;
end;

procedure TTableManager.SetAliasNull(const Value : string);
begin
   if Trim(Value) = '' then
     Exit;
   FAliasNull := Value;
end;

procedure TTableManager.SetAliasFalse(const Value : string);
begin
   if Trim(Value) = '' then
     Exit;
   FAliasFalse := Value;
end;

procedure TTableManager.SetUserID(const Value : string);
begin
  if Value.Length > 6 then
    FUserID := Copy(Value, 1, 6)
  else
    FUserID := Value;
end;

procedure TTableManager.TreeViewClick(Sender: TObject);
begin
  if FTableList.ItemIndex <> -1 then
    ActiveTable := TableOf(FTableList.Items[FTableList.ItemIndex]);
end;

function TTableManager.CommandFromSQL(SQL: TStrings): TSQLType;
begin
    RemoveComments(SQL);
    if SQL.Count = 0 then
      Result := ctUnknown
    else if PosRestrict('SELECT', SQL.Text, False) <> 0 then
      Result := ctSelect
    else if PosRestrict('INSERT', SQL.Text, False) <> 0 then
      Result := ctInsert
    else if PosRestrict('UPDATE', SQL.Text, False) <> 0 then
      Result := ctUpdate
    else if PosRestrict('DELETE', SQL.Text, False) <> 0 then
      Result := ctDelete
    else
      Result := ctUnknown;
end;

function TTableManager.ExecSQL(SQL : TStrings; ActivateTable : Boolean = false) : integer;
var
  s, s2 : string;
  P, P2 : integer;
  TBL : TILTable;
  sl : TStringList;
  fldlist : string;

  function GetTable(SQ : string) : TILTable;
  var
    s2 : string;
    i : integer;
    P : integer;
  begin
      Result := nil;
      P := Length(SQ) + 2;
      s2 := '';
      for i := P to Length(S) do
      begin
        if (S[i] = #$D) or (S[i] = ' ') then
          break
        else
          S2 := S2 + S[i];
      end;
      if S2 <> '' then
      begin
       Result := TableOf(S2);
       if Result <> nil then
       begin
         System.Delete(S, P, Length(S2));
         sl.Text := S;
       end
       else
       begin
         Comment1 := s2;
         HandleILException(E_InvalidTableName);
       end;
      end;
  end;

begin
  Result := 0;
  Tbl := nil;
  sl := TStringList.Create;
  try
    sl.Assign(SQL);
    RemoveComments(sl);
    S := Sl.Text;
    S := TrimBlanks(S, false);
    if S = '' then
      Exit;
    P := PosRestrict('INSERT TABLE', S, False);
    if P = 1 then
    begin
       S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
       S := TrimBlanks(S, false);
       Tbl := GetTable('INSERT TABLE');
       if Tbl <> nil then
         Result := Tbl.DBWork.ExecSQL(sl)
       else
        Result := ActiveTable.DBWork.ExecSQL(sl);
    end
    else
    begin
      P := PosRestrict('UPDATE TABLE', S, False);
      if P = 1 then
      begin
        S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
        S := TrimBlanks(S, false);
        Tbl := GetTable('UPDATE TABLE');
        if Tbl <> nil then
         Result := Tbl.DBWork.ExecSQL(sl)
       else
        Result := ActiveTable.DBWork.ExecSQL(sl);
      end
      else
      begin
        P := PosRestrict('SELECT', S, False);
        if P = 1 then
        begin
          P2 := PosRestrict('FROM', S, False);
          if P2 <> 0 then
          begin
            P := PosRestrict('WHERE', S, False);
            { Straighten lines before where }
            if P <> 0 then
            begin
              s2 := Copy(S, 1, P-1);
              s2 := StringReplace(S2, #$D#$A, ' ', [rfReplaceAll]);
              System.Delete(S, 1, P-1);
              S := s2 + s;
            end
            else
              S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
            S := TrimBlanks(S);
            P2 := PosRestrict('FROM', S, False);
            P := Length('SELECT') + 2;
            fldlist := Copy(S, P, P2-P-1);
            System.Delete(S, P, P2 + Length('FROM')-P);
            S := TrimBlanks(S);
            Tbl := GetTable('SELECT');
            if Tbl <> nil then
            begin
             S := sl.Text;
             Insert(fldlist, s, P);
             sl.Text := S;
             Result := Tbl.DBView.ExecSQL(sl);
            end
            else
             Result := ActiveTable.DBView.ExecSQL(sl);
          end
          else
            Result := ActiveTable.DBView.ExecSQL(sl);
        end
        else
        begin
          P := PosRestrict('DELETE TABLE', S, False);
          if P = 1 then
          begin
            S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
            S := TrimBlanks(S, false);
            Tbl := GetTable('DELETE TABLE');
            if Tbl <> nil then
              Result := Tbl.DBWork.ExecSQL(sl)
            else
              Result := ActiveTable.DBWork.ExecSQL(sl);
          end;
        end;
      end;
    end;
    if Tbl = nil then
      Result := ActiveTable.ExecSQL(SQL)
    else if (Tbl <> nil) and ActivateTable then
      ActiveTable := Tbl;
  finally
    sl.Free;
  end;
end;

function TTableManager.CanExecCommand(const ACommand : TCommandMode; const AnOption : TBaseCommandOption) : integer;
var
 s : string;
 sl : TStringList;

 function SortAlike : Boolean;
 var
   i : integer;
   Cnt : integer;

 begin

      Result := True;
      begin
        if (AnOption = ovSortDesc) and
         (ActiveTable.Columns.SortOrdering = soAscending) then
        begin
          Result := False;
          Exit;
        end
        else if ((AnOption = ovSortAsc) or (AnOption = ovDefault)) and
         (ActiveTable.Columns.SortOrdering = soDescending) then
        begin
          Result := False;
          Exit;
        end;
      end;
      Cnt := -1;
      for i := 0 to WorkFieldCount-1 do
      begin
        if not WorkFields[i].IsVisible then
          Continue;
        inc(Cnt);
        if ActiveTable.Columns.SortFields.IndexOf
        (WorkFields[i].Field.FieldName) <> Cnt then
        begin
          Result := False;
          Break;
        end;
      end;
      if Cnt+1 <> ActiveTable.Columns.SortFields.Count then
        Result := false;
 end;

 function ColumnsAlike : Boolean;
 var
   i : integer;
   Cnt : integer;

 begin
      Result := True;
      Cnt := -1;
      for i := 0 to WorkFieldCount-1 do
      begin
        if not WorkFields[i].IsVisible then
          Continue;
        inc(Cnt);
        if ActiveTable.Columns.IndexOf
        (WorkFields[i].Field.FieldName) <> Cnt then
        begin
          Result := False;
          Break;
        end;
      end;
      if Cnt+1 <> ActiveTable.Columns.Count then
        Result := false;
 end;

 function ChecRecordState : Integer;
 var
   R1, R2 : TILDatarecord;
   i : integer;
 begin
   R1 := nil;
   if ActiveRecord <> nil then
     R1 := ActiveRecord;
   R2 := ActiveTable.DBWork.CreateDataRecord;
   Result := ActiveTable.DBWork.ReadControls(R2);
   if Result <> 0 then
     Exit;
   try
    if ActiveTable.DBCommons.IsRecordEmpty(R2) then
    begin
      for I := 0 to ActiveTable.Fields.Count-1 do
       if ActiveTable.Fields.Items[i].FieldType = ftAutoIncrement then
         Exit;
      Result := -1;
    end
    else if R1 <> nil then
    begin
      if ActiveTable.DBCommons.CompareRecords(R1, R2, True) then
        Result := -1;
    end;
   finally
     R2.Free;
   end;
 end;

begin
     if not OptionInCommand(ACommand, AnOption) then
     begin
        Result := E_CMDOptionNotFound;
        Exit;
     end;

     Result := 0;
     if not Connected then
      Result := E_NoConnection;
     if Result <> 0 then
      Exit;

     try

     case ACommand of
     cmSearch:
     begin
         if (SearchModel = smAndOrMixed) and (AnOption <> ovDefault) then
         begin
           Result := E_OptDefaultNeeded;
           Exit;
         end
         else if (SearchModel = smAndAligned) and (AnOption = ovSearchOr) then
         begin
           Result := E_OptSearchAndNeeded;
           Exit;
         end;
         Result := ActiveTable.DBView.CanSearch;
     end;
     cmUpdate:
     begin
       if AnOption = ovUpdateImage then
       begin
         Result := ActiveTable.DBWork.CanUpdateImage;
       end
       else
       begin
         Result := Activetable.DBWork.CanUpdateData;
       end;
     end;
     cmColumns:
     begin
       if (ActiveTable.Viewer = nil) then
         Result := -1
       else if not ColumnsAlike then
         Result := 0
       else
         Result := -1;
     end;
     cmSort: if not SortAlike then
       Result := 0
     else
       Result := -1;
     cmSQL :
     begin
       S := '';
       if Assigned(FOnGetSQLText) then
       begin
          FOnGetSQLText(Self, S);
       end
       else
       begin
          Result := -1;
          Exit;
       end;
       sl := TStringList.Create;
       try
        sl.Text := S;
        Result := ActiveTable.DBCommons.CanExecSQL(Sl);
       finally
         sl.Free;
       end;
     end;
   end;

   except on E:Exception do
   begin
    if E is ILException then
    with E as ILException do
    begin
     Result := ErrorCode;
    end
    else
      raise;
    end;
   end;
end;

function TTableManager.ExecCommand(const AnOption : TBaseCommandOption) : integer;
var
  i : integer;
  S : string;
  SL : TStrings;
  Opt : TBaseCommandOption;

  begin
   Result := -1;
   if not UseCommandModes then
     Exit;
   Opt := AnOption;
   if CommandMode = cmSearch then
     CreateSearchFieldOrder;
   I := CanExecCommand(CommandMode, Opt);
   if I <> 0 then
     HandleILException(I, '');

   S := '';
   Result := 0;

   case CommandMode of
     cmSearch:
       begin
          if SearchModel = smAndOrAligned then
          begin
           if Opt = ovSearchAnd then
            ActiveTable.SetLogOp(flAnd)
           else if Opt = ovSearchOr then
            ActiveTable.SetLogOp(flOr);
          end
          else if Opt = ovSearchAnd then
            ActiveTable.SetLogOp(flAnd);
          CreateSearchFieldOrder;
          Result := ActiveTable.DBView.Search;
       end;
     cmUpdate:
       begin
          if Opt = ovUpdateImage then
          begin
           Result := ACtiveTable.DBWork.UpdateImageFromSource(false);
          end
          else if Opt in [ovDefault, ovUpdateData] then
          begin
            Result := ActiveTable.DBWork.Update(nil);
          end;
       end;
     cmSort:
       begin
          if Opt = ovDefault then
            Opt := ovSortAsc;
          if Opt = ovSortDesc then
            ActiveTable.Columns.SortOrdering := soDescending
          else if Opt = ovSortAsc then
            ActiveTable.Columns.SortOrdering := soAscending;
          ActiveTable.Columns.SortFields.Clear;
          for i := 0 to WorkFieldCount-1 do
          begin
              if WorkFields[i].IsVisible then
              begin
               ActiveTable.Columns.SortFields.Add(WorkFields[i].Field.FFieldName);
              end;
          end;
          ActiveTable.Columns.Apply;
       end;
     cmColumns:
       begin
         ActiveTable.Columns.Clear;
         for I := 0 to WorkFieldCount-1 do
         begin
           if WorkFields[i].IsVisible then
           begin
             ActiveTable.Columns.Add(WorkFields[i].Field.FFieldName);
           end;
         end;
         ActiveTable.Columns.Apply;
       end;
     cmSQL :
       begin
         sl := TStringList.Create;
         try
           if Assigned(FOnGetSQLText) then
           begin
            S := '';
            FOnGetSQLText(Self, S);
            sl.Text := S;
           end
           else
             Exit;
           Result := ActiveTable.DBCommons.ExecSQL(sl);
         finally
           sl.Free;
         end;
       end;

   end;
end;

function TTableManager.CheckScript(Statements : TStrings; var ErrorIndex : integer) : integer;
var
  Statement : TStringList;
  S : string;
  i : integer;
  Cnt : integer;
  TBL : TILTable;
  P : integer;
  SQText : string;
begin
    Result := 0;
    ErrorIndex := 0;
    Cnt := 0;
    Statement := TStringList.Create;
    try
      for I := 0 to Statements.Count-1 do
      begin
        if Trim(Statements[i]) = '' then
          Continue;
        Statement.Add(Statements[i]);
        s := Trim(Statements[i]);
        if S[Length(S)] = ';' then
        begin
          Inc(Cnt);
          System.Delete(S, Length(s), 1);
          S := Trim(S);
          Statement[Statement.Count-1] := S;
          if not (CommandFromSQL(Statement) in [ctUnKnown, ctSelect]) then
          begin
            Tbl := TableFromSQL(Statement);
            if TBL = nil then
            begin
              Result := E_InvalidTableName;
              ErrorIndex := Cnt;
              Exit;
            end;
            SQText := Statement.Text;
            P := PosRestrict(Tbl.TableName, SQText, false);
            System.Delete(SQText, P, Length(tbl.TableName));
            Statement.Text := SQText;
            Result := TBL.DBCommons.CanExecSQL(Statement);
            if Result <> 0 then
            begin
              ErrorIndex := Cnt;
              Exit;
            end;
          end
          else
          begin
            Result := -1;
            ErrorIndex := Cnt;
            Exit;
          end;

          Statement.Clear;
        end;
      end;
    finally
      Statement.Free;
    end;
end;

function TTableManager.CheckScript(const AFileName : string; var ErrorIndex : integer) : integer;
var
  sl : TStringList;
begin
    ErrorIndex := -1;
    sl := TStringList.Create;
    try
      sl.LoadFromFile(AFilename);
      Result := CheckScript(sl, ErrorIndex);
    finally
      sl.Free;
    end;
end;

function TTableManager.ExecScript(const AFileName : string; var ErrIndex : integer) : integer;
var
  sl : TStringList;
begin
    sl := TStringList.Create;
    try
      sl.LoadFromFile(AFilename);
      Result := ExecScript(sl, ErrIndex);
    finally
      sl.Free;
    end;
end;

function TTableManager.ExecScript(Statements : TStrings; var ErrIndex : integer) : integer;
var
  Statement : TStringList;
  S : string;
  i : integer;
  Res : integer;
  Cnt : integer;
  Scrpt : Boolean;
  N : integer;
begin
    Result := CheckScript(Statements, ErrIndex);
    if Result <> 0 then
    begin
      Exit;
    end;
    Statement := TStringList.Create;
    Scrpt := not FTransactionActive;
    if not FTransactionActive then
     BeginTrans;
    try
    Cnt := Statements.Count;
    N := 0;
    try
      for I := 0 to Statements.Count-1 do
      begin
        if Trim(Statements[i]) = '' then
          Continue;
        Statement.Add(Statements[i]);
        s := Trim(Statements[i]);
        if S[Length(S)] = ';' then
        begin
          inc(N);
          System.Delete(S, Length(s), 1);
          S := Trim(S);
          Statement[Statement.Count-1] := S;
          try
            Res := ExecSQL(Statement);
          except on E:Exception do
           begin
              E.Message := E.Message + ' (Statement number ' + IntToStr(N) + ')';
              raise;
           end;
          end;
          Result := Result + Res;
          Statement.Clear;
        end;
      end;
    finally
      Statement.Free;
      for I := 0 to TableCount-1 do
         Tables[i].FUpdateTarget := utrDataset;
    end;
    if Scrpt then
    begin
      if Cnt = 0 then
       RollbackTrans
      else
       CommitTrans;
    end

    except
      if Scrpt then
      begin
        RollbackTrans;
        raise;
      end;
    end;
end;


procedure TILControlPanel.CheckCPScroller;
var
 W,  H : integer;
 Box : TScrollBox;
begin
  if Parent is TScrollBox then
  begin
     W := AbsoluteWidth;
     H := AbsoluteHeight;

     Box := TScrollBox(Parent);
     if (W > Box.ClientWidth) or (H > Box.ClientHeight) then
     begin
        Align := alNone;
        if (W > Box.ClientWidth) then
          Width := W + 10
        else
          Width := Box.ClientWidth;
        if (H > Box.ClientHeight) then
          Height := H + 10
        else
          Height := Box.ClientHeight;
     end
     else
       Align := alClient;
  end;

end;

procedure TTableManager.ChangeWorkFields;
begin
  if Assigned(FOnChangeWorkfields) then
     FOnChangeWorkFields(self);
end;

procedure TTableManager.ResolveWorklist;
var
 i : integer;
 HI : TILWorkField;
 P : TILControlPanel;
 Handled : Boolean;
begin
  if WorkList = nil then
    Exit;
  if not UseCommandModes then
     Exit;
     P := nil;
   HI := nil;
   if WorkList.Count > 0 then
   begin
     HI := WorkList.Items.Objects[0] as TILWorkField;
      if HI.FPanel <> nil then
        P := HI.Panel;
   end;

   FPanelBuild := True;
   try
   if Assigned(FOnClickWorkList) then
   begin
     Handled := false;
     FOnClickWorkList(Self, P, Handled);
     if Handled then
     Exit;
   end;

   for I := 0 to WorkList.Count-1 do
   begin
     HI := WorkList.Items.Objects[i] as TILWorkField;
     if HI.FPanel <> nil then
       P := HI.FPanel;

     if CommandMode in [cmSearch, cmUpdate] then
     begin
       HI.Control.Visible := WorkList.Checked[i];
       HI.IsVisible := HI.Control.Visible;
       if Assigned(HI.ControlLabel) then
         HI.ControlLabel.Visible := HI.Control.Visible;
       if P = nil then
       begin
         if HI.ValueLogOpButton <> nil then
           HI.ValueLogOpButton.Visible := HI.IsVisible;
         if HI.FieldLogOpButton <> nil then
           HI.FieldLogOpButton.Visible := HI.IsVisible;
       end;
     end
     else
       HI.IsVisible := FWorkList.Checked[i];
   end;
   if HI = nil then
     Exit;
   i := Buble;
   if (i <> -1) and (WorkList <> nil) then
   begin
     WorkList.ItemIndex := i;
   end;
   if CommandMode in [cmSearch, cmUpdate] then
   begin
     if P <> nil then
     begin
      DoShowWorkFields(P);
     end;
     if (CommandMode = cmUpdate) and (HI.Table.FocusedRecord <> nil) then
      if not HI.Table.MultiSelected  or (HI.Table.SelCount = 0) then
       HI.Table.DBWork.WriteControls(HI.Table.FocusedRecord);
   end
   else if CommandMode = cmColumns then
   begin
      HI.Table.Columns.SetColumns(WorkList);
      HI.Table.Columns.Apply;
   end
   else if CommandMode = cmSort then
   begin

   end;
   ChangeWorkFields;
   if CommandMode = cmSearch then
     CreateSearchFieldOrder;
   finally
      FPanelBuild := false;
   end;
end;

procedure TTableManager.WorkListClick(Sender: TObject);
begin
   ResolveWorkList;
end;

function TTableManager.WorkFieldOf(AControl : TWinControl) : TILWorkField;
var
  i : integer;
  WF : TILWorkField;
begin
   Result := nil;
   for I := 0 to WorkFieldCount-1 do
   begin
      WF := WorkFields[i];
      if WF.Control = AControl then
      begin
        Result := WF;
        Break;
      end;
   end;
end;

function TTableManager.CanMoveWorkField(const FromIndex, ToIndex : integer) : Boolean;
var
  H : TILWorkField;
begin
    Result := false;
    if not UseCommandModes then
      Exit;
    if (FromIndex  < 0) or (FromIndex > FWorkFields.Count -1)
    or (ToIndex  < 0) or (ToIndex > FWorkFields.Count -1)
    then
     Exit;
    if CommandMode = cmSearch then
    begin
      if ActiveTable.AlignOperators then
        Exit;
    end;
    if not (CommandMode in [cmSort, cmColumns,cmSearch]) then
      Exit;
    H := WorkFields[FromIndex];
    if not H.Visible then
      Exit;
    H := WorkFields[ToIndex];
    if not H.Visible then
      Exit;
    Result := True;
end;

function TTableManager.MoveWorkField(const FromIndex: Integer; const ToIndex: Integer) : Boolean;
begin
    Result := False;
    if not UseCommandModes then
      Exit;
    Result := CanMoveWorkField(FromIndex, ToIndex);
    if not Result then
      Exit;
    if Assigned(WorkList) then
    begin
     WorkList.Items.Move(FromIndex, ToIndex);
     WorkList.ItemIndex := ToIndex;
    end;
    FWorkFields.Move(FromIndex, ToIndex);
    if (CommandMode = cmSearch) and not ActiveTable.AlignOperators then
      CreateSearchFieldOrder;
    if CommandMode in [cmSearch, cmUpdate] then
    begin
      if (WorkFieldCount > 0) and (WorkFields[0].Panel <> nil) then
        DoShowWorkFields(WorkFields[0].Panel);
    end
    else if CommandMode = cmSort then
     UpdateSortOrder
    else if CommandMode = cmColumns then
     UpdateColumnOrder;
    if CommandMode = cmSearch then
      CreateSearchFieldOrder;
    if ActiveTable.HasViewer then
      ActiveTable.Viewer.Invalidate;
    ChangeWorkFields;
end;

procedure TTableManager.CreateSearchFieldOrder;
var
  i : integer;
  s : string;
begin
  if not UseCommandModes then
    Exit;
  if ActiveTable.AlignOperators then
    Exit;
  if CommandMode <> cmSearch then
    Exit;
  s := '';
  for I := 0 to WorkFieldCount -1 do
  begin
      if not WorkFields[i].Control.Visible
      or not ActiveTable.DBCommons.IsControlActive(WorkFields[i].Control) then
        Continue;
      s := s + WorkFields[i].Field.FieldName;
      if i < WorkFieldCount-1 then
        s := s + ',';
  end;
  SearchFieldOrder := s;
end;

procedure TTableManager.UpdateColumnOrder;
var
  i : integer;
  H : TILWorkField;
begin
   if CommandMode <> cmColumns then
     Exit;
   ActiveTable.Columns.Clear;
   for I := 0 to WorkFieldCount-1 do
   begin
      H := WorkFields[i];
      if H.IsVisible then
       ActiveTable.Columns.Add(H.Field.FieldName);
   end;
   ActiveTable.Columns.Apply;
end;

procedure TTableManager.UpdateSortOrder;
var
  i : integer;
  H : TILWorkField;
begin
   if CommandMode <> cmSort then
     Exit;
   ActiveTable.Columns.SortFields.Clear;
   for I := 0 to WorkFieldCount-1 do
   begin
      H := WorkFields[i];
      if H.IsVisible then
       ActiveTable.Columns.SortFields.Add(H.Field.FieldName);
   end;
end;

procedure TTableManager.SetTableList(const Value : TListBox);
begin
  if Value = FTableList then
    Exit;
  if Value = nil then
  begin
    FTableList.Items.Clear;
    FTableList := nil;
    Exit;
  end;
  FTableList := Value;
  FTableList.OnClick := TreeViewClick;
  if not(csLoading in ComponentState) then
  begin
    PopulateTableList(FTableList);
    if FTableList.Items.Count > 0 then
      FTableList.ItemIndex := 0;
  end;
  FTableList.FreeNotification(Self);
end;

procedure TTableManager.SetTransLogAge(Value : integer);
begin
  if (Value < 0) or ((Value < 10) and (Value <> 0)) then
    Exit;
  FTransLogAge := Value;
end;

procedure TTableManager.SetWorkList(const Value : TCheckListBox);
begin
  if Value = FWorkList then
    Exit;
  if not UseCommandModes then
    Exit;
  if Value = nil then
  begin
    FWorkList.Items.Clear;
    FWorkList := nil;
    Exit;
  end;
  FWorkList := Value;
  FWorkList.OnClickCheck := WorklistClick;
  FWorkList.FreeNotification(Self);
end;

procedure TTableManager.SetWorkStyle(const Value : TWorkStyle);
var
  i : integer;
begin
   if Value <> FWorkStyle then
   begin
   FWorkStyle := Value;
   for I := 0 to RegBars.Count-1 do
     TILActionBar(RegBars[i]).ShowButtons;
   end;
end;

{procedure TTableManager.SetSharedImages(const Value : TILImageTable);
begin
  FSharedImages := Value;
  if FSharedImages <> nil then
    FSharedImages.FreeNotification(Self);
end;}

procedure TTableManager.LaunchQuickBuilder(const Operation : TQuickMode);
var
 F : Tfrm_QuickBuilder;
begin
   if not (csDesigning in ComponentState) and (Operation <> qbAlter) then
     HandleILException(0, 'This operation is design time only.');
   if (Operation = qbCreateDB) and (Database <> '') then
     HandleILException(E_DBLoaded)
   else if (Operation in [qbAlter, qbCreateChildTable]) and (Database = '') then
     HandleILException(E_DBNotLoaded);
   F := TFrm_QuickBuilder.Create(Application);
   F.TM := self;
   F.Actn := Operation;

   if Operation = qbCreateDB then
     F.Clear
   else if Operation = qbCreateChildTable then
     F.InitChild
   else
     F.Fill;

   F.ShowModal;
end;

procedure TTableManager.Synchronize;
var
  i : integer;
begin
    DoLoaded;
    for I := 0 to TableCount-1 do
    begin
      Tables[i].Columns.AssignSource;
    end;
end;


procedure TTableManager.CreateNodes;
var
  i: integer;
  M: TILTable;

begin
  Clear;
  if RootTable = nil then
    Exit;
  TTableNode.Create(RootTable, Self);
  RootTable.FActive := True;
  { Get all components that can be traced to the root }
  for I := 0 to RegTables.Count-1 do
  begin
      begin
        M := TILTable(RegTables[i]);
        while M.MasterSource <> nil do
          M := M.MasterSource as TILDataTable;
        if (M = RootTable) and (RegTables[i] <> RootTable) then
        begin
          TTableNode.Create(TILDataTable(RegTables[i]), Self);
        end;
      end;
  end;
end;

function TTableManager.DefaultColWidth(FieldDef : TILFieldDef) : integer;
begin
   Result := 100;
   case Fielddef.DataType of
     dtString: Result := 150;
     dtText: Result := 200;
     dtNumber:
     begin
        if FieldDef.LookupType = looTableKey then
          Result := 150
        else
         Result := 50;
     end;
     dtBoolean: Result := 50;
     dtDate: Result := 125;
     dtResource: Result := 150;
     dtLongText: Result := 150;
   end;
end;

procedure TTableManager.AddinheritedFields(ATable : TILTable);
var
  Prnt : TILTable;
  i, j : integer;
  Fld, PrntFld : TILFieldDef;
  Indx : integer;

  function GetFieldID : integer;
  var
    I : integer;
  begin
    Result := -1;
    for I := 0 to ATable.Fields.Count - 1 do
      Result := Max(ATable.Fields.Items[i].FieldID, Result);
    inc(Result);
  end;

  function PrntExists(SourceFld : TILFieldDef; SourceTbl : TILTable) : Boolean;
  var
    i : integer;
  begin
     Result := false;
     for i := 0 to ATable.Fields.Count-1 do
     if (ATable.Fields.Items[i].LookupType = looParentField)
     and (ATable.Fields.Items[i].LookupTable = SourceTbl)
     and SameText(ATable.Fields.Items[i].LookupField, SourceFld.FieldName)
     then
     begin
        Result := True;
        Break;
     end;
  end;

  procedure DoAdd(SourceFld : TILFieldDef; SourceTbl : TILTable);
  var
    NewFld : TILFieldDef;
    Nam : string;
    Indx : integer;
  begin
     if SourceFld.FieldType = ftStoredEvaluation then
        Exit
      else if (SourceFld.FieldType = ftSystemData) then
      begin
        if SameText(SourceFld.FieldName, 'SYS_LinkID')
        or SameText(SourceFld.FieldName, 'SYS_Level')
        or SameText(SourceFld.FieldName, 'SYS_SiblingIndex')  then
         Exit;
      end;
      if PrntExists(SourceFld, SourceTbl) then
        Exit;
      NewFld := ATable.Fields.Add;
      NewFld.Assign(SourceFld);
      NewFld.Ilib := ATable;
      Nam := SourceFld.FieldName;
      NewFld.FFieldName := '';
      Indx := ATable.Fields.IndexOf(Nam);
      if Indx = -1 then
         NewFld.FFieldName := Nam
      else
         NewFld.FFieldName := ATable.TableName + '_' + Nam;
      NewFld.FLookupType := looParentField;
      NewFld.FLookupTable := SourceTbl;
      NewFld.FLookupField := SourceFld.FieldName;
      NewFld.FLookupSearchField := '';
      NewFld.FIndexed := false;

      NewFld.FFieldID := GetFieldID;
      if NewFld.ColumnWidth = 0 then
       NewFld.FColumnWidth := DefaultColWidth(NewFld);
  end;

begin
    { First syncronize properties of exisiting persistent parent fields.
    These will originally have the string data type}
    for I := 0 to ATable.Fields.Count-1 do
    begin
      if ATable.Fields.Items[i].LookupType = looParentField then
      begin
         Indx := ATable.Fields.Items[i].LookupTable.Fields.IndexOf(ATable.Fields.Items[i].LookupField);
         if Indx = -1 then
           Continue;
         PrntFld := ATable.Fields.Items[i].LookupTable.Fields.Items[Indx];
         Fld := ATable.Fields.Items[i];
         Fld.FDataType := PrntFld.DataType;
         Fld.FDecimals := PrntFld.FDecimals;
         Fld.FIndexed := false;
      end;
    end;

    if not ATable.ParentFields then
      Exit;
    Prnt := ATable.MasterSource;
    if Prnt <> nil then

    for I := 0 to Prnt.Fields.Count-1 do
    begin
      PrntFld := Prnt.Fields.Items[i];
      DoAdd(PrntFld, Prnt);
    end;
    if ATable is TILDataTable then
    begin
      for I := 0 to ATable.Fields.Count-1 do
      begin
        Fld := ATable.Fields.Items[i];
        if Fld.LookupType = looTableKey then
        begin
          Prnt := Fld.LookupTable;
          for J := 0 to Prnt.Fields.Count-1 do
          begin
            PrntFld := Prnt.Fields.Items[j];
            if SameText(PrntFld.FieldName, Fld.LookupField)
            or SameText(PrntFld.FieldName, Fld.LookupSearchField) then
             Continue;
            DoAdd(PrntFld, Prnt);
          end;
        end;
      end;
   end;
end;

function TTableManager.SyncTable(ATable : TILTable) : string;
{ Compare db properties to the db properties in the metafile on start up.
If there are differnces, db properties are syncronized with the metafile.
Anomalies may occure when different applications share the same database
Returns the table classname: empty if table not found.
}
var
 ANode: IXMLNode;
 Meta: string;
 P1, P2 : integer;
 S, S2 : string;
 N : integer;
 TblName : string;
 TblName2 : string;
 Defs : TILFieldDefs;
 TBL : TILTable;
 F, F2 : integer;
 Cnt : integer;
 FieldName : string;
 Indx : integer;
 FD : TILFieldDef;
 T1 : integer;
 X : TXMLDocument;
 sl : TStringList;
 FieldNames : TStringList;
 i : integer;
 Deleted : Boolean;
begin
    if not FileExists(FMetaName) then
    begin
      Comment1 := FMetaName;
      HandleILException(E_FileNotExists);
    end;
   Result := '';
   X := TXMLDocument.Create(Self);
   X.Active := True;
   X.DocumentElement := X.CreateNode('Ilib', ntElement, '');
   X.LoadFromFile(FMetaName);
   try
    ANode := X.DocumentElement.ChildNodes[0];

    Meta := ANode.Xml;

    P1 := Pos('<DBName>', Meta);
    P2 := Pos('</DBName>', Meta);
    Inc(P1, Length('<DBName>'));
    S := Copy(Meta, P1, P2 - P1);
     FDatabase := S;

    P1 := Pos('<Encoding>', Meta);
    P2 := Pos('</Encoding>', Meta);
    Inc(P1, Length('<Encoding>'));
    S := Copy(Meta, P1, P2 - P1);
    if S = '' then
      FUnicode := True
    else
     FUnicode := (StrToInt(S) = 0);

    P1 := Pos('<TransHistory>', Meta);
    P2 := Pos('</TransHistory>', Meta);
    Inc(P1, Length('<TransHistory>'));
    S := Copy(Meta, P1, P2 - P1);
    FTransHistory := (S = '1');

    { Find this table }
      P1 := Pos('<TableName>', Meta);
      P2 := Pos('</TableName>', Meta);
      T1 := Pos('<Table>', Meta);
      Inc(P1, Length('<TableName>'));
      TblName := Copy(Meta, P1, P2 - P1);

      if ATable is TILView then
      begin
        if TILView(ATable).SourceTable = nil then
          Exit;
        TblName2 := TILView(ATable).SourceTable.TableName;
      end
      else
        TblName2 := ATable.TableName;
      while not SameText(TblName, TblName2) do
      begin

        P1 := Pos('<TableName>', Meta, P2);
        P2 := Pos('</TableName>', Meta, P1);
        T1 := Pos('<Table>', Meta, T1 + 1);
        Inc(P1, Length('<TableName>'));
        TblName := Copy(Meta, P1, P2 - P1);
        if TblName = '' then
        { Table not found in meta, must be deleted }
        begin
           Exit;
        end;
      end;


      P1 := Pos('<TableClass>', Meta, P1);
      P2 := Pos('</TableClass>', Meta, P1);
      Inc(P1, Length('<TableClass>'));
      S := Copy(Meta, P1, P2 - P1);
      Result := S;
      if ATable is TILView then
       Result := 'TILView'
      else if SameText(S, 'TILTreeViewTable') then
      begin
        if not (ATable is TILTreeViewTable) then
          Exit;
      end
      else if SameText(S, 'TILDataTable') then
      begin
        if not (ATable is TILDataTable) then
          Exit;
      end
      else if SameText(S, 'TILImageTable') then
      begin
        if not (ATable is TILImageTable) then
         Exit;
      end
      else if SameText(S, 'TILLookupTable') then
      begin
        if not (ATable is TILLookupTable) then
          Exit;
      end;

      { Cut out the metadata for this table }
      P2 := Pos('</Table>', Meta, P1);
      P1 := T1 + Length('<Table>');
      Meta := Copy(Meta, P1, P2 - P1);

      Defs := ATable.FFields;
      TBL := ATable;

      P1 := Pos('<ErrorState>', Meta);
      P2 := Pos('</ErrorState>', Meta);
      Inc(P1, Length('<ErrorState>'));
      S := Copy(Meta, P1, P2 - P1);
      Tbl.FErrorState := StrToInt(S);

      P1 := Pos('<IsRoot>', Meta);
      P2 := Pos('</IsRoot>', Meta);
      Inc(P1, Length('<IsRoot>'));
      S := Copy(Meta, P1, P2 - P1);
      if (S = '1') and not (ATable is TILView) then
      begin
        FRootTable := ATable as TILDataTable;
      end;

      P1 := Pos('<Field>', Meta);
      P2 := Pos('</Field>', Meta);
      Inc(P1, Length('<Field>'));
      S := Copy(Meta, P1, P2 - P1);

      if S = '' then
      begin
        Exit;
      end;
      FieldNames := TStringList.Create;
      try
      Cnt := -1;
      repeat
        Inc(Cnt);
        F := Pos('<FieldName>', S);
        F2 := Pos('</FieldName>', S);
        Inc(F, Length('<FieldName>'));
        s2 := Copy(S, F, F2 - F);

        if s2 = '' then
         exit;

        FieldName := s2;
        Indx := Defs.IndexOf(s2);
        if (Indx = -1) or (Indx <> cnt) then
        { Field added }
        begin
          Importing := True;
          FD := Defs.Add;
          Importing := False;
        end
        else
          FD := Defs.Items[Indx];

        { Track fieldnames to later find deltions }
        FieldNames.Add(FieldName);
        F := Pos('<FieldName>', S);
        F2 := Pos('</FieldName>', S);
        Inc(F, Length('<FieldName>'));
        s2 := Copy(S, F, F2 - F);
        if not SameText(FD.FFieldName,s2) then
          FD.FFieldName := s2;

        F := Pos('<FieldID>', S);
        F2 := Pos('</FieldID>', S);
        Inc(F, Length('<FieldID>'));
        s2 := Copy(S, F, F2 - F);
        if s2 = '' then
         s2 := IntToStr(FD.FFields.IndexOf(FD.FieldName));
          FD.FFieldID := StrToInt(s2);

        F := Pos('<FieldType>', S);
        F2 := Pos('</FieldType>', S);
        Inc(F, Length('<FieldType>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FFieldType <> TFieldType(StrToInt(s2)) then
          FD.FFieldType := TFieldType(StrToInt(s2));

        F := Pos('<DataType>', S);
        F2 := Pos('</DataType>', S);
        Inc(F, Length('<DataType>'));
        s2 := Copy(S, F, F2 - F);
        if (FD.FDataType <> TDataType(StrToInt(s2))) then
          FD.FDataType := TDataType(StrToInt(s2));

        F := Pos('<Decimals>', S);
        F2 := Pos('</Decimals>', S);
        Inc(F, Length('<Decimals>'));
        s2 := Copy(S, F, F2 - F);
        if (FD.FDecimals <> StrToInt(s2)) then
          FD.FDecimals := StrToInt(s2);

        F := Pos('<DateFormat>', S);
        F2 := Pos('</DateFormat>', S);
        Inc(F, Length('<DateFormat>'));
        s2 := Copy(S, F, F2 - F);
        if (FD.FDateFormat <> s2) then
          FD.FDateFormat := s2;

        F := Pos('<Identifier>', S);
        F2 := Pos('</Identifier>', S);
        Inc(F, Length('<Identifier>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FIdentifier <> Boolean(StrToInt(s2)) then
          FD.FIdentifier := Boolean(StrToInt(s2));

        F := Pos('<Indexed>', S);
        F2 := Pos('</Indexed>', S);
        Inc(F, Length('<Indexed>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FIndexed <> Boolean(StrToInt(s2)) then
         FD.FIndexed := Boolean(StrToInt(s2));

        F := Pos('<IndexPriority>', S);
        F2 := Pos('</IndexPriority>', S);
        Inc(F, Length('<IndexPriority>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FIndexPriority <> TIndexPriority(StrToInt(s2)) then
          FD.FIndexPriority := TIndexPriority(StrToInt(s2));

        F := Pos('<LookupType>', S);
        F2 := Pos('</LookupType>', S);
        Inc(F, Length('<LookupType>'));
        s2 := Copy(S, F, F2 - F);
        if FD.LookupType <> TLookupType(StrToInt(s2)) then
          FD.FLookupType := TLookupType(StrToInt(s2));

        F := Pos('<LookupField>', S);
        F2 := Pos('</LookupField>', S);
        Inc(F, Length('<LookupField>'));
        s2 := Copy(S, F, F2 - F);
        if FD.LookupField <> s2 then
          FD.FLookupField := s2;

        F := Pos('<LookupSearchField>', S);
        F2 := Pos('</LookupSearchField>', S);
        Inc(F, Length('<LookupSearchField>'));
        s2 := Copy(S, F, F2 - F);
        if FD.LookupSearchField <> s2 then
          FD.FLookupSearchField := s2;

        { Lookuptable is linked in SyncRelations }

        FD.FColumnWidth := DefaultColWidth(FD);

        F := Pos('<Alignment>', S);
        F2 := Pos('</Alignment>', S);
        Inc(F, Length('<Alignment>'));
        s2 := Copy(S, F, F2 - F);
        if s2 <> '' then
        if (FD.FAlignment <> TAlignment(StrToInt(s2))) then
          FD.FAlignment := TAlignment(StrToInt(s2));

        F := Pos('<PickValues>', S);
        F2 := Pos('</PickValues>', S);
        Inc(F, Length('<PickValues>'));
        s2 := Copy(S, F, F2 - F);
        if s2 <> '' then
        begin
         sl := TStringList.Create;
         try
          CreateListFromDelimiter('~', s2, sl);
          FD.PickValues.Assign(sl);
         finally
           sl.Free;
         end;
        end;

        F := Pos('<InitialValue>', S);
        F2 := Pos('</InitialValue>', S);
        Inc(F, Length('<InitialValue>'));
        s2 := Copy(S, F, F2 - F);
        if not SameText(s2, FD.InitialValue) then
        begin
         FD.FInitialValue := s2;
         if FD.FieldType = ftAutoIncrement then
         begin
           N := FD.GetNextAutoValue(false);
           if StrToInt(S2) > N then
             FD.SetNextAutoValue(StrToInt(S2));
         end;
        end;

        F := Pos('<Mandatory>', S);
        F2 := Pos('</Mandatory>', S);
        Inc(F, Length('<Mandatory>'));
        s2 := Copy(S, F, F2 - F);
        if FD.Mandatory <> Boolean(StrToInt(s2)) then
          FD.FMandatory := Boolean(StrToInt(s2));

        F := Pos('<Unique>', S);
        F2 := Pos('</Unique>', S);
        Inc(F, Length('<Unique>'));
        s2 := Copy(S, F, F2 - F);
        if FD.Unique <> Boolean(StrToInt(s2)) then
           FD.FUnique := Boolean(StrToInt(s2));

        F := Pos('<MaxLength>', S);
        F2 := Pos('</MaxLength>', S);
        Inc(F, Length('<MaxLength>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FMaxLength <> StrToInt(s2) then
          FD.FMaxLength := StrToInt(s2);

        F := Pos('<MaxValues>', S);
        F2 := Pos('</MaxValues>', S);
        Inc(F, Length('<MaxValues>'));
        s2 := Copy(S, F, F2 - F);
        if FD.MaxValues <> StrToInt(s2) then
         FD.FMaxValues := StrToInt(s2);

        F := Pos('<ReportField>', S);
        F2 := Pos('</ReportField>', S);
        Inc(F, Length('<ReportField>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FReportField <> Boolean(StrToInt(s2)) then
          FD.FReportField := Boolean(StrToInt(s2));

        F := Pos('<ReportCaption>', S);
        F2 := Pos('</ReportCaption>', S);
        Inc(F, Length('<ReportCaption>'));
        s2 := Copy(S, F, F2 - F);
        if FD.FReportCaption <> s2 then
          FD.FReportCaption := s2;

        F := Pos('<LTPageSize>', S);
        F2 := Pos('</LTPageSize>', S);
        Inc(F, Length('<LTPageSize>'));
        s2 := Copy(S, F, F2 - F);
        if s2 = '' then
          FD.FLTPageSize := ltp4KB
        else
         if (FD.LTPageSize <> TLTPageSize(StrToInt(s2))) then
           FD.FLTPageSize := TLTPageSize(StrToInt(s2));

        P1 := Pos('<Field>', Meta, P1 + 1);
        P2 := Pos('</Field>', Meta, P2 + 1);
        Inc(P2, Length('</Field>'));
        S := Copy(Meta, P1, P2 - P1);
      until P1 = 0;

      P1 := Pos('<TableName>', Meta);
      P2 := Pos('</TableName>', Meta);
      Inc(P1, Length('<TableName>'));
      S := Copy(Meta, P1, P2 - P1);
      if ATable is TILView then
        Tbl.FTableName := ATable.TableName
      else if not SameText(Tbl.TableName, S) then
         Tbl.FTableName := S;

      P1 := Pos('<TableSpace>', Meta);
      P2 := Pos('</TableSpace>', Meta);
      Inc(P1, Length('<TableSpace>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.TableSpace <> StrToInt(S) then
         Tbl.FTableSpace := StrToInt(S);

      P1 := Pos('<RecordFreeSpace>', Meta);
      P2 := Pos('</RecordFreeSpace>', Meta);
      Inc(P1, Length('<RecordFreeSpace>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.RecordFreeSpace <> StrToInt(S) then
         Tbl.FRecordFreeSpace := StrToInt(S);

      P1 := Pos('<RecordSize>', Meta);
      P2 := Pos('</RecordSize>', Meta);
      Inc(P1, Length('<RecordSize>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.RecordSize <> StrToInt(S) then
         Tbl.FRecordSize := StrToInt(S);

      P1 := Pos('<LocalImageSize>', Meta);
      P2 := Pos('</LocalImageSize>', Meta);
      Inc(P1, Length('<LocalImageSize>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.FImageLocalSize <> TImageLocalSize(StrToInt(S)) then
         Tbl.FImagelocalSize := TImageLocalSize(StrToInt(S));

      P1 := Pos('<RecordLayout>', Meta);
      P2 := Pos('</RecordLayout>', Meta);
      Inc(P1, Length('<RecordLayout>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.RecordLayout <> TRecordLayout(StrToInt(S)) then
         Tbl.FRecordLayout := TRecordLayout(StrToInt(S));

      P1 := Pos('<Relations>', Meta);
      P2 := Pos('</Relations>', Meta);
      Inc(P1, Length('<Relations>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.Relations <> TRelations(StrToInt(S)) then
         Tbl.FRelations := TRelations(StrToInt(S));

      P1 := Pos('<Alias>', Meta);
      P2 := Pos('</Alias>', Meta);
      Inc(P1, Length('<Alias>'));
      S := Copy(Meta, P1, P2 - P1);
      if Tbl.Alias <> S then
         Tbl.FAlias := S;

      P1 := Pos('<DateSearchFormat>', Meta);
      P2 := Pos('</DateSearchFormat>', Meta);
      Inc(P1, Length('<DateSearchFormat>'));
      S := Copy(Meta, P1, P2 - P1);
      if S = '' then
         Tbl.DateSearchFormat := dsfDate
      else if Tbl.DateSearchFormat <> TDateSearchFormat(StrToInt(S)) then
         Tbl.DateSearchFormat := TDateSearchFormat(StrToInt(S));

      P1 := Pos('<ParentFields>', Meta);
      P2 := Pos('</ParentFields>', Meta);
      Inc(P1, Length('<ParentFields>'));
      S := Copy(Meta, P1, P2 - P1);
      if S = '' then
         Tbl.FParentFields := false
      else if Tbl.FParentFields <> Boolean(StrToInt(S)) then
         Tbl.FParentFields := Boolean(StrToInt(S));

      P1 := Pos('<Shared>', Meta);
      P2 := Pos('</Shared>', Meta);
      Inc(P1, Length('<Shared>'));
      S := Copy(Meta, P1, P2 - P1);
      if S <> '' then
       if Boolean(StrToInt(S)) then
         FSharedImages := Tbl as TILImageTable;

      { Check deltions }
      repeat
      Deleted := false;
      for I := 0 to ATable.Fields.Count-1 do
      begin
          if (FieldNames.IndexOf(ATable.Fields.Items[i].FieldName) = -1)
          and not ((ATable.Fields.Items[i].LookupType = looParentField) and TBL.ParentFields)
          then
          begin
              if ATable.DataHandler <> nil then
              begin
                N := ATable.DataHandler.DataFields.IndexOf(ATable.Fields.Items[i].FieldName);
                if N <> -1 then
                  ATable.DataHandler.DataFields.Items[N].FieldName := '';
              end;
              Importing := True;
              ATable.Fields.Delete(i);
              Importing := False;
              Deleted := True;
              Break;
          end;
      end;
      until not Deleted;

      finally
        FieldNames.Free;
      end;

      if NOT Assigned (ATable.Columns) then
      begin
        ATable.FColumns := TILColumns.Create(ATable, ATable.Fields);
      end;

   finally
     X.Free;
   end;
end;

procedure TTableManager.SyncRelations(ATable: TILTable);
var
 ANode: IXMLNode;
 Meta: string;
 P1, P2 : integer;
 S : string;
 TblName, TblName2 : string;
 TBL : TILTable;
 T1 : integer;
 i : integer;
 F, F2 : integer;
 FD : TILFieldDef;
 s2 : string;
 Defs : TILFieldDefs;
 FieldName : string;
 Indx : integer;
 X : TXMLDocument;
begin
    X := TXMLDocument.Create(Self);
    X.Active := True;
    X.DocumentElement := X.CreateNode('Ilib', ntElement, '');
    X.LoadFromFile(FMetaName);
    ANode := X.DocumentElement.ChildNodes[0];
     Meta := ANode.Xml;
    X.Free;
    { Find this table }
    P1 := Pos('<TableName>', Meta);
    P2 := Pos('</TableName>', Meta);
    T1 := Pos('<Table>', Meta);
    Inc(P1, Length('<TableName>'));
    TblName := Copy(Meta, P1, P2 - P1);
    if ATable is TILView then
    begin
     if TILView(ATable).SourceTable = nil then
       Exit;
     TblName2 := TILView(ATable).SourceTable.TableName;
    end
    else
     TblName2 := ATable.TableName;

    while not SameText(TblName, TblName2) do
    begin
        P1 := Pos('<TableName>', Meta, P2);
        P2 := Pos('</TableName>', Meta, P1);
        T1 := Pos('<Table>', Meta, T1 + 1);
        Inc(P1, Length('<TableName>'));
        TblName := Copy(Meta, P1, P2 - P1);
    end;

    { Cut out the metadata for this table }
    P2 := Pos('</Table>', Meta, P1);
    P1 := T1 + Length('<Table>');
    Meta := Copy(Meta, P1, P2 - P1);

    if not (ATable is TILView) then
    begin
      P1 := Pos('<MasterSource>', Meta);
      P2 := Pos('</MasterSource>', Meta);
      Inc(P1, Length('<MasterSource>'));
      S := Copy(Meta, P1, P2 - P1);
      if S <> '' then
      begin
         for I := 0 to RegTables.Count-1 do
         begin
           TBL := TILTable(RegTables[i]);
           if SameText(Tbl.TableName, S) then
           begin
             ATable.FMasterSource := TBL as TILDataTable;
             Break;
           end;
         end;
      end;
    end;

    { Create master links }
    P1 := Pos('<LookupType>', Meta);
    P2 := Pos('</LookupType>', Meta);
    Inc(P1, Length('<LookupType>'));
    S := Copy(Meta, P1, P2 - P1);
    while P1 <> 0 do
    begin
      if S = '3' then
      begin
         P1 := Pos('<LookupTable>', Meta, P1);
         P2 := Pos('</LookupTable>', Meta, P2);
         Inc(P1, Length('<LookupTable>'));
         S := Copy(Meta, P1, P2 - P1);
         if S <> '' then
         for I := 0 to RegTables.Count-1 do
         begin
           TBL := TILTable(RegTables[i]);
           if SameText(Tbl.TableName, S) then
           begin
             ATable.FMasterLinks.Add(TBL);
             Break;
           end;
         end;
      end;
      P1 := Pos('<LookupType>', Meta, P1+1);
      if P1 <> 0 then
      begin
        Inc(P1, Length('<LookupType>'));
        P2 := Pos('</LookupType>', Meta, P2 + 1);
        S := Copy(Meta, P1, P2 - P1);
      end;
    end;

    Defs := ATable.FFields;

    P1 := Pos('<Field>', Meta);
    P2 := Pos('</Field>', Meta);
    Inc(P1, Length('<Field>'));
    S := Copy(Meta, P1, P2 - P1);

    if S = '' then
      Exit;

    repeat
        F := Pos('<LookupTable>', S);
        F2 := Pos('</LookupTable>', S);
        Inc(F, Length('<LookupTable>'));
        s2 := Copy(S, F, F2 - F);
        if s2 <> '' then
        begin
          F := Pos('<FieldName>', S);
          F2 := Pos('</FieldName>', S);
          Inc(F, Length('<FieldName>'));
          FieldName := Copy(S, F, F2 - F);

          Indx := Defs.IndexOf(FieldName);
          FD := Defs.Items[Indx];

          for I := 0 to RegTables.Count-1 do
           begin
             TBL := TILTable(RegTables[i]);
             if SameText(Tbl.TableName, S2) then
             begin
               FD.FLookupTable := TBL;
               Break;
             end;
           end;
        end;

       P1 := Pos('<Field>', Meta, P1 + 1);
       P2 := Pos('</Field>', Meta, P2 + 1);
       Inc(P2, Length('</Field>'));
       S := Copy(Meta, P1, P2 - P1);

    until P1 = 0;
    AddInheritedFields(ATable);
end;

procedure TTableManager.SyncDetailLinks(ATable : TilTable);
var
  i, j : integer;
  TBL : TILTable;
begin
    {Create detail links }
    for I := 0 to RegTables.Count-1 do
    begin
      TBL := TILTable(RegTables[i]);
      for j  := 0 to Tbl.Fields.Count-1 do
      begin
        if (Tbl.Fields.Items[j].LookupType = looTableKey)
        and SameText(Tbl.Fields.Items[j].LookupTable.Tablename, ATable.TableName) then
        begin
          ATable.FDetailLinks.Add(Tbl);
        end;
      end;
    end;

end;

function TTableManager.TableFromSQL(SQL : Tstrings) : TILTable;
var
  s : string;
  P1, P2 : integer;
  L : integer;
begin
   Result := nil;
   L := 0;
   RemoveComments(SQL);
   S := SQL.Text;
   S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
   P1 := PosRestrict('FROM', S, false);
   if P1 = 0 then
    P1 := PosRestrict('TABLE', S, false)
   else
     L := Length('FROM');
   if P1 = 0 then
     Exit;
   if L = 0 then
     L := Length('TABLE');
   P1 := P1 + L;
   while S[P1] = ' ' do
     inc(P1); { Start of table name }
   P2 := P1;
   while (S[P2] <> ' ') and (S[P2] <> #$D) do
    Inc(P2); { End of table name}
   S := Copy(S, P1, P2-P1);
   Result := TableOf(S);

end;

function TTableManager.TablesOfMeta : TStringList;
var
 ANode: IXMLNode;
 Meta: string;
 P1, P2 : integer;
 TblName : string;
 X : TXMLDocument;
begin
    X := TXMLDocument.Create(Self);
    X.Active := True;
    X.DocumentElement := X.CreateNode('Ilib', ntElement, '');
    X.LoadFromFile(FMetaName);
    ANode := X.DocumentElement.ChildNodes[0];
    Meta := ANode.Xml;
    X.Free;
    { Find this table }
    Result := TStringList.Create;
    P1 := Pos('<TableName>', Meta);
    P2 := Pos('</TableName>', Meta);
    Inc(P1, Length('<TableName>'));
    TblName := Copy(Meta, P1, P2 - P1);
    while TblName <> '' do
    begin
      Result.Add(TblName);
      P1 := Pos('<TableName>', Meta, P1);
      P2 := Pos('</TableName>', Meta, P1);
      Inc(P1, Length('<TableName>'));
      TblName := Copy(Meta, P1, P2 - P1);
    end;
end;

function TTableManager.FindMissingTables : integer;
var
  sl : TStringList;
  i : integer;
  T : TILReplicator;

  function IsReg(S : string) : Boolean;
  var
   i : integer;
  begin
    Result := False;
    for I := 0 to RegTables.Count-1 do
      if SameText(TILTable(RegTables[i]).TableName, S) then
      begin
        Result := True;
        Break;
      end;
  end;

begin
   Result := 0;
   sl := TablesOfMeta;
   T := TILReplicator.Create(FMetaName, Self);
   try
     for I := 0 to sl.Count-1 do
     begin
       if not IsReg(sl[i]) then
       begin
         T.Replicate(sl[i]);
         inc(Result);
       end;
     end;
   finally
     sl.Free;
     T.Free;
   end;
end;

function TTableManager.CheckStates : TILTable;
var
  i : integer;
begin
   Result := nil;
   if FSkipValidation then
     Exit;
   for I := 0 to TableCount-1 do
   begin
      if Tables[i].FErrorState <> 0 then
      begin
        Result := Tables[i];
        Break;
      end;
   end;
end;

function TTableManager.OptionInCommand(const ACommand : TCommandMode; const Option : TBaseCommandOption) : Boolean;
{ True if the option is registered with cmdmode }
begin
    if Option = ovDefault then
    begin
      Result := True;
      Exit;
    end;
    Result := False;
    case ACommand of
      cmSearch:
      begin
        if not (Option in [ovSearchAnd, ovSearchOr]) then
          Exit;
      end;
      cmUpdate:
      begin
        if not (Option in [ovUpdateData,ovUpdateImage]) then
          Exit;
      end;
      cmSort:
      begin
        if not (Option in [ovSortAsc, ovSortDesc]) then
          Exit;
      end;
      cmColumns: Exit;
      cmSQL: Exit;
    end;
    Result := True;
end;

procedure TTableManager.RemoveGarbage;
begin
  if RootTable = nil then
    HandleILException(E_MissingRootTable);
  FGarbaging := True;
  try
    CleanUp(nil);
  finally
    FGarbaging := false;
  end;
end;

procedure TTableManager.SaveIndexes;
 var
   LI : TILSaveIndex;
 begin
      LI := TILSaveIndex.Create(True);
      LI.FreeOnTerminate := True;
      LI.Start;
 end;

procedure TTableManager.RunTransactions(ChangedTables : TList);
var
  Log : TStringList;
  i : integer;
  TL : TILTranslogItem;
  Tbl : TILTable;
  Cnt : integer;
  MH : TILMessageHandler;
  Ind : integer;
  SavCnt : integer;
begin
  FtranslogBuffer.SaveToFile(FRollbackName); {If Hardware failure }
  Log := LoadTranslog;
  Cnt := 0;
  for I := 0 to TMan.TableCount-1 do
  begin
      Ind := -1;
      MH := TMan.GetMessageHandler(TMan.Tables[i], Ind);
      while MH <> nil do
      begin
        MH.FSaveState := -1;
        MH := TMan.GetMessageHandler(TMan.Tables[i], Ind);
      end;
  end;
  SavCnt := 0;
  try
   try
    for i := 0 to Log.Count-1 do
     begin
       inc(SavCnt);
       TL := Log.Objects[i] as TILTranslogItem;
       TBL := TableOf(TL.TableName);
       if TL.FTransType in[dboUpdate, dboDelete] then
       begin
         TBL.FSF_Data[TL.RecordIndex] := TL.FTrans;
       end
       else
         TBL.FSF_Data.Add(TL.FTrans);
       inc(Cnt);
       if (ChangedTables.IndexOf(TBL) = -1) then
           ChangedTables.Add(TBL);
     end;

     for I := 0 to ChangedTables.Count-1 do
     begin
       Ind := -1;
       MH := TMan.GetMessageHandler(TILTable(ChangedTables[i]), Ind);
       while MH <> nil do
       begin
        MH.FSaveCount := SavCnt;
        MH := TMan.GetMessageHandler(TILTable(ChangedTables[i]), Ind);
       end;
     end;

     System.SysUtils.Deletefile(FRollbackName);
   except on E:Exception do
    begin
      FCfgFile.WriteInteger('Transaction', 'WriteCount', Cnt);
      HandleILException(0, 'An unexpected error occured during writing records to the database. ' +
      'Please close and restart your application to make the system attempt to restore your data. ' +
      'If the error continues, please contact your vendor with the message: "' + E.Message + '"');
    end;
   end;

   if TransHistory then
     SaveTranslog;

   ActiveTable.Dataset.Sort;
   if (ActiveTable.Dataset.Position <> -1) and Activetable.HasViewer then
      ActiveTable.Viewer.ScrollTo(ActiveTable.Dataset.Position);

  finally
    Log.Free;
  end;
end;

 function TTableManager.CleanUp(ATable : TILTable) : integer;
 type
   TOldInd = record
     TableName : string;
     Indexes : TStringList;
   end;
 var
    OldInds : array of TOldInd; { Conversion tables }
    DelCnt: integer; { Number of dels regiserd on main table }
    I: Integer;
    EstmLoops : integer;
    ProgCounter : integer;
    TmpImgs: TFileStream;
    TmpTextLobs : TFileStream;
    TmpTransLog : TFileStream;
    TmpData : TSubFile;
    TmpLib : TFileContainer;
    TrLog : TStringList;

  procedure RemoveTempFiles;
  begin
     System.SysUtils.DeleteFile(_FTmpLibPath);
     System.SysUtils.DeleteFile(_FTmpThumbName);
     System.SysUtils.DeleteFile(_FTmpTransLogName);
     System.SysUtils.DeleteFile(_FTmpTextLobsName);
  end;


  procedure RemoveWorkFiles;
  begin
     System.SysUtils.DeleteFile(_FLibPath);
     System.SysUtils.DeleteFile(_FTableCfgName);
     System.SysUtils.DeleteFile(_FThumbName);
     System.SysUtils.DeleteFile(_FTransLogName);
     System.SysUtils.DeleteFile(_FTextLobsName);
     RemoveTempFiles;
  end;

  function GetValue(Tag, XML : string) : string;
  var
    P1, P2 : integer;
    STag, NTag : string;
    begin
      STag := '<' + Tag + '>';
      NTag :=  '</' + Tag + '>';
      P1 := Pos(Stag, XML);
      P2 := Pos(NTag, XML);
      inc (P1,Length(STag));
      Result := Copy(XML, P1, P2-P1);
      if (Pos('F', Tag) = 1) and TryStrToInt(Copy(Tag, 2, Length(Tag)), P1) then
        Result := GetValue('Vl', Result);
     end;

  function SetValue(Tag, Value, XML : string) : string;
  var
    P1, P2, N : integer;
    STag, NTag : string;
    begin
      STag := '<' + Tag + '>';
      NTag :=  '</' + Tag + '>';
      P1 := Pos(Stag, XML);
      P2 := Pos(NTag, XML);
      inc (P1,Length(STag));
      if (Pos('F', Tag) = 1) and TryStrToInt(Copy(Tag, 2, Length(Tag)), N)
      and (Pos('<Vl>', Trim(Value)) <> 1) then { Multi value}
      begin
        STag := '<Vl>';
        NTag :=  '</Vl>';
        P1 := Pos(Stag, XML, P1);
        P2 := Pos(NTag, XML, P1);
        inc(P1, Length(STag));
      end;
      System.Delete(XML, P1, P2-P1);
      Insert(Value, XML, P1);
      Result := XML;
    end;


  function IsDel(XML : string) : Boolean;
  var
    S : string;
    begin
      if Pos('{Mov', XML) = 1 then
      begin
       Result := True;
       Exit;
      end;
      S := GetValue('St', XML);
      Result := SameText(S, 'Deleted') or
       SameText(S, 'DT') or
       SameText(S, 'DD') or
       SameText(S, 'DeletedTrans') or
       SameText(S, 'Moved') or
       SameText(S, 'MO');
    end;

  procedure SaveImgToTemp(AThumbnail: Vcl.Graphics.TBitmap; var Offset, Count: int64);
    var
      MS: TMemoryStream;
      P: int64;
  begin
    MS := TMemoryStream.Create;
    try
      AThumbnail.SaveToStream(MS);
      P := TmpImgs.Seek(0, SoEnd);
      AThumbnail.SaveToStream(TmpImgs);
      Offset := P;
      Count := MS.Size;
    finally
      MS.Free;
    end;
  end;

  function GetNewRef(OldVal : string; ARefTable : TILTable) : string;
    var
      i : integer;
      Indx : integer;
      OldValInd : integer;
    begin
      OldValInd := 0;
      for I := 0 to High(OldInds) do
      if OldInds[i].TableName = ARefTable.TableName then
      begin
        OldValInd := i;
        Break;
      end;
      Result := '';
      if OldInds[OldValInd].Indexes.Find(OldVal, Indx) then
        Result := FillZeroes(IntToStr(Indx), 7);
    end;

  procedure DoRemove(DelTbl : TILTable; var D : integer);
  var
    i, j: integer;
    Cnt: integer;
    Data: TSubFile;
    Deleted: Boolean;
    NewCnt: integer;
    NVal : string;
    OVal : string;
    Tag : string;
    S2 : string;
    N : integer;
    P : integer;
    Indx : integer;
    S : string;

  begin
    SetLength(OldInds, Length(OldInds) + 1);
    OldInds[High(OldInds)].Indexes := TStringList.Create;
    OldInds[High(OldInds)].Indexes.Sorted := True;
    OldInds[High(OldInds)].Indexes.Duplicates := dupIgnore;
    OldInds[High(OldInds)].Indexes.OwnsObjects := True;
    OldInds[High(OldInds)].TableName := DelTbl.TableName;;

    Data := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(DelTbl.FDataName)];
    Cnt := Data.Count;
    Data.Position := 0;
    TmpData := TmpLib.Files[TmpLib.IndexOf(DelTbl.FDataName)];
    TmpData.Clear;
    NewCnt := -1;

    for i := 0 to Cnt - 1 do
    begin
      Deleted := false;
      S := Data.AbsItems[i];
      inc(ProgCounter);
      TMan.Progress(DelTbl, iloRemoveDeleted, EstmLoops, ProgCounter);
      if DelTbl is TILImageTable then
      begin
      { Not ref. safe }
       TmpData.Add(S);
       Continue;
      end;
      if IsDel(S) then
      begin
          if Pos('{Mov', S) = 1 then
          begin
             P := Pos('}', S);
             N := StrToInt(Copy(S,5, P-5));
             S2 := Data.AbsItems[N];
             while Pos('{Mov', S2) = 1 do
             begin
               N := StrToInt(Copy(S2,5, P-5));
               S2 := Data.AbsItems[N];
             end;
             { S2 is end of chain}

             Oval := GetValue('St', S2);

             if not IsDel(S2) and (Oval <> 'X')  {Aleady processed } then
             begin
               Inc(NewCnt);
               Oval := GetValue('In', S2);
               S2 := SetValue('In', IntToStr(NewCnt), S2);
               TmpData.Add(S2);
               Oval := FillZeroes(OVal, 7);
               OldInds[High(OldInds)].Indexes.Add(OVal);
               S2 := SetValue('St', 'X', S2); { Mark as handeled}
               Data.Items[N] := S2;
             end;
          end;
          Inc(D);
          Continue;
      end
      else
      begin
        if Deltbl.IsDetail then
        begin
          Indx := DelTbl.Fields.IndexOf('SYS_LinkId');
          Tag := 'F' + IntToStr(DelTbl.Fields.Items[Indx].FieldID);
          OVal := GetValue(Tag, S);
          Oval := FillZeroes(OVal, 7);
          NVal := GetNewref(OVal, DelTbl.ParentTable);
          if NVal = '' then { Orphan }
            Deleted := True
        end;
        if not Deleted then
          Inc(NewCnt);
      end;
      if not Deleted then
      begin
        OVal := GetValue('St', S);
        if Oval = 'X' then
        begin
          Dec(NewCnt);
          Continue;
        end;
        OVal := GetValue('In', S);
        Oval := FillZeroes(OVal, 7);
        OldInds[High(OldInds)].Indexes.Add(OVal);
        NVal := IntToStr(NewCnt);
        S := SetValue('In', Nval, S);
        S := SetValue('St', '', S);
        TmpData.Add(S);
      end;
    end;

    for j := 0 to DelTbl.ChildCount-1 do
      DoRemove(DelTBL.Children[j], D) ;

  end;

  procedure RestoreRefs(ATable : TILTable);
  var
    i, j : integer;
    RefMap : array[1..10] of integer;
    RefCount : integer;
    Data: TSubFile;
    S : string;
    Def : TILFieldDef;
    Tag : string;
    OVal, NVal : string;
    Indx : integer;

  begin
    { First check table refs:}
    RefCount := 0;
    for I := 0 to ATable.Fields.Count-1 do
    begin
      if ATable.Fields.Items[i].LookupType = looTableRef then
      begin
         inc(RefCount);
         RefMap[RefCount] := i;
      end;
    end;

    Data := TmpLib.Files[TmpLib.IndexOf(ATable.FDataName)];
    Data.Position := 0;
    for i := 0 to Data.Count-1 do
    begin
      inc(ProgCounter);
      TMan.Progress(ATable, iloRemoveDeleted, EstmLoops, ProgCounter);
      S := Data.AbsItems[i];

      for j := 1 to RefCount do
      begin
         Def := ATable.Fields.Items[RefMap[j]];
         Tag := 'F' + IntToStr(Def.FieldID);
         OVal := GetValue(Tag, S);
         Oval := FillZeroes(OVal, 7);
         NVal := GetNewref(OVal, Def.LookupTable);
         S := SetValue(Tag, Nval, S);
      end;

      if ATable.IsDetail then
      begin
        Indx := ATable.Fields.IndexOf('SYS_LinkId');
        Tag := 'F' + IntToStr(ATable.Fields.Items[Indx].FieldID);
        OVal := GetValue(Tag, S);
        Oval := FillZeroes(OVal, 7);
        NVal := GetNewref(OVal, ATable.ParentTable);
        S := SetValue(Tag, Nval, S);
      end
      else if ATable is TILTreeViewTable then
      begin
        Indx := ATable.Fields.IndexOf('SYS_LinkId');
        Tag := 'F' + IntToStr(ATable.Fields.Items[Indx].FieldID);
        OVal := GetValue(Tag, S);
        if OVal <> '' then
        begin
          Oval := FillZeroes(OVal, 7);
          NVal := GetNewref(OVal, ATable);
          S := SetValue(Tag, Nval, S);
        end;
      end;
      Data[i] := S;
    end;
  end;

  procedure RestoreTransLog(Filename : string);
  var
    TrLog : TStringList;
    LogItm : TILTransLogItem;
    I : integer;
    MS : TStringStream;
    S, S2, S3 : string;
    RecInd, SysInd : string;
    T, MT : TILTable;
    SysTag : string;
    Indx : integer;
    OldSys : string;
  begin
      TrLog := TMan.LoadTransLog(Filename);
      EstmLoops := TrLog.Count;
      if Tman.Unicode then
        MS := TStringStream.Create('', TEncoding.Utf8)
      else
        MS := TStringStream.Create;
      try
      for i := 0 to TrLog.Count-1 do
      begin
        LogItm := TILTransLogItem(TrLog.Objects[i]);
        T := TMan.TableOf(LogItm.TableName);
        inc(ProgCounter);
        TMan.Progress(T, iloProcessTranslog, EstmLoops, ProgCounter);

        if T = nil then {Will not happen?. May be if table name has changed ...}
          Continue;

        RecInd := GetNewRef(FillZeroes(IntToStr(LogItm.RecordIndex), 7), T);
        if RecInd <> '' then
        begin
          if LogItm.OldData <> '' then
            S2 := SetValue('In', RecInd, LogItm.OldData)
          else
            S2 := '';
          S3 := SetValue('In', RecInd, LogItm.Trans);

          if T.IsDetail then
          begin
            MT := T.MasterSource;
            Indx := T.Fields.IndexOf('SYS_LinkID');
            SysTag := 'F' + IntToStr(T.Fields.Items[Indx].FieldID);
            if S2 <> '' then
            begin
             OldSys := GetValue(SysTag, S2);
             SysInd := GetNewRef(FillZeroes(OldSys, 7), MT);
             S2 := SetValue(SysTag, SysInd, S2);
            end;
            OldSys := GetValue(SysTag, S3);
            SysInd := GetNewRef(FillZeroes(OldSys, 7), MT);
            S3 := SetValue(SysTag, SysInd, S3);
          end;
          S := TMan.EncodeTrans(LogItm.TableName, StrToInt(RecInd), LogItm.FTransType, S2, S3, LogItm.TransID);

          MS.Clear;
          MS.WriteString(S);
          MS.Position := 0;
          TmpTranslog.CopyFrom(MS, MS.Size);
        end;
      end;
      finally
        TrLog.Free;
        MS.Free;
      end;
    end;

  function ComputeLoops : integer;
  var
    i : integer;
  begin
    Result := 0;
    for I := 0 to Tman.TableCount-1 do
      Result := Result + TMan.Tables[i].RecordCount;
     Result := Result * 2;
  end;

  procedure FreeTransLog;
  var
    i : integer;
  begin
    TrLog.Free;
    for I := 0 to High(OldInds) do
      OldInds[i].Indexes.Free;

  end;

  procedure DoImgMain(ATable : TILTable);
  var
    i: integer;
    Cnt: integer;
    Data: TSubFile;
    S: string;
    Offs, OldOffs, Sz, Fn : string;
    Rec : TILDataRecord;

    procedure AddConversion(NewOffs : string);
    var
      NVal : TNewVal;
    begin
      NVal := TNewVal.Create;
      NVal.NV := NewOffs;
      OldInds[0].Indexes.AddObject(FillZeroes(OldOffs, 10), NVal);
    end;

  begin {DoImgMain}
    Data := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(ATable.FDataName)];
    Cnt := Data.Count;
    Data.Position := 0;
    for i := 0 to Cnt - 1 do
    begin
      inc(ProgCounter);
      TMan.Progress(ATable, iloRecreateImages, EstmLoops, ProgCounter);
      S := Data[i];
      Rec := ATable.XMLToRecord(S, True);
      if Rec.DeletedMoved or Rec.DeletedTrans then
      begin
        Rec.Free;
        Continue;
      end;

      Sz := GetValue('ISz', S);
      Offs := GetValue('IOf', S);
      Fn := GetValue('Ifl', S);

      if Rec.ThumbnailSize <= 0 then
      begin
        Rec.Free;
        Continue;
      end;

      if Rec.FImgTableIndex <> -1 then
      begin
        Rec.Free;
        Continue;
      end;
      try
        if Rec.FileName = '' then
        begin
          Rec.Image := ATable.GetThumbnail(Rec.ThumbnailOffset, Rec.ThumbnailSize);
        end
        else
        begin
          if IsUrl(Rec.FileName) then
          begin
            if not CheckUrl(Rec.FileName) then
            begin
              OldOffs := GetValue('Ifl', S);
              S := SetValue('Ifl', '', S);
              S := SetValue('IOf', '0000000000', S);
              S := SetValue('ISz', '0000000000', S);
              Data[i] := S;
              Continue;
            end;
          end
          else if not FileExists(Rec.FileName) then
          begin
            OldOffs := GetValue('Ifl', S);
            S := SetValue('Ifl', '', S);
            S := SetValue('IOf', '0000000000', S);
            S := SetValue('ISz', '0000000000', S);
            Data[i] := S;
            Continue;
          end;
          Rec.LoadImage(Rec.FileName);
        end;
        SaveImgToTemp(Rec.Image, Rec.FThumbnailOffset, Rec.FThumbnailSize);
        OldOffs := GetValue('IOf', S);
        S := SetValue('Ifl', Fn, S);
        S := SetValue('IOf', FillZeroes(IntToStr(Rec.FThumbnailOffset), 10), S);
        S := SetValue('ISz', FillZeroes(IntToStr(Rec.FThumbnailSize), 10), S);
        AddConversion(FillZeroes(IntToStr(Rec.FThumbnailOffset), 10));
        Data[i] := S;
      finally
        Rec.Free;
      end;
    end;
  end;

  procedure DoTextLobs(ATable : TILTable);
  var
    i, j: integer;
    Cnt: integer;
    Data: TSubFile;
    S, Txt: string;
    Fn : string;
    N : integer;
    M : TMemoryStream;
    Oval, NVal : string;
    Rec : TILDatarecord;

    procedure AddConversion(NewOffs, OldOffs : string);
    var
      NVal : TNewVal;
    begin
      NVal := TNewVal.Create;
      NVal.NV := NewOffs;
      OldInds[1].Indexes.AddObject(FillZeroes(OldOffs, 10), NVal);
    end;

  begin { Do text lobs}
    Data := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(ATable.FDataName)];
    Cnt := Data.Count;
    Data.Position := 0;
    for i := 0 to Cnt - 1 do
    begin
      inc(ProgCounter);
      TMan.Progress(ATable, iloRecreateTextLobs, EstmLoops, ProgCounter);
      S := Data[i];
      Rec := ATable.XMLToRecord(S, True);

      if Rec.DeletedMoved or Rec.DeletedTrans then
      begin
        Rec.Free;
        Continue;
      end;

      try
       Oval := GetValue('Ifl', S);
       if TryStrToInt(Oval, N) then
       begin
          Txt := Rec.FileName;
          NVal := IntToStr(Rec.SaveFileResourceName(-1, Txt, TmpTextLobs));
          AddConversion(NVal, Oval);
          S := SetValue('Ifl', Nval, S);
       end;

       for j := 0 to Rec.Count-1 do
       begin
        if (Rec[j].FieldDef.LookupType = looParentField) then
          Continue;
        if Rec[j].FieldDef.DataType = dtLongText then
        begin
          M := Rec[j].AsStream;
          if M = nil then
            Continue
          else
          try
            Fn := 'F' + IntToStr(Rec[j].FieldDef.FieldID);
            Oval := GetValue(Fn, S);
            N := Rec[j].SaveLongTextStream(M, 1, TmpTextLobs);
            NVal := IntToStr(N);
            AddConversion(Nval, Oval);
            S := SetValue(Fn, Nval, S);
          finally
           M.Free;
          end;
        end
        else if Rec[j].FieldDef.DataType = dtResource then
        begin
          if Rec[j].FresFilePos > -1 then
          begin
           Fn := 'F' + IntToStr(Rec[j].FieldDef.FieldID);
           Oval := GetValue(Fn, S);
            if Rec[j].FValues.Count = 0 then
             S := SetValue(Fn, '', S)
            else
            begin
              Txt := Rec[j].FValues[0];
              NVal := IntToStr(Rec.SaveFileResourceName(-1, Txt, TmpTextLobs));
              S := SetValue(Fn, NVal, S);
              AddConversion(NVal, Oval);
            end;
          end;
        end;
       end;

       Data[i] := S;
      finally
        Rec.Free;
      end;
    end;
  end;

   procedure UpdateTransLog;
   var
     Tr : string;
     sl : TStringList;
     i, j : integer;
     L : TILTransLogItem;
     Vl : string;
     T : TILTable;
     Tg : string;

     function SetNewVal(ValType : integer; Tg, OldVal, Trans : string) : string;
     var
       NVl : TNewVal;
       Indx : integer;
     begin
       Result := Trans;
       if OldInds[ValType].Indexes.Find(FillZeroes(OldVal, 10), Indx) then
       begin
         NVl := OldInds[ValType].Indexes.Objects[Indx] as TNewVal;
         Result := SetValue(Tg, NVl.NV, Trans);
       end;
     end;

   begin
     if not TransHistory then
       Exit;
     TrLog := LoadTransLog(FTranslogName);
     ProgCounter := 0;
     EstmLoops := TrLog.Count;
     FProgressTicker := GetTickCount;
     sl := TStringList.Create;
     try
     for I := 0 to TrLog.Count-1 do
     begin
       L := TrLog.Objects[i] as TILTransLogItem;
       Vl := GetValue(XImgOffset, L.FTrans);
       if Vl <> '' then
       begin
        if GetValue(XImgSize, L.FTrans) <> '0000000000'  then
         L.Ftrans := SetNewVal(0, XImgOffset, Vl, L.FTrans);
       end;
       Vl := GetValue(XImgFile, L.FTrans);
       if Vl <> '' then
        L.Ftrans := SetNewVal(1, XImgFile, Vl, L.FTrans);
       if L.OldData <> '' then
       begin
        Vl := GetValue(XImgOffset, L.FOldData);
        if Vl <> '' then
        begin
         if GetValue(XImgSize, L.FOldData) <> '0000000000'  then
          L.FOldData := SetNewVal(0, XImgOffset, Vl, L.FOldData);
        end;
        Vl := GetValue(XImgFile, L.FOldData);
        if Vl <> '' then
         L.FOldData := SetNewVal(1, XImgFile, Vl, L.FOldData);
       end;

       T := TableOf(L.TableName);
       inc(ProgCounter);
       Progress(T, iloProcessTranslog, EstmLoops, ProgCounter);

       for j := 0 to T.Fields.Count-1 do
       begin
         if T.Fields.Items[j].DataType in [dtResource, dtLongText] then
         begin
           Tg := 'F' + IntToStr(T.Fields.Items[j].FieldID);
           Vl := GetValue(Tg, L.FTrans);
           if Vl <> '' then
             L.FTrans := SetNewVal(1, Tg, Vl, L.FTrans);
           if L.OldData <> '' then
           begin
            Vl := GetValue(Tg, L.FOldData);
            if Vl <> '' then
              L.FOldData := SetNewVal(1, Tg, Vl, L.FOldData);
            Vl := GetValue(Tg, L.FOldData);
            if Vl <> '' then
              L.FOldData := SetNewVal(1, Tg, Vl, L.FOldData);
           end;
         end;
       end;

       Tr := EncodeTrans(L.TableName, L.RecordIndex, L.TransType, L.FOldData, L.FTrans, L.FTransID);
       System.Delete(Tr, Length(Tr)-1, 2);
       sl.Add(Tr);
     end;
     FTransLogFile.Size := 0;
     sl.SaveToStream(FTransLogFile);
     finally
       sl.Free;
     end;
   end;


begin {CleanUp}
    Result := 0;
    { Errors occuring from here has hardcoded messages }
    if csDesigning in ComponentState then
      HandleILException(E_NotDesignTime);
    if not Connected then
      HandleILException(E_NoConnection); { The indexes must be loaded}
    begin
      { Make tempmLib }
      Connected := false;
      if not CopyFile(pchar(FLibPath), pchar(_FTmpLibPath), false) then
      begin
        HandleILException(0,'Could not create workfile (' +
          GetLastErrorText + ')');
      end;

      { Make backup copies }
      MakeBackups(0);
      ConnectSilent;
      DelCnt := 0;
      TmpLib := TFileContainer.Create(_FTmpLibPath, 0, false);
      TmpLib.Unicode := Unicode;
      if TransHistory then
        TmpTransLog := TFileStream.Create(_FTmpTransLogName, fmCreate);
      TmpLib.Unicode := Unicode;
      try
        try
          EstmLoops := ComputeLoops;
          FProgressTicker := GetTickCount;
          ProgCounter := 0;

          DoRemove(RootTable, DelCnt);
          for I := 0 to FreeTableCount-1 do
             DoRemove(FreeTables[i], DelCnt);

          if DelCnt > 0 then
          begin
           for I := 0 to TableCount-1 do
            RestoreRefs(Tables[i]);
           if TransHistory then
           begin
             Progress(nil, iloChangeOperation, 0, 0);
             FProgressTicker := GetTickCount;
             ProgCounter := 0;
             RestoreTranslog(FTransLogName);
           end;
          end;
        except
          on E: Exception do
          begin
            RestoreFromBackups(0);
            RemoveTempFiles;
            HandleILException(0,'An error occured (' + E.Message +'). Changes were rolled back');
          end;
        end;
      finally
        TmpLib.Free;
        if TransHistory then
          TmpTransLog.Free;
        for I := 0 to High(OldInds) do
        begin
          OldInds[i].Indexes.Free;
        end;
        Finalize(OldInds);
      end;
      Connected := false;
      if DelCnt > 0 then
      begin
        { Make tmplib the real lib }
        try
          if not System.SysUtils.DeleteFile(FLibPath) then
            HandleILException(0,'Error deleteing old data file');
          if not System.SysUtils.RenameFile(_FTmpLibPath, FLibPath) then
            HandleILException(0,'Error renaming new data file');
          if Tman.TransHistory then
          begin
           if not System.SysUtils.DeleteFile(FTranslogName) then
             HandleILException(0,'Error deleteing old trans log file');
           if not System.SysUtils.RenameFile(_FTmpTransLogName, FTranslogName) then
             HandleILException(0,'Error renaming new trans log file');
          end;

          ConnectSilent;
          Progress(nil, iloChangeOperation, 0, 0);
          RebuildAllIndexes;
        except
          RestoreFromBackups(0);
          RemoveTempFiles;
          raise;
        end;
      end
      else
        ConnectSilent;
    end;

    {State is now that deleted rows are removed from all tables. Syslinks are corrected,
     according to new parent record indexes and transaction items refering to removed records
     are excluded.
     Remove unused images:}

    begin
      if TransHistory then
      { Coversion }
      begin
       SetLength(OldInds, 2);
       OldInds[0].Indexes := TStringList.Create;
       OldInds[0].Indexes.Duplicates := dupIgnore;
       OldInds[0].Indexes.Sorted := True;
       OldInds[0].Indexes.OwnsObjects := True;
       OldInds[1].Indexes := TStringList.Create;
       OldInds[1].Indexes.Sorted := True;
       OldInds[1].Indexes.Duplicates := dupIgnore;
       OldInds[1].Indexes.OwnsObjects := True;
      {Dim 0: Image offsets, dim 1 textlob offsets}
      end;

      if FileExists(_FTmpThumbName) then
      begin
        TmpImgs := TFileStream.Create(_FTmpThumbName, fmOpenReadWrite);
        TmpImgs.Size := 0;
      end
      else
        TmpImgs := TFileStream.Create(_FTmpThumbName, fmCreate);
      try
       try
        EstmLoops := 0;
        ProgCounter := 0;
        for I := 0 to TableCount-1 do
        begin
          EstmLoops := EstmLoops + Tables[i].RecordCount;
        end;
        FProgressTicker := GetTickCount;
        Progress(nil, iloChangeOperation, 0, 0);
        for I := 0 to TableCount-1 do
        begin
          DoImgMain(Tables[i]);
        end;
        FThumbs.Size := 0;
        TmpImgs.Position := 0;
        FThumbs.CopyFrom(TmpImgs, TmpImgs.Size);
       except
        on E: Exception do
        begin
          RestoreFromBackups(1);
          FreeTransLog;
          RemoveTempFiles;
          HandleILException(0,'Error creating image file (' + E.Message +
            '). Work aborted');
        end;
      end;
      finally
        TmpImgs.Free;
      end;
    end;

    { Remove unused text lobs }
    begin
      Progress(nil, iloChangeOperation, 0, 0);
      if FileExists(_FTmpTextLobsName) then
      begin
        TmpTextLobs := TFileStream.Create(_FTmpTextLobsName, fmCreate);
        TmpTextLobs.Size := 0;
      end
      else
        TmpTextLobs := TFileStream.Create(_FTmpTextLobsName, fmCreate);
      EstmLoops := 0;
      ProgCounter := 0;
      for I := 0 to TableCount-1 do
      begin
        EstmLoops := EstmLoops + Tables[i].RecordCount;
      end;
      try
        try
          FProgressTicker := GetTickCount;
          for I := 0 to TableCount-1 do
          begin
            DoTextLobs(Tables[i]);
          end;
          if TransHistory then
          begin
             Progress(nil, iloChangeOperation, 0, 0);
          end;
          FTextLobs.Size := 0;
          TmpTextLobs.Position := 0;
          FTextLobs.CopyFrom(TmpTextLobs, TmpTextLobs.Size);
          if TransHistory then
          begin
            UpdateTransLog;
            FreeTransLog;
          end;
        except on E: Exception do
         begin
          RestoreFromBackups(1);
          FreeTransLog;
          RemoveTempFiles;
          HandleILException(0,'Error creating text lobs file (' + E.Message +
            '). Work aborted');
         end;
        end;
      finally
        TmpTextLobs.Free;
        Progress(nil, iloFinished, 0, 0);
      end;
    end;
    RemoveWorkFiles;
end;

procedure TTableManager.RestoreFromBackups(Option : integer);
begin
      Connected := false;
      System.SysUtils.DeleteFile(FLibPath);
      System.SysUtils.RenameFile(_FLibPath, FLibPath);
      System.SysUtils.DeleteFile(FTableCfgName);
      System.SysUtils.RenameFile(_FTableCfgName, FTableCfgName);
      System.SysUtils.DeleteFile(FTextLobsName);
      System.SysUtils.RenameFile(_FTextLobsName, FTextLobsName);
      System.SysUtils.DeleteFile(FTransLogName);
      System.SysUtils.RenameFile(_FTransLogName, FTransLogName);
      ConnectSilent;
end;

procedure TTableManager.MakeBackups(Option : integer);
begin
      { Make backup copies }
    Connected := false;
    try
      if FileExists(_FLibPath) then
        System.SysUtils.DeleteFile(_FLibPath);
      if not CopyFile(pchar(FLibPath), pchar(_FLibPath), false) then
      begin
        Comment1 := 'data file';
        Comment2 := GetLastErrorText;
        HandleILException(E_NoBackup, '');
      end;

      if FileExists(_FTranslogName) then
       System.SysUtils.DeleteFile(_FTransLogName);
      if not CopyFile(pchar(FTranslogName), pchar(_FTransLogName), false) then
      begin
        Comment1 := 'trans log file';
        Comment2 := GetLastErrorText;
        HandleILException(E_NoBackup, '');
      end;

      if FileExists(_FTableCfgName) then
        System.SysUtils.DeleteFile(_FTableCfgName);
      if not CopyFile(pchar(FTableCfgName), pchar(_FTableCfgName), false) then
      begin
        Comment1 := 'configuration file';
        Comment2 := GetLastErrorText;
        HandleILException(E_NoBackup, '');
      end;

      if FileExists(_FTextLobsName) then
        System.SysUtils.DeleteFile(_FTextLobsName);
      if not CopyFile(pchar(FTextLobsName), pchar(_FTextLobsName), false) then
      begin
        Comment1 := 'text objects file';
        Comment2 := GetLastErrorText;
        HandleILException(E_NoBackup, '');
      end;
    finally
        ConnectSilent;
    end;
end;

procedure TTableManager.ClearTranslog;
begin
  if (FCfgFile <> nil) then
  begin
    FCfgFile.WriteInteger('Transaction', 'TransID', 0);
    FCfgFile.WriteInteger('Transaction', 'Completion', TrCommitted);
    FCfgFile.WriteInteger('Transaction', 'Count', 0);
    FCfgFile.WriteInteger('Transaction', 'Active', 0);
  end;
  if Assigned(FTranslogFile) then
  begin
    FTransLogFile.Size := 0;
  end;
  FTranslogBuffer.Clear;
  FTranslogIndex.Clear;
end;

procedure TTableManager.SaveTranslog;
begin
  FTransLogFile.Position := FTransLogFile.Size;
  FTransLogBuffer.Position := 0;
  FTranslogFile.CopyFrom(FTransLogBuffer, FTranslogBuffer.Size);
  FTranslogBuffer.Clear;
  FTranslogIndex.Clear;
end;

procedure TTableManager.SetTransState(State : integer);
var
  i, j, k : integer;
  M : TList;
  Tbl : TILTable;
  MH : TILMessageHandler;
  Ind : integer;
begin
    if FTransactionActive and not (State in [trCommittedActive, trPendingActive, trRolledBack]) then
      Exit;
    M := nil;
    if (State in [TrCommitted, TrCommittedActive]) then
    begin
       if FTransLogBuffer.Size > 0 then
       begin
        M := TList.Create;
        RunTransactions(M);
        TMan.FAffectedTables.Clear;
        for I := 0 to M.Count-1 do
          TMan.FAffectedTables.Add(M[i]);
       end;
    end;

    {Update states}

    if (State = TrPending) { Internal trans}
    and (FCfgFile.ReadInteger('Transaction', 'Completion', TrCommitted) = TrPending) then
    { Already pending }
      Exit;
    if State in [TrPending, TrPendingActive] then
    begin
       if State = TrPendingActive then {Call from BeginTrans }
       begin
         FCfgFile.WriteInteger('Transaction', 'Active', 1);
         FTransID := FCfgFile.ReadInteger('Transaction', 'TransID', 0) + 1;
         FCfgFile.WriteInteger('Transaction', 'TransID', FTransID);
         FTransactionActive := True;
       end;
       if not FTransactionActive then
       { Dont start a new transaction when a user transaction is active}
       begin
        FTransID := FCfgFile.ReadInteger('Transaction', 'TransID', 0) + 1;
        FCfgFile.WriteInteger('Transaction', 'TransID', FTransID);
       end;
    end;
    if FTransactionActive and (State in [TrRolledback, TrCommittedActive]) then
    begin
        FCfgFile.WriteInteger('Transaction', 'Active', 0);
        FTransactionActive := false;
    end;
    FCfgFile.WriteInteger('Transaction', 'Completion', State);
      { 0 incompleted, 1 suspended, 2 completed 3 rolled back}

     if (State in [TrCommitted, TrCommittedActive,  trRolledback]) then
     begin
       FTranslogBuffer.Clear;
       FTranslogIndex.Clear;
     end;

     FTransOwner := nil;

     if (State in [TrCommitted, TRCommittedActive]) then
      SaveIndexes;

     try
     { Check if there are mem tables based on this table}
     if (State in [TrCommitted, TRCommittedActive]) and (M <> nil) then
     for i := 0 to TMan.FreeTableCount-1 do
       if TMan.FreeTables[i] is TILMemTable then
       begin
           if TILMemTable(Tman.FreeTables[i]).BaseTables.Count = 0 then
           begin
             TMan.InvalidateMemTables(TILMemTable(Tman.FreeTables[i]));
             Continue;
           end;
           for j := 0 to TILMemTable(Tman.FreeTables[i]).BaseTables.Count-1 do
           begin
             for k := 0 to M.Count-1 do
             if SameText(TILMemTable(Tman.FreeTables[i]).BaseTables[j], TILTable(M[k]).Tablename) then
             begin
               TMan.InvalidateMemTables(TILMemTable(Tman.FreeTables[i]));
               Break;
             end;
           end;
       end;
     finally
       if M <> nil then
         M.Free;
     end;

     if (State in [TrCommitted, TRCommittedActive]) then
     begin
      for I := 0 to TMan.TableCount-1 do
      begin
        TBL := TMan.Tables[i];
        Ind := -1;
        MH := GetMessageHandler(TBL, Ind);
        while MH <> nil do
        begin
         if MH.SaveCount > -1 then
          MH.AddCount(dboSave,MH.SaveCount, False);
         MH := GetMessageHandler(TBL, Ind);
        end;
      end;
     end;
end;

function TTableManager.GetTransState(var TrID : integer) : integer;
begin
      Result := FCfgFile.ReadInteger('Transaction', 'Completion', TrCommitted);
      TrID := FCfgFile.ReadInteger('Transaction', 'TransID', 0);
end;

function TTableManager.EncodeTrans(TblName : string; RecIndex : integer; TrType :
 TDBOperation; OldData, TrData : string; TransID : integer) : string;
var
  TL : TILTranslogItem;
begin
  Result := '';
  if FDisableTransLog then
    Exit;
  TL := TILTranslogItem.Create;
  try
   TL.Trans := TrData;
   TL.OldData := OldData;
   TL.TableName := TblName;
   TL.TransType := TrType;
   TL.RecordIndex := RecIndex;
   TL.TransID := TransID;
   Result := TL.TableName + '|' + FillZeroes(IntToStr(TL.RecordIndex) ,7);
   TL.Trans := StringReplace(TL.Trans, #$D#$A, '~', [rfReplaceAll]); {~~ Text line breaks }
   TL.OldData := StringReplace(TL.OldData, #$D#$A, '~~', [rfReplaceAll]);
   Result := Result + '|' + IntToStr(Ord(TL.TransType)) + '|' + IntToStr(TL.FTransID) + '|' + TL.Trans + '##' +  TL.OldData + #$D#$A;
  finally
    TL.Free;
  end;
  { Linebreaks delimt trans. ## Delimits old and new data }
end;

procedure TTableManager.AddToTransLog(TblName : string; RecIndex : integer;
 TrType : TDBOperation; OldData, TrData : string; DataRec : TILDataRecord);
var
  S : string;
  MS : TStringStream;
  TP : TTransPos;
  T : TIlTable;
  I : integer;
begin
  if FDisableTransLog then
    Exit;
  T := TMan.TableOf(TblName);
  if TMan.FTransOwner = nil then
    TMan.FTransOwner := T;
  S := EncodeTrans(TblName, RecIndex, TrType, OldData, TrData, FTransID);
  if Unicode then
    MS := TStringStream.Create('', TEncoding.Utf8)
  else
    MS := TStringStream.Create;
  try
   MS.WriteString(S);
   MS.Position := 0;
   FTransLogBuffer.Position := FTranslogBuffer.Size;
   TP := TTransPos.Create;
   TP.Position := FTranslogBuffer.Size;
   TP.Size := Ms.Size;
   TP.Operation := TrType;
   TP.TableName := TblName;
   FTransLogBuffer.CopyFrom(MS, MS.Size);
   S := TblName + FillZeroes(IntToStr(RecIndex), 7);
   FTranslogIndex.AddObject(S, TP);
  finally
    MS.Free;
  end;
  if T.HasHandler and Assigned(T.Datahandler.FOnTrigger) then
  begin
    for I := 0 to High(Tman.FUpdateStack) do
    begin
      if (Tman.FUpdateStack[i].Table = T) and (Tman.FUpdateStack[i].UpdateID <> TMan.FUpdateIDCounter) then
      begin
        if TMan.FUpdater = nil then
          TMan.RollbackTrans;
        HandleILException(E_CircularTrigger);
      end;

    end;
    T.Datahandler.FonTrigger(T, TrType, DataRec);
  end;

end;

procedure TTableManager.AddToUpdateStack(ATable : TILTable; ID : integer);
var
  SI : TUpdateStackItem;
begin
   SI.UpdateID := ID;
   SI.Table := ATable;
   SetLength(FUpdateStack, Length(FUpdateStack) + 1);
   FUpdateStack[High(FUpdateStack)] := SI;
end;

procedure TTableManager.RemoveFromUpdateStack;
begin
   SetLength(FUpdateStack, Length(FUpdateStack) - 1);
end;

function TTableManager.LoadTransLog(FileName : string = '') : TStringList;
var
  sl : TStringList;
  i : integer;
  s, s2, s3, s4, s5 : string;
  P : integer;
  TL : TILTranslogItem;
begin
  sl := TStringList.Create;
  if SameText(FileName, FTranslogName) then
    FTransLogFile.Free;
  try
    Result := TStringList.Create(True);
    if FileExists(FTransLogName) then
    begin
      if Unicode then
       sl.DefaultEncoding := TEncoding.Utf8;
      if FileName <> '' then
        sl.LoadFromFile(FileName)
      else
      begin
        FTranslogBuffer.Position := 0;
        sl.LoadFromStream(FTranslogBuffer);
      end;
    end;

    if sl.Count = 0 then
      Exit;

    { Create the mem log list }

    for I := 0 to sl.Count-1 do
    begin
      s := sl[i];
      P := Pos('|', S);
      s2 := Copy(S, 1, P-1); {TableName}
      System.Delete(S, 1, P);
      P := Pos('|', S);
      s3 := Copy(S, 1, P-1); {RecordIndex}
      System.Delete(S, 1, P);
      P := Pos('|', S);
      s4 := Copy(S, 1, P-1); {Transtype}
      System.Delete(S, 1, P);
      P := Pos('|', S);
      s5 := Copy(S, 1, P-1); {TransID}
      System.Delete(S, 1, P); { S is now the xml record data, if any }
      S := StringReplace(S, '~~', #$D#$A, [rfReplaceAll]);

      TL := TILTranslogItem.Create;
      TL.FTableName := s2;
      TL.FRecordIndex := StrToInt(s3);
      TL.FTransType := TDBOperation(StrToInt(s4));
      P := Pos('##', S);
      TL.FTrans := Copy(s , 1, P-1);
      Inc(P, 2);
      TL.OldData := Copy(s, P, Length(S));
      TL.FTransID := StrToInt(s5);

      Result.AddObject(S2+S3, TL)
    end;

  finally
   sl.Free;
   if SameText(FileName, FTranslogName) then
     FTransLogFile := TFileStream.Create(FTranslogName, fmOpenReadWrite);
  end;
end;

procedure TTableManager.BeginTrans;
var
 ID : integer;
begin
    if not Connected then
      HandleILException(E_NoConnection);
    if (GetTransState(ID) = TrPending)
    or FTransactionActive then
      HandleILException(E_TransActive);
    FTransactionActive := True;
    SetTransState(TrPendingActive);
end;

procedure TTableManager.CommitTrans;
begin
    if not FTransactionActive then
      Exit;
    SetTransState(TrCommittedActive);
    FTransactionActive := false;
end;

function TTableManager.RollbackTrans : integer;
var
  i: Integer;
  Tbl : TILTable;
  R : TILDataRecord;
  Log : TstringList;
  TR : TILTranslogItem;
  S : string;
  Cnt : integer;

begin
    Result := 0;
    FTransOwner := nil;
    if not FTransactionActive then
      Exit;
    FTransactionActive := false;

    if not FileExists(FRollbackName) then
    { Som kind of failure, nothing was written }
    begin
      if FTransLogBuffer.Size > 0 then
        DoLoadIndexes;
      SetTransState(TRRolledBack);
      Exit;
    end;
    { Failure when writing records }
    Log := LoadTransLog(FRollbackName);
    try
     Cnt := FCfgFile.ReadInteger('Transaction', 'WriteCount', 0);
     for i := 0 to Cnt-1 do
     begin
       TR := Log.Objects[i] as TILTranslogItem;
       TBL := TableOf(TR.TableName);
       if TR.FTransType = dboUpdate then
         TBL.FSF_Data[TR.RecordIndex] := TR.OldData
       else if TR.FTransType = dboDelete then
       begin
         R := TBL.RecordsX[TR.RecordIndex];
         try
          R.FState := 'NO';
          S := TBL.RecordToXML(R, False);
          TBL.FSF_Data[TR.RecordIndex] := S;
         finally
          R.Free;
         end;
       end
       else
       begin
         R := TBL.RecordsX[TR.RecordIndex];
         try
          R.DeletedTrans := True;
          S := TBL.RecordToXML(R, False);
          TBL.FSF_Data[TR.RecordIndex] := S;
         finally
          R.Free;
         end;
       end;
     end;
     System.SysUtils.Deletefile(FRollbackName);
     SetTransState(TRRolledBack);
     DoLoadIndexes;
    finally
      Log.Free;
    end;
end;

procedure TTableManager.ClearDatasets;
var
  i : integer;
begin
   if not Connected then
     Exit;
   if RootTable = nil then
     Exit;
   if not CheckUnsaved(RootTable) then
     Exit;
   for I := TableNodeCount-1 downto 0 do
     TreeTables[i].Dataset.Clear;
end;

procedure TTableManager.RenameDatabase(const OldName, NewName : string);
var
  Reg : TRegistry;
begin
    if SameText(OldName, NewName) then
      Exit;
    if not NameCheck(NewName) then
      HandleIlException(E_InvalidChar);
    Connected := false;
    if not RenameFile(Location + '\' + OldName + '.dat', Location + '\' + NewName + '.dat') then
      HandleILException(0,'Could not rename files (' + GetLastErrorText + ')' );
    RenameFile(Location + '\' + OldName + '_Imgs.dat', Location + '\' + NewName + '_Imgs.dat');
    RenameFile(Location + '\' + OldName + '_TextLobs.dat', Location + '\' + NewName + '_TextLobs.dat');
    RenameFile(Location + '\' + OldName + '.dat', Location + '\' + NewName + '.dat');
    RenameFile(Location + '\' + OldName + '.cfg', Location + '\' + NewName + '.cfg');
    RenameFile(Location + '\' + OldName + '_Meta.XML', Location + '\' + NewName + '_Meta.XML');
    RenameFile(Location + '\' + OldName + '_Trans.LOG', Location + '\' + NewName + '_Trans.LOG');
    Reg := TRegistry.Create(KEY_WRITE);
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      Reg.MoveKey('\Software\ImageLibrary\' + OldName, '\Software\ImageLibrary\' + NewName, True);
    finally
      Reg.Free;
    end;
    FDatabase := NewNAME;
    SetNames;
end;

procedure TTableManager.MoveDatabase(const OldDir, NewDir : string);
var
  Reg : TRegistry;
begin
    if SameText(OldDir, NewDir) then
      Exit;
    Connected := false;
    if not RenameFile(OldDir + '\' + Database + '.dat', NewDir + '\' + Database + '.dat') then
      HandleILException(0,'Could not rename files (' + GetLastErrorText + ')' );
    RenameFile(OldDir + '\' + Database + '_Imgs.dat', NewDir + '\' + Database + '_Imgs.dat');
    RenameFile(OldDir + '\' + Database + '_TextLobs.dat', NewDir + '\' + Database + '_TextLobs.dat');
    RenameFile(OldDir + '\' + Database + '.dat', NewDir + '\' + Database + '.dat');
    RenameFile(OldDir + '\' + Database + '.cfg', NewDir + '\' + Database + '.cfg');
    RenameFile(OldDir + '\' + Database + '_Meta.XML', NewDir + '\' + Database + '_Meta.XML');
    RenameFile(OldDir + '\' + Database + '_Trans.LOG', NewDir + '\' + Database + '_Trans.LOG');
    Reg := TRegistry.Create(KEY_WRITE);
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      Reg.OpenKey('\Software\ImageLibrary\' + Database, false);
      Reg.WriteString('Location', NewDir);
    finally
      Reg.Free;
    end;
    Location := NewDir;
    SetNames;
end;

procedure TTableManager.RecreateDatbase;
begin
   if Connected then
     Connected := false;
   SetNames;
   if FileExists(FTableCfgName) then
     System.SysUtils.DeleteFile(FTableCfgName);
   if FileExists(FLibPath) then
     System.SysUtils.DeleteFile(FLibPath);
   if FileExists(FThumbName) then
     System.SysUtils.DeleteFile(FThumbName);
   if FileExists(FMetaName) then
     System.SysUtils.DeleteFile(FMetaName);
   if FileExists('bak_' + FLibPath) then
     System.SysUtils.DeleteFile('bak_' + FLibPath);
   if FileExists('bak_' + FTableCfgName) then
     System.SysUtils.DeleteFile('bak_' + FTableCfgName);
   CreateDatabase;
end;

{procedure TTableManager.InitDatabase;
var
  F : string;
begin
     if Trim(Database) = '' then
       HandleILException(E_MissingDBName);
     if not DirectoryExists(Location) then
     begin
       Comment1 := Location;
       HandleILException(E_InvalidDir);
     end;
     F := Location + '\' + Database + '.dat';
     Comment1 := F;
     if FileExists(F) then
     begin
       Comment1 := '';
       HandleILException(E_FileExists);
     end;
     SetNames;
     CreateFiles;
end;}

procedure TTableManager.CreateTableFromMeta(const TableName : string; const Metafile : string);
  var
    i: integer;
    XDoc : TXMLDocument;
    ANode, Meta, TblName : iXMLNode;
    TRep : TILReplicator;
    Tbl : TILTable;
  begin
    XDoc := TXMLDocument.Create(Self);
    try
    XDoc.Active := True;
    XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
    XDoc.LoadFromFile(Metafile);
    ANode := XDoc.DocumentElement;
    Meta := ANode.ChildNodes['Meta'];

    TRep := TILReplicator.Create(Metafile, Self, True);
    try
      TRep.Replicate(TableName);
    finally
      TRep.Free;
    end;

    try
      begin
        for i := 0 to Meta.ChildNodes.Count - 1 do
        begin
          if Meta.ChildNodes[i].NodeName = 'Table' then
          begin
           TblName := Meta.ChildNodes[i].ChildNodes.FindNode('TableName');
           Tbl := TableOf(TblName.Text);
           if SameText(Tbl.TableName, TableName) then
           begin
             OpenFiles;
             try
              DoCreateTable(Tbl);
             finally
               CloseFiles;
             end;
             Break;
           end;
          end;
        end;
      end;
    except
      raise;
    end;
    finally
      XDoc.Free;
    end;
  end;


procedure TTableManager.CreateDBFromMeta(const Metafile : string);
  var
    F: string;
    i: integer;
    Lc : TIniFile;
    T : string;
    L : string;
    DBName : string;
    Loc : string;
    XDoc : TXMLDocument;
    DBThumbs, DBLt : TFilestream;
    ANode, Meta, TblName : iXMLNode;
    TRep : TILReplicator;
    Tbl : TILTable;
  begin
    XDoc := TXMLDocument.Create(Self);
    try
    XDoc.Active := True;
    XDoc.DocumentElement := XDoc.CreateNode('Ilib', ntElement, '');
    XDoc.LoadFromFile(Metafile);
    ANode := XDoc.DocumentElement;
    Meta := ANode.ChildNodes['Meta'];
    DBName := Meta.ChildNodes['DBName'].Text;

    Loc := ExtractFileDir(Metafile);
    F := Loc + '\' + DBName + '.dat';
    T := Loc + '\' + DBName + '_Imgs.dat';
    L := Loc + '\' + DBName + '_TextLobs.dat';
    if FileExists(F) then
      HandleILException(E_FileExists);
    DBThumbs := TFileStream.Create(T, fmCreate);
    DBLt := TFileStream.Create(L, fmCreate);
    FLibContainer := TFileContainer.Create(F, (8 * 30) , False);
    FLibContainer.Unicode := Unicode;

    TRep := TILReplicator.Create(Metafile, Self, True);
    try
      TRep.Replicate;
    finally
      TRep.Free;
    end;

   { 8 files per table. MaxLib files = 30 }
    try
      begin
        for i := 0 to Meta.ChildNodes.Count - 1 do
        begin
          if Meta.ChildNodes[i].NodeName = 'Table' then
          begin
           TblName := Meta.ChildNodes[i].ChildNodes.FindNode('TableName');
           Tbl := TableOf(TblName.Text);
           DoCreateTable(Tbl);
           Break;
          end;
        end;
      end;
      DBThumbs.Free;
      DBLt.Free;
      FLibContainer.Free;
      FLibContainer := nil;

      Lc := TIniFile.Create(Loc + '\' + DBName + '.cfg');
      try
        Lc.WriteString('Loacation','Location', Loc);
      finally
        Lc.Free;
      end;
    except
      raise;
    end;
    finally
      XDoc.Free;
    end;
  end;


procedure TTableManager.CreateDatabase;

  { Creates the db tree structure }
  var
    F: string;
    i, j: integer;
  begin
    Comment1 := '';
    if Trim(Database) = '' then
       HandleILException(E_MissingDBName);
    if not Assigned(RootTable) then
      HandleILException(E_MissingRootTable);
    if (RootTable.TableName = '') or (Location = '') then
      HandleILException(E_NoTableNameLoc);
    if not DirectoryExists(Location) then
    begin
      Comment1 := Location;
      HandleILException(E_InvalidDir);
    end;
    F := Location + '\' + Database + '.dat';
    Comment1 := F;
    if FileExists(F) then
      HandleILException(E_FileExists);

    { Check table names }
    if RootTable.TableName = '' then
      HandleILException(E_NoTableNameLoc);
    for i := 0 to RootTable.AllChildCount - 1 do
    begin
        if (RootTable.AllChildren[i].TableName = '') then
         HandleILException(E_NoTableNameLoc);
    end;
    if FSharedImages <> nil then
     if (FSharedImages.TableName = '') then
      HandleILException(E_NoTableNameLoc);
    RegTables.Clear;

    for I := 0 to High(FTreeTables) do
      begin
         for j  := 0 to FTreeTables[i].Fields.Count-1 do
         begin
            if FTreeTables[i].Fields.Items[j].LookupTable <> nil then
            begin
               if FTreeTables[i].Fields.Items[j].LookupTable.TableName = '' then
                 HandleILException(E_NoTableNameLoc);
            end;
         end;
      end;

    SetNames;
    CreateFiles;
    try
      begin
        DoCreateTable(Root.Table);
        for i := 0 to RootTable.AllChildCount - 1 do
        begin
          DoCreateTable(RootTable.AllChildren[i]);
        end;
      end;

      if FSharedImages <> nil then
      begin
        DoCreateTable(FSharedImages);
        RegisterFreeTable(FSharedImages);
      end;
      for I := 0 to High(FTreeTables) do
      begin
         for j  := 0 to FTreeTables[i].Fields.Count-1 do
         begin
            if FTreeTables[i].Fields.Items[j].LookupTable <> nil then
            begin
               if FLibContainer.IndexOf(FTreeTables[i].Fields.Items[j].LookupTable.TableName) <> -1 then
                  Continue;
               DoCreateTable(FTreeTables[i].Fields.Items[j].LookupTable);
               RegisterFreeTable(FTreeTables[i].Fields.Items[j].LookupTable);
            end;
         end;
      end;
      FindFreeTables;
      for I := 0 to High(FFreeTables) do
      begin
          if FLibContainer.IndexOf(FFreeTables[i].TableName) <> -1 then
            Continue;
          RegisterFreeTable(FFreeTables[i]);
          DoCreateTable(FFreeTables[i]);
      end;

      CloseFiles;
      FCreatingTable := True;
      { To supress table validating which will fail because of incomplete meta data }
      Connected := True;
      FCreatingTable := False;

      ShowMessage('Database ' + Database + ' successfully created. ' +
      IntToStr(TableCount) + ' tables added. Database is ready for use.');
    except
      raise;
    end;
  end;

procedure TTableManager.CreateChildTable(AParentTable,
  AChildTable: TILDataTable);
  var
    F: string;
  begin
    Comment1 := '';
    if AParentTable = nil then
    begin
      if AChildTable <> RootTable then
       HandleILException(E_MissingParent);
    end;
    if AChildTable = nil then
      HandleILException(E_MissingChild);
    if AParentTable = AChildTable then
      HandleILException(E_ParentEqChild);
    if not Assigned(AChildTable.MasterSource) and not (RootTable = AChildTable) then
      HandleILException(E_MissingMasterSource);
    if (AChildTable.TableName = '') then
      HandleILException(E_NoTableNameLoc);
    if Connected then
      HandleILException(E_CreateConnected);
    SetNames;
    OpenFiles;
    F := AChildTable.TableName + '_Data';
    if FLibContainer.IndexOf(F) <> -1 then
    begin
      CloseFiles;
      HandleILException(E_FileExists);
    end;
    DoCreateTable(AChildTable);
    FindFreeTables;
    CloseFiles;
  end;

procedure TTableManager.CreateFreeTable(ATable: TILTable);
var
  F: string;
begin
  if Root = nil then
    HandleILException(E_NoConnection);
  if (RootTable = ATable) then
  begin
    CreateChildTable(nil, ATable as TILDataTable);
    Exit;
  end;
  if (ATable.TableName = '') or (Location = '') then
    HandleILException(E_NoTableNameLoc);
  F := Location + '\' + Database + '.dat';

  if not FileExists(F) then
  begin
     HandleILException(E_LibNotCreated);
  end;

  if Connected then
    HandleILException(E_CreateConnected);
  SetNames;
  OpenFiles;
  F := ATable.TableName + '_Data';
  if FLibContainer.IndexOf(F) <> -1 then
  begin
    CloseFiles;
    HandleILException(E_FileExists);
  end;
  DoCreateTable(ATable);
  RegisterFreeTable(ATable);
  FindFreeTables;
  CloseFiles;
end;

procedure TTableManager.CreateTable(AParentTable, AChildTable : TILDataTable);
var
 IsConn : Boolean;
begin
   if AChildTable <> nil then
    if not HasUserFields(AChildTable) then
     HandleILException(E_TableNoFields);
   IsConn := Connected;
   if IsConn then
     Connected := false;
   CreateChildTable(AParentTable, AChildTable);
   FSkipValidation := True;
   Connected := true;
   ShowMessage('Child table ' + AChildTable.TableName + ' successfully created. ' + c_TBLAddNote);

end;

procedure TTableManager.CreateTable(ATable : TILTable);
var
  IsConn : Boolean;
begin
   if ATable = nil then
     Exit;
   if ATable.MasterSource <> nil then
     HandleILException(E_FreeTableMaster);
   if not HasUserFields(ATable) then
     HandleILException(E_TableNoFields);
   IsConn := Connected;
   if IsConn then
     Connected := false;
   if ATable is TILLookupTable then
   begin
     CreateLookupTable(ATable as TILLookupTable);
   end
   else
   begin
     CreateFreeTable(ATable);
   end;
   FSkipValidation := True;
   Connected := True;
   if ATable is TILLookupTable then
   begin
     if ATable is TILImageTable then
       ShowMessage('Image table ' + ATable.TableName + ' successfully created. ' + c_TblAddNote)
     else
       ShowMessage('Lookup table ' + ATable.TableName + ' successfully created. ' + c_TblAddNote);
   end
   else
   begin
     ShowMessage('Table ' + ATable.TableName + ' successfully created. ' + c_TblAddNote);
   end;
end;

procedure TTableManager.ClearTableTree;
begin
   if RootTable = nil then
     HandleILException(E_MissingRootTable);
   RootTable.ClearTable;
end;

procedure TTableManager.ClearDatabase;
var
  i : integer;
  sl : TStringList;
begin
  if RootTable = nil then
    HandleILException(E_MissingRootTable);
  RootTable.ClearTable;
  for I := 0 to FreeTableCount-1 do
    FreeTables[i].ClearTable;
  if FThumbs <> nil then
    FThumbs.Size := 0;
  if FTextLobs <> nil then
    FTextLobs.Size := 0;
  if FileExists(FTableCfgName) then
  begin
   if Assigned(FCfgFile) then
   begin
     FCfgFile.Free
   end;
   sl := TStringList.Create;
   try
    sl.LoadFromFile(FTableCfgName);
    sl.Clear;
    sl.SaveToFile(FTableCfgName);
   finally
     sl.Free;
   end;
   FCfgFile := TIniFile.Create(FTableCfgName);
  end;
end;

procedure TTableManager.RegisterFreeTable(ATable: TILTable);
var
  i, j : integer;
  T : TTreeTables;
begin

  for I := 0 to FreeTableCount-1 do
  begin
    if FreeTables[i] = ATable then
      Exit;
  end;

  { if it is in the regular table list, delete it from there}
  for I := 0 to High(TreeTables) do
  begin
    if TreeTables[i] = ATable then
    begin
      SetLength(T, Length(FTreeTables)-1);
      for j := 0 to High(FTreeTables) do
      begin
          if j < i then
           T[j] := FTreeTables[j]
          else if j > i then
           T[j-1] := FTreeTables[j];
      end;
      Finalize(FTreeTables);
      FTreeTables := Copy(T, 0, Length(T));
      Finalize(T);
      Break;
    end;
  end;

  SetLength(FFreeTables, Length(FFreeTables) + 1);
  FFreeTables[High(FFreeTables)] := ATable;
end;

procedure TTableManager.UnregisterFreeTable(ATable: TILTable);
var
  i, j : integer;
  T : TFreeTables;
begin
  for I := 0 to FreeTableCount-1 do
  begin
    if FreeTables[i] = ATable then
    begin
      SetLength(T, Length(FFreeTables)-1);
      for j := 0 to High(FFreeTables) do
      begin
          if j < i then
           T[j] := FFreeTables[j]
          else if j > i then
           T[j-1] := FFreeTables[j];
      end;
      Finalize(FFreeTables);
      FFreeTables := Copy(T, 0, Length(T));
      Finalize(T);
      Break;
    end;
  end;
end;

procedure TTableManager.CreateLookupTable(ALookupTable : TIlLookupTable);
begin
   CreateFreeTable(ALookupTable);
end;

procedure TTableManager.PopulateTableList(AList : TListBox);
var
  i : integer;
begin
   AList.Clear;
   for i := 0 to TMan.TableCount-1 do
   begin
     if TMan.Tables[i].DataHandler <> nil then
     begin
       if not TMan.Tables[i].DataHandler.HideInTableList then
         AList.Items.Add(TMan.Tables[i].Alias);
     end;
   end;
   if AList.Count > 0 then
   begin
    AList.Sorted := True;
    AList.ItemIndex := AList.Items.IndexOf(ActiveTable.TableName);
   end;
end;

function TTableManager.CheckDupTable(ATableName : string) : Boolean;
var
  i : integer;
begin
  Result := false;
  for I := 0 to TableCount-1 do
    if SameText(Tables[i].TableName, ATableName) then
    begin
      Result := True;
      Break;
    end;
end;

function TTableManager.CheckDupDB(ADBName : string) : Boolean;
var
  Reg : TRegistry;
begin
   Reg := TRegistry.Create(KEY_READ);
   try
    Reg.RootKey := HKEY_CURRENT_USER;
    Result := Reg.KeyExists('\Software\ImageLibrary\' + ADBName);
   finally
    Reg.Free;
   end;
end;

function TTableManager.NodeOf(ATable: TILTable): TTableNode;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to TableNodeCount - 1 do
  begin
    if TableNodes[i].Table = ATable then
    begin
      Result := TableNodes[i];
      Break;
    end;
  end;
end;

function TTableManager.TreeTableOf(const TableName: string): TILDataTable;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to TableNodeCount - 1 do
    if SameText(TreeTables[i].TableName, TableName)
    or SameText(TreeTables[i].Alias, TableName) then
    begin
      Result := TreeTables[i];
      Break;
    end;
end;

function TTableManager.FreeTableOf(const TableName: string): TILTable;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to FreeTableCount-1 do
    if SameText(FreeTables[i].TableName, TableName)
    or SameText(FreeTables[i].Alias, TableName) then
    begin
      Result := FreeTables[i];
      Break;
    end;
end;

function TTableManager.TableOf(const TableName: string): TILTable;
begin
  Result := TreeTableOf(TableName);
  if Result = nil then
   Result := FreeTableOf(TableName);
end;

function TTableManager.TableIndexOf(ATable : TILTable) : integer;
var
  i : integer;
begin
   Result := -1;
   for I := 0 to TableCount-1 do
     if Tables[i] = ATable then
     begin
       Result := i;
       Break;
     end;
end;

function TTableManager.IsRelated(AParent, ATable: TILTable): Boolean;
var
  i: integer;
  N: TTableNode;
begin
{ Not documented, only IsInPath }
  Result := false;
  N := NodeOf(AParent);
  if N = nil then
    Exit;
  for i := 0 to N.CountAll - 1 do
  begin
    if N.AllChildren[i].Table = ATable then
    begin
      Result := True;
      Break;
    end;
  end;
end;

procedure TTableManager.ClearTableLists;
begin
  Finalize(FTreeTables);
  Finalize(FFreeTables);
  RegTables.Clear;
  while RegHandlers.Count > 0 do
  begin
    TILData(RegHandlers[0]).Free;
  end;
  RegHandlers.Clear;
end;

function TTableManager.TableIndex(ATable : TILTable) : integer;
var
  i : integer;
begin
   Result := -1;
   for I := 0 to TableNodeCount-1 do
    if TreeTables[i] = ATable then
    begin
        Result := I;
        Break;
    end;
end;

function TTableManager.IsInPath(Table1, Table2: TILTable): Boolean;
var
  N1, N2: TTableNode;
  Child, Prnt: TTableNode;
begin
  Result := false;
  if Table1 = Table2 then
  begin
    Result := True;
    Exit;
  end;
  if Table1.IsFreeTable or Table2.IsFreeTable then
  begin
    Exit;
  end;
  N1 := NodeOf(Table1);
  N2 := NodeOf(Table2);

  if N1.Level = N2.Level then
  begin
    Exit;
  end;
  if N1.Level > N2.Level then
  begin
    Child := N1;
    Prnt := N2;
  end
  else
  begin
    Child := N2;
    Prnt := N1;
  end;
  while (Child <> Prnt) and (Child <> nil) do
  begin
    Child := Child.Parent;
  end;
  Result := (Child = Prnt);
end;

function TTableManager.IsInLinkedPath(Table1, Table2: TILTable; var Master : TilTable): Boolean;
var
  i : integer;
begin
  Result := false;
  if (Table1 Is TILLookupTable) or (Table2 Is TILLookupTable) then
    Exit;

  Master := nil;
  if IsInPath(Table1, Table2) then
   Exit;
  for I := 0 to Table1.Fields.Count-1 do
  begin
    if Table1.Fields.Items[i].LookupType = looTableKey then
    begin
        if Table1.Fields.Items[i].LookupTable = Table2 then
        begin
          Result := True;
          Master := Table2;
          Break;
        end;
    end;
  end;
  if Result then
    Exit;
  for I := 0 to Table2.Fields.Count-1 do
  begin
    if Table2.Fields.Items[i].LookupType = looTableKey then
    begin
        if Table2.Fields.Items[i].LookupTable = Table1 then
        begin
          Result := True;
          Master := Table1;
          Break;
        end;
    end;
  end;
end;

function TTableManager.IsLinkedChild(ATable: TILDataTable) : Boolean;
var
  i : integer;
begin
     Result := false;
     for i := 0 to ATable.Fields.Count-1 do
     if (ATable.Fields.Items[i].LookupType = looTableKey)
     then
     begin
       Result := True;
       Break;
     end;
end;

function TTableManager.IsLinkedChild(AParentTable: TILDataTable; ATestTable: TILDataTable) : Boolean;
var
  i : integer;
begin
     Result := false;
     for i := 0 to ATestTable.Fields.Count-1 do
     if (ATestTable.Fields.Items[i].LookupType = looTableKey)
     and (AtestTable.Fields.Items[i].LookupTable = AParentTable)
     then
     begin
       Result := True;
       Break;
     end;
end;

function TTableManager.IsInSearchPath(Table1, Table2: TILTable): Boolean;
var
  T : TILTable;
begin
  Result := IsInPath(Table1, Table2)
  or IsInLinkedPath(Table1, Table2, T);

  { This gets too complicated. Ambigous paths with more than one parent :

  if not Result then
  begin
    if Table1.MasterLinkCount > 0 then
    for I := 0 to Table1.MasterLinkCount-1 do
    begin
      if IsInPath(Table1.MasterLinks[i], Table2) then
      begin
        Result := True;
        Break;
      end;
    end
    else if Table2.MasterLinkCount > 0 then
    for I := 0 to Table2.MasterLinkCount-1 do
    begin
      if IsInPath(Table2.MasterLinks[i], Table1) then
      begin
        Result := True;
        Break;
      end;
    end;
  end;  }
end;


function TTableManager.IsTableRegistered(ATable : TILTable) : Boolean;
var
  LCOpen : Boolean;
begin
    LCOpen := True;
    if FLibContainer = nil then
    begin
      SetNames;
      FLibContainer := TFileContainer.Create(FLibPath, 0, True);
      FLibContainer.Unicode := Unicode;
      LCOpen := False;
    end;
    ATable.SetNames(ATable);
    Result := (FLibContainer.IndexOf(ATable.FDataName) <> -1);
    if not LCOpen then
      FreeAndNil(FLibContainer);
end;

function TTableManager.IsChild(AParent, ATable: TILTable): Boolean;
var
  i: integer;
  N: TTableNode;
begin
  Result := false;
  N := NodeOf(AParent);
  if N = nil then
    Exit;
  for i := 0 to N.Count - 1 do
  begin
    if N.Children[i].Table = ATable then
    begin
      Result := True;
      Break;
    end;
  end;
end;

function TTableManager.SaveAll: integer;
var
  i : integer;
  ErrIndx : integer;
  ErrCode : integer;
  ErrTable : TILTable;
  S : string;
begin
  if not Connected then
    HandleILException(E_NoConnection);
  if IsReadOnly then
    HandleILException(E_ReadOnly);

  ErrCode := CanSaveAll(ErrTable, ErrIndx);
  if ErrCode <> 0 then
  begin
    if ErrTable <> nil then
      S := ' Table: ' + ErrTable.TableName + '. Record: ' + IntToStr(ErrIndx)
    else
      S := '';
    HandleILException(ErrCode, S);
  end;
  Result := 0;
  FSavingAll := True;
  SetTransState(TrPending);
  try
    for I := 0 to TableCount-1 do
    begin
     try
       if not Tables[i].Dataset.Changed then
        continue;
        Result := Result + Tables[i].Dataset.Save;
     except on E:Exception do
      begin
        HandleTransError(E);
      end;
     end;
    end;
    SetTransState(TrCommitted);
  finally
    FSavingAll := false;
  end;
end;

function TTableManager.CanSaveAll(var ErrorTable : TILTable; var ErrorIndex : integer) : integer;
var
  i : integer;
  ErrIndx : integer;
  ErrCode : integer;
begin
  if not Connected then
    HandleILException(E_NoConnection);
  Result := -1;
  ErrorIndex := -1;
  ErrorTable := nil;
  for I := 0 to TableCount-1 do
  begin
    if not Tables[i].Dataset.Changed and not Tman.AutoUpdate then
      continue;
    Comment1 := '';
    ErrCode := Tables[i].Dataset.CanSave(ErrIndx);
    if ErrCode = -1 then
      Continue;
    if ErrCode <> 0 then
    begin
         Result := ErrCode;
         ErrorIndex := ErrIndx;
         ErrorTable := TreeTables[i];
         Break;
    end
    else
     Result := 0;
  end;
end;

function TTableManager.CheckUnsaved(ATable : TILTable) : Boolean;
var
    Cancl, Allow : Boolean;
    Res, E : integer;
begin
    Result := True;
    if Assigned(FOnUnsaved) or AutoSave then
    begin
      if ATable.Dataset.Changed then
      begin
        Res := ATable.DBWork.CanSave(E);
        if AutoSave then
        begin
          if Res = 0 then
            ATable.DBWork.Save
          else
            HandleILException(Res);
        end
        else if Assigned(FOnUnsaved) then
        begin
          Allow := false;
          Cancl := false;
          FOnUnsaved(ATable, Allow, Cancl);
          if Cancl then
          begin
            Result := false;
            Exit;
          end;
          if Allow then
          begin
           if Res <> 0 then
           begin
             if Assigned(TableList) then
              TableList.ItemIndex := TableList.Items.IndexOf(ActiveTable.Alias);
             HandleILException(Res);
           end;
           ATable.DBWork.Save;
          end;
        end;
      end;
    end;
end;

function TTableManager.ChildrenChanged(ParentTable : TILDataTable) : integer;
var
  N : TTableNode;
  i : integer;
  Cnt : integer;
begin
    Result := 0;
    N := NodeOf(ParentTable);
    if N = nil then
      Exit;
    Cnt := N.CountAll;
    for I := 0 to Cnt-1 do
    begin
       if N.AllChildren[i].Table.Dataset.Changed then
         inc(Result);
    end;

end;

procedure TTableManager.Delete(ATable: TILTable);
var
  Ind: integer;
  T: TTreeTables;
  i: integer;
begin
  Ind := -1;
  for i := 0 to TableNodeCount - 1 do
    if FTreeTables[i] = ATable then
    begin
      Ind := i;
      Break;
    end;
  if Ind = -1 then
  begin
    UnregisterFreeTable(ATable);
    Exit;
  end;

  TTableNode(FNodes[Ind]).Free;
  FNodes.Delete(Ind);
  SetLength(T, TableNodeCount);
  for i := 0 to High(FTreeTables) do
  begin
    if i < Ind then
      T[i] := FTreeTables[i]
    else if i = Ind then
      Continue
    else
      T[i - 1] := FTreeTables[i];
  end;
  Finalize(FTreeTables);
  FTreeTables := Copy(T, 0, Length(T));
  Finalize(T);
end;

procedure TTableManager.Add(ATable: TILDataTable);
begin
  TTableNode.Create(ATable, Self);
end;

procedure TTableManager.Clear;
var
  i: integer;
begin
  Finalize(FTreeTables);
  for i := 0 to FNodes.Count - 1 do
    TTableNode(FNodes[i]).Free;
  FNodes.Clear;
end;

procedure TTableManager.Loaded;
begin
  inherited;
  SetIntPlayerIcon(PlayerIcon);
  if Database = '' then
    Exit;
  if Assigned(FTableList) then
    begin
      PopulateTableList(FTableList);
      if FTableList.Items.Count > 0 then
      begin
        FTableList.ItemIndex := 0;
      end;
    end;
  if Assigned(ActiveTable) then
    if Assigned(FOnActivateTable) then
      FOnActivateTable(ActiveTable);
  PostMessage(FHWnd, WM_Loaded, 0, 0);
end;

procedure TTableManager.Progress(Table : TObject; const Operation : TILOperation; const Count, Progress : integer);
var
  Ticks : integer;
  MBar : TILMessageBar;
  i : integer;
  AText : string;
begin
  MBar := nil;
  for I := 0 to RegMbs.Count-1 do
    if TILMessageBar(RegMbs[i]).Table = Table then
    begin
      MBar := TILMessageBar(RegMbs[i]);
      Break;
    end;

  if not Assigned(FOnProgress) and (MBar = nil) then
    Exit;
  if FProgressTicker = 0 then
    FProgressTicker := GetTickCount;

  Ticks := GetTickCount - FProgressTicker;
  if (FProgressFreq = 0) or (Ticks >= FProgressFreq)
  or (Operation in [iloChangeOperation, iloFinished]) then
  begin
   if Assigned(FOnProgress) then
     FOnProgress(Table, Operation, Count, Progress);
   if MBar <> nil then
   begin
    AText := GetProgressText(Operation);
    MBar.UpdateMessage(Operation, AText, Progress, Count);
   end;

   FProgressTicker := GetTickCount;
  end;
end;

function TTableManager.GetLocation: string;
begin
    Result := FLocation;
end;

procedure TTableManager.SetLocation(const Value : string);
begin
  FLocation := Value;
end;

function TTableManager.GetFreeTableCount : integer;
begin
   Result := Length(FFreeTables);
end;

function TTableManager.GetFreeTables(Index : integer) : TILTable;
begin
   Result := FFreeTables[Index];
end;

function TTableManager.GetTableCount : integer;
begin
   Result := TableNodeCount + FreeTableCount;
end;

function TTableManager.GetTables(Index : integer) : TILTable;
begin
   if Index <= TableNodeCount-1 then
     Result := TreeTables[Index]
   else
     Result := FreeTables[Index-TableNodeCount];
end;

function TTableManager.GetTables(TableName : string) : TILTable;
begin
   Result := TableOf(TableName);
end;

function TTableManager.GetAutoSave : Boolean;
begin
  if (ActiveTable = nil) or (ActiveTable.MultiSelected) then
    Result := False
  else
    Result := FAutoSave;
end;

function TTableManager.GetAutoUpdate : Boolean;
begin
  if (ActiveTable = nil) or (ActiveTable.MultiSelected) then
    Result := False
  else
    Result := FAutoUpdate;
end;

function TTableManager.GetPanels(Index : integer) : TILControlPanel;
begin
   Result := TILControlPanel(FPanels[Index]);
end;

function TTableManager.GetPanelCount : integer;
begin
   Result := FPanels.Count;
end;

function TTableManager.GetMessageHandler(ATable : TILTable;
 var LastIndex : integer) : TILMessageHandler;
var
  i : integer;
  MH : TILMessageHandler;
begin
  Result := nil;
  for I := 0 to RegMBh.Count-1 do
  begin
    if (LastIndex <> -1) and (I <= LastIndex) then
     Continue;
    MH := TILMessageHandler(RegMbh[i]);
    if (MH.Table = ATable) then
    begin
       Result := MH;
       LastIndex := I;
       Break;
    end;
  end;
end;

function TTableManager.GetReportTrue : string;
  var
    S : string;
  begin
    { Translation overrides }
    S := Translator.GetTranslation(T_TRUE, TRANSL_TERMS);
    if not SameText(S, T_TERMS[T_TRUE]) then
      Result := S
    else
    begin
     if TMan <> nil then
      Result := TMan.AliasTrue
    else
      Result := 'Yes';
    end;
  end;

  function TTableManager.IsValueNull(AValue : string) : Boolean;
  begin
    Result := SameText(AValue, ReportNull) or SameText('Null', AValue);
  end;

  function TTableManager.GetReportNull : string;
  var
    S : string;
  begin
    { Translation overrides }
    S := Translator.GetTranslation(T_NULL, TRANSL_TERMS);
    if not SameText(S, T_TERMS[T_NULL]) then
      Result := S
    else
    begin
     if TMan <> nil then
      Result := TMan.AliasNull
    else
      Result := 'Null';
    end;
  end;

  function TTableManager.GetReportFalse : string;
  var
    S: string;
  begin
    S := Translator.GetTranslation(T_FALSE, TRANSL_TERMS);
    if not SameText(S, T_TERMS[T_FALSE]) then
      Result := S
    else
    begin
     if TMan <> nil then
      Result := TMan.AliasFalse
    else
      Result := 'No';
    end;
  end;

function TTableManager.GetConnected: Boolean;
begin
  if RootTable = nil then
    Result := false
  else
    Result := RootTable.Connected;
end;

procedure TTableManager.FindFreeTables;
  var
   i : integer;
   Tbl : TilTable;
  begin
     for I := 0 to RegTables.Count-1 do
     begin
        Tbl := TILTable(RegTables[i]);
        if Tbl.TableName = '' then
          Continue;
        if TreeTableOf(Tbl.TableName) <> nil then
          Continue;
        if FreeTableOf(Tbl.TableName) <> nil then
          Continue;
         RegisterFreeTable(Tbl);
     end;
end;

function TTableManager.HasUserFields(ATable : TILTable) : Boolean;
begin
  if ATable is TILTreeViewTable then
  begin
    if ATable.Fields.IndexOf('SYS_LinkID') = -1 then
      Result := false
    else
      Result := ATable.Fields.Count > 3;
  end
  else if ATable.IsDetail then
  begin
     if ATable.Fields.IndexOf('SYS_LinkID') = -1 then
       Result := false
     else
       Result := ATable.Fields.Count > 1
  end
  else
    Result := ATable.Fields.Count > 0;
end;

function TTableManager.GetDatabase : string;
begin
   Result := FDatabase;
end;

function TTableManager.GetSaveOnDelete : Boolean;
begin
  if AutoSave then
   Result := True
  else
   Result := FSaveOnDelete;
end;

procedure TTableManager.SetDatabase(const Value : string);
var
 T : TILReplicator;
 Tbl : TILTable;
 Reg : TRegistry;

 procedure OpenCfg;
 begin
    if Assigned(FCfgFile) then
      FcfgFile.Free;
    FCfgFile := TIniFile.Create(FTablecfgName);
 end;

 procedure ResolveNames(DB : string);
 begin
     FLocation := GetAppdataFolderPath + '\ImageLibrary\' + DB;
     if not DirectoryExists(FLocation) then
     begin
        Reg := TRegistry.Create;
        try
          Reg.RootKey := HKEY_CURRENT_USER;
          if Reg.OpenKeyReadOnly('\Software\ImageLibrary\' + DB) then
          begin
             FLocation := Reg.ReadString('Location');
             if not DirectoryExists(FLocation) then
             begin
               FLocation := GetCurrentDir;
               FIsServer := True;
             end
             else
             begin
               FIsServer := not (Pos('\\', FLocation) = 1);
               { Server stores location as local path. Clients uses UNC format }
               FServer := UNCFromFileName(FLocation);
             end;
          end;
          if not FIsServer and (csDesigning in ComponentState) then
            HandleILException(0, 'Access to database denied. You cannot do design time changes ' +
            'when accessing a remote computer. Close Delphi without saving and run your application outside the IDE.');
        finally
          Reg.Free;
        end;
     end;
     SetNames;
 end;

begin
  if SameText(FDatabase, Value) then
    Exit;

  if (csLoading in ComponentState) then
  begin
    FDatabase := Value;
    SetNames;
    if FDatabase <> '' then
    begin
     ResolveNames(FDatabase);
     OpenCfg;
    end;
    Exit;
  end;

  if (Regtables.Count > 0) then
  begin
    while RegTables.Count > 0 do
    begin
        Tbl := TILTable(RegTables[0]);
        FreeAndNil(TBL);
    end;
    RegTables.Clear;
    Finalize(FFreeTables);
    Finalize(FTreeTables);
  end;
  if Value = '' then
  begin
    FDatabase := '';
    Exit;
  end;

  Comment1 := Value;
  FDatabase := Value;
  SetNames;
  ResolveNames(Value);
  if not FileExists(FLibPath) then
  begin
    FDatabase := '';
    SetNames;
    Comment1 := FLibPath;
    HandleILException(E_FileNotExists, FLibPath);
  end;

  if not FileExists(FMetaName) then
  begin
    FDatabase := '';
    SetNames;
    Comment1 := FMetaName;
    HandleILException(E_FileNotExists, FMetaName);
  end;
  OpenCfg;
  T := TILReplicator.Create(FMetaName, Self);
  try
   T.Replicate;
  finally
   T.Free;
  end;
  ExternalMeta := True;
end;

procedure TTableManager.SetUnicode(const Value : Boolean);
begin
    FUnicode := Value;
end;

procedure TTableManager.SetConnected(const Value : Boolean);
var
  i: integer;
  IsRef : Boolean;
  ErrTbl : TILTable;

 function IsReferenced(Tbl : TILTable) : Boolean;
 var
   i, j : integer;
   T : TILTable;
 begin
    Result := false;
    for I := 0 to RegTables.Count-1 do
    begin
      T := TILTable(RegTables[i]);
      if T = Tbl then
        Continue;
      if TBL = SharedImages then
      begin
        Result := True;
        Exit;
      end;
      for j := 0 to T.Fields.Count-1 do
        if T.Fields.Items[j].LookupTable = Tbl then
        begin
           Result := True;
           Exit;
        end;
    end;
 end;

 procedure CheckTransState;
 var
   TS : integer;
   Dt : TDateTime;
 begin
   if IsReadOnly then
     Exit;
   TS := GetTransState(FTransID);
   if TS in [TrPending, TrPendingActive] then
   begin
    if Assigned(FOnAlert) then
      FOnAlert(Self, alRollback);
    if TS = trPendingActive then
      FTransactionActive := True; { Else rollback does nothing}
    RollbackTrans;
   end
   else if not TransHistory and (FTranslogFile.Size > 0) then
   begin
     ClearTransLog;
     FTransactionActive := false;
   end;

   if TransLogAge <> 0 then
   begin
      Dt := Today - TranslogAge;
      TruncateTransLog(Dt);

   end;

 end;

begin
  if Value = Connected then
    Exit;
  if Value and not FIsLoaded then
    HandleILException(E_DBNotIdle);
  while FSystemBusy do
    Application.ProcessMessages;
  if Value then
  begin
   SetNames; { Set essensial variable names }
   if not FileExists(FLibPath) then
   begin
    Comment1 := FLibPath;
    HandleILException(E_FileNotExists);
   end;
   if RootTable = nil then
   begin
    HandleILException(E_MissingRootTable);

   end;
   if Connected then
     Exit;
  end;

  if not Value and not Connected then
    Exit;

  if Value then
  begin
     ErrTbl := CheckStates;
     if (ErrTbl <> nil) and not ExtDesigner then
     begin
       Comment1 := ErrTbl.TableName;
       HandleILException(E_TBLErrorState);
     end;
    if IsFileInUse(FLibPath) then
    begin
     IsReadOnly := True;
     if Assigned(FOnAlert) then
       FOnAlert(Self, alReadOnly);
    end;
    OpenFiles; { Open the database files in lib container }
    FIsDBEmpty := FLibContainer.IsEmpty;
    FindFreeTables;
    { Check if essensial table props are present}
    for I := 0 to TableNodeCount-1 do
    begin
       if TreeTables[i].TableName = '' then
        HandleILException(E_MissingTableName, TreeTables[i].Name);
    end;
    for i := 0 to FreeTableCount-1 do
    begin
     IsRef := IsReferenced(FreeTables[i]);
     if (FreeTables[i].TableName = '') then
     begin
       if IsRef then
         HandleILException(E_MissingTableName, FreeTables[i].Name);
     end;
    end;
  end;

  { Connect / diconnect tables.
  Connecting a table sets the table variables that points to the files within the lib container.
  If one connection fails, an exeception is raised and all table connections are closed.
  Disconnecting a table calls Diconnect for each table. In turn this method calls Closetable which
  writes the necessary info to the cfg file and save meta data}

  { Open / Close Tree structure }

  RootTable.Connected := Value;
  { The root table connects all of its childs }
  for i := 0 to FreeTableCount-1 do
  begin
     if FreeTables[i].TableName <> '' then
       FreeTables[i].Connected := Value;
  end;

  if not Value then
    CloseFiles
  else
  begin
   if not IsReadOnly then
   begin
      if not FileExists(FTransLogName) then
        FTransLogFile := TFileStream.Create(FTranslogName, fmCreate)
      else if not Assigned(FTransLogFile) then
      begin
        if IsFileInUse(FTransLogName) and not IsReadOnly then
        begin
          IsReadOnly := True;
          if Assigned(FOnAlert) then
           FOnAlert(Self, alReadOnly);
       end
       else
        FTransLogFile := TFileStream.Create(FTranslogName, fmOpenReadWrite);
      end;
   end;

   if not FConnectSilent then
   begin
    CheckTransstate;
    DoLoadIndexes;
    if Assigned(FOnActivateTable) then
      FOnActivateTable(RootTable);
   end;

   if Value and not FConnectSilent then
   begin
       { Init mem tables. All tables must be opened before doing this }
       InvalidateMemTables;
       if RootTable.HasViewer then
        RootTable.Viewer.Invalidate;

   end;
  end;
end;

procedure TTableManager.InvalidateMemTables(ATable : TILMemTable = nil);
var
  i : integer;
  M : TILMemTable;
begin
     for i := 0 to FreeTableCount-1 do
     begin
       if FreeTables[i] is TilmemTable then
       begin
        if ATable <> nil then
          if ATable <> FreeTables[i] then
            Continue;
        M := TILMemTable(FreeTables[i]);
        if Assigned(M.FOnInvalidateTable) then
        begin
          M.FOnInvalidateTable(FreeTables[i]);
        end;
       end;
     end;
end;

procedure TTableManager.OpenFiles;
begin
   if FIsReadOnly then
   begin
     FThumbs := TFileStream.Create(FThumbName, fmOpenRead or fmShareDenyNone);
     FTextLobs := TFileStream.Create(FTextLobsName, fmOpenRead or fmShareDenyNone);
   end
   else
   begin
     FThumbs := TFileStream.Create(FThumbName, fmOpenReadWrite or fmShareDenyNone);
     FTextLobs := TFileStream.Create(FTextLobsName, fmOpenReadWrite or fmShareDenyNone);
   end;
   FLibContainer := TFileContainer.Create(FLibPath, 0, FIsReadOnly);
   FLibContainer.Unicode := Unicode;
   if not Assigned(FCFGFile) then
     FCfgFile := TINIFile.Create(FTableCfgName);
end;

procedure TTableManager.CreateFiles;
begin
   FThumbs := TFileStream.Create(FThumbName, fmCreate);
   FTextLobs := TFileStream.Create(FTextLobsName, fmCreate);
   FLibContainer := TFileContainer.Create(FLibPath,
   (8 * MaxLibFiles) , FIsReadOnly);
   { 8 files per table. MaxLib files = 30 }
   FLibContainer.Unicode := Unicode;
end;

procedure TTableManager.CloseFiles;
begin
   FLibContainer.Free;
   FLibContainer := nil;
   FTransLogFile.Free;
   FTransLogFile := nil;
   FThumbs.Free;
   FThumbs := nil;
   FTextLobs.Free;
   FTextLobs := nil;
   FCfgFile.Free;
   FCfgFile := nil;
end;

procedure TTableManager.SetNames;
var
  F : string;
  L : string;
begin
   if FDatabase = '' then
   begin
     FLocation := '';
     FLibPath := '';
     FTableCfgName := '';
     FThumbName := '';
     FMetaName := '';
     FTransLogName := '';
     FUserIniName := '';
     FTextLobsName := '';

     _FLibPath := '';
     _FTableCfgName := '';
     _FThumbName := '';
     _FMetaName := '';
     _FTransLogName := '';
     _FTextLobsName := '';
     _FUserIniName := '';

     _FTmpLibPath := '';
     _FTmpThumbName := '';
     _FTmpTextLobsName := '';
     _FTmpTransLogName := '';
     Exit;
   end;
   L := FLocation;
   F := L + '\' + Database + '.dat';
   FLibPath := F;
   FTableCfgName := L + '\' + Database + '.cfg';
   FThumbName := L + '\' + Database + '_Imgs.dat';
   FTextLobsName := L + '\' + Database + '_TextLobs.dat';
   FMetaName := L + '\' + Database + '_Meta.XML';
   FTransLogName := L + '\' + Database + '_Trans.LOG';
   FRollbackName := L + '\' + Database + '_Rollback.LOG';
   FUserIniName := L + '\User.cfg';

   _FLibPath := L + '\Bak_' + Database + '.dat';;
   _FTableCfgName := L + '\Bak_' + Database + '.cfg';
   _FThumbName := L + '\Bak_' + Database + '_Imgs.dat';
   _FTextLobsName := L + '\Bak_' + Database + '_TextLobs.dat';
   _FMetaName := L + '\Bak_' + Database + '_Meta.XML';
   _FTransLogName := L + '\Bak_' + Database + '_Trans.LOG';
   _FUserIniName := L + '\Bak_User.cfg';

   _FTmpThumbName := L + '\Tmp_' + Database + '_Imgs.dat';
   _FTmpTextLobsName := L + '\Tmp_' + Database + '_TextLobs.dat';
   _FTmpTransLogName := L + '\Tmp_' + Database + '_Trans.LOG';
   _FTmpLibPath := L + '\Tmp_' + Database + '.dat';


end;

function TTableManager.LogOpButtonOfControl(AControl : TWinControl; const LogopType : TLogOpType) : TILLogOpButton;
 var
    i : integer;
  begin
     Result := nil;
     for I := 0 to Owner.ComponentCount-1 do
     begin
         if Owner.Components[i] is TILLogOpButton then
         if (TILLogOpButton(Owner.Components[i]).FSearchControl = AControl)
         and (TILLogOpButton(Owner.Components[i]).LogopType = LogOpType)
         then
         begin
           Result := Owner.Components[i] as TILLogOpButton;
           Break;
         end;
       end;
  end;

 procedure TTableManager.LoadIndexes;
 var
   i : integer;
 begin
     for I := 0 to TableCount-1 do
     begin
       if (Tables[i] is TILView) or (Tables[i] is TILMemTable) then
         Continue;
       if not Assigned(Tables[i].FKeyList) then
       begin
         Tables[i].FKeyList := TList.create;
       end;
       if not Assigned(Tables[i].FLinkList) then
       begin
         Tables[i].FLinkList := TList.create;
       end;
       Tables[i].LoadKeyList;
       Tables[i].LoadLinkList;
     end;

     for I := 0 to TableCount-1 do
     begin
       if not (Tables[i] is TILView) or (Tables[i] is TILMemTable) then
         Continue;
       Tables[i].FKeyList := TILView(Tables[i]).SourceTable.FKeyList;
       Tables[i].FLinkList := TILView(Tables[i]).SourceTable.FLinkList;
     end;
 end;

 procedure TTableManager.DoLoadIndexes;
 var
   LI : TILLoadIndex;
 begin
      while TMan.FMainThreadBusy do
       sleep(50);

      FSystemBusy := True;
      if (csDesigning in ComponentState) then
        Exit;
      LI := TILLoadIndex.Create(True);
      LI.FreeOnTerminate := True;
      LI.Start;
 end;

  procedure TTableManager.DoCreateTable(ATable: TILTable);

  var
    Spec: TFileSpec;
    Space : integer;

  begin
    ATable.SetNames(ATable);
    { The XML data structures }
    Spec.FileName := ATable.FDataName;
    Spec.FileSpace := ATable.FTableSpace;
    if ATable.RecordLayout = rlVariableSize then
      Spec.RecordSpace := -ATable.RecordFreeSpace
    else if ATable.RecordLayout = rlFixedSize then
    begin
      Space := ATable.ComputeRecordSize;
      if ATable.RecordSize < Space then
        HandleILException(E_RecordSizeTooSmall);
      Spec.RecordSpace := ATable.RecordSize;
    end
    else
      Spec.RecordSpace := ATable.RecordSize + c_PageBase;
    Spec.FixedStringSize := false;
    Spec.Sorted := false;
    Spec.Duplicates := DupIgnore;
    ATable.FSF_Data := FLibContainer.Add(Spec);

    { The index table that links the index keys to the rcords }
    Spec.FileName := ATable.FLinkListName;
    Spec.FileSpace := Round(ATable.TableSpace * 10 / 100);
    { 10 % of table space, by experience }
    Spec.RecordSpace := 18;
    Spec.FixedStringSize := True;
    Spec.Sorted := True;
    Spec.Duplicates := dupAccept;
    ATable.FSF_LinkList := FLibContainer.Add(Spec);

    { Index key list }
    Spec.FileName := ATable.FKeyListName;
    Spec.FileSpace := Round(ATable.TableSpace * 5 / 100);
    { 5 % of table space, by experience }
    Spec.RecordSpace := ATable.IndexSpace;
    Spec.FixedStringSize := false;
    Spec.Sorted := True;
    Spec.Duplicates := DupIgnore;
    ATable.FSF_KeyList := FLibContainer.Add(Spec);

  end;

   { Panel methods : }

   function TTableManager.FindPanel(ATable : TILTable; ACommand : TCommandMode) : TILControlPanel;
   var
     i, j : integer;
   begin
      Result := nil;
      for I := 0 to PanelCount-1 do
      begin
         for j := 0 to Panels[i].Count- 1 do
         begin
           if ATable = Panels[i].Items[j].Table then
           begin
             if (ACommand = cmUpdate) and (Panels[i].Items[j].Usage = duData) then
             begin
               Result := Panels[i];
               Exit;
             end
             else if (ACommand in [cmSearch, cmSort, cmColumns]) and (Panels[i].Items[j].Usage = duSearch) then
             begin
               Result := Panels[i];
               Exit;
             end

           end;
         end;
      end;
   end;

   procedure TTableManager.RemoveWorkField(ADataField : TILDataField);
   var
     PanFound : Boolean;
     Pan : TILControlPanel;
     I, J : integer;
     Tbl : TILTable;
     FDef : TILFieldDef;
   begin
       if Tman = nil then
         Exit;
       PanFound := false;
       Tbl := ADataField.Table;
       FDef := ADataField.FieldDef;
       FDef.DataControl := nil;
       FDef.DataLabel := nil;
       FDef.SearchControl := nil;
       FDef.SearchLabel := nil;

       for I := 0 to RegPanels.Count-1 do
       begin
         Pan := TILControlPanel(RegPanels[i]);
         FPanelBuild := True;
         for j := 0 to Pan.Count-1 do
         if (Pan.Items[j].FField = FDef)
         then
         begin
           if Pan.Items[j].ControlLabel <> nil then
           begin
              if Pan.Items[j].Usage = duData then
              FreeAndNil(Pan.Items[j].Field.FDataLabel)
             else
              FreeAndNil(Pan.Items[j].Field.FSearchLabel);
           end;
           PostMessage(Tbl.FHWnd, wm_UpdatePanel, 0, Integer(Pan));
           PanFound := True;
             Break;
         end;
         if PanFound then
            Break;
       end;
       if not PanFound then
         PostMessage(Tbl.FHWnd, wm_UpdatePanel, 0, 0);
   end;

   procedure TTableManager.RecreateWorkFields;
   var
     i : integer;
   begin
     if not UseCommandModes then
       Exit;
     if CommandMode in [cmSort, cmColumns, cmSQL] then
       Exit;

     for I := 0 to FWorkHistory.Count-1 do
      TWorkHistoryElement(FWorkHistory[i]).Free;
     FWorkHistory.Clear;

     CreateWorkFields(ActiveTable, CommandMode);
     UpdateWorkList;
     if (ActiveRecord <> nil) then
     begin
      if ((CommandMode = cmUpdate) and not ActiveTable.MultiSelected or (ActiveTable.SelCount = 0))
      or (CommandMode = cmSearch)
      then
        ActiveTable.DBWork.WriteControls(ActiveRecord);
     end;
     DoShowWorkFields(FindPanel(ActiveTable, CommandMode));
     if CommandMode = cmSearch then
       WriteBrowser(FindPanel(ActiveTable, cmUpdate));
   end;

   procedure TTableManager.CreateWorkFields(const ATable: TILTable; const CommandMode: TCommandMode);
   var
     i : integer;
     Indx : integer;
     P : TILControlPanel;
     PI, PI2 : TILWorkField;
     W : TWorkHistoryElement;
     FldDef : TILFieldDef;
     Accept : Boolean;

     procedure DoCreateCtrl(FldDef : TILFieldDef; Usage : TDataUsage);
     var
       PI : TILWorkField;
     begin
        PI := TILWorkField.Create;
        if Usage = duData then
        begin
          PI.FControl := FldDef.DataControl;
          PI.FLabel := FldDef.DataLabel;
        end
        else if CommandMode = cmSearch then
        begin
          PI.FControl := FldDef.SearchControl;
          PI.FLabel := FldDef.SearchLabel;
          PI.FFieldLogOpButton := LogOpButtonOfControl(PI.FControl, ltField);
          PI.FValueLogOpButton := LogOpButtonOfControl(PI.FControl, ltValue)
        end;
        PI.FPanel := nil;
        PI.Usage := Usage;
        PI.FTable := FldDef.Ilib;
        PI.FField := FldDef;

        if CommandMode = cmSort then
          PI.IsVisible := (PI.Table.Columns.SortFields.IndexOf(PI.Field.Fieldname) <> -1)
        else if CommandMode = cmColumns then
          PI.IsVisible := (PI.Table.Columns.IndexOf(PI.Field.Fieldname) <> -1)
        else
          PI.FIsVisible := PI.FControl.Visible;
        PI.FCommand := CommandMode;
        PI.FCreator := wcManager;
        FWorkFields.AddObject(PI.Field.Caption, PI);
     end;

   begin
     if not UseCommandModes then
       Exit;
     if ATable = nil then
       Exit;
     if ATable.DataHandler = nil then
       Exit;
     if ATable.DataHandler.DisableCommandModes then
       Exit;
     for I := 0 to FWorkFields.Count-1 do
       TILWorkField(FWorkFields.Objects[i]).Free;
     FWorkFields.Clear;

     W := GetFromHistory(ATable, Commandmode);
     if W <> nil then
     begin
       for I := 0 to W.FWorkFields.Count-1 do
       begin
         PI := TILWorkField.Create;
         PI2 := TILWorkField(W.FWorkFields.Objects[i]);
         PI.FControl := PI2.FControl;
         PI.FLabel := PI2.FLabel;
         PI.FPanel := PI2.FPanel;
         PI.Usage := PI2.Usage;
         PI.FTable := PI2.FTable;
         PI.FField := PI2.Field;
         if CommandMode = cmSort then
          PI.IsVisible := (PI.Table.Columns.SortFields.IndexOf(PI.Field.Fieldname) <> -1)
        else if CommandMode = cmColumns then
          PI.IsVisible := (PI.Table.Columns.IndexOf(PI.Field.Fieldname) <> -1)
        else
          PI.FIsVisible := PI2.IsVisible;
         PI.FCommand := PI2.Command;
         PI.FCreator := wcManager;
         PI.FFieldLogOpButton := PI2.FieldLogopButton;
         PI.FValueLogOpButton := PI2.ValueLogopButton;
         PI.FExpanded := PI2.Expanded;
         PI.FExpandControl := PI2.FExpandControl;
         if (CommandMode = cmSearch) and (TableCount > 1) then
          FWorkFields.AddObject(PI.Table.Alias + '.' + PI.Field.Caption, PI)
         else
          FWorkFields.AddObject(PI.Field.Caption, PI);
       end;
       W.FTbl.WriteControls(W.FValues, TILWorkField(FWorkFields.Objects[0]).Usage);
       ChangeWorkFields;
       EXIT;
     end;

     P := nil;
     if CommandMode = cmColumns then
     begin
       for i := 0 to ATable.Columns.Count-1 do
       begin
          PI := TILWorkField.Create;
          PI.FTable := ATable;
          Indx := ATable.Fields.IndexOf(ATable.Columns[i].FFieldName);
          PI.Field := ATable.Fields.Items[Indx];
          PI.IsVisible := True;
          PI.Command := cmColumns;
          PI.FCreator := wcManager;
          FWorkFields.AddObject(PI.Field.Caption, PI);
       end;
       for i := 0 to ATable.Datahandler.DataFields.Count-1 do
       begin
          FldDef := ATable.DataHandler.DataFields.Items[i].FieldDef;
          if ATable.DataHandler.DataFields.Items[i].FieldName = '' then
            Continue;
          if FldDef.FieldType = ftStoredEvaluation then
            Continue;
          Indx := ATable.Columns.IndexOf(FldDef.FieldName);
          if (Indx = -1) then
          begin
           PI := TILWorkField.Create;
           PI.FTable := ATable;
           PI.Field := FldDef;
           PI.IsVisible := False;
           PI.Command := cmColumns;
           PI.FCreator := wcManager;
           FWorkFields.AddObject(PI.Field.Caption, PI);
          end;
       end;

       ChangeWorkFields;
       Exit;
     end
     else if CommandMode = cmSort then
     begin
       for i := 0 to ATable.Columns.Sortfields.Count-1 do
       begin
          PI := TILWorkField.Create;
          PI.FTable := ATable;
          Indx := ATable.Fields.IndexOf(ATable.Columns.Sortfields[i]);
          PI.Field := ATable.Fields.Items[Indx];
          PI.IsVisible := True;
          PI.Command := cmSort;
          PI.FCreator := wcManager;
          FWorkFields.AddObject(PI.Field.Caption, PI);
       end;
       for i := 0 to ATable.Datahandler.DataFields.Count-1 do
       begin
          FldDef := ATable.DataHandler.DataFields.Items[i].FieldDef;
          if ATable.DataHandler.DataFields.Items[i].FieldName = '' then
            Continue;
          if FldDef.FieldType = ftStoredEvaluation then
            Continue;
          if FldDef.DataType in [dtLongText, dtText] then
            Continue;

          Indx := ATable.Columns.Sortfields.IndexOf(FldDef.FieldName);
          if (Indx = -1) then
          begin
           PI := TILWorkField.Create;
           PI.FTable := ATable;
           PI.Field := FldDef;
           PI.IsVisible := False;
           PI.Command := cmSort;
           PI.FCreator := wcManager;
           FWorkFields.AddObject(PI.Field.Caption, PI);
          end;
       end;
       ChangeWorkFields;
       Exit;
     end;
     if ATable <> nil then
       P := FindPanel(ATable, CommandMode);
     if P <> nil then
     begin
      FPanelBuild := True;
      try
       for I := 0 to P.Count-1 do
       begin
         if not (csDesigning in ComponentState) then
        begin
         if (P.Items[i].Table <> ATable) and (CommandMode <> cmSearch) then
          Continue;
         if (CommandMode = cmUpdate) and (P.Items[i].Usage = duSearch) then
          Continue;
         if (CommandMode = cmSearch) and (P.Items[i].Usage = duData) then
          Continue;
         if (CommandMode = cmSearch) then
         begin
            if (P.Items[i].FTable <> ActiveTable)
             and not P.Items[i].FTable.Datahandler.PropagateSearchFields then
              Continue;
            if not ActiveTable.IsFreeTable and not IsInSearchPath(ActiveTable, P.Items[i].Table) then
             Continue;
            if (ActiveTable.MasterLinkCount > 0) then
            begin
               if not IsInSearchPath(ActiveTable, P.Items[i].Table) then
                 Continue;
            end
            else if ActiveTable.IsFreeTable and (P.Items[i].FTable <> ActiveTable) then
                Continue;

            if Assigned(ActiveTable.DataHandler.FOnCreateWorkfield) then
            begin
              Accept := True;
              ActiveTable.DataHandler.FOnCreateWorkfield(ActiveTable, P.Items[i], Accept);
              if not Accept then
                Continue;
            end;

         end;
         if Assigned(ActiveTable.DataHandler.FOnCreateWorkfield) then
          begin
              ActiveTable.DataHandler.FOnCreateWorkfield(ActiveTable, P.Items[i], Accept);
              if not Accept then
                Continue;
          end;
        end;

        PI := TILWorkField.Create;
        PI.FControl := P.Items[i].FControl;
        PI.FLabel := P.Items[i].FLabel;
        PI.FPanel := P.Items[i].FPanel;
        PI.Usage := P.Items[i].Usage;
        PI.FTable := P.Items[i].FTable;
        PI.FField := P.Items[i].Field;
        if CommandMode = cmSearch then
        begin
          Indx := PI.Table.DataHandler.DataFields.IndexOf(PI.Field.FieldName);
          if Indx <> -1 then
             PI.FIsVisible := PI.Table.DataHandler.DataFields.Items[Indx].DacSearchVisible
           else
             PI.FIsVisible :=  false;
        end
        else if CommandMode = cmUpdate then
        begin
          if (PI.Table.DataHandler.Table = ATable) then
          begin
             Indx := PI.Table.DataHandler.DataFields.IndexOf(PI.Field.FieldName);
             if Indx <> -1 then
               PI.FIsVisible := PI.Table.DataHandler.DataFields.Items[Indx].DacUpdateVisible
             else
               PI.FIsVisible :=  false;
          end
          else
           PI.FIsVisible := false;
        end
        else
          PI.FIsVisible := True;
        PI.FCommand := CommandMode;
        PI.FCreator := wcManager;
        PI.FFieldLogOpButton := P.Items[i].FieldLogopButton;
        PI.FValueLogOpButton := P.Items[i].ValueLogopButton;
        PI.FExpanded := P.Items[i].Expanded;
        PI.FExpandControl := P.Items[i].FExpandControl;
        if (CommandMode = cmSearch) and (TableCount > 1) then
          FWorkFields.AddObject(PI.Table.Alias + '.' + PI.Field.Caption, PI)
        else
          FWorkFields.AddObject(PI.Field.Caption, PI);
       end;
      finally
        FPanelBuild := false;
      end;
     end
     else
     begin
       for I := 0 to ATable.Fields.Count-1 do
       begin
          if not ATable.Fields.Items[i].ReportField then
            Continue;
          if CommandMode  = cmSearch then
          begin
           if ATable.Fields.Items[i].SearchControl <> nil then
           begin
              DoCreateCtrl(ATable.Fields.Items[i], duSearch);
            end
          end
          else if CommandMode  = cmUpdate then
          begin
           if ATable.Fields.Items[i].DataControl <> nil then
           begin
            DoCreateCtrl(ATable.Fields.Items[i], duData);
           end;
          end
          else if CommandMode <> cmSQL then
          begin
            DoCreateCtrl(ATable.Fields.Items[i], duSearch);
          end;
       end;
     end;
     ChangeWorkFields;
   end;

   function TTableManager.GetWorkFields(Index : integer) : TILWorkField;
   begin
     if not UseCommandModes then
     begin
       Result := nil;
       Exit;
     end;
     Result := TILWorkField(FWorkFields.Objects[Index]);
   end;

   function TTableManager.GetWorkFieldCount : integer;
   begin
     if not UseCommandModes then
     begin
       Result := 0;
       Exit;
     end;
     Result := FWorkFields.Count;
   end;

   procedure TTableManager.EnablePanel(APanel : TILControlPanel; Enabl : Boolean);
   begin
     if (WorkStyle = wsCommandModes) and not (csDesigning in ComponentState) then
     if not (csDesigning in ComponentState) then

       APanel.EnableCtrls(Enabl);
   end;

   procedure TTableManager.EnablePanels;
   var
    i, j : integer;
    Fld : TILFieldDef;
    Usg : TDataUsage;
    LB : TILLogOpButton;
    InPth : Boolean;
    Enabl : BooleaN;

    function IsWorkField(Fld : TILFieldDef; var Usg : TDataUsage) : Boolean;
    var
      I : integer;
      W : TILWorkField;
    begin
      Result := False;
      for I := 0 to WorkFieldCount-1 do
      begin
         W := WorkFields[i];
         if W.FPanel <> nil then
           Exit;
         if W.Control <> nil then
         begin
           if Fld.DataControl = W.Control then
           begin
             Result := True;
             Usg := duData;
           end
           else if Fld.SearchControl = W.Control then
           begin
             Result := True;
             Usg := duSearch;
           end;
           if Result then
             Exit;
         end;
      end;
    end;

   begin
     if csDesigning in ComponentState then
       Exit;
     if WorkStyle <> wsCommandModes then
     { Disable all controls outside the active table}
     begin
       for I := 0 to PanelCount-1 do
       begin
          for j := 0 to Panels[i].Count-1do
            Panels[i].Items[j].Enabled :=
            (Panels[i].Items[j].Table = TMan.ActiveTable)
            or ((Panels[i].Items[j].Usage = duSearch)
            and Tman.IsInSearchPath(Panels[i].Items[j].Table, TMan.ActiveTable));
       end;
       Exit;
     end;
     for I := 0 to PanelCount-1 do
     begin
       if CommandMode in [cmSort, cmColumns] then
         EnablePanel(Panels[i], False)
       else if Panels[i] <> FindPanel(ActiveTable, CommandMode) then
         EnablePanel(Panels[i], False)
       else
       begin
         EnablePanel(Panels[i], True);
       end;
     end;

     { Check controls that are not in panels }
     for I := 0 to TableCount-1 do
     begin
       for j := 0 to Tables[i].Fields.Count-1 do
       begin
           Fld := Tables[i].Fields.Items[j];
           if IsWorkField(Fld, Usg) then
           begin
              if Usg = duData then
              begin
                if Fld.DataControl = nil then
                  Continue;
                if Fld.DataLabel <> nil then
                begin
                  Fld.DataLabel.Enabled := not Fld.IsReadOnly or (csDesigning in ComponentState);
                  Fld.DataControl.Enabled := Fld.DataLabel.Enabled;
                end
                else
                  Fld.DataControl.Enabled := not Fld.IsReadOnly or (csDesigning in ComponentState);
              end
              else
              begin
                if Fld.SearchControl = nil then
                  Continue;
                InPth := IsInSearchPath(ActiveTable, Fld.Ilib);
                Enabl := (InPth or (csDesigning in ComponentState)) or (Inpth and ActiveTable.DataHandler.FreeSearch);
                if Fld.SearchLabel <> nil then
                  Fld.SearchLabel.Enabled := Enabl;
                Fld.SearchControl.Enabled := Enabl;
                LB := LogOpButtonOfControl(Fld.SearchControl, ltField);
                if LB <> nil then
                  LB.Enabled := Enabl;
                LB := LogOpButtonOfControl(Fld.SearchControl, ltValue);
                if LB <> nil then
                  LB.Enabled := Enabl;
              end
           end
           else
           begin
             if (Fld.DataControl <> nil) and not (Fld.DataControl.Parent is TILControlPanel) then
             begin
                if Fld.DataLabel <> nil then
                  Fld.DataLabel.Enabled := (csDesigning in ComponentState);
                Fld.DataControl.Enabled := (csDesigning in ComponentState);
             end;
             if (Fld.SearchControl <> nil) and not (Fld.SearchControl.Parent is TILControlPanel) then
             begin
                InPth := IsInSearchPath(ActiveTable, Fld.Ilib);
                Enabl := (csDesigning in ComponentState) or (Inpth and ActiveTable.DataHandler.FreeSearch);
                if Fld.SearchLabel <> nil then
                  Fld.SearchLabel.Enabled := Enabl;
                Fld.SearchControl.Enabled :=  Enabl;
                LB := LogOpButtonOfControl(Fld.SearchControl, ltField);
                if LB <> nil then
                  LB.Enabled := Enabl;
                LB := LogOpButtonOfControl(Fld.SearchControl, ltValue);
                if LB <> nil then
                  LB.Enabled := Enabl;
             end;
           end;
           if Fld.DataControl is TDateTimePicker then
           with Fld.DataControl as TDateTimePicker do
           begin
             if Fld.DateFormat <> '' then
               Format := Fld.DateFormat
             else if Translator.FFormatSettings.ShortDateFormat <> FormatSettings.ShortDateFormat then
               Format := Translator.FFormatSettings.ShortDateFormat;
           end;
           if Fld.SearchControl is TDateTimePicker then
           with Fld.SearchControl as TDateTimePicker do
           begin
             if Fld.DateFormat <> '' then
               Format := Fld.DateFormat
             else if Translator.FFormatSettings.ShortDateFormat <> FormatSettings.ShortDateFormat then
               Format := Translator.FFormatSettings.ShortDateFormat;
           end;
       end;
     end;
end;

procedure TTableManager.InitWorkFields;
var
  Pan : TILControlPanel;
  S : string;
  i : integer;
  Active : TILTable;
  Tbl : TILTable;
begin
   Active := ActiveTable;
   if not UseCommandModes then
   begin
     for I := 0 to RegPanels.Count-1 do
      begin
        TILControlPanel(RegPanels[i]).ShowCtrls(TILControlPanel(RegPanels[i]).FItems);
      end;
     Exit;
   end;
   Active.FActive := false;
   for I := 0 to TMan.TableCount-1 do
   begin
     Tbl := TMan.Tables[i];
     Tbl.FActive := True;
     CreateWorkFields(Tbl, cmSearch);
     Tbl.FActive := False;
     Pan := FindPanel(TBl, cmSearch);
     if Pan <> nil then
     begin
      if Tbl.DataHandler.DisableCommandModes then
      begin
        Pan.ShowCtrls(Pan.FItems);
        Exit;
      end;
      Pan.ShowCtrls(FWorkFields);
     end;
   end;
   Active.FActive := True;
   FWorkHistory.Clear;
   CreateSearchFieldOrder;
   EnablePanels;
   if ActiveTable.HasHandler and Assigned(ActiveTable.DataHandler.FOnUpdateLongText)
   then
    begin
       S := '';
       ActiveTable.DataHandler.FOnUpdateLongText(ActiveTable.DataHandler, nil, S, [ltaDisable])
    end;
   WriteBrowser(FindPanel(ActiveTable, cmUpdate));
   CreateWorkFields(ActiveTable, cmSearch);
end;



function TTableManager.Buble : integer;
var
  i : integer;
  HI : TILWorkField;
  Cnt : integer;
  Indx : integer;
  Moved : Boolean;
begin
  Result := -1;
  if ActiveTable = nil then
    Exit;
  if not (CommandMode in [cmSearch, cmSort, cmColumns]) then
    Exit;
  if ActiveTable.AlignOperators and (CommandMode = cmSearch) then
    Exit;
  Cnt := 0;
  Indx := 0;
  repeat
   Moved := false;
   for i := Indx to Worklist.Count-1 do
   begin
     HI := WorkList.Items.Objects[i] as TILWorkField;
     if HI.IsVisible then
     begin
      WorkList.Items.Move(i, Cnt);
      FWorkFields.Move(i, Cnt);
      Moved := True;
      inc(Cnt);
      Indx := i+1;
      Break;
     end;
   end;
  until not Moved;
  Result := Cnt-1;
end;

procedure TTableManager.DoShowWorkFields(APanel : TILControlPanel);
var
  i : integer;
  P : TILControlPanel;

begin
  if (FWorkFields.Count = 0) then
  begin
     for I := 0 to RegPanels.Count-1 do
      begin
        P := TILControlPanel(RegPanels[i]);
        if P = APanel then
          P.Clear;
      end;
  end
  else
  if APanel <> nil then
    APanel.ShowCtrls(FWorkFields)

end;

procedure TTableManager.WriteBrowser(APanel : TILControlPanel);
var
  P : TILControlPanel;
begin
  if APanel = nil then
    Exit;
  P := FindPanel(ActiveTable, cmSearch);
  if P = APanel then
    Exit;
  APanel.Clear;
  APanel.ShowCtrls(ActiveTable, duData);
end;

function TTableManager.UseCommandModes : Boolean;
begin
    Result := (WorkStyle = wsCommandModes);
    if Result and (ActiveTable <> nil) and (ActiveTable.DataHandler <> nil) then
     if ActiveTable.DataHandler.DisableCommandModes then
       Result := false;
end;

procedure TTableManager.HandleTransError(E : Exception);
begin
   Comment1 := E.Message;
   TMan.RollbackTrans;
   HandleILException(E_TransError);
end;

procedure TTableManager.RebuildAllIndexes;
  var
      i : integer;
    begin
      TMan.Root.Table.RebuildIndexes;
      for I := 0 to TMan.Root.CountAll-1 do
      begin
        if TMan.Root.AllChildren[i].Table.Fields.HasIndexes then
        begin
         TMan.Progress(TMan.Root.AllChildren[i].Table, iloChangeOperation, 0, 0);
         TMan.Root.AllChildren[i].Table.RebuildIndexes;
        end;
      end;
      for I := 0 to TMan.FreeTableCount-1 do
      begin
        if TMan.FreeTables[i].Fields.HasIndexes then
        begin
          TMan.Progress(TMan.FreeTables[i], iloChangeOperation, 0, 0);
          TMan.FreeTables[i].RebuildIndexes;
        end;
      end;
    end;

procedure TTableManager.UpdateWorkList;
var
  I: Integer;
begin
  if not UseCommandModes then
    Exit;
  if WorkList = nil then
    Exit;
  if csDesigning in ComponentState then
  begin
    WorkList.Clear;
    Exit;
  end;
  { Worklist is not maintained design time }
  WorkList.Items := FWorkFields;
  if WorkList.Count > 0 then
      WorkList.ItemIndex := 0;
   for I := 0 to WorkList.Count-1 do
   begin
     if WorkFields[i].IsVisible then
       Worklist.State[i] := cbChecked;
   end;
  Buble;
end;

procedure TTableManager.SaveToHistory(Tbl : TILTable; const Cmd : TCommandMode; WorkFields: TStringList);
var
  W : TWorkHistoryElement;
  Usg : TDataUsage;
  PI : TILWorkField;
  i: Integer;
  PI2 : TILWorkField;
  Indx : integer;
begin
   if not UseCommandModes then
     Exit;
   if WorkFields.Count = 0 then
     Exit;
   W := GetFromHistory(Tbl, Cmd);
   if W <> nil then
   begin
      Indx := FWorkHistory.IndexOf(W);
      FWorkHistory.Delete(Indx);
      W.Free;

   end;
   W := TWorkHistoryElement.Create;
   W.FTbl := Tbl;
   W.FCmd := Cmd;
   W.FWorkFields := TStringList.Create;
   for i := 0 to WorkFields.Count-1 do
   begin
     PI2 := TILWorkField(WorkFields.Objects[i]);
     PI := TILWorkField.Create;
     PI.FControl := PI2.FControl;
     PI.FLabel := PI2.FLabel;
     PI.FPanel := PI2.FPanel;
     PI.Usage := PI2.Usage;
     PI.FTable := PI2.FTable;
     PI.FField := PI2.Field;
     PI.FIsVisible := PI2.IsVisible;
     PI.FCommand := PI2.Command;
     PI.FFieldLogOpButton := PI2.FieldLogopButton;
     PI.FValueLogOpButton := PI2.ValueLogopButton;
     PI.FExpanded := PI2.Expanded;
     PI.FExpandControl := PI2.FExpandControl;
     if (CommandMode = cmSearch) and (TableCount > 1) then
       W.FWorkFields.AddObject(PI.Table.Alias + '.' + PI.Field.Caption, PI)
     else
       W.FWorkFields.AddObject(PI.Field.Caption, PI);
   end;

   W.FValues := TILDataRecord.Create(Tbl, false);
   Usg := TILWorkField(WorkFields.Objects[0]).Usage;
   Tbl.ReadControls(W.FValues, Usg);
   FWorkHistory.Add(W);
end;

function TTableManager.GetFromHistory(Tbl : TILTable; const Cmd : TCommandMode) : TWorkHistoryElement;
var
  i : integer;
  W : TWorkHistoryElement;
begin
  Result := nil;
  for I := 0 to FWorkHistory.Count-1 do
  begin
     W := TWorkHistoryElement(FWorkHistory[i]);
     if (W.FTbl = Tbl) and (W.FCmd = Cmd) then
     begin
       Result := W;
       Break;
     end;
  end;
end;

function TTableManager.GetProgressText(Operation : TILOperation) : string;
begin
    case Operation of
      iloSearch: Result := Translator.GetTranslation(T_PGSearch, TRANSL_TERMS);
      iloUpdate: Result := Translator.GetTranslation(T_PGUpdate, TRANSL_TERMS);
      iloDelete: Result := Translator.GetTranslation(T_PGDelete, TRANSL_TERMS);
      iloRemoveDeleted: Result := Translator.GetTranslation(T_PGRemoveDeleted, TRANSL_TERMS);
      iloRecreateImages: Result := Translator.GetTranslation(T_PGRecreateImages, TRANSL_TERMS);
      iloProcessTransLog: Result := Translator.GetTranslation(T_PGProcessTransLog, TRANSL_TERMS);
      iloRecreateTextLobs: Result := Translator.GetTranslation(T_PGRecreateTextLobs, TRANSL_TERMS);
      iloAlterIndexSpace: Result := Translator.GetTranslation(T_PGAlterIndexSpace, TRANSL_TERMS);
      iloAlterRecordSize: Result := Translator.GetTranslation(T_PGAlterRecordSize, TRANSL_TERMS);
      iloExtractIndexes: Result := Translator.GetTranslation(T_PGExtractIndexes, TRANSL_TERMS);
      iloBuildIndexes: Result := Translator.GetTranslation(T_PGBuildIndexes, TRANSL_TERMS);
      iloAlterTableSpace: Result := Translator.GetTranslation(T_PGAlterTableSpace, TRANSL_TERMS);
      iloAlterRecordFreeSpace: Result := Translator.GetTranslation(T_PGAlterRecordFreeSpace, TRANSL_TERMS);
      iloAlterRecordLayout: Result := Translator.GetTranslation(T_PGAlterRecordLayout, TRANSL_TERMS);
      iloReadFile: Result := Translator.GetTranslation(T_PGReadFile, TRANSL_TERMS);
    end;
end;

function TTableManager.GetMBarFromTable(ATable : TILTable) : TObject;
var
  i : integer;
  MB : TILMessageBar;
begin
  Result := nil;
  for I := 0 to RegMBs.Count-1 do
  begin
     MB := TILMessageBar(RegMBs[i]);
     if MB.Table = ATable then
     begin
       Result := MB;
       Break;
     end;
  end;
end;

{ TTranslator -------------------------------------------------------------- }

procedure TTranslator.SetLanguage(Language: string);
var
  Reg: TRegistry;
  F : string;
begin
  F := Application.ExeName;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey('Software\Embarcadero\Locales', True) then
      Reg.WriteString(F, Language)
    else if Reg.OpenKey('\Software\CodeGear\Locales', True) then
      Reg.WriteString(F, Language)
    else if Reg.OpenKey('\Software\Borland\Locales', True) then
      Reg.WriteString(F, Language)
    else if Reg.OpenKey('\Software\Borland\Delphi\Locales', True) then
      Reg.WriteString(F, Language);
  finally
    Reg.Free;
  end;
end;

function TTranslator.GetLanguage : string;
var
  Reg: TRegistry;
  F : string;
begin
  F := Application.ExeName;
  Reg := TRegistry.Create;
  Result := 'EN';
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey('Software\Embarcadero\Locales', False) then
      Result := Reg.ReadString(F)
    else if Reg.OpenKey('\Software\CodeGear\Locales', False) then
      Result := Reg.ReadString(F)
    else if Reg.OpenKey('\Software\Borland\Locales', False) then
      Result := Reg.ReadString(F)
    else if Reg.OpenKey('\Software\Borland\Delphi\Locales', False) then
      Result := Reg.ReadString(F);
  finally
    Reg.Free;
  end;
end;

function TTranslator.GetTranslation(const ID: integer; const TranslType: integer): string;
begin
    if TranslType = TRANSL_USER_ERRORS then
    begin
        Result := E_ErrorText[ID];
    end
    else
    begin
        Result := T_Terms[ID];
    end;
end;

function TTranslator.GetTranslation(const FieldNames : Boolean; const TheName: string): string;
var
  i: integer;
  s : string;
begin
  Result := '';
  if FieldNames then
  for i := 0 to High(TranslateFieldCaps) do
  begin
    s := Trim(TranslateFieldCaps[i].Name);
    if SameText(s, TheName) then
    begin
      Result := TranslateFieldCaps[i].Translation;
      Break;
    end;
  end
  else
  for i := 0 to High(TranslateTableCaps) do
  begin
    s := Trim(TranslateTableCaps[i].Name);
    if SameText(s, TheName) then
    begin
      Result := TranslateTableCaps[i].Translation;
      Break;
    end;
  end
end;

function TTranslator.FormatNum(const Anumber : integer) : string;
begin
   Result := FormatFloat('0,', ANumber, FormatSettings);

end;

function TTranslator.FormatNum(const Anumber : Extended; Decimals : integer) : string;
var
  FmtStr : string;
  i : integer;
begin
   FmtStr := '0';
   if Decimals > 0 then
   begin
    FmtStr := '0.';
    for I := 1 to Decimals do
      FmtStr := FmtStr + '0';
   end;
   Result := FormatFloat(FmtStr, ANumber, FormatSettings);

end;

function TTranslator.FormatDT(const ADate : TDateTime; const AFormat : string) : string;
begin
   if AFormat <> '' then
    Result := FormatDateTime(AFormat, ADate, FFormatSettings)
   else
    Result := FormatDateTime('', ADate, FFormatSettings)
end;

function TTranslator.FormatStr(const AStr, A1, A2, A3 : string) : string;
begin
    Result := Format(AStr, [A1, A2, A3]);
end;

procedure TTranslator.AddFieldName(const FieldName: string; const Translation: string);
var
  i : integer;
begin
  if not NameCheck(Translation) then
    HandleILException(E_InvalidChar);
  if not NameCheck(FieldName) then
    HandleILException(E_InvalidChar);
  for I := 1 to High(FTranslateFieldCaps) do
  begin
    if SameText(FieldName, FTranslateFieldCaps[i].Translation) then
    begin
      Comment1 := FieldName;
      HandleILException(E_DupFieldName)
    end;
  end;
  SetLength(FTranslateFieldCaps, Length(TranslateFieldCaps) + 1);
  TranslateFieldCaps[High(TranslateFieldCaps)].Name := FieldName;
  TranslateFieldCaps[High(TranslateFieldCaps)].Translation := Translation;
end;

procedure TTranslator.AddTableName(const TableName: string; const Translation: string);
var
  i : integer;
begin
  if not NameCheck(TableName) then
    HandleILException(E_InvalidChar);
  if not NameCheck(Translation) then
    HandleILException(E_InvalidChar);
  for I := 1 to High(FTranslateTableCaps) do
  begin
    if SameText(TableName, FTranslateTableCaps[i].Translation) then
    begin
      Comment1 := TableName;
      HandleILException(E_DupTableName)
    end;
  end;
  SetLength(FTranslateTableCaps, Length(TranslateTableCaps) + 1);
  TranslateFieldCaps[High(TranslateFieldCaps)].Name := TableName;
  TranslateFieldCaps[High(TranslateFieldCaps)].Translation := Translation;
end;

constructor TTranslator.Create;
var
  Lang : string;
begin
    Lang := Language;
    if Lang <> '' then
      FFormatSettings := TFormatSettings.Create(Language)
    else
      FFormatSettings := TFormatSettings.Create;
    FEnFormatSettings := TFormatSettings.Create('EN');
end;

procedure TTranslator.ClearCaps;
begin
  Finalize(FTranslateFieldCaps);
  Finalize(FTranslateTableCaps);
end;

destructor TTranslator.Destroy;
begin
  ClearCaps;
  inherited;
end;

{ TILWorkField ------------------------------------------------------------ }

procedure TILWorkField.Expand(RowCount : integer = 3);
var
 i : integer;
 Cnt : integer;
 MinWidth : integer;
 PLWidth : integer;
 RNum : integer;
 Indx : integer;

 procedure UpdatePanel(CmdMode : TCommandMode; AOldCtrl, ANewCtrl : TWinControl);
 var
   P : TILControlPanel;
   i : integer;
 begin
    P := Tman.FindPanel(Table, CmdMode);
    if P = nil then
      Exit;
    for I := 0 to P.Count-1 do
    begin
      if P.Items[i].FControl = AOldCtrl then
      begin
       P.Items[i].FControl := ANewCtrl;
       break;
      end;

    end;
 end;

begin
   if FExpanded then
     Exit;
   if FControl = nil then
     Exit;
   RNum := 0;
   if FControl is TILSearchGrid then
     if TilSearchGrid(FControl).RowCount >= RowCount then
       Exit;
   if FControl is TIlResourceList then
     Exit;
   if (FUsage = duData) and not Table.MultiSelected then
     Exit;

   FOrigControl := FControl;
   Tman.FPanelBuild := True;
   try
   FControl.Visible := False;
   if not Assigned(FExpandControl) then
   begin
       if FUsage = duSearch then
       begin
         FExpandControl := TILSearchGrid.Create(FControl.Parent);
         FExpandControl.Parent := FControl.Parent;
         if FExpandControl.Name = '' then
          FExpandControl.Name := FOrigControl.Name + 'Exp';
         RNum := 0;
       end
       else
       begin
         FExpandControl := TILUpdateGrid.Create(FControl.Parent);
         FExpandControl.Parent := FControl.Parent;
         if FExpandControl.Name = '' then
          FExpandControl.Name := FOrigControl.Name + 'Exp';
         TILUpdateGrid(FExpandControl).Mode := gmUpdateActions;
         RNum := 1;
       end;
       FExpandControl.RowCount := RowCount + 1;
       FExpandControl.AlignValueCol := True;
       FExpandControl.AutoSizeHeight := True;
       FExpandControl.Left := FControl.Left;
       FExpandControl.Top := FControl.Top;

       if FUsage = duSearch then
       with FExpandControl as TILSearchGrid do
       begin
         if FPanel <> nil then
         begin
          CoPickList := (doCoPicklist in FPanel.DualCtrlOptions);
          if not (doLogPicklist in FPanel.DualCtrlOptions) then
            LogPickList := false;
         end
         else
         begin
           LogPickList := True;
         end;
         PLWidth := 0;
         if CoPickList then
           PLWidth := ColWidths[CoOpColIndex];
         if LogPickList then
           PLWidth := PLWidth + FExpandControl.ColWidths[LogOpColIndex];
         if FControl.ClientWidth < 50 then
           MinWidth := 50 + PLWidth
         else
           MinWidth := FControl.ClientWidth + PlWidth;
       end
       else
       with FExpandControl as TILUpdateGrid do
       begin
         PLWidth := GetActionColWidth + 25;
         if FControl.ClientWidth < 50 then
           MinWidth := 50 + PLWidth
         else
           MinWidth := FControl.ClientWidth + PlWidth;
       end;
       if FControl.Left + MinWidth > FControl.Parent.ClientWidth then
         MinWidth := MinWidth - (MinWidth + FControl.Left - FControl.Parent.Width) - 6;
       FExpandControl.Width := MinWidth;

       if Field.LookupType = looPickValues then
       begin
         if (FControl is TComboBox) then
           with FControl as TComboBox
           do FExpandControl.PickValues.Assign(Items)
         else if FControl is TRadioGroup then
           with FControl as TRadioGroup
           do FExpandControl.PickValues.Assign(Items)
         else if FControl is TCheckListBox then
           with FControl as TCheckListBox
           do FExpandControl.PickValues.Assign(Items);
       end;
       FControl := FExpandControl;

       if Fusage = duSearch then
       begin
         FField.FSearchControl := FExpandControl;
         FExpandControl.Field := FField;
       end
       else
       begin
         FField.FDataControl := FExpandControl;
         FExpandControl.Field := FField;
       end;
       if FField.IsLookup then
         Table.SetLookup(FExpandControl, FField);
       if FFieldLogOpButton <> nil then
          FFieldLogOpButton.FSearchControl := FExpandControl;
       if FValueLogOpButton <> nil then
          FValueLogOpButton.FSearchControl := FExpandControl;
       Indx := Table.DataHandler.DataFields.IndexOf(FField.FieldName);
       if Indx <> -1 then
          Table.DataHandler.DataFields.Items[Indx].FSearchControl := FExpandControl;

   end
   else
   begin
     FExpandControl.Visible := True;
     if Fusage = duSearch then
     begin
       FField.FSearchControl := FExpandControl;
     end
     else
     begin
       FField.FDataControl := FExpandControl;
       if TILUpdateGrid(FExpandControl).Mode = gmUpdateActions then
         RNum := 1;
     end;
     FControl := FExpandControl;
   end;

   if FUsage = duSearch then
     UpdatePanel(cmSearch, FOrigControl, FExpandControl)
   else
     UpdatePanel(cmUpdate, FOrigControl, FExpandControl);

   FExpandControl.ClearValues;
   if FOrigControl is TComboBox then
    with FOrigControl as TComboBox
    do FExpandControl.Values[RNum] := Text
   else if FOrigControl is TRadioGroup then
    with FOrigControl as TRadioGroup
    do FExpandControl.Values[RNum] := Items[ItemIndex]
   else if (FOrigControl is TEdit) or (FOrigControl is TLabeledEdit)  then
   with FOrigControl as TEdit
     do FExpandControl.Values[RNum] := Text
   else if FOrigControl is TDateTimePicker then
   with FOrigControl as TDateTimePicker
     do FExpandControl.Values[RNum] := DateTimeToStr(DateTime, Translator.FFormatSettings)
   else if FOrigControl is TCheckBox then
   begin
     with FOrigControl as TCheckBox
     do begin
       if State = cbChecked then
         FExpandControl.Values[Rnum] := TMan.ReportTrue
       else if State = cbUnChecked then
         FExpandControl.Values[Rnum] := TMan.ReportFalse
       else
         FExpandControl.Values[RNum] := '';
     end
   end
   else if FOrigControl is TMemo then
   with FOrigControl as TMemo do
   begin
    for I := 0 to Lines.Count-1 do
     FExpandControl.Values[i+RNum] := Lines[i];
   end
   else if FOrigControl is TCheckListBox then
   with FOrigControl as TCheckListBox do
   begin
      FExpandControl.ClearValues;
      Cnt := 0;
      for I := 0 to Items.Count-1 do
      begin
        if Checked[i] then
        begin
          if FExpandControl.RowCount-RNum < Cnt +1 then
            FExpandControl.RowCount := FExpandControl.RowCount + 1;
          FExpandControl.Cells[FExpandControl.ValueColIndex, Cnt+Rnum] := Items[i];
          inc(Cnt);
        end;
      end;
   end;
   if (Field.PickValues.Count > 0) then
     FExpandControl.RelocateComboBox
   else if FField.LookupType in [looTableKey, looTableRef] then
     FExpandControl.RelocateComboBox;

   FExpanded := True;
   finally
     TMan.FPanelBuild := false;
   end;
   if Panel <> nil then
   begin
       Panel.ShowCtrls(Panel.FItems);
   end;
   if FExpandControl.CanFocus then
     FExpandControl.SetFocus;
   Tman.ChangeWorkFields;
end;

procedure TILWorkField.Collapse;
var
  Dt : TDateTime;
  i : integer;
  Indx : integer;
  RNum : integer;

 procedure UpdatePanel(CmdMode : TCommandMode; AOldCtrl, ANewCtrl : TWinControl);
 var
   P : TILControlPanel;
   i : integer;
 begin
    P := Tman.FindPanel(Table, CmdMode);
    if P = nil then
      Exit;
    for I := 0 to P.Count-1 do
    begin
      if P.Items[i].FControl = AOldCtrl then
      begin
       P.Items[i].FControl := ANewCtrl;
       break;
      end;
    end;
 end;

begin
   if not FExpanded then
     Exit;
   if FControl = nil then
     Exit;
   if FUsage = duSearch then
   begin
     FExpandControl := FField.SearchControl as TILCustomGrid;
     RNum := 0;
     if FFieldLogOpButton <> nil then
       FFieldLogOpButton.FSearchControl := FOrigControl;
     if FValueLogOpButton <> nil then
       FValueLogOpButton.FSearchControl := FOrigControl;
     Indx := Table.DataHandler.DataFields.IndexOf(FField.FieldName);
     if Indx <> -1 then
       Table.DataHandler.DataFields.Items[Indx].FSearchControl := FOrigControl;
     UpdatePanel(cmSearch, FExpandControl, FOrigControl);
   end
   else
   begin
     FExpandControl := FField.DataControl as TILCustomGrid;
     if TILUpdateGrid(FExpandControl).Mode = gmSimple then
       RNum := 0
     else
       RNum := 1;
     UpdatePanel(cmUpdate, FExpandControl, FOrigControl);
   end;
   TMan.FPanelBuild := true;
   try
   FExpandControl.Visible := false;
   FOrigControl.Visible := True;
   if FUsage = duSearch then
     FField.FSearchControl := FOrigControl
   else
     FField.FDataControl := FOrigControl;
   FControl := FOrigControl;

   if FControl is TComboBox then
    with FControl as TComboBox
    do ItemIndex := Items.IndexOf(FExpandControl.Values[RNum])
   else if FControl is TRadioGroup then
    with FControl as TRadioGroup
    do ItemIndex := Items.IndexOf(FExpandControl.Values[RNum])
   else if (FControl is TEdit) or (FControl is TLabeledEdit) then
    with FControl as TCustomEdit
     do Text := FExpandControl.Values[RNum]
   else if FControl is TDateTimePicker then
     with FControl as TDateTimePicker do
     begin
        if TryStrToDatetIme(FExpandControl.Values[RNum], Dt, Translator.FFormatSettings) then
          DateTime := StrToDateTime(FExpandControl.Values[RNum], Translator.FFormatSettings)
     end
   else if FControl is TCheckBox then
   begin
     with FControl as TCheckBox
     do begin
       if SameText(TMan.ReportTrue,FExpandControl.Values[RNum]) then
         Checked := True
       else if SameText(TMan.ReportFalse,FExpandControl.Values[RNum]) then
         Checked := False
       else if AllowGrayed then
         State := cbGrayed;
     end
   end
   else if FControl is TMemo then
   with FControl as TMemo do
   begin
     Lines.Clear;
     for I := RNum to FExpandControl.Cols[FExpandControl.ValueColIndex].Count-1 do
       if FExpandControl.Values[i] <> '' then
       Lines.Add(FExpandControl.Values[i])
   end
   else if FControl is TCheckListBox then
   with FControl as TCheckListBox do
   begin
      CheckAll(cbGrayed);
      for I := RNum to FExpandControl.Cols[FExpandControl.ValueColIndex].Count-1 do
      begin
       if FExpandControl.Values[i] <> '' then
       begin

         Indx := Items.IndexOf(FExpandControl.Values[i]);
         if Indx <> -1 then
          Checked[Indx] := True;
       end;
      end;
   end;
   finally
     TMan.FPanelBuild := false;
   end;
   FExpanded := false;
   if Panel <> nil then
   begin
       Panel.ShowCtrls(Panel.FItems);
   end;
   FControl.SetFocus;
   TMan.ChangeWorkFields;
end;

function TILWorkField.CanExpand : Boolean;
begin
   Result := false;
   if Table.DataHandler = nil then
    Exit;
   if not Visible then
     Exit;
   if Expanded then
     Exit;
   if (Control is TILSearchGrid) or (Control is TILUpdateGrid)
   or (Control is TILResourceList) then
     Exit;
   if (FUsage = duData) and not Table.MultiSelected then
     Exit;
   Result := True;
end;

function TILWorkField.CanCollapse : Boolean;
begin
   Result := false;
   if not Visible then
     Exit;
   if not ((Control is TILSearchGrid) or (Control is TILUpdateGrid)) then
     Exit;
   if Expanded then
     Result := True;
end;

procedure TILWorkField.SetVisible(const Value : Boolean);
begin
  if IsVisible <> Value then
  begin
   IsVisible := Value;
   TMan.UpdateWorklist;
  end;
end;

function TILWorkField.GetVisible : Boolean;
begin
     Result := IsVisible;
end;

procedure TILWorkField.SetEnabled(const Value : Boolean);
begin
  if Control.Enabled = Value then
    Exit;
  Control.Enabled := Value;
  if ControlLabel <> nil then
   ControlLabel.Enabled := Value;
  if FieldLogOpButton <> nil then
   FieldLogOpButton.Enabled := Value;
  if ValueLogOpButton <> nil then
  begin
    if (Control is TCheckListBox) and (Field.MaxValues = 1)
    and (ValueLogOpButton.LogopType = ltValue)then
    begin
      ValueLogOpButton.Enabled := false;
    end
    else
      ValueLogOpButton.Enabled := Value;
  end;
end;

function TILWorkField.GetEnabled : Boolean;
begin
     Result := FControl.Enabled;
end;

{ TILControlPanel -------------------------------------------------------------}

function SortList(Item1, Item2 : pointer) : integer;
var
 P1, P2 : TILWorkField;
 N1 : TTableNode;
 N2 : TTableNode;
begin
  if Tman = nil then
    HandleILException(E_ManagerNotFound);
  P1 := TILWorkField(Item1);
  P2 := TILWorkField(Item2);
  if (P1.Usage = duData) and (P2.Usage = duSearch) then
    Result := 1
  else if (P1.Usage = duSearch) and (P2.Usage = duData) then
    Result := -1
  else if not P1.Field.Ilib.IsFreeTable and not P2.Field.Ilib.IsFreeTable then
  begin
    N1 := TMan.NodeOf(P1.Field.Ilib);
    N2 := TMan.NodeOf(P2.Field.Ilib);
    if N1.Level < N2.Level then
      Result := -1
    else if N1.Level > N2.Level then
      Result := 1
    else
    begin
       if N1.Table = N2.Table then
       begin
         if P1.Field.FieldId < P2.Field.FieldID then
           Result := -1
         else if P1.Field.FieldID > P2.Field.FieldID then
           Result := 1
         else
           Result := 0;
       end
       else
       Result := CompareText(N1.Table.TableName, N2.Table.TableName);
    end;
  end
  else
    Result := CompareText(P1.Field.Ilib.TableName, P2.Field.Ilib.TableName);
end;

function TILControlPanel.GeTILWorkField(Index : integer) : TILWorkField;
begin
    Result := TILWorkField(FItems[Index]);
end;

function TILControlPanel.GetCount : integer;
begin
    Result := FItems.Count;
end;

function TILControlPanel.GetVisibleCount : integer;
var
  i : integer;
begin
   Result := 0;
   for I := 0 to Count-1 do
     if Items[i].Visible then
       inc(Result);
end;

function TILControlPanel.GetVisibles(Index : integer) : TILWorkField;
var
  i, ind : integer;
begin
  Result := nil;
  Ind := 0;
  for I := 0 to Count-1 do
  begin
    if Items[i].Visible then
    begin
       if Index = Ind then
       begin
         Result := Items[i];
         break;
       end
       else
         inc(ind);
    end;
  end;
end;

function TILControlPanel.GetTVParams : TILTreeViewParams;
begin
  Result := nil;
  if Tman = nil then
    Exit
  else if TMan.ActiveTable is TILTreeViewTable then
     Result := TILTreeViewTable(TMan.ActiveTable).TreeViewParams;
end;

function TILControlPanel.GetAbsoluteWidth : integer;
{ Computes how much horiz space controls take up. }
var
  i : integer;
begin
   Result := 0;
   for I := 0 to ControlCount-1 do
     begin
        if (Controls[i].Visible) or (csDesigning in ComponentState) then
          if Controls[i].Left + Controls[i].Width > Result  then
            Result := Controls[i].Left + Controls[i].Width;
     end;
end;

function TILControlPanel.GetAbsoluteHeight : integer;
{ Computes how much vert space controls take up }
var
  i : integer;
begin
   Result := 0;
   for I := 0 to ControlCount-1 do
     begin
        if Controls[i].Visible or (csDesigning in ComponentState) then
          if Controls[i].Top + Controls[i].Height > Result  then
            Result := Controls[i].Top + Controls[i].Height;
     end;
end;

procedure TILControlPanel.DoShowCtrl(const Index : integer; AnItem : TILWorkField; X, Y : integer);
var
  FieldDef : TILFieldDef;
  StartX : integer;

  function LBVisible(LbType : TLogopType) : Boolean;
  begin
     Result := false;
     if LBType = ltField then
     begin
        if (AnItem.FieldLogOpButton <> nil) then
       begin
          AnItem.FieldLogOpButton.Visible := True;
          Result := True;
       end
     end
     else
     begin
       if (AnItem.ValueLogOpButton <> nil) and not AnItem.Expanded then
       begin
         AnItem.ValueLogOpButton.Visible := True;
         Result := True;
       end
     end;
  end;
begin
   StartX := X;
   FieldDef := AnItem.Field;
   if AnItem.Usage = duSearch then
     begin
       if LBVisible(ltField) then
       begin
           if (Index > 0)  then
           begin
             AnItem.FieldLogOpButton.Left := X;
             AnItem.FieldLogOpButton.Top := Y;
             if Assigned(FOnDrawLogOpButton) then
              AnItem.FieldLogOpButton.Invalidate;
           end
           else
           begin
             if csDesigning in ComponentState then
             begin
              AnItem.FieldLogOpButton.Left := -20;
              AnItem.FieldLogOpButton.Top := -20;
             end
             else
               AnItem.FieldLogOpButton.Visible := false;
           end;
           X := X + 19;
       end;

       if FieldDef.SearchLabel <> nil then
       begin
          FieldDef.SearchLabel.Left := X;
          FieldDef.SearchLabel.Top := Y;
          if AnItem.IsVisible then
          begin
            FieldDef.SearchLabel.Visible := True;
          end;
       end;

       if (AnItem.Visible and LBVisible(ltValue))
       or ((csDesigning in ComponentState) and (AnItem.ValueLogOpButton <> nil))
       then
       begin
          FieldDef.SearchControl.Left := StartX + FLabelSpace + 3;
          AnItem.ValueLogOpButton.Left := FieldDef.SearchControl.Left - 17;
          AnItem.ValueLogOpButton.Top := Y;
          if Assigned(FOnDrawLogOpButton) then
           AnItem.ValueLogOpButton.Invalidate;
       end
       else
         FieldDef.SearchControl.Left := StartX + FLabelSpace + 3;
       FieldDef.SearchControl.Top := Y;
       FieldDef.SearchControl.Visible := AnItem.IsVisible;
     end
     else
     begin
       if FieldDef.DataLabel <> nil then
       begin
           FieldDef.DataLabel.Left := X;
           FieldDef.DataControl.Left := X + FLabelSpace + 3;
           FieldDef.DataLabel.Top := Y;
           if AnItem.IsVisible then
           begin
            FieldDef.DataLabel.Visible := True;
           end;
       end;
       FieldDef.DataControl.Top := Y;
       FieldDef.DataControl.Visible := AnItem.IsVisible;
     end;
end;

procedure TILControlPanel.ShowCtrls(Ctrls : TList);
var
  I, J: Integer;
  Y : integer;
  Itm : TILWorkField;
  Found : Boolean;
begin
  if Ctrls.Count = 0 then
   Exit;
  try
  Y := CtrlsTop;
  Clear;
  ComputeLabelSpace;
  TMan.FPanelBuild := True;
  if (LogOpButton <> nil) and (TILWorkField(Ctrls[0]).Usage = duSearch) and ShowLogOpButtons then
  begin
    LogOpButton.Left := CtrlsLeft;
    LogOpButton.Top := CtrlsTop;
    LogOpButton.Visible := True;
    inc(Y, LogOpButton.Height + 5);
    if Assigned(FOnDrawLogOpButton) then
      LogOpButton.Invalidate;
  end;

  for I := 0 to Ctrls.Count-1 do
  begin
     Itm := TILWorkField(Ctrls[i]);
     if not Itm.IsVisible and not (csDesigning in componentState) then
       Continue;
     if (Itm.Table <> TMan.ActiveTable) and not TMan.UseCommandModes and not (csDesigning in componentState) then
     begin
       { If the panel does not contain controls of the active table, dont remove exisiting ones }
       begin
          Found := false;
          for J := 0 to Count-1 do
          begin
            if Items[i].FTable = TMan.ActiveTable then
            begin
              Found := True;
              Break;
            end;
          end;
          if Found then
            Continue;
       end;
     end;
     DoShowCtrl(I, Itm, CtrlsLeft, Y);
     if Itm.Usage = duSearch then
       inc(Y,Itm.Field.SearchControl.Height + ControlSpacing)
     else
       inc(Y,Itm.Field.DataControl.Height + ControlSpacing)
  end;
  finally
    TMan.FPanelBuild := False;
  end;
  if EnableScroller then
    CheckCPScroller;
end;

procedure TILControlPanel.ShowCtrls(Ctrls : TStrings);
var
  L : TList;
  I : integer;
begin
    L := TList.Create;
    try
      for I := 0 to Ctrls.Count-1 do
        L.Add(Ctrls.Objects[i]);
      ShowCtrls(L);
    finally
      L.Free;
    end;
end;

procedure TILControlPanel.ShowCtrls(ATable : TILTable; AUsage : TDataUsage);
var
  I: Integer;
  Itm : TILWorkField;
  L : TList;
begin
  if ATable = nil then
    Exit;
  L := TList.Create;
  try
  for I := 0 to Count-1 do
  begin
     Itm := Items[i];
     if (Itm.Table <> ATable) or not Itm.IsVisible or (Itm.Usage <> AUsage) then
     begin
       Continue;
     end;
     L.Add(Itm);
   end;
   ShowCtrls(L);
  finally
    L.Free;
  end;
end;

procedure TILControlPanel.Clear;
var
 i : integer;
begin
    if LogOpButton <> nil then
    begin
      LogOpButton.Visible := false;
    end;
    TMan.FPanelBuild := True;
    for I := 0 to Count-1 do
    begin
      Items[i].Control.Visible := false;
      if Items[i].ControlLabel <> nil then
         Items[i].ControlLabel.Visible := false;
      if Items[i].FieldLogopButton <> nil then
         Items[i].FieldLogOpButton.Visible := false;
      if Items[i].ValueLogopButton <> nil then
         Items[i].ValueLogOpButton.Visible := false;
    end;
    TMan.FPanelBuild := false;
end;

procedure TILControlPanel.EnableCtrls(Enable : Boolean);
var
 i : integer;
 Enabl : Boolean;
begin
    for I := 0 to Count-1 do
    begin
      if Items[i].Table.HasHandler and Items[i].Table.DataHandler.DisableCommandModes then
      begin
        Items[i].Enabled := True;
        if LogOpButton <> nil then
          LogOpButton.Enabled := True;
        continue;
      end;
      Enabl := Enable;
      if TMan = nil then
      begin
        if csDesigning in Componentstate then
          Items[i].Enabled := true
        else
          Items[i].Enabled := Enabl;
        Continue;
      end;
      if (Items[i].Usage = duSearch) and Items[i].Table.DataHandler.FreeSearch
      and (TMan.CommandMode = cmUpdate) then
       Enabl := True

      else if TMan.FindPanel(Tman.ActiveTable, TMan.CommandMode) = nil then
        Enabl := false;

      if Enabl then
      begin
        if Items[i].Usage = duData then
        begin
            Enabl := not Items[i].Field.IsReadOnly and (TMan.CommandMode = cmUpdate);
        end
        else
        begin
            Enabl := ((TMan.CommandMode = cmSearch) or
             (Items[i].Table.DataHandler.FreeSearch)) and TMan.IsInSearchPath(TMan.ActiveTable, Items[i].Table);
        end;
      end;
     if csDesigning in ComponentState then
       Enabl := True;

     Items[i].Enabled := Enabl;

     if Items[i].FControl is TDateTimePicker then
     with Items[i].FControl as TDateTimePicker do
      begin
         if Items[i].FField.DateFormat <> '' then
             Format := Items[i].FField.DateFormat
         else if Translator.FFormatSettings.ShortDateFormat <> FormatSettings.ShortDateFormat then
             Format := Translator.FFormatSettings.ShortDateFormat;
      end;
      if Items[i].Enabled then
        Items[i].FControl.Repaint;
    end;

    if LogOpButton <> nil then
    if csDesigning in ComponentState then
      LogOpButton.Enabled := True
    else
      LogOpButton.Enabled := Enable;
end;

constructor TILControlPanel.Create(AOwner : TComponent);
begin
   inherited;
   FCtrlsTop := 5;
   FCtrlsLeft := 5;
   FDualCtrlOptions := [doLogPicklist];
   FControlSpacing := 3;
   FItems := TList.Create;
   RegPanels.Add(Self);
end;

destructor TILControlPanel.Destroy;
var
  i : integer;
  Indx : integer;
begin
   inherited;
   for I := 0 to FItems.Count-1 do
     TILWorkField(FItems[i]).Free;
   FItems.Free;
   Indx := RegPanels.IndexOf(Self);
   if Indx <> -1 then
   begin
     RegPanels.Delete(Indx);
     if TMan = nil then
     begin
       Exit;
     end;
     Indx := TMan.FPanels.IndexOf(Self);
     if Indx <> -1 then
      TMan.FPanels.Delete(Indx);
   end;
   if (TMan <> nil) and TMan.UseCommandModes then
   while TMan.FWorkHistory.Count > 0 do
   begin
        TWorkHistoryElement(TMan.FWorkHistory[0]).Free;
        TMan.FWorkHistory.Delete(0);
    end;
end;

procedure TILControlPanel.CreateLogOpButtons;
var
  i : integer;
  LB : TILLogOpButton;
  Itm : TILWorkField;

  function GetName(Pref : string) : string;
  var
    n : integer;
  begin
    Result := Pref + Itm.Field.Caption;
    n := 0;
    while FindComponent(Result) <> nil do
    begin
       inc(n);
       Result := Result + IntToStr(n);
    end;
  end;

  function LBExist(AType : TLogOpType) : Boolean;
  var
    i : integer;
  begin
     Result := false;
     for I := 0 to ControlCount-1 do
     begin
       if Controls[i] is TILLogOpButton then
       with Controls[i] as TILLogOpButton do
       begin
          if AType = ltFieldAligned then
          begin
            if (LogOpType = AType) then
            begin
              Result := True;
              Break;
            end;
          end;
          if (SearchControl = Itm.Control)
          and (LogOpType = AType) then
          begin
            Result := True;
            Break;
          end;

       end;
     end;
  end;

begin
  if not Assigned(TMan) then
    Exit;
  TMan.FPanelBuild := True;
  try
   if (TMan.SearchModel = smAndOrAligned)
   and not LBExist(ltFieldAligned) then
   begin
      LB := TILLogOpButton.Create(Self);
      LB.FTable := TMan.ActiveTable;
      LB.Parent := Self;
      LB.Visible := False;
      LB.LogOpType := ltFieldAligned;
      LB.Name := 'lob_' + LB.Table.TableName;
      LogOpButton := LB;
      if Assigned(FOnDrawLogOpButton) then
        LogOpButton.FOnDrawButton := FOnDrawLogOpButton;
   end;
   for i := 0 to Count-1 do
   begin
     Itm := Items[i];
     if Itm.Usage <> duSearch then
       Continue;
     if TMan.SearchModel = smAndOrMixed then
     begin
       if LBExist(ltField) then
         Continue;
     end;
     if (Itm.Control is TMemo) or (Itm.Control is TCheckListBox) then
     begin
        if LBExist(ltValue) then
         Continue;
     end;

     if TMan.SearchModel = smAndOrMixed then
     begin
        LB := TILLogOpButton.Create(Self);
        LB.FTable := Itm.Table;
        LB.Parent := Self;
        LB.Visible := False;
        LB.LogOpType := ltField;
        LB.Name := GetName('flb_');
        LB.SearchControl := Itm.Field.SearchControl;
        Itm.FFieldLogOpButton := LB;
        if Assigned(FOnDrawLogOpButton) then
          LB.FOnDrawButton := FOnDrawLogOpButton;
     end;
     if (Itm.Control is TMemo) or (Itm.Control is TCheckListBox) then
     begin
        LB := TILLogOpButton.Create(Self);
        LB.Parent := Self;
        LB.FTable := Itm.Table;
        LB.LogOpType := ltValue;
        LB.Visible := False;
        LB.Name := GetName('vlb_');
        LB.SearchControl := Itm.Field.SearchControl;
        Itm.FValueLogOpButton := LB;
        if Assigned(FOnDrawLogOpButton) then
          LB.FOnDrawButton := FOnDrawLogOpButton;
     end;
   end;
  finally
    TMan.FPanelBuild := false;
  end;
end;

procedure TILControlPanel.RemoveLogopButtons;
var
  i : integer;
  Itm : TILWorkField;
begin
 if TMan = nil then
   Exit;
 TMan.FPanelBuild := True;
 try
   if Assigned(LogOpButton) then
     FreeAndNil(FLogOpButton);
   for i := 0 to Count-1 do
   begin
      Itm := Items[i];
      if Itm.FieldLogOpButton <> nil then
        FreeAndNil(Itm.FFieldLogOpButton);
      if Itm.ValueLogOpButton <> nil then
        FreeAndNil(Itm.FValueLogOpButton);
   end;
 finally
   TMan.FPanelBuild := false;
 end;
end;

procedure TILControlPanel.CreateItems(Init : Boolean);
var
  i, Indx : integer;
  P : TILWorkField;
  Fld : TilFieldDef;
  Usg : TDataUsaGE;

 function IsDac(ACtrl : TControl; var Usage : TDataUsage) : TilFieldDef;
 var
   i : integer;
   Indx : integer;
 begin
   Result := nil;
   if not (ACtrl is TWinControl) then
    Exit;
   if TMan = nil then
     Exit;
   for I := 0 to TMan.TableCount-1 do
   begin
      Indx := TMan.Tables[i].Fields.IndexOfControl(ACtrl as TWinControl);
      if Indx <> -1 then
      begin
        Result := TMan.Tables[i].Fields.Items[Indx];
        if Result.SearchControl = ACtrl then
          Usage := duSearch
        else
          Usage := duData;
        Break;
      end;
   end;
 end;

begin
  if TMan = nil then
    Exit;
  TMan.FPanelBuild := True;
  try
   for I := 0 to FItems.Count-1 do
    TILWorkField(FItems[i]).Free;
   FItems.Clear;

  for I := 0 to ControlCount-1 do
  begin
    Fld := IsDac(Controls[i], Usg);
    if Fld <> nil then
    begin
       P := TILWorkField.Create;
       P.FControl := Controls[i] as TWinControl;
       if Init then
       begin
          if not Fld.Ilib.HasHandler then
           Indx := -1
          else
           Indx := Fld.Ilib.DataHandler.DataFields.IndexOf(Fld.FieldName);
          if Indx <> -1 then
          begin
            if Usg = duSearch then
              P.IsVisible := Fld.Ilib.DataHandler.DataFields.Items[Indx].DacSearchVisible
            else
              P.IsVisible := Fld.Ilib.DataHandler.DataFields.Items[Indx].DacUpdateVisible;
          end
          else
            P.IsVisible := True;
       end
       else
         P.IsVisible := True;
       P.Usage := Usg;
       if Usg = duSearch then
       begin
         P.FLabel := Fld.SearchLabel;
       end
       else
         P.FLabel := Fld.DataLabel;
       P.Field := Fld;
       P.FPanel := Self;
       P.FTable := Fld.Ilib;
       FItems.Add(P);
    end;
  end;
  FItems.Sort(@SortList);
  finally
    TMan.FPanelBuild := False;
  end;
end;

procedure TILControlPanel.Notification(AComponent: TComponent; Operation: TOperation);
begin
    inherited Notification(AComponent, Operation);
    if csDestroying in ComponentState then
      Exit;
    if Tman = nil then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent = FDataSource then
        FDataSource := nil
    end;
end;

procedure TILControlPanel.SetCtrlsLeft(const Value : integer);
begin
     if CtrlsLeft = Value then
       Exit;
     FCtrlsLeft := Value;
     ShowCtrls(FItems);
end;

procedure TILControlPanel.SetCtrlsTop(const Value : integer);
begin
     if CtrlsTop = Value then
       Exit;
     FCtrlsTop := Value;
     ShowCtrls(FItems);
end;

procedure TILControlPanel.SetControlSpacing(const Value : integer);
begin
     if FControlSpacing = Value then
       Exit;
     FControlSpacing := Value;
     ShowCtrls(FItems);
end;

function TILControlPanel.GetEnableScroller : Boolean;
begin
  if not (Parent is TScrollBox) then
    Result := false
  else
    Result := FEnableScroller;
end;

procedure TILControlPanel.SetEnableScroller(const Value : Boolean);
begin
  if (csLoading in ComponentState) or (Tman = nil) then
  begin
    FEnableScroller := Value;
    Exit;
  end;
  if Value <> FEnableScroller then
  begin
       if not (Parent is TScrollBox) then
         HandleILException(E_NotInScrollbox);
       TMan.FPanelBuild := True;
       Align := alClient;
       TMan.FPanelBuild := False;
       FEnableScroller := Value;
       ShowCtrls(FItems);
  end;
end;

procedure TILControlPanel.SetShowLogOpButtons(const Value : Boolean);
begin
   if Tman = nil then
     Exit;
   FShowLogOpButtons := Value;
   if Value then
   begin
     RemoveLogOpButtons;
     CreateLogOpButtons;
   end
   else
     RemoveLogOpButtons;
   if Tman.UseCommandModes then
   begin
     ShowCtrls(Tman.FWorkFields);
   end
   else
    ShowCtrls(FItems);
end;


procedure TILControlPanel.Loaded;
begin
  inherited;
end;

procedure TILControlPanel.AlignControls(AControl : TControl; var Rect : TRect);
var
  i : integer;
  Found :Boolean;
begin
   inherited;
   if TMan = nil then
     Exit;
   if TMan.FPanelBuild then
     Exit;
   if AControl = nil then
     Exit;
   Found := false;
   for I := 0 to Count-1 do
   begin
     if (Items[i].FControl = AControl)
     or (Items[i].FLabel = AControl) then
     begin
       Items[i].FIsVisible := AControl.Visible;
       Found := True;
     end;
   end;

   if not Found then
     Exit;

   if Assigned(TMan) and (TMan.UseCommandModes)
     and not (csDesigning in ComponentState) then
   begin
     ShowCtrls(TMan.FWorkFields);
   end
   else
     ShowCtrls(FItems);
end;

procedure TILControlPanel.SetDataSource(const Value : TILData);
begin
   if csLoading in ComponentState then
   begin
     FDataSource := Value;
     Exit;
   end;
   Comment1 := '';
   FDataSource := Value;
   if Value <> nil then
   begin
     Value.FreeNotification(Self);
   end;
end;

procedure TILControlPanel.GenerateControls(const Value : TControlType);
var
  Usage : TDataUsage;
  FieldDef : TILFieldDef;
  i : integer;
  Indx : integer;

  function GetName(Pref : string) : string;
  var
    n : integer;
  begin
    Result := Pref + FieldDef.Caption;
    n := 0;
    while Owner.FindComponent(Result) <> nil do
    begin
       inc(n);
       Result := Result + IntToStr(n);
    end;
  end;

  procedure SetUsg(Ctrl : TWinControl);
  var
    Indx : integer;
  begin
    Indx := FDataSource.DataFields.IndexOf(FieldDef.FieldName);
     if Usage = duData then
    begin
      FieldDef.DataControl := Ctrl;
      FieldDef.DataControl.Name := GetName ('d_');
      FDataSource.DataFields.Items[Indx].FDataControl := Ctrl;
    end
    else
    begin
      FieldDef.SearchControl := Ctrl;
      FieldDef.SearchControl.Name := GetName('s_');
      FDataSource.DataFields.Items[Indx].FSearchControl := Ctrl;
    end;
  end;

  procedure CreateListBox;
  var
    Ctrl : TListBox;
  begin
    Ctrl := TListBox.Create(Owner);
    Ctrl.Parent := Self;
    if FieldDef.LookupType = looPickValues then
     Ctrl.Items.Assign(FieldDef.PickValues);
    Ctrl.Height := (Ctrl.ItemHeight + 2) * Ctrl.Count;
    SetUsg(Ctrl);
  end;

  procedure CreateRadio;
  var
    Ctrl : TRadioGroup;
  begin
    Ctrl := TRadioGroup.Create(Owner);
    Ctrl.Parent := Self;
    if FieldDef.LookupType = looPickValues then
      Ctrl.Items.Assign(FieldDef.PickValues);
    Ctrl.Height := Ctrl.Items.Count * (Canvas.TextHeight('Ilib') + 2) + 20;
    SetUsg(Ctrl);
    Ctrl.Caption := '';
  end;

  procedure CreateCombo;
  var
    Ctrl : TComboBox;
  begin
     Ctrl := TComboBox.Create(Owner);
     Ctrl.Parent := Self;
     if FieldDef.LookupType = looPickValues then
     begin
       Ctrl.Items.Assign(FieldDef.PickValues);
     end;
     if (Usage = duSearch) and not (FieldDef.DataType = dtBoolean) then
       Ctrl.Style := csDropDown
     else
      Ctrl.Style := csDropDownList;
     SetUsg(Ctrl);
  end;

  procedure CreateCheckList;
  var
    Ctrl : TCheckListBox;
  begin
     Ctrl := TCheckListBox.Create(Owner);
     Ctrl.Parent := Self;
     if FieldDef.LookupType = looPickValues then
     begin
       Ctrl.Items.Assign(FieldDef.PickValues);
     end;
     if Ctrl.Count < 5 then
       Ctrl.Height := (Ctrl.ItemHeight + 2) * Ctrl.Count;
     SetUsg(Ctrl);
     Ctrl.AllowGrayed := True;
     Ctrl.CheckAll(cbGrayed);

  end;

  procedure CreateDTP;
  var
    Ctrl : TDateTimePicker;
  begin
     Ctrl := TDateTimePicker.Create(Owner);
     Ctrl.Parent := Self;
     Ctrl.ShowCheckbox := True;
     Ctrl.Checked := false;
     Ctrl.Format := '';
     SetUsg(Ctrl);
  end;

  procedure CreateCheckBox;
  var
    Ctrl : TCheckBox;
    x : integer;
  begin
     Ctrl := TCheckBox.Create(Owner);
     Ctrl.Parent := Self;
     Ctrl.AllowGrayed := True;
     Ctrl.State := cbGrayed;
     SetUsg(Ctrl);
     Ctrl.Caption := '';
     x := GetSystemMetrics( SM_CXMENUCHECK );
     Ctrl.Width := x+1;
  end;

  procedure CreateEdit;
  var
    Ctrl : TEdit;
  begin
     Ctrl := TEdit.Create(Owner);
     Ctrl.Parent := Self;
     SetUsg(Ctrl);
     Ctrl.Text := '';
  end;

  procedure CreateMemo;
  var
    Ctrl : TMemo;
  begin
     Ctrl := TMemo.Create(Owner);
     Ctrl.Parent := Self;
     Ctrl.Height := 60;
     Ctrl.WordWrap := false;
     SetUsg(Ctrl);
     Ctrl.Clear;
  end;

  procedure CreateReslist;
  var
    Ctrl : TILResourceList;
  begin
     Ctrl := TILResourceList.Create(Owner);
     Ctrl.Parent := Self;
     Ctrl.Visible := True;
     if FieldDef.MaxValues = 1 then
       Ctrl.SingleLine := True
     else
      Ctrl.Height := 60;
     SetUsg(Ctrl);
  end;

  procedure CreateControl;
  begin
    if Usage = duData then
    begin
       if (FieldDef.DataType in [dtString, dtNumber])
       and not FieldDef.IsMultiVal then
       begin
        if (Assigned(FieldDef.PickValues) and (FieldDef.PickValues.Count > 0) and (FieldDef.DataType = dtString))
        or (FieldDef.LookupType in [looTableKey, looTableRef])
        then
        begin
           CreateCombo
        end
        else
          CreateEdit;
       end
       else if (FieldDef.DataType in [dtString, dtNumber]) and FieldDef.IsMultiVal then
       begin
        if FieldDef.LookupType = looPickValues then
        begin
         if Assigned(FieldDef.PickValues) and (FieldDef.PickValues.Count > 0) and (FieldDef.DataType = dtString) then
          CreateCheckList
        end
        else
          CreateMemo;
       end
       else if (FieldDef.DataType in [dtResource]) then
       begin
          CreateResList;
       end
       else if (FieldDef.DataType = dtText) then
         CreateMemo
       else if (FieldDef.DataType = dtDate) then
         CreateDTP
       else if (FieldDef.DataType = dtBoolean)then
         CreateCheckBox;
       FieldDef.DataControl.Width := FieldDef.ColumnWidth;

       if FieldDef.DataLabel = nil then
       begin
         FieldDef.DataLabel := TLabel.Create(Owner);
         FieldDef.DataLabel.Name := GetName('dlbl_');
         FieldDef.DataLabel.Caption := FieldDef.Caption;
         FieldDef.DataLabel.Parent := Self;
         Indx := FDataSource.DataFields.IndexOf(FieldDef.FieldName);
         FDataSource.DataFields.Items[Indx].FDataLabel := FieldDef.DataLabel;
       end;
    end

    else
    begin
      if (FieldDef.SearchControl = nil) then
      begin
       if (FieldDef.DataType in [dtString, dtNumber, dtResource, dtDate]) then
       begin
          if (FieldDef.LookupType = looPickValues) and
          (FieldDef.PickValues.Count > 0) and (FieldDef.DataType = dtString) then
          begin
            CreateCheckList
          end
          else
          begin
             CreateEdit
          end;
       end
       else if (FieldDef.DataType in [dtText, dtLongText]) then
         CreateEdit
       else if (FieldDef.DataType = dtBoolean) and not FieldDef.Mandatory then
         CreateCombo
       else if (FieldDef.DataType = dtBoolean) then
         CreateCheckBox;
       FieldDef.SearchControl.Width := FieldDef.ColumnWidth;
      end;

      if FieldDef.SearchLabel = nil then
      begin
        FieldDef.SearchLabel := TLabel.Create(Owner);
        FieldDef.SearchLabel.Name := GetName('slbl_');
        FieldDef.SearchLabel.Caption := FieldDef.Caption;
        FieldDef.SearchLabel.Parent := Self;
        Indx := FDataSource.DataFields.IndexOf(FieldDef.FieldName);
        FDataSource.DataFields.Items[Indx].FSearchLabel := FieldDef.SearchLabel;
      end;
     end;
    end;

begin
  if Value = ctUndefined then
  begin
    FControlType := Value;
    Exit;
  end;

   if (TMan = nil) then
    HandleILException(E_ManagerNotFound)
  else if FDataSource = nil then
    HandleILException(E_MissingSourceTable);

  if Value = ctDataControls then
    Usage := duData
  else
    Usage := duSearch;
  FControlList := TList.Create;
  FSourceBuild := True;
  try
   for I := 0 to FDatasource.DataFields.Count-1 do
   begin
      if FDataSource.DataFields.Items[i].FieldName = '' then
        Continue;
      FieldDef := FDataSource.DataFields.Items[i].FieldDef;
      if (FieldDef.LookupType = looParentField) then
      { It is not a good idea to include lookup parents fields.
      This field type is not indexed and it looks confusing to
      show two fields doing the same. The source field
      is the proper field to search
      Used as data field it is not shown in update operations, but may
      be visible when browsing (///not quite clear) }
        Continue;
      if FieldDef.ReportField and not ((Usage = duData) and (FieldDef.DataType = dtLongText)) then
      { Long text must have their own data controls, defined by user }
      begin
        if (( (Usage = duData) and (FieldDef.DataControl = nil) ) and not FieldDef.IsReadOnly)
        or ( (Usage = duSearch) and (FieldDef.SearchControl = nil) ) then
        begin
         CreateControl;
         FControlList.Add(FieldDef);
        end;
      end;
   end;
  finally
    FSourceBuild := false;
    FControlList.Free;
  end;

  CreateItems;

  if TMan.UseCommandModes then
  begin
    if Usage = duSearch then
      TMan.CreateWorkFields(FDataSource.Table, cmSearch)
    else
    begin
       TMan.CreateWorkFields(FDataSource.Table, cmUpdate);
    end;
  end;

  if ShowLogOpButtons then
    CreateLogOpButtons;
  ComputeLabelSpace;
  if TMan.UseCommandModes then
  begin
    if csDesigning in ComponentState then
     ShowCtrls(FItems)
    else
     ShowCtrls(TMan.FWorkFields);
    TMan.UpdateWorkList;
  end
  else
    ShowCtrls(FItems);
end;

procedure TILControlPanel.ComputeLabelSpace;
var
  Fld : TilFieldDef;
  DL, SL, LL, L : integer;
  I : integer;
begin
   FLabelSpace := 0;
   if Tman = nil then
     Exit;
   LL := 0;
   if TMan.SearchModel = smAndOrMixed then
     LL := 18;
   for I := 0 to Count-1 do
   begin
       Fld := TILWorkField(FItems[i]).Field;
       SL := 0;
       DL := 0;
       if (Fld.DataLabel <> nil) and (Items[i].Usage = duData) then
        DL := Fld.DataLabel.Width;
       if (Fld.SearchLabel <> nil) and (Items[i].Usage = duSearch) then
        SL := Fld.SearchLabel.Width;
       L := Max(SL, DL);
       FLabelSpace := Max(FLabelSpace, L);
       if ShowLogOpButtons then
       if Items[i].Usage = duSearch then
       begin
         if (Items[i].FControl is TCheckListBox)
         or (Items[i].FControl is TMemo) then
         begin
           if TMan.SearchModel = smAndOrMixed then
             LL := 36
           else
             LL := 15;
         end;
       end;
   end;
   Inc(FLabelSpace, LL);
end;

{ TILLogOpButton ---------------------------------------------------------}

  procedure TILLogOpButton.SetTable(const Value : TILTable);
  begin
       if Value = FTable then
         Exit;
       if Value = nil then
        Exit;
       if not (csLoading in ComponentState) then
         if TMan = nil then
       begin
        HandleILException(E_ManagerNotFound);
       end;
       FTable := Value;
       if not (csLoading in ComponentState) then
         FSearchControl := nil;
       FTable.FreeNotification(Self);
  end;

 procedure TILLogOpButton.SetSearchControl(const Value : TWinControl);
 var
   Ctrl : TWinControl;
 begin
    if FSearchControl = Value then
      Exit;
    Ctrl := FSearchControl;
    FSearchControl := Value;
    if (csDesigning in ComponentState) and not (csLoading in ComponentState) then
    begin
     if Table = nil then
     begin
      FSearchControl := Ctrl;
      HandleILException(E_TableNotAssigned);
     end;

     if TMan = nil then
     begin
        FSearchControl := Ctrl;
        HandleILException(E_ManagerNotFound);
     end;

     if Table.Fields.IndexOfControl(Value) = -1 then
     begin
         FSearchControl := Ctrl;
         Comment1 := Value.Name;
         Comment2 := Table.TableName;
         HandleILException(E_CtrlNotFound);
     end;

     if FieldDef <> nil then
     if FieldDef.SearchControl <> Value then
     begin
       FSearchControl := Ctrl;
       Comment1 := Value.Name;
       Comment2 := 'search';
       HandleILException(E_DacNotRegistered);
     end;
    end;

    if (FSearchControl is TCheckListBox) and (FieldDef.MaxValues = 1) and (LogopType = ltValue)then
    begin
      Enabled := False;
      FState := loOr;
    end;
 end;

 procedure TILLogopButton.SetBordered(const Value : Boolean);
 begin
   if FBordered = Value then
     Exit;
   FBordered := Value;
   Invalidate;
 end;

 procedure TILLogopButton.SetLogopType(const Value : TLogOpType);
 begin
   if csLoading in ComponentState  then
   begin
     FLogOpType := Value;
     Exit;
   end;
   if (csDesigning in ComponentState) and not (csLoading in ComponentState) then
    if Table = nil then
      HandleILException(E_TableNotAssigned);
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    if (Value = ltField) and (TMan.SearchModel <> smAndOrMixed) then
    begin
        Comment1 := 'ltField';
        Comment2 := 'smAndOrMixed';
        HandleILException(E_LogOpBtnType);
    end
    else if (Value = ltFieldAligned) and (TMan.SearchModel = smAndOrMixed) then
    begin
        Comment1 := 'ltFieldAligned';
        Comment2 := 'smAndAligned or smAndOrAligned';
        HandleILException(E_LogOpBtnType);
    end;
    FLogOpType := Value;
    if FLogOpType = ltFieldAligned then
      FSearchControl := nil;
    if (SearchControl is TCheckListBox) and (LogOpType = ltValue) and (FieldDef.MaxValues = 1)  then
    begin
      State := loOr;
    end;
 end;

 procedure TILLogOpButton.SetState(const Value : TLogOp);
 var
   Tbl : TILTable;
 begin
    if FState = Value then
      Exit;
    if (FSearchControl = nil) and (LogopType in [ltField, ltValue]) then
      Exit;
    if not Enabled then
      Exit;
    if (Table = nil) or (TMan = nil)
    or ((SearchControl = nil) and (LogOptype <> ltFieldAligned)) then
      Exit;

    if TMan.SearchModel = smAndOrAligned then
    begin
        Tbl := TMan.ActiveTable;
    end
    else
      Tbl := Table;

    if FState = loAnd then
    begin
       if TMan.SearchModel = smAndAligned then
         Exit;
       Tbl.SetLogOp(flOr);
       if LogOpType = ltFieldAligned then
         Tbl.SetLogOp(flOr)
       else if LogOpType in [ltField, ltValue] then
       begin
         if LogOpType = ltField then
           Tbl.SetLogOp(FSearchControl, flOr)
         else
           Tbl.SetLogOp(FSearchControl, vlOr)
       end;
    end
    else
    begin
      if (SearchControl is TCheckListBox) and (LogOpType = ltValue) and (FieldDef.MaxValues = 1)  then
         Exit;
       Tbl.SetLogOp(flAnd);
       if LogOpType = ltFieldAligned then
         Tbl.SetLogOp(flAnd)
       else if LogOpType in [ltField, ltValue] then
       begin
         if LogOpType = ltField then
           Tbl.SetLogOp(FSearchControl, flAnd)
         else
           Tbl.SetLogOp(FSearchControl, vlAnd)
       end;
    end;
    if FState = loAnd then
      FState := loOr
    else
      FState := loAnd;
   Invalidate;
 end;


 function TILLogOpButton.GetFieldDef : TILFieldDef;
 var
   Indx : integer;
 begin
    Result := nil;
    if Table = nil then
      Exit;
    Indx := Table.Fields.IndexOfControl(FSearchControl);
    if Indx <> -1 then
      Result := Table.Fields.Items[Indx];
 end;

 procedure TILLogOpButton.Paint;
 var
   W, H : integer;
   L, T : integer;
   S : string;
   Handled : Boolean;
 begin
   if FState = loAnd then
   begin
    S := c_LogAnd;
   end
   else
   begin
     S := c_LogOr;
   end;
   Handled := False;
   if Assigned(FOnDrawButton) then
   begin
     FOnDrawButton(Self, Canvas, Handled);
     if Handled then
       Exit;
   end;
   Canvas.Brush.Style := bsSolid;
   Canvas.Brush.Color := Color;
   Canvas.Font.Assign(Font);
   if not Enabled then
   begin
     Canvas.Pen.Color := clSilver;
     Canvas.Font.Color := clSilver;
   end
   else
   begin
     Canvas.Pen.Color := Font.Color;
     Canvas.Font.Color := Font.Color;
   end;
   if Bordered then
     Canvas.Rectangle(ClientRect)
   else
     Canvas.FillRect(ClientRect);
   Canvas.Brush.Style := bsClear;
   W := Canvas.TextWidth(S);
   H := Canvas.TextHeight(S);
   L := (ClientWidth - W) div 2;
   T := (ClientHeight - H) div 2;
   Canvas.TextOut(L,T, S)
 end;

procedure TILLogOpButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
    inherited Notification(AComponent, Operation);
    if csDestroying in ComponentState then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent = Table then
        FTable := nil
    end;
end;

constructor TILLogOpButton.Create(AOwner : TComponent);
 begin
     inherited;
     Width := 15;
     Height := 15;
     FBordered := True;
 end;

procedure TILLogOpButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: integer);
begin
    if FState = loAnd then
      State := loOr
    else
      State := loAnd;
    inherited;
end;

{ TILComboBox ---------------------------------------------------------------- }

procedure TILComboBox.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  Message.Result := Message.Result or DLGC_WANTTAB
end;

{ TILEdit ------------------------------------------------------------------- }

procedure TILEdit.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  Message.Result := Message.Result or DLGC_WANTTAB
end;

{ TILResourceList -----------------------------------------------------------}

constructor TILResourceList.Create(AnOwner : TComponent);
begin
  inherited;
  Visible := false;
  Parent := AnOwner as TWinControl;
  ImageIndex := -1;
  Style := lbOwnerDrawFixed;
  Canvas.Font.Assign(Font);
  OnDrawItem := DrawItm;
  OnMouseLeave := MouseLeave;
  FAddresses := TStringList.Create;
  Items.Add('');
  FAddresses.Add('');
end;

destructor TILResourceList.Destroy;
begin
  FAddresses.Free;
  inherited;
end;

procedure TILResourceList.KeyDown(var Key: Word; Shift: TShiftState);
var
  Dlg : TILResourceDialog;
begin
  inherited;
  if Shift <> [] then
    Exit;
  if Key = vk_Insert then
  begin
     if SingleLine and not Empty then
      Exit;
      if Assigned(FOnAddResource) then
       FOnAddResource(Self)
      else
      begin
        Dlg := TILResourceDialog.Create(Self);
        try
          if Dlg.Execute then
          begin
             AddResource(Dlg.NameControl.Text, Dlg.AddressControl.Text);
          end;
        finally
          Dlg.Free;
        end;

      end;
  end
  else if (Key = vk_Delete) then
  begin
    DeleteResource(ItemIndex);
  end;
end;

procedure TILResourceList.MouseDown(Button: TMouseButton; Shift: TShiftState;
         X, Y: integer);
begin
    inherited;
    FMouseDown := True;
    Invalidate;
end;

procedure TILResourceList.MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: integer);
var
  Dlg : TILResourceDialog;
  R : TRect;
begin
   inherited;
   FMouseDown := false;
   if FMouseInBtn then
   begin
     if Assigned(FOnUpdateResource) then
       FOnUpdateResource(Self)
     else
     begin
        Dlg := TILResourceDialog.Create(Self);
        try
        Dlg.NameControl.Text := Items[ItemIndex];
        Dlg.AddressControl.Text := Addresses[ItemIndex];
        if Dlg.Execute then
          UpdateResource(ItemIndex, Dlg.NameControl.Text, Dlg.AddressControl.Text);
        SetFocus;

        finally
          Dlg.Free;
        end;
     end;
     R := GetBtnRect(ItemIndex);
     FMouseInBtn := PtInRect(R, Point(X,Y));
   end;
   Invalidate;
end;

procedure TILResourceList.MouseMove(Shift: TShiftState; X, Y: integer);
var
  Itm : integer;
  R : TRect;
  InBtn : Boolean;
begin
    inherited;
    InBtn := FMouseInBtn;
    FMouseInBtn := false;
    Itm := ItemAtPos(Point(X,Y), True);
    if Itm = -1 then
    begin
     if InBtn then
       Invalidate;
     Exit;
    end;
    if Itm <> ItemIndex then
    begin
     if InBtn then
       Invalidate;
     Exit;
    end;
    R := GetBtnRect(Itm);
    FMouseInBtn := PtInRect(R, Point(X,Y));
    if (FMouseInBtn and not InBtn) or (InBtn and not FMOuseInBtn) then
      Invalidate;
end;

procedure TILResourceList.MouseLeave;
begin
   if FMouseInBtn then
   begin
    FMouseInBtn := false;
    Invalidate;
   end;
end;

procedure TILResourceList.AddResource(const AName, AAddress : string);
begin
    if SingleLine and (Items.Count > 0) then
    begin
      HandleIlException(E_ResSingleLine);
      Exit;
    end;
    if (Trim(AName) = '') or (Trim(AAddress) = '') then
      HandleIlException(E_MissingResElement);

    if Empty then
    begin
      Items[0] := AName;
      FAddresses[0] := AAddress;
    end
    else
    begin
      Items.Add(AName);
      ItemIndex := Count-1;
      FAddresses.Add(AAddress);
    end;
end;

procedure TILResourceList.UpdateResource(const Index : integer; const AName, AAddress : string);
begin
    if (Trim(AName) = '') or (Trim(AAddress) = '') then
      HandleIlException(E_MissingresElement);
    Items[Index] := AName;
    FAddresses[Index] := AAddress;
end;

procedure TILResourceList.DeleteResource(const Index : integer);
var
  Indx : integer;
begin
    if Empty then
      Exit;
    if Count = 1 then
    begin
      Items[0] := '';
      FAddresses[0] := '';
    end
    else
    begin
      Indx := ItemIndex;
      Items.Delete(Index);
      FAddresses.Delete(Index);
      if Indx = Count then
        Dec(Indx);
      ItemIndex := Indx;
    end;
    Invalidate;
end;

procedure TILResourceList.CNDrawItem(var Message: TWMDrawItem);
begin
  Exclude(TOwnerDrawState(LongRec
(Message.DrawItemStruct^.itemState).Lo),odFocused);
  inherited;
end;

procedure TILResourceList.Resize;
begin
  if SingleLine then
  begin
    ComputeHeight;
  end;
  inherited;
end;

procedure TILResourceList.DrawItm(Control: TWinControl; Index: Integer;
   Rect: TRect; State: TOwnerDrawState);
var
  R : TRect;
  P : TPicture;
begin
    if (odFocused in State) or (odSelected in State) or Empty then
    begin
      Canvas.Brush.Color := clHighLight;
      Canvas.Font.Color := clHighLightText;
    end
    else
    begin
      Canvas.Brush.Color := Color;
      Canvas.Font.Color := clWindowText;
    end;
    Canvas.Brush.Style := bsSolid;
    Canvas.FillRect(Rect);
    Canvas.Brush.Style := bsClear;
    Canvas.TextOut(Rect.Left + 1, Rect.Top, Items[Index]);
    if (odFocused in State) or (odSelected in State) or Empty then
    begin
     R := GetBtnRect(Index);
     P := TPicture.Create;
     try
      if FMouseInBtn and FMouseDown then
      begin
        Canvas.Brush.Color := clBtnFace;
        Canvas.Pen.Color := clBlack;
      end
      else if FMouseInBtn then
      begin
        Canvas.Brush.Color := clBtnHighLight;
        Canvas.Pen.Color := clActiveBorder;
      end
      else
      begin
        Canvas.Pen.Color := clInactiveBorder;
        Canvas.Brush.Color := clBtnFace;
      end;
      Canvas.Brush.Style := bsSolid;
      Canvas.FillRect(R);

      if ImageIndex <> -1 then
      begin
         Images.GetIcon(ImageIndex, P.Icon);
         Canvas.Draw(R.Left, R.Top, P.Icon);
         Canvas.Brush.Style := bsClear;
         if FMouseInBtn then
          Canvas.Rectangle(R);
      end
      else
      begin
         if FMouseInBtn then
         begin
          Canvas.Rectangle(R);
         end;
         Canvas.Brush.Style := bsClear;
         Canvas.Font.Color := clWindowText;
         Canvas.TextOut(R.Left+2, R.Top, '...');
      end

     finally
      P.Free;
     end;
    end;
end;

function TILResourceList.GetFieldDef : TILFieldDef;
var
  i : integer;
  Tbl : TILTable;
  Indx : integer;
begin
   Result := nil;
   for I := 0 to TMan.TableCount-1 do
   begin
      Tbl := TMan.Tables[i];
      Indx := Tbl.Fields.IndexOfControl(Self);
      if Indx <> -1 then
      begin
        Result := Tbl.Fields.Items[Indx];
        Break;
      end;
   end;
end;

procedure TILResourceList.SetAddresses(Index : integer; Value : string);
begin
    FAddresses[Index] := Value;
end;

function TILResourceList.GetAddresses(Index : integer) : string;
begin
  Result := FAddresses[Index];
end;

procedure TILResourceList.SetImageIndex(const Value : TImageIndex);
begin
    if Images = nil then
     FImageIndex := -1
    else
     FImageIndex := Value;
end;

function TILResourceList.GetBtnRect(const Index : integer) : TRect;
begin
   Result := ItemRect(Index);
   Result.Left := Result.Right-16;
end;

procedure TILResourceList.SetImages(const Value : TImageList);
begin
   FImages := Value;
end;

procedure TILResourceList.SetSingleLine(const Value : Boolean);
begin
  if Value = FSingleLine then
    Exit;
  FSingleLine := Value;
  Items.Clear;
  FAddresses.Clear;
  Items.Add('');
  FAddresses.Add('');
  ItemIndex := 0;
  ComputeHeight;
  Invalidate;
end;

procedure TILResourceList.ComputeHeight;
begin
  if SingleLine then
  begin
     Height := ItemHeight + 4;
  end;
end;

function TILResourceList.GetEmpty : Boolean;
begin
  Result := (Count = 0)
  or ((Count = 1) and (Items[0] = ''));
end;

procedure TILResourceList.SetEmpty(const Value : Boolean);
begin
  if Value = Empty then
    Exit;
  Items.Clear;
  FAddresses.Clear;
  Items.Add('');
  FAddresses.Add('');
  ItemIndex := 0;
end;

function TILResourceList.GetResourceStr(Index : integer) : string;
begin
    Result := Items[Index] + '@' + Addresses[Index];
end;

{ TILCustomGrid ----------------------------------------------------------------}

Constructor TILCustomGrid.Create(AOWner:TComponent);
begin
  inherited Create(Aowner);
  EditorMode := True;
  fComboBox:=TILComboBox.Create(self);
  fComboBox.Parent:=Self;
  fComboBox.Visible:=FALSE;
  fComboBox.OnChange := ComboChanged;
  fComboBox.OnKeyDown := ComboKeyDown;
  fComboBox.Width := 0;
  RowCount := 1;
  FComboRow := 0;
  FixedRows := 0;
  FixedCols := 0;
  ColCount := 1;
  DefaultRowHeight := 20;
  Options := Options + [goEditing];
  Options := Options - [goRangeSelect];
  Width := 170;
  FPickValues := TStringList.Create;
end;

procedure TILCustomGrid.Notification(AComponent: TComponent;
    Operation: TOperation);
begin
    inherited Notification(AComponent, Operation);
    if csDestroying in ComponentState then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent is TILTable then
      begin
        FField := nil;
      end;
    end;
end;


function TILCustomGrid.GetTable : TILTable;
begin
  if Field = nil then
    Result := nil
  else
    Result := TILTable(Field.Collection.Owner);
end;

procedure TILCustomGrid.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  Message.Result := Message.Result or DLGC_WANTTAB
end;

procedure TILCustomGrid.WMHScroll(var Message : TWMHScroll);
begin
  inherited;
  RelocateComboBox;
end;

procedure TILCustomGrid.WMVScroll(var Message : TWMHScroll);
begin
  inherited;
  RelocateComboBox;
end;

procedure TILCustomGrid.KeyPress(var Key: Char);
begin
  if Table = nil then
    Exit;
  if Key = #13 then
   if Assigned(Table.LookupControl) and Table.LookupControl.Visible then
   begin
    Cells[Col, Row] := Table.LookupControl.Items[Table.LookupControl.ItemIndex];
    Table.LookupControl.Visible := False;
   end;
  inherited KeyPress(Key);
  if Key = #0 then
    Exit;
  if Key = #27 then
   if Assigned(Table.LookupControl) and Table.LookupControl.Visible then
    Table.LookupControl.Visible := False;
  if CharInSet(Key, [#13, #27]) then
    Exit;
  if Field = nil then
    Exit;
  PostMessage(Handle, wm_Lookups, 0, 0);
   { The edit text of inplace editor is not yet reflected }

end;

function TILCustomGrid.GetValue(Index : integer) : string;
begin
  if HasCellCombo(ValueColIndex, Index) then
    Result := Combo.Text
  else if Assigned(InplaceEditor) and InPlaceEditor.Visible and (ValueColIndex = Col) and (Index = Row) then
    Result := InplaceEditor.EditText
  else
    Result := Cells[ValueColIndex, Index];
end;

procedure TILCustomGrid.SetValue(Index : integer; Value : string);
var
  i : integer;
begin
   if HasCellCombo(ValueColIndex, Index) and (Combo.Style = csDropDownList) then
   begin
     if Value = '' then
       Combo.ItemIndex := -1
     else
     begin
       i := Combo.Items.IndexOf(Value);
       if i = -1 then
        Exit
       else
        Combo.ItemIndex := i;
     end;
  end;
  if (Cells[ValueColIndex, Index] <> Value) then
   Cells[ValueColIndex, Index] := Value;
end;


function TILCustomGrid.GetValueRowCount : integer;
begin
  Result := RowCount;
end;

procedure TILCustomGrid.SetValueRowCount(const Value : Integer);
var
  Last : integer;
  i : integer;
  MinRows : integer;
begin
  if Self is TILUpdateGrid then
    MinRows := TILUpdateGrid(Self).StartRow +1
  else
    MinRows := 1;
  if Value < MinRows then
    Exit;
  if Value <> ValueRowCount then
  begin
    Last := -1;
    if Value > RowCount then
     Last := RowCount;
    RowCount := Value;
    if Last > -1 then
     for I := Last to RowCount-1 do
       Values[i] := '';
    if AutoSizeHeight then
      DoAutoSizeHeight;
    SupplyOperators;
  end;
end;

procedure TILCustomGrid.SupplyOperators;
begin
  if FAutoSizeHeight then
   DoAutoSizeHeight;
end;

procedure TILCustomGrid.Resize;
begin
  inherited;
  if AlignValueCol then
   DoAlignValueCol;
  if AutoSizeHeight and not fsemafor then
    DoAutoSizeHeight;
end;

Destructor TILCustomGrid.Destroy;
begin
  fComboBox.Free;
  FPickValues.Free;
  inherited destroy;
end;

procedure TILCustomGrid.WMCommand(var msg: TWMCommand);
begin
  If EditorMode and ( msg.Ctl = fComboBox.Handle ) Then
  begin
    inherited
  end
  Else
    If msg.Ctl = 0 Then
      msg.result :=
        SendMessage( msg.ctl, CN_COMMAND,
                     TMessage(msg).wparam,
                     TMessage(msg).lparam );
end;

procedure TILCustomGrid.DoAutoSizeHeight;
var
  H, SH : integer;
  sbVisible : Boolean;
begin

    if AutoSizeHeight and (RowCount <= 10) then
      H := (RowCount * DefaultRowHeight) + RowCount + 5
    else
      H := (10 * DefaultRowHeight) + RowCount + 5;
    if ClientHeight < GridHeight then
    begin
      H := H - (GridHeight - ClientHeight)
    end
    else
    begin
      SBVisible := GetWindowLong(Handle, GWL_STYLE) and WS_HSCROLL <> 0;
      if SBVisible then
      begin
       sh := GetSystemMetrics(SM_CXVSCROLL);
       H := H + SH;
      end;
    end;
    Height := H;
end;

procedure TILCustomGrid.SetAlignValueCol(const Value : Boolean);
begin
  if Value = FAlignValueCol then
    Exit;
  FAlignValueCol := Value;
  if Value then
  begin
    DoAlignValueCol;
  end;
end;

function TILCustomGrid.GetPickValues : TStringList;
begin
  if Field <> nil then
    Result := Field.PickValues
  else
    Result := FPickValues; { Empty, to avoid nils }
end;

procedure TILCustomGrid.SetField(const Value : TILFieldDef);
begin
  if Value = FField then
    Exit;
  FField := Value;
  ResetColumns;
end;

procedure TILCustomGrid.SetAutoSizeHeight(const Value : Boolean);
begin
    if Value = FAutoSizeHeight then
      Exit;
    FAutoSizeHeight := Value;
    if Value then
    begin
      FSemafor := True; { Avoidis calling DoAutoSize height in Resize}
      try
       DoAutoSizeHeight;
      finally
       FSemafor := False;
      end;

    end;
end;

procedure TILCustomGrid.HideCombobox;
begin
  fcomboBox.Visible:=false;
end;

function TILCustomGrid.GetInPlaceText : string;
begin
   Result := InPlaceEditor.EditText;
end;

procedure TILCustomGrid.ComboChanged(Sender  : TObject);
begin
   Cells[Col, Row] := fComboBox.Text;
end;

function TILCustomGrid.HasCellCombo(const ACol, ARow: integer) : Boolean;
begin
   Result := (ACol = FComboCol) and (ARow = FComboRow) and fComboBox.Visible;
end;

procedure TILCustomGrid.ComboKeyDown(Sender : TObject; var Key : Word; Shift : TShiftstate);
{ When combo style shifts from dropdownlist to dropdown as a result of moving
  column focus when pressing tab, an extra keydown message is sent. Why?
  The FDropStyleChanged and FCBKeyDown flags ensures that the second message is not processed.
  See also RelocateComboBox
}
begin
   if FDropStyleChanged then
   begin
     FDropStyleChanged := false;
     EXIT;
   end;
   FCBKeyDown := True;
   try
    if Key <> vk_Tab then
    begin
      Exit;
    end;
    if not ((Shift = []) or (Shift = [ssShift])) then
      Exit;
    Cells[Col, Row] := fComboBox.Text;
    if Shift = [] then
    begin
     if Col = ColCount - 1  then
     begin
      if RowCount-1 = Row then
       Exit;
      Row := Row + 1;
      Col := 0;
     end
     else
     Col := Col +1;
     { Changing col calls Click --> RelocateCombobox}
    end
    else
    begin
     if (Col = 0) and (Row = 1) then
       Exit;
     if Col > 0 then
       Col := Col-1
     else
     begin
       Row := Row-1;
       Col := ColCount-1;
     end;
    end;
   finally
     FCBKeyDown := false;
   end;

   FOldRow := Row;
   FOldCol := Col;
end;

procedure TILSearchGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: integer);
begin
   inherited;
end;

procedure TILSearchGrid.Click;
begin
  inherited;
  if fComboBox.Visible then
  { Transfer value to cell }
  begin
    if FOldCol = ValueColIndex then
       Cells[FOldCol, FOldRow] := fComboBox.Text
     else if (fComboBox.Text <> '') then
       Cells[FOldCol, FOldRow] := fComboBox.Text;
  end;
  HideCombobox;
  RelocateComboBox;
  FOldCol := Col;
  FOldRow := Row;
  if Table <> nil then
   if Assigned(Table.LookupControl) and Table.LookupControl.Visible then
    Table.LookupControl.Visible := False;

end;

procedure TILCustomGrid.WMLOOKUPS(var Message: TMessage);
begin
  Table.ShowLookups(Self);
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

{ TILUpdateGrid -------------------------------------------------------------}

constructor TILUpdateGrid.Create(AOWner: TComponent);
var
   S1 : string;
   procedure SetTransl(tagName : string);
   begin
     FTranslations.Add(tagName +  ' : ' + S1 );
   end;
begin
  inherited Create(Aowner);
  RowCount := 1;
  ColCount := 1;
  FixedCols := 0;
  FIxedRows := 0;
  Scrollbars := ssNone;
  ColWidths[0] := 100;
  Height := DefaultRowHeight + 5;
  Width := 100 + 5;
  FTranslations := TStringList.Create;
  S1 := Translator.GetTranslation(T_Set, TRANSL_TERMS);
  SetTransl(tag_Set);
  S1 := Translator.GetTranslation(T_Add, TRANSL_TERMS);
  SetTransl(tag_Add);
  S1 := Translator.GetTranslation(T_Delete, TRANSL_TERMS);
  SetTransl(tag_Delete);
  S1 := Translator.GetTranslation(T_Replace, TRANSL_TERMS);
  SetTransl(tag_Replace);
  S1 := Translator.GetTranslation(T_Action, TRANSL_TERMS);
  SetTransl(tag_Action);
  S1 := Translator.GetTranslation(T_Value, TRANSL_TERMS);
  SetTransl(tag_Value);
  Col := 0;
  Row := 0;
end;

function TILUpdateGrid.GetTranslation(Tagname: string) : string;
var
  i : integer;
  P : integer;
  s : string;
begin
   Result := Tagname;
   Delete(Result, 1, 1);
   Delete(Result, Length(Result), 1);
   for I := 0 to FTranslations.Count -1 do
   begin
      if Pos(Tagname, FTranslations[i]) <> 0 then
      begin
          s := FTranslations[i];
          P := Pos(':', s);
          if P = 0 then
            Exit;

          Result := Copy(S, P +1, Length(S));
          Result := Trim(Result);
          Break;
      end;
   end;
end;

function TILUpdateGrid.GetTranslTag(Transl: string) : string;
var
  i : integer;
  P : integer;
  s : string;
begin
   Result := '';
   for I := 0 to FTranslations.Count -1 do
   begin
      if Pos(Transl, FTranslations[i]) <> 0 then
      begin
          s := FTranslations[i];
          if s[1] <> '[' then
            Exit;
          P := Pos(']', s);
          if P = 0 then
            Exit;
          Result := Copy(S, 1, P);
          Break;
      end;
   end;
end;

function TILUpdateGrid.GetTagValue(Index : integer) : string;
begin
  Result := '';
  if Mode = gmSimple then
    Exit;
  if HasCellCombo(0, Index) then
    Result := FComboBox.Text
  else
    Result := Cells[0, Index];
end;

function TILUpdateGrid.ValueColIndex : integer;
begin
   if Mode = gmSimple then
    Result := 0
   else
    Result := 1;
end;

function TILUpdateGrid.StartRow : integer;
begin
   if Mode = gmSimple then
    Result := 0
   else
    Result := 1;
end;

destructor TILUpdateGrid.Destroy;
begin
  FTranslations.Free;
  inherited;
end;

function TILUpdateGrid.GetActionColWidth(Cnvs : TCanvas = nil) : integer;
var
  n : integer;
  s : string;
  Cnv : TCanvas;
begin
  if (Parent = nil) and (Cnvs = nil) then
  begin
    Result := 40;
    Exit;
  end;
  if Cnvs <> nil then
   Cnv := Cnvs
  else
   Cnv := Canvas;
  Cnv.Font.Assign(Font);
  s := GetTranslation(tag_Action);
  Result := Cnv.TextWidth(s);
  s := GetTranslation(tag_Set);
  n := Cnv.TextWidth(s);
  if n > Result then
    Result := n;
  s := GetTranslation(tag_Add);
  n := Cnv.TextWidth(s);
  if n > Result then
    Result := n;
  s := GetTranslation(tag_Delete);
  n := Cnv.TextWidth(s);
  if n > Result then
    Result := n;
  s := GetTranslation(tag_Replace);
  n := Cnv.TextWidth(s);
  if n > Result then
    Result := n;
end;

procedure TILUpdateGrid.RelocateComboBox;
{ For FDropStyleChanged see ComboKeyDown}
var
  Used : Boolean;
  sl : TStringList;
begin
  if csDesigning in ComponentState then
  begin
    fComboBox.Width := 0;
    Exit;
  end;
  Used := True;
  if (Col = 0) and (Mode = gmUpdateActions) then
  begin
      fComboBox.Style := csDropDownList;
      fComboBox.Clear;
      fComboBox.Items.Add(GetTranslation(tag_Set));
      if ((Field <> nil) and Field.IsMultival)
       or (Field = nil) then
        fComboBox.Items.Add(GetTranslation(tag_Add));
      fComboBox.Items.Add(GetTranslation(tag_Delete));
      fComboBox.Items.Add(GetTranslation(tag_Replace));
      fComboBox.ItemIndex := fComboBox.Items.IndexOf(Cells[Col, Row]);
  end
  else if (Field <> nil) and (Field.LookupType = looPickValues) and (PickValues.Count > 0) then
  begin
     FDropStyleChanged := (fComboBox.Style <> csDropDownList) and FCBKeyDown;
     fComboBox.Style := csDropDownList;
     fComboBox.Items.Assign(PickValues);
     fComboBox.Text := Cells[Col, Row];
  end
  else if (Field <> nil) and (Field.LookupType in [looTableRef, looTableKey]) then
  begin
     FDropStyleChanged := (fComboBox.Style <> csDropDownList) and FCBKeyDown;
     fComboBox.Style := csDropDownList;
     sl := Table.GetLookups(Field.FieldName, '*', duData);
     sl.Insert(0, TMan.ReportNull);
     fComboBox.Items.Assign(sl);
     sl.Free;
     fComboBox.ItemIndex := fComboBox.Items.IndexOf(Cells[Col, Row]);
  end
  else
    Used := false;
  if Used then
  begin
   fcombobox.boundsrect := CellRect( Selection.Left, Selection.Top);
   FComboCol := Col;
   FComboRow := Row;
   fcomboBox.Visible:=TRUE;
   FComboBox.BringToFront;
   FComboBox.SetFocus;
  end
  else
    fcomboBox.Visible:=False;
  fComboBox.OnChange := ComboChanged;
end;

procedure TILUpdateGrid.Click;
var
  s : string;
begin
  inherited;
  if fComboBox.Visible then
  { Transfer value to cell }
  begin
    if (FOldCol = 0) or (FOldCol = 2) then
       Cells[FOldCol, FOldRow] := fComboBox.Text
     else if (fComboBox.Text <> '') then
       Cells[FOldCol, FOldRow] := fComboBox.Text;
  end;
  HideCombobox;
  s := GetTranslation(tag_Value);
  if (Col in [0,1]) and (Cells[0, Row] = GetTranslation(tag_Replace)) then
  begin
     Cells[1,0] := s + '1 ; ' + s + '2';
     if Col = 0 then
       RelocateComboBox;
  end
  else
  begin
    Cells[1,0] := s;
    RelocateComboBox;
  end;
  FOldCol := Col;
  FOldRow := Row;
  if Table <> nil then
   if Assigned(Table.LookupControl) and Table.LookupControl.Visible then
    Table.LookupControl.Visible := False;

end;

procedure TILUpdateGrid.Loaded;
begin
  inherited;
  SetMode(FMode);
end;

procedure TILUpdateGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
 X, Y: integer);
var
  Coord : TGridCoord;
begin
   { StringGrid tends to hide the combo when clicking on fixed row}
   if Mode = gmSimple then
   begin
     inherited;
     Exit;
   end;
   Coord := MouseCoord(X,Y);
   inherited;
   if Coord.Y = 0 then
   begin
     if FComboBox.Visible and HasCellCombo(0, 1) then
     begin
       FComboBox.BringToFront;
       FComboBox.SetFocus;
     end;
   end
end;

procedure TILUpdateGrid.SetMode(const Value : TUpdateGridMode);
var
  i : integer;
  W : integer;
begin
  FMode := Value;
  if csLoading in ComponentState then
    Exit;   { Sets in Loaded}
  if FMode = gmSimple then
  begin
    if FixedCols = 1 then
      ColWidths[0] := ColWidths[1];
    FixedCols := 0;
    FIxedRows := 0;
    for I := 0 to RowCount-1 do
      Cells[0, i] := '';

    ColCount := 1;
    Col := 0;
    Row := 0;
    FOldCol := 0;
    FOldRow := 0;
    DoAutoSizeHeight;
    DoAlignValueCol;
    RelocateComboBox;
    Exit;
  end;
  W := ColWidths[0];
  ColCount := 2;
  if RowCount < 2 then
    RowCount := 2;
  FixedRows := 1;
  Cells[0, 0] := GetTranslation(tag_Action);
  Cells[1, 0] := GetTranslation(tag_Value);
  ColWidths[0] := GetActionColWidth + 30;
  ColWidths[1] := W;
  for I := 1 to RowCount-1 do
  begin
     Cells[0, I] := GetTranslation(tag_Set);
     Cells[1, I] := '';
  end;
  Col := 0;
  Row := 1;
  FOldCol := 0;
  FOldRow := 1;
  FComboRow := 1;
  FComboCol := 0;
  DoAutoSizeHeight;
  DoAlignValueCol;
  if Parent <> nil then
    PostMessage(Handle, wm_ShowActionCombo, 0, 0);
end;

procedure TILUpdateGrid.WMSHOWACTIONCOMBO(var Message: TMessage);
begin
    RelocateComboBox;
end;

procedure TILUpdateGrid.DoAlignValueCol;
var
  CW : integer;
  R : TRect;
  ValCol : integer;
begin
 if not AlignValueCol then
   Exit;
 if Mode = gmSimple then
 begin
   ValCol := 0;
   ColWidths[ValCol] := ClientWidth -1;
   RelocateComboBox;
   Exit;
 end
 else
   ValCol := 1;
 CW := ColWidths[ValCol];
 R := CellRect(ValCol, 0);
 if GridWidth = ClientWidth then
  begin
     CW := CW - (CW - R.Width);
     ColWidths[ValCol] := CW-1;
  end
  else if GridWidth < ClientWidth then
  begin
     CW := CW + (ClientWidth-GridWidth)-1;
     ColWidths[ValCol] := CW;
  end;
  RelocateComboBox;
end;

procedure TILUpdateGrid.ResetColumns;
{ Only called from the Translator object: SetTranslations }
var
   I : integer;
   S1 : string;
   procedure SetTransl(tagName : string);
   begin
     FTranslations.Add(tagName +  ' : ' + S1 );
   end;
begin
  FTranslations.Clear;
  S1 := Translator.GetTranslation(T_Set, TRANSL_TERMS);
  SetTransl(tag_Set);
  S1 := Translator.GetTranslation(T_Add, TRANSL_TERMS);
  SetTransl(tag_Add);
  S1 := Translator.GetTranslation(T_Delete, TRANSL_TERMS);
  SetTransl(tag_Delete);
  S1 := Translator.GetTranslation(T_Replace, TRANSL_TERMS);
  SetTransl(tag_Replace);
  S1 := Translator.GetTranslation(T_Action, TRANSL_TERMS);
  SetTransl(tag_Action);
  S1 := Translator.GetTranslation(T_Value, TRANSL_TERMS);
  SetTransl(tag_Value);
  if Mode = gmUpdateActions then
  begin
   Cells[0,0] := Translator.GetTranslation(T_Action, TRANSL_TERMS);
   Cells[1,0] := Translator.GetTranslation(T_Value, TRANSL_TERMS);
   for I := 1 to RowCount-1 do
     Cells[0, i] := Translator.GetTranslation(T_Set, TRANSL_TERMS);
  end;
  RelocateComboBox;
end;

procedure TILUpdateGrid.SupplyOperators;
var
  LastRow : integer;
  i : integer;
begin
  if Mode = gmSimple then
    Exit;
  LastRow := -1;
  for I := 0 to RowCount-1 do
  begin
    if Cells[0, i] = '' then
     begin
       LastRow := i;
       Break;
     end;
  end;

  if LastRow <> -1 then
  for i := LastRow to RowCount-1 do
      Cells[0, i] := GetTranslation(tag_Set);
  inherited;
end;

procedure TILUpdateGrid.ClearValues;
var
  I: Integer;
begin
  for I := StartRow to RowCount-1 do
  begin
    if Mode = gmSimple then
     Cells[0, I] := ''
    else
     Cells[1, I] := '';
  end;
end;

{ TILSearchGrid -----------------------------------------------------------}

procedure TILSearchGrid.KeyDown(var Key: Word; Shift : TShiftState);
begin
  Inherited;
end;

procedure TILSearchGrid.RelocateComboBox;
{ For FDropStyleChanged see ComboKeyDown}
var
  Used : Boolean;
  procedure SetCoOps;
  begin
    fComboBox.Style := csDropDownList;
    fComboBox.Clear;
    fComboBox.Items.Add('=');
    if Field <> nil then
     if Field.IsMultiVal then
     FComboBox.Items.Add('==');
    fComboBox.Items.Add('<>');
    fComboBox.Items.Add('>');
    fComboBox.Items.Add('<');
    fComboBox.Items.Add('>=');
    fComboBox.Items.Add('<=');
    fComboBox.ItemIndex := fComboBox.Items.IndexOf(Cells[Col, Row]);
  end;
begin
  if csDesigning in ComponentState then
  begin
    Exit;
    FComboBox.Width := 0;
  end;
  Used := True;
  if Col = LogOpColIndex then
  begin
      fComboBox.Style := csDropDownList;
      fComboBox.Clear;
      fComboBox.Items.Add(c_LogAnd);
      fComboBox.Items.Add(c_LogOr);
      fComboBox.ItemIndex := fComboBox.Items.IndexOf(Cells[Col, Row]);
    end
  else if (Col = CoOpColIndex) then
    SetCoOps
  else if (Col = ValueColIndex) and (Field <> nil)
  and (Field.LookupType = looPickValues) and (PickValues.Count > 0) then
  begin
     FDropStyleChanged := (fComboBox.Style <> csDropDown) and FCBKeyDown;
     fComboBox.Style := csDropDown;
     fComboBox.Items.Assign(PickValues);
     fComboBox.Text := Cells[Col, Row];
  end
  else
    Used := false;
  if Used then
  begin
   fcombobox.boundsrect := CellRect( Selection.Left, Selection.Top);
   FComboCol := Col;
   FComboRow := Row;
   fcomboBox.Visible:=TRUE;
   FComboBox.BringToFront;
   FComboBox.SetFocus;
  end
  else
    fcomboBox.Visible:=False;
end;

procedure TILSearchGrid.ClearValues;
var
  i : integer;
begin
   for I := 0 to Rowcount-1 do
     Values[I] := '';
end;

procedure TILSearchGrid.DoAlignValueCol;
var
  R : TRect;
begin
 FSemafor := True;
 R := CellRect(ValueColIndex, 0);
 LeftCol := 0;
 try
 if GridWidth = ClientWidth then
  begin
     ValueColWidth := ValueColWidth - (ValueColWidth - R.Width);
  end
  else if GridWidth < ClientWidth then
     ValueColWidth := ValueColWidth + (ClientWidth-GridWidth)-1;
 finally
   FSemafor := false;
 end;
 RelocateComboBox;
end;

procedure TILSearchGrid.SupplyOperators;
var
  LastRow : integer;
  i : integer;
  Indx : integer;
begin
  LastRow := -1;
  Indx := -1;
  if CoOpColIndex <> -1 then
    Indx := CoOpColIndex
  else if LogOpColIndex <> -1 then
    Indx := LogOpColIndex;
  if Indx = -1 then
  begin
    inherited;
    Exit;
  end;

  for I := 0 to RowCount-1 do
  begin
    if Cells[Indx, i] = '' then
     begin
       LastRow := i;
       Break;
     end;
  end;

  if LastRow <> -1 then
  for i := LastRow to RowCount-1 do
  begin
    if CoOpColIndex <> - 1 then
      Cells[CoOpColIndex, i] := '=';
    if LogOpColIndex <> -1 then
    begin
      Cells[LogOpColIndex, i] := c_LogAnd;
    end;
  end;
  if not ShowFieldLogOp then
    Cells[LogOpColIndex, 0] := '';
  inherited;
end;

procedure TILSearchGrid.SetLogPickList(const Value : Boolean);
begin
  if Value = FLogPickList then
    Exit;
  FLogPickList := Value;
  ResetColumns;
end;

procedure TILSearchGrid.SetShowFieldLogOp(const Value : Boolean);
begin
  if Value = FShowFieldLogOp then
    Exit;

  FShowFieldLogOp := Value;
  if Assigned(Table) and Assigned(Tman) then
  begin
      if not Value and (TMan.SearchModel in [smAndAligned, smAndOrAligned])then
        FLogPicklist := false
      else
        FLogPickList := True;
  end;
  ResetColumns;
end;


Constructor TILSearchGrid.Create(AOWner:TComponent);
var
   I : integer;
begin
  inherited Create(Aowner);
  RowCount := 4;
  ColCount := 3;
  FCoPickList := True;
  FLogPickList := True;
  FShowFieldLogOp := True;
  for I := 0 to 4 do
  begin
     Cells[0,I] := c_LogAnd;
     Cells[1,I] := '=';
  end;

  ColWidths[0] := 40;
  ColWidths[1] := 35;
  ColWidths[2] := 170-75;
end;

function TILSearchGrid.ValueColIndex : integer;
begin
  Result := ColCount-1;
end;

function TILSearchGrid.LogOpColIndex : integer;
begin
  if LogPickList then
    Result := 0
  else
    Result := -1
end;

function TILSearchGrid.CoOpColIndex : integer;
begin
  if not CoPickList then
   Result := -1
  else if LogPickList then
    Result := 1
  else
    Result := 0;
end;

function TILSearchGrid.GetLogOp(Index : integer) : string;
begin
  if not LogPickList then
  begin
    Result := '';
    Exit;
  end;
  if fComboBox.Visible and (Col = LogOpColIndex)
  and (fComboBox.Text <> Cells[Col, Row]) then
    Cells[Col, Row] := FComboBox.Text;
  Result := Cells[LogOpColIndex, Index];
end;

function TILSearchGrid.GetCoOp(Index : integer) : string;
begin
  if not CoPicklist then
  begin
    Result := '';
    Exit;
  end;
  if fComboBox.Visible and (Col = CoOpColIndex)
  and (fComboBox.Text <> Cells[Col, Row]) then
     Cells[Col, Row] := FComboBox.Text;
  Result := Cells[CoOpColIndex, Index]
end;

procedure TILSearchGrid.ResetColumns;
var
  i, j : integer;
  RowStart : integer;
  Op : string;

  procedure SetFieldLogOp;
  begin
    if ShowFieldLogOp then
      Cells[0, RowStart] := c_LogAnd
    else
      Cells[0, RowStart] := '';
  end;

begin
  Op := c_LogAnd;
  if Assigned(Table) then
  begin
    if Table.AlignOperators then
     Op := ''
  end;

  RowStart := 0;
  for i := RowStart to RowCount-1 do
   for j := 0 to ColCount-1 do
     Cells[j, i] := '';

  { Either log or co picklists }
  if (CoPickList and not LogPicklist)
  or (not CoPicklist and LogPickList) then
  begin
    ColCount := 2;
    ColWidths[1] := ValueColWidth;
    if CoPickList then
    begin
      ColWidths[0] := CoOpColWidth;
      for I := RowStart to RowCount-1 do
      begin
       Cells[0,I] := '=';
       Cells[1,I] := '';
      end;
    end
    else
    begin
      ColWidths[0] := LogOpColWidth;
      for I := RowStart to RowCount-1 do
      begin
       Cells[0,I] := Op;
       Cells[1,I] := '';
      end;
      { When aligned ops, the first operator controls the value operators.
      The other operator fields are blank and inaccessable.

      When not aligned the first operator is the field logop, possibly inaccessable,
      depending on the value of ShowFieldLogOp }
      SetFieldLogOp;
    end;
  end

  { Both picklists }
  else if CoPickList and LogPickList then
  begin
    ColCount := 3;
    ColWidths[0] := LogOpColWidth;
    ColWidths[1] := CoOpColWidth;
    ColWidths[2] := ValueColWidth;
    for I := RowStart to RowCount-1 do
    begin
       Cells[0,I] := Op;
       Cells[1,I] := '=';
       Cells[2,I] := '';
    end;
    SetFieldLogOp;
  end

  { No picklists }
  else if not CoPickList and not LogPickList then
  begin
    ColCount := 1;
    ColWidths[0] := ValueColWidth;
    for I := RowStart to RowCount-1 do
     Cells[0,I] := '';
  end;
  Col := ValueColIndex;
  FOldRow := 0;
  FOldCol := Col;
  if AlignValueCol then
    DoAlignValueCol;
end;

procedure TILSearchGrid.WMALIGNLOGOPS(var Message: TMessage);
begin
  if TMan.SearchModel in [smAndOrMixed] then
    FLogPickList := True;
  ResetColumns;
end;

procedure TILSearchGrid.WMSHOWVALUECOMBO(var Message : TMessage);
begin
    if (Field <> nil) and (Field.LookupType = looPickValues) then
     if PickValues.Count > 0 then
      RelocateComboBox;
end;

function TILSearchGrid.GetValueColWidth : integer;
begin
    Result := ColWidths[ValueColIndex];
end;

procedure TILSearchGrid.SetValueColWidth(const Value : integer);
begin
   if AlignValueCol and not FSemafor then
     Exit;
   ColWidths[ValueColIndex] := Value;
end;

function TILSearchGrid.GetLogColWidth : integer;
begin
    Result := Canvas.TextWidth(c_LogAnd) + 25;
end;

function TILSearchGrid.GetCoColWidth : integer;
begin
    Result := Canvas.TextWidth('<>') + 25;
end;

procedure TILSearchGrid.SetLogColWidth(const Value : integer);
begin
    ColWidths[LogOpColIndex] := Value;
end;

procedure TILSearchGrid.SetCoColWidth(const Value : integer);
begin
    ColWidths[CoOpColIndex] := Value;
end;


procedure TILSearchGrid.SetCoPickList(const Value : Boolean);
begin
  if FCoPickList = Value then
    Exit;
  FCoPickList := Value;
  ResetColumns;
end;

procedure TILSearchGrid.Loaded;
begin
  if (Field <> nil) and (Field.LookupType = looPickValues) then
  if PickValues.Count > 0 then
  begin
     PostMessage(Handle, wm_ShowValueCombo, 0, 0);
  end;
  ResetColumns;
  inherited;
end;

function TILSearchGrid.SelectCell(ACol, ARow : Longint) : Boolean;
var
  R : integer;
begin
  if not Assigned(Table) or not LogPickList then
  begin
    Result := True;
    Exit;
  end;
  R := 0;
  if Table.AlignOperators and (ACol = 0) and (ARow = R) then
    Result := True
  else if Table.AlignOperators and (ACol = 0) and (ARow > R) then
    Result := false
  { If aligned ops and not show ops, then there will be no logop picklist }
  else if not Table.AlignOperators and (ACol = 0) and (ARow = R) and ShowFieldLogOp then
    Result := True
  else if not Table.AlignOperators and (ACol = 0) and (ARow = R) and not ShowFieldLogOp then
    Result := false
  else
    Result := True;
end;

{ TILImage -----------------------------------------------------------------}

  procedure TILImage.CreateParams(var Params: TCreateParams);
  begin
    inherited CreateParams(Params);
    Params.Style := Params.Style or ws_Border;
  end;

  procedure TILImage.UpdateImage;
  var
    P : TPicture;
    LTID : integer;
    R : TILDataRecord;
  begin
      if InZoomWindow then
        Exit;
      if Tman.UseCommandModes and (TMan.CommandMode <> cmUpdate) then
        raise Exception.Create('Please switch to update mode to change the image');

      if FSourceField <> nil then
      begin
        LTID := StrToInt(FSourcefield.Table.GetLookupTableID(FSourceField.FieldDef.LookupTable,
        FSourceField.FieldDef.LookupField, FSourceField.AsString));
        R := FSourceField.FieldDef.LookupTable.RecordsX[LTID];
        if FFileName <> '' then
          R.UpdateImage(FFileName)
        else
        begin
           P := TPicture.Create;
           try
            P.Assign(FSourceBM);
            R.UpdateImage(P)
           finally
             P.Free;
             R.Free;
           end;
        end;
      end
      else if FSourceRecord <> nil then
      begin
        if FFileName <> '' then
          FSourceRecord.UpdateImage(FFileName)
        else
        begin
           P := TPicture.Create;
           try
            P.Assign(FSourceBM);
            FSourceRecord.UpdateImage(P)
           finally
             P.Free;
           end;
        end;
      end;
  end;

  function TILImage.GetSourcePicture : TPicture;
  begin
    if SourceBitmap = nil then
    begin
      Result := nil;
      Exit;
    end;
    Result := TPicture.Create;
    Result.Assign(SourceBitmap);
  end;

  function TILImage.GetSourceBitmap : VCL.Graphics.TBitmap;
  begin
       if FSourceBM.Width > 0 then
         Result := FSourceBM
       else
         Result := nil;
  end;

  function TILImage.GetDrawBitmap : VCL.Graphics.TBitmap;
  begin
       if FDrawBM.Width > 0 then
         Result := FDrawBM
       else
         Result := nil;
  end;

  function TILImage.GetJPEGScale : TJPEGScale;
  begin
    Result := jsFullSize;
    case JPEGQuality of
      jqHigh: Result := jsFullSize;
      jqGood: Result := jsHalf;
      jqLow: Result := jsQuarter;
      jqDraft: Result := jsEighth;
      jqAuto: Result := jsFullSize;
    end;
  end;

  function TILImage.GetNextRect : TRect;
  var
    Y : integer;
    X : integer;
    W, H : integer;
  begin
    Canvas.Font.Name := 'Tahoma';
    Canvas.Font.Charset := SYMBOL_CHARSET;
    Canvas.Font.Size := 18;
    W := Canvas.TextWidth(chr(161)) + 5;
    H := Canvas.TextHeight(chr(161)) + 5;
    X := Left + Width - W ;
    Y := (Top + Height) div 2 - H;
    Result := Rect(X, Y, W + X, H + Y);
  end;

  function TILImage.GetPrevRect : TRect;
  var
    Y : integer;
    X : integer;
    W, H : integer;
  begin
    Canvas.Font.Name := 'WingDings 3';
    Canvas.Font.Charset := SYMBOL_CHARSET;
    Canvas.Font.Size := 18;
    W := Canvas.TextWidth(chr(162)) + 5;
    H := Canvas.TextHeight(chr(162)) + 5;
    X := Left + 5 ;
    Y := (Top + Height) div 2 - H;
    Result := Rect(X, Y, W + X, H + Y);
  end;

  function TILImage.GetTable : TILDataTable;
  begin
    if FSourceField <> nil then
      Result := FSourceField.Table as TILDataTable
    else if FSourceRecord <> nil then
      Result := FSourceRecord.Table as TIlDataTable
    else
      Result := nil;
  end;

  function TILImage.GetDataset : TILDataSet;
  begin
     if FSourceField <> nil then
      Result := FSourceField.Table.Dataset
    else if FSourceRecord <> nil then
      Result := FSourceRecord.Table.Dataset
    else
      Result := nil;
  end;

  procedure TILImage.Resize;
  begin
    if FSourceBM = nil then
    begin
      inherited;
      Exit;
    end;
    FBM.Assign(FSourceBM);
    ScaleBitmap(FBM, ClientWidth, ClientHeight);
    SetDrawBM;
    inherited;
    Invalidate;
  end;

  procedure TILImage.MouseMove(Shift: TShiftState; X, Y: integer);
  var
    R : TRect;
  begin
    if not InZoomWindow or (SourceRecord = nil) then
    begin
      inherited;
      Exit;
    end;
    FMouseInNext := false;
    FMouseInPrev := false;
    R := GetNextRect;
    if PtInRect(R, Point(X,Y)) then
    begin
      FMouseInNext := True;
      SetDrawBM;
      Invalidate;
      inherited;
      Exit;
    end;
    R := GetPrevRect;
    if PtInRect(R, Point(X,Y)) then
    begin
      FMouseInPrev := True;
      SetDrawBM;
      Invalidate;
    end
    else
    begin
      SetDrawBM;
      Invalidate;
    end;
    Inherited;
 end;

  procedure TILImage.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: integer);
  var
    R : TRect;
    W, H : integer;

    procedure DoPlayerEvent;
    var
      R : TRect;
      XPos, YPos : integer;
    begin
        if SourceRecord = nil then
          Exit;
        if Assigned(TMan.OnClickPlayerIcon) then
        begin
          if Assigned(TMan.FIntPlayerBM) or Assigned(TMan.FIntPlayerIcon)
          and (SourceRecord.MediaType in [MED_AUDIO, MED_VIDEO]) then
          begin
              XPos := (ClientWidth div 2) - 10;
              YPos := (ClientHeight div 2) - 10;
              R := Rect(XPos, YPos, XPos + 20, YPos + 20);
              if PtInRect(R, Point(X,Y)) then
                TMan.FOnClickPlayerIcon(Self, SourceRecord, X, Y);
          end;
        end;
    end;

  begin
       SetFocus;
       if InZoomWindow and (SourceRecord = nil) then
       begin
         Inherited;
         Exit;
       end;
       if InZoomWindow then
       begin
         if Dataset.Count = 1 then
          Exit;
         if not Dataset.EOF and FMouseInNext then
         begin
           Dataset.Next;
         end
         else if not Dataset.BOF and FMouseInPrev then
         begin
           Dataset.Previous;
         end;
         DoPlayerEvent;
         Exit;
       end;
       if FSourceField = nil then
       begin
         DoPlayerEvent;
         inherited;
         Exit;
       end;
       FDrawBM.Canvas.Font.Name := 'Wingdings 3';
       FDrawBM.Canvas.Font.Charset := SYMBOL_CHARSET;
       FDrawBM.Canvas.Font.Size := 14;
       W:= FDrawBM.Canvas.TextWidth(Chr(161));
       H:= FDrawBM.Canvas.TextHeight(Chr(161));
       R.Left := 5;
       R.Top := 5;
       R.Width := W;
       R.Height := H;
       if PtInRect(R, Point(X,Y)) then
         ShowRecordImage
       else
         DoPlayerEvent;

       inherited;
  end;

  procedure TILImage.WMGetDlgCode(var Message: TWMGetDlgCode);
  begin
    inherited;
    Message.Result := Message.Result or DLGC_WANTTAB or DLGC_WANTARROWS;
  end;

  procedure TILImage.WndOnShow(Sender : TObject);
  begin
     if Dataset = nil then
       Exit;
     if DataSet.Count = 0 then
       ClearSource
     else
       LoadFromRecord(Dataset.FocusedRecord);
     if Dataset.Ilib.HasViewer then
       Dataset.Ilib.Viewer.ZoomWindow.Caption := 'Image ' + IntToStr(Dataset.FocusedRecord.Index+1) + '/' +
        IntToStr(Dataset.Count);
  end;

  procedure TILImage.KeyDown(var Key : word; Shift : TShiftState);
  begin
      if InZoomWindow and (SourceRecord <> nil) then
      begin
        if (Key = VK_Right) or ((Key = vk_Tab) and (Shift = [])) then
          Dataset.Next
        else if (Key = VK_Left) or ((Key = vk_Tab) and (Shift = [ssShift])) then
          Dataset.Previous
        else if Key = VK_End then
          Dataset.Last
        else if Key = VK_Home then
          Dataset.First;
        if Dataset.Ilib.HasViewer then
         Dataset.Ilib.Viewer.ZoomWindow.Caption := 'Image ' + IntToStr(Dataset.FocusedRecord.Index+1) + '/' +
           IntToStr(Dataset.Count);

      end;
      inherited;
  end;

  procedure TILImage.SetDrawBM;
  var
    X, Y : integer;
    W, H : integer;
  begin
     if SourceBitmap = nil then
       Exit;
     FDrawBM.Width := ClientWidth;
     FDrawBM.Height := ClientHeight;
     X := (ClientWidth - FBM.Width) div 2;
     Y := (ClientHeight - FBM.Height) div 2;
     if TStyleManager.IsCustomStyleActive then
        FDrawBM.Canvas.Brush.Color := StyleServices.GetStyleColor(scGrid)
       else
        FDrawBM.Canvas.Brush.Color := Color;
     FDrawBM.Canvas.FillRect(Rect(0,0,ClientWidth, ClientHeight));
     FDrawBM.Canvas.Draw(X, Y, FBM);
     if EmptySource then
       Exit;
     if (FSourceField <> nil) and not InZoomWindow then
     begin
       FDrawBM.Canvas.Brush.Style := bsClear;
       FDrawBM.Canvas.Font.Name := 'Wingdings 3';
       FDrawBM.Canvas.Font.Charset := SYMBOL_CHARSET;
       FDrawBM.Canvas.Font.Size := 14;
       FDrawBM.Canvas.Font.Color := clLime;
       FDrawBM.Canvas.TextOut(X+5, Y+5, Chr(161));
     end
     else if InZoomWindow and ShowNavButtons and (SourceRecord <> nil) then
     begin
       FDrawBM.Canvas.Font.Name := 'Wingdings 3';
       FDrawBM.Canvas.Font.Charset := SYMBOL_CHARSET;
       FDrawBM.Canvas.Brush.Style := bsClear;
       FDrawBM.Canvas.Font.Size := 18;
       if not Dataset.BOF then
         FDrawBM.Canvas.TextOut(GetPrevRect.Left, GetPrevRect.Top, Chr(161));
       if not Dataset.EOF then
         FDrawBM.Canvas.TextOut(GetNextRect.Left, GetNextRect.Top, Chr(162));
     end;

     if (SourceRecord <> nil) and (SourceRecord.MediaType in [MED_AUDIO, MED_VIDEO])
     and (Assigned(TMan.FIntPlayerIcon) or Assigned(TMan.FIntPlayerBM)) then
     begin
        if Assigned(TMan.FIntPlayerIcon) then
        begin
          W := TMan.FIntPlayerIcon.Width div 2;
          H := TMan.FIntPlayerIcon.Height div 2;
        end
        else
        begin
          W := TMan.FIntPlayerBM.Width div 2;
          H := TMan.FIntPlayerBM.Height div 2;
        end;
        X := X + (FBM.Width div 2) - W;
        Y := Y + (FBM.Height div 2) - H;
        if Assigned(TMan.FIntPlayerBM) then
          FDrawBM.Canvas.Draw(X, Y, TMan.FIntPlayerBM)
        else
          FDrawBM.Canvas.Draw(X, Y, TMan.FIntPlayerIcon);
     end;
  end;

  procedure TILImage.Paint;
  begin
     if (csDesigning in ComponentState) or (DrawBitmap = nil) then
     begin
       if TStyleManager.IsCustomStyleActive then
        Canvas.Brush.Color := StyleServices.GetStyleColor(scGrid)
       else
        Canvas.Brush.Color := Color;
       Canvas.FillRect(Rect(0,0, ClientWidth, ClientHeight));
       Exit;
     end;
     Canvas.Draw(0,0, FDrawBM);
  end;

  procedure TILImage.WMERASEBKGND(var Msg: TWMEraseBkgnd);
  begin
    Msg.Result := 1;
  end;

  constructor TILImage.Create(AOwner : TComPonent);
  begin
      inherited;
      ControlStyle := ControlStyle + [csReplicatable, csPannable, csOpaque];
      Width := 200;
      Height := 150;
      FBM := VCL.Graphics.TBitmap.Create;
      FSourceBM := VCL.Graphics.TBitmap.Create;
      FDrawBM := VCL.Graphics.TBitmap.Create;
      JPEGQuality := jqAuto;
  end;

  destructor TILImage.Destroy;
  begin
      FBM.Free;
      FSourceBM.Free;
      FDrawBM.Free;
      inherited;
  end;

  procedure TILImage.LoadImage(AnImage : TPicture);
  begin
      LoadFromPicture(AnImage);
  end;

  procedure TILImage.LoadImage(AFileName : string);
  begin
      if AFileName = CLIPBOARD_IMAGE then
        LoadFromClipBoard
      else
        LoadFromFile(AFileName);
  end;

  procedure TILImage.LoadFromFile(FileName : string);
  { General load from file. If source is present, it is updated }
  var
    Y, W, H, i : integer;
    S, S2 : string;
    Err, Linebreak : Boolean;
    Scl : TJPegScale;
    Res : Hresult;
    IsJPG : Boolean;
    XPics, YPics : integer;

    procedure DrawLine;
    var
      W, X : integer;
    begin
      W := FBM.Canvas.TextWidth(S2);
      X := ((ClientWidth-10) div 2) - (W div 2) + 5;
      FBM.Canvas.TextOut(X, Y, S2);
      inc(Y, FBM.Canvas.TextHeight('I')+ 5);
      System.Delete(S, 1, i);
    end;

  function GetJpegSize(out width, height : integer): boolean;
  var n: integer;
    b: byte;
    w: word;
    jpeg : TFileStream;
  begin
    result := false;
    jPeg := TFileStream.Create(FileName, fmOpenRead);
    try
      n := jpeg.Size-8;
      jpeg.Position := 0;
      if n<=0 then
        exit;
      jpeg.Read(w,2);
      if w<>$D8FF then
        exit; // invalid format
      jpeg.Read(b,1);
      while (jpeg.Position<n) and (b=$FF) do
      begin
        jpeg.Read(b,1);
        case b of
        $C0..$C3:
        begin
          jpeg.Seek(3,soFromCurrent);
          jpeg.Read(w,2);
          height := swap(w);
          jpeg.Read(w,2);
          width := swap(w);
          jpeg.Read(b,1);
          Result := true; // JPEG format OK
          exit;
        end;
        $FF:
         jpeg.Read(b,1);
        $D0..$D9, $01:
        begin
          jpeg.Seek(1,soFromCurrent);
          jpeg.Read(b,1);
        end;
        else
        begin
          jpeg.Read(w,2);
          jpeg.Seek(swap(w)-2, soFromCurrent);
          jpeg.Read(b,1);
        end;
      end;
    end;
    finally
      jPeg.Free;
    end;
  end;

begin
      FreeAndNil(FBM);
      if FileName = CLIPBOARD_IMAGE then
      begin
         LoadFromClipBoard;
         Exit;
      end;

      if IsVideoFile(Filename) or IsAudioFile(FileName) then
      begin
       W := 500;
       FBM := Vcl.Graphics.TBitmap.Create;
       if FSourceRecord <> nil then
         FSourceRecord.Ilib.GetImgDimensions(W, H);
       if W = -1 then
         W := 500;
       res := GetMediaThumb(FileName, W, FBM);
       if Res <> S_OK then
       begin
          FBM.Free;
          HandleILException(E_ImageNotAvailable)
       end;
      end;

      Err := false;
      Scl := jsFullSize;
      IsJPG := (ImageLibrary.GetFileType(FileName) = iftJPG);
      try

       if (JPegQuality = jqAuto) and IsJPG then
       begin
         GetJPegSize(XPics, YPics);

         if (XPics <= ClientWidth) or (YPics <= ClientHeight) then
           Scl := jsFullSize
         else if (XPics div 2 <= ClientWidth) or (YPics div 2 <= ClientHeight) then
           Scl := jsHalf
         else if (XPics div 4 <= ClientWidth) or (YPics div 4 <= ClientHeight) then
           Scl := jsQuarter
         else
           Scl := jsEighth;

         if Scl = jsHalf then
         begin
           if (XPics div 2 < ClientWidth)
            and (YPics div 2 < ClientHeight) then
             Scl := jsFullSize;
         end
         else if Scl = jsQuarter then
         begin
           if (XPics div 4 < ClientWidth)
            and (YPics div 4 < ClientHeight) then
             Scl := jsHalf;
         end
         else if Scl = jsEighth then
         begin
           if (XPics div 8 < ClientWidth)
            and (YPics div 8 < ClientHeight) then
             Scl := jsQuarter;
         end
       end
       else if IsJPG then
         Scl := GetJPEGScale;

       if not IsVideoFile(Filename) and not IsAudioFile(FileName) then
        FBM := GetBitmapFromFile(FileName, Scl);
       if FBM = nil then
        HandleILException(E_UnsupportedFileFormat, Filename);

      Except on E:Exception do
       begin
        if FBM = nil then
          raise;
        ClearSource;
        FBM.Width := ClientWidth;
        FBM.Height := ClientHeight;
        S := E.Message;
        S2 := '';
        LineBreak := True;
        Y := ClientHeight div 2;
        while LineBreak do
        begin
         S2 := '';
         LineBreak := false;
         for I := 1 to Length(S) do
         begin
           S2 := S2 + S[i];
           W := FBM.Canvas.TextWidth(S2);
           if W >= (ClientWidth-10) then
           begin
            DrawLine;
            LineBreak := True;
            Break;
           end;
         end;
        end;
        if S2 <> '' then
         DrawLine;
        FSourceBM.Assign(FBM);
        FDrawBM.Assign(FSourceBM);
        Err := True;
       end;

      end;

      if not Err then
      begin
       if IsJPG then
        CheckOrientation(FileName, FBM);
       FSourceBM.Assign(FBM);
       ScaleBitmap(FBM, ClientWidth, ClientHeight);
       SetDrawBM;
      end;
      FFileName := FileName;
      if not FInternalLoad and not Err and not EmptySource then
        UpdateImage;
      Invalidate;
  end;

  procedure TILImage.LoadFromPicture(APicture : TPicture);
   { General load from picture. If source is present, it is updated }
  begin
      if APicture = nil then
      begin
        Clear;
        Exit;
      end;
      FBM.Assign(APicture.Graphic);
      FSourceBM.Assign(FBM);
      ScaleBitmap(FBM, ClientWidth, ClientHeight);
      FFileName := '';
      SetDrawBM;
      if not EmptySource then
        UpdateImage;
      Invalidate;
  end;

  procedure TILImage.LoadFromClipBoard;
  { General load from clip board. If source is present, it is updated }
  var
    Url : Boolean;
    Img : TWicImage;

    procedure DoLoad;
    begin
        FSourceBM.Assign(FBM);
        ScaleBitmap(FBM, ClientWidth, ClientHeight);
        Invalidate;
        FFileName := '';
        SetDrawBM;
        if not FInternalLoad and not EmptySource then
          UpdateImage;
    end;
  begin
     if InZoomWindow then
       HandleILException(E_UpdateZoomed);
     if ClipBoard.HasFormat(cf_Bitmap) then
     begin
        FBM.LoadFromClipboardFormat(cf_Bitmap,
           ClipBoard.GetAsHandle(cf_Bitmap), 0);
        DoLoad;
     end
     else if ClipBoard.HasFormat(cf_Text) then
     begin
        Url := false;
        FFileName := GetAddressFromClipBrd(Url);
        if FFileName = '' then
          HandleILException(E_ImageNotAvailable);
        {$IfDef UseIndy}
        if Url then
        begin
          Img := DownLoadImage(FFileName);
          try
            FBM.Assign(Img);
          finally
            if Img <> nil then
              Img.Free;
          end;
          DoLoad;
        end
        else
        {$EndIf}
        begin
          if Url then
           HandleILException(E_ImageNotAvailable)
          else
           LoadFromFile(FFileName);
        end;
     end

  end;

  procedure TILImage.LoadFromField(AField : TILField);
  var
    LTID : Integer;
    R : TILDataRecord;
  begin
    if AField = nil then
    begin
      ClearSource;
      Exit;
    end;
    if (TMan = nil) or not (TMan.ActiveTable = AField.Table) then
      Exit;
    if not (AField.FieldDef.LookupTable is TILImageTable)  then
      Exit;
    LTID := StrToInt(Afield.Table.GetLookupTableID(AField.FieldDef.LookupTable,
    AField.FieldDef.LookupField, AField.AsString));
    R := AField.FieldDef.LookupTable.RecordsX[LTID];
    try
      LoadFromRecord(R);
      FSourceField := AField;
      FSourceRecord := nil;
      SetDrawBM;
    finally
      R.Free;
    end;
  end;

  procedure TILImage.LoadFromRecord(ARecord : TILDataRecord);
  var
   BM : VCL.Graphics.Tbitmap;
  begin
    if ARecord = nil then
    begin
      ClearSource;
      Exit;
    end;

    if (ARecord.FileName <> '') and (ARecord.MediaType <> MED_VIDEO) then
    begin
      FInternalLoad := True;
      try
        FSourceRecord := ARecord;
        LoadFromFile(ARecord.FileName);
      finally
        FInternalLoad := false;
      end;
    end
    else
    begin
      if Arecord.FImageChanged or (ARecord.RecordIndex = -1)  then
      begin
       if not ARecord.FImageFetched then
         ARecord.FetchImage;
       BM :=  ARecord.FImage;
      end
      else
       BM := ARecord.Table.GetImage(ARecord);
      try
       if BM = nil then
       begin
         FSourceRecord := ARecord;
         FSourceField := nil;
         InternalClear;
         Exit;
       end;
       FBM.Assign(BM);
       FSourceBM.Assign(FBM);
       ScaleBitmap(FBM, ClientWidth, ClientHeight);
       FSourceRecord := ARecord;
       FSourceField := nil;
       SetDrawBM;
       Invalidate;
      finally
        if (BM <> nil) and (BM <> ARecord.FImage) then
          BM.Free;
      end;
    end;
  end;

  procedure TILImage.SetSourceRecord(const Value : TILDataRecord);
  begin
      LoadFromRecord(Value);
  end;

  procedure TILImage.SetSourceField(const Value : TILField);
  begin
      LoadFromField(Value);
  end;

  procedure TILImage.ShowRecordImage;
  begin
     if EmptySource then
       Exit;
     if (FSourceRecord <> nil) then
       if FSourceRecord = FSourceRecord.Table.FocusedRecord then
         Exit;
     if FSourceField = nil then
       Exit;
     LoadFromRecord(FSourceField.Table.FocusedRecord);
  end;

  procedure TILImage.InternalClear;
  begin
    FBM.SetSize(0,0);
    FDrawBM.SetSize(0,0);
    FSourceBM.SetSize(0,0);
    Invalidate;
  end;

  procedure TILImage.Clear;
  begin
    InternalClear;
    if not EmptySource then
      FSourceRecord.UpdateImage(nil);
  end;

  procedure TILImage.ClearSource;
  begin
    InternalClear;
    FSourceField := nil;
    FSourceRecord := nil;
  end;

  procedure TILImage.RemoveSource;
  begin
    FSourceField := nil;
    FSourceRecord := nil;
  end;

  function TILImage.EmptySource : Boolean;
  begin
     Result := (FSourceField = nil) and (FSourceRecord = nil);
  end;

{ TILTreeView --------------------------------------------------------------- }

procedure TILTreeView.BuildTree(AdataSet: TILDataset);
var
  S: string;
  TN: TTreeNode;
  i: integer;
  R: TILDataRecord;
  AddPrnt : Boolean;

  procedure AddChildren(ATreeNode: TTreeNode; AParent: TILDataRecord);
  var
    i: integer;
    CN: TTreeNode;
    R: TILDataRecord;
    AddChld : Boolean;
  begin

    for i := 0 to AdataSet.Count - 1 do
    begin
      R := AdataSet[i];
      if R.RecordIndex <> -1 then
      begin
        if (R = AParent) or R[c_SysFieldNames[SYS_LinkID]].IsEmpty then
          Continue
      end
      else
      if (R = AParent) or (R.TVParentRecord = nil) then
        Continue;

      AddChld := false;
      if R.RecordIndex <> -1 then
      begin
       if R[c_SysFieldNames[SYS_LinkID]].AsInteger = AParent.RecordIndex then
         AddChld := True;
      end
      else
         AddChld := (R.TVParentRecord = AParent);
      if AddChld then
      begin
        if TVTable.TreeViewLabelField = '' then
          S := '??'
        else
          S := R[TVTable.TreeViewLabelField].AsString;
        if R.FIsDeleted then
         S := '(' + S + ')';
        CN := Items.AddChild(ATreeNode, S);
        CN.HasChildren := True;
        CN.Data := R;
        AddChildren(CN, R);
      end;
    end;
  end;

begin
  Building := True;
  Items.Clear;
  if TVTable.HasDatasetTemps then
    Exit;
  Items.BeginUpdate;

  try

  for i := 0 to AdataSet.Count - 1 do
  begin
    R := AdataSet[i];
    if R.RecordIndex <> -1 then
     AddPrnt := (R[c_SysFieldNames[SYS_LinkID]].IsEmpty)
    else
     AddPrnt := (R.TVParentRecord = nil);
    if AddPrnt then
    begin
      if TVTable.TreeViewLabelField = '' then
        S := '??'
      else
        S := AdataSet[i][TVTable.TreeViewLabelField].AsString;
      if R.FIsDeleted then
        S := '(' + S + ')';
      TN := Items.AddChild(nil, S);
      TN.Data := R;
      TN.HasChildren := True;
      AddChildren(TN, R);
    end;
  end;

  Building := false;
  R := TVTable.FocusedRecord;
  if R <> nil then
  begin
    TN := NodeOfRecord(R);
    if TN <> nil then
      Selected := TN;
  end;
  if (TVTable.TreeViewLabelField = '') or (TVTable.Dataset.IsReadOnly) then
    ReadOnly := True;
  finally
    Items.EndUpdate;
  end;
end;

function TILTreeView.NodeOfRecord(ARecord: TILRecord): TTreeNode;
var
  i: integer;
  R: TILDataRecord;
begin
  Result := nil;
  if ARecord = nil then
   Exit;
  for i := 0 to Items.Count - 1 do
  begin
    R := TILDataRecord(Items[i].Data);
    if R = ARecord then
    begin
      Result := Items[i];
      Break;
    end;
  end;

end;

procedure TILTreeView.WMREFRESHTREEVIEW(var Message: TMessage);
var
  N: TTreeNode;
  R: TILDataRecord;
  Indx: integer;
begin
  BuildTree(TVTable.Dataset);
  Indx := TVTable.Dataset.IndexOf(Message.WParam);
  R := TVTable.Dataset[Indx];
  N := NodeOfRecord(R);
  Selected := N;
  N.Expand(false);
end;

function TILTreeView.CanExpand(Node: TTreeNode): Boolean;
var
  R: TILDataRecord;
  Cnt: integer;
begin
  Result := inherited CanExpand(Node);
  if not Result then
    Exit;
  if Node = nil then
    Exit;
  if (Node.Count = 0) and Node.HasChildren then
  begin
    Result := false;
    R := TILDataRecord(Node.Data);
    Cnt := TVTable.ExpandNode(R, false);
    if Cnt = 0 then
    begin
      Node.HasChildren := false;
      Exit;
    end;
    PostMessage(Handle, WM_RefreshTreeView, R.RecordIndex, 0);
  end
  else
    Result := True;
end;

function TILTreeView.CanChange(Node: TTreeNode): Boolean;
var
  R: TILDataRecord;
  P: integer;
begin
  Result := inherited CanChange(Node);
  if not Result then
    Exit;
  if Building then
    Exit;
  P := TVTable.Dataset.Position;
  R := TILDataRecord(Node.Data);
  TVTable.Dataset.Position := R.Index;
  if (P = TVTable.Dataset.Position) and (Selected <> nil) then
    Result := false
  else if TVTable.Viewer <> nil then
    TVTable.Viewer.ScrollTo(R.Index);
end;

procedure TILTreeView.Expand(Node: TTreeNode);
begin
  inherited;
end;

procedure TILTreeView.Change(Node: TTreeNode);
begin
  inherited;
end;

procedure TILTreeView.Edit(const Item: TTVItem);
var
  R: TILDataRecord;
begin
  inherited;
  R := TILDataRecord(Selected.Data);
  R[TVTable.TreeViewLabelField].AsString := Selected.Text;
  TVTable.Inval;
end;

procedure TILTreeView.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if (Key = vk_Delete) and (Selected <> nil) and not ReadOnly
  and not IsEditing then
    Delete(Selected);
end;

{ TILRecordViewer -----------------------------------------------------------}

constructor TILRecordViewer.Create(AOwner : TComponent);
begin
    inherited;
    ViewStyle := vsReport;
    Columns.Add;
    Columns.Add;
    FFieldCaption := 'Field';
    FValueCaption := 'Value';
    Columns[0].Caption := FFieldCaption;
    Columns[1].Caption := FValueCaption;
    ReadOnly := True;
    RowSelect := True;
    FViewElements := [reData];
end;

destructor TILRecordViewer.Destroy;
begin
  inherited;
end;

procedure TILRecordViewer.SetImageControl(const Value : TILImage);
begin
  if Value = FImageControl then
    Exit;
  FImageControl := Value;
  if Value <> nil then
    Value.FreeNotification(Self);
end;

procedure TILRecordViewer.SetFieldCaption(const Value : string);
begin
  if Value = FFieldCaption then
    Exit;
  FFieldCaption := Value;
  Columns[0].Caption := Value;
end;

procedure TILRecordViewer.SetValueCaption(const Value : string);
begin
  if Value = FValueCaption then
    Exit;
  FValueCaption := Value;
  Columns[1].Caption := Value;

end;

procedure TILRecordViewer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if csDestroying in ComponentState then
    Exit;
  if Operation = opRemove then
  begin
    if AComponent = ImageControl then
    begin
      FImageControl := nil;
    end;
  end;
end;

procedure TILRecordViewer.ShowRecord(ARecord : TILDataRecord);
  var
    Rec: TILDataRecord;
    i, j: integer;
    Fld: TILFieldDef;
    S: string;
    Indx: integer;
    Li: TListItem;
    Val: string;
    Prop: TILPropertyValue;
    Tbl : TILTable;
    W, FW, VW : integer;
    Dt : TDateTime;
    N : integer;
    StateIndx : integer;
    NF : Extended;

    function GetBoolVal(Bool: string): string;
    begin
      Result := Bool;
      if (Bool = '0') and (TrimBlanks(Tman.ReportFalse) <> '') then
        Result := TMan.ReportFalse
      else if (Bool = '1') and (TrimBlanks(TMan.ReportTrue) <> '') then
        Result := TMan.ReportTrue;
    end;

    procedure SetW(Cap, Val : string);
    begin
      W := Canvas.TextWidth(Cap);
      FW := Max(FW, W);
      W := Canvas.TextWidth(Val);
      VW := Max(VW, W);
    end;

    procedure DoStateEvent(LI : TListItem);
    begin
      StateIndx := -1;
        if Assigned(FOnStateImage) and Assigned(StateImages) then
         FOnStateImage(Self, Li.Index, StateIndx);
      Li.StateIndex := StateIndx;
    end;

  begin
    if ARecord = nil then
    begin
      Clear;
      Exit;
    end;
    TBL := ARecord.Ilib;
    FW := Canvas.TextWidth('Field');
    VW := Canvas.TextWidth('Value');
    Clear;
    Rec := ARecord;

    if ImageControl <> nil then
    begin
       ImageControl.LoadFromRecord(Rec);
    end;

    if reData in FViewElements then
    begin
      for i := 0 to Rec.Count - 1 do
      begin
        Indx := Tbl.FFields.IndexOf(Rec[i].FieldName);
        Fld := Tbl.FFields.Items[Indx];
        if Fld.FieldType = ftStoredEvaluation then
          continue;
        if not Fld.ReportField then
          Continue;
        if SameText(Fld.FieldName, c_SysFieldNames[SYS_LinkID]) then
          Continue;
        if (Fld.DataType = dtDate) and not Rec[i].IsEmpty and
          (Rec[i].AsDateTime = 0) then
          if not ShowEmptyFields then
            Continue;
        if (Rec[i].FValues.Count = 0) and not ShowEmptyFields then
          Continue;
        if not Rec.Ilib.HasHandler then
          Indx := -1
        else
          Indx := Rec.Ilib.DataHandler.DataFields.IndexOf(Fld.FieldName);
        if (Indx <> -1) and (Rec.Ilib.DataHandler.DataFields.Items[Indx].Caption <> '') then
          S := Rec.Ilib.DataHandler.DataFields.Items[Indx].Caption
        else
          S := Fld.Caption;

        Li := Items.Add;
        Li.ImageIndex := -1;
        Li.Caption := S;
        DoStateEvent(LI);
        W := Canvas.TextWidth(S);
        FW := Max(FW, W);

        for j := 0 to Rec[i].FValues.Count - 1 do
        begin
          if Fld.LookupType = looTableKey then
            Val := rec[i].DisplayValue
          else if Fld.FDataType = dtBoolean then
            Val := GetBoolVal(Rec[i].FValues[j])
          else if Fld.FDataType in [dtResource] then
            Val := GetResourceDisplayElement(Rec[i].FValues[j])
          else if Fld.DataType = dtLongText then
            Val := Rec[i].GetSampleLongText(50)
          else if (Fld.DataType = dtDate) and (Rec[i].AsDateTime = 0) then
            Val := ''
          else
            Val := Rec[i][j];
          if (Fld.DataType = dtDate) and (Fld.DateFormat <> '') and (Val <> '') then
          begin
            Dt := StrToDateTime(Val, Translator.FFormatSettings);
            Val := FormatDateTime(Fld.DateFormat, Dt);
          end;
          if (Fld.DataType = dtNumber) and (Fld.Decimals = 0) and (Val <> '') then
          begin
             if Fld.Decimals > 0 then
             begin
                NF := StrToFloat(Val, Translator.FFormatSettings);
                Val := Translator.FormatNum(NF, Fld.Decimals);
             end
             else if Fld.Decimals = 0 then
             begin
               N := StrToInt(Val);
               Val := Translator.FormatNum(N);
             end;
          end;

          if j = 0 then
            Li.SubItems.Add(Val)
          else
          begin
            Li := Items.Add;
            Li.ImageIndex := -1;
            Li.SubItems.Add(Val)
          end;
          W := Canvas.TextWidth(Val);
          VW := Max(VW, W);
        end;
      end;
    end;

    if (reMediaProps in FViewElements) and Assigned(Rec.PropertyValues) then
    begin
        for i := 0 to Rec.PropertyValues.Count - 1 do
        begin
          Prop := Rec.PropertyValues.Items[i];
          Val := Prop.DisplayValue;
          Li := Items.Add;
          Li.Caption := Prop.DisplayName;
          DoStateEvent(LI);
          Li.SubItems.Add(Val);
          Li.ImageIndex := -1;
          SetW(Li.Caption, Val);
        end;
    end;

    if (reRecordIndex in FViewElements) then
    begin
       Val := IntToStr(Rec.RecordIndex);
       Li := Items.Add;
       Li.Caption := 'SYS_RecordIndex';
       DoStateEvent(LI);
       Li.SubItems.Add(Val);
       Li.ImageIndex := -1;
       SetW(Li.Caption, Val);
    end;

    if (reRecordID in FViewElements) then
    begin
       Val := IntToStr(Rec.RecordID);
       Li := Items.Add;
       Li.Caption := 'SYS_RecordID';
       DoStateEvent(LI);
       Li.SubItems.Add(Val);
       Li.ImageIndex := -1;
       SetW(Li.Caption, Val);
    end;

    if (reLinkId in FViewElements) then
    begin
       Indx := Rec.IndexOf('SYS_LinkID');
       if (Indx <> -1) then
       begin
         if Rec['SYS_LinkID'].IsEmpty then
          N := -1
         else
          N := Rec['SYS_LinkID'].AsInteger;
         Val := IntToStr(N);
         Li := Items.Add;
         Li.Caption := 'SYS_LinkId';
         DoStateEvent(LI);
         Li.SubItems.Add(Val);
         Li.ImageIndex := -1;
         SetW(Li.Caption, Val);
       end;
    end;

    if (reFileName in FViewElements) then
    begin
       Val := Rec.FileName;
       if ((Val = '') and ShowEmptyFields) or (Val <> '') then
       begin
         Li := Items.Add;
         Li.Caption := 'SYS_FileName';
         DoStateEvent(LI);
         Li.SubItems.Add(Val);
         Li.ImageIndex := -1;
         SetW(Li.Caption, Val);
       end;
    end;

    if reRegDate in FViewElements then
    begin
       Dt := Rec.RegDate;
       Val := DateTimeToStr(Dt, Translator.FFormatSettings);
       Li := Items.Add;
       Li.Caption := 'SYS_RegDate';
       DoStateEvent(LI);
       Li.SubItems.Add(Val);
       Li.ImageIndex := -1;
       SetW(Li.Caption, Val);
    end;

    if reChangeDate in FViewElements then
    begin
       Dt := Rec.ChangedDate;
       Val := DateTimeToStr(Dt, Translator.FFormatSettings);
       Li := Items.Add;
       Li.Caption := 'SYS_ChangedDate';
       DoStateEvent(LI);
       Li.SubItems.Add(Val);
       Li.ImageIndex := -1;
       SetW(Li.Caption, Val);
    end;

    if reState in FViewElements then
    begin
         Li := Items.Add;
         Li.Caption := 'SYS_State';
         DoStateEvent(LI);
         if Rec.DeletedMoved then
           Val := 'Moved'
         else if Rec.Deleted then
           Val := 'Deleted'
         else
           Val := 'Normal';
         Li.SubItems.Add(Val);
         Li.ImageIndex := -1;
         SetW(Li.Caption, Val);
    end;

    if (reUserId in FViewElements) then
    begin
       Val := Rec.UserID;
       if ((Val = '') and ShowEmptyFields) or (Val <> '') then
       begin
         Li := Items.Add;
         Li.Caption := 'UserID';
         DoStateEvent(LI);
         Li.SubItems.Add(Val);
         Li.ImageIndex := -1;
         SetW(Li.Caption, Val);
       end;
    end;

    if Assigned(StateImages) then
      W := StateImages.Width
    else
      W := 0;
    Columns[0].Width := FW + W + 15;
    Columns[1].Width := VW + W + 15;
  end;


{ TILCellEditor ------------------------------------------------------------ }

constructor TILCellEditor.Create(AViewer: TILDatasetViewer);
begin
  Ilib := AViewer.Table;
  FViewer := AViewer;
  inherited Create;
  if csDesigning in Ilib.ComponentState then
    Exit;
  FComboCtrl := TILComboBox.Create(AViewer);
  FComboCtrl.Visible := false;
  FComboCtrl.Parent := AViewer;
  FComboCtrl.Height := FViewer.ReportLineHeight;

  FMemoCtrl := TMemo.Create(AViewer);
  FMemoCtrl.Visible := False;
  FMemoCtrl.Parent := AViewer;
  FMemoCtrl.WantTabs := True;

  FEditCtrl := TILEdit.Create(AViewer);
  FEditCtrl.Visible := false;
  FEditCtrl.Parent := AViewer;
  FEditCtrl.Height := FViewer.ReportLineHeight;

  FResourceCtrl := TILResourcelist.Create(AViewer);
  FResourceCtrl.Visible := false;
  FResourceCtrl.Parent := AViewer;
  FResourceCtrl.SingleLine := True;
  FResourceCtrl.Height := FViewer.ReportLineHeight;

  FComboCtrl.OnKeyDown := ComboKeyDown;
  FComboCtrl.OnChange := ComboChange;
  FComboCtrl.OnCloseUp := ComboCloseUp;
  FMemoCtrl.OnKeyDown := MemoKeyDown;
  FMemoCtrl.OnChange := MemoChange;
  FEditCtrl.OnKeyDown := EditKeyDown;
  FEditCtrl.OnChange := EditChange;
  FResourceCtrl.OnKeyDown := ResourceKeyDown;
end;

destructor TILCellEditor.Destroy;
begin
  inherited;
end;

procedure TILCellEditor.ProcessTab(const Fwd: Boolean);
var
  RR: TILReportRow;
  AtRight: Boolean;
  F, L, R : Integer;
begin
  if Fwd then
  begin
    if (Cell.Index < Ilib.Columns.Count - 1) then
    begin
      RR := FViewer.Report.Items[Cell.FOwner.Index];
      FViewer.Report.SelectedCell := RR.Items[Cell.Index + 1];
      if FViewer.Report.SelectedCell.Position = -1 then
        FViewer.Report.SelectedCell.Position := 0;

    end
    else
    begin
      RR := FViewer.Report.Items[Cell.FOwner.Index];
      if RR.DatasetIndex < Ilib.Dataset.Count - 1 then
      begin
        Ilib.Dataset.Next;
        RR := FViewer.Report.Items
          [FViewer.Report.RecordIndexToRowIndex(Ilib.Dataset.Position)];
        FViewer.Report.SelectedCell := RR.Items[0];
        if FViewer.Report.SelectedCell.Position = -1 then
          FViewer.Report.SelectedCell.Position := 0;

      end;
    end;
  end
  else
  begin
    if (Cell.Index > 0) then
    begin
      RR := FViewer.Report.Items[Cell.FOwner.Index];
      FViewer.Report.SelectedCell := RR.Items[Cell.Index - 1];
      if FViewer.Report.SelectedCell.Position = -1 then
        FViewer.Report.SelectedCell.Position := 0;

    end
    else
    begin
      RR := FViewer.Report.Items[Cell.FOwner.Index];
      if RR.DatasetIndex > 0 then
      begin
        Ilib.Dataset.Previous;
        RR := FViewer.Report.Items
          [FViewer.Report.RecordIndexToRowIndex(Ilib.Dataset.Position)];
        FViewer.Report.SelectedCell := RR.Items[Ilib.Columns.Count - 1];
        if FViewer.Report.SelectedCell.Position = -1 then
          FViewer.Report.SelectedCell.Position := 0;
      end;
    end;
  end;

  if not FViewer.IsColumnVisible(Cell.Index, AtRight) then
  begin
    if AtRight then
      FViewer.LeftColumn := FViewer.LeftColumn + 1
    else
      FViewer.LeftColumn := FViewer.LeftColumn - 1;
  end;

  FViewer.GetVisibles(F, L);
  R := FViewer.Report.RecordIndexToRowIndex(Ilib.Dataset.Position);
  if (R < F) or (R > L) then
    FViewer.ScrollTo(Ilib.Dataset.Position);
  FViewer.Invalidate;
end;

procedure TILCellEditor.ProcessKey(Key: Word; Shift: TShiftState);
var
  Msg: TMsg;
  Done: Boolean;
begin
  Done := false;
  if (Key = vk_Tab) and (Shift = []) then
  begin
    ProcessTab(True);
    Done := True;
  end
  else if (Key = vk_Tab) and (Shift = [ssShift]) then
  begin
    ProcessTab(false);
    Done := True;
  end
  else if Key = vk_Return then
  begin
    if ActiveCtrl is TMemo then
    begin
      if Shift <> [ssCtrl] then
        Exit;
    end;
    UpdateValue;
    Cancel;
    Done := True;
    FViewer.SetFocus;
  end
  else if Key = vk_Escape then
  begin
    if not FLookupVisible then
    begin
      Cancel;
      if Cell.IsEmpty and (Cell.Data[Cell.FieldIndex].FValues.Count > 0) then
        FViewer.Report.DeleteSelected;
      Done := True;
      FViewer.SetFocus;
    end;
  end;
  if Done then
    PeekMessage(Msg, 0, WM_CHAR, WM_CHAR, PM_REMOVE);
end;

procedure TILCellEditor.MemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ProcessKey(Key, Shift);
end;

procedure TILCellEditor.ResourceKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ProcessKey(Key, Shift);
end;

procedure TILCellEditor.MemoChange(Sender: TObject);
begin

end;

procedure TILCellEditor.ComboKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ProcessKey(Key, Shift);
end;

procedure TILCellEditor.ComboChange(Sender: TObject);
begin
end;

procedure TILCellEditor.ComboCloseup(Sender: TObject);
begin
   UpdateValue;
   Cancel;
end;

procedure TILCellEditor.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ProcessKey(Key, Shift);
end;

procedure TILCellEditor.EditChange(Sender: TObject);
begin
  if FSemafor then
   Exit;
  if Cell.FieldDef.IsLookup then
  begin
    Ilib.ShowLookups(FEditCtrl, Cell.FieldDef.FieldName);
  end;
end;

procedure TILCellEditor.MoveTo(ACell: TILReportCell);
var
  Ctrl: TWinControl;
  sl: TStrings;
  R: TRect;
  IsEditing: Boolean;
  N, A : string;
  Pt : TPoint;
begin

  IsEditing := false;
  if Assigned(FCell) and FCell.FEditing then
  begin
    UpdateValue;
    Cancel;
    IsEditing := True;
    FViewer.Invalidate;
  end;
  FCell := ACell;
  Ctrl := nil;
  sl := nil;

  FComboCtrl.Parent := Screen.ActiveForm;
  FEditCtrl.Parent := Screen.ActiveForm;
  FMemoCtrl.Parent := Screen.ActiveForm;
  FResourceCtrl.Parent := Screen.ActiveForm;

  if Assigned(Cell.FieldDef.DataControl) then
    Ctrl := Cell.FieldDef.DataControl
  else if Assigned(Cell.FieldDef.SearchControl) then
    Ctrl := Cell.FieldDef.SearchControl;

  if FCell.FieldDef.LookupType = looPickValues then
  begin
   if FCell.FieldDef.PickValues.Count > 0 then
    sl := FCell.FieldDef.PickValues
  end
  else if FCell.FieldDef.LookupType in [looTableKey, looTableRef] then
  begin
    sl := Ilib.GetLookups(FCell.FieldDef.FieldName, '*', duData);
  end
  else if (Ctrl is TCombobox) and (sl = nil) then
  begin
    sl := TCombobox(Ctrl).Items;
  end
  else if (Ctrl is TCheckListBox) and (sl = nil) then
  begin
    sl := TCheckListBox(Ctrl).Items;
  end
  else if (Ctrl is TRadioGroup) and (sl = nil) then
  begin
    sl := TRadioGroup(Ctrl).Items;
  end
  else if (Ctrl is TILResourceList) and (sl = nil) then
  begin
  end
  else if (Ctrl is TListBox) and (sl = nil) then
  begin
    sl := TListBox(Ctrl).Items;
  end;

  if (Cell.FieldDef.DataType = dtBoolean) and (sl = nil) then
  begin
    FComboCtrl.Clear;
    FComboCtrl.Style := csDropDown;
    FComboCtrl.Items.Add(TMan.ReportFalse);
    FComboCtrl.Items.Add(Tman.ReportTrue);
    if Cell.Value = '0' then
      FComboCtrl.ItemIndex := 0
    else if Cell.Value = '1' then
      FComboCtrl.ItemIndex := 1
    else
      FComboCtrl.ItemIndex := -1;
    FActiveCtrl := FComboCtrl;
  end
  else if (Cell.FieldDef.DataType = dtText) and (sl = nil) then
  begin
    FMemoCtrl.Lines.Assign(Cell.Data[Cell.FieldIndex].FValues);
    FActiveCtrl := FMemoCtrl;
  end
  else if (Cell.FieldDef.DataType in [dtResource]) and (sl = nil) then
  begin
    if Cell.Data[Cell.FieldIndex].FValues.Count > 0 then
    begin
    GetResourceElements(Cell.Data[Cell.FieldIndex].FValues[Cell.Position], N, A);
    FResourceCtrl.Items[0] := N;
    FResourceCtrl.Addresses[0] := A;
    end;
    FActiveCtrl := FResourceCtrl;
  end
  else if sl = nil then
  begin
    if Cell.FieldDef.IsLookup then
      Ilib.SetLookup(FEditCtrl, nil)
    else
      FEditCtrl.OnKeyDown := EditKeyDown;;
    FActiveCtrl := FEditCtrl;
  end;
  if sl <> nil then
  begin
    FComboCtrl.Style := csDropDownList;
    FComboCtrl.Items.Assign(sl);
    FComboCtrl.ItemIndex := sl.IndexOf(Cell.Value);
    FActiveCtrl := FComboCtrl;
    if FCell.FieldDef.LookupType in [looTableKey, looTableRef] then
     FreeAndNil(sl);
  end;
  R := Cell.ItemRect;

  Pt := FViewer.ClientToScreen(POint(R.Left, R.Top));
  Pt := Screen.ActiveForm.ScreenToClient(Pt);

  FActiveCtrl.Left := Pt.X;
  FActiveCtrl.Top := Pt.Y;
  FActiveCtrl.Width := R.Width;
  if (sl = nil) and (FActiveCtrl = FEditCtrl) then
  begin
    FSemafor := True; { Ignore the on change event }
    if (Cell.FieldDef.DataType = dtNumber) and (Cell.FieldDef.Decimals > 0)
    and not Cell.Field.IsEmpty then
    begin
      FEditCtrl.Text := Cell.Field.FValues[Cell.Position];
    end
    else
     FEditCtrl.Text := Cell.Value;
    FSemafor := false;
  end;
  if IsEditing then
    Edit;
end;

procedure TILCellEditor.Edit;
begin
  if Cell.CanEdit <> 0 then
    Exit;
  Cell.FEditing := True;
  if (FActiveCtrl is TILresourceList) then
  with FActiveCtrl as TILResourceList do
  begin
    if Cell.Data.IsNew then
      Empty := True;
  end;
  FActiveCtrl.Visible := True;
  FActiveCtrl.SetFocus;
  FActiveCtrl.BringToFront;
  if Assigned(FViewer.FOnEditCell) then
    FViewer.FOnEditCell(FViewer, Cell);
end;

procedure TILCellEditor.UpdateValue;
var
  Vl: string;
  Del : Boolean;
  MH : TILMessageHandler;
  Ind : integer;
begin
  if FActiveCtrl is TCombobox then
    Vl := FComboCtrl.Items[FComboCtrl.ItemIndex]
  else if (FActiveCtrl is TEdit) then
    Vl := FEditCtrl.Text
  else if FActiveCtrl is TILResourceList then
    Vl := FResourceCtrl.ResourceStr[0]
  else
    Vl := FMemoCtrl.Text;

  if Cell.FieldDef.DataType = dtBoolean then
  begin
    if Vl = TMan.ReportTrue then
      Vl := '1'
    else
      Vl := '0';
  end;
  Del := False;
  if ((Trim(Vl) = '') or TMan.IsValueNull(Vl)) and (Cell = FViewer.Report.SelectedCell) then
  begin
    Cancel;
    FViewer.Report.DeleteSelected;
    Del := True;
  end;

  if Cell.Value <> Vl then
  begin
    if not Del then
      Cell.Value := Vl;
  end;
  Ilib.WriteControls(Cell.Owner.Data, duData);

  if Del then
  begin
    Ind := -1;
    MH := Tman.GetMessageHandler(FViewer.Table, Ind);
    while MH <> nil do
    begin
       MH.AddCount(dboUpdate, -1);
       MH := TMan.GetMessageHandler(FViewer.Table, Ind);
    end;
  end;
end;

procedure TILCellEditor.Cancel;
begin
  if not Cell.FEditing then
    Exit;
  ActiveCtrl.Visible := false;
  Cell.FEditing := false;
  MoveTo(Cell); { Reset ctrl values }
  FViewer.Invalidate;
  FViewer.SetFocus;
end;

{ TILReportCell ------------------------------------------------------------ }

constructor TILReportCell.Create(AnOwner: TILReportRow; AFieldDef: TILFieldDef);
begin
  FOwner := AnOwner;
  Ilib := FOwner.Ilib;
  FFieldDef := AFieldDef;
  inherited Create;
end;

destructor TILReportCell.Destroy;
begin
  inherited;
end;

function TILReportCell.IsEmpty : Boolean;
var
  V : string;
begin
    V := Value;
    Result := (V='') or TMan.IsValueNull(V);
end;

function TILReportCell.CanEdit : integer;
begin
  Result := -1;
  if not (roEditable in FViewer.Reportoptions) then
    Exit;

  if (TMan.NodeOf(Ilib) <> nil) then
  begin
    if not (Ilib = TMan.ActiveTable) then
      Exit;
  end;
  if TMan.ActiveTable <> Ilib then
    Exit;
  if Fviewer.Report.SelectedCell <> Self then
    Exit;

  if FieldDef.DataType = dtLongText then
    Exit;

  if FieldDef.IsReadOnly then
  begin
    Comment1 := FieldDef.Caption;
    Result := E_ReadOnly;
    Exit;
  end;
  if Editing then
    Exit;
  if Data.IsDeleted then
    Exit;
  {if Ilib.MultiSelected then
  begin
    if Assigned(TMan.FOnAlert) then
      TMan.FOnAlert(Tman, alMulSelNoEdit);
    Exit;
  end;
  Removed this contraint 13.3
  }
  if Tman.UseCommandModes and (Tman.CommandMode <> cmUpdate) then
    Exit;
  Result := 0;
end;

procedure TILReportCell.CancelEdit;
begin
  if Editing then
  begin
    FViewer.CellEditor.Cancel;
  end;
end;

procedure TILReportCell.Edit;
var
  Res : integer;
begin
   Res := CanEdit;
   if Res <> 0 then
     HandleILException(Res);
   FViewer.CellEditor.Edit;
end;

function TILReportCell.GetIndex: integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to Ilib.Columns.Count - 1 do
  begin
    if Ilib.Columns[i].ReportField then
    begin
      Inc(Result);
    end;
    if Ilib.Columns[i] = FieldDef then
    begin
      Break;
    end;
  end;
end;

function TILReportCell.GetItemRect: TRect;
var
  X, Y: integer;
  RowNum: integer;
  i: integer;
  Found: Boolean;
begin
  X := FViewer.GetRptLeftMargin;
  Y := Fviewer.ReportLineHeight + 2 + FViewer.GetTopMargin;
  RowNum := FOwner.Index - Fviewer.FirstItemIndex;
  Y := Y + RowNum * Fviewer.ReportLineHeight;
  Found := false;
  for i := Ilib.FViewer.FLeftColumn - 1 downto 0 do
  begin
    X := X - (Owner.Items[i].FFieldDef.ColumnWidth + Ilib.FViewer.FColumnSpacing);
    if i = index then
    begin
      Found := True;
      Break;
    end;
  end;

  if not Found then
  begin
    X := FViewer.GetRptLeftMargin;
    for i := Ilib.FViewer.FLeftColumn to FOwner.Count - 1 do
    begin
      if i = index then
      begin
        Break;
      end;
      X := X + Owner.Items[i].FFieldDef.ColumnWidth + Ilib.FViewer.FColumnSpacing;
    end;
  end;
  Result.Top := Y;
  Result.Bottom := Y + FViewer.ReportLineHeight;
  Result.Left := X;
  Result.Right := X + FFieldDef.ColumnWidth + Ilib.Fviewer.FColumnSpacing;
end;

function TILReportCell.GetImageRect: TRect;
begin
  Result := Rect(0, 0, 0, 0);
  if not(FieldDef.LookupTable is TILImageTable) then
    Exit;
  if IsEmpty then
    Exit;
  if not(roShowFieldThumbnails in FViewer.ReportOptions) then
    Exit;
  Result := ItemRect;
  Result.Right := Result.Left + Fviewer.GetReportLineHeight - 2;
end;

function TILReportCell.GetImage: Vcl.Graphics.TBitmap;
var
  LTID: integer;
  R: TILDataRecord;
begin
  Result := nil;
  if FFieldDef.LookupTable = nil then
    Exit;
  if not(roShowFieldThumbnails in FViewer.ReportOptions) then
    Exit;
  LTID := StrToInt(Ilib.GetLookupTableID(FFieldDef.LookupTable, FFieldDef.LookupField, Value));
  R := FFieldDef.LookupTable.Records[LTID];
  try
    Result := Vcl.Graphics.TBitmap.Create;
    Result.Assign(R.Image);
  finally
    R.Free;
  end;
end;

function TILReportCell.GetImageFileName: string;
var
  LTID: integer;
  R: TILDataRecord;
begin
  Result := '';
  if FFieldDef.LookupTable = nil then
    Exit;
  if not(roShowFieldThumbnails in FViewer.ReportOptions) then
    Exit;
  LTID := StrToInt(Ilib.GetLookupTableID(FFieldDef.LookupTable, FFieldDef.LookupField, Value));
  R := FFieldDef.LookupTable.RecordsX[LTID];
  try
    Result := R.FileName;
  finally
    R.Free;
  end;
end;

procedure TILReportCell.SetVisible(const Value : Boolean);
begin

end;

function TILReportCell.GetValue: string;
var
  Rec: TILDataRecord;
  ValNum: integer;
  Indx: integer;
  NF : Extended;
  NI : integer;
begin
  Rec := Owner.FData;
  ValNum := FOwner.FExpansionIndex;
  Indx := FieldIndex;

  if Rec[Indx].SuppressOutput then
  begin
    Result := '';
    Exit;
  end;
  if Rec[Indx].Count - 1 < ValNum then
  begin
    Result := '';
    Exit;
  end;
  if (FieldDef.LookupType = looTableKey)
  and (FieldDef.DataType = dtNumber) then
    Result := Rec[FieldDef.FieldName].DisplayValue
  else if (FieldDef.DataType = dtText) and FEditing then
    Result := Rec[Indx].FValues.Text
  else if FieldDef.DataType = dtLongText then
  begin
    Result := Rec[Indx].GetSampleLongText(50);
  end
  else if (FieldDef.Decimals > 0)
  and (FieldDef.DataType = dtNumber) then
  begin
      NF := StrToFloat(Rec[Indx][ValNum], Translator.FFormatSettings);
      Result := Translator.FormatNum(NF, FieldDef.Decimals);
  end
  else
  begin
    Result := Rec[Indx][ValNum];
    if (FieldDef.DataType = dtNumber) and (FieldDef.Decimals = 0)  then
    begin
      NI := StrToInt(Result);
      Result := Translator.FormatNum(NI);
    end;
  end;
end;

function TILReportCell.GetFieldIndex: integer;
begin
  Result := Owner.FData.IndexOf(FieldDef.FieldName);
end;

function TILReportCell.GetData: TILDataRecord;
begin
  Result := Owner.FData;
end;

function TILReportCell.GetPosition : integer;
begin
  Result := Owner.ExpansionIndex;
  if Result > Data[FieldIndex].Count -1 then
   Result := -1;
end;

function TILReportCell.GetField : TILField;
begin
    Result := Data[FieldIndex];
end;

function TILReportCell.GetTable : TILTable;
begin
    Result := Data.Table;
end;


procedure TILReportCell.SetPosition(const Value : integer);
var
  Indx : integer;
begin
  if Value = Position then
    Exit;
  if Value > Data[FieldIndex].Count-1 then
    Exit;
  if Value < 0 then
    Exit;
  Indx := Owner.OwnerReport.RecordIndexToRowIndex(Data.Index);
  Owner.OwnerReport.SelectedCell := Owner.OwnerReport.Cells[Indx + Value, Index];
end;

procedure TILReportCell.SetSelected(const Value : Boolean);
var
  i: integer;
  C: TILReportCell;
  R: TILReportRow;
  Ind: integer;
begin
  FSelected := Value;
  if (FieldDef.DataType = dtText) and Owner.Expanded then
  begin
    Ind := Owner.Index;
    R := Owner.FOwnerReport.Items[Owner.Index];
    while R.ExpansionIndex > 0 do
    begin
      Dec(Ind);
      R := Owner.FOwnerReport.Items[Ind];
    end;
    R := Owner.FOwnerReport.Items[Ind];
    for i := 0 to Owner.ExpansionCount
      do { not -1 because counting starts after the base row }
    begin
      C := R.Items[Index];
      C.FSelected := Value;
      R := Owner.FOwnerReport.Items[Ind];
      Inc(Ind);
    end;
  end;
end;

procedure TILReportCell.SetValue(Val: string);
var
  Rec: TILDataRecord;
  ValNum: integer;
  N: Extended;
  DT: TDateTime;
  Emp: Boolean;
  I : integer;
  KeyID : string;
  Err : integer;
  MH : TILMessageHandler;
  Ind : integer;

  procedure HandleChanged;
  begin
    Table.Dataset.FChngTracker.Add(Rec);
    if Assigned(FViewer.FOnSetValue) then
     FViewer.FOnSetValue(FViewer, Self);
  end;

begin
  Rec := Owner.FData;
  Emp := (Trim(Val) = '') or TMan.IsValueNull(Val);
  KeyId := '';
  Comment1 := Rec[FieldIndex].FieldDef.Caption + ': ' + Val;
  if (Rec[FieldIndex].FieldDef.DataType = dtNumber) and not Emp then
  begin
    if Rec[FieldIndex].FieldDef.LookupType = looTableKey then
    begin
      KeyID := Rec[FieldIndex].DecodeTableKey(Val);
      if KeyID = '' then
      begin
        CancelEdit;
        HandleILException(E_UnresolvedTableKey);
      end;
      Val := KeyID;
    end;
    if not TryStrToFloat(Val, N, Translator.FFormatSettings) then
      HandleILException(E_InvalidNumber)
  end
  else if (Rec[FieldIndex].FieldDef.DataType = dtDate) and not Emp then
  begin
    if not TryStrToDateTime(Val, DT, Translator.FFormatSettings) then
      HandleILException(E_InvalidDate, Val)
  end
  else if (Rec[FieldIndex].FieldDef.DataType = dtBoolean) and not Emp then
  begin
    if not(Val = Tman.ReportTrue) and not(Val = Tman.ReportFalse) and
      not(Val = '0') and not(Val = '1') then
      HandleILException(E_FieldDataTypeMismatch)
  end;

  if Rec[FieldIndex].FValues = Rec[FieldIndex].FOldValues then
  begin
    Rec[FieldIndex].FOldValues := TStringList.Create;
    Rec[FieldIndex].FOldValues.Assign(Rec[FieldIndex].FValues);
  end;

  if (Rec[FieldIndex].FieldDef.DataType = dtText) then
  begin
    if not Emp then
      Rec[FieldIndex].FValues.Text := Val;
    if Rec[FieldIndex].FieldDef.ImageText then
      Rec[FieldIndex].UpdateImageText;
  end;

  if Rec[FieldIndex].FieldDef.DataType = dtText then
  begin
    HandleChanged;
    if not Rec.IsNew then
    begin
      Ind := -1;
      MH := TMan.GetMessageHandler(ILib, Ind);
      while MH <> nil do
      begin
        MH.AddCount(dboUpdate, -1);
        MH := TMan.GetMessageHandler(ILib, Ind);
      end;
    end;

    if Tman.AutoSave and (Table.DBWork.CanSave(Err) = 0) then
      Table.DBWork.Save;
     Exit;
  end;

  ValNum := FOwner.FExpansionIndex;

  if (Rec[FieldIndex].FieldDef.LookupType = looTableRef) then
  begin
    if Ilib.GetLookupTableID(Rec[FieldIndex].FieldDef.LookupTable,
      Rec[FieldIndex].FieldDef.LookupField, Val) = ''
    then
    begin
      Comment1 := Val;
      HandleILException(E_LookupNotRegistered);
    end;
  end;

  if Rec[FieldIndex].Count - 1 < ValNum then
  begin
    if not Emp then
      Rec[FieldIndex].FValues.Add(Val);
  end
  else
  begin
    if Emp then
    begin
      if Editing then
         FViewer.CellEditor.Cancel
      else
         Rec[FieldIndex].FValues[Valnum] := '';
    end
    else
      Rec[FieldIndex].FValues[ValNum] := Val;
  end;
  if Rec[FieldIndex].FieldDef.ImageText then
    Rec[FieldIndex].UpdateImageText;
  if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
  begin
    for I := 0 to Rec.Count-1 do
    begin
      if (Rec[i].FieldDef.FieldType = ftCalculated) and not Rec[i].FieldDef.FUserCalc then
      try
        Ilib.DataHandler.FOnCalcField(Self, Rec, Rec[i]);
      except

      end;
    end;
  end;
  if Assigned(Ilib.DataHandler) then
    for I := 0 to Ilib.DataHandler.DataFields.Count-1 do
    begin
       if Ilib.DataHandler.DataFields.Items[i].ImageTag then
       begin
         if SameText(Rec[FieldIndex].FieldDef.FieldName,
          Ilib.DataHandler.DataFields.Items[i].FieldName) then
          begin
            TILDataTable(Ilib).GetFromImageTable(Rec[FieldIndex].DisplayValue, Rec);
          end;
         Break;
       end;
    end;
  HandleChanged;
  Ind := -1;
  Ilib.DBWork.WriteControls(Ilib.FocusedRecord);


  MH := TMan.GetMessageHandler(ILib, Ind);
  while MH <> nil do
  begin
      MH.AddCount(dboUpdate, -1);
      MH := TMan.GetMessageHandler(ILib, Ind);
  end;

  if Tman.AutoSave and (Table.DBWork.CanSave(Err) = 0) then
      Table.DBWork.Save;
  if Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsDataUpdated);
end;

{ TReportRow--------------------------------------------------------------- }

function TILReportRow.GetItems(Index: integer): TILReportCell;
begin
  Result := TILReportCell(FColumnList[Index]);
end;

function TILReportRow.GetCount: integer;
begin
  Result := FColumnList.Count;
end;

function TILReportRow.GetDatasetIndex: integer;
begin
  Result := Data.Index;
end;

function TILReportRow.IsDeleted : Boolean;
begin
  Result := Data.FIsDeleted;
end;

function TILReportRow.GetCheckRect: TRect;
var
  R: TRect;
begin
  R := Rect(0, 0, 0, 0);
  if not Ilib.FViewer.ShowCheckBoxes then
    Exit;
  R := ItemRect;
  R.Top := R.Top + (R.Height - c_CheckBoxWidth) div 2;
  R.Left := R.Left + 3;
  R.Right := R.Left + c_CheckBoxWidth;
  R.Bottom := R.Top + c_CheckBoxWidth;
  Result := R;
end;

function TILReportRow.GetExpandBtnRect: TRect;
var
  R: TRect;
begin
  R := Rect(0, 0, 0, 0);
  if not(roShowExpandButtons in Ilib.FViewer.ReportOptions) and
    (Ilib.FViewer.UpdateMarkers = upmNone) then
    Exit;
  R := ItemRect;
  R.Left := 0;
  R.Right := Ilib.FViewer.GetRptLeftMargin;
  Result := R;
end;

function TILReportRow.GetImageRect: TRect;
var
  R: TRect;
begin
  R := Rect(0, 0, 0, 0);
  if not Ilib.FViewer.CanReportDisplayImages then
    Exit;
  R := FOwnerReport.ExpansionRect[Index];
  R.Left := 0;
  R.Right := Ilib.FViewer.GetRptImageSize;
  R.Bottom := R.Top + Ilib.FViewer.GetRptImageSize;
  Result := R;
end;

function TILReportRow.GetExpanded: Boolean;
begin
  Result := (FExpansionCount > 0);
end;

function TILReportRow.GetItemRect: TRect;
var
  X, Y: integer;
  RowNum: integer;
  i: integer;
  Lft: integer;
begin
  X := Ilib.FViewer.GetRptLeftMargin;
  Y := Ilib.FViewer.ReportLineHeight + 2 + Ilib.FViewer.GetTopMargin;
  RowNum := Index - Ilib.FViewer.FirstItemIndex;
  Y := Y + RowNum * Ilib.FViewer.ReportLineHeight;

  for i := Ilib.FViewer.FLeftColumn - 1 downto 0 do
  begin
    X := X - (Items[i].FFieldDef.ColumnWidth + Ilib.FViewer.FColumnSpacing);
  end;
  Lft := X;
  X := Ilib.FViewer.GetRptLeftMargin;
  for i := Ilib.FViewer.FLeftColumn to Count - 1 do
  begin
    X := X + Items[i].FFieldDef.ColumnWidth + Ilib.FViewer.FColumnSpacing;
  end;

  Result.Top := Y;
  Result.Bottom := Y + Ilib.FViewer.ReportLineHeight;
  Result.Left := Lft;
  Result.Right := X;
end;

constructor TILReportRow.Create(AnOwner: TILReport);
var
  i: integer;
  C: TILReportCell;
begin
  FOwnerReport := AnOwner;
  Ilib := FOwnerReport.Ilib;
  FColumnList := TList.Create;
  for i := 0 to Ilib.Columns.Count - 1 do
  begin
    if Ilib.FColumns[i].ReportField then
    begin
      C := TILReportCell.Create(Self, Ilib.FColumns[i]);
      C.FViewer := Ilib.FViewer;
      FColumnList.Add(C);
    end;
  end;
  inherited Create;
end;

function TILReportRow.CanExpand: Boolean;
var
  i: integer;
begin
  Result := false;
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  for i := 0 to Count - 1 do
  begin
    if Data[Items[i].FieldIndex].Count > 1 then
    begin
      Result := True;
      Break;
    end;
  end;

end;

procedure TILReportRow.Clear;
var
  i: integer;
  C: TILReportCell;
begin
  for i := 0 to Count - 1 do
  begin
    C := TILReportCell(FColumnList[i]);
    C.Free;
  end;
  FColumnList.Clear;
end;

destructor TILReportRow.Destroy;
begin
  Clear;
  FColumnList.Free;
  inherited Destroy;
end;

{ TILReport }

constructor TILReport.Create(TableOwner: TILTable);
begin
  FColumnsList := TList.Create;
  Ilib := TableOwner;
  if Ilib.Dataset <> nil then
    Populate;
  inherited Create;
end;

destructor TILReport.Destroy;
begin
  Clear;
  FColumnsList.Free;
  inherited Destroy;
end;

procedure TILReport.Populate;
var
  i: integer;
  Rec: TILDataRecord;
  C: TILReportRow;
begin
  if not Assigned(Ilib.Columns) then
    Exit;
  if Ilib.FViewer = nil then
    Exit;
  SelectedCell := nil;
  if roEditable in Ilib.FViewer.ReportOptions then
    Ilib.FViewer.FCellEditor.FCell := nil;
  Clear;
  for i := 0 to Ilib.Dataset.Count - 1 do
  begin
    Rec := Ilib.Dataset.Items[i];
    C := TILReportRow.Create(Self);
    C.FData := Rec;
    C.FIndex := i;
    FColumnsList.Add(C);
  end;
  if Ilib.FViewer.CanReportDisplayImages then
    for i := 0 to Ilib.Dataset.Count - 1 do
    begin
      DoExpand(i, Ord(Ilib.FViewer.FReportImageSize) + 2)
    end
  else if (roAutoExpand in Ilib.FViewer.ReportOptions) and (Ilib.Dataset.Position <> -1)
  then
    Expand(Ilib.Dataset.Position);
  Ilib.FViewer.FCellUnderMouse := nil;
end;

function TILReport.InfoFromPos(const X, Y: integer): TReportInfo;
var
  Pt: TPoint;
  Indx: integer;
  R: TRect;
begin
  Pt := Point(X, Y);
  Result.DataRecord := nil;
  Result.Cell := nil;
  Result.Row := nil;
  Result.Graphics := rgNone;
  if Ilib.FViewer.ViewMode <> vmReport then
    Exit;
  Indx := RowAtXY(X, Y);
  if Indx = -1 then
    Exit;
  Result.Row := Items[Indx];
  Result.DataRecord := Items[Indx].Data;
  R := Items[Indx].ImageRect;
  if PtInRect(R, Pt) then
  begin
    Result.Graphics := rgRecordImage;
    Exit;
  end;
  Result.Cell := CellAtXY(X, Y);
  if Result.Cell <> nil then
  begin
    R := Result.Cell.ImageRect;
    if PtInRect(R, Pt) then
      Result.Graphics := rgFieldImage;
  end
  else
  begin
    if Ilib.FViewer.ShowCheckBoxes then
    begin
      R := Ilib.FViewer.GetItemRect(rtCheckRect, Result.Row.Index);
      if not R.IsEmpty then
      begin
        Result.Graphics := rgCheckBox;
        Exit;
      end;
    end;
    if Ilib.FViewer.UpdateMarkers <> upmNone then
    begin
      R := Ilib.FViewer.GetItemRect(rtExpandBtnRect, Result.Row.Index);
      if not R.IsEmpty then
      begin
        Result.Graphics := rgUpdateMarker;
      end;
    end;
  end;
end;

function TILReport.Expand(const Index: integer): integer;
begin
  Result := DoExpand(Index, 0);
end;

function TILReport.DoExpand(Index: integer; const MinCount: integer): integer;
{ Returns the item number following the expansion }
var
  C, C2: TILReportRow;
  i: integer;
  N: integer;
  L: integer;
  NewIndx: integer;
  Cl: TILReportCell;
  SelIndx: integer;
begin
  Result := -1;
  SelIndx := -1;
  if Index = -1 then
    Exit;
  Index := RecordIndexToRowIndex(Index);
  C := Items[Index];
  if C.Expanded then
    Exit;
  { Check if a text column is selected : }
  for i := 0 to C.Count - 1 do
  begin
    Cl := C.Items[i];
    if Cl.Selected then
    begin
      if Cl.FieldDef.DataType = dtText then
      begin
        SelIndx := i;
      end;
      Break;
    end;
  end;

  L := 0;
  for i := 0 to C.Count - 1 do
  begin
    N := C.Data[C.Items[i].FieldIndex].Count;
    if N > L then
      L := N
  end;
  if L < MinCount then
    L := MinCount;
  NewIndx := Index;
  C.FExpansionCount := L - 1;
  for i := 1 to L - 1 do
  begin
    C2 := TILReportRow.Create(Self);
    C2.FData := C.Data;
    C2.FExpansionIndex := i;
    C2.FExpansionCount := L - 1;
    Inc(NewIndx);
    C2.FIndex := NewIndx; { Insert after the first item }
    FColumnsList.Insert(NewIndx, C2);
    if SelIndx <> -1 then
    begin
      C2.Items[SelIndx].FSelected := True;
    end;
  end;

  { Increment the rest }
  for i := NewIndx + 1 to Count - 1 do
  begin
    C := Items[i];
    C.FIndex := i;
  end;
  Result := NewIndx + 1;
  Ilib.FViewer.UpdateScrollRange;
  Ilib.FViewer.UpdateScrollPos;
  Ilib.FViewer.Invalidate;

end;

function TILReport.Expanded(Index: integer): Boolean;
begin
  Result := false;
  if Index = -1 then
    Exit;
  Index := RecordIndexToRowIndex(Index);
  Result := Items[Index].Expanded;
end;

function TILReport.Collapse(Index: integer): integer;
{ Returns the item number following the collapsed item }
var
  C: TILReportRow;
  i: integer;
  L: integer;
  NewIndx: integer;
begin
  Result := -1;
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  if Index = -1 then
    Exit;
  Index := RecordIndexToRowIndex(Index);
  C := Items[Index];
  Result := -1;
  if not C.Expanded then
    Exit;

  while C.ExpansionIndex <> 0 do
  begin
    Dec(Index);
    C := Items[Index];
  end;

  if SelectedCell <> nil then
  begin
    if SelectedCell.FOwner.ExpansionIndex > 0 then
      SelectedCell := C.Items[SelectedCell.Index];
  end;

  L := C.ExpansionCount;
  C.FExpansionCount := 0;
  NewIndx := Index + 1;
  for i := 1 to L do
  begin
    C := Items[NewIndx];
    C.Free;
    FColumnsList.Delete(NewIndx);
  end;
  { Decrement the rest }
  for i := NewIndx to Count - 1 do
  begin
    C := Items[i];
    C.FIndex := C.FIndex - L;
  end;
  Ilib.FViewer.FCellUnderMouse := nil;
  Result := NewIndx;
  Ilib.FViewer.UpdateScrollRange;
  Ilib.FViewer.UpdateScrollPos;
  Ilib.Fviewer.Invalidate;
end;

procedure TILReport.ExpandAll;
var
  i: integer;
  RIndex: integer;
  C: TILReportRow;
begin
  i := 0;
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  repeat
    C := Items[i];
    if C.Expanded then
    begin
      Inc(i, C.ExpansionCount + 1);
      Continue;
    end;
    RIndex := Items[i].Data.Index;
    i := Expand(RIndex);
  until i >= Count;
end;

procedure TILReport.ExpandSelection;
var
  i: integer;
begin
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  for i := 0 to Ilib.SelCount - 1 do
  begin
    Expand(Ilib.Selection[i].Index);
  end;
end;

procedure TILReport.CollapseSelection;
var
  i: integer;
begin
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  for i := 0 to Ilib.SelCount - 1 do
  begin
    Collapse(Ilib.Selection[i].Index);
  end;
end;

procedure TILReport.CollapseAll;
var
  i: integer;
  C: TILReportRow;
  RIndex: integer;
begin
  if Ilib.FViewer.CanReportDisplayImages then
    Exit;
  i := 0;
  repeat
    C := Items[i];
    if not C.Expanded then
    begin
      Inc(i);
      Continue;
    end;
    RIndex := Items[i].Data.Index;
    i := Collapse(RIndex);
  until i >= Count;
end;

procedure TILReport.Redraw;
begin
  Populate;
  Ilib.FViewer.Invalidate;
end;

procedure TILReport.Clear;
var
  i: integer;
begin
  if not(csDestroying in Ilib.ComponentState) then
  begin
    if SelectedCell <> nil then
    begin
      SelectedCell := nil;
    end;
  end;
  for i := 0 to Count - 1 do
    TILReportRow(FColumnsList[i]).Free;
  FColumnsList.Clear;
  Ilib.FViewer.FCellEditor.Cell := nil;
end;

procedure TILReport.AddSelected;
var
  CIndex: integer;
  RIndex: integer;
  Rec: TILDataRecord;
  Res : integer;
begin
  if SelectedCell = nil then
    HandleILException(-1);
  Res := CanAddSelected;
  if Res <> 0 then
  begin
    Comment1 := SelectedCell.FieldDef.Caption;
    HandleILException(Res);
    Exit;
  end;
  Rec := SelectedCell.Owner.Data;
  CIndex := Rec.IndexOf(SelectedCell.FieldDef.FieldName);
  if Rec[CIndex].FValues.Count = 0 then
  begin
    Ilib.FViewer.FCellEditor.Edit;
    Exit;
  end;
  if SelectedCell.FieldDef.DataType in [dtBoolean, dtNumber] then
    Rec[CIndex].FValues.Add('0')
  else if SelectedCell.FieldDef.DataType in [dtString, dtText, dtResource] then
    Rec[CIndex].FValues.Add('')
  else if SelectedCell.FieldDef.DataType = dtDate then
    Rec[CIndex].FValues.Add(DateTimeToStr(Now, Translator.FFormatSettings))
  else
    Exit; { Never happens }
  if Rec[CIndex].FValues.Count > 1 then
  begin
    if Expanded(Rec.Index) then
      Collapse(Rec.Index);
    Expand(Rec.Index);
    RIndex := RecordIndexToRowIndex(Rec.Index);
    SelectedCell := Cells[RIndex + Rec[CIndex].Count-1 , SelectedCell.Index];
    Ilib.FViewer.FCellEditor.Edit;
  end;
end;

procedure TILReport.DeleteSelected;
var
  CIndex, VIndex: integer;
  Rec: TILDataRecord;
  RIndex: integer;
  Last : Boolean;
  Expnd : Boolean;
  Res : integer;
begin
  if SelectedCell = nil then
    HandleILException(-1);
  Res := CanDeleteSelected;
  if Res <> 0 then
  begin
    if Res <> -1 then
    begin
      Comment1 := SelectedCell.FieldDef.Caption;
      HandleILException(Res);
    end;
    Exit;
  end;
  if SelectedCell.Editing then
    Ilib.FViewer.FCellEditor.Cancel;

  Rec := SelectedCell.Owner.Data;
  CIndex := Rec.IndexOf(SelectedCell.FieldDef.FieldName);
  RIndex := SelectedCell.FOwner.Index;
  VIndex := SelectedCell.Owner.ExpansionIndex;
  if Rec[CIndex].FValues = Rec[CIndex].FOldValues then
  begin
    Rec[CIndex].FOldValues := TStringList.Create;
    Rec[CIndex].FOldValues.Assign(Rec[CIndex].FValues);
  end;
  Last := (VIndex = Rec[CIndex].FValues.Count-1);
  Expnd := SelectedCell.Owner.Expanded;
  if Rec[CIndex].FValues.Count > 0 then
   Rec[CIndex].FValues.Delete(VIndex);
  if Rec[CIndex].FValues.Count < 2 then
  begin
    if Expnd then
      Collapse(Rec.Index);
    if (Rec[CIndex].FValues.Count = 0) then
     SelectedCell := Cells[RIndex, SelectedCell.Index]
    else if Last then
      SelectedCell := Cells[RIndex - 1, SelectedCell.Index];
    if SelectedCell.Owner.CanExpand then
      Expand(SelectedCell.Data.Index);
  end
  else
  begin
    Collapse(Rec.Index);
    Expand(Rec.Index);
    SelectedCell := Cells[RIndex - 1, SelectedCell.Index];
  end;
  Ilib.Dataset.FChngTracker.Add(Rec);
  Ilib.FViewer.Invalidate;
end;

function TILReport.CanAddSelected : integer;
var
  Rec : TILDataRecord;
  CIndex : integer;
begin
  Result := -1;
  if not (roEditable in Ilib.FViewer.ReportOptions) then
    Exit;
  if TMan = nil then
    Exit;
  if (TMan.NodeOf(Ilib) <> nil) then
  begin
    if not (Ilib = TMan.ActiveTable) then
      Exit;
  end;
  if SelectedCell = nil then
    Exit;
  Rec := SelectedCell.Owner.Data;
  if Rec.IsDeleted then
    Exit;
  CIndex := Rec.IndexOf(SelectedCell.FieldDef.FieldName);
  if (SelectedCell.FieldDef.DataType = dtText) and not (SelectedCell.IsEmpty)  then
    Exit;
  if (SelectedCell.FieldDef.MaxValues = Rec[CIndex].Count) and
    not(SelectedCell.FieldDef.MaxValues = 0) then
  begin
    Comment1 := SelectedCell.FieldDef.Caption;
    Result := E_FieldTooManyValues;
    Exit;
  end;
  if SelectedCell.Editing then
    Exit;
  if SelectedCell.FieldDef.IsReadOnly then
  begin
    Comment1 := SelectedCell.FieldDef.Caption;
    Result := E_ReadOnly;
    Exit;
  end;
  Result := 0;
end;

function TILReport.CanDeleteSelected : integer;
var
  Rec : TILDataRecord;
begin

  Result := -1;
  if not (roEditable in Ilib.FViewer.ReportOptions) then
    Exit;
  if TMan = nil then
     Exit;
  if (TMan.NodeOf(Ilib) <> nil) then
  begin
    if not (Ilib = TMan.ActiveTable) then
      Exit;
  end;
  if SelectedCell = nil then
    Exit;
  if SelectedCell.Data[SelectedCell.FieldIndex].Count = 0 then
  begin
    Exit;
  end;

  if SelectedCell.Data.IsNew and (SelectedCell.Data.IsDataCleared(True)) then
  begin
    Result := 0;
    Exit;
  end;

  Rec := SelectedCell.Owner.Data;
  if Rec.IsDeleted then
    Exit;
  if Ilib.IsReadOnly then
  begin
    Comment1 := Ilib.Name;
    Result := E_ReadOnly;
    Exit;
  end;
  if SelectedCell.FieldDef.Mandatory and (Ilib.FocusedRecord.RecordIndex <> -1) then
  begin
    Comment1 := SelectedCell.FieldDef.Caption;
    Result := E_DeleteMandatory;
    Exit;
  end;
  if SameText(SelectedCell.FieldDef.FieldName, 'SYS_LINKID') then
    Exit;
  if (Ilib is TILTreeViewTable)
    and SameText(SelectedCell.FieldDef.FieldName, 'Index') then
     Exit;

  Result := 0;
end;

function TILReport.GetItems(Index: integer): TILReportRow;
begin
  Result := TILReportRow(FColumnsList[Index]);
end;

function TILReport.GetCount: integer;
begin
  Result := FColumnsList.Count;
end;

function TILReport.GetFirstVisibleItem: integer;
begin
  Result := Ilib.FViewer.FirstItemIndex;
end;

function TILReport.GetLastVisibleItem: integer;
var
  F: integer;
begin
  Ilib.FViewer.GetVisibles(F, Result);
end;

function TILReport.GetFirstVisibleColumn: integer;
begin
  Result := Ilib.FViewer.FLeftColumn;
end;

function TILReport.GetLastVisibleColumn: integer;
var
  F: integer;
begin
  Ilib.Columns.GetVisibleColumns(F, Result);
end;

function TILReport.RowAtXY(const X, Y: integer): integer;
var
  i: integer;
  C: TILReportRow;
  R: TRect;
begin
  Result := -1;
  if FirstVisibleItem = -1 then
    Exit;
  for i := FirstVisibleItem to LastVisibleItem do
  begin
    C := Items[i];
    R := C.ItemRect;
    R.Left := 0;
    if PtInRect(R, Point(X, Y)) then
    begin
      Result := i;
      Break;
    end;
  end;
end;

function TILReport.CellAtXY(const X: integer; const Y: integer): TILReportCell;
var
  i, j: integer;
  F, L: integer;
  YPos: integer;
  R: TRect;
  C : TILReportCell;
begin
  Result := nil;
  F := FirstVisibleItem;
  L := LastVisibleItem;
  if F = -1 then
    Exit;

  YPos := Ilib.FViewer.ReportLineHeight * 2 + 2 + Ilib.FViewer.GetTopMargin;
  for i := F to L do
  begin
    if (YPos >= Y) then
    begin
      for j := 0 to Items[i].Count - 1 do
      begin
        R := Items[i].Items[j].ItemRect;
        if PtInRect(R, Point(X, Y)) then
        begin
          C := Items[i].Items[j];
          if C.Data = nil then
            Exit
          else if (C.Owner.ExpansionIndex > 0)
          and (C.Owner.ExpansionIndex > C.Data[C.FieldIndex].Count-1) then
          begin
            Exit;
          end;
          Result := Items[i].Items[j];
          Exit;
        end;
      end;
      Break;
    end;
    Inc(YPos, Ilib.FViewer.ReportLineHeight);
  end;
end;

function TILReport.RecordIndexToRowIndex(const RecordIndex: integer): integer;
var
  i: integer;
begin
  Result := -1;
  i := RecordIndex;
  while i < Count do
  begin
    if Items[i].DatasetIndex = RecordIndex then
    begin
      Result := i;
      if Items[i].ExpansionIndex > 0 then
        Dec(Result, Items[i].ExpansionIndex);
      Break;
    end;
    Inc(i);
  end;

end;

function TILReport.GetExpansionRect(Index: integer): TRect;
var
  i: integer;
  C: TILReportRow;
  R: TRect;
begin
  if Index = -1 then
  begin
    Exit;
  end;
  C := Items[Index];

  if not C.Expanded then
  begin
    Result := C.ItemRect;
    Exit;
  end;

  while C.ExpansionIndex > 0 do
  begin
    Dec(Index);
    C := Items[Index];
  end;
  Result := C.ItemRect;
  for i := Index + 1 to Index + C.ExpansionCount do
  begin
    C := Items[i];
    R := C.ItemRect;
    Result.Bottom := R.Bottom;
  end;
end;

function TILReport.GetCells(Row, Cell: integer): TILReportCell;
begin
  Result := Items[Row].Items[Cell];
end;

procedure TILReport.SetSelectedCell(const Value : TILReportCell);
begin
  if Value = FSelectedCell then
    Exit;
  if Assigned(FSelectedCell) then
  begin
    if FSelectedCell.FEditing then
    begin
      Ilib.FViewer.FCellEditor.UpdateValue;
      if not FSelectedCell.FEditing then
        Ilib.FViewer.FCellEditor.Edit;
      if Ilib.FViewer.FCellEditor.FMouseMoved then
        Ilib.FViewer.FCellEditor.Cancel;
    end;
    FSelectedCell.Selected := false;
  end;
  FSelectedCell := Value;
  if Value <> nil then
  begin
    FSelectedCell.Selected := True;
    if roEditable in Ilib.FViewer.ReportOptions then
      Ilib.FViewer.FCellEditor.MoveTo(FSelectedCell);
  end;
end;

{ TSelectField --------------------------------------------------------------
  Handles the fields of a select statement
}

function TSelectField.CalcValue(Data: TILDataRecord): string;
var
  i: integer;
  Op: string;
  DT: TDataType;
  S, s2: string;
  sAcc: string;
  NAcc: Extended;
  Dacc: Extended;
  N: Extended;
  Ni : integer;
  Literal: Boolean;
  Indx: integer;
  ParFieldString: string;
  SelField: TSelectField;
  res: string;
  HasLiterals : boolean;
  IsDate : Boolean;

begin
  Result := '';
  if FieldType <> stCalc then
    Exit;
  sAcc := '';
  NAcc := 0;
  Dacc := 0;
  i := 0;
  DT := dtString;
  if FFormula.Count = 0 then
    Exit;
  HasLiterals := false;
  try
  while i < FFormula.Count do
  begin
    S := FFormula[i];
    if Pos(c_SubstParStart, S) <> 0 then
    { Parenthesis group }
    begin
      if CharInSet(S[1], ['+', '-', '*', '/']) then
      begin
        Op := S[1];
        Delete(S, 1, 1);
      end
      else
        Op := '';
      S := StringReplace(S, c_SubstParStart, '(', [rfReplaceAll]);
      S := StringReplace(S, c_SubstParEnd, ')', [rfReplaceAll]);
      ParFieldString := GetInnerParenthesis(S, s2);
      SelField := nil;
      while ParFieldString <> '' do
      begin
        try
          SelField := TSelectField.Create(FOwner, ParFieldString, FFieldSource);
          if SelField.FError <> 0 then
          begin
            Result := '#Error: ' + GetILErrorText(Selfield.FError);
            Exit;
          end;
          res := SelField.CalcValue(Data);
          if res = '?' then
          begin
            Result := '?';
            Exit;
          end;
          { Replace the parenthesis content with the calculated result }
          S := StringReplace(S, s2, res, []);
          ParFieldString := GetInnerParenthesis(S, s2);
        finally
          SelField.Free;
        end;
      end;
      SelField := TSelectField.Create(FOwner, S, FFieldSource);
      try
        FFormula.Delete(i);
        S := SelField.FFormula[0];
        if S[1] = '+' then
          Delete(S, 1, 1);
        if Op = '' then
          Op := '+';
        S := Op + S;
        SelField.FFormula[0] := S;

        FFormula.Insert(i, SelField.FFormula[0]);
      finally
        SelField.Free;
      end;
      i := 0;
    end
    else
      Inc(i);
    { S now contains the calculated values }
  end;

  { If one of the operands is date, make all operands dates }
  IsDate := false;
  for I := 0 to FFormula.Count-1 do
    begin
     S := FFormula[i];
     if S[3] = '4' then
     begin
       IsDate := True;
       Break;
     end;
    end;

  if IsDate then
  begin
    for I := 0 to FFormula.Count-1 do
    begin
     S := FFormula[i];
     S[3] := '4';
     FFormula[i] := S;
    end;
  end;

  for i := 0 to FFormula.Count - 1 do
  begin
    S := FFormula[i];
    Op := S[1];
    Literal := (S[2] = '1');
    DT := TDataType(StrToInt(S[3]));
    Delete(S, 1, 3);

    if Pos(c_SubstParStart, S) = 0 then
    begin
      if DT in [dtText, dtString, dtResource] then { Only addition }
      begin
        if Literal then
        begin
          StringReplace(S, '''', '', [rfReplaceAll]);
          sAcc := sAcc + S;
        end
        else
        begin
          Indx := Data.IndexOf(S);
          if Data[Indx].Count = 0 then
          begin
            sAcc := sAcc + '?';
          end
          else if DT = dtText then
          begin
            sAcc := sAcc + Data[Indx].FValues.Text;
          end
          else if Data[Indx].Count > 1 then
          begin
            Result := '#Too many values';
            Exit;
          end
          else
          begin
            sAcc := sAcc + Data[Indx].AsString;
          end;
        end;
      end
      else if DT = dtNumber then
      begin
        if Literal then
        begin
          if i = 0 then
          begin
            NAcc := StrToFloat(S, Translator.FFormatSettings);
            Continue;
          end;
          N := StrToFloat(S, Translator.FFormatSettings);
          if Op = '+' then
            NAcc := NAcc + N
          else if Op = '-' then
            NAcc := NAcc - N
          else if Op = '*' then
            NAcc := NAcc * N
          else
            NAcc := NAcc / N
        end
        else
        begin
          Indx := Data.IndexOf(S);
          if Data[Indx].Count = 0 then
          begin
            Result := '?';
            Exit;
          end
          else if Data[Indx].Count > 1 then
          begin
            Result := '#Too many values';
            Exit;
          end;

          if i = 0 then
          begin
            NAcc := StrToFloat(Data[Indx][0], Translator.FFormatSettings);
            Continue;
          end;
          if Op = '+' then
            NAcc := NAcc + StrToFloat(Data[Indx][0], Translator.FFormatSettings)
          else if Op = '-' then
            NAcc := NAcc - StrToFloat(Data[Indx][0], Translator.FFormatSettings)
          else if Op = '*' then
            NAcc := NAcc * StrToFloat(Data[Indx][0], Translator.FFormatSettings)
          else if Op = '/' then
            NAcc := NAcc / StrToFloat(Data[Indx][0], Translator.FFormatSettings);
        end;
      end
      else if DT = dtDate then
      begin
        if Literal then
        begin
          if  TryStrToFloat(s, N, Translator.FFormatSettings) then { Number math }
          begin
           Ni := Round(N);
           if i = 0 then
            Dacc := ni
           else if Op = '-' then
            Dacc := Dacc - ni
           else if Op = '+' then
            Dacc := Dacc + ni
           else if Op = '*' then
            Dacc := Dacc * ni
           else if Op = '/' then
            Dacc := Round(Dacc / ni);
            HasLiterals := True;
          end
          else
          begin
           if i = 0 then
            Dacc := StrToDateTime(S, Translator.FFormatSettings)
           else if Op = '-' then
            Dacc := Dacc - StrToDateTime(S, Translator.FFormatSettings)
           else
            Dacc := Dacc + StrToDateTime(S, Translator.FFormatSettings);
          end;

        end
        else
        begin
          Indx := Data.IndexOf(S);
          if Data[Indx].Count = 0 then
          begin
            Result := '?';
            Exit;
          end
          else if Data[Indx].Count > 1 then
          begin
            Result := '#Too many values';
            Exit;
          end;
          if i = 0 then
            Dacc := StrToDateTime(Data[Indx][0], Translator.FFormatSettings)
          else if Op = '-' then
            Dacc := Dacc - StrToDateTime(Data[Indx][0], Translator.FFormatSettings)
          else
            Dacc := Dacc + StrToDateTime(Data[Indx][0], Translator.FFormatSettings)
        end;
      end;
    end;
  end;
  if DT in [dtString, dtText, dtResource] then
    Result := sAcc
  else if DT = dtNumber then
    Result := FloatToStr(NAcc, Translator.FFormatSettings)
  else if DT = dtDate then
  begin
    if (FFormula.Count = 1) or HasLiterals then
      Result := DateToStr(Dacc, Translator.FFormatSettings)
    else
      Result := FloatToStr(Dacc, Translator.FFormatSettings); { Difference }
  end;
  except
     HandleILException(E_Formula);
  end;
end;

function TSelectField.GetFieldsFromExpression(const Expression: string): TStringList;

var
  i: integer;
  P, P2: integer;
  S: string;
  Expr : string;

const
  Delimiters = [#6, #7, #10, #13, #32 .. #47, #58 .. #64, #91 .. #95];

  function IsWord: Boolean;
  var
    NothingLeft, NothingRight: Boolean;
  begin

    NothingLeft := (P = 1) or (CharInSet(Expr[P - 1], Delimiters)) or
      (CharInSet(Expr[P - 1], ['0' .. '9']));
    NothingRight := (Length(S) = Length(Expr)) or
      (CharInSet(Expr[P + Length(S)], Delimiters));
    Result := NothingLeft and NothingRight;
  end;

begin
  Expr := Expression.ToUpper;
  Result := TStringList.Create;
  P2 := 1;
  for i := 0 to FFieldSource.Count - 1 do
  begin
    S := FFieldSource.Items[i].FieldName;
    P := PosRestrict(S, Expr, False, P2);
    if (P <> 0) and not(FFieldSource.Items[i].FieldType = ftCalculated) then
    begin
      if IsWord then
        Result.Add(Copy(Expr, P, Length(S)));
      P2 := P + 1;
    end;
  end;
end;

function TSelectField.HasFieldType(const FieldString: string; const DataType: TDataType): Boolean;
var
  sl: TStringList;
  i: integer;
  Indx: integer;
begin
  Result := false;
  sl := GetFieldsFromExpression(FieldString);
  try
    for i := 0 to sl.Count - 1 do
    begin
      Indx := FFieldSource.IndexOf(sl[i]);
      if FFieldSource.Items[Indx].DataType = DataType then
      begin
        Result := True;
        Break;
      end;
    end;
  finally
    sl.Free;
  end;
end;

procedure TSelectField.ParseField(FieldString: string);
var
  P, P2: integer;
  S, s2: string;
  sl: TStringList;
  AggCheck: Boolean;
  Indx: integer;
  OldFuncs: TAggFuncs;
  AilTable : TILTable;

  function CheckAggFields(AggName: string): Boolean;
  var
    P: integer;
    P2: integer;
    Indx: integer;
  begin
    Result := false;
    P := PosCaseless(AggName, FieldString);
    while CharInQuotes(FieldString, P, False) do
      P := PosCaseless(AggName, FieldString, P + 1);
    if P <> 0 then
    begin
      P := Pos('(', FieldString, P);
      P2 := Pos(')', FieldString, P);
      s2 := Copy(FieldString, P + 1, P2 - P - 1);
      s2 := TrimBlanks(s2);
      if (s2 <> '*') and not(s2 = '') then
      begin
        Indx := FFieldSource.IndexOf(s2);
        if Indx = -1 then
          Indx := FFieldSource.IndexOfCaption(s2);
        if Indx = -1 then
        begin
          Comment1 := s2;
          FError := E_InvalidFieldName;
          Exit;
        end;
      end;
      if not SameText('COUNT', AggName) then
      begin
        Indx := FFieldSource.IndexOf(s2);
        if Indx = -1 then
          Indx := FFieldSource.IndexOfCaption(s2);
        if Indx = -1 then
        begin
          Comment1 := s2;
          FError := E_InvalidFieldName;
          Exit;
        end;
        if not(FFieldSource.Items[Indx].DataType = dtNumber) then
        begin
          Comment1 := s2;
          FError := E_NotANumberField;
          Exit;
        end;
      end;
      if (Length(FieldString) <> Length(AggName + '(' + s2 + ')')) or (s2 = '')
      then
      begin
        FError := E_SyntaxError;
        Exit;
      end;

      if FieldName = '' then
        FFieldName := FieldString;
      FFieldType := stFunction;
      FFormula.Add(AggName);
      FAggregateField := s2;
      Result := True;
    end;
  end;

  function SetSubstitutes(FieldString: string): string;
  var
    P, P2: integer;
    Cnt1, Cnt2: integer;
    Cnt: integer;
    First, Last: integer;
    A: array [1 .. 1000] of Byte;
    i: integer;
    S: string;
    sl: TStringList;
    Indx: integer;
    dt : TDateTime;
  begin
    { Get paranthesis groups at same level. These will be resolved at calc time.
      The operators and parenthesis are substituded as not to conflict with the delimiters

      Example
      (a+ (b-c) + (d - c)) + (5 * 10)
      Extracts to groups }

    { First rule out non number fields }
    P := Pos('''', FieldString);
    if P <> 0 then
    begin
      P2 := Pos('''', FieldString, P + 1);
      if P2 <> 0 then
      begin
        S := Copy(FieldString, P+1, P2-P-1);
        if not TryStrToDateTime(s, dt, Translator.FFormatSettings) then
        begin
          Result := FieldString;
          Result := StringReplace(Result, '(', '', [rfReplaceAll]);
          Result := StringReplace(Result, ')', '', [rfReplaceAll]);
          Exit;
        end;
      end;

    end;
    sl := GetFieldsFromExpression(FieldString);
    try
      for i := 0 to sl.Count - 1 do
      begin
        Indx := FFieldSource.IndexOf(sl[i]);
        if not(FFieldSource.Items[Indx].DataType in [dtNumber, dtDate]) then
        begin
          Result := FieldString;
          Result := StringReplace(Result, '(', '', [rfReplaceAll]);
          Result := StringReplace(Result, ')', '', [rfReplaceAll]);
          Exit;
        end;
      end;
    finally
      sl.Free;
    end;

    repeat
      Cnt := 0;
      P := Pos('(', FieldString);
      FillChar(A, 1000, 0);
      while P <> 0 do
      begin
        if P > Cnt then
          Cnt := P;
        A[P] := 1;
        P := PosRestrict('(', FieldString, False, P + 1);
      end;

      P := Pos(')', FieldString);
      while P <> 0 do
      begin
        if P > Cnt then
          Cnt := P;
        A[P] := 2;
        P := Pos(')', FieldString, P + 1);
      end;

      Cnt1 := 0;
      Cnt2 := 0;
      First := 0;
      Last := 0;
      for i := 1 to Cnt do
      begin
        if A[i] = 1 then
        begin
          Inc(Cnt1);
          if First = 0 then
            First := i;
        end
        else if A[i] = 2 then
          Inc(Cnt2);
        if (Cnt1 > 0) and (Cnt1 = Cnt2) then
        begin
          Last := i;
          Break;
        end;
      end;

      if Cnt1 <> Cnt2 then
      begin
        FError := E_MissingParenthesis;
        Exit;
      end;
      { Substitute operators }
      S := Copy(FieldString, First, Last - First + 1);
      S := StringReplace(S, '+', c_SubstAdd, [rfReplaceAll]);
      S := StringReplace(S, '-', c_SubstSub, [rfReplaceAll]);
      S := StringReplace(S, '*', c_SubstMul, [rfReplaceAll]);
      S := StringReplace(S, '/', c_SubstDiv, [rfReplaceAll]);
      S := StringReplace(S, '(', c_SubstParStart, [rfReplaceAll]);
      S := StringReplace(S, ')', c_SubstParEnd, [rfReplaceAll]);

      Delete(FieldString, First, Last - First + 1);
      Insert(S, FieldString, First);
      P := Pos('(', FieldString);
    until P = 0;
    if PosRestrict(')', FieldString, False) <> 0 then
      FError := E_MissingParenthesis
    else
      Result := FieldString;
  end;

  function FieldToList(var FieldString: string): TStringList;
  var
    P: integer;
    Found: Boolean;
  begin
    P := PosRestrict('(', FieldString, False);
    Result := TStringList.Create;
    if P <> 0 then
    begin
      { Organize paranthesisis groups at same levels. Operators within parenthesis
        are substituted not to conflict with operator search outside groups. }
      FieldString := SetSubstitutes(FieldString);
      if FError <> 0 then
        Exit;
    end;

    { Break up string at operators outside parenthesis }
    repeat
      Found := false;
      P := PosRestrict('*', FieldString, False);
      if P <> 0 then
      begin
        FieldString[P] := c_SubstMul;
        Insert(#5, FieldString, P); { Delimiter }
        Found := True;
      end;
      P := PosRestrict('/', FieldString, False);
      if P <> 0 then
      begin
        FieldString[P] := c_SubstDiv;
        Insert(#5, FieldString, P); { Delimiter }
        Found := True;
      end;
      P := PosRestrict('+', FieldString, False);
      if P <> 0 then
      begin
        FieldString[P] := c_SubstAdd;
        if (P > 1) and not(CharInSet(FieldString[P - 1], ['*', '/', '-', '+',
          c_SubstAdd, c_SubstSub, c_SubstDiv, c_SubstMul])) then
          Insert(#5, FieldString, P); { Delimiter }
        Found := True;
      end;
      P := PosRestrict('-', FieldString, False);
      if P <> 0 then
      begin
        FieldString[P] := c_SubstSub;
        if (P > 1) and not(CharInSet(FieldString[P - 1], ['*', '/', '-', '+',
          c_SubstAdd, c_SubstSub, c_SubstDiv, c_SubstMul])) then
          Insert(#5, FieldString, P); { Delimiter }
        Found := True;
      end;
    until not Found;

    { Restore operators to normal }
    FieldString := StringReplace(FieldString, c_SubstMul, '*', [rfReplaceAll]);
    FieldString := StringReplace(FieldString, c_SubstDiv, '/', [rfReplaceAll]);
    FieldString := StringReplace(FieldString, c_SubstAdd, '+', [rfReplaceAll]);
    FieldString := StringReplace(FieldString, c_SubstSub, '-', [rfReplaceAll]);
    CreateListFromDelimiter(#5, FieldString, Result);
    TrimList(Result, false);
    if Result.Count = 0 then
      FError := E_SyntaxError;
  end;

  function IsFunction(FieldString: string): Boolean;
  begin
    Result := SameText(FieldString, 'COUNT') or SameText(FieldString, 'MAX') or
      SameText(FieldString, 'MIN') or SameText(FieldString, 'SUM') or
      SameText(FieldString, 'AVG');
  end;

  procedure ParseFormula(FieldString: string);
  var
    P: integer;
    sl: TStringList;
    i: integer;
    S: string;
    L: integer;
    Op: string;
  begin
    if IsFunction(FieldString) then
      Exit; { Added in CheckAgg }
    sl := FieldToList(FieldString);
    if FError <> 0 then
    begin
      sl.Free;
      Exit;
    end;
    try
      for i := 0 to sl.Count - 1 do
      begin
        S := TrimBlanks(sl[i], false);
        if i > 0 then
        begin
          Op := S[1];
          Delete(S, 1, 1);
          S := TrimBlanks(S, false);
        end;
        P := Pos(c_SubstParStart, S);
        if P = 0 then
        begin
          P := Pos('''', S);
          if P = 1 then
          begin
            P2 := Pos('''', S, P + 1);
            L := Length(S);
            if P2 <> L then
            begin
              FError := E_MissingQuote;
              Exit;
            end;
          end
          else if P > 0 then
          begin
            FError := E_QuoteError;
            Exit;
          end;
        end;
        if i > 0 then
          S := Op + S;
        FFormula.Add(S);
      end;
      TrimList(FFormula, false);
    finally
      if Assigned(sl) then
        sl.Free;
    end;
  end;

  procedure CheckFormula;
  var
    i: integer;
    dt, dt1, dt2: TDataType;
    dte: TDateTime;
    S: string;
    Op: string;
    N: Extended;
    Indx: integer;
    isLiteral: Boolean;
    P: integer;
    StrField: Boolean;
    Cnt: integer;
  begin
    Op := '+';
    Cnt := -1;
    dt1 := dtString;
    dt2 := dtString;
    for i := 0 to FFormula.Count - 1 do
    begin
      Inc(Cnt);
      S := FFormula[i];
      if IsFunction(S) then
      begin
        Cnt := -1;
        Continue;
      end;
      P := PosRestrict(c_SubstParStart, S, False);
      if P <> 0 then
      begin
        Cnt := -1;
        FFieldType := stCalc;
        Continue;
        { Paranthesis are resolved at calc time }
      end;

      if i > 0 then
      begin
        Op := S[1];
        Delete(S, 1, 1);
      end;

      StrField := HasFieldType(S, dtString) or HasFieldType(S, dtText);
      if (Pos('''', S) = 0) and not StrField then
      begin
        if TryStrToFloat(S, N, Translator.FFormatSettings) then
          begin
            if not Odd(Cnt) then
              dt1 := dtNumber
            else
              dt2 := dtNumber;
            IsLiteral := True;
        end
        else
        begin
          Indx := FFieldSource.IndexOf(S);
          if Indx <> -1 then
          begin
            if not Odd(Cnt) then
              dt1 := FFieldSource.Items[Indx].DataType
            else
              dt2 := FFieldSource.Items[Indx].DataType;
            isLiteral := false;
          end
          else
          begin
            Comment1 := S;
            FError := E_InvalidFieldname;
            Exit;
          end;
        end;
      end
      else
      begin
        S := StringReplace(S, '''', '', [rfReplaceAll]);
        if TryStrToDateTime(S, dte, Translator.FFormatSettings) then
        begin
          if not Odd(Cnt) then
            dt1 := dtDate
          else
            dt2 := dtDate;
        end
        else
        begin
          if not Odd(Cnt) then
            dt1 := dtString
          else
            dt2 := dtString;
        end;
        isLiteral := not StrField;
      end;
      if Odd(Cnt) then
        dt := dt2
      else
        dt := dt1;
      if isLiteral then
        FFormula[i] := Op + '1' + IntToStr(integer(dt)) + S
      else
        FFormula[i] := Op + '0' + IntToStr(integer(dt)) + S;
      FFieldType := stCalc;

      if Odd(Cnt) then { Check each pair }
      begin
        Comment1 := '';

        if (dt1 = dtDate) and (dt2 = dtDate) and (Op <> '-') then
        begin
           Comment1 := Op;
           FError := E_OperatorMismatch;
           Exit;
        end;
        if ((dt1 = dtDate) or (dt2 = dtDate)) and
        (Op <> '-') and (Op <> '+') then
        begin
          Comment1 := Op;
          FError := E_OperatorMismatch;
          Exit;
        end;

        if ((dt1 = dtDate) or (dt2 = dtDate))
        and (dt1 <> dt2) then
        begin
           if not (dt1 in [dtNumber, dtDate])
           or not (dt2 in [dtNumber, dtDate]) then
           begin
             Comment1 := ''; /// Field name sould be set
             FError := E_InvalidDate;
             Exit;
           end;
        end;

        if not (dt1 = dtDate) and not (dt2 = dtDate) then
        if (dt1 <> dt2) and not(Ilib.IsStringType(dt1) and
          Ilib.IsStringType(dt2)) then
        begin
          Comment1 := ''; /// Field name sould be set
          FError := E_InvalidDate;
          Exit;
        end;
        if ((Op = '*') or (Op = '/') or (Op = '-')) and Ilib.IsStringType(dt1)
        then
        begin
          Comment1 := Op;
          FError := E_OperatorMismatch;
          Exit;
        end
        else if ((Op = '*') or (Op = '/'))  and (dt1 = dtDate) then
        begin
          Comment1 := Op;
          FError := E_OperatorMismatch;
          Exit;
        end;
      end;
    end;
  end;

  procedure SetPriority(var FieldString: string);
  { If field is a number calculation, sets the priority of mul and div expressions }
  var
    P: integer;
    sl: TStringList;
    i: integer;
    S, s2, s3: string;
  begin
    { Delete all blanks outside quoutes }
    P := PosRestrict(' ', FieldString, False);
    while P <> 0 do
    begin
      Delete(FieldString, P, 1);
      P := PosRestrict(' ', FieldString, False);
    end;
    if (PosRestrict('*', FieldString, False) = 0) and
      (PosRestrict('/', FieldString, False) = 0) then
      Exit;

    P := PosRestrict('(', FieldString, False);
    { Resolve parenthesis. Open up parenthesis that contain mul/div operators }
    if P <> 0 then
    begin
      S := GetInnerParenthesis(FieldString, s2);
      if Pos('#Error', S) = 1 then
      begin
        FError := E_MissingParenthesis;
        Exit;
      end;
      while S <> '' do
      begin
        if (PosRestrict('*', S, False) <> 0) or (PosRestrict('/', S, False) <> 0) then
          { Open up : }
          FieldString := StringReplace(FieldString, s2, S, [rfReplaceAll])
        else
        begin
          { Let be, but replace parenthesis with substitutes so that next turn of loop
            doesnt get caught in the previous one }
          s3 := s2;
          s2[1] := c_SubstParStart;
          s2[Length(s2)] := c_SubstParEnd;
          { Prevent eternal loop : }
          FieldString := StringReplace(FieldString, s3, s2, [rfReplaceAll]);
        end;
        S := GetInnerParenthesis(FieldString, s2);
        if Pos('#Error', S) = 1 then
        begin
          FError := E_MissingParenthesis;
          Exit;
        end;
      end;
      { Restore normal parenthesis }
      FieldString := StringReplace(FieldString, c_SubstParStart, '(',
        [rfReplaceAll]);
      FieldString := StringReplace(FieldString, c_SubstParEnd, ')',
        [rfReplaceAll]);
    end;

    { Break up string at operators outside parenthesis }
    sl := FieldToList(FieldString);
    try
      if FError <> 0 then
        Exit;

      { Check mul and div operators, If found, eclose expression in parenthesis.
        This will ensure that the CalValue proc treats theese first }
      i := 0;
      if (sl.Count = 1) and (sl[0] = '*') then
      begin
        Comment1 := '';
        FError := E_InvalidFieldName;
        Exit;
      end;

      while i < sl.Count do
      begin
        S := sl[i];
        if S = '*' then { Missing number }
        begin
          FError := E_SyntaxError;
          Exit;
        end;

        if (S[1] = '*') then
        begin
          if i - 1 = -1 then
          begin
            FError := E_SyntaxError;
            Exit;
          end;
          S := S + ')';
          s2 := sl[i - 1];
          if CharInSet(s2[1], ['*', '/', '+', '-']) then
            Insert('(', s2, 2)
          else
            s2 := '(' + s2;
          S := s2 + S;
          sl[i - 1] := S;
          sl.Delete(i);
        end
        else
          Inc(i);
      end;

      i := 0;
      while i < sl.Count do
      begin
        S := sl[i];
        if S = '/' then { Missing number }
        begin
          FError := E_SyntaxError;
          Exit;
        end;
        if S[1] = '/' then
        begin
          if i - 1 = -1 then
          begin
            FError := E_SyntaxError;
            Exit;
          end;
          S := S + ')';
          s2 := sl[i - 1];
          if CharInSet(s2[1], ['*', '/', '+', '-']) then
            Insert('(', s2, 2)
          else
            s2 := '(' + s2;
          S := s2 + S;
          sl[i - 1] := S;
          sl.Delete(i);
        end
        else
          Inc(i);
      end;
      FieldString := sl.Text;
      FieldString := StringReplace(FieldString, #$D#$A, '', [rfReplaceAll]);
      FieldString := StringReplace(FieldString, c_SubstParStart, '(',
        [rfReplaceAll]);
      FieldString := StringReplace(FieldString, c_SubstParEnd, ')',
        [rfReplaceAll]);

    finally
      sl.Free;
    end;
  end;

  function Split : TILTable;
  var
    P : integer;
    TblName : string;
    Indx : integer;
  begin
    Result := nil;
    P := PosRestrict('.', FieldString, False);
    if P > 0 then
    begin
      TblName := Trim(Copy(FieldString, 1, P - 1));
      if TMan.TableOf(TblName) = nil then
      begin
        Exit;
      end;
      Delete(FieldString, 1, P);
      FieldString := Trim(FieldString);
      Result := TMan.TableOf(TblName);
      if Result <> nil then
      begin
        if FieldString <> '*' then
        begin
          Indx := Result.Fields.IndexOf(FieldString);
          if Indx = -1 then
            Result := nil;
        end;
      end;
    end;

  end;

begin { Parse field }
  if FOwner <> nil then
    OldFuncs := FOwner.FAggFuncs;
  FieldString := TrimBlanks(FieldString);

  { Check for field from other table }
  if Ilib.IsFreeTable then
    AilTable := nil
  else
   AilTable := Split;
  if (AilTable <> nil) and (AilTable <> ILib) then
  begin
    FFieldName := FieldString;
    FFieldType := stAlien;
    FTableName := AilTable.TableName;
    FAggregateFunc := agNone;
    Exit;
  end;

  { Check for simple field }
  Indx := FFieldSource.IndexOf(FieldString);
  if Indx <> -1 then
  begin
    FFieldName := FieldString;
    FFieldType := stField;
    FAggregateFunc := agNone;
    Exit;
  end
  else
  begin
    Indx := FFieldSource.IndexOfCaption(FieldString);
    if Indx <> -1 then
    begin
      FFieldName := FFieldSource.Items[Indx].FieldName;
      FFieldType := stField;
      FAggregateFunc := agNone;
      Exit;
    end;
  end;

  { Next look for the AS keyword at end of string }
  P := PosRestrict(' AS ', FieldString, False);
  if P <> 0 then
  { Som kind of calc field }
  begin
    S := Copy(FieldString, P + 1, Length(FieldString));
    sl := TStringList.Create;
    try
      CreateListFromDelimiter(' ', S, sl);
      TrimList(sl);
      if sl.Count = 2 then
      begin
        FFieldName := sl[1];
        FFieldType := stCalc;
        Delete(FieldString, P, Length(FieldString));
      end;
    finally
      sl.Free;
    end;
  end;

  { Look for aggregate functions. Can be only one }
  if FOwner <> nil then
  begin
   AggCheck := CheckAggFields('COUNT');
   if not AggCheck then
   begin
    if FError <> 0 then
      Exit;
   end
   else
   begin
    Include(FOwner.FAggFuncs, agCount);
    FAggregateFunc := agCount;
   end;
   AggCheck := CheckAggFields('SUM');
   if not AggCheck then
   begin
    if FError <> 0 then
      Exit;
   end
   else
   begin
    Include(FOwner.FAggFuncs, agSum);
    FAggregateFunc := agSum;
   end;

   AggCheck := CheckAggFields('MIN');
   if not AggCheck then
   begin
    if FError <> 0 then
      Exit;
   end
   else
   begin
    Include(FOwner.FAggFuncs, agMin);
    FAggregateFunc := agMin;
   end;

   AggCheck := CheckAggFields('MAX');
   if not AggCheck then
   begin
    if FError <> 0 then
      Exit;
   end
   else
   begin
    Include(FOwner.FAggFuncs, agMax);
    FAggregateFunc := agMax;
   end;

   AggCheck := CheckAggFields('AVG');
   if not AggCheck then
   begin
    if FError <> 0 then
      Exit;
   end
   else
   begin
    Include(FOwner.FAggFuncs, agAvg);
    FAggregateFunc := agAvg;
   end;

   if FOwner.FAggFuncs <> OldFuncs then
   begin
    Exit;
   end;
  end;

  { Now we have ruled out simple fields functions and the AS key word.
    Check for expressions }
  if (PosRestrict('(', FieldString, False) <> 0) and
    (PosRestrict(')', FieldString, False) <> 0) then
    FFieldType := stCalc
    { We consider all expression containing parenthesis as calc fields }
  else if (PosRestrict('(', FieldString, False) <> 0) or
    (PosRestrict(')', FieldString, False) <> 0) then
  begin
    FError := E_MissingParenthesis;
    Exit;
  end;
  if (FOwner = nil) or FOwner.FBuilding then
    SetPriority(FieldString); { Div and mul comes first }
  if FError <> 0 then
    Exit;

  if FFieldName = '' then
  { Expressions must have a substitute name. No AS was defined }
  begin
    FFieldName := FieldString;
    FFieldName := StringReplace(FFieldName, ' ', '_', [rfReplaceAll]);
  end;
  ParseFormula(FieldString);
  if FError = 0 then
    CheckFormula;
end;

constructor TSelectField.Create(Owner: TSelectFields; FieldString: string;
  FieldSource: TILFieldDefs);
begin
  FOwner := Owner;
  FFormula := TStringList.Create;
  FFieldSource := FieldSource;
  Ilib := FieldSource.Owner as TILTable;
  ParseField(FieldString);
  inherited Create;
end;

destructor TSelectField.Destroy;
begin
  if Assigned(FFormula) then
    FFormula.Free;
  inherited Destroy;
end;

{ TSelectFields -------------------------------------------------------------- }

function TSelectFields.GetCount: integer;
begin
  Result := FSelectFields.Count;
end;

function TSelectFields.GetItems(Index: integer): TSelectField;
begin
  Result := TSelectField(FSelectFields[Index]);
end;

function TSelectFields.GetAliens(Index : integer) : TSelectField;
var
  i, ind : integer;
begin
   Result := nil;
   Ind := -1;
   for I := 0 to Count-1 do
    if Items[i].FFieldType = stAlien then
    begin
       inc(Ind);
       if Ind = Index then
       begin
         Result := Items[i];
         break;
       end;
    end;
end;

function TSelectFields.GetAlienCount : integer;
var
  i : integer;
begin
  Result := 0;
  for I := 0 to Count-1 do
    if Items[i].FFieldType = stAlien then
      inc(Result);
end;

function TSelectFields.IsLiterals: Boolean;
var
  i: integer;
  sl: TStringList;
  S: string;
begin
  Result := True;
  if Count = 0 then
  begin
    Result := false;
    Exit;
  end;
  for i := 0 to Count - 1 do
  begin
    S := Items[i].FFormula.Text;
    if S = '' then
    begin
      Result := false;
      Exit;
    end;
    sl := Items[i].GetFieldsFromExpression(S);
    try
      if sl.Count > 0 then
      begin
        Result := false;
        Break;
      end;
    finally
      sl.Free;
    end;
  end;
end;

constructor TSelectFields.Create(Owner: TSearcher; Fields: TStringList;
  FieldSource: TILFieldDefs);
{ Fields is the section extracted from TSearcher.GetSections }
type
  TAlienField = record
    TableName : string;
    FieldName : string;
    ColIndex : integer;
  end;
var
  i: integer;
  SelField: TSelectField;
  SectionFields: TStringList;
  S: string;
  P: integer;

  function Split : integer;
  var
    S, Fname, TName : string;
    i, j, P, P2 : integer;
    TBL : TilTable;
    Cnt : integer;
    IsAll : Boolean;
    FAlienFields : array of TAlienField;
  begin
     Result := 0;
     Finalize(FAlienFields);
     for I := 0 to SectionFields.Count-1 do
     begin
        IsAll := false;
        TName := Ilib.TableName;
        S := SectionFields[i];
        P := PosRestrict(ILib.TableName + '.', S, False);
        if P <> 0 then
        begin
          P2 := Pos('.', S, P);
          Delete(S, P, P2-P+1);
          SectionFields[i] := S;
        end;

        P := PosRestrict('.', S, False);
        if P <> 0 then
        begin
          TName := Trim(Copy(S, 1, P-1));
          FName := Trim(Copy(S, P + 1, Length(S)));
        end
        else
          FName := Trim(S);

        Tbl := TMan.TableOf(TName);
        if (Tbl = nil) and (P <> 0) then
        { Must be an expression. Check for foreign field }

        begin
         for j := 0 to Tman.TableCount-1 do
           if TMan.Tables[j] <> ILib then
           if PosRestrict(TMan.Tables[j].TableName + '.', S, False) <> 0 then
           begin
             Comment1 := TMan.Tables[j].TableName;
             Result := E_ForeignOperands;
             Exit;
           end;
        end;

        if (FName = '*') and (TBL <> nil) then
        begin
          Cnt := Tbl.Fields.Count;
          IsAll := True;
        end
        else
          Cnt := 1;

        for j := 0 to Cnt-1 do
        begin
          if IsAll then
          begin
            if not Tbl.Fields.Items[j].ReportField then
             Continue;
            FName := Tbl.Fields.Items[j].FieldName;
          end
          else
          begin
          end;

          SetLength(FAlienFields, Length(FAlienFields) + 1);
          FAlienFields[High(FAlienFields)].TableName := TName;
          FAlienFields[High(FAlienFields)].FieldName := FName;
          FAlienFields[High(FAlienFields)].ColIndex := J;
        end;
     end;
     if Length(FAlienFields) > 0 then
     begin
       SectionFields.Clear;
       for I := 0 to High(FAlienFields) do
       begin
         if SameText(FAlienFields[i].TableName, ILib.TableName) then
           SectionFields.Add(FAlienFields[i].FieldName)
         else
           SectionFields.Add(FAlienFields[i].TableName + '.' + FAlienFields[i].FieldName);
       end;
     end;
  end;

begin
  inherited Create;
  FOwner := Owner;
  FSelectFields := TList.Create;
  Ilib := FOwner.Ilib;
  S := Fields.Text;
  P := PosRestrict(',', S, True);
  while P <> 0 do
  begin
   { if CharInQuotes(S, P, False) then
    begin
      P := Pos(',', S, P + 1);
      Continue;
    end;}
    S[P] := #1;
    P := PosRestrict(',', S, True, P + 1);
  end;
  S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
  SectionFields := TStringList.Create;
  CreateListFromDelimiter(#1, S, SectionFields);
  FError := Split;
  if FError <> 0 then
  begin
    FBuilding := false;
    SectionFields.Free;
    Exit;
  end;
  FBuilding := True;
  FAggFuncs := [];
  try
    if (SectionFields.Count = 1) and (SectionFields[0] = '*') then
    begin
      if FieldSource = Ilib.Fields then
        Ilib.Columns.AssignSource;
      Exit;
    end;
    for i := 0 to SectionFields.Count - 1 do
    begin
      SelField := TSelectField.Create(Self, SectionFields[i], FieldSource);
      if SelField.FError <> 0 then
      begin
        FError := SelField.FError;
        SelField.Free;
        Exit;
      end;
      FSelectFields.Add(SelField);
    end;
  finally
    FBuilding := false;
    SectionFields.Free;
  end;

end;

destructor TSelectFields.Destroy;
var
  i: integer;
  SelField: TSelectField;
begin
  for i := 0 to Count - 1 do
  begin
    SelField := TSelectField(FSelectFields[i]);
    SelField.Free;
  end;
  FSelectFields.Free;
  inherited Destroy;
end;

procedure TSelectFields.Clear;
var
  i: integer;
  SelField: TSelectField;
begin
  Ilib.Fields.ClearCalcFields;
  Ilib.Columns.Clear;
  for i := 0 to Count - 1 do
  begin
    SelField := TSelectField(FSelectFields[i]);
    SelField.Free;
  end;
  FSelectFields.Clear;
end;

function TSelectFields.IndexOfCalcField(const FieldName: string): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to Count - 1 do
  begin
    if SameText(FieldName, Items[i].FieldName) then
      if Items[i].FieldType = stCalc then
      begin
        Result := i;
        Break;
      end;
  end;
end;


{ TExpressionHandler ---------------------------------------------------------
  Base class for handling select, update and insert expsressions
}

procedure TExpressionHandler.SetExpression(const Value : TStringList);
begin
  if Value = FExpression then
    Exit;
  TrimList(Value);
  FExpression.Assign(Value);
  FPrepared := false;
end;

function TExpressionHandler.Execute(ADataRecord: TILDataRecord): Boolean;
begin
  Result := false;
end;

function TExpressionHandler.Validate: integer;
begin
  Result := Prepare;
end;

function TExpressionHandler.Prepare: integer;
begin
  while TMan.FSystemBusy do
  begin
   Application.ProcessMessages;
   sleep(50);
  end;
  Result := 0;
  RemoveComments(FExpression);
end;

function TExpressionHandler.DecomposeElement(AnElement: string;
  Decomp: TStringList): integer;
begin
  Result := 0;
end;

function TExpressionHandler.TrimOperators(AnExpression: string): string;
begin
  Result := '';
end;

function TExpressionHandler.RecordToExpression(ARecord: TILRecord): TStringList;
begin
  Result := nil;
end;

constructor TExpressionHandler.Create(TableOwner: TILTable;
  AnExpression: TStringList);
begin
  Ilib := TableOwner;
  TrimList(AnExpression);
  FExpression := TStringList.Create;
  FExpression.Assign(AnExpression);
  FSections[0] := TStringList.Create;
  FSections[1] := TStringList.Create;
  FSections[2] := TStringList.Create;
  FPrivate := True;
  if Self is TUpdater then
  with Self as TUpdater do
  begin
     FRecordIndex := -1;
     Tman.FMainThreadBusy := True;
  end;
  inherited Create;
end;

constructor TExpressionHandler.Create(TableOwner: TILTable; ARecord: TILRecord);
begin
  Ilib := TableOwner;
  FExpression := RecordToExpression(ARecord);
  FSections[0] := TStringList.Create;
  FSections[1] := TStringList.Create;
  FSections[2] := TStringList.Create;
  FPrivate := True;
  inherited Create;
end;

destructor TExpressionHandler.Destroy;
var
  i: integer;
begin
  i := 0;
  while Assigned(FDecomps[i]) do
  begin
    FDecomps[i].Free;
    Inc(i);
  end;
  if Assigned(FExpression) then
    FExpression.Free;
  if Self is TSearcher then
    with Self as TSearcher do
    begin
      if Assigned(FFetchRecords) then
        FFetchRecords.Free;
    end;
  FSections[0].Free;
  FSections[1].Free;
  FSections[2].Free;
  inherited Destroy;
end;

{ TSearcher ------------------------------------------------------------------
  The search implementation of TExpressionHandler }

function TSearcher.TrimOperators(AnExpression: string): string;
begin
  while Pos(c_NotOperator + ' ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, c_NotOperator + ' ',
      c_NotOperator, [rfReplaceAll]);
  while Pos(' ' + c_NotOperator, AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' ' + c_NotOperator,
      c_NotOperator, [rfReplaceAll]);

  while Pos('= ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, '= ', '=', [rfReplaceAll]);
  while Pos(' =', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' =', '=', [rfReplaceAll]);

  while Pos('< ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, '< ', '<', [rfReplaceAll]);
  while Pos(' <', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' <', '<', [rfReplaceAll]);

  while Pos('> ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, '> ', '>', [rfReplaceAll]);
  while Pos(' >', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' >', '>', [rfReplaceAll]);

  while Pos('/ ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, '/ ', '/', [rfReplaceAll]);
  while Pos(' /', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' /', '/', [rfReplaceAll]);

  while Pos('\ ', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, '\ ', '\', [rfReplaceAll]);
  while Pos(' \', AnExpression) <> 0 do
    AnExpression := StringReplace(AnExpression, ' \', '\', [rfReplaceAll]);
  Result := AnExpression;
end;

function TSearcher.RecordToExpression(ARecord: TILRecord): TStringList;
var
  i, j: integer;
  S, Expr: string;
  P: integer;
  Val, Val2: string;
  IsNot: string;
  FldName: string;
  IsEmpty: Boolean;
  LogOp: string;
  LineLogOp: string;
  TypedOps: Boolean;
begin
  Result := TStringList.Create;
  for i := 0 to ARecord.Count - 1 do
  begin
    FldName := ARecord[i].FieldDef.Ilib.TableName + '.' + ARecord[i].FieldName;
    Expr := '';
    if ARecord[i].Expression.Count > 0 then
      ARecord[i].FValues.Assign(ARecord[i].Expression);
    TypedOps := (ARecord[i].FieldDef.SearchControl is TILSearchGrid) and
      not(Ilib.AlignOperators);
    for j := 0 to ARecord[i].Count - 1 do
    begin
      S := ARecord[i][j];
      S := TrimBlanks(S);
      if S = '' then
        Continue;
      S := SetESCCodes(S);
      IsNot := '';
      if TypedOps then
      begin
        if CharInSet(S[1], [c_LogAnd, c_LogOr]) then
        begin
          if S[1] = c_LogAnd then
            LogOp := '\'
          else
            LogOp := '/';
          Delete(S, 1, 1);
          S := TrimBlanks(S);
        end
        else
          LogOp := '\';
        if j = 0 then
          LineLogOp := LogOp;
        if S = '' then
          Continue;
      end;

      if (S[1] = c_NotOperator) then
      begin
        IsNot := c_NotOperator;
        Delete(S, 1, 1);
      end;

      if (CharInSet(S[1], ['>', '<', '=', c_NotOperator])) and (Length(S) > 1)
      then
      begin
        if CharInSet(S[2], ['=', '>', '<']) then
          P := 3
        else
          P := 2;
        Val := Trim(Copy(S, P, Length(S)));
        Delete(S, P, Length(S));
        if not TypedOps then
        begin
          if ARecord[i].FieldDef.ValueLogOp in [vlAnd, vlAndAll] then
            S := '\' + IsNot + FldName + S
          else
            S := '/' + IsNot + FldName + S;
        end
        else
        begin
          S := LogOp + IsNot + FldName + S
        end;
      end
      else
      begin
        Val := S;
        if not TypedOps then
        begin
          if ARecord[i].FieldDef.ValueLogOp = vlAnd then
            S := '\' + IsNot + FldName + '='
          else if ARecord[i].FieldDef.ValueLogOp = vlAndAll then
            S := '\' + IsNot + FldName + '=='
          else
            S := '/' + IsNot + FldName + '=';
        end
        else
        begin
          if LogOp = '==' then
            S := '\' + IsNot + FldName + '=='
          else
            S := LogOp + IsNot + FldName + '=';
        end;
      end;
      IsEmpty := TMan.IsValueNull(Val);
      Val2 := '';
      if (ARecord[i].FieldDef.DataType in [dtString, dtText, dtLongText, dtDate, dtResource])
        and not IsEmpty then
      begin
        P := Pos('..', Val);
        if P <> 0 then
        begin
          Val2 := Copy(Val, P + 2, Length(Val));
          Val := Copy(Val, 1, P - 1);
        end;
        if Val = '' then
          Exit;
        if Val[1] <> '''' then
           Val := '''' + Val;
        if Val[Length(Val)] <> '''' then
           Val := Val + '''';
        Val := TrimBlanks(Val);
        if Val2 <> '' then
        begin
          if Val2[1] <> '''' then
            Val2 := '''' + Val2;
          if Val2[Length(Val2)] <> '''' then
            Val2 := Val2 + '''';
          Val2 := TrimBlanks(Val2);
          Val := Val + '..' + Val2;
        end;

      end;
      S := S + Val;
      Expr := Expr + S;
    end;
    if TypedOps then
    begin
      if LogOp = '==' then
        Expr := '\' + Expr
      else
        Expr := LineLogOp + Expr;
    end
    else if Ilib.AlignOperators then
    begin
      if (Ilib.Fields.FieldLogOp = flAnd) then
        Expr := '\' + Expr
      else
        Expr := '/' + Expr;
    end
    else
    begin
      if ARecord[i].FieldDef.FieldLogOp = flAnd then
        Expr := '\' + Expr
      else
        Expr := '/' + Expr;
    end;
    if Pos('//', Expr) = 1 then
      Expr[2] := '\';
    Result.Add(Expr);
  end;
end;

function TSearcher.GetFromIndexes(var State: TIndexSearchState;
  RootDataset: TILDataset = nil): TStringList;
{ Indexed search
  This function does the work of finding records from a search expression
  conatining indexed fields.

  The function returns an array of record numbers representing the records that
  fully of partially match the search criteria. The main search engine then processes
  the numbers to pupulate the result set with real records.

  What kind of processing is decided by the State return value:

  issComplete: GetFromIndexes was able to identify the exact number of matches. The
  main search engine simply look up the records and adds them to the result set.

  issInclude: GetFromIndexes could not determine the exact number of matching records
  due to fields that were not indexed or indexes that for some reason were excluded.
  The record numbers returned contains all the matching records + the ones that
  could not be filtered out because of the lacking indexes.

  The main search engine performs a scan limited to the record numbers delivered,
  in order to strip off the aliens.

  issScan : None of the fields were indexed or indexes could not be applied.
  GetFromIndexes could not process the query. The main engine must do a full scan of the table
}
type
  TInterval = record
    St, Nd: integer;
  end;

  TIndexInfo = record
    Indexed: Boolean;
    LogOp: string;
    LineLogOp: string;
    Prio: TIndexPriority end;
    TFieldIntervals = array of TInterval;
    TExpressionInterval = array of TFieldIntervals;
    TExpressionIntervals = array of TExpressionInterval;

    { An interval defines a sequence of record numbers obtained from the index linklist. This
      sequenece is computed by looking up the value in the key list and
      checking it against the conditoinal operator. An interval may broken down i sub intervals,
      stored in the TFieldIntervals array. The inertvals of all fields in a single expression
      (a "line" in the FExpression stringlist) are stored in the TExpressionInterval
      while the inervals of all lines are wrapped in the TExpressionIntervals array.

      Field intervals: (Start1, End1), (Start1, End2) ...
      The Analyze and GetKeys sub procs do fill the interval arrays.
    }

  var
    Intervals: TExpressionIntervals;
    LineIndexes: array [0 .. 29, 0 .. 29] of TIndexInfo;
    TM: LongWord;
    MaxPrio: TIndexPriority;
    RCounter: integer;
    HasAndAlls: Boolean;

    function GetKeys(TBL: TILTable; Value: string; LogOp, CO: string;
      FieldName: string; var List: TFieldIntervals): integer;

  var
    Indx, Indx2: integer;
    Found1, Found2 : Boolean;
    i: integer;
    S, s2: string;
    res: integer;
    P: integer;
    N : TKey;

  function IntervalOverlaps(var StIndx, NdIndx: integer): Boolean;

  var
    i, j, k: integer;
    Interval: TInterval;
  begin
    Result := false;
    Exit;
    for i := 0 to High(Intervals) do
      for j := 0 to High(Intervals[i]) do
        for k := 0 to High(Intervals[i, j]) do
        begin
          Interval := Intervals[i, j, k];
          if (StIndx = Interval.St) and (NdIndx = Interval.Nd) then
          begin
            Result := True;
            Exit;
          end;
        end;
  end;

  function IsLinked(KeyID: string; var RecordCount: integer;
    var FldIndxStr: string): Boolean;

  var
    Indx, Indx2: integer;
    S, s2: string;
    N : TKey;
  begin
    Result := false;
    RecordCount := 0;
    if FieldName = 'SYS_Deleted' then
      Indx := 99
    else
    begin
     Indx := Tbl.Fields.IndexOf(FieldName);
     Indx := Tbl.Fields.Items[Indx].FieldID;
    end;
    FldIndxStr := FillZeroes(IntToStr(Indx), 2);
    S := KeyID + '|' + FldIndxStr;
    Tbl.FindKey(1, S, Indx);
    if Indx < Tbl.FLinkList.Count then
    begin
      N := TKey(Tbl.FLinkList[Indx]);
      s2 := N.Key;
      Result := (Pos(S, s2) = 1);
      if Result then
      begin
        Tbl.FindKey(1, S + 'x', Indx2);
        Dec(Indx2);
        if not IntervalOverlaps(Indx, Indx2) then
        begin
          RecordCount := Indx2 - Indx + 1;
          SetLength(List, Length(List) + 1);
          List[Length(List) - 1].St := Indx;
          List[Length(List) - 1].Nd := Indx2;
        end
        else
        begin
          RecordCount := 0;
          SetLength(List, Length(List) + 1);
          List[Length(List) - 1].St := 0;
          List[Length(List) - 1].Nd := -1;
        end
      end;
    end;
  end;

  function GetWilds: integer;

  var
    Indx1, Indx2: integer;
    i: integer;
    S: string;
    res: integer;
    AtEnd : Boolean;
    SearchVal : string;
    N : TKey;

   procedure DoFullScan;
    var
     i : integer;
    begin
      for I := 0 to TBL.FKeyList.Count-1 do
      begin
        N := TKey(TBL.FKeyList[i]);
        if Masks.MatchesMask(N.Key.ToUpper, Value.ToUpper) then
        begin
          if N.Key <> c_EmptyIndexName then
          begin
            if IsLinked(FillZeroes(IntToStr(N.ID), 7), res, S) then
              Result := result + Res;
          end;
        end;
      end;
    end;


  begin { GetWilds }
    Result := 0;
    res := 0;
    Indx1 := Pos('*', Value);
    Indx2 := Pos('?', Value);
    if (Indx1 > 0) and (Indx2 > 0) then
      Indx1 := Min(Indx1, Indx2)
    else if Indx2 <> 0 then
      Indx1 := Indx2;

    if Indx1 = 1 then
    begin
      DoFullScan;
      Exit;
    end;
    AtEnd := (Indx1 = Length(Value));

    begin
      SearchVal := Copy(Value, 1, Indx1-1);
      Tbl.FindKey(0, SearchVal, Indx);
      for i := Indx to Tbl.FKeyList.Count - 1 do
      begin
        N := TKey(Tbl.FKeyList[i]);
        if Masks.MatchesMask(N.Key.ToUpper, Value.ToUpper) then
        begin
          if N.Key <> c_EmptyIndexName then
          begin
            if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
            begin
              Result := Result + res;
            end;
          end;
        end
        else if AtEnd then
          Break
        else
        begin
          S := Copy(N.Key, 1, Length(SearchVal));
          if AnsiComparetext(S, SearchVal) > 0 then
            Break;
        end;
      end;
    end;
  end;

  function GetLess: integer;
  var
    i: integer;
    S: string;
    res: integer;
    N : TKey;
  begin
    Result := 0;
    res := 0;

    Tbl.FindKey(0, Value, Indx);
    N := TKey(TBL.FKeyList[Indx]);
    if N.Key.ToUpper >= Value.ToUpper then
      Dec(Indx);
    if Indx =  Tbl.FKeyList.Count then
      Dec(Indx);
    for i := 0 to Indx do
    begin
      N := TKey(TBL.FKeyList[I]);
      if N.Key <> c_EmptyIndexName then
      begin
        if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
        begin
          Result := Result + res;
        end;
      end;
    end;
  end;

  function GetGreater: integer;
  var
    i: integer;
    S: string;
    res: integer;
    N : TKey;
  begin
    Result := 0;
    res := 0;

    Tbl.FindKey(0, Value, Indx);
    if Indx = Tbl.FKeyList.Count then
      Exit;
    N := TKey(Tbl.FKeyList[Indx]);
    if N.Key.ToUpper <= Value.ToUpper then
      Inc(Indx);
    if Indx = Tbl.FKeyList.Count then
      Exit;
    for i := Indx to Tbl.FKeyList.Count - 1 do
    begin
      N := TKey(TBL.FKeyList[I]);
      if N.Key <> c_EmptyIndexName then
       if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
        begin
          Result := Result + res;
        end;
    end;
  end;

  begin { Get keys }
    Result := 0;
    res := 0;
    if HasWildCards(Value) then
    begin
      if CO = '=' then
        Result := GetWilds
      else
        State := issScan;
      Exit;
    end;

    if (CO = '=') or (CO = '==') then
    begin
      P := Pos(c_IntervalDelimiter, Value);
      if P <> 0 then
      begin
        S := Copy(Value, 1, P - 1);
        s2 := Copy(Value, P + 1, Length(Value));
        Found1 := Tbl.FindKey(0, S, Indx);
        Found2 := Tbl.FindKey(0, s2, Indx2);
        if not Found1 then
        begin
           N := TKey(Tbl.FKeyList[Indx]);
           if N.Key < S then
             Inc(Indx);
           if Indx = Tbl.FKeyList.Count then
            Exit;
        end;
        if not Found2 then
        begin
           if Indx2 = Tbl.FKeyList.Count then
             Dec(Indx)
           else
           begin
            N := TKey(Tbl.FKeyList[Indx2]);
            if N.Key > S2 then
             Dec(Indx2);
           end;
        end;

        for i := Indx to Indx2 do
        begin
          if Indx2 < Tbl.FKeyList.Count then
          begin
            N := TKey(Tbl.FKeyList[i]);
            if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
            begin
              Result := Result + res;
            end;
          end;
        end;
      end
      else if Tbl.FindKey(0, Value, Indx) then
      begin
        N := TKey(Tbl.FKeyList[Indx]);
        if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
        begin
          Result := Result + res;
        end;
      end;
    end

    else if CO = '<>' then
    begin
      Result := GetGreater;
      Result := Result + GetLess;
    end
    else if CO = '>' then
    begin
      Result := GetGreater;
    end
    else if CO = '>=' then
    begin
      Tbl.FindKey(0, Value, Indx);
      for i := Indx to Tbl.FKeyList.Count - 1 do
      begin
        N := TKey(Tbl.FKeyList[I]);
        if N.Key <> c_EmptyIndexName then
        if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
        begin
          Result := Result + res;
        end;
      end;
    end
    else if CO = '<' then
    begin
       Result := GetLess;
    end
    else if CO = '<=' then
    begin
      Tbl.FindKey(0, Value, Indx);
      for i := 0 to Indx do
      begin
        N := TKey(Tbl.FKeyList[I]);
        if N.Key <> c_EmptyIndexName then
         if Indx < Tbl.FKeyList.Count then
          if IsLinked(FillZeroes(IntToStr(N.Id), 7), res, S) then
          begin
            Result := Result + res;
          end;
      end;
    end;
  end; { GetKeys }

  procedure JoinFields(List1, List2: TStringList);

  { Joins List1 with list2, returning the join in list1 }
  var
    i: integer;
    sl: TStringList;
  begin
    if List1.Count = 0 then
      Exit
    else if List2.Count = 0 then
    begin
      List1.Clear;
      Exit;
    end
    else
    begin
      sl := TStringList.Create;
      try
        for i := 0 to List1.Count - 1 do
        begin
          if List2.IndexOf(List1[i]) <> -1 then
            sl.Add(List1[i]);
        end;
        List1.Assign(sl);
      finally
        sl.Free;
      end;
    end;
  end;

  function GetRecords(TBL: TILTable; Interval: TInterval; List: TStringList;
    TotalRecords: integer): Boolean;

  { Collect the records found in Analyze }
  var
    i: integer;
    S: string;
    Go: Boolean;
    Limit: Word;
    M : TILTable;
    N : TKey;

  function CheckPath(ChildID: integer): Boolean;
  var
    i: integer;
    IP: Boolean;
  begin
    Result := True;
    if RootDataset = nil then
      Exit;
    IP := false;
    for i := 0 to RootDataset.Count - 1 do
    begin
      if InPath(RootDataset[i].RecordIndex, ChildID, True) then
      begin
        IP := True;
        Break;
      end;
    end;
    Result := IP;
  end;

  function GetRecID(LinkStr: string): string;

  var
    P: integer;
  begin
    P := Pos('|', LinkStr);
    P := Pos('|', LinkStr, P + 1);
    Delete(LinkStr, 1, P);
    Result := LinkStr;
  end;

  function ConvertToMaster(LinkStr: string): string;

  var
    LibIndx: integer;
    srec: string;
    Rec: TILDataRecord;
    RecIndx: integer;
    T: TILTable;

  begin
    Result := '';
    LibIndx := StrToInt(GetRecID(LinkStr));
    srec := Tbl.FSF_Data[LibIndx];
    Rec := Tbl.XMLToRecord(srec, True);
    try
      RecIndx := Rec.IndexOf(c_SysFieldNames[SYS_LinkID]);
      if not Rec.Deleted then
        Result := Rec[RecIndx][0];
    finally
      Rec.Free;
    end;

    if Result = '' then
      Exit;
    { Move upwards in hierarchy to reach the calling master }
    T := Tbl.MasterSource;
    while (T <> nil) and (T <> Ilib) do
    begin
      srec := T.FSF_Data[StrToInt(Result)];
      Rec := T.XMLToRecord(srec, True);
      try
        RecIndx := Rec.IndexOf(c_SysFieldNames[SYS_LinkID]);
        if not Rec.Deleted then
          Result := Rec[RecIndx][0]
        else
        begin
          Result := '';
          Break;
        end;
      finally
        Rec.Free;
      end;
      T := T.MasterSource;
    end;
  end;

  procedure ConvertToPathlinkMaster(LinkStr: string; L: TStringList);
  var
    LibIndx: integer;
    F : TILFieldDef;
    sl : TStringList;
    R : TILDataRecord;
    DS : TILDataset;
  begin
    LibIndx := StrToInt(GetRecID(LinkStr));
    { Find record in child table }
    R := Tbl.RecordsX[LibIndx];
    { Find Lookup definition field in child table }
    F := Tbl.GetLookupKeyField(Ilib);
    {Search in master table for the value of the field defined as lookupe key }
    sl := TStringList.Create;
    try
      if F.DataType = dtNumber then
        sl.Add('select * where ' + F.LookupField + ' = ' + R[F.FieldName].AsString)
      else
        sl.Add('select * where ' + F.LookupField + ' = ' + '''' + R[F.FieldName].AsString + '''');
      DS := Ilib.DBView.SearchPrivate(sl);
      try
        if DS.Count > 0 then
        begin
          S := IntToStr((DS[0].RecordIndex));
          S := FillZeroes(S, 7);
          L.Add(S);
        end;
        finally
          DS.Free;
        end;
      finally
        SL.Free;
        R.Free;
      end;
  end;

  procedure ConvertToPathlinkDetails(LinkStr: string; L: TStringList);
  var
    LibIndx: integer;
    F : TILFieldDef;
    sl : TStringList;
    R : TILDataRecord;
    DS : TILDataset;
    i : integer;
  begin
    { Get record index in master table }
    LibIndx := StrToInt(GetRecID(LinkStr));
    { Get lookup field in detail table table }
    F := Ilib.GetLookupKeyField(Tbl);
    { Get matching record in master table }
    R := Tbl.RecordsX[LibIndx];
    sl := TStringList.Create;
    try
      { Search for link field value in detail table }
      if F.DataType = dtNumber then
        sl.Add('select * where ' + F.FieldName + ' = ' + R[F.LookupField].AsString)
      else
        sl.Add('select * where ' + F.FieldName + ' = ' + '''' + R[F.LookupField].AsString + '''');
      DS := Ilib.DBView.SearchPrivate(sl);
      try
        for i := 0 to DS.Count -1 do
        begin
          S := IntToStr((DS[i].RecordIndex));
          S := FillZeroes(S, 7);
          L.Add(S);
        end;
        finally
          DS.Free;
        end;
      finally
        SL.Free;
        R.Free;
      end;
  end;


  procedure ConvertToDetails(MasterTbl: TILTable; LinkStr: string;
    L: TStringList);
  var
    LibIndx: integer;
    i, j: integer;
    TreeData: TTreeData;
    S : string;
  begin
    LibIndx := StrToInt(GetRecID(LinkStr));
    TreeData := Ilib.SearchDownTree(MasterTbl, LibIndx, Ilib);
    for i := 0 to High(TreeData) do
    begin
      if TreeData[i].Table = Ilib then
        for j := 0 to TreeData[i].Dataset.Count - 1 do
        begin
          if CheckPath(TreeData[i].Dataset[j].RecordIndex) then
          begin
            S := IntToStr(TreeData[i].Dataset[j].RecordIndex);
            S := FillZeroes(S, 7);
            L.Add(S);
            Inc(RCounter);
          end;
        end;
      TreeData[i].Dataset.Free;
    end;
  end;

  procedure ConvertToSysLinks(LinkStr: string; L: TStringList);

  var
    LibIndx: integer;
    DS: TILDataset;
    i: integer;
    DR: TILDataRecord;
    S : string;
  begin
    LibIndx := StrToInt(GetRecID(LinkStr));
    DR := Tbl.RecordsX[LibIndx];
    try
      LibIndx := DR['Sys_LinkID'].AsInteger;
    finally
      DR.Free;
    end;
    DS := Ilib.DBView.SearchPrivate('Select * where SYS_LinkID = ' + IntToStr(LibIndx));
    try
      for i := 0 to DS.Count - 1 do
      begin
        if CheckPath(DS[i].RecordIndex) then
        begin
          S := IntToStr((DS[i].RecordIndex));
          S := FillZeroes(S, 7);
          L.Add(S);
          Inc(RCounter);
        end;
      end;
    finally
      DS.Free;
    end;
  end;

  begin
    Result := True;
    Go := false;
    { If the collection process consumes too much time, consider
      reverting to table scan }
    if MaxPrio = ipHigh then
      Limit := 2000
    else if MaxPrio = ipIndefinite then
      Limit := 3000
    else
      Limit := 4000;

    for i := Interval.St to Interval.Nd do
    begin
      N := TKey(Tbl.FLinkList.Items[i]);
      S := N.Key;
      S := FIllZeroes(S, 7);
      if TMan.IsInLinkedPath(TBL, ILIb, M) then
      begin
         if M = Ilib then
            ConvertToPathLinkMaster(S, List)
         else
            ConvertToPathLinkDetails(S, List);
      end

      else if TBL = Ilib then
      { Query is run from the same table as the field. Simply add the record }
      begin
        S := GetRecID(S);
        if S <> '' then
        begin
          if CheckPath(StrToInt(S)) then
          begin
            S := FIllZeroes(S, 7);
            List.Add(S);
            Inc(RCounter);
          end;
        end;
      end
      else if Ilib.IsMaster and Tbl.IsDetail
      and (TMan.NodeOf(Ilib).Level < TMan.NodeOf(Tbl).Level) then
      { Query is run from a mastersource and the field is in a detail record.
        Convert detail link ids to master entry ids }
      begin
        S := ConvertToMaster(S);
        if S <> '' then
        begin
          if CheckPath(StrToInt(S)) then
          begin
            S := FIllZeroes(S, 7);
            List.Add(S);
            Inc(RCounter);
          end;
        end;
      end
      else if Ilib.IsDetail and Tbl.IsMaster
      and (TMan.NodeOf(Ilib).Level > TMan.NodeOf(Tbl).Level) then
      { Query is run from a detail source. Critearia is in master. Convert master record ids to detail entry ids }
      begin
        ConvertToDetails(TBL, S, List);
      end
      else if (TBL <> Ilib) and (Tbl.IsDetail and Ilib.IsDetail) then
      { Query is run from a detail source. Criteria is in another detail. Convert detail sys-link ids to detail entry ids }
      begin
        ConvertToSysLinks(S, List);
      end;
      if ((GetTickCount - TM) > Limit) and not Go then
      begin
        if (RCounter = 0) or (TotalRecords div RCounter > 1) then
        { If less than half of the records left, proceed. }
        begin
          Result := false;
          Break;
        end
        else
          Go := True;
      end;
    end;
  end;

  function IsIndexExcluded(LineNo, FieldPos: integer): Boolean;

  var
    i: integer;
    Indx: integer;
    Elements: TStringList;
    Prio: TIndexPriority;
    Cnt: integer;
    OrInd : integer;

    { An indexed field can be exluded if it has a lower priority than the
      max field priority of the line. Candidates can only be picked from a
      contigous row of ands at the end of the line. There might be better
      methods. However, this one wil affect the most commonly used search logics. }
  begin
    Result := not LineIndexes[LineNo, FieldPos].Indexed;
    if Result then
      Exit;

    { Check if there is a single field line AND logop with max prio that is not LineNo }

    OrInd := -1;
    { Start after the rightmost OR }
    for I := FExpression.Count-1 downto 0 do
    if LineIndexes[i, 0].LineLogOp = '/' then
    begin
       OrInd := i;
       Break;
    end;


    for I := OrInd +1 to FExpression.Count-1 do
    begin
      if LineIndexes[i, 0].Indexed and(FDecomps[i].Count = 2) then
      begin
          if LineIndexes[i, 0].Prio = ipHigh then
          begin
            if LineNo <> i then
            begin
              Result := True;
              Exit;
            end;
          end;
      end;
    end;

    Cnt := FDecomps[LineNo].Count;
    Elements := TStringList.Create;
    try
      Indx := -1;
      for i := Cnt - 1 downto 1 do
      { Loop back to track the ands at the end }
      begin
        if LineIndexes[LineNo, i - 1].Indexed then
        begin
          if LineIndexes[LineNo, i - 1].LogOp = '\' then
          begin
            Indx := i - 1;
          end
          else
            Break;
        end;
      end;
    finally
      Elements.Free;
    end;

    if Indx = -1 then
    begin
      Result := false;
      Exit;
    end;
    Prio := ipLow;
    for i := Indx to Cnt - 2 do
    begin
      if LineIndexes[LineNo, i].Indexed then
      begin
        if LineIndexes[LineNo, i].Prio > Prio then
          Prio := LineIndexes[LineNo, i].Prio;
      end;
    end;
    Result := (LineIndexes[LineNo, FieldPos].Prio < Prio);
  end;

  function Analyze: integer;

  var
    i, j, k: integer;
    slFields: TStringList;
    slFieldElements: TStringList;
    FldName: string;
    TblName: string;
    LogOp: string;
    Value: string;
    LineRCount: integer;
    TBL: TILTable;
    FldIndx: integer;
    LT: TILTable;
    DS: TILDataset;
    LTFname: string;
  begin
    Result := 0;
    slFieldElements := TStringList.Create;
    try
      for i := 0 to FExpression.Count - 1 do
      begin
        SetLength(Intervals, i + 1);
        slFields := FDecomps[i];

        LogOp := slFields[0];
        LineRCount := 0;

        for j := 1 to slFields.Count - 1 do
        begin
          slFieldElements.Clear;
          CreateListFromDelimiter('~', slFields[j], slFieldElements);
          { Contains 0: AndOr operator, 1: FieldName, 2: CO operator, 3: Value , 4: IsNot
            Test each field }
          if slFieldElements[2] = '==' then
            HasAndAlls := True;
          TBL := SplitTableField(slFieldElements[1], TblName, FldName);

          Value := slFieldElements[3];
          if slFieldElements[4] = '1' then { Set NOT to unequal }
            slFieldElements[2] := '<>';
          SetLength(Intervals[i], j);
          { Get the linklist intervals for each field in the line expression }
          if IsIndexExcluded(i, j - 1) then
          begin
            SetLength(Intervals[i][j - 1], Length(Intervals[i][j - 1]) + 1);
            Intervals[i][j - 1][Length(Intervals[i][j - 1]) - 1].St := 1;
            Intervals[i][j - 1][Length(Intervals[i][j - 1]) - 1].Nd := -1;
            State := issInclude;
            Continue;
          end;

          FldIndx := Tbl.Fields.IndexOf(FldName);
          if (Tbl.Fields.Items[FldIndx].LookupTable <> nil) and (Tbl.Fields.Items[FldIndx].LookupType = looTableRef)then
          begin
            LT := Tbl.Fields.Items[FldIndx].LookupTable;
            LTFname := LT.Fields.Items[LT.Fields.IndexOf(Tbl.Fields.Items[FldIndx].LookupField)].FieldName;
            DS := LT.DBView.SearchPrivate('Select * where ' + LTFname + ' = ' + '''' + Value + '''');
            try
              for k := 0 to DS.Count - 1 do
              begin
                LineRCount := LineRCount +
                  GetKeys(TBL, IntToStr(DS[k].RecordIndex), slFieldElements[0],
                  slFieldElements[2], FldName, Intervals[i][j - 1])
              end;
            finally
              DS.Free;
            end;

          end
          else
            { GetKeys searches for the key values and sets the linklist intervals }
            LineRCount := LineRCount + GetKeys(TBL, Value, slFieldElements[0],
              slFieldElements[2], FldName, Intervals[i][j - 1]);
          Result := Result + LineRCount;
        end;
      end;
    finally
      slFieldElements.Free;
    end;
  end;

  function ResolveIndexes: Boolean;

  var
    i, j: integer;
    Cnt, MaxCnt: integer;
    IsIndexed: Boolean;
    slFields: TStringList;
    slFieldElements: TStringList;
    F: TILFieldDef;
    FieldLogOp: string;
    LastOp: string;
    LineLogOp: string;
    LastIndexed: Boolean;

    function GetField(FieldName: string): TILFieldDef;

  var
    TBL: TILTable;
    Indx: integer;
    TblName, FldName: string;
  begin
    TBL := SplitTableField(FieldName, TblName, FldName);
    Indx := Tbl.Fields.IndexOf(FldName);
    Result := Tbl.Fields.Items[Indx];
  end;

  function Evaluate(FirstField: Boolean; Active: Boolean;
    LogOp: string): Boolean;
  begin
    Result := True;
    if LogOp = '/' then
    begin
      if not Active then
      begin
        State := issScan;
        { Unindexed ors cannot be resolved, Scan is necessary,
          if not modifies later
          Invalidate all previous indexes: }
      end
      else
      begin
        { An indexed or preceeded by an unindexed and must scan }
        if (LastOp = '\') and not LastIndexed then
          State := issScan;
      end;
    end
    else { And logop }
    begin
      if Active then
      begin
        if State = issScan then
          State := issInclude
          { The righmost and will always provide a maximum result set }
      end
      else
      begin
        if State = issComplete then
        { An unindexed and reduces a complete state to scan if it was precede by an or.
          If it was preceeded by an indexed and it reduces complete to include }
        begin
          if (LastOp = '/') or (FirstField and (i = 0)) then
          begin
            State := issScan;
          end
          else
            State := issInclude
        end
        else if State = issInclude then
        { An unindexed and keeps an include state if it was preceded by an indexed and,
          else it reverts to scan }
        begin
          if LastOp = '/' then
            State := issScan
        end
        else
          State := issScan;
      end;
    end;
  end;

  procedure ClearIndexes;

  var
    j: integer;
  begin
    for j := 0 to 29 do
      LineIndexes[i, j].Indexed := false;
  end;

  begin
    MaxPrio := ipLow;
    for i := 0 to 29 do
      for j := 0 to 29 do
        LineIndexes[i, j].Indexed := false;
    slFieldElements := TStringList.Create;

    try
      { Check if there are indexes that can provide the maximum result set }
      for i := 0 to FExpression.Count - 1 do
      begin
        State := issComplete;
        HasAndAlls := false;
        slFields := FDecomps[i];
        LineLogOp := slFields[0];
        FieldLogOp := '\';
        LastIndexed := false;
        LastOp := '\';

        for j := 1 to slFields.Count - 1 do
        { Evaluate each field in the expression line. The Evaluate procedure determines
          if a max result can be obtained. If not the expression line
          is excluded from the index search. This does not ncessarily means a full scan,
          succeceeding lines may modify the full scan to an include }
        begin
          slFieldElements.Clear;
          CreateListFromDelimiter('~', slFields[j], slFieldElements);
          { Contains AndOr operator, FieldName, CO operator, Value , IsNot }
          F := GetField(slFieldElements[1]);
          if j = 1 then
            FieldLogOp := LineLogOp
          else if j > 1 then
            FieldLogOp := slFieldElements[0];

          IsIndexed := F.Indexed;
          { The following conditions cannot be handled: }
          if (slFieldElements[2] = '<>') or
            ((F.DataType = dtDate) and HasWildCards(slFieldElements[3])) or
            ((F.DataType = dtText) and not IsWholeWord(slFieldElements[3]))
          { Index only works on single words }
          then
            IsIndexed := false;
          LastIndexed := IsIndexed;
          LastOp := FieldLogOp;
          Evaluate(j = 1, IsIndexed, FieldLogOp);
          if State = issScan then
            ClearIndexes
          else
          begin
            LineIndexes[i, j - 1].Indexed := IsIndexed;
            LineIndexes[i, j - 1].Prio := F.IndexPriority;
            LineIndexes[i, j - 1].LogOp := FieldLogOp;
            LineIndexes[i, j - 1].LineLogOp := LineLogOp;
          end;

        end;
      end;

      { Now, we have a map of the indexes to be applied on each line, so we can
        evaluate the wholde expression }
      MaxCnt := 0;
      Cnt := 0;
      IsIndexed := false;
      for i := 0 to FExpression.Count - 1 do
      begin
        slFields := FDecomps[i];
        for j := 1 to slFields.Count - 1 do
        begin
          if LineIndexes[i, j - 1].Indexed then
          begin
            Inc(Cnt);
            IsIndexed := True;
          end;
          Inc(MaxCnt);
        end;
        if not IsIndexed then
          State := issScan;
        { This is overruled if the last line is indexed and linelogop is and }
      end;
      if (State = issScan) and IsIndexed then
      begin
        if (LineIndexes[FExpression.Count - 1, 0].LineLogOp = '\') then
        begin
          Result := True;
          State := issInclude;
          Exit;
        end
        else
        begin
          Result := false;
          Exit;
        end;
      end;

      if Cnt = 0 then
        State := issScan
      else if Cnt < MaxCnt then
        State := issInclude
      else
        State := issComplete;
      Result := (State <> issScan);
    finally
      slFieldElements.Free;
    end;
  end;

  var
    i, j, k: integer;
    slFields, slFieldElements: TStringList;
    ErrCode: integer;
    FldName: string;
    LogOp: string;
    TBL: TILTable;
    TblName: string;
    FieldRecords: array of TStringList;
    Interval: TInterval;
    ExpressionRecords: array of TStringList;
    RCount: integer;
    FirstExpr: Boolean;
    IsMThreadBusy : Boolean;

  begin
    IsMThreadBusy := TMan.FMainThreadBusy;
    TMan.FMainThreadBusy := True;
    try

    while Tman.FSystemBusy do
      Sleep(50);
    Result := nil;
    if not Ilib.Connected then
      Exit;

    if not FPrepared then
    begin
      ErrCode := Prepare;
      if ErrCode <> 0 then
        HandleILException(ErrCode, '');
    end;

    Result := TStringList.Create;
    Result.Sorted := True;
    Result.Duplicates := DupIgnore;
    if FExpression.Count = 0 then
    begin
      State := issScan;
      Exit;
    end;
    if not ResolveIndexes then
    begin
      Exit;
    end;

    slFieldElements := TStringList.Create;
    try
      { Search the KeyList and detect the intervals in the linklist }
      RCount := Analyze;
      if RCount = 0 then
      begin
        if State = issInclude then
          for i := 0 to FExpression.Count - 1 do
          begin
            if Pos(c_SysFieldNames[SYS_LinkID], FExpression[i]) <> 0 then
            begin
              State := issComplete;
              Break;
            end;

          end;

        Exit;
      end;
      TM := GetTickCount;
      RCounter := 0;

      for i := 0 to FExpression.Count - 1 do
      begin
        slFields := FDecomps[i];
        LogOp := slFields[0];
        for j := 1 to slFields.Count - 1 do
        begin
          slFieldElements.Clear;
          CreateListFromDelimiter('~', slFields[j], slFieldElements);
          TBL := SplitTableField(slFieldElements[1], TblName, FldName);
          SetLength(FieldRecords, j);
          FieldRecords[j - 1] := TStringList.Create;
          FieldRecords[j - 1].Sorted := True;
          FieldRecords[j - 1].Duplicates := DupIgnore;
          for k := 0 to Length(Intervals[i][j - 1]) - 1 do
          begin
            Interval := Intervals[i][j - 1, k];
            if Interval.Nd = -1 then
            begin
              FieldRecords[j - 1].Add('-1');
              Continue;
            end;
            { If there is more than one interval, bind together with or }
            if not GetRecords(TBL, Interval, FieldRecords[j - 1], RCount) then
            { Collects the records for one field inside an expression line }
            begin
              { Takes too long, revert to table scan }
              Result.Clear;
              State := issScan;
              Exit;
            end;
          end;
        end;
        { Concatenate the field records into one expression record }
        SetLength(ExpressionRecords, i + 1);
        ExpressionRecords[i] := TStringList.Create;
        ExpressionRecords[i].Sorted := True;
        ExpressionRecords[i].Duplicates := DupIgnore;
        FirstExpr := false;
        for j := 1 to slFields.Count - 1 do
        begin
          slFieldElements.Clear;
          CreateListFromDelimiter('~', slFields[j], slFieldElements);
          LogOp := slFieldElements[0];
          if (FieldRecords[j - 1].Count > 0) and (FieldRecords[j - 1][0] = '-1')
          then
          begin
            ExpressionRecords[i].Add('-1');
            Continue;
          end;

          if not FirstExpr then
          begin
            ExpressionRecords[i].Assign(FieldRecords[j - 1]);
            FirstExpr := True;
          end
          else
          begin
            if LogOp = '\' then
            begin
              JoinFields(ExpressionRecords[i], FieldRecords[j - 1]);
            end
            else
            begin
              ExpressionRecords[i].AddStrings(FieldRecords[j - 1]);
            end;
          end;
        end;
        SetLength(Intervals[i], 0);
        for j := 0 to High(FieldRecords) do
          FieldRecords[j].Free;
        SetLength(FieldRecords, 0);
      end;

      { Last step concatenate the expression records to one single result record }
      FirstExpr := false;
      for i := 0 to FExpression.Count - 1 do
      begin
        slFields := FDecomps[i];
        LogOp := slFields[0];

        if (ExpressionRecords[i].Count > 0) and (ExpressionRecords[i][0] = '-1')
        then
          Continue;
        if not FirstExpr then
        begin
          Result.Assign(ExpressionRecords[i]);
          FirstExpr := True;
        end
        else
        begin
          if LogOp = '\' then
            JoinFields(Result, ExpressionRecords[i])
          else
          begin
            Result.AddStrings(ExpressionRecords[i]);
          end;
        end;
      end;

      if HasAndAlls and (State = issComplete) then
        State := issInclude;

    finally
      slFieldElements.Free;
      for i := 0 to High(ExpressionRecords) do
        ExpressionRecords[i].Free;
      for i := 0 to High(FieldRecords) do
        FieldRecords[i].Free;
    end;
    i := Result.IndexOf('-1');
    while i > -1 do
    begin
      Result.Delete(i);
      i := Result.IndexOf('-1');
    end;
    finally
      if not IsMThreadBusy then
        Tman.FMainThreadBusy := false;
    end;
  end;

  function TSearcher.InPath(ParentID, ChildID: integer;
    IncludeRoot: Boolean): Boolean;

  var
    Rec: TILDataRecord;
    LinkID: integer;
  begin
    Result := false;
    if IncludeRoot and (ParentID = ChildID) then
    begin
      Result := True;
      Exit;
    end;
    Rec := Ilib.RecordsX[ChildID];
    while not Rec['SYS_LinkID'].IsEmpty do
    begin
      LinkID := Rec['SYS_LinkID'].AsInteger;
      FreeAndNil(Rec);
      if LinkID = ParentID then
      begin
        Result := True;
        Break;
      end;
      Rec := Ilib.Records[LinkID];
    end;
    if Assigned(Rec) then
      Rec.Free;
  end;

  function TSearcher.Join(var OutData : TILDataset): integer;
  type
    TILPath = array[0..25] of TILTable;
  var
    PathCount : integer;
    ILPath : TILPath;

    function PathIndexOf(TableName, FieldName: string): integer;
    var
      i, j : integer;
      Tbl : TILTable;
    begin
      Result := -1;
      for I := 0 to FSelectfields.AlienCount-1 do
      begin
        if SameText(FieldName, FSelectfields.Aliens[i].FieldName)
        and SameText(TableName, FSelectfields.Aliens[i].TableName) then
        begin
          Tbl := TMan.TableOf(FSelectfields.Aliens[i].TableName);
          for j := 0 to PathCount do
            if ILPath[j] = Tbl then
            begin
              Result := j;
              Exit;
            end;
        end;
      end;

    end;

    function HasField(TableName : string) : Boolean;
    var
      i : integer;
    begin
       Result := false;
       for I := 0 to FSelectFields.AlienCount-1 do
       begin
         if SameText(TableName, FSelectFields.Aliens[i].TableName) then
         begin
             Result := True;
             Break;
         end;
       end;
    end;

    function GetILPath(var Pth : TILPath) : integer;
    var
      N, N2 : TTableNode;
      i : integer;
      L : TILTable;
      Cnt : integer;
      TmpPth : TILPath;
    begin
      N := TMan.NodeOf(ILIb);
      Result := 1;
      Pth[0] := Ilib;
      L := ILib;
      for I := 0 to TMan.TableNodeCount-1 do
      begin
        N2 := TMan.NodeOf(TMan.TreeTables[i]);
        if N2.Level >= N.Level then
        if Tman.IsInSearchPath(L, TMan.TreeTables[i]) then
        begin
         if HasField(TMan.TreeTables[i].TableName) then
         begin
           L := TMan.TreeTables[i];
           Pth[Result] := L;
           inc(Result);

         end;
        end;
      end;

      { Check missing links}
      Cnt := 1;
      TmpPth[0] := Pth[0];
      for I := 1 to Result-1 do
      begin
         if i < Result then
         begin
          if not TMan.IsLinkedChild(TIlDataTable(Pth[i-1]), TilDataTable(Pth[i]))  then
          begin
           if Pth[i].ParentTable <> Pth[i-1] then
           begin
              TmpPth[Cnt] := Pth[i].ParentTable;
              inc(Cnt);
           end;
          end;
          TmpPth[Cnt] := Pth[i];
          inc(Cnt);
         end;
      end;
      Pth := TmpPth;
      Result := Cnt;
    end;

    procedure AddToList(DR : TILDataRecord; First : Boolean; PathIndx : integer);
    var
      j : integer;
      Indx : integer;
      FName, FName1, FName2, FName3: string;
      Rec : TILDataRecord;
      OrigRecIndex : integer;
      Tbl : TILTable;
    begin
         OrigRecIndex := OutData[OutData.Count-1].RecordIndex;
         if not First then
         begin
           Rec := TILDataRecord.Create(ILib, True);
           Rec.FRecordIndex := DR.RecordIndex;
         end
         else
         begin
           Rec := OutData[OutData.Count-1];
           Rec.FRecordIndex := OutData[OutData.Count-1].RecordIndex;
         end;

         for J := 0 to FSelectFields.AlienCount-1 do
         begin
           FName1 := FSelectFields.Aliens[J].FieldName;
           FName2 := FSelectFields.Aliens[J].TableName + '.' + FSelectFields.Aliens[J].FieldName;
           Indx := DR.IndexOf(FName1);
           if Indx = -1 then
             Indx := DR.IndexOf(FName2);
           if (Indx = -1)
           or not SameText(DR.Ilib.TableName, FSelectFields.Aliens[J].TableName) then
            Continue;
           begin
             if Rec.IndexOf(FName1) = -1 then
               FName := FName2
             else
               FName := FName1;
             if DR.IndexOf(FName1) = -1 then
               FName3 := FName2
             else
               FName3 := FName1;
             Rec[FName].FValues.Assign(DR[FName3].FValues);
             Rec[FName].FOldValues.Assign(DR[FName3].FOldValues);
           end;
         end;
         if not First then
         begin
           { Fill blanks }
           for J := 0 to Rec.Count-1 do
           begin
             if Rec[J].IsEmpty then
             begin
               if PathIndx = PathCount then
                 Tbl := ILPath[PathCount-1]
               else
                 Tbl := ILPath[PathIndx];

               Indx := OutData[OutData.Count-1].IndexOf(Rec[j].FieldName);
               if ((Indx <> -1) and (Rec.RecordIndex = OutData[OutData.Count-1].RecordIndex))
               or (PathIndexOf(Tbl.TableName, Rec[j].FieldName) = -1) { First table in path }
               then
               begin
                 Rec[j].FValues.Assign(OutData[OutData.Count-1][Indx].FValues);
                 Rec[j].FOldValues.Assign(OutData[OutData.Count-1][Indx].FOldValues);
                 Rec[j].FSuppressOutput := Ilib.HasViewer and (roSuppressRepeatingValues in Ilib.Viewer.ReportOptions);
               end;
             end;
           end;
           OutData[OutData.Count-1].FRecordIndex := OrigRecIndex;
           OutData.Add(Rec);
         end;
    end;

    procedure DoJoin(SourceRecord : TILDataRecord; FirstChild : Boolean; PathIndx : integer);
    var
      AilTable : TILTable;
      DS : TILDataset;
      i : integer;
      PthIndex : integer;
      M : TILTable;
    begin
       PthIndex := PathIndx;
       AddToList(SourceRecord, FirstChild, PthIndex);
       Inc(Result);
       if PthIndex < PathCount then
        begin
            DS := nil;
            AilTable := ILPath[PthIndex];
            if TMan.IsInLinkedPath(SourceRecord.Ilib, AilTable, M) then
            begin
              for I := 0 to AilTable.Fields.Count-1 do
                if (AilTable.Fields.Items[i].LookupType = looTableKey)
                and (AilTable.Fields.Items[i].LookupTable = SourceRecord.ILIB) then
                begin
                   if AilTable.Fields.Items[i].DataType = dtNumber then
                    DS := AilTable.DBView.SearchPrivate('select * where ' + AilTable.Fields.Items[i].FieldName +
                     ' = ' + IntToStr(SourceRecord[AilTable.Fields.Items[i].LookupField].AsInteger))
                   else
                    DS := AilTable.DBView.SearchPrivate('select * where ' + AilTable.Fields.Items[i].FieldName +
                    ' = ' + '''' + SourceRecord[AilTable.Fields.Items[i].LookupField].AsString + '''');
                   Break;
                end;
            end
            else
             DS := AilTable.DBView.SearchPrivate('select * where sys_linkid = ' +
              IntToStr(SourceRecord.RecordIndex));
            if DS <> nil then
            try
              for I := 0 to DS.Count-1 do
                DoJoin(DS[i], (I = 0), PthIndex + 1);
            finally
              DS.Free;
            end;
        end;
    end;

  begin
     Result := 0;
     PathCount := GetILPath(ILPath);
     OutData.FIsReadOnly := True;
     OutData.FIsJoined := True;
     DoJoin(OutData[OutData.Count-1], True, 1);
  end;

  function TSearcher.Search(OutDataset: TILDataset; AMaxCount: integer;
    SkipImages: Boolean = false): TILDataset;

  var
    SearchRec: TILDataRecord;
    AggCounter: integer;
    AvgCounter: integer;
    Fld: TILField;
    ErrCode: integer;
    RootDataset: TILDataset;
    i: integer;
    Indx : integer;
    IsMThreadBusy : Boolean;

    procedure AddMatch(Rec: TILDataRecord);

  var
    i: integer;
  begin
    for i := 0 to Rec.Count - 1 do
    begin
      if Assigned(Rec[i].FOldValues) and (Rec[i].FOldValues <> Rec[i].FValues) then
        Rec[i].FOldValues.Free;
      Rec[i].FOldValues := Rec[i].FValues;
    end;
    Rec.FOldState := Rec.FState;
    Result.Add(Rec);
    if FSelectFields.AlienCount > 0 then
     Join(Result);
  end;

  function GetSearchRec(Indx: integer; IsScan : Boolean): TILDataRecord;
  var
    S: string;
    i : integer;
    SelIndx: integer;
    Val: string;
    SelField: TSelectField;
    F : TStringList;
    sl : TStrings;
    Fld : TILField;
  begin
    if ILib is TILMemTable then
    begin
      Result := ILib.DBWork.CreateDataRecord;
      Result.Assign(TilMemTable(Ilib).FTableRecords[Indx]);
      for I := 0 to Ilib.Fields.Count-1 do
      begin
        if (ILib.Fields.Items[i].FieldType = ftCalculated)
        and (ILib.Fields.Items[i].FieldId = -1) then
        begin
         Fld := TILField.Create(Ilib, ILib.Fields.Items[i].FieldName, ILib.Fields);
         Result.FFields.Insert(i, Fld);
        end;
      end;
    end
    else
    begin
      if IsScan then
       S := Ilib.FSF_Data.AbsItems[Indx]
      else
       S := Ilib.FSF_Data[Indx];
      if FPrivate then
        Result := Ilib.XMLToRecord(S, True, FFieldSource)
      else
        Result := Ilib.XMLToRecord(S, True);
    end;
    if Result.RecordIndex = -1 then
     Result.FRecordIndex := Indx;
    F := TStringList.Create;
    try
    for i := 0 to Result.Count - 1 do
    begin
      if Result[i].FieldDef.FieldType = ftCalculated then
      begin
        if TMan.DLLAccess then
          Continue;
        if Assigned(Ilib.DataHandler.FOnCalcField) and (Result[i].FieldDef.FieldID <> -1) then
        begin
          ILib.DataHandler.FOnCalcField(Ilib, Result, Result[i]);
          Result[i].FIsCalculated := True;
          Continue;
        end;

        SelIndx := FSelectFields.IndexOfCalcField (Result[i].FieldDef.FFieldName);
        if SelIndx <> -1 then
        begin
          SelField := FSelectFields.Items[SelIndx];
          F.Assign(Selfield.FFormula);
          { Keep the original formula to prevent hangup }
          Val := SelField.CalcValue(Result);
          SelField.FFormula.Assign(F);
          { Restore formula }
          Result[i].AsString := Val;
        end;
      end
      else if (Result[i].FieldDef.LookupType = looParentField)
      and not (FPrivate and (Ilib.MasterLinkCount > 0)) then
      {Dont call GetLookupParentValue when private is true and ilib is a link table. May lead
      to stack overflow when more than one parent field in the table. GLPV calls search
      privatly which may lead to and endless GLPV loop}
      begin
        sl := Ilib.GetLookupParentValue(Result[i].FieldDef.LookupTable, Result[i].FieldDef.LookupField, Result);
        if sl <> nil then
        begin
         Result[i].FValues.Assign(sl);
         sl.Free;
        end;
      end;
    end;
    finally
      F.Free;
    end;
  end;

  function CanAdd: Boolean;
  begin
    Result := True;
    if Ilib.FDatasetSearch then
    begin
      if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnFetchRecord) then
      begin
        Ilib.DataHandler.FOnFetchRecord(Self, SearchRec, Result);
      end;
    end;
  end;

  function HasCountAll: Boolean;

  var
    i: integer;
  begin
    Result := false;
    for i := 0 to FSelectFields.Count - 1 do
    begin
      if FSelectFields.Items[i].FAggregateFunc = agCount then
      begin
        if FSelectFields.Items[i].FAggregateField = '*' then
        begin
          Result := True;
          Break;
        end;
      end;
    end;
  end;

  function GetAggField(Func: TAggFunc): TILField;

  var
    i: integer;
  begin
    Result := nil;
    for i := 0 to FSelectFields.Count - 1 do
    begin
      if FSelectFields.Items[i].FAggregateFunc = Func then
      begin
        if Func = agCount then
        begin
          if FSelectFields.Items[i].FAggregateField = '*' then
          begin
            Result := nil;
            Break;
          end;
        end;
        Result := SearchRec[FSelectFields.Items[i].FAggregateField];
        Break;
      end;
    end;
  end;

  function GetFuncField(Func: TAggFunc): TILField;

  var
    i: integer;
  begin
    Result := nil;
    for i := 0 to FSelectFields.Count - 1 do
    begin
      if FSelectFields.Items[i].FAggregateFunc = Func then
      begin
        Result := SearchRec[FSelectFields.Items[i].FFieldName];
        Break;
      end;
    end;
  end;

  function GetLiteralField(FieldName: string): TILField;

  var
    i: integer;
  begin
    Result := nil;
    for i := 0 to FSelectFields.Count - 1 do
    begin
      if SameText(FSelectFields.Items[i].FieldName, FieldName) then
      begin
        Result := SearchRec[FSelectFields.Items[i].FFieldName];
        Break;
      end;
    end;
  end;

  procedure Aggregate;

  var
    Field: TILField;
    i: integer;
    N: Extended;
  begin
    if agCount in FSelectFields.FAggFuncs then
    begin
      Field := GetAggField(agCount);
      if Field = nil then
      begin
        FSelectFields.FAggVals[agCount] := FSelectFields.FAggVals[agCount] + 1;
        Inc(AggCounter);
      end
      else
      begin
        if (Field.FValues.Count > 0) then
        begin
          FSelectFields.FAggVals[agCount] := FSelectFields.FAggVals
            [agCount] + 1;
        end;
      end;
      Inc(AggCounter);
    end;

    if agMin in FSelectFields.FAggFuncs then
    begin
      Field := GetAggField(agMin);
      for i := 0 to Field.FValues.Count - 1 do
      begin
        N := StrToFloat(Field.FValues[i], Translator.FFormatSettings);
        if N < FSelectFields.FAggVals[agMin] then
          FSelectFields.FAggVals[agMin] := N;
      end;
      Inc(AggCounter, Field.Count);
    end;
    if agMax in FSelectFields.FAggFuncs then
    begin
      Field := GetAggField(agMax);
      for i := 0 to Field.FValues.Count - 1 do
      begin
        N := StrToFloat(Field.FValues[i], Translator.FFormatSettings);
        if N > FSelectFields.FAggVals[agMax] then
          FSelectFields.FAggVals[agMax] := N;
      end;
      Inc(AggCounter, Field.Count);
    end;
    if agSum in FSelectFields.FAggFuncs then
    begin
      Field := GetAggField(agSum);
      for i := 0 to Field.FValues.Count - 1 do
      begin
        N := StrToFloat(Field.FValues[i], Translator.FFormatSettings);
        FSelectFields.FAggVals[agSum] := FSelectFields.FAggVals[agSum] + N;
      end;
      Inc(AggCounter, Field.Count);
    end;
    if agAvg in FSelectFields.FAggFuncs then
    begin
      Field := GetAggField(agAvg);
      for i := 0 to Field.FValues.Count - 1 do
      begin
        N := StrToFloat(Field.FValues[i], Translator.FFormatSettings);
        FSelectFields.FAggVals[agAvg] := FSelectFields.FAggVals[agAvg] + N;
      end;
      Inc(AvgCounter, Field.Count);
    end;

  end;

  function CheckPath(ChildID: integer): Boolean;

  var
    i: integer;
    IP: Boolean;
  begin
    Result := True;
    if RootDataset = nil then
      Exit;
    IP := false;
    for i := 0 to RootDataset.Count - 1 do
    begin
      if InPath(RootDataset[i].RecordIndex, ChildID, True) then
      begin
        IP := True;
        Break;
      end;
    end;
    Result := IP;
  end;

  procedure SearchLib;

  var
    i: integer;
    OldResume: integer;
    FetchCount: integer;
    Indx: integer;
    Loops : integer;
    Cnt : integer;
  begin
    OldResume := FResumeIndex;
    FetchCount := 0;
    Loops := Ilib.RecordCount - FResumeIndex;
    TMan.FProgressTicker := GetTickCount;
    Cnt := Ilib.RecordCount;
    for i := FResumeIndex to Cnt - 1 do
    begin
      if not FPrivate then
       TMan.Progress(Ilib, iloSearch, Loops, i);
      try
        Ilib.FSearching := True;
        SearchRec := GetSearchRec(i, True);
        Ilib.FSearching := false;
        if SearchRec = nil then
          Continue;
        if SearchRec.Deleted then
        begin
          SearchRec.Free;
          Continue;
        end
        else if not CheckPath(SearchRec.RecordIndex) then
        begin
          SearchRec.Free;
          Continue;
        end;

        if (FIndexSearchState = issExclude) and
          FFetchRecords.Find(FillZeroes(IntToStr(SearchRec.RecordIndex), 7),
          Indx) then
        begin
          if FSelectFields.FAggFuncs <> [] then
          begin
            Aggregate;
            SearchRec.Free;
          end
          else if CanAdd then
          begin
            AddMatch(SearchRec);
            Inc(FetchCount);
          end
          else
            SearchRec.Free;
        end
        else if Execute(SearchRec) then
        begin
          if FSelectFields.FAggFuncs <> [] then
          begin
            Aggregate;
            SearchRec.Free;
          end
          else if CanAdd then
          begin
            AddMatch(SearchRec);
            Inc(FetchCount);
          end
          else
            SearchRec.Free;
        end
        else
          SearchRec.Free;
        if MaxCount > 0 then { DatasetMaxcount overrules the local MaxCount }
        begin
          if Result.Count = MaxCount then
          begin
            FResumeIndex := 0;
            Break;
          end;
        end
        else if (Self = Ilib.GetSearcher(ILib)) and (FetchCount = Ilib.DatasetMaxCount)
          and (Ilib.DatasetMaxCount > 0) and
          (FSelectFields.FAggFuncs = []) then
        begin
          FResumeSearchType := 0;
          FResumeIndex := i + 1;
          Break;
        end;
      except
        Result.Free;
        raise;
      end;
    end;

    FFetchCount := FFetchCount + FetchCount;

    if OldResume = FResumeIndex then
      FResumeIndex := 0;
  end;

  procedure FetchLib;

  var
    i: integer;
    OldResume: integer;
    FetchCount: integer;
    Loops : integer;
  begin
    OldResume := FResumeIndex;
    FetchCount := 0;
    Loops := FFetchRecords.Count - FResumeIndex;
    TMan.FProgressTicker := GetTickCount;
    for i := FResumeIndex to FFetchRecords.Count - 1 do
    begin
      if not FPrivate then
       TMan.Progress(Ilib, iloSearch, Loops, i);
      SearchRec := GetSearchRec(StrToInt(FFetchRecords[i]), False);
      if SearchRec.Deleted then
      begin
        SearchRec.Free;
        Continue;
      end;
      if FIndexSearchState = issInclude then
      begin
        if Execute(SearchRec) then
        begin
          if FSelectFields.FAggFuncs <> [] then
          begin
            Aggregate;
            SearchRec.Free;
          end
          else if CanAdd then
          begin
            AddMatch(SearchRec);
            Inc(FetchCount);
          end
          else
            SearchRec.Free;
        end
        else
          SearchRec.Free;
      end
      else
      begin
        if FSelectFields.FAggFuncs <> [] then
        begin
          Aggregate;
          SearchRec.Free;
        end
        else if CanAdd then
        begin
          AddMatch(SearchRec);
          Inc(FetchCount);
        end
        else
          SearchRec.Free;
      end;
      if MaxCount > 0 then
      begin
        if Result.Count = MaxCount then
        begin
          FResumeIndex := 0;
          Break;
        end;
      end
      else if (Self = Ilib.GetSearcher(Ilib)) and (FetchCount = Ilib.DatasetMaxCount)
        and (Ilib.DatasetMaxCount > 0) and
        (FSelectFields.FAggFuncs = []) then
      begin
        FResumeSearchType := 1;
        FResumeIndex := i + 1;
        Break;
      end
    end;
    if not FPrivate then
       TMan.Progress(Ilib, iloFinished, 0, 0);
    FFetchCount := FFetchCount + FetchCount;
    if OldResume = FResumeIndex then
      FResumeIndex := 0;
  end;

  procedure CalcLiterals;

  var
    i: integer;
  begin
    SearchRec := TILDataRecord.Create(Ilib, True);
    for i := 0 to FSelectFields.Count - 1 do
    begin
      Fld := GetLiteralField(FSelectFields.Items[i].FFieldName);
      Fld.AsString := FSelectFields.Items[i].CalcValue(SearchRec);
    end;
    Result.Add(SearchRec);
  end;

  procedure CalcCountAll;

  var
    Fld: TILField;
  begin
    SearchRec := TILDataRecord.Create(Ilib, True);
    Fld := GetFuncField(agCount);
    Fld.AsNumber := Ilib.RecordCount - Ilib.DeletedCount;
    Result.Add(SearchRec);
  end;

  procedure MergeDatasets;

  var
    i, j, ID: integer;
    Linked: Boolean;
    R: TILDataRecord;
    IP: Boolean;
  begin
    { Remove roots without childs }
    i := 0;
    while i < RootDataset.Count do
    begin
      IP := false;
      for j := 0 to Result.Count - 1 do
      begin
        if (RootDataset[i].RecordIndex = Result[j].RecordIndex) then
        begin
          Result[j][c_SysFieldNames[SYS_LinkID]].Clear;
          Break;
        end;
        if InPath(RootDataset[i].RecordIndex, Result[j].RecordIndex, True) then
        begin
          IP := True;
          Break;
        end;
      end;
      if not IP then
        RootDataset.Delete(i)
      else
        Inc(i);
    end;

    { Add the root records }
    RootDataset.OwnsRecords := false;
    for i := 0 to RootDataset.Count - 1 do
    begin
      RootDataset[i].FOwnerDataset := Result;
      RootDataset[i][c_SysFieldNames[SYS_LinkID]].Clear;
      Result.FDataset.Insert(0, RootDataset[i]);
    end;
    { Check for missing links in childs }
    i := RootDataset.Count;
    while i < Result.Count do
    begin
      if Result[i][c_SysFieldNames[SYS_LinkID]].IsEmpty then
      begin
        Inc(i);
        Continue;
      end;
      ID := Result[i][c_SysFieldNames[SYS_LinkID]].AsInteger;
      Linked := false;
      for j := 0 to Result.Count - 1 do
      begin
        if Result[j].RecordIndex = ID then
        begin
          Linked := True;
          Break;
        end;
      end;
      if not Linked then
      begin
        R := Ilib.Records[ID];
        R.FOwnerDataset := Result;
        Result.FDataset.Insert(i, R);
        i := RootDataset.Count;
      end
      else
        Inc(i);
    end;
  end;

  procedure Finish;
  begin
    if ((FColumnSource.FSortFields.Count > 0) and (Result.Count > 0))
     then
      begin
         if FPrivate then
           Result.Sort(FColumnSource.FSortFields, FColumnSource.SortOrdering);
      end;
      if FPrivate then
      begin
       Result.FColumnSource := FColumnSource;
       Result.FPrivateFields := FFieldSource;
       if Assigned(FTVParamsSource) then
         FreeAndNil(FTVParamsSource);
      end
      else
      begin
        Result.FPrivateFields := nil;
        TMan.Progress(Ilib, iloFinished, 0, 0);
      end;
  end;

  begin
    IsMThreadBusy := TMan.FMainThreadBusy;
    TMan.FMainThreadBusy := True;
    try
    if not Ilib.Connected then
      Exit;
    if not FPrepared then
    begin
      ErrCode := Prepare;
      if ErrCode <> 0 then
      begin
        if FPrivate then
          FColumnSource.Free;
        if Assigned(FTVParamsSource) and FPrivate then
          FreeAndNil(FTVParamsSource);
        HandleILException(ErrCode, '');
      end;
    end;

    RootDataset := nil;

    if (FSelectFields.FAggFuncs = []) and
    FTreeViewing and Assigned(FRootCondition) and (FRootCondition.Count > 0)  then {sql}
    begin
      RootDataset := Ilib.DBView.SearchPrivate(FRootCondition);
      FreeAndNil(FRootCondition);
      if (FExpression.Count = 0) and not FTVParamsSource.Recursive then
      begin
        if RootDataset.Count > 0 then
          for i := 0 to RootDataset.Count - 1 do
          begin
            RootDataset[i].FOwnerDataset := RootDataset;
            RootDataset[i][c_SysFieldNames[SYS_LinkID]].Clear;
          end;
        RootDataset.OwnsRecords := True;
        Result := RootDataset;
        Finish;
        Exit;
      end;
    end
    else if (FSelectFields.FAggFuncs = []) and FTreeViewing and (FTVParamsSource.RootCondition.Count > 0) then
    begin
      RootDataset := Ilib.DBView.SearchPrivate(FTVParamsSource.RootCondition);
      if (FTVParamsSource.WhereCondition.Count = 0) and not FTVParamsSource.Recursive
      then
      begin
        if RootDataset.Count > 0 then
          for i := 0 to RootDataset.Count - 1 do
          begin
            RootDataset[i].FOwnerDataset := RootDataset;
            RootDataset[i][c_SysFieldNames[SYS_LinkID]].Clear;
          end;
        RootDataset.OwnsRecords := True;
        Result := RootDataset;
        Finish;
        Exit;
      end;
      FExpression.Free;
      FExpression := RecordToExpression(FTVParamsSource.WhereCondition);
      Prepare;
    end;

    AggCounter := 0;
    AvgCounter := 0;
    if FResumeIndex <> 0 then
    begin
      if OutDataset = nil then
      begin
        FResumeIndex := 0;
        FFetchCount := 0;
      end
      else
      begin
        OutDataset.BeginUpdate;
        try
          Result := OutDataset;
          if FResumeSearchType = 0 then
            SearchLib
          else
            FetchLib;
          Finish;
          Exit;
        finally
          OutDataset.EndUpdate;
        end;
      end;
    end
    else
      FFetchCount := 0;

    if Assigned(FFetchRecords) then
    begin
      FFetchRecords.Free;
      FFetchRecords := nil;
    end;
    if OutDataset = nil then
      Result := TILDataset.Create(Ilib)
    else
      Result := OutDataset;
    if FPrivate then
      Result.FPrivateFields := FFieldSource
    else
     Result.FPrivateFields := nil;

    if (FSelectFields.FAggFuncs = []) and FSelectFields.IsLiterals then
    begin
      CalcLiterals;
      Finish;
      Exit;
    end;
    if (FSelectFields.FAggFuncs = [agCount]) and HasCountAll and
      (FSections[1].Count = 0) then
    begin
      CalcCountAll;
      Finish;
      Exit;
    end;

    MaxCount := AMaxCount;
    Ilib.FTickStart := GetTickCount;
    if Ilib is TilMemTable then
      FIndexSearchState := issScan
    else
      FFetchRecords := GetFromIndexes(FIndexSearchState, RootDataset);
    if (FSelectFields.FAggFuncs <> []) and (FExpression.Count = 0) then
    begin
      FIndexSearchState := issScan;
    end;
    try
      FillChar(FSelectFields.FAggVals, 0, SizeOf(FSelectFields.FAggVals));
      FSelectFields.FAggVals[agMin] := MaxInt;
      FSelectFields.FAggVals[agMax] := -MaxInt;
      if (FIndexSearchState = issComplete) and (FFetchRecords.Count = 0) then
        Exit;

      if not FPrivate then
        Result.BeginUpdate;
      try
        if FIndexSearchState = issScan then
        begin
          SearchLib;
        end
        else
        begin
          FetchLib;
        end;
      finally
        if not FPrivate then
          Result.EndUpdate;
      end;

      if FSelectFields.FAggFuncs <> [] then
      begin
        SearchRec := TILDataRecord.Create(Ilib, True);
        if agCount in FSelectFields.FAggFuncs then
        begin
          Fld := GetFuncField(agCount);
          Fld.AsNumber := FSelectFields.FAggVals[agCount];
        end;
        if agMax in FSelectFields.FAggFuncs then
        begin
          Fld := GetFuncField(agMax);
          Fld.AsNumber := FSelectFields.FAggVals[agMax];
        end;
        if agMin in FSelectFields.FAggFuncs then
        begin
          Fld := GetFuncField(agMin);
          Fld.AsNumber := FSelectFields.FAggVals[agMin];
        end;
        if agSum in FSelectFields.FAggFuncs then
        begin
          Fld := GetFuncField(agSum);
          Fld.AsNumber := FSelectFields.FAggVals[agSum];
        end;
        if agAvg in FSelectFields.FAggFuncs then
        begin
          Fld := GetFuncField(agAvg);
          Fld.AsNumber := FSelectFields.FAggVals[agAvg] / AvgCounter;
        end;
        Result.Add(SearchRec);
      end;
      if RootDataset <> nil then
      begin
        MergeDatasets;
       { Connect records }
       for I := 0 to Result.Count-1 do
       begin
        if not Result[i][c_SysFieldNames[SYS_LinkID]].IsEmpty then
        begin
         Indx := Result.IndexOf(Result[i][c_SysFieldNames[SYS_LinkID]].AsInteger);
         if Indx <> -1 then
         begin
           Result[i].TVParentRecord := Result[Indx];
         end;
        end;
       end;
      end;
    finally
      Ilib.FTickEnd := GetTickCount;
      if (RootDataset <> nil) and (RootDataset <> Result) then
        RootDataset.Free;
      Finish;
    end;
    finally
      if not IsMThreadBusy then
        TMan.FMainThreadBusy := false;
    end;
  end;

  function TSearcher.Search: TILDataset;
  begin
    Result := Search(nil, 0, True);
  end;

  function TSearcher.Resume(Dataset: TILDataset): Boolean;
  begin
    Result := false;
    if FResumeIndex = 0 then
      Exit;
    Search(Dataset, 0, True);
    Result := True;
  end;

  function TSearcher.CanResume: Boolean;
  begin
    Result := (FResumeIndex > 0);
  end;

  destructor TSearcher.Destroy;
  begin
    inherited Destroy;
    if Assigned(FSelectFields) then
      FSelectFields.Free;
    if Assigned(FRootCondition) then
      FRootCondition.Free;
  end;

  function TSearcher.GetRemainingRecords: integer;

  var
    Avg: Extended;
  begin
    if FResumeIndex = 0 then
      Result := 0
    else if FResumeSearchType = 0 then
    begin
      Avg := FFetchCount / FResumeIndex;
      Result := -Round((Ilib.RecordCount - FFetchCount) * Avg);
    end
    else
    begin
      if FIndexSearchState = issComplete then
        Result := FFetchRecords.Count - FFetchCount
      else
      begin
        Avg := FFetchCount / FResumeIndex;
        Result := -Round((FFetchRecords.Count - FFetchCount) * Avg);
      end;
    end;
  end;

  function TSearcher.GetSections: integer;

  var
    S, s2: string;
    P, PS, PW, PO, POSIB, PST, PSTC: integer;
    i: integer;
    WhereFound: Boolean;
    OrderFound: Boolean;

    procedure Normalize(Expr: TStringList);

  var
    i: integer;
    sl: TStringList;
    S: string;
    P: integer;
    Cnt: integer;
  begin
    sl := TStringList.Create;
    try
      for i := 0 to Expr.Count - 1 do
      begin
        Cnt := 0;
        P := 1;
        if i = 0 then
        begin
          sl.Add(Expr[0]);
          Continue;
        end;
        S := Expr[i];
        P := PosRestrict('AND ', S, False, P);
        if P = 1 then
          Inc(Cnt);
        P := 1;
        P := PosRestrict(' AND ', S, False, P);
        if P <> 0 then
          Inc(Cnt);
        P := 1;
        P := PosRestrict('OR ', S, False, P);
        if P = 1 then
          Inc(Cnt);
        P := 1;
        P := PosRestrict(' OR ', S, False, P);
        if P <> 0 then
          Inc(Cnt);
        if Cnt = 1 then
          sl[sl.Count - 1] := sl[sl.Count - 1] + ' ' + S
        else
          sl.Add(S);
      end;
      Expr.Assign(sl);
    finally
      sl.Free;
    end;
  end;

  function PosKeyWord(KeyWord, S: string): integer;
  begin
    Result := PosRestrict(KeyWord, S, False);
    if Result = 0 then
      Exit;
    if Length(S) = Length(KeyWord) then
      Exit;
    if (S[Result + Length(KeyWord)] <> ' ') and
      (S[Result + Length(KeyWord)] <> #$D)
      then
      Result := 0;
    if (Result <> 0) and (Result <> 1) then
    begin
      if (S[Result-1] <> #$A)
      and(S[Result-1] <> ' ') then
        Result := 0;
    end;
  end;

  function PosOP(Op : string; S: string; Start : integer): integer;
  begin
    Result := PosRestrict(Op, S, False, Start);
    if Result = 0 then
      Exit;
    if Length(S) = Length(Op) then
      Exit;
    if (S[Result + Length(Op)] <> ' ') and
      (S[Result + Length(Op)] <> #$D)
      and (S[Result + Length(Op)] <> '(')
      then
      Result := 0;
    if (Result <> 0) and (Result <> 1) then
    begin
      if (S[Result-1] <> #$A)
      and(S[Result-1] <> ' ') then
        Result := 0;
    end;
  end;

  procedure SetRootCondition;

  var
    PstCond: string;
    P: integer;
  begin
    if PST <> 0 then
      P := PST
    else
      P := PSTC;
    if PST <> 0 then
      P := P + Length('START WITH')
    else
      P := P + Length('START WITH RECURSIVE');
    if PW <> 0 then
    begin
      PstCond := Copy(S, P, PW - P);
      Delete(S, P, PW - P);
    end
    else if PO <> 0 then
    begin
      PstCond := Copy(S, P, PO - P);
      Delete(S, P, PO - P);
    end
    else
    begin
      PstCond := Copy(S, P, Length(S));
      Delete(S, P, Length(S));
    end;

    if PST <> 0 then
      S := StringReplace(S, 'START WITH', '', [rfIgnoreCase])
    else
      S := StringReplace(S, 'START WITH RECURSIVE', '', [rfIgnoreCase]);
    PW := PosKeyWord('WHERE', S);
    PO := PosKeyWord('ORDER BY', S);
    FExpression.Text := S;

    if not Assigned(FRootCondition) then
      FRootCondition := TStringList.Create;
    FRootCondition.Text := PstCond;
  end;


  procedure SetTVParams;
  begin

    FTVParamsSource.Recursive := (PSTC <> 0);
    if (POSIB <> 0) then
      FTVParamsSource.SortingMethod := tsBySiblings
    else
      FTVParamsSource.SortingMethod := tsByRecords;
    if PST <> 0 then
      FTVParamsSource.SearchCtrlMode := scmStartWith
    else
      FTVParamsSource.SearchCtrlMode := scmWhere;
  end;

  function ExpandOps(var Expr : string; var Err : integer) : Boolean;
  var
   P, P1, P2, P3 : integer;
   S : string;
   sl : TStringList;
   FName : string;
   i : integer;
   j : integer;
   Op : string;
   Indx : integer;
  begin
   Err := 0;
   Result := false;
   for j := 1 to 3 do
   begin
    if J = 1 then
     Op := 'IN'
    else if J = 2 then
     Op := 'CONTAINS'
    else
     Op := 'EQUALS';
    P := PosOp(Op, Expr, 1);
    while P <> 0 do
    begin
      Result := True;
      { Get the fieldname }
      P2 := P-1;
      while (P2 > 0) and (CharInSet(Expr[P2], [' '])) do
      dec(P2);
      { Now at end of field name }
      P3 := P2;
      while (P3 > 0) and (CharInSet(Expr[P3], ['.', '_', 'a'..'z', 'A'..'Z', '0'..'9'])) do
        dec(P3);
      Inc(P3);
      { Now at start of field name }
      FName := Copy(Expr, P3, P2-P3+1);
      Indx := Ilib.Fields.IndexOf(FName);
      if Indx = -1 then
      begin
        Comment1 := Fname;
        Err := E_InvalidFieldName;
        Exit;
      end;
      if ((Op = 'EQUALS') or (Op = 'CONTAINS'))
      and (Ilib.Fields.Items[Indx].MaxValues = 1) then
      begin
        Comment1 := Op;
        Err := E_IllegalMVFOperator;
        Exit;
      end;

      P1 := Pos('(', Expr, P);
      P2 := Pos(')', Expr, P);
      S := Copy(Expr, P1, P2-P1+1);
      if S = '' then
      begin
        Err := E_MissingParenthesis;
        Exit;
      end;
      System.Delete(Expr, P, P2-P+1);
      if Op = 'EQUALS' then
      begin
       Insert(' == ', Expr, P);
       Inc(P, 4);
      end
      else
      begin
       Insert(' = ', Expr, P);
       Inc(P, 3);
      end;
      System.Delete(S, 1, 1);
      System.Delete(S, Length(S), 1);
      sl := TStringList.Create;
      try
      CreateListFromDelimiter(',', S, sl);
      for i := 0 to sl.Count-1 do
      begin
         if I = 0 then
         begin
           Insert(sl[i], Expr, P);
           inc(P, Length(Sl[i]))
         end
         else
         begin
           if Op = 'IN' then
           begin
            S := ' OR ' + FName + ' = ' + sl[i];
            Insert(S, Expr, P);
            inc(P, Length(S));
           end
           else if Op = 'CONTAINS' then
           begin
            S := ' AND ' + FName + ' = ' + sl[i];
            Insert(S, Expr, P);
            inc(P, Length(S));
           end
           else
           begin
            S := ' AND ' + FName + ' == ' + sl[i];
            Insert(S, Expr, P);
            inc(P, Length(S));
           end
         end;
      end;
      finally
        sl.Free;
      end;
      P := PosOp(Op, Expr, P);
    end;
   end;
  end;

  begin
    for i := 0 to 2 do
      FSections[i].Clear;
    for I := 0 to FExpression.Count-1 do
    begin
      S := FExpression[i];
      S := SetESCCodes(S);
      FExpression[i] := S;
    end;
    S := FExpression.Text;
    S := TrimBlanks(S, false);
    PS := PosKeyWord('SELECT', S);
    if PS <> 1 then
      PS := 0;
    PST := 0;
    PSTC := PosKeyWord('START WITH RECURSIVE', S);
    if (PSTC <> 0) and not(Ilib is TILTreeViewTable) then
    begin
      Result := E_StartWith;
      Exit;
    end;
    if (PSTC = 0) then
    begin
      PST := PosKeyWord('START WITH', S);
      if (PST <> 0) and not(Ilib is TILTreeViewTable) then
      begin
        Result := E_StartWith;
        Exit;
      end;
    end;

    if (PST <> 0) and (PS = 0) then
    begin
      Result := E_MissingSelect;
      Exit;
    end;

    PW := PosKeyWord('WHERE', S);
    PO := PosKeyWord('ORDER BY', S);
    POSIB := PosKeyWord('ORDER BY SIBLINGS', S);
    if (POSIB <> 0) and not(Ilib is TILTreeViewTable) then
    begin
      Result := E_OrderBySiblings;
      Exit;
    end;
    WhereFound := (PW <> 0);
    OrderFound := (PO <> 0);

    if ((PST <> 0) or (PSTC <> 0)) and (Ilib.TVTable <> nil) then
      { Assign FRootCondion. Relevant with sql statements }
      SetRootCondition;
    if ((PW <> 0) or (PS <> 0))  and (Ilib.TVTable <> nil) then
     if Assigned(FTVParamsSource) then
      SetTVParams;

    if PS = 0 then
    begin
      FExpression.Insert(0, 'SELECT');
    end;
    if (PW = 0) and (PS = 0) then
    begin
      FExpression.Insert(1, 'WHERE');
    end
    else if (PW = 0) and (PS <> 0) then { Select without where }
    begin
      if PO = 0 then
        FExpression.Add('WHERE')
      else
      begin
        for i := 0 to FExpression.Count - 1 do
        begin
          P := PosKeyWord('ORDER BY', FExpression[i]);
          if P <> 0 then
          begin
            S := FExpression[i];
            Insert('WHERE ', S, P);
            FExpression[i] := S;
            Break;
          end;
        end;
      end;
    end
    else if (PS = 0) and (PW <> 0) then
    begin
      Result := E_MissingSelect;
      { Where without select, this is considered an error }
      Exit;
    end;

    if PO = 0 then
      FExpression.Add('ORDER BY');
    S := FExpression.Text;

    PW := PosKeyWord('WHERE', S);
    s2 := Copy(S, 1, PW - 1);
    Delete(s2, 1, Length('SELECT'));
    s2 := TrimBlanks(s2, false);
    FSections[0].Text := s2;

    Delete(S, 1, PW - 1);
    S := TrimBlanks(S);
    PO := PosKeyWord('ORDER BY SIBLINGS', S);
    if PO = 0 then
      PO := PosKeyWord('ORDER BY', S);

    s2 := Copy(S, 1, PO - 1);
    Delete(s2, 1, Length('WHERE'));
    s2 := TrimBlanks(s2, false);
    FSections[1].Text := s2;

    Delete(S, 1, PO - 1);
    if POSIB <> 0 then
      Delete(S, 1, Length('ORDER BY SIBLINGS'))
    else
      Delete(S, 1, Length('ORDER BY'));
    S := TrimBlanks(S, false);
    FSections[2].Text := S;

    TrimList(FSections[0], false);
    TrimList(FSections[1], false);
    TrimList(FSections[2], false);
    if (FSections[1].Count = 0) and WhereFound then
    begin
      Result := E_MissingExpression;
      Exit;
    end;
    if (FSections[2].Count = 0) and OrderFound then
    begin
      Result := E_MissingExpression;
      Exit;
    end;

    Normalize(FSections[1]);
    { Expand IN operators }
    S := FSections[1].Text;
    if ExpandOps(S, Result) then
      FSections[1].Text := S;
    if Result <> 0 then
      Exit;

    FExpression.Assign(FSections[1]);


    if Assigned(FSelectFields) then
      FreeAndNil(FSelectFields);
    try
      FSelectFields := TSelectFields.Create(Self, FSections[0], FFieldSource);
      Result := FSelectFields.FError;
    except
      if Assigned(FSelectFields) then
        FSelectFields := nil;
      Result := E_SyntaxError;
    end;
  end;

  function TSearcher.Execute(ADataRecord: TILDataRecord): Boolean;

  type
    { TLineHit = record
      Op : string;
      Hit : Boolean;
      end;
    }
    THit = record
      Op: string;
      Hit: Boolean;
      RecValueCount: integer;
    end;

  const
    c_LineOp = 0;
    c_FieldName = 1;
    c_Co = 2;
    c_Value = 3;
    c_IsNot = 4;

  var
    TblName: string;
    FldName: string;
    TBL: TILTable;

    Hits: array [0 .. 29, 1 .. 30] of THit;

    function EvaluateField(AField, ACo, AValue: string; IsNot: Boolean)
      : Boolean;

  var
    Indx: integer;
    i: integer;
    Fld: TILField;
    Hit: Boolean;
    S: string;
    P: integer;
    AValue2: string;
    sl: TStringList;
    TextSearch, ResSearch: Boolean;
    Fnd : Boolean;
    Dt : TDateTime;

  begin
    Indx := ADataRecord.IndexOf(AField);
    Fld := ADataRecord[Indx];
    if (ACo = '<>') then
      Result := True
    else
      Result := false;
    if ACo = '==' then
    begin
      ACo := '=';
    end;
    if TMan.IsValueNull(AValue) or (AValue = '') then
    begin
      if IsNot or (ACo = '<>') then
        Result := (Fld.Count <> 0)
      else
        Result := (Fld.Count = 0);
      Exit;
    end
    else if (Fld.Count = 0) and (Fld.FieldDef.FieldType <> ftStoredEvaluation) then
    begin
      Result := false;
      Exit;
    end;

    Hit := false;
    TextSearch := false;
    ResSearch := false;

    if Fld.FieldDef.FieldType = ftStoredEvaluation then
    begin
      if Tman.DLLAccess then
      begin
        Result := false;
        Exit;
      end;
      Fnd := False;
      if Assigned(Ilib.DataHandler.FOnEvaluateField) then
      begin
         Ilib.DataHandler.FOnEvaluateField(Ilib, ADatarecord, Fld.FieldDef.FieldName, AValue, Fnd);
      end;

      if not Fnd then
      begin
        Result := false;
        Exit;
      end
      else
      begin
        if Fld.Count = 0 then
          Fld.FValues.Add(AValue);
        Fld.FValues[0] := AValue;
      end;

    end;

    if (Fld.FieldDef.DataType = dtLongText) then
    begin
       Result := false;
       if Assigned(Ilib.DataHandler.FOnSearchLongText) then
        Ilib.DataHandler.FOnSearchLongText(Ilib, Fld, AValue, Result);
       Exit;
       {S := Fld.AsText;
       sl2 := TStringList.Create;
       sl2.Text := S;
       sl := GetWords(sl2);
       sl2.Free;
       TextSearch := True;}
    end
    else if (Fld.FieldDef.DataType = dtText) and (Fld.FieldDef.Indexed) and
      IsWholeWord(AValue) then
    begin
      sl := GetWords(Fld.FValues);
      TextSearch := True;
    end
    else if (Fld.FieldDef.DataType = dtResource) then
    begin
      sl := Ilib.GetResNames(Fld.FValues);
      ResSearch := True;
    end
    else
      sl := Fld.FValues;
    try
      for i := 0 to sl.Count - 1 do
      begin
        if (Fld.FieldDef.DataType in [dtNumber, dtDate]) and
          not HasWildCards(AValue) then
          begin
          S := Ilib.SetDateNumberFormat(sl[i], Fld.FieldDef.DataType,
            Fld.FieldDef.Decimals);
          end
        else
          S := sl[i];
        if Fld.FieldDef.LookupTable <> nil then
        begin
          P := Pos(' | ', S);
          if P <> 0 then
           Delete(S, P, Length(S));
        end;
        if ACo = '=' then
        begin
          P := Pos(c_IntervalDelimiter, AValue);
          if P <> 0 then
          begin
            AValue2 := Copy(AValue, P + 1, Length(AValue));
            AValue := Copy(AValue, 1, P - 1);
            Result := (AnsiCompareText(S, AValue) >= 0) and
              (AnsiCompareText(S, AValue2) <= 0);
          end
          else if Ilib.IsStringType(Fld.FieldDef.DataType) or
            ((Fld.FieldDef.DataType = dtDate) and HasWildCards(AValue)) then
          begin
            if Fld.FieldDef.DataType = dtDate then
            begin
              if ILib.DateSearchFormat = dsfDate then
              begin
                DT := StrToDateTime(S, Translator.FFormatSettings);
                S := DateToStr(DT, Translator.FFormatSettings);
              end;
            end;
            Result := Masks.MatchesMask(S.ToUpper, AValue.ToUpper);
          end
          else
            Result := (AnsiCompareText(S, AValue) = 0);
        end
        else if ACo = '>' then
        begin
          Result := (S > AValue);
        end
        else if ACo = '<' then
        begin
          Result := (S < AValue);
        end
        else if ACo = '<=' then
        begin
          Result := (S < AValue) or
           (AnsiCompareText(S, AValue) = 0)
        end
        else if ACo = '>=' then
        begin
          Result := (S > AValue) or
           (AnsiCompareText(S, AValue) = 0);
        end
        else if ACo = '<>' then
        begin
          Hit := Masks.MatchesMask(S.ToUpper, AValue.ToUpper);
          if Hit then
          begin
            Result := false;
            Break;
          end;
        end
        else
          Result := (AnsiCompareText(S, AValue) >= 0);
        if Result and not((ACo = '<>') and not Hit) then
          Break;
      end;
    finally
      if TextSearch or ResSearch then
        sl.Free;
    end;
    if Result and IsNot then
      Result := false
    else if not Result and IsNot then
      Result := True;
  end;

  procedure SetHit(LineNo, Index: integer; res: Boolean; LogOp: string;
    ValCount: integer);
  begin
    Hits[LineNo, Index].Hit := res;
    Hits[LineNo, Index].Op := LogOp;
    Hits[LineNo, Index].RecValueCount := ValCount;
  end;

  procedure CheckAndAll(LineNo: integer);

  var
    N: integer;
    i: integer;
    Cnt: integer;
  begin
    Cnt := 0;
    N := Hits[LineNo, 1].RecValueCount;
    for i := 1 to 30 do
    begin
      if Hits[LineNo, i].RecValueCount = N then
        Inc(Cnt)
      else
        Break;
    end;
    if Cnt <> N then
    begin
      for i := 1 to Cnt do
        Hits[LineNo, i].Hit := false;
    end;
  end;

  function EvaluateAll: Boolean;

  type
    TLHits = array [0 .. 29] of Boolean;

  var
    i, j: integer;
    Lhit: Boolean;
    LHits: TLHits;
  begin
    Lhit := false;
    for i := 0 to FExpression.Count - 1 do
    begin
      j := 1;
      while Hits[i, j].Op <> '' do
      begin
        if j = 1 then
        begin
          Lhit := Hits[i, j].Hit;
          Inc(j);
          Continue;
        end;
        if Hits[i, j].Op = '\' then
          Lhit := Lhit and Hits[i, j].Hit
        else
          Lhit := Lhit or Hits[i, j].Hit;
        Inc(j);
      end;
      LHits[i] := Lhit;
    end;
    Result := True;
    for i := 0 to FExpression.Count - 1 do
    begin
      if i = 0 then
      begin
        Result := LHits[0];
        Continue;
      end;
      if FDecomps[i][c_LineOp] = '\' then
        Result := Result and LHits[i]
      else
        Result := Result or LHits[i];
    end;
  end;

  procedure InitHits;

  var
    i, j: integer;
  begin
    for i := 0 to 29 do
      for j := 1 to 30 do
      begin
        Hits[i, j].Hit := false;
        Hits[i, j].Op := '';
        Hits[i, j].RecValueCount := 0;

      end;
  end;

  function UncomposeElement(Elements: TStringList): TStringList;

  var
    S: string;
    sdt: string;
    Indx: integer;
    FldDef: TILFieldDef;
    DT: TDateTime;
    fmt: TFormatSettings;
    P : integer;
    M : TILTable;
    i : integer;
  begin
    Result := TStringList.Create;
    Indx := Tbl.Fields.IndexOf(FldName);
    FldDef := Tbl.Fields.Items[Indx];
    fmt := TFormatSettings.Create;
    S := Elements[c_value];
    P := Pos(c_IntervalDelimiter, S);
    if P <> 0 then
    begin
     Delete(S, P, 1);
     Insert('..', S, P);
     Elements[c_value] := S;
    end;

    if S <> '' then
    begin
     if FldDef.DataType = dtDate then
     begin
      sdt := Copy(Elements[c_Value], 1, 4);
      sdt := sdt + '.' + Copy(Elements[c_Value], 5, 2);
      sdt := sdt + '.' + Copy(Elements[c_Value], 7, 2);
      fmt.ShortDateFormat := 'yyyy.mm.dd';
      fmt.LongTimeFormat := 'hh:nn:ss';
      fmt.TimeSeparator := ':';
      fmt.DateSeparator := '.';
      if Tbl.DateSearchFormat = dsfDate then
      begin
        DT := StrToDate(sdt, fmt);
        S := DateToStr(DT, Translator.FFormatSettings);
        S := '''' + S + '''';
      end
      else
      begin
        sdt := sdt + ' ' + Copy(Elements[c_Value], 9, 2);
        sdt := sdt + ':' + Copy(Elements[c_Value], 11, 2);
        sdt := sdt + ':' + Copy(Elements[c_Value], 13, 2);
        DT := StrToDateTime(sdt, fmt);
        S := DateTimeToStr(DT, fmt);
        S := '''' + S + '''';
      end;
     end
     else if Ilib.IsStringType(FldDef.DataType) then
      S := '''' + Elements[c_Value] + ''''
     else
      S := Elements[c_Value];
    end;
    if S = '' then
      S := TMan.ReportNull;
    S := Elements[c_FieldName] + Elements[c_Co] + S;
    Result.Add(S);
    if TMan.IsInLinkedPath(Ilib, TBL, M) then
    begin
       if M = TBL then
       for I := 0 to ADataRecord.Count-1 do
       begin
         if (ADataRecord[i].FieldDef.LookupTable = TBL)
         and (ADataRecord[i].FieldDef.LookupType = looTableKey) then
         begin
            if Tbl.IsStringType(ADataRecord[i].FieldDef.Datatype) then
              Result.Add('\' + Tbl.TableName + '.' + ADataRecord[i].FieldDef.LookupField +
               '=' + '''' + ADataRecord[i].AsString + '''')
            else
              Result.Add('\' + Tbl.TableName + '.' + ADataRecord[i].FieldDef.LookupField +  '=' + ADataRecord[i].AsString);
            Break;
         end;
       end
       else
       begin
         FldDef := Tbl.GetLookupKeyField(Ilib);
         if Tbl.IsStringType(FldDef.Datatype) then
           Result.Add('\' + Tbl.TableName + '.'  + FldDef.FieldName +  '=' + '''' +
             ADataRecord[FldDef.LookupField].AsString + '''')
         else
           Result.Add('\' + Tbl.TableName + '.'  + FldDef.FieldName +  '=' +
             ADataRecord[FldDef.LookupField].AsString)
       end;
    end
    else
    if TMan.IsChild(Ilib, TBL) then
      Result.Add('\SYS_LinkID=' + IntToStr(ADataRecord.RecordIndex));
  end;

  function CheckMaster(MasterTbl: TILTable; Elements: TStringList): Boolean;

  var
    DR: TILDataRecord;
    DR2: TILDataRecord;
    MS: TILTable;
    ID: integer;
  begin
    DR2 := ADataRecord;
    MS := Ilib.MasterSource;
    DR := MS.RecordsX[ADataRecord['SYS_LinkId'].AsInteger];
    try
      while MS <> MasterTbl do
      begin
        ID := DR['SYS_LinkId'].AsInteger;
        DR.Free;
        MS := MS.MasterSource;
        DR := MS.RecordsX[ID];
      end;

      ADataRecord := DR;
      Result := EvaluateField(FldName, Elements[c_Co], Elements[c_Value],
        (Elements[c_IsNot] = '1'));
    finally
      ADataRecord := DR2;
      DR.Free;
    end;
  end;

  var
    i, j: integer;
    slFields, slFieldElements, sl: TStringList;
    res: Boolean;
    LineOp: string;
    ErrCode: integer;
    Indx: integer;
    DR: TILDataRecord;
    M : TILTable;
    RIndx : integer;
  begin

    if not FPrepared then
    begin
      ErrCode := Prepare;
      if ErrCode <> 0 then
        HandleILException(ErrCode, '');
    end;
    if FExpression.Count = 0 then
    begin
      Result := True;
      Exit;
    end;
    InitHits;

    slFieldElements := TStringList.Create;
    try
      for i := 0 to FExpression.Count - 1 do
      begin
        begin
          slFields := FDecomps[i];

          for j := 1 to slFields.Count - 1 do
          begin
            slFieldElements.Clear;
            CreateListFromDelimiter('~', slFields[j], slFieldElements);
            { Contains AndOr operator, FieldName, CO operator, Value , IsNot
              Test each field }
            LineOp := slFieldElements[c_LineOp];
            if slFieldElements.Count = 4 then { No value }
              slFieldElements.Insert(3, TMan.ReportNull);
            if (j = 0) and (slFields.Count > 1) then
            begin
              { The second field element should always have the same
                log op as the first field element }
              sl := TStringList.Create;
              try
                CreateListFromDelimiter('~', slFields[j + 1], sl);
                slFieldElements[c_LineOp] := sl[0];
              finally
                sl.Free;
              end;
            end;
            TBL := SplitTableField(slFieldElements[c_FieldName],
              TblName, FldName);
            if TBL <> Ilib then
            begin
              if TMan.IsInLinkedPath(TBL, ILib, M) then
              begin
                 sl := UncomposeElement(slFieldElements);
                 try
                 if M = TBL then
                 begin
                   DR := Ilib.SearchUpLink(Sl, Tbl, ADataRecord);
                   try
                     Res := (DR <> nil);
                   finally
                     if DR <> nil then
                       DR.Free;
                   end;
                 end
                 else
                 begin
                    res := Tbl.Exists(sl, RIndx)
                 end;
                 finally
                   sl.Free;
                 end;
              end
              else if Ilib.IsMaster or (Ilib.IsDetail and Tbl.IsDetail) then
              { Criteria is defined in a detail record and the originator is master or another detail }
              begin
                sl := UncomposeElement(slFieldElements);
                try
                  if TMan.IsRelated(Ilib, TBL) then
                    { Tbl is child }
                    res := Tbl.Exists(sl, RIndx)
                  else
                  begin
                    { Tbl is Master }
                    DR := Ilib.SearchUpTree(sl, TBL, ADataRecord);
                    res := (DR <> nil);
                    if DR <> nil then
                      DR.Free;
                  end;
                finally
                  sl.Free;
                end;
              end
              else { Originator is a detail and criteria is in the master }
                res := CheckMaster(TBL, slFieldElements);
            end
            else
              res := EvaluateField(FldName, slFieldElements[c_Co],
                slFieldElements[c_Value], (slFieldElements[c_IsNot] = '1'));
            Indx := ADataRecord.IndexOf(FldName);
            if slFieldElements[c_Co] = '==' then
              SetHit(i, j, res, LineOp, ADataRecord[Indx].FValues.Count)
            else
              SetHit(i, j, res, LineOp, -1)
          end;
        end;
        if slFieldElements.Count > 0 then
         if slFieldElements[c_Co] = '==' then
          CheckAndAll(i);
      end;
    finally
      slFieldElements.Free;
    end;
    Result := EvaluateAll;
  end;

  function TSearcher.Prepare: integer;

  var
    slFields: TStringList;
    Ind: integer;

    procedure FreeDecomps;

  var
    i: integer;
  begin
    i := 0;
    while Assigned(FDecomps[i]) do
    begin
      FDecomps[i].Free;
      FDecomps[i] := nil;
      Inc(i);
    end;
  end;

  function CreateSelectFields: integer;

  var
    i: integer;
    sl: TStringList;
    S: string;
    P: integer;
    Asc: Boolean;
    Tbl : TILTable;
    Indx : integer;

    procedure SetDefaults(SectionType: integer);
    { 0 = select, 1,= order, 2 = both }

  var
    sl: TStringList;
    Indx: integer;
    N: integer;
    i: integer;
  begin
    if FPrivate then
      Exit;
    sl := TStringList.Create;
    try
      if SectionType in [0, 2] then
      begin
        Tman.FCfgFile.ReadSection(Ilib.TableName + '_ColumnSequence', sl);
        if sl.Count = 0 then
          Ilib.Columns.AssignSource
        else
          for i := 0 to sl.Count - 1 do
          begin
            Indx := Ilib.Fields.IndexOf(sl[i]);
            N := Tman.FCfgFile.ReadInteger(Ilib.TableName + '_ColumnSequence', sl[i],
              Ilib.Fields.Items[Indx].ColumnWidth);
            FColumnSource.Add(sl[i]);
            FColumnSource.Items[i].FColumnWidth := N;
          end;
      end;

      if SectionType in [1, 2] then
      begin
        sl.Clear;
        Tman.FCfgFile.ReadSection(Ilib.TableName + '_ColumnSort', sl);
        for i := 0 to sl.Count - 1 do
        begin
          N := Tman.FCfgFile.ReadInteger(Ilib.TableName + '_ColumnSort', sl[i], 0);
          FColumnSource.SortFields.Add(sl[i]);
          FColumnSource.SortOrdering := TSortOrdering(N);
        end;
      end;
    finally
      sl.Free;
    end;
  end;

  begin
    Result := 0;
    if FColumnSource = Ilib.Columns then
      Ilib.Fields.ClearCalcFields;

    if FSelectFields.Count > 0 then
      FColumnSource.Clear;

    if (FSections[0].Count = 0) and (FSections[2].Count = 0) then
      SetDefaults(2)
    else if (FSections[0].Count = 0) then
      SetDefaults(0)
    else if (FSections[2].Count = 0) then
      SetDefaults(1);

    for i := 0 to FSelectFields.Count - 1 do
    begin
      Tbl := nil;
      if FSelectFields.Items[i].FFieldType in [stCalc, stFunction, stAlien] then
      begin
         if FSelectFields.Items[i].FFieldType = stAlien then
         begin
           Tbl := TMan.TableOf(FSelectFields.Items[i].FTableName);
           Indx := Tbl.Fields.IndexOf(FSelectFields.Items[i].FFieldName);
           if FPrivate then
             FSelectFields.Items[i].FFieldDef :=
             FFieldSource.AddAlienField(Tbl, Tbl.Fields.Items[Indx])
           else
            FSelectFields.Items[i].FFieldDef :=
             Ilib.Fields.AddAlienField(Tbl, Tbl.Fields.Items[Indx]);
         end
         else if FSelectFields.Items[i].FFieldType = stCalc then
         begin
           if FPrivate then
             FSelectFields.Items[i].FFieldDef :=
              FFieldSource.AddCalcField(FSelectFields.Items[i].FieldName)
           else
             FSelectFields.Items[i].FFieldDef :=
              Ilib.Fields.AddCalcField(FSelectFields.Items[i].FieldName);
         end;
        FSelectFields.Items[i].FFieldDef.FReportField := True;
        if FSelectFields.Items[i].FFieldType = stFunction then
        begin
          FSelectFields.Items[i].FFieldDef :=
           FFieldSource.AddCalcField(FSelectFields.Items[i].FieldName);

          FSelectFields.Items[i].FFieldDef.FDataType := dtNumber;
          if FSelectFields.Items[i].FAggregateFunc = agCount then
           FSelectFields.Items[i].FFieldDef.FDecimals := 0
          else
           FSelectFields.Items[i].FFieldDef.FDecimals := 2;
        end;
        if FSelectFields.Items[i].FFieldType = stAlien then
          FColumnSource.Add(Tbl.TableName + '.' + FSelectFields.Items[i].FieldName)
        else
          FColumnSource.Add(FSelectFields.Items[i].FieldName);
      end
      else
        FColumnSource.Add(FSelectFields.Items[i].FieldName);
    end;

    if FSections[2].Count > 0 then
    { Order by }
    begin
      Asc := True;
      S := FSections[2].Text;
      S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
      P := PosCaseless('Asc', S);
      if P = 0 then
      begin
        P := PosCaseless('Desc', S);
        if P <> 0 then
          Asc := false;
      end;
      if P <> 0 then
        Delete(S, P, Length(S));
      sl := TStringList.Create;
      try
        CreateListFromDelimiter(',', S, sl);
        TrimList(sl);
        if sl.Count = 0 then
          Exit;
        FColumnSource.SortFields.Clear;
        for i := 0 to sl.Count - 1 do
        begin
          FColumnSource.SortFields.Add(sl[i]);
        end;
      finally
        sl.Free;
      end;
      if Asc then
        FColumnSource.SortOrdering := soAscending
      else
        FColumnSource.SortOrdering := soDescending;
    end;
  end;

  function CheckSelectFields: integer;
  var
    i, j: integer;
    sl: TStringList;
    S: string;
    P: integer;
    Indx: integer;
    Cols: TILColumns;
    TBL : TILTable;
    TBLS : array[0..20] of TILTable;
    HasAliens : boolean;
    Lvl : integer;
    FName : string;
  begin
    Result := 0;
    HasAliens := false;
    { Check for errors }
    for i := 0 to FSelectFields.Count - 1 do
    begin
      if FSelectFields.Items[i].FError <> 0 then
      begin
        Result := FSelectFields.Items[i].FError;
        Exit;
      end;
      if HasAliens or (FSelectfields.AlienCount > 0) then
      begin
        HasAliens := True;
        if FSelectFields.Items[i].FTableName <> '' then
         TBLS[i] := TMan.Tableof(FSelectFields.Items[i].FTableName)
        else
         TBLS[i] := ILIB;
      end;
    end;
    if HasAliens then
    { Check for valid paths }
    begin
      if ILib is TILTreeViewTable then
      begin
         Result := E_TreeViewJoin;
         Exit;
      end;
      Lvl := TMan.NodeOf(Ilib).Level;
      i := 0;
      repeat
        Tbl := TBLS[i];
        if (TBL <> ILib) and not TBL.IsFreeTable then
        begin
          if TMan.NodeOf(TBL).Level < Lvl then
          begin
             Comment1 := TBL.Alias;
             Result := E_ParentJoin;
             Exit;
          end;
        end;
        for j := 0 to FSelectFields.Count - 1 do
        begin
           if TBLS[j] <> ILIb then

           if not TMan.IsInSearchPath(ILib, TBLS[j]) then
           begin
             Comment1 := Tbl.Alias + ', ' + TBLS[j].Alias;
             Result := E_TableOutOfPath;
             Exit;
           end;
        end;
        inc(i);
      until i = FSelectFields.Count;

    end;

    { Create local field of columns, not to conflict with
      any Connected report }
    Cols := TILColumns.Create(Ilib, FFieldSource);
    if FSelectFields.Count > 0 then { 0 = select * }
      Cols.Clear;
    try
      for i := 0 to FSelectFields.Count - 1 do
      begin
        if FSelectFields.Items[i].FFieldType = stAlien then
        begin
           Tbl := TMan.TableOf(FSelectFields.Items[i].FTableName);
           Indx := Tbl.Fields.IndexOf(FSelectFields.Items[i].FFieldName);
           FSelectFields.Items[i].FFieldDef :=
            FFieldSource.AddAlienField(Tbl, Tbl.Fields.Items[Indx]);
           FName := Tbl.TableName + '.' + FSelectFields.Items[i].FFieldName;
        end
        else if FSelectFields.Items[i].FFieldType = stCalc then
        begin
           FSelectFields.Items[i].FFieldDef :=
           FFieldSource.AddCalcField(FSelectFields.Items[i].FieldName);
           FName := FSelectFields.Items[i].FieldName;
        end
        else if FSelectFields.Items[i].FFieldType = stFunction then
        begin
          FSelectFields.Items[i].FFieldDef :=
           FFieldSource.AddCalcField(FSelectFields.Items[i].FieldName);
          FName := FSelectFields.Items[i].FieldName;

          FSelectFields.Items[i].FFieldDef.FDataType := dtNumber;
          if FSelectFields.Items[i].FAggregateFunc = agCount then
           FSelectFields.Items[i].FFieldDef.FDecimals := 0
          else
           FSelectFields.Items[i].FFieldDef.FDecimals := 2;
          FName := FSelectFields.Items[i].FieldName;
        end
        else
        begin
          Ind := Ilib.Fields.IndexOf(FSelectFields.Items[i].FieldName);
          if not Ilib.Fields.Items[Ind].ReportField then
          begin
            Comment1 := Ilib.Fields.Items[Ind].Caption;
            Result := E_NotReportField;
            Exit;
          end;
          FName := FSelectFields.Items[i].FieldName;
        end;
        Cols.Add(FName);
      end;

      if FSections[2].Count > 0 then
      begin
        S := FSections[2].Text;
        S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
        P := PosCaseless('Asc', S);
        if P = 0 then
        begin
          P := PosCaseless('Desc', S);
        end;
        if P <> 0 then
          Delete(S, P, Length(S));
        sl := TStringList.Create;
        try
          CreateListFromDelimiter(',', S, sl);
          TrimList(sl);
          if sl.Count = 0 then
            Exit;
          for i := 0 to sl.Count - 1 do
          begin
            Indx := FFieldSource.IndexOf(sl[i]);
            if Indx <> -1 then
            begin
              S := StringReplace(S, sl[i],
                FFieldSource.Items[Indx].FieldName, []);
              FSections[2].Text := S
            end
            else
            begin
              Comment1 := sl[i];
              Result := E_InvalidFieldName;
              Exit;
            end;

            if Cols.IndexOf(sl[i]) = -1 then
            begin
              Comment1 := sl[i];
              Result := E_NotASelectedField;
              Exit;
            end;
          end;
        finally
          sl.Free;
        end;
      end;
    finally
      Cols.Free;
    end;
  end;

  procedure FreeSWConditions;
  begin
    if Assigned(FRootCondition) then
      FreeAndNil(FRootCondition);
  end;

  var
    i: integer;
    SR : TSearcher;
  begin
    Result := inherited Prepare;

    if Result <> 0 then
    begin
      FreeSWConditions;
      Exit;
    end;

    for i := 0 to FTableCount - 1 do
      FTablePath[i] := nil;
    if not Ilib.IsFreeTable then
    begin
      FTableCount := 1;
      FTablePath[0] := Ilib;
    end
    else
      FTableCount := 0;
    if FChecking or FPrivate then
    begin
      FFieldSource := TILFieldDefs.Create(Ilib, TILFieldDef);
      FFieldSource.Assign(Ilib.Fields);
      FFieldSource.ClearCalcFields(false);
      if FPrivate then
      begin
       FColumnSource := TILColumns.Create(Ilib, FFieldSource);
       if Ilib.TVTable <> nil then
       with Ilib.TVTable do
       begin
          FTVParamsSource := TILTreeViewParams.Create(TVTable);
       end;
      end;
    end
    else
    begin
      FFieldSource := Ilib.Fields;
      FColumnSource := Ilib.Columns;
      FFieldSource.ClearCalcFields;
      if Ilib.TVTable <> nil then
        FTVParamsSource := Ilib.TVTable.TreeViewParams;
    end;
    try
      Result := GetSections;
      FreeDecomps;
      if Result <> 0 then
      begin
        FreeSWConditions;
        if FPrivate and Assigned(FColumnSource) then
          FreeAndNil(FColumnSource);
        if FPrivate and Assigned(FTVParamsSource) then
         FreeAndNil(FTVParamsSource);
        Exit;
      end;

      for i := 0 to FExpression.Count - 1 do
      begin
        slFields := TStringList.Create;
        Result := DecomposeElement(FExpression[i], slFields);
        if Result <> 0 then
        begin
          FreeDecomps;
          slFields.Free;
          Break;
        end;
        FDecomps[i] := slFields;
      end;

      if (Result = 0) and Assigned(FRootCondition) and FChecking then
      { SQL statement }
      begin
        SR := Ilib.CreateSearcher(Ilib, FRootCondition);
        SR.FPrivate := True;
        try
          SR.FChecking := True;
          Result := SR.Validate;
          FreeAndNil(FRootCondition);
        finally
          SR.Free;
        end;
      end;

      if (Result <> 0) then
      begin
        FreeSWConditions;
        if FPrivate and Assigned(FColumnSource) then
         FreeAndNil(FColumnSource);
        if FPrivate and Assigned(FTVParamsSource) then
         FreeAndNil(FTVParamsSource);
      end;

      if Result = 0 then
      begin
        { Check for invalid path }
        for i := 1 to FTableCount - 1 do
        begin
          if not TMan.IsInSearchPath(FTablePath[0], FTablePath[i]) then
          begin
             Comment1 := FTablePath[i].Alias;
             Result := E_TableOutOfPath;
             Break;
          end;

        end;
        if Result <> 0 then
           Exit;
      end;

      if Result = 0 then
      begin
        FPrepared := True;
        if (FSections[0].Count > 0) then
        begin
          Result := CheckSelectFields;
          if Result <> 0 then
            Exit;
          if not FChecking then
            CreateSelectFields;
        end;
      end;

      if Result <> 0 then
      begin
       if FPrivate and Assigned(FColumnSource) then
        FreeAndNil(FColumnSource);
       if FPrivate and Assigned(FTVParamsSource) then
         FreeAndNil(FTVParamsSource);
       FreeSWConditions;
      end;

    finally
      if FChecking then
        FFieldSource.Free;
      if FChecking and Assigned(FColumnSource) then
        FColumnSource.Free;
      if FChecking and Assigned(FTVParamsSource) then
        FTVParamsSource.Free;
    end;
  end;

  function TSearcher.DecomposeElement(AnElement: string; Decomp: TStringList): integer;

  { [/][\]FieldName1<Comparision operator>Value[/][\]Fieldname2<Comparision Operator>Value ...

    / = OR, \ = AND
  }
  const
    substGreater = #1;
    substLess = #2;
    substEq = #3;
    substAnd = #4;
    substOr = #5;
    substNot = #6;
    substAndAnd = #7;
    substOrOr = #8;
    substNotNot = #9;
    substIntervalDelimiter = #11;

  function RestoreReserved(var S: string): integer;
  begin
    Result := 0;
    S := StringReplace(S, substIntervalDelimiter, '..', [rfReplaceAll]);
    S := StringReplace(S, substEq, '=', [rfReplaceAll]);
    S := StringReplace(S, substGreater, '>', [rfReplaceAll]);
    S := StringReplace(S, substLess, '<', [rfReplaceAll]);
    S := StringReplace(S, substOr, '/', [rfReplaceAll]);
    S := StringReplace(S, substAnd, '\', [rfReplaceAll]);
    S := StringReplace(S, substOrOr, 'OR', [rfReplaceAll, rfIgnoreCase]);
    S := StringReplace(S, substAndAnd, 'AND', [rfReplaceAll, rfIgnoreCase]);
    S := StringReplace(S, substNot, c_NotOperator, [rfReplaceAll]);
    S := StringReplace(S, substNotNot, 'NOT', [rfReplaceAll, rfIgnoreCase]);

  end;


  function CheckDataType(DataType: TDataType; Value: string): integer;

  var
    DT: TDateTime;
    N: Extended;
    S: string;
  begin
    Result := 0;
    S := Value;
    RestoreReserved(S);
    if TMan.IsValueNull(S) or (S = '''' + '''') then
      Exit;
    if DataType = dtDate then
    begin
      if (Pos('?', S) = 0) and (Pos('*', S) = 0) then
        if not TryStrToDateTime(S, DT, Translator.FFormatSettings) then
        begin
          Comment1 := S;
          Result := E_InvalidDate;
          Exit;
        end;
    end
    else
    begin
      if not TryStrToFloat(S, N, Translator.FFormatSettings) then
      begin
        Comment1 := S;
        Result := E_InvalidNumber;
        Exit;
      end;
    end;
  end;

  function SplitFieldValue(AFieldValue: string; First: Boolean;
    var FieldName, AndOr, Value, CO: string; var IsNot: Boolean): integer;

  var
    P: integer;
    COLength: integer;
    Indx: integer;
    Eq: Boolean;
    i: integer;
    TBL: TILTable;
    TblName: string;
    S, Value2: string;
    N: Extended;
    IsInterval: Boolean;
    ValCount: integer;
    DS : TILDataset;
    SR : TSearcher;
    SQL : TStringList;
  begin
    Result := 0;
    COLength := 1;
    Eq := false;
    IsNot := false;
    { Get AndOr }
    P := Pos(c_NotOperator, AFieldValue);
    if P <> 0 then
    begin
      i := 1;
      while CharInSet(AFieldValue[i], ['\', '/']) do
        Inc(i);
      if P <> i then
      begin
        Result := E_NotOpMisplaced;
        Exit;
      end;
      IsNot := True;
      Delete(AFieldValue, P, 1);
    end;
    if not First then
    begin
      P := Pos('/', AFieldValue);
      if P = 0 then
        P := Pos('\', AFieldValue);
      if P = 0 then
      { No logoperator found at beginning of line, not allowed }
      begin
        Result := E_MissingOperator;
        Exit;
      end
      else
      begin
        AndOr := AFieldValue[1];
        System.Delete(AFieldValue, 1, 1);
      end;
    end
    else
    begin
      AndOr := '\'; { First element is the same a the next and/or op.
        Will be corrected if necessary }
    end;

    { Get fieldname }
    IsInterval := false;
    P := Pos('<', AFieldValue);
    if P = 0 then
      P := Pos('>', AFieldValue);
    if P = 0 then { Must be == or = }
    begin
      P := Pos('==', AFieldValue);
      if P = 0 then
      begin
        P := Pos('=', AFieldValue);
        if P = 0 then
        begin
          Result := E_MissingOperator;
          Exit;
        end
        else
        begin
          if Pos('..', AFieldValue) <> 0 then
          begin
            IsInterval := True;
          end;
          Eq := True;
        end;
      end;
    end;

    if not IsInterval then
    { Cannot mix inervals with other operator }
    begin
      if Pos('..', AFieldValue) <> 0 then
      begin
        Result := E_SyntaxError;
        Exit;
      end;
    end;
    if P = Length(AFieldValue) then
    begin
      Result := E_MissingValue;
      Exit;
    end;
    { If not co is equal or between get possible second equal operator }
    if not Eq and (CharInSet(AFieldValue[P + 1], ['=', '>'])) and not IsInterval
    then
    begin
      if P + 1 = Length(AFieldValue) then
      begin
        Result := E_MissingValue;
        Exit;
      end;
      COLength := 2;
    end;
    { Extract field name }
    FieldName := Copy(AFieldValue, 1, P - 1);
    { Extract the co }
    CO := Copy(AFieldValue, P, COLength);
    System.Delete(AFieldValue, 1, P + COLength - 1);
    { Field value: }
    Value := AFieldValue;
    if IsInterval then
    begin
      P := Pos('..', Value);
      S := Copy(Value, 1, P - 1);
      S := TrimBlanks(S);
      Delete(Value, 1, P + 1);
      Value2 := TrimBlanks(Value);
      Value := S;
    end;

    { Check if fieldname exists: }
    TBL := SplitTableField(FieldName, TblName, FieldName);
    if TBL = nil then
    begin
      Comment1 := TblName;
      Result := E_InvalidTableName;
      Exit;
    end;

    Indx := Tbl.Fields.IndexOf(FieldName);
    if Indx = -1 then
    begin
      Indx := Tbl.Fields.IndexOfCaption(FieldName);
      if Indx <> -1 then
        FieldName := Tbl.Fields.Items[Indx].FieldName
      else
      begin
        Comment1 := FieldName;
        Result := E_InvalidFieldName;
        Exit;
      end;
    end;

    if Value = EmptyValue then
    begin
      Result := -1;
      Exit;
    end;

    { Check that Value is quoted at begin and end and only two quotes }

    RestoreReserved(Value);

    if True then

    if (Tbl.Fields.Items[Indx].LookupType = looParentField)
    and (FPrivate and (TBl.MasterLinkCount > 0)) then
    begin

    end;

    if (Tbl.Fields.Items[Indx].DataType in [dtString, dtText, dtDate, dtResource, dtLongText])
    or ((Tbl.Fields.Items[Indx].LookupType = looTableKey)
       and (Tbl.Fields.Items[Indx].DataType = dtNumber)and not TryStrToFloat(Value, N))
    then
    begin
      if IsInterval then
        ValCount := 2
      else
        ValCount := 1;
      for i := 1 to ValCount do
      begin
        if i = 1 then
          S := Value
        else
          S := Value2;
        if TMan.IsValueNull(S) then
          S := '''' + '''';
        P := Pos('''', S);
        if P <> 1 then
        begin
            Result := E_MissingQuote;
        end
        else
          P := Pos('''', S, P + 1);
        if (P <> Length(S)) then
          Result := E_MissingQuote
        else
        begin
          P := Pos('''', S, P + 1);
          if (P <> 0) then
            Result := E_MissingOperator;
        end;
        if Result <> 0 then
          Break;
        if i = 1 then
          Value := S
        else
          Value2 := S;
      end;

      if (Tbl.Fields.Items[Indx].LookupType = looTableKey) and (Value <> '''' + '''') then
      begin
        SQL := TStringList.Create;
        SQL.Add('select * where ' + Tbl.Fields.Items[Indx].LookupSearchField + ' = ' + Value);
        SR := Ilib.CreateSearcher(Tbl.Fields.Items[Indx].LookupTable, SQL);
        DS := nil;
        try
          SR.FTreeViewing := (Ilib is TILTreeViewTable);
          SR.FPrivate := True;
          try
          Result := SR.Prepare;
          if Result <> 0 then
           Exit;
          DS := SR.Search;
          except
             SR.Free;
             SQL.Free;
             if DS <> nil then
               DS.Free;
             Result := E_InvalidLookupRef;
             Exit;
          end;
        finally
          SR.Free;
          SQL.Free;
        end;

        try
           if DS.Count > 0 then
           begin
              Value := (DS[0][Tbl.Fields.Items[Indx].LookupField].AsString);
           end
           else
           begin
             Result := E_InvalidLookupRef;
             Exit;
           end;
        finally
            DS.Free;
        end;
      end;
    end

    else

    begin
      if IsInterval then
        ValCount := 2
      else
        ValCount := 1;
      for i := 1 to ValCount do
      begin
        if i = 1 then
          S := Value
        else
          S := Value2;
        if TMan.IsValueNull(S) then
          S := ''
        else if Pos('''', S) <> 0 then
        begin
          Comment1 := Tbl.Fields.Items[Indx].FieldName;
          Result := E_QuotesNotNeeded;
        end;
        if (Result = 0) and (S <> '') then
        begin
          if Tbl.Fields.Items[Indx].DataType = dtBoolean then
          begin
            if SameText(S, Tman.ReportTrue) then
              S := '1'
            else if SameText(S, Tman.ReportFalse) then
              S := '0';
          end
          else if Tbl.Fields.Items[Indx].DataType = dtNumber then
          begin
            if not TryStrToFloat(S, N, Translator.FFormatSettings) then
            begin
              Comment1 := S;
              Result := E_InvalidNumber;
            end;
          end;
        end;
        if Result <> 0 then
          Break;
        if i = 1 then
          Value := S
        else
          Value2 := S;
      end;

    end;
    if Result <> 0 then
      Exit;

    if Value = '' then
    begin
      { Make fully qualified field name befor exiting }
      FieldName := TblName + '.' + FieldName;
      Exit;
    end;
    { Formalize value, if necessary }
    if (Tbl.Fields.Items[Indx].DataType in [dtDate, dtNumber]) and
      not TMan.IsValueNull(Value) then
      try
        Result := CheckDataType(Tbl.Fields.Items[Indx].DataType, Value);
        if Result <> 0 then
          Exit;
        Value := Tbl.SetDateNumberFormat(Value, Tbl.Fields.Items[Indx].DataType,
          Tbl.Fields.Items[Indx].Decimals);

        if IsInterval then
        begin
          Result := CheckDataType(Tbl.Fields.Items[Indx].DataType, Value2);
          if Result <> 0 then
            Exit;
          Value2 := Tbl.SetDateNumberFormat(Value2,
            Tbl.Fields.Items[Indx].DataType, Tbl.Fields.Items[Indx].Decimals);
        end;
      except
        Comment1 := Value;
        Result := E_IllegalValue;
      end;
    if (Tbl.Fields.Items[Indx].DataType = dtDate) and not TMan.IsValueNull(Value)
    then
    { Restore quotes, deleted in setdatnumberformat }
    begin
      Value := '''' + Value + '''';
      if IsInterval then
        Value2 := '''' + Value2 + '''';
    end;
    { Make fully qualified field name }
    FieldName := TblName + '.' + FieldName;
    if IsInterval then
      Value := Value + c_IntervalDelimiter + Value2;
  end;

  function ReplaceReserved(var S: string): integer;

  var
    P1, P2: integer;
    s2: string;
  begin
    Result := 0;
    P1 := Pos('''', S);
    while P1 <> 0 do
    begin
      P2 := Pos('''', S, P1 + 1);
      if P2 = 0 then
      begin
        Result := E_MissingQuote;
        Exit;
      end;
      s2 := Copy(S, P1, P2 - P1 + 1);
      s2 := StringReplace(s2, '=', substEq, [rfReplaceAll]);
      s2 := StringReplace(s2, '>', substGreater, [rfReplaceAll]);
      s2 := StringReplace(s2, '<', substLess, [rfReplaceAll]);
      s2 := StringReplace(s2, '/', substOr, [rfReplaceAll]);
      s2 := StringReplace(s2, '\', substAnd, [rfReplaceAll]);
      s2 := StringReplace(s2, c_NotOperator, substNot, [rfReplaceAll]);
      s2 := StringReplace(s2, '..', substIntervalDelimiter, [rfReplaceAll]);
      s2 := StringReplace(s2, 'OR', substOrOr, [rfReplaceAll, rfIgnoreCase]);
      s2 := StringReplace(s2, 'AND', substAndAnd, [rfReplaceAll, rfIgnoreCase]);
      s2 := StringReplace(s2, 'NOT', substNotNot, [rfReplaceAll, rfIgnoreCase]);

      System.Delete(S, P1, P2 - P1 + 1);
      Insert(s2, S, P1);
      P1 := Pos('''', S, P2 + 1);
    end;
  end;

  function ReplaceUserLogOps(var S: string): integer;
  begin
    Result := 0;
    S := StringReplace(S, ' AND ', '\', [rfReplaceAll, rfIgnoreCase]);
    S := StringReplace(S, ' OR ', '/', [rfReplaceAll, rfIgnoreCase]);
    if Pos('AND ', S.ToUpper) = 1 then
      S := StringReplace(S, 'AND ', '\', [rfIgnoreCase]);
    if Pos('OR ', S.ToUpper) = 1 then
      S := StringReplace(S, 'OR ', '/', [rfIgnoreCase]);
    if Pos('NOT ', S.ToUpper) = 1 then
      S := StringReplace(S, 'NOT ', c_NotOperator, [rfIgnoreCase]);
    S := StringReplace(S, ' NOT ', c_NotOperator, [rfReplaceAll, rfIgnoreCase]);
  end;

  var
    P1, P2, P3: integer;
    HasOp: Boolean;
    sl: TStringList;
    i: integer;
    F, AO, CO, V: string;
    TBL: TILTable;
    TblName, FldName: string;

    { Receives a line with posible multi field expressions
      Splits each field into and/or operator, field name, comparision operator, and value
      and returns them in a stringlist }
    HitAllCheck: TStringList;
    Indx: integer;
    IsNot: Boolean;
    TF: Char;
    FirstExpr: Boolean;

  begin
    FirstExpr := (AnsiCompareText(FExpression[0], AnElement) = 0);
    { First replace reserved characters within quotes }
    Result := ReplaceReserved(AnElement);
    if Result <> 0 then
      Exit;
    { Next "normalize" user defined logical operators }
    Result := ReplaceUserLogOps(AnElement);
    if Result <> 0 then
      Exit;

    AnElement := TrimOperators(AnElement);

    { Next get the element or/and. If not found it is interpreted as AND }
    HasOp := True;
    if (AnElement[1] <> '/') and (AnElement[1] <> '\') then
    begin
      Decomp.Add('\');
      HasOp := false;
    end
    else if AnElement[1] = '/' then
      Decomp.Add('/')
    else if AnElement[1] = '\' then
      Decomp.Add('\');

    if HasOp then
      System.Delete(AnElement, 1, 1);
    P1 := Pos('/', AnElement);
    P2 := Pos('\', AnElement);
    while (P1 <> 0) or (P2 <> 0) do
    begin
      if P1 = 0 then
        P3 := P2
      else if P2 = 0 then
        P3 := P1
      else if P1 < P2 then
        P3 := P1
      else
        P3 := P2;
      Insert('~', AnElement, P3); { Create a delimiter }
      P1 := P3 + 2;
      P2 := P3 + 2;
      P1 := Pos('/', AnElement, P1);
      P2 := Pos('\', AnElement, P2);
    end;
    sl := TStringList.Create;
    HitAllCheck := TStringList.Create;
    HitAllCheck.Sorted := True;
    HitAllCheck.Duplicates := DupIgnore;
    try
      if not FirstExpr then
      { Not the first element, must start with a logop }
      begin
        if not HasOp then
        begin
          Result := E_MissingOperator;
          Exit;
        end
      end;
      CreateListFromDelimiter('~', AnElement, sl);
      for i := 0 to sl.Count - 1 do
      begin
        if TrimBlanks(sl[i]) = '' then
          Continue;
        Result := SplitFieldValue(sl[i], (i = 0), F, AO, V, CO, IsNot);
        if Result <> 0 then
        begin
          if Result = -1 then { Empty value}
          begin
            Result := 0;
            Continue;
          end
          else
            Exit;
        end;
        if (CO = '=') or (CO = '==') then
          HitAllCheck.Add(F + CO);
        if (CO = '=') then
        begin
          if HitAllCheck.Find(F + '==', Indx) then
          begin
            Result := E_IllegalMixOfEqualCO;
            Exit;
          end
        end
        else if (CO = '==') then
        begin
          if HitAllCheck.Find(F + '=', Indx) then
          begin
            Result := E_IllegalMixOfEqualCO;
            Exit;
          end;
        end;

        { Check if quotes ok }
        TBL := SplitTableField(F, TblName, FldName);
        Indx := Tbl.Fields.IndexOf(FldName);
        if (Tbl.Fields.Items[Indx].DataType in [dtString, dtText, dtDate,
          dtResource, dtLongText]) and not TMan.IsValueNull(V) then
        begin
          if (V[1] <> '''') or (V[Length(V)] <> '''') then
          begin
            Result := E_MissingQuote;
            Exit;
          end
          else
          begin { Delet quote, no longer use for it }
            V := StringReplace(V, '''', '', [rfReplaceAll]);
            if Tbl.Fields.Items[Indx].DataType <> dtText then
              V := TrimBlanks(V);
          end;
        end;
        RestoreReserved(V);
        if IsNot then
          TF := '1'
        else
          TF := '0';

        Decomp.Add(AO + '~' + F + '~' + CO + '~' + V + '~' + TF);
      end;
    finally
      sl.Free;
      HitAllCheck.Free;
    end;
  end;

  function TSearcher.SplitTableField(FieldName: string;
    var TblName, FldName: string): TILTable;

  var
    P: integer;

  procedure CollectTable;
  var
    i: integer;
  begin
    for i := 0 to FTableCount - 1 do
    begin
      if FTablePath[i] = Result then
        Exit;
    end;
    Inc(FTableCount);
    FTablePath[FTableCount - 1] := Result;
  end;

  begin
    P := Pos('.', FieldName);
    if P = 0 then
    begin
      TblName := Ilib.TableName;
      FldName := FieldName;
      Result := Ilib;
    end
    else
    begin
      Result := nil;
      TblName := Copy(FieldName, 1, P - 1);
      FldName := Copy(FieldName, P + 1, Length(FieldName));
      Result := TMan.TableOf(TblName)
    end;
    if not Ilib.IsFreeTable then
      CollectTable;
  end;

  { TUpdater ----------------------------------------------------------------- }

  constructor TUpdater.Create(TableOwner: TILTable; ARecord: TILRecord);

  var
    sl: TStringList;
  begin
    { If Create is called without an action the default is umaReplaceAll }
    Ilib := TableOwner;
    sl := RecordToExpression(ARecord);
    try
      inherited Create(TableOwner, sl);
    finally
      sl.Free;
    end;
    TMan.FMainThreadBusy := True;
    FRecordIndex := -1;
  end;

  destructor TUpdater.Destroy;
  begin
    TMan.FMainThreadBusy := False;
    if Assigned(FInsertRecord) then
      FInsertRecord.Free;
    inherited Destroy;
  end;

  function TUpdater.Update: integer;

  var
    UpdateRec: TILDataRecord;
    ErrCode: integer;
    S : string;
    IsPending : Boolean;
    TrID : integer;
    Ind : integer;
    MH : TILMessageHandler;
    Res : integer;
    DeletedNewCount : integer;

   procedure CheckLongText(ARecord : TILDataRecord);
   var
    i : integer;
    Cnt : integer;
   begin
    if not Assigned(Ilib.Datahandler.FOnUpdateLongText) then
      Exit;
    if ARecord = nil then
      Exit;
    Cnt := ARecord.Count;
    for I := 0 to Cnt-1 do
    if (ARecord[i].FieldDef.DataType = dtLongText)
    and ARecord[i].TextSourceChanged then
    begin
      S := '';
      Ilib.Datahandler.FOnUpdateLongText(Ilib.DataHandler, ARecord[i], S, [ltaUpload]);
      ARecord[i].TextSourceChanged := false;
      ARecord[i].FTextUpdated := True;
    end;
   end;


   function DoUpdate: integer;
   var
    Srch: TSearcher;
    DS: TILDataset;
    i, j: integer;
    Missing: string;
    Indx, TagIndx: integer;
    ImageFile: string;
    ImageTagValue : string;
    NewTagValue : string;
    R : TILDatarecord;
    Loops : integer;
    sl : TStrings;
    FN : string;

    procedure CheckDataset(RecIndx : integer);
    var
      DSIndx : integer;
    begin
      if TMan.DLLAccess then
        Exit;
      DSIndx := Ilib.Dataset.IndexOf(RecIndx);
      if DSIndx <> -1 then
        Ilib.Dataset[DSIndx].Refresh;
    end;

    procedure CheckImageTag;
    var
      j : integer;
    begin
         if not Ilib.HasHandler then
           Exit;
         for j := 0 to Ilib.DataHandler.DataFields.Count-1 do
         begin
            if Ilib.DataHandler.DataFields.Items[j].ImageTag then
            begin
              TagIndx := UpdateRec.IndexOf(Ilib.DataHandler.DataFields.Items[j].FieldName);
              ImageTagValue := Ilib.DataHandler.GetImageTagValue(UpdateRec);
              Break;
            end;
         end;
    end;

    procedure SetTagImage;
    begin
        if not Ilib.HasHandler then
           Exit;
        NewTagValue := Ilib.DataHandler.GetImageTagValue(UpdateRec);
        if not SameText(NewTagValue, ImageTagValue) then
        begin
           if NewTagValue = '' then
            Ilib.DeleteImage(UpdateRec.RecordIndex)
           else
            TILDataTable(Ilib).GetFromImageTable(NewTagValue, UpdateRec);
        end;
    end;

  begin
    Result := 0;
    ErrCode := 0;
    DS := nil;
    If FUpdateTarget = utrDataSet then
    begin
      if Ilib.Dataset.IsReadOnly then
        HandleILException(E_DatasetReadOnly, '');
      for i := 0 to Ilib.SelCount - 1 do
      begin
        ImageFile := '';
        TagIndx := -1;
        UpdateRec := Ilib.Dataset.Selection[i];
        Indx := UpdateRec.IndexOf(c_SysFieldNames[SYS_FileName]);
        CheckImageTag;
        if Indx <> -1 then
        begin
          ImageFile := UpdateRec[Indx].AsString;
          if not IsMediaFile(ImageFile) then
          begin
            Comment1 := ImageFile;
            HandleILException(E_UnsupportedFileFormat, '');
          end;
        end;
        Execute(Ilib.Dataset.Selection[i]);
        if Ilib.Dataset.Selection[i].Changed then
        begin
          Ilib.Dataset.FChngTracker.Add(Ilib.Dataset.Selection[i]);
          if Indx <> -1 then
          begin
            if not SameText(UpdateRec[Indx].AsString, ImageFile) then
            begin
              if ImageFile = '' then
              begin
                UpdateRec[Indx].FValues.Clear;
                if Assigned(UpdateRec.FImage) then
                  FreeAndNil(UpdateRec.FImage);
                UpdateRec[Indx].AsString := '';
              end
              else
              begin
                UpdateRec[Indx].AsString := '';
                UpdateRec.LoadFileData(ImageFile);
                UpdateRec.InitFields(nil);
              end;
            end;
          end;
          if TagIndx <> -1 then
          begin
            SetTagImage;
          end;
          Inc(Result);
          if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
          for J := 0 to UpdateRec.Count-1 do
          begin
            if (UpdateRec[J].FieldDef.FieldType = ftCalculated) and not UpdateRec[J].FieldDef.FUserCalc then
            try
             Ilib.DataHandler.FOnCalcField(Self, UpdateRec, UpdateRec[J]);
            except

            end;
          end;

          for J := 0 to UpdateRec.Count-1 do
          begin
            { Check ParentField lookups }
            if UpdateRec[j].FieldDef.LookupType = looParentField then
            begin
             sl := Ilib.GetLookupParentValue
              (UpdateRec[j].FieldDef.LookupTable, UpdateRec[j].FieldDef.LookupField, UpdateRec);
             if sl = nil then
              Continue;
             UpdateRec[j].FValues.Assign(sl);
             sl.Free;
            end;
          end;
        end;
        CheckLongText(Ilib.Dataset.Selection[i]);
      end;
      if Result = 0 then
        Result := -1; { No selection }
      Exit;
    end;

    if FSections[2].Count > 0 then
    begin
      Srch := Ilib.CreateSearcher(Ilib, FSections[2]);
      Srch.FPrivate := True;
      try
        ErrCode := Srch.Validate;
        if ErrCode <> 0 then
          Exit;
        DS := Srch.Search(nil, 0, True);
        Loops := Ds.Count;
        TMan.FProgressTicker := GetTickCount;
        for i := 0 to DS.Count - 1 do
        begin
          TMan.Progress(Ilib, iloUpdate, Loops, i);
          ImageFile := '';
          TagIndx := -1;
          if DS.Items[i].Deleted then
            Continue;
          UpdateRec := DS.Items[i];
          Indx := UpdateRec.IndexOf(c_SysFieldNames[SYS_FileName]);
          if Indx <> -1 then
          begin
            ImageFile := UpdateRec[Indx].AsString;
            if not IsMediaFile(ImageFile) then
            begin
              Comment1 := ImageFile;
              HandleILException(E_UnsupportedFileFormat, '');
            end;
          end;
          CheckImageTag;
          Execute(UpdateRec);
          if UpdateRec.Changed then
          begin
            if Ilib.CheckMandatories(UpdateRec, Missing) then
            begin
              R := Ilib.RecordsX[UpdateRec.RecordIndex];
              try
                if not Ilib.CompareIDS(UpdateRec, R) then
                begin
                  if Ilib.IndexOfID(UpdateRec, FN) <> -1 then
                    HandleILException(E_EntryExists, FN);
                 if Ilib.IndexOfUnique(UpdateRec, FN) <> -1 then
                    HandleILException(E_EntryExists, FN);
                end;
              finally
                R.Free;
              end;

              Ilib.Records[UpdateRec.RecordIndex] := UpdateRec;
              if Indx <> -1 then
              begin
                if not SameText(UpdateRec[Indx].AsString, ImageFile) then
                begin
                  if ImageFile = '' then
                    Ilib.DeleteImage(UpdateRec.RecordIndex)
                  else
                    Ilib.UpdateImage(UpdateRec.RecordIndex, ImageFile);
                end;
              end;
              if TagIndx <> -1 then
              begin
                SetTagImage;
              end;
              Inc(Result);
              CheckDataset(UpdateRec.RecordIndex);
            end
            else
            begin
              Comment1 := Missing;
              HandleILException(E_MandatoryValue);
            end;
          end;
        end;
        Ind := -1;
        MH := TMan.GetMessageHandler(Ilib, Ind);
        while MH <> nil do
        begin
           MH.AddCount(dboUpdate, Loops,false);
           if Loops = 0 then
           begin
             MH.AddCount(dboSave, Loops,false);
             MH.LastOperation := dboSave;
           end;
           MH := TMan.GetMessageHandler(Ilib, Ind);
        end;
        TMan.Progress(Ilib, iloFinished, 0, 0);
      finally
        Srch.Free;
        if Assigned(DS) then
          DS.Free;
      end;
    end
    else
    begin
      if FRecordIndex > -1 then
      begin
         Loops := 1;
      end
      else
        Loops := Ilib.RecordCount;
      for i := 0 to Loops - 1 do
      begin
        TMan.Progress(Ilib, iloUpdate, Loops, i);
        ImageFile := '';
        TagIndx := -1;
        if FRecordIndex > -1 then
           UpdateRec := Ilib.RecordsAbs[FRecordIndex]
        else
           UpdateRec := Ilib.RecordsAbs[i];
        if UpdateRec.Deleted then
        begin
          UpdateRec.Free;
          Continue;
        end;
        Indx := UpdateRec.IndexOf(c_SysFieldNames[SYS_FileName]);
        if Indx <> -1 then
           ImageFile := UpdateRec[Indx].AsString;
        CheckImageTag;
        try
          Execute(UpdateRec);
          if UpdateRec.Changed then
          begin
            if Ilib.CheckMandatories(UpdateRec, Missing) then
            begin
              R := Ilib.RecordsX[UpdateRec.RecordIndex];
              try
                if not Ilib.CompareIDS(UpdateRec, R) then
                 if Ilib.IndexOfID(UpdateRec, FN) <> -1 then
                    HandleILException(E_EntryExists, FN);
                 if Ilib.IndexOfUnique(UpdateRec, FN) <> -1 then
                   HandleILException(E_EntryExists, FN);
              finally
                  R.Free;
              end;

              Ilib.Records[UpdateRec.RecordIndex] := UpdateRec;
              if Indx <> -1 then
              begin
                if not SameText(UpdateRec[Indx].AsString, ImageFile) then
                begin
                  if ImageFile = '' then
                    Ilib.DeleteImage(UpdateRec.RecordIndex)
                  else
                    Ilib.UpdateImage(UpdateRec.RecordIndex, ImageFile);
                end;
                if TagIndx <> -1 then
                begin
                  SetTagImage;
                end;
              end;
              CheckDataset(UpdateRec.RecordIndex);
              Inc(Result);
            end
            else
            begin
              Comment1 := Missing;
              HandleILException(E_MandatoryValue);
            end;
          end;
        finally
          if Assigned(UpdateRec) then
            UpdateRec.Free;
        end;
      end;
      Ind := -1;
      MH := TMan.GetMessageHandler(Ilib, Ind);
      while MH <> nil do
      begin
           MH.AddCount(dboUpdate, Loops,false);
           if Loops = 0 then
           begin
             MH.AddCount(dboSave, Loops,false);
             MH.LastOperation := dboSave;
           end;
           MH := TMan.GetMessageHandler(Ilib, Ind);
      end;
      TMan.Progress(Ilib, iloFinished, 0, 0);
    end;
  end;

  function DoInsert: integer;

  var
    i, j: integer;
    Indx: integer;
    Data: TILDataRecord;
    sl: TStringList;
    ImageFile: string;
    DS : TILDataset;
    LnkID : integer;
    Cnt : integer;
    N : integer;
    S : string;
    Err : Boolean;

  procedure CreateSubLines(Sublines: string; SubList : TStringList);
  var
    P: integer;
  begin
    P := PosRestrict(';', Sublines, False);
    while P <> 0 do
    begin
      Sublines[P] := #1;
      P := PosRestrict(';', Sublines, False);
    end;
    CreateListFromDelimiter(#1, Sublines, SubList);
  end;

  procedure CheckImgTag;
  var
    Val : string;
  begin
    if (TMan.SharedImages <> nil) and Ilib.HasHandler then
      Val := Ilib.DataHandler.GetImageTagValue(Data);
    if Val <> '' then
      Data.GetFromImageTable(Val);
  end;

  begin
    Result := 0;
    ImageFile := '';
    Err := false;
    if Ilib.IsDetail and (FSections[0].Count = 0) and (FUpdateTarget = utrTable) then
    begin
     HandleILException(E_MissingConnectBy)
    end
    else if (Ilib.Relations = relOne) and (Ilib.RecordCount - Ilib.DeletedCount  > 0) then
      HandleILException(E_RelConstraint);

    if (FSections[0].Count = 0) and (Ilib is TILTreeViewTable) then
    begin
      if FSections[1].Count > 0 then
        Indx := FSections[1].IndexOf('SYS_Level')
      else
        Indx := Ilib.Fields.IndexOf('SYS_Level');
      FSections[2][Indx] := '0';

      if FSections[1].Count > 0 then
        Indx := FSections[1].IndexOf('SYS_SiblingIndex')
      else
        Indx := Ilib.Fields.IndexOf('SYS_SiblingIndex');
      Cnt := TILTreeViewTable(Ilib).GetTVChildCount(-1);
      FSections[2][Indx] := IntToStr(Cnt);
    end

    else if FSections[0].Count > 0 then { Connect by clause }
    begin
      if Ilib.IsDetail then
      begin
        if FSections[0][0] = 'PRIOR' then
        begin
          DS := TILDataSet.Create(ILIB.MasterSource);
          DS.Add(ILIB.MasterSource.RecordsX[ILIB.MasterSource.RecordCount-1])
        end
        else if (FSections[0].Count = 1) and TryStrToInt(FSections[0][0], Indx) then
        begin
          DS := TILDataSet.Create(ILIB);;
          DS.Add(ILIB.RecordsX[Indx])
        end
        else
        DS := Ilib.MasterSource.DBView.SearchPrivate(FSections[0]);
      end
      else { TreeViewTable }
      begin
        if FSections[0][0] = 'PRIOR' then
        begin
          DS := TILDataSet.Create(ILIB);
          { Get last inserted child: }
          Data := ILIB.RecordsX[ILIB.RecordCount-1];
          try
          { Get index of last inserted parent }
          if Data['SYS_LinkID'].IsEmpty then
            Indx := Data.RecordIndex
          else
            Indx := Data['SYS_LinkID'].AsInteger;
          finally
            Data.Free;
          end;
          DS.Add(ILIB.RecordsX[Indx]);
        end
        else
          DS := Ilib.DBView.SearchPrivate(FSections[0]);
      end;
      try
        if (DS.Count = 0) or (DS.Count > 1) then
          HandleILException(E_ParentResolve);
        LnkID := DS[0].RecordIndex;

        if Ilib.Relations = relOne then
        begin
          sl := TStringList.Create;
          try
           sl.Add('SYS_LinkID = ' + IntToStr(LnkID));
           Ilib.DBView.SearchCount('SYS_LinkID', N, Sl);
           if N > 0 then
             HandleILException(E_RelConstraint);
           finally
            sl.Free;
           end;
        end;

        if FSections[1].Count > 0 then
          Indx := FSections[1].IndexOf('SYS_LinkID')
        else
          Indx := Ilib.Fields.IndexOf('SYS_LinkID');
        FSections[2][Indx] := IntToStr(LnkID);

        if Ilib is TILTreeViewTable then
        begin
          if FSections[1].Count > 0 then
            Indx := FSections[1].IndexOf('SYS_Level')
          else
            Indx := Ilib.Fields.IndexOf('SYS_Level');
          FSections[2][Indx] := IntToStr(DS[0]['Sys_Level'].AsInteger + 1);

          if FSections[1].Count > 0 then
            Indx := FSections[1].IndexOf('SYS_SiblingIndex')
          else
            Indx := Ilib.Fields.IndexOf('SYS_SiblingIndex');
          Cnt := TILTreeViewTable(Ilib).GetTVChildCount(DS[0].RecordIndex);
          FSections[2][Indx] := IntToStr(Cnt);
        end;
      finally
       DS.Free;
      end;
    end;

    sl := TStringList.Create;
    Data := Ilib.CreateDataRecord;
    try
      for i := 0 to FSections[2].Count - 1 do { Value list }
      begin
        If TMan.IsValueNull(FSections[2][i]) then
          FSections[2][i] := '';
        if PosRestrict(';', FSections[2][i], False) <> 0 then
        begin
          CreateSubLines(FSections[2][i], sl);
        end;
        if FSections[1].Count > 0 then { Fieldlist exists }
        begin
          if SameText(FSections[1][i], '$Image') then
          begin
            ImageFile := FSections[2][i];
            if not IsMediaFile(ImageFile) then
            begin
               Comment1 := ImageFile;
               Err := True;
               HandleILException(E_UnsupportedFileFormat);
            end;
          end
          else if sl.Count > 0 then
          begin
            Indx := Data.IndexOf(FSections[1][i]);
            for j := 0 to sl.Count - 1 do
             if not sl[j].IsEmpty then
              Data[Indx].AsString := sl[j]
          end
          else
          begin
            Indx := Data.IndexOf(FSections[1][i]);
            if SameText(Data[Indx].FFieldName, c_SysFieldNames[SYS_FileName])
            then
            begin
              ImageFile := FSections[2][i];
              if not IsMediaFile(ImageFile) then
              begin
                Comment1 := ImageFile;
                Err := True;
                HandleILException(E_UnsupportedFileFormat, '');
              end;
            end
            else if not FSections[2][i].IsEmpty then
            begin
              if Data[Indx].FieldDef.DataType in [dtText, dtLongtext] then
               Data[Indx].AsText := FSections[2][i]
              else
               Data[Indx].AsString := FSections[2][i];
            end;
          end;
        end
        else
        begin
          if (Pos('$Image', FSections[2][i]) = 1) then
          begin
            ImageFile := FSections[2][i];
            System.Delete(ImageFile, 1, 6);
            S := ExtractFileDir(ImageFile);
            if S = '' then
              ImageFile := GetCurrentDir + '\' + ImageFile;
            if not IsMediaFile(ImageFile) then
            begin
               Comment1 := ImageFile;
               Err := True;
               HandleILException(E_UnsupportedFileFormat);
            end;
          end
          else if sl.Count > 0 then
          begin
            for j := 0 to sl.Count - 1 do
              if not sl[j].IsEmpty then
               Data[i].AsString := sl[j]
          end
          else
          begin
            if SameText(Data[i].FFieldName, c_SysFieldNames[SYS_FileName]) then
            begin
              ImageFile := FSections[2][i];
            end
            else if not FSections[2][i].IsEmpty then
            begin
              if Data[i].FieldDef.DataType in [dtLongtext, dtText] then
               Data[i].AsText := FSections[2][i]
              else
               Data[i].AsString := FSections[2][i];
            end;
          end;
        end;
        sl.Clear;
      end;
      CheckImgTag;

      if (Ilib is TILTreeViewTable) and (FUpdateTarget = utrDataset) then
      begin
         TILTreeViewTable(Ilib).TreeViewParams.InsertMode := timChild;
      end;

      if FInsertIntoRecord then
        FInsertRecord := Data
      else if FUpdateTarget = utrTable then
      begin
       try
        if (ImageFile <> '') then
        begin
          Ilib.AddToTable(Data, ImageFile);
        end
        else
          Ilib.AddToTable(Data);
       except
          Err := True;
          raise;
       end;
      end
      else
      begin
        try
        if (ImageFile <> '') then
          Ilib.AddToDataset(Data, ImageFile)
        else
          Ilib.AddToDataset(Data);
        except
          Err := True;
          raise;
        end;
        CheckLongText(Data);
      end;
      Inc(Result);
    finally
      if Err or ((FUpdateTarget = utrTable) and not FInsertIntoRecord) then
        Data.Free;
      sl.Free;
    end;
  end;

  function DoDelete: integer;

  var
    Srch: TSearcher;
    DS: TILDataset;
    i: integer;
    Loops : integer;
  begin
    Result := 0;
    DeletedNewCount := 0;
    if UpdateTarget = utrDataSet then
    begin
      if ILib.Dataset.Count = 0 then
      begin
        Result := -1;
        Exit;
      end;
      i := 0;
      Ilib.Dataset.BeginUpdate;
      try
        while i < Ilib.Dataset.Count do
        begin
          begin
            if Ilib.Dataset.Items[i].Selected then
            begin
              if Ilib.Dataset.Items[i].IsNew then
                inc(DeletedNewCount);
              Ilib.Dataset.Delete(i);
              Ilib.Dataset.FChngTracker.Add(Ilib.Dataset.Items[i]);
              Inc(Result);
              if not Tman.SaveOnDelete or (Ilib.HasHandler and Ilib.DataHandler.Multiselected) then
                inc(i);
            end
            else
             inc(i);
          end;
        end;
      finally
        Ilib.Dataset.EndUpdate;
      end;
      if TMan.SaveOnDelete and Ilib.HasHandler and Ilib.DataHandler.MultiSelected then
      begin
        Ilib.DBWork.Save;
      end;

    end
    else
    begin
      DS := nil;
      Srch := Ilib.CreateSearcher(Ilib, FSections[2]);
      Srch.FPrivate := True;
      try
        DS := Srch.Search(nil, 0, True);
        Loops := DS.Count;
        Ind := -1;
        MH := TMan.GetMessageHandler(Ilib, Ind);
        while MH <> nil do
        begin
           MH.AddCount(dboDelete, Loops, False);
           if Loops = 0 then
           begin
             MH.LastOperation := dboSave;
             MH.AddCount(dboSave, Loops,false);
           end;
           MH := TMan.GetMessageHandler(Ilib, Ind);
        end;
        for i := 0 to DS.Count - 1 do
        begin
          TMan.Progress(Ilib, iloDelete, Loops, i);
          Res := Ilib.DeleteFromTable(DS.Items[i].RecordIndex);
          Inc(Result, Res);
        end;
        TMan.Progress(Ilib, iloFinished, 0, 0);
      finally
        Srch.Free;
        if Assigned(DS) then
          DS.Free;
      end;
    end;
  end;

  procedure DoAddCount(Op : TDBOperation);
  var
    Ind : integer;
    MH : TILMessageHandler;
    Cnt : integer;
  begin
     Ind := -1;
     Cnt := Result;
     if Op = dboDelete then
     begin
      dec(Cnt, DeletedNewCount);
     end;
     MH := TMan.GetMessageHandler(ILib, Ind);
     while MH <> nil do
     begin
       if DeletedNewCount > 0 then
        Dec(cnt, DeletedNewCount);
       if Cnt >= 0 then
       begin
         if Op = dboUpdate then
           MH.AddCount(Op, -1)
         else
           MH.AddCount(Op, Cnt, False);
         MH.Target := UpdateTarget;
       end;
       MH := TMan.GetMessageHandler(ILib, Ind);
     end;
  end;

  begin
    Result := 0;
    DeletedNewCount := 0;
    if Ilib.IsReadOnly then
      HandleILException(E_ReadOnly);
    ErrCode := Prepare;
    if ErrCode <> 0 then
      HandleILException(ErrCode, '');
    IsPending := (TMan.GetTransState(TrId) = TRPending);
    Inc(TMan.FUpdateIDCounter);
    if (UpdateTarget = utrTable) or TMan.AutoSave then
    begin
      TMan.FUpdater := Self;
      TMan.AddToUpdateStack(Ilib, TMan.FUpdateIDCounter);
    end;

    try
    try
     {True: An internal transaction is active, dont start a new one }
     if (UpdateTarget = utrTable) and not IsPending and (Ilib = TMan .FTransOwner) then
      TMan.SetTransState(trPending);
     case FOperation of
        dboUpdate:
        begin
          Result := DoUpdate;
          if UpdateTarget = utrDataset then
           DoAddCount(dboUpdate);
        end;
        dboInsert:
        begin
          Result := DoInsert;
          if UpdateTarget = utrDataset then
           DoAddCount(dboInsert); { Added in AddToTable}
        end;
        dboDelete:
        begin
          Result := DoDelete;
          if UpdateTarget = utrDataset then
          begin
            DoAddCount(dboDelete); { Added in DeleteFromTable}
          end;
        end;
     end;
     if ErrCode <> 0 then
        HandleILException(ErrCode, '');

     except on E:Exception do
     begin
        TMan.HandleTransError(E);
     end;
    end;

    if (UpdateTarget = utrTable) and not IsPending and (Ilib = TMan.FTransOwner) then
      TMan.SetTransState(trCommitted);

    if (FUpdateTarget = utrDataSet) then
    begin
        if Ilib.HasViewer then
          Ilib.Viewer.Report.Redraw;
        Ilib.WriteControls(Ilib.FocusedRecord, duData);
    end;
    finally
     if (UpdateTarget = utrTable) or TMan.AutoSave then
      TMan.RemoveFromUpdateStack;
     TMan.FUpdater := nil;
    end;
  end;

  function TUpdater.Execute(ADataRecord: TILDataRecord): Boolean;
  var
    i, j: integer;
    Xpr: string;
    sl: TStringList;
    Indx, Indx2: integer;
    Found: Boolean;
    ErrCode: integer;
    AllSuccess: Boolean;
    Hit: Boolean;
    ChngCount: integer;
    OldCopy: Boolean;
    S, s2: string;
    { There are no checks for maxvalues or mandatories. Theese will be caught on
      save time }

  begin
    if not FPrepared then
    begin
      ErrCode := Prepare;
      if ErrCode <> 0 then
        HandleILException(ErrCode, '')
    end;

    OldCopy := false;
    if (ADataRecord.FOwnerDataset <> nil) or (FUpdateTarget = utrTable)
    then { Not a TestUpdate }
      for i := 0 to ADataRecord.Count - 1 do
        if ADataRecord[i].FValues = ADataRecord[i].FOldValues then
        begin
          ADataRecord[i].FOldValues := TStringList.Create;
          ADataRecord[i].FOldValues.Assign(ADataRecord[i].FValues);
          OldCopy := True;
        end;

    AllSuccess := True;
    ChngCount := 0;
    sl := TStringList.Create;
    try
      for i := 0 to FDecomps[0].Count - 1 do
      begin
        Xpr := FDecomps[0][i];
        CreateListFromDelimiter('~', Xpr, sl);
        if sl[1].ToUpper = '$IMAGE' then
         Indx := 100
        else
         Indx := ADataRecord.IndexOf(sl[1]);
        if (Indx <> 100) and (ADataRecord[Indx].FieldDef.FieldType in [ftCalculated, ftStoredEvaluation]) then
        begin
          Continue;
        end;
        if sl[0] = 'DELETE' then
        begin
          if sl[2] = '*' then
          begin
            if (ADataRecord[Indx].FValues.Count <> 0) then
            begin
              ADataRecord[Indx].FValues.Clear;
              Inc(ChngCount);
            end;
          end
          else
            repeat
              Found := false;
              Hit := false;
              for j := 0 to ADataRecord[Indx].Count - 1 do
              begin
                if ADataRecord[Indx].FieldDef.DataType = dtText then
                begin
                  S := ADataRecord[Indx].FValues[j];
                  if PosCaseless(sl[2], S) <> 0 then
                  begin
                    S := ReplaceText(S, sl[2], '');
                    ADataRecord[Indx].FValues[j] := S;
                    Inc(ChngCount);
                    Found := True;
                    Break;
                    Hit := True;
                  end;
                end
                else if Masks.MatchesMask(ADataRecord[Indx].FValues[j].ToUpper, sl[2].ToUpper)
                then
                begin
                  ADataRecord[Indx].FValues.Delete(j);
                  Inc(ChngCount);
                  Found := True;
                  Hit := True;
                  Break;
                end;
              end;
              if not Hit then
                AllSuccess := false;
            until not Found;
        end
        else if sl[0] = 'ADD' then
        begin
          Found := false;
          if not (ADataRecord[Indx].FieldDef.DataType in [dtText, dtLongText]) then
            for j := 0 to ADataRecord[Indx].Count - 1 do
            begin
              if AnsiCompareText(ADataRecord[Indx].FValues[j], sl[3]) = 0 then
              begin
                Found := True;
                Break;
              end;
            end;
          if not Found then
          begin
            if TMan.IsValueNull(sl[3]) then
            begin
              if ADataRecord[Indx].FieldDef.DataType in [dtText, dtLongText] then
                ADataRecord[Indx].FValues.Add('')
              else
                ADataRecord[Indx].FValues.Clear;
            end
            else
            begin
              ADataRecord[Indx].FValues.Add(sl[3]);
            end;
            Inc(ChngCount);
          end
          else
            AllSuccess := false;
        end
        else if sl[0] = 'SET' then
        begin
          if Indx = 100 then
          begin
            if TMan.IsValueNull(sl[3]) or (sl[3] = '') then
            begin
               ADataRecord.UpdateImage(nil);
            end
            else
            begin
              ADataRecord.UpdateImage(sl[3]);
            end;
          end
          else if (ADataRecord[Indx].FieldDef.DataType = dtLongText)
          and not TMan.IsValueNull(sl[3]) then
          begin
            ADataRecord[Indx].AsText := sl[3];
          end
          else
          begin
            ADataRecord[Indx].FValues.Clear;
            if not TMan.IsValueNull(sl[3]) then
            begin
              ADataRecord[Indx].FValues.Add(sl[3]);
            end;
          end;
          Inc(ChngCount);
        end
        else { Replace }
        begin
          Found := false;
          Indx2 := 0;
          if TMan.IsValueNull(sl[2]) then
            Found := (ADataRecord[Indx].FValues.Count = 0)
          else
            for j := 0 to ADataRecord[Indx].Count - 1 do
            begin
              if ADataRecord[Indx].FieldDef.DataType = dtText then
              begin
                S := ADataRecord[Indx].FValues[j];
                if PosCaseless(sl[2], S) <> 0 then
                begin
                  if TMan.IsValueNull(sl[3]) then
                    s2 := ''
                  else
                    s2 := sl[3];
                  S := ReplaceText(S, sl[2], s2);
                  ADataRecord[Indx].FValues[j] := S;
                  Inc(ChngCount);
                  Found := True;
                end;
              end
              else if Masks.MatchesMask(ADataRecord[Indx].FValues[j].ToUpper, sl[2].ToUpper)
              then
              begin
                Found := True;
                Indx2 := j;
                Break;
              end;
            end;
          if Found and not(ADataRecord[Indx].FieldDef.DataType = dtText) then
          begin
            if TMan.IsValueNull(sl[2]) then
              ADataRecord[Indx].FValues.Add(sl[3])
            else if TMan.IsValueNull(sl[3]) then
              ADataRecord[Indx].FValues.Clear
            else
              ADataRecord[Indx].FValues[Indx2] := sl[3];
            Inc(ChngCount);
          end
          else
            AllSuccess := false;
        end;
        if (Indx <> 100) and ADataRecord[Indx].FieldDef.ImageText then
        begin
          ADataRecord[Indx].UpdateImageText;
        end;
      end;
      Result := AllSuccess;
      for i := 0 to ADataRecord.Count - 1 do
        if (ADataRecord[i].FValues.Count = 1) and
          (ADataRecord[i].FValues[0] = '') then
          ADataRecord[i].FValues.Clear;

      if (ChngCount = 0) and OldCopy then
        for i := 0 to ADataRecord.Count - 1 do
        begin
          ADataRecord[i].FOldValues.Free;
          ADataRecord[i].FOldValues := ADataRecord[i].FValues;
        end;
    finally
      sl.Free;
    end;
  end;

  function TUpdater.GetSections: integer;

  var
    i: integer;
    S, s2: string;
    P1, P2, PU, PIN, PD, PW, CB, CBP: integer;
    CommandFound: Boolean;
    sl: TStringList;
    WhereFound: Boolean;
    N : integer;
    InsByLiteral : Boolean;

  function CheckMandatories(sl: TStringList): integer;
  var
    i: integer;
  begin
    Result := 0;
    for i := 0 to Ilib.Fields.Count - 1 do
    begin
      if Ilib.Fields.Items[i].Mandatory and
        not SameText(Ilib.Fields.Items[i].FieldName, c_SysFieldNames[SYS_LinkID])
        and not SameText(Ilib.Fields.Items[i].FieldName, c_SysFieldNames[SYS_Level])
        and not SameText(Ilib.Fields.Items[i].FieldName, c_SysFieldNames[SYS_SiblingIndex])
        and not (Ilib.Fields.Items[i].LookupType = looParentField)
        and not (Ilib.Fields.Items[i].FieldType = ftStoredEvaluation)
      then
      begin
        if (sl.IndexOf(Ilib.Fields.Items[i].FieldName) = -1)
        and (Ilib.Fields.Items[i].InitialValue = '')  then
        begin
          Comment1 := Ilib.Fields.Items[i].Caption;
          Result := E_MissingMandatoryField;
          Exit;
        end;
      end;
    end;
  end;

  function CheckFieldList: integer;

  var
    sl: TStringList;
    Indx: integer;
    S: string;
    i: integer;
  begin
    S := FSections[1].Text;
    S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
    sl := TStringList.Create;
    try
      CreateListFromDelimiter(',', S, sl);
      TrimList(sl);
      for i := 0 to sl.Count - 1 do
      begin
        if SameText('$Image', Sl[i]) then
          Continue;
        Indx := Ilib.Fields.IndexOf(sl[i]);
        if Indx = -1 then
        begin
          Indx := Ilib.Fields.IndexOfCaption(sl[i]);
          if Indx <> -1 then
            sl[i] := Ilib.Fields.Items[Indx].FieldName
          else
          begin
            Comment1 := sl[i];
            Result := E_InvalidFieldName;
            Exit;
          end;
        end;
      end;
      { Check mandatories fields }
      Result := CheckMandatories(sl);
      if Result <> 0 then
        Exit;
      FSections[1].Assign(sl);
    finally
      sl.Free;
    end;
  end;

  function CheckDataType(var Val: string; Def: TILFieldDef): integer;

  var
    N: Extended;
    DT: TDateTime;
  begin
    Result := 0;
    if Def = nil then
      Exit;
    if Ilib.IsStringType(Def.DataType) and not TMan.IsValueNull(Val) then
    begin
      if (Pos('''', Val) <> 1) or (Pos('''', Val, 2) <> Length(Val)) then
      begin
        Result := E_MissingQuote;
        Exit;
      end;
    end
    else if (Def.DataType = dtNumber) and not TMan.IsValueNull(Val) then
    begin
      if not TryStrToFloat(Val, N, Translator.FFormatSettings) then
      begin
        Comment1 := Val;
        Result := E_InvalidNumber;
        Exit;
      end;
    end
    else if (Def.DataType = dtDate) and not TMan.IsValueNull(Val) then
    begin
      if not TryStrToDateTime(Val, DT, Translator.FFormatSettings) then
      begin
        Comment1 := Val;
        Result := E_InvalidDate;
        Exit;
      end;
    end
    else if (Def.DataType = dtBoolean) and not TMan.IsValueNull(Val) then
    begin
      if not SameText(Val, Tman.ReportTrue) and
        not SameText(Val, Tman.ReportFalse) and not SameText(Val, '0')
        and not SameText(Val, '1') then
      begin
        Comment1 := Val;
        Result := E_IllegalValue;
        Exit;
      end;
      if SameText(Val, TMan.ReportTrue) then
        Val := '1'
      else if SameText(Val, Tman.ReportFalse) then
        Val := '0';
    end;
  end;

  function CheckSubLines(Sublines: string; Def: TILFieldDef): integer;

  var
    P: integer;
    sl: TStringList;
    i: integer;
    S: string;
  begin
    Result := 0;
    sl := TStringList.Create;
    try
      P := PosRestrict(';', Sublines, False);
      while P <> 0 do
      begin
        Sublines[P] := #1;
        P := PosRestrict(';', Sublines, False);
      end;
      CreateListFromDelimiter(#1, Sublines, sl);
      TrimList(sl, false);
      if (Def.MaxValues <> 0) and (sl.Count > Def.MaxValues) and not (Def.DataType = dtText) then
      begin
        Comment1 := Def.Caption;
        Result := E_FieldTooManyValues;
        Exit;
      end;
      for i := 0 to sl.Count - 1 do
      begin
        S := sl[i];
        Result := CheckDataType(S, Def);
        if Result <> 0 then
          Exit;
        sl[i] := S;
      end;
    finally
      sl.Free;
    end;
  end;

  function CheckValueList: integer;

  var
    sl: TStringList;
    Indx: integer;
    P: integer;
    S: string;
    Def: TILFieldDef;
    Val: string;
    i: integer;
    HasEmpties : Boolean;
  begin
    Result := 0;
    S := FSections[2].Text;
    S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
    sl := TStringList.Create;
    try
     CreateListFromDelimiter(',', S, sl);
     i := 0;
     HasEmpties := false;
     while i < sl.Count do
     begin
      if SameText(Trim(sl[i]), EmptyValue) then
      begin
        sl.Delete(i);
        HasEmpties := True;
      end
      else
      begin
        sl[i] := sl[i] + ',';
        inc(i);
      end;
     end;
     if HasEmpties then
     begin
       FSections[2].Assign(sl);
       S := FSections[2].Text;
       S := StringReplace(S, #$D#$A, '', [rfReplaceAll]);
       if S[Length(S)] = ','  then
        Delete(S, Length(S), 1);
     end;
     sl.Clear;

     P := PosRestrict(',', S, False);
     while P <> 0 do
     begin
      S[P] := #1;
      P := PosRestrict(',', S, False);
     end;
     CreateListFromDelimiter(#1, S, sl);
     if FSections[1].Count > 0 then
      { Field list exists, check congruence }
     begin
        if sl.Count <> FSections[1].Count then
        begin
          Result := E_MissingValue;
          Exit;
        end;
     end
     else if Ilib.Fields.Count <> sl.Count then
      { Check count congruence for fields }
     begin
        Result := E_MissingInsertFields;
        Exit;
      end;

      for i := 0 to sl.Count - 1 do
      begin
        if FSections[1].Count > 0 then
        begin
          Indx := Ilib.Fields.IndexOf(FSections[1][i]);
          if SameText(FSections[1][i], '$Image') then
            Def := nil
          else
            Def := Ilib.Fields.Items[Indx];
        end
        else
        begin
          Def := Ilib.Fields.Items[i];
        end;
        if (Def <> nil) and Def.IsReadOnly then
        begin
          Comment1 := Def.FieldName;
          Result := E_FieldReadOnly;
          Exit;
        end;
        Val := sl[i];
        Val := TrimBlanks(Val);
        if (Val = '') or (Val = '''' + '''') or TMan.IsValueNull(Val) then
        begin
          Val := TMan.ReportNull;
          sl[i] := TMan.ReportNull;
        end;
        if (Def <> nil) and (Def.FieldType in [ftCalculated, ftAutoIncrement, ftStoredEvaluation]) then
        begin
          Val := TMan.ReportNull;
          sl[i] := TMan.ReportNull;
        end
        else if (Def <> nil) and TMan.IsValueNull(Val) and Def.Mandatory and not SameText(Def.FieldName,
          c_SysFieldNames[SYS_LinkID])
          and not SameText(Def.FieldName, c_SysFieldNames[SYS_Level])
          and not SameText(Def.FieldName, c_SysFieldNames[SYS_SiblingIndex])
          and (Def.InitialValue = '') then
        begin
          Comment1 := Def.Caption;
          Result := E_MissingMandatoryField;
          Exit;
        end

        else
        begin
          if PosRestrict(';', Val, False) <> 0 then
            Result := CheckSubLines(Val, Def)
          else
            Result := CheckDataType(Val, Def);
        end;
        if Result <> 0 then
          Exit;
        Val := StringReplace(Val, '''', '', [rfReplaceAll]);
        sl[i] := Val;
        if Val = '' then
          Val := TMan.ReportNull;
      end;
      FSections[2].Assign(sl);
    finally
      sl.Free;
    end;
  end;

  function PosKeyWord(KeyWord, S: string): integer;
  begin
    Result := PosRestrict(KeyWord, S, False);
    if Result = 0 then
      Exit;
    if Length(S) = Length(KeyWord) then
      Exit;
    if (S[Result + Length(KeyWord)] <> ' ') and
      (S[Result + Length(KeyWord)] <> #$D)
      and not (SameText('VALUES', KeyWord)
        and (S[Result + Length(KeyWord)] = '('))
      then
      Result := 0;
    if (Result <> 0) and (Result <> 1) then
    begin
      if (S[Result-1] <> #$A)
      and(S[Result-1] <> ' ') then
        Result := 0;
    end;
  end;

  procedure CompleteValueList;
  var
    i : integer;
    Indx : integer;
    Map : array of boolean;
  begin
    SetLength(Map, Ilib.Fields.Count);
    for I := 0 to High(Map) do Map[i] := false;
    for I := 0 to FSections[1].Count-1 do
    begin
       if SameText(FSections[1][i], '$Image') then
       begin
         if i <> FSections[1].Count-1 then
           HandleILException(E_ImageSpecifier);
         FSections[2][i] := '$Image' + FSections[2][i];
         continue;
       end;
       Indx := Ilib.Fields.IndexOf(FSections[1][i]);
       begin
         Map[Indx] := True;
       end;
    end;
    for I := 0 to High(Map) do
    begin
      if not Map[i] then
      begin
        FSections[2].Insert(I, TMan.ReportNull)
      end;
    end;
    FSections[1].Clear;
    Finalize(Map);
  end;

  function PosSet(ASet : string; S: string; Start : integer): integer;
  begin
    Result := PosRestrict(ASet, S, False, Start);
    if Result = 0 then
      Exit;
    if Length(S) = Length(ASet) then
      Exit;
    if (S[Result + Length(ASet)] <> ' ') and
      (S[Result + Length(ASet)] <> #$D)
      then
      Result := 0;
    if (Result <> 0) and (Result <> 1) then
    begin
      if (S[Result-1] <> #$A)
      and(S[Result-1] <> ' ') then
        Result := 0;
    end;
  end;

  function ExpandSets(var Expr : string; var Err : integer) : Boolean;
  var
   P, P1, P2 : integer;
   S, S2 : string;
   sl : TStringList;
   FName : string;
   i : integer;
   Indx : integer;
  begin
    Err := 0;
    Result := false;
    P := PosSet('SET', Expr, 1);
    while P <> 0 do
    begin
      { Get the fieldname }
      P1 := P + 4; { Start of FieldName }
      P2 := P1;
      while (P2 < Length(Expr)) and (CharInSet(Expr[P2], ['_', 'a'..'z', 'A'..'Z', '0'..'9'])) do
        inc(P2);
      { Now at end of field name }
      if (Expr[P2] <> '(') and (Expr[P2+1] <> '(') then
      begin
        P := PosSet('SET', Expr, P + 1);
        Continue;
      end;

      Result := True;

      FName := Copy(Expr, P1, P2-P1);
      Indx := Ilib.Fields.IndexOf(FName);
      if Indx = -1 then
      begin
        Comment1 := Fname;
        Err := E_InvalidFieldName;
        Exit;
      end;

      if (Ilib.Fields.Items[Indx].MaxValues = 1) then
      begin
        Comment1 := 'SET';
        Err := E_IllegalMVFOperator;
        Exit;
      end;

      P1 := Pos('(', Expr, P);
      P2 := Pos(')', Expr, P);
      S := Copy(Expr, P1, P2-P1+1); { Get the value list }
      if S = '' then
      begin
        Err := E_MissingParenthesis;
        Exit;
      end;

      System.Delete(Expr, P, P2-P+1); {Delete SET expression }
      System.Delete(S, 1, 1); { Remove parentheses from list }
      System.Delete(S, Length(S), 1);
      if SameText(S, EmptyValue) then
      begin
         S2 := 'SET ' + Fname + ' = ' + EmptyValue;
         Insert(S2, Expr, P);
         P := PosSet('SET', Expr, P+1);
         Continue;
      end;
      sl := TStringList.Create;
      try
      S2 := 'SET ' + Fname + ' = ' + TMan.ReportNull; { First clear the field }
      Insert(S2, Expr, P);
      inc(P, Length(S2));
      if Expr[P] <> #$D then
       Insert(#$D#$A, Expr, P);
      inc(P, 2);

      CreateListFromDelimiter(',', S, sl);
      for i := 0 to sl.Count-1 do
      begin
         S := 'ADD ' + Fname + ' = ' + sl[i];
         Insert(S, Expr, P);
         inc(P, Length(S));
         if i < sl.Count-1 then
         begin
           Insert(#$D#$A, Expr, P);
           inc(P, 2);
         end;
      end;
      finally
        sl.Free;
      end;
      P := PosSet('SET', Expr, P);
    end;
  end;

  begin
    if FExpression.Count = 0 then
    begin
      Result := E_MissingExpression;
      Exit;
    end;

    Result := 0;
    CommandFound := false;
    WhereFound := false;
    for i := 0 to 2 do
      FSections[i].Clear;
    S := FExpression.Text;
    S := TrimBlanks(S, false);
    PU := PosRestrict('  ', S, False);
    while PU <> 0 do
    begin
      Delete(S, PU, 1);
      PU := PosRestrict('  ', S, False);
    end;

    PU := PosKeyWord('UPDATE DATASET', S);
    if PU = 1 then
    begin
      if Ilib.Dataset.Count = 0 then
      begin
        Result := -1;
        Exit;
      end;
      Delete(S, 1, Length('UPDATE DATASET'));
      S := TrimBlanks(S);
      FExpression.Text := S;
      TrimList(FExpression, false);
      S := FExpression.Text;
      if ExpandSets(S, Result) then
      begin
        If Result <> 0 then
          Exit;
        FExpression.Text := S;
      end;
      FUpdateTarget := utrDataSet;
      Exit;
    end;
    PU := PosKeyWord('UPDATE TABLE', S);
    if PU = 1 then
    begin
      FOperation := dboUpdate;
      FUpdateTarget := utrTable;
      TrimList(FExpression, false);
      S := FExpression.Text;
      if (FRecordIndex > -1) and TMan.DLLAccess then
      {DLL logic. Set by UpdateTable which points to a specific record}
      begin
        PW := PosKeyWord('WHERE', S);
        if PW <> 0 then
        begin
          Result := E_NoUpdateWHERE;
          Exit;
        end;
      end;

      if ExpandSets(S, Result) then
      begin
        If Result <> 0 then
          Exit;
        FExpression.Text := S;
      end;
      CommandFound := True;
    end;

    if not CommandFound then
    begin
      PIN := PosKeyWord('INSERT DATASET', S);
      if PIN <> 1 then
        PIN := 0
      else
        FUpdateTarget := utrDataSet;
      if PIN = 0 then
      begin
        PIN := PosKeyWord('INSERT TABLE', S);
        if PIN <> 1 then
          PIN := 0
        else
          FUpdateTarget := utrTable;
      end;
      if PIN <> 0 then
      begin
        FOperation := dboInsert;
        CommandFound := True;
      end;
    end;

    if not CommandFound then
    begin
       PD := PosKeyWord('DELETE DATASET', S);
      if PD <> 1 then
        PD := 0
      else
      begin
        if ILib.Dataset.Count = 0 then
        begin
          Result := -1;
          Exit;
        end;
        FUpdateTarget := utrDataSet;
      end;
      if PD = 0 then
      begin
        PD := PosKeyWord('DELETE TABLE', S);
        if PD <> 1 then
          PD := 0
        else
          FUpdateTarget := utrTable;
      end;
      if PD <> 0 then
      begin
        FOperation := dboDelete;
        CommandFound := True;
      end;
    end;

    if not CommandFound then
    begin
      Result := E_MissingCommand;
      Exit;
    end;

    PW := PosKeyWord('WHERE', S);
    if PW <> 0 then
    begin
      WhereFound := True;
      { No where clause in insert or if target = dataset }
      if (FOperation = dboInsert) or (FUpdateTarget = utrDataSet) then
      begin
        Result := E_InappropriateWhere;
        Exit;
      end;
    end;

    if (PW = 0) and (FOperation <> dboInsert) then
    begin
      FExpression.Add('WHERE');
    end;

    S := FExpression.Text;
    S := TrimBlanks(S, false);

    FSections[0].Clear;

    if (FOperation = dboDelete) then
    begin
      { Section1 catches any garbage between delete and where }
      PW := PosRestrict('WHERE', S, False);
      s2 := Copy(S, 1, PW - 1);
      if FUpdateTarget = utrTable then
        Delete(s2, 1, Length('Delete table'))
      else
        Delete(s2, 1, Length('Delete dataset'));
      s2 := TrimBlanks(s2, false);
      FSections[1].Text := s2;
    end
    else if (FOperation = dboUpdate) then
    begin
      PW := PosKeyWord('WHERE', S);
      s2 := Copy(S, 1, PW - 1);
      Delete(s2, 1, Length('Update table'));
      s2 := TrimBlanks(s2, false);

      {Replace comma delimiters with returns}
      P1 := PosRestrict(',', s2, False);
      while P1 <> 0 do
      begin
        System.Delete(s2, P1, 1);
        Insert(#$D#$A, s2, P1);
        P1 := PosRestrict(',', s2, False, P1);
      end;

      FSections[1].Text := s2;
      { Section 1 = set, add, delete etc }
    end
    else if FOperation = dboInsert then
    begin
      if (FUpdateTarget = utrDataSet) and (ILib is TILTreeViewTable) then
        if Ilib.Dataset.Position = -1 then
        begin
          Result := E_UndefinedLinkRecord;
          Exit;
        end;

      CB := PosKeyWord('CONNECT BY', S);
      CBP := PosKeyWord('CONNECT BY PRIOR', S);
      if CB <> 0 then
      begin
          if (not (Ilib is TilTreeViewTable) and not Ilib.IsDetail)
          or (FUpdateTarget = utrDataset)then
          begin
           if FUpdateTarget = utrDataset then
             Result := E_InappropriateWhere
           else
             Result := E_ConnectByRoot;
           Exit;
          end;
      end
      else if PosRestrict('CONNECT BY', S, False) <> 0 then { Some other place }
      begin
           Result := E_SyntaxError;
           Exit;
      end;

      InsByLiteral := False;
      if CB <> 0 then
      begin
        if CBP <> 0 then
         Delete(S, CB, Length('CONNECT BY PRIOR'))
        else
         Delete(S, CB, Length('CONNECT BY'));
        P1 := PosCaseless('VALUES', S);
        P2 := Pos('(', S);
        if P2 < P1 then
          P1 := P2;
        if CBP = 0 then
        begin
          FSections[0].Text := Copy(S, CB, P1-CB);
          TrimList(FSections[0]);
          { Use searcher to test where clause : }

          if (FSections[0].Count > 0) then
          begin
           if TryStrToInt(FSections[0][0], N) then { Literal }
           begin
              if (FSections[0].Count > 0) then
              begin
               Result := E_ConnectByLiteral;
               Exit;
              end;
              InsByLiteral := True;
           end;

           if not InsByLiteral then
           begin
             sl := TStringList.Create;
             try
               sl.Assign(FSections[0]);
               sl.Insert(0, 'Select *');
               sl.Insert(1, 'Where');
               if Ilib.IsDetail then
                 Result := Ilib.MasterSource.CheckSearchExpression(sl)
               else
                 Result := Ilib.CheckSearchExpression(sl);
               if Result <> 0 then
                 Exit;
             finally
               sl.Free;
             end;
           end;

          end
          else
          begin
            Result := E_MissingExpression;
            Exit;
          end;
        end
        else
        begin
          FSections[0].Text := 'PRIOR';
        end;
      end;

      s2 := '';
      P1 := PosKeyWord('VALUES', S);
      if P1 <> 0 then
        Delete(S, P1, Length('Values'))
      else
      begin
        Result := E_MissingValues;
        Exit;
      end;
      P2 := PosRestrict('(', S, False);
      if P2 = 0 then
      begin
        Result := E_MissingParenthesis;
        Exit;
      end;

      if P2 < P1 then { FieldList start }
      begin
        P1 := P2;
        P2 := PosRestrict(')', S, False, P1);
        if P2 = 0 then
        begin
          Result := E_MissingParenthesis;
          Exit;
        end;
        s2 := Copy(S, P1 + 1, P2 - P1 - 1);
        s2 := TrimBlanks(s2, false);
        Delete(S, P1, P2 - P1 + 1); { Fieldlist in section 1 for insert }
        FSections[1].Text := s2;
        Result := CheckFieldList;
        if Result <> 0 then
          Exit;
      end;
      { Value list : }
      P1 := PosRestrict('(', S, False);
      if P1 = 0 then
      begin
        Result := E_MissingParenthesis;
        Exit;
      end;
      P2 := Pos(')', S);
      if P2 = 0 then
      begin
        Result := E_MissingParenthesis;
        Exit;
      end;
      s2 := Copy(S, P1 + 1, P2 - P1 - 1);
      s2 := TrimBlanks(s2, false);
      FSections[2].Text := s2; { Value list in section 2 for insert }

      Result := CheckValueList;
      if Result <> 0 then
        Exit;

      { Remove field list and insert nulls for fields not included }
      if FSections[1].Count > 0 then
      begin
        CompleteValueList;
      end;
    end;

    if (FOperation <> dboInsert) then
    begin
      { Conditon in section 2 for update and delete}
      PW := PosKeyWord('WHERE', S);
      Inc(PW, Length('WHERE'));
      if PW > Length(S) then
        FSections[2].Clear
      else
      begin
        s2 := Copy(S, PW, Length(S));
        FSections[2].Text := s2;
      end;
    end;

    TrimList(FSections[0], false);
    TrimList(FSections[1], false);
    TrimList(FSections[2], false);

    if (FOperation = dboUpdate) and (FSections[1].Count = 0) then
    begin
      Result := E_MissingUpdateSpec;
      Exit;
    end;

    if (FOperation = dboDelete) and (FSections[1].Count <> 0) then
    begin
      Result := E_SyntaxError;
      Exit;
    end;


    { Use searcher to test where clause : }
    if (FSections[2].Count > 0) and (FOperation <> dboInsert) then
    begin
      sl := TStringList.Create;
      try
        sl.Assign(FSections[2]);
        sl.Insert(0, 'Select *');
        sl.Insert(1, 'Where');
        Result := Ilib.CheckSearchExpression(sl);
      finally
        sl.Free;
      end;
    end
    else if WhereFound then
      Result := E_MissingExpression;
    if Result = 0 then
      FExpression.Assign(FSections[1]);

  end;

  function TUpdater.Prepare: integer;

  var
    i: integer;
  begin
    Result := inherited Prepare;
    if FPrepared then
      Exit;
    if Assigned(FDecomps[0]) then
    begin
      FDecomps[0].Free;
    end;
    Result := GetSections;
    if Result <> 0 then
      Exit;

    if Ilib.IsReadOnly then
    begin
      Comment1 := Ilib.Name;
      Result := E_ReadOnly;
    end
    else if (FUpdateTarget = utrDataSet) and (FOperation = dboUpdate) then
      if Ilib.Dataset.IsReadOnly then
        Result := E_DatasetReadOnly;
    if Result <> 0 then
      Exit;

    if FOperation <> dboUpdate then
      { For the other types the checking and decompositions are done i GetSections }
      Exit;
    FDecomps[0] := TStringList.Create;
    for i := 0 to FExpression.Count - 1 do
    begin
      Result := DecomposeElement(FExpression[i], FDecomps[0]);
      if Result <> 0 then
      begin
        FDecomps[0].Free;
        FDecomps[0] := nil;
        Break;
      end;
    end;
    if Result = 0 then
      FPrepared := True;
  end;

  function TUpdater.DecomposeElement(AnElement: string;
    Decomp: TStringList): integer;

  var
    P: integer;
    N: Extended;
    FldName: string;
    Indx: integer;
    Val1, Val2: string;
    AnAction: string;
    S: string;
    KeyWords: Boolean;
    KeyWord: string;
    NeedQuotes: Boolean;
    DT: TDateTime;
    S2 : string;

    function CheckQuotes(S: string): integer;
    begin
      Result := 0;
      if TMan.IsValueNull(S) or SameText(S, EmptyValue) or (S = '') then
        Exit;
      if (Length(S) < 2) then
      begin
        Comment1 := S;
        Result := E_MissingQuote;
      end
      else if (S[1] <> '''') or (S[Length(S)] <> '''') then
      begin
        Result := E_MissingQuote;
      end;
    end;

    procedure Swap;

    { When expressions come from a record (uaExpression action), the
      filedname must be inserted after the key word }
  var
    P: integer;
    S, s2, s3, s4: string;
  begin
    S := AnElement;
    P := Pos(' ', S);
    if P = 0 then
      Exit;
    s2 := Copy(S, 1, P - 1); { Possibly fieldname }
    Delete(S, 1, P);
    P := Pos(' ', S);
    if P = 0 then
      Exit;
    s4 := Copy(S, 1, P - 1);
    s3 := s4.ToUpper;
    Delete(S, 1, P);
    if (s3 = 'ADD') or (s3 = 'DELETE') OR (s3 = 'REPLACE') or (s3 = 'SET') then
    begin
      AnElement := s3 + ' ' + s2 + ' ' + S;
    end;
  end;

  begin
    Result := 0;
    Swap;
    P := Pos(' ', AnElement);
    if P = 0 then
    begin
      S := AnElement.ToUpper;
    end
    else
    begin
      s2 := Copy(AnElement, 1, P - 1);
      S := s2.ToUpper;
      Delete(AnElement, 1, P);
    end;

    KeyWords := (S = 'ADD') or (S = 'DELETE') OR (S = 'REPLACE') or (S = 'SET');
    if not KeyWords then
    begin
      Result := E_MissingKeyword;
      Exit;
    end
    else
    begin
      KeyWord := S;
      P := Pos(' ', AnElement);
      if (P = 0) and (KeyWord <> 'DELETE') then
      begin
        if (AnElement.ToUpper = '$IMAGE') and (KeyWord = 'SET') then
         Indx := 100
        else Indx := Ilib.Fields.IndexOf(AnElement);
        if Indx = -1 then
        begin
          Comment1 := AnElement;
          Result := E_InvalidFieldName;
        end
        else
        begin
          Comment1 := AnElement;
          Result := E_MissingValue;
        end;
        Exit;
      end
      else if P = 0 then
      begin
        FldName := AnElement;
        P := Length(AnElement);
      end
      else
        FldName := Copy(AnElement, 1, P - 1);

      if (FldName.ToUpper = '$IMAGE') and (KeyWord = 'SET') then
       Indx := 100
      else Indx := Ilib.Fields.IndexOf(FldName);
      if Indx = -1 then
      begin
        Indx := Ilib.Fields.IndexOfCaption(FldName);
        if Indx = -1 then
        begin
          Comment1 := FldName;
          Result := E_InvalidFieldName;
          Exit;
        end
        else
        begin
          FldName := Ilib.Fields.Items[Indx].FieldName;
        end;
      end;

      if (Indx <> 100) and (Ilib.Fields.Items[Indx].IsReadOnly) and not Ilib.FIndexing then
      begin
          Comment1 := FldName;
          Result := E_FieldReadOnly;
          Exit;
      end;

      if KeyWord = 'ADD' then
      begin
        if Ilib.Fields.Items[Indx].MaxValues = 1 then
        begin
          Result := E_AddSingleLineField;
          Exit;
        end;
      end;
      Delete(AnElement, 1, P);
    end;
    NeedQuotes := (Indx = 100) or (Ilib.Fields.Items[Indx].DataType in [dtString, dtText, dtDate,
      dtResource, dtLongText]);
    if Indx <> 100 then
    begin
     AnElement := TrimBlanks(AnElement, not (Ilib.Fields.Items[Indx].DataType in [dtText, dtLongText]));
    end;
    AnElement := TrimOperators(AnElement);

    if KeyWord = 'REPLACE' then
    begin
      P := Pos('=', AnElement);
      if P = 0 then
      begin
        Result := E_MissingOperator;
        Exit;
      end;
    end;

    if (KeyWord = 'DELETE') or (KeyWord = 'ADD') or (KeyWord = 'SET') then
    begin
      P := PosRestrict('=', AnElement, False);
      if P <> 0 then
        AnElement := StringReplace(AnElement, '=', ' ', []);
      AnElement := TrimBlanks(AnElement, (Indx = 100)
       or not (Ilib.Fields.Items[Indx].DataType in [dtText, dtLongText]));
      Val1 := AnElement;
    end
    else
    begin
      Val1 := Copy(AnElement, 1, P - 1);
      Delete(AnElement, 1, P);
      AnElement := TrimBlanks(AnElement, not (Ilib.Fields.Items[Indx].DataType
        in [dtText, dtLongText]));
    end;

    if (Val1 = '') and (KeyWord <> 'DELETE') then
    begin
      if Indx = 100 then
        Comment1 := '$Image'
      else
        Comment1 := Ilib.Fields.Items[Indx].Caption;
      Result := E_MissingValue;
      Exit
    end;

    if (KeyWord = 'DELETE') or (KeyWord = 'ADD') or (KeyWord = 'SET') then
    begin
      if NeedQuotes then
        Result := CheckQuotes(Val1);
      if Result <> 0 then
        Exit;
      if KeyWord = 'DELETE' then
        Val2 := TMan.ReportNull
      else
      begin
        Val2 := Val1;
        Val1 := TMan.ReportNull;
      end;
    end
    else
      Val2 := AnElement;

    if not NeedQuotes and ((Pos('''', Val1) <> 0) or (Pos('''', Val2) <> 0))
    then
    begin
      if Indx = 100 then
        Comment1 := '$Image'
      else
       Comment1 := Ilib.Fields.Items[Indx].FieldName;
      Result := E_QuotesNotNeeded;
      Exit;
    end;

    if (KeyWord = 'DELETE') and ((Val1 = '') or TMan.IsValueNull(Val1) or SameText(Val1, EmptyValue)) then
    { Delete without a value means delete all }
    begin
      if NeedQuotes then
        Val1 := '''' + '*' + ''''
      else
        Val1 := '*';
    end
    { Check type values }
    else if (Indx <> 100) and (Ilib.Fields.Items[Indx].DataType = dtBoolean) then
    begin
      if SameText(Val1, TMan.ReportTrue) then
        Val1 := '1'
      else if SameText(Val1, Tman.ReportFalse) then
        Val1 := '0';
      if SameText(Val2, TMan.ReportTrue) then
        Val2 := '1'
      else if SameText(Val2, TMan.ReportFalse) then
        Val2 := '0';
      if not TMan.IsValueNull(Val1) and not SameText(Val1, EmptyValue) and (Val1 <> '') then
      begin
        if (Val1 <> '0') and (Val1 <> '1') then
        begin
          Comment1 := Val1;
          Result := E_IllegalValue;
          Exit;
        end;
      end;
      if not TMan.IsValueNull(Val2) and not SameText(Val2, EmptyValue) and (Val2 <> '') then
      begin
        if (Val2 <> '0') and (Val2 <> '1') then
        begin
          Comment1 := Val2;
          Result := E_IllegalValue;
          Exit;
        end;
      end;
    end
    else if (Indx <> 100) and (Ilib.Fields.Items[Indx].DataType = dtNumber) then
    begin
      if not TMan.IsValueNull(Val1) and not not SameText(Val1, EmptyValue) and (Val1 <> '') then
      begin
        if not TryStrToFloat(Val1, N,Translator.FFormatSettings ) then
        begin
          Comment1 := Val1;
          Result := E_InvalidNumber;
          Exit;
        end;
      end;
      if not TMan.IsValueNull(Val2) and not SameText(Val2, EmptyValue) and (Val2 <> '') then
      begin
        if not TryStrToFloat(Val2, N, Translator.FFormatSettings) then
        begin
          Comment1 := Val2;
          Result := E_InvalidNumber;
          Exit;
        end;
      end;
    end;

    if (Val1 = '') or (Val1 = '''' + '''') then
      Val1 := TMan.ReportNull;
    if not TMan.IsValueNull(Val1) and NeedQuotes and (Val1.ToUpper <> EmptyValue.ToUpper) then
    begin
      Result := CheckQuotes(Val1);
      if Result <> 0 then
        Exit;
      Delete(Val1, 1, 1);
      Delete(Val1, Length(Val1), 1);
      if Val1 = '' then
        Val1 := TMan.ReportNull;
    end;
    if (Val2 = '') or (Val2 = '''' + '''') then
      Val2 := TMan.ReportNull;
    if not TMan.IsValueNull(Val2) and (Val2.ToUpper <> EmptyValue.ToUpper) then
    begin
      if NeedQuotes then
      begin
        Result := CheckQuotes(Val2);
        Delete(Val2, 1, 1);
        Delete(Val2, Length(Val2), 1);
      end;
    end;

    { Check date }
    if (Indx <> 100) and (Ilib.Fields.Items[Indx].DataType = dtDate) then
    begin
      if (KeyWord <> 'DELETE') and (KeyWord <> 'REPLACE') and
        not TMan.IsValueNull(Val1) and not SameText(Val1, EmptyValue) then { Wilcards allowed on Value 1 }
      begin
        if not TryStrToDateTime(Val1, DT, Translator.FFormatSettings) then
        begin
          Comment1 := Val1;
          Result := E_InvalidDate;
          Exit;
        end;
      end;
      if not TMan.IsValueNull(Val2) and not SameText(Val2, EmptyValue) then
      begin
        if not TryStrToDateTime(Val2, DT, Translator.FFormatSettings) then
        begin
          Comment1 := Val2;
          Result := E_InvalidDate;
          Exit;
        end;
      end;
    end;

    AnAction := KeyWord;
    if Val2 <> EmptyValue then
      Decomp.Add(AnAction + '~' + FldName + '~' + Val1 + '~' + Val2);
  end;

  function TUpdater.TrimOperators(AnExpression: string): string;
  begin
    while PosRestrict('= ', AnExpression, False) <> 0 do
      AnExpression := StringReplace(AnExpression, '= ', '=', [rfReplaceAll]);
    while PosRestrict(' =', AnExpression, False) <> 0 do
      AnExpression := StringReplace(AnExpression, ' =', '=', [rfReplaceAll]);
    Result := AnExpression;
  end;

  function TUpdater.RecordToExpression(ARecord: TILRecord): TStringList;

  var
    i, j: integer;
    S, s2, s3: string;
    qt: string;
    P: integer;
    OrigVal: string;
    OpState: integer;
  const
    m_Add = '>';
    m_Delete = '<';

    function GetMethod(var Value: string): TUpdateMethod;
    begin
      if Pos(m_Add, S) = 1 then
      begin
        Result := umAddValue;
      end
      else if Pos(m_Delete, S) = 1 then
      begin
        Result := umRemoveValue;
      end
      else if PosRestrict('=', Value, False) <> 0 then
      begin
        Result := umReplaceValue;
      end
      else
        Result := umReplaceFields;
      if Result in [umAddValue, umRemoveValue] then
      begin
        Delete(Value, 1, 1);
        Value := TrimBlanks(Value, false);
      end;
    end;

    function GetOpState(Indx: integer): integer;

  var
    j: integer;
    OpFound: Boolean;
    OpNotFound: Boolean;
  begin
    { 0 No ops found
      1 Mix
      2 Ops found }
    Result := 0;
    OpFound := false;
    OpNotFound := false;
    begin
      for j := 0 to ARecord[Indx].Count - 1 do
      begin
        if Pos(m_Add, ARecord[Indx][j]) = 1 then
          OpFound := True
        else if Pos(m_Delete, ARecord[Indx][j]) = 1 then
          OpFound := True
        else if PosRestrict('=', ARecord[Indx][j], False) <> 0 then
          OpFound := True
        else
          OpNotFound := True;
        if OpFound and OpNotFound then
        begin
          Result := 1; { Mix }
          Exit;
        end;

      end;
    end;
    if OpFound then
      Result := 2
  end;

  begin
    Result := TStringList.Create;
    FUpdateTarget := Ilib.UpdateTarget;
    if FUpdateTarget = utrDataSet then
      Result.Add('UPDATE DATASET')
    else
      Result.Add('UPDATE TABLE');

    for i := 0 to ARecord.Count - 1 do
    begin
      OpState := GetOpState(i);
      if OpState = 1 then
      begin
        FError := E_UpdateOperators;
        Exit;
      end;

      if ARecord[i].FExpression.Count > 0 then
        ARecord[i].FValues.Assign(ARecord[i].FExpression);

      if ARecord[i].FieldDef.DataType in [dtString, dtText, dtDate, dtResource, dtLongText]
      then
        qt := ''''
      else
        qt := '';

      if OpState = 0 then
      begin
        S := 'DELETE ' + ARecord[i].FieldName;
        Result.Add(S);
      end;

      for j := 0 to ARecord[i].Count - 1 do
      begin
        { Get update method }
        S := ARecord[i][j];
        OrigVal := S;
        FUpdateMethod := GetMethod(S);

        if (Pos('''', S) = 1) and (Pos('''', S, 2) = Length(S)) then
        begin
          Delete(S, 1, 1);
          Delete(S, Length(S), 1);
        end;
        ARecord[i].FValues[j] := S;

        case FUpdateMethod of
          umReplaceAll, umReplaceFields:
            begin
              if (ARecord[i].FieldDef.MaxValues = 1) then
                S := 'SET ' + ARecord[i].FieldName + ' ' + qt +
                  ARecord[i][j] + qt
              else
                S := 'ADD ' + ARecord[i].FieldName + ' ' + qt +
                  ARecord[i][j] + qt;
            end;
          umAddValue:
            begin
              if (ARecord[i].FieldDef.MaxValues = 1) then
                S := 'SET ' + ARecord[i].FieldName + ' ' + qt +
                  ARecord[i][j] + qt
              else
                S := 'ADD ' + ARecord[i].FieldName + ' ' + qt +
                  ARecord[i][j] + qt
            end;
          umRemoveValue:
            begin
              S := 'DELETE ' + ARecord[i].FieldName + ' ' + qt +
                ARecord[i][j] + qt;
            end;
          umReplaceValue:
            begin
              S := ARecord[i][j];
              P := Pos('=', S);
              if P <> 0 then
              begin
                s2 := Copy(S, 1, P - 1);
                s2 := TrimBlanks(s2);
                if (Pos('''', s2) = 1) and (Pos('''', s2, 2) = Length(s3)) then
                begin
                  Delete(s2, 1, 1);
                  Delete(s2, Length(s2), 1);
                end;
                s3 := Copy(S, P + 1, Length(S));
                s3 := TrimBlanks(s3);
                if (Pos('''', s3) = 1) and (Pos('''', s3, 2) = Length(s3)) then
                begin
                  Delete(s3, 1, 1);
                  Delete(s3, Length(s3), 1);
                end;
                S := 'REPLACE ' + ARecord[i].FieldName + ' ' + qt + s2 + qt +
                  '=' + qt + s3 + qt;
              end
              else
                S := 'REPLACE ' + ARecord[i].FieldName + ' ' + qt +
                  ARecord[i][j] + qt; { Must contain = val1 val2 }
            end;
          umExpression:
            begin
              S := ARecord[i].FieldName + ' ' + ARecord[i][j];
              { Must contain a valid expression evt with quotes }
            end;
        end;
        Result.Add(S);
        ARecord[i].FValues[j] := OrigVal;
      end;
    end;
  end;

  { TFileContainer ------------------------------------------------------------
    This class reads and writes multiple files stored within one physical file }

  function TFileContainer.GetCount: integer; { Number of files }

  var
    P: int64;
  begin
    P := FStream.Position;
    FStream.Position := 0;
    FStream.Read(Result, SizeOf(integer));
    FStream.Position := P;
  end;

  constructor TFileContainer.Create(const FileName: string; const MaxFiles: integer;
    const ReadOnly: Boolean);

  var
    Cnt: integer;
    i: integer;
    Info: TFileSpecEx;
  begin
    FFileName := FileName;
    FMaxFiles := MaxFiles;
    if FileExists(FileName) then
    begin
      if ReadOnly then
        FStream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone)
      else
        FStream := TFileStream.Create(FileName, fmOpenReadWrite or
          fmShareDenyNone);
      Cnt := Count;
      for i := 0 to Cnt - 1 do
      begin
        SetLength(FFiles, Length(FFiles) + 1);
        Info := ReadHeader(i);
        FFiles[High(FFiles)] := TSubFile.Create(Info.FileName, Self);
      end;
      { Link index tables }
      for i := 0 to Cnt - 1 do
      begin
        if (FFiles[i].RecordSpace <= 0) or FFiles[i].Paged then
        begin
          FFiles[i].FIndexTable :=
            Files[IndexOf(FFiles[i].FFileName + '_IndexTable')];
        end;
      end;
    end
    else
    begin
      FStream := TFileStream.Create(FileName, fmCreate);
      FStream.Size := MaxFiles * SizeOf(TFileSpecEx) + SizeOf(integer);
      Cnt := 0;
      FStream.Position := 0;
      FStream.Write(Cnt, SizeOf(integer));
    end;
  end;

  destructor TFileContainer.Destroy;

  var
    i: integer;
    Cnt: integer;
  begin
    Cnt := Count;
    for i := 0 to Cnt - 1 do
      Files[i].Free;
    inherited Destroy;
    FStream.Free;
  end;

  function TFileContainer.IsEmpty : Boolean;
  var
    i : integer;
    Cnt : integer;
  begin
     Result := True;
     Cnt := Count;
     for I := 0 to Cnt-1 do
     begin
       if Files[i].Count > 0 then
       begin
         Result := false;
         Break;
       end;
     end;

  end;

  function TFileContainer.Add(FileSpec: TFileSpec): TSubFile; { Adds a file }

  var
    Cnt: integer;
    Buf: integer;
    FSpec: TFileSpecEx;
    Dup: TDuplicates;
    Sort: Boolean;
    RS : integer;
  begin
    if (FileSpec.RecordSpace <= 0) and FileSpec.FixedStringSize then
      HandleILException(E_FixedNotStatic);
    { Fixed string must have a static recordspace }
    FStream.Position := 0;
    FStream.Read(Cnt, SizeOf(integer));
    FStream.Position := SizeOf(integer) + (Count * SizeOf(TFileSpecEx));
    FSpec.FileSize := 0;
    FSpec.IndexTable := (Pos('_IndexTable', String(FileSpec.FileName)) <> 0);
    FSpec.FileName := FileSpec.FileName;
    FSpec.Sorted := FileSpec.Sorted;
    FSpec.Duplicates := FileSpec.Duplicates;
    FSpec.RecordSpace := FileSpec.RecordSpace;
    if FSpec.RecordSpace = c_PageBase then
      RS := 700
    else if FSpec.RecordSpace > c_PageBase then
      RS := FSpec.RecordSpace - c_PageBase
    else if FSpec.RecordSpace <= 0 then
      RS := 700
    else
      RS := FSpec.RecordSpace;
    FSpec.FileSpace := RS * 100;

    FSpec.FixedStringSize := FileSpec.FixedStringSize;
    FSpec.Offset := FStream.Size;
    Dup := FileSpec.Duplicates;
    Sort := FileSpec.Sorted;
    FStream.Write(FSpec, SizeOf(TFileSpecEx));
    FStream.Size := FStream.Size + FSpec.FileSpace;
    FStream.Position := 0;
    Buf := Cnt + 1;
    FStream.Write(Buf, SizeOf(integer));
    Result := TSubFile.Create(FSpec.FileName, Self);
    SetLength(FFiles, Length(FFiles) + 1);
    FFiles[High(FFiles)] := Result;
    Result.Position := 0;
    if (FSpec.RecordSpace <= 0) or (FSpec.RecordSpace >= c_PageBase)
    then { Index table needed }
    begin
      FileSpec.Sorted := Sort;
      FileSpec.FileName := FSpec.FileName + '_IndexTable';
      FileSpec.FixedStringSize := True;
      FileSpec.Sorted := Sort;
      FileSpec.Duplicates := Dup;
      FileSpec.RecordSpace := 10;
      FileSpec.FileSpace := 1000;
      Result.FIndexTable := Add(FileSpec);
    end;
  end;

  function TFileContainer.ReadHeader(const Index: integer): TFileSpecEx;

  var
    P: integer;
  begin
    P := SizeOf(integer) + Index * SizeOf(TFileSpecEx);
    FStream.Position := P;
    FStream.Read(Result, SizeOf(TFileSpecEx));
  end;

  procedure TFileContainer.WriteHeader(const Index: integer; Info: TFileSpecEx);

  var
    Exp: int64;
    i: integer;
    Cnt: integer;
    AddIn : integer;

    function GetAddIn : Int64;
    var
     RS : integer;
    begin
       Result := 0;
       if Info.RecordSpace <= 0 then
         RS := 700
       else
         RS := Info.RecordSpace;
       if Info.FileSize > RS * 100 then
         Result := RS * 200
       else if Info.FileSize > RS * 200 then
         Result := RS * 300
       else if Info.FileSize > RS * 300 then
         Result := RS * 400;
       if Result = 0 then
         Result := RS * 100;

    end;
  begin
    Exp := 0;
    if Info.FileSize > Info.FileSpace then
    begin
      AddIn := GetAddIn;
      Exp := (Info.FileSize - Info.FileSpace) + AddIn;
      Grow(Info.Offset + Info.FileSpace, Exp);
      Info.FileSpace := Info.FileSpace + Exp;
    end;
    FStream.Position := SizeOf(integer) + Index * SizeOf(TFileSpecEx);
    FStream.Write(Info, SizeOf(TFileSpecEx));
    { Increment offset for subsequent files }
    Cnt := Count;
    if Exp > 0 then
      for i := Index + 1 to Cnt - 1 do
      begin
        Info := ReadHeader(i);
        Inc(Info.Offset, Exp);
        FStream.Position := SizeOf(integer) + i * SizeOf(TFileSpecEx);
        FStream.Write(Info, SizeOf(TFileSpecEx));
        FFiles[i].FFileSpec := Info;
      end;
  end;

  function TFileContainer.GetFiles(Index: integer): TSubFile;
  begin
    Result := FFiles[Index];
  end;

  procedure TFileContainer.Grow(const FromPosition: int64; const ByteCount: integer);

  { Adds space to a file }
  var
    MS: TMemoryStream;
  begin
    FStream.Position := FromPosition;
    MS := TMemoryStream.Create;
    try
      MS.CopyFrom(FStream, FStream.Size - FromPosition);
      FStream.Size := FStream.Size + ByteCount;
      FStream.Position := FromPosition + ByteCount;
      MS.Position := 0;
      FStream.CopyFrom(MS, MS.Size);
    finally
      MS.Free;
    end;
  end;

  function TFileContainer.IndexOf(const FileName: shortstring): integer;

  var
    i: integer;
    Cnt: integer;
    Info: TFileSpecEx;
  begin
    Cnt := Count;
    Result := -1;
    for i := 0 to Cnt - 1 do
    begin
      Info := ReadHeader(i);
      if SameText(FileName, Info.FileName) then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  { TSubFile -----------------------------------------------------------------
    The files witin the container }

  procedure TSubFile.SetPosition(const Value : int64);
  begin
    FStream.Position := FFileSpec.Offset + Value;
    FPosition := Value;
  end;

  function TSubFile.GetPosition: int64;
  begin
    if (FStream.Position >= Offset) and (FStream.Position <= Size + Offset) then
      Result := FStream.Position - Offset
    else
      Result := FPosition;
  end;

  procedure TSubFile.SetSize(const Value: int64);
  begin
    FFileSpec.FileSize := Value;
    FOwner.WriteHeader(FOwner.IndexOf(FFileName), FFileSpec);
    FFileSpec := FOwner.ReadHeader(FOwner.IndexOf(FFileName));
  end;

  function TSubFile.GetSize: int64;
  begin
    Result := FFileSpec.FileSize;
  end;

  function TSubFile.GetOffset: int64;
  begin
    Result := FFileSpec.Offset;
  end;

  function TSubFile.GetSpace: int64;
  begin
    Result := FFileSpec.FileSpace;
  end;

  function TSubFile.GetIsIndexTable: Boolean;
  begin
    Result := FFileSpec.IndexTable;
  end;

  function TSubFile.GetSorted: Boolean;
  begin
    Result := FFileSpec.Sorted;
  end;

  function TSubFile.GetDuplicates: TDuplicates;
  begin
    Result := FFileSpec.Duplicates;
  end;

  function TSubFile.GetRecordSpace: integer;
  begin
    if FFileSpec.RecordSpace >= c_PageBase then {Paged }
    begin
     Result := FFileSpec.RecordSpace - c_PageBase;
    end
    else
     Result := FFileSpec.RecordSpace;
  end;

  function TSubFile.GetFixedStringSize: Boolean;
  begin
    Result := FFileSpec.FixedStringSize;
  end;

  function TSubFile.GetCount: int64; { Counts the number of records }
  var
    i, p : integer;
    TR : TTransPos;
    TblName : string;
  begin
    if (RecordSpace > 0) and not Paged then
    begin
      if not FixedStringSize then
        Result := Size div (SizeOf(integer) * 2 + RecordSpace)
      else
        Result := Size div RecordSpace;
    end
    else
    begin
      Result := FIndexTable.Size div 10;
    end;
    if (Tman.TransLogbuffer.Size > 0) and (Pos('_Data', FFilename) = Length(FFileName)-4)  then
    for I := 0 to TMan.TransLogIndex.Count-1 do
    begin
       P := Pos('_Data', FFileName);
       TblName := Copy(FFileName, 1, P-1);
       TR := TTransPos(Tman.TransLogIndex.Objects[i]);
       if (TR.Operation = dboInsert) and Sametext(Tr.TableName, TblName) then
         inc(Result);
    end;
  end;

  function TSubFile.GetIndex: integer;
  begin
    Result := FOwner.IndexOf(FFileName);
  end;

  constructor TSubFile.Create(const FileName: shortstring; OwnerFile: TFileContainer);
  begin
    FStream := OwnerFile.FStream;
    FFileName := FileName;
    FOwner := OwnerFile;
    FFileSpec := FOwner.ReadHeader(FOwner.IndexOf(FFileName));
    FPaged := (FFileSpec.RecordSpace >= c_PageBase);
  end;

  function TSubFile.GetPagedRecord(const PagePos : integer) : string;
  var
    PgSize, Lng, NextPg : integer;
    SS : TStringStream;
  begin
    Result := '';
    FStream.Position := PagePos;
    FStream.Read(PgSize, SizeOf(Integer));
    FStream.Read(Lng, SizeOf(Integer));
    FStream.Read(NextPg, SizeOf(Integer));
    if Lng = 0 then
      Exit;
    SS := GetSS('');
    try
     SS.CopyFrom(FStream, Lng);
     SS.Position := 0;
     Result := SS.ReadString(SS.Size);
     if (NextPg > 0) and (Lng = PgSize-SizeOf(Integer)*2-SizeOf(Integer)) then
     begin
      Result := Result + GetPagedRecord(NextPg);
     end;
    finally
      SS.Free;
    end;
  end;

  function TSubFile.SetPagedRecord(const Value : string; PagePos : integer) : integer;
  var
    SS : TStringStream;
    PS : Integer;
    RS : Integer;
    NextPage : Integer;
    NetFrame : Word;
    ThisStr, Rest : string;
    MS : TMemoryStream;
  begin
     { First 4 bytes are page frame size, next 4 is actual datasize within page,
     last 4 bytes point to next page if any }
     if PagePos = 0 then
     { A new page is added }
     begin
       FPageN := True;
       MS := StringToStream(Value);
       FPageN := false;
       try
        Result := AddStream(MS);
       finally
         MS.Free;
       end;
       Exit;
     end;
     FStream.Position := PagePos;
     Result := PagePos;
     FStream.Read(PS, SizeOf(Integer)); { Get total page size}
     Rest := '';
     ThisStr := '';
     NetFrame := PS - (Sizeof(Integer)*3); { The netto frame size - length specifiers }
     if Fowner.Unicode then
       SS := TStringStream.Create(Value, TEncoding.Utf8)
     else
       SS := TStringStream.Create(Value);
     RS := SS.Size;
     SS.Free;
     if RS > NetFrame then {Text too long to fit frame}
     begin
       ThisStr := Copy(Value, 1, NetFrame);
       Rest := Copy(Value, NetFrame+1, RS);
       RS := NetFrame;
     end
     else
     begin
       ThisStr := Value;
     end;
     FStream.Position := Result + SizeOf(Integer)*2;
     FStream.Read(NextPage, SizeOf(Integer));
     FStream.Position := Result + Sizeof(Integer)*3;
     if Fowner.Unicode then
       SS := TStringStream.Create(ThisStr, TEncoding.Utf8)
     else
       SS := TStringStream.Create(ThisStr);
     try
     SS.Position := 0;
     FStream.CopyFrom(SS, SS.Size);
     FStream.Position := Result + SizeOf(Integer);
     FStream.Write(RS, SizeOf(Integer));
     if Rest <> '' then
     begin
        if NextPage < 0 then
          NextPage := Abs(NextPage);
        NextPage := SetPagedRecord(Rest, NextPage);
        FStream.Position := Result + Sizeof(Integer)*2;
        FStream.Write(NextPage, SizeOf(Integer));
     end
     else if NextPage > 0 then
     begin
       FStream.Position := Result + Sizeof(Integer)*2;
       NextPage := -NextPage;
       FStream.Write(NextPage, SizeOf(Integer));
     end;
     finally
       SS.Free;
     end;
  end;

  function TSubFile.GetItems(Index: integer): string;

  var
    L: integer;
    SS: TStringStream;
    P : integer;
    LInt : integer;
    Found : Boolean;
  begin
    if (Tman.TransLogBuffer.Size > 0) then
    begin
      Result := GetFromTransLog(Index);
      if Result <> '' then
        Exit;
    end;

    if (Index < 0) or (Index > Count-1) then
    begin
      HandleILException(E_RecordNotFound, IntToStr(Index));
    end;

    LInt := Index;

    if Paged then
    begin
      Position := PositionOfRecord(LInt);
      Result := GetPagedRecord(FStream.Position);
      Exit;
    end;

    Found := False;
    SS := GetSS('');
    try
      while not Found do
      begin
        if not FixedStringSize then
        begin
          L := SizeOfData(LInt);
        end
        else
          L := RecordSpace;
        if L = 0 then
        begin
          Result := '';
          Exit;
        end;

        Position := PositionOfRecord(LInt);
        if not FixedStringSize then
          Position := Position + SizeOf(integer) * 2;
        SS.CopyFrom(FStream, L);
        SS.Position := 0;
        Result := SS.ReadString(L);
        P := Pos('{Mov', Result);
        if P = 1 then
        begin
          P := Pos('}', Result);
          LInt := StrToInt(Copy(Result,5, P-5));
          SS.Clear;
        end
        else
          Found := True;
      end;
    finally
      SS.Free;
    end;
  end;

  function TSubFile.GetAbsItems(Index: integer): string;
  var
    L: integer;
    SS: TStringStream;
  begin
    if (Tman.TransLogBuffer.Size > 0) then
    begin
      Result := GetFromTransLog(Index);
      if Result <> '' then
        Exit;
    end;

    if Paged then
    begin
      Result := Items[Index];
      Exit;
    end;
    SS := GetSS('');
    try
        if not FixedStringSize then
        begin
          L := SizeOfData(Index);
        end
        else
          L := RecordSpace;
        if L = 0 then
        begin
          Result := '';
          Exit;
        end;
        Position := PositionOfRecord(Index);
        if not FixedStringSize then
          Position := Position + SizeOf(integer) * 2;
        SS.CopyFrom(FStream, L);
        SS.Position := 0;
        Result := SS.ReadString(L);
    finally
      SS.Free;
    end;
  end;

  procedure TSubFile.SetSorted(const Value : Boolean);
  begin
    FFileSpec.Sorted := Value;
    FOwner.WriteHeader(Index, FFileSpec);
  end;

  function TSubFile.HasSpace(const Index : Integer; const Arecord : string) : integer;
  var
    Ind : integer;
  begin
     Result := -1;
     if Paged then Exit;
     if RecordSpace <= 0 then
     begin
       Ind := RealIndex(Index);
       if (Length(ARecord) + (SizeOf(integer) * 2) > SizeOfRecord(Ind)) then
         Result := Ind;
     end;
  end;

  procedure TSubFile.ReplaceAll(AStream : TMemoryStream);
  begin
    Size := AStream.Size;
    AStream.Position := 0;
    Position := 0;
    FStream.CopyFrom(AStream, AStream.Size);
  end;

  function TSubFile.RealIndex(const SourceIndex: Integer) : integer;
  var
    L: integer;
    SS: TStringStream;
    P : integer;
    LInt : integer;
    Found : Boolean;
    S : string;
  begin
    if Paged then
    begin
      Result := SourceIndex;
      Exit;
    end;
    LInt := SourceIndex;
    Found := False;
    Result := -1;
    SS := GetSS('');
    try
      while not Found do
      begin
        L := SizeOfData(LInt);
        if L = 0 then
        begin
          Exit;
        end;

        Position := PositionOfRecord(LInt);
        Position := Position + SizeOf(integer) * 2;
        SS.CopyFrom(FStream, L);
        SS.Position := 0;
        S := SS.ReadString(L);
        P := Pos('{Mov', S);
        if P = 1 then
        begin
          P := Pos('}', S);
          LInt := StrToInt(Copy(S,5, P-5));
          SS.Clear;
        end
        else
          Found := True;
      end;
      Result := LInt;
    finally
      SS.Free;
    end;
  end;

  procedure TSubFile.SetItems(Index: integer; Value: string);
  var
    NewSpace: integer;
    OldSpace: integer;
    SS: TStringStream;
    S : string;
    Ind : integer;
    Lng : integer;
    P : integer;

  begin
    if Sorted then
      HandleILException(E_ChangeSorted);
    if RecordSpace <= 0 then
     Index := RealIndex(Index);

    if Paged then
    begin
       Position := PositionOfRecord(Index);
       SetPagedRecord(Value, FStream.Position);
       Exit;
    end;

    Position := PositionOfRecord(Index) + SizeOf(integer);
    FStream.Read(OldSpace, SizeOf(integer));
    SS := GetSS(Value);
    Lng := SS.Size;
    SS.Free;

    if Lng + (SizeOf(integer) * 2) > SizeOfRecord(Index) then
    begin
      if (RecordSpace > 0) then
      begin
        HandleILException(E_SpaceOverflow);
      end;
      Ind := Add(Value);
      S := '{Mov' + IntToStr(Ind) + '}';
      Value := S;
      SS := GetSS(Value);
      Lng := SS.Size;
      SS.Free;
      NewSpace := Lng + Abs(RecordSpace) + SizeOf(integer) * 2;
    end
    else
      NewSpace := OldSpace;
    SS := GetSS(Value);
    try
      P := SS.Size;
      Position := PositionOfRecord(Index);
      if not FixedStringSize then
      begin
        FStream.Write(P, SizeOf(integer));
        FStream.Write(NewSpace, SizeOf(integer));
      end;
      SS.Position := 0;
      FStream.CopyFrom(SS, SS.Size)
    finally
      SS.Free;
    end;
  end;

  procedure TSubFile.SaveToStream(const Index: integer; Stream: TMemoryStream);

  var
    Cnt: integer;
  begin
    Cnt := SizeOfRecord(Index);
    Position := PositionOfRecord(Index);
    Stream.CopyFrom(FStream, Cnt);
  end;

  procedure TSubFile.InsertStream(Source: TStream; const AtPosition: int64);
  begin
    Size := Size + Source.Size;
    MoveDown(AtPosition, Source.Size);
    Position := AtPosition;
    Source.Position := 0;
    FStream.CopyFrom(Source, Source.Size);
  end;

  procedure TSubFile.DeleteStream(const FromPosition: int64; const ByteCount: int64);
  begin
    MoveUp(FromPosition, ByteCount);
    Size := Size - ByteCount;
  end;

  function TSubFile.AddStream(Source: TStream) : int64;
  begin
    Size := Size + Source.Size;
    Position := Size - Source.Size;
    Result := FStream.Position;
    Source.Position := 0;
    FStream.CopyFrom(Source, Source.Size);
  end;

  function TSubFile.PositionOfRecord(const Index: integer): int64;

  var
    SS: TStringStream;
    S: string;
  begin
    if Assigned(FIndexTable) then
    { Variable record size }
    begin
      SS := TStringStream.Create;
      try
        FIndexTable.Position := Index * 10;
        SS.CopyFrom(FIndexTable.FStream, 10);
        SS.Position := 0;
        S := SS.ReadString(10);
        Result := StrToInt(S);
      finally
        SS.Free;
      end;
    end
    else if not FixedStringSize then
      Result := (SizeOf(integer) * 2 + FFileSpec.RecordSpace) * Index
    else
      Result := FFileSpec.RecordSpace * Index
  end;

  procedure TSubFile.MoveDown(const FromPosition: int64; const ByteCount: int64);

  var
    MS: TMemoryStream;
  begin
    Position := FromPosition;
    MS := TMemoryStream.Create;
    try
      MS.CopyFrom(FStream, Size - ByteCount - FromPosition);
      Position := FromPosition + ByteCount;
      MS.Position := 0;
      FStream.CopyFrom(MS, MS.Size);
    finally
      MS.Free;
    end;
  end;

  procedure TSubFile.MoveUp(const FromPosition: int64; const ByteCount: int64);

  var
    MS: TMemoryStream;
  begin
    Position := FromPosition;
    MS := TMemoryStream.Create;
    try
      MS.CopyFrom(FStream, Size - FromPosition);
      Position := FromPosition - ByteCount;
      MS.Position := 0;
      FStream.CopyFrom(MS, MS.Size);
    finally
      MS.Free;
    end;
  end;

  function TSubFile.GetSS(const Value : string) : TStringStream;
  begin
    if FOwner.Unicode and not IsIndexTable and not FixedStringSize then
      Result := TStringStream.Create(Value, TEncoding.UTF8)
    else
      Result := TStringStream.Create(Value);
  end;

  function TSubFile.PageSize : integer;
  begin
      Result := Recordspace;
  end;

  function TSubFile.GetFromTransLog(Index : integer) : string;
  var
    S : String;
    P : integer;
    Idx : integer;
    TP : TTransPos;
    SS : TStringStream;
  begin
    Result := '';
    S := FFilename;
    P := Pos('_', S);
    S := Copy(S, 1, P-1);
    S := S + FillZeroes(IntToStr(Index), 7);
    if Tman.TransLogIndex.Find(S, Idx) then
    begin
      TP := TTransPos(Tman.TransLogIndex.Objects[Idx]);
      Tman.TransLogBuffer.Position := TP.Position;
      if Tman.Unicode then
        SS := TStringStream.Create('', TEncoding.UTF8)
      else
        SS := TStringStream.Create;
      try
      SS.CopyFrom(Tman.TransLogBuffer, TP.Size);
      SS.Position := 0;
      Result := SS.ReadString(TP.Size);
      P := Pos('##', Result);
      Result := Copy(Result, 1, P-1);
      P := Pos('<Rc>', Result);
      System.Delete(Result, 1, P-1);
      finally
        SS.Free;
      end;
      Tman.TransLogBuffer.Position := Tman.TransLogBuffer.Size;
    end;
  end;

  function TSubFile.StringToStream(const AString: string; FreeSpace : integer = 0): TMemoryStream;

  var
    L: integer;
    SS: TStringStream;
    Sint : integer;
    Diff : integer;
    PSize : integer;
  begin
    SS := GetSS('');
    try
      SS.WriteString(AString);
      L := SS.Size;
      Result := TMemoryStream.Create;
      if Paged then
      begin
        if FPageN and (PageSize <> 0 )then {Subsequent pages get a page size of string length + 50}
        begin
           SInt := L + 50;
        end
        else
          SInt := PageSize;

        PSize := SInt;
        Result.Write(PSize, SizeOf(Integer)); { page size }
        Result.Write(L, SizeOf(Integer)); { Length of string}
        SInt := 0;
        Result.Write(SInt, SizeOf(Integer)); { Next page }
        SS.Position := 0;
        Result.CopyFrom(SS, SS.Size);

        Diff := PSize - Result.Size;
        if Diff > 0 then
         Result.Size := Result.Size + Diff
        else if Diff < 0 then
        { Page size is too small, expand it }
        begin
          SInt := PSize + Abs(Diff);
          if PageSize = 0 then { Compact, give 10 bts slack }
          begin
            SInt := SInt + 10;
            Result.Size := result.Size + 10;
            Result.Position := 0;
          end;

          Result.Position := 0;
          Result.Write(SInt, SizeOf(Integer));
        end;
        Result.Position := 0;
        Exit;
      end
      else if FixedStringSize then
      begin
        SS.Position := 0;
        Result.CopyFrom(SS, SS.Size);
        Exit;
      end;
      Result.Write(L, SizeOf(integer)); { The string length specifier }
      if (RecordSpace <= 0) then
      begin
        L := L + Abs(RecordSpace) + FreeSpace + SizeOf(integer) * 2;
        SS.Size := SS.Size + Abs(RecordSpace) + FreeSpace;
      end
      else
      begin
        SS.Size := RecordSpace;
        L := RecordSpace + SizeOf(integer) * 2;
      end;
      Result.Write(L, SizeOf(integer)); { The record space specifier }
      SS.Position := 0;
      Result.CopyFrom(SS, SS.Size);
    finally
      SS.Free;
    end;
  end;

  procedure TSubFile.DoInsert(const Index: integer; const S: string);

  var
    P: Longint;
    MS: TMemoryStream;
  begin
    if Index = Count then
    begin
      Add(S);
      Exit;
    end;

    P := PositionOfRecord(Index);
    MS := StringToStream(S);
    try
      InsertStream(MS, P);
      if Assigned(FIndexTable) then
      begin
        FIndexTable.FInsertCount := MS.Size;
        FIndexTable.Add(FillZeroes(IntToStr(P), 10));
      end;
      if IsIndexTable then
      begin
        RenumberIndex(Self, Index + 1, Self.FInsertCount);
      end;
    finally
      MS.Free;
    end;
  end;

  procedure TSubFile.Insert(const Index: integer; const S: string); { Inserts a record }

  var
    N: integer;
  begin
    N := Length(S) + (SizeOf(integer) * 2);
    if FixedStringSize then
      Dec(N, SizeOf(integer) * 2);
    if N > RecordSpace then
    begin
      if (RecordSpace > 0) then
        HandleILException(E_SpaceOverflow);
    end;
    if Sorted or (Index = Count) then
      Add(S)
    else
      DoInsert(Index, S);
  end;

  procedure TSubFile.Delete(const Index: integer); { Deletes a record }

  var
    P1: Longint;
    P2: Longint;
    SS: TStringStream;
    i: integer;
    Cnt: integer;
    Strng: string;
  begin
    if Index = Count - 1 then
    begin
      Size := Size - SizeOfRecord(Index);
      if Assigned(FIndexTable) then
        FIndexTable.Delete(Index);
      Exit;
    end;

    P1 := PositionOfRecord(Index);
    P2 := PositionOfRecord(Index + 1);
    Cnt := P2 - P1;
    DeleteStream(P2, Cnt);
    if Assigned(FIndexTable) then
    begin
      FIndexTable.FDeleteCount := Cnt;
      FIndexTable.Delete(Index);
    end;
    if IsIndexTable then
    begin
      SS := TStringStream.Create;
      try
        Position := PositionOfRecord(Index);
        for i := Index to Count - 1 do
        begin
          SS.Clear;
          P2 := Position;
          SS.CopyFrom(FStream, 10);
          SS.Position := 0;
          P1 := StrToInt(SS.ReadString(10));
          P1 := P1 - FDeleteCount;
          Strng := IntToStr(P1);
          Strng := FillZeroes(Strng, 10);
          SS.Clear;
          SS.WriteString(Strng);
          SS.Position := 0;
          Position := P2;
          FStream.CopyFrom(SS, SS.Size);
          Position := P2 + RecordSpace;
        end;
      finally
        SS.Free;
      end;
    end;
  end;

  function TSubFile.Add(const S: string; FreeSpace : integer = 0): integer; { Adds a record }

  var
    MS: TStream;
    Indx: integer;
    P: integer;
    N: integer;
    Lng : integer;
    SS2 : TStringStream;

    procedure DoAdd;

  var
    SS: TStringStream;
    Strng: string;
  begin
    AddStream(MS);
    if Assigned(FIndexTable) then
    begin
     SS := TStringStream.Create;
     try
        Strng := FillZeroes(IntToStr(P), 10);
        SS.WriteString(Strng);
        FIndexTable.AddStream(SS);
      finally
        SS.Free;
      end;
    end;
  end;

  begin
    SS2 := GetSS(S); { Get string as stream}
    Lng := SS2.Size;
    SS2.Free;
    N := Lng + (SizeOf(integer) * 2);
    if Paged then
    { Paged layout }
    begin
       MS := StringToStream(S);
       try
        P := Size;
        DoAdd;
        Result := Count - 1;
       finally
        MS.Free;
       end;
       Exit;
    end
    else if FixedStringSize then
      Dec(N, SizeOf(integer) * 2);
    if (N > RecordSpace) and (RecordSpace > 0) then
    begin
      Comment1 := IntToStr(RecordSpace) + ' (' + S + ')';
      HandleILException(E_SpaceOverflow);
    end;

    Result := -1;
    P := Size;

    MS := StringToStream(S, FreeSpace);
    try
      if Sorted and (Count > 0) then
      begin
        if Find(S, Indx) and (Duplicates = DupIgnore) and not IsIndexTable then
          Exit;
        if Indx < Count then
        begin
          DoInsert(Indx, S);
          Result := Indx;
        end
        else
        begin
          DoAdd;
          Result := Count - 1;
        end;
      end
      else
      begin
        DoAdd;
        Result := Count - 1;
      end;
    finally
      MS.Free;
    end;
  end;

  function TSubFile.Find(const AString: string; var Index: integer): Boolean;

  { Same code as TStringList.Find ! }
  var
    L, H, i, C: integer;
  begin
    Result := false;
    L := 0;
    H := Count - 1;
    while L <= H do
    begin
      i := (L + H) shr 1;
      C := AnsiCompareText(Items[i], AString);
      if C < 0 then
        L := i + 1
      else
      begin
        H := i - 1;
        if C = 0 then
        begin
          Result := True;
          if Duplicates <> dupAccept then
            L := i;
        end;
      end;
    end;
    Index := L;
  end;

  procedure TSubFile.Clear;

  var
    MS: TMemoryStream;
    i: integer;
    Buf: Byte;
  begin
    MS := TMemoryStream.Create;
    try
      MS.Size := FFileSpec.FileSpace;
      Buf := 0;
      MS.Position := 0;
      for i := 0 to FFileSpec.FileSpace - 1 do
        MS.Write(Buf, 1);
      Position := 0;
      MS.Position := 0;
      FStream.CopyFrom(MS, MS.Size);
      Size := 0;
      if Assigned(FIndexTable) then
        FIndexTable.Clear;
    finally
      MS.Free;
    end;
  end;

  procedure TSubFile.Truncate(const NewFileSpace: integer);

  var
    MS: TMemoryStream;
    MovePos: int64;
    MoveCnt: int64;
    Info: TFileSpecEx;
    Cnt: integer;
    i: integer;
  begin
    Clear;
    MS := TMemoryStream.Create;
    try
      MovePos := Offset + FFileSpec.FileSpace;
      MoveCnt := FStream.Size - MovePos;
      FStream.Position := MovePos;
      MS.CopyFrom(FStream, MoveCnt);
      MS.Position := 0;
      FStream.Position := Offset + NewFileSpace;
      FStream.CopyFrom(MS, MS.Size);
      FStream.Size := FStream.Size - FFileSpec.FileSpace + NewFileSpace;
      FFileSpec.FileSpace := NewFileSpace;
      FFileSpec.FileSize := 0;
      { Decrement offset for subsequent files }
      Cnt := FOwner.Count;

      for i := Index + 1 to Cnt - 1 do
      begin
        Info := FOwner.ReadHeader(i);
        Dec(Info.Offset, MoveCnt);
        FStream.Position := SizeOf(integer) + i * SizeOf(TFileSpecEx);
        FStream.Write(Info, SizeOf(TFileSpecEx));
        FOwner.FFiles[i].FFileSpec := Info;
      end;
      FOwner.WriteHeader(FOwner.IndexOf(FFileName), FFileSpec);
    finally
      MS.Free;
    end;
  end;

  function TSubFile.SizeOfData(const Index: integer): integer;
  var
    S : string;
    SS : TStringStream;
  begin
    if FixedStringSize then
      Result := RecordSpace
    else if Paged then
    begin
      S := Items[Index];
      if FOwner.Unicode then
       SS := TStringStream.Create(S, TEncoding.Utf8)
      else
       SS := TStringStream.Create(S);
      Result := SS.Size;
      SS.Free;
    end
    else
    begin
     Position := PositionOfRecord(Index);
     FStream.Read(Result, SizeOf(integer));
    end;
  end;

  function TSubFile.SizeOfRecord(const Index: integer): integer;
  var
    NP : integer;

    procedure GetNP(AnNP: integer; var NextNP : integer);
    var
      N : integer;
    begin
      FStream.Position := AnNP;
      FStream.Read(N, SizeOf(Integer));
      Result := Result + N;
      FStream.Position := FStream.Position + SizeOf(Integer);
      FStream.Read(NextNP, SizeOf(Integer));
    end;

  begin
    if FixedStringSize then
    begin
      Result := RecordSpace;
      Exit;
    end;

    if Paged then
    begin
      Position := PositionOfRecord(Index);
      FStream.Read(Result, SizeOf(integer)); {Page size}
      FStream.Position := FStream.Position + SizeOf(Integer); {Move to next page }
      FStream.Read(NP, SizeOf(integer));
      while NP > 0 do
       GetNP(NP, NP);

    end
    else
    begin
      Position := PositionOfRecord(Index) + SizeOf(integer);
      FStream.Read(Result, SizeOf(integer));
    end;
  end;

  function TSubFile.PageCount(Index : integer) : integer;
  var
    NP : integer;

  procedure GetNP(AnNP: integer; var NextNP : integer);
    begin
      inc(result);
      FStream.Position := AnNP;
      FStream.Position := FStream.Position + SizeOf(Integer)*2;
      FStream.Read(NextNP, SizeOf(Integer));
    end;

  begin
     Result := 1;
     if not Paged then
       Exit;

     Position := PositionOfRecord(Index);
     FStream.Position := FStream.Position + SizeOf(Integer) * 2; {Move to next page }
     FStream.Read(NP, SizeOf(integer));
      while NP > 0 do
       GetNP(NP, NP);
  end;

  { function TSubFile.SizeOfFreeSpace(Index : integer) : integer;
    begin
    Result := SizeOfRecord(Index) - SizeOfData(Index);
    end; }

  procedure TSubFile.RenumberIndex(AnIndexTable: TSubFile; const FromIndex: integer;
    const ByteCount: integer);

  var
    i: integer;
    SS: TStringStream;
    Strng: string;
    P, P2: integer;
  begin
    SS := TStringStream.Create;
    try
      AnIndexTable.Position := AnIndexTable.PositionOfRecord(FromIndex);
      for i := FromIndex to AnIndexTable.Count - 1 do
      begin
        SS.Clear;
        P2 := AnIndexTable.Position;
        SS.CopyFrom(AnIndexTable.FStream, 10);
        SS.Position := 0;
        P := StrToInt(SS.ReadString(10));
        P := P + ByteCount;
        Strng := IntToStr(P);
        Strng := FillZeroes(Strng, 10);
        SS.Clear;
        SS.WriteString(Strng);
        SS.Position := 0;
        AnIndexTable.Position := P2;
        AnIndexTable.FStream.CopyFrom(SS, SS.Size);
        AnIndexTable.Position := P2 + AnIndexTable.RecordSpace;
      end;
    finally
      SS.Free;
    end;
  end;

  { TILUpdateTest ------------------------------------------------------------- }

  constructor TILUpdateTest.Create;
  begin
    FResults := TList.Create;
  end;

  destructor TILUpdateTest.Destroy;

  var
    i: integer;
    TR: TILTestResult;
  begin
    for i := 0 to Count - 1 do
    begin
      TR := TILTestResult(FResults[i]);
      TR.Free;
    end;
    FResults.Free;
  end;

  procedure TILUpdateTest.Add(ATestResult: TILTestResult);
  begin
    FResults.Add(ATestResult);
  end;

  procedure TILUpdateTest.SetItem(Index: integer; Value: TILTestResult);
  begin
    FResults[Index] := Value;
  end;

  function TILUpdateTest.GetItem(Index: integer): TILTestResult;
  begin
    Result := TILTestResult(FResults[Index]);
  end;

  function TILUpdateTest.GetCount: integer;
  begin
    Result := FResults.Count;
  end;

  { TilField ------------------------------------------------------------------- }

  constructor TILField.Create(TableOwner: TILTable; AFieldName: string;
    OwnerFields: TILFieldDefs);

  var
    Indx: integer;
  begin
    Ilib := TableOwner;
    FFieldDefs := OwnerFields;
    Indx := FFieldDefs.IndexOf(AFieldName);
    if (Indx = -1) and not (AFieldName = 'Dummy') then
    begin
      Comment1 := AFieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    FFieldName := AFieldName;
    FValues := TStringList.Create;
    FOldValues := FValues;
    FExpression := TStringList.Create;
    FResFilePos := -1;
    FLookupRecordIndex := -1;
    FValues.OnChange := Change;
  end;

  destructor TILField.Destroy;
  begin
    if FOldValues <> FValues then
      FOldValues.Free;
    FValues.Free;
    FExpression.Free;
    if Assigned(FLTStream) then
      FLTStream.Free;
    Inherited Destroy;
  end;

  function TILField.GetCount: integer;
  begin
    Result := FValues.Count;
  end;

  function TILField.GetFieldName : string;
  begin
     Result := FFieldName;
  end;

  function TILField.GetItems(Index: integer): string;
  begin
    Result := FValues[Index];
  end;

  function TILField.IsEmpty: Boolean;
  begin
    Result := (FValues.Count = 0);
  end;

  function TILField.IsNull: Boolean;
  begin
    Result := (FValues.Count = 1)
    and TMan.IsValueNull(FValues[0])
    or (FValues[0] = '')
  end;

  procedure TILField.Change(Sender: TObject);
  var
    N: TTreeNode;
  begin
    if not( (FOwnerRecord <> nil) and (FOwnerRecord is TILDatarecord) and
    (TILDataRecord(FOwnerRecord).OwnerDataset = Ilib.Dataset)) then
     Exit;
    if Ilib.TVControl = nil then
      Exit
    else if Ilib.TVControl.Building then
      Exit;
    if SameText(FieldName, ILIB.TVTable.TreeViewLabelField) then
    begin
       N := ILIB.TVControl.NodeOfRecord(FOwnerRecord);
       if N <> nil then
          N.Text := AsString;
    end;
  end;

  function TILField.GetAsString: string;
  var
    S : string;
  begin
    if FieldDef.FDataType = dtLongText then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := '';
    if not IsEmpty then
    begin
      if FieldDef.FDataType = dtText then
      begin
        S := FValues.Text;
        S := StringReplace(S, #$D#$A, ' ', [rfReplaceAll]);
        Result := S;
      end
      else
        Result := FValues[0];
    end;
  end;

  function TILField.GetIndex: integer;
  begin
    Result := -1;
    if OwnerRecord = nil then
      Exit;
    Result := OwnerRecord.IndexOf(FieldName);
  end;

  function TILField.GetAsNumber: Real;
  begin
    Comment1 := FieldDef.Caption;
    if IsEmpty then
      HandleILException(E_FieldIsEmpty, '');
    if not(FieldDef.FDataType = dtNumber) then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := StrToFloat(FValues[0], Translator.FFormatSettings);
    Comment1 := '';
  end;

  function TILField.GetAsInteger: integer;
  begin
    Comment1 := FieldDef.Caption;
    if IsEmpty then
      HandleILException(E_FieldIsEmpty, '');
    if not(FieldDef.FDataType = dtNumber) then
      HandleILException(E_FieldDataTypeMismatch, '')
    else if FieldDef.Decimals > 0 then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := StrToInt(FValues[0]);
    Comment1 := '';
  end;

  function TILField.GetAsBoolean: Boolean;
  begin
    Comment1 := FieldDef.Caption;
    if IsEmpty then
      HandleILException(E_FieldIsEmpty, '');
    if not(FieldDef.FDataType = dtBoolean) then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := (FValues[0] = '1');
    Comment1 := '';
  end;

  function TILField.GetAsDateTime: TDateTime;
  begin
    Comment1 := FieldDef.Caption;
    if IsEmpty then
      HandleILException(E_FieldIsEmpty, '');
    if not(FieldDef.FDataType = dtDate) then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := StrToDateTime(FValues[0], Translator.FFormatSettings);
    Comment1 := '';
  end;

  function TILField.GetAsDate: TDate;
  var
    DT : TDateTime;
    Y, M, D : Word;
  begin
    DT := AsDateTime;
    DecodeDate(DT, Y, M, D);
    Result := EncodeDate(Y, M, D);
  end;

  function TILField.GetAsText: string;
  begin
    Result := '';
    Comment1 := FieldDef.Caption;
    if IsEmpty then
    begin
      Comment1 := '';
      Exit;
    end;
    if not(FieldDef.FDataType in [dtText, dtLongText]) then
      HandleILException(E_FieldDataTypeMismatch, '');
    if (FieldDef.FDataType = dtLongText) then
    begin
      if FTextUpdated then
        Result := FLongText
      else if FValues.Count > 0 then
        Result := GetTextLob(StrToInt(FValues[0]));
    end
    else
      Result := FValues.Text;
    if Length(Result) > 2 then
    begin
      if Result[Length(Result)] = #$A then
        System.Delete(Result, Length(Result)-1, 2);
    end;

    Comment1 := '';
  end;

  function TILField.GetTextLob(const Position: Integer) : string;
  var
    RecSize : Word;
    PageSize : Word;
    NextPage : Integer;
    SS : TStringStream;
  begin
    Result := '';
    if FValues[0] = '1' then
      Exit;
    TMan.FTextLobs.Position := Position;
    TMan.FTextLobs.Read(PageSize, SizeOf(Word));
    TMan.FTextLobs.Read(RecSize, SizeOf(Word));
    TMan.FTextLobs.Read(NextPage, SizeOf(Integer));
    if RecSize = 0 then
      Exit;
    if Tman.Unicode then
      SS := TStringStream.Create('', TEncoding.Utf8)
    else
      SS := TStringStream.Create;

    try
     SS.CopyFrom(Tman.FTextLobs, RecSize);
     SS.Position := 0;
     Result := SS.ReadString(SS.Size);
     if (NextPage > 0) and (RecSize = PageSize-SizeOf(Word)*2-SizeOf(Integer)) then
     begin
      Result := Result + GetTextLob(NextPage);
     end;
    finally
      SS.Free;
    end;
  end;

  procedure TILField.GetStreamedTextLob(const Position: Integer; TheStream : TMemoryStream);
  var
    RecSize : Word;
    PageSize : Word;
    NextPage : Integer;
  begin
    if FValues[0] = '1' then
      Exit;
    TMan.FTextLobs.Position := Position;
    TMan.FTextLobs.Read(PageSize, SizeOf(Word));
    TMan.FTextLobs.Read(RecSize, SizeOf(Word));
    TMan.FTextLobs.Read(NextPage, SizeOf(Integer));
    if RecSize = 0 then
      Exit;
    TheStream.CopyFrom(Tman.FTextLobs, RecSize);
    if (NextPage > 0) and (RecSize = PageSize-SizeOf(Word)*2-SizeOf(Integer)) then
    begin
      GetStreamedTextLob(NextPage, TheStream);
    end;
  end;

  function TILField.GetSampleLongText(const MaxLength : integer) : string;
  var
   P : Integer;
   RecSize : Word;
   PageSize : Word;
   NextPage : Integer;
   SS : TStringStream;
   SampleText : string;
   Mx : integer;
  begin
    if not(FieldDef.FDataType = dtLongText) then
      HandleILException(E_FieldDataTypeMismatch, '');
    Result := '';
    if FValues.Count = 0 then
       Exit;
    if FValues[0] = '1' then
      Exit;
    Mx := MaxLength;
    if Mx > 256 then
      Mx := 256;
    if Assigned(Table.DataHandler) and
    Assigned(Table.Datahandler.FOnUpdateLongText) then
    begin
      SampleText := '';
      Table.Datahandler.FOnUpdateLongText(Table, Self, SampleText, [ltaSampleText]);
      if Length(SampleText) <> 0 then
      begin
        if Length(SampleText) > 256 then
          SampleText := Copy(SampleText, 1, 256);
        Result := SampleText;
        Exit;
      end;
    end;

    P := StrToInt(FValues[0]);
    TMan.FTextLobs.Position := P;
    TMan.FTextLobs.Read(PageSize, SizeOf(Word));
    TMan.FTextLobs.Read(RecSize, SizeOf(Word));
    TMan.FTextLobs.Read(NextPage, SizeOf(Integer));
    if Mx > RecSize then
      Mx := RecSize;
    if Tman.Unicode then
     SS := TStringStream.Create('', TEncoding.Utf8)
    else
     SS := TStringStream.Create;
    try
     SS.CopyFrom(Tman.FTextLobs, Mx);
     SS.Position := 0;
     Result := SS.ReadString(SS.Size);
    finally
      SS.Free;
    end;
  end;

  function TILField.GetAsValues: TStringList;
  begin
    Result := TStringList.Create;
    Result.Assign(FValues);
  end;

  procedure TILField.SetAsValues(const Value : TStringList);
  begin
     SetOldState;
     FValues.Assign(Value);
  end;

  procedure TILField.SetAsNumber(const Value : Real);
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType = dtNumber) then
      HandleILException(E_FieldDataTypeMismatch, '');
    SetOldState;
    if IsEmpty or (FieldDef.MaxValues = 0) then
      FValues.Add(FloatToStr(Value, Translator.FFormatSettings))
    else if FieldDef.MaxValues = 1 then
      FValues[0] := FloatToStr(Value, Translator.FFormatSettings)
    else if FValues.Count < FieldDef.MaxValues then
      FValues.Add(FloatToStr(Value, Translator.FFormatSettings))
    else
      HandleILException(E_FieldTooManyValues, '');
    if FieldDef.ImageText then
      UpdateImageText;
    Comment1 := '';
  end;

  procedure TILField.SetAsInteger(const Value : integer);
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType = dtNumber) then
      HandleILException(E_FieldDataTypeMismatch, '');
    SetOldState;
    if IsEmpty or (FieldDef.MaxValues = 0) then
      FValues.Add(IntToStr(Value))
    else if FieldDef.MaxValues = 1 then
      FValues[0] := IntToStr(Value)
    else if FValues.Count < FieldDef.MaxValues then
      FValues.Add(IntToStr(Value))
    else
      HandleILException(E_FieldTooManyValues, '');
    if FieldDef.ImageText then
      UpdateImageText;
    Comment1 := '';
  end;

  procedure TILField.SetNullValue;
  begin
    SetOldState;
    FValues.Clear;
    FValues.Add(TMan.ReportNull);
  end;

  procedure TILField.Clear;
  begin
    SetOldState;
    FValues.Clear;
  end;

  function TILField.IndexOf(const Value: string): integer;
  begin
    Result := FValues.IndexOf(Value);
  end;

  procedure TILField.SetAsBoolean(const Value : Boolean);

  var
    Val: string;
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType = dtBoolean) then
      HandleILException(E_FieldDataTypeMismatch, '');
    if Value then
      Val := '1'
    else
      Val := '0';
    SetOldState;
    if IsEmpty or (FieldDef.MaxValues = 0) then
      FValues.Add(Val)
    else if FieldDef.MaxValues = 1 then
      FValues[0] := Val
    else if FValues.Count < FieldDef.MaxValues then
      FValues.Add(Val)
    else
      HandleILException(E_FieldTooManyValues, ' (' + FieldDef.FieldName + ')');
    if FieldDef.ImageText then
      UpdateImageText;
    Comment1 := '';
  end;

  procedure TILField.SetAsDateTime(const Value : TDateTime);
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType = dtDate) then
      HandleILException(E_FieldDataTypeMismatch, '');
    SetOldState;
    if IsEmpty or (FieldDef.MaxValues = 0) then
      FValues.Add(DateTimeToStr(Value, Translator.FFormatSettings))
    else if FieldDef.MaxValues = 1 then
      FValues[0] := DateTimeToStr(Value, Translator.FFormatSettings)
    else if FValues.Count < FieldDef.MaxValues then
      FValues.Add(DateTimeToStr(Value))
    else
      HandleILException(E_FieldTooManyValues, ' (' + FieldDef.FieldName + ')');
    if FieldDef.ImageText then
      UpdateImageText;
    Comment1 := '';
  end;

  procedure TILField.SetAsDate(const Value : TDate);
  begin
    SetAsDateTime(Value);
  end;

  procedure TILField.SetAsText(const Value : string);
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType in [dtText, dtLongText]) then
      HandleILException(E_FieldDataTypeMismatch, '');
    SetOldState;
    if FieldDef.FDataType = dtLongText then
    begin
      if IsEmpty then
        FValues.Add('1');
      FLongText := Value;
      if Assigned(FLTStream) then
        FreeAndNil(FLTStream);
      FTextUpdated := True;
      FTextSourceChanged := false;
      if Ilib.HasHandler and Assigned(Ilib.Datahandler.FOnDatasetChange) then
       Ilib.Datahandler.FOnDatasetChange(Ilib, dsDataUpdated);

    end
    else
    begin
     FValues.Text := Value;
     if FieldDef.ImageText then
       UpdateImageText;
    end;
    Comment1 := '';
  end;

  procedure TILField.SaveLT;
  begin
     if Assigned(FLTStream) then
       SaveLongTextStream
     else
       SaveLongText;
  end;

  procedure TILField.SetAsStream(const Value : TMemoryStream);
  begin
    Comment1 := FieldDef.Caption;
    if not(FieldDef.FDataType = dtLongText) then
      HandleILException(E_FieldDataTypeMismatch, '');
    SetOldState;
    if IsEmpty then
       FValues.Add('1');
    if not Assigned(FLTStream) then
      FLTStream := TMemoryStream.Create;
    FLongText := '';
    Value.Position := 0;
    FLTStream.Clear;
    FLTStream.CopyFrom(Value, Value.Size);
    FTextUpdated := True;
    FTextSourceChanged := false;
    if Ilib.HasHandler and Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsDataUpdated);

    Comment1 := '';
   end;

  function TILField.GetAsStream : TMemoryStream;
  begin
    Result := nil;
    Comment1 := FieldDef.Caption;
    if IsEmpty then
    begin
      Comment1 := '';
      Exit;
    end;
    if not(FieldDef.FDataType = dtLongText) then
      HandleILException(E_FieldDataTypeMismatch, '');
    if FTextUpdated then
      Result := FLtStream
    else if FValues.Count > 0 then
    begin
       Result := TMemoryStream.Create;
       GetStreamedTextLob(StrToInt(FValues[0]), Result);
    end;
    Result.Position := 0;
    Comment1 := '';
  end;

  function TILField.GetDisplayValue : string;
  var
    DS : TILDataset;
    i : integer;
  begin
    if (FieldDef.LookupType = looTableKey)
    and (FieldDef.DataType = dtNumber) then
    begin
      if AsString <> '' then
      begin
        DS := FieldDef.LookupTable.DBView.SearchPrivate('select * where ' +
        FieldDef.Lookupfield + ' = ' + AsString );
        try
         if DS.Count = 0 then
           Result := ''
         else
         begin
            Result := DS[0][FieldDef.LookupSearchField].AsString;
         end;
         finally
           DS.Free;
         end;
         DS := FieldDef.LookupTable.DBView.SearchPrivate('select * where ' +
          FieldDef.LookupSearchField + ' = ' + '''' + Result + '''');
         try
           if DS.Count > 1 then
           begin
              for i := 0 to DS.Count -1 do
                if DS[i][FieldDef.LookupField].Asstring = AsString then
                begin
                  Result := Result + ' [' + AsString + ']';
                end;

           end;
         finally
           DS.Free;
         end;
       end
       else
         Result := '';
    end
    else if FieldDef.DataType = dtLongText then
      Result := GetSampleLongText(50)
    else
      Result := AsString;
  end;

  function TILField.SaveLongText(const Value : string; const Position : Integer) : Integer;
  var
    SS : TStringStream;
    PS : Word;
    LTP : TLTPageSize;
    RS : LongWord;
    Page : Integer;
    NetFrame : Word;
    ThisStr, Rest : string;
    NewText : Boolean;
    ExactFill : Boolean;
  begin
     { First 2 bytes are page frame size, next 2 is actual datasize within page,
     last 4 bytes point to next page if any }
     NewText := Position = 1;
     if NewText then
     begin
      LTP := FieldDef.LTPageSize;
      case LTP of
       ltp1Kb : PS := 1000;
       ltp2Kb : PS := 2000;
       ltp4Kb : PS := 4000;
       ltp8Kb : PS := 8000;
       ltp16Kb : PS := 16000;
       ltp32Kb : Ps := 32000;
      end;
     end
     else
     begin
       TMan.FTextLobs.Position := Position;
       TMan.FTextLobs.Read(PS, SizeOf(Word));
     end;
     Rest := '';
     ThisStr := '';
     ExactFill := false;
     NetFrame := PS - (Sizeof(Word)*2 + SizeOf(Integer));
     RS := Length(Value);
     if RS > NetFrame then
     begin
       ThisStr := Copy(Value, 1, NetFrame);
       Rest := Copy(Value, NetFrame+1, RS);
       RS := NetFrame;
     end
     else
     begin
       ExactFill := RS = NetFrame;
       ThisStr := Value;
     end;
     Page :=0;
     if NewText then
      TMan.FTextLobs.Position := TMan.FTextLobs.Size
     else
      TMan.FTextLobs.Position := Position;
     Result := TMan.FTextLobs.Position;
     if NewText then
     begin
        TMan.FTextLobs.Write(PS, SizeOf(Word));
        TMan.FTextLobs.Write(RS, SizeOf(Word));
        TMan.FTextLobs.Write(Page, SizeOf(Integer));
     end
     else
     begin
       TMan.FTextLobs.Position := Result + SizeOf(Word)*2;
       TMan.FTextLobs.Read(Page, SizeOf(Integer));
       TMan.FTextLobs.Position := Result + Sizeof(Word)*2 + SizeOf(Integer);
     end;
     if Tman.Unicode then
       SS := TStringStream.Create(ThisStr, TEncoding.Utf8)
     else
       SS := TStringStream.Create(ThisStr);
     if NewText then
       SS.Size := NetFrame;
     try
     SS.Position := 0;
     TMan.FTextLobs.CopyFrom(SS, SS.Size);
     Tman.FTextLobs.Position := Result + SizeOf(Word);
     TMan.FTextLobs.Write(RS, SizeOf(Word));
     if Rest <> '' then
     begin
        if Page = 0 then
          Page := 1
        else if Page < 0 then
          Page := Abs(Page);
        Page := SaveLongText(Rest, Page);
        TMan.FTextLobs.Position := Result + Sizeof(Word)*2;
        TMan.FTextLobs.Write(Page, SizeOf(Integer));
     end
     else if ExactFill and (Page > 0) then
     begin
       TMan.FTextLobs.Position := Result + Sizeof(Word)*2;
       Page := -Page;
       TMan.FTextLobs.Write(Page, SizeOf(Integer));
     end;
     finally
       SS.Free;
     end;
  end;

  function TILField.SaveLongText : Integer;
  var
    S : string;
  begin
     Result := SaveLongText(FLongText, StrToInt(FValues[0]));
     if FValues[0] = '1' then
     begin
        Str(Result, S);
        FValues[0] := S;
     end;
     FTextSourceChanged := false;
     FTextUpdated := false;
     FLongText := '';
  end;

  function TILField.SaveLongTextStream(const Value : TStream; const Position : Integer;
   AFile : TFileStream = nil) : Integer;
  var
    PS : Word;
    RS : LongWord;
    Page : Integer;
    LTP : TLTPageSize;
    NetFrame : Word;
    ThisStr, Rest : TMemoryStream;
    ExactFill : Boolean;
    NewText : Boolean;

  begin
     { First 2 bytes are page frame size, next 2 is actual datasize within page,
     last 4 bytes point to next page if any }
     if AFile = nil then
       AFile := TMan.FTextLobs;
     NewText := Position = 1;
     if NewText then
     begin
      LTP := FieldDef.LTPageSize;
      case LTP of
       ltp1Kb : PS := 1000;
       ltp2Kb : PS := 2000;
       ltp4Kb : PS := 4000;
       ltp8Kb : PS := 8000;
       ltp16Kb : PS := 16000;
       ltp32Kb : Ps := 32000;
      end;
     end
     else
     begin
       AFile.Position := Position;
       AFile.Read(PS, SizeOf(Word));
     end;
     Rest := nil;
     ExactFill := false;
     ThisStr := TMemoryStream.Create;
     try
     NetFrame := PS - (Sizeof(Word)*2 + SizeOf(Integer));
     Value.Position := 0;
     RS := Value.Size;
     if RS > NetFrame then
     begin
       Rest := TMemoryStream.Create;
       ThisStr.CopyFrom(Value, NetFrame);
       Value.Position := NetFrame;
       Rest.CopyFrom(Value, Value.Size-Value.Position);
       RS := NetFrame;
     end
     else
     begin
       ThisStr.CopyFrom(Value, Value.Size);
       ExactFill := RS = NetFrame;
     end;
     Page :=0;
     if NewText then
      AFile.Position := AFile.Size
     else
      AFile.Position := Position;
     Result := AFile.Position;
     if NewText then
     begin
        AFile.Write(PS, SizeOf(Word));
        AFile.Write(RS, SizeOf(Word));
        AFile.Write(Page, SizeOf(Integer));
     end
     else
     begin
       AFile.Position := Result + SizeOf(Word)*2;
       AFile.Read(Page, SizeOf(Integer));
       AFile.Position := Result + Sizeof(Word)*2 + SizeOf(Integer);
     end;
     if NewText then
       ThisStr.Size := NetFrame;
     ThisStr.Position := 0;
     AFile.CopyFrom(ThisStr, ThisStr.Size);
     AFile.Position := Result + SizeOf(Word);
     AFile.Write(RS, SizeOf(Word));
     if Rest <> nil then
     begin
        if Page = 0 then
          Page := 1
        else if Page < 0 then
          Page := Abs(Page);
        Page := SaveLongTextStream(Rest, Page);
        AFile.Position := Result + Sizeof(Word)*2;
        AFile.Write(Page, SizeOf(Integer));
        Rest.Free;
     end
     else if ExactFill and (Page > 0) then
     begin
       AFile.Position := Result + Sizeof(Word)*2;
       Page := -Page;
       AFile.Write(Page, SizeOf(Integer));
     end;
     finally
       ThisStr.Free;
     end;
  end;

  function TILField.SaveLongTextStream : Integer;
  var
    S : string;
  begin
     Result := 1;
     if not Assigned(FltStream) then
       Exit;
     Result := SaveLongTextStream(FLTStream, StrToInt(FValues[0]));
     if FValues[0] = '1' then
     begin
        Str(Result, S);
        FValues[0] := S;
     end;
     FTextSourceChanged := false;
     FTextUpdated := false;
     FLTStream.Clear;
  end;

  function TILField.LongTextSize : integer;
  var
    P : integer;
    Sz : word;
  begin
    Result := 0;
    if (FValues.Count = 0) or (Fvalues[0] = '1') then
      Exit;
    P := StrToInt(Fvalues[0]);
    TMan.FTextLobs.Position := P;
    TMan.FTextLobs.Position := TMan.FTextLobs.Position  + SizeOf(Word);
    TMan.FTextLobs.Read(Sz, SizeOf(Word));
    Result := Sz;
    TMan.FTextLobs.Read(P, SizeOf(Integer));
    while P > 0 do
    begin
      TMan.FTextLobs.Position := P;
      TMan.FTextLobs.Position := TMan.FTextLobs.Position  + SizeOf(Word);
      TMan.FTextLobs.Read(Sz, SizeOf(Word));
      Result := Result + Sz;
      TMan.FTextLobs.Read(P, SizeOf(Integer));
    end;
  end;

  procedure TILField.SetAsString(const Value : string);

  var
    N: Extended;
    i: integer;
    DT: TDateTime;
  begin
    Comment1 := FieldDef.Caption;
    if FieldDef.DataType = dtNumber then
    begin
      if FieldDef.Decimals > 0 then
      begin
        if not TryStrToFloat(Value, N, Translator.FFormatSettings) then
          HandleILException(E_InvalidNumber, '');
      end
      else
      begin
        if not TryStrToInt(Value, i) then
          HandleILException(E_InvalidNumber, '');
      end;
    end
    else if FieldDef.DataType = dtDate then
    begin
      if not TryStrToDateTime(Value, DT, Translator.FFormatSettings) then
        HandleILException(E_InvalidDate, Value);
    end
    else if FieldDef.DataType = dtBoolean then
    begin
      if not(Value = '0') and not(Value = '1') and not(Value = Tman.ReportTrue)
        and not(Value = Tman.ReportFalse) then
        HandleILException(E_InvalidBoolean, '');

    end;

    if FValues.Count > 0 then
    begin
      if FieldDef.DataType = dtText then
      begin
        SetOldState;
        FValues.Text := Value;
      end
      else if FieldDef.MaxValues = 1 then
      begin
        SetOldState;
        FValues[0] := Value;
      end
      else if (FieldDef.MaxValues > 0) and (FValues.Count = FieldDef.MaxValues)
      then
      begin
        HandleILException(E_FieldTooManyValues, '');
      end
      else
      begin
        SetOldState;
        FValues.Add(Value);
      end;
    end
    else
    begin
      SetOldState;
      FValues.Add(Value);
    end;
    if FieldDef.ImageText then
      UpdateImageText;
    Comment1 := '';
  end;

  function TILField.DecodeTableKey(SearchVal : string) : string;
  var
    DS : TILDataSet;
    P1, P2 : integer;
  begin
    Result := '';
    P1 := PosRestrict('[', SearchVal, False);
    P2  := PosRestrict(']', SearchVal, False);
    if (P1 <> 0) and (P2 = Length(SearchVal)) then
    begin
      Result := Copy(SearchVal, P1 +1, P2-P1-1);
      Exit;
    end;
    DS := FieldDef.LookupTable.DBView.SearchPrivate('select * where ' +
    FieldDef.LookupSearchField + ' = ' + '''' + SearchVal + '''');
    try
       if DS.Count > 0 then
       begin
          Result := DS[0][FieldDef.LookupField].AsString;
       end;
    finally
      DS.Free;
    end;
  end;

  function TILField.GetFieldDef;
  var
    Indx: integer;
  begin
    Indx := FFieldDefs.IndexOf(FFieldName);
    Result := FFieldDefs.Items[Indx];
  end;

  function TILField.GetLookupSourceRecord : TILDataRecord;
  var
    N : string;
    DS : TILDataset;
    LV : string;
  begin
    Result := nil;
    if not (FieldDef.LookupType in [looTableKey, looTableRef]) then
      Exit;
    if AsString = '' then
      Exit;
    if FieldDef.LookupType = looTableRef then
    begin
        N := Ilib.GetLookupTableID(FieldDef.LookupTable, FieldDef.LookupField, AsString);
        Result := FieldDef.LookupTable.RecordsX[StrToInt(N)];
    end
    else
    begin
       if FieldDef.DataType = dtNumber then
         LV := AsString
       else
         LV := '''' + AsString + '''';
       DS := FieldDef.LookupTable.DBView.SearchPrivate(FieldDef.LookupField + ' = ' + LV);
       try
         if DS.Count = 0 then
           Result := nil
         else if DS.Count > 1 then
           HandleILException(0, 'Ambigous records') {Should never happen !?}
         else
         begin
           Result :=  FieldDef.LookupTable.CreateDataRecord;
           Result.Assign(DS[0]);
         end;
       finally
          DS.Free;
       end;
    end;
  end;

  procedure TILField.ResetLTFlags;
  begin
    FTextSourceChanged := false;
  end;

  procedure TILField.SetOldState;
  begin
    if FValues = FOldValues then
    begin
      FOldValues := TStringList.Create;
      FOldValues.Assign(FValues);
    end;
    if (OwnerRecord is TILDataRecord) then
      Ilib.Dataset.FChngTracker.Add(TILDataRecord(OwnerRecord));
  end;

  procedure TILField.SetTextSourceChanged(Value : Boolean);
  begin
    if Value = FTextSourceChanged then
      Exit;
    FTextSourceChanged := Value;
    if Table.HasViewer then
      Table.Viewer.Invalidate;
  end;

  procedure TILField.UpdateImageText;

  var
    i: integer;
  begin
    if FOwnerRecord = nil then
      Exit;
    if not(FOwnerRecord is TILDataRecord) then
      Exit;
    with FOwnerRecord as TILDataRecord do
    begin
      FImageText := '';
      for i := 0 to FValues.Count - 1 do
      begin
        FImageText := FImageText + FValues[i];
        if i < FValues.Count - 1 then
          FImageText := FImageText + #13#10;
      end;
    end;
    if Ilib.HasViewer then
      Ilib.Viewer.Invalidate;
    if Ilib.HasHandler and Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsDataUpdated);
  end;

  { TilTestResult --------------------------------------------------------- }

  constructor TILTestResult.Create;
  begin
    FAdds := TStringList.Create;
    FDeletes := TStringList.Create;
  end;

  destructor TILTestResult.Destroy;
  begin
    FAdds.Free;
    FDeletes.Free;
  end;

  { TilRecord ---------------------------------------------------------------- }

  function TILRecord.GetItems(Index: integer): TILField;
  var
    Q : Boolean;
  begin
    Result := TILField(FFields[Index]);
    Q := CalcQualified(Index);
    if Q and not Result.FieldDef.FUserCalc then
    begin
     try
      if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
        Ilib.DataHandler.FOnCalcField(Ilib, Self as TILDataRecord, Result);
     except

     end;
     Result.FIsCalculated := True;
    end
    else if Q and Result.FieldDef.FUserCalc then
    begin
      Result.FieldDef.CalcUserCalc(Self as TILDataRecord);
      Result.FIsCalculated := True;
    end;
  end;

  function TILRecord.GetItems(FieldName: string): TILField;
  var
    Indx: integer;
    Q : Boolean;
    FD : TILFieldDefs;
  begin
    FD := GetActiveFieldSource;
    Indx := IndexOf(FieldName);
    if Indx = -1 then
    begin
      Indx := Fd.IndexOfCaption(FieldName);
      if Indx = -1 then
      begin
        Comment1 := FieldName;
        HandleILException(E_InvalidFieldName, '');
      end;
      FieldName := FD.Items[Indx].FieldName;
      Indx := IndexOf(FieldName);
      if Indx = -1 then
      begin
        Comment1 := FieldName;
        HandleILException(E_InvalidFieldName, '');
      end;
    end;
    Result := TILField(FFields[Indx]);
    Q := CalcQualified(Indx);
    if Q and not Result.FieldDef.FUserCalc
    then
    begin
     try
       if Assigned(ILib.DataHandler) and Assigned(ILib.DataHandler.FOnCalcField) then
        ILib.DataHandler.FOnCalcField(ILib, Self as TILDataRecord, Result);
     except

     end;
     Result.FIsCalculated := True;
    end
    else if Q and Result.FieldDef.FUserCalc then
    begin
      Result.FieldDef.CalcUserCalc(Self as TILDataRecord);
      Result.FIsCalculated := True;
    end;
    Comment1 := '';
  end;

  procedure TILRecord.SetItems(Index: integer; Value: TILField);
  begin
    FFields[Index] := Value;
  end;

  procedure TILRecord.SetItems(FieldName: string; Value: TILField);

  var
    Indx: integer;
  begin
    Indx := IndexOf(FieldName);
    FFields[Indx] := Value;
  end;

  function TILRecord.IndexOf(const FieldName: string): integer;

  { Returns the field index correspoonding to Fieldname }
  var
    i: integer;
  begin
    Result := -1;
    for i := 0 to Count - 1 do
    begin
      if SameText(Items[i].FieldName, FieldName) then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  function TILRecord.IsDataCleared(All : Boolean): Boolean;

  { Returns true if all contrls are empty }
  var
    i: integer;
    R : TIldataRecord;
  begin
    Result := True;
    if Self is TILDatarecord  then
    with Self as TILDataRecord do
    begin
      if FImage <> nil then
      begin
       Result := false;
       Exit;
      end;
    end;
    if (Count = 0) and (TMan.ActiveRecord <> nil) and (TMan.CommandMode = cmUpdate) then
    begin
      for I := 0 to TMan.ActiveRecord.Count-1 do
       if TMan.ActiveRecord[i].FieldDef.DataType = dtLongText then
       begin
         if TMan.ActiveRecord[i].Count > 0 then
         begin
           Result := false;
           Exit;
         end;
       end;
    end;
    for i := 0 to Count - 1 do
    begin
      if (Items[i].FieldDef.DataType = dtLongText) then
      begin
        R := TMan.ActiveRecord;
        if R[i].Count = 0 then
          Continue
        else
        begin
          Result := false;
          Break;
        end;
      end;
      if (Items[i].FValues.Count > 0) and not (Items[i].FValues[0] = '') then
      begin
        if All then
        begin
          Result := false;
          Break;
        end
        else
        begin
          if not SameText(Items[i].FFieldName, 'SYS_LinkID')
          and not SameText(Items[i].FFieldName, 'SYS_SiblingIndex')
          and not SameText(Items[i].FFieldName, 'SYS_Level')
          and not (Items[i].FieldDef.FieldType in [ftAutoIncrement, ftCalculated])
          and not (Items[i].FieldDef.LookupType = looParentField)
          and not  ((Items[i].FieldDef.InitialValue <> '')
            and (CompareText(Items[i].FValues[0], Items[i].FieldDef.InitialValue) = 0))
            then
          begin
            Result := false;
            Break;
          end;
        end;
      end;
    end;
  end;

  function TILRecord.HasTempFields: Boolean;

  var
    i: integer;
  begin
    Result := false;
    for i := 0 to Count - 1 do
    begin
      if Items[i].FieldDef.FTemporary then
      begin
        Result := True;
        Break;
      end;
    end;
  end;

  function TILRecord.ValueOf(const AFieldName: string): TStringList;

  { Returns the value of AField from ADatasetrecord }
  var
    i: integer;
    F : TILField;
    Q : Boolean;
  begin
    Result := nil;
    for i := 0 to Count - 1 do
    begin
      if SameText(Items[i].FieldName, AFieldName) then
      begin
        F := TIlField(FFields[I]);
        Q := CalcQualified(i);
        if Q and not F.FieldDef.FUserCalc then
        begin
           try
           if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
            Ilib.DataHandler.FOnCalcField(Ilib, Self as TILDatarecord, F);
           except

           end;
           F.FIsCalculated := True;
        end
        else if Q and F.FieldDef.FUserCalc then
        begin
          F.FieldDef.CalcUserCalc(Self as TILDataRecord);
          F.FIsCalculated := True;
        end;

        Result := TStringList.Create;
        Result.Assign(Items[i].FValues);
        Break;
      end;
    end;
  end;

  function TILRecord.GetCount: integer;
  begin
    Result := FFields.Count;
  end;

  constructor TILRecord.Create(TableOwner: TILTable);
  begin
    FFields := TList.Create;
    Ilib := TableOwner;
  end;

  destructor TILRecord.Destroy;

  var
    i: integer;
  begin
    for i := 0 to Count - 1 do
    begin
      TILField(FFields[i]).Free;
    end;
    FFields.Free;
    inherited Destroy;
  end;

  procedure TILRecord.Assign(Source: TILRecord);

  var
    i: integer;
    Field: TILField;
    TBL: TILTable;
  begin
    Clear;

    for i := 0 to Source.Count - 1 do
    begin
      TBL := Source[i].FieldDef.Ilib;
      Field := TILField.Create(TBL, Source[i].FieldName, Tbl.FFields);
      Field.FValues.Assign(Source[i].FValues);
      Field.FOwnerRecord := Self;
      Add(Field);
    end;
  end;

  procedure TILRecord.Add(AValue: TILField);
  begin
    FFields.Add(AValue);
  end;

  procedure TILRecord.Delete(const Index: integer);

  var
    Fld: TILField;
  begin
    Fld := TILField(FFields[Index]);
    Fld.Free;
    FFields.Delete(Index);
  end;

  procedure TILRecord.Clear;

  var
    i: integer;
    Fld: TILField;
  begin
    for i := 0 to Count - 1 do
    begin
      Fld := TILField(FFields[i]);
      Fld.Free;
    end;
    FFields.Clear;
  end;

  procedure TILRecord.ClearData;

  { Clears the user data }
  var
    i: integer;
  begin
    for i := 0 to Count - 1 do
    begin
      Items[i].FValues.Clear;
      Items[i].FLongText := '';
      Items[i].FTextUpdated := false;
      Items[i].FTextSourceChanged := false;
    end;
  end;

  function TILRecord.CalcQualified(const FieldIndex : integer) : Boolean;
  var
     F : TILField;
   begin
      if not Assigned(Ilib.DataHandler) then
      begin
        Result := false;
        Exit;
      end;
      F := TIlField(FFields[FieldIndex]);
      Result := (F.FieldDef.FieldType = ftCalculated) and not Tman.DLLAccess
         and not F.FieldDef.FTemporary and not F.FIsCalculated and not Ilib.FIndexing
         and Assigned(ILib.DataHandler.FOnCalcField) and
         (Self is TILDatarecord)
         and ((Ilib.Dataset.Source.IndexOf(Self) <> -1) or Ilib.FPrivateCalc);
   end;

  function TILRecord.GetActiveFieldSource : TILFieldDefs;
  begin
    if Self is TILDataRecord then
    with Self as TILDataRecord do
    begin
     if (OwnerDataset <> nil) and (OwnerDataset.FPrivateFields <> nil)  then
      Result := OwnerDataset.FPrivateFields
     else
      Result := Ilib.Fields;
    end
    else
      Result := Ilib.Fields;
  end;


  { TILDataRecord ----------------------------------------------------------- }

  function TILDataRecord.GetIndex: integer;
  begin
    if FOwnerDataset = nil then
      Result := -1
    else
      Result := FOwnerDataset.IndexOf(Self);
  end;

  function TILDataRecord.GetTVParentRecord : TILDataRecord;
  var
    Indx : integer;
  begin
      if not Items[c_SysFieldNames[SYS_LinkID]].IsEmpty then
      begin
        Indx := Ilib.Dataset.IndexOf(Items[c_SysFieldNames[SYS_LinkID]].AsInteger);
        Result := Ilib.Dataset[Indx];
        FTVParentRecord := Result;
      end
      else
      begin
        Result := FTVParentRecord;
      end;
  end;

  procedure TILDataRecord.SetTVParentRecord(const Value : TILDataRecord);
  begin
      if FTVParentRecord = Value then
        Exit;
      FTVParentRecord := Value;
      if (FTVParentRecord = nil) or (FTVParentRecord.RecordIndex = -1) then
        Items[c_SysFieldNames[SYS_LinkID]].Clear
      else if not FTVParentRecord.IsNew then
        Items[c_SysFieldNames[SYS_LinkID]].AsInteger := FTVParentRecord.RecordIndex;
  end;

  procedure TILDataRecord.SetTVSiblingIndex(const Value : integer);
  begin
      if FTVSiblingIndex = Value then
        Exit;
      FTVSiblingIndex := Value;
      Items[c_SysFieldNames[SYS_SiblingIndex]].AsInteger := Value;
  end;

  function TILDataRecord.GetTVSiblingIndex : integer;
  begin
      if not Items[c_SysFieldNames[SYS_SiblingIndex]].IsEmpty then
      begin
        Result := Items[c_SysFieldNames[SYS_SiblingIndex]].AsInteger;
      end
      else
        Result := FTVSiblingIndex;
  end;

  procedure TILDataRecord.SetTVLevel(const Value : integer);
  begin
      Items[c_SysFieldNames[SYS_Level]].AsInteger := Value;
  end;

  function TILDataRecord.GetTVLevel : integer;
  begin
      if not Items[c_SysFieldNames[SYS_Level]].IsEmpty then
      begin
        Result := Items[c_SysFieldNames[SYS_Level]].AsInteger;
      end
      else
        Result := -1;
  end;

  function TILDataRecord.GetTVParentID : integer;
  var
    Indx : integer;
  begin
      if not Items[c_SysFieldNames[SYS_LinkID]].IsEmpty then
      begin
        Indx := Ilib.Dataset.IndexOf(Items[c_SysFieldNames[SYS_LinkID]].AsInteger);
        Result := Ilib.Dataset[Indx].RecordIndex;
      end
      else
        Result := FTVParentID;
  end;

  function TILDataRecord.GetDeleted : Boolean;
  begin
      Result := SameText(FState, 'Deleted') or
       SameText(FState, 'DT') or
       SameText(FState, 'DD') or
       SameText(FState, 'DeletedTrans') or
       SameText(FState, 'Moved') or
       SameText(FState, 'MO'); { All this for backwards compability }
       if not Result and not Ilib.FIndexing then
       { If indexing is going on, orphans are kept aside. When doing a deletion count orphans must be subtratced}
         Result := (Assigned(Ilib.MasterSource) and Ilib.IsRecordOrphan(RecordIndex));
  end;

  function TILDataRecord.GetDeletedUser : Boolean;
  begin

      Result := SameText(FState, 'DD')
      or SameText(FState, 'Deleted');
  end;

  function TILDataRecord.GetDeletedMoved : Boolean;
  begin
     Result := SameText(FState, 'MO') or
     SameText(FState, 'Moved');
  end;

  procedure TILDataRecord.SetDeletedUser(const Value : Boolean);
  begin
     if Value then
       FState := 'DD'
     else
       FState := '';
  end;

  function TILDataRecord.GetDeletedTrans : Boolean;
  begin
     Result := SameText(FState, 'DT') or
     SameText(FState, 'DeletedTrans');
  end;


  procedure TILDataRecord.SetDeletedTrans(const Value : Boolean);
  begin
     if Value then
       FState := 'DT'
     else
       FState := '';
  end;

  function TILDataRecord.GetTextLob(const Position : integer) : string;
  var
    RecSize : Word;
    PageSize : Word;
    NextPage : Integer;
    SS : TStringStream;
  begin
       Result := '';
       TMan.FTextLobs.Position := Position;
       TMan.FTextLobs.Read(PageSize, SizeOf(Word));
       TMan.FTextLobs.Read(RecSize, SizeOf(Word));
       TMan.FTextLobs.Read(NextPage, SizeOf(Integer));
       if RecSize = 0 then
         Exit;
       if TMan.Unicode then
         SS := TStringStream.Create('', TEncoding.Utf8)
       else
         SS := TStringStream.Create;
       try
        SS.CopyFrom(Tman.FTextLobs, RecSize);
        SS.Position := 0;
        Result := SS.ReadString(SS.Size);
        if (NextPage > 0) and (RecSize = PageSize-SizeOf(Word)*2-SizeOf(Integer)) then
        begin
         Result := Result + GetTextLob(NextPage);
        end;
       finally
         SS.Free;
       end;
  end;

  function TILDataRecord.SaveFileResourceName(const Position : integer; AName : string; AFile : TFileStream = nil) : integer;
  var
    NetFrame : integer;
    Ps, Rs : integer;
    SS : TStringStream;
    Pg : integer;
    IsNew : Boolean;
  begin
     PS := 100;
     { Resource names are not paged. There is a minimum size of 100,
     which is increaed if length of string is longer. If the name
     is later updated a new string is allocated in the lob file if
     the new name does not fit. }
     Pg := 0;
     if AFile = nil then
      AFile := TMan.FTextLobs;
     IsNew := Position = -1;
     if IsNew then
       AFile.Position := AFile.Size
     else
       Afile.Position := Position;
     Result := AFile.Position;
     NetFrame := PS - (Sizeof(Word)*2 + SizeOf(Integer));
     if not TMan.FIsServer then
       AName := CollapseUnc(AName);

     if Tman.Unicode then
       SS := TStringStream.Create(AName, TEncoding.Utf8)
     else
       SS := TStringStream.Create(AName);
     try
       RS := SS.Size;
       if RS > NetFrame then
       begin
         if not IsNew then
         begin
           AFile.Position := AFile.Size;
           Result := AFile.Position;
         end;
         NetFrame := RS + Sizeof(Word)*2 + SizeOf(Integer);
       end;
       AFile.Write(PS, SizeOf(Word));
       AFile.Write(RS, SizeOf(Word));
       AFile.Write(Pg, SizeOf(Integer)); { Next page pos }
       SS.Size := NetFrame;
       SS.Position := 0;
       AFile.CopyFrom(SS, SS.Size);
       AFile.Position := Result + SizeOf(Word);
       AFile.Write(RS, SizeOf(Word));
     finally
       SS.Free;
     end;
  end;

  function TILDataRecord.GetFileName : string;
  begin
     Result := FFileName;
  end;

  function TILDataRecord.GetChanged: Boolean;
  var
    i, j: integer;
  begin

    Result := false;
    if Ilib.IsReadOnly then
      Exit;
    if Ilib.MultiSelected and ((OwnerDataset = Ilib.Dataset) and not Selected) then
        Exit;
    if FIsDeleted then
    begin
      Result := True;
      Exit;
    end;
    if IsNew then
    begin
      Result := not HasTempFields;
      Exit;
    end;
    if FImage <> FOldImage then
    begin
      Result := True;
      Exit;
    end;
    if FOldState <> FState then
    begin
      Result := True;
      Exit;
    end;
    if FImageChanged then
    begin
      Result := True;
      Exit;
    end;
    if FOldImageText <> FImageText then
    begin
      Result := True;
      Exit;
    end;

    for i := 0 to Count - 1 do
    begin
      if Items[i].FieldDef.FieldType in [ftCalculated, ftStoredEvaluation] then
        Continue;
      if Items[i].FieldDef.LookupType = looParentField then
        Continue;
      if Items[i].FOldValues <> Items[i].FValues then
      begin
        if Items[i].FValues.Count <> (Items[i].FOldValues.Count) then
        begin
          Result := True;
          Exit;
        end
        else if Items[i].FieldDef.DataType = dtLongtext then
        begin
            Result := Items[i].FTextUpdated;
            Exit;
        end;

        for j := 0 to Items[i].FValues.Count - 1 do
        begin
          if Items[i].FValues[j] <> Items[i].FOldValues[j] then
          begin
            Result := True;
            Exit;
          end;
        end;
      end
      else if Items[i].FieldDef.DataType = dtLongtext then
      begin
          Result := Items[i].FTextUpdated;
          Exit;
      end;
    end;
  end;

  function TILDataRecord.MustStoreLocally(const AFileName: string): Boolean;
  begin
    if not Ilib.StoreImagesLocally then
    begin
      if IsUrl(AFileName) then
        Result := True
      else if MediaType in [MED_VIDEO, MED_AUDIO] then
        Result := True
      else if (AFileName = '') and (MediaType = MED_IMAGE) then
        Result := True
      else
        Result := false;
    end
    else
      Result := True;
  end;

  function TILDataRecord.GetImageText: string;
  begin
    Result := '';
    if FImageText <> '' then
      Result := FImageText;
  end;

  function TILDataRecord.GetTable : TILTable;
  begin
    Result := Ilib;
  end;

  function TILDataRecord.GetColCount : integer;
  begin
     Result := 0;
     if FOwnerDataset.ColumnSource <> nil then
       Result := FOwnerDataset.ColumnSource.Count;
  end;

  function TILDataRecord.GetColumns(Index : integer) : TILField;
  var
    Ind : integer;
  begin
    Result := nil;
    if FOwnerDataset.ColumnSource = nil then
      Exit;
    if (Index < 0) or (Index > FOwnerDataset.ColumnSource.Count-1) then
    begin
      Comment1 := IntToStr(Index);
      HandleILException(E_OutOfBounds);
    end;

    Ind := IndexOf(FOwnerDataset.ColumnSource[Index].FieldName);
    if Ind = -1 then
      Exit;
    Result := Items[Ind];
  end;

  function TILDataRecord.GetImageTextField: TILField;
  var
    i: integer;
  begin
    Result := nil;
    for i := 0 to Count - 1 do
      if Items[i].FieldDef.ImageText then
      begin
        Result := Items[i];
      end;
  end;

  procedure TILDataRecord.SetSelected(const Value : Boolean);
  var
    SlCnt : integer;
    MH : TILMessageHandler;
    Ind : integer;
  begin
    if OwnerDataset <> Ilib.Dataset then
      Exit;
    if Value = FSelected then
      Exit;
    if not (Ilib.FMultiSelect) or not (Ilib = Tman.ActiveTable) then
    begin
      Exit;
    end;

    //if (Ilib.Selcount = 1) and Ilib.Dataset.Changed then
    //begin
    //  if Assigned(TMan.FOnAlert) then
    //   TMan.FOnAlert(TMan, alMulSelNoSelect);
    //  Exit;
    //end;
    {To avoid confusions, it is only possible to start multiselect on a saved dataset
    13.3.21 Removed this constraint}

    SlCnt := Ilib.Dataset.SelCount;
    if not Value and (OwnerDataset = Ilib.Dataset) and (SlCnt = 1) then
      Ilib.ResetDataControls(slCnt);
    if Ilib.HasViewer and Assigned(Ilib.Viewer.CellEditor.Cell) then
      Ilib.Viewer.CellEditor.Cancel;


    if not Value and Assigned(Ilib.DataHandler) and Ilib.MultiSelected and Changed then
      Refresh;

    FSelected := Value;
    Ilib.Dataset.FSelState := -1;
    if not Value and (Ilib.SelCount <= 1) and Assigned(Ilib.Dataset.FocusedRecord) then
    begin
      Ilib.DoDeselectAll;
      Ilib.Dataset.FocusedRecord.FSelected := True;
      Ilib.Dataset.FSelState := -1;
    end;

    if not Value then
      FIsDeleted := false;

    Ind := -1;
    MH := TMan.GetMessageHandler(Ilib, Ind);
    while MH <> nil do
    begin
        MH.DatasetChanged(dboNone);
        MH := TMan.GetMessageHandler(Ilib, Ind);
    end;

    if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnSelect) then
      Ilib.DataHandler.FOnSelect(Ilib, Index);

 end;

  procedure TILDataRecord.SetImageText(const Value : string);
  begin
    if FImageText = Value then
      Exit;
    FImageText := Value;
    if Ilib.HasViewer then
      Ilib.Viewer.Invalidate;
  end;

  function TILDataRecord.NeedsUpdate: Boolean;
  var
    ASource: TILRecord;
    TR: TILTestResult;
    i : integer;
  begin
    Result := false;
    if not Ilib.Connected then
      Exit;
    for I := 0 to Count-1 do
      if Items[i].FieldDef.DataType = dtLongText then
      begin
        if Items[i].TextSourceChanged then
        begin
          Result := True;
          Exit;
        end;

      end;
    if Ilib.IsControlsEmpty(duData) then
      Exit;
    if HasTempFields then
      Exit;
    ASource := TILRecord.Create(Ilib);
    try
      Ilib.ReadControls(ASource, duData);
      TR := Ilib.TestUpdate(ASource, Self, Result);
      if TR <> nil then
        TR.Free;
    finally
      ASource.Free;
    end;
  end;

  function TILDataRecord.NeedsUpdate(Expression: TStrings): Boolean;

  var
    TR: TILTestResult;
  begin
    Result := false;
    if Ilib.Dataset.IsReadOnly then
      Exit;
    if not Ilib.Connected then
      Exit;
    Ilib.FCommanded := True;
    try
      TR := Ilib.TestUpdate(Self, Expression, Result);
      if TR <> nil then
        TR.Free;
    finally
      Ilib.FCommanded := false;
    end;
  end;

  procedure TILDataRecord.EditImageText;

  var
    R: TRect;
  begin
    if Ilib.FViewer = nil then
      Exit;
    if Ilib.FViewer.ViewMode <> vmImages then
      Exit;
    if Ilib.Dataset.IsReadOnly then
      Exit;
    if FEditingImageText then
      Exit;
    if Ilib.FViewer.ImageTextSpace = 0 then
      Exit;
    if ImageTextField = nil then
      Exit;
    if TMan.UseCommandModes and ((Tman.ActiveTable <> Ilib) or (TMan.CommandMode <> cmUpdate)) then
      Exit;

    if not Assigned(Ilib.FImgTxtEditor) then
    begin
      Ilib.FImgTxtEditor := TMemo.Create(Ilib.FViewer);
      Ilib.FImgTxtEditor.Parent := Ilib.FViewer;
      Ilib.FImgTxtEditor.OnKeyDown := Ilib.FViewer.ImgTxtEditorKeyDown;
    end;
    R := Ilib.FViewer.GetItemRect(rtTextRect, Index);
    Ilib.FImgTxtEditor.Top := R.Top;
    Ilib.FImgTxtEditor.Left := R.Left;
    Ilib.FImgTxtEditor.Width := R.Width;
    Ilib.FImgTxtEditor.Height := R.Height;
    Ilib.FImgTxtEditor.Lines.Text := ImageText;
    Ilib.FImgTxtEditor.WantTabs := True;
    Ilib.FImgTxtEditor.Visible := True;
    Ilib.FImgTxtEditor.SetFocus;
    FEditingImageText := True;
  end;

  procedure TILDataRecord.CancelEditImageText;
  begin
    if not FEditingImageText then
      Exit;
    Ilib.FImgTxtEditor.Visible := false;
    FEditingImageText := false;
    if Ilib.HasViewer then
     Ilib.FViewer.Invalidate;
  end;

  procedure TILDataRecord.SetEditImageText;

  var
    F: TILField;
    MH : TILMessageHandler;
    Ind : integer;
  begin
    if not FEditingImageText then
      Exit;
    F := ImageTextField;
    if F.FValues = F.FOldValues then
    begin
      F.FOldValues := TStringList.Create;
      F.FOldValues.Assign(F.FValues);
      FOldImageText := FImageText;
    end;
    F.FValues.Text := Ilib.FImgTxtEditor.Lines.Text;
    ImageText := F.FValues.Text;
    Ilib.FImgTxtEditor.Visible := false;
    FEditingImageText := false;

    Ilib.WriteControls(Self, duData);
    Ilib.Dataset.FChngTracker.Add(Self);

    Ind := -1;
    MH := TMan.GetMessageHandler(ILib, Ind);
    while MH <> nil do
    begin
      MH.AddCount(dboUpdate, -1);
      MH := TMan.GetMessageHandler(ILib, Ind);
    end;

    if Ilib.HasViewer then
     Ilib.FViewer.Invalidate;
    if TMan.AutoSave then
      ILib.DBWork.Save;
  end;

  procedure TILDataRecord.Refresh;

  { Rereads the data }
  var
    R: TILDataRecord;
    Ind: integer;
    FOwnerDS : TILDataset;
    i : integer;
    sl : TStrings;
    s : string;
  begin
    if RecordIndex = -1 then
      Exit;
    Ind := RecordIndex;
    FOwnerDS := OwnerDataset;
    if Ilib is TILMemTable then
      R := TILMemTable(Ilib).RecordOf(RecordIndex)
    else
      R := Ilib.Records[RecordIndex];
    try
      Assign(R);
      FOwnerDataset := FOwnerDS;
      FRecordIndex := Ind;
      S := '';
      for I := 0 to Count-1 do
      begin
        if Items[i].FieldDef.LookupType = looParentField then
        begin
          sl := Ilib.GetLookupParentValue(Items[i].FieldDef.LookupTable, Items[i].FieldDef.LookupField, Self);
          Items[i].FValues.Assign(sl);
          sl.Free;
        end
        else if (Items[i].FieldDef.DataType = dtLongText) and Ilib.HasHandler
         and Assigned(Ilib.DataHandler.FOnUpdateLongText) then
         Ilib.DataHandler.FOnUpdateLongText(Ilib.Datahandler, Items[i], S, [ltaDownload]);
      end;
    finally
      if not (Ilib is TILMemTable) then
       R.Free;
    end;
    FIsDeleted := false;
    Ilib.Dataset.FChngTracker.Add(Self);
    if ILib.HasViewer then
      ILib.Inval;
    Ilib.WriteControls(Self, duData);
  end;

  procedure TILDataRecord.Save;

  { Updates lib with the record data and saves }
  var
    AdataSet: TILDataset;
  begin
    if not Changed then
      Exit;
    AdataSet := TILDataset.Create(Ilib);
    ADataset.FChngTracker.Delete(Index);
    AdataSet.FOwnsRecords := false;
    try
      FOwnerDataset := AdataSet;
      AdataSet.Add(Self);
      AdataSet.Save;
    finally
      AdataSet.Free;
    end;
  end;

  function TILDataRecord.CheckDuplicates(Checking : Boolean) : integer;
  var
    FN : string;
    Idx : integer;
    R : TILDataRecord;
    Alike : Boolean;
  I: Integer;

  function HasUniqueDups(TargetRec : TILDatarecord) : Boolean;
  var
    i, j : integer;
    Indx : integer;
    SourceRec : TILDatarecord;
    IdSame : Boolean;
    HasID : Boolean;
    Cnt : integer;
    S : string;
  begin
     Result := false;
     S := '';
     Cnt := Ilib.Dataset.ChangeCount;
     for I := 0 to Cnt-1 do
     begin
       SourceRec := Ilib.Dataset.FChngTracker.ChngRecords[i];
       if TargetRec.FIsDataControls then
       begin
       if SourceRec = Ilib.Dataset.FocusedRecord then
         Continue;
       end
       else if SourceRec = TargetRec then
         Continue;

       HasID := false;
       IDSame := True;
       for j := 0 to SourceRec.Count-1 do
       begin
        if SourceRec[j].FieldDef.Unique then
        begin
          Indx := TargetRec.IndexOf(SourceRec[j].FieldName);
          if SameText(SourceRec[j].AsString, TargetRec[Indx].AsString)
          and not SourceRec[j].IsEmpty then
          begin
            Comment1 := SourceRec[j].FieldName + ': ' + SourceRec[j].AsString;
            TargetRec.FDupRec := SourceRec;
            Result := True;
            Break;
          end;
        end;
        if SourceRec[j].FieldDef.Identifier then
        begin
          HasID := True;
          S := SourceRec[j].FieldName;
          Indx := TargetRec.IndexOf(SourceRec[j].FieldName);
          if not SameText(SourceRec[j].AsString, TargetRec[Indx].AsString)
          and not SourceRec[j].IsEmpty then
          begin
            IdSame := False;
          end;
        end;
       end;
       if Result then
         Break;
       if HasID and IDSame then
       begin
         Comment1 := S;
         TargetRec.FDupRec := SourceRec;
         Result := True;
         Break;
       end;
     end;
  end;

  function CheckUniques(Idents : Boolean) : Boolean;
  var
    Indx : integer;
    DSRec : TILDataRecord;
  begin
      {Checks if the record on disk is overruled by a record in dataset }
      Result:= True;
      Indx := OwnerDataset.IndexOf(Idx);
      if Indx <> -1 then
      begin
         DsRec := OwnerDataset.Items[Indx];
         if Idents then
           Result := Ilib.DBCommons.CompareIdents(DsRec, R) = nil
         else
           Result := Ilib.DBCommons.CompareUniques(DsRec, R) = nil;
      end;
  end;

  begin
   Result := 0;

   for I := 0 to OwnerDataset.ChangeCount-1 do
   begin
     R := OwnerDataset.FChngTracker.ChngRecords[i];
     R.FValidState := 0;
    if HasUniqueDups(R) then
    begin
       Result := E_EntryExists;
       if not Checking then
         HandleILException(E_EntryExists, '');
       Exit;
    end;

    Idx := Ilib.IndexOfID(R, FN);
    if{ (Ilib.SelCount = 1) and }(Idx <> -1) then
    begin
      Alike := not CheckUniques(True);
      if Alike then
      begin
       Result := E_EntryExists;
       Comment1 := FN;
       if not Checking then
       begin
        HandleILException(E_EntryExists);
       end;
       FDupRecID := Idx;
       Exit;
      end;
    end;

    Idx := Ilib.IndexOfUnique(R, FN);
    if {(Ilib.SelCount = 1) and }Idx <> -1 then
    begin
      Alike := not CheckUniques(False);
      if Alike then
      begin
       Result := E_EntryExists;
       Comment1 := FN;
       if not Checking then
       begin
        HandleILException(E_EntryExists, FN);
       end;
       FDupRecID := Idx;
       Exit;
      end;
    end;
   end;

  end;

  function TILDataRecord.Validate(const Checking: Boolean = false; const Tracking : integer = 0): integer;

  { Validates data before saving }
  var
    S, N, A: string;
    Indx: integer;
    i : integer;

  function CheckMaxLengths: integer;
  var
    i, j: integer;
  begin
    Result := 0;
    for i := 0 to Count - 1 do
    begin
      if Items[i].FieldDef.LookupTable <> nil then
        Continue;
      if Items[i].FieldDef.MaxLength <> 0 then
      begin
        for j := 0 to Items[i].Count - 1 do
        begin
          S := Items[i].FValues[j];
          if Items[i].FieldDef.DataType = dtResource then
          begin
            GetResourceElements(S, N, A);
            S := N;
          end;
          if Length(S) > Items[i].FieldDef.MaxLength then
          begin
            Comment1 := S;
            if Checking then
            begin
              Result := E_TextTooLong;
              Exit;
            end
            else
            begin
              HandleILException(E_TextTooLong, '');
            end;
          end;
        end;
      end
      else
      begin
        if (Items[i].FieldDef.FieldType = ftUserData) and
          (Items[i].FieldDef.DataType in [dtNumber, dtString, dtText, dtResource])
          then
          for j := 0 to Items[i].Count - 1 do
          begin
            S := Items[i].FValues[j];
            if Items[i].FieldDef.DataType = dtNumber then
            begin
              if Length(S) > Ilib.FNumberMaxLength then
              begin
                Comment1 := S;
                if Checking then
                begin
                  Result := E_TextTooLong;
                  Exit;
                end
                else
                begin
                  HandleILException(E_TextTooLong, '');
                end;
              end;
            end
            else if Items[i].FieldDef.DataType = dtText then
            begin
              if Length(S) > Ilib.FTextMaxLength then
              begin
                Comment1 := S;
                if Checking then
                begin
                  Result := E_TextTooLong;
                  Exit;
                end
                else
                begin
                  HandleILException(E_TextTooLong, '')
                end;
              end;
            end
            else if Items[i].FieldDef.DataType = dtResource then
            begin
              GetResourceElements(S, N, A);
              if Length(N) > Ilib.FNamesMaxLength then
              begin
                Comment1 := N;
                if Checking then
                begin
                  Result := E_TextTooLong;
                  Exit;
                end
                else
                begin
                  HandleILException(E_TextTooLong, '')
                end;
              end;
            end
            else
            begin
              if Length(S) > Ilib.FNamesMaxLength then
              begin
                Comment1 := S;
                if Checking then
                begin
                  Result := E_TextTooLong;
                  Exit;
                end
                else
                begin
                  HandleILException(E_TextTooLong, '');
                end;
              end;
            end;
          end;
      end;
    end;
  end;

  function CheckKeyListRecordSize: integer;

  var
    MS: TStringStream;
    S: string;
    i, j: integer;
    Indx: integer;
    N, A: string;
  begin
    Result := 0;
    if Ilib.FSF_KeyList.RecordSpace <= 0 then
      Exit;
    if Tman.Unicode then
      MS := TStringStream.Create('', TEncoding.Utf8)
    else
      MS := TStringStream.Create;
    try
      for i := 0 to Ilib.Fields.Count - 1 do
      begin
        if not Ilib.Fields.Items[i].Indexed then
          Continue;
        if Ilib.Fields.Items[i].LookupTable <> nil then
          Continue;
        Indx := IndexOf(Ilib.Fields.Items[i].FieldName);
        if Indx = -1 then
          Continue;
        for j := 0 to Items[Indx].Count - 1 do
        begin
          S := Items[Indx].FValues[j];
          if Items[Indx].FieldDef.DataType = dtResource then
          begin
            GetResourceElements(S, N, A);
            S := N;
          end
          else if Items[Indx].FieldDef.DataType = dtText then
            Exit; { Dont itereate through the words, let the database take care of it }

          MS.Clear;
          MS.WriteString(S);
          if MS.Size > Ilib.FSF_KeyList.RecordSpace then
          begin
            Comment1 := S;
            if Checking then
            begin
              Result := E_TextTooLong;
              Exit;
            end
            else
            begin
              HandleILException(E_TextTooLong, '');
            end;
          end;
        end;
      end;
    finally
      MS.Free;
    end;
  end;

  function CheckTypes: integer;
  var
    i, j: integer;
    DT: TDateTime;
    Fld: TILField;
    E: Extended;
    B: integer;
    s1, s2: string;
  begin
    Result := 0;
    for i := 0 to Count - 1 do
    begin
      Fld := Items[i];
      if (Fld.Count = 1) and TMan.IsValueNull(Fld.FValues[0])  then
        Continue;
      for j := 0 to Fld.Count - 1 do
      begin
        if Items[i].FieldDef.FDataType = dtDate then
        begin
          if (Fld.FValues[j] <> '')
           and not TryStrToDateTime(Fld.FValues[j], DT, Translator.FFormatSettings)
          then
          begin
            Comment1 := Fld.FValues[j];
            if Checking then
            begin
              Result := E_InvalidDate;
              Exit;
            end
            else
            begin
              HandleILException(E_InvalidDate);
            end;
          end;
          if (Fld.FValues[j] <> '') then
            Fld.FValues[j] := DateTimeToStr(DT, Translator.FFormatSettings); { Create full format }
        end
        else if Items[i].FieldDef.FDataType = dtBoolean then
        begin
          if not(Fld.FValues[j] = '0') and not(Fld.FValues[j] = '1') then
          begin
            Comment1 := Fld.FValues[j];
            if Checking then
            begin
              Result := E_IllegalValue;
              Exit;
            end
            else
              HandleILException(E_IllegalValue, '');
          end;
        end
        else if Items[i].FieldDef.FDataType = dtNumber then
        begin
          if not SameText(Items[i].FieldDef.FieldName,
            c_SysFieldNames[SYS_LinkID]) then
          begin
            { Will get its value in ConnectToMaster }
            if (Fld.FValues[j] <> '') and not TMan.IsValueNull(Fld.FValues[j])
             and not TryStrToFloat(Fld.FValues[j], E, Translator.FFormatSettings)
            then
            begin
              Comment1 := Fld.FValues[j];
              if Checking then
              begin
                Result := E_InvalidNumber;
                Exit;
              end
              else
                HandleILException(E_InvalidNumber);
            end;
            { Check decimals }
            B := Pos(FormatSettings.DecimalSeparator, Fld.FValues[j]);
            if B <> 0 then
            begin
              s2 := Copy(Fld.FValues[j], B + 1, Length(S));
              s1 := Copy(Fld.FValues[j], 1, B - 1);
              if Length(s2) > Items[i].FieldDef.Decimals then
              begin
                Comment1 := Fld.FValues[j];
                if Checking then
                  Result := E_TooManyDecimals
                else
                  HandleILException(E_TooManyDecimals, '');
                Exit;
              end;
            end;
          end;
        end;
      end;
    end;
  end;

  function InPickList : integer;
  var
    i, j : integer;
    Fld : TILField;
  begin
    Result := 0;
    for i := 0 to Count - 1 do
    begin
      Fld := Items[i];
      if Fld.FieldDef.PickValues.Count > 0 then
      for j := 0 to Fld.Count - 1 do
      begin
         if (Fld.FieldDef.PickValues.IndexOf(Fld.FValues[j]) = -1)
         and not TMan.IsValueNull(Fld.FValues[j]) then
         begin
           Comment1 := Fld.FValues[j];
           Result := E_ValuePickList;
           Exit;
         end;
      end;
    end;
  end;


  begin
    Result := -1;
    if (Tracking = 0) and not FIsDataControls then
    begin
      if OwnerDataset.FChngTracker.Count > 0 then
        Result := 0;
      for I := 0 to OwnerDataset.FChngTracker.Count-1 do
      begin
        if OwnerDataset.FChngTracker.ChngRecords[i].FValidState <> 0 then
        begin
          Result := OwnerDataset.FChngTracker.ChngRecords[i].FValidState;
          if not Checking then
            HandleILException(Result);
          Break;
        end;
      end;
      Exit;
    end;
    if IsDataCleared(True) and  not IsNew then
    begin
      Exit;
    end;
    if Assigned(Ilib.MasterSource) then
    begin
      Indx := IndexOf(c_SysFieldNames[SYS_LinkID]);
      if Indx = -1 then
      begin
        if Checking then
        begin
          Result := E_MissingLinkField;
          Exit;
        end
        else
          HandleILException(E_MissingLinkField, '');
      end;
    end;
    if Ilib.Fields.Count <> Count then
    begin
      if not Checking then
        HandleILException(E_RecMissingFields, '');
    end;

    if not FIsDataControls then {Checked in ReadControls}
     Result := CheckTypes
    else
     Result := 0;
    if Result <> 0 then
    begin
      if not Checking then
        HandleILException(Result);
      Exit;
    end;

    if not FIsDataControls then {Checked in ReadControls}
     Result := CheckMaxLengths
    else
     Result := 0;
    if Result <> 0 then
    begin
      if not Checking then
        HandleILException(Result);
      Exit;
    end;

    if not FIsDataControls then {Checked in ReadControls}
     Result := CheckKeyListRecordSize
    else
      Result := 0;
    if Result <> 0 then
    begin
      if not Checking then
        HandleILException(Result);
      Exit;
    end;

    if Checking and not Ilib.PrecheckConstraints and not TMan.AutoSave and FIsDataControls then
       Exit;

    if not Ilib.CheckMandatories(Self, S) then
    begin
      Comment1 := S;
      Result := E_MandatoryValue;
      if not Checking then
      begin
        HandleILException(E_MandatoryValue, S);
      end;
      Exit;
    end;

    Result := CheckDuplicates(Checking);
    if Result <> 0 then
      Exit;

    if Ilib.TVTable <> nil  then
    begin
      if Ilib.TVTable.TreeViewParams.InsertMode <> timRoot then
      begin
        if ILib.FocusedRecord = nil then
        begin
          Result := E_TVReference;
          if Checking then
          begin
            Exit;
          end
          else
            HandleILException(E_TVReference, '');
        end;
      end;
    end;

    Result := InPicklist;
    if Result <> 0 then
    begin
      if not Checking then
        HandleILException(E_ValuePicklist);
      Exit;
    end;
  end;

  procedure TILDataRecord.Assign(ASource: TILDataRecord);

  var
    Prop: TILPropertyValue;
    i: integer;
    Field: TILField;
  begin
    Clear;
    for i := 0 to ASource.Count - 1 do
    begin
      Field := TILField.Create(Ilib, ASource[i].FieldName, Ilib.FFields);
      Field.FValues.Assign(ASource[i].FValues);
      Field.FOldValues := Field.FValues;
      Field.FOwnerRecord := Self;
      Field.FTextSourceChanged := ASource[i].TextSourceChanged;
      Field.FTextUpdated := ASource[i].FTextUpdated;
      Field.FLongText := ASource[i].FLongText;
      Field.FSuppressOutput := ASource[i].FSuppressOutput;
      Field.FResFilePos := ASource[i].FResFilePos;
      Field.FUsage := ASource[i].FUsage;
      if Assigned(ASource[i].FLTStream) then
      begin
          Field.FltStream := TmemoryStream.Create;
          ASource[i].FLTStream.Position := 0;
          Field.FltStream.CopyFrom(ASource[i].FLTStream, ASource[i].FLTStream.Size);
      end;
      Add(Field);
    end;
    if Assigned(ASource.Image) then
    begin
      if Assigned(Image) then
      begin
        if FOwnsImage then
          FImage.Free;
      end;
      FImage := Vcl.Graphics.TBitmap.Create;
      FImage.Assign(ASource.Image);
      FOldImage := FImage;
      FOwnsImage := True;
    end
    else if Assigned(FImage) and OwnsImage then
      FreeAndNil(FImage);
    FState := ASource.FState;
    FOldState := ASource.FOldState;
    FOldFileName := ASource.FFileName;
    FOldImageText := ASource.FImageText;
    FOldTableIndex := ASource.FOldTableIndex;
    FOldMediaType := ASource.FOldMediaType;
    FRegDate := ASource.FRegDate;
    FChangedDate := ASource.FChangedDate;
    FFileName := ASource.FileName;
    FImgTableIndex := ASource.FImgTableIndex;
    FUserID := ASource.UserID;
    FImageText := ASource.ImageText;
    FRecordIndex := ASource.RecordIndex;
    FRecordID := ASource.RecordID;
    FMediaType := ASource.FMediaType;
    FThumbnailOffset := ASource.ThumbnailOffset;
    FImageChanged := ASource.FImageChanged;
    FThumbnailSize := ASource.ThumbnailSize;
    FOwnsImage := True;
    FImgFilePos := ASource.FImgFilePos;
    FImageFetched := ASource.FImageFetched;
    if Assigned(ASource.PropertyValues) then
    begin
      if Assigned(PropertyValues) then
        FPropertyValues.Free;
      FPropertyValues := TILPropertyValues.Create;
      FPropertyValues.Ilib := Ilib;
      for i := 0 to ASource.PropertyValues.Count - 1 do
      begin
        Prop := TILPropertyValue.Create;
        Prop.FOwner := FPropertyValues;
        Prop.FPID := ASource.PropertyValues.Items[i].FPID;
        Prop.FValue := ASource.PropertyValues.Items[i].Value;
        Prop.FDataType := ASource.PropertyValues.Items[i].DataType;
        FPropertyValues.Add(Prop);
      end;
    end;
  end;

  procedure TILDataRecord.AssignFields(ASource: TILRecord);

  { Copies data from ASource into corresponding fields. }
  var
    i: integer;
    FldIndx: integer;
    Fld: TILField;
  begin
    for i := 0 to ASource.Count - 1 do
    begin
      FldIndx := IndexOf(ASource[i].FieldName);
      if FldIndx <> -1 then
      begin
        Fld := Items[FldIndx];
        Fld.FValues.Assign(ASource[i].FValues);
        Fld.FLongText := ASource[i].FLongText;
        Fld.FTextSourceChanged := ASource[i].TextSourceChanged;
        Fld.FTextUpdated := ASource[i].FTextUpdated;
        if Assigned(ASource[i].FLTStream) then
        begin
          Fld.FltStream := TmemoryStream.Create;
          ASource[i].FLTStream.Position := 0;
          Fld.FltStream.CopyFrom(ASource[i].FLTStream, ASource[i].FLTStream.Size);
        end;
      end
      else
      begin
        Fld := TILField.Create(Ilib, ASource[i].FieldName, Ilib.FFields);
        Fld.FOwnerRecord := Self;
        Fld.FValues.Assign(ASource[i].FValues);
        Add(Fld);
      end;
    end;
  end;

  procedure TILDataRecord.AssignMedia(ASource: TILDataRecord);

  { Copies image and image reletated data from ASource into corresponding fields.
    Exisiting images are destroyed }
  var
    i: integer;
    Prop: TILPropertyValue;
    RestoreOld: Boolean;
    Indx: integer;
  begin
    RestoreOld := false;
    FFileName := ASource.FileName;
    FThumbnailOffset := ASource.FThumbnailOffset;
    FThumbnailSize := ASource.FThumbnailSize;
    FMediaType := ASource.MediaType;
    if (FOldImage <> FImage) and Assigned(FOldImage) then
    begin
      FreeAndNil(FOldImage);
      RestoreOld := True;
    end;
    if Assigned(FImage) then
      FreeAndNil(FImage);
    if Assigned(ASource.FImage) then
    begin
      FImage := Vcl.Graphics.TBitmap.Create;
      FImage.Assign(ASource.FImage);
      if RestoreOld then
        FOldImage.Assign(ASource.FOldImage);
    end;
    FImageText := ASource.ImageText;
    FOldImageText := ASource.FOldImageText;
    FImageChanged := True;
    FImageFetched := ASource.FImageFetched;
    FOwnsImage := True;
    if not RestoreOld then
      FOldImage := FImage;

    if Assigned(ASource.PropertyValues) then
    begin
      if Assigned(PropertyValues) then
        FPropertyValues.Free;
      FPropertyValues := TILPropertyValues.Create;
      FPropertyValues.Ilib := Ilib;
      for i := 0 to ASource.PropertyValues.Count - 1 do
      begin
        Prop := TILPropertyValue.Create;
        Prop.FOwner := FPropertyValues;
        Prop.FPID := ASource.PropertyValues.Items[i].FPID;
        Prop.FValue := ASource.PropertyValues.Items[i].Value;
        Prop.FDataType := ASource.PropertyValues.Items[i].DataType;
        FPropertyValues.Add(Prop);
        Indx := IndexOf(c_SysFieldNames[Prop.FPID]);
        if Indx <> -1 then
        begin
          Items[Indx].FValues.Clear;
          Items[Indx].FValues.Add(Prop.Value);
        end;
      end;
    end;

  end;

  function TILDataRecord.HasLocalImage: Boolean;
  begin
    Result := (ThumbnailSize > 0);
  end;

  function TILDataRecord.IsNew: Boolean;
  begin
    Result := (FRecordIndex = -1);
  end;

  procedure TILDataRecord.CopyImageLink;

  var
    S: string;
    AData: THandle;
    APalette: HPalette;
    MyFormat: Word;
  begin
    if ThumbnailSize <= 0 then
      HandleILException(E_ImageLinkNotFound, '');
    S := 'ILib,' + FileName + ',' + IntToStr(ThumbnailOffset) + ',' +
      IntToStr(ThumbnailSize);

    ClipBoard.Open;
    ClipBoard.Clear;
    ClipBoard.AsText := S;
    if Assigned(FImage) then
    begin
      FImage.SaveToClipBoardFormat(MyFormat, AData, APalette);
      ClipBoard.SetAsHandle(cf_Bitmap, AData);
    end;
    ClipBoard.Close;
  end;

  procedure TILDataRecord.AddToImageTable(const TagName: string);
  begin
    if TMan = nil then
      Exit;
    if TMan.FSharedImages = nil then
      HandleILException(E_MissingLookupTable);
    TMan.FSharedImages.Add(TagName, Self);
  end;

  function TILDataRecord.GetFromImageTable(const TagName: string) : Boolean;
  begin
    Result := false;
    if TMan = nil then
      Exit;
    if TMan.FSharedImages = nil then
      Exit;
    Result := TMan.FSharedImages.ShareWith(TagName, Self);
  end;

  procedure TILDataRecord.CopyImage;

  var
    S: string;
    AData: THandle;
    APalette: HPalette;
    MyFormat: Word;
  begin
    if not Assigned(FImage) then
      HandleILException(E_CopyNoImage, '');
    ClipBoard.Open;
    ClipBoard.Clear;
    ClipBoard.AsText := S;
    FImage.SaveToClipBoardFormat(MyFormat, AData, APalette);
    ClipBoard.SetAsHandle(cf_Bitmap, AData);
    ClipBoard.Close;
  end;

  function TILDataRecord.CanPasteImage(var HasLink: Boolean): Boolean;

  var
    S: string;
  begin
    HasLink := false;
    Result := ClipBoard.HasFormat(cf_Bitmap);
    if Result and ClipBoard.HasFormat(CF_TEXT) then
    begin
      S := ClipBoard.AsText;
      HasLink := (Pos('ILib,', S) = 1);
    end;
  end;

  procedure TILDataRecord.PasteImageLink;

  var
    S: string;
    sl: TStringList;
    Rec: TILDataRecord;
  begin
    if ClipBoard.HasFormat(CF_TEXT) then
    begin
      Rec := Self;
      S := ClipBoard.AsText;
      if Pos('ILib,', S) = 1 then
      begin
        sl := TStringList.Create;
        try
          CreateListFromDelimiter(',', S, sl);
          Rec.FFileName := sl[1];
          Rec.FThumbnailOffset := StrToInt(sl[2]);
          Rec.FThumbnailSize := StrToInt(sl[3]);
          if ClipBoard.HasFormat(cf_Bitmap) then
          begin
            Rec.FImage.LoadFromClipboardFormat(cf_Bitmap,
              ClipBoard.GetAsHandle(cf_Bitmap), 0);
          end;
        finally
          sl.Free;
        end;
        Ilib.Inval;
        Exit;
      end
      else
        HandleILException(E_ClipBrdNoLink, '');
    end
    else
      HandleILException(E_ClipBrdNoLink, '');
  end;

  procedure TILDataRecord.PasteImage;

  var
    Bitmap: Vcl.Graphics.TBitmap;
    Pic: TPicture;
  begin
    if not ClipBoard.HasFormat(cf_Bitmap) then
    begin
      HandleILException(E_ClipBrdNoImage, '');
    end;
    Bitmap := Vcl.Graphics.TBitmap.Create;
    Pic := TPicture.Create;
    try
      Bitmap.LoadFromClipboardFormat(cf_Bitmap,
        ClipBoard.GetAsHandle(cf_Bitmap), 0);
      Pic.Assign(Bitmap);
      UpdateImage(Pic)
    finally
      Bitmap.Free;
      Pic.Free;
    end;
  end;

  procedure TILDataRecord.RemoveImage;
  begin
    if Assigned(FImage) then
    begin
      if FImage = FOldImage then
      begin
        FOldImage := Vcl.Graphics.TBitmap.Create;
        FOldImage.Assign(FImage);
      end;
      FreeAndNil(FImage);
    end;
    FOldTableIndex := FImgTableIndex;
    FImgTableIndex := -1;
    FFileName := '';
    FThumbnailOffset := 0;
    FThumbnailSize := 0;
    Ilib.Inval;
  end;

  procedure TILDataRecord.UpdateImage(AFileName: string);

  { Replaces the image, if any,  contained in ADataRecord with the one contained in AFilename }
  var
    ItmIdx: integer;
    P : TPicture;
    Url : Boolean;
    Img : TWicImage;
    NormalFile : Boolean;
    Ind : integer;
    MH : TILMessageHandler;
  begin
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    if SameText(AFileName, CLIPBOARD_IMAGE) then
    begin
     P := nil;
     NormalFile := false;
     try
      if Clipboard.HasFormat(CF_BITMAP) then
      begin
        P := TPicture.Create;
        P.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(cf_bitmap), 0);
      end
      else if Clipboard.HasFormat(CF_Text) then
      begin
         Url := false;
         AFileName := GetAddressFromClipBrd(Url);
         if AFilename = '' then
            HandleILException(E_ImagenotAvailable)
         {$IfDef UseIndy}
         else if Url then
         begin
           P := TPicture.Create;
           Img := DownLoadImage(AFileName);
           try
             P.Assign(Img);
           finally
             if Img <> nil then
               Img.Free;
           end;
         end
         {$EndIf}
         else if Url then
          HandleILException(E_ImagenotAvailable)
         else
           NormalFile := True;
      end
      else
         HandleILException(E_MissingImage);
      UpdateImage(P);
     finally
      P.Free;
     end;
     if not NormalFile then
      Exit;
    end;

    FOldMediaType := FMediaType;
    if IsVideoFile(AFileName) then
      FMediaType := MED_VIDEO
    else if IsImageFile(AFileName) then
      FMediaType := MED_IMAGE
    else if IsAudioFile(AFileName) then
      FMediaType := MED_AUDIO;
    if not IsUrl(AFileName) then
      if not FileExists(AFileName) then
      begin
        Comment1 := AFileName;
        HandleILException(E_FileNotExists);
      end;
    if SameText(FFileName, AFileName) then
      Exit;
    ItmIdx := FRecordIndex; { Will be reset in LoadFiledata }
    if Assigned(FImage) then
    begin
      if FImage = FOldImage then
      begin
        FOldImage := Vcl.Graphics.TBitmap.Create;
        FOldImage.Assign(FImage);
      end;
      FreeAndNil(FImage);
    end;
    FOldFileName := FFileName;
    LoadFileData(AFileName);
    FRecordIndex := ItmIdx;
    AssignSystemValues;
    FImageChanged := True;
    FOldTableIndex := FImgTableIndex;
    FImgTableIndex := -1;
    Ilib.Dataset.FChngTracker.Add(Self);
    if TMan.DLLAccess then
      Exit;

    Ind := -1;
    MH := TMan.GetMessageHandler(Ilib, Ind);
    while MH <> nil do
    begin
       MH.AddCount(dboUpdate, -1);
       MH := TMan.GetMessageHandler(Ilib, Ind);
    end;
    Ilib.Inval;
    if Ilib.HasHandler and Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsImageUpdated);
    if TMan.AutoSave then
      Ilib.Dataset.Save;
  end;

  procedure TILDataRecord.UpdateImage(AnImage: TPicture);

  { Replaces the image, if any,  contained in ADataRecord with the one contained in AnImage }
  var
    W, H: integer;
    Ind : integer;
    MH : TILMessageHandler;
  begin
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    FOldMediaType := FMediaType;
    FMediaType := MED_IMAGE;
    if not MustStoreLocally('') then
      Exit;
    if Assigned(FImage) then
    begin
      if FImage = FOldImage then
      begin
        FOldImage := Vcl.Graphics.TBitmap.Create;
        FOldImage.Assign(FImage);
      end;
      FreeAndNil(FImage);
    end;
    if Assigned(AnImage) then
    begin
      FImage := Vcl.Graphics.TBitmap.Create;
      FImage.Assign(AnImage.Bitmap);
      if Filename <> '' then
      begin
        Ilib.GetImgDimensions(W, H);
        ScaleBitmap(FImage, W, H);
      end;
      FImageFetched := True;
    end;
    FOldFileName := FFileName;
    FFileName := '';
    FImageChanged := True;
    FOldTableIndex := FImgTableIndex;
    FImgTableIndex := -1;
    FOwnsImage := True;
    Ilib.Dataset.FChngTracker.Add(Self);
    if TMan.DLLAccess then
      Exit;

    Ind := -1;
    MH := TMan.GetMessageHandler(Ilib, Ind);
    while MH <> nil do
    begin
       MH.AddCount(dboUpdate, -1);
       MH := TMan.GetMessageHandler(Ilib, Ind);
    end;

    Ilib.Inval;
    if Ilib.HasHandler and Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsImageUpdated);
    if TMan.AutoSave then
      Ilib.Dataset.Save;

  end;

  procedure TILDataRecord.FetchImage;

  var
    PP: TPropertyProvider;
  begin
    if ThumbnailSize = 0 then
    begin
      if FFileName <> '' then
      begin
        try
        LoadImage(FFileName);
        except
          FImage := nil;
        end;
        FOldImage := FImage;
        if FImage <> nil then
        begin
          PP := TPropertyProvider.Create(FileName);
          Ilib.FImageMem := Ilib.FImageMem + PP.GetPropValueAsVariant
            (PID_FILE_SIZE);
          PP.Free;
        end;
      end
      else
        FImage := nil;
    end
    else
    begin
      FImage := Ilib.GetThumbnail(ThumbnailOffset, ThumbnailSize);
      FOldImage := FImage;
      Ilib.FImageMem := Ilib.FImageMem + ThumbnailSize;
    end;

    FImageFetched := True;
  end;

  constructor TILDataRecord.Create(TableOwner: TILTable; const CreateFields: Boolean);
  begin
    inherited Create(TableOwner);
    FRecordIndex := -1;
    FImgTableIndex := -1;
    FOldTableIndex := -1;
    FTVParentID := -1; { TreeViewTable uses this }
    FTVSiblingIndex := -1;
    FImgFilePos := -1;
    FDupRecID := -1;
    FRegDate := Now;
    if TableOwner is TILTreeViewTable then
    begin
      FSiblingList := TStringList.Create;
      FSiblingList.OwnsObjects := True;
    end;
    FChangedDate := Now;
    if CreateFields then
      InitFields(nil);

  end;

  destructor TILDataRecord.Destroy;
  begin
    if FOwnsImage then
    begin
      if Assigned(FOldImage) and (FOldImage <> FImage) then
        FOldImage.Free;
      if Assigned(FImage) then
        FImage.Free;
    end;
    if Assigned(PropertyValues) then
      PropertyValues.Free;
    if Assigned(FSiblingList) then
      FSiblingList.Free;
    inherited Destroy;
  end;

  procedure TILDataRecord.InitFields(InitialValues: TILDataRecord;
    const IncludeTemps: Boolean = True);

  { Creates all fields and gives value to system fields.
    Sets initial values according to fields in InitialValues }
  var
    i, j: integer;
    Fld: TILField;
    Indx: integer;
    sl: TStringList;
  begin
    for i := 0 to Ilib.FFields.Count - 1 do
    begin
      if Ilib.FFields.Items[i].FTemporary and not IncludeTemps then
        Continue;
      Fld := TILField.Create(Ilib, Ilib.FFields.Items[i].FieldName,
        Ilib.FFields);
      Fld.FOwnerRecord := Self;
      if Ilib.FFields.Items[i].FieldType = ftSystemData then
      begin
        sl := ValueOfSysField(Fld.FieldName);
        try
          for j := 0 to sl.Count - 1 do
          begin
            Fld.FValues.Add(sl[j]);
          end;
        finally
          sl.Free;
        end;
      end
      else if InitialValues <> nil then
      begin
        Indx := InitialValues.IndexOf(Fld.FieldName);
        if (Indx <> -1) then
        begin
          Fld.FValues.Assign(InitialValues.Items[Indx].FValues);
        end;
      end;
      Add(Fld);
    end;
  end;

  procedure TILDataRecord.AssignSystemValues;

  { Assign values to existing system fields }
  var
    i, j: integer;
    FldDef: TILFieldDef;
    sl: TStringList;
  begin
    if IsUrl(FileName) then
      Exit;
    for i := 0 to Count - 1 do
    begin
      FldDef := Items[i].FieldDef;
      if FldDef.FieldType = ftSystemData then
      begin
        if SameText(FldDef.FFieldName, c_SysFieldNames[SYS_LinkID])
        or SameText(FldDef.FFieldName, c_SysFieldNames[SYS_Level])
        or SameText(FldDef.FFieldName, c_SysFieldNames[SYS_SiblingIndex])
        then
          Continue;

        if SameText(FldDef.FFieldName, PidNameOf(PID_PHOTO_CAMDATE))
        then
        begin
          sl := ValueOfPropField(FldDef.FieldName);
          if sl.Count = 0 then
          begin
            sl.Free;
            Continue;
          end;

          try
            if (Items[i].Count = 0) and (sl.Count = 1) then
              Items[i].FValues.Add(sl[0]);
          finally
            sl.Free;
          end;
          if Items[i].AsDateTime <> 0 then
            Continue;
        end;
        sl := ValueOfSysField(FldDef.FieldName);
        Items[i].FValues.Clear;
        try
          for j := 0 to sl.Count - 1 do
          begin
            Items[i].FValues.Add(sl[j]);
          end;
        finally
          sl.Free;
        end;
      end;
    end;
  end;

  procedure TILDataRecord.ConnectToMaster(const MasterID: integer);

  var
    S: string;
    Indx: integer;
    sl : TStringList;
    N : integer;
    i : integer;
  begin
    if ILib.Relations = relOne then
    begin
       sl := TStringList.Create;
       try
         sl.Add('SYS_LinkID = ' + IntToStr(MasterID));
         Ilib.DBView.SearchCount('SYS_LinkID', N , sl);
         if N > 0 then
           HandleILException(E_RelConstraint);
       finally
         sl.Free;
       end;

    end;
    S := FillZeroes(IntToStr(MasterID), 7);
    Indx := IndexOf(c_SysFieldNames[SYS_LinkID]);
    if Items[Indx].FValues.Count = 0 then
      Items[Indx].FValues.Add(S)
    else
      Items[Indx].FValues[0] := S;
    if OwnerDataset <> nil then
    begin
    for I := 0 to OwnerDataset.ChangeCount-1 do
      if OwnerDataset.FChngTracker.ChngRecords[i].FValidState <> 0 then
       HandleILException(OwnerDataset.FChngTracker.ChngRecords[i].FValidState, S);
    end;
    //if Ilib.IndexOfID(Self, S) <> -1 then
    //  HandleILException(E_EntryExists, S);
    //if Ilib.IndexOfUnique(Self, S) <> -1 then
    //  HandleILException(E_EntryExists, S);

  end;

  procedure TILDataRecord.ConnectToMaster(ATVParentRecord: TILDataRecord);

  begin
    FTVParentRecord := ATVParentRecord;
  end;

  procedure TILDataRecord.LoadImage(const AFileName: string);

  var
    W, H: integer;
    res: HRESULT;

  procedure GetThumb;
  begin
      FImage := Vcl.Graphics.TBitmap.Create;
      res := GetMediaThumb(AFileName, W, FImage);
      if not Succeeded(res) then
      begin
        FreeAndNil(FImage);
      end;
  end;
  begin
    Ilib.GetImgDimensions(W, H);
    if IsVideoFile(AFileName) or IsAudioFile(AFileName) then
    begin
      if (W = -1)  then
        W := 500;
      GetThumb;
    end
    else if (W <> -1) and (W <= 200) and not IsUrl(FileName)
    then { Bigger than 200 drops quality seriously }
    begin
      GetThumb;
    end
    else
      FImage := GetBitmapFromFile(AFileName, Ilib.GetJPEGQuality);
    if FImage = nil then
    begin
      Comment1 := AFilename;
      HandleILException(E_UnsupportedFileFormat);
    end;
    if (W > 200) or (W = -1) then
    begin
      if GetFileType(AFileName) = iftJPG then
        CheckOrientation(AFileName, FImage);
    end;
    ScaleBitmap(FImage, W, H);
    OwnsImage := True;
    FImageFetched := True;
  end;

  procedure TILDataRecord.LoadVideoFrame;

  var
    W, H: integer;
    res: HRESULT;

  begin
    FImage := Vcl.Graphics.TBitmap.Create;
    Ilib.GetImgDimensions(W, H);
    if W = -1 then
      W := 500;
    res := GetMediaThumb(AFileName, W, FImage);
    if not Succeeded(res) then
    begin
      FreeAndNil(FImage);
      HandleILException(E_ImageNotAvailable);
    end;
    ScaleBitmap(FImage, W, H);
    FImageFetched := True;
    OwnsImage := True;
  end;

  procedure TILDataRecord.CreateProperties(const AFileName: string);

  var
    i: integer;
    Props: TPropertyProvider;
    Prop: TILPropertyValue;
  begin
    if AFileName = '' then
      Exit;
    if Ilib.DataHandler = nil then
      Exit;
    if Assigned(PropertyValues) then
      Exit;
    if IsUrl(AFileName) then
    begin
      if (Ilib.DataHandler.PropertyList.Count > 0) and not Assigned(FPropertyValues) then
      begin
        FPropertyValues := TILPropertyValues.Create;
        FPropertyValues.Ilib := Ilib;
      end;
      { Create an empty list }
      Exit;
    end;
    Props := nil;
    if Ilib.DataHandler.PropertyList.Count > 0 then
      Props := TPropertyProvider.Create(AFileName);
    try
      for i := 0 to Ilib.DataHandler.PropertyList.Count - 1 do
      begin
        if MediaType = MED_IMAGE then
        begin
          if (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_MED_') <> 0) or
            (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_VIDEO_') <> 0) or
            (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_AUDIO_') <> 0)
          then
            Continue;
        end
        else if MediaType = MED_VIDEO then
        begin
          if (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_IMAGE_') <> 0)
            or (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_PHOTO_') <> 0)
            or (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_AUDIO_') <> 0)
          then
            Continue;
        end
        else if MediaType = MED_AUDIO then
        begin
          if (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_IMAGE_') <> 0)
            or (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_PHOTO_') <> 0)
            or (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_VIDEO_') <> 0)
            or (PosCaseless(Ilib.DataHandler.PropertyList.Items[i].PIDName, '_GPS_') <> 0)
          then
            Continue;
        end;

        Prop := TILPropertyValue.Create;
        Prop.FValue := Props.GetPropValueAsVariant
          (Ilib.DataHandler.PropertyList.Items[i].PID);
        if VarIsNull(Prop.FValue) then
        begin
          Prop.Free;
          Continue;
        end;
        Prop.FPID := Ilib.DataHandler.PropertyList.Items[i].PID;
        Prop.FDataType := Ilib.DataHandler.PropertyList.Items[i].DataType;

        if not Assigned(PropertyValues) then
        begin
          FPropertyValues := TILPropertyValues.Create;
          FPropertyValues.Ilib := Ilib;
        end;
        Prop.FOwner := FPropertyValues;
        PropertyValues.Add(Prop)
      end;
    finally
      if Assigned(Props) then
        Props.Free;
    end;
  end;

  procedure TILDataRecord.LoadFileData(const AFileName: string);
  { Sets the system data of a datarecord when a new file is added.
    Also called from UpdateImage }
  begin
    if not IsMediaFile(AFileName) then
      Exit;
    FFileName := AFileName;
    if IsVideoFile(AFileName) or IsAudioFile(AFileName) then
    begin
      LoadVideoFrame(AFileName);
      if IsVideoFile(AFileName) then
        FMediaType := MED_VIDEO
      else
        FMediaType := MED_Audio;
      CreateProperties(AFileName);
      Exit;
    end;
    FMediaType := MED_IMAGE;
    LoadImage(AFileName);

    { Set properties: }
    CreateProperties(AFileName);
  end;

  procedure TILDataRecord.CalcFields;
  var
    i : integer;
  begin
    for I := 0 to Count-1 do
    begin
      if (Items[i].FieldDef.FieldType = ftCalculated) and not Items[i].FieldDef.FUserCalc
      and Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
      begin
         Ilib.DataHandler.FOnCalcField(Ilib, Self, Items[i]);
         Items[i].FIsCalculated := True;
      end
      else if (Items[i].FieldDef.FieldType = ftCalculated) and
      Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnCalcField) then
      begin
        Items[i].FieldDef.CalcUserCalc(Self);
        Items[i].FIsCalculated := True;
      end;

    end;
  end;

  procedure TILDataRecord.SaveToFile(const AFileName : string);
  { Saves the record to a file }
  var
    SL : TstringList;
    S : string;
  begin
     SL := TStringList.Create;
     try
     S := ILib.RecordToXML(Self, false);
     SL.Add(S);
     SL.SaveToFile(AFileName);
     finally
       SL.Free;
     end;
  end;

  procedure TILDataRecord.LoadFromFile(const AFileName : string);
  { Loads a record from a file and assigns it to the record }
  var
    SL : TStringList;
    R : TILDataRecord;
  begin
    SL := TStringList.Create;
    try
      SL.LoadFromFile(AFileName);
      R := ILIB.XMLToRecord(SL[0], True);
      try
        Assign(R);
      finally
        R.Free;
      end;
    finally
      SL.Free;
    end;
  end;

  { TILDataSet ----------------------------------------------------------------- }

  function TILDataset.GetItems(Index: integer): TILDataRecord;
  begin
    if Index < 0 then
      Result := nil
    else
      Result := TILDataRecord(FDataset[Index]);
  end;

  function TILDataset.GetSelection(Index: integer): TILDataRecord;

  var
    i: integer;
    Indx: integer;
    MulSel : Boolean;
  begin
    Result := nil;
    MulSel := Ilib.MultiSelected;
    if MulSel and (Index < 0) or (Index > Selcount-1) then
      HandleIlException(E_OutOfBounds, IntToStr(Index));
    if not MulSel then
    begin
      Result := FocusedRecord;
      Exit;
    end;
    if FSelState = 0 then
      Result := TILDataRecord(FSelBuffer[Index])
    else
    begin
     Indx := 0;
     FSelBuffer.Clear;
     for i := 0 to Count - 1 do
     begin
       if Items[i].Selected then
       begin
         if Indx = Index then
         begin
           Result := Items[i];
           FSelBuffer.Add(Items[i]);
           Break;
         end;
         Inc(Indx);
       end;
     end;
     FSelState := 0;
    end;
  end;

  constructor TILDataset.Create(TableOwner: TILTable);
  begin
    Ilib := TableOwner;
    FOwnsRecords := True;
    FDataset := TList.Create;
    FPosition := -1;
    FSelBuffer := TList.Create;
    FChngTracker := TChangeTracker.Create;
  end;

  function TILDataset.CanSave(var ErrorIndex : integer) : integer;
  var
    i : integer;
  begin
     Result := -1;
     if not TMan.Connected then
     begin
       Result := E_NoConnection;
       Exit;
     end;
     ErrorIndex := -1;
     if not Changed then
       Exit;
     if FIsReadOnly then
       Exit;

     for I := 0 to ChangeCount-1 do
     begin
        begin
          Result := FChngTracker.ChngRecords[i].FValidState;
          if Result <> 0 then
          begin
           ErrorIndex := FChngTracker.ChngRecords[i].Index;
           Comment1 := FChngTracker.ChngRecords[i].FValidText;
           Exit;
          end;
        end;
     end;
  end;

  function TILDataset.CanRevert(const SelectionOnly : Boolean = true) : Boolean;
  var
    i : integer;
  begin
      Result := False;
      if not Changed then
        Exit;
      if SelectionOnly then
      for I := 0 to SelCount-1 do
      begin
        if not Selection[i].IsNew and Selection[i].Changed then
        begin
          Result := True;
          Break;
        end;
      end

      else for I := 0 to Count-1 do
      begin
        if not Items[i].IsNew and Items[i].Changed then
        begin
          Result := True;
          Break;
        end;
      end;
  end;

  procedure TILDataset.Clear(const Recursive : Boolean = True);
  var
    i: integer;
    MH : TILMessageHandler;
  begin
    if Count = 0 then
      Exit;

    if (Self = Ilib.Dataset) then
      Ilib.ClearEdits;
    if Ilib.HasViewer then
    begin
     if (Ilib.Viewer.ZoomImage <> nil) and (Self = Ilib.Dataset) then
      Ilib.Viewer.ZoomImage.ClearSource;
    end;
    if (Self = Ilib.Dataset) and (ILIB.DataHandler.SummaryControl <> nil) then
    begin
      if Ilib.FActive then
      begin
        Ilib.DataHandler.SummaryControl.Items.Clear;
      end;
    end;
    if Items[0].HasTempFields then
      Ilib.Fields.ClearCalcFields;

    if FOwnsRecords then
      for i := 0 to Count - 1 do
      begin
        Items[i].Free;
      end;
    FDataset.Clear;
    FPosition := -1;
    if (Self = Ilib.Dataset) and Ilib.HasViewer then
    begin
      Ilib.Viewer.FReport.Clear;
    end;
    if (ILIB.TVControl <> nil) and (Self = Ilib.Dataset) then
    begin
      ILIB.TVControl.Building := True;
      ILIB.TVControl.Items.Clear;
      ILIB.TVControl.Building := false;
    end;
    if (Self = Ilib.Dataset) and (Ilib.FViewer <> nil) then
    with Ilib.FViewer do
    begin
      FirstItemIndex := 0;
      FTLImages := 0;
      FTLReport := 0;
      UpdateScrollRange;
      UpdateScrollPos;
      Ilib.Inval;
    end;
    FChngTracker.Clear;
    i := -1;
    MH := TMan.GetMessageHandler(ILib, i);
    while MH <> nil do
    begin
     MH.Reset;
     MH := TMan.GetMessageHandler(ILib, i);
    end;
    RunFocus(-1, Recursive);
  end;

  function TILDataset.Validate(var ErrorIndex: integer): integer;
  var
    i: integer;
    R: TILDataRecord;
  begin
    Result := -1;
    ErrorIndex := -1;
    if Count = 0 then
      Exit;
    for i := 0 to Count - 1 do
    begin
      R := Items[i];
      if R.Changed then
        Result := R.Validate(True);
      if Result > 0 then
      begin
        ErrorIndex := i;
        Break;
      end;
    end;
  end;

  destructor TILDataset.Destroy;
  var
    i: integer;
  begin
    if FOwnsRecords then
    begin
      for i := 0 to Count - 1 do
      begin
        Items[i].Free;
      end;
    end;
    if FColumnSource <> nil then
    begin
      FColumnSource.FFieldSource.Free;
      FColumnSource.Free;
    end;
    FDataset.Free;
    FSelBuffer.Free;
    FChngTracker.Free;
    inherited Destroy;
  end;

  procedure TILDataset.BeginUpdate;
  begin
    FUpdating := True;
  end;

  procedure TILDataset.EndUpdate;
  begin
    FUpdating := false;
    if FTriggerChange and Assigned(Ilib.DataHandler) then
    begin
      Ilib.Datahandler.CreateSummary(nil);
      if Assigned(Ilib.DataHandler.FOnFocusRecord) then
        PostMessage(Ilib.FHwnd, WM_RunFocus, 0, 0);
    end;
    FTriggerChange := false;
    if (Self = Ilib.Dataset) and Ilib.HasViewer then
      Ilib.Viewer.FReport.Redraw;
    if Ilib.HasViewer then
    begin
      Ilib.FViewer.UpdateScrollRange;
      Ilib.FViewer.UpdateScrollPos;
    end;
  end;

  procedure TILDataset.Next;
  begin
    Position := Position + 1;
  end;

  procedure TILDataset.Previous;
  begin
    Position := Position - 1;
  end;

  procedure TILDataset.First;
  begin
    Position := 0;
  end;

  procedure TILDataset.Last;
  begin
    Position := Count - 1;
  end;

  function TILDataset.EOF : Boolean;
  begin
    Result := (Count = 0) or (Position = Count-1);
  end;

  function TILDataset.BOF : Boolean;
  begin
    Result := (Count = 0) or (Position = 0);
  end;

  procedure TILDataset.MoveTo(const Index: integer);
  begin
    Position := Index;
  end;

  function TILDataset.Save: integer;

  var
    i, j: integer;
    OldPos: integer;
    Rec: TILDataRecord;
    HasDeletions : Boolean;
    S : string;

  procedure RemoveDeletions;
  var
    i : integer;
    DR : TILDataRecord;
  begin
      i := 0;
      while i < Count do
      begin
         DR := Items[i];
         if DR.FIsDeleted then
         begin
           FChngTracker.Delete(DR);
           DR.Free;
           FDataset.Delete(i);
         end
         else
         begin
           inc(i);
         end;
      end;
      FSelstate := -1;
      if Ilib.FViewer <> nil then
      begin
        Ilib.FViewer.Report.Redraw;
        Ilib.FViewer.UpdateScrollRange;
        Ilib.FViewer.UpdateScrollPos;
        Ilib.Fviewer.Invalidate;
      end;
      if Position > Count -1 then
        FPosition := Count-1;
      if Self = Ilib.Dataset then
        Ilib.Dataset.RunFocus(Ilib.Dataset.Position, False);
  end;

  begin
    if IsReadOnly then
      HandleILException(E_ReadOnly);
    Result := 0;
    if not Ilib.Connected then
      Exit;
    if IsReadOnly then
      HandleILException(E_DatasetReadOnly, '');
    { Validate }
    if not(Self = Ilib.Dataset) then
    begin
      for i := 0 to Count - 1 do
        Items[i].FSelected := True;
    end;

    if ILib.Dataset.FocusedRecord <> nil then
    begin
     if TMan.AutoUpdate and ILib.DBWork.NeedsUpdate(ILib.Dataset.FocusedRecord) then
     begin
      ILib.DbWork.Update(nil);
      S := '';
      for J := 0 to Ilib.FocusedRecord.Count-1 do
      begin
        if (Ilib.FocusedRecord[j].FieldDef.DataType = dtLongText)
        and Ilib.FocusedRecord[j].TextSourceChanged
        and Assigned(ILib.DataHandler.FOnUpdateLongText) then
          ILib.DataHandler.FOnUpdateLongText(ILib, Ilib.FocusedRecord[j], S, [ltaUpload]);
      end;
     end;
    end;

    for i := 0 to ChangeCount - 1 do
    begin
      Rec := FChngTracker.ChngRecords[i];
      if Ilib.MultiSelected and not Rec.Selected then
        Continue;

      if Rec.IsDataCleared(True) and not Rec.FIsDeleted then
       Continue;
      if Rec.FValidState <> 0 then
        HandleILException(Rec.FValidState, Rec.FValidText);
    end;

    OldPos := Position;
    HasDeletions := false;
    if not TMan.FSavingAll and (Ilib = TMan.FTransOwner) then
    begin
     Tman.SetTransState(TrPending);
    end;
    FUpdater := TMan.FUpdater;
    if Tman.FUpdater = nil then
    begin
        Inc(TMan.FUpdateIDCounter);
        TMan.AddToUpdateStack(Ilib, TMan.FUpdateIDCounter);
    end;
    try
    try
    for i := 0 to ChangeCount - 1 do
    begin
      Rec := FChngTracker.ChngRecords[i];
      if Ilib.MultiSelected and not Rec.Selected then
        Continue;
      if Rec.IsDataCleared(True) and not Rec.FIsDeleted then
        Continue;
      if Rec.IsNew then
      begin
        Ilib.DoAddToTable(Rec);
        Rec.FImageChanged := false;
        Inc(Result);
      end
      else
      begin
        if Rec.Changed then
        begin
          if Rec.FIsDeleted then
          begin
            HasDeletions := True;
            if not Rec.Deleted then
            { DeleteFromTable has al1ready deleted this record in a TreeViewTabele cascade delete}
              Ilib.DeleteFromTable(Rec.RecordIndex, True);
            inc(Result);
            Continue;
          end
          else if Rec.FImageChanged then
          begin
            if Rec.FImgTableIndex <> -1 then
            begin

            end
            else if Assigned(Rec.Image) and Rec.MustStoreLocally(Rec.FileName) then
            begin
              Ilib.SaveImage(Rec.FImage, Rec.FThumbnailOffset,
                Rec.FThumbnailSize);
            end
            else
            begin
              Rec.FThumbnailOffset := 0;
              Rec.FThumbnailSize := 0;
              Rec.FFileName := '';
            end;
          end;
          Ilib.Records[Rec.FRecordIndex] := Rec;
          Rec.FImageChanged := false;
          Inc(Result);
        end;
      end;
    end;
    except on E:Exception do
     begin
        if TMan.FSavingAll or (FUpdater <> nil) then
          raise;
        if ILException(E).ErrorCode = E_TransError then
          raise;
        TMan.HandleTransError(E);
     end;
    end;

    if not TMan.FSavingAll and (Ilib = TMan.FTransOwner) then
    begin
     Tman.SetTransState(TrCommitted);
    end;

    if HasDeletions then
      RemoveDeletions;

    Ilib.Inval;

    if (Self = Ilib.Dataset) then
    begin
      if not HasDeletions then { Remove deltions sets pos }
      begin
       if Count > 0 then
       begin
        if (OldPos = -1)then
         Position := 0
        else if OldPos < Count then
         FPosition := OldPos
        else
         Position := 0;
       end
       else
        Position := -1;
      end;
      FChngTracker.Clear;
      if Ilib.HasHandler then
        Ilib.Datahandler.CreateSummary(nil);
      if HasDeletions and (Ilib.TVControl <> nil) then
        Ilib.TVControl.BuildTree(Self);
      if Ilib.MultiSelected then
        Ilib.DeselectAll;
    end;
    finally
      if FUpdater = nil then
      begin
        TMan.RemoveFromUpdateStack;
      end;
      FUpdater := nil;
    end;
  end;

  function TILDataset.Delete(const Index: integer) : integer;

  var
    i: integer;
    DS: TILDataset;
    PRec : TILDataRecord;
    R: TILDataRecord;
    Indx: integer;
    N: TTreeNode;
    Nw : Boolean;
    MustRedraw : Boolean;
    MH : TILMessageHandler;
  begin
    Result := 1;
    if (Count = 0) then
    begin
      Result := 0;
      Exit;
    end;
    if Position = -1 then
      FPosition := Index;
    if Self <> Ilib.Dataset then
    begin
      if FOwnsRecords then
       Items[Index].Free;
      FDataset.Delete(Index);
      Exit;
    end;

    if Ilib.Dataset.IsReadOnly then
      HandleILException(E_ReadOnly);
    if (Count = 1) and Items[Index].HasTempFields then
      Ilib.Fields.ClearCalcFields;
    Ilib.ClearEdits;
    if (Count = 1) and Items[Index].FEditingImageText then
    begin
      Items[Index].CancelEditImageText;
    end;
    if ILib.TVControl <> nil then
    begin
       N := Ilib.TVControl.NodeOfRecord(Items[Index]);
       if Ilib.TVControl.IsEditing and (N <> nil) then
          N.EndEdit(True);
    end;

    FSelState := -1;

    PRec := Items[Index];
    Nw := PRec.IsNew;
    if not Nw then
    begin
      if TMan.SaveOnDelete and not Ilib.DataHandler.MultiSelected then
      begin
       if Ilib.DBWork.DeleteFromTable(PRec.RecordIndex) then
       begin
         Result := 1;
         I := -1;
         MH := TMan.GetMessageHandler(Ilib, I);
         while MH <> nil do
         begin
          MH.AddCount(dboDelete, 1);
          MH := TMan.GetMessageHandler(ILib, I);
         end;
       end;
       Exit;
      end;
    end;

    if ILib.TVTable <> nil then
      ILib.TVTable.FillSiblings(PRec.FTVParentRecord);

    MustRedraw := false;

    if (ILIB.TVTable <> nil) then
    with Ilib.TVTable do
    begin
       DS := GetPathNodes(PRec);
        try
          for i := 0 to DS.Count - 1 do
          begin
            Indx := Self.IndexOf(DS[i].RecordIndex);
            if Indx <> -1 then
            begin
              R := FDataset[Indx];
              if R.IsNew then
              begin
                Items[Indx].Free;
                Self.FDataset.Delete(Indx);
                inc(Result);
                MustRedraw := True;
              end
              else
                R.FIsDeleted := True;
            end;
          end;
        finally
          DS.Free;
        end;
    end;

    if ILib.TVTable <> nil then
    begin
      ILib.TVTable.DeleteSiblingList(PRec.FTVParentRecord, Items[Index]);
      Ilib.TVTable.RenumberSiblings(PRec.FTVParentRecord, 0);
    end;

    if Nw then
    begin
     Ilib.Dataset.FChngTracker.Delete(Items[Index]);
     Items[Index].Free;
     FDataset.Delete(Index);
    end
    else
    begin
     Items[Index].FIsDeleted := True;
    end;
    Inc(Result);

    if FUpdating then
    begin
      if Position = FDataset.Count then
        FPosition := Position - 1;
      if (Position <> FDataset.Count) then
        FTriggerChange := True;
      Exit;
    end;
    for i := 0 to Ilib.AllChildCount - 1 do
    begin
      Ilib.AllChildren[i].Dataset.Clear;
    end;

    if Nw then
    begin
      if Position = FDataset.Count then
      begin
        Position := Position - 1;
      end
      else
      begin
        RunFocus(Position, False);
      end;
    end;


    if Ilib.HasViewer then
    if MustRedraw or (Nw and (Self = Ilib.Dataset) and (Ilib.FViewer <> nil)) then
    begin
      Ilib.FViewer.FReport.Redraw;
      Ilib.FViewer.UpdateScrollRange;
      Ilib.FViewer.UpdateScrollPos;
    end;
    if Count = 0 then
      Ilib.Inval;

    if Ilib.TVTable <> nil then
    begin
     TILTreeViewTable(Ilib).TreeViewControl.BuildTree(Self);
    end;


    if (Ilib.TVTable <> nil) and Nw then
    begin
      N := Ilib.TVControl.NodeOfRecord(Ilib.FocusedRecord);
      if N <> nil then
       TILTreeViewTable(Ilib).TreeViewControl.Selected := N;
    end;

    if not NW then
     Ilib.Dataset.FChngTracker.Add(Ilib.Dataset.Items[Index]);

    begin
     I := -1;
     MH := TMan.GetMessageHandler(ILib, I);
     while MH <> nil do
     begin
      if Nw then
        MH.AddCount(dboInsert, -1)
      else
        MH.AddCount(dboDelete, 1);
      MH := TMan.GetMessageHandler(ILib, I);
     end;
    end;
    if Ilib.HasViewer then
      Ilib.FViewer.Invalidate;
  end;

  function TILDataset.IndexOf(ADataRecord: TILDataRecord): integer;
  begin
    Result := FDataset.IndexOf(ADatarecord);
  end;

  function TILDataset.IndexOf(const ARecordIndex: integer): integer;

  var
    i: integer;
    DR: TILDataRecord;
  begin
    Result := -1;
    for i := 0 to Count - 1 do
    begin
      DR := TILDataRecord(Items[i]);
      if ARecordIndex = DR.RecordIndex then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  procedure TILDataset.Exchange(const Index1, Index2: integer);
  begin
    FDataset.Exchange(Index1, Index2);
  end;

  procedure TILDataset.DoSort(Compare: TListSortCompare);
  begin
    SortLib := Ilib;
    SortDataset := self;
    if Ilib.TVControl <> nil then
     Ilib.TVControl.Building := True;
    try
     FDataset.Sort(Compare);
    finally
      if Ilib.TVControl <> nil then
       Ilib.TVControl.Building := False;
    end;
  end;

  procedure TILDataset.Sort(FieldNames: TStringList; const Ordering: TSortOrdering);

  var
    Rec: TILDataRecord;
  begin
    if Count = 0 then
      Exit;
    if (FieldNames.Count = 0) then
      Exit;
    if ILib.HasViewer and (Ilib.Viewer.Report.SelectedCell <> nil) then
    begin
      if Ilib.Viewer.Report.SelectedCell.Editing then
        Ilib.FViewer.FCelleditor.Cancel;
    end;
    SortFieldnames := FieldNames;
    SortAscending := (Ordering = soAscending);
    Rec := nil;
    if Position <> -1 then
      Rec := Items[Position];
    DoSort(@SortProc);
    if not (Self = Ilib.Dataset) then
      Exit;
    if Rec <> nil then
      FPosition := IndexOf(Rec);

    if Ilib.FViewer <> nil then
    begin
     Ilib.Viewer.CreateReport;
     Ilib.FViewer.Invalidate;
    end;
    if Ilib.TVControl <> nil then
      Ilib.TVControl.BuildTree(Self);
  end;

  procedure TILDataset.Sort;
  begin
      Sort(Ilib.Columns.SortFields, Ilib.Columns.SortOrdering);
  end;

  function TILDataset.ToXML(SelectionOnly : Boolean) : string;
  var
    i, j, k: integer;
    R: TILDataRecord;
    S: String;
    Cnt: integer;
    F : TILField;
  begin
    begin
      S := '<ILib>';
      if not SelectionOnly then
        Cnt := Count
      else
        Cnt := SelCount;
      for i := 0 to Cnt - 1 do
      begin
        if not SelectionOnly then
          R := Items[i]
        else
          R := Selection[i];
        S := S + '<Rc>';
        S := S + '<RecordIndex>' + IntToStr(R.RecordIndex) + '</RecordIndex>';
        S := S + '<FileName>' + R.FileName + '</FileName>';
        for j := 0 to R.ColCount-1 do
        begin
          F := R[R.Columns[j].FieldName];
          if F.Count = 0 then
            S := S + '<' + R.Columns[j].FieldName + '>' + '</' + R.Columns[j].FieldName + '>'
          else if F.Count = 1 then
            S := S + '<' + R.Columns[j].FieldName + '>' +
              F.AsString + '</' + R.Columns[j].FieldName + '>'
          else
          begin
            S := S + '<' + R.Columns[j].FieldName + '>';
            for k := 0 to F.Count-1 do
            begin
                S := S + '<Vl>' + F.Items[k] + '</Vl>';
            end;
            S := S + '</' + R.Columns[j].FieldName + '>' ;
          end;
        end;
        S := S + '</Rc>';
      end;
      S := S + '</ILib>';
      Result := S;
    end;
  end;

  procedure TILDataset.SaveAsXML(const FileName: string; const SelectionOnly: Boolean);
  var
    i: integer;
    R: TILDataRecord;
    S: String;
    FS: TFileStream;
    MS: TStringStream;
    Cnt: integer;
  begin
    if FileName = '' then
    begin
      S := '<ILib>';
      if not SelectionOnly then
        Cnt := Count
      else
        Cnt := SelCount;
      for i := 0 to Cnt - 1 do
      begin
        if not SelectionOnly then
          R := Items[i]
        else
          R := Selection[i];
        S := S + Ilib.RecordToXML(R, True);
      end;
      S := S + '</ILib>';
      ClipBoard.AsText := S;
      Exit;
    end;

    if FileExists(FileName) then
    begin
      FS := TFileStream.Create(FileName, fmOpenWrite);
      FS.Size := 0;
    end
    else
      FS := TFileStream.Create(FileName, fmCreate);
    MS := TStringStream.Create('', TEncoding.Utf8);
    try
      MS.WriteString('<Ilb>');
      MS.Position := 0;
      FS.CopyFrom(MS, MS.Size);
      if not SelectionOnly then
        Cnt := Count
      else
        Cnt := SelCount;
      for i := 0 to Cnt - 1 do
      begin
        if not SelectionOnly then
          R := Items[i]
        else
          R := Selection[i];
        S := Ilib.RecordToXML(R, True);
        MS.Size := 0;
        MS.WriteString(S);
        MS.Position := 0;
        FS.CopyFrom(MS, MS.Size);
      end;
      MS.Size := 0;
      MS.WriteString('</Ilb>');
      MS.Position := 0;
      FS.CopyFrom(MS, MS.Size);
    finally
      FS.Free;
      MS.Free;
    end;
  end;

  function TILDataset.ReadXMLFile(const FileName: string): integer;
  begin
    Result := Ilib.DoReadXMLFile(FileName, 1);
  end;

  function TILDataset.ToJSON(SelectionOnly : Boolean) : string;
  var
    i: integer;
    R: TILDataRecord;
    Cnt: integer;
  begin
    Result := '';
    if SelectionOnly then
      Cnt := SelCount
    else
      Cnt := Count;
    if Cnt = 0 then
      Exit;
    Result := '[' + #13#10;
    for i := 0 to Cnt - 1 do
    begin
      if SelectionOnly then
        R := Selection[i]
      else
        R := Items[i];
      Result := Result + Ilib.RecordToJSON(R);
      if i < Cnt-1 then
        Result := Result +',' + #13#10
      else
        Result := Result + #13#10;
    end;
    Result := Result + ']';
  end;

  procedure TILDataset.SaveAsJSON(const AFileName : string; const SelectionOnly: Boolean);
    var
    S: string;
    Flds : TStringList;
  begin
    S := ToJSon(SelectionOnly);
    Flds := TStringList.Create;
    try
    if AFileName = '' then
      ClipBoard.AsText := S
    else
    begin
      Flds.Text := S;
      Flds.SaveToFile(AFileName);
    end;
    finally
      Flds.Free;
    end;
  end;

  procedure TILDataset.SaveAsText(const AFileName : string; const ColumnSeparator, TextQualifier:
   Char; const IncludeTitles: Boolean;  const SelectionOnly: Boolean);
  { Saves the dataset as a plain text tile with columns separators and titles}

  var
    i, j, k: integer;
    R: TILDataRecord;
    S, s2: string;
    Sep: string;
    Cnt: integer;
    Flds : TStringList;
    sl : TStringList;
    Fld : TILFieldDef;
    F : TILField;

  begin
    if (TextQualifier <> '"') and (TextQualifier <> '''') then
      HandleILException(0, 'Invaid text qualifier. Must be " or ' + '''');

    S := '';
    if SelectionOnly then
      Cnt := SelCount
    else
      Cnt := Count;
    if Cnt = 0 then
      Exit;
    Flds := TStringList.Create;
    try
    R := Items[0];
    for I := 0 to R.ColCount-1 do
    begin
      Flds.Add(R.Columns[i].FFieldName);
    end;

    for i := 0 to Cnt - 1 do
    begin
      if SelectionOnly then
        R := Selection[i]
      else
        R := Items[i];

      for j := 0 to Flds.Count - 1 do
      begin
        Fld := Ilib.Fields.Items[Ilib.Fields.IndexOf(Flds[j])];
        if IncludeTitles and (i = 0) then
        begin
          if j < R.ColCount - 1 then
            S := S + Fld.Caption + ColumnSeparator
          else
            S := S + Fld.Caption;
        end;
      end;

      if IncludeTitles and (i = 0) then
      begin
          S := S + #13#10;
      end;

      for k := 0 to R.ColCount - 1 do
      begin
        if k < R.ColCount - 1 then
          Sep := ColumnSeparator
        else
          Sep := '';
        F := R[R.Columns[k].FieldName];
        if R.Columns[k].FieldDef.DataType = dtLongText then
        begin
           s2 := '';
           if Assigned(Ilib.DataHandler) and Assigned(Ilib.DataHandler.FOnUpdateLongText) then
           begin
              Ilib.DataHandler.FOnUpdateLongText(Ilib, F, s2, [ltaSampleText]);
           end;
        end
        else if (F.Count > 1) and not (R.Columns[k].FieldDef.DataType = dtText) then
        begin
           sl := F.AsValues;
           try
             s2 := sl.Text;
             if Length(s2) > 2 then
             begin
              if s2[Length(s2)] = #$A then
               System.Delete(s2, Length(s2)-1, 2);
             end;
           finally
              sl.Free;
            end;
        end
        else if R.Columns[k].FieldDef.DataType = dtText then
        begin
            s2 := F.AsText;
        end
        else
        begin
           s2 := F.AsString;
        end;
        if Ilib.IsStringType(R.Columns[k].FieldDef.DataType) then
        begin
           s2 := TextQualifier + s2 + TextQualifier;
        end
        else if R.Columns[k].FieldDef.DataType = dtBoolean then
        begin
           if s2 = '0' then
             s2 := TMan.ReportFalse
           else
             s2 := TMan.ReportTrue
        end;
        S := S + s2 + Sep;
      end;
      S := S + #13#10;

      if i < SelCount - 1 then
      begin
         S := S +  #13#10;
      end;
    end;
    if AFileName = '' then
      ClipBoard.AsText := S
    else
    begin
      Flds.Text := S;
      Flds.SaveToFile(AFileName);
    end;
    finally
      Flds.Free;
    end;
  end;

  procedure TILDataset.RevertChanges(const SelectionOnly: Boolean);
  var
    i: integer;
    Indx: integer;
    Sel: Boolean;
    RunFoc : Boolean;
    MH : TILMessageHandler;
  begin
    if (Self <> Ilib.Dataset) then
      Exit;
    if not Changed then
      Exit;
    if Ilib.HasViewer and (Ilib.Viewer.Report.SelectedCell <> nil) then
    begin
      if Ilib.Viewer.Report.SelectedCell.Editing then
        Ilib.FViewer.CellEditor.Cancel;
    end;

    RunFoc := false;
    while ChangeCount > 0 do
    begin
      if SelectionOnly then
        if not FChngTracker.ChngRecords[0].Selected then
        begin
          Continue;
        end;
      begin
        if FChngTracker.ChngRecords[0].IsDeleted and Ilib.IsMaster and (Ilib.ChildCount > 0) then
          RunFoc := True;
        Sel := FChngTracker.ChngRecords[0].Selected;
        Indx := FChngTracker.ChngRecords[0].Index;
        if FChngTracker.ChngRecords[0].RecordIndex <> -1 then
        begin
         FChngTracker.ChngRecords[0].Refresh;
         Items[Indx].FSelected := Sel;
        end
        else
        begin
          if FPosition = Count-1 then
            dec(FPosition);
          FChngTracker.Delete(0);
          FDataset.Delete(Indx);
          RunFoc := True;
        end;
      end;
    end;

    FSelState := -1;

    if Ilib.TVTable <> nil then
    begin
     { Reconnect records }
      for I := 0 to Count-1 do
      begin
        if not Items[i][c_SysFieldNames[SYS_LinkID]].IsEmpty then
        begin
         Indx := IndexOf(Items[i][c_SysFieldNames[SYS_LinkID]].AsInteger);
         if Indx <> -1 then
         begin
           Items[i].FTVParentRecord := Items[Indx];
         end;
        end;
      end;
    end;

    if Ilib.HasViewer then
      Ilib.Viewer.FReport.Redraw;
    Ilib.Datahandler.CreateSummary(nil);
    if Position > -1 then
      Ilib.WriteControls(Items[Position], duData);

    if Ilib.TVControl <> nil then
      Ilib.TVControl.BuildTree(Self);
    if RunFoc then
      RunFocus(Position, False)
    else
     Ilib.Inval;
    I := -1;
    FChngTracker.Clear;
    MH := TMan.GetMessageHandler(ILib, I);
    while MH <> nil do
    begin
      MH.Reset;
      MH := TMan.GetMessageHandler(ILib, I);
    end;
  end;

  procedure TILDataset.SetPosition(const Value : integer);
  begin
    if FPosition = Value then
      Exit;
    if (Value > Count - 1) then
      Exit;
    if (Value = -1) and (Count > 0) then
      Exit;
    if Value < -1 then
      Exit;
    if (Self <> Ilib.Dataset) then
      Exit;
    RunFocus(Value, True);
    if ILib.HasViewer then
    begin
    if not Ilib.Viewer.IsImageFullyVisible(Position) then
      Ilib.Viewer.ScrollTo(Position);
    end;
  end;

  procedure TILDataset.RunFocus(Posit: integer; RecursiveClear : Boolean = True);
  begin
    Ilib.FFocHandler.Execute(Self, Posit);
  end;

  function TILDataSet.GetParentRecord : TILDataRecord;
  begin
    Result := nil;
    if (not Ilib.IsDetail and not (Ilib is TILTreeViewTable)) or (Ilib.Dataset.Count = 0) then
      Exit;
    if Ilib.Dataset[0]['Sys_LinkID'].IsEmpty then
      Exit;
    if Ilib is TILTreeViewTable then
      Result := Ilib.RecordsX[Ilib.Dataset[0]['Sys_LinkID'].AsInteger]
    else
      Result := Ilib.MasterSource.RecordsX[Ilib.Dataset[0]['Sys_LinkID'].AsInteger];
  end;

  function TILDataset.GetCount;
  begin
    Result := FDataset.Count;
  end;

  function TILDataset.GetSelCount;
  var
    i: integer;
  begin
    Result := 0;
    if FSelState = 0 then
      Result := FSelBuffer.Count
    else
    begin
      FSelBuffer.Clear;
      for i := 0 to Count - 1 do
      begin
        if Items[i].Selected then
        begin
          Inc(Result);
          FSelBuffer.Add(Items[i]);
        end;
      end;
      FSelState := 0;
    end;
  end;

  function TILDataset.GetChangeCount : integer;
  begin
    Result := FChngTracker.Count;
  end;

  function TILDataset.GetChanged: Boolean;
  begin
    Result := FChngTracker.Count > 0;
  end;

  function TILDataset.GetIsReadOnly: Boolean;
  begin
    Result := Ilib.IsReadOnly;
    if not Result then
      Result := Ilib.Fields.HasTempFields;
  end;

  function TILDataset.GetColumnSource : TILColumns;
  begin
    if Self = Ilib.Dataset then
      Result := ILib.Columns
    else
      Result := FColumnSource;
  end;

  function TILDataset.GetFocusedRecord: TILDataRecord;
  begin
    Result := nil;
    if Position = -1 then
      Exit;
    Result := Items[Position];
  end;

  procedure TILDataset.Add(ADataRecord: TILDataRecord);
  begin
    ADataRecord.FOwnerDataset := Self;
    FDataset.Add(ADataRecord);
    if not FUpdating then
      if (Self = Ilib.Dataset) and Ilib.HasViewer then
        Ilib.Viewer.FReport.Redraw;
  end;

  procedure TILDataset.ExchangeMedia(const Index: integer; WithDataset: TILDataset;
    const WithIndex: integer);

  var
    DR: TILDataRecord;
  begin
    DR := TILDataRecord.Create(Ilib, false);
    try
      DR.Assign(Items[Index]);
      Items[Index].AssignMedia(WithDataset.Items[WithIndex]);
      WithDataset.Items[WithIndex].AssignMedia(DR);
    finally
      DR.Free;
    end;
    Ilib.Inval;
    WithDataset.Ilib.Inval;
  end;

  { TILFieldDef ---------------------------------------------------------------- }

  procedure TILFieldDef.Assign(Source: TPersistent);

  var
    wSrc: TILFieldDef;
  begin
    if Source is TILFieldDef then
    begin
      wSrc := TILFieldDef(Source);
      FFieldName := wSrc.FieldName;
      FFieldID := wSrc.FieldId;
      FReportField := wSrc.FReportField;
      FFieldType := wSrc.FFieldType;
      FDataType := wSrc.FDataType;
      FReportCaption := wSrc.FReportCaption;
      FTemporary := wSrc.FTemporary;
      FMandatory := wSrc.Mandatory;
      FIndexed := wSrc.Indexed;
      FIndexPriority := wSrc.IndexPriority;
      FMaxValues := wSrc.MaxValues;
      FMaxLength := wSrc.MaxLength;
      FIdentifier := wSrc.Identifier;
      FLookupField := wSrc.LookupField;
      FLookupTable := wSrc.LookupTable;
      FLookupType := wSrc.LookupType;
      FLookupSearchField := wSrc.LookupSearchField;
      FColumnWidth := wSrc.ColumnWidth;
      FUnique := wSrc.Unique;
      FDecimals := wSrc.Decimals;
      FValueLogop := wSrc.ValueLogOp;
      FFieldLogOp := wSrc.FieldLogOp;
      FUserCalc := wSrc.FUserCalc;
      FPickValues.Assign(wSrc.PickValues);
      Ilib := wSrc.Ilib;
    end
    else
      inherited;
  end;

  constructor TILFieldDef.Create(Collection: TCollection);

  var
    Nm: string;
    Cnt: integer;
  begin
    inherited;
    { Default is dtString }
    Ilib := TILTable(Collection.Owner);
    FFields := TILFieldDefs(Collection);
    FFieldType := ftUserData;
    FValueLogOp := vlAnd;
    FIndexPriority := ipIndefinite;
    FColumnWidth := 100;
    FMaxLength := 50;
    FMaxValues := 1;
    FLTPageSize := ltp4KB;
    FReportField := True;
    FPickValues := TStringList.Create;
    FDecimals := -1;
    Cnt := Collection.Count;
    Nm := 'Field' + IntToStr(Collection.Count);
    while FFields.IndexOf(Nm) <> -1 do
    begin
      Inc(Cnt);
      Nm := 'Field' + IntToStr(Cnt);
    end;
    FFieldName := Nm;
    FFieldID := Cnt;
  end;

  destructor TILFieldDef.Destroy;
  begin
    FPickValues.Free;
    inherited;
  end;

  function TILFieldDef.IsMultiVal: Boolean;
  begin
    Result := ((MaxValues = 0) or (MaxValues > 1)) and not(DataType = dtText);
  end;

  function TILFieldDef.IsLookup;
  begin
    Result := not (LookupType in [looNone, looPickValues])
  end;

  procedure TILFieldDef.PropagatePickValues(const Clear: Boolean);
  var
    i : integer;
    sl  : TStringList;
  begin
    if LookupType <> looPickValues then
      Exit;
    sl := TStringList.Create;
    if Clear then
    begin
     sl.Assign(FPickValues);
     FPickValues.Clear;
    end;
    try
    for I := 0 to FFields.Count-1 do
    begin
      FFields.Items[i].AssignPickValues(FFields.Items[i].SearchControl);
      FFields.Items[i].AssignPickValues(FFields.Items[i].DataControl);
    end;
    if Clear then
      FPickValues.Assign(sl);
    finally
      sl.Free;
    end;
  end;


  procedure TILFieldDef.SetPickValues(const Value : TStringList);
  var
    i : integer;
    Found : Boolean;
  begin
    if (LookupType = looPickValues) and (InitialValue <> '')
    and not (csLoading in Ilib.ComponentState) then
    begin
     Found := false;
     for i := 0 to Value.Count-1 do
     begin
       if Value.IndexOf(InitialValue) <> -1 then
       begin
          Found := True;
          Break;
       end;
     end;
     if not Found then
     begin
       FInitialValue := '';
       ShowMessage('Note! Initial value ' +  InitialValue + ' removed from picklist lookup');
     end;
    end;

    FPickValues.Assign(Value);
    PropagatePickValues(False);
  end;

  function IsValidCtrl(ACtrl: TWinControl): Boolean;
  begin
       Result := (ACtrl is TEdit) or (ACtrl is TLabeledEdit) or (ACtrl is TMemo) or (ACtrl is TCombobox) or
      (ACtrl is TListBox) or (ACtrl is TCheckbox) or (ACtrl is TRadioGroup) or
      (ACtrl is TDateTimePicker) or (ACtrl is TCheckListBox) or
      (ACtrl is TILSearchGrid) or (ACtrl is TILUpdateGrid);
  end;

  procedure TILFieldDef.AssignPickValues(ACtrl : TWinControl);
  begin
       if ACtrl is TComboBox then
        with ACtrl as TComboBox do
        begin
           Style := csDropDownList;
           Items.Assign(PickValues);
        end
       else if ACtrl is TCheckListBox then
        with ACtrl as TCheckListBox do
        begin
           AllowGrayed := True;
           Items.Assign(PickValues);
           CheckAll(cbGrayed);
           if (Count < 5) and (Count > 0) then
             Height := (ItemHeight+2) * Count;
        end
        else if ACtrl is TRadioGroup then
        with ACtrl as TRadioGroup do
        begin
           Items.Assign(PickValues);
        end
        else if ACtrl is TListBox then
        with ACtrl as TListBox do
        begin
           Items.Assign(PickValues);
           if (Count < 5) and (Count > 0) then
             Height := (ItemHeight+2) * Count;
        end;
  end;

  procedure TILFieldDef.SetDataControl(const Value: TWinControl);
  var
    P : TILControlPanel;
  procedure SetHandler(Ctrl : TWinControl; Lbl : TLabel);
  var
    Indx : integer;
  begin
    if Assigned(Ilib.DataHandler) then
     begin
        Indx := Ilib.DataHandler.DataFields.IndexOf(FieldName);
        if Indx <> -1 then
        begin
          Ilib.DataHandler.DataFields.Items[Indx].FDataControl := Ctrl;
          Ilib.DataHandler.DataFields.Items[Indx].FDataLabel := Lbl;
        end;
      end;
  end;

  begin
    if Value = FDataControl then
      Exit;
    if csDestroying in Ilib.ComponentState then
      Exit;
    if csLoading in ILIB.ComponentState then
    begin
      FDataControl := Value;
      if FDataControl is TILUpdateGrid then
        TILUpdateGrid(FDataControl).Field := Self
      else if FDataControl is TILResourceList then
      begin
         TILResourceList(FDataControl).SingleLine := (MaxValues = 1);
      end;
      if FDataControl <> nil then
      begin
        FDataControl.FreeNotification(Ilib);
      end;
      if FDataControl is TILUpdateGrid then
      begin
       Ilib.FreeNotification(FDataControl);
      end;
      Exit;
    end;
    if DataType = dtLongText then
      Exit;

    P := nil;
    if Value = nil then
    begin
      if FDataControl is TILUpdateGrid then
        TILUpdateGrid(FDataControl).Field := nil;
      if (FDataControl.Parent is TILControlPanel) then
      begin
        P := TILControlPanel(FDataControl.Parent);
      end;

      if P <> nil then
      begin
        FreeAndNil(FDataControl);
        if Assigned(FDataLabel) then
          FreeAndNil(FDataLabel);
        P.CreateItems;
        P.ComputeLabelSpace;
        P.ShowCtrls(P.FItems);
      end
      else
        FDataControl := nil;
      SetHandler(nil, nil);

      Exit;
    end;
    if Value is TILSearchGrid then
      HandleILException(E_SearchGridDataCtrl, '')
    else if not IsValidCtrl(Value) then
      HandleILException(E_CtrlMismatch, '')
    else if (MaxValues = 1) and (Value is TChecklistbox) then
      HandleILException(E_CtrlDataTypeMismatch)
    else if not FFields.CheckCtrlDataType(Value, FDataType, duData) then
      HandleILException(E_CtrlDataTypeMismatch)
    else
    begin
       if (FSearchControl <> nil) and (FSearchControl.Parent is TILControlPanel) then
         if Value = SearchControl then
          HandleILException(E_PanelCtrls);
      if FDataControl is TILUpdateGrid then
        TILUpdateGrid(FDataControl).Field := nil;
      FDataControl := Value;
    end;
    if FDataControl <> nil then
    begin
      if LookupType = looPickValues then
       if PickValues.Count > 0 then
        AssignPickValues(FDataControl);
      if LookupType = looTableKey then
      begin
      if FDataControl is TEdit then
      with FDataControl as TEdit do
      begin
        ReadOnly := True;
      end;
      end;
    end;
    if FDataControl <> nil then
    begin
        FDataControl.FreeNotification(Ilib);
    end;
    if FDataControl is TILUpdateGrid then
    begin
       Ilib.FreeNotification(FDataControl);
    end;
    if FDataControl <> nil then
    begin
      if FDataControl is TILResourceList then
        TILResourceList(FDataControl).SingleLine := (MaxValues = 1);
      if (FDataControl.Parent is TILControlPanel)
      and not (csLoading in TILControlPanel(FDataControl.Parent).ComponentState)
      and not TILControlPanel(FDataControl.Parent).FSourceBuild then
      with FDataControl.Parent as TILControlPanel do
      begin
        if DataLabel = nil then
        begin
          DataLabel := TLabel.Create(Owner);
          DataLabel.Name := 'dlbl' + IntToStr(Abs(TMan.TableIndex(Ilib))) + '_' + Self.Caption;
          DataLabel.Caption := Self.Caption;
          DataLabel.Parent := FDataControl.Parent;
        end;
        CreateItems;
        ComputeLabelSpace;
        ShowCtrls(FItems);
      end;
    end;
    SetHandler(DataControl, DataLabel);
  end;

  function TILFieldDef.GetInitialValue : string;
  begin
    if SameText(FInitialValue, 'Today') and (DataType = dtDate)
    and not (csDesigning in Ilib.ComponentState) then
      Result := DateTimeToStr(Today, Translator.FFormatSettings)
    else
      Result := FInitialValue;
  end;


  procedure TILFieldDef.SetSearchControl(const Value: TWinControl);
  var
    P : TILControlPanel;

  procedure SetHandler(Ctrl : TWinControl; Lbl : TLabel);
  var
    Indx : integer;
  begin
    if Assigned(Ilib.DataHandler) then
     begin
        Indx := Ilib.DataHandler.DataFields.IndexOf(FieldName);
        if Indx <> -1 then
        begin
          Ilib.DataHandler.DataFields.Items[Indx].FSearchControl := Ctrl;
          Ilib.DataHandler.DataFields.Items[Indx].FSearchLabel := Lbl;
        end;
      end;
  end;

  begin
    if Value = SearchControl then
      Exit;
    if csDestroying in Ilib.ComponentState then
      Exit;
    if csLoading in Ilib.ComponentState then
    begin
      FSearchControl := Value;
      if FSearchControl <> nil then
        FSearchControl.FreeNotification(Ilib);
      if Value is TILSearchGrid then
      begin
       TILSearchGrid(Value).Field := Self;
       Ilib.FreeNotification(FSearchControl);
      end;
      Exit;
    end;

    if Value = nil then
    begin
      P := nil;
      if FSearchControl is TILSearchGrid then
        TILSearchGrid(FSearchControl).Field := nil;
      if (FSearchControl.Parent is TILControlPanel) then
      begin
        P := TILControlPanel(FSearchControl.Parent);
      end;

      if P <> nil then
      begin
        FreeAndNil(FSearchControl);
        if Assigned(FSearchLabel) then
          FreeAndNil(FSearchLabel);
        P.CreateItems;
        P.ComputeLabelSpace;
        P.ShowCtrls(P.FItems);
      end
      else
        FSearchControl := nil;
      SetHandler(nil, nil);
      Exit;
    end;
    if not IsValidCtrl(Value) then
      HandleILException(E_CtrlMismatch)
    else if not FFields.CheckCtrlDataType(Value, FDataType, duSearch) then
      HandleILException(E_CtrlDataTypeMismatch)
    else if Value is TILUpdateGrid then
      HandleILException(E_SearchGridDataCtrl)
    else if Value is TILResourceList then
      HandleIlException(E_ResSearchControl)
    else if (FDataControl <> nil) and (FDataControl.Parent is TILControlPanel) then
    begin
     if Value = DataControl then
      HandleILException(E_PanelCtrls)
    end
    else if FSearchControl is TILSearchGrid then
      TILSearchGrid(FSearchControl).Field := nil;
    FSearchControl := Value;
    if FSearchControl <> nil then
        FSearchControl.FreeNotification(Ilib);

    if Value is TILSearchGrid then
    begin
       TILSearchGrid(Value).Field := Self;
       Ilib.FreeNotification(FSearchControl);
    end;

    if LookupType = looPickValues then
      if Assigned(PickValues) and (PickValues.Count > 0) then
        AssignPickValues(FSearchControl);
    if FSearchControl <> nil then
    begin
      if (FSearchControl.Parent is TILControlPanel)
      and not (csLoading in TILControlPanel(FSearchControl.Parent).ComponentState)
      and not TILControlPanel(FSearchControl.Parent).FSourceBuild then
      with FSearchControl.Parent as TILControlPanel do
      begin
        if SearchLabel = nil then
        begin
          SearchLabel := TLabel.Create(Owner);
          SearchLabel.Name :=
           'slbl' + IntToStr(Abs(TMan.TableIndex(Ilib))) + '_' + Self.Caption;
          SearchLabel.Caption := Self.Caption;
          SearchLabel.Parent := FSearchControl.Parent;
        end;
        CreateItems;
        ComputeLabelSpace;
        ShowCtrls(FItems);
      end;
    end;
    SetHandler(SearchControl, SearchLabel);
  end;

  procedure TILFieldDef.SetDataLabel(const Value : TLabel);
  begin
      if FDataLabel = Value then
        Exit;
      FDataLabel := Value;
      if FDataLabel <> nil then
        FDataLabel.FreeNotification(Ilib);
  end;

  procedure TILFieldDef.SetSearchLabel(const Value : TLabel);
  begin
      if FSearchLabel = Value then
        Exit;
      FSearchLabel := Value;
      if FSearchLabel <> nil then
        FSearchLabel.FreeNotification(Ilib);
  end;

  procedure TILFieldDef.SetDataType(Value : TDataType);
  begin
     if csLoading in TILTable(FFields.Owner).Componentstate then
    begin
      FDataType := Value;
      Exit;
    end;
     if Value = FDataType then
       Exit;
     FDataType := Value;
     if FDataType <> dtNumber then
       FDecimals := -1;
     if FDataType = dtText then
       FMaxValues := 0
     else if not (FDataType in [dtString, dtResource]) then
       FMaxValues := 1;
  end;

  procedure TILFieldDef.SetDecimals(Value : integer);
  begin
    if csLoading in TILTable(FFields.Owner).Componentstate then
    begin
      FDecimals := Value;
      Exit;
    end;
    if Value = FDecimals then
      Exit;
    if DataType <> dtNumber then
      Exit;
    if Value > 5 then
      Value := 5;
    FDecimals := Value;
  end;

  procedure TILFieldDef.SetMaxValues(const Value : integer);
  begin
    if csLoading in TILTable(FFields.Owner).Componentstate then
    begin
      FMaxValues := Value;
      Exit;
    end;
    if Value = FMaxValues then
      Exit;
    if not (DataType in[dtString, dtResource]) then
      Exit;
    FMaxValues := Value;
  end;

  procedure TILFieldDef.SetFieldName(const Value : string);
  var
    i : integer;
    D : TILdata;
    OldName : string;
  begin
    if csLoading in TILTable(FFields.Owner).Componentstate then
    begin
      FFieldName := Value;
      Exit;
    end;

    if Value = FFieldName then
      Exit;
    if not NameCheck(Value) then
      HandleILException(E_InvalidFieldName, Value);
    for I := 0 to FFields.Count-1 do
    begin
      if SameText(FFields.Items[i].FFieldName, Value)
      and (FFields.Items[i] <> Self) then
        HandleILException(E_DupFieldName, Value);
    end;
    OldName := FFieldName;
    FFieldName := Value;
    if csDesigning in TILTable(FFields.Owner).Componentstate then
    begin
       D := TILTable(FFields.Owner).DataHandler;
       if (D <> nil) and (D.DataFields.Count > 0) then
        begin
          I := D.DataFields.IndexOf(OldName);
          if i <> -1 then
          begin
           D.DataFields.Items[i].FFieldName := FFieldName;
          end;
        end;
    end;

  end;

  procedure TILFieldDef.SetFieldLogOp(const Value : TFieldLogOp);
  begin
    if Value = FFieldLogOp then
      Exit;
    { Synchronize }
    FFieldLogOp := Value;
  end;

  function TILFieldDef.GetNextAutoValue(Update : Boolean) : integer;
  begin
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
     Result := Tman.FCfgFile.ReadInteger(Ilib.TableName, FieldName + '_NextAutoValue', 1);
     if Update then
        Tman.FCfgFile.WriteInteger(Ilib.TableName, FieldName + '_NextAutoValue', Result+1);

  end;

  procedure TILFieldDef.SetNextAutoValue(AValue : integer);
  begin
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
      Tman.FCfgFile.WriteInteger(Ilib.TableName, FieldName + '_NextAutoValue', AValue);
  end;


  function TILFieldDef.GetDisplayName: string;
  begin
    Result := FFieldName;
  end;

  function TILFieldDef.GetCaption: string;
  var
    S : string;
  begin
    S := Translator.GetTranslation(True, FieldName);
    if S <> '' then
      Result := S
    else if FReportCaption <> '' then
     Result := FReportCaption
    else
     Result := FieldName;
  end;

  function TILFieldDef.GetValueLogOp: TValueLogOp;
  begin
    Result := vlAnd;
    if (TMan = nil) then
      Exit;
    if (TMan.SearchModel = smAndAligned) then
    begin
      if not IsMultival and (SearchControl is TCheckListBox) then
        Result := vlOr
      else
        Result := FValueLogOp;
    end
    else if (TMan.SearchModel = smAndOrAligned) then
    begin
      if not IsMultival and (SearchControl is TCheckListBox) then
        Result := vlOr
      else
        Result := FValueLogop;
    end
    else
      Result := FValueLogOp;
  end;

  function TILFieldDef.GetFieldLogOp: TFieldLogOp;
  begin
    if (TMan <> nil) and
      (TMan.SearchModel = smAndAligned) then
      Result := flAnd
    else
      Result := FFieldLogOp;
  end;

  function TILFieldDef.GetIsReadOnly : Boolean;
  begin
    Result := True;
    if FieldType in [ftCalculated, ftAutoIncrement, ftStoredEvaluation] then
      Exit;
    if LookUpType = looParentField then
      Exit;
    if FTemporary then
      Exit;
    if Ilib.IsReadOnly or Ilib.Dataset.IsReadOnly then
      Exit;
    if FieldType = ftSystemData then
    begin
      if SameText(FieldName, PidNameOf(PID_PHOTO_CAMDATE)) then
        Result := False;
    end
    else
      Result := False;
  end;

  function TILFieldDef.CalcUserCalc(DataRec : TILDataRecord) : string;
  var
    SelField : TSelectField;
    SF : TSelectFields;
    sl : TStringList;
  begin
     sl := TStringList.Create;
     SF := TSelectFields.Create(Ilib.GetSearcher(ILib), sl, FFields);
     SelField := TSelectField.Create(SF, FFormula, FFields);
     try
      Result := SelField.CalcValue(DataRec);
     finally
       sl.Free;
       SelField.Free;
       SF.Free;
     end;
  end;

  { TILPropertyItem ----------------------------------------------------}

  procedure TILPropertyItem.SetPIDName(const Value : string);

  var
    ID: integer;
  begin
    ID := PIDOf(Value);
    if ID = -1 then
    begin
      ID := SysIdOf(Value);
      if ID = -1 then
        HandleILException(E_UnknownPID);
    end;
    FPIDName := Value;
    FPID := ID;
    FDataType := GetSysDataType(FPIDName);
  end;

  constructor TILPropertyItem.Create(Collection: TCollection);
  begin
    inherited Create(Collection);
    FPropertyItems := TILPropertyList(Collection);
  end;

  destructor TILPropertyItem.Destroy;
  begin
    inherited Destroy;
  end;

  function TILPropertyItem.GetDisplayName: string;
  begin
    if FPIDName <> '' then
    begin
      if FDisplayName <> '' then
        Result := FDisplayName
      else
        Result := FPIDName
    end
    else
      Result := inherited;
  end;

  procedure TILPropertyItem.Assign(Source: TPersistent);

  var
    wSrc: TILPropertyItem;
  begin
    if Source is TILPropertyItem then
    begin
      wSrc := TILPropertyItem(Source);
      FPID := wSrc.PID;
      FPIDName := wSrc.PIDName;
      FDisplayName := wSrc.DisplayName;
      FDataType := wSrc.DataType;
      FPropertyItems := wSrc.FPropertyItems;
    end
    else
      inherited;
  end;

  { TILPropertyList ------------------------------------------------------------- }

  function TILPropertyList.GetItem(AIndex: integer): TILPropertyItem;
  begin
    Result := TILPropertyItem(inherited Items[AIndex]);
  end;

  procedure TILPropertyList.SetItem(AIndex: integer;
    const Value: TILPropertyItem);
  begin
    inherited SetItem(AIndex, Value);
  end;

  function TILPropertyList.Add: TILPropertyItem;
  begin
    Result := TILPropertyItem(inherited Add);
  end;

  function TILPropertyList.IndexOf(const ItemName: string): integer;

  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if (AnsiCompareText(Items[loop].PIDName, ItemName) = 0) then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILPropertyList.IndexOfPID(const PID: integer): integer;

  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if Items[loop].PID = PID then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  procedure TILPropertyList.Assign(Source: TPersistent);

  var
    wSrc: TILPropertyList;
    loop: integer;
  begin
    if (Source is TILPropertyList) then
    begin
      wSrc := TILPropertyList(Source);
      Clear;
      for loop := 0 to wSrc.Count - 1 do
        Add.Assign(wSrc.Items[loop]);
    end
    else
      inherited;
  end;

  { TILFieldDefs }

  procedure TILFieldDefs.Notify(Item: TCollectionItem;
    Action: TCollectionNotification);

  var
    IL: TILTable;
    Pan : TilControlPanel;
    I, j : integer;
    PanFound : Boolean;
  begin

    if Importing then
    begin
      inherited Notify(Item, Action);
      Exit;
    end;

    IL := TILTable(Owner);
    if (IL = nil) or (IL.Root = nil) then
    begin
      inherited Notify(Item, Action);
      Exit;
    end;
    if (csDesigning in IL.ComponentState) and
      not(csLoading in IL.ComponentState) and
      not(csDestroying in IL.ComponentState) and Assigned(Tman)
      and not (TMan.NodeOf(IL) = nil) then
      begin
        if (Action = cnExtracting) then
        begin
           PanFound := false;
           for I := 0 to RegPanels.Count-1 do
           begin
             Pan := TILControlPanel(RegPanels[i]);
             with Owner as TILTable do
              TMan.FPanelBuild := True;
             for j := 0 to Pan.Count-1 do
               if (Pan.Items[j].FField = Item)
               then
               begin
                   if Pan.Items[j].ControlLabel <> nil then
                   begin
                     if Pan.Items[j].Usage = duData then
                       FreeAndNil(Pan.Items[j].Field.FDataLabel)
                     else
                       FreeAndNil(Pan.Items[j].Field.FSearchLabel);
                   end;
                   PostMessage(IL.FHWnd, wm_UpdatePanel, 0, Integer(Pan));
                   PanFound := True;
                   Break;
               end;
               if PanFound then
                 Break;
           end;
           if not PanFound then
             PostMessage(IL.FHWnd, wm_UpdatePanel, 0, 0);
        end;
      end;
      if (Owner is TILMemTable) and Assigned(Tman) then
      begin
       if (csDesigning in IL.ComponentState)
       and not(csDestroying in IL.ComponentState)
       and not(csLoading in IL.ComponentState) and
       (Action = cnExtracting) then
       with Owner as TILMemTable do
       begin
          if DataHandler <> nil then
          begin
           i := DataHandler.DataFields.IndexOf(TILFieldDef(Item).FieldName);
            if i <> -1 then
           DataHandler.DataFields.Delete(i);
          end;
       end;
      end;

    inherited Notify(Item, Action);
  end;

  procedure TILFieldDefs.Update(Item: TCollectionItem);

  var
    IL: TILTable;
  begin
    IL := TILTable(Owner);
    if (csDesigning in IL.ComponentState) and
      not(csLoading in IL.ComponentState) and
      not(csDestroying in IL.ComponentState) and Assigned(IL.Columns) then
    begin
      PostMessage(IL.FHwnd, WM_UpdateRptColumns, 0, 0);
    end;
    inherited Update(Item);
  end;

  function TILFieldDefs.Add: TILFieldDef;
  begin
    Result := TILFieldDef(inherited Add);
    Result.FFieldName := 'Field_' + IntToStr(NextID);
  end;

  function TILFieldDefs.AddCalcField(const FieldName: string): TILFieldDef;
  begin
    if IndexOf(FieldName) <> -1 then
     HandleILException(E_FieldNameExists, ' (' + FieldName + ')');
    Result := Add;
    Result.FFieldType := ftCalculated;
    Result.FMaxValues := 0;
    Result.FFieldName := FieldName;
    Result.FTemporary := True;
    Result.FFieldID := -1;
  end;

  function TILFieldDefs.AddAlienField(Table : TILTable; SourceField : TILFieldDef): TILFieldDef;
  begin
    Result := Add;
    Result.Assign(SourceField);
    Result.FFieldName := Table.TableName + '.' + Result.FFieldName;
    Result.FTemporary := True;
    Result.FAlien := True;
    Result.FFieldID := -1;
  end;

  procedure TILFieldDefs.ClearCalcFields(const SyncColumns: Boolean = True);
  var
    i: integer;
    Fld: TILFieldDef;
    IL: TILTable;
    Indx: integer;
  begin
    IL := TILTable(Owner);
    i := 0;
    while i < Count do
    begin
      Fld := Items[i];
      if Fld.FTemporary then
      begin
        Indx := -1;
        if SyncColumns then
          Indx := IL.Columns.IndexOf(Fld.Caption);
        Delete(i);
        i := 0;
        if Indx <> -1 then
        begin
          IL.Columns.Delete(Indx);
        end;
      end
      else
        Inc(i);
    end;
  end;

  procedure TILFieldDefs.Assign(Source: TPersistent);

  var
    wSrc: TILFieldDefs;
    loop: integer;
  begin
    if (Source is TILFieldDefs) then
    begin
      wSrc := TILFieldDefs(Source);
      Clear;
      for loop := 0 to wSrc.Count - 1 do
        Add.Assign(wSrc.Items[loop]);
    end
    else
      inherited;
  end;

  function TILFieldDefs.GetItem(AIndex: integer): TILFieldDef;
  begin
    Result := TILFieldDef(inherited Items[AIndex]);
  end;

  function TILFieldDefs.IndexOf(const ItemName: string): integer;

  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if (AnsiCompareText(Items[loop].FieldName, ItemName) = 0) then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILFieldDefs.IndexOfControl(AControl: TWinControl): integer;

  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if (AControl = Items[loop].FDataControl) or
        (AControl = Items[loop].FSearchControl) then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILFieldDefs.IndexOfLabel(ALabel : TLabel): integer;
  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if (ALabel = Items[loop].FDataLabel) or
        (ALabel = Items[loop].FSearchLabel) then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILFieldDefs.IndexOfID(const AnID : integer): integer;
  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if Items[loop].FieldID = AnID then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILFieldDefs.IndexOfCaption(const ACaption: string): integer;
  var
    loop: integer;
  begin
    Result := -1;
    loop := 0;
    while (Result = -1) and (loop < Count) do
    begin
      if (AnsiCompareText(Items[loop].Caption, ACaption) = 0) or
        (AnsiCompareText(Items[loop].FFieldName, ACaption) = 0) then
        Result := loop
      else
        Inc(loop);
    end;
  end;

  function TILFieldDefs.HasLookups: Boolean;
  var
    i: integer;
  begin
    Result := false;
    for i := 0 to Count - 1 do
    begin
      if Items[i].IsLookup then
      begin
        Result := True;
        Break;
      end;
    end;

  end;

  function TILFieldDefs.HasIndexes: Boolean;

  var
    i: integer;
  begin
    Result := false;
    for i := 0 to Count - 1 do
    begin
      if Items[i].Indexed then
      begin
        Result := True;
        Break;
      end;
    end;
  end;

  function TILFieldDefs.HasTempFields: Boolean;

  var
    i: integer;
  begin
    Result := false;
    for i := 0 to Count - 1 do
    begin
      if Items[i].FTemporary then
      begin
        Result := True;
        Break;
      end;
    end;
  end;

  procedure TILFieldDefs.SetItem(AIndex: integer; const Value: TILFieldDef);
  begin
    inherited SetItem(AIndex, Value);
  end;

  procedure TILFieldDefs.SetFieldLogOp(const Value : TFieldLogOp);
  var
    i: integer;
  begin
    if (Value = FFieldLogOp) then
      Exit;
    FFieldLogOp := Value;
    { Synchronize }
    for i := 0 to TMan.TableCount - 1 do
    begin
      TMan.Tables[i].Fields.FFieldLogOp := Value;
    end;
  end;

  function TILFieldDefs.GetFieldLogOp: TFieldLogOp;
  begin
    if (TMan <> nil) and
      (TMan.SearchModel = smAndAligned) then
      Result := flAnd
    else
      Result := FFieldLogOp;
  end;

  function TILFieldDefs.CheckCtrlDataType(ACtrl: TWinControl;
    const DataType: TDataType; const Usage: TDataUsage): Boolean;
  begin
    Result := True;
    if ACtrl = nil then
      Exit;
    if (ACtrl is TILResourceList)
    then
    begin
     if not (DataType in [dtResource]) or (Usage = duSearch) then
     begin
       Result := false;
     end;
     Exit;
    end;

    case DataType of
      dtString:
        Result := not(ACtrl is TDateTimePicker) and not(ACtrl is TCheckbox);
      dtText:
        Result := (ACtrl is TMemo) or (ACtrl is TEdit) or (ACtrl is TLabeledEdit) or
          (ACtrl is TILSearchGrid) or (ACtrl is TILUpdateGrid);
      dtNumber:
        Result := not(ACtrl is TDateTimePicker) and not(ACtrl is TCheckbox);
      dtBoolean:
        Result := not(ACtrl is TCheckListBox) and not(ACtrl is TDateTimePicker);
      dtDate:
        Result := (ACtrl is TDateTimePicker) or (ACtrl is TEdit) or (ACtrl is TLabeledEdit)
           or (ACtrl is TMemo) or (ACtrl is TILSearchGrid) or (ACtrl is TILUpdateGrid);
      dtResource:
        Result := (ACtrl is TMemo) or (ACtrl is TEdit) or
          (ACtrl is TILSearchGrid) or (ACtrl is TILUpdateGrid);
      dtLongText:
      begin
         if Usage = duData then
           Result := false
         else
           Result := (ACtrl is TEdit) or (ACtrl is TLabeledEdit);
      end;
    end;
  end;

  { TILPropertyValues holds the property values in data record -------------------- }

  function TILPropertyValues.GetProperty(Index: integer): TILPropertyValue;
  begin
    Result := TILPropertyValue(FList[Index]);
  end;

  function TILPropertyValues.GetCount: integer;
  begin
    Result := FList.Count;
  end;

  constructor TILPropertyValues.Create;
  begin
    inherited Create;
    FList := TList.Create;
  end;

  destructor TILPropertyValues.Destroy;

  var
    i: integer;
  begin
    for i := 0 to FList.Count - 1 do
      TILPropertyValue(FList[i]).Free;
    FList.Free;
    inherited Destroy;
  end;

  procedure TILPropertyValues.Add(AProperty: TILPropertyValue);
  begin
    FList.Add(AProperty);
  end;

  procedure TILPropertyValues.Delete(const Index: integer);
  begin
    TILPropertyValue(FList[Index]).Free;
    FList.Delete(Index);
  end;

  { TILPropertyValue the items of TILPropertyValues -------------------------------------- }

  function TILPropertyValue.GetDisplayName: string;

  var
    Indx: integer;
  begin
    Result := '';
    if FOwner.Ilib.DataHandler = nil then
     Exit;
    Indx := FOwner.Ilib.DataHandler.PropertyList.IndexOfPID(FPID);
    Result := FOwner.Ilib.DataHandler.PropertyList.Items[Indx].DisplayName;
  end;

  function TILPropertyValue.GetDisplayValue: string;

  function Get(Val: Variant): string;

  var
    DT: TDateTime;
  begin
    Result := '';
    if DataType = dtString then
      Result := Val
    else if DataType = dtDate then
    begin
      DT := Val;
      Result := DateTimeToStr(DT, Translator.FFormatSettings);
    end
    else if DataType = dtNumber then
    begin
      Result := FloatToStr(Value, Translator.FFormatSettings)
    end
    else if DataType = dtBoolean then
    begin
      if Value = 0 then
        Result := TMan.ReportFalse
      else
        Result := TMan.ReportTrue;
    end;
  end;

  function GetFromArray: string;

  var
    i: integer;
  begin
    for i := 0 to VarArrayHighBound(Value, 1) do
    begin
      if i = 0 then
        Result := Result + FloatToStr(VarArrayGet(Value,[i]))
      else
        Result := #13#10 + Result + FloatToStr(VarArrayGet(Value, [i]));
    end;
  end;

  begin
    Result := '';
    if VarIsNull(Value) then
    begin
      Exit;
    end;
    if Assigned(FOwner.Ilib.DataHandler) and Assigned(FOwner.Ilib.DataHandler.FOnFormatProperty) then
    begin
      FOwner.Ilib.DataHandler.FOnFormatProperty(FOwner.Ilib, Self, Result);
      if Result <> '' then
        Exit;
    end;

    if VarIsArray(Value) then
      Result := GetFromArray
    else
      Result := Get(Value);
  end;

  { TILDB ----------------------------------------------------------------- }

  constructor TILDB.Create(AnOwner: TILTable);
  begin
    inherited Create;
    Ilib := AnOwner;
  end;

  destructor TILDB.Destroy;
  begin
    inherited Destroy;
  end;

  procedure TILDB.CheckConn;
  begin
    if (TMan = nil)
    or not Tman.Connected then
      HandleILException(E_NoConnection);
  end;

  procedure TILDB.AddFieldSpec(Fields: TILRecord; const FieldName, Spec: string);
  { Short hand way of setting conditions }
  var
    i: integer;
    F: TILField;
    Sp : string;
  begin
    i := Ilib.FFields.IndexOf(FieldName);
    if i = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    i := Fields.IndexOf(FieldName);
    if i = -1 then
    begin
      F := TILField.Create(Ilib, FieldName, Ilib.FFields);
      F.FOwnerRecord := Fields;
      Fields.Add(F);
      i := Fields.IndexOf(FieldName);
    end;
    Sp := Trim(Spec);
    if Sp <> '' then
      Fields.Items[i].FValues.Add(Sp);
  end;

  procedure TILDB.ClearControls(AControl : TWinControl = nil);
  begin
    if Self is TILDBView then
      Ilib.ClearControls(duSearch, AControl)
    else
      Ilib.ClearControls(duData, AControl);
  end;

  function TILDB.ControlsEmpty: Boolean;
  begin
    if Self is TILDBView then
      Result := Ilib.IsControlsEmpty(duSearch)
    else
      Result := Ilib.IsControlsEmpty(duData);
  end;

  function TILDB.ExecSQL(AnExpression: TStrings): integer;
  begin
    if Self is TILDBView then
    begin
      Ilib.Root.FSearchOriginator := Ilib;
      Result := Ilib.ExecSQL(AnExpression, [ctSelect]);
    end
    else
      Result := Ilib.ExecSQL(AnExpression, [ctUpdate, ctInsert, ctDelete])
  end;

  function TILDB.ReadControls(Fields: TILRecord): integer;
  begin
    if Self is TILDBView then
      Result := Ilib.ReadControls(Fields, duSearch)
    else
      Result := Ilib.ReadControls(Fields, duData);
  end;

  procedure TILDB.WriteControls(Fields: TILRecord);
  begin
    if Self is TILDBView then
      Ilib.WriteControls(Fields, duSearch)
    else
      Ilib.WriteControls(Fields, duData);
  end;

  { TILDBCommons }

  function TILDBCommons.ExecSQL(Expression: TStrings;
    SQLTypes: TSQLTypes = []): integer;
  begin
    if TMan.CommandFromSQL(Expression) = ctSelect then
    begin
      Ilib.Root.FSearchOriginator := Ilib;
    end;
    Result := Ilib.ExecSQL(Expression, SQLTypes);
  end;

  function TILDBCommons.CanExecSQL(SQL : TStrings) : integer;
  var
   Ct : TSQLType;
   MBar : TILMessageBar;
   sl : TStringList;
   MH : TILMessageHandler;
   Ind : integer;

   procedure GetMBar;
   var
     i : integer;
   begin
      MBar := nil;
      for I := 0 to RegMBs.Count-1 do
      begin
        if TILMessageBar(RegMbs[i]).Table = ILib then
        begin
          Mbar := TILMessageBar(RegMbs[i]);
        end;
      end;
   end;

   procedure SetMsg;
   begin
     Ind := -1;
     MH := TMan.GetMessageHandler(Ilib, Ind);
     while MH <> nil do
     begin
       MH.SQLChanged(ct, Result);
       MH := TMan.GetMessageHandler(Ilib, Ind);
     end;
   end;

  begin
      while Tman.FSystemBusy do
        Sleep(50);
      Comment1 := '';
      Ind := -1;

      ct := ctSelect;
      GetMBar;
      if SQL.Count = 0 then
      begin
        Result := -1;
        SetMsg;
        Exit;
      end;
      sl := TStringList.Create;
      try
        sl.Assign(SQL);
        TrimList(sl, false);
        if sl.Count = 0 then
        begin
          Result := -1;
          SetMsg;
          Exit;
        end;
      finally
        sl.Free;
      end;

      Ct := TMan.CommandFromSQL(SQL);
      if ILib.Dataset.IsReadOnly  then
      begin
        if (ct = ctUpdate)
        or (ct = ctInsert)
        or (ct = ctDelete)
        then
        begin
          Result := E_DatasetReadOnly;
          SetMsg;
          Exit;
        end;
      end;

      if Ct = ctUnknown then
      begin
        Result := E_MissingCommand
      end
      else if Ct = ctSelect then
       Result := Ilib.CheckSearchExpression(SQL)
      else
       Result := Ilib.CheckUpdateExpression(SQL);
      SetMsg;
  end;

  function TILDBCommons.CompareIDS(OldData, NewData: TILDataRecord): Boolean;
  begin
    Result := Ilib.CompareIDS(OldData, NewData);
  end;

  function TILDBCommons.CompareUniques(Record1, Record2: TILDataRecord): TilField;
  begin
    Result := ILib.CompareUniques(Record1, Record2);
  end;

  function TILDBCommons.CompareIdents(Record1, Record2: TILDataRecord): TilField;
  begin
    Result := ILib.CompareIdents(Record1, Record2);
  end;

  function TILDBCommons.CompareRecords(Record1, Record2: TILRecord;
    Delta: Boolean): Boolean;
  begin
    Result := Ilib.CompareRecords(Record1, Record2, Delta);
  end;

  function TILDBCommons.IndexOfID(ADataRecord: TILDataRecord; var FieldNames : string ): integer;
  begin
    Result := Ilib.IndexOfID(ADataRecord, FieldNames);
  end;

  function TILDBCommons.IsControlActive(AControl: TControl): Boolean;
  begin
    Result := Ilib.IsControlActive(AControl)
  end;

  function TILDBCommons.IsRecordOrphan(const RecordIndex: integer): Boolean;
  begin
    Result := Ilib.IsRecordOrphan(RecordIndex);
  end;

  function TILDBCommons.IsRecordEmpty(ARecord: TILRecord): Boolean;
  var
    i: integer;
  begin
    Result := True;
    for i := 0 to ARecord.Count - 1 do
      if ARecord[i].FValues.Count > 0 then
      begin
        Result := false;
        Break;
      end;
  end;

  { TILDBAdmin }
  procedure TILDBAdmin.AlterField(FieldName: string; Alter: TAlterFieldOption;
    Value: Variant);
  begin
    Ilib.AlterField(FieldName, Alter, Value);
  end;

  procedure TILDBAdmin.AlterIndexSpace;
  begin
    Ilib.AlterIndexSpace;
  end;

  procedure TILDBAdmin.AlterRecordFreeSpace;
  begin
    Ilib.AlterRecordFreeSpace;
  end;

  procedure TILDBAdmin.AlterRecordSize;
  begin
    Ilib.AlterRecordSize;
  end;

  procedure TILDBAdmin.AlterRecordLayout(const NewLayout : TRecordlayout);
  begin
    Ilib.AlterRecordLayout(NewLayout);
  end;

  procedure TILDBAdmin.AlterTable(const Option: TILOperation; const NewSpace: integer;
    NewIndexes, NewLookups: TStringList);
  begin
    Ilib.AlterTable(Option, NewSpace, NewIndexes, NewLookups);
  end;

  procedure TILDBAdmin.AlterTableSpace(const NewSpace: integer);
  begin
    Ilib.AlterTableSpace(NewSpace)
  end;

  procedure TILDBAdmin.ClearTable;
  begin
    Ilib.ClearTable;
  end;

  procedure TILDBAdmin.RebuildIndexes;
  begin
    Ilib.RebuildIndexes;
  end;

  procedure TILDBAdmin.RenameTable(const OldName, NewName : string);
  begin
    Ilib.RenameTable(OldName, NewName);
  end;

  { TILDBView ----------------------------------------------------------
    Groups the search methods }

  constructor TILDBView.Create(AnOwner: TILTable);
  begin
    inherited Create(AnOwner);
    Ilib := AnOwner;
  end;

  function TILDBView.GetSearchEngine: TSearcher;
  begin
    Result := Ilib.GetSearcher(ILib);
  end;

  function TILDBView.ExpressionToSQL(AnExpression : TStringList) : TStringList;
  { Converta the internal format to ands and ors}
  var
    i : integer;
    S : string;
  begin
    Result := TStringList.Create;
    for I := 0 to AnExpression.Count-1 do
        begin
          S := AnExpression[i];
          S := StringReplace(S, '\\', ' And ', [rfReplaceAll]);
          S := StringReplace(S, '\', ' And ', [rfReplaceAll]);
          S := StringReplace(S, '//', ' Or ', [rfReplaceAll]);
          S := StringReplace(S, '/', ' Or ', [rfReplaceAll]);
          Result.Add(S);
        end;
  end;

  function TILDBView.ControlsToSQL(AllInPath : Boolean) : TStringList;
  { Reads the controls and construct a logical expression. If AllInPath,
  all active controls in the path are read, else only the controls of the table}
  var
    R : TILDataRecord;
    sl : TStringList;
  begin
    R := Ilib.CreateDatarecord;
    R.Clear;
    try
      Ilib.FReadSearchPathControls := AllInPath;
      Ilib.ReadControls(R, duSearch);
      Ilib.FReadSearchPathControls := True;
      sl := Ilib.GetSearcher(ILib).RecordToExpression(R);
      try
        Result := ExpressionToSQL(sl);
      finally
        sl.Free;
      end;
    finally
      R.Free;
    end;
  end;

  function TILDBView.Exists(Expression: TStringList; var RecordIndex : integer): Boolean;
  begin
    CheckConn;
    Result := Ilib.Exists(Expression, RecordIndex);
  end;

  function TILDBView.CheckSource(Source: TStrings): integer;
  begin
    if Tman.CommandFromSQL(Source) <> ctSelect then
      Result := E_NotAViewCommand
    else
      Result := Ilib.CheckSearchExpression(Source);
  end;

  function TILDBView.CheckSource: integer;
  begin
    Result := Ilib.CheckSearchExpression;
  end;

  function TILDBView.CheckSource(Source: TILRecord): integer;
  begin
    Result := Ilib.CheckSearchExpression(Source)
  end;

  function TILDBView.CanSearch : integer;
  var
   TVParams : TILTreeViewParams;
  begin
       if not TMan.Connected then
       begin
         Result := -1;
         Exit;
       end;
       if not Ilib.DataHandler.FreeSearch then
        if Tman.UseCommandModes and (TMan.CommandMode <> cmSearch) then
       begin
         Result := -1;
         Exit;
       end;

       TVParams := nil;
       if Ilib is TILTreeViewTable then
         TVParams := TILTreeViewTable(ILib).TreeViewParams;
       if ILib is TILTreeViewTable then
       begin
         Result := ILib.DBView.CheckSource(TVParams.WhereCondition);
         if Result = 0 then
           Result := ILib.DBView.CheckSource(TVParams.RootCondition);
       end
       else
         Result := ILib.DBView.CheckSource;
  end;


  function TILDBView.Search(Source: TObject = nil): integer;
  var
    sl: TStringList;
    res: integer;
  begin
    CheckConn;
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    if ILib.Columns.Count = 0 then
      HandleILException(0, 'No columns defined');
    Result := 0;
    if not TMan.CheckUnsaved(Ilib) then
      Exit;
    Ilib.Root.FSearchOriginator := Ilib;
    if Assigned(Source) then
    begin
      if Source is TStrings then
      begin
          Result := ExecSQL(Source as TStrings);
      end
      else if Source is TILRecord then
      begin
        Result := Ilib.Search(Source as TILRecord);
      end
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
    begin
       sl := TStringList.Create;
       try
         Res := Ilib.SearchCtrlsToExpression(sl, True);
         if res <> 0 then
           HandleILException(res, '');
         Result := Ilib.Search(sl);
       finally
         sl.Free;
       end;
    end;
  end;


  function TILDBView.Search(const FieldOrder: string; Source : TObject): integer;
  begin
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    TMan.SearchFieldOrder := FieldOrder;
    Result := Search(Source);
  end;

  function TILDBView.Search(const AnExpression : string) : integer;
  var
    sl : TStringList;
  begin
    sl := TStringList.Create;
    sl.Add(AnExpression);
    try
      Result := Search(sl);
    finally
      sl.Free;
    end;
  end;

  procedure TILDBView.FilterRecords(AFilterTable : TILDataTable);
  var
    i : integer;
    IsLinked : Boolean;
  begin
    CheckConn;
    if not (Ilib is TILDataTable) then
      Exit;

    if Ilib = TMan.RootTable then
      Exit;

    if (AFilterTable <> nil) and not TMan.IsInSearchPath(Ilib, AFilterTable) then
      HandleILException(E_TableOutOfPath, AFilterTable.TableName);

    IsLinked := false;
    for I := 0 to Ilib.MasterLinkCount-1 do
    begin
      if AFilterTable = Ilib.MasterLinks[i] then
      begin
        IsLinked := True;
        Break;
      end;
    end;
    if IsLinked then
    begin
      AFilterTable.FindLinkedRecords;
      Exit;
    end
    else
      Ilib.MasterSource.GetChildRecords(TIlDataTable(Ilib));
  end;

  function TILDBView.ControlsToExpression(Expression: TStringList;
    const IncludeDetailTables: Boolean = false): integer;
  begin
    Result := Ilib.SearchCtrlsToExpression(Expression, IncludeDetailTables);
  end;

  function TILDBView.SearchDeleted: TILDataset;
  begin
    CheckConn;
    Result := Ilib.SearchDeleted;
  end;

  function TILDBView.DoSearchAgg(const FieldName, Func: string; var AResult : Real;
   Condition : TSTringList = nil): Integer;

  var
    sl: TStringList;
    Wh: string;
    DS : TILDataset;
  begin
    CheckConn;
    Result := 0;
    if not TMan.CheckUnsaved(ILib) then
      Exit;
    DS := nil;
    if Trim(FieldName) = '' then
      HandleILException(E_MissingFieldName, '');
    sl := TStringList.Create;
    try
      if Condition <> nil then
        sl.Assign(Condition)
      else
      if ControlsToExpression(sl) > 0 then
        Exit;
      if sl.Count = 0 then
        Wh := ''
      else
        Wh := ' WHERE';
      sl.Insert(0, 'SELECT ' + Func + '(' + FieldName + ')' + Wh);
      if Condition = nil then
        Result := ExecSQL(sl)
      else
      begin
        DS := SearchPrivate(sl);
        Result := DS.Count;
      end;
      if Result <> 0 then
        AResult := DS[0][DS[0].Count-1].AsNumber;
    finally
      if DS <> nil then
        DS.Free;
      sl.Free;
      if Condition <> nil then
        Ilib.Fields.ClearCalcfields;
    end;
  end;

  function TILDBView.SearchCount(const FieldName: string;
     var AResult : integer; Condition : TStringList = nil): integer;

  var
    sl: TStringList;
    Wh: string;
    DS : TILDataset;
    R : TILDataRecord;
    S : string;
  begin
    CheckConn;
    if TMan = nil then
     HandleILException(E_ManagerNotFound);
    Result := 0;
    AResult := 0;
    if not TMan.CheckUnsaved(ILib) then
      Exit;
    DS := nil;
    sl := TStringList.Create;
    try
      if Condition <> nil then
        sl.Assign(Condition)
      else
       if ControlsToExpression(sl) > 0 then
        Exit;
      if sl.Count = 0 then
        Wh := ''
      else
        Wh := ' WHERE';
      if FieldName = '' then
        sl.Insert(0, 'SELECT COUNT(*)' + Wh)
      else
      begin
        sl.Insert(0, 'SELECT COUNT(' + FieldName + ')');
        if sl.Count > 1 then
          sl[1] := wh + ' ' + sl[1] ;
      end;
      if Condition = nil then
      begin
        Result := ExecSQL(sl);
      end
      else
      begin
        DS := SearchPrivate(sl);
        Result := DS.Count;
        if Result <> 0 then
        begin
          R := DS[0];
          S := R[R.Count-1].AsString;
          AResult := StrToInt(S);
        end;
      end;
    finally
      sl.Free;
      if DS <> nil then
        DS.Free;
      if Condition <> nil then
        Ilib.Fields.ClearCalcFields();
    end;
  end;

  function TILDBView.SearchMax(const FieldName: string; var AResult : Real;
    Condition : TStringList = nil): Integer;
  begin
    Result := DoSearchAgg(FieldName, 'MAX', Aresult, Condition);
  end;

  function TILDBView.SearchMin(const FieldName: string; var AResult : Real;
    Condition : TStringList = nil): Integer;
  begin
    Result := DoSearchAgg(FieldName, 'MIN', AResult, Condition);
  end;

  function TILDBView.SearchAvg(const FieldName: string; var AResult : Real;
   Condition : TStringList = nil): Integer;
  begin
    Result := DoSearchAgg(FieldName, 'AVG', AResult, Condition);
  end;

  function TILDBView.SearchSum(const FieldName: string; var AResult : Real;
    Condition : TStringList = nil): Integer;
  begin
    Result := DoSearchAgg(FieldName, 'SUM', AResult, Condition);
  end;

  function TILDBView.SearchPrivate(AnExpression: TStringList; const PerformCalc : Boolean = false): TILDataset;
  var
    SR: TSearcher;
    i : integer;
  begin
    CheckConn;
    SR := Ilib.CreateSearcher(Ilib, AnExpression);
    SR.FTreeViewing := (Ilib is TILTreeViewTable);
    try
      SR.FPrivate := True;
      Result := SR.Search;
    finally
      SR.Free;
    end;
    if PerformCalc then
      for I := 0 to Result.Count-1 do
        Result[i].CalcFields;
  end;

  function TILDBView.SearchPrivate(ARecord: TILRecord; const PerformCalc : Boolean = false): TILDataset;
  var
    SR: TSearcher;
    i : integer;
  begin
    CheckConn;
    SR := Ilib.CreateSearcher(Ilib, ARecord);
    SR.FTreeViewing := (Ilib is TILTreeViewTable);
    try
      SR.FPrivate := True;
      Result := SR.Search;
    finally
      SR.Free;
    end;
    if PerformCalc then
      for I := 0 to Result.Count-1 do
        Result[i].CalcFields;
  end;

  function TILDBView.SearchPrivate(AnExpression: string; const PerformCalc : Boolean = false): TILDataset;
  var
    sl : TStringList;
  begin
    sl := TStringList.Create;
    sl.Add(AnExpression);
    try
      Result := SearchPrivate(sl, PerformCalc);
    finally
      sl.Free;
    end;
  end;

  procedure TILDBView.SetLogOp(const FieldName: string;
    const LogOp: TFieldLogOp);
  begin
    Ilib.SetLogOp(FieldName, LogOp);
  end;

  procedure TILDBView.SetLogOp(const FieldName: string;
    const LogOp: TValueLogOp);
  begin
    Ilib.SetLogOp(FieldName, LogOp);
  end;

  procedure TILDBView.SetLogOp(SearchControl: TWinControl;
    const LogOp: TFieldLogOp);
  begin
    Ilib.SetLogOp(SearchControl, LogOp);
  end;

  procedure TILDBView.SetLogOp(SearchControl: TWinControl;
    const LogOp: TValueLogOp);
  begin
    Ilib.SetLogOp(SearchControl, LogOp);
  end;

  procedure TILDBView.SetLogOp(LogOp: TFieldLogOp);
  begin
    Ilib.SetLogOp(LogOp);
  end;

  { TILDBWork ----------------------------------------------------------- }

  function TILDBWork.CheckSource(Source: TStrings): integer;
  var
    C: TSQLType;
  begin
    if Ilib.IsReadOnly then
      begin
        Comment1 := Ilib.Name;
        Result := E_ReadOnly;
        Exit;
      end;
    C := TMan.CommandFromSQL(Source);
    if not(C in [ctUpdate, ctInsert, ctDelete]) then
      Result := E_NotAWorkCommand
    else
      Result := Ilib.CheckUpdateExpression(Source);
    if (Result = 0) and (C = ctUpdate) and
      (PosRestrict('DATASET', Source.Text, False) <> 0) then
      if Ilib.Dataset.IsReadOnly then
        Result := E_DatasetReadOnly;
  end;

  function TILDBWork.CheckSource(Source: TILRecord): integer;
  begin
    if Ilib.IsReadOnly then
      begin
        Comment1 := Ilib.Name;
        Result := E_ReadOnly;
        Exit;
      end;
    if Source is TILDataRecord then
      with Source as TILDataRecord do
        Result := Validate(True)
    else
      Result := Ilib.CheckUpdateExpression(Source);
    if Result = 0 then
      if Ilib.Dataset.IsReadOnly then
        Result := E_DatasetReadOnly;
  end;

  function TILDBWork.CheckSource(const UpdateType: TUpdateType): integer;

  var
    R: TILDataRecord;
  begin
    if Ilib.IsReadOnly then
      begin
        Comment1 := Ilib.Name;
        Result := E_ReadOnly;
        Exit;
      end;
    if UpdateType = dboInsert then
    begin
      R := CreateDataRecord;
      try
        if Ilib.IsDetail then
        begin
          if (Ilib.MasterSource.Dataset.Position <> -1) and
            (Ilib.MasterSource.FocusedRecord.RecordIndex = -1)
          then
          begin
            Result := E_SaveMasterRecord;
            Exit;
          end
          else if (Ilib.MasterSource.Dataset.Position = -1) then
          begin
            if not Ilib.HasHandler or not Assigned(Ilib.DataHandler.FOnMissingLink) then
            begin
              Result := E_UndefinedLinkRecord;
              Exit;
            end;
          end;
        end;
        Result := ReadControls(R);
        if Result <> 0 then
          Exit;
        Result := R.Validate(True);
      finally
        R.Free;
      end;
    end
    else
    begin
      Result := Ilib.CheckUpdateExpression;
    end;
    if (Result = 0) and (UpdateType = dboUpdate) then
    begin
      if Ilib.Dataset.IsReadOnly then
        Result := E_DatasetReadOnly;
    end;
  end;

  function TILDBWork.ValidateDataset(var ErrorIndex: integer): integer;
  begin
    Result := Ilib.Dataset.Validate(ErrorIndex);
  end;

  function TILDBWork.GetDisableIndexUpdate: Boolean;
  begin
    Result := Ilib.FDisableIndexUpdate;
  end;

  procedure TILDBWork.SetDisableIndexUpdate(const Value : Boolean);
  begin
    if Value <> Ilib.FDisableIndexUpdate then
      Ilib.FDisableIndexUpdate := Value;
  end;

  procedure TILDBWork.SetUpdateTarget(const Value : TUpdateTarget);
  begin
    ILIb.UpdateTarget := Value;
  end;

  function TILDBWork.GetUpdateTarget : TUpdateTarget;
  begin
     Result := ILIb.UpdateTarget;
  end;

  function TILDBWork.DoInsert(Target: TUpdateTarget; Source: TILRecord;
    FileName: string; Image: TPicture): integer;

  var AutSave : Boolean;
  S : string;
  I: Integer;
  begin
    CheckConn;
    Result := 0;
    AutSave := TMan.AutoSave;
    if not AutSave then
     TMan.AutoSave := (Target = utrTable);
    try
    if Assigned(Source) then
    begin
      if Source is TILDataRecord then
      begin
         if FileName <> '' then
           Result := Ilib.AddToDataset(Source as TILDataRecord, FileName)
         else if Assigned(Image) then
           Result := Ilib.AddToDataset(Source as TILDataRecord, Image)
         else
           Result := Ilib.AddToDataset(Source as TILDataRecord);
      end
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
    begin
      if FileName <> '' then
        Result := Ilib.AddToDataset(FileName)
      else if Assigned(Image) then
        Result := Ilib.AddToDataset(Image)
      else
        Result := Ilib.AddToDataset;
    end;

    finally
      TMan.AutoSave := AutSave;
    end;
    S := '';
    if Assigned(ILib.DataHandler.FOnUpdateLongText) then
    begin
      ILib.DataHandler.FOnUpdateLongText(Ilib, nil, S, [ltaClearEditors]);
      if Ilib.FocusedRecord <> nil then
      for I := 0 to Ilib.FocusedRecord.Count-1 do
        Ilib.FocusedRecord[i].ResetLTFlags;
    end;
    if Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsInserted);

  end;

  function TILDBWork.Insert: integer;
  begin
    Result := InsertEmptyRecord;
  end;

  function TILDBWork.Insert(Source: TILRecord): integer;
  begin
    Result := DoInsert(utrDataset, Source, '', nil);
  end;

  function TILDBWork.Insert(Image: TPicture; Source: TILRecord): integer;
  begin
    Result := DoInsert(utrDataset, Source, '', Image);
  end;

  function TILDBWork.Insert(const MediaFile: string; Source: TILRecord): integer;
  begin
    Result := DoInsert(utrDataset, Source, MediaFile, nil);
  end;

  function TILDBWork.Insert(MediaFiles: TStrings; const ReplaceDataset: Boolean): integer;
  begin
    CheckConn;
    if MediaFiles = nil then
      HandleILException(E_MissingImage);
    Result := Ilib.AddToDataset(MediaFiles, ReplaceDataset);
    if Result > 0 then
     if Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsInserted);
  end;

  function TILDBWork.InsertEmptyRecord;
  var
  R : TILDataRecord;
  Precheck : boolean;
  Err : integer;
  begin
    CheckConn;
    R := CreateDataRecord;
    R.OwnsImage := True;
    Precheck := Ilib.PrecheckConstraints;
    Ilib.PrecheckConstraints := false;
    try
     try
      Result := Insert(R);
      if Ilib.FViewer <> nil then
        Ilib.FViewer.ScrollTo(Ilib.Dataset.Count-1);
      if TMan.UseCommandModes then
      begin
        Tman.CommandMode := cmUpdate;
        if Assigned(TMan.FOnChangeCommandMode) then
          TMan.FOnChangeCommandMode(Self);
      end;
     except
      R.Free;
      raise;
     end;
    finally
      Ilib.PrecheckConstraints := Precheck;
    end;
  end;

  constructor TILDBWork.Create(AnOwner: TILTable);
  begin
    inherited Create(AnOwner);
    Ilib := AnOwner;
    FRecordIndex := -1;
  end;

  function TILDBWork.NeedsUpdate(Target: TILDataRecord;
    Source: TObject = nil): Boolean;

  var
    U: TUpdater;
  begin
    Result := false;
    if Ilib.Dataset.IsReadOnly then
    begin
      Exit;
    end;
    if Assigned(Source) then
    begin
      if Source is TILDataRecord then
        HandleILException(E_InvalidSource, Source.ClassName)
      else if Source is TStrings then
        Result := Target.NeedsUpdate(Source as TStrings)
      else if Source is TILRecord then
      begin
        Ilib.UpdateTarget := utrDataSet;
        U := TUpdater.Create(Ilib, Source as TILRecord);
        try
          if U.FError <> 0 then
            Result := false
          else
            Result := Target.NeedsUpdate(U.Expression);
        finally
          U.Free;
        end;
      end
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
      Result := Target.NeedsUpdate;
  end;

  function TILDBWork.NeedsUpdateDataset(Source: TObject = nil): Boolean;

  var
    i: integer;
  begin
    Result := false;
    if Ilib.Dataset.IsReadOnly then
      Exit;
    for i := 0 to Ilib.Dataset.SelCount - 1 do
    begin
      if NeedsUpdate(Ilib.Dataset.Selection[i], Source) then
      begin
        Result := True;
        Break;
      end;
    end;
  end;

  function TILDBWork.ControlsToExpression: TStringList;
  begin
    Result := Ilib.DataCtrlsToExpression;
  end;

  function TILDBWork.Delete(const AnIndex: integer): integer;
  var
    i : integer;
    News : integer;
  begin
      CheckConn;
      Result := 0;
      if AnIndex = -1 then
      begin
        for I := 0 to Ilib.Selcount-1 do
        begin
          if not Ilib.Selection[i].IsNew then
          begin
            Ilib.Dataset.Delete(Ilib.Selection[i].Index);
            Inc(Result);
          end;
        end;
        News := Ilib.Selcount-Result;

        while News > 0 do
        begin
          for I := 0 to Ilib.SelCount-1 do
          if Ilib.Selection[i].IsNew then
          begin
           Ilib.Dataset.Delete(Ilib.Selection[i].Index);
           Inc(Result);
           Dec(News);
           Break;
          end;
        end;
      end
      else
        Result := Ilib.Dataset.Delete(AnIndex);
      if Result >0 then
      begin
        if Assigned(Ilib.Datahandler.FOnDatasetChange) then
         Ilib.Datahandler.FOnDatasetChange(Ilib, dsDeleted);
         if TMan.AutoSave then
           Save;
      end;

  end;

  function TILDBWork.DeleteFromTable(const ARecordIndex: integer): Boolean;
  begin
      CheckConn;
      Result := false;
      if (Ilib = TMan.FTransOwner) then
        TMan.SetTransState(trPending);
      Inc(TMan.FUpdateIDCounter);
      TMan.AddToUpdateStack(Ilib, TMan.FUpdateIDCounter);
      try
      try
       Result := (Ilib.DeletefromTable(ARecordIndex) > 0);
      except on E:Exception do
       begin
        TMan.HandleTransError(E);
       end;
      end;
      if (Ilib = TMan.FTransOwner) then
        TMan.SetTransState(trCommitted);
      finally
        TMan.RemoveFromUpdateStack;
      end;
  end;

  function TILDBWork.Undelete(const RecordIndex: integer): Boolean;
  begin
    CheckConn;
    Result := false;
    if (Ilib = TMan.FTransOwner) then
     TMan.SetTransState(trPending);
    TMan.AddToUpdateStack(Ilib, TMan.FUpdateIDCounter);
    try
    try
     Result := Ilib.Undelete(RecordIndex);
    except on E:Exception do
      begin
        TMan.HandleTransError(E);
      end;
     end;
    if (Ilib = TMan.FTransOwner) then
      TMan.SetTransState(trCommitted);
    finally
      TMan.RemoveFromUpdateStack;
    end;
  end;

  procedure TILDBWork.ClearDataset;
  begin
    if not TMan.CheckUnsaved(Ilib) then
      Exit;
    Ilib.Dataset.Clear;
    if Assigned(Ilib.Datahandler.FOnDatasetChange) then
        Ilib.Datahandler.FOnDatasetChange(Ilib, dsCleared);
  end;

  function TILDBWork.TestUpdate(Target: TILDataRecord; var Updated: Boolean;
    Source: TObject = nil): TILTestResult;
  var
    F: TILRecord;
    sl: TStringList;
  begin
    Result := nil;
    if Assigned(Source) then
    begin
      if Source is TStrings then
      begin
        sl := TStringList.Create;
        try
          sl.Assign(Source as TStrings);
          Result := Ilib.TestUpdate(Target, sl, Updated)
        finally
          sl.Free;
        end;
      end
      else if Source is TILRecord then
        Result := Ilib.TestUpdate(Source as TILRecord, Target, Updated)
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
    begin
      F := TILRecord.Create(Ilib);
      try
        ReadControls(F);
        Result := Ilib.TestUpdate(F, Target, Updated);
      finally
        F.Free;
      end;
    end;
  end;

  function TILDBWork.TestUpdateDataset(Source: TObject = nil): TILUpdateTest;
  var
    UPD: TUpdater;
    sl: TStringList;
  begin
    Result := nil;
    if Assigned(Source) then
    begin
      if Source is TStrings then
      begin
        sl := TStringList.Create;
        try
          sl.Assign(Source as TStrings);
          Result := Ilib.TestUpdateDataset(sl)
        finally
          sl.Free;
        end;
      end
      else if Source is TILRecord then
      begin
        Ilib.UpdateTarget := utrDataSet;
        UPD := TUpdater.Create(Ilib, Source as TILRecord);
        try
          if UPD.FError <> 0 then
            Exit
          else
            Result := Ilib.TestUpdateDataset(UPD.Expression);
        finally
          UPD.Free;
        end;
      end
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
      Result := Ilib.TestUpdateDataset;
  end;

  function TILDBWork.Update(Source: TILRecord): integer;

  begin
    CheckConn;
    Result := 0;
    if Ilib.HasViewer then
      if Ilib.Viewer.Report.SelectedCell <> nil then
        Ilib.Viewer.Report.SelectedCell.CancelEdit;
    if Assigned(Source) then
    begin
      if Source is TILRecord then
      begin
          Result := Ilib.UpdateDataset(Source as TILRecord)
      end
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
    begin
       Result := Ilib.UpdateDataset;
    end;
    if Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsDataUpdated);

  end;

  function TILDBWork.UpdateRecord(TargetRecord: TILDataRecord;
    Source: TILRecord): integer;
  var
    Fields: TILRecord;
  begin
    Result := 0;
    if Ilib.HasViewer then
      if Ilib.Viewer.Report.SelectedCell <> nil then
        Ilib.Viewer.Report.SelectedCell.CancelEdit;
    if Assigned(Source) then
    begin
      if Source is TILRecord then
        Result := Ilib.UpdateRecord(Source as TILRecord, TargetRecord)
      else
        HandleILException(E_InvalidSource, Source.ClassName);
    end
    else
    begin
      Fields := TILRecord.Create(Ilib);
      try
        ReadControls(Fields);
        Result := Ilib.UpdateRecord(Fields, TargetRecord);
      finally
        Fields.Free;
      end;
    end;
  end;

  {$IfDef DLLAccess}
  function TILDBWork.UpdateTable(ASource : TStrings; ARecordIndex : integer) : integer;
  { Used by DLL }
  begin
     CheckConn;
     FRecordIndex := ARecordIndex;
     try
      Result := ExecSQL(ASource);
     finally
      FRecordIndex := -1;
     end;
  end;

  function TILDBWork.UpdateRecordImage(AStream : Pointer; Size : integer; ARecordIndex : integer) : integer;
  { Used by DLL }
  var
    P : TPicture;
    M : TMemoryStream;
    R : TILDataRecord;
  begin
    CheckConn;
    Result := -1;
    R := ILib.RecordsX[ArecordIndex];
    try
    if (AStream = nil) or (Size = 0) then
    begin
       R.UpdateImage(nil);
       R.Save;
       Exit;
    end;
    M := TMemoryStream.Create;
    M.WriteBuffer(AStream^, Size);
    M.Position := 0;
    P := TPicture.Create;
    try
     P.LoadFromStream(M);
     R.FMediaType := MED_IMAGE;
     if P.Bitmap.Width = 0 then
     begin
       Exit;
     end;
     R.UpdateImage(P);
     R.Save;
    finally
      M.Free;
      P.Free;
    end;
    finally
      R.Free;
    end;
    Result := 1;
  end;

  function TILDBWork.UpdateRecordImage(AFileName : string; ARecordIndex : integer) : integer;
  { Used by DLL }
  var
    R : TILDataRecord;
  begin
    CheckConn;
    R := ILib.RecordsX[ArecordIndex];
    try
     R.UpdateImage(AFileName);
     R.Save;
    finally
      R.Free;
    end;
    Result := 1;
  end;
  {$EndIf}

  procedure TILDBWork.ExchangeMedia(Dataset1, Dataset2: TILDataset;
    const Index1, Index2: integer);
  begin
    Dataset1.ExchangeMedia(Index1, Dataset2, Index2);
  end;

  function TILDBWork.UpdateImageFromSource(Inserting : Boolean) : Integer;
  var
   imgTag : string;
   FName : string;
   FNames : TStringList;
   Dlg : TOpenPictureDialog;
   Pic : TPicture;
   Url : Boolean;

  function UseClipBrdImage(var AFilename : string; var URL : Boolean) : Boolean;
  begin
    Result := false;
    AFileName := '';
    Url := false;
    if Clipboard.HasFormat(CF_BITMAP) then
    begin
      if MessageDlg('Use image on the clipboard?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
       Result := True;
      end;
    end
    else
    begin
      AFileName := GetAddressFromClipBrd(Url);
      if AFileName <> '' then
      begin
        if MessageDlg('Use image ' + AFilename + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        begin
         Result := True;
        end;
      end;
    end;
  end;

  procedure RunHandler;
  var
    R : TILDataRecord;
    ImgInd : Integer;
  begin
    if not Ilib.HasHandler then
      Exit;
    ImgInd := -1;
    Ilib.Datahandler.OnImageTableRequest(Ilib, ImgInd);
    if ImgInd = -1 then
    begin
        Result := 0;
        Exit;
    end;
    R := TMan.SharedImages.RecordsX[ImgInd];
    try
      Tman.SharedImages.ShareWith(R[0].AsString, Ilib.FocusedRecord);
    finally
      R.Free;
    end;

  end;

begin
  CheckConn;
  Result := 0;
  if not Ilib.HasHandler then
    Exit;
  if Ilib.DataHandler.ImageSource = isNoImages then
  begin
    Exit;
  end;

  Result := 1;
  FNames := TstringList.Create;
  try
    if ILib.DataHandler.ImageSource = isManual then
    begin
       if not Assigned(ILib.DataHandler.FOnUpdateImage) then
         Exit;
       Pic := nil;
       FName := '';
       ILib.DataHandler.FOnUpdateImage(ILib.DataHandler, Pic, FName);
       if (Pic = nil) and (FName = '') then
         Exit;
       if Pic <> nil then
         ILib.DBWork.UpdateImage(ILib.FocusedRecord.Index, Pic)
       else
         ILib.DBWork.UpdateImage(ILib.FocusedRecord.Index, FName)
    end
    else if ILib.DataHandler.ImageSource = isImageTable then
    begin
      if TMan.SharedImages = nil then
        Exit;
      ImgTag := ILib.DataHandler.GetImageTagValue(ILib.FocusedRecord);
      if ImgTag <> '' then
      begin
       if not ILib.FocusedRecord.GetFromImageTable(ImgTag) then
       begin
         if not Assigned(ILib.Datahandler.OnImageTableRequest) then
         begin
           Result := 0;
           ShowMessage('Could not find any matching images with the tag ' + ImgTag);
         end
         else
         begin
           RunHandler;
         end;
       end;
      end
      else if Assigned(ILib.Datahandler.OnImageTableRequest) then
      begin
         RunHandler;
      end
      else
       Exit;

    end
    else if (ILib.DataHandler.ImageSource = isClipboard) then
    begin
      if not Clipboard.HasFormat(CF_BITMAP) then
        HandleILException(E_ClipbrdNoImage);
      FName := CLIPBOARD_IMAGE;
    end
    else if (ILib.DataHandler.ImageSource in [isFile, isFileOrClipboard]) then
    begin
     if (ILib.DataHandler.ImageSource = isFileOrClipBoard) and UseClipBrdImage(FName, Url) then
     begin
        FName := CLIPBOARD_IMAGE;
     end
     else
     begin
       Dlg := TOpenPictureDialog.Create(Ilib);
       if (ILib.DataHandler.ImageSource = isFile) and not Inserting then
         Dlg.Options := Dlg.Options - [ofAllowMultiSelect]
       else
         Dlg.Options := Dlg.Options + [ofAllowMultiSelect];
       try
        if not Dlg.Execute then
        begin
         Result := 0;
         Exit;
        end;
        FNames.Assign(DLG.Files);
       finally
        Dlg.Free;
       end;
     end;
    end;

    if (ILib.DataHandler.ImageSource <> isImageTable) then
    begin
      if (ILib.DataHandler.ImageSource in [isFile, isFileOrClipBoard]) and Inserting then
      begin
        ILib.DBWork.Insert(FNames, False);
        Result := FNames.Count;
      end
      else
      begin
        if FName = CLIPBOARD_IMAGE then
         ILib.DBWork.UpdateImage(TMan.ActiveRecord.Index, FName)
        else
         ILib.DBWork.UpdateImage(TMan.ActiveRecord.Index, FNames[0]);
        Result := 1;
      end;

    end;
  finally
    FNames.Free;
  end;
end;

  procedure TILDBWork.UpdateImage(const AnIndex: integer; const AFileName: string);
  var
    R: TILDataRecord;
  begin
    begin
      R := Ilib.Dataset.Items[AnIndex];
      R.UpdateImage(AFileName);
    end;
  end;

  function TILDBWork.LinkImage(const AnIndex : integer; const TagName : string) : Boolean;
  var
    R: TILDataRecord;
  begin
    begin
      R := Ilib.Dataset.Items[AnIndex];
      Result := R.GetFromImageTable(Tagname);
    end;
  end;

  procedure TILDBWork.UpdateImage(const AnIndex: integer; AnImage: TPicture);
  var
    R: TILDataRecord;
  begin
    begin
      R := Ilib.Dataset.Items[AnIndex];
      R.UpdateImage(AnImage);
    end;
  end;

  function TILDBWork.CreateDataRecord: TILDataRecord;
  begin
    Result := Ilib.CreateDataRecord;
  end;

  function TILDBWork.Save: integer;
  begin
    CheckConn;
    Result := Ilib.Dataset.Save;
    if Assigned(Ilib.Datahandler.FOnDatasetChange) then
       Ilib.Datahandler.FOnDatasetChange(Ilib, dsSaved);
  end;

  function TILDBWork.CanSave(var ErrorIndex : integer): integer;
  begin
    if not Tman.Connected then
      Result := E_NoConnection
    else
    Result := Ilib.Dataset.CanSave(ErrorIndex);
  end;

  function TILDBWork.CanUpdateData : integer;
  var
    R : TILDataRecord;
    i : integer;
  begin
     if not Tman.Connected then
      Result := E_NoConnection
     else if Ilib.IsReadOnly then
     begin
      Comment1 := Ilib.Name;
      Result := E_ReadOnly;
     end
    else if Ilib.Dataset.IsReadOnly then
      Result := E_DatasetReadOnly
    else if Tman.UseCommandmodes and (TMan.CommandMode <> cmUpdate) then
      Result := -1
    else
      Result := 0;
    if Result <> 0 then
      Exit;
    Result := Ilib.DBWork.CheckSource(dboUpdate);
    if Result = 0 then
     if not Ilib.DBWork.NeedsUpdateDataset then
      Result := -1
     else
     begin
      R := ILib.CreateDataRecord;
      R.FIsDataControls := True;
      R.FOwnerDataset := Ilib.Dataset;
      try
       Result := ReadControls(R);
       if Result <> 0 then
         Exit;
       R.FRecordIndex := Ilib.Selection[0].RecordIndex;
       Result := R.Validate(True);
       if Result = 0 then
       for I := 1 to Ilib.SelCount-1 do
       begin
         R.FRecordIndex := Ilib.Selection[i].RecordIndex;
         Result := R.CheckDuplicates(True);
         if Result <> 0 then
           Break;
       end;
      finally
        R.Free;
      end;

     end;
  end;

  function TILDBWork.CanUpdateImage : integer;
  var
    DF : TILDataField;
  begin
    if not Tman.Connected then
      Result := E_NoConnection
    else if Ilib.IsReadOnly then
    begin
      Comment1 := Ilib.Name;
      Result := E_ReadOnly;
    end
    else if Ilib.Dataset.IsReadOnly then
      Result := E_DatasetReadOnly
    else if Tman.UseCommandmodes and (TMan.CommandMode <> cmUpdate) then
      Result := -1
    else
      Result := 0;
    if Result <> 0 then
      Exit;

     if ILib.HasHandler and Assigned(Tman.ActiveRecord)
     and (ILib.Datahandler.ImageSource <> isNoImages) then
     begin
       if (ILib.Datahandler.ImageSource = isManual)
        and not Assigned(ILib.Datahandler.FOnUpdateImage) then
          Result := -1
       else if (ILib.Datahandler.ImageSource = isImageTable) and
       not Assigned(ILib.Datahandler.FOnImageTableRequest)  then
       begin
         DF := ILib.DataHandler.ImageTagField;
         if (DF = nil) or (DF.Field.IsEmpty) or (DF.Field.IsNull) then
           Result := -1
         else
           Result := 0;
       end
       else
        Result := 0;
     end
     else
       Result := -1;
  end;

  function TILDBWork.CanInsert : Integer;
  begin
     if not Tman.Connected then
      Result := E_NoConnection
     else if Ilib.IsReadOnly then
     begin
      Comment1 := Ilib.Name;
      Result := E_ReadOnly;
     end
     else if Ilib.Dataset.IsReadOnly then
      Result := E_DatasetReadOnly
     else
      Result := 0;
     if Result <> 0 then
      Exit;

     if not ILib.PrecheckConstraints then
     begin
       Result := 0;
       Exit;
     end;
     Result := CheckSource(dboInsert);
  end;

  function TILDBWork.CanDelete(Index : integer) : integer;
  var
    i , j : integer;
    sl : TStringList;
    Fld : TILFieldDef;
    DS : TILDataset;
    Rec : TILDataRecord;
  begin
    if not Tman.Connected then
      Result := E_NoConnection
    else if Ilib.IsReadOnly then
    begin
      Comment1 := Ilib.Name;
      Result := E_ReadOnly;
    end
    else if Ilib.Dataset.IsReadOnly then
      Result := E_DatasetReadOnly
    else
      Result := 0;
    if Result <> 0 then
      Exit;

    if Index = -1 then
      Exit; { New}
    Rec := Ilib.RecordsX[Index];
    sl := TStringList.Create;
    try
       for i := 0 to TMan.TableCount - 1 do
       begin
          for j := 0 to TMan.Tables[i].Fields.Count - 1 do
          begin
              Fld := TMan.Tables[i].Fields.Items[j];
              if (Fld.LookupTable = Ilib) and (Fld.LookupType in [looTableKey, looTableRef]) then
              { This table is used as a lookup table. Check if the record to be deleted contains a lookup value in use }
              begin
                sl.Clear;
                if Fld.DataType = dtString then
                  sl.Add('SELECT * Where ' + Fld.FieldName + ' = ' + '''' + Rec[Fld.LookupField].AsString + '''')
                else
                  sl.Add('SELECT * Where ' + Fld.FieldName + ' = ' + Rec[Fld.LookupField].AsString);
                DS := TMan.Tables[i].DBView.SearchPrivate(sl);
                try
                  if DS.Count > 0 then
                  begin
                    Comment1 := Fld.Caption + ' (' + Rec[Fld.LookupField].FValues[0] + ')' ;
                    Result := E_LookupDelete;
                  end;
                finally
                  DS.Free;
                end;
              end;
          end;
       end;
    finally
     Rec.Free;
     sl.Free;
    end;
  end;

  procedure TILDBWork.RevertChanges(const SelectionOnly : Boolean);
  begin
    CheckConn;
    Ilib.Dataset.RevertChanges(SelectionOnly);
    if Assigned(Ilib.Datahandler.FOnDatasetChange) then
      Ilib.Datahandler.FOnDatasetChange(Ilib, dsReverted);
  end;

  function TILDBWork.CanRevert : Boolean;
  begin
    Result:= Tman.Connected;
    if Result then
      Result := Ilib.Dataset.CanRevert;
  end;

  procedure TILDBWork.SaveAsXML(const AFileName: string);
  begin
    CheckConn;
    Ilib.SaveAsXML(AFileName);
  end;

  procedure TILDBWork.SaveAsJSON(const AFileName: string);
  begin
    CheckConn;
    ILib.SaveAsJSON(AFileName);
  end;

  function TILDBWork.ReadXMLFile(const AFileName: string): integer;
  begin
    CheckConn;
    Result := Ilib.ReadXMLFile(AFileName);
  end;

  procedure TILDBWork.CalcControls;
  begin
    Ilib.CalcControls;
  end;

  procedure TILDBWork.CalcRecord(ARecord : TILDataRecord);
  begin
    ARecord.CalcFields;
  end;

  function TILDBWork.UpdateFileNames(const OldPattern, NewPattern : string;
    const Occurence : integer; Query : TStringList = nil) : integer;
  var
    i : integer;
    R : TILDataRecord;
    NP, NP2 : string;
    F : string;
    OP : string;
    P, P2 : integer;
    Occ : integer;

    procedure DoUpdate(Indx : integer; FName : string);
    var
      Posit : integer;
    begin
      Posit := R.SaveFileResourceName(R.FImgFilePos, FName);
      if Posit <> R.FImgFilePos then
      begin
         R.FImgFilePos := Posit;
         R.FFileName := FName;
         R.FOldFileName := FName;
         Ilib.Records[Indx] := R;
      end;
    end;
  begin
     CheckConn;
     Result := 0;
     Ilib.DisableIndexUpdate := True;
     TMan.FDisableTransLog := True;
     try
     for I := 0 to Ilib.RecordCount-1 do
     begin
       R := Ilib.RecordsAbs[i];
       try
        if R.DeletedMoved or (R.FileName = '') or (R.FImgTableIndex <> -1) then
        { File names in image tables are treated in the source table }
        begin
         Continue;
        end;
        if Occurence <= -1 then
        begin
          F := StringReplace(R.FFileName, OldPattern, NewPattern, [rfReplaceAll]);
          if not SameText(F, R.FileName) then
          begin
            if Assigned(Query) then
              Query.Add(R.FileName + ' > ' + F)
            else
             DoUpdate(i, F);
            inc(Result);
          end;
          Continue;
        end;

        F := R.FileName.ToUpper;
        NP2 := NewPattern;
        NP := NewPattern.ToUpper;
        OP := OldPattern.ToUpper;
        P := Pos(OP, F);
        if P = 0 then
        begin
          Continue;
        end;

        Occ := 1;
        while Occ < Occurence do
        begin
          P2 := Pos(OP, F, P + 1);
          if P2 = 0 then
          begin
            P := 0;
            Break;
          end
          else
          begin
            P := P2;
            inc(Occ);
          end;
        end;

        if P <> 0 then
        begin
         F := R.FileName;
         System.Delete(F, P, Length(OldPattern));
         System.Insert(NP2, F, P);
         if Assigned(Query) then
           Query.Add(R.FileName + ' > ' + F)
         else
           DoUpdate(i, F);
        end;
       finally
         R.Free;
       end;
     end;
     finally
       Ilib.DisableIndexUpdate := false;
       TMan.FDisableTransLog := false;
     end;
  end;

  { TILDataFields ---------------------------------------------------}

  procedure TILDataFields.Assign(Source: TPersistent);

  var
    wSrc: TILDataFields;
    loop: integer;
  begin
    if (Source is TILDataFields) then
    begin
      wSrc := TILDataFields(Source);
      Clear;
      for loop := 0 to wSrc.Count - 1 do
        Add.Assign(wSrc.Items[loop]);
    end
    else
      inherited;
  end;

  function TILDataFields.GetItem(AIndex: integer): TILDataField;
  begin
    Result := TILDataField(inherited Items[AIndex]);
  end;

  procedure TILDataFields.SetItem(AIndex: integer; const Value: TILDataField);
  begin
    inherited SetItem(AIndex, Value);
  end;

  function TILDataFields.GetTable : TILTable;
  begin
   Result := TILData(Owner).Table;
  end;

  function TILDataFields.Add: TILDataField;
  begin
    Result := TILDataField(inherited Add);
  end;

  procedure TILDataFields.Notify(Item: TCollectionItem;
      Action: TCollectionNotification);
  begin
        if (Tman = nil) or (Table = nil) or (csDestroying in TILData(Owner).ComponentState) then
        begin
          inherited;
          Exit;
        end;
        if (Action = cnExtracting) then
        begin
           with Item as TILDatafield do
           begin
             if FieldDef = nil then
             begin
               inherited;
               Exit;
             end;
           end;
           TMan.RemoveWorkField(Item as TILDataField);
           if TILDataField(Item).FFieldName <> '' then
           with Owner as TILData do
           begin
             if SortKey.IndexOf(TILDataField(Item).FFieldName) <> -1 then
             begin
              SortKey.Clear;
              SortOrdering := soAscending;
             end;
             if (Table <> nil) and (Table.Columns <> nil) then
             begin
              Table.Columns.AssignSource;
              if Table.HasViewer then
                Table.Viewer.Invalidate;
             end;
           end;
        end;
        inherited;
  end;

  procedure TILDataFields.Update(Item: TCollectionItem);
  begin
    if (Item <> nil) then
    begin
      if (Table <> nil) and (Table.Columns <> nil)
      and (TILDataField(Item).FieldName <> '')
      then
      begin
        Table.Columns.AssignSource;
        Table.Inval;
      end;
    end
    else if (Table <> nil) and (Table.Columns <> nil) and not (csLoading in Table.ComponentState)
    then
    begin
      Table.Columns.AssignSource;
      if TILData(Owner).SaveColumns
       and (csDesigning in TILData(Owner).ComponentState) then
       Table.SaveColumns;
      if Table.HasViewer then
       Table.Viewer.Invalidate;
    end;
    inherited;
  end;

  function TILDataFields.IndexOf(const FieldName: string): integer;
  var
    i : integer;
  begin
    Result := -1;
    for I := 0 to Count-1 do
      if SameText(Items[i].FFieldName, FieldName) then
      begin
        Result := i;
        Break;
      end;
  end;

  { TILDataField -------------------------------------------------------------}

  procedure TILDataField.SetFieldName(const Value: string);
  var
    Indx : integer;
    i : integer;
    procedure CheckLookup;
    begin
      if (FieldDef <> nil) and (FieldDef.LookupType in [LooHistory, LooTableRef, LooTableKey]) then
       begin
        if DataControl <> nil then
         SetLookup(DataControl);
        if SearchControl <> nil then
         SetLookup(SearchControl);
       end;
    end;

    procedure ResetSort;
    begin
      with FDataFields.Owner as TILData do
       begin
        if FSortKey.IndexOf(FFieldName) <> -1 then
        begin
         FSortKey.Clear;
         FSortOrdering := soAscending;
        end;
       end;
    end;

  begin
     if csLoading in TILData(FDataFields.Owner).ComponentState then
     begin
       FFieldName := value;
       CheckLookup;
       Exit;
     end;
     if SameText(FFieldName, Value) then
       Exit;
     if Trim(Value) = '' then
     begin
       ResetSort;
       Tman.RemoveWorkField(self);
       FFieldName := '';
       FColumnWidth := 50;
       if (Table <> nil) and (Table.Columns <> nil) then
          Table.Columns.AssignSource;
       Exit;
     end;
     if (Table <> nil) then
     begin
       Indx := Table.Fields.IndexOf(Value);
       if Indx = -1 then
       begin
          Comment1 := Value;
          HandleILException(E_InvalidFieldname, Value);
       end;
       for I := 0 to FDataFields.Count-1 do
         if SameText(FDataFields.Items[i].FieldName, Value) then
           HandleILException(E_DupFieldname);
       if FFieldName <> '' then
       begin
         TMan.RemoveWorkField(Self);
       end;
       ResetSort;
       FFieldName := value;
       if FieldDef.DataType = dtLongText then
       begin
          DataControl := nil;
          Exit;
       end;
       CheckLookup;
       if (Table.Columns <> nil) then
          Table.Columns.AssignSource;
       ColumnWidth := FieldDef.ColumnWidth;
       if Table.HasViewer then
         Table.Viewer.Invalidate;
     end
     else
       HandleILException(E_TableNotAssigned);
  end;

  procedure TILDataField.SetDataControl(const Value: TWinControl);
  begin
     if Value = FDataControl then
       Exit;
     if (FieldDef <> nil) and (FieldDef.LookupType = looParentField) then
      Exit;

     if (FieldDef <> nil) and (FieldDef.DataType = dtLongText) then
       Exit;

     FDataControl := Value;
     if (FDataControl <> nil) and Assigned(TMan) and (Value.Parent is TILControlPanel) then
     begin
       TMan.FPanelBuild := True;
       FDataControl.Visible := DacUpdateVisible;
       TMan.FPanelBuild := False;
     end;

     if FieldDef <> nil then
     begin
       FieldDef.DataControl := Value;
       if FieldDef.LookupType in [looHistory, looTableRef, LooTableKey] then
         SetLookup(Value);
     end;
  end;

  procedure TILDataField.SetSearchControl(const Value: TWinControl);
  begin
     if Value = FSearchControl then
       Exit;
     if (FieldDef <> nil) and (FieldDef.LookupType = looParentField) then
      Exit;

     FSearchControl := Value;
     if (FSearchControl <> nil) and Assigned(TMan) and (Value.Parent is TILControlPanel) then
     begin
      TMan.FPanelBuild := True;
      FSearchControl.Visible := DacSearchVisible;
      TMan.FPanelBuild := False;
     end;

     if FieldDef <> nil then
     begin
      FieldDef.SearchControl := Value;
      if FieldDef.LookupType in [looHistory, looTableRef, looTableKey] then
         SetLookup(Value);
     end;
  end;

  procedure TILDataField.SetDataLabel(const Value : TLabel);
  begin
     if Value = FDataLabel then
       Exit;
     if (FieldDef <> nil) and (FieldDef.DataType = dtLongText) then
       Exit;
     if (FieldDef <> nil) and (FieldDef.LookupType = looParentField) then
       Exit;

     FDataLabel := Value;
     FOrigLabelFont.Assign(FDataLabel.Font);

     if (FDataLabel <> nil) and Assigned(TMan) and (Value.Parent is TILControlPanel) then
     begin
       TMan.FPanelBuild := True;
       FDataLabel.Visible := DacUpdateVisible;
       TMan.FPanelBuild := False;
     end;
     if FieldDef <> nil then
      FieldDef.DataLabel := Value;
  end;

  procedure TILDataField.SetSearchLabel(const Value : TLabel);
  begin
     if Value = FSearchLabel then
       Exit;
     FSearchLabel := Value;
     if (FieldDef <> nil) and (FieldDef.LookupType = looParentField) then
      Exit;

     if (FSearchLabel <> nil) and Assigned(TMan) and (Value.Parent is TILControlPanel) then
     begin
       TMan.FPanelBuild := True;
       FSearchLabel.Visible := DacSearchVisible;
       TMan.FPanelBuild := False;
     end;
     if FieldDef <> nil then
       FieldDef.SearchLabel := Value;
  end;

  procedure TILDataField.SetUseAsImageText(const Value : Boolean);
  begin
     if Value = FUseAsImageText then
       Exit;
     FUseAsImageText := Value;
     if FieldDef <> nil then
       FieldDef.FImageText := Value;
  end;

  procedure TILDataField.SetImageTag(const Value : Boolean);
  var
    i : integer;
  begin
    if Value = FImageTag then
      Exit;
    if not Value then
    begin
      FImageTag := false;
      Exit;
    end;
    if csLoading in TILData(FDataFields.Owner).ComponentState then
    begin
      FImageTag := Value;
      Exit;
    end;

    if FieldName = '' then
      HandleILException(E_MissingFieldname);
    if (FieldDef.DataType <> dtString) then
      if (FieldDef.LookupType <> looTableKey) then
       HandleILException(E_StrImageTag);
    if not (Table is TILDataTable) then
       HandleIlException(E_DataTableImageTag);
    for I := 0 to Collection.Count-1 do
      if TILDatafield(Collection.Items[i]).ImageTag then
    begin
      TILDatafield(Collection.Items[i]).FImageTag := false;
      Break;
    end;
    FImageTag := Value;

  end;

  procedure TILDataField.SetDacUpdateVisible(Value : Boolean);
  begin
    if Value = FDacUpdateVisible then
      Exit;
    FDacUpdateVisible := Value;
    if Assigned(Datalabel) then
      DataLabel.Visible := Value;
    if Assigned(DataControl) then
      DataControl.Visible := Value;
  end;

  procedure TILDataField.SetDacSearchVisible(Value : Boolean);
  begin
    if Value = FDacSearchVisible then
      Exit;
    FDacSearchVisible := Value;
    if Assigned(Searchlabel) then
      SearchLabel.Visible := Value;
    if Assigned(SearchControl) then
      SearchControl.Visible := Value;
  end;

  procedure TILDataField.SetColumnVisible(const Value : Boolean );
  begin
    if Value = FColumnVisible then
      Exit;
    FColumnVisible := Value;
    if (FieldName <> '') and (Table <> nil) and (Table.Columns <> nil) then
    begin
      if not Value then
      begin
        if TILData(FDataFields.Owner).SortKey.IndexOf(FieldName) <> -1 then
        begin
          TILData(FDataFields.Owner).SortKey.Clear;
          TILData(FDataFields.Owner).SortOrdering := soAscending;
        end;
      end;
      Table.Columns.AssignSource;
      if TILData(FDataFields.Owner).SaveColumns
      and (csDesigning in TILData(FDataFields.Owner).ComponentState)
      and not (csLoading in TILData(FDataFields.Owner).ComponentState) then
        Table.SaveColumns;
      if Table.HasViewer then
         Table.Viewer.Invalidate;
    end;
  end;

  procedure TILDataField.SetColumnWidth(const Value : integer);
  begin
      if Value = ColumnWidth then
        Exit;
      if (Value < 10) or (Value > 1000) then
        Exit;
      FColumnWidth := Value;
      if FieldDef <> nil then
      begin
        FieldDef.FColumnWidth := Value;
        if TILData(FDataFields.Owner).SaveColumns
        and (csDesigning in TILData(FDataFields.Owner).ComponentState) then
         Table.SaveColumns;
        if Table.HasViewer then
          Table.Viewer.Invalidate;
      end;
  end;

  procedure TILDataField.SetAlignment(const Value : TAlignment);
  begin
      if Value = Alignment then
        Exit;
      FAlignment := Value;
      if FieldDef <> nil then
      begin
       FieldDef.FAlignment := Value;
       if TILData(FDataFields.Owner).SaveColumns
       and (csDesigning in TILData(FDataFields.Owner).ComponentState) then
         Table.SaveColumns;
       if Table.HasViewer then
        Table.Viewer.Invalidate;
      end;

  end;

  function TILDataField.GetColumnWidth : integer;
  begin
      Result := FColumnWidth;
  end;

  function TILDataField.GetAlignment : TAlignment;
  begin
     Result := FAlignment;
  end;

  function TILDataField.GetFieldDef : TILFieldDef;
  var
    Indx : integer;
  begin
     Result := nil;
     if (FieldName <> '') and (Table <> nil) then
     begin
       Indx := Table.Fields.IndexOf(Fieldname);
       if Indx <> -1 then
        Result := Table.Fields.Items[Table.Fields.IndexOf(Fieldname)]
     end;
  end;

  function TILDataField.GetTable: TILTable;
  begin
     Result := FDataFields.Table;
  end;

  function TILDataField.GetField : TILField;
  begin
    Result := nil;
    if (Table = nil) or (FieldDef = nil) then
      Exit;
    if Table.FocusedRecord = nil then
      Exit;
    Result := Table.FocusedRecord[FieldName];
  end;

  function TILDataField.GetDisplayName: string;
  begin
    if FieldName <> '' then
     Result := FieldName
    else
     Result := inherited;
  end;

  constructor TILDataField.Create(Collection: TCollection);
  begin
    inherited;
    FDataFields := TILDataFields(Collection);
    FColumnVisible := True;
    FDacUpdateVisible := True;
    FColumnWidth := 50;
    ForigLabelFont := TFont.Create;
  end;

  destructor TILDataField.Destroy;
  begin
    inherited;
    FOrigLabelFont.Free;
  end;

  procedure TILDataField.Assign(Source: TPersistent);
  var
    wSrc: TILDataField;
  begin
    if Source is TILDataField then
    begin
      wSrc := TILDataField(Source);
      FFieldName := wSrc.FieldName;
      FUseAsImageText := wSrc.UseAsImageText;
      FColumnVisible := wSrc.ColumnVisible;
      FAlignment := wSrc.Alignment;
      FColumnWidth := wSrc.ColumnWidth;
      FCaption := wSrc.Caption;
      FImageTag := wSrc.ImageTag;
      FDacSearchVisible := wSrc.DacSearchVisible;
      FDacUpdateVisible := wSrc.DacUpdateVisible;
      FDataControl := wSrc.DataControl;
      FDataFields := wSrc.FDataFields;
      FDataLabel := wSrc.DataLabel;
      FFieldName := wSrc.FieldName;
      FSearchControl := wSrc.SearchControl;
      FSearchLabel := wSrc.SearchLabel;
      FUseAsImageText := wSrc.UseAsImageText;
    end
    else
      inherited;
  end;

  procedure TILDataField.SetCaption(const Value : string);
  begin
    if FCaption = Value then
      Exit;
    if csLoading in TILData(FDataFields.Owner).ComponentState then
    begin
      FCaption := Value;
      Exit;
    end;
    FCaption := Value;
    if (TILData(FDataFields.Owner).Table <> nil) and (csDesigning in TILData(FDataFields.Owner).ComponentState)
    and TILData(FDataFields.Owner).Table.HasViewer then
       TILData(FDataFields.Owner).Table.Viewer.Invalidate;
  end;

 procedure TILDataField.CtrlChng(Sender : TObject);
 var
   D : TILData;
   MH : TILMessageHandler;
   Ind : integer;
   R : TILRecord;

  procedure DoEvents;
  begin
   if (DataControl = Sender) and Assigned(FieldDef.FSCDataChange) then
     FieldDef.FSCDataChange(Sender)
   else if (SearchControl = Sender) and Assigned(FieldDef.FSCSearchChange) then
     FieldDef.FSCSearchChange(Sender)
   else if Assigned(D.FOnChangeDAC) then
   begin
       R := TILRecord.Create(D.Table);
       try
         if DataControl = Sender then
           D.DBWork.ReadControls(R)
         else
           D.DBView.ReadControls(R);
         Ind := R.IndexOf(FieldName);
         if Ind <> -1 then
         begin
           if DataControl = Sender then
             D.FOnChangeDAC(Sender, Self, duData, R.Items[Ind].AsString)
           else
             D.FOnChangeDAC(Sender, Self, duSearch, R.Items[Ind].AsString)
         end;
       finally
        R.Free;
       end;
   end;
  end;

 begin
    if Table <> nil then
    begin
      D := TILData(FDataFields.Owner);
      if Table.FWritingControls then
      begin
        DoEvents;
        Exit;
      end;
      if FieldDef = nil then
       Exit;
      if FieldDef.LookupType in [looHistory, looTableRef, looTableKey] then
       Table.ShowLookups(Sender as TWinControl);
      Ind := -1;
      MH := TMan.GetMessageHandler(Table, Ind);
      if DataControl = Sender then
      begin
        while MH <> nil do
        begin
         MH.ControlsChanged(duData);
         MH := TMan.GetMessageHandler(Table, Ind);
        end;
        DoEvents;
      end
      else
      begin
       while MH <> nil do
        begin
         MH.ControlsChanged(duSearch);
         MH := TMan.GetMessageHandler(Table, Ind);
        end;
        DoEvents;
      end;
    end;
 end;

 procedure TILDataField.SetLookup(Ctrl : TWinControl);
 var
   Usage : TDataUsage;
 begin
   if Ctrl = nil then
     Exit;
   Usage := duData;
   if Assigned(FieldDef) then
   begin
      if SearchControl = Ctrl then
        Usage := duSearch;
   end;
   if Ctrl is TMemo then
   with Ctrl as TMemo do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng
   end
   else if Ctrl is TEdit then
   with Ctrl as TEdit do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng
   end
   else if Ctrl is TLabeledEdit then
   with Ctrl as TLabeledEdit do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng;
   end
   else if Ctrl is TComboBox then
   with Ctrl as TComboBox do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng;
   end
   else if Ctrl is TListBox then
   with Ctrl as TListBox do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnClick
      else
        FieldDef.FSCSearchChange := OnClick
     end;
     OnClick := CtrlChng;
   end
   else if Ctrl is TRadioGroup then
   with Ctrl as TRadioGroup do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnClick
      else
        FieldDef.FSCSearchChange := OnClick
     end;
     OnClick := CtrlChng;
   end
   else if Ctrl is TCheckListBox then
   with Ctrl as TCheckListBox do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnClick
      else
        FieldDef.FSCSearchChange := OnClick
     end;
     OnClick := CtrlChng;
   end
   else if Ctrl is TCheckBox then
   with Ctrl as TCheckBox do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnClick
      else
        FieldDef.FSCSearchChange := OnClick
     end;
     OnClick := CtrlChng;
   end
   else if Ctrl is TILSearchGrid then
   with Ctrl as TILSearchGrid do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng;
   end
   else if Ctrl is TILUpdateGrid then
   with Ctrl as TILUpdateGrid do
   begin
     if Assigned(FieldDef) then
     begin
      if Usage = duData then
        FieldDef.FSCDataChange := OnChange
      else
        FieldDef.FSCSearchChange := OnChange
     end;
     OnChange := CtrlChng;
   end
 end;

 { TILData ----------------------------------------------------------------}

  constructor TILData.Create(AOwner : TComponent);
  begin
      inherited;
      FDataFields := TILDataFields.Create(Self, TILDataField);
      FPropertyList := TILPropertyList.Create(Self, TILPropertyItem);
      FDataSetMaxCount := 10000;
      FPrecheckConstraints := True;
      RegHandlers.Add(Self);
      FPropagateSearchFields := True;
      FSortKey := TStringList.Create;
      FFilter := TStringList.Create;
      FAutoCreateFields := True;
      FMDOptions := [mdLookupParents];
  end;

  destructor TILData.Destroy;
  begin
     inherited;
     FDataFields.Free;
     FPropertyList.Free;
     FSortKey.Free;
     FFilter.Free;
     if RegHandlers.IndexOf(Self) <> -1 then
       RegHandlers.Delete(RegHandlers.IndexOf(Self));
  end;

  procedure TILData.BeforeDestruction;
  begin
    inherited;
  end;

  procedure TILData.Notification(AComponent: TComponent; Operation: TOperation);
  begin
    inherited Notification(AComponent, Operation);
    if (csDestroying in ComponentState) or (csLoading in ComponentState) then
      Exit;
    if Tman = nil then
      Exit;
    if Table = nil then
      Exit;
    if TMan = nil then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent = FSummaryControl then
        FSummaryControl := nil
      else if AComponent = Table then
      begin
         DataFields.Clear;
         SortKey.Clear;
         SortOrdering := soAscending;
         FTable := nil;
      end
      else if AComponent = TreeViewControl then
         FTreeViewControl := nil
      else if AComponent = ImageControl then
        FImageControl := nil;
    end;
  end;

  procedure TILData.SetPrecheckConstraints(const Value : Boolean);
  begin
    if Value = FPrecheckConstraints then
      Exit;
    FPrecheckConstraints := Value;
    if Table <> nil then
      Table.PrecheckConstraints := Value;
  end;

  procedure TILData.SetSortKey(const Value : TStringList);
  var
    i : integer;
    Indx : integer;
  begin
    if csLoading in ComponentState then
    begin
      FSortKey.Assign(Value);
      Exit;
    end;
    if (Table <> nil) and (Table.Columns <> nil) then
    begin
      for I := 0 to Value.Count-1 do
      begin
         indx:=  DataFields.IndexOf(Value[i]);
         Comment1 := Value[i];
         if Indx = -1 then
           HandleILException(E_InvalidSortField);
         if not DataFields.Items[Indx].FieldDef.ReportField
          or (DataFields.Items[Indx].FieldDef.FieldType = ftStoredEvaluation)
          or (DataFields.Items[Indx].FieldDef.DataType in [dtLongText, dtText]) then
           HandleILException(E_IllegalSortField);
         Comment1 := '';
      end;
      FSortKey.Assign(Value);
      Table.Columns.AssignSource;
      if SaveColumns
      and (csDesigning in ComponentState)
      and not (csLoading in ComponentState) then
        Table.SaveColumns;
    end;
  end;

  procedure TILData.SetFilter(const Value : TStringList);
  begin
    FFilter.Assign(Value);
  end;

  procedure TILData.SetSortOrdering(const Value : TSortOrdering);
  begin
    if Value = FSortOrdering then
      Exit;
    FSortOrdering := Value;
    if (Table <> nil) and (Table.Columns <> nil) then
      Table.Columns.AssignSource;
    if SaveColumns  and (Table <> nil)
    and (csDesigning in ComponentState)
    and not (csLoading in ComponentState) then
      Table.SaveColumns;
  end;

  procedure TILData.SetSaveColumns(const Value : Boolean);
  begin
    if Value = FSaveColumns then
      Exit;
    FSaveColumns := Value;
    if Tman = nil then
      Exit;
    if Table = nil then
      Exit;
    if not FSaveColumns then
    begin
        Tman.FCfgFile.EraseSection(Table.TableName + '_ColumnSequence');
        Tman.FCfgFile.EraseSection(Table.TableName + '_ColumnSort');
    end
    else
    if (csDesigning in ComponentState)
    and not (csLoading in ComponentState) then
      Table.SaveColumns;
  end;

  procedure TILData.Loaded;
  begin
    inherited;
    if Table <> nil then
    begin
      Table.FMultiSelect := FMultiSelect;
      Table.DatasetMaxCount := DataSetMaxCount;
      Table.PrecheckConstraints := PrecheckConstraints;
      if FTable is TILTreeViewTable then
      begin
        TILTreeViewTable(FTable).FTreeViewControl := TreeViewControl;
        if TreeViewControl <> nil then
          TILTreeViewTable(FTable).FTreeViewControl.FTVTable := TILTreeViewTable(FTable);
        TILTreeViewTable(FTable).FTreeViewLabelField := TreeViewLabelField;
      end
      else
      begin
        FTreeViewControl := nil;
        FTreeViewLabelField := '';
      end;
    end;
  end;

  procedure TILData.SetDatasetMaxCount(const Value : integer);
  begin
     if csLoading in ComponentState then
     begin
       FDatasetMaxCount := Value;
       Exit;
     end;

     if FDatasetMaxCount = Value then
       Exit;

     if Value < 0 then
      Exit;
     if Value < 500 then
      Exit;
     FDatasetMaxCount := Value;

     if Table <> nil then
       Table.DatasetMaxCount := Value;
  end;

  procedure TILData.SetMultiSelect(const Value : Boolean);
  begin
     if csLoading in ComponentState then
     begin
       FMultiSelect := Value;
       Exit;
     end;
     if FMultiSelect = Value then
        Exit;
     FMultiSelect := Value;
     if Table <> nil then
       Table.FMultiSelect := Value;
  end;

  procedure TILData.SetTable(const Value : TILTable);
  var
    i : integer;
    D : TILData;

  procedure SetValues;
  begin
   if FTable =  nil then
     Exit;
   FTable.FDataHandler := Self;
   FTable.FMultiSelect := FMultiSelect;
   FTable.FDatasetMaxCount := DataSetMaxCount;
   FTable.FPrecheckConstraints := PrecheckConstraints;
  end;

  begin
    if Value = nil then
     Exit;
    if (FTable = Value) then
     Exit;
    if csLoading in ComponentState then
    begin
      FTable := Value;
      SetValues;
      Exit;
    end;

    if (csDesigning in ComponentState) then
    for I := 0 to RegHandlers.Count-1 do
    begin
       D := TILData(RegHandlers[i]);
       if (D <> Self) and (Pos('_Internal', D.Name) = 0) then
        if D.Table = Value then
        begin
            HandleILException(E_DataDupTable, D.Name);
        end;
    end;

    FTreeViewControl := nil;
    FTreeViewLabelField := '';

    if FTable <> nil then
    begin
     if TMan = nil then
      HandleILException(E_ManagerNotFound);
     DataFields.Clear;
     if FTable.HasViewer then
     begin
       FTable.Viewer.Table := nil;
     end;
     FTable.FDataHandler := nil;
   end;
   FTable := Value;
   if FTable <> nil then
   begin
    SetValues;
    if AutoCreateFields then
     FillDataFields;
    FreeNotification(FTable);
   end;
   if (TMan.TableList <> nil) and not HideInTableList then
     TMan.PopulateTableList(TMan.TableList);
  end;

  function TILData.GetReadOnly : Boolean;
  begin
      Result := FIsReadOnly;
  end;

  procedure TILData.SetDataset(const Value : TILDataset);
  begin
     if Table = nil then
       Exit;
     Table.Dataset := Value;
  end;

  function TILData.GetDataset : TILDataset;
  begin
     if Table = nil then
      Result := nil
     else
      Result := Table.Dataset;
  end;

  function TILData.GetDBView : TILDBView;
  begin
     if Table = nil then
      Result := nil
     else
      Result := Table.DBView;
  end;

  function TILData.GetDBWork : TILDBWork;
  begin
     if Table = nil then
      Result := nil
     else
      Result := Table.DBWork;
  end;

  function TILData.GetDBCommons : TILDBCommons;
  begin
     if Table = nil then
      Result := nil
     else
      Result := Table.DBCommons;
  end;

  function TILData.GetMultiSelected : Boolean;
  begin
    Result := FMultiSelect and (Dataset.SelCount > 1);
  end;

  function TILData.GetImageTagField : TILDataField;
  var
    i : integer;
  begin
     Result := nil;
     for I := 0 to DataFields.Count-1 do
      if Datafields.Items[i].ImageTag then
     begin
       Result := Datafields.Items[i];
       Break;
     end;
  end;

  procedure TILData.SetDataFields(const Value : TILDataFields);
  begin
     FDataFields.Assign(Value);
  end;

  procedure TILData.SetPropertyList(const Value : TILPropertyList);
  begin
     FPropertyList.Assign(Value);
  end;

  procedure TILData.SetPropagateSearchFields(Value : Boolean);
  begin
    if Value = FPropagateSearchFields then
      Exit;
    FPropagateSearchFields := Value;
    if Tman <> nil then
    if TMan.UseCommandModes and Tman.Connected and not (csDesigning in ComponentState)
    and (TMan.CommandMode = cmSearch) then
    begin
      Tman.CreateWorkFields(Table, cmSearch);
      TMan.UpdateWorkList;
      TMan.DoShowWorkFields(Tman.FindPanel(Tman.ActiveTable, cmSearch));
    end;

  end;

  procedure TILData.SetSummaryControl(const Value : TILRecordViewer);
  begin
    if Value = FSummaryControl then
      Exit;
    FSummaryControl := Value;
    if Value <> nil then
      Value.FreeNotification(Self);
  end;

  procedure TILData.SetTreeViewControl(const Value : TILTreeView);
  begin
    if Value = FTreeViewControl then
      Exit;
    if csLoading in ComponentState then
    begin
      FTreeViewControl := Value;
      if FTreeViewControl <> nil then
        FTreeViewControl.FreeNotification(Self);
      Exit;
    end;
    if Table <> nil then
    begin
        if (Table is TILTreeViewTable) then
        begin
          FTreeViewControl := Value;
          TILTreeViewTable(Table).TreeViewControl := Value;
        end
        else
          Exit;
    end;
    if FTreeViewControl <> nil then
      FTreeViewControl.FreeNotification(Self);
  end;

  procedure TILData.SetTreeViewLabelField(const Value : string);
  begin
    if Value = FTreeViewLabelField then
      Exit;
    if csLoading in ComponentState then
    begin
      FTreeViewLabelField := Value;
      Exit;
    end;
    if Table <> nil then
    begin
       if (Table is TILTreeViewTable) then
       begin
         TILTreeViewTable(Table).TreeViewLabelField := Value;
         FTreeViewLabelField := Value;
       end
       else
        Exit;
    end;
  end;

  procedure TILData.SetAutoCreateFields(const Value : Boolean);
  begin
    if Value = FAutoCreateFields then
      Exit;
    FAutoCreateFields := Value;
    if csLoading in ComponentState then
    begin
       Exit;
    end;
    if Value and Assigned(Table) then
      FillDataFields;
  end;

  procedure TILData.SetHideInTableList(Value : Boolean);
  begin
    if FHideInTableList = Value then
      Exit;
    FHideInTableList := Value;
    if csLoading in ComponentState then
      Exit;
    if TMan <> nil then
      if Assigned(TMan.FTableList) then
      TMan.PopulateTableList(TMan.FTableList);
  end;

  procedure TILData.FillDataFields;
  var
    i : integer;
    DF : TILDataField;
    Fld : TILFieldDef;
  begin
      for I := 0 to Table.Fields.Count-1 do
      begin
        Fld := Table.Fields.Items[i];
        if not Fld.ReportField then
          Continue;
        if DataFields.IndexOf(Fld.FieldName) <> -1 then
          Continue;
        DF := DataFields.Add;
        DF.FFieldName := Fld.FieldName;
        DF.FColumnWidth := Fld.ColumnWidth;
        if not TMan.UseCommandModes then
         DF.FDacSearchVisible := True;
      end;
      Table.Columns.AssignSource;
  end;

  procedure TILData.InitLookups;
  var
    i : integer;
  begin
    for I := 0 to DataFields.Count-1 do
    begin
      if DataFields.Items[i].Table = nil then
        Continue;
      if DataFields.Items[i].FieldName = '' then
        Continue;
      begin
        if DataFields.Items[i].DataControl <> nil then
        begin
           DataFields.Items[i].SetLookup(DataFields.Items[i].DataControl);
        end;
        if DataFields.Items[i].SearchControl <> nil then
        begin
           DataFields.Items[i].SetLookup(DataFields.Items[i].SearchControl);
        end;
      end;
    end;
  end;

  procedure TILData.AssignColumns;
  {Syncs with column object }
  var
    i : integer;
    Indx : integer;
    Fld : TILDataField;
  begin
    if Table = nil then
      Exit;
    for I := 0 to DataFields.Count-1 do
    begin
       Fld := DataFields.Items[i];
       if Fld.FieldName = '' then
         Continue;
       Indx := Table.Columns.IndexOf(Fld.FieldName);
       if Indx = -1 then
       begin
         Fld.FColumnVisible := false;
       end
       else
       begin
         Fld.FColumnVisible := True;
       end;
       Indx := Table.Fields.IndexOf(Fld.FieldName);
       if Indx <> -1 then
         Fld.FColumnWidth := Table.Fields.Items[Indx].ColumnWidth;
    end;
    FSortKey.Assign(Table.Columns.SortFields);
    FSortOrdering := Table.Columns.SortOrdering;
  end;

  procedure TILData.CreateSummary(DataRecord: TILDataRecord);

  function Shared: Boolean;
  var
    i: integer;
    M: TILTable;
    Cnt: integer;
    DH : TILData;
  begin
    Result := false;
    if Table = nil then
      Exit;
    Cnt := 0;
    if Table.MasterSource = nil then
      M := Table
    else
      M := Table.MasterSource;
    if M.DataHandler = nil then
      Exit;
    DH := M.DataHandler;
    if DH.SummaryControl = SummaryControl then
      Inc(Cnt);
    Inc(Cnt);
    for i := 0 to M.AllChildCount - 1 do
    begin
      if M.AllChildren[i].DataHandler = nil then
       Continue;
      if M.AllChildren[i].DataHandler.SummaryControl = SummaryControl then
        Inc(Cnt);
    end;
    Result := (Cnt > 1) or (M.AllChildCount = 0);
  end;

  begin
    if not Table.Connected or not((Table = TMan.ActiveTable) and Shared) then
      Exit;
    if SummaryControl = nil then
      Exit;
    if DataRecord = nil then
      if Dataset.Count = 0 then
      begin
        SummaryControl.Clear;
        Exit;
      end
      else
        DataRecord := Table.FocusedRecord;
    SummaryControl.ShowRecord(DataRecord);
 end;

 function TILData.GetImageTagValue(Data : TILDatarecord) : string;
var
    I : integer;
    Indx : integer;
    s : string;
    DS : TILDataset;
  begin
    Result := '';
    if Data = nil then
      Exit;
    if (TMan.SharedImages <> nil) then
      begin
       for I := 0 to DataFields.Count-1 do
        if DataFields.Items[i].ImageTag then
        begin
         Indx := Data.IndexOf(DataFields.Items[i].FieldName);
         if DataFields.Items[i].FieldDef.LookupType = looTableKey then
         begin
          s := 'select * where ' + DataFields.Items[i].FieldDef.LookupField + ' = ' +
           Data[Indx].AsString;
          DS := DataFields.Items[i].FieldDef.LookupTable.DBView.SearchPrivate(S);
          try
            if DS.Count > 0 then
             Result := DS[0][DataFields.Items[i].FieldDef.LookupSearchField].AsString;
          finally
            DS.Free;
          end;
         end
         else
          Result := Data[Indx].AsString;
        end;
      end;
  end;

  { TILDatasetViewer -------------------------------------------------------}

  function VertScrollBarVisible(WindowHandle: THandle): Boolean;
  begin
    Result:= (GetWindowlong(WindowHandle, GWL_STYLE) AND WS_VSCROLL) <> 0
  end;

  function HorzScrollBarVisible(WindowHandle: THandle): Boolean;
  begin
    Result:= (GetWindowlong(WindowHandle, GWL_STYLE) AND WS_HSCROLL) <> 0
  end;

  constructor TILDatasetViewer.Create(AOwner: TComponent);
  begin
    inherited Create(AOwner);
    ControlStyle := ControlStyle + [csReplicatable, csPannable, csOpaque,
      csAcceptsControls];
    Width := 120;
    Height := 120;
    FImageRowSpacing := 10;
    FImageItemSpacing := 10;
    FImageLayout := ilGrid;
    FImageWidth := 120;
    FImageHeight := 120;
    FImageTextSpace := 0;
    FDrawBM := Vcl.Graphics.TBitmap.Create;
    FCaptionAlignment := taCenter;
    FGridLineColor := clActiveBorder;
    FItemUnderMouse := -1;
    FLineSpacing := 5;
    FColumnSpacing := 5;
    FReportImageSize := risMedium;
    FReportOptions := [];
    FColumnClicked := -1;
    FHighLightColumn := -1;
    FSplitterClicked := -1;
    FImageOptions := [ioWrapPartialVisibles];
    FShowCheckBoxes := false;
    FFocusRectangle := True;
    FViewMode := vmReport;
  end;

  destructor TILDatasetViewer.Destroy;
  begin
    FDrawBM.Free;
    if Assigned(FReport) then
    begin
      FReport.Free;
      FCellEditor.Free;
    end;
    inherited Destroy;
  end;

  function TILDatasetViewer.GetStyleColor(const Element: TStyleElements;
    const StyleTarget: TStyleTarget): TColor;
  begin
    Result := clBlack;
    if Element = [seFont] then
    begin

      if TStyleManager.IsCustomStyleActive And (seFont In StyleElements) and
        not(csDesigning in ComponentState) then
      begin
        if StyleTarget = stFont then
          Result := StyleServices.GetStyleFontColor(sfGridItemNormal)
        else if StyleTarget = stFontHot then
          Result := StyleServices.GetStyleFontColor(sfButtonTextHot)
        else if StyleTarget = stFontCaption then
          Result := StyleServices.GetStyleFontColor(sfButtonTextHot)
      end
      else
        Result := Font.Color;
    end
    else if Element = [seClient] then
    begin
      if TStyleManager.IsCustomStyleActive And (seClient In StyleElements) and
        not(csDesigning in ComponentState) then
      begin
        if StyleTarget = stBackground then
          Result := StyleServices.GetStyleColor(scGrid)
        else if StyleTarget = stHotTrack then
          Result := clSilver
        else if StyleTarget = stFocused then
          Result := StyleServices.GetSystemColor(clHighLight)
        else if StyleTarget = stCheckbox then
          Result := StyleServices.GetSystemColor(clBtnHighlight)
        else if StyleTarget = stSelCol then
          Result := StyleServices.GetStyleColor(scButtonFocused)
        else if StyleTarget = stHotCol then
          Result := clSilver
        else if StyleTarget = stActiveColHeading then
          Result := StyleServices.GetStyleColor(scButtonFocused)
        else if StyleTarget = stInactiveColHeading then
          Result := StyleServices.GetStyleColor(scButtonNormal)
        else if StyleTarget = stColPressed then
          Result := StyleServices.GetStyleColor(scButtonPressed)
        else if StyleTarget = stCaption then
          Result := StyleServices.GetSystemColor(clBlue)
        else if StyleTarget = stInactiveCaption then
          Result := StyleServices.GetSystemColor(clGray);
      end
      else
      begin
        if StyleTarget = stBackground then
          Result := Color
        else if StyleTarget = stHotTrack then
          Result := HotTrackClr
        else if StyleTarget = stFocused then
          Result := SelectedClr
        else if StyleTarget = stCheckbox then
          Result := clWhite
        else if StyleTarget = stActiveColHeading then
          Result := RGB(242, 247, 253)
        else if StyleTarget = stInactiveColHeading then
          Result := RGB(253, 253, 253)
        else if StyleTarget = stSelCol then
          Result := clHighLight
        else if StyleTarget = stHotCol then
          Result := HotTrackClr
        else if StyleTarget = stColPressed then
          Result := clBtnFace
        else if StyleTarget = stCaption then
          Result := clHighLight
        else if StyleTarget = stInactiveCaption then
          Result := clInActiveCaption
        else
          Result := Color;
      end;
    end
    else if Element = [seBorder] then
    begin
      if TStyleManager.IsCustomStyleActive And (seBorder In StyleElements) and
        not(csDesigning in ComponentState) then
      begin
        if StyleTarget = stGridlines then
         Result := StyleServices.GetStyleColor(scBorder)
        else if StyleTarget = stCheckBoxBorder then
          Result := StyleServices.GetSystemColor(clBtnShadow)
        else if StyleTarget = stImageBorder then
          Result := StyleServices.GetStyleColor(scBorder)
        else if StyleTarget = stBorder then
          Result := StyleServices.GetStyleColor(scBorder)
      end
      else
      begin
        if StyleTarget = stGridlines then
          Result := RGB(222, 231, 241)
        else
          Result := clBlack;
      end;
    end;
  end;

  function TILDatasetViewer.CanReportDisplayImages : Boolean;
  begin
    Result := false;
    if (Table = nil) or (Table.DataHandler = nil) then
      Exit;
    Result := (ReportImages = riEmbed) and (Table.DataHandler.ImageSource <> isNoImages);
  end;

  function TILDatasetViewer.CanScroll(const Forwrd: Boolean; const ScrollDir: integer): Boolean;
  var
    First, Last: integer;
  begin
    if Table = nil then
    begin
      Result := false;
     Exit;
    end;
    if (Table.Dataset = nil) or (Table.Dataset.Count = 0) then
    begin
      Result := false;
      Exit;
    end;
    GetVisibles(First, Last);
    if First = -1 then
    begin
      Result := false;
      Exit;
    end;
    if Forwrd then
    begin
      if (First = Last) and (Table.Dataset.Count <= GetColCount) then
        { Count fills one single row }
        Result := false
      else
      begin
        if not(ioWrapPartialVisibles in ImageOptions) then
          Result := (Last < OutputCount - 1) or
            not IsImageFullyVisible(Last, false)
        else
          Result := (Last < OutputCount - 1);
      end;
    end
    else
    begin
      Result := (FirstItemIndex > 0);
    end;
  end;

  procedure TILDatasetViewer.SetReportOptions(const Value : TReportOptions);
  begin
    if FReportOptions = Value then
      Exit;
    FReportOptions := Value;
    Invalidate;
  end;

  procedure TILDatasetViewer.SetGridlineColor(Value : TColor);
  begin
     if Value <> FGridlineColor then
     begin
       FGridlineColor := Value;
       Invalidate;
     end;

  end;

  procedure TILDatasetViewer.SetReportImages(const Value : TReportImages);
  begin
    if Value = FReportImages then
      Exit;
    FReportImages := Value;
    if Table = nil then
      Exit;
    if Table.IsRunning then
    begin
      CreateReport;
      UpdateScrollRange;
      UpdateScrollPos;
      Table.Dataset.Position := 0;
      FFirstItemIndex := 0;
    end;
    Invalidate;
  end;

  function TILDatasetViewer.GetTable : TILTable;
  begin
      Result := FTable;
  end;

  procedure TILDatasetViewer.SetReportImageSize(const Value : TReportImageSize);
  begin
    if Value = FReportImageSize then
      Exit;
    FReportImageSize := Value;
    Invalidate;
  end;

  function TILDatasetViewer.GetImageWidth: integer;
  begin
    Result := FImageWidth;
  end;

  function TILDatasetViewer.GetImageHeight: integer;
  begin
    Result := FImageHeight;
  end;

  function TILDatasetViewer.GetReportLineHeight: integer;
  begin
    if Font <> Canvas.Font then
      Canvas.Font.Assign(Font);
    Result := Canvas.TextHeight('ImageLib') + ReportLineSpacing + 3;
  end;

  procedure TILDatasetViewer.SetImageWidth(const Value : integer);
  begin
    if FImageWidth = Value then
      Exit;
    FImageWidth := Value;
  end;

  procedure TILDatasetViewer.SetImageHeight(const Value : integer);
  begin
    if FImageHeight = Value then
      Exit;
    FImageHeight := Value;
  end;

  procedure TILDatasetViewer.SetCaption(const Value : string);
  begin
    if FCaption = Value then
      Exit;
    FCaption := Value;
    Invalidate;
  end;

  procedure TILDatasetViewer.SetCaptionAlignment(const Value : TAlignment);
  begin
    if FCaptionAlignment = Value then
      Exit;
    FCaptionAlignment := Value;
    Invalidate;
  end;

  function TILDatasetViewer.GetImageRowSpacing: integer;
  begin
    Result := FImageRowSpacing + ImageTextSpace;
  end;

  function TILDatasetViewer.GetTopMargin: integer;
  { Top margin for images }
  begin
    if ViewMode = vmImages then
      Result := 10
    else
      Result := 1;
    if Caption <> '' then
    begin
      Canvas.Font.Assign(Font);
      Canvas.Font.Size := Canvas.Font.Size + 2;
      Result := Result + Canvas.TextHeight('X') + 6;
      Canvas.Font.Assign(Font);
    end;
  end;

  function TILDatasetViewer.GetLeftMargin: integer;

  var
    ScrollInfo: TScrollInfo;
    W: integer;
  begin
    if ioWrapPartialVisibles in ImageOptions then
      Result := ImageItemSpacing
    else
    begin
      ScrollInfo.cbSize := SizeOf(ScrollInfo);
      ScrollInfo.fMask := SIF_ALL;
      GetScrollInfo(Handle, SB_HORZ, ScrollInfo);
      if ScrollInfo.nPos > 0 then
      begin
        W := GetColCount * (ImageWidth + ImageItemSpacing);
        Result := ImageItemSpacing - (W - ClientWidth) - ImageItemSpacing;
      end
      else
        Result := ImageItemSpacing;
    end;
  end;

  function TILDatasetViewer.GetRptLeftMargin: integer;
  begin
    Result := 0;
    if CanReportDisplayImages then
      Result := ReportLineHeight * (Ord(FReportImageSize) + 2) + 5
    else if (roShowExpandButtons in FReportOptions) or (UpdateMarkers <> upmNone)
    then
      Result := 20;
  end;

  function TILDatasetViewer.GetRptImageSize: integer;
  begin
    Result := ReportLineHeight * (Ord(FReportImageSize) + 2);
  end;

  function TILDatasetViewer.GetRptVertSpace: integer;
  begin
    Result := ClientHeight - (ReportLineHeight + 2 + GetTopMargin);
  end;

  function TILDatasetViewer.GetImgVertSpace: integer;
  begin
    Result := ClientHeight - GetTopMargin;
  end;

  procedure TILDatasetViewer.SetImageRowSpacing(const Value : integer);
  begin
    if FImageRowSpacing = Value then
      Exit;
    FImageRowSpacing := Value;
  end;

  procedure TILDatasetViewer.SetFirstItemIndex(const Value : integer);

  var
    F, L: integer;
    Val : integer;
  begin
    if FFirstItemIndex = Value then
      Exit;
    Val := Value;
    if Value < 0 then
      Val := 0
    else if Value > (OutputCount - 1) then
      Val := OutputCount - 1;
    if Value < 0 then
      Val := 0;
    if FFirstItemIndex = Val then
      Exit;
    FFirstItemIndex := Val;

    if Table.FSearcher.CanResume then
    begin
      GetVisibles(F, L);
      if L = OutputCount - 1 then
      begin
        if Assigned(Table.DataHandler) and Assigned(Table.DataHandler.FBeforeResume) then
          Table.DataHandler.FBeforeResume(Self);
        Table.GetSearcher(Table).Resume(Table.Dataset);
        if Table.Columns.FSortFields.Count > 0 then
          Table.FDataset.Sort(Table.Columns.FSortFields, Table.FColumns.SortOrdering);
        if Assigned(Table.DataHandler) and Assigned(Table.DataHandler.FAfterResume) then
          Table.DataHandler.FAfterResume(Self);
      end;
    end;
    FScrolling := True; { Reset in DrawReport }
    UpdateScrollPos;
    Invalidate;
  end;

  procedure TILDatasetViewer.SetLeftColumn(const Value : integer);
  begin
    if FLeftColumn = Value then
      Exit;
    FLeftColumn := Value;
    FScrolling := True; { Reset in DrawReport }
    UpdateScrollPos;
    Invalidate;
  end;

  procedure TILDatasetViewer.SetUpdateMarkers(const Value : TUpdateMarkers);
  begin
    if FUpdateMarkers = Value then
      Exit;
    FUpdateMarkers := Value;
    if not(csLoading in ComponentState) then
      Invalidate;
  end;

  procedure TILDatasetViewer.WMHScroll(var Message: TWMHScroll);

  var
    ScrollInfo: TScrollInfo;
    OldPos: integer;
    F, L: integer;
  begin
    inherited;
    if Table = nil then
      Exit;
    if ViewMode = vmReport then
      OldPos := FLeftColumn
    else
      OldPos := FirstItemIndex;
    case Message.ScrollCode of
      SB_ENDSCROLL:
        Exit;
      SB_LINELEFT:
        begin
          if ViewMode = vmReport then
          begin
            if FLeftColumn > 0 then
              LeftColumn := LeftColumn - 1;
          end
          else
          begin
            if FImageLayout = ilGrid then
            begin
              if FImgHScrollPos = 0 then
                Exit;
              FImgHScrollPos := 0;
              UpdateScrollPos;
              Invalidate;
            end
            else
            begin
              FirstItemIndex := FirstItemIndex - 1;
            end;
          end;
        end;
      SB_LINERIGHT:
        begin
          if ViewMode = vmReport then
          begin
            LeftColumn := LeftColumn + 1;
            if FLeftColumn > Table.Columns.Count - 1 then
              LeftColumn := Table.Columns.Count - 1;
          end
          else
          begin
            if FImageLayout = ilGrid then
            begin
              if FImgHScrollPos = 1 then
                Exit;
              FImgHScrollPos := 1;
              UpdateScrollPos;
              Invalidate;
            end
            else
              FirstItemIndex := FirstItemIndex + 1;
          end;
        end;
      SB_PAGELEFT:
        begin
          if ViewMode = vmReport then
          begin
            Table.Columns.GetVisibleColumns(F, L);
            if L = 0 then
              Exit;
            LeftColumn := LeftColumn - L - F - 1;
            if FLeftColumn < 0 then
              LeftColumn := 0;
          end
          else
          begin
            if FImageLayout = ilGrid then
            begin
              if FImgHScrollPos = 0 then
                Exit;
              FImgHScrollPos := 0;
              UpdateScrollPos;
              Invalidate;
            end
            else
              FirstItemIndex := FirstItemIndex - GetColCount;
          end;
        end;
      SB_PAGERIGHT:
        if ViewMode = vmReport then
        begin
          Table.Columns.GetVisibleColumns(F, L);
          LeftColumn := LeftColumn + L - F + 1;
          if FLeftColumn > Table.Columns.Count - 1 then
            LeftColumn := L;
        end
        else
        begin
          if FImageLayout = ilGrid then
          begin
            if FImgHScrollPos = 1 then
              Exit;
            FImgHScrollPos := 1;
            UpdateScrollPos;
            Invalidate;
          end
          else
            FirstItemIndex := FirstItemIndex + GetColCount;
        end;
      SB_THUMBPOSITION:
        ;
      SB_THUMBTRACK:
        begin
          ScrollInfo.cbSize := SizeOf(ScrollInfo);
          ScrollInfo.fMask := SIF_ALL;
          GetScrollInfo(Handle, SB_HORZ, ScrollInfo);
          if ViewMode = vmReport then
          begin
            LeftColumn := ScrollInfo.nTrackPos div 50;
          end
          else
          begin
            if FImageLayout = ilGrid then
            begin
              FImgHScrollPos := ScrollInfo.nTrackPos;
              UpdateScrollPos;
              Invalidate;
            end
            else
              FirstItemIndex := ScrollInfo.nTrackPos;
          end;
        end;
    end;
    if (OldPos <> FirstItemIndex) and (ViewMode <> vmReport) then
    begin
      if FirstItemIndex < 0 then
        FirstItemIndex := 0
      else if FirstItemIndex > Table.Dataset.Count - 1 then
        FirstItemIndex := Table.Dataset.Count - 1;
    end
    else if (OldPos <> FLeftColumn) and (ViewMode = vmReport) then
    begin
      if FLeftColumn < 0 then
        LeftColumn := 0
      else if FLeftColumn > Table.Columns.Count - 1 then
        LeftColumn := Table.Columns.Count - 1;
    end;;
  end;

  procedure TILDatasetViewer.WMVScroll(var Message: TWMVScroll);

  var
    ScrollInfo: TScrollInfo;
    OldPos: integer;
  begin
    inherited;
    if Table = nil then
      Exit;
    OldPos := FirstItemIndex;
    case Message.ScrollCode of
      SB_LINEDOWN:
        if CanScroll(True, SB_VERT) then
          FirstItemIndex := FirstItemIndex + GetColCount;
      SB_LINEUP:
        if CanScroll(false, SB_VERT) then
          FirstItemIndex := FirstItemIndex - GetColCount;
      SB_PAGEDOWN:
        if CanScroll(True, SB_VERT) then
          FirstItemIndex := FirstItemIndex + (GetColCount * GetRowCount);
      SB_PAGEUP:
        if CanScroll(false, SB_VERT) then
          FirstItemIndex := FirstItemIndex - (GetColCount * GetRowCount);
      SB_THUMBPOSITION:
        ;
      SB_THUMBTRACK:
        begin
          ScrollInfo.cbSize := SizeOf(ScrollInfo);
          ScrollInfo.fMask := SIF_TRACKPOS;
          GetScrollInfo(Handle, SB_VERT, ScrollInfo);
          if ViewMode = vmReport then
          begin
            FirstItemIndex := ScrollInfo.nTrackPos;
          end
          else
          begin
            FirstItemIndex := ScrollInfo.nTrackPos * GetColCount;
          end;
        end;
    end;
    if FirstItemIndex <> OldPos then
    begin
      if FirstItemIndex < 0 then
        FirstItemIndex := 0
      else
      begin
        if OutputCount - FirstItemIndex < GetRowCount then
          FirstItemIndex := OutputCount - GetRowCount;
      end;
    end;
  end;

  procedure TILDatasetViewer.WMMouseLeave(var Message: TWMMouse);
  begin
    Inherited;
    if FItemUnderMouse <> -1 then
    begin
      if Assigned(FOnMouseLeaveItem) then
        FOnMouseLeaveItem(Self, FItemUnderMouse);
    end;
    FItemUnderMouse := -1;
    Invalidate;
  end;

  procedure TILDatasetViewer.CMDESIGNHitTest(var Message: TCMDesignHitTest);
  var
    P : TPOint;
  begin
    P.X := Message.Pos.X;
    P.Y := Message.Pos.Y;
    if (Table = nil) or (ViewMode = vmImages)
    or( (FSplitterClicked = -1) and
    (Table.Columns.ColumnSplitterAtXY(P.X, P.Y) = -1)) then
      Message.Result := 0
    else
     Message.Result := HTClient;
  end;

  procedure TILDatasetViewer.WMERASEBKGND(var Msg: TWMEraseBkgnd);
  begin
    Msg.Result := 1;
  end;

  procedure TILDatasetViewer.WMSETFOCUS(var Msg: TWMSetFocus);
  begin
    if Table = nil then
      Exit;
    if TMan = nil then
      Exit;
 end;

  procedure TILDatasetViewer.WMGetDlgCode(var Message: TWMGetDlgCode);
  begin
    inherited;
    Message.Result := Message.Result + DLGC_WANTARROWS + DLGC_WANTTAB;
  end;

  procedure TILDatasetViewer.GetVisibles(var First, Last: integer);

  { Returns the items that are fully visible in a vertical layout }
  var
    ColCnt, RowCnt, Avails: integer;
  begin
    if (Table.Dataset = nil) or (Table.Dataset.Count = 0) then
    begin
      First := -1;
      Last := -1;
      Exit;
    end;
    if (Table.Dataset.Count > 0) and (FirstItemIndex = -1) then
      FirstItemIndex := 0;

    First := FirstItemIndex;


    if ViewMode = vmReport then
    begin
      Last := First + GetRowCount - 1;
      if Last > OutputCount - 1 then
        Last := OutputCount - 1;
      Exit;
    end
    else if (ImageLayout = ilGrid) then
    begin
      ColCnt := GetColCount;
      RowCnt := GetRowCount;
      Avails := ColCnt * RowCnt;
      if First + Avails >= Table.Dataset.Count then
        Last := Table.Dataset.Count - 1
      else
        Last := First + Avails - 1;
    end
    else if ImageLayout = ilStripHorz then
    begin
      Last := First + GetColCount - 1;
    end
    else
    begin
      Last := First + GetRowCount - 1;
    end;
    if Last > OutputCount - 1 then
      Last := OutputCount - 1;
  end;

  function TILDatasetViewer.GetColCount: integer;
  begin
    if (ViewMode = vmReport) or (ImageLayout = ilStripVert) then
      Result := 1
    else
    begin
      Result := (ClientWidth div (ImageWidth + ImageItemSpacing));
      { Max visibles horz }
      if Result = 0 then
        Result := 1;
      if not(ioWrapPartialVisibles in ImageOptions) then
      begin
        if (ClientWidth mod (ImageWidth + ImageItemSpacing)) <> ClientWidth then
          Inc(Result);
      end;
    end;
  end;

  function TILDatasetViewer.GetRowCount: integer;
  var
    Size: integer;
  begin
    if ViewMode = vmReport then
    begin
      Size := GetRptVertSpace;
      Result := Size div ReportLineHeight;
      Exit;
    end;
    if ImageLayout = ilStripHorz then
      Result := 1
    else
    begin
      Result := (GetImgVertSpace div (ImageHeight + GetImageRowSpacing));
      if Result = 0 then
        Result := 1;
      if not(ioWrapPartialVisibles in ImageOptions) then
      begin
        if (GetImgVertSpace mod (ImageHeight + GetImageRowSpacing)) <> GetImgVertSpace
        then
          Inc(Result);
      end;
    end;
  end;

  procedure TILDatasetViewer.SetTable(const Value : TILTable);
  var
    i : integer;
  begin
    if Value = FTable then
      Exit;
    if (Value <> nil) and not (csLoading in ComponentState) and (TMan <> nil) then
    begin
      {if (Value <> nil) and (Value.DataHandler = nil) then
        HandleILException(E_ViewerNoHandler, Value.TableName);}

      for I := 0 to TMan.TableCount-1 do
      begin
        if TMan.Tables[i].HasViewer then
          if TMan.Tables[i].Viewer.Ftable = Value then
            HandleILException(E_ViewerTable);
      end;
    end;

    if (Value <> FTable) and (FTable <> nil) and Assigned(FReport) then
    begin
       FreeAndNil(FReport);
       FreeAndNil(FCellEditor);
    end;

    if FTable <> nil then
      FTable.FViewer := nil;

    FTable := Value;

    if FTable <> nil then
    begin
      FTable.FreeNotification(Self);
      FreeNotification(FTable);
      FTable.FViewer := Self;
    end;

    if FTable <> nil then
    begin
      FCellEditor := TILCellEditor.Create(Self);
      FReport := TILReport.Create(Ftable);
      if not Assigned(FTable.Columns) then
        FTable.FColumns := TILColumns.Create(FTable, Ftable.Fields);
      UpdateScrollrange;
      UpdateScrollPos;
      ScrollIntoView(FTable.Dataset.Position);
      if FTable.Dataset.Position <> -1 then
       FTable.Dataset.RunFocus(Table.Dataset.Position, False);
    end
    else
    begin
      SetScrollRange(Self.Handle, SB_HORZ, 0, 0, false);
      SetScrollRange(Self.Handle, SB_VERT, 0, 0, false);
    end;
    if Assigned(FTable) and Assigned(FTable.Columns) and  (FTable.Columns.Count = 0) then
      FTable.Columns.AssignSource;
    Invalidate;
  end;

  procedure TILDatasetViewer.SetImageLayout(const Value : TImageLayout);
  begin
    if Value = FImageLayout then
      Exit;
    FImageLayout := Value;
    Invalidate;
  end;

  function TILDatasetViewer.ProcessString(const AString: string; const MaxWidth: integer): string;

  var
    ElipsisW: integer;
  begin
    Result := AString;
    if FDrawBM.Canvas.TextWidth(Result) > MaxWidth then
    begin
      ElipsisW := Canvas.TextWidth('...');
      Result := Copy(AString, 1, Length(AString) - 1);
      while FDrawBM.Canvas.TextWidth(Result) + ElipsisW > MaxWidth do
      begin
        Result := Copy(Result, 1, Length(Result) - 1);
        if Result = '' then
          Break;
      end;
      if Result = '' then
        Result := Copy(AString, 1, 2)
      else
        Result := Result + '...'
    end;
  end;

  function TILDatasetViewer.ProcessAlignment(const A : TAlignment; const AString: string;
    const LeftPos : integer; const MaxWidth: integer): integer;
  begin
    if A = taLeftJustify then
    begin
      Result := LeftPos;
      Exit;
    end;
    if Pos('...', AString) <> 0 then
    begin
      Result := LeftPos;
      Exit;
    end;
    if A = taCenter then
    begin
      Result := LeftPos + (MaxWidth div 2) - (Canvas.TextWidth(AString) div 2);
    end
    else
    begin
      Result := LeftPos + MaxWidth - Canvas.TextWidth(AString);
    end;
  end;

  procedure TILDatasetViewer.ReleaseImageMem(ExcludeStart, ExcludeEnd: integer);
  { Releases memory used by images when reaches 10 mb }
  var
    i: integer;
    PP: TPropertyProvider;
    Rec: TILDataRecord;
  begin
    if FImageMem < 10000000 then
      Exit;
    for i := 0 to Table.Dataset.Count - 1 do
    begin
      Rec := Table.Dataset.Items[i];
      if ((i < ExcludeStart) or (i > ExcludeEnd)) and Rec.FImageFetched and
        Assigned(Rec.FImage) and (Rec.RecordIndex <> -1) and not Rec.FImageChanged
      then
      begin
        FreeAndNil(Rec.FImage);
        Rec.FOldImage := nil;

        Table.Dataset.Items[i].FImageFetched := false;
        if Table.Dataset.Items[i].MustStoreLocally(Table.Dataset.Items[i].FileName) then
          FImageMem := FImageMem - Table.Dataset.Items[i].ThumbnailSize
        else
        begin
          PP := TPropertyProvider.Create(Table.Dataset.Items[i].FileName);
          FImageMem := FImageMem - PP.GetPropValueAsVariant(PID_FILE_SIZE);
          PP.Free;
        end;
      end;
    end;
  end;

  procedure TILDatasetViewer.UpdateScrollRange;
  var
    RCnt: integer;
    RptColCount : integer;
  begin
    { Reset range befor computing}
    FSemaphor := True;
    SetScrollRange(Self.Handle, SB_HORZ, 0, 0, false);
    SetScrollRange(Self.Handle, SB_VERT, 0, 0, false);
    FSemaphor := false;
    if Table = nil then
      Exit;
    if FReport = nil then
      Exit;
     if (csDesigning in ComponentState) or (csLoading in ComponentState) or
     not Assigned(Table.Columns) or (Table.Columns.Count = 0) then
       Exit;
    FSemaphor := True;
    { To prevent the proc being retriggered by Resize }
    try
    if ViewMode = vmReport then
    begin
      if CanReportDisplayImages or (UpdateMarkers <> upmNone) or
      (roShowExpandButtons in ReportOptions) then
        RptColCount := Table.Columns.Count + 1
      else
        RptColCount := Table.Columns.Count;

      if OutputCount * ReportLineHeight > GetRptVertSpace then
        SetScrollRange(Self.Handle, SB_VERT, 0, OutputCount, True)
      else
        SetScrollRange(Self.Handle, SB_VERT, 0, 0, False);
      if (Table.Columns.TotalWidth <= ClientWidth) and (FLeftColumn = 0) then
      begin
        SetScrollRange(Self.Handle, SB_HORZ, 0, 0, False);
      end
      else
      begin
        SetScrollRange(Self.Handle, SB_HORZ, 0, RptColCount * 50, True);
      end;
    end
    else if ImageLayout = ilStripHorz then
    begin
      SetScrollRange(Self.Handle, SB_VERT, 0, 0, True);
      SetScrollRange(Self.Handle, SB_HORZ, 0, OutputCount, True);
    end
    else if ImageLayout = ilStripVert then
    begin
      SetScrollRange(Self.Handle, SB_HORZ, 0, 0, True);
      SetScrollRange(Self.Handle, SB_VERT, 0, OutputCount, True);
    end
    else
    begin
      RCnt := OutputCount div ColCount;
      if OutputCount mod ColCount <> 0 then
        Inc(RCnt);
      if (ioWrapPartialVisibles in ImageOptions) then
      begin
        if RCnt * (ImageHeight + GetImageRowSpacing) <= GetImgVertSpace then
          SetScrollRange(Self.Handle, SB_VERT, 0, 0, True)
        else
          SetScrollRange(Self.Handle, SB_VERT, 0, RCnt, True);
        SetScrollRange(Self.Handle, SB_HORZ, 0, 0, True);
      end
      else
      begin
        if RCnt * (ImageHeight + GetImageRowSpacing) <= GetImgVertSpace then
          SetScrollRange(Self.Handle, SB_VERT, 0, 0, True)
        else
          SetScrollRange(Self.Handle, SB_VERT, 0, RCnt, True);
        SetScrollRange(Self.Handle, SB_HORZ, 0, 2, True);
      end;

    end;
    finally
      FSemaphor := false;
    end;
  end;

  procedure TILDatasetViewer.UpdateScrollPos;
  var
    ScrollInfo: TScrollInfo;
    H, hn, W, wn: integer;
  begin
    if Table = nil then
      Exit;
    if FReport = nil then
      Exit;
    if not Assigned(Table.Columns) or (Table.Columns.Count = 0) then
      Exit;

    if (csDesigning in ComponentState) or (csLoading in ComponentState) then
      Exit;
    if Table.Dataset = nil then
      Exit;
    FSemaphor := True;
    { To prevent the proc being retriggered by resize }
    try
    if (Table.Dataset.Count < 2) and (ViewMode = vmImages) then
    begin
      Exit;
    end;

    W := 0; wn := 0;
    H := 0; hn := 0;

    ScrollInfo.cbSize := SizeOf(ScrollInfo);
    ScrollInfo.fMask := SIF_ALL;
    if GetScrollInfo(Handle, SB_HORZ, ScrollInfo) then
    begin
      if (ViewMode = vmReport) then
      begin
        W := 50;
      end
      else if (ImageLayout = ilStripHorz) then
      begin
        W := GetColCount;
      end
      else if (ImageLayout = ilGrid) and not(ioWrapPartialVisibles in ImageOptions)
      then
      begin
        W := 2;
      end
      else
        W := 0;
      wn := ScrollInfo.nPage;
    end
    else
      ShowScrollbar(Handle, SB_HORZ, false);

    if GetScrollInfo(Handle, SB_VERT, ScrollInfo) then
    begin
      if ViewMode = vmReport then
      begin
        if OutputCount > 0 then
        begin
          H := GetRowCount
        end
        else
          H := 0;
      end
      else
      begin
        H := GetRowCount
      end;
      hn := ScrollInfo.nPage;
    end
    else
      ShowScrollbar(Handle, SB_VERT, false);

    ScrollInfo.fMask := SIF_PAGE;
    if (ViewMode = vmReport) then
    begin
      if W <> wn then
      begin
        ScrollInfo.nPage := W;
        SetScrollInfo(Handle, SB_HORZ, ScrollInfo, True);
      end;
      if (H <> hn) and (ReportLineHeight * OutputCount > GetRptVertSpace) then
      begin
        ScrollInfo.nPage := H + 1;
        SetScrollInfo(Handle, SB_VERT, ScrollInfo, True);
      end;
    end
    else if ImageLayout = ilStripHorz then
    begin
      if W <> wn then
      begin
        ScrollInfo.nPage := W;
        SetScrollInfo(Handle, SB_HORZ, ScrollInfo, True);
      end;
    end
    else { Image grid }
    begin
      if H <> hn then
      begin
        ScrollInfo.nPage := GetRowCount + 1;
        SetScrollInfo(Handle, SB_VERT, ScrollInfo, True);
      end;
      if not(ioWrapPartialVisibles in ImageOptions) and
      (W <> wn) then
      begin
        ScrollInfo.nPage := W;
        SetScrollInfo(Handle, SB_HORZ, ScrollInfo, True);
      end;
    end;

    if (ViewMode = vmReport) then
    begin
      if HorzScrollbarVisible(Handle) then
        SetScrollPos(Handle, SB_HORZ, FLeftColumn * 50, True);
      if VertScrollbarVisible(Handle) then
        SetScrollPos(Handle, SB_VERT, FirstItemIndex, True);
      Exit;
    end;
    if ImageLayout = ilStripHorz then
    begin
       if HorzScrollbarVisible(Handle) then
         SetScrollPos(Handle, SB_HORZ, FirstItemIndex, True);
    end
    else if ImageLayout = ilStripVert then
    begin
      if VertScrollbarVisible(Handle) then
        SetScrollPos(Handle, SB_VERT, FirstItemIndex, True);
    end
    else
    begin
      if VertScrollbarVisible(Handle) then
        SetScrollPos(Handle, SB_VERT, FirstItemIndex div GetColCount, True);
      if not(ioWrapPartialVisibles in ImageOptions) then
      begin
        if HorzScrollbarVisible(Handle) then
          SetScrollPos(Handle, SB_HORZ, FImgHScrollPos, True)
      end
    end;
    finally
      FSemaphor := false;
    end;
  end;

  procedure TILDatasetViewer.Resize;
  begin
    if FSemaphor then
    begin
      inherited;
    { Triggered from within teh UpdateScrollpos or UpdateScrollRange procs }
      Exit;
    end;
    UpdateScrollRange;
    UpdateScrollPos;
    inherited;
  end;

  procedure TILDatasetViewer.Loaded;
  begin
     if (FTable <> nil) and (FTable.DataHandler <> nil) then
       FTable.Datahandler.FreeNotification(Self);
     inherited;
  end;


  procedure TILDatasetViewer.Paint;
  begin
    if Table = nil then
    begin
      if TStyleManager.IsCustomStyleActive then
        Canvas.Brush.Color := StyleServices.GetStyleColor(scGrid)
       else
        Canvas.Brush.Color := Color;
      Canvas.FillRect(ClientRect);
      DrawCaption(Canvas);
      Exit;
    end;
    if Table.Connected then
    begin
      if ViewMode in [vmImages] then
        DrawImages
      else
        DrawReport;
      Canvas.Draw(0, 0, FDrawBM);
    end
    else
    begin
      if (ViewMode = vmReport) then
      begin
        if Table.Columns.Count > 0 then
        begin
          DrawReport;
          Canvas.Draw(0, 0, FDrawBM);
        end
        else
        begin
          if TStyleManager.IsCustomStyleActive then
            Canvas.Brush.Color := StyleServices.GetStyleColor(scGrid)
          else
            Canvas.Brush.Color := Color;
          Canvas.FillRect(ClientRect);
           DrawCaption(Canvas);
        end;
      end
      else
      begin
        DrawImages;
        Canvas.Draw(0, 0, FDrawBM);
      end;
    end;
  end;

  procedure TILDatasetViewer.DrawImages;

  var
    i: integer;
    Rec: TILDataRecord;
    X, Y: integer;
    CenterX, CenterY: integer;
    First, Last: integer;
    ItmH, ItmW: integer;
    MaxCols, ColCnt: integer;
    OutImage: Vcl.Graphics.TBitmap;

    procedure Center(TopX, TopY: integer; var cx, cy: integer);

  var
    W, H: integer;
  begin
    if not Assigned(Rec.FImage) then
    begin
      W := ImageWidth;
      H := ImageHeight;
    end
    else
    begin
      W := OutImage.Width;
      H := OutImage.Height;
    end;
    cx := TopX + (ItmW - W) div 2;
    cy := TopY + (ItmH - H) div 2;
  end;

  procedure DrawCheckMark;
  begin
    if not ShowCheckBoxes then
      Exit;
    FDrawBM.Canvas.Brush.Style := bsSolid;
    FDrawBM.Canvas.Pen.Color := clBlack;
    FDrawBM.Canvas.Brush.Color := clWhite;
    FDrawBM.Canvas.Rectangle(X, Y, X + c_CheckBoxWidth, Y + c_CheckBoxWidth);
    if Rec.Selected then
    begin
      if not Assigned(TMan.FCheckmarkBM) then
        TMan.FCheckmarkBM := TMan.LoadResource('Checkmark');
      if TMan.FCheckmarkBM <> nil then
        FDrawBM.Canvas.Draw(X + 2, Y + 2, TMan.FCheckmarkBM);
    end;
  end;

  procedure DrawPlayerIcon;
  var
    XPos, YPos : integer;
    W, H : integer;
  begin
    begin
      if Assigned(TMan.FIntPlayerIcon) then
      begin
       W := Tman.FIntPlayerIcon.Width div 2;
       H := Tman.FIntPlayerIcon.Height div 2;
      end
      else
      begin
       W := Tman.FIntPlayerBM.Width div 2;
       H := Tman.FIntPlayerBM.Height div 2;
      end;

      XPos := X + (ImageWidth div 2) - W;
      YPos := Y + (ImageHeight div 2) - H;
      if Assigned(TMan.FIntPlayerBM) then
        FDrawBM.Canvas.Draw(XPos, YPos, Tman.FIntPlayerBM)
      else
        FDrawBM.Canvas.Draw(XPos, YPos, Tman.FIntPlayerIcon);
    end;
  end;

  procedure DrawChangeMarker;

  var
    Clr: TColor;
    Needs: Boolean;
    R: TRect;
    Handled: Boolean;
    M: TUpdateMarkers;
  begin
    if FUpdateMarkers = upmNone then
      Exit;
    Needs := false;
    Clr := clBtnFace;
    M := upmNone;
    if (FUpdateMarkers in [upmPreviewChange, upmBoth]) and Rec.Selected then
    begin
      Needs := Rec.NeedsUpdate;
      M := upmPreviewChange;
      Clr := clYellow;
    end;

    if not Needs and (FUpdateMarkers in [upmChanged, upmBoth]) then
    begin
      Needs := Rec.Changed;
      M := upmChanged;
      if Rec.Validate(True) = 0 then
        Clr := clLime
      else
        Clr := clRed;
    end;

    if Needs then
    begin
      Handled := false;
      if Assigned(FOnDrawUpdateMarker) then
      begin
        R := GetItemRect(rtItemRect, Rec.Index);
        FOnDrawUpdateMarker(Self, M, FDrawBM.Canvas, R, Handled);
        if Handled then
          Exit;
      end;
      if Assigned(Rec.Image) then
      begin
        R.Left := CenterX + 1;
        R.Top := CenterY + 1;
        R.Bottom := R.Top + Rec.Image.Height - 2;
      end
      else
      begin
        R := GetItemRect(rtItemRect, Rec.Index);
        R.Left := R.Left + 1;
        R.Top := Y + 20;
        R.Bottom := R.Bottom - ImageTextSpace - 1;
      end;
      R.Right := R.Left + 5;
      FDrawBM.Canvas.Brush.Color := Clr;
      FDrawBM.Canvas.FillRect(R);
    end;
  end;

  function DrawImage(Indx: integer): Boolean;
  var
    W, H: integer;
    LeftPos: integer;
    TopPos: integer;
    TextRect: TRect;
    ItmRect: TRect;
    Hot: Boolean;
    Foc: Boolean;
    sl: TStringList;
    i: integer;
    Space: integer;
    S: string;
  begin
    Result := True;
    if not Rec.FImageFetched then
      Rec.FetchImage;
    FDrawBM.Canvas.Pen.Width := 1;
    FDrawBM.Canvas.Pen.Color := GetStyleColor([seBorder], stImageBorder);
    OutImage := Rec.Image;
    try
      if Assigned(Rec.Image) then
      begin
        if not(Rec.Image.Width = ImageWidth) or
          not(Rec.Image.Height = ImageHeight) then
        begin
          OutImage := Vcl.Graphics.TBitmap.Create;
          OutImage.Assign(Rec.Image);
          ScaleBitmap(OutImage, ImageWidth, ImageHeight, false);
        end;
      end;

      Center(X, Y, CenterX, CenterY);
      TextRect := Rect(X, Y + ImageHeight, X + ImageWidth,
        Y + ImageHeight + ImageTextSpace);
      ItmRect := Rect(X, Y, X + ItmW, Y + ItmH);

      FDrawBM.Canvas.Font.Assign(Font);

      if Assigned(FOnDrawImageItem) then
      begin
        FOnDrawImageItem(Self, FDrawBM.Canvas, Rec, ItmRect, TextRect, Result);
        if not Result then
          Exit;
      end;
      Foc := (Indx = Table.Dataset.Position);
      Hot := (HotTrack and (FItemUnderMouse = Indx));
      if Foc and not FocusRectangle then
      begin
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stSelCol)
      end
      else if Hot then
      begin
        if Rec.Selected then
          FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stFocused)
        else
          FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stHotTrack)
      end
      else if Rec.Selected then
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stFocused)
      else
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stBackground);

      if Rec.Selected or Hot or Foc then
      begin
        FDrawBM.Canvas.Brush.Style := bsSolid;
        FDrawBM.Canvas.FillRect(Rect(X, Y, X + ItmW,
          Y + ItmH + ImageTextSpace));
      end;
      FDrawBM.Canvas.Brush.Style := bsClear;
      if not Assigned(Rec.FImage) then
      begin
        FDrawBM.Canvas.Rectangle(ItmRect);
      end
      else
      begin
        if Rec.IsDeleted then
          DrawAlphaBlend(OutImage.Canvas.Handle, Rect(0,0,OutImage.Width,OutImage.Height) , 180, clWhite);
        FDrawBM.Canvas.Draw(CenterX, CenterY, OutImage);
      end;

      if Rec.IsDeleted and not Assigned(Rec.FImage) then
      begin
        FDrawBM.Canvas.MoveTo(ItmRect.Left, ItmRect.Top);
        FDrawBM.Canvas.LineTo(ItmRect.Right, ItmRect.Bottom);
        FDrawBM.Canvas.MoveTo(ItmRect.Right, ItmRect.Top);
        FDrawBM.Canvas.LineTo(ItmRect.Left, ItmRect.Bottom);
      end;

      if (ImageTextSpace > 0) and (Rec.ImageText <> '') then
      begin
        FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFont);
        W := FDrawBM.Canvas.TextWidth(Rec.ImageText);
        H := FDrawBM.Canvas.TextHeight(Rec.ImageText);
        Space := H;
        if W > ImageWidth then
          LeftPos := X + 1
        else
          LeftPos := X + (ImageWidth - W) div 2;
        TopPos := Y + ImageHeight + (RectHeight(TextRect) - H) div 2;

        sl := TStringList.Create;
        sl.Text := Rec.ImageText;
        try
          if sl.Count = 1 then
          begin
            S := ProcessString(Rec.ImageText, ImageWidth);
            FDrawBM.Canvas.TextRect(TextRect, LeftPos, TopPos - 3, S);
          end
          else
            for i := 0 to sl.Count - 1 do
            begin
              S := ProcessString(sl[i], ImageWidth);
              FDrawBM.Canvas.TextRect(TextRect, LeftPos, TextRect.Top, S);
              Inc(TextRect.Bottom, H + 3);
              Inc(TextRect.Top, H + 3);
              Inc(Space, H + 3);
              if Space > ImageTextSpace then
                Break;
            end;
        finally
          sl.Free;
        end;
      end;
      if Assigned(Rec.Image) then
      begin
        FDrawBM.Canvas.Rectangle(CenterX, CenterY, CenterX + OutImage.Width,
          CenterY + OutImage.Height);
      end;
      if (Indx = Table.Dataset.Position) or Rec.Selected or Hot then
      begin
        FDrawBM.Canvas.Pen.Width := 1;
        if Indx = Table.Dataset.Position then
        begin
          if (Table = TMan.ActiveTable) or Table.IsFreeTable then
            FDrawBM.Canvas.Pen.Style := psSolid
          else
            FDrawBM.Canvas.Pen.Style := psDot;
          FDrawBM.Canvas.Pen.Color := clRed;
        end
        else if Rec.Selected then
          FDrawBM.Canvas.Pen.Color := GetStyleColor([seClient], stFocused);
        if ((Indx = Table.Dataset.Position) or Rec.Selected) and FocusRectangle then
          FDrawBM.Canvas.Rectangle(X - 2, Y - 2, X + ItmW + 2,
            Y + ItmH + ImageTextSpace + 2)
        else if Hot then
        begin
          FDrawBM.Canvas.Pen.Color := GetStyleColor([seClient], stHotTrack);
          FDrawBM.Canvas.Rectangle(X - 1, Y - 1, X + ItmW + 1,
            Y + ItmH + ImageTextSpace + 1);
        end;
      end;
      FDrawBM.Canvas.Pen.Style := psSolid;
    finally
      if (OutImage <> nil) and (OutImage <> Rec.Image) then
        OutImage.Free;
    end;

    if Assigned(FAfterDrawImageItem) then
      FAfterDrawImageItem(Self, FDrawBM.Canvas, Rec, ItmRect, TextRect);
  end;

  begin
    ItmH := ImageHeight;
    ItmW := ImageWidth;
    FDrawBM.Width := ClientWidth;
    FDrawBM.Height := ClientHeight;
    FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stBackground);
    FDrawBM.Canvas.Brush.Style := bsSolid;
    FDrawBM.Canvas.FillRect(ClientRect);
    if Caption <> '' then
      DrawCaption(FDrawBM.Canvas);
    if (csDesigning in ComponentState) then
      Exit;
    GetVisibles(First, Last);
    if First = -1 then
      Exit;
    X := GetLeftMargin;
    Y := GetTopMargin;
    if (ImageLayout = ilStripHorz) then
    begin
      for i := First to Last do { The last visible item }
      begin
        Rec := Table.Dataset.Items[i];
        if DrawImage(i) then
        begin
          DrawCheckMark;
          if (Rec.MediaType in [MED_VIDEO, MED_AUDIO])
           and (Assigned(TMan.FIntPlayerIcon) or Assigned(Tman.FIntPlayerBM)) then
            DrawPlayerIcon;
          DrawChangeMarker;
        end;
        Inc(X, ImageWidth + ImageItemSpacing);
      end;
    end
    else if ImageLayout = ilStripVert then
    begin
      for i := First to Last do { The last visible item }
      begin
        Rec := Table.Dataset.Items[i];
        if DrawImage(i) then
        begin
          DrawCheckMark;
          if (Rec.MediaType in [MED_VIDEO, MED_AUDIO])
           and (Assigned(TMan.FIntPlayerIcon) or Assigned(Tman.FIntPlayerBM)) then
            DrawPlayerIcon;
          DrawChangeMarker;
        end;
        Inc(Y, ImageHeight + GetImageRowSpacing);
      end;
    end
    else { ilGrid }
    begin
      MaxCols := GetColCount;
      ColCnt := 1;
      for i := First to Last do { The last visible item }
      begin
        Rec := Table.Dataset.Items[i];
        if ColCnt > MaxCols then
        begin
          Inc(Y, ImageHeight + GetImageRowSpacing);
          X := GetLeftMargin;
          ColCnt := 1;
        end;
        if DrawImage(i) then
        begin
          DrawCheckMark;
          if (Rec.MediaType in [MED_VIDEO, MED_AUDIO])
          and (Assigned(TMan.FIntPlayerIcon) or Assigned(Tman.FIntPlayerBM)) then
            DrawPlayerIcon;
          DrawChangeMarker;
        end;
        Inc(X, ImageWidth + ImageItemSpacing);
        Inc(ColCnt);
      end;
    end;

    ReleaseImageMem(First, Last);
  end;

  procedure TILDatasetViewer.DrawReport;

  const
    Margin = 6;

  var
    i, j: integer;
    F, L: integer;
    FI, Li: integer;
    H, W: integer;
    FldDef: TILFieldDef;
    R: TRect;
    X, Y: integer;
    S: string;
    TxtH: integer;
    TxtY: integer;
    Clr: TColor;
    M: integer;
    Pt: TPoint;
    FirstY: integer;
    C: TILReportRow;
    Accepted: Boolean;
    HotTracking: Boolean;
    Rec: TILDataRecord;
    RecordChanged: Boolean;
    FocRect: TRect;
    ExpRects: array of TRect;
    FixedFontClr, FixedColClr : Boolean;
    LTID: integer;
    BM: Vcl.Graphics.TBitmap;
    SImgValue: string;
    D : TDateTime;
    Cnv : TCanvas;

    procedure DrawDivideLine(X, Y: integer);
    begin
      Clr := GridlineColor;
      FDrawBM.Canvas.Pen.Color := Clr;
      FDrawBM.Canvas.MoveTo(X, Y);
      FDrawBM.Canvas.LineTo(X, Y + H);
      FDrawBM.Canvas.MoveTo(X - W - FColumnSpacing, Y + H );
      FDrawBM.Canvas.LineTo(X + W + FColumnSpacing, Y + H);
      FDrawBM.Canvas.Pen.Color := clBlack;
    end;

    procedure ResetCanvas;
    begin
      FDrawBM.Canvas.Pen.Color := clBlack;
      FDrawBM.Canvas.Brush.Style := bsSolid;
      FDrawBM.Canvas.Brush.Color := Color;
      FDrawBM.Canvas.Font.Assign(Font);
    end;

    procedure DrawCheckBox(X, Y: integer; Sel: Boolean; Rec: TILDataRecord);

  var
    BG: TColor;
  begin
    if not ShowCheckBoxes then
      Exit;
    BG := FDrawBM.Canvas.Brush.Color;
    FDrawBM.Canvas.Brush.Color := clWhite;
    FDrawBM.Canvas.Pen.Color := clGray;
    TxtY := Y + (ReportLineHeight - c_CheckBoxWidth) div 2 - 1;
    FDrawBM.Canvas.Rectangle(Rect(X + 3, TxtY, X + 3 + c_CheckBoxWidth,
      TxtY + c_CheckBoxWidth));
    if Sel then
    begin
      if not Assigned(TMan.FCheckmarkBM) then
       TMan.FCheckmarkBM := TMan.LoadResource('Checkmark');
      if TMan.FCheckmarkBM <> nil then
        FDrawBM.Canvas.Draw(X + 5, TxtY + 2, TMan.FCheckmarkBM);
    end;
    FDrawBM.Canvas.Brush.Color := BG;
  end;


  procedure DrawExpandButton(Y: integer; C: TILReportRow);

  var
    X: integer;
    S: string;
    N: string;
    Sz: integer;
  begin
    if not(roShowExpandButtons in ReportOptions) or CanReportDisplayImages
    then
      Exit;
    Y := Y + 2;
    X := 6;
    S := '';
    N := FDrawBM.Canvas.Font.Name;
    Sz := FDrawBM.Canvas.Font.Size;
    FDrawBM.Canvas.Font.Size := 10;
    FDrawBM.Canvas.Font.Name := 'Courier New';
    if C.Expanded then
      S := '-'
    else if C.CanExpand then
      S := '+';
    if not FixedFontClr then
      FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFont);
    FDrawBM.Canvas.TextOut(X, Y, S);
    FDrawBM.Canvas.Font.Name := N;
    FDrawBM.Canvas.Font.Size := Sz;
  end;

  procedure DrawSelected(Indx: integer);

  var
    R: TRect;
  begin
    FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stFocused);
    R := FReport.Items[Indx].ItemRect;
    R.Top := R.Top;
    R.Left := GetRptLeftMargin + 1;
    R.Right := R.Right - 1;
    FDrawBM.Canvas.FillRect(R);
  end;

  procedure DrawHotTrack(Indx: integer; Sel: Boolean);

  var
    R: TRect;
  begin
    if not FHotTrack then
      Exit;
    if not FixedFontClr and not FixedColClr then
      FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFont);
    if Sel then
      FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stFocused)
    else
      FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stHotTrack);
    R := FReport.Items[Indx].ItemRect;
    R.Left := GetRptLeftMargin;
    R.Top := R.Top + 1;
    R.Left := GetRptLeftMargin + 1;
    R.Right := R.Right - 1;
    FDrawBM.Canvas.FillRect(R);
  end;

  procedure DrawFocusBackground(Indx: integer);
  begin
    FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stSelCol);
    R := FReport.Items[Indx].ItemRect;
    R.Top := R.Top;
    R.Left := GetRptLeftMargin + 1;
    R.Right := R.Right - 1;
    FDrawBM.Canvas.FillRect(R);
  end;

  procedure DrawFocusRect(Yp: integer; ItmIndx: integer; Collect: Boolean);

  var
    i: integer;
    Lm, TM: integer;
    DrawTop, DrawLeft: Boolean;

    procedure DrawRect(R: TRect);

  var
    R2: TRect;
  begin
    if DrawTop and DrawLeft then
    begin
      inc(R.Top);
      FDrawBM.Canvas.Rectangle(R);
    end
    else
    begin
      R2 := Table.Columns.RectFromLeftColumn(0);
      if DrawTop then
      begin
        FDrawBM.Canvas.MoveTo(Lm, R.Top);
        FDrawBM.Canvas.LineTo(R2.Right, R.Top);
      end;
      { Always draw bottom. Not visible if outside client rect }
      FDrawBM.Canvas.MoveTo(Lm, R.Bottom);
      FDrawBM.Canvas.LineTo(R2.Right, R.Bottom);
      if DrawLeft then
      begin
        FDrawBM.Canvas.MoveTo(Lm, TM);
        FDrawBM.Canvas.LineTo(Lm, R.Bottom);
      end;
      { Always draw right. Not visible if outside client rect }
      if R.Top < TM then
        FDrawBM.Canvas.MoveTo(R2.Right, TM)
      else
        FDrawBM.Canvas.MoveTo(R2.Right, R.Top);
      FDrawBM.Canvas.LineTo(R2.Right, R.Bottom);
    end;
  end;

  begin
    if Collect then
    { We collect the rectangles within the loop, draw
      when loop is finished to put them on top }
    begin
      R := FReport.ExpansionRect[ItmIndx];
      if C.DatasetIndex = Table.Dataset.Position then
        FocRect := R;
      Inc(R.Left, 1);
      Inc(R.Top, 1);
      Dec(R.Right, 1);
      Dec(R.Bottom, 1);
      if (C.DatasetIndex = Table.Dataset.Position) then
      begin
        SetLength(ExpRects, Length(ExpRects) + 1);
        ExpRects[High(ExpRects)] := R;
      end
      else if FReport.Expanded(C.DatasetIndex) then
      begin
        SetLength(ExpRects, Length(ExpRects) + 1);
        ExpRects[High(ExpRects)] := R;
      end;

      Exit;
    end;

    Lm := GetRptLeftMargin;
    TM := GetTopMargin + ReportLineHeight + 2;

    FDrawBM.Canvas.Pen.Width := 1;
    FDrawBM.Canvas.Brush.Style := bsClear;

    if (roOutlineExpansion in ReportOptions) then
    begin
      FDrawBM.Canvas.Pen.Color := clGray;
      for i := 0 to High(ExpRects) do
      begin
        DrawTop := True;
        DrawLeft := True;
        if Abs(ExpRects[i].Top - FocRect.Top) > 2 then
        begin
          if (ExpRects[i].Left < Lm) and (Lm > 0) and (FLeftColumn > 0) then
            DrawLeft := false;
          if (ExpRects[i].Top < TM) then
            DrawTop := false;
          DrawRect(ExpRects[i])
        end;
      end;
    end;

    DrawTop := True;
    DrawLeft := True;
    if (FocRect.Width > 0) then
    begin
      if (FocRect.Left < Lm) and (Lm > 0) and (FLeftColumn > 0) then
        DrawLeft := false;
      if (FocRect.Top < TM) then
        DrawTop := false;

      if FocusRectangle then
      begin
        if (Table = TMan.ActiveTable) or (Table.IsFreeTable and not (Table.MasterLinkCount > 0)) then
          FDrawBM.Canvas.Pen.Style := psSolid
        else
          FDrawBM.Canvas.Pen.Style := psDot;
        FDrawBM.Canvas.Pen.Color := clRed;
        DrawRect(FocRect);
        FDrawBM.Canvas.Pen.Style := psSolid;
      end;
    end;
  end;

  function RecordUnderMouse(ItmIndx: integer): integer;
  begin
    if ItmIndx = -1 then
      Result := -1
    else
      Result := FReport.Items[ItmIndx].DatasetIndex;
  end;

  procedure DrawGridLine(X, Y: integer; C: TILReportRow; Indx: integer);

  { x, y = upper left of cell }
  var
    W: integer;
  begin
    FDrawBM.Canvas.Pen.Color := GridlineColor;
    FDrawBM.Canvas.MoveTo(X, Y);
    FDrawBM.Canvas.LineTo(X, Y + ReportLineHeight);
    W := C.Items[Indx].FFieldDef.ColumnWidth;
    if (C.Items[Indx].FFieldDef.DataType <> dtText) or
      ((C.Items[Indx].FFieldDef.DataType = dtText) and
      (C.ExpansionIndex = C.ExpansionCount)) then
    begin
      FDrawBM.Canvas.LineTo(X + W + FColumnSpacing, FDrawBM.Canvas.PenPos.Y);
    end;
    if Indx = C.Count - 1 then
    begin
      FDrawBM.Canvas.MoveTo(X + W + FColumnSpacing, Y + ReportLineHeight);
      FDrawBM.Canvas.LineTo(X + W + FColumnSpacing, Y - ReportLineHeight);
    end;

  end;

  procedure DrawGridlines;

  var
    i, j: integer;
    X, Y: integer;
    R: TRect;
  begin
    if not(roGridlines in ReportOptions) then
      Exit;
    Y := ReportLineHeight * 2 + 2 + GetTopMargin;
    FDrawBM.Canvas.Pen.Color := GridlineColor;
    R := Table.Columns.RectFromLeftColumn(0);
    if csDesigning in ComponentState then
    begin
      for i := 0 to GetRowCount do
      begin
        FDrawBM.Canvas.MoveTo(0, Y);
        FDrawBM.Canvas.LineTo(R.Right, Y);
        Inc(Y, ReportLineHeight);
      end;
      Y := ReportLineHeight + 2;
      X := GetRptLeftMargin;
      FDrawBM.Canvas.MoveTo(X, Y);
      FDrawBM.Canvas.LineTo(X, ClientHeight);
      X := Table.Columns[FLeftColumn].ColumnWidth + FColumnSpacing +
        GetRptLeftMargin;
      for i := FLeftColumn + 1 to Table.Columns.Count - 1 do
      begin
        FDrawBM.Canvas.MoveTo(X, Y);
        FDrawBM.Canvas.LineTo(X, ClientHeight);
        Inc(X, Table.Columns[i].ColumnWidth + FColumnSpacing);
      end;
      FDrawBM.Canvas.MoveTo(X, Y);
      FDrawBM.Canvas.LineTo(X, ClientHeight);
      Exit;
    end;

    Y := ReportLineHeight + 2 + GetTopMargin;
    X := GetRptLeftMargin;
    for i := F to L do
    begin
      C := FReport.Items[i];
      for j := FI to Li do
      begin
        DrawGridLine(X, Y, C, j);
        Inc(X, C.Items[j].FFieldDef.ColumnWidth + FColumnSpacing);
      end;
      Inc(Y, ReportLineHeight);
      X := GetRptLeftMargin;
    end;
  end;

  procedure DrawHoverImage;

  var
    Pt: TPoint;
    C: TILReportRow;
    Indx: integer;
    BM: Vcl.Graphics.TBitmap;
    R: TRect;
    Cl : TILReportCell;
  begin
    if not(ReportImages = riHover) and  not(ReportImages = riHoverCtrl)then
      Exit;
    if (ReportImages = riHoverCtrl) and not FHoverCtrl then
      Exit;

    BM := nil;
    Pt := ScreenToClient(Point(Mouse.CursorPos.X, Mouse.CursorPos.Y));
    Cl := Report.CellAtXY(Pt.X, Pt.Y);
    if Cl <> nil then
    begin
      BM := Cl.Image;
    end;
    try
     Indx := Report.RecordIndexToRowIndex(FItemUnderMouse);
     C := FReport.Items[Indx];
     if C.Expanded then
       C := FReport.Items[Indx - C.ExpansionIndex];
     if BM = nil then
     begin
       if not Assigned(C.Data.Image) and not C.Data.FImageFetched then
       begin
         C.Data.FetchImage;
       end;
       if Assigned(C.Data.Image)  then
       begin
         BM := VCL.Graphics.TBitmap.Create;
         BM.Assign(C.Data.Image);
       end;
     end;

     if Assigned(BM) then
     begin
        if (BM.Width <> 120) or (BM.Height <> 120) then
          ScaleBitmap(BM, 120, 120);
        R := C.ItemRect;
        Pt.X := GetRptLeftMargin;
        if R.Top - BM.Height < GetTopMargin then
          Pt.Y := R.Bottom + 1
        else
          Pt.Y := R.Top - BM.Height - 1;
       FDrawBM.Canvas.Draw(Pt.X, Pt.Y, BM);
     end;
    finally
      if Assigned(BM) then
        BM.Free;
    end;

  end;

  procedure DrawEmbededImage(C: TILReportRow); { The first in the record }

  var
    BM: Vcl.Graphics.TBitmap;
    R: TRect;
    Sz: integer;
  begin
    if not CanReportDisplayImages then
      Exit;
    if not Assigned(C.Data.Image) and not C.Data.FImageFetched then
    begin
      C.Data.FetchImage;
    end;
    Sz := GetRptImageSize;
    if Assigned(C.Data.Image) then
    begin
      BM := Vcl.Graphics.TBitmap.Create;
      try
        BM.Assign(C.Data.Image);
        ScaleBitmap(BM, Sz - 1, Sz - 1, false);
        R := C.ImageRect;
        FDrawBM.Canvas.Draw(R.Left, R.Top, BM);
      finally
        BM.Free;
      end;
    end
    else
    begin
      R := C.ImageRect;
      R.Left := R.Left + 2;
      R.Width := Sz;
      R.Height := Sz - 1;
      FDrawBM.Canvas.Brush.Style := bsClear;
      FDrawBM.Canvas.Pen.Color := GetStyleColor([seBorder], stGridlines);
      FDrawBM.Canvas.Rectangle(R);
    end;
  end;

  procedure DrawSelectedColumn(C: TILReportCell);
  begin
    FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stHotCol);
    if not FixedFontClr and not FixedColClr then
      FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFontHot);
    R := C.ItemRect;
    if (C.Index = 0) and ShowCheckBoxes then
    begin
      R.Left := R.Left + c_CheckBoxWidth + 3;
    end;
    FDrawBM.Canvas.FillRect(R);
  end;

  procedure DrawHotColumn(C: TILReportCell);

  var
    Pt: TPoint;
    R: TRect;
    Indx: integer;
    Rec: TILDataRecord;
    MaxVal: integer;
  begin
    Rec := C.Owner.Data;
    if C.FieldDef.MaxValues = 0 then
      MaxVal := MaxInt
    else
      MaxVal := C.FieldDef.MaxValues;
    Indx := Rec.IndexOf(C.FieldDef.FieldName);
    if (C.IsEmpty and (C.Owner.ExpansionIndex > Rec[Indx].Count)) or
      (MaxVal = C.Owner.ExpansionIndex) then
      Exit;
    if C.FEditing then
      Exit;
    R := C.ItemRect;
    Pt := ScreenToClient(Point(Mouse.CursorPos.X, Mouse.CursorPos.Y));
    if PtInRect(R, Pt) then
    begin
      if not FixedFontClr then
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stHotCol);
      if not FixedColClr then
        FDrawBM.Canvas.Font.Color := clBlack;

      if (FLeftColumn = 0) and (C.Index = FLeftColumn) and
        (C.Owner.ExpansionIndex = 0) then
      begin
        R.Left := R.Left + Margin + c_CheckBoxWidth;
      end;
      FDrawBM.Canvas.FillRect(R);
    end;
  end;

  procedure UpdateEditCtrl(C: TILReportCell);

  var
    R, R2: TRect;
    Pt : TPoint;
  begin
    R := C.ItemRect;
    if FCellEditor.ActiveCtrl = FCellEditor.FMemoCtrl then
    begin
      if (C.FieldDef.DataType = dtText) and C.Owner.Expanded then
      begin
        R2 := C.Owner.OwnerReport.ExpansionRect[C.Owner.Index];
        R.Top := R2.Top;
        R.Bottom := R2.Bottom;
        FCellEditor.ActiveCtrl.Height := R2.Height;
      end;
    end;
      Pt := ClientToScreen(POint(R.Left, R.Top));
      Pt := Screen.ActiveForm.ScreenToClient(Pt);

    FCellEditor.ActiveCtrl.Left := Pt.X;
    FCellEditor.ActiveCtrl.Top:= Pt.Y;
    FCellEditor.ActiveCtrl.Width := R.Width;
  end;

  procedure DrawImageColumn;
  begin
    if CanReportDisplayImages then
    begin
      TxtY := GetTopMargin + ((H - TxtH) div 2) - 1;
      R := Rect(0, GetTopMargin, GetRptLeftMargin, GetTopMargin + H);
      FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient],
        stInactiveColHeading);
      FDrawBM.Canvas.FillRect(R);
      FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFontCaption);
      FDrawBM.Canvas.TextOut(Margin, TxtY, Translator.GetTranslation(T_Image,
        TRANSL_TERMS));
      DrawDivideLine(GetRptLeftMargin, GetTopMargin);
    end;
  end;

  procedure DrawColumnHeaders;

  var
    i: integer;
    R2: TRect;
    M : integer;
    Indx : integer;
  begin
    { Draw the column headers; }
    TxtH := FDrawBM.Canvas.TextHeight('ImageLibrary');
    H := ReportLineHeight;
    Y := GetTopMargin;
    X := GetRptLeftMargin;
    TxtY := Y + ((H - TxtH) div 2) - 1;

    DrawImageColumn;

    R := Table.Columns.RectFromLeftColumn(Y);
    Inc(R.Bottom);

    for i := FLeftColumn to Table.Columns.Count - 1 do
    begin
      ResetCanvas;
      M := Margin;
      S := '';
      if not Table.HasHandler then
        Indx := -1
      else
        Indx := Table.DataHandler.DataFields.IndexOf(Table.Columns[i].FieldName);
      if Indx <> -1 then
        S := Table.DataHandler.DataFields.Items[Indx].Caption;
      if S = '' then
      begin
       if (Table.Columns[i].Caption <> '') then
         S := Table.Columns[i].Caption
       else
        S := Table.Columns[i].FFieldName;
      end;
      W := Table.Columns[i].ColumnWidth;
      S := ProcessString(S, W - M);
      M := ProcessAlignment(Table.Columns[i].Alignment, S, M, W - M);

      FDrawBM.Canvas.Font.Style := [];
      FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFontCaption);
      if roColumnClickSort in FReportOptions then
      begin
        if FColumnClicked <> -1 then
          R2 := Table.Columns.ColumnHeaderRect[FColumnClicked];
        Pt := ScreenToClient(Point(Mouse.CursorPos.X, Mouse.CursorPos.Y));
        if (FColumnClicked = -1) or
          ((FColumnClicked <> -1) and not PtInRect(R2, Pt)) then
        { Col not clicked or Col clicked and mouse outside col }
        begin
          Indx := Table.Columns.ColumnHeaderAtXY(Pt.X, Pt.Y);
          if (Indx = i) and (Table.Columns.ColumnSplitterAtXY(Pt.X, Pt.Y) = -1) then { Mouse over col, but not down }
          begin
            FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stActiveColHeading);
            FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFontCaption);
            R := Table.Columns.ColumnHeaderRect[Indx];
            Inc(R.Left);
            GradientFillCanvas(FDrawBM.Canvas, FDrawBM.Canvas.Brush.Color,
             GetGradEndColor(FDrawBM.Canvas.Brush.Color), R, gdVertical);
            FDrawBM.Canvas.Brush.Style := bsClear;
            FDrawBM.Canvas.Pen.Color := clGray;
            FDrawBM.Canvas.Rectangle(R);
            FDrawBM.Canvas.TextOut(X + M, TxtY, S);
            FDrawBM.Canvas.Brush.Style := bsSolid;
          end
          else
          begin { Not mouse over, may be down, may be not }
            FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stInactiveColHeading);
            R := Table.Columns.ColumnHeaderRect[i];
            Inc(R.Left);
            inc(R.Bottom);
            GradientFillCanvas(FDrawBM.Canvas, FDrawBM.Canvas.Brush.Color,
             GetGradEndColor(FDrawBM.Canvas.Brush.Color), R, gdVertical);
            FDrawBM.Canvas.Brush.Style := bsClear;
            FDrawBM.Canvas.TextOut(X + M, TxtY, S);
            FDrawBM.Canvas.Brush.Style := bsSolid;
          end;
        end
        else
        begin
          Indx := Table.Columns.IndexOf(Table.Columns[FColumnClicked]
            .FieldName);
          if Indx = i then { Button down over col }
          begin
            R := Table.Columns.ColumnHeaderRect[FColumnClicked];
            Inc(R.Left);
            FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stColPressed);
            FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFontHot);
            GradientFillCanvas(FDrawBM.Canvas, FDrawBM.Canvas.Brush.Color,
              GetGradEndColor(FDrawBM.Canvas.Brush.Color), R, gdVertical);
            FDrawBM.Canvas.Brush.Style := bsClear;
            FDrawBM.Canvas.Pen.Color := clGray;
            FDrawBM.Canvas.Rectangle(R);
            FDrawBM.Canvas.Pen.Color := clBtnFace;
            FDrawBM.Canvas.TextOut(X + M + 1, TxtY + 1, S);
            FDrawBM.Canvas.Brush.Style := bsSolid;
          end
          else
          begin { Not mouse over }
            FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient],
              stInactiveColHeading);
            R := Table.Columns.ColumnHeaderRect[i];
            Inc(R.Left);
            GradientFillCanvas(FDrawBM.Canvas, FDrawBM.Canvas.Brush.Color,
             GetGradEndColor(FDrawBM.Canvas.Brush.Color), R, gdVertical);
            FDrawBM.Canvas.Brush.Style := bsClear;
            FDrawBM.Canvas.TextOut(X + M, TxtY, S);
            FDrawBM.Canvas.Brush.Style := bsSolid;
          end;
        end
      end
      else
      begin { Static, not clickable }
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient],
          stInactiveColHeading);
        R := Table.Columns.ColumnHeaderRect[i];
        Inc(R.Left);
        GradientFillCanvas(FDrawBM.Canvas, FDrawBM.Canvas.Brush.Color,
         GetGradEndColor(FDrawBM.Canvas.Brush.Color), R, gdVertical);
        FDrawBM.Canvas.Brush.Style := bsClear;
        FDrawBM.Canvas.TextOut(X + M, TxtY, S);
        FDrawBM.Canvas.Brush.Style := bsSolid;
      end;
      DrawDivideLine(X, Y);
      Inc(X, W + FColumnSpacing);
    end;

  end;

  function SetMarkerColor(RowIndx: integer): Boolean;

  var
    Needs: Boolean;
    R: TRect;
    Handled: Boolean;
    Cansv : Integer;

  begin
    Result := false;
    Needs := false;
    Handled := false;
    if CanReportDisplayImages then
      R := GetItemRect(rtImageRect, RowIndx)
    else
      R := GetItemRect(rtExpandBtnRect, RowIndx);
    if (FUpdateMarkers in [upmPreviewChange, upmBoth]) and Rec.Selected then
    begin
      Needs := Rec.NeedsUpdate;
      if Needs then
      begin
        if Assigned(FOnDrawUpdateMarker) then
        begin
          FOnDrawUpdateMarker(Self, upmPreviewChange, FDrawBM.Canvas,
            R, Handled);
          if not Handled then
            FDrawBM.Canvas.Brush.Color := clYellow
          else
            Exit;
        end
        else
          FDrawBM.Canvas.Brush.Color := clYellow;
      end;
    end;

    if not Needs and (FUpdateMarkers in [upmChanged, upmBoth]) then
    begin
      Needs := Rec.Changed;
      if Needs then
      begin
        Cansv := Rec.Validate(True);
        if Assigned(FOnDrawUpdateMarker) then
        begin
          FOnDrawUpdateMarker(Self, upmChanged, FDrawBM.Canvas, R, Handled);
          if not Handled then
          begin
            if CanSv = 0 then
              FDrawBM.Canvas.Brush.Color := clLime
            else
              FDrawBM.Canvas.Brush.Color := clRed;
          end
          else
            Exit;
        end
        else if CanSv = 0 then
          FDrawBM.Canvas.Brush.Color := clLime
        else
          FDrawBM.Canvas.Brush.Color := clRed;
      end;
    end;
    if Needs then
    begin
      if CanReportDisplayImages then
      begin
         R.Left := R.Right -5;
         Inc(R.Top,2);
         Dec(R.Bottom, 2);
      end
      else
      begin
        Inc(R.Top,2);
        Dec(R.Bottom, 2);
        Inc(R.Left, 2);
        Dec(R.Right, 2);
      end;

      FDrawBM.Canvas.FillRect(R);
    end;
    Result := Needs;
  end;

  begin
    FScrolling := false;
    FDrawBM.Width := ClientWidth;
    FDrawBM.Height := ClientHeight;
    FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stBackground);
    FDrawBM.Canvas.Brush.Style := bsSolid;
    FDrawBM.Canvas.FillRect(ClientRect);
    FDrawBM.Canvas.Font.Assign(Font);
    if (csDesigning in ComponentState) then
    begin
      DrawCaption(FDrawBM.Canvas);
    end;
    if Table.Columns.Count = 0 then
      Exit;

    DrawColumnHeaders;

    { Draw the data }
    if (csDesigning in ComponentState) then
    begin
      DrawGridlines;
      Exit;
    end;

    FocRect := Rect(0, 0, 0, 0);

    if Table.Dataset.Count = 0 then
    begin
      DrawCaption(FDrawBM.Canvas);
      Exit;
    end;
    GetVisibles(F, L);
    if F = -1 then
    begin
      DrawCaption(FDrawBM.Canvas);
      Exit;
    end;
    ResetCanvas;
    Y := ReportLineHeight + 2 + GetTopMargin;
    Rec := nil;
    for i := F to L do
    begin
      HotTracking := false;
      FixedFontClr := false;
      C := FReport.Items[i];
      if C.Data <> Rec then
      begin
        Rec := C.Data;
        RecordChanged := True
      end
      else
        RecordChanged := false;
      if C.IsDeleted then
      begin
        FDrawBM.Canvas.Font.Color := clSilver;
        FixedFontClr := True;
      end
      else
        FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFont);

      X := GetRptLeftMargin;

      if (Table.Dataset.Position = C.DatasetIndex) and not FocusRectangle then
      begin
        DrawFocusBackground(i);
      end
      else if (FItemUnderMouse = C.DatasetIndex) and HotTrack and
        ((Table.Dataset.Position <> C.DatasetIndex) or not(roEditable in ReportOptions))
      then { Dont hottrack a focused record unless not edidtable }
      begin
        DrawHotTrack(i, Table.Dataset.Items[C.DatasetIndex].Selected);
        if not FixedFontClr and not FixedColClr then
          FDrawBM.Canvas.Font.Color := clBlack; { Always black on silver }
        HotTracking := True;
      end
      else if Table.Dataset.Items[C.DatasetIndex].Selected then
      begin
        DrawSelected(i);
      end
      else
      begin
        FDrawBM.Canvas.Brush.Color := GetStyleColor([seClient], stBackground);
      end;

      if (FLeftColumn = 0) and (C.ExpansionIndex = 0) then
        DrawCheckBox(X, Y, C.Data.Selected, C.Data);

      if (C.ExpansionIndex = 0) then
        DrawExpandButton(Y, C);

      if Assigned(FOnDrawReportRow) then
      begin
        Accepted := True;
        Clr := FDrawBM.Canvas.Font.Color;
        FOnDrawReportRow(Self, FDrawBM.Canvas, C, Accepted);
        if not Accepted then
        begin
          if RecordChanged then
            DrawEmbededImage(C);
          Inc(Y, H);
          Continue;
        end
        else if Clr <> FDrawBM.Canvas.Font.Color then
          FixedFontClr := True;

      end;

      FI := FReport.FirstVisibleColumn;
      if (FCellEditor.FCell <> nil) and (FCellEditor.FCell.FEditing) then
        Li := Table.Columns.Count -
          1 { In order to reposition the edit control }
      else
        Li := FReport.LastVisibleColumn;

      for j := FI to Li do
      begin
        FixedColClr := false;
        if (j = FLeftColumn) and (FLeftColumn = 0) and ShowCheckBoxes then
          M := Margin + 3 + c_CheckBoxWidth { Checkbox margin }
        else
          M := Margin;
        if C.Items[j].FEditing then
          UpdateEditCtrl(C.Items[j]);
        FldDef := C.Items[j].FieldDef;
        W := FldDef.ColumnWidth;
        S := C.Items[j].Value;
        FDrawBM.Canvas.Font.Style := [];
        if C.Items[j].FieldDef.DataType = dtBoolean then
        begin
          if S = '0' then
            S := TMan.ReportFalse
          else if S = '1' then
            S := TMan.ReportTrue
          else
            S := '';
        end
        else if C.Items[j].FieldDef.DataType in [dtResource] then
        begin
          S := GetResourceDisplayElement(S);
          FDrawBM.Canvas.Font.Color := clBlue;
          FDrawBM.Canvas.Font.Style := [fsUnderline];
        end;

        SImgValue := S;
        if Assigned(FOnDrawReportCell) or Assigned(FOnDrawReportCellEx) then
        begin
          if Assigned(FOnDrawReportCell) then
          begin
            FOnDrawReportCell(Self, FDrawBM.Canvas.Font, C.Items[j], S);
            FixedColClr := (Font.Color <> FDrawBM.Canvas.Font.Color);
          end;
        end
        else if (FldDef.DataType = dtDate) and (FldDef.DateFormat <> '') and (S <> '') then
        begin
          D := StrToDateTime(S, Translator.FFormatSettings);
          S := FormatDateTime(FldDef.DateFormat, D);
        end;
        if C.IsDeleted then
        begin
          FDrawBM.Canvas.Font.Color := clSilver;
          FixedColClr := false;
        end;

        if (C.Items[j].FFieldDef.LookupTable is TILImageTable) and
          (roShowFieldThumbnails in ReportOptions) then
        begin
          S := ProcessString(S, W - M - H - 2);
          M := ProcessAlignment(FldDef.Alignment, S, M, W - M - H - 2);
        end
        else
        begin
          S := ProcessString(S, W - M);
          M := ProcessAlignment(FldDef.Alignment, S, M, W - M);
        end;
        TxtY := Y + (ReportLineHeight - TxtH) div 2 - 1;
        Clr := FDrawBM.Canvas.Brush.Color;
        if (roEditable in ReportOptions) and C.Items[j].Selected then
          DrawSelectedColumn(C.Items[j])
        else if HotTrack and (Table.Dataset.Position = C.DatasetIndex) and
          (roEditable in ReportOptions) then
          { Only hottrack columns on focused record }
          DrawHotColumn(C.Items[j]);
        LTID := -1;
        if (roShowFieldThumbnails in ReportOptions) and
          (C.Items[j].FFieldDef.LookupTable is TILImageTable) and
          (SImgValue <> '') and not C.Items[j].FEditing then
        begin
          LTID := StrToInt(Table.GetLookupTableID(C.Items[j].FFieldDef.LookupTable,
            C.Items[j].FFieldDef.LookupField, SImgValue));
          BM := TILImageTable(C.Items[j].FFieldDef.LookupTable).GetBitmap(LTID, H);
          FDrawBM.Canvas.Draw(X, Y + (H-BM.Height) div 2, BM);
          BM.Free;
          Inc(X, H - 2);
        end;
        if not C.Items[j].FEditing then
        begin
          if (C.Data[C.Data.IndexOf(C.Items[j].FFieldDef.FieldName)].Count > 1)
            and not C.Expanded then
          begin
            FDrawBM.Canvas.TextOut(X + M, TxtY, S + ' * ');
          end
          else
            FDrawBM.Canvas.TextOut(X + M, TxtY, S);
        end;
        FDrawBM.Canvas.Brush.Color := Clr;
        if not HotTracking and not FixedFontClr then
        begin
          FDrawBM.Canvas.Font.Color := GetStyleColor([seFont], stFont);
        end;

        if Assigned(FOnDrawReportCellEx) then
        begin
            Cnv := TCanvas.Create;
            Cnv.Brush.Assign(FDrawBM.Canvas.Brush);
            Cnv.Font.Assign(FDrawBM.Canvas.Font);
            Cnv.Pen.Assign(FDrawBM.Canvas.Pen);
            try
              FOnDrawReportCellEx(Self, FDrawBM.Canvas, C.Items[j], S);
            finally
              FDrawBM.Canvas.Brush.Assign(Cnv.Brush);
              FDrawBM.Canvas.Font.Assign(Cnv.Font);
              FDrawBM.Canvas.Pen.Assign(Cnv.Pen);
              Cnv.Free;
            end;
        end;

        if LTID <> -1 then
          Dec(X, H - 2);
        Inc(X, W + FColumnSpacing);
      end;

      Inc(Y, H);
      FirstY := Y;
      if (Table.Dataset.Position = C.DatasetIndex) or (C.ExpansionCount > 0) then
      begin
        if RecordChanged and (Table.Dataset.Position = C.DatasetIndex) or
          ((Table.Dataset.Position <> C.DatasetIndex) and
          (roOutlineExpansion in ReportOptions)) then
          DrawFocusRect(FirstY - H, i, True);
      end;
      if RecordChanged then
        DrawEmbededImage(C);
      if (UpdateMarkers in [upmPreviewChange, upmChanged, upmBoth]) and RecordChanged
      then
        SetMarkerColor(i);
    end;

    DrawGridlines;
    DrawFocusRect(0, 0, false);
    DrawCaption(FDrawBM.Canvas);
    if ((ReportImages = riHover) or (ReportImages = riHoverCtrl)) and (FItemUnderMouse <> -1) then
    begin
      DrawHoverImage;
    end;
    if CanReportDisplayImages then
    begin
      if Report.Items[FFirstItemIndex].ExpansionIndex > 0 then
        DrawImageColumn;
    end;
  end;

  procedure TILDatasetViewer.DrawCaption(Cnv: TCanvas);

  var
    R: TRect;
    Pt: TPoint;
  begin
    if Caption = '' then
      Exit;
    R.Left := 0;
    R.Top := 0;
    R.Width := Width;
    if (Table = TMan.ActiveTable) then
      Cnv.Brush.Color := GetStyleColor([seClient], stCaption)
    else
      Cnv.Brush.Color := GetStyleColor([seClient], stInactiveCaption);
    Cnv.Font.Assign(Font);
    Cnv.Font.Size := Font.Size + 2;
    R.Height := Cnv.TextHeight('X') + 6;
    GradientFillCanvas(Cnv, Cnv.Brush.Color, GetGradEndColor(Cnv.Brush.Color), R, gdVertical);
    Pt := CenterText(Caption, FDrawBM.Canvas, R);
    if CaptionAlignment = taLeftJustify then
      Pt.X := 5
    else if CaptionAlignment = taRightJustify then
      Pt.X := Width - Cnv.TextWidth(Caption) - 5;

    if Table = TMan.ActiveTable then
       Cnv.Font.Color := clHighLightText
    else
      Cnv.Font.Color := GetStyleColor([seFont], stFontCaption);
    Cnv.Brush.Style := bsClear;
    Cnv.TextOut(Pt.X, Pt.Y, Caption);
    Cnv.Font.Assign(Font);
  end;

  procedure TILDatasetViewer.CreateParams(var Params: TCreateParams);
  begin
    inherited CreateParams(Params);
    if not(csDesigning in ComponentState) then
    begin
      Params.Style := Params.Style or WS_HScroll or WS_VScroll;
    end;
    Params.Style := Params.Style or ws_Border;
  end;

  procedure TILDatasetViewer.Notification(AComponent: TComponent;
    Operation: TOperation);
  begin
    inherited Notification(AComponent, Operation);
    if csDestroying in ComponentState then
      Exit;
    if Tman = nil then
      Exit;
    if Operation = opRemove then
    begin
      if (AComponent = FTable) or ((FTable <> nil) and (AComponent = FTable.DataHandler)) then
      begin
        Table := nil;
      end
      else if AComponent = ZoomWindow then
      begin
        FZoomWindow := nil;
        FZoomImage := nil;
      end;

    end;
  end;


  procedure TILDatasetViewer.KeyDown(var Key: Word; Shift: TShiftState);
  var
    CF, Cl: integer;
    Pt: TPoint;
    Indx, RIndx: integer;
    F, L: integer;
    WS : Widestring;
  begin
    inherited KeyDown(Key, Shift);
    if Table = nil then
      Exit;
    if Tman = nil then
      Exit;
    if Table.NoOutput then
      Exit;
    if (Key = vk_Control)
    and (TMan.FTableUnderMouse <> nil)
    and (TMan.FTableUnderMouse.FViewer.ReportImages = riHoverCtrl) and (Shift = [ssCtrl]) then
    begin
      TMan.FTableUnderMouse.FViewer.FHoverCtrl := True;
      TMan.FTableUnderMouse.Viewer.Invalidate;
    end;

    if (ViewMode = vmReport) and (FCellUnderMouse <> nil) and
      (Shift = [ssAlt, ssCtrl]) and
      (FCellUnderMouse.FieldDef.DataType in [dtResource]) then
    begin
      Cursor := crHandPoint;
    end;

    { Check expanding report rows }
    if (ViewMode = vmReport) and (Key = VK_Space) and not CanReportDisplayImages
    then
    begin
      Pt := ScreenToClient(Point(Mouse.CursorPos.X, Mouse.CursorPos.Y));
      Indx := FReport.RowAtXY(Pt.X, Pt.Y);
      if Indx <> -1 then
      begin
        RIndx := FReport.Items[Indx].DatasetIndex;
        if FReport.Items[Indx].Expanded then
          FReport.Collapse(RIndx)
        else
          FReport.Expand(RIndx);
      end;
      Exit;
    end
    { Check cycling report cells }
    else if (ViewMode = vmReport) and ((Key = vk_Tab) or (Key = VK_Right) or
      (Key = VK_Left)) and (roEditable in ReportOptions) and
      Assigned(Report.SelectedCell) and not Report.SelectedCell.FEditing then
    begin
      if Key = vk_Tab then
      begin
        if Shift = [] then
        begin
          FCellEditor.ProcessTab(True);
        end
        else if Shift = [ssShift] then
        begin
          FCellEditor.ProcessTab(false);
        end;
      end
      else if Key = VK_Right then
        FCellEditor.ProcessTab(True)
      else
        FCellEditor.ProcessTab(false);
      Exit;
    end
    { Check entering edit mode }
    else if (ViewMode = vmReport) and (roEditable in ReportOptions) and
      Assigned(Report.SelectedCell) and not Report.SelectedCell.FEditing then
    begin
      if (Key = vk_Return) then
      begin
        FCellEditor.Edit;
        Exit;
      end
      else if Char(Key).IsLetterOrDigit and (Shift = []) then
      begin
        if FCellEditor.Cell.FFieldDef.LookupType = looTableKey then
        begin
           Exit;
        end;
        FCellEditor.Edit;
        if FCellEditor.ActiveCtrl = FCellEditor.FComboCtrl then
        begin
          WS := VKeyToWidestring(Key);
          FCellEditor.FComboCtrl.Text := WS;
          FCellEditor.FComboCtrl.OnChange(Self);
          if FCellEditor.FComboCtrl.SelLength = 0 then
            FCellEditor.FComboCtrl.Selstart := 1
          else
          begin
            FCellEditor.FComboCtrl.Selstart := 1;
            FCellEditor.FComboCtrl.SelLength :=
              Length(FCellEditor.FComboCtrl.Text) - 1;
          end;
        end
        else if FCellEditor.ActiveCtrl = FCellEditor.FMemoCtrl then
        begin
          WS := VKeyToWidestring(Key);
          FCellEditor.FMemoCtrl.Text := WS;
          FCellEditor.FMemoCtrl.OnChange(Self);
          if FCellEditor.FMemoCtrl.SelLength = 0 then
            FCellEditor.FMemoCtrl.Selstart := 1
          else
          begin
            FCellEditor.FMemoCtrl.Selstart := 1;
            FCellEditor.FMemoCtrl.SelLength :=
              Length(FCellEditor.FMemoCtrl.Text) - 1;
          end;
        end
        else if FCellEditor.ActiveCtrl = FCellEditor.FEditCtrl then
        begin
          WS := VKeyToWidestring(Key);
          FCellEditor.FEditCtrl.Text := WS;
          FCellEditor.FEditCtrl.OnChange(Self);
          if FCellEditor.FEditCtrl.SelLength = 0 then
            FCellEditor.FEditCtrl.Selstart := 1
          else
          begin
            FCellEditor.FEditCtrl.Selstart := 1;
            FCellEditor.FEditCtrl.SelLength :=
              Length(FCellEditor.FEditCtrl.Text) - 1;
          end;
        end;
        Exit;
      end;
    end;

    { Vertical scrolling is taken care of in separate procs.
      The simpler horz scrolling takes place here. }

    GetVisibles(F, L);
    if F = -1 then
      Exit;
    if ViewMode = vmReport then
    begin
      Table.Columns.GetVisibleColumns(CF, Cl);
      L := FReport.Items[L].DatasetIndex;
    end;
    if Key = VK_Next then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_PAGEDOWN, 0)
      else
        ScrollImages(SB_PAGEDOWN, 0);
    end

    else if Key = VK_Prior then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_PAGEUP, 0)
      else
        ScrollImages(SB_PAGEUP, 0);
    end

    else if Key = vk_Down then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_LINEDOWN, 0)
      else
        ScrollImages(SB_LINEDOWN, 0)
    end

    else if Key = vk_Up then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_LINEUP, 0)
      else
        ScrollImages(SB_LINEUP, 0)
    end

    else if Key = VK_Left then
    begin
      if (ViewMode = vmReport) and (CF > 0) then
      begin
        LeftColumn := LeftColumn - 1;
      end
      else
      begin
        if Table.Dataset.Position = 0 then
          Exit;
        if ImageLayout in [ilStripHorz, ilGrid] then
          Table.Dataset.Position := Table.Dataset.Position - 1;
        if Table.Dataset.Position < F then
        begin
          if ImageLayout in [ilStripVert, ilGrid] then
            FirstItemIndex := FirstItemIndex - GetColCount
          else
            FirstItemIndex := FirstItemIndex - 1;
        end;
        if not IsImageFullyVisible(Table.Dataset.Position) then
        begin
          if FImgHScrollPos = 0 then
            FImgHScrollPos := 1
          else
            FImgHScrollPos := 0;
          UpdateScrollPos;
          Invalidate;
        end;
      end;
    end

    else if Key = VK_Right then
    begin
      if (ViewMode = vmReport) and (Cl < Table.Columns.Count - 1) then
      begin
        LeftColumn := LeftColumn + 1;
      end
      else
      begin
        if Table.Dataset.Position = Table.Dataset.Count - 1 then
          Exit;
        if ImageLayout in [ilStripHorz, ilGrid] then
          Table.Dataset.Position := Table.Dataset.Position + 1;
        if FirstItemIndex > Table.Dataset.Count - 1 then
          FirstItemIndex := Table.Dataset.Count - 1;
        if (Table.Dataset.Position > L) and (ImageLayout in [ilStripVert, ilGrid])
        then
          FirstItemIndex := FirstItemIndex + GetColCount
        else if (Table.Dataset.Position > L) and (ImageLayout = ilStripHorz) then
          FirstItemIndex := FirstItemIndex + 1;

        if not IsImageFullyVisible(Table.Dataset.Position) then
        begin
          if FImgHScrollPos = 0 then
            FImgHScrollPos := 1
          else
            FImgHScrollPos := 0;
          UpdateScrollPos;
          Invalidate;
        end;
      end;
    end

    else if Key = VK_Home then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_TOP, 0)
      else
        ScrollImages(SB_TOP, 0);
    end

    else if Key = VK_End then
    begin
      if ViewMode = vmReport then
        ScrollReport(SB_BOTTOM, 0)
      else
        ScrollImages(SB_BOTTOM, 0);
    end;
  end;

  procedure TILDatasetViewer.KeyUp(var Key: Word; Shift: TShiftState);
  begin
    if Table = nil then
    begin
      inherited KeyUp(Key, Shift);
      Exit;
    end;
    if (Key = vk_Control) and
    (TMan <> nil) and (TMan.FTableUnderMouse <> nil)
    and (TMan.FTableUnderMouse.FViewer.ReportImages = riHoverCtrl) and not (ssCtrl in Shift) then
    begin
      TMan.FTableUnderMouse.FViewer.FHoverCtrl := False;
      TMan.FTableUnderMouse.Viewer.Invalidate;
    end;
    Cursor := crDefault;
    inherited KeyUp(Key, Shift);
  end;

  procedure TILDatasetViewer.MouseDown(Button: TMouseButton; Shift: TShiftState;
    X, Y: integer);

  var
    Indx: integer;
    R: TRect;
    N: integer;
    First, Last: integer;
    i: integer;
    Sel: Boolean;
    ColumnHit: Boolean;
    SplitterHit: Boolean;
    RecordIndx: integer;
    RptCell: TILReportCell;
    RptRow : TILReportRow;

    procedure GetRange;

  var
    i: integer;
    LeftIndx, RightIndx: integer;
    RightRange, LeftRange: integer;
    InSelection: Boolean;
  begin
    First := RecordIndx;
    Last := RecordIndx;
    if Table.Dataset.SelCount <= 1 then
    begin
      if Table.Dataset.SelCount = 1 then
        N := Table.Dataset.IndexOf(Table.Selection[0])
      else
        N := Table.Dataset.Position;
      if N < RecordIndx then
      begin
        First := N;
        Last := RecordIndx;
      end
      else if N > RecordIndx then
      begin
        First := RecordIndx;
        Last := N;
      end;
      Exit;
    end;
    N := Table.Dataset.IndexOf(Table.Selection[0]); { Get first selected Item }
    LeftIndx := -1;
    RightIndx := -1;
    InSelection := false;
    for i := RecordIndx downto 0 do
    begin
      if Table.Dataset.Items[i].Selected then
      begin
        LeftIndx := i;
        InSelection := True;
      end
      else if InSelection then
      begin
        LeftIndx := i + 1;
        Break;
      end;
    end;
    InSelection := false;
    for i := RecordIndx to Table.Dataset.Count - 1 do
    begin
      if Table.Dataset.Items[i].Selected then
      begin
        RightIndx := i;
        InSelection := True;
      end
      else if InSelection then
      begin
        RightIndx := i - 1;
        Break;
      end;
    end;
    if LeftIndx <> -1 then
      LeftRange := RecordIndx - LeftIndx
    else
      LeftRange := -1;
    if RightIndx <> -1 then
      RightRange := RightIndx - RecordIndx
    else
      RightRange := -1;

    if RightRange > LeftRange then
    begin
      First := RecordIndx;
      Last := RightIndx
    end
    else if LeftRange > RightRange then
    begin
      First := LeftIndx;
      Last := RecordIndx;
    end;
    if (RightIndx = -1) and (LeftIndx = -1) then
      First := -1;
  end;

  procedure ExecRes;
  var
    res: integer;
    Handled: Boolean;
  begin
    if RptCell = nil then
      Exit;
    if not(RptCell.FieldDef.DataType in [dtResource]) then
      Exit;
    Handled := false;
    if Assigned(FOnExecResource) then
    begin
      FOnExecResource(Self, RptCell.Value, Handled);
    end;
    if not Handled then
    begin
      res := ExecuteResource(RptCell.Value);
      if res <> 0 then
        HandleILException(0, GetLastErrorText);
    end;
  end;

  procedure SetColumnState(FocusChanged: Boolean);

  var
    Indx: integer;
    Rec: TILDataRecord;
    MaxVal: integer;
    R : TRect;
  begin
    if RptCell = nil then
      Exit;
    if not(roEditable in ReportOptions) or Table.Dataset.IsReadOnly then
      Exit;
    if Report.SelectedCell <> nil then
    begin
      if (Report.SelectedCell <> RptCell) and (Report.CanDeleteSelected = 0)
      and (Report.SelectedCell.IsEmpty) then
      begin
        Report.DeleteSelected;
        Exit;
      end;
    end;
    if RptCell.FieldDef.MaxValues = 0 then
      MaxVal := MaxInt
    else
      MaxVal := RptCell.FieldDef.MaxValues;
    Rec := RptCell.Owner.Data;
    Indx := Rec.IndexOf(RptCell.FieldDef.FieldName);
    if (RptCell.Value.IsEmpty and (RptCell.Owner.ExpansionIndex >
      Rec[Indx].Count)) or (MaxVal = RptCell.Owner.ExpansionIndex) then
      Exit;
    if not RptCell.Selected then
    begin
      FCellEditor.FMouseMoved := True;
      try
        FReport.SelectedCell := RptCell;
      finally
        FCellEditor.FMouseMoved := false;
      end;
    end
    else if not FocusChanged then
    begin
      R := RptCell.ImageRect;
      if not PtInRect(R, Point(X,Y)) then
       FCellEditor.Edit;
    end;
  end;

  procedure SetPos;
  begin
    if (Table.Dataset.Position = RecordIndx) and (ViewMode = vmReport) then
      SetColumnState(false)
    else
    begin
      Table.Dataset.Position := RecordIndx;
      SetColumnState(True);
    end;
  end;

  begin
    inherited MouseDown(Button, Shift, X, Y);
    if Table = nil then
      Exit;
    if Caption <> '' then
    begin
      SetFocus;
      R := GetItemRect(rtCaptionrect, 0);
      if PtInRect(R, Point(X,Y)) then
      begin
       if Assigned(FOnClickCaption) then
        FOnClickCaption(Self);
        Exit;
      end;

    end;
    if Table.NoOutput then
      Exit;
    ColumnHit := false;
    RptCell := nil;
    SplitterHit := false;
    SetFocus;
    if Button <> mbLeft then
      Exit;
    Indx := ItemAtXY(X, Y);

    if Indx = -1 then
    begin
      if ViewMode = vmReport then
      begin
        Indx := Table.Columns.ColumnSplitterAtXY(X, Y);
        if Indx <> -1 then
          SplitterHit := True
        else
        begin
          Indx := Table.Columns.ColumnHeaderAtXY(X, Y);
          if Indx <> -1 then
            ColumnHit := True
          else
            Exit;
        end;
      end
      else
        Exit;
    end;

    if (ViewMode = vmReport) and not SplitterHit and not ColumnHit then
    begin
      RecordIndx := FReport.Items[Indx].DatasetIndex;
      R := FReport.Items[Indx].ExpandBtnRect;
      if PtInRect(R, Point(X, Y)) and (roShowExpandButtons in ReportOptions)
      then
      begin
        if FReport.Items[Indx].Expanded then
          FReport.Collapse(RecordIndx)
        else if FReport.Items[Indx].CanExpand then
          FReport.Expand(RecordIndx);
        Exit;
      end;
      RptCell := FReport.CellAtXY(X, Y);
      if RptCell <> nil then
      begin
       R := GetItemRect(rtCheckRect, Indx);
       if PtInRect(R, Point(X, Y)) then
          RptCell := nil;
      end;
    end
    else
    begin
      RecordIndx := Indx;
    end;

    if (ViewMode = vmImages) and (roEditable in ReportOptions) and
      (Table.Dataset.Position = RecordIndx) then
    begin
      R := GetItemRect(rtTextRect, Indx);
      if PtInRect(R, Point(X, Y)) then
      begin
        Table.Dataset.Items[RecordIndx].EditImageText;
        Exit;
      end;
    end;

    if (Shift = [ssLeft, ssShift]) and Table.FMultiSelect and (Table = TMan.ActiveTable) then
    { Range select. Works like this:
      The nearest selected item becomes the start/end index to Indx. If there is no selected item
      the current position becoms the start/end index. If the indx item is selected, any selection
      outside the range is deleted before the selection is set. If the indx item is not selected
      the range selection, if any,  is cancelled, except the index item }
    begin
      GetRange; { If click inside a range First and Last will be the same }
      if First <> -1 then
      begin
        if First <> Last then
        begin
          Table.DoDeselectAll;
          { Select the range }
          for i := First to Last do
            Table.Dataset.Items[i].FSelected := True;
          Table.Dataset.FSelstate := -1;
        end
        else
        begin
          Table.DoDeselectAll; { Deselect all except the index item }
          Table.Dataset.Items[Indx].FSelected := True;
          Table.Dataset.FSelstate := -1;
        end;
        if Assigned(Table.DataHandler) and Assigned(Table.DataHandler.FOnSelect) then
          Table.DataHandler.FOnSelect(Self, -1);
        Invalidate;
      end;
      Exit;
    end;

    if Shift = [ssLeft, ssCtrl, ssShift] then
    begin
      Table.ResetDataControls;
      Table.DoDeselectAll;
    end;

    if (Shift = [ssLeft, ssCtrl]) and Table.FMultiSelect then
    begin
      Sel := Table.Dataset.Items[RecordIndx].Selected;
      Table.Dataset.Items[RecordIndx].Selected := not Sel;
    end
    else if (Shift = [ssCtrl, ssAlt, ssLeft]) and (ViewMode = vmReport) then
      ExecRes
    else if Shift = [ssLeft] then
    begin
      if ColumnHit then
      begin
        FColumnClicked := Indx;
        Invalidate;
        Exit;
      end
      else if SplitterHit then
      begin
        FSplitterClicked := Indx;
        Invalidate;
        Exit;
      end;

      if not ShowCheckBoxes and (Table.Dataset.Position <> RecordIndx) then
      begin
        if RptCell <> nil then
          SetColumnState(True);
        Table.Dataset.Position := RecordIndx;
        Invalidate;
        Exit;
      end;

      if RptCell <> nil then
      begin
        SetColumnState(False);
        if Assigned(RptCell) then
        if not RptCell.Editing and Assigned(FOnClickCell) then
        begin
          R := RptCell.ImageRect;
          if PtInRect(R, Point(X,Y)) then
            FOnClickCell(Self, RptCell, True)
          else
            FOnClickCell(Self, RptCell, False);
        end;
        Table.Dataset.Position := RecordIndx;
        Invalidate;
        Exit;
      end;

      R := GetItemRect(rtCheckRect, Indx);
      if not Table.FMultiSelect then
      begin
        if PtInRect(R, Point(X, Y)) and (Table.Dataset.Position = RecordIndx) then
          Exit;
      end;

      Sel := Table.Dataset.Items[RecordIndx].Selected;
      if Table.FMultiSelect and ((ViewMode = vmReport) and
        (FReport.Items[Indx].ExpansionIndex = 0)) or (ViewMode = vmImages) then
      begin
        if PtInRect(R, Point(X, Y)) then
        begin
          Table.Dataset.Items[RecordIndx].Selected := not Sel;
        end
        else
        begin
          SetPos;
        end;
      end
      else
      begin
        SetPos;
      end;
    end;
    if Assigned(RptCell) then
    if not RptCell.Editing and Assigned(FOnClickCell) then
    begin
        R := RptCell.ImageRect;
        if PtInRect(R, Point(X,Y)) then
          FOnClickCell(Self, RptCell, True)
        else
          FOnClickCell(Self, RptCell, False);
    end;
    if (Shift = [ssLeft]) and (Assigned(FOnClickImage) or Assigned(Tman.FOnClickPlayerIcon)) then
    begin
       if (ViewMode = vmReport) and Assigned(FOnClickImage) then
       begin
         if CanReportDisplayImages then
         begin
          Indx := Report.RowAtXY(X,Y);
          if Indx = -1 then
            Exit;
          RptRow := Report.Items[Indx];
          Indx := RptRow.Data.Index;
          R := GetItemRect(rtImageRect, Indx);
          if PtInRect(R, Point(X,Y)) then
            FOnClickImage(Self, Table.Dataset.Items[Indx], X, Y);
         end;
       end
       else
       begin
         if Assigned(FOnClickImage) then
         begin
           R := GetItemRect(rtItemRect, Indx);
           if PtInRect(R, Point(X,Y)) then
             FOnClickImage(Self, Table.Dataset.Items[Indx], X, Y);
         end;
         if Assigned(TMan.FOnClickPlayerIcon) then
         begin
           R := GetItemRect(rtPlayerIconRect, Indx);
           if PtInRect(R, Point(X,Y)) then
             TMan.FOnClickPlayerIcon(Self, Table.Dataset.Items[Indx], X, Y);
         end;
       end;
    end;
    Invalidate;
  end;

  procedure TILDatasetViewer.MouseUp(Button: TMouseButton; Shift: TShiftState;
    X, Y: integer);
  var
    R: TRect;
  begin
    inherited;
    if Table = nil then
      Exit;
    if ViewMode <> vmReport then
      Exit;
    if Table.NoOutput then
      Exit;
    if FColumnClicked <> -1 then
    begin
      R := Table.Columns.ColumnHeaderRect[FColumnClicked];
      if roColumnClickSort in FReportOptions then
      begin
        if PtInRect(R, Point(X, Y)) then
          ColumnClick(FColumnClicked);
      end;
    end;
    FSplitterClicked := -1;
    FColumnClicked := -1;
    Invalidate;
  end;

  procedure TILDatasetViewer.MouseMove(Shift: TShiftState; X, Y: integer);

  var
    Indx: integer;
    ItemEnteredOrLeft: Boolean;
    C: TILReportCell;
    R: TRect;
    CellChanged: Boolean;
    Cur : HCURSOR;

  procedure SetColWidths;
  var
    R: TRect;
    Dist: integer;
    DefLeft: TILFieldDef;
    Indx : integer;
  begin
    Cur := Screen.Cursors[crHSplit];
    SetCursor(Cur);
    DefLeft := Table.Columns[FSplitterClicked];
    R := Table.Columns.ColumnHeaderRect[FSplitterClicked];
    Dist := X - R.Right; { Column at left grows if positive }
    if (DefLeft.FColumnWidth + Dist < 20) then
      Exit;
    DefLeft.FColumnWidth := DefLeft.FColumnWidth + Dist;
    Indx := Table.DataHandler.DataFields.IndexOf(DefLeft.FieldName);
    if Indx <> -1 then
       Table.DataHandler.DataFields.Items[Indx].FColumnWidth := DefLeft.ColumnWidth;
    Invalidate;
  end;

  begin
    inherited MouseMove(Shift, X, Y);
    if Dragging then
      Exit;
    if Table = nil then
      Exit;
    if TMan <> nil then
     TMan.FTableUnderMouse := Table;
    FHoverCtrl := (Shift = [ssCtrl]) and (ReportImages = riHoverCtrl)
      and (Table.Dataset.Count > 0);
    if FHoverCtrl then
      Invalidate;

    if Table.NoOutput then
      Exit;
    if FColumnClicked <> -1 then
    begin
      R := Table.Columns.RectFromLeftColumn(0);
      InvalidateRect(Handle, @R, false);
      Exit;
    end;
    if FSplitterClicked <> -1 then
    begin
      SetColWidths;
      UpdateScrollRange;
      UpdateScrollPos;
      FCellUnderMouse := nil;
      Exit;
    end;
    C := nil;
    CellChanged := false;
    ItemEnteredOrLeft := false;
    Indx := ItemAtXY(X, Y);
    if (ViewMode = vmReport) and (Indx <> -1) then
      Indx := FReport.Items[Indx].DatasetIndex;

    if Indx <> FItemUnderMouse then
    begin
      if FItemUnderMouse <> -1 then
      begin
        if Assigned(FOnMouseLeaveItem) then
          FOnMouseLeaveItem(Self, FItemUnderMouse);
        ItemEnteredOrLeft := True;
      end;
      if Indx <> -1 then
      begin
        if Assigned(FOnMouseEnterItem) then
          FOnMouseEnterItem(Self, Indx);
        ItemEnteredOrLeft := True;
      end;
      FItemUnderMouse := Indx;
      Cursor := crDefault;
    end;
    if (ViewMode = vmReport) and ((Shift = []) or (Shift = [ssAlt, ssCtrl]))
    then
    begin
      begin
        Indx := Table.Columns.ColumnSplitterAtXY(X, Y);
        if Indx <> -1 then
        begin
          Cur := Screen.Cursors[crHSplit];
          SetCursor(Cur);
        end
        else if Shift <> [ssAlt, ssCtrl] then
        begin
          Cur := Screen.Cursors[crDefault];
          SetCursor(Cur);
        end;
        if Indx = -1 then
          Indx := Table.Columns.ColumnHeaderAtXY(X, Y);
        if ((Indx <> -1) and (roColumnClickSort in FReportOptions)) then
        begin
          Invalidate;
          FHighLightColumn := Indx;
        end
        else if (roColumnClickSort in FReportOptions) then
        begin
          FHighLightColumn := -1;
          Invalidate;
        end;
      end;
      CellChanged := false;
      C := FReport.CellAtXY(X, Y);
      if C <> FCellUnderMouse then
      begin
        if FCellUnderMouse <> nil then
        begin
          if Assigned(FOnMouseLeaveCell) then
            FOnMouseLeaveCell(Self, FCellUnderMouse);
          if C <> nil then
          begin
            if Assigned(FOnMouseEnterCell) then
              FOnMouseEnterCell(Self, C);
          end;
        end
        else if C <> nil then
        begin
          if Assigned(FOnMouseEnterCell) then
            FOnMouseEnterCell(Self, C);
        end;
        FCellUnderMouse := C;
        CellChanged := True;
      end;
    end;
    if (ViewMode = vmReport) and (Shift = [ssCtrl, ssAlt]) then
    begin
      if (FCellUnderMouse <> nil) and (C.FieldDef.DataType = dtResource) then
        Cursor := crHandPoint
      else
        Cursor := crDefault;
    end;

    if (ViewMode = vmReport) and (ReportImages = riHover) and
      (FItemUnderMouse <> -1) then
      Invalidate
    else if HotTrack and ItemEnteredOrLeft and
      not((FReport.SelectedCell <> nil) and FReport.SelectedCell.FEditing) then
      Invalidate
    else if CellChanged then
      Invalidate;
  end;

  {procedure TILDatasetViewer.DataAtXY(X, Y: integer; var ColumnIndex, ItemIndex,
    FieldIndex: integer);

  var
    i: integer;
    W: integer;
    P: integer;
  begin
    ColumnIndex := -1;
    ItemIndex := ItemAtXY(X, Y);
    if ItemIndex = -1 then
      Exit;
    W := 0;
    P := Table.Columns.ColumnPos(FLeftColumn, false);
    X := P + X;
    for i := 0 to Table.Columns.Count - 1 do
    begin
      W := W + Table.Columns[i].ColumnWidth + FColumnSpacing;
      if X < W then
      begin
        ColumnIndex := i;
        FieldIndex := Table.FFields.IndexOf(Table.Columns[i].FieldName);
        Break;
      end;
    end;
  end;}

  procedure TILDatasetViewer.ColumnClick(Index: integer);
  var
    SortFields: TStringList;
    Ordr: TSortOrdering;
  begin
    if roColumnClickSort in FReportOptions then
    begin
      SortFields := TStringList.Create;
      try
        SortFields.Add(Table.Columns[Index].FieldName);
        if Table.Columns.FClickSortIndex = Index then
        begin
          if Table.Columns.FClickSortOrdering = soAscending then
            Ordr := soDescending
          else
            Ordr := soAscending;
          Table.Columns.FClickSortOrdering := Ordr;
        end
        else
        begin
          Table.Columns.FClickSortIndex := Index;
          Table.Columns.FClickSortOrdering := soAscending;
          Ordr := soAscending;
        end;
        Table.Dataset.Sort(SortFields, Ordr);
      finally
        SortFields.Free;
      end;
    end;
  end;

  function TILDatasetViewer.IsImageFullyVisible(const ItemIndex: integer;
    const AtRight: Boolean): Boolean;

  var
    R: TRect;
  begin
    R := GetItemRect(rtItemRect, ItemIndex);
    if AtRight then
      Result := (R.Right <= ClientWidth)
    else
      Result := (R.Bottom <= ClientHeight)
  end;

  function TILDatasetViewer.IsImageFullyVisible(const ItemIndex: integer): Boolean;

  var
    R: TRect;
  begin
    R := GetItemRect(rtItemRect, ItemIndex);
    Result := not((R.Left < 0) or (R.Right > ClientWidth) or (R.Top < 0) or
      (R.Bottom > ClientHeight));
  end;

  function TILDatasetViewer.IsColumnVisible(const ColumnIndex: integer;
    var AtRight: Boolean): Boolean;

  var
    F, L: integer;
  begin
    Table.Columns.GetVisibleColumns(F, L);
    Result := (ColumnIndex >= F) and (ColumnIndex <= L);
    if not Result then
      AtRight := (ColumnIndex > L);
  end;

  procedure TILDatasetViewer.ImgTxtEditorKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);
  begin
    if Key = vkEscape then
      Table.Dataset.Items[Table.Dataset.Position].CancelEditImageText
    else if (Key = vkReturn) and (Shift = [ssCtrl]) then
      Table.Dataset.Items[Table.Dataset.Position].SetEditImageText
    else if (Key = vkTab) and (Shift = [ssCtrl]) and
      (Table.Dataset.Position < Table.Dataset.Count - 1) then
    begin
      Key := 0;
      Table.Dataset.Items[Table.Dataset.Position].SetEditImageText;
      Table.Dataset.Position := Table.Dataset.Position + 1;
      Table.Dataset.Items[Table.Dataset.Position].EditImageText;

    end
    else if (Key = vkTab) and (Shift = [ssCtrl, ssShift]) and
      (Table.Dataset.Position > 0) then
    begin
      Key := 0;
      Table.Dataset.Items[Table.Dataset.Position].SetEditImageText;
      Table.Dataset.Position := Table.Dataset.Position - 1;
      Table.Dataset.Items[Table.Dataset.Position].EditImageText;
    end;
  end;

  function TILDatasetViewer.ImageTextFits(const TheText: string): Boolean;
  var
    H, W: integer;
    i: integer;
    sl: TStringList;
  begin
    Result := false;
    if ImageTextSpace = 0 then
      Exit;
    Result := True;
    sl := TStringList.Create;
    try
      sl.Text := TheText;
      Canvas.Font.Assign(Font);
      H := 0;
      W := 0;
      for i := 0 to sl.Count - 1 do
      begin
        H := H + Canvas.TextHeight(sl[i]);
        W := W + Canvas.TextWidth(TheText);
        if (W > ImageWidth) or (H > ImageTextSpace) then
        begin
          Result := false;
          Break;
        end;
      end;
    finally
      sl.Free;
    end;
  end;

  procedure TILDatasetViewer.ScrollReport(const ScrollCode: integer; Delta: integer);
  const
    Up = 0;
    Down = 1;
  var
    LineIndx: integer;

  function LineOfPosition(Posit: integer): integer;
  var
    i: integer;
  begin
    Result := FReport.Items[Posit].DatasetIndex;
    if Result < Posit then
    begin
      while True do
      begin
        Inc(Result);
        if FReport.Items[Result].DatasetIndex = Posit then
        begin
          Break;
        end;
      end
    end
    else
    begin
      while Result > Posit do
      begin
        Dec(Result);
        if FReport.Items[Result].DatasetIndex= Posit then
        begin
          Break;
        end;
      end;
    end;
    while FReport.Items[Result].ExpansionIndex > 0 do
      Dec(Result);

    Exit;

    for i := 0 to FReport.Count - 1 do
    begin
      if FReport.Items[i].DatasetIndex = Posit then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  function RecordsToLines(Records: integer; Direction: integer): integer;

  { Converts a number of records scrolled to the corresponding line number }
  var
    Start: integer;
    i: integer;
    Cnt: integer;
  begin
    Result := -1;
    { Get linenumber of Dataset position }
    Start := LineOfPosition(Table.Dataset.Position);
    if Start = -1 then
      Exit;
    Cnt := 0; { Counts the records }
    if Direction = Down then
    begin
      { Get past the focused item }
      for i := Start + 1 to FReport.Count - 1 do
        if FReport.Items[i].ExpansionIndex = 0 then
        begin
          Result := i;
          Cnt := 1;
          Start := i;
          Break;
        end;
      if Records = 1 then
        Exit;
      for i := Start + 1 to FReport.Count - 1 do
      begin
        Inc(Result);
        if FReport.Items[i].ExpansionIndex = 0 then
        begin
          Inc(Cnt);
          if Cnt = Records then
            Break;
        end;
      end;
    end
    else
    begin
      for i := Start - 1 downto 0 do
        if FReport.Items[i].ExpansionIndex = 0 then
        begin
          Result := i;
          Cnt := 1;
          Start := i;
          Break;
        end;
      if Records = 1 then
        Exit;
      for i := Start - 1 downto 0 do
      begin
        Dec(Result);
        if FReport.Items[i].ExpansionIndex = 0 then
        begin
          Inc(Cnt);
          if Cnt = Records then
            Break;
        end;
      end;
    end;
    if Result < 0 then
      Result := 0
    else if Result > FReport.Count - 1 then
      Result := FReport.Count - 1;
  end;

  function PageRecords: integer;

  var
    i: integer;
  begin
    Result := 0;
    for i := FReport.FirstVisibleItem to FReport.LastVisibleItem do
    begin
      if FReport.Items[i].ExpansionIndex = 0 then
        Inc(Result);
    end;
  end;

  function EOF: Boolean;
  begin
    Result := (Table.Dataset.Position = Table.Dataset.Count - 1);
  end;

  function BOF: Boolean;
  begin
    Result := (Table.Dataset.Position = 0);
  end;

  begin
    LineIndx := -1;
    case ScrollCode of
      SB_SCROLLBY:
        begin
          if Delta < 0 then
          begin
            if BOF then
              Exit;
            LineIndx := RecordsToLines(Abs(Delta), Up);
          end
          else
          begin
            if EOF then
              Exit;
            LineIndx := RecordsToLines(Delta, Down);
          end;

        end;
      SB_SCROLLTO:
        begin
          if (Delta < 0) or (Delta > Table.Dataset.Count - 1) then
            Exit;
          LineIndx := LineOfPosition(Delta);
        end;
      SB_LINEDOWN:
        begin
          if Report.SelectedCell <> nil then
          begin
            if Report.SelectedCell.Owner.Expanded and
            (Report.SelectedCell.Position <
              Report.SelectedCell.Data[Report.SelectedCell.FieldIndex].Count-1) then
              begin
                Report.SelectedCell.Position := Report.SelectedCell.Position + 1;
                Invalidate;
                Exit;
              end;
          end;
          if EOF then
          begin
           if (roAutoAppend in ReportOptions) and (Table = TMan.ActiveTable)
           and not Table.FocusedRecord.IsDataCleared(True) then
           begin
            if TMan.UseCommandModes then
            begin
              TMan.CommandMode := cmUpdate;
              if Assigned(TMan.FOnChangeCommandMode) then
                TMan.FOnChangeCommandMode(Self);
            end;
            Table.DBWork.Insert;
           end;
           Exit;
          end;
          LineIndx := RecordsToLines(1, Down);
        end;
      SB_LINEUP:
        begin
          if Report.SelectedCell <> nil then
          begin
            if Report.SelectedCell.Position > 0 then
              begin
                Report.SelectedCell.Position := Report.SelectedCell.Position-1;
                Invalidate;
                Exit;
              end;
          end;
          if BOF then
            Exit;
          LineIndx := RecordsToLines(1, Up);
        end;
      SB_PAGEDOWN:
        begin
          if EOF then
            Exit;
          LineIndx := RecordsToLines(PageRecords, Down)
        end;
      SB_PAGEUP:
        begin
          if BOF then
            Exit;
          LineIndx := RecordsToLines(PageRecords, Up)
        end;
      SB_BOTTOM:
        begin
          if EOF then
            Exit;
          LineIndx := RecordsToLines(Table.Dataset.Count-1, Down);
        end;
      SB_TOP:
        begin
          LineIndx := 0;
        end;
    end;

    if LineIndx < 0 then
      LineIndx := 0
    else if LineIndx > FReport.Count - 1 then
      LineIndx := FReport.Count - 1;

    if FReport.Count div GetRowCount = 0 then
    begin
      Table.Dataset.Position := FReport.Items[LineIndx].DatasetIndex;
      Exit;
    end;
    if LineIndx < FReport.FirstVisibleItem then
      FirstItemIndex := LineIndx
    else if LineIndx > FReport.LastVisibleItem then
      FirstItemIndex := LineIndx - GetRowCount + 1;
    Table.Dataset.Position := FReport.Items[LineIndx].DatasetIndex;
  end;

  procedure TILDatasetViewer.ScrollImages(const ScrollCode: integer; Delta: integer);

  var
    F, L: integer;
    LastChunkStart: integer;

    procedure SetDelta;

  var
    F, L: integer;
    NewPos: integer;
  begin
    if Table.Dataset.Position + Delta > Table.Dataset.Count - 1 then
      Delta := Table.Dataset.Count - 1 - Delta;
    if Table.Dataset.Position + Delta < 0 then
      Delta := FirstItemIndex + Delta;
    NewPos := 0;
    GetVisibles(F, L);
    if (Table.Dataset.Position + Delta < F) or (Table.Dataset.Position + Delta > L) then
      FirstItemIndex := Table.Dataset.Position + Delta;
    if FirstItemIndex > LastChunkStart then
    begin
      FirstItemIndex := LastChunkStart;
    end
    else if FirstItemIndex < 0 then
    begin
      FirstItemIndex := 0;
      NewPos := 0;
    end
    else
      NewPos := Table.Dataset.Position + Delta;
    Table.Dataset.Position := NewPos;
  end;

  begin
    GetVisibles(F, L);
    if F = -1 then
      Exit;
    if ImageLayout = ilGrid then
    begin
      LastChunkStart := Table.Dataset.Count - (GetColCount * GetRowCount);
      if not(ioWrapPartialVisibles in ImageOptions) then
        Inc(LastChunkStart, GetColCount);
    end
    else if ImageLayout = ilStripHorz then
    begin
      LastChunkStart := Table.Dataset.Count - GetColCount;
      if not(ioWrapPartialVisibles in ImageOptions) then
        Inc(LastChunkStart);
    end
    else
    begin
      LastChunkStart := Table.Dataset.Count - GetRowCount;
      if not(ioWrapPartialVisibles in ImageOptions) then
        Inc(LastChunkStart, 1);
    end;
    if LastChunkStart < 0 then
      LastChunkStart := 0;

    if ScrollCode = SB_SCROLLBY then
    begin
      SetDelta;
    end

    else if ScrollCode = SB_SCROLLTO then
    begin
      Delta := Delta - Table.Dataset.Position;
      SetDelta;
    end

    else if ScrollCode = SB_PAGEDOWN then
    begin
      if Table.Dataset.Position = Table.Dataset.Count - 1 then
        Exit;
      if ImageLayout = ilGrid then
      begin
        FirstItemIndex := FirstItemIndex + GetColCount * GetRowCount;
      end
      else if ImageLayout = ilStripHorz then
      begin
        FirstItemIndex := FirstItemIndex + GetColCount;
      end
      else
      begin
        FirstItemIndex := FirstItemIndex + GetRowCount;
      end;
      if FirstItemIndex > LastChunkStart then
      begin
        FirstItemIndex := LastChunkStart;
      end;
      Table.Dataset.Position := FirstItemIndex;
    end

    else if ScrollCode = SB_PAGEUP then
    begin
      if Table.Dataset.Position = 0 then
        Exit;
      if ImageLayout = ilGrid then
      begin
        FirstItemIndex := FirstItemIndex - GetColCount * GetRowCount;
      end
      else if ImageLayout = ilStripHorz then
      begin
        FirstItemIndex := FirstItemIndex - GetColCount;
      end
      else
      begin
        FirstItemIndex := FirstItemIndex - GetRowCount;
      end;
      if FirstItemIndex < 0 then
        FirstItemIndex := 0;
      Table.Dataset.Position := FirstItemIndex;
    end

    else if ScrollCode = SB_LINEDOWN then
    begin
      if Table.Dataset.Position = Table.Dataset.Count - 1 then
        Exit;
      if (ImageLayout in [ilStripVert, ilGrid]) then
      begin
        Table.Dataset.Position := Table.Dataset.Position + GetColCount;
        if (Table.Dataset.Position > L) or not IsImageFullyVisible(L) then
        begin
          FirstItemIndex := FirstItemIndex + GetColCount;
        end;
      end;
    end

    else if ScrollCode = SB_LINEUP then
    begin
      if Table.Dataset.Position = 0 then
        Exit;
      if (ImageLayout in [ilStripVert, ilGrid]) then
      begin
        Table.Dataset.Position := Table.Dataset.Position - GetColCount;
        if Table.Dataset.Position < F then
          FirstItemIndex := FirstItemIndex - GetColCount;
        if FirstItemIndex < 0 then
          FirstItemIndex := 0;
      end;
    end

    else if ScrollCode = SB_TOP then
    begin
      FirstItemIndex := 0;
      Table.Dataset.Position := 0;
    end

    else if ScrollCode = SB_BOTTOM then
    begin
      if Table.Dataset.Position = Table.Dataset.Count - 1 then
        Exit;
      FirstItemIndex := LastChunkStart;
      Table.Dataset.Position := Table.Dataset.Count - 1;
      if not IsImageFullyVisible(Table.Dataset.Position, false) then
        FirstItemIndex := FirstItemIndex + GetColCount;
    end;
  end;

  procedure TILDatasetViewer.ScrollBy(const Delta: integer);
  begin
    if ViewMode = vmReport then
      ScrollReport(SB_SCROLLBY, Delta)
    else
      ScrollImages(SB_SCROLLBY, Delta);
  end;

  procedure TILDatasetViewer.ScrollTo(const Index: integer);
  begin
    if ViewMode = vmReport then
      ScrollReport(SB_SCROLLTO, Index)
    else
      ScrollImages(SB_SCROLLTO, Index)
  end;

  procedure TILDatasetViewer.ScrollIntoView(Index: integer);

  var
    F, L: integer;
    SpareItms, Decr: integer;
  begin
    GetVisibles(F, L);
    SpareItms := OutputCount - Index;
    if SpareItms < RowCount * ColCount then
    begin
      SpareItms := RowCount * ColCount;
      Index := OutputCount - SpareItms + ColCount;
    end;
    if (Index < F) or (Index > L) then
    begin
      Decr := Index mod ColCount;
      FirstItemIndex := Index - Decr;
    end;
  end;

  procedure TILDatasetViewer.SetBounds(ALeft: integer; ATop: integer; AWidth: integer;
    AHeight: integer);
  begin
     inherited
  end;

  procedure TILDataSetViewer.UpdateScrollers;
  begin
    UpdateScrollRange;
    UpdateScrollPos;
  end;

  function TILDatasetViewer.GetViewMode: TViewMode;
  begin
    Result := FViewMode;
  end;

  procedure TILDatasetViewer.SetViewMode(const Value : TViewMode);
  begin
    if FViewMode = Value then
      Exit;
    if table = nil then
    begin
      FViewMode := value;
      Exit;
    end;
    if not Assigned(Table.Columns) then
      Table.FColumns := TILColumns.Create(Table, Table.Fields);
    { Happens at designtime }

    if (FViewMode = vmReport) and (Report.SelectedCell <> nil) and
      Report.SelectedCell.Editing then
    begin
      CellEditor.UpdateValue;
      CellEditor.ActiveCtrl.Visible := false;
    end
    else if (FViewMode = vmImages) and (Table.Dataset.Position <> -1) then
    begin
      if Table.FocusedRecord.FEditingImageText then
        Table.FocusedRecord.SetEditImageText;
    end;

    if FViewMode = vmImages then
      FTLImages := FirstItemIndex
    else
      FTLReport := FirstItemIndex;

    FViewMode := Value;
    if FViewMode = vmReport then
      FirstItemIndex := FTLReport
    else
      FirstItemIndex := FTLImages;

    if (FViewMode = vmReport) and (Table.Dataset.Count > 0) then
    begin
      CreateReport;
    end;
    ScrollIntoView(Table.Dataset.Position);
    if Table.Columns.Count > 0 then
    begin
      UpdateScrollRange;
      UpdateScrollPos;
    end;
    Invalidate;
  end;

  function TILDatasetViewer.DoZoom(const Width, Height : integer) : TILImage;
  var
    AParentForm : TForm;
  begin
    if FZoomWindow <> nil then
    begin
      Result := FZoomImage;
      Exit;
    end;

     AParentForm := Owner as TForm;

    if (AParentForm = Owner) and (ZoomWindow = nil) then
    begin
      FZoomWindow := TForm.Create(AParentForm);
      FZoomWindow.Caption := Translator.GetTranslation(T_IMAGE, TRANSL_TERMS);
    end
    else if AParentForm <> Owner then
       FZoomWindow := AParentForm;

    Result := TILImage.Create(AParentForm);
    if Table.DataHandler.ImageControl <> nil then
      Result.ShowNavButtons := Table.DataHandler.ImageControl.ShowNavButtons;
    Result.Parent := FZoomWindow as TWinControl;
    Result.Align := alClient;
    FZoomWindow.OnShow := Result.WndOnShow;

    if (Width = 0) or (Height = 0) then
    begin
      FZoomWindow.WindowState := wsMaximized
    end
    else
    begin
       FZoomWindow.Width := Width;
       FZoomWindow.Height := Height;
       FZoomWindow.Position := poScreenCenter;
    end;
    Result.JPEGQuality := jqGood;
    Result.FInZoomWindow := True;
    FZoomWindow.Visible := True;
    FZoomWindow.FreeNotification(Self);
    FZoomImage := Result;
    Result.SetFocus;
  end;

  procedure TILDatasetViewer.ShowZoomWindow(ARecord : TILDataRecord; const Width, Height : integer);
  var
    Img : TILImage;
  begin
    Img := DoZoom(Width, Height);
    Img.LoadFromRecord(ARecord);
    ZoomWindow.Visible := True;
    ZoomWindow.OnShow(nil);
  end;

  procedure TILDatasetViewer.ShowZoomWindow(AField : TILField; const Width, Height : integer);
  var
    Img : TILImage;
  begin
    Img := DoZoom(Width, Height);
    Img.LoadFromField(AField);
    ZoomWindow.Visible := True;
    ZoomWindow.OnShow(nil);
  end;

  procedure TILDatasetViewer.CreateReport;
  begin
    FReport.Populate;
  end;

  procedure TILDatasetViewer.SetShowCheckBoxes(const Value : Boolean);
  begin
     if (FShowCheckBoxes = Value) or ((Table <> nil) and  not Table.FMultiSelect) then
      Exit;
    FShowCheckBoxes := Value;
    Invalidate;
  end;

  function TILDatasetViewer.GetOutputCount: integer;
  begin
    Result := 0;
    if ViewMode = vmReport then
      Result := FReport.Count
    else if (ViewMode in [vmImages]) and (Table.Dataset <> nil) then
      Result := Table.Dataset.Count;
  end;

  function TILDatasetViewer.GetItemRect(const ARectType: TRectType;
    const ItemIndex: integer): TRect;

  var
    X, Y: integer;
    First, Last: integer;
    i: integer;
    CenterX, CenterY: integer;
    MaxCols, ColCnt: integer;

    procedure Center(TopX, TopY: integer; var cx, cy: integer; Indx: integer);

  var
    W, H: integer;
    Rec: TILDataRecord;
  begin
    if ARectType = rtImageRect then
    begin
      Rec := Table.Dataset.Items[Indx];
      W := Rec.Image.Width;
      H := Rec.Image.Height;
      cx := TopX + (ImageWidth - W) div 2;
      cy := TopY + (ImageHeight - H) div 2;
    end;
  end;

  function GetRect: TRect;

  var
    cbX, cbY: integer;
  begin
    if ARectType = rtItemRect then
    begin
      if ViewMode = vmReport then
        Result := Rect(X, Y, X + ImageWidth, Y + ImageHeight)
      else
        Result := Rect(X, Y, X + ImageWidth, Y + ImageHeight + ImageTextSpace);
    end
    else if ARectType = rtTextRect then
    begin
      if ViewMode = vmReport then
        Result := Rect(0, 0, 0, 0)
      else
        Result := Rect(X, Y + ImageHeight - ImageTextSpace, X + ImageWidth,
          Y + ImageHeight + ImageTextSpace);
    end
    else if ARectType = rtCheckRect then
    begin
      if ViewMode = vmReport then
      begin
        cbX := X + 3;
        cbY := Y + (ReportLineHeight - c_CheckBoxWidth) div 2;
        Result := Rect(cbX, cbY, cbX + c_CheckBoxWidth, cbY + c_CheckBoxWidth);
      end
      else
        Result := Rect(X, Y, X + c_CheckBoxWidth, Y + c_CheckBoxWidth);
    end
    else if ARectType = rtImageRect then
    begin
      if ViewMode = vmReport then
        Result := Rect(0, 0, 0, 0)
      else
        Result := Rect(CenterX, CenterY, CenterX + ImageWidth,
          CenterY + ImageHeight - ImageTextSpace);
    end
    else if ARectType = rtPlayerIconRect then
    begin
      if (ViewMode = vmReport) or (not Assigned(TMan.FIntPlayerIcon) and not Assigned(TMan.FIntPlayerBM))
      or not (FReport.Items[ItemIndex].Data.MediaType in [MED_AUDIO, MED_VIDEO])  then
        Result := Rect(0, 0, 0, 0)
      else
      begin
         cbX := X + (ImageWidth div 2) - 10;
         cbY := Y + (ImageHeight div 2) - 10;
         Result := Rect(cbX, cbY, cbX + 20, cbY + 20);
      end;
    end
  end;

  begin
    Result := Rect(0, 0, 0, 0);
    if Table = nil then
      Exit;
    if Table.Dataset = nil then
    begin
      Exit;
    end;
    if ARectType = rtCaptionRect then
    begin
      if Caption <> '' then
      begin
        Result.Left := 0;
        Result.Top := 0;
        Result.Width := Width;
        Canvas.Font.Assign(Font);
        Canvas.Font.Size := Font.Size + 2;
        Result.Height := Canvas.TextHeight('X') + 6;
      end;
      Exit;
    end;
    GetVisibles(First, Last);
    if First = -1 then
      Exit;
    if (ViewMode = vmReport) then
    begin
      X := GetRptLeftMargin;
      Y := ReportLineHeight;
    end
    else
    begin
      X := GetLeftMargin;
      Y := GetTopMargin;
    end;

    if ViewMode = vmReport then
    begin
      if ARectType = rtItemRect then
        Result := FReport.Items[ItemIndex].ItemRect
      else if ARectType = rtCheckRect then
        Result := FReport.Items[ItemIndex].CheckRect
      else if ARectType = rtImageRect then
        Result := FReport.Items[ItemIndex].ImageRect
      else
        Result := FReport.Items[ItemIndex].ExpandBtnRect;
    end
    else if (ImageLayout = ilStripHorz) then
    begin
      for i := First to Last do
      begin
        Center(X, Y, CenterX, CenterY, i);
        if i = ItemIndex then
        begin
          Result := GetRect;
          Break;
        end;
        Inc(X, ImageWidth + ImageItemSpacing);
      end;
    end
    else if (ImageLayout = ilStripVert) then
    begin
      for i := First to Last do { The last visible item }
      begin
        Center(X, Y, CenterX, CenterY, i);
        if i = ItemIndex then
        begin
          Result := GetRect;
          Break;
        end;
        Inc(Y, ImageHeight + GetImageRowSpacing);
      end;
    end
    else { ilGrid }
    begin
      MaxCols := GetColCount;
      ColCnt := 1;
      for i := First to Last do { The last visible item }
      begin
        if ColCnt > MaxCols then
        begin
          Inc(Y, ImageHeight + GetImageRowSpacing);
          X := GetLeftMargin;
          ColCnt := 1;
        end;
        Center(X, Y, CenterX, CenterY, i);
        if i = ItemIndex then
        begin
          Result := GetRect;
          Break;
        end;
        Inc(X, ImageWidth + ImageItemSpacing);
        Inc(ColCnt);
      end;
    end;
  end;

  function TILDatasetViewer.ItemAtXY(const X: integer; const Y: integer): integer;

  var
    R: TRect;
    i: integer;
    F, L: integer;
  begin
    Result := -1;
    if ViewMode = vmReport then
    begin
      Result := FReport.RowAtXY(X, Y);
      Exit;
    end;
    GetVisibles(F, L);
    if F = -1 then
      Exit;
    for i := F to L do
    begin
      R := GetItemRect(rtItemRect, i);
      if PtInRect(R, Point(X, Y)) then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  { TChangeTracker ---------------------------------------------------------}
    function TChangeTracker.GetCount : integer;
    begin
       Result := FChngRecords.Count;
    end;

    function TChangeTracker.GetChngRecord(Index : integer) : TILDataRecord;
    begin
       Result := TILDataRecord(FChngRecords[Index]);
    end;

    constructor TChangeTracker.Create;
    begin
       FChngRecords := TList.Create;
    end;

    destructor TChangeTracker.Destroy;
    begin
      FChngRecords.Free;
      inherited;
    end;

    procedure TChangeTracker.Clear;
    begin
       FChngRecords.Clear;
    end;

    procedure TChangeTracker.Delete(Index : integer);
    begin
      FChngRecords.Delete(Index);
    end;

    procedure TChangeTracker.Delete(Arecord: TILDataRecord);
    var
      Index : integer;
    begin
      Index := FChngRecords.IndexOf(ARecord);
      if Index <> -1 then
        FChngRecords.Delete(Index);
    end;

    procedure TChangeTracker.Add(ARecord : TILDataRecord);
    var
      Indx : integer;
    begin
      if ARecord.FIsDataControls then
        Exit;

      ARecord.FValidText := '';
      Indx := FChngRecords.IndexOf(ARecord);
      if Indx = -1 then
      begin
         if Arecord.Changed then
         begin
          FChngRecords.Add(ARecord);
          ARecord.FValidState := ARecord.Validate(True, 1);
          if ARecord.FValidState <> 0 then
            ARecord.FValidText := Comment1;
         end;
      end
      else
      begin
        if not ARecord.Changed then
          Delete(Indx)
        else
        begin
          ARecord.FValidState := ARecord.Validate(True, 1);
          if ARecord.FValidState <> 0 then
            ARecord.FValidText := Comment1;
        end;
      end;

    end;

  { TILTable -------------------------------------------------------------- }

  constructor TILTable.Create(AOwner: TComponent);
  var
    sl: TStringList;
  begin
    inherited Create(AOwner);
    FFields := TILFieldDefs.Create(Self, TILFieldDef);
    FDBWork := TILDBWork.Create(Self);
    FDBView := TILDBView.Create(Self);
    FDBAdmin := TILDBAdmin.Create;
    FDBCommons := TILDBCommons.Create;
    FDBCommons.Ilib := Self;
    FDBAdmin.Ilib := Self;
    FFocHandler := TFocusHandler.Create;
    FTableSpace := 300000;
    FFirstRecordID := 1;
    FDatasetMaxCount := 10000;
    FNamesMaxLength := 100;
    FNumberMaxLength := 15;
    FTextMaxLength := 2000;
    FWordMaxLength := 50;
    FDataHandlers := TList.Create;
    FReadSearchPathControls := True;
    FImageLocalSize := ils200;
    FDataset := TILDataset.Create(Self);
    FPrecheckConstraints := True;
    FTranslateLookups := True;
    sl := TStringList.Create;
    FSearcher := TSearcher.Create(Self, sl);
    FSearcher.FPrivate := False;
    FMasterLinks := TList.Create;
    FDetailLinks := TList.Create;
    sl.Free;
    FUpmCtrlRecord := TILRecord.Create(Self);
    FXMLDoc := TXMLDocument.Create(Self);
    FXMLDoc.Active := True;
    FXMLDoc.DocumentElement := FXMLDoc.CreateNode('Ilib', ntElement, '');
    RegTables.Add(Self);
    FHWnd := AllocateHWnd(WndMethod);
    //System.ReportMemoryLeaksOnShutdown := True;
  end;

  destructor TILTable.Destroy;
  var
    Indx : integer;
  begin
    if Connected then
      CloseTable;
    FXMLDoc := nil;
    inherited Destroy;
    if Assigned(FDataset) then
      FreeAndNil(FDataset);
    FFields.Free;
    if Assigned(FColumns) then
     FColumns.Free;
    FSearcher.Free;
    FDBView.Free;
    FDBWork.Free;
    FDBAdmin.Free;
    FDBCommons.Free;
    FMasterLinks.Free;
    FDetailLinks.Free;
    FFocHandler.Free;
    FDataHandlers.Free;
    FUPMCtrlRecord.Free;
    if Assigned(FLinkList)  and not (Self is TILView) then
    begin
     ClearIndex;
     FLinkList.Free;
    end;
    if Assigned(FKeyList) and not (Self is TILView) then
    begin
     FKeyList.Free;
    end;
    Indx := regTables.IndexOf(Self);
    if Indx <> -1 then
      RegTables.Delete(Indx);
    DeallocateHWnd(FHWnd);
  end;

  function TILTable.IsRunning: Boolean;
  begin
    Result := (ComponentState = []) or (ComponentState = [csFreeNotification]);
  end;

  function TILTable.IsMaster: Boolean;
  begin
    Result := (ChildCount > 0);
  end;

  function TILTable.IsDetail: Boolean;
  begin
    Result := Assigned(MasterSource);
  end;

  function TILTable.IsRoot: Boolean;
  begin
    Result := (MasterSource = nil) and (Self is TILDataTable)
    and not IsFreeTable;
  end;

  function TILTable.IsFreeTable: Boolean;
  var
    i : integer;
  begin
    Result := false;
    if Assigned(TMan) then
    begin
      for I := 0 to TMan.FreeTableCount-1 do
      if TMan.FreeTables[i] = self then
      begin
        Result := True;
        Break;
      end;
    end
    else
      Result := True;
  end;

  procedure TILTable.Inval;
  begin
      if HasViewer then FViewer.Invalidate;
      if DataHandler = nil then
        Exit;
      if (DataHandler.ImageControl <> nil) then
      begin
         if FocusedRecord = nil then
         begin
           DataHandler.ImageControl.ClearSource;
           if HasViewer then
           if Assigned(Viewer.ZoomImage) and Viewer.ZoomWindow.Visible then
            Viewer.ZoomImage.ClearSource;
         end
         else
         begin
             if TMan.ExtDesigner and not (Self = Tman.ActiveTable) then
             { If running from the designer. The designer lets one single
             image conrol be shared among all handlers. Only show images
             of the active table }
               Exit;
             DataHandler.ImageControl.LoadFromRecord(FocusedRecord);
             if HasViewer then
             if Assigned(Viewer.ZoomImage) and Viewer.ZoomWindow.Visible then
             begin
               Viewer.ZoomImage.LoadFromRecord(FocusedRecord);
             end;
         end;
      end;
  end;

  function TILTable.HasViewer : Boolean;
  begin
      Result := (FViewer <> nil);
  end;

  function TILTable.NoOutput: Boolean;
  begin
    Result := (FViewer = nil) or
      ((FViewer.ViewMode in [vmImages]) and (Dataset.Count = 0)) or
      ((FViewer.ViewMode = vmReport) and (Columns.Count = 0));
  end;

  procedure TILTable.SaveColumns;
  var
    i : integer;
    Indx : integer;
  begin
        Tman.FCfgFile.EraseSection(TableName + '_ColumnSequence');
        for i := 0 to Columns.Count - 1 do
        begin
          if not Columns.Items[i].FTemporary then
            Tman.FCfgFile.WriteInteger(TableName + '_ColumnSequence',
              Columns[i].FFieldName, Columns[i].FColumnWidth);
        end;
        Tman.FCfgFile.EraseSection(TableName + '_ColumnSort');
        for i := 0 to Columns.SortFields.Count - 1 do
        begin
          Indx := Fields.IndexOf(Columns.SortFields[i]);
          if Indx <> -1 then
            if not Fields.Items[Indx].FTemporary then
              Tman.FCfgFile.WriteInteger(TableName + '_ColumnSort',
                Columns.SortFields[i],
                integer(Columns.SortOrdering));
        end;
  end;

  function TILTable.ComputeRecordSize : integer;
  var
   i : integer;
   StrSize : integer;
   StrCnt : integer;
   Reduct : integer;

  begin
    Result := 0;
    StrCnt := 0;
    StrSize := 0;
    for I := 0 to Fields.Count-1 do
    begin
      if Fields.Items[i].MaxValues <> 0 then
         Result := Result + 11 +
          (9 + ComputeFieldSpace(Fields.Items[i], True)) * Fields.Items[i].MaxValues
     {11 is the field id tags, 9 size of value tags}
       else
         Result := Result + 11 + (9 + ComputeFieldSpace(Fields.Items[i], True)) * 3;
          { 4 values as a guess, but this should not be called }

      if (Fields.Items[i].MaxLength = 0) and (Fields.Items[i].DataType in [dtText, dtString, dtResource]) then
      begin
        inc(StrCnt);
        if Fields.Items[i].DataType  = dtText then
          inc(StrSize, 2000)
        else
          inc(StrSize, 100);
       end;
    end;

    if StrCnt > 1 then
    begin
      { Reuce with 10 pst for each maxlength string, but not more than 50 % }
      Reduct := Round(StrSize * StrCnt * 10 / 100);
      if Reduct > StrSize div 2 then
        Reduct := StrSize div 2;
      Result := Result - Reduct;
    end;

    Result := Result + 204; {204 sys section }
  end;

  function TILTable.ComputeAllFieldsSpace : integer;
  var
  i : integer;
  begin
    Result := 0;
    for I := 0 to Fields.Count-1 do
    begin
      Result := Result +  ComputeFieldSpace(Fields.Items[i]);
    end;
  end;

  function TILTable.ComputeFieldSpace(AField : TILFieldDef; const RecSize : Boolean = false) : integer;
    var
    Tmp: integer;
  begin
    Result := 0;
    if AField.MaxLength > 0 then
    begin
        if AField.DataType = dtText then
        begin
          if RecSize then
            Result := AField.MaxLength
          else
            Result := FWordMaxLength;
        end
        else if Result < AField.MaxLength then
          Result := AField.MaxLength;
      end
    else
    begin
        Tmp := 0;
        if AField.DataType = dtText then
        begin
          if RecSize then
            Tmp := FTextMaxLength
          else
            Tmp := 50;
        end
        else if AField.DataType = dtDate then
          Tmp := 24
        else if AField.DataType = dtBoolean then
          Tmp := 1
        else if AField.DataType = dtString then
          Tmp := FNamesMaxLength
        else if AField.DataType in [dtResource, dtLongText] then
          Tmp := 10
        else if AField.DataType = dtNumber then
          Tmp := FNumberMaxLength;
        if Result < Tmp then
          Result := Tmp;
    end;
    Result := Result + 12; { For the counters at start of key }
    Result := Result + 10; { For the key val id numbers}
    if Result < 26 then
      Result := 26;
  end;

  function TILTable.GetTVControl : TILTreeView;
  begin
      if Self is TILTreeViewTable then
      with Self as TILTreeViewTable do
        Result := TreeViewControl
      else
        Result := nil;
  end;

  function TILTable.GetTVTable : TILTreeViewTable;
  begin
    if Self is TILTreeViewTable then
      Result := Self as TILTreeViewTable
    else
      Result := nil;
   end;

  function TILTable.GetDatahandler : TILData;
  begin
      Result := FDataHandler;
  end;

  function TILTable.GetDatahandlers(Index : integer) : TILData;
  begin
      Result := TILData(FDataHandlers[Index]);
  end;

  function TILTable.GetDatahandlerCount : integer;
  begin
      Result := FDataHandlers.Count;
  end;

  procedure TILTable.ResetDataControls(SelCnt : integer = -1);
  { Called from selecting procedues (MouseDown, DeselectAll, SetSelected)
    when there are no more seleced records. Resets the data control by fetching
    the data from the record in focus, as the controls may not reflect the
    record }
  begin
    if SelCnt = -1 then
      SelCnt := SelCount;
    if (Dataset.Position <> -1) then
    begin
      if (SelCnt = 1) and
        not(Dataset.Items[Dataset.Position] = Selection[0]) then
        WriteControls(Dataset.Items[Dataset.Position], duData)
        { Dont refresh if the selected and the one i focus is
          one and the same }
      else if (SelCnt > 0) and not(Dataset.Items[Dataset.Position] = Selection[0]) then
        WriteControls(Dataset.Items[Dataset.Position], duData);
    end
  end;

  function TILTable.KeyCount(const AValue: string): integer;
  var
    Indx: integer;
    S: string;
    N : TKey;
  begin
    Result := 0;
    if FindKey(0, AValue, Indx) then
    begin
      N := TKey(FKeyList[Indx]);
      S := FillZeroes(IntToStr(N.id), 7);
      FindKey(1, S, Indx);
      while (Indx < FLinkList.Count) and (Pos(S, TKey(FLinkList[Indx]).Key) = 1) do
      begin
        Inc(Result);
        if Result > 1 then
          Break;
        Inc(Indx);
      end;
    end;
  end;

procedure TILTable.SaveIndex;
begin
  SaveKeyList;
  SaveLinkList;
  GetKeyWordID(True);
end;

procedure TILTable.LoadKeyList;
var
   i : integer;
   cnt : integer;
   S, S2, S3 :  string;
   P, P2 : integer;
   N : TKey;
begin
  for I := 0 to FKeyList.Count-1 do
    TKey(FKeyList[i]).Free;
  FKeyList.Clear;
  sleep(10);
  {It sometime happens with rollbacks that the key list poinst to the data file, or at
  least it fetches records from the data file. Mysterious. A little halt helps
  to bring it on the right track.}

  Cnt := FSF_KeyList.Count;
  if Cnt > 0 then
  for i := 0 to Cnt-1 do
  begin
    S := FSF_KeyList.Items[i];
    P := Pos('{', S);
    P2 := 0;
    while P <> 0 do
    begin
      P2 := P;
      P := Pos('{', S, P+1);
    end;
    P := P2;

    S2 := Copy(S, 1, P-1);
    P2 := Pos('}', S, P);
    S3 := Copy(S, P +1, P2-P-1);
    N := TKey.Create;
    N.Key := S2;
    if S3 = '' then
      Sleep(0);
    N.ID := StrToInt(S3);
    FKeyList.add(N);
  end;
end;

procedure TILTable.LoadLinkList;
  var
   i : integer;
   cnt : integer;
   S :  string;
   N : TKey;
begin
  for I := 0 to FLinkList.Count-1 do
    TKey(FLinkList[i]).Free;
  FLinkList.Clear;
  Cnt := FSF_LinkList.Count;
  for i := 0 to Cnt-1 do
  begin
    S := FSF_LinkList.Items[i];
    N := TKey.Create;
    N.Key := S;
    N.ID := 0;
    FLinkList.add(N);
  end;
end;

procedure TILTable.SaveKeyList;
var
  M, MS : TMemoryStream;
  i : integer;
  N : TKey;
  S : string;
  Cnt : integer;
begin
  if DisableIndexUpdate then
    Exit;
  M := TMemoryStream.Create;
  try
  Cnt := FKeyList.Count;
  for I := 0 to Cnt-1 do
  begin
     N := TKey(FKeyList[i]);
     S := N.Key + '{' + IntToStr(N.ID) + '}';
     MS := FSF_KeyList.StringToStream(S);
     try
     MS.Position := 0;
     M.CopyFrom(MS, Ms.Size);

     finally
       MS.Free;
     end;
  end;
   FSF_KeyList.ReplaceAll(M);
  finally
    M.Free;
  end;
end;

procedure TILTable.SaveLinkList;
var
  M, MS : TMemoryStream;
  S : string;
  N : TKey;
  i : integer;
begin
  if DisableIndexUpdate then
    Exit;
  M := TMemoryStream.Create;
  try
  for I := 0 to FLinkList.Count-1 do
  begin
     N := TKey(FLinkList[i]);
     S := N.Key;
     MS := FSF_LinkList.StringToStream(S);
     try
     MS.Position := 0;
     M.CopyFrom(MS, Ms.Size);
     finally
       MS.Free;
     end;
   end;
   FSF_LinkList.ReplaceAll(M);
  finally
   M.Free;
  end;
end;

  procedure TILTable.UpdateIndexes(ADataRecord: TILDataRecord;
    const ATableIndex: integer; const AMovedRecord : integer);

  { Updates the index files, based on what is deleted from and what is added
    to ADataRecord }
  var
    i, j: integer;
    Found: Boolean;
    Val, Val2: string;
    Indx, FldIndx: integer;
    S, FldName: string;
    TR: TILTestResult;
    Updated: Boolean;
    TargetRec: TILDataRecord;
    SourceRec: TILDataRecord;
    KeyInt: integer;
    FldDef: TILFieldDef;
    RecIndx: integer;
    RefRec: TILDataRecord;
    TextFields: TStringList;
    N : TKey;

  function GetTRValue(Added: Boolean; Index: integer): string;
  var
    P: integer;
    S: string;
    N, A: string;
  begin
    if Added then
      S := TR.Adds[Index]
    else
      S := TR.Deletes[Index];
    P := Pos(':', S);
    Result := Copy(S, P + 2, Length(S));
    if FldDef.DataType = dtResource then
    begin
      GetResourceElements(Result, N, A);
      Result := N;
    end;
  end;

  function GetTRFieldName(Added: Boolean; Index: integer): string;

  var
    P: integer;
    S: string;
  begin
    if Added then
      S := TR.Adds[Index]
    else
      S := TR.Deletes[Index];
    P := Pos(':', S);
    Result := Copy(S, 1, P - 1);
  end;

  function CreateLinkStr(KeyIndex, FieldIndex, RecIndex: integer): string;

  var
    S, s2: string;
  begin
    S := IntToStr(KeyIndex);
    S := FillZeroes(S, 7);
    S := S + '|';
    if FieldIndex < 90 then
     s2 := IntToStr(Fields.Items[FieldIndex].FieldID)
    else
     s2 := IntToStr(FieldIndex);
    s2 := FillZeroes(s2, 2);
    S := S + s2 + '|';
    s2 := IntToStr(RecIndex);
    s2 := FillZeroes(s2, 7);
    Result := S + s2;
  end;

  procedure SetEmpty(FieldIndx, RecId: integer);
  var
    N : TKey;
  begin
   if (FKeyList.Count > 0) and (TKey(FKeyList[0]).Key = '')
    then { Already in keylist, update linklist }
    begin
      N := TKey(FKeyList[0]);
      S := CreateLinkStr(N.ID, FieldIndx, RecId);
      if not FindKey(1, S, Indx) then
      begin
        AddKey(1, S, 0, Indx);
        FIndexUpdated := True;
      end;
    end
    else
    begin
      KeyInt := GetKeyWordID(false);
      Indx := AddKey(0, '', KeyInt, -2);
      S := CreateLinkStr(KeyInt, FieldIndx, RecId);
      AddKey(1, S, 0, -1);
      FIndexUpdated := True;
    end;
  end;

  procedure ExtractTextFields(TRs: TStringList; Adds: Boolean);

  var
    i: integer;
    FldName: string;
    FldIndx: integer;
  begin
    for i := 0 to TRs.Count - 1 do
    begin
      FldName := GetTRFieldName(Adds, i);
      FldIndx := Fields.IndexOf(FldName);
      if not Fields.Items[FldIndx].Indexed then
        Continue;
      if Fields.Items[FldIndx].DataType = dtText then
      begin
        TextFields.Add(FldName);
      end;
    end;
  end;

  procedure SetQuotes(ARec: TILDataRecord);

  var
    i, j: integer;
    S: string;
  begin
    for i := 0 to ARec.Count - 1 do
    begin
      if IsStringType(ARec[i].FieldDef.DataType) then
        for j := 0 to ARec[i].Count - 1 do
        begin
          S := ARec[i].FValues[j];
          S := '''' + S + '''';
          ARec[i].FValues[j] := S;
        end;
    end;
  end;

  function AbsoluteSame(V1, V2 : string) : Boolean;
  begin
    if Pos('''', V1) = 1 then
     Delete(V1, 1,1);
    if Pos('''', V1) = Length(V1) then
     Delete(V1, Length(V1),1);

    if Pos('''', V2) = 1 then
     Delete(V2, 1,1);
    if Pos('''', V2) = Length(V2) then
     Delete(V2, Length(V2),1);

    Result := SameText(V1, V2);
  end;

  procedure AddDelete;
  var
    RecIndx : integer;

    procedure DoSet;
    var
      N : TKey;
    begin
          if FindKey(0, 'Delete', Indx) then
          begin
            { Already in key list. Only update reference }
            N := TKey(FKeyList[Indx]);
            KeyInt := N.id;
            S := CreateLinkStr(KeyInt, 99, RecIndx);
            if not FindKey(1, S, Indx) then
              Indx := AddKey(1, S, 0, Indx);
          end
          else
          begin
            { Not in key list, add it and update ref. }
            KeyInt := GetKeyWordID(false);
            AddKey(0, 'Delete', KeyInt, Indx);
            S := CreateLinkStr(KeyInt, 99, RecIndx);
            AddKey(1, S, 0, -1);
          end;
    end;

  begin
     if ADataRecord.Deleted and (ADataRecord.FOldState <> 'DD') then
     begin
       RecIndx := FSF_Data.RealIndex(ATableIndex);
       DoSet;
     end;
     if AMovedRecord <> -1 then
     begin
       RecIndx := AMovedRecord;
       DoSet;
     end;
  end;

  procedure AddUndelete;
  var
    N : TKey;
  begin
        if FindKey(0, 'Delete', Indx) then
        begin
          N := TKey(FKeyList[Indx]);
          S := CreateLinkStr(N.id, 99, ATableIndex);
          if KeyCount('Delete') = 1
          then { Only this record refrences the field value. Delete from keylist }
          begin
            N.Free;
            FKeyList.Delete(Indx);
          end;
          if FindKey(1, S, Indx)
          then { Delete the reference to this field }
          begin
            N := TKey(FLinkList[Indx]);
            N.Free;
            FLinkList.Delete(Indx);
          end;
        end;
  end;

  begin
    if FDisableIndexUpdate or not Connected then
      Exit;

    if (ADataRecord.Deleted and (ADataRecord.FOldState <> 'DD')) or (AMovedRecord <> -1) then
    begin
       AddDelete;
       FIndexUpdated := True;
       if AMovedRecord = -1 then
       begin
        Exit;
       end;
    end
    else if not ADataRecord.Deleted and
     ((ADataRecord.FOldState = 'DD') or (ADataRecord.FOldState = 'DT')) then
    begin
       AddUndelete;
       FIndexUpdated := True;
       Exit;
    end;

    if not Fields.HasIndexes then
      Exit;
    if ATableIndex = -1 then
      TargetRec := CreateDataRecord
    else
    begin
      TargetRec := RecordsX[ATableIndex];
    end;

    for I := 0 to ADataRecord.Count-1 do
    begin
      for j := 0 to ADataRecord[i].Count-1 do
      begin
         S := ADataRecord[i].FValues[j];
         S := SetESCCodes(S);
         ADataRecord[i].FValues[j] := S;
      end;
    end;

    RefRec := TILDataRecord.Create(Self, false);
    SourceRec := TILDataRecord.Create(Self, false);

    RefRec.Assign(TargetRec);
    SourceRec.Assign(ADataRecord);
    { Run test, i.e comparing the record with the stored data. This will return the adds and deletes }
    FIndexing := True;
    try
     try
      TR := TestUpdate(SourceRec, TargetRec, Updated);
     except
       RefRec.Free;
       SourceRec.Free;
       TargetRec.Free;
       if TR <> nil then
         TR.Free;
       raise;
     end;
    finally
      FIndexing := False;
    end;
    if TR = nil then
      HandleILException(E_CreateIndexes);

    TextFields := TStringList.Create;
    TextFields.Sorted := True;
    TextFields.Duplicates := DupIgnore;
    try
      { if new record, add refs to Null }
      if TR = nil then
        Exit;
      if SourceRec.FRecordIndex = -1 then
      begin
        for i := 0 to SourceRec.Count - 1 do
        begin
          if not (SourceRec[i].FieldDef.Indexed) or (SourceRec[i].FieldDef.DataType = dtLongText) then
            Continue;
          if SourceRec[i].FValues.Count = 0 then
            SetEmpty(i, RecordCount);
        end;
      end;
      { Extract text fields. These are handled by themseleves }
      ExtractTextFields(TR.Adds, True);
      ExtractTextFields(TR.Deletes, false);


      { If the same value is in both lists, it means a change of caps.
        Delete those from the lists as they are alredy indexed }

      repeat
        Found := false;
        for i := 0 to TR.Adds.Count - 1 do
        begin
          FldName := GetTRFieldName(True, i);
          FldIndx := Fields.IndexOf(FldName);
          FldDef := Fields.Items[FldIndx];
          if not IsStringType(FldDef.DataType) then
            Continue;
          Val := GetTRValue(True, i);
          for j := 0 to TR.Deletes.Count - 1 do
          begin
            FldName := GetTRFieldName(false, j);
            FldIndx := Fields.IndexOf(FldName);
            FldDef := Fields.Items[FldIndx];
            if not IsStringType(FldDef.DataType) then
              Continue;
            Val2 := GetTRValue(false, j);
            if AbsoluteSame(Val, Val2) then
            begin
              TR.Adds.Delete(i);
              TR.Deletes.Delete(j);
              Found := True;
              Break;
            end;
          end;
          if Found then
            Break;
        end;
      until not Found;

      { Check adds }
      for i := 0 to TR.Adds.Count - 1 do
      begin
        FldName := GetTRFieldName(True, i);
        FldIndx := Fields.IndexOf(FldName);
        if not(Fields.Items[FldIndx].Indexed) or
          (TextFields.IndexOf(FldName) <> -1)
          or (Fields.Items[FldIndx].DataType = dtLongText) then
          Continue;
        RecIndx := SourceRec.IndexOf(FldName);
        FldDef := Fields.Items[FldIndx];

        Val := GetTRValue(True, i);
        Val := SetDateNumberFormat(Val, FldDef.DataType, FldDef.Decimals);
        if Length(Val) > IndexSpace then
          Val := Copy(Val, 1, IndexSpace);
        if FindKey(0, Val, Indx) then
        begin
          { Already in key list. Only update reference }
          N := TKey(FKeyList[Indx]);
          KeyInt := N.id;
          if SourceRec.FRecordIndex = -1 then
            { Insertion, not yet got its id, which will be the last item in the library }
            S := CreateLinkStr(KeyInt, FldIndx, RecordCount)
          else
            S := CreateLinkStr(KeyInt, FldIndx, SourceRec.FRecordIndex);
          if not FindKey(1, S, Indx) then
          begin
            AddKey(1, S, 0, Indx);
            FIndexUpdated := True;
          end;
        end
        else
        begin
          { Not in key list, add it and update ref. }
          KeyInt := GetKeyWordID(false);
          AddKey(0, Val, KeyInt, Indx);
          if SourceRec.FRecordIndex = -1 then
            { Insertion, not yet got its id, which will be the last item in the library }
            S := CreateLinkStr(KeyInt, FldIndx, RecordCount)
          else
            S := CreateLinkStr(KeyInt, FldIndx, SourceRec.FRecordIndex);
          AddKey(1, S, 0, -1);
          FIndexUpdated := True;
        end;


        if (SourceRec.FRecordIndex <> -1) and
          (RefRec[RecIndx].FValues.Count = 0) and
          (SourceRec[RecIndx].FValues.Count > 0) then
        { Delete the reference to Null }
        begin
          FindKey(0, '', Indx);
          N := TKey(FKeylist[Indx]);
          S := CreateLinkStr(N.ID, FldIndx, SourceRec.FRecordIndex);
          if KeyCount('') = 1 then
          begin
            N.Free;
            FKeyList.Delete(0);
            FIndexUpdated := True;
          end;
          if FindKey(1, S, Indx) then
          begin
            N := TKey(FLinkList[Indx]);
            N.Free;
           {Delete the reference to this field }
            FLinkList.Delete(Indx);
            FIndexUpdated := True;
          end;
        end;
      end;

      for i := 0 to TR.Deletes.Count - 1 do
      begin
        FldName := GetTRFieldName(false, i);
        FldIndx := Fields.IndexOf(FldName);
        if not Fields.Items[FldIndx].Indexed or
          (TextFields.IndexOf(FldName) <> -1)
        or (Fields.Items[FldIndx].DataType = dtLongText) then
          Continue;
        FldDef := Fields.Items[FldIndx];
        RecIndx := SourceRec.IndexOf(FldName);
        Val := GetTRValue(false, i);
        Val := SetDateNumberFormat(Val, FldDef.DataType, FldDef.Decimals);
        if FindKey(0, Val, Indx) then
        begin
          N := TKey(FKeyList[Indx]);
          S := CreateLinkStr(N.ID, FldIndx, SourceRec.FRecordIndex);
          if KeyCount(Val) = 1
          then { Only this record refrences the field value. Delete from keylist }
          begin
            N.Free;
            FKeyList.Delete(Indx);
            FIndexUpdated := True;
          end;
          if FindKey(1, S, Indx) then
          begin
            N := TKey(FLinkList[Indx]);
            N.Free;
            FLinkList.Delete(Indx);
            FIndexUpdated := True;
          end;
        end;

        if (SourceRec.FRecordIndex <> -1) and
          (RefRec[RecIndx].FValues.Count > 0) and
          (SourceRec[RecIndx].FValues.Count = 0) then
        { Add a new reference to Null }
        begin
          SetEmpty(FldIndx, SourceRec.FRecordIndex);
        end;
      end;

      for i := 0 to TextFields.Count - 1 do
      begin
        UpdateTextIndex(SourceRec, TextFields[i], ATableIndex);
      end;

    finally
      if TR <> nil then
        TR.Free;
      TargetRec.Free;
      RefRec.Free;
      SourceRec.Free;
      TextFields.Free;
    end;
  end;

  function TILTable.GetResNames(Vals: TStringList): TStringList;

  var
    i: integer;
    N, A: string;
  begin
    Result := TStringList.Create;
    for i := 0 to Vals.Count - 1 do
    begin
      GetResourceElements(Vals[i], N, A);
      if N <> '' then
        Result.Add(N);
    end;
  end;

  procedure TILTable.RebuildIndexes;

  var
    Cnt: integer;
    i, k: integer;
    Rec: TILDataRecord;
    j: integer;
    Val: string;
    snum1, snum2, snum3: string;
    Indx: integer;
    FldDef: TILFieldDef;
    s_linkList: TStringList;
    s_Keylist: TStringList;
    s_KeyIds: TStringList;
    TotLoops: integer;
    LoopCounter: integer;
    sl: TStringList;
    Accept: Boolean;

    function GetLinkIndex(F : TILField) : integer;
    var
      Indx : integer;
    begin
       Indx := Fields.IndexOf(F.FieldName);
       Result := Fields.Items[Indx].FieldID;
    end;

    function NeedsSpace : Boolean;
    var
      Data : TSubFile;
    begin
      if not Connected then
      begin
        TMan.SetNames;
        Tman.FLibContainer := TFileContainer.Create(Tman.FLibPath, 0, True);
        Data := TMan.FLibContainer.FFiles[TMan.FLibContainer.IndexOf(FKeyListName)];
        Result := (IndexSpace > Data.RecordSpace);
        TMan.CloseFiles;
      end
      else
       Result := (IndexSpace > FSF_KeyList.RecordSpace);

    end;

  begin
    if TMan = nil then
      HandleILException(E_ManagerNotFound);
    if csDesigning in ComponentState then
      HandleILException(E_NotDesigntime);
    if not Tman.Connected then
      HandleILException(E_NoConnection);
    if (Self is TILView) or (Self is TILMemTable) then
      HandleILException(0, 'Operation not appliccable for this table type');
    if not Fields.HasIndexes then
      Exit;
    if not TMan.FGarbaging and NeedsSpace then
    begin
      DBAdmin.AlterIndexSpace;
      { This runs RebuildIndexes}
      Exit;
    end;

    TMan.FProgressTicker := GetTickCount;
    TotLoops := RecordCount * 2;
    if not Assigned(FKeyList) then
    begin
      FKeyList := TList.create;
      FLinkList := TList.Create;
    end
    else
     ClearIndex;
    s_linkList := TStringList.Create;
    s_Keylist := TStringList.Create;
    s_Keylist.Sorted := True;
    s_Keylist.Duplicates := DupIgnore;
    s_KeyIds := TStringList.Create;
    FTranslateLookups := false;
    LoopCounter := 0;
    FIndexing := True;

    try
      { Create the keylist }
      for i := 0 to RecordCount - 1 do
      begin
        Inc(LoopCounter);
        TMan.Progress(Self, iloExtractIndexes, TotLoops, LoopCounter);
        Rec := RecordsAbs[i];
        {if Rec.DeletedMoved or Rec.DeletedTrans then
        begin
          Rec.Free;
          Continue;
        end;}
        if Rec.Deleted then
        begin
          if not Rec.DeletedTrans and not Rec.DeletedMoved and not Rec.DeletedUser then
          { Deleted orphans are not indexed as deleted }
          begin
           Rec.Free;
           Continue;
          end;

          Cnt := s_Keylist.Count;
          s_Keylist.Add('Delete');
          if Cnt < s_Keylist.Count then
          begin
            snum1 := IntToStr(s_Keylist.Count - 1);
            snum1 := FillZeroes(snum1, 7);
            s_KeyIds.Add(snum1);
          end;
        end;
        for j := 0 to Rec.Count - 1 do
        begin
          FldDef := Rec[j].FieldDef;
          if not FldDef.Indexed then
            Continue;
          if Rec[j].FValues.Count = 0 then
          begin
            Val := c_EmptyIndexName;
            if s_Keylist.Count > 0 then
            begin
              if s_Keylist[0] <> Val then
              begin
                s_Keylist.Add(Val);
                snum1 := IntToStr(s_Keylist.Count - 1);
                snum1 := FillZeroes(snum1, 7);
                s_KeyIds.Add(snum1);
              end
            end
            else
            begin
                s_Keylist.Add(Val);
                snum1 := IntToStr(s_Keylist.Count - 1);
                snum1 := FillZeroes(snum1, 7);
                s_KeyIds.Add(snum1);
            end;
            Continue;
         end;

          if FldDef.DataType = dtText then
            sl := GetWords(Rec[j].FValues)
          else if FldDef.DataType = dtResource then
            sl := GetResNames(Rec[j].FValues)
          else
            sl := Rec[j].FValues;
          try
            for k := 0 to sl.Count - 1 do
            begin
              if (FldDef.DataType = dtText)
              and Assigned(DataHandler.FOnIndexWord) then
              begin
                Accept := True;
                DataHandler.FOnIndexWord(Self, sl[k], Accept);
                if not Accept then
                  Continue;
              end;
              Val := SetDateNumberFormat(sl[k], FldDef.DataType,
                FldDef.Decimals);
              Cnt := s_Keylist.Count;
              if Length(Val) > IndexSpace then
                Val := Copy(Val, 1, IndexSpace);
              s_Keylist.Add(Val);
              if Cnt < s_Keylist.Count then
              begin
                snum1 := IntToStr(s_Keylist.Count - 1);
                snum1 := FillZeroes(snum1, 7);
                s_KeyIds.Add(snum1);
              end;
            end;
          finally
            if FldDef.DataType in [dtText, dtResource] then
              sl.Free;
          end;
        end;
        Rec.Free;
      end;
      { Create the links }
      for i := 0 to RecordCount - 1 do
      begin
        Inc(LoopCounter);
        TMan.Progress(Self, iloExtractIndexes, TotLoops, LoopCounter);
        Rec := RecordsAbs[i];
        if Rec.Deleted then
        begin
          s_Keylist.Find('Delete', Indx);
          snum1 := IntToStr(Indx);
          snum1 := FillZeroes(snum1, 7);
          snum2 := IntToStr(Rec.RecordIndex);
          snum2 := FillZeroes(snum2, 7);
          snum3 := '99';
          snum3 := FillZeroes(snum3, 2);
          s_linkList.Add(snum1 + '|' + snum3 + '|' + snum2);
        end;
        for j := 0 to Rec.Count - 1 do
        begin
          FldDef := Rec[j].FieldDef;
          if not FldDef.Indexed then
            Continue;

          if Rec[j].FValues.Count = 0 then
          begin
            Val := c_EmptyIndexName;
            s_Keylist.Find(Val, Indx);
            snum1 := IntToStr(Indx);
            snum1 := FillZeroes(snum1, 7);
            snum2 := IntToStr(Rec.RecordIndex);
            snum2 := FillZeroes(snum2, 7);
            snum3 := IntToStr(GetLinkIndex(Rec[j]));
            snum3 := FillZeroes(snum3, 2);
            s_linkList.Add(snum1 + '|' + snum3 + '|' + snum2);
            Continue;
          end;

          if FldDef.DataType = dtText then
            sl := GetWords(Rec[j].FValues)
          else if FldDef.DataType = dtResource then
            sl := GetResNames(Rec[j].FValues)
          else
            sl := Rec[j].FValues;
          try
            for k := 0 to sl.Count - 1 do
            begin
              if (FldDef.DataType = dtText)
              and Assigned(DataHandler.FOnIndexWord) then
              begin
                Accept := True;
                DataHandler.FOnIndexWord(Self, sl[k], Accept);
                if not Accept then
                  Continue;
              end;
              Val := SetDateNumberFormat(sl[k], FldDef.DataType,
                FldDef.Decimals);
              s_Keylist.Find(Val, Indx);
              snum1 := IntToStr(Indx);
              snum1 := FillZeroes(snum1, 7);
              snum2 := IntToStr(Rec.RecordIndex);
              snum2 := FillZeroes(snum2, 7);
              snum3 := IntToStr(GetLinkIndex(Rec[j]));
              snum3 := FillZeroes(snum3, 2);
              s_linkList.Add(snum1 + '|' + snum3 + '|' + snum2);
            end;
          finally
            if FldDef.DataType in [dtText, dtResource] then
              sl.Free;
          end;
        end;
        Rec.Free;
      end;

      s_linkList.Sorted := True;
      FSF_KeyList.Sorted := false;
      if Assigned(FSF_KeyList.FIndexTable) then
        FSF_KeyList.FIndexTable.Sorted := false;
      FSF_LinkList.Sorted := false;
      TMan.Progress(Self, iloChangeOperation, 0, 0);
      LoopCounter := 0;
      TotLoops := s_Keylist.Count + s_linkList.Count;
      try
        Cnt := s_Keylist.Count;
        for i := 0 to Cnt - 1 do
        begin
          Inc(LoopCounter);
          TMan.Progress(Self, iloBuildIndexes, TotLoops, LoopCounter);
          AddKey(0,s_Keylist[i], StrToInt(s_KeyIds[i]),-1);
        end;
        Cnt := s_linkList.Count;
        for i := 0 to Cnt - 1 do
        begin
          Inc(LoopCounter);
          TMan.Progress(Self, iloBuildIndexes, TotLoops, LoopCounter);
          AddKey(1, s_Linklist[i], 0, -1);
        end;
        FNextKeyValId := s_KeyIds.Count + 1;
        Tman.FCfgFile.WriteInteger(TableName, 'NextKeyIndex', FNextKeyValId);
      finally
        FSF_KeyList.Sorted := True;
        if Assigned(FSF_KeyList.FIndexTable) then
          FSF_KeyList.FIndexTable.Sorted := True;
        FSF_LinkList.Sorted := True;
      end;

      SaveIndex;

    finally
      FIndexing := false;
      s_linkList.Free;
      s_Keylist.Free;
      s_KeyIds.Free;
      FTranslateLookups := True;
    end;
    if not TMan.FGarbaging then
     TMan.Progress(Self, iloFinished, 0, 0);
  end;

  procedure TILTable.TextDifference(OldText, NewText, Adds,
    Deletes: TStringList);

  var
    slOld, slNew: TStringList;
    i: integer;
  begin
    slOld := GetWords(OldText);
    slNew := GetWords(NewText);

    { Check adds }
    for i := 0 to slNew.Count - 1 do
    begin
      if slOld.IndexOf(slNew[i]) = -1 then
        Adds.Add(slNew[i]);
    end;

    { Check Deletes }
    for i := 0 to slOld.Count - 1 do
    begin
      if slNew.IndexOf(slOld[i]) = -1 then
        Deletes.Add(slOld[i]);
    end;
    try
      slOld.Free;
      slNew.Free;
    finally

    end;
  end;

  procedure TILTable.ClearTextIndex(const RecordIndex: integer; const FieldName: string);

  var
    Indx, KeyIndx: integer;
    S, s2: string;
    sl: TStringList;
    R: TILDataRecord;
    i: integer;
    ValIndx: string;
    FldIndx : integer;
    N, N2 : TKey;

  begin
    FldIndx := Fields.IndexOf(FieldName);
    if FldIndx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    Indx := Fields.Items[FldIndx].FieldID;
    if Fields.Items[FldIndx].DataType <> dtText then
    begin
      Comment1 := FieldName;
      HandleILException(E_FieldDataTypeMismatch, FieldName);
    end;

    S := '|' + FillZeroes(IntToStr(Indx), 2) + '|' +
      FillZeroes(IntToStr(RecordIndex), 7);
    R := RecordsX[RecordIndex];
    Indx := R.IndexOf(FieldName);
    sl := GetWords(R[Indx].FValues);
    try
      for i := 0 to sl.Count - 1 do
      begin
        if FindKey(0, sl[i], KeyIndx) then
        begin
          N := TKey(FKeyList[KeyIndx]);
          ValIndx := FillZeroes(IntToStr(N.id), 7);
          s2 := ValIndx + S;
          if FindKey(1, s2, Indx) then
          begin
            N2 := TKey(FLinkList[Indx]);
            N2.Free;
            FLinkList.Delete(Indx);
          end
          else
            Continue;

          { Check if this was the last reference to this value }
          FindKey(1, ValIndx, Indx);
          if Indx = FLinkList.Count then
          begin
            N := TKey(FKeyList[KeyIndx]);
            N.Free;
            FKeyList.Delete(KeyIndx);
          end
          else
          begin
            N := TKey(FLinkList[Indx]);
            s2 := N.Key;
            if Pos(ValIndx, s2) <> 1 then
            begin
              N := TKey(FKeyList[KeyIndx]);
              N.Free;
              FKeyList.Delete(KeyIndx);
            end
          end;
        end;
      end;
    finally
      sl.Free;
      R.Free;
    end;
  end;

  procedure TILTable.UpdateTextIndex(DataRecord: TILDataRecord;
    const FieldName: string; RecordIndex: integer);

  var
    FldIndx, Indx, KeyIndx: integer;
    S, s2: string;
    i: integer;
    ValIndx: integer;
    ValIndxS: string;
    Accept: Boolean;
    Dels, Adds: TStringList;
    OldRec: TILDataRecord;
    N, N2 : TKey;

  begin
    Indx := Fields.IndexOf(FieldName);
    FldIndx := Indx;
    Comment1 := '';
    if Indx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName);
    end;
    if Fields.Items[Indx].DataType <> dtText then
    begin
      Comment1 := FieldName;
      HandleILException(E_FieldDataTypeMismatch);
    end;
    Indx := DataRecord.IndexOf(FieldName);
    if DataRecord.RecordIndex <> -1 then
      Adds := TStringList.Create
    else
      Adds := GetWords(DataRecord[Indx].FValues);
    Dels := TStringList.Create;

    try
      if RecordIndex <> -1 then
      begin
        OldRec := RecordsX[RecordIndex];
        try
          TextDifference(OldRec[Indx].FValues, DataRecord[Indx].FValues,
            Adds, Dels);
        finally
          OldRec.Free;
        end;
      end;

      if RecordIndex = -1 then
        RecordIndex := RecordCount;
      for i := 0 to Dels.Count - 1 do
      begin
        if FindKey(0, Dels[i], KeyIndx) then
        begin
          N := TKey(FKeyList[KeyIndx]);
          ValIndxS := FillZeroes(IntToStr(N.id), 7);
          S := '|' + FillZeroes(IntToStr(Fields.Items[FldIndx].FieldId), 2) + '|' +
            FillZeroes(IntToStr(RecordIndex), 7);
          s2 := ValIndxS + S;
          if FindKey(1, s2, Indx) then
          begin
            N2 := TKey(FLinkList[Indx]);
            N2.Free;
            FLinkList.Delete(Indx);
            FIndexUpdated := True;
          end
          else
            Continue;

         { Check if this was the last reference to this value }
          FindKey(1, ValIndxS, Indx);
          if Indx = FLinkList.Count then
          begin
            N := TKey(FKeyList[KeyIndx]);
            N.Free;
            FKeyList.Delete(KeyIndx);
            FIndexUpdated := True;
          end
          else
          begin
            N := TKey(FLinkList[Indx]);
            s2 := N.Key;
            if Pos(ValIndxS, s2) <> 1 then
            begin
              N := TKey(FKeyList[KeyIndx]);
              N.Free;
              FKeyList.Delete(KeyIndx);
              FIndexUpdated := True;
            end
          end;
        end;
      end;

      for i := 0 to Adds.Count - 1 do
      begin
        S := '|' + FillZeroes(IntToStr(Fields.Items[FldIndx].FieldID), 2) + '|' +
          FillZeroes(IntToStr(RecordIndex), 7);
        if Assigned(DataHandler.FOnIndexWord) then
        begin
          Accept := True;
          DataHandler.FOnIndexWord(Self, Adds[i], Accept);
          if not Accept then
            Continue;
        end;
        KeyIndx := AddKey(0, Adds[i], 0, -2);
        if KeyIndx <> -1 then
        begin
          N := TKey(FKeyList[KeyIndx]);
          ValIndx := FNextKeyValId;
          N.id := ValIndx;
          S := FillZeroes(IntToStr(ValIndx), 7);
          S := S + '|' + FillZeroes(IntToStr(Fields.Items[FldIndx].FieldID), 2) + '|' +
            FillZeroes(IntToStr(RecordIndex), 7);
        end
        else
        begin
          FindKey(0, Adds[i], ValIndx);
          N := TKey(FKeyList[ValIndx]);
          S := FillZeroes(IntToStr(N.Id), 7) + S;
        end;

        AddKey(1, S, 0, -2);
        Inc(FNextKeyValId);
        FIndexUpdated := True;
      end;

    finally
      Adds.Free;
      Dels.Free;
      GetKeyWordID(false);
    end;
  end;

  procedure TILTable.BuildTextIndex(DataRecord: TILDataRecord;
    const FieldName: string; const RecordIndex: integer);

  { Must be called after the indexes have been cleared }
  var
    Indx : integer;
    S: string;
    sl: TStringList;
    i: integer;
    ValIndx: integer;
    Accept: Boolean;

  begin
    Indx := Fields.IndexOf(FieldName);
    if Indx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    if Fields.Items[Indx].DataType <> dtText then
      HandleILException(E_FieldDataTypeMismatch, FieldName);

    Indx := DataRecord.IndexOf(FieldName);
    sl := GetWords(DataRecord[Indx].FValues);
    try
      for i := 0 to sl.Count - 1 do
      begin
        if Assigned(DataHandler.FOnIndexWord) then
        begin
          Accept := True;
          DataHandler.FOnIndexWord(Self, sl[i], Accept);
          if not Accept then
            Continue;
        end;


        ValIndx := FNextKeyValId;
        AddKey(0, sl[i], ValIndx, -2);
        S := FillZeroes(IntToStr(ValIndx), 7);
        S := S + '|' + FillZeroes(IntToStr(Fields.Items[Indx].FieldID), 2) + '|' +
          FillZeroes(IntToStr(DataRecord.RecordIndex), 7);
        AddKey(1, S, 0, -1);
        Inc(FNextKeyValId);
      end;
    finally
      sl.Free;
      GetKeyWordID(false);
    end;
  end;

{  procedure TILTable.RebuildTextIndex(DataRecord: TILDataRecord;
    FieldName: string; RecordIndex: integer);
  begin
    ClearTextIndex(RecordIndex, FieldName);
    BuildTextIndex(DataRecord, FieldName, RecordIndex);
  end;}

  function TILTable.SetDateNumberFormat(const AValue: string; const ADataType: TDataType;
    const DecimalCount: integer; const UseLocale : Boolean = false): string;

  var
    fmt: TFormatSettings;
    DT: TDateTime;
    N, F: Extended;
    i: int64;
    S: string;
    P: integer;
    Neg: Boolean;
    DeciCount : integer;
    Val : string;
  begin
    Val := AValue;
    Result := AValue;
    DeciCount := DecimalCount;
    if DeciCount < 0 then
     DeciCount := 0;
    if (AValue = '') or (AValue = '''' + '''') then
      Exit;
    if ADataType = dtDate then
    begin
      if (Pos('*', AValue) <> 0) or (Pos('?', AValue) <> 0) then
        Exit;
      DT := StrToDateTime(AValue, Translator.FFormatSettings);
      fmt := TFormatSettings.Create(Translator.Language);
      if DateSearchFormat = dsfDate then
      begin
        Result := FormatDateTime('yyyymmdd', DT);
      end
      else
      begin
        Result := FormatDateTime('yyyymmddhhnnss', DT);
      end;
      Result := TrimBlanks(Result);
      { If value has sec min hour setting, a blank is added at end }
    end
    else if ADataType = dtNumber then
    begin
      Neg := (AValue[1] = '-');
      if Neg then
      begin
        Delete(Val, 1, 1);
      end;
      N := StrToFloat(Val, Translator.FFormatSettings);
      if Neg then
      begin
        N := MaxFloat - N;
      end;

      if not UseLocale then
      begin
        Val := FloatToStr(N, Translator.FEnFormatSettings);
      end;
      F := Frac(N);
      i := Trunc(N);
      S := IntToStr(i);

      Result := FillZeroes(S, 10);
      if F <> 0 then
      begin
        S := Val;
        if UseLocale then
          P := Pos(Translator.FFormatSettings.DecimalSeparator, S)
        else
         P := Pos(Translator.FEnFormatSettings.DecimalSeparator, S);
        S := Copy(S, P + 1, Length(S));
        if Length(S) > DeciCount then
          S := Copy(S, 1, DeciCount);
        while Length(S) < DeciCount do
          S := S + '0';
        if UseLocale then
          Result := Result + Translator.FFormatSettings.DecimalSeparator + S
        else
          Result := Result + Translator.FEnFormatSettings.DecimalSeparator + S;
      end;
      if Neg then
        Result := NegSign + Result;
    end;
  end;

  function TILTable.HasHandler : Boolean;
  begin
    Result := (FDataHandler <> nil);
  end;

  function TILTable.HasDatasetTemps: Boolean;
  { Checks if on or more tables contains dataset with temp fields.
    In that state it is not posbile to work with the datasets }

  var
    i: integer;
    TBL: TILTable;
    DetTbl: TILTable;
  begin
    Result := false;
    if IsDetail then
      TBL := MasterSource
    else
      TBL := Self;
    for i := 0 to Tbl.AllChildCount - 1 do
    begin
      DetTbl := Tbl.AllChildren[i];
      if DetTbl.Dataset.Count > 0 then
      begin
        Result := DetTbl.Dataset.Items[0].HasTempFields;
        if Result then
          Break;
      end;
    end;
    if not Result then
      if Tbl.Dataset.Count > 0 then
        Result := Tbl.Dataset.Items[0].HasTempFields;
  end;

  procedure TILTable.ResetDataset;

  { Clears all datasets. If datasets contain tmporary fields, resets the
    columns list. This method can be called from both master and detail }
  var
    i: integer;
    TBL: TILTable;
    HasTemps: Boolean;
    DetTbl: TILTable;
  begin
    if IsDetail then
      TBL := MasterSource
    else
      TBL := Self;
    for i := 0 to Tbl.AllChildCount - 1 do
    begin
      DetTbl := Tbl.AllChildren[i];
      if DetTbl.Dataset.Count > 0 then
      begin
        HasTemps := DetTbl.Dataset.Items[0].HasTempFields;
        DetTbl.Dataset.Clear;
        if HasTemps then
        begin
          DetTbl.GetSearcher(DetTbl).FSelectFields.Clear;
          DetTbl.Columns.RestoreDefaults;
        end;
      end;
    end;

    if Tbl.Dataset.Count > 0 then
    begin
      HasTemps := Tbl.Dataset.Items[0].HasTempFields;
      Tbl.Dataset.Clear;
      if HasTemps then
      begin
        Tbl.GetSearcher(Tbl).FSelectFields.Clear;
        Tbl.Columns.RestoreDefaults;
      end;
    end;
  end;

  function TILTable.GetRecordCount: integer;
  begin
    if not Connected then
      Result := 0
    else if (Self is TILMemTable) then
    with Self as TILMemTable do
    begin
      Result := TILMemTable(Self).FTableRecords.Count;
    end
    else
      Result := FSF_Data.Count;
  end;

  function TILTable.GetSelection(Index: integer): TILDataRecord;
  begin
    Result := nil;
    if Dataset = nil then
      Exit;
    Result := Dataset.Selection[Index];
  end;

  function TILTable.GetActiveSearchCtrls(Index: integer): TILFieldDef;

  var
    i, j: integer;
    Ind: integer;
  begin
    Ind := -1;
    Result := nil;
    for i := 0 to TMan.TableCount-1 do
    begin
      if (TMan.Tables[i] <> Self) and not TMan.Tables[i].IsFreeTable then
        if not FReadSearchPathControls or not TMan.IsInSearchPath(Self, TMan.Tables[i]) then
          Continue;
      for j := 0 to TMan.Tables[i].Fields.Count - 1 do
      begin
        if (TMan.Tables[i].Fields.Items[j].SearchControl <> nil) then
        begin
          if IsControlActive(TMan.Tables[i].Fields.Items[j].SearchControl) then
            Inc(Ind);
          if Ind = Index then
          begin
            Result := TMan.Tables[i].Fields.Items[j];
            Exit;
          end;
        end;
      end;
    end;
  end;

  function TILTable.GetActiveSearchCtrlCount: integer;

  var
    i, j: integer;
  begin
    Result := 0;
    for i := 0 to TMan.Tablecount-1 do
    begin
      if (TMan.Tables[i] <> Self) and not TMan.Tables[i].IsFreeTable then
      begin
        if not FReadSearchPathControls or not TMan.IsInSearchPath(Self, TMan.Tables[i]) then
          Continue;
      end;
      for j := 0 to TMan.Tables[i].Fields.Count - 1 do
      begin
        if (TMan.Tables[i].Fields.Items[j].SearchControl <> nil) then
        begin
          if IsControlActive(TMan.Tables[i].Fields.Items[j].SearchControl) then
            Inc(Result);
        end;
      end;
    end;
  end;

  procedure TILTable.CalcControls;
  var
    R : TILDataRecord;
    i : integer;
  begin
    R := TILDataRecord.Create(Self, True);
    try
    R.ClearData;
    ReadControls(R, duData);
    for I := 0 to Fields.Count-1 do
    begin
      if (Fields.Items[i].FieldType = ftCalculated) and not Fields.Items[i].FUserCalc then
      if Assigned(DataHandler.FOnCalcField) then
      begin
         DataHandler.FOnCalcField(Self, R, R[i]);
      end;
    end;
    WriteControls(R, duData);
    finally
      R.Free;
    end;
  end;

  function TILTable.GetChildCount: integer;
  var
    N: TTableNode;
  begin
    Result := 0;
    if not Assigned(TMan) then
      Exit;
    N := TMan.NodeOf(Self);
    if N <> nil then
      Result := N.Count;
  end;

  function TILTable.GetAllChildCount: integer;

  var
    N: TTableNode;
  begin
    Result := 0;
    if IsFreeTable then
     Exit;
    if not Assigned(TMan) then
      Exit;
    N := TMan.NodeOf(Self);
    if N <> nil then
      Result := N.CountAll;
  end;

  function TILTable.GetChildrn(Index: integer): TILTable;

  var
    N: TTableNode;
  begin
    Result := nil;
    if IsFreeTable then
     Exit;
    if not Assigned(TMan) then
      Exit;
    N := TMan.NodeOf(Self);
    if N <> nil then
      Result := N.Children[Index].Table;
  end;

  function TILTable.GetAllChildren(Index: integer): TILTable;

  var
    N: TTableNode;
  begin
    Result := nil;
    if IsFreeTable then
     Exit;
    if not Assigned(TMan) then
      Exit;
    N := TMan.NodeOf(Self);
    if N <> nil then
      Result := N.AllChildren[Index].Table;
  end;

  function TILTable.GetParentTable : TILTable;
  var
    N: TTableNode;
  begin
    Result := nil;
    if IsFreeTable then
     Exit;
    if not Assigned(TMan) then
      Exit;
    N := TMan.NodeOf(Self);
    if (N <> nil) and (N.Parent <> nil) then
      Result := N.Parent.Table;
  end;

  function TILTable.IsTableEmpty : Boolean;
    function Check : Boolean;
    var
      SF : TSubFile;
      Indx : integer;
    begin
      Result := True;
      Indx := TMan.FLibContainer.IndexOf(FDataName);
      if Indx <> -1 then
      begin
        SF := TMan.FLibContainer.Files[Indx];
        Result := (SF.Count = 0);
      end;
    end;

  begin
       Result := True;
       if Tman = nil then
         Exit;
       if TMan.Connected then
         Result := (RecordCount = 0)
       else
       begin
         SetNames(Self);
         if not Assigned(TMan.FLibContainer) then
         begin
           TMan.FLibContainer := TFileContainer.Create(TMan.FLibPath, 0, True);
           TMan.FLibContainer.Unicode := TMan.Unicode;
           try
            Result := Check;
           finally
             FreeAndNil(TMan.FLibContainer );
           end;
         end
         else
         begin
            Result := Check;
         end;
       end;
  end;

  function TILTable.DeletedCount : integer;
  var
    FD : TILFieldDef;
    SR : TSearcher;
    sl, sl2 : TStringList;
    Complete : TIndexSearchState;
    R : TILDataRecord;
    Tbl : TILTable;
    i, j : integer;
    DelCnt : integer;
  begin
    DelCnt := 0;
    Result := 0;
    SR := nil;
    if IsDetail then
    {Check orphans }
    begin
      {Get the deleted records in mastersource }
      TBL := MasterSource;
      FD := Tbl.Fields.Add;
      FD.FIndexed := True;
      FD.FReportField := false;
      FD.FFieldName := 'SYS_Deleted';
      sl := TStringList.Create;

      sl.Add('Select * where SYS_Deleted = ' + '''' + 'Delete' + '''' );
      try
        SR := TBL.CreateSearcher(TBL, Sl);
        SR.FPrivate := True;
      finally
        sl.Free;
      end;

      SR.FChecking := True;
      sl := SR.GetFromIndexes(Complete);
      SR.Free;
      Tbl.Fields.Delete(Tbl.Fields.Count-1);

      { Count the matching records in detail. These are the orphans. They are not marked
      as deleted except if the deletion occured before they became orphans }

      for I := 0 to sl.Count-1 do
      begin
         R := Tbl.RecordsAbs[StrToInt(sl[i])];
         if R.DeletedMoved then
         begin
           R.Free;
           Continue;
         end;
         sl2 := TStringList.Create;
         sl2.Add('Select * where SYS_LinkID = ' + IntToStr(R.RecordIndex));
         R.Free;
         SR := CreateSearcher(Self, Sl2);
         SR.FPrivate := True;
         SR.FChecking := True;
         sl2.Free;
         sl2 := SR.GetFromIndexes(Complete);
         inc(Result, sl2.Count);
         SR.Free;
         { Count "hard" deletions within orphans. They must be subtracted when counting
         the actual deleted records in the table }
         for J := 0 to sl2.Count-1 do
         begin
             R := RecordsX[StrToInt(sl2[J])];
             if R.DeletedUser or R.DeletedTrans then
               inc(DelCnt);
             R.Free;
         end;
         sl2.Free;
      end;
      sl.Free;
    end;

    { Count actual deleted }
    FD := Fields.Add;
    FD.FIndexed := True;
    FD.FReportField := false;
    FD.FFieldName := 'SYS_Deleted';
    sl := TStringList.Create;
    try
     sl.Add('Select * where SYS_Deleted = ' + '''' + 'Delete' + '''' );
     try
      SR := CreateSearcher(Self, Sl);
      SR.FPrivate := True;
     finally
      sl.Free;
     end;

     SR.FChecking := True;
     sl := SR.GetFromIndexes(Complete);
     inc(Result, sl.Count);
    finally
      sl.Free;
      SR.Free;
      Fields.Delete(Fields.Count-1);
    end;
    Dec(Result, DelCnt);
  end;

  function TILTable.Count : integer;
  begin
    Result := RecordCount-DeletedCount;
  end;

  function TILTable.GetSelCount: integer;
  begin
    Result := 0;
    if Dataset = nil then
      Exit;
    Result := Dataset.SelCount;
  end;

  procedure TILTable.GetImgDimensions(var W, H: integer);
  begin
    case ImageLocalSize of
      ils100:
        begin
          W := 100;
          H := 100;
        end;
      ils200:
        begin
          W := 200;
          H := 200;
        end;
      ils300:
        begin
          W := 300;
          H := 300;
        end;
      ils400:
        begin
          W := 400;
          H := 400;
        end;
      ils500:
        begin
          W := 500;
          H := 500;
        end;
      ils0, ilsFullSize:
        begin
          W := -1;
          H := -1;
        end;
    end;
  end;

  procedure TILTable.SetMultiSelect(const Value : Boolean);
  begin
    if FMultiSelect = Value then
      Exit;
    FMultiSelect := Value;
    Dataset.FSelState := -1;
    if not FMultiSelect and (FViewer <> nil) then
      FViewer.FShowCheckBoxes := false;
    DoDeselectAll;
    if (Dataset.Position <> -1) then
    begin
      FocusedRecord.FSelected := True;
      Dataset.FSelstate := -1;
    end;
    if HasViewer then Viewer.Invalidate;
  end;

  function TILTable.GetAlignOperators: Boolean;
  begin
    if TMan = nil then
      Result := True
    else
      Result := (TMan.SearchModel in [smAndAligned, smAndOrAligned])
  end;

  function TILTable.GetStoreImagesLocally : Boolean;
  begin
      Result := (ImageLocalSize <> ils0);
  end;

  function TILTable.CompareRecords(Record1, Record2: TILRecord;
    const Delta: Boolean): Boolean;

  { Compare values of corresponding fields }
  var
    i, j: integer;
    sl1, sl2: TStringList;
    Indx: integer;
    R1, R2: TILRecord;
  begin
    if not Delta and (Record1.Count <> Record2.Count) then
    begin
      Result := false;
      Exit;
    end;
    R1 := TILRecord.Create(Self);
    R2 := TILRecord.Create(Self);
    R1.Assign(Record1);
    R2.Assign(Record2);
    try
      Result := True;
      i := 0;
      { Remove Null fields }
      while i < R1.Count do
      begin
        if R1[i].Count = 1 then
        begin
          if TMan.IsValueNull(R1[i][0]) then
            R1.Delete(i)
          else
            Inc(i);
        end
        else
          Inc(i);
      end;
      i := 0;
      while i < R2.Count do
      begin
        if R2[i].Count = 1 then
        begin
          if TMan.IsValueNull(R2[i][0]) then
            R2.Delete(i)
          else
            Inc(i);
        end
        else
          Inc(i);
      end;

      for i := 0 to R1.Count - 1 do
      begin
        Indx := R2.IndexOf(R1[i].FieldName);
        if (Indx = -1) then
        begin
          if Delta then
            Continue
          else
          begin
            Result := false;
            Break;
          end;
        end;
        sl1 := R1.ValueOf(R1[i].FieldName);
        sl2 := R2.ValueOf(R1[i].FieldName);
        try
          if sl1.Count <> sl2.Count then
          begin
            Result := false;
            Exit;
          end;
          for j := 0 to sl1.Count - 1 do
            If (sl2.IndexOf(sl1[j]) = -1) then
            begin
              Result := false;
              Break;
            end;
          if not Result then
            Break;
        finally
          sl1.Free;
          sl2.Free;
        end;
      end;

      if not Result then
        Exit;

      for i := 0 to R2.Count - 1 do
      begin
        Indx := R1.IndexOf(R2[i].FieldName);
        if (Indx = -1) then
        begin
          if Delta then
            Continue
          else
          begin
            Result := false;
            Break;
          end;
        end;
        sl1 := R1.ValueOf(R2[i].FieldName);
        sl2 := R2.ValueOf(R2[i].FieldName);
        try
          if sl1.Count <> sl2.Count then
          begin
            Result := false;
            Exit;
          end;
          for j := 0 to sl1.Count - 1 do
            If (sl2.IndexOf(sl1[j]) = -1) then
            begin
              Result := false;
              Break;
            end;
          if not Result then
            Break;
        finally
          sl1.Free;
          sl2.Free;
        end;
      end;
    finally
      R1.Free;
      R2.Free;
    end;
  end;

  procedure TILTable.WndMethod(var Msg : TMessage);
  var
  Handled: Boolean;
  Pan : TILControlPanel;
  L : TILFieldDef;
  Indx : integer;
begin
  Handled := True;
  case Msg.Msg of
    WM_RunFocus :
    begin
       if TMan = nil then
         Exit;
       Datahandler.FOnFocusRecord(self);
    end;
    WM_UpdateRptColumns:
    begin
      if TMan = nil then
        Exit;
      Columns.AssignSource;
      if HasViewer then Viewer.Invalidate;
    end;
    WM_UpdatePanel:
    { Called when a field or a dac is deleted. Rebuilds the panel and
    the manager dml work fields}
    begin
      { L is the field def. See TILTable.Notification and TILFieldDef.Notify.}
      if TMan = nil then
        Exit;
      Pan := nil;
      if Msg.LParam <> 0 then
      begin
        Pan := TILControlPanel(Msg.LParam);
        if RegPanels.IndexOf(Pan) = -1 then
          Exit;
      end;
      L := nil;
      if Msg.WParam <> 0 then
        L := TILFieldDef(Msg.WParam);
      if Pan <> nil then
      with Pan do
      begin
        if L <> nil then
        begin
           if FDeletionUSG = duSearch then
             FreeAndNil(L.FSearchLabel)
           else
             FreeAndNil(L.FDataLabel);

           if DataHandler = nil then
            Indx := -1
           else
            Indx := DataHandler.DataFields.IndexOf(L.FieldName);
           if Indx <> -1 then
           begin
             if FDeletionUSG = duSearch then
               DataHandler.DataFields.Items[Indx].FSearchLabel := nil
             else
               DataHandler.DataFields.Items[Indx].FDataLabel := nil;
           end;
        end;

        if ShowLogOpButtons then
          RemoveLogOpButtons;
        CreateItems;
        if ShowLogOpButtons then
          CreateLogOpButtons;
        if (TMan.UseCommandModes) then
        begin
           if csDesigning in ComponentState then
           begin
              if FDeletionUSG = duSearch then
                TMan.CreateWorkFields(Self, cmSearch)
              else
              begin
                TMan.CreateWorkFields(Self, cmUpdate);
              end;
           end
           else
              TMan.CreateWorkFields(TMan.ActiveTable, TMan.CommandMode);

           ShowCtrls(TMan.FWorkFields);
           TMan.UpdateWorkList;
        end
        else
        begin
          ShowCtrls(FItems);
        end;
      end
      else if (TMan.UseCommandModes) then
      begin
        TMan.CreateWorkFields(TMan.ActiveTable, TMan.CommandMode);
        TMan.UpdateWorkList;
      end
    end;
    else
      Handled := False;
  end;
  if Handled then
    Msg.Result := 0
  else
    Msg.Result := DefWindowProc(FHWnd, Msg.Msg,
      Msg.WParam, Msg.LParam);
end;

  procedure TILTable.SetTableName(const Value : string);
  var
    i : integer;
  begin
    if Value = '' then
      Exit;
    if (csLoading in ComponentState) then
     FTableName := Value
    else if (Self is TILView) then
    begin
     for I := 0 to TMan.TableCount-1 do
       begin
         if (TMan.Tables[i] <> Self) then
         begin
           if SameText(Value, TMan.Tables[i].TableName) then
             HandleILException(E_DUPTableName);
         end;
       end;
      FTableName := Value;
    end
    else
      FTableName := Value;
  end;

  procedure TILTable.SaveImage(AThumbnail: Vcl.Graphics.TBitmap;
    var Offset, Count: int64);

  var
    MS: TMemoryStream;
    P: int64;
  begin
    MS := TMemoryStream.Create;
    try
      AThumbnail.SaveToStream(MS);
      P := TMan.FThumbs.Seek(0, SoEnd);
      AThumbnail.SaveToStream(TMan.FThumbs);
      Offset := P;
      Count := MS.Size;
    finally
      MS.Free;
    end;
  end;

  function TILTable.GetJPEGQuality;

  var
    W, H: integer;
  begin
    GetImgDimensions(W, H);
    if (W > 400) or (H > 400) or (W = -1) then
      Result := jsHalf
    else if (W > 200) or (H > 200) then
      Result := jsQuarter
    else
      Result := jsEighth;
  end;

  procedure TILTable.GetDetails;

  { Retrievs detail records of a one to many image relationship }
  var
    MID: integer;
    sl, sl2: TStringList;
    i : integer;
    FSQL : TILSQL;
  begin
    if not IsDetail then
      Exit;
    Fields.ClearCalcFields();
    if FMasterSource.Dataset.FIsJoined then
      Exit;

    if FMasterSource.Dataset.Position = -1 then
    begin
      Dataset.Clear;
      Exit;
    end;

    MID := FMasterSource.Dataset.Items[FMasterSource.Dataset.Position].RecordIndex;
    if MID = -1 then
    begin
      Dataset.Clear;
      Exit;
    end;
    sl := TStringList.Create;
    try
      sl.Add(c_SysFieldNames[SYS_LinkID] + '=' + IntToStr(MID));

      if Datahandler.FilterState = fsUseDacs then
      begin
        sl2 := DBView.ControlsToSQL(False);
        try
         for I := 0 to sl2.Count-1 do
          sl.Add(sl2[i]);
        finally
         sl2.Free;
        end;
      end
      else if Datahandler.FilterState = fsUseFilter then
      begin
        FSQL := TILSQL.Create(Self);
        try
          FSQL.FNoCheck := True;
          FSQL.FIsFilter := True;
          FSQL.TargetTable := Self;
          FSQL.SetSQLStatement(DataHandler.Filter);
          if Assigned(Datahandler.FOnFilter) then
          begin
           Datahandler.FOnFilter(DataHandler, FSQL.Params);
          end;
          if FSQL.Params.Count > 0 then
          begin
            FSQL.SetValues;
            FDataHandler.Filter.Assign(FSQL.FValuedSQL);
          end;

          for i := 0 to Datahandler.Filter.Count-1 do
          begin
           if i = 0 then
             sl.Add('And ' + Datahandler.Filter[i])
           else
             sl.Add(Datahandler.Filter[i]);
          end;
          if FSQL.Params.Count > 0 then
            FDataHandler.Filter.Assign(FSQL.SQLStatement);
        finally
          FSQL.Free;
        end;
      end;

      if Self.Search(sl) = 0 then
      begin
        Dataset.Clear;
        Inval;
      end;
    finally
      sl.Free;
    end;
  end;

  function TILTable.GetKeyWordID(Save : Boolean) : integer;
  begin
    Result := FNextKeyValId;
    Inc(FNextKeyValId);
    if Save then
     Tman.FCfgFile.WriteInteger(TableName, 'NextKeyIndex', FNextKeyValId);
  end;

  function TILTable.GetLookupKeyField(ALookupTable : TILTable) : TILFieldDef; {Gets the lookup field looTableKey}
  var
    i : integer;
  begin
    Result := nil;
    for i := 0 to Fields.Count-1 do
    begin
      if (Fields.Items[i].LookupType = looTableKey) and (Fields.Items[i].LookupTable = AlookupTable) then
      begin
        Result := Fields.Items[i];
        Break;
      end;
    end;
  end;

  procedure TILTable.Loaded;
  var
    i : integer;
  begin
    if TMan <> nil then
     FreeNotification(TMan);
    for I := 0 to Fields.Count-1 do
    begin
      if Fields.Items[i].DataControl is TCheckListBox then
      with Fields.Items[i].DataControl as TCheckListBox do
      begin
        AllowGrayed := True;
        CheckAll(cbGrayed);
      end;
      if Fields.Items[i].SearchControl is TCheckListBox then
      with Fields.Items[i].SearchControl as TCheckListBox do
      begin
        AllowGrayed := True;
        CheckAll(cbGrayed);
      end;

    end;
    Inherited;
  end;

  procedure TILTable.Notification(AComponent: TComponent;
    Operation: TOperation);

  var
    i, j: integer;
    Dac : TControl;
    Pan : TILControlPanel;
    IsCtrl : Boolean;
    PanFound : Boolean;
    indx : integer;
    D : TILData;

    procedure SetHandler(FieldName : string; AControl : TControl);
    begin
      if DataHandler = nil then
        Indx := -1
      else
        Indx := DataHandler.DataFields.IndexOf(FieldName);
      if Indx = -1 then
        Exit;
      if AControl = DataHandler.DataFields.Items[Indx].FDataControl then
        DataHandler.DataFields.Items[Indx].FDataControl := nil
      else if AControl = DataHandler.DataFields.Items[Indx].FSearchControl then
        DataHandler.DataFields.Items[Indx].FSearchControl := nil
      else if AControl = DataHandler.DataFields.Items[Indx].FSearchLabel then
        DataHandler.DataFields.Items[Indx].FSearchLabel := nil
      else if AControl = DataHandler.DataFields.Items[Indx].FDataLabel then
        DataHandler.DataFields.Items[Indx].FDataLabel := nil;
    end;

  begin
    inherited Notification(AComponent, Operation);
    if AComponent = nil then
      Exit;
    if TMan = nil then
      Exit;
    if (csDestroying in ComponentState) or ((TMan <> nil) and (csDestroying in TMan.ComponentState)) then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent = MasterSource then
        FMasterSource := nil
      else if AComponent = FViewer then
      begin
        FViewer := nil;
      end
      else if (AComponent = DataHandler) then
      begin
        D := DataHandler;
        for I := 0 to D.DataFields.Count-1 do
        begin
          Indx := Fields.IndexOf(D.DataFields.Items[i].FieldName);
          if Indx <> -1 then
          begin
            if Assigned(Fields.Items[Indx].DataControl) then
            begin
               if Fields.Items[Indx].DataControl.Parent is TILControlPanel then
               begin
                 Fields.Items[Indx].FDataControl := nil;
                 if Assigned(Fields.Items[Indx].DataLabel) then
                 begin
                   Fields.Items[Indx].FDataLabel := nil;
                 end;
               end
               else
               begin
                 Fields.Items[Indx].DataControl := nil;
                 Fields.Items[Indx].DataLabel := nil;
               end;
            end;
            if Assigned(Fields.Items[Indx].SearchControl) then
            begin
               if Fields.Items[Indx].SearchControl.Parent is TILControlPanel then
               begin
                 Fields.Items[Indx].SearchControl := nil;
                 begin
                   Fields.Items[Indx].FSearchLabel := nil;
                 end;
               end
               else
               begin
                 Fields.Items[Indx].SearchControl := nil;
                 Fields.Items[Indx].SearchLabel := nil;
               end;
            end;
          end;
        end;
        FDataHandler := nil;
        TMan.SyncTable(Self);
      end;

      Dac := nil;
      for i := 0 to Fields.Count - 1 do
      begin
        if Fields.Items[i].FDataControl = AComponent then
        begin
          Dac := Fields.Items[i].FDataControl;
          Fields.Items[i].FDataControl := nil;
          SetHandler(Fields.Items[i].FieldName, Dac)
        end;
        if Fields.Items[i].FSearchControl = AComponent then
        begin
          Dac := Fields.Items[i].FSearchControl;
          Fields.Items[i].FSearchControl := nil;
          SetHandler(Fields.Items[i].FieldName, Dac)
        end;
        if Fields.Items[i].FSearchLabel = AComponent then
        begin
          Dac := Fields.Items[i].FSearchLabel;
          Fields.Items[i].FSearchLabel := nil;
          SetHandler(Fields.Items[i].FieldName, Dac)
        end;
        if Fields.Items[i].FDataLabel = AComponent then
        begin
          Dac := Fields.Items[i].FDataLabel;
          Fields.Items[i].FDataLabel := nil;
          SetHandler(Fields.Items[i].FieldName, Dac);
        end;
        if Fields.Items[i].FLookupTable = AComponent then
        begin
          Fields.Items[i].FLookupTable := nil;
          Fields.Items[i].FLookupField := '';
          Fields.Items[i].FLookupSearchField := '';
        end;
      end;
      if Dac <> nil then
      begin
        PanFound := False;
        for I := 0 to RegPanels.Count-1 do
        begin
          Pan := TILControlPanel(RegPanels[i]);
          TMan.FPanelBuild := True;
          for j := 0 to Pan.Count-1 do
            if (Pan.Items[j].Control = Dac) or
            (Pan.Items[j].ControlLabel = Dac)
            then
            begin
              IsCtrl := (Pan.Items[j].Control = Dac);
              if IsCtrl and (Pan.Items[j].ControlLabel <> nil) then { Not deleted }
              begin
                { Set usage flag to know which label to dlete }
                if Pan.Items[j].Usage = duSearch then
                  Pan.FDeletionUSG := duSearch
                else
                  Pan.FDeletionUSG := duData;
                PostMessage(FHWnd, wm_UpdatePanel, Integer(Pan.Items[j].Field), Integer(Pan));
              end
              else
              begin
                Pan.FDeletionUSG := Pan.Items[j].Usage;
                PostMessage(FHWnd, wm_UpdatePanel, 0, Integer(Pan));
              end;
              PanFound := True;
              Break;
            end;
            if PanFound then
              Break;
        end;
        if not PanFound then
           PostMessage(FHWnd, wm_UpdatePanel, 0, 0);
      end;
    end;
  end;

  procedure TILTable.SetData(const Value: TILFieldDefs);
  begin
    FFields.Assign(Value);
  end;

  procedure TILTable.SetNames(ATable: TILTable);
  var
    TBL : TILTable;

  begin
    if ATable.TableName = '' then
      HandleILException(E_MissingTableName, ATable.Name);
    if ATable is TILMemTable then
      Exit;
    if ATable is TILView then
      Tbl := TILView(ATable).SourceTable
    else
      Tbl := ATable;

    ATable.FKeyListName := Tbl.TableName + '_Keys';
    ATable.FLinkListName := Tbl.TableName + '_KeyLinks';
    ATable.FKeyIDsName := Tbl.TableName + '_KeyIDS';
    ATable.FDataName := Tbl.TableName + '_Data';
    ATable.FComputerName := GetCompName('');
  end;

  function TILTable.GetThumbnail(const Offset, Count: int64; const Scale : Boolean = True): Vcl.Graphics.TBitmap;

  var
    MS: TMemoryStream;
  begin
    MS := TMemoryStream.Create;
    Result := Vcl.Graphics.TBitmap.Create;
    try
      TMan.FThumbs.Position := Offset;
      MS.CopyFrom(TMan.FThumbs, Count);
      MS.Position := 0;
      try
        Result.LoadFromStream(MS);
      except
        Result.Free;
        raise;
      end;
      if HasViewer and Scale then
      if not(FViewer.ImageWidth = Result.Width) or not(FViewer.ImageHeight = Result.Height) then
        ScaleBitmap(Result, FViewer.ImageWidth, FViewer.ImageHeight, false);
    finally
      MS.Free;
    end;
  end;

  function TILTable.GetImage(DataRecord: TILDataRecord): Vcl.Graphics.TBitmap;
  var
    MS: TMemoryStream;
  begin
    Result := nil;
    if DataRecord.ThumbnailSize > 0 then
    begin
      MS := TMemoryStream.Create;
      Result := Vcl.Graphics.TBitmap.Create;
      try
        TMan.FThumbs.Position := DataRecord.ThumbnailOffset;
        MS.CopyFrom(TMan.FThumbs, DataRecord.ThumbnailSize);
        MS.Position := 0;
        try
          Result.LoadFromStream(MS);
        except
          Result.Free;
          raise;
        end;
      finally
        MS.Free;
      end;
    end
    else if DataRecord.FileName <> '' then
    begin
      if not FileExists(DataRecord.FileName) then
      begin
        Comment1 := DataRecord.FileName;
        HandleILException(E_FileNotExists);
      end;
      Result := GetBitmapFromFile(DataRecord.FileName, GetJPEGQuality);
      if Result = nil then
      begin
        Comment1 := DataRecord.FileName;
        HandleILException(E_UnsupportedFileFormat);
      end;
      if GetFileType(DataRecord.FileName) = iftJPG then
        CheckOrientation(DataRecord.FileName, Result);
      if FImageLocalSize = ils100 then
        ScaleBitmap(Result, 100, 100)
      else if FImageLocalSize = ils200 then
        ScaleBitmap(Result, 200, 200)
      else if FImageLocalSize = ils300 then
        ScaleBitmap(Result, 300, 300)
      else if FImageLocalSize = ils400 then
        ScaleBitmap(Result, 400, 400)
      else if FImageLocalSize = ils500 then
        ScaleBitmap(Result, 500, 500)
    end;
  end;

  function TILTable.GetImage(const DatasetIndex: integer): Vcl.Graphics.TBitmap;

  { The public version }
  var
    DataRec: TILDataRecord;
  begin
    if (DatasetIndex < 0) or (DatasetIndex > Dataset.Count - 1) then
    begin
      Comment1 := IntToStr(DatasetIndex);
      HandleILException(E_OutOfBounds);
    end;
    DataRec := Dataset.Items[DatasetIndex];
    Result := GetImage(DataRec);
  end;

  function TILTable.GetConnected: Boolean;
  begin
    Result := FConnected and Assigned(TMan) and Assigned(TMan.FLibContainer);
  end;

  procedure TILTable.SetConnected(const Value : Boolean);
  begin
    if Value and Connected then
      Exit;
    if not Value and not Connected then
      Exit;

    if Value then
      Connect
    else
      Disconnect;
  end;

  procedure TTableManager.ConnectSilent;
  begin
    FConnectSilent := True;
    try
      Connected := True;
    finally
     FConnectSilent := false;
    end;
  end;

  function TILTable.GetAlias : string;
  var
    S : string;
  begin
    S := Translator.GetTranslation(False, TableName);
    if S <> '' then
      Result := S
    else if FAlias <> '' then
     Result := FAlias
    else
     Result := TableName;
  end;

  function TILTable.XMLToRecord(const ALibstring: string; const SkipImage: Boolean = false; FieldSource : TILFieldDefs = nil)
    : TILDataRecord;

  { Converts a libstring (XML) to a datarecord }
  var
    Val: string;
    Fld: TILField;
    Indx: integer;
    i, j: integer;
    DT: TDateTime;
    S: string;
    ImgRec: TILDataRecord;
    LInt : integer;
    NF : Double;
    NI : integer;
    sl : TStringList;
    TagNm : string;
    OldTags : Integer;
    FD : TILFieldDefs;

  function GetValue(AFieldName: string; AFieldID : integer; AnIndex: integer): string;
  var
    P1, P2: integer;
    StartIdx, EndIdx, StartPos: integer;
    Substr: string;
    i: integer;

  begin
    if AnIndex <> -1 then { User fields }
    begin
      StartPos := Pos('<Data>', ALibstring);
      if StartPos = 0 then
       StartPos := Pos('<'+ XData + '>', ALibString);
      P1 := PosCaseless('<F' + IntToStr(AFieldID) + '>', ALibstring, StartPos);
      P2 := PosCaseless('</F' + IntToStr(AFieldID) + '>', ALibstring, StartPos);
      StartIdx := P1 + Length('<F' + IntToStr(AFieldID) + '>');
      if P1 = 0 then
      begin
       P1 := PosCaseless('<' + AFieldName + '>', ALibstring, StartPos);
       P2 := PosCaseless('</' + AFieldName + '>', ALibstring, StartPos);
       if (AFieldName = 'Value') or (AFieldName = XValue) then
         P2 := Pos('</' + AFieldName + '>', ALibstring, P2 + 1);
       StartIdx := P1 + Length('<' + AFieldName + '>');
      end;

      EndIdx := P2;
      Substr := Copy(ALibstring, StartIdx, EndIdx - StartIdx);
      StartPos := 1;
      for i := 0 to AnIndex do
      begin
        P1 := Pos('<Value>', Substr, StartPos);
        if P1 = 0 then
        begin
          P1 := Pos('<' + XValue + '>', Substr, StartPos);
          if P1 = 0 then
          begin
            Result := '#EOF';
            Break;
          end;
        end;
        if i = AnIndex then
        begin
          P2 := Pos('</Value>', Substr, P1);
          if P2 = 0 then
          begin
            P2 := Pos('</' + XValue + '>', Substr, P1);
            StartIdx := P1 + Length('<' + XValue + '>');
          end
          else
           StartIdx := P1 + Length('<Value>');
          EndIdx := P2;
          Result := Copy(Substr, StartIdx, EndIdx - StartIdx);
          Exit;
        end;
        StartPos := P1 + 1;
      end;
    end
    else { System fields }
    begin
      P1 := PosCaseless('<' + AFieldName + '>', ALibstring);
      P2 := PosCaseless('</' + AFieldName + '>', ALibstring);
      if P1 = 0 then
        Result := ''
      else
      begin
       StartIdx := P1 + Length('<' + AFieldName + '>');
       EndIdx := P2;
       Result := Copy(ALibstring, StartIdx, EndIdx - StartIdx);
      end;
    end;
  end;

  function ReplaceAmpers(S : string) : string;
  begin
    Result := S;
    Result := StringReplace(Result, '&amp;', '&', [rfReplaceAll]);
    Result := StringReplace(Result, '&lt;', '<', [rfReplaceAll]);
    Result := StringReplace(Result, '&gt;', '>', [rfReplaceAll]);
    Result := StringReplace(Result, '&quot;', '"', [rfReplaceAll]);
    Result := StringReplace(Result, '&apos;', '''', [rfReplaceAll]);
  end;

  function GetSysTag(OldTagname, NewTagname : string) : string;
  { This is a fix when xml structure was changed }
  begin
      if OldTags = 0 then
        Result := OldTagName
      else if OldTags = 1 then
        Result := NewTagName;
  end;

  begin
    if Pos('<Record>', ALibString) <> 0 then
      OldTags := 0
    else
      OldTags := 1;

    Result := TILDataRecord.Create(Self, false);
    try
    Result.Clear;
    Result.FOwnsImage := True;

    if Pos('{Mov', ALibString) = 1 then
    begin
      Result.FState := 'MO';
      Exit;
    end;

    { First create the system fields }
    TagNm := GetSysTag('RecordIndex', XRecIndex);
    S := GetValue(TagNm,0, -1);
    if S = '' then
      Result.FRecordIndex := -1
    else
      Result.FRecordIndex := StrToInt(S);

    TagNm := GetSysTag('RecordId', XRecId);
    S := GetValue(TagNm, 0, -1);
    if S = '' then
      Result.FRecordID := -1
    else
      Result.FRecordID := StrToInt(S);

    TagNm := GetSysTag(c_SysFieldNames[SYS_RecordState], XRecState);
    Result.FState := GetValue(TagNm, 0, -1);

    TagNm := GetSysTag(c_SysFieldNames[SYS_RegDate], XRegDate);
    Result.FRegDate :=
      StrToFloat(GetValue(TagNm, 0, -1), Translator.FEnFormatSettings);

    TagNm := GetSysTag(c_SysFieldNames[SYS_ChangedDate], XChangeDate);
    Result.ChangedDate :=
      StrToFloat(GetValue(TagNm, 0, -1), Translator.FEnFormatSettings);

    TagNm := GetSysTag(c_SysFieldNames[SYS_MediaType], XMedia);
    if GetValue(TagNm, 0, -1) = '' then
    begin
      TagNm := GetSysTag(c_SysFieldNames[SYS_FileName], XImgFile);
      S := GetValue(TagNm, 0, -1);
      if TryStrToint(S, NI) then
      begin
        S := Result.GetTextLob(NI);
      end;
      if IsImageFile(S) then
        Result.FMediaType := MED_IMAGE
      else if IsVideoFile(S) then
        Result.FMediaType := MED_VIDEO
      else if IsAudioFile(S) then
        Result.FMediaType := MED_AUDIO
      else
        Result.FMediaType := MED_NONE;
    end
    else
    begin
      TagNm := GetSysTag(c_SysFieldNames[SYS_MediaType], XMedia);
      Result.FMediaType :=
        StrToInt(GetValue(TagNm, 0, -1));
    end;

    TagNm := GetSysTag(c_SysFieldNames[SYS_ThumbnailOffset], XImgOffset);
    Result.FThumbnailOffset := StrToInt(GetValue(TagNm, 0, -1));

    TagNm := GetSysTag(c_SysFieldNames[SYS_ThumbnailSize], XImgSize);
    Result.FThumbnailSize := StrToInt(GetValue(TagNm, 0, -1));

    TagNm := GetSysTag(c_SysFieldNames[SYS_FileName], XImgFile);
    Result.FFileName := GetValue(TagNm, 0, -1);
    Result.FileName := ReplaceAmpers(Result.FileName);

    Result.FUserID := GetValue(XUserID, 0, -1);

    { Check refs to image table. The reference is stored in thumbnailoffset when filename is <ImageTable>.
    It is stored in FImgTableIndex and saved back at saving point }

    if Result.FileName = IMAGE_TABLE then
    begin
      if TMan.SharedImages = nil then
        HandleILException(E_MissingLookupTable);
      Result.FImgTableIndex := Result.FThumbnailOffset;
      ImgRec := TMan.SharedImages.RecordsX[Result.FImgTableIndex];
      try
        Result.FThumbnailOffset := ImgRec.FThumbnailOffset;
        Result.FThumbnailSize := ImgRec.FThumbnailSize;
        Result.FileName := ImgRec.FileName;
        if not TMan.FIsServer then
          Result.FFileName := ToUNC(ImgRec.FFileName, TMan.FServer);
        Result.FImgFilePos := -1;
      finally
        ImgRec.Free;
      end;
    end
    else if TryStrToint(Result.FFileName, NI) then
    begin
        if NI > -1 then
         Result.FFileName := Result.GetTextLob(NI)
        else
         Result.FFileName := '';
        Result.FImgFilePos := NI;
        if not TMan.FIsServer then
          Result.FFileName := ToUNC(Result.FFileName, TMan.FServer);
    end
    else
    begin
        Result.FImgFilePos := -1;
    end;

    { User fields }
    if FieldSource = nil then
      FD := Fields
    else
      FD := FieldSource;
    for i := 0 to FD.Count - 1 do
    begin
      Indx := 0;
      Fld := TILField.Create(Self, FD.Items[i].FieldName, FD);
      Fld.FOwnerRecord := Result;
      if not(FD.Items[i].FieldType = ftCalculated) and not (FD.Items[i].LookupType = looParentField)  then
      begin
        Val := GetValue(FD.Items[i].FieldName, FD.Items[i].FieldID, Indx);
      end
      else
      begin
        Val := '';
        Result.Add(Fld);
        Continue;
      end;
      while Val <> '#EOF' do
      begin
        if (Fld.FieldDef.LookupType in [looTableRef, looTableKey]) and (Fld.FieldDef.LookupTable = nil) then
          HandleILException(E_MissingLookupTable);
        if IsStringType(Fld.FieldDef.DataType) then
          Val := RevertESQCodes(Val);
        if (Fld.FieldDef.LookupType = looTableRef) and FTranslateLookups then
        begin
          if not TryStrToInt(Val, LInt) then
            HandleILException(E_InvalidLookupRef);
          Val := GetLookupTableValue(Fld.FieldDef.LookupTable, Fld.FieldDef.LookupField, LInt);
        end;
        if IsStringType(Fld.FieldDef.DataType) then
        begin
          Val := ReplaceAmpers(Val);
        end;
        if (Fld.FieldDef.DataType = dtDate) and (Val <> '') then
        begin
          if not TryStrToFloat(Val, NF, Translator.FEnFormatSettings) then
            DT := 0
          else
            DT := StrToFloat(Val, Translator.FEnFormatSettings);
          Val := DateTimeToStr(DT, Translator.FFormatSettings);
        end
        else if (Fld.FieldDef.DataType = dtNumber) and (Val <> '') then
        begin
          { Float values are stored in eng. Convert to locale format }
          if Fld.FieldDef.Decimals > 0 then
          begin
            NF := StrToFloat(Val, Translator.FEnFormatSettings);
            Val := FLoatToStr(NF, Translator.FFormatSettings);
          end;

        end
        else if (Fld.FieldDef.DataType = dtResource) and (Val <> '') then
        begin
          if TryStrToint(Val, NI) then
          begin
            if NI > -1 then
              Val := Result.GetTextLob(NI)
            else
              Val := '';
            Fld.FResFilePos := NI;
            { Val is a concatenation of all strings in the value list. Split them.}
            sl := TStringList.Create;
            try
              Fld.FValues.Clear;
              CreateListFromDelimiter(c_ResourceStrSeparator, Val, sl);
              for j := 0 to sl.Count-1 do
              begin
                Fld.FValues.Add(sl[j]);
              end;
            finally
              sl.Free;
            end;
            Break;
          end
        end;

        if Val <> '' then
        begin
          Fld.FValues.Add(Val);
        end;
        Inc(Indx);
        Val := GetValue(FD.Items[i].FFieldName, FD.Items[i].FFieldID, Indx);
      end;
      Result.Add(Fld);

      if Fld.FieldDef.ImageText and (Result.ImageText = '') then
        for j := 0 to Fld.FValues.Count - 1 do
        begin
          Result.ImageText := Result.ImageText + Fld.FValues[j];
          if j < Fld.FValues.Count - 1 then
            Result.ImageText := Result.ImageText + #13#10;
          Result.FOldImageText := Result.ImageText;
        end;

    end;

    if SkipImage then
    begin
      Result.FImage := nil;
      Result.FImageFetched := false;
    end
    else if Result.FThumbnailSize > 0 then
    begin
      Result.FImage := GetThumbnail(Result.FThumbnailOffset,
        Result.FThumbnailSize);
      Result.FImageFetched := True;
    end
    else if not Result.MustStoreLocally(Result.FileName) then
    begin
      if Result.FileName <> '' then
        Result.LoadImage(Result.FileName)
      else
        Result.FImage := nil;
    end;

    except
      Result.Free;
      raise;
    end;

  end;

  function TILTable.RecordToJSON(ARecord : TILDataRecord) : string;
  var
    j, k: integer;
    S: string;
    F : TILField;

  begin
    S := '{';
    S := S + #13#10;
    S := S + '"RecordIndex": ' + '"' + IntToStr(ARecord.RecordIndex) + '",' + #13#10;
    S := S + '"Filename": ' + '"' + ARecord.FileName + '",' + #13#10;
    for j := 0 to ARecord.ColCount - 1 do
    begin
        F := ARecord[ARecord.Columns[j].FieldName];
        if F.FieldDef.DataType = dtLongText then
          Continue;
        S := S + '"' + ARecord.Columns[j].FieldName + '": ';
        if F.Count = 0 then
          S := S + '""'
        else if F.Count = 1 then
          S := S + '"' + F.AsString + '"'
        else
        begin
          S := S + '[';
          for k := 0 to F.Count-1 do
          begin
            if k < F.Count-1 then
              S := S + '"' + F.Items[k] + '",' + #13#10
            else
            begin
              S := S + '"' + F.Items[k] + '"' + ']';
            end;
          end;
        end;

        if j< ARecord.ColCount-1 then
          S := S + ',' + #13#10
        else
          S := S + #13#10;
    end;
    S := S + '}';
    Result := S;
  end;


  function TILTable.RecordToXML(ARecord: TILDataRecord; const KeepFieldnames : Boolean;
   const ConvertLookups : Boolean = false): string;

  { Converts ARecord to a XML-string. Adds the FileName and ThumbIndex fields if
    necessary }
  var
    i, j: integer;
    Val: string;
    ANode, C1, C2, C3: IXMLNode;
    DT: TDateTime;
    NF : Extended;

    procedure OrderColumns;
    var
      i : integer;
      Indx : integer;
    begin
       for I := 0 to Columns.Count-1 do
       begin
         Indx := ARecord.IndexOf(Columns[i].FieldName);
         ARecord.FFields.Move(Indx, i);
       end;
    end;

  begin
    ANode := FXMLDoc.DocumentElement.AddChild(XRec);
    C1 := ANode.AddChild(XSystem);
    C2 := C1.AddChild(XRecIndex);
    C2.Text := IntToStr(ARecord.RecordIndex);
    C2 := C1.AddChild(XRecID);
    C2.Text := IntToStr(ARecord.RecordID);
    C2 := C1.AddChild(XRegDate);
    C2.Text := FloatToStr(ARecord.RegDate, Translator.FEnFormatSettings);
    C2 := C1.AddChild(XChangeDate);
    C2.Text := FloatToStr(ARecord.ChangedDate, Translator.FEnFormatSettings);
    C2 := C1.AddChild(XRecState);
    C2.Text := ARecord.FState;
    C2 := C1.AddChild(XMedia);
    C2.Text := IntToStr(ARecord.MediaType);

    C2 := C1.AddChild(XImgOffset);
    if ARecord.FImgTableIndex = -1 then
      C2.Text := FillZeroes(IntToStr(ARecord.ThumbnailOffset), 10)
    else
      { ImgTable ref stores its refernse index in Thumbnail offset }
      C2.Text := FillZeroes(IntToStr(ARecord.FImgTableIndex), 10);
    C2 := C1.AddChild(XImgSize);
    if ARecord.FImgTableIndex = -1 then
      C2.Text := FillZeroes(IntToStr(ARecord.ThumbnailSize), 10)
    else
      C2.Text := '0000000000';

    C2 := C1.AddChild(XImgFile);
    if ARecord.FImgTableIndex <> -1 then
    begin
      C2.Text := IMAGE_TABLE;
    end
    else if ARecord.FImgFilePos <> -1 then
     C2.Text := IntToStr(ARecord.FImgFilePos)
    else
     C2.Text := ARecord.FileName;

    if ARecord.UserID <> '' then
    begin
       C2 := C1.AddChild(XUserID);
       C2.Text := ARecord.UserID;
    end;


    { Write the user data : }
    if KeepFieldNames then
      OrderColumns;
    C1 := ANode.AddChild(XData);
    for i := 0 to ARecord.Count - 1 do
    begin
      if ARecord[i].FieldDef.FieldType in [ftCalculated, ftStoredEvaluation] then
        Continue;
      if ARecord[i].FieldDef.LookupType = looParentField then
        Continue;
      if KeepFieldNames and (Columns.IndexOf(ARecord.Items[i].FieldDef.FieldName) = -1) then
        Continue;

      if KeepFieldNames then
        C2 := C1.AddChild(ARecord.Items[i].FieldName)
      else
        C2 := C1.AddChild('F' + IntToStr(ARecord.Items[i].FieldDef.FieldID));
      for j := 0 to ARecord[i].FValues.Count - 1 do
      begin
        Val := ARecord[i].FValues[j];
        if not(ARecord[i].FieldDef.DataType = dtText) then
          Val := TrimBlanks(Val);
        if IsStringType(ARecord[i].FieldDef.DataType) then
        begin
          Val := SetESCCodes(Val);
          { Sets codes for wildcard symbols as literals }
        end;
        if ConvertLookups and (ARecord[i].FieldDef.LookupType = looTableRef) then
        { Calls from Optimize method only }
        begin
          Val := GetLookupTableID(ARecord[i].FieldDef.LookupTable, ARecord[i].FieldDef.LookupField,
            Val);
        end;
        if SameText(ARecord[i].FieldName, c_SysFieldNames[SYS_LinkID]) then
          Val := FillZeroes(Val, 7);
        if ARecord[i].FieldDef.DataType = dtDate then
        begin
          DT := StrToDateTime(Val, Translator.FFormatSettings);
          Val := FloatToStr(DT, Translator.FEnFormatSettings);
        end
        else if ARecord[i].FieldDef.Datatype = dtNumber then
        begin
          { Save float numbers in standard english format }
          if ARecord[i].FieldDef.Decimals > 0 then
          begin
            NF := StrToFloat(Val, Translator.FFormatSettings);
            Val := FloatToStr(NF, Translator.FEnFormatSettings);
          end;
        end;
        C3 := C2.AddChild(XValue);
        C3.Text := Val;
      end;
    end;
    Result := ANode.Xml;
    FXMLDoc.DocumentElement.ChildNodes.Clear;
  end;

  function TILDataRecord.ValueOfPropField(const PID: integer): TStringList;

  var
    i, j: integer;
    V: Variant;
    Cnt: integer;
    Found: Boolean;
    PP: TPropertyProvider;
  begin
    Result := TStringList.Create;
    if IsUrl(FileName) then
      Exit;
    Found := false;

    if Assigned(PropertyValues) then
      for i := 0 to PropertyValues.Count - 1 do
      begin
        if PropertyValues.Items[i].FPID = PID then
        begin
          if VarIsArray(PropertyValues.Items[i].Value) then
          begin
            V := PropertyValues.Items[i].Value;
            Cnt := VarArrayHighBound(V, 1);
            for j := 0 to Cnt do
            begin
              Result.Add(VarToStr(V[j]));
            end;
          end
          else
            Result.Add(VarToStr(PropertyValues.Items[i].Value));
          Found := True;
        end;
      end;
    if not Found and (FileName <> '') then
    begin
      PP := TPropertyProvider.Create(FileName);
      try
        V := PP.GetPropValueAsVariant(PID);
        if VarIsNull(V) then
          Exit;
        if VarIsArray(V) then
        begin
          Cnt := VarArrayHighBound(V, 1);
          for j := 0 to Cnt do
          begin
            Result.Add(VarToStr(V[j]));
          end;
        end
        else
        begin
          Result.Add(VarToStr(V));
        end;

      finally
        PP.Free;
      end;
    end;

  end;

  function TILDataRecord.ValueOfPropField(const PIDName: string): TStringList;

  var
    PID: integer;
  begin
    PID := PIDOf(PIDName);
    Result := ValueOfPropField(PID);
  end;

  function TILDataRecord.ValueOfSysField(const ASysNameID: integer)
    : TStringList;

  var
    Indx: integer;
  begin
    if ASysNameID >= 100 then
    begin
      Result := ValueOfPropField(ASysNameID);
      Exit;
    end;
    Result := TStringList.Create;
    if ASysNameID = SYS_RecordID then
      Result.Add(IntToStr(RecordID))
    else if ASysNameID = SYS_ThumbnailOffset then
      Result.Add(IntToStr(ThumbnailOffset))
    else if ASysNameID = SYS_ThumbnailSize then
      Result.Add(IntToStr(ThumbnailSize))
    else if ASysNameID = SYS_FileName then
      Result.Add(FileName)
    else if ASysNameID = SYS_RegDate then
      Result.Add(DateTimeToStr(RegDate, Translator.FFormatSettings))
    else if ASysNameID = SYS_ChangedDate then
      Result.Add(DateTimeToStr(ChangedDate, Translator.FFormatSettings))
    else if ASysNameID = SYS_RecordState then
      Result.Add(FState)
    else if ASysNameID = SYS_LinkID then
    begin
      Indx := IndexOf(c_SysFieldNames[SYS_LinkID]);
      if Indx <> -1 then
        Result.Add(Items[Indx].AsString);
    end
    else if ASysNameID = SYS_Level then
    begin
      Indx := IndexOf(c_SysFieldNames[SYS_Level]);
      if Indx <> -1 then
        Result.Add(Items[Indx].AsString);
    end
    else if ASysNameID = SYS_SiblingIndex then
    begin
      Indx := IndexOf(c_SysFieldNames[SYS_SiblingIndex]);
      if Indx <> -1 then
        Result.Add(Items[Indx].AsString);
    end;
  end;

  function TILDataRecord.ValueOfSysField(const ASysName: string): TStringList;

  var
    i: integer;
  begin
    Result := nil;
    if PosCaseless('PID_', ASysName) <> 0 then
    begin
      Result := ValueOfPropField(ASysName);
      Exit;
    end;
    for i := 1 to c_SysFieldCount do
      if SameText(ASysName, c_SysFieldNames[i]) then
      begin
        Result := ValueOfSysField(i);
        Break;
      end;
  end;

  function TILTable.IndexOfID(ADataRecord: TILDataRecord; var FieldNames : string): integer;
  var
    Xpr: TStringList;
    i: integer;
    S: string;
    SR: TSearcher;
    Records: TStringList;
    Complete: TIndexSearchState;
    R : TILDataRecord;
  begin
    FieldNames := '';
    Result := -1;
    if not Connected then
      Exit;
    if DisableIndexUpdate then
      Exit;
    if FSF_Data.Count = 0 then
      Exit;
    SR := nil;
    Records := nil;
    Xpr := TStringList.Create;
    try
      S := '';
      for i := 0 to ADataRecord.Count - 1 do
      begin
        FieldNames := FieldNames + ADataRecord[i].FieldName;
        if i <  ADataRecord.Count - 1 then
          FieldNames := FieldNames + ', ';

        if ADataRecord[i].FieldDef.Identifier then
        begin
          if (ADataRecord[i].Count = 0) then
          begin
            Exit;
          end;
          if Xpr.Count > 0 then
            S := '\';
          if ADataRecord[i].FieldDef.DataType in [dtString, dtResource] then
            S := S + ADataRecord[i].FieldDef.FieldName + '=' + '''' +
              ADataRecord[i].AsString + ''''
          else
            S := S + ADataRecord[i].FieldDef.FieldName + '=' + ADataRecord
              [i].AsString;
          S := SetESCCodes(S);
          Xpr.Add(S);
          S := '';
        end;
      end;
      if Xpr.Count = 0 then
        Exit;
      SR := CreateSearcher(Self, Xpr);
      SR.FChecking := True;
      SR.FPrivate := True;
      Result := SR.Validate;
      if Result <> 0 then
       Exit;
      Result := -1;
      Records := SR.GetFromIndexes(Complete);
      if (Records <> nil) and (Records.Count > 0) and (Complete = issComplete)
      then
      begin
        if StrToInt(Records[0]) = ADataRecord.RecordIndex then
          Exit;
        R := RecordsX[StrToInt(Records[0])];
        try
        if R.Deleted then
          Records.Clear
        else
        begin
          Result := StrToInt(Records[0]);

        end;
        finally
          R.Free;
        end;
      end;
    finally
      Xpr.Free;
      if SR <> nil then
        SR.Free;
      if Records <> nil then
        Records.Free;
    end;
  end;

  function TILTable.IndexOfUnique(ADataRecord : TILDataRecord; var FieldName : string) : integer;
  var
    SR : TSearcher;
    SL : TStringList;
    S : string;
    Complete : TIndexSearchState;
    Records: TStringList;
    F : TILField;
    i : integer;
    R : TILDataRecord;
  begin
   Result := -1;
   FieldName := '';
   SL := TStringList.Create;
   try
   for I := 0 to ADataRecord.Count-1 do
   begin
     F := ADataRecord[i];
     SL.Clear;
     if not F.FieldDef.Unique then
       Continue;
     if F.IsEmpty then
       Continue;

     S := '';
     if F.FieldDef.DataType in [dtString, dtResource, dtDate] then
      S := S + F.FieldName + ' = ' + '''' + F.AsString + ''''
     else
      S := S + F.FieldName + ' = ' + F.AsString;
     S := SetESCCodes(S);
     SL.Add(S);
     SR := CreateSearcher(Self, SL);
     SR.FChecking := True;
     Result := SR.Validate;
     if Result <> 0 then
     begin
       SR.Free;
       Exit;
     end;
     Result := -1;
     Records := SR.GetFromIndexes(Complete);
     try
       if (Records <> nil) and (Records.Count > 0) and (Complete = issComplete) then
       begin
        if StrToInt(Records[0]) = ADataRecord.RecordIndex then
          Exit;
        R := RecordsX[StrToInt(Records[0])];
        try
         if R.Deleted then
           Records.Clear
         else
         begin
           Result := StrToInt(Records[0]);
           FieldName := F.FieldName + ': ' + R[F.FieldName].AsString;
           Break;
         end;
        finally
          R.Free;
        end;
      end;
     finally
       SR.Free;
       if Records <> nil then
        Records.Free;
     end;
   end;
   finally
     SL.Free;
   end;
  end;

  procedure TILTable.SaveResourceStrings(ARecord : TILDataRecord);
  { When saving changes or inserting new resource strings must be saved to
  text lobs with a file positions reference }
  var
    i, j : integer;
    Sres : string;
  begin
      SRes := '';
      for I := 0 to ARecord.Count-1 do
      begin
        if ARecord[i].FieldDef.DataType = dtResource then
        begin
          { Resources are concatenated and saved as one single string in text lobs}
          ARecord[i].FResourceStrings := TStringList.Create;
          { Preserve literal names to use in RestoreResourceStrings}
          ARecord[i].FResourceStrings.Assign(ARecord[i].FValues);
          for j := 0 to ARecord[i].FValues.Count-1 do
          begin
            SRes := SRes + ARecord[i].FValues[j];
            if J < ARecord[i].FValues.Count-1 then
              SRes := SRes + c_ResourceStrSeparator;
          end;
          { Store file pos in first entry, will be expanded when read from disk }
          ARecord[i].FResFilePos := ARecord.SaveFileResourceName(-1, SRes);
          ARecord[i].FValues.Clear;
          ARecord[i].FValues.Add(IntToStr(ARecord[i].FResFilePos));
          { Save it as a single number }
        end;
      end;
  end;

  procedure TILTable.RestoreResourceStrings(ARecord : TILDataRecord);
  { After saving the resource string to text lobs, the literal names must be restored }
  var
    i : integer;
  begin
     for I := 0 to ARecord.Count-1 do
      begin
        if ARecord[i].FieldDef.DataType = dtResource then
        begin
          ARecord[i].FValues.Assign(ARecord[i].FResourceStrings);
        end;
        ARecord[i].FResourceStrings.Free;
      end;
  end;

  procedure TILTable.SetRecordStates(ARecord : TILDataRecord);
  begin
    ARecord.FOldImage := ARecord.FImage;
    ARecord.FOldState := ARecord.FState;
    ARecord.FOldImageText := ARecord.FImageText;
    ARecord.FOldFileName := ARecord.FFileName;
    ARecord.FOldTableIndex := ARecord.FImgTableIndex;
    ARecord.FOldMediaType := ARecord.FMediaType;
  end;

  function TILTable.GetRecords(Index: integer): TILDataRecord;
  var
    IsMThreadBusy : Boolean;
  begin
    IsMThreadBusy := TMan.FMainThreadBusy;
    TMan.FMainThreadBusy := True;
    try
    Result := nil;
    if not Connected then
      Exit;
    if Self is TILMemTable then
    with Self as TILMemTable do
    begin
      Result := DBWork.CreateDataRecord;
      Result.Assign(FTableRecords[Index]);
    end
    else
      Result := XMLToRecord(FSF_Data[Index]);
    if Result.RecordIndex = -1 then
      Result.FRecordIndex := Index;
    SetRecordStates(Result);
    finally
      if not IsMThreadBusy then
        TMan.FMainThreadBusy := False;
    end;
  end;

  function TILTable.GetRecordsX(Index: integer): TILDataRecord;
  { Record without image }
  var
    IsMThreadBusy : Boolean;
  begin
    Result := nil;
    if not Connected then
      Exit;
    IsMThreadBusy := TMan.FMainThreadBusy;
    TMan.FMainThreadBusy := True;
    try

    if Self is TILMemTable then
    begin
      Result := DBWork.CreateDataRecord;
      Result.Assign(TILMemTable(Self).FTableRecords[Index]);
    end
    else
      Result := XMLToRecord(FSF_Data[Index], True);
    if Result.RecordIndex = -1 then
     Result.FRecordIndex := Index;
    SetRecordStates(Result);
    finally
      if not IsMThreadBusy then
        TMan.FMainThreadBusy := False;
    end;
  end;

  function TILTable.GetRecordsAbs(Index: integer): TILDataRecord;
  { Record with absolute index, ignoring move refrences }
  var
    IsMThreadBusy : Boolean;
  begin
    Result := nil;
    IsMThreadBusy := TMan.FMainThreadBusy;
    TMan.FMainThreadBusy := True;
    try
    if not Connected then
      Exit;
    if Self is TILMemTable then
    begin
      Result := DBWork.CreateDataRecord;
      Result.Assign(TILMemTable(Self).FTableRecords[Index]);
    end
    else
      Result := XMLToRecord(FSF_Data.AbsItems[Index], True);
    if Result.RecordIndex = -1 then
     Result.FRecordIndex := Index;
    SetRecordStates(Result);
    finally
      if not IsMThreadBusy then
        TMan.FMainThreadBusy := False;
    end;
  end;

  procedure TILTable.SetRecords(Index: integer; Value: TILDataRecord);
  var
    S, STrans: string;
    Indx: integer;
    i, j: integer;
    DR: TILDataRecord;
    MovRec : integer;
    sl : TStringList;
    DBOp : TDBOperation;
    RIndx : integer;
    IsMthreadBusy : Boolean;
  begin
    if not Connected then
      Exit;

    IsMthreadBusy := TMan.FMainThreadBusy;
    while TMan.FSystemBusy do
     sleep(50);
    TMan.FMainThreadBusy := True;
    try
    if IsReadOnly then
      HandleILException(E_ReadOnly, Name);
    if Value.Count < Fields.Count then
      HandleILException(E_RecMissingFields, '');
    if (Value.FImgTableIndex <> -1) and (TMan.SharedImages = nil)  then
      HandleILException(E_MissingLookupTable);
    Value.FChangedDate := Now;
    Indx := Value.IndexOf(c_SysFieldNames[SYS_ChangedDate]);
    if Indx <> -1 then
      Value[Indx].AsDateTime := Value.FChangedDate;
    DR := DBWork.CreateDataRecord;
    DR.Assign(Value);
    {Save the original values in a tmp variable. This is because
    lookup fields of type looTableRef must be converted from real values to ids whn saved.
    The real values are restored at the end.}
    try
      for i := 0 to Value.Count - 1 do
      begin
        if (Value[i].FieldDef.LookupType = looTableRef) then
          for j := 0 to Value[i].Count - 1 do
          begin
            S := GetLookupTableID(Value[i].FieldDef.LookupTable, Value[i].FieldDef.LookupField,
              Value[i].FValues[j]);
            if S = '' then
            begin
              Comment1 := Value[i].FValues[j];
              HandleILException(E_LookupNotRegistered);
            end;
            Value[i].FValues[j] := S;
          end
          { Check path link integrity }
          else if (Value[i].FieldDef.LookupType = looTableKey) and not Value[i].IsEmpty then
          begin
            sl := TStringList.Create;
            if Value[i].FieldDef.FDataType = dtNumber then
              sl.Add(Value[i].FieldDef.LookupField + ' = ' + Value[i].AsString)
            else
              sl.Add(Value[i].FieldDef.LookupField + ' = ' + '''' + Value[i].AsString + '''');
            try
             if not Value[i].FieldDef.LookupTable.Exists(sl, RIndx) then
             begin
               Comment1 := Value[i].AsString;
               HandleILException(E_InvalidTableKey);
             end;
             finally
               sl.Free;
             end;
          end;
      end;

      if (Index <> -1) and not Value.Deleted then
      begin
        Strans := FSF_Data[Index]
      end
      else
        STrans := '';

      if Index = -1 then
      begin
        Value.FRecordIndex := RecordCount;
        Value.FState := 'NO';
      end;

      if (Value.FFileName <> '') and (Value.FImgTableIndex = -1) then
      begin
        if (Value.FFileName <> Value.FOldFileName) or (Value.FImgFilePos = -1) then
          Value.FImgFilePos := Value.SaveFileResourceName(-1, Value.FFileName);
          {We don't reuse pages for file names, in order to keep the transaction loggin intact }

      end
      else
         Value.FImgFilePos := -1;

      SaveResourceStrings(Value);
      { This converts literal resource names to file pos }

      { Text lobs are not part of transaction }
      for I := 0 to Value.Count-1 do
      if Value[i].FieldDef.DataType = dtLongText then
        if Value[i].FTextUpdated then
           Value[i].SaveLT;

      S := RecordToXML(Value, False);

      RestoreResourceStrings(Value);
      { Back to literals }

      if Index <> -1 then
        MovRec := FSF_Data.HasSpace(Index, S)
      else
        MovRec := -1;

      { Indexes are not saved in trans file. They are restored auto
      on roll back }
      UpdateIndexes(Value, Index, MovRec);

      if (Index <> -1) and not Value.Deleted and not Value.FUndeleted then
      begin
        DBOp := dboUpdate;
        TMan.AddToTransLog(TableName, Value.RecordIndex, DBOp, STrans, S, Value);
      end
      else if (Index = -1) or Value.FUndeleted then
      begin
        DBOp := dboInsert;
        TMan.AddToTransLog(TableName, RecordCount-1, DBOp, '', S, Value);
      end
      else
      begin
        DBOp := dboDelete;
        TMan.AddToTransLog(TableName, Value.RecordIndex, DBOp, '', S, Value);
      end;

      { Restore original values, except long text references that are actually
      looked up in theis procedure}
      for i := 0 to DR.Count - 1 do
       if Dr[i].FieldDef.DataType <> dtLongText then
        for j := 0 to DR[i].Count - 1 do
         Value[i].FValues[j] := DR[i].FValues[j];

      for i := 0 to Value.Count - 1 do
      { Check table key refrences }
      begin
        if (Self is TILDataTable) and (Value[i].FOldValues <> Value[i].FValues)
        and (Value[i].FValues.Count > 0) and (Value[i].FOldValues.Count > 0)
        and not SameText(Value[i].FValues[0], Value[i].FOldValues[0])
        and IsFieldLookupSource(looTableKey, Value[i].FieldDef) then
          TILDataTable(Self).UpdateTableKey(Value[i].FieldDef.FieldName,
            Value[i].FValues[0], Value[i].FOldValues[0]);
        Value[i].FieldDef.FIsUpdated := True;
      end;

      if Self is TILDataTable then
      begin
        { if lookup source, sync clients}
        TILDataTable(Self).UpdateParentField(Value);
        { If has parent fields, sync them}
        TILDataTable(Self).FetchParentFields(Value);
      end;
      ResetOldValues(Value);
    finally
      DR.Free;
    end;
    finally
      if not IsMthreadBusy then
        TMan.FMainThreadBusy := false;
    end;
  end;

  function TILTable.GetIndexSpace: integer;

  var
    i: integer;
    Tmp: integer;
  begin
    Result := 0;
    for i := 0 to Fields.Count - 1 do
    begin
      if not Fields.Items[i].Indexed then
        Continue;
      Tmp := ComputeFieldSpace(Fields.Items[i]);
      if Tmp > Result then
        Result := Tmp;
    end;
    if Result < 26 then
      Result := 26;
    { There is always one indexed field the "Deleted" field }
  end;


  function TILTable.GetMasterSource: TILDataTable;
  begin
    Result := FMasterSource;
  end;

  function TILTable.GetMasterLinks(Index : integer) : TILDataTable;
  begin
     Result := TILDataTable(FMasterLinks[Index]);
  end;

  function TILTable.GetMasterLinkCount : integer;
  begin
     Result := FMasterLinks.Count;
  end;

    function TILTable.GetDetailLinks(Index : integer) : TILDataTable;
  begin
     Result := TILDataTable(FDetailLinks[Index]);
  end;

  function TILTable.GetDetailLinkCount : integer;
  begin
     Result := FDetailLinks.Count;
  end;

  function TILTable.GetMultiSelected : Boolean;
  begin
    Result := FMultiSelect and (Dataset.SelCount > 1);
  end;

  procedure TILTable.ClearEdits;
  var
    i: integer;
  begin
    if (FViewer <> nil) and (roEditable in FViewer.ReportOptions) then
    begin
      if Assigned(FViewer.FCellEditor.ActiveCtrl) then
        FViewer.FCellEditor.ActiveCtrl.Visible := false;
      if Assigned(FViewer.FCellEditor.Cell) then
        FViewer.FCellEditor.Cell.FEditing := false;

      for i := 0 to AllChildCount - 1 do
      begin
        if AllChildren[i].FViewer <> nil then
        begin
        if Assigned(AllChildren[i].FViewer.FCellEditor.ActiveCtrl) then
          AllChildren[i].FViewer.FCellEditor.ActiveCtrl.Visible := false;
        if Assigned(AllChildren[i].FViewer.FCellEditor.Cell) then
          AllChildren[i].FViewer.FCellEditor.Cell.FEditing := false;
        end;
      end;
    end;
  end;

  function TILTable.GetDataset : TILDataset;
  begin
       Result := FDataset;
  end;

  function TILTable.GetFields : TILFieldDefs;
  begin
      Result := FFields;
  end;

  function TILTable.GetColumns : TILColumns;
  begin
      Result := FColumns;
  end;

  procedure TILTable.SetDataset(const Value : TILDataset);
  var
    i : integer;
    MH : TILMessageHandler;
  begin
    if Value = FDataset then
      Exit;
    if Value = nil then
      HandleILException(E_DatasetNil, '');
    if Value.Count > 0 then
    begin
      for I := 0 to Value[0].Count-1 do
        if Fields.IndexOf(Value[0][i].FFieldName) = -1 then
          HandleILException(E_DatasetMismatch, '');
    end;

    FImgHScrollPos := 0;
    ClearEdits;
    if HasViewer then
    if (Viewer.ZoomImage <> nil) then
      Viewer.ZoomImage.ClearSource;
    Dataset.Free;

    FDataset := Value;
    FDataset.Ilib := Self;

    if FDataset.Count > 0 then
    begin
     if HasViewer then
     with FViewer do
     begin
      FirstItemIndex := 0;
      FTLImages := 0;
      FTLReport := 0;
      CreateReport;
      UpdateScrollRange;
      UpdateScrollPos;
      Viewer.Invalidate;
     end;
     Dataset.Sort;
     Dataset.Position := 0;
    end;
    if TVControl <> nil then
    begin
      TVControl.BuildTree(Dataset);
    end;
    if Self is TILTreeViewTable then
    with Self as TILTreeViewTable do
     FillSiblings(nil);
    if DataHandler <> nil then
     if Assigned(Datahandler.FOnDatasetChange) then
      Datahandler.FOnDatasetChange(Self, dsPopulated);
    begin
     I := -1;
     MH := TMan.GetMessageHandler(Self, i);
     while MH <> nil do
     begin
      MH.Reset;
      MH.AddCount(dboSearch, Dataset.Count, False);
      MH.FSearchState := 0;
      MH := TMan.GetMessageHandler(Self, i);
     end;
    end;
    Dataset.FChngTracker.Clear;

  end;

  procedure TILTable.SetDatasetMaxCount(const Value : integer);
  begin
    if Value < 0 then
      Exit;
    if Value < 500 then
      Exit;
    FDatasetMaxCount := Value;
  end;

  procedure TILTable.SetLookup(AControl: TWinControl; AField: TILFieldDef);
  { Called from field controller /tblman to restore lookp ups when controls are expanded / collapsed }
  begin
    if AControl is TEdit then
      with AControl as TEdit do
      begin
        if AField <> nil then
        { This is the case when called from FCellEditor }
        begin
          AField.FSCKeyDown := OnKeyDown;
          AField.FSCExit := OnExit;
        end;
        OnKeyDown := ControlsKeyDown;
        OnExit := ControlsExit;
      end
    else if AControl is TLabeledEdit then
      with AControl as TLabeledEdit do
      begin
        if AField <> nil then
        { This is the case when called from FCellEditor }
        begin
          AField.FSCKeyDown := OnKeyDown;
          AField.FSCExit := OnExit;
        end;
        OnKeyDown := ControlsKeyDown;
        OnExit := ControlsExit;
      end
    else if AControl is TMemo then
      with AControl as TMemo do
      begin
        AField.FSCKeyDown := OnKeyDown;
        AField.FSCExit := OnExit;
        OnKeyDown := ControlsKeyDown;
        OnExit := ControlsExit;
      end
    else if AControl is TILSearchGrid then
      with AControl as TILSearchGrid do
      begin
        AField.FSCKeyDown := OnKeyDown;
        AField.FSCExit := OnExit;
        OnKeyDown := ControlsKeyDown;
        OnExit := ControlsExit;
      end
    else if AControl is TILUpdateGrid then
      with AControl as TILUpdateGrid do
      begin
        AField.FDCKeyDown := OnKeyDown;
        AField.FDCExit := OnExit;
        OnKeyDown := ControlsKeyDown;
        OnExit := ControlsExit;
      end
  end;

  procedure TILTable.SetLookups(const Reset : Boolean = false);

  var
    SCtrl, DCtrl: TWinControl;
    i: integer;
  begin
    if FLookupsAssigned and not Reset then
      Exit;
    if not FFields.HasLookups then
    begin
      { Keep a pointer to the lookup id to avoid avs }
      if not Assigned(FLookupControl) then
      begin
       FLookupControl := TListBox.Create(Self);
       FLookupControl.Visible := false;
      end;
      Exit;
    end;

    for i := 0 to FFields.Count - 1 do
    begin
      DCtrl := nil;
      SCtrl := nil;
      if FFields.Items[i].IsLookup then
      begin
        if Assigned(FFields.Items[i].DataControl) then
          DCtrl := FFields.Items[i].DataControl;
        if Assigned(FFields.Items[i].SearchControl) then
          SCtrl := FFields.Items[i].SearchControl;
        if (SCtrl <> nil) or (DCtrl <> nil) then
          FLookupsAssigned := True;

        if Assigned(SCtrl) then
        begin
          if SCtrl is TEdit then
            with SCtrl as TEdit do
            begin
              FFields.Items[i].FSCKeyDown := OnKeyDown;
              FFields.Items[i].FSCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end
          else if SCtrl is TLabeledEdit then
            with SCtrl as TLabeledEdit do
            begin
              FFields.Items[i].FSCKeyDown := OnKeyDown;
              FFields.Items[i].FSCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end
          else if SCtrl is TMemo then
            with SCtrl as TMemo do
            begin
              FFields.Items[i].FSCKeyDown := OnKeyDown;
              FFields.Items[i].FSCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;

            end
          else if SCtrl is TILSearchGrid then
            with SCtrl as TILSearchGrid do
            begin
              FFields.Items[i].FSCKeyDown := OnKeyDown;
              FFields.Items[i].FSCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;

            end;
        end; { is assigned }

        if Assigned(DCtrl) then
        begin
          if DCtrl is TEdit then
            with DCtrl as TEdit do
            begin
              FFields.Items[i].FDCKeyDown := OnKeyDown;
              FFields.Items[i].FDCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end
          else if DCtrl is TLabeledEdit then
            with DCtrl as TLabeledEdit do
            begin
              FFields.Items[i].FDCKeyDown := OnKeyDown;
              FFields.Items[i].FDCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end
          else if DCtrl is TMemo then
            with DCtrl as TMemo do
            begin
              FFields.Items[i].FDCKeyDown := OnKeyDown;
              FFields.Items[i].FDCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end
          else if DCtrl is TILUpdateGrid then
            with DCtrl as TILUpdateGrid do
            begin
              FFields.Items[i].FDCKeyDown := OnKeyDown;
              FFields.Items[i].FDCExit := OnExit;
              OnKeyDown := ControlsKeyDown;
              OnExit := ControlsExit;
            end;
        end; { is assigned }
      end; { is lookup field }
    end; { loop }

    FLookupControl := TListBox.Create(Self);
    FLookupControl.Visible := false;
    FLookupControl.OnClick := LookUpClick;
  end;

  procedure TILTable.ClearTable;

  var
    i, j: integer;
    IsConn : Boolean;
    T : TILTable;
    procedure DoClear(Tbl : TILTable);
    begin
      Tbl.FSF_Data.Clear;
      Tbl.FSF_LinkList.Clear;
      Tbl.FSF_KeyList.Clear;
      Tman.FCfgFile.WriteInteger(TableName, 'NextRecordID', FirstRecordID);
      ClearIndex;
    end;

    begin
      IsConn := True;
      if not TMan.Connected then
      begin
        TMan.FSkipValidation := True;
        IsConn := false;
        TMan.Connected := True;
      end
      else
        DBWork.ClearDataset;
      { Check if table has lookup references }
      for I := 0 to TMan.TableCount-1 do
      begin
        T := nil;
        if TMan.Tables[i] = Self then
          T := TMan.Tables[i];
        if T <> nil then
        if T.RecordCount > 0 then
        for j := 0 to T.Fields.Count-1 do
        begin
           if T.Fields.Items[j].LookupTable = Self then
           begin
             { This is handeled by adm app}
             HandleILException(E_ClearTable, T.TableName);
           end;
        end;
      end;
      try
      DoClear(Self);
      for i := 0 to AllChildCount - 1 do
        DoClear(AllChildren[i]);

      if IsRoot and (TMan.FreeTableCount = 0) then
      begin
        TMan.FThumbs.Size := 0;
        TMan.FTextLobs.Size := 0;
        if FileExists(TMan.FTableCfgName) then
        begin
          if Assigned(TMan.FCfgFile) then
           TMan.FCfgFile.Free;
          System.SysUtils.DeleteFile(TMan.FTableCfgName);
        end;
      end;
      finally
        TMan.FSkipValidation := false;
        if not IsConn then
          TMan.Connected := false;
      end;
      TMan.ClearTranslog;
    end;

  procedure TILTable.OpenTable(const AFileName: string);

  var
    Indx: integer;

    procedure Configure;
    var
    i: integer;
    sl: TStringList;
    N: integer;
    Indx: integer;
    F: string;
    FD: TILFieldDef;
    IPr: integer;
    RCIndex: integer;
    S : string;
    Dt : TDataType;
    SC : Boolean;
    Frm : string;
  begin
      { Check calc fields:}
      if Self is TILDataTable then
      begin
        for I := 1 to 20 do
        begin
         S := 'Calc_' + TableName + '_' + IntTostr(i);
         if Tman.FCfgFile.SectionExists(S) then
         begin
          F := Tman.FCfgFile.ReadString(S, 'FieldName', '');
          Dt := TDataType(Integer(Tman.FCfgFile.ReadInteger(S, 'DataType', 0)));
          N := Tman.FCfgFile.ReadInteger(S, 'ColumnWidth', 50);;
          Frm := Tman.FCfgFile.ReadString(S, 'Formula', '');
          SC := Tman.FCfgFile.ReadBool(S, 'SearchControl', False);
          with Self as TILDataTable do
            AddUserCalcField(F, Dt, N, Frm, SC, False);
         end;
        end;
      end;

      for i := 0 to Fields.Count - 1 do
      begin
        F := Fields.Items[i].FieldName;
        if Tman.FCfgFile.SectionExists(TableName + '_' + F) then
        begin
          FD := Fields.Items[i];
          FD.FIdentifier := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'Identifier',
            FD.Identifier);
          FD.FUnique := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'Unique',
            FD.Unique);
          FD.FMaxLength := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'MaxLength',
            FD.MaxLength);
          FD.FMaxValues := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'MaxValues',
            FD.MaxValues);
          FD.FDecimals := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'Decimals',
            FD.Decimals);
          IPr := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'IndexPriority', -1);
          if IPr <> -1 then
            FD.FIndexPriority := TIndexPriority(IPr);
          FD.FMandatory := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'Mandatory',
            FD.FMandatory);
          FD.FIndexed := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'Indexed',
            FD.Indexed);
          FD.FReportField := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'ReportField',
            FD.ReportField);
          FD.FReportCaption := Tman.FCfgFile.ReadString(TableName + '_' + F,
            'ReportCaption', FD.ReportCaption);
          FD.FColumnWidth := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'ColumnWidth',
            FD.ColumnWidth);
          FD.FImageText := Tman.FCfgFile.ReadBool(TableName + '_' + F, 'ImageText',
            FD.ImageText);
          FD.FInitialValue := Tman.FCfgFile.ReadString(TableName + '_' + F,
            'InitialValue', FD.InitialValue);
          FD.FDateFormat := Tman.FCfgFile.ReadString(TableName + '_' + F,
            'DateFormat', FD.DateFormat);
          IPr := Tman.FCfgFile.ReadInteger(TableName + '_' + F, 'Alignment', -1);
          if IPr <> -1 then
            FD.FAlignment := TAlignment(IPr);
        end;
      end;

      sl := TStringList.Create;
      try
        Tman.FCfgFile.ReadSection(TableName + '_ColumnSequence', sl);
        if sl.Count > 0 then
        begin
          Columns.Clear;
        end;
        RCIndex := 0;
        for i := 0 to sl.Count - 1 do
        begin
          Indx := Fields.IndexOf(sl[i]);
          if Indx = -1 then
          begin
            Continue;
          end;
           if not Fields.Items[Indx].ReportField then
            Continue;
           N := Tman.FCfgFile.ReadInteger(TableName + '_ColumnSequence', sl[i],
            Fields.Items[Indx].ColumnWidth);
           Columns.Add(sl[i]);
           Columns.Items[RCIndex].FColumnWidth := N;
           Inc(RCIndex);
        end;

        sl.Clear;
        Tman.FCfgFile.ReadSection(TableName + '_ColumnSort', sl);
        if sl.Count > 0 then
        begin
           Columns.SortFields.Clear;
        end;
        for i := 0 to sl.Count - 1 do
        begin
          Indx := Fields.IndexOf(sl[i]);
          if Indx = -1 then
            Continue;
          N := Tman.FCfgFile.ReadInteger(TableName + '_ColumnSort', sl[i], 0);
          Columns.SortFields.Add(sl[i]);
          Columns.SortOrdering := TSortOrdering(N);
        end;
      finally
        sl.Free;
      end;
      if DataHandler <> nil then
        DataHandler.AssignColumns;
  end;

  procedure InitKeyValID;
  begin
      FNextKeyValId := Tman.FCfgFile.ReadInteger(TableName, 'NextKeyIndex', -1);
      Inc(FNextKeyValId);
  end;

  begin
    { Create essisial names }
    SetNames(Self);
    if not (Self is TILMemTable) then
    begin
      Indx := TMan.FLibContainer.IndexOf(FDataName);
      if Indx = -1 then
      begin
        Comment1 := Alias;
        TMan.Connected := False;
        HandleILException(E_TableNotInDB);
      end;
    end;

    if not Assigned(FColumns) then
      FColumns := TILColumns.Create(Self, Fields);
    if FColumns.Count = 0 then
      FColumns.AssignSource;

    if (Self is TILMemTable) then
    begin
      FConnected := True;
      Exit;
    end;

    Indx := TMan.FLibContainer.IndexOf(FDataName);
    FSF_Data := TMan.FLibContainer.Files[Indx];
    FSF_LinkList := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(FLinkListName)];
    FSF_KeyList := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(FKeyListName)];
    InitKeyValID;
    FConnected := True;
    try
      Configure;
    except
      TMan.Connected := False;
      raise;
    end;
    SetLookups;
  end;

  procedure TILTable.ClearIndex;
  var
   i : integer;
   N : TKey;
  begin
   if Assigned(FKeyList) then
   begin
     for i := 0 to FKeylist.Count-1 do
     begin
       N := TKey(FKeyList[i]);
       N.Free;
     end;
     FKeyList.Clear;
   end;

   if Assigned(FLinkList) then
   begin
     for i := 0 to FLinklist.Count-1 do
     begin
       N := TKey(FLinkList[i]);
       N.Free;
     end;
     FLinkList.Clear;
   end;
  end;

  function TILTable.FindKey(KeyType : integer; AKey : string; var Index : integer) : Boolean;
    var
    L, H, i, C: integer;
    N : TKey;
  begin
    Result := false;
    L := 0;
    if KeyType = 0 then
      H := FKeyList.Count - 1
    else
      H := FLinkList.Count - 1;
    while L <= H do
    begin
      i := (L + H) shr 1;
      if KeyType = 0 then
        N := TKey(FKeyList[i])
      else
        N := TKey(FLinkList[i]);
      C := AnsiCompareText(N.Key, AKey);
      if C < 0 then
        L := i + 1
      else
      begin
        H := i - 1;
        if C = 0 then
        begin
          Result := True;
           L := i;
        end;
      end;
    end;
    Index := L;
  end;

  function TILTable.AddKey(KeyType: Integer; AKey: string; AKeyID : integer; Index : integer) : integer;
  var
    Indx : integer;
    L : TList;
    N : TKey;
  begin
    Result := -1;
    if Index = -2 then
    begin
     if FindKey(KeyType, AKey, Indx) then
       Exit;
    end
    else
     Indx := Index;

    if KeyType = 0 then
      L := FKeyList
    else
      L := FLinkList;
    N := TKey.Create;
    N.Key := AKey;
    if Keytype = 0  then
    begin
      N.ID := AKeyID
    end
    else
      N.ID := 0;
    if (Indx < L.Count) and not (Indx = -1) then
    begin
       L.Insert(Indx, N);
       Result := Indx;
    end
    else
    begin
       L.Add(N);
       Result := L.Count - 1;
    end;
  end;

  procedure TILTable.CloseTable;
  { Called from Disconnect }
  begin
    if not Connected then
      Exit;
    if TMan.FLibPath = '' then
      Exit;
    if csDestroying in ComponentState then
      Exit;
    if Self is TILMemTable then
    with Self as TILMemTable do
    begin
      FTableRecords.Clear;
    end;
    Dataset.Clear;
    if HasViewer then
     FViewer.Repaint;
    FConnected := false;
  end;

  function TILTable.Connect: integer;
  { Note the result of this function is not used. Checking is handled
  by table manager.

  It is not possbile to do sigle table connect. Either all or none}
  var
    i: integer;
    D: TILTable;
    F: string;
  begin
    Result := 0;
    if TableName = '' then
    begin
      HandleILException(E_MissingTableName, Name);
    end;

    F := TMan.FLibPath;
    try
      OpenTable(F);
    except
      TMan.Connected := False;
      raise;
    end;
    for i := 0 to AllChildCount - 1 do
    begin
      D := AllChildren[i];
      F := TMan.FLibPath;
      D.OpenTable(F);
    end;
  end;

  procedure TILTable.Disconnect;
  var
    i: integer;
    Master: TILTable;
  begin
    if not IsRoot and not IsFreeTable then
     HandleILException(E_MethodMasterTable);
    if not IsRoot then
      CloseTable
    else
    begin
      Master := Self;
      Master.CloseTable;
      for i := 0 to AllChildCount - 1 do
      begin
        AllChildren[i].CloseTable;
      end;
    end;
  end;

  procedure TILTable.SaveAsJSON(const AFileName: string);
  var
    i : integer;
    S : string;
    R : TILDataRecord;
    sl : TStringList;
    Cnt : integer;
  begin
     S := '';
     Cnt := RecordCount;
     if Cnt-DeletedCount = 0 then
       Exit;
     S := '[';
     for I := 0 to Cnt-1 do
     begin
      R := RecordsAbs[i];
      try
        if R.Deleted then
        begin
         Continue;
        end;
        S := S + RecordToJSON(R);
        if i < Cnt-1 then
          S := S +',' + #13#10
        else
          S := S + #13#10;
      finally
         R.Free;
      end;
     end;
     S := S + ']';
     sl := TStringList.Create;
     try
      sl.Text := S;
      sl.SaveToFile(AFileName);
     finally
       sl.Free;
     end;
  end;

  procedure TILTable.SaveAsXML(const AFileName: string);
  var
    i: integer;
    R: TILDataRecord;
    S: String;
    FS: TFileStream;
    MS: TStringStream;
  begin
    if not Connected then
      Exit;
    if FileExists(AFileName) then
    begin
      FS := TFileStream.Create(AFileName, fmOpenWrite);
      FS.Size := 0;
    end
    else
      FS := TFileStream.Create(AFileName, fmCreate);
    MS := TStringStream.Create('', TEncoding.UTF8);
    try
      MS.WriteString('<' + TableName +'>');
      MS.Position := 0;
      FS.CopyFrom(MS, MS.Size);
      for i := 0 to RecordCount - 1 do
      begin
        R := RecordsAbs[i];
        if R.Deleted then
        begin
         R.Free;
         Continue;
        end;
        try
          S := RecordToXML(R, True);
        finally
          R.Free;
        end;

        MS.Size := 0;
        MS.WriteString(S);
        MS.Position := 0;
        FS.CopyFrom(MS, MS.Size);
      end;
      MS.Size := 0;
      MS.WriteString('</' + TableName +'>');
      MS.Position := 0;
      FS.CopyFrom(MS, MS.Size);
    finally
      FS.Free;
      MS.Free;
    end;
  end;

  function TILTable.GetLookups(const AFieldName : string; AnExpression: string;
   const Usage : TDataUsage): TStringList;

  var
    KeyVal, KeyID, sOut: string;
    LnkStr: string;
    FieldIndx, KeyIndx, LinkIndx: integer;
    SFieldIndx: string;
    KeyCnt, LinkCnt: integer;
    Expr: string;
    Cnt: integer;
    RecId: string;
    i: integer;
    DS : TILDataset;
    T : TILTable;
    sl : TStringList;
    F, F2 : string;
    R : TilDataRecord;
    N : TKey;
    IsRefLoo : Boolean;
    S : string;
    IsDup : Boolean;

  begin
    Result := TStringList.Create;
    FieldIndx := FFields.IndexOf(AFieldName);
    if FieldIndx = -1 then
    begin
      Comment1 := AFieldName;
      HandleILException(E_InvalidFieldName, '');
    end;

    if TrimBlanks(AnExpression) = '' then
       Exit;
    IsRefLoo := (FFields.Items[FieldIndx].LookupType in [looTableRef, looTableKey]);
    if (Trim(AnExpression) = '*') and not IsRefLoo then
      Exit;

    if (AnExpression[Length(AnExpression)] = '*') then
      Delete(AnExpression, Length(AnExpression), 1);


    if (FFields.Items[FieldIndx].DataType <> dtString)
      and not (FFields.Items[FieldIndx].LookupType = looTableKey) then
      Exit;
    if (FFields.Items[FieldIndx].LookupType = looNone) then
      Exit;
    if (FFields.Items[FieldIndx].LookupType = looTableRef) and (FFields.Items[FieldIndx].LookupTable <> nil) then
    { This is looTableRef }
    begin
      T := FFields.Items[FieldIndx].LookupTable;
      Sl := TStringList.Create;
      try
       for I := 0 to T.Fields.Count-1 do
       begin
        if T.Fields.Items[i].FieldName = FFields.Items[FieldIndx].LookupField then
        begin
          sl.Add(T.Fields.Items[i].FieldName + ' = ' + '''' + AnExpression + '*' + '''');
          F := T.Fields.Items[i].FieldName;
          Break;
        end;
       end;

       DS := T.DBView.SearchPrivate(sl);
       try
        if DS.Count = 0 then
          Exit;
        sl.Clear;
        sl.Add(F);
        DS.Sort(sl, soAscending);
        sl.Clear;
        for I := 0 to DS.Count-1 do
        begin
          Result.AddObject(DS[i][F].AsString, TObject(DS[i].RecordIndex))
        end;
       finally
        DS.Free;
       end;
      finally
        sl.Free;
      end;
      Exit;
    end;

    if (FFields.Items[FieldIndx].LookupType = looTableKey) and (FFields.Items[FieldIndx].LookupTable <> nil) then
    { This is looTableKey }
    begin
      T := FFields.Items[FieldIndx].LookupTable;
      Sl := TStringList.Create;
      try
       for I := 0 to T.Fields.Count-1 do
       begin
        if T.Fields.Items[i].FieldName = FFields.Items[FieldIndx].LookupSearchField then
        begin
          sl.Add(T.Fields.Items[i].FieldName + ' = ' + '''' + AnExpression + '*' + '''');
          F := T.Fields.Items[i].FieldName;
          Break;
        end;
       end;
       F2 := Fields.Items[FieldIndx].LookupField;

       DS := T.DBView.SearchPrivate(sl);
       try
        if DS.Count = 0 then
          Exit;
        sl.Clear;
        sl.Add(F);
        DS.Sort(sl, soAscending);
        sl.Clear;
        S := '';
        IsDup := False;
        for I := 0 to DS.Count-1 do
        begin
          if (I > 0) and (S = DS[i][F].AsString) then
          begin
             if not IsDup then
             begin
               Result[i-1] := Result[i-1] + ' [' + DS[i-1][F2].AsString + ']';
               IsDup := True;
             end;
             Result.AddObject(DS[i][F].AsString + ' [' + DS[i][F2].AsString + ']', TObject(DS[i].RecordIndex));
          end
          else
          begin
            Result.AddObject(DS[i][F].AsString, TObject(DS[i].RecordIndex));
            IsDup := false;
          end;
          S := DS[i][F].AsString;

        end;
       finally
        DS.Free;
       end;
      finally
        sl.Free;
      end;
      Exit;
    end;

    { This is looHistory }
    FieldIndx := Fields.Items[FieldIndx].FieldID;
    SFieldIndx := IntToStr(FieldIndx);
    SFieldIndx := FillZeroes(SFieldIndx, 2);
    FindKey(0, AnExpression, KeyIndx);
    if KeyIndx = FKeyList.Count then
      Exit;
    N := TKey(FKeyList[KeyIndx]);
    sOut := N.Key;
    KeyVal := sOut.ToUpper;
    Expr := AnExpression.ToUpper;
    KeyCnt := FKeyList.Count;
    LinkCnt := FLinkList.Count;
    Cnt := 0;
    while (Pos(Expr, KeyVal) = 1) and (KeyIndx < KeyCnt) and (Cnt < 500) do
    begin
      KeyID := FillZeroes(IntToStr(N.ID), 7);
      LnkStr := KeyID + '|' + SFieldIndx;
      FindKey(1, LnkStr, LinkIndx);
      if LinkIndx <> LinkCnt then
      begin
        N := TKey(FLinkList[LinkIndx]);
        LnkStr := N.Key;
        if (Pos('|' + SFieldIndx + '|', LnkStr) <> 0) and
          (Pos(KeyID + '|', LnkStr) = 1) then
        begin
          RecId := Copy(LnkStr, 12, 7);
          R := RecordsX[StrToInt(RecId)];
          try
          if not R.Deleted then
            Result.Add(sOut);
          finally
            R.Free;
          end;
          Inc(Cnt);
        end;
      end;
      Inc(KeyIndx);
      if KeyIndx < KeyCnt then
      begin
        N := TKey(FKeyList[KeyIndx]);
        sOut := N.Key;
        KeyVal := sOut.ToUpper;
      end
      else
        Break;
    end;
  end;

  function TILTable.GetLookupTableID(LookupTable: TILTable; const LookupField : string;
    const Value: string): string;

  { Converts the literal Value to a lookup record id }
  var
    sl: TStringList;
    SR: TSearcher;
    DS: TILDataset;
    Indx : integer;
  begin
    Result := '';
    sl := TStringList.Create;
    try
      begin
        if LookupTable is TILLookUpTable then
        with LookupTable as TILLookUpTable do
        sl.Add('Select * where ' + FieldName + ' = ' + '''' +
          Value + '''')
        else
        begin
          Indx := LookupTable.Fields.IndexOf(LookupField);
          sl.Add('Select * where ' + LookupTable.Fields.Items[Indx].FieldName + ' = ' + '''' +
            Value + '''')
        end;
      end;

      SR := CreateSearcher(LookupTable, sl);
      SR.FPrivate := True;
      DS := SR.Search(nil, 0, True);
      try
        if DS.Count = 1 then
          Result := IntToStr(DS[0].RecordIndex);
      finally
        DS.Free;
        SR.Free;
      end;
    finally
      sl.Free;
    end;
  end;


  function TILTable.GetLookupTableValue(LookupTable: TILTable; const LookupField : string;
    const RecordIndex: integer): string;

  var
    LRec: TILDataRecord;
  begin
    LRec := LookupTable.RecordsX[RecordIndex];
    try
      Result := LRec[LookupField].AsString;
    finally
     LRec.Free;
    end;
  end;

  function TILTable.GetLookupParentValue(LookupTable : TILTable; const LookupField : string;
   LRec : TILDataRecord): Tstrings;
    var
    LinkID : integer;
    i : integer;
    F : TILFieldDef;
    FF : TILField;
    Indx : integer;
    DS : TILDataset;
    R : TILDatarecord;
    PF : TILFieldDef;
  begin
    Result := nil;
    LinkID := -1;
    if (LookupTable = MasterSource) then
    begin
      LinkId := Lrec['SYS_LinkID'].AsInteger;
    end
    else
    begin
      { Find the field that links to master }
      F := nil;
      PF := nil;
      for I := 0 to Fields.Count-1 do
      begin
        if (Fields.Items[i].LookupType = looParentField)
        and (Fields.Items[i].LookupTable = LookupTable)
        and (Fields.Items[i].LookupField = LookupField) then
        begin
          PF := Fields.Items[i];
          Break;
        end;
      end;

      if PF = nil then
        Exit;

      for I := 0 to Fields.Count-1 do
      begin
        if (Fields.Items[i].LookupType = looTableKey)
        and (Fields.Items[i].LookupTable = PF.LookupTable) then
        begin
          F := Fields.Items[i];
          Break;
        end;
      end;
      if F = nil then
        Exit;
      Indx := LRec.IndexOf(F.FieldName);
      FF := Lrec[Indx];

      if FF.AsString = '' then
      begin
        Exit;
      end;

      {Search for the value in lookuptable}
      if F.DataType = dtString then
        DS := LookupTable.DBView.SearchPrivate('select * where ' + F.LookupField + ' = ' + '''' + FF.AsString + '''')
      else
        DS := LookupTable.DBView.SearchPrivate('select * where ' + F.LookupField + ' = ' + FF.AsString);
      try
        if DS.Count > 0 then
          LinkId := DS[0].RecordIndex;
      finally
        DS.Free;
      end;
    end;

    if LinkID <> -1 then
    begin
      R := LookupTable.RecordsX[LinkID];
      try
        Result := TStringList.Create;
        Result.Assign(R[LookupField].FValues);
      finally
        R.Free;
      end;
    end;
  end;

  procedure TILTable.ShowLookups(AParentControl: TWinControl);
  var
    Indx: integer;
  begin
    Indx := FFields.IndexOfControl(AParentControl);
    if Indx = -1 then
      Exit;
    ShowLookups(AParentControl, FFields.Items[Indx].FieldName);
  end;

  procedure TILTable.ShowLookups(AParentControl: TWinControl;
    FieldName: string);

  var
    Indx: integer;
    sl: TStringList;
    linenumber: integer;
    Txt: string;
    H: integer;
    Usg : TDataUsage;
    Pt : TPoint;
  begin
    if not Connected or FDontLookUp then
      Exit;
    Indx := FFields.IndexOf(FieldName);
    if Indx = -1 then
      Exit;

    if Fields.Items[Indx].SearchControl = AParentControl then
      Usg := duSearch
    else
      Usg := duData; {Cell editor }

    FLookupControl.Parent := Screen.ActiveForm;
    FLookupControl.Width := AParentControl.Width;
    FLookupControl.Visible := false;
    Pt := AParentControl.Parent.ClientToScreen(POint(AParentControl.Left, AParentControl.Top));
    Pt := Screen.ActiveForm.ScreenToClient(Pt);
    FLookupControl.Left := Pt.X;
    if (AParentControl is TILSearchGrid) or (AParentControl is TILUpdateGrid) then
      with AParentControl as TILCustomGrid do
      begin
        Txt := InplaceText;
        FLookupControl.Top := Pt.Y + AParentControl.Height + 1;
        FLookupControl.Left := Pt.X + Cellrect(Col, Row).Left;
        FLookupControl.Width := Cellrect(Col, Row).Width;
      end
    else if AParentControl is TMemo then
    begin
      linenumber := SendMessage(AParentControl.Handle, EM_LINEFROMCHAR,
        TMemo(AParentControl).Selstart, 0);
      Txt := TMemo(AParentControl).Lines[linenumber];
      FLookupControl.Top := Pt.Y + AParentControl.Height + 1;
    end
    else if (AParentControl is TEdit) then
    begin
      Txt := TEdit(AParentControl).Text;
      FLookupControl.Top := Pt.Y + AParentControl.Height + 1;
    end
    else if (AParentControl is TLabeledEdit) then
    begin
      Txt := TLabeledEdit(AParentControl).Text;
      FLookupControl.Top := Pt.Y + AParentControl.Height + 1;
    end
    else if AParentControl is TCombobox then
    begin
      Txt := TCombobox(AParentControl).Text;
      FLookupControl.Top := Pt.Y + AParentControl.Height + 1;
    end;
    if Txt = '' then
      Exit;
    sl := GetLookups(FieldName, Txt, Usg);
    try
      FLookupControl.Visible := (sl.Count > 0) and
        not((sl.Count = 1) and (AnsiCompareText(sl[0], Txt) = 0));
      if not FLookupControl.Visible then
      begin
        Exit;
      end;

      FLookupControl.Items.Assign(sl);
      H := sl.Count * (FLookupControl.ItemHeight + 4);
      if H > 200 then
        H := 200;
      FLookupControl.Height := H;
      FLookupControl.ItemIndex := 0;
      FLookupOwner := AParentControl;
      FLookupControl.BringToFront;
    finally
      sl.Free;
    end;
  end;

  function TILTable.DeleteFromTable(const Index: integer): integer;
  begin
    Result := DeleteFromTable(Index, False);
  end;

  function TILTable.DeleteFromTable(const Index: integer; const Deletions : Boolean): integer;

  { When Deletions is true, the caller is Dataset.Save. Save removes the records from the dataset }
  var
    Rec: TILDataRecord;
    i, j: integer;
    DS: TILDataset;
    OldPos : integer;
    PrntRec : TILDatarecord;
    L : TStringList;
    Res : integer;

    procedure CheckDataset(RecIndex : integer);
    var
      DSIndx : integer;
      DR : TILDataRecord;
    begin
       if TMan.DLLAccess then
         Exit;
       DSIndx := Dataset.IndexOf(RecIndex);
       if DSIndx <> -1 then
       begin
         DR := Dataset[DsIndx];
         if Deletions then
           DR.DeletedUser := True
         else
         begin
          DR.Free;
          Dataset.FDataset.Delete(DSIndx);
         end;
       end;
    end;

  begin
    if IsReadOnly then
      HandleILException(E_ReadOnly, Name);
    Result := 0;
    if (Index < 0) or (Index >= RecordCount) then
    begin
      Comment1 := IntToStr(Index);
      HandleILException(E_OutOfBounds);
    end;

    DS := nil;
    if (TVTable <> nil) then
    begin
      DS := TVTable.GetPathNodes(Index);
    end;
    OldPos := Dataset.Position;
    Rec := RecordsX[Index];
    PrntRec := nil;
    if TVTable <> nil then
    begin
      PrntRec := TVTable.GetParentRecord(Rec.RecordIndex);
      TVTable.FillSiblings(PrntRec);
    end;

    try
      if Rec.Deleted then
      begin
        Rec.Free;
        Exit;
      end;
      try
       { Check integrity }
       begin
        Res := DBWork.CanDelete(Index);
        if Res > 0 then
         HandleILException(res);
       end;

       Rec.DeletedUser := True;
       Rec.ChangedDate := Now;
       Records[Index] := Rec;
       if TVTable <> nil then
         TVTable.DeleteSiblingList(PrntRec, Rec);

       if not Deletions then
        CheckDataset(Index);

      finally
        Rec.Free;
      end;
      Result := 1;

      if (Self is TILTreeViewTable) then
      with Self as TILTreeViewTable do
      { Delete sub nodes}
      begin
          for i := 0 to DS.Count - 1 do
          begin
            Rec := RecordsX[DS[i].RecordIndex];
            try
              Rec.DeletedUser := True;
              Rec.ChangedDate := Now;
              Records[DS[i].RecordIndex] := Rec;
              CheckDataset(DS[i].RecordIndex);
              {If Deletions the data set record, if present, is marked as deleted,
              not removed from dataset list. The Save method will then ignore this record}
              inc(Result);
            finally
              Rec.Free;
            end;
          end;
      end;
    finally
      if Assigned(DS) then
        DS.Free;
    end;

    if TVTable <> nil then
    { Check renumbering }
    begin
      if PrntRec = nil then
       L := TVTable.FRootSiblings
      else
       L := PrntRec.FSiblingList;
      for I := 0 to L.Count-1 do
      begin
        Rec := RecordsX[TILDataRecord(L.Objects[i]).RecordIndex];
        Rec['SYS_SiblingIndex'].AsInteger := I;
        Records[TILDataRecord(L.Objects[i]).RecordIndex] := Rec;
        Rec.Free;
        j := Dataset.IndexOf(TILDataRecord(L.Objects[i]).RecordIndex);
        if j <> -1 then
        begin
         Dataset[j]['SYS_SiblingIndex'].FValues[0] := IntToStr(i);
        end;
      end;
    end;

    if PrntRec <> nil then
      PrntRec.Free;

    if not Deletions then
    begin
      if HasViewer then
      begin
       FViewer.Report.Redraw;
       FViewer.UpdateScrollRange;
       FViewer.UpdateScrollPos;
      end;
      if Dataset.Count = 0 then
        Dataset.Position := -1
      else if OldPos > Dataset.Count -1 then
        Dataset.Position :=  Dataset.Count -1
      else
        Dataset.RunFocus(Dataset.Position, False);
      if (TVControl <> nil) then
       TVControl.BuildTree(Dataset);
    end;

    Dataset.FSelState := -1;
    if Dataset.Count = 0 then
      Inval;
    if HasHandler then
     if Assigned(DataHandler.FOnDatasetChange) then
      DataHandler.FOnDatasetChange(Self, dsSaved);

  end;

  function TILTable.IsRecordOrphan(const RecordIndex: integer): Boolean;

  var
    R: TILDataRecord;
    ID: integer;
  begin
    Result := false;
    if IsRoot or IsFreeTable then
      Exit;
    if RecordIndex = -1 then
      Exit;
    R := RecordsX[RecordIndex];
    try
      ID := StrToInt(R[c_SysFieldNames[SYS_LinkID]].AsString);
    finally
      R.Free;
    end;
    R := MasterSource.RecordsX[ID];
    try
      Result := R.Deleted and not R.DeletedTrans;
    finally
      R.Free;
    end;

  end;


  function TILTable.Undelete(const Index: integer): Boolean;
   var
    Rec: TILDataRecord;
    ID: integer;
  begin
    Result := false;
    if not Connected then
      Exit;
    Rec := RecordsX[Index];
    try
      if not Rec.Deleted or Rec.DeletedTrans then
        Exit;
      Rec.DeletedUser := false;
      Rec.FUndeleted := True;
      Records[Index] := Rec;
      Result := True;
    finally
      Rec.Free;
    end;
    if Self is TILTreeViewTable then
      with Self as TILTreeViewTable do
      begin
        Rec := RecordsX[Index];
        if not Rec[c_SysFieldNames[SYS_LinkID]].IsEmpty then
          ID := Rec[c_SysFieldNames[SYS_LinkID]].AsInteger
        else
          ID := -1;
        Rec.Free;
        try
          while ID <> -1 do
          begin
            Rec := RecordsX[ID];
            try
              if not Rec.Deleted or Rec.DeletedTrans then
                Break;
              Rec.DeletedUser := false;
              Rec.FUndeleted := True;
              Records[ID] := Rec;
              if not Rec[c_SysFieldNames[SYS_LinkID]].IsEmpty then
                ID := Rec[c_SysFieldNames[SYS_LinkID]].AsInteger
              else
                ID := -1;
            finally
              Rec.Free;
            end;
          end;
        finally
        end;
      end;
  end;

  procedure TILTable.ResetOldValues(ADataRecord: TILDataRecord);

  var
    i: integer;
  begin
    for i := 0 to ADataRecord.Count - 1 do
    begin
      if ADataRecord[i].FOldValues <> ADataRecord[i].FValues then
        ADataRecord[i].FOldValues.Free;
      ADataRecord[i].FOldValues := ADataRecord[i].FValues;
    end;
    if Assigned(ADataRecord.FOldImage) and
      (ADataRecord.FImage <> ADataRecord.FOldImage) then
      ADataRecord.FOldImage.Free;
    ADataRecord.FOldImage := ADataRecord.FImage;
    ADataRecord.FOldState := ADataRecord.FState;
    ADataRecord.FOldFileName := ADataRecord.FFileName;
    ADataRecord.FOldImageText := ADataRecord.FImageText;
    ADataRecord.FOldTableIndex := ADataRecord.FImgTableIndex;
    ADataRecord.FOldMediaType := ADataRecord.FMediaType;
  end;

  procedure TILTable.SelectAll;

  var
    i: integer;
  begin
    if Dataset = nil then
      Exit;
    if not FMultiSelect then
      Exit;
    DataSet.FSelState := -1;
    for i := 0 to Dataset.Count - 1 do
    begin
      Dataset.Items[i].FSelected := True;
    end;
    if Dataset.Count > 0 then
    begin
      if HasHandler and Assigned(DataHandler.FOnSelect) then
      begin
        DataHandler.FOnSelect(Self, -1);
      end;
      if HasViewer then
        Viewer.Invalidate;
    end;
  end;

  procedure TILTable.DoDeselectAll;

  var
    i: integer;
  begin
    if Dataset = nil then
      Exit;
    if Dataset.Count = 0 then
      Exit;
    if not FMultiSelect then
      Exit;
    for i := 0 to Dataset.Count - 1 do
    begin
      Dataset.Items[i].FSelected := false;
      Dataset.Items[i].FIsDeleted := false;
    end;
    Dataset.Focusedrecord.FSelected := True;
    //DBWork.RevertChanges(False);
    Dataset.FSelState := -1;
  end;

  procedure TILTable.DeselectAll;
  begin
    if Dataset = nil then
      Exit;
    if not MultiSelected then
      Exit;
    DoDeselectAll;
    ResetDataControls;
    if HasHandler and Assigned(DataHandler.FOnSelect) then
    begin
      DataHandler.FOnSelect(Self, -1);
    end;
    if (Dataset.Count > 0) and HasViewer then Viewer.Invalidate;
  end;

  procedure TILTable.ClearControls(const Usage: TDataUsage; AControl : TWinControl = nil);

  var
    i, j: integer;
    FldDef: TILFieldDef;
    GC: TWinControl;
    Cnt: integer;
    ARecord : TilDatarecord;
    S : string;
    IsEmpty : Boolean;

    procedure NotifyMsgHandler;
    var
      MH : TILMessageHandler;
      Ind : integer;
    begin
      Ind := -1;
      MH := TMan.GetMessageHandler(Self, Ind);
      while MH <> nil do
      begin
         MH.ControlsChanged(Usage);
         MH := TMan.GetMessageHandler(Self, Ind);
      end;
    end;

    procedure CallChng;
    var
      i : integer;
    begin
       for I := 0 to Datahandler.DataFields.Count-1 do
       begin
         if Usage = duSearch then
         begin
           if DataHandler.DataFields.Items[i].SearchControl = GC then
           begin
              DataHandler.DataFields.Items[i].CtrlChng(GC);
              Break;
           end;
         end
         else
         begin
           if DataHandler.DataFields.Items[i].DataControl = GC then
           begin
              DataHandler.DataFields.Items[i].CtrlChng(GC);
              Break;
           end;
         end;
       end;

    end;

  begin
   FDontLookup := True;
   try
    if Usage = duSearch then
      Cnt := ActiveSearchCtrlCount
    else
      Cnt := Fields.Count;
    for i := 0 to Cnt - 1 do
    begin
      if Usage = duSearch then
        FldDef := ActiveSearchCtrls[0] { Reduced with 1 per loop }
      else
        FldDef := Fields.Items[i];
      if not (FldDef.Table = self) then
         Continue;
      if Usage = duSearch then
        GC := FldDef.FSearchControl
      else
        GC := FldDef.FDataControl;
      if (GC = nil) and (AControl = nil) then
      begin
       if (FldDef.DataType = dtLongText) and (Usage = duData)
       and HasHandler and Assigned(Datahandler.FOnUpdateLongText) then
       begin
         ARecord := Dataset.FocusedRecord;
         S := '';
         if ARecord <> nil then
         for j := 0 to ARecord.Count-1 do
         begin
            if SameText(ARecord[j].FieldName, FldDef.FieldName) then
            begin
              Datahandler.FOnUpdateLongText(Datahandler, ARecord[j], S, [ltaClearEditors]);
              ARecord[j].ResetLTFlags;
            end;
         end
         else
          Datahandler.FOnUpdateLongText(Datahandler, nil, S, [ltaClearEditors]);
       end;
       Continue;
      end;

      if Assigned(GC) and ((AControl = nil) or (AControl = GC)) then
      begin
        if GC is TMemo then
          with GC as TMemo do
          begin
            IsEmpty := (Lines.Count = 0);
            Lines.Clear;
            if IsEmpty then
              CallChng;
          end
        else if GC is TCustomEdit then
          with GC as TCustomEdit do
          begin
            IsEmpty := Text = '';
            Text := '';
            if IsEmpty then
              CallChng;
          end
        else if (GC is TILSearchGrid) or (GC is TILUpdateGrid) then
          with GC as TILCustomGrid do
          begin
            for j := 1 to RowCount - 1 do
            begin
              Values[j] := '';
            end;
            CallChng;
          end
        else if GC is TILResourceList then
          with GC as TILResourceList do
          begin
            Empty :=True;
            CallChng;
          end
        else if GC is TListBox then
          with GC as TListBox do
          begin
            ItemIndex := -1;
            CallChng;
          end
        else if GC is TCheckbox then
          with GC as TCheckbox do
          begin
            if AllowGrayed then
              State := cbGrayed
            else
              State := cbUnChecked;
            CallChng;
          end
        else if GC is TRadioGroup then
          with GC as TRadioGroup do
          begin
            ItemIndex := -1;
            CallChng;
          end
        else if GC is TCombobox then
          with GC as TCombobox do
          begin
            if Style in [csDropDownList, csOwnerdrawFixed, csOwnerdrawVariable]
            then
              ItemIndex := -1
            else
              Text := '';
            CallChng;
          end
        else if GC is TCheckListBox then
          with GC as TCheckListBox do
          begin
            for j := 0 to Items.Count - 1 do
            begin
              AllowGrayed := True;
              State[j] := cbGrayed;
            end;
            CallChng;
          end
        else if GC is TDateTimePicker then
          with GC as TDateTimePicker do
          begin
            if ShowCheckBox then
            begin
              Checked := false;
              CallChng;
            end;
          end;
        if GC = AControl then
          Break;
      end;
    end;
    finally
     FDontLookup := false
    end;
    if HasViewer and (Usage = duData) and (FViewer.UpdateMarkers <> upmNone) then
    begin
      FUpmCtrlRecord.Clear;
      Viewer.Invalidate;
    end;
    NotifyMsgHandler;
  end;

  function TILTable.ReadControls(ARecord: TILRecord; const Usage: TDataUsage) : integer;

  { Reads control values into ADataRecord
    A control is regarded as Null if it contains no data or is disabled/invisible }
  var
    i, j, k: integer;
    FldDef: TILFieldDef;
    GC: TWinControl;
    Fld: TILField;
    Cnt: integer;
    ReadOnly: Boolean;
    Indx: integer;
    S, s2: string;
    MaxVal: integer;
    B: integer;
    sl: TStringList;
    TagStr: string;
    P: integer;
    TBL: TILTable;
    Vals: TStringList;
    LOp : TValueLogOp;
    DS : TILDataSet;
    V1, V2 : string;

    procedure DoError(ErrorCode: integer);
    begin
      Fld.Free;
      ARecord.Clear;
      Result := ErrorCode;
    end;

    function CheckContentType(var S: string; CheckOps: Boolean = True): Boolean;

  var
    TestD: TDateTime;
    CoOp: string;
    P: integer;
    N: Extended;

    function CheckElements(Delimiter: string; DoFloat: Boolean): Boolean;

  var
    s1, s2: string;
  begin
    Result := True;
    P := Pos(Delimiter, S);
    if P <> 0 then
    begin
      s1 := Copy(S, 1, P - 1);
      s1 := TrimBlanks(s1);
      s2 := Copy(S, P + 2, Length(S));
      s2 := TrimBlanks(s2);
      if DoFloat then
        Result := TryStrToFloat(s1, N, Translator.FFormatSettings) or
          ((s2 <> '') and not TryStrToFloat(s2, N, Translator.FFormatSettings))
      else
        Result := TryStrToDateTime(s1, TestD, Translator.FFormatSettings) or
          ((s2 <> '') and not TryStrToDateTime(s2, TestD, Translator.FFormatSettings));
      if not Result then
      begin
        S := '';
        Exit;
      end;
    end
  end;

  procedure CheckCoOps;
  var
    E : Extended;
  begin
    if not CheckOps then
      Exit;
    if (Pos('>=', S) = 1) or (Pos('<=', S) = 1) or (Pos('==', S) = 1) then
    begin
      CoOp := Copy(S, 1, 2);
      System.Delete(S, 1, 2);
      S := TrimBlanks(S);
    end
    else if (Pos('>', S) = 1) or (Pos('<', S) = 1) or (Pos('=', S) = 1) then
    begin
      CoOp := Copy(S, 1, 1);
      System.Delete(S, 1, 1);
      S := TrimBlanks(S);
    end;
    if (FldDef.DataType = dtNumber) and (Pos('-', S) = 1) then
    begin
     if not TryStrToFloat(S, E, Translator.FFormatSettings) then
       Exit;
     Delete(S, 1, 1);
     E := StrToFloat(S, Translator.FFormatSettings);
     E := MaxFloat - E;
     S := FloatToStr(E, Translator.FFormatSettings);
     S := NegSign + S;
    end;

  end;

  procedure CheckDataOps;
  begin
    if (Pos('>', S) = 1) or (Pos('<', S) = 1) or (Pos('=', S) = 1) then
    begin
      CoOp := Copy(S, 1, 1);
      System.Delete(S, 1, 1);
      S := TrimBlanks(S);
    end;
  end;

  begin
    Result := True;
    if S = '' then
      Exit;
    CoOp := '';
    if PosRestrict(TMan.ReportNull, S, False) <> 0 then
    begin
      if Length(S) <> Length(TMan.ReportNull) then
      begin
        P := Pos('<>', S);
        if P <> 1 then
          Result := False;
        Exit;
      end;
    end;
    if TMan.IsValueNull(s) then
    begin
      S := TMan.ReportNull;
      Exit;
    end;
    if FldDef.DataType = dtNumber then
    begin
      if Usage = duSearch then
      begin
        P := Pos('..', S);
        if P > 0 then
        begin
          Result := CheckElements('..', True);
          Exit;
        end
        else
          CheckCoOps;
      end
      else
      begin
        P := Pos('=', S);
        if P > 0 then
        begin
          Result := CheckElements('=', True);
          Exit;
        end
        else
          CheckDataOps;
      end;
    end

    else if FldDef.DataType = dtDate then
    begin
      if Usage = duSearch then
      begin
        P := Pos('..', S);
        if P <> 0 then
        begin
          Result := CheckElements('..', false);
          Exit;
        end
        else
          CheckCoOps;
      end
      else
      begin
        P := Pos('=', S);
        if P <> 0 then
        begin
          Result := CheckElements('=', false);
          Exit;
        end
        else
          CheckDataOps;
      end;

      if (Pos('?', S) = 0) and (Pos('*', S) = 0)
      then { Wild cards allowed with dates }
        if (S <> '') and not TryStrToDateTime(S, TestD, Translator.FFormatSettings) then
        begin
          Result := false;
          Exit;
        end;
      S := CoOp + S;
    end
    else if FldDef.DataType = dtBoolean then
    begin
      if (S <> '0') and (S <> '1') and (S <> TMan.ReportTrue) and (S <> TMan.ReportFalse)
      then
      begin
        Result := false;
      end;
    end
  end;

  function CheckDups(Values: TStringList): integer;

  var
    sl: TStringList;
    i: integer;
  begin
    Result := 0;
    Comment1 := '';
    if Values.Count = 0 then
    begin
      Exit;
    end;
    sl := TStringList.Create;
    try
      sl.Sorted := True;
      sl.Duplicates := DupIgnore;
      sl.AddStrings(Values);
      if sl.Count < Values.Count then
      begin
        Comment1 := Fld.FieldDef.Caption;
        Result := E_FieldDupValues;
      end;
    finally
      sl.Free;
    end;

    { Check for null mixes }
    if Values.Count > 1 then
      for i := 0 to Values.Count - 1 do
      begin
        if TMan.IsValueNull(Values[i]) then
        begin
          Comment1 := Fld.FieldDef.Caption;
          Result := E_FieldNullMixed;
          Break;
        end;
      end;
  end;

  begin
    if (Usage = duSearch) and (TVTable <> nil) then
      with TVTable do
        if TreeViewParams.SearchCtrlMode = scmStartWith then
        begin
          TreeViewParams.RootCondition.Clear;
        end
        else
          TreeViewParams.WhereCondition.Clear;
    if (Usage = duSearch) and not(Self is TILLookupTable) then
      Cnt := ActiveSearchCtrlCount
    else
      Cnt := FFields.Count;
    Result := 0;

    for i := 0 to Cnt - 1 do
    begin
      if (Usage = duSearch) and not(Self is TILLookupTable) then
        FldDef := ActiveSearchCtrls[i]
      else
        FldDef := FFields.Items[i];

      if FldDef.MaxValues = 0 then
        MaxVal := MaxInt
      else
        MaxVal := FldDef.MaxValues;

      ReadOnly := FldDef.IsReadOnly;
      if ReadOnly and (Usage = duData) then
      begin
        Continue;
      end;

      Indx := ARecord.IndexOf(FldDef.FFieldName);

      if FldDef.Ilib.Fields.IndexOf(FldDef.FieldName) = -1 then
        Continue;

      if Usage = duSearch then
      begin
        GC := FldDef.FSearchControl;
      end
      else
        GC := FldDef.FDataControl;
      if not Assigned(GC) then
        Continue;
      if (Usage = duData) and (not GC.Visible or not GC.Enabled) then
        Continue;

      Fld := TILField.Create(Self, FldDef.FieldName, FldDef.Ilib.FFields);
      Fld.FOwnerRecord := ARecord;
      Fld.FUsage := Usage;

      if GC is TILSearchGrid then
        with GC as TILSearchGrid do
        begin
          sl := TStringList.Create;
          try
            sl.Assign(Cols[ValueColIndex]);
            TrimList(sl);
            if CheckDups(sl) <> 0 then
            begin
              Comment1 := Fld.FieldDef.Caption;
              DoError(E_FieldDupValues);
              Exit;
            end;
          finally
            sl.Free;
          end;
          for j := 0 to RowCount - 1 do
          begin
            S := Values[j];
            if Trim(S) = '' then
              Continue;
            if not CheckContentType(S, false) then
            begin
              Comment1 := Fld.FieldDef.Caption;
              if FldDef.DataType = dtNumber then
                DoError(E_InvalidNumber)
              else if FldDef.DataType = dtDate then
                DoError(E_InvalidDate)
              else
               DoError(E_FieldDataTypeMismatch);
              Exit;
            end;
            LOp := FldDef.ValueLogOp;
            if (LogOp[j] <> '') and AlignOperators then
            begin
              if LogOp[j] = '&' then
                FldDef.ValueLogOp := vlAnd
              else
                FldDef.ValueLogOp := vlOr;
              S := CoOp[j] + ' ' + S;
            end
            else
            begin
              if LogOp[j] = '' then
                FldDef.ValueLogOp := LOp;
              S := LogOp[j] + ' ' + CoOp[j] + ' ' + S;
            end;
            S := Trim(S);
            Fld.FValues.Add(S);
          end;
        end
      else if GC is TILUpdateGrid then
        with GC as TILUpdateGrid do
        begin
          for j := StartRow to RowCount - 1 do
          begin
            S := Values[j];
            if Trim(S) = '' then
              Continue;
            if Mode = gmUpdateActions then
            begin
              TagStr := GetTranslTag(TagValue[j]);
              if TagStr = tag_Add then
                S := '> ' + S
              else if TagStr = tag_Delete then
                S := '< ' + S
              else if TagStr = tag_Replace then
              begin
                P := PosRestrict(';', S, False);
                if P = 0 then
                begin
                  DoError(E_SyntaxError);
                  Exit;
                end;
                S[P] := '=';
              end
              else if TagStr <> tag_Set then
              begin
                DoError(-1);
                Exit;
              end;
            end;
            if not CheckContentType(S) then
            begin
              Comment1 := Fld.FieldDef.Caption;
              if FldDef.DataType = dtNumber then
                DoError(E_InvalidNumber)
              else if FldDef.DataType = dtDate then
                DoError(E_InvalidDate)
              else
               DoError(E_FieldDataTypeMismatch);
              Exit;
            end;
            Fld.FValues.Add(S);
          end;
        end

      else if GC is TMemo then
        with GC as TMemo do
        begin
          Fld.FValues.Assign(Lines);
          if not(FldDef.DataType = dtText) then
            TrimList(Fld.FValues);
          if (Fld.Count > MaxVal) and (Usage = duData) and not (FldDef.DataType = dtText) then
          begin
            Comment1 := Fld.FieldDef.Caption;
            DoError(E_FieldTooManyValues);
            Exit;
          end
          else if CheckDups(Fld.FValues) <> 0 then
          begin
            Comment1 := Fld.FieldDef.Caption;
            DoError(E_FieldDupValues);
            Exit;
          end
          else if Fld.FValues.Count = 0 then
            Fld.FValues.Add(TMan.ReportNull);
        end

      else if GC is TCustomEdit then
        with GC as TCustomEdit do
        begin
          if not(FldDef.DataType = dtText) then
            S := TrimBlanks(Text)
          else
            S := Text;
          if S = '' then
            Fld.FValues.Add(TMan.ReportNull)
          else
          begin
            s2 := S;
            if not CheckContentType(S) then
            begin
              Comment1 := Fld.FieldDef.Caption;
              if FldDef.DataType = dtNumber then
                DoError(E_InvalidNumber)
              else if FldDef.DataType = dtDate then
                DoError(E_InvalidDate)
              else
               DoError(E_FieldDataTypeMismatch);
              Exit;
            end;
            if (FldDef.LookupType = looTableKey)
            and (FldDef.DataType = dtNumber)
            and not TMan.IsValueNull(S)
            then
            begin
              S := Fld.DecodeTableKey(S);
              if S <> '' then
               Fld.FValues.Add(S);
            end

            else if (Usage = duData) and (FldDef.DataType = dtNumber) then
            begin
              B := Pos(Translator.FFormatSettings.DecimalSeparator, S);
              if B <> 0 then
              begin
                V2 := '';
                V1 := S;
                P := Pos('=', S);
                if P <> 0 then
                begin
                  V2 := Copy(S, P + 1, Length(S));
                  V1 := Copy(S, 1, P-1);
                end;

                B := Pos(Translator.FFormatSettings.DecimalSeparator, V1);
                s2 := Trim(Copy(V1, B + 1, Length(V1)));
                S := Copy(V1, 1, B - 1);
                if Length(s2) > FldDef.Decimals then
                begin
                  Comment1 := S;
                  DoError(E_TooManyDecimals);
                  Exit;
                end;
                if V2 <> '' then
                begin
                  B := Pos(Translator.FFormatSettings.DecimalSeparator, V2);
                  s2 := Trim(Copy(V2, B + 1, Length(V2)));
                  S := Copy(V2, 1, B - 1);
                  if Length(s2) > FldDef.Decimals then
                  begin
                    Comment1 := S;
                    DoError(E_TooManyDecimals);
                    Exit;
                  end;
                end;
              end;
            end;
            if not ((FldDef.LookupType = looTableKey)
            and (FldDef.DataType = dtNumber)) or TMan.IsValueNull(s) then
              Fld.FValues.Add(Text);
          end;
        end

      else if GC is TILResourceList then
        with GC as TILResourceList do
        begin
            if Empty then
             Fld.FValues.Add(TMan.ReportNull)
            else
            for J := 0 to Items.Count-1 do
              Fld.FValues.Add(ResourceStr[j])
        end
      else if GC is TListBox then
        with GC as TListBox do
        begin
          if ItemIndex <> -1 then
            Fld.FValues.Add(Items[ItemIndex])
          else
            Fld.FValues.Add(TMan.ReportNull);
        end
      else if GC is TCheckbox then
        with GC as TCheckbox do
        begin
          if State = cbChecked then
            Fld.FValues.Add('1')
          else if State = cbUnChecked then
            Fld.FValues.Add('0')
          else
            Fld.FValues.Add(TMan.ReportNull);
        end
      else if GC is TRadioGroup then
        with GC as TRadioGroup do
        begin
          if ItemIndex <> -1 then
            Fld.FValues.Add(Items[ItemIndex])
          else
            Fld.FValues.Add(TMan.ReportNull)
        end
      else if GC is TCombobox then
        with GC as TCombobox do
        begin
          if ItemIndex <> -1 then
          begin
            if (FldDef.LookupType = looTableKey)
            and (FldDef.DataType = dtNumber)
            then
            begin
              DS := FldDef.LookupTable.DBView.SearchPrivate('select * where ' +
               FldDef.LookupSearchField + ' = ' +
              '''' + Text + '''');
              try
              if DS.Count > 0 then
              begin
                Fld.FValues.Add(DS[0][FldDef.LookupField].AsString);
              end;
              finally
                DS.Free;
              end;
            end
            else
             Fld.FValues.Add(Text)
          end
          else
            Fld.FValues.Add(TMan.ReportNull);
        end
      else if GC is TCheckListBox then
        with GC as TCheckListBox do
        begin
          for j := 0 to Items.Count - 1 do
          begin
            if State[j] = cbChecked then
            begin
              Fld.FValues.Add(Items[j]);
            end
            else if (State[j] = cbUnChecked) and (Usage = duSearch) then
              Fld.FValues.Add('<> ' + Items[j])
          end;
          if Fld.FValues.Count = 0 then
            Fld.FValues.Add(TMan.ReportNull)
          else if (Fld.Count > MaxVal) and (Usage = duData) then
          begin
            Comment1 := Fld.FieldDef.Caption;
            DoError(E_FieldTooManyValues);
            Exit;
          end;
        end
      else if GC is TDateTimePicker then
        with GC as TDateTimePicker do
        begin
          if Usage = duData then
          begin
            if not ReadOnly then
            begin
              if Checked then
                Fld.FValues.Add(DateTimeToStr(DateTime, Translator.FFormatSettings))
              else
              begin
                Fld.FValues.Clear;
                Fld.FValues.Add(TMan.ReportNull);
              end;
            end
            else
            begin
              if Checked then
                Fld.FValues.Add(DateTimeToStr(DateTime, Translator.FFormatSettings))
              else
              begin
                Fld.FValues.Clear;
                Fld.FValues.Add(TMan.ReportNull);
              end;
            end;
          end;
        end;

      if Indx = -1 then
        ARecord.Add(Fld)
      else
      begin
        ARecord[Indx].FValues.Assign(Fld.FValues);
        Fld.Free;
      end; { I }

    end;
    if (Usage = duSearch) and (TVTable <> nil) then
      with TVTable do
        if TreeViewParams.SearchCtrlMode = scmStartWith then
          TreeViewParams.RootCondition.Assign(ARecord)
        else
          TreeViewParams.WhereCondition.Assign(ARecord);

    if Usage = duData then
    begin
      if not CompareRecords(FUpmCtrlRecord, ARecord, True) then
      begin
        if HasViewer and (Viewer.FUpdateMarkers <> upmNone) then
          Viewer.Invalidate;
      end;
      FUpmCtrlRecord.Assign(ARecord);
    end;
  end;

  function TILTable.DataCtrlsToExpression: TStringList;

  var
    Rec: TILRecord;
    S: TUpdater;
  begin
    Rec := TILRecord.Create(Self);
    ReadControls(Rec, duData);
    S := TUpdater.Create(Self, Rec);
    try
      if S.FError <> 0 then
        HandleILException(S.FError, '');
      Result := TStringList.Create;
      Result.Assign(S.Expression);
    finally
      Rec.Free;
      S.Free;
    end;
  end;

  function TILTable.SearchCtrlsToExpression(Expression: TStringList;
    const IncludeDetailTables: Boolean = false): integer;
  var
    Rec: TILRecord;
    S: TSearcher;
    sl: TStringList;

  function AddStrings(TBL: TILTable): integer;
  var
    S: TSearcher;
  begin
    S := nil;
    try
      Result := Tbl.ReadControls(Rec, duSearch);
      if Result <> 0 then
      begin
        Expression.Clear;
        Exit;
      end;
      S := CreateSearcher(TBL, Rec);
      sl.Assign(S.Expression);
      Expression.AddStrings(sl);
    finally
      Rec.Clear;
      sl.Clear;
      if Assigned(S) then
        S.Free;
    end;
  end;

  procedure SetSequence;

  var
    sl: TStringList;
    TmpSl: TStringList;
    i, j: integer;
    Cnt: integer;
  begin
    TMan.FSearchFieldOrder := Trim(TMan.FSearchFieldOrder);
    if TMan.FSearchFieldOrder = '' then
    begin
      if TMan.SearchModel = smAndOrMixed then
      begin
        if TMan.UseCommandModes then
          TMan.CreateSearchFieldOrder
        else
          HandleILException(E_SearchFieldOrder);
      end
      else
       Exit;
    end;
    Cnt := 0;
    sl := TStringList.Create;
    TmpSl := TStringList.Create;
    try
      CreateListFromDelimiter(',', TMan.FSearchFieldOrder, sl);
      TrimList(sl);
      for i := 0 to sl.Count - 1 do
      begin
        for j := 0 to Expression.Count - 1 do
        begin
          if PosRestrict(sl[i], Expression[j], False) <> 0 then
          begin
            TmpSl.Add(Expression[j]);
            Inc(Cnt);
            Break;
          end;
        end;
      end;
      if Cnt <> Expression.Count then
        HandleILException(E_SearchFieldOrder);
      Expression.Assign(TmpSl);
      TMan.FSearchFieldOrder := '';
    finally
      sl.Free;
      TmpSl.Free;
    end;

  end;

  begin
    S := nil;
    Rec := TILRecord.Create(Self);
    try
      Result := ReadControls(Rec, duSearch);
      if Result <> 0 then
        Exit;
      if Rec.Count = 0 then
      begin
        Result := 0;
        Exit;
      end;
      S := CreateSearcher(Self, Rec);
      Expression.Assign(S.Expression);
    finally
      Rec.Free;
      if Assigned(S) then
        S.Free;
    end;
    if Expression.Count = 0 then
    begin
      Result := -1;
      Exit;
    end;
    SetSequence;

  end;

  procedure TILTable.WriteControls(ARecord: TILRecord; const Usage: TDataUsage;
  const SkipInvisibles : Boolean = false);

  { Fills gui controls with data from ADataRecord }
  var
    i, j: integer;
    FldDef: TILFieldDef;
    GC: TWinControl;
    Val: string;
    sl: TStringList;
    Indx: integer;
    Chck: Boolean;
    S, A: string;

    function GetVal(AFieldName: string): TStringList;
    var
      dt : TDateTime;
      i : integer;

    begin
      if ARecord[AFieldName].FieldDef.LookupType = looTableKey then
      begin

      end;
      Result := ARecord.ValueOf(AFieldName);
      if (Result.Count = 1) and TMan.IsValueNull(Result[0]) then
        Result.Clear
      else
      begin
        if TMan.UseCommandModes and (ARecord[AFieldName].FieldDef.DataType = dtDate)
        and not (TMan.CommandMode = cmUpdate) and (Usage = duData) and
        (ARecord[AFieldName].FieldDef.DateFormat <> '') and not (GC is TDateTimePicker)
        then
        begin
            for I := 0 to Result.Count-1 do
            begin
             Dt := StrToDateTime(Result[i], Translator.FFormatSettings);
             Result[i] := FormatDateTime(ARecord[AFieldName].FieldDef.DateFormat, Dt);
            end;
        end;
      end;
    end;

    procedure CallChng;
    var
      i : integer;
    begin
       for I := 0 to Datahandler.DataFields.Count-1 do
       begin
         if Usage = duSearch then
         begin
           if DataHandler.DataFields.Items[i].SearchControl = GC then
           begin
              DataHandler.DataFields.Items[i].CtrlChng(GC);
              Break;
           end;
         end
         else
         begin
           if DataHandler.DataFields.Items[i].DataControl = GC then
           begin
              DataHandler.DataFields.Items[i].CtrlChng(GC);
              Break;
           end;
         end;
       end;

    end;

  begin
      if ARecord = nil then
        Exit;
      FWritingControls := True;
      FDontLookUp := True;
      if Assigned(FLookupControl) and FLookupControl.Visible then
        FLookupControl.Visible := false;
      try
        for i := 0 to ARecord.Count - 1 do
        begin
          S := '';
          ARecord[i].FUsage := Usage;
          FldDef := ARecord[i].FieldDef;
          if Usage = duData then
          begin
            if not Assigned(FldDef.FDataControl) then
            begin
              Continue;
            end;
            if (FldDef.FDataControl = nil) or (not FldDef.FDataControl.Visible and SkipInvisibles) then
              Continue;
          end;

          if Usage = duData then
          begin
            GC := FldDef.FDataControl;
          end
          else
          begin
            GC := FldDef.FSearchControl;
            if GC = nil then
              Continue;
            if not FldDef.FSearchControl.Visible and SkipInvisibles then
              Continue;
          end;

          if GC is TMemo then
            with GC as TMemo do
            begin
              sl := GetVal(FldDef.FieldName);
              try
                Lines.Assign(sl);
              finally
                sl.Free;
              end;
              ReadOnly := FldDef.IsReadOnly and (Usage = duData);
            end

          else if GC is TCustomEdit then
            with GC as TCustomEdit do
            begin
               if (FldDef.LookupType = looTableKey)
               and (FldDef.DataType = dtNumber) then
               begin
                 Text := ARecord[FldDef.FieldName].DisplayValue;
                 ReadOnly := FldDef.IsReadOnly;
               end
               else
               begin
                 sl := GetVal(FldDef.FieldName);
                 try
                  if (sl.Count = 0) then
                   Text := ''
                  else
                   Text := sl[0];
                  ReadOnly := FldDef.IsReadOnly and (Usage = duData);
                finally
                 sl.Free;
                end;
               end;

            end

          else if GC is TILSearchGrid then
            with GC as TILSearchGrid do
            begin
              sl := GetVal(FldDef.FieldName);
              try
                for j := 0 to sl.Count - 1 do
                begin
                  S := sl[j];
                  if (S[1] = c_LogOr) or (S[1] = c_LogAnd) then
                  begin
                    System.Delete(S, 1, 2);
                    sl[j] := S;
                  end;
                end;

                Cols[ValueColIndex].Assign(sl);
                if FldDef.LookupType = looPickValues then
                  if PickValues.Count > 0 then
                  RelocateCombobox;
              finally
                sl.Free;
              end;
            end

          else if GC is TILUpdateGrid then
            with GC as TILUpdateGrid do
            begin
              sl := GetVal(FldDef.FieldName);
              try
                for j := 0 to sl.Count - 1 do
                begin
                  Values[j + StartRow] := sl[j];
                end;
                if FldDef.LookupType = looPickValues then
                begin
                 if PickValues.Count > 0 then
                  RelocateCombobox;
                end
                else if FldDef.LookupType in [looTableRef, looTableKey] then
                  RelocateCombobox;
              finally
                sl.Free;
              end;
            end

          else if GC is TILResourceList then
            with GC as TILResourceList do
            begin
               sl := GetVal(FldDef.FieldName);
               try
               if Sl.Count = 0 then
                 Empty := True
               else
               begin
                Items.Clear;
                FAddresses.Clear;
                for j := 0 to sl.Count-1 do
                begin
                  GetResourceElements(sl[j], S, A);
                  Items.Add(S);
                  FAddresses.Add(A);
                end;
                ItemIndex := 0;
               end;
               finally
                 sl.Free;
               end;
            end

          else if GC is TListBox then
            with GC as TListBox do
            begin
              sl := GetVal(FldDef.FieldName);
              try
                if (sl.Count = 0) then
                  TListBox(GC).ItemIndex := -1
                else
                begin
                  Val := sl[0];
                  TListBox(GC).ItemIndex := TListBox(GC).Items.IndexOf(Val);
                end;
              finally
                sl.Free;
              end;
            end

          else if GC is TCheckbox then
            with GC as TCheckbox do
            begin
              sl := GetVal(FldDef.FieldName);
              try
                if sl.Count = 0 then
                begin
                  if AllowGrayed then
                    State := cbGrayed
                  else
                    Checked := false;
                end
                else
                begin
                  Val := sl[0];
                  if Val = '0' then
                    Checked := false
                  else if Val = '1' then
                    Checked := True
                  else
                    State := cbGrayed;
                end
              finally
                sl.Free;
              end;
            end

          else if GC is TRadioGroup then
          begin
            sl := GetVal(FldDef.FieldName);
            try
              if sl.Count = 0 then
                TRadioGroup(GC).ItemIndex := -1
              else
              begin
                Val := sl[0];
                TRadioGroup(GC).ItemIndex := TRadioGroup(GC).Items.IndexOf(Val);
              end;
            finally
              sl.Free;
            end;
          end

          else if GC is TCombobox then
          begin
            if (FldDef.LookupType in [looTableKey, looTableRef]) then
            begin
               Indx := FldDef.LookupTable.Fields.IndexOf(FldDef.LookupField);
               if (TComboBox(GC).Items.Count = 0)
               or FldDef.LookupTable.Fields.Items[Indx].FIsUpdated then
              begin
               FldDef.LookupTable.Fields.Items[Indx].FIsUpdated := false;
               sl := GetLookups(FldDef.FieldName, '*', duData);
               TComboBox(GC).Items.Assign(sl);
               sl.Free;
              end;
            end;

            if (FldDef.LookupType = looTableKey)
            and (FldDef.DataType = dtNumber) then
            begin
                TComboBox(GC).ItemIndex :=
                 TComboBox(GC).Items.IndexOf(ARecord[FldDef.FieldName].DisplayValue);
            end
            else
            begin
              sl := GetVal(FldDef.FieldName);
              try
                if sl.Count = 0 then
                begin
                  if TCombobox(GC).Style in [csDropDownList, csOwnerdrawFixed,
                    csOwnerdrawVariable] then
                    TCombobox(GC).ItemIndex := -1
                  else
                    TCombobox(GC).Text := '';
                end
                else
                begin
                  Val := sl[0];
                  if TCombobox(GC).Style in [csDropDownList, csOwnerdrawFixed,
                    csOwnerdrawVariable] then
                    TCombobox(GC).ItemIndex := TCombobox(GC).Items.IndexOf(Val)
                  else
                    TCombobox(GC).Text := Val;
                end;
            finally
              sl.Free;
            end;
            end;
          end

          else if GC is TCheckListBox then
          begin
            sl := GetVal(FldDef.FieldName);
            try
              if sl.Count = 0 then
                TCheckListBox(GC).CheckAll(cbGrayed)
              else
                TCheckListBox(GC).CheckAll(cbUnChecked);
              for j := 0 to sl.Count - 1 do
              begin
                Indx := TCheckListBox(GC).Items.IndexOf(sl[j]);
                if Indx <> -1 then
                  TCheckListBox(GC).Checked[Indx] := True;
              end;
            finally
              sl.Free;
            end;

          end

          else if GC is TDateTimePicker then
            with GC as TDateTimePicker do
            begin
              Chck := True;
              Format := '';
              sl := GetVal(FldDef.FieldName);
              try
                if sl.Count > 0 then
                begin
                  DateTime := StrToDateTime(sl[0], Translator.FFormatSettings);
                  if DateTime = 0 then
                    Chck := false
                end
                else
                begin
                  S := DateTimeToStr(Today, Translator.FFormatSettings);
                  DateTime := StrToDateTime(s, Translator.FFormatSettings);
                  Chck := false;
                end;
                if ShowCheckBox then
                  Checked := Chck;
              finally
                sl.Free
              end;
            end;

          CallChng;
        end;
        if (Usage = duData) and not Dataset.IsReadOnly then
        begin
          FUpmCtrlRecord.Assign(ARecord);
        end;
      finally
        FDontLookUp := false;
        FWritingControls := false;
      end;
  end;

  function TILTable.IsControlsEmpty(const Usage: TDataUsage): Boolean;
  var
    ARecord: TILRecord;
    res: integer;
  begin
    if Usage = duSearch then
      ARecord := TILRecord.Create(Self)
    else
      ARecord := TILRecord.Create(Self);
    try
      res := ReadControls(ARecord, Usage);
      if res <> 0 then
        Result := false
      else
        Result := (ARecord.Count = 0);
    finally
      ARecord.Free;
    end;
  end;

  procedure TILTable.AlterIndexSpace;
  { Changes the indexspace of the table }
  begin
    AlterTable(iloAlterIndexSpace, IndexSpace, nil, nil);
  end;

  procedure TILTable.AlterRecordFreeSpace;
  { Changes the recordspace of the table }
  begin
    AlterTable(iloAlterRecordFreeSpace, RecordFreeSpace, nil, nil);
  end;

  procedure TILTable.AlterRecordSize;
  { Changes the recordspace of the table }
  begin
    AlterTable(iloAlterRecordSize, RecordSize, nil, nil);
  end;

  procedure TILTable.AlterRecordLayout(const NewLayout : TRecordLayout);
  { Changes the recordspace of the table }
  begin
    AlterTable(iloAlterRecordLayout, Integer(NewLayout), nil, nil);
  end;

  procedure TILTable.RenameTable(OldName, NewName : string);
  var
    Data : TSubFile;
    Indx : integer;
    sl : TStringList;
    s : string;
    OldN, NewN : string;
    LCOpen : Boolean;

    procedure DoRename;
    begin
      Indx := TMan.FLibContainer.IndexOf(OldName);
      Data := TMan.FLibContainer.FFiles[Indx];
      Data.FFileSpec.FileName := NewName;
      Data.FFileName := Data.FFileSpec.FileName;
      TMan.FLibContainer.WriteHeader(Indx, Data.FFileSpec);
    end;

  begin
      OldN := OldName;
      NewN := NewName;
      OldName := OldName + '_Data';
      NewName := NewName + '_Data';
      if SameText(OldName, NewName) then
        Exit;

      LCOpen := True;
      try
      if TMan.FLibContainer = nil then
      begin
        TMan.SetNames;
        TMan.FLibContainer := TFileContainer.Create(TMan.FLibPath, 0, False);
        TMan.FLibContainer.Unicode := Tman.Unicode;
        LCOpen := False;
      end;
      SetNames(Self);
      DoRename;

      OldName := OldN + '_Data_IndexTable';
      NewName := NewN + '_Data_IndexTable';
      if TMan.FLibContainer.IndexOf(OldName) <> -1 then
        DoRename;

      OldName := OldN + '_KeyLinks';
      NewName := NewN + '_KeyLinks';
      DoRename;

      OldName := OldN + '_Keys';
      NewName := NewN + '_Keys';
      DoRename;

      FTableName := NewN;
      SetNames(Self);

      sl := TStringList.Create;
      try
        sl.LoadFromFile(TMan.FTableCfgName);
        s := sl.Text;
        s := StringReplace(S, '[' + OldN, '['+ NewN, [rfReplaceAll]);
        s := StringReplace(S, '[Calc_' + OldN, '[Calc_'+ NewN, [rfReplaceAll]);
        sl.Text := s;
        sl.SaveToFile(TMan.FTableCfgName);
      finally
        sl.Free;
      end;
      finally
        if not LCOpen then
        FreeAndNil(TMan.FLibContainer);
      end;
  end;

  procedure TILTable.AlterTableSpace(const NewSpace: integer);
  { Changes the tablespace of the table }
  begin
    AlterTable(iloAlterTableSpace, NewSpace, nil, nil);
  end;

  procedure TILTable.AlterField(const FieldName: string; const Alter: TAlterFieldOption;
    const Value: Variant);
  var
    Indx: integer;
    OldLength: integer;
  begin
    Indx := Fields.IndexOf(FieldName);
    Comment1 := FieldName;
    try
    if Indx = -1 then
      HandleILException(E_InvalidFieldName);
    if Alter = afIndexed then
    begin
      if Value = Fields.Items[Indx].Indexed then
        Exit;
      if Fields.Items[Indx].Identifier then
        HandleILException(E_DropIndexIdentifier);
      Fields.Items[Indx].FIndexed := Value;
      RebuildIndexes;
    end
    else if Alter = afMaxLength then
    begin
      if Value = Fields.Items[Indx].MaxLength then
        Exit;

      OldLength := Fields.Items[Indx].MaxLength;
      Fields.Items[Indx].FMaxLength := Value;
      if Fields.Items[Indx].Indexed and (Value > OldLength) then
      begin
        AlterIndexSpace;
      end;
    end
    else if Alter = afMaxValues then
    begin
      if Value = Fields.Items[Indx].MaxValues then
        Exit;
      Fields.Items[Indx].FMaxValues := Value;
    end
    else if Alter = afDecimals then
    begin
      if Value = Fields.Items[Indx].Decimals then
        Exit;
      Fields.Items[Indx].FDecimals := Value;
    end
    else if Alter = afInitialValue then
    begin
      if Value = Fields.Items[Indx].InitialValue then
        Exit;
      Fields.Items[Indx].FInitialValue := Value;
    end
    else if Alter = afDateFormat then
    begin
      if Value = Fields.Items[Indx].DateFormat then
        Exit;
      Fields.Items[Indx].FDateFormat := Value;
    end
    else if Alter = afMandatory then
    begin
      if Value = Fields.Items[Indx].Mandatory then
        Exit;
      Fields.Items[Indx].FMandatory := Value;
    end
    else if Alter = afReportField then
    begin
      if Value = Fields.Items[Indx].ReportField then
        Exit;
      Fields.Items[Indx].FReportField := Value;
    end
    else if Alter = afReportCaption then
    begin
      if SameText(Value, Fields.Items[Indx].ReportCaption) then
        Exit;
      Fields.Items[Indx].FReportCaption := Value;
    end
    else if Alter = afIdentifier then
    begin
      if Fields.Items[Indx].Identifier = Value then
        Exit;
      if not Fields.Items[Indx].Indexed then
        HandleILException(E_NotAnIndexedField);
      Fields.Items[Indx].FIdentifier := Value;
    end
    else if Alter = afIndexPriority then
    begin
      if Fields.Items[Indx].FIndexPriority = Value then
        Exit;
      Fields.Items[Indx].FIndexPriority := Value;
    end
    else if Alter = afAlignment then
    begin
      if Fields.Items[Indx].FAlignment = Value then
        Exit;
      Fields.Items[Indx].FAlignment := Value;
    end
    else if Alter = afColumnWidth then
    begin
      if Fields.Items[Indx].FColumnWidth = Value then
        Exit;
      Fields.Items[Indx].FColumnWidth := Value;
    end
    else if Alter = afImagetext then
    begin
      if Fields.Items[Indx].FImageText = Value then
        Exit;
      Fields.Items[Indx].FImageText := Value;
    end;

    { Update config file }
      case Alter of
        afIdentifier:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'Identifier', Value);
        afUnique:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'Unique', Value);
        afMaxLength:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'MaxLength', Value);
        afMaxValues:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'MaxValues', Value);
        afIndexPriority:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'IndexPriority',
            integer(Value));
        afMandatory:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'Mandatory', Value);
        afIndexed:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'Indexed', Value);
        afReportField:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'ReportField', Value);
        afReportCaption:
          Tman.FCfgFile.WriteString(TableName + '_' + FieldName, 'ReportCaption', Value);
        afColumnWidth:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'ColumnWidth', Value);
        afImagetext:
          Tman.FCfgFile.WriteBool(TableName + '_' + FieldName, 'ImageText', Value);
        afDecimals:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'Decimals', Value);
        afInitialValue:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'InitialValue', Value);
        afDateFormat:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'DateFormat', Value);
        afAlignment:
          Tman.FCfgFile.WriteInteger(TableName + '_' + FieldName, 'Alignment', Integer(Value));

      end;
    finally
      Comment1 := '';
    end;
  end;

  function TILTable.GetNextRecordID;
  begin
      Result := Tman.FCfgFile.ReadInteger(TableName, 'NextRecordID', 1);
      Tman.FCfgFile.WriteInteger(TableName, 'NextRecordID', Result + 1);
  end;

  function TILTable.GetFocusedRecord : TILDataRecord;
  begin
    Result := Dataset.FocusedRecord;
  end;

  procedure TILTable.DoAddToTable(ADataRecord: TILDataRecord);

  { Adds a ADatarecord to the library. The record must have been loaded
    Called from Save procs of TILDataset and TILDatarecord  which do the saving }
  var
    Offs, Count: int64;
    S: string;
    i, j: integer;
    DRec, R: TILDataRecord;
    DS: TILDataset;
    Lvl : integer;
    sl : TStringList;
    RecIndx : integer;
    Idx : integer;

  procedure SetSiblingIndex;
  var
     SID: integer;
     SSID : string;
     PID: integer;
     i: integer;
     DR : TILDataRecord;
     Chng: Boolean;
     Indx: integer;

   begin
    with Self as TILTreeViewTable do
    begin
      PID := ADatarecord.TVParentID;
      if ADatarecord.TVSiblingIndex = -1 then
        SID := GetTVChildCount(PID)
      else
      begin
        SID := ADataRecord.TVSiblingIndex;
        { Renumber siblings }
        DS := GetTVChildren(PID);
        try
          Chng := false;
          for i := 0 to DS.Count - 1 do { Except the one in question }
          begin
            DR := DS[i];
            if DR[c_SysFieldNames[SYS_SiblingIndex]].AsInteger >= SID then
            begin
              DR[c_SysFieldNames[SYS_SiblingIndex]].AsInteger :=
                DR[c_SysFieldNames[SYS_SiblingIndex]].AsInteger + 1;
              Chng := True;
            end;
          end;
          if Chng then
          begin
            DS.Save;
            { Refresh the changed records }
            for i := 0 to DS.Count - 1 do
            begin
              DR := DS[i];
              if DR[c_SysFieldNames[SYS_SiblingIndex]].AsInteger > SID then
              begin
                Indx := Dataset.IndexOf(DR.RecordIndex);
                if Indx <> -1 then
                  Dataset[Indx].Refresh;
              end;
            end;
          end;
        finally
          DS.Free;
        end;
      end;
    end;
    SSid := IntToStr(SID);
    SSid := FillZeroes(Ssid, 7);

    ADataRecord[c_SysFieldNames[SYS_SiblingIndex]].AsString := SSID;
    DRec[c_SysFieldNames[SYS_SiblingIndex]].AsString := SSID;
  end;


  begin
    if IsReadOnly then
      HandleILException(E_ReadOnly, Name);
    for i := 0 to ADataRecord.Count - 1 do
      if (ADataRecord[i].Count > 0) and TMan.IsValueNull(ADatarecord[i][0]) then
        ADataRecord[i].FValues.Clear;

    DRec := CreateDataRecord;
    { Used to convert lookup values to lookup ids }
    DRec.Assign(ADataRecord);
    try
      for i := 0 to DRec.Count - 1 do
      begin
        if DRec[i].FieldDef.LookupType = looTableRef then
        begin
          for j := 0 to DRec[i].Count - 1 do
          begin
            if DRec[i].IsEmpty then
              Continue;

            S := GetLookupTableID(DRec[i].FieldDef.LookupTable, DRec[i].FieldDef.LookupField,
              DRec[i].FValues[j]);
            if S = '' then
            begin
              Comment1 := DRec[i].FValues[j];
              HandleILException(E_LookupNotRegistered);
            end;
            DRec[i].FValues[j] := S;
          end;
        end

        { Check path link integrity }
        else if (DRec[i].FieldDef.LookupType = looTableKey) and not DRec[i].IsEmpty then
        begin
          sl := TStringList.Create;
          if DRec[i].FieldDef.FDataType = dtNumber then
            sl.Add(DRec[i].FieldDef.LookupField + ' = ' + DRec[i].AsString)
          else
            sl.Add(DRec[i].FieldDef.LookupField + ' = ' + '''' + DRec[i].AsString + '''');
          try
           if not DRec[i].FieldDef.LookupTable.Exists(sl, Idx) then
           begin
             Comment1 := DRec[i].Asstring;
             HandleILException(E_InvalidTableKey);
           end;
           finally
             sl.Free;
           end;
        end;
      end;

      if (TVTable <> nil) and (UpdateTarget = utrDataSet) then
        with TVTable do
        begin
           if ADataRecord[c_SysFieldNames[SYS_Level]].IsEmpty then
           begin
            Lvl := GetLevel;
            ADataRecord[c_SysFieldNames[SYS_Level]].AsInteger := Lvl;
            DRec[c_SysFieldNames[SYS_Level]].AsInteger := ADataRecord[c_SysFieldNames[SYS_Level]].AsInteger;
           end;

           if ADataRecord.TVParentRecord <> nil then
           { If the record has a parent that is not saved trace back to the first record in the parent chain that is
           saved and save its child. Repeat this until the record has a saved parent }
           begin
             repeat
              if (ADatarecord.TVParentRecord.RecordIndex = -1)
              and (ADataRecord[c_SysFieldNames[SYS_Level]].AsInteger <> 0) then
              begin
               R := ADataRecord.TVParentRecord;
               while (R <> nil) and (R.TVParentRecord <> nil) and (R.TVParentRecord.RecordIndex = -1) do
                R := R.TVParentRecord;
               if R <> nil then
                DoAddToTable(R)
               else
                HandleILException(0, 'Broken link');  { Will not happen ... }
              end;
             until (ADataRecord.TVParentRecord = nil)
              or (ADatarecord.TVParentRecord.RecordIndex <> -1);

             ADataRecord.TVParentID := ADatarecord.TVParentRecord.RecordIndex;
           end;
           if ADataRecord.TVParentID <> -1 then
           begin
            ADataRecord.ConnectToMaster(ADataRecord.TVParentID);
            if ADataRecord[c_SysFieldNames[SYS_SiblingIndex]].IsEmpty then
             SetSiblingIndex;    { DONE : Change it? May not ever be called, but keep an eye}
            DRec[c_SysFieldNames[SYS_LinkID]].AsInteger := ADataRecord[c_SysFieldNames[SYS_LinkID]].AsInteger;
           end;
           Inval;
        end

      else if IsDetail and (UpdateTarget = utrDataset) then
      begin
        try
          if (FMasterSource.Dataset.Position = -1) then
          begin
            RecIndx := -1;
            if HasHandler and Assigned(DataHandler.FOnMissingLink) then
            begin
              Datahandler.FOnMissingLink(Self, FMasterSource, RecIndx);
              if RecIndx = -1 then
                Abort;
            end;
            if RecIndx = -1 then
             HandleILException(E_UndefinedLinkRecord, '');
          end
          else
          begin
            RecIndx := FMasterSource.Dataset.Items
             [FMasterSource.Dataset.Position].RecordIndex;
            if FMasterSource.FocusedRecord.RecordIndex = -1 then
              HandleILException(E_SaveMasterRecord, '');
          end;
          ADataRecord.ConnectToMaster(RecIndx);
        except
          if ADataRecord.OwnerDataset <> nil then
            ADataRecord.OwnerDataset.Delete(ADataRecord.Index);
          raise;
        end;
      end;
      if Assigned(ADataRecord.Image) and ADataRecord.MustStoreLocally
        (ADataRecord.FileName) then
      begin
        SaveImage(ADataRecord.Image, Offs, Count);
        ADataRecord.FThumbnailOffset := Offs;
        ADataRecord.FThumbnailSize := Count;
        DRec.FThumbnailOffset := Offs;
        DRec.FThumbnailSize := Count;
      end;

      SetInitials(DRec);
      UpdateIndexes(DRec, -1, -1);
      ADataRecord.AssignSystemValues;
      DRec.AssignSystemValues;
      DRec.FState := 'NO';
      if DRec.FRecordID <= 0 then
        DRec.FRecordID := GetNextRecordID;
      DRec.FRecordIndex := RecordCount;

      for j := 0 to DRec.Count-1 do
      begin
         if DRec[j].FieldDef.DataType = dtLongText then
          if DRec[j].FTextUpdated then
          begin
            DRec[j].SaveLT;
            ADataRecord[j].FValues[0] := DRec[j].FValues[0];
          end;
      end;

      if (Drec.FFileName <> '') and (DRec.FImgTableIndex = -1) then
      begin
          Drec.FImgFilePos := Drec.SaveFileResourceName(-1, Drec.FFileName);
          ADatarecord.FImgFilePos := DRec.FImgFilePos;
      end;

      SaveResourceStrings(DRec);

      S := RecordToXML(DRec, False);

      RestoreResourceStrings(Drec);
      for I := 0 to ADataRecord.Count-1 do
      begin
        if ADataRecord[i].FieldDef.DataType = dtResource then
          ADataRecord[i].FResFilePos := Drec[i].FResFilePos;
        ADataRecord[i].FieldDef.FIsUpdated := True;
      end;
    finally
      DRec.Free;
    end;
    ADataRecord.FRecordID := DRec.RecordID;
    ADataRecord.FRecordIndex := RecordCount;

    TMan.AddToTransLog(TableName, RecordCount, dboInsert, '', S, ADataRecord);

    ResetOldValues(ADataRecord);
    if Self is TILDataTable then
      TILDataTable(Self).FetchParentFields(ADataRecord);
    for I := 0 to ADataRecord.Count-1 do
    begin
       ADataRecord[i].FTextUpdated := false;
       ADataRecord[i].FTextSourceChanged := false;
    end;
  end;

  procedure TILTable.SetInitials(ADataRecord : TILDataRecord);
  var
    I : integer;
  begin
    for I := 0 to ADataRecord.Count-1 do
    begin
        if ((ADataRecord[i].FieldDef.FieldType in [ftUserData, ftAutoIncrement])
        or SameText(ADataRecord[i].FieldDef.FFieldName, PidNameOf(PID_PHOTO_CAMDATE)))
        and (ADataRecord[i].FieldDef.LookupType <> looTableRef)
        and ADataRecord[i].IsEmpty then
        begin
           if ADataRecord[i].FieldDef.FieldType = ftAutoIncrement then
           begin
             ADataRecord[i].AsInteger := ADataRecord[i].FieldDef.GetNextAutoValue(True);
           end
           else if ADataRecord[i].FieldDef.InitialValue <> '' then
           begin
               ADataRecord[i].AsString := ADataRecord[i].FieldDef.InitialValue;
           end;
        end;
    end;
    if HasHandler and Assigned(Datahandler.FOnInitRecord) then
      Datahandler.FOnInitRecord(Datahandler, dboInsert, ADataRecord);
  end;

  procedure TILTable.IncludeParentFields(DataRec : TILDataRecord);
  var
   j : integer;
   sl : TStrings;
   Smpl : string;
  begin
       for j := 0 to DataRec.Count-1 do
        begin
         { Check ParentField lookups and long texts }
         if DataRec[j].FieldDef.LookupType = looParentField then
         begin
          sl := GetLookupParentValue
           (DataRec[j].FieldDef.LookupTable, DataRec[j].FieldDef.LookupField, DataRec);
          if sl <> nil then
          begin
           DataRec[j].FValues.Assign(sl);
           sl.Free;
          end;
         end
         else  if (DataRec[j].FieldDef.DataType = dtLongText)
         and HasHandler and Assigned(Datahandler.FOnUpdateLongText) then
         begin
           Smpl := '';
           Datahandler.FOnUpdateLongText(DataHandler, DataRec[j], Smpl, [ltaUpload]);
           DataRec[j].TextSourceChanged := false;
           DataRec[j].FTextUpdated := True;
          end;
        end;
  end;

  procedure TILTable.DoAddToDataset(ADataRecord: TILDataRecord;
    const AnImageFile: string; AnImage: TPicture);

  { Does the core operations when adding a new record to a dataset }
  var
    W, H, P: integer;
    S: string;
    Indx: integer;
    i: integer;
    PR: TILDataRecord;
    SCnt1, SCnt2 : integer;
    RecIndx : integer;
    MH : TILMessageHandler;
    Err : integer;
  begin
     if (TMan.AutoSave) and (FocusedRecord <> nil) then
     begin
       if FocusedRecord.Changed then
       begin
         i := FocusedRecord.Validate(True);
         if i > 0 then
          HandleILException(i)
         else if i = 0 then
           DBWork.Save;
       end;

     end;
     if HasViewer and Assigned(Viewer.Report.FSelectedCell) then
     begin
      if Viewer.Report.FSelectedCell.FEditing then
      begin
          Viewer.FCellEditor.Cancel;
      end;
    end
    else if (Dataset.Count <> 0) and (Dataset.Position <> -1) and (Dataset.Position < Dataset.Count) then
      if Dataset[Dataset.Position].FEditingImageText then
        Dataset[Dataset.Position].CancelEditImageText;

    if IsReadOnly then
      HandleILException(E_ReadOnly,  Name)
    else if (Relations = relOne) and not IsDetail and (RecordCount - DeletedCount  > 0) then
         HandleILException(E_RelConstraint)
    else if Dataset.IsReadOnly then
      Dataset.Clear;

    for i := 0 to ADataRecord.Count - 1 do
    begin
      if (ADataRecord[i].Count > 0) and TMan.IsValueNull(ADataRecord[i][0]) then
        ADataRecord[i].FValues.Clear;
      if ADataRecord[i].FieldDef.LookupType = looTableKey then
      begin

      end;
    end;

    ADataRecord.FRecordID := GetNextrecordID;
    ADataRecord.FOwnerDataset := Dataset;

    if TVTable  <> nil then
    { Special processing for treeview table }
    with TVTable do
    begin
        if (TreeViewParams.InsertMode in[timSiblingBefore, timSiblingAfter])
        and (FocusedRecord = nil) then
          HandleILException(E_TVReference);
        PR := nil;
        if TreeViewParams.InsertParentRecord <> nil then
        begin
          if TreeViewParams.InsertParentRecord.IsDeleted then
            HandleILException(E_TVReference);
          ADatarecord.FTVParentRecord:= TreeViewParams.InsertParentRecord;
          PR := TreeViewParams.InsertParentRecord;
          ADataRecord.ConnectToMaster(PR);
        end;

        case TreeViewParams.InsertMode of
          timRoot: TreeViewParams.SetInsertChild(nil);
          timChild: TreeViewParams.SetInsertChild(PR);
          timSiblingBefore: TreeViewParams.SetInsertBefore(FocusedRecord);
          timSiblingAfter: TreeViewParams.SetInsertAfter(FocusedRecord);
        end;

        { If new sibling index is not at end, renumebring is necessary. Renumbering
        is only possible when all siblings are present in dataset }
        if TreeViewParams.InsertParentRecord <> nil then
        begin
         if (TreeViewParams.FInsertSiblingIndex <> -1)
         and (ADatarecord.TVParentRecord.RecordIndex <> -1) then
         begin
           Scnt1 := GetTVChildCount(ADatarecord.TVParentRecord);
           Scnt2 := GetTVChildCount(ADatarecord.TVParentRecord.RecordIndex);
           if SCnt1 < SCnt2 then
             HandleILException(E_SiblingsIncomplete);
         end;
        end
        else if TreeViewParams.InsertSiblingIndex <> -1 then
        begin
          Scnt1 := GetTVChildCount(nil);
          Scnt2 := GetTVChildCount(-1);
           if SCnt1 < SCnt2 then
            HandleILException(E_SiblingsIncomplete);
        end;
        ADatarecord.TVSiblingIndex := TreeViewParams.InsertSiblingIndex;
    end

    else if IsDetail then
    begin
      if (FMasterSource.Dataset.Position = -1) then
      begin
        RecIndx := -1;
        if HasHandler and Assigned(Datahandler.FOnMissingLink) then
        begin
          Datahandler.FOnMissingLink(Self, FMasterSource, RecIndx);
          if RecIndx = -1 then
            Abort;
        end;
        if RecIndx = -1 then
         HandleILException(E_UndefinedLinkRecord, '');
      end
      else
      begin
        RecIndx := FMasterSource.Dataset.Items[FMasterSource.Dataset.Position].RecordIndex;
        if FMasterSource.FocusedRecord.RecordIndex = -1 then
          HandleILException(E_SaveMasterRecord, '');
      end;
      ADataRecord.ConnectToMaster(RecIndx);
    end;

    if Assigned(AnImage) then
    begin
      ADataRecord.FImage := Vcl.Graphics.TBitmap.Create;
      ADataRecord.FOwnsImage := True;
      ADataRecord.FImage.Assign(AnImage.Graphic);
      if ADataRecord.FileName <> '' then
      begin
        GetImgDimensions(W, H);
        ScaleBitmap(ADataRecord.FImage, W, H);
      end;
      ADataRecord.FImageFetched := True;
      ADataRecord.FMediaType := MED_IMAGE;
    end
    else if AnImageFile <> '' then
    begin
      ADataRecord.LoadFileData(AnImageFile);
    end
    else if (TMan.SharedImages <> nil) and HasHandler then
    begin
      for I := 0 to DataHandler.DataFields.Count-1 do
        if DataHandler.DataFields.Items[i].ImageTag then
        begin
          Indx := ADataRecord.IndexOf(DataHandler.DataFields.Items[i].FieldName);
          with Self as TILDataTable do
          GetFromImageTable(ADatarecord[Indx].AsString, ADataRecord);
        end;
    end;

    ADataRecord.AssignSystemValues;

    { Check initials }
    SetInitials(ADataRecord);

    if PrecheckConstraints then
      if not CheckMandatories(ADataRecord, S) then
      begin
        Comment1 := S;
        HandleILException(E_MandatoryValue);
      end;

    if TVTable <> nil then
    with TVTable do
    begin
       ADataRecord[c_SysFieldNames[SYS_Level]].AsInteger := GetLevel;
       i := GetSiblingIndex;
       ADataRecord[c_SysFieldNames[SYS_SiblingIndex]].AsInteger := i;
       AddToSiblingList(i, TreeViewParams.InsertParentRecord, ADataRecord);
       RenumberSiblings(TreeViewParams.InsertParentRecord, 0);
    end;

    if Fields.HasTempFields then
      ResetDataset;

    Dataset.Add(ADataRecord);
    P := Dataset.Position;
    Dataset.FPosition := Dataset.Count-1;
    if HasViewer then
     Viewer.CreateReport;

    begin
      if not MultiSelected then
      begin
        if P <> -1 then
         Dataset.Items[P].FSelected := false;
        Dataset.Items[Dataset.Position].FSelected := True;
      end;
    end;

    Dataset.RunFocus(Dataset.Position, False);
    if HasHandler and Assigned(DataHandler.FOnCalcField) then
    begin
      for I := 0 to ADataRecord.Count-1 do
      begin
        if (ADataRecord[i].FieldDef.FieldType = ftCalculated)
         and not ADataRecord[i].FieldDef.FuserCalc then
         try
          DataHandler.FOnCalcField(Self, ADataRecord, ADataRecord[i]);
         except
          { ignore if error }
         end;
      end;
    end;
    IncludeParentFields(ADataRecord);
    if HasViewer then
    begin
      if roEditable in Viewer.ReportOptions then
       FViewer.Report.SelectedCell :=
        Viewer.Report.Cells[FViewer.Report.RecordIndexToRowIndex(Dataset.Position), 0];
      FViewer.UpdateScrollRange;
      FViewer.Scrollto(Dataset.Count-1);
    end;
    WriteControls(ADataRecord, duData);
    if TVControl <> nil then
     TVControl.BuildTree(Dataset);
    Inval;
    Dataset.FChngTracker.Add(ADataRecord);
    if TMan.AutoSave and (DBWork.Cansave(Err) = 0) then
      DBWork.Save;
    I := -1;
    MH := TMan.GetMessageHandler(Self, I);
    while MH <> nil do
    begin
     MH.AddCount(dboInsert, 1);
     MH := TMan.GetMessageHandler(Self, I);
    end;
  end;

  function TILTable.AddToDataset(Data: TILDataRecord): integer;
  { Adds a ADatarecord without an image to Dataset }
  begin
     DoAddToDataset(Data, '', nil);
     Result := Dataset.Count - 1;
  end;

  function TILTable.AddToDataset: integer;

  { Adds a data record without an image, reading the data controls }
  var
    Rec: TILDataRecord;
    res: integer;
  begin
    Result := 0;
    Rec := CreateDataRecord;
    res := ReadControls(Rec, duData);
    if res > 0 then
    begin
      Rec.Free;
      HandleILException(res, '');
    end;
    try
      Result := AddToDataset(Rec);
    except
      Rec.Free;
    end;
  end;

  function TILTable.AddToDataset(MediaFile: string): integer;

  { Adds a data record with an image, reading the data controls }
  var
    Rec: TILDataRecord;
    res: integer;
  begin
    Result := 0;
    Rec := CreateDataRecord;
    res := ReadControls(Rec, duData);
    if res > 0 then
    begin
      Rec.Free;
      HandleILException(res, '');
    end;
    try
      Result := AddToDataset(Rec, MediaFile);
    except
     Rec.Free;
    end;
  end;

  function TILTable.AddToDataset(Data: TILDataRecord; MediaFile: string): integer;
  var
    P : TPicture;
    Url : Boolean;
    img : TWicImage;
    NormalFile : Boolean;
  { Adds a ADatarecord with an image to Dataset }
  begin
    Result := -1;
    Url := false;
    if SameText(MediaFile, CLIPBOARD_IMAGE) then
    begin
      NormalFile := false;
      if Clipboard.HasFormat(CF_BITMAP) then
      begin
       P := TPicture.Create;
       try
         P.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(cf_bitmap), 0);
         Result := AddToDataset(Data, P);
       finally
         P.Free;
       end;
      end
      else if Clipboard.HasFormat(CF_Text) then
      begin
        Url := false;
        Mediafile := GetAddressFromClipbrd(Url);
        if MediaFile = '' then
          HandleILException(E_MissingImage)
        {$IfDef UseIndy}
        else if Url then
        begin
          P := TPicture.Create;
          Img := DownLoadImage(MediaFile);
          try
            P.Assign(Img);
            Result := AddToDataset(Data, P);
          finally
            if Img <> nil then
              Img.Free;
            P.Free;
          end;
        end
        {$EndIf}
        else if Url then
          HandleILException(E_ImageNotAvailable)
        else
          NormalFile := True;
      end
      else
        HandleILException(E_MissingImage);
      if not NormalFile then
       Exit;
    end;

    if not IsMediaFile(MediaFile) then
    begin
      Comment1 := MediaFile;
      Data.Free;
      HandleILException(E_UnsupportedFileFormat, '');
    end;
    try
      DoAddToDataset(Data, MediaFile, nil);
    except
      Data.Free;
      raise;
    end;
    Result := Dataset.Count - 1;
  end;

  function TILTable.AddToDataset(Data: TILDataRecord;
    AnImage: TPicture): integer;
  { Adds a ADatarecord with an image to Dataset }
  begin
    try
      DoAddToDataset(Data, '', AnImage);
    except
      Data.Free;
      raise;
    end;
    Result := Dataset.Count - 1;
  end;

  function TILTable.AddToDataset(AnImage: TPicture): integer;

  var
    Rec: TILDataRecord;
    res: integer;
  begin
    Rec := CreateDataRecord;
    res := ReadControls(Rec, duData);
    Result := 0;
    if res > 0 then
    begin
      Rec.Free;
      HandleILException(res, '');
    end;
    try
      Result := AddToDataset(Rec, AnImage);
    except
      Rec.Free;
    end;
  end;

  function TILTable.AddToTable(Data: TILDataRecord): integer;

  var
    AdataSet: TILDataset;
    MH : TILMessageHandler;
    Ind : integer;
  begin
    AdataSet := TILDataset.Create(Self);
    try
      AdataSet.FOwnsRecords := false;
      Data.FRecordIndex := -1;
      try
        Data.AssignSystemValues;
      except
        Data.Free;
        raise;
      end;
      AdataSet.Add(Data);
      Ind := -1;
      MH := TMan.GetMessageHandler(Self, Ind);
      while MH <> nil do
      begin
        MH.AddCount(dboInsert, 1);
        MH := TMan.GetMessageHandler(Self, Ind);
      end;
    AdataSet.Save;
    finally
      AdataSet.Free;
    end;
    Result := RecordCount - 1;
  end;

  function TILTable.AddToTable(Data: TILDataRecord; const MediaFile: string): integer;
  var
    AdataSet: TILDataset;
    MH : TILMessageHandler;
    Ind : integer;
  begin
    try
      if not IsMediaFile(MediaFile) then
      begin
        Comment1 := MediaFile;
        HandleILException(E_UnsupportedFileFormat, '');
      end;
      AdataSet := TILDataset.Create(Self);
      try
        AdataSet.FOwnsRecords := false;
        Data.LoadFileData(MediaFile);
        Data.FRecordIndex := -1;
        Data.AssignSystemValues;
        AdataSet.Add(Data);
        Ind := -1;
        MH := TMan.GetMessageHandler(Self, Ind);
        while MH <> nil do
        begin
          MH.AddCount(dboInsert, 1);
          MH := TMan.GetMessageHandler(Self, Ind);
        end;
        AdataSet.Save;
      finally
        AdataSet.Free;
      end;

      Result := RecordCount - 1;
    except
      Data.Free;
      raise;
    end;
  end;

  function TILTable.GetReadOnly;
  begin
    Result := false;
    if Assigned(TMan) then
     Result := TMan.IsReadOnly;
    if not Result then
      Result := FIsReadOnly;
  end;

  function TILTable.DoReadImages(ImageFiles: TStrings;
    const ReplaceDataset: Boolean): integer;

  { Reads images in Imagefiles and import tem to Dataset }
  var
    i: integer;
    DataRec: TILDataRecord;
    F, L: integer;
    FName: string;
    Cnt: integer;
    PreCheck : Boolean;
    RecIndx : integer;
    MH : TILMessageHandler;
  begin
    if IsDetail and (FMasterSource.Dataset.Position = -1) then
    begin
      RecIndx := -1;
      if HasHandler and Assigned(DataHandler.FOnMissingLink) then
      begin
        DataHandler.FOnMissingLink(Self, FMasterSource, RecIndx);
        if RecIndx = -1 then
          Abort;
      end;
      if RecIndx = -1 then
       HandleILException(E_UndefinedLinkRecord, '');
    end
    else if IsDetail then
    begin
       RecIndx := FMasterSource.Dataset.Items[FMasterSource.Dataset.Position].RecordIndex;
       if (MasterSource.FocusedRecord.RecordIndex = -1) then
         HandleILException(E_SaveMasterRecord, '');
    end;
    if HasDatasetTemps then
      HandleILException(E_UpdateTempFields, '');
    Result := ImageFiles.Count;
    if ReplaceDataset or Fields.HasTempFields then
    begin
      if Fields.HasTempFields then
        ResetDataset
      else
        Dataset.Clear;
    end;
    Dataset.BeginUpdate;
    Precheck := PrecheckConstraints;
    PrecheckConstraints := false;
    try
      Cnt := ImageFiles.Count;
      TMan.FProgressTicker := GetTickCount;
      for i := 0 to Cnt - 1 do
      begin
        TMan.Progress(Self, iloReadFile, Cnt, i);
        FName := ImageFiles[i];
        if not IsMediaFile(FName) then
        begin
          Dec(Result);
          Continue;
        end;

        DataRec := TILDataRecord.Create(Self, false);
        DataRec.FOwnerDataset := Dataset;
        DataRec.LoadFileData(FName);
        DataRec.InitFields(nil);

        if IsDetail and Connected then
        begin
          try
            DataRec.ConnectToMaster(RecIndx);
          except
            Dec(Result);
            Continue;
          end;
        end;

        SetInitials(DataRec);
        IncludeParentFields(DataRec);
        Dataset.Add(DataRec);
        Dataset.FChngTracker.Add(DataRec);
        if HasViewer then
        begin
         FViewer.GetVisibles(F, L);
         if (i <= L) and (FViewer.ViewMode in [vmImages]) then
           FViewer.Repaint;
        end;
      end;
      if HasViewer then
        FViewer.FFirstItemIndex := 0;
    finally
      Dataset.EndUpdate;
      PrecheckConstraints := Precheck;
    end;
    Dataset.Position := Dataset.Count-1;
    TMan.Progress(Self, iloFinished, 0, 0);
    if TMan.UseCommandModes then
      TMan.CommandMode := cmUpdate;
    i := -1;
    MH := TMan.GetMessageHandler(Self, i);
    while MH <> nil do
    begin
      MH.AddCount(dboInsert, Result, False);
      MH := TMan.GetMessageHandler(Self, i);
    end;
    if TMan.AutoSave then
      DBWork.Save;

  end;

  function TILTable.DoReadXMLFile(const FileName: string; const Target: integer): integer;

  { Reads data from an xmlfile into Dataset or Table. Field structure must correspond
    to field list. Image data is not relevant.
    Cannot contain caculated or system fields.
    Target 0 = Table, 1 = Dataset }
  var
    i: integer;
    ALibstring: string;
    ANode: IXMLNode;
    Rec: TILDataRecord;

    function GetValue(AFieldName: string; AnIndex: integer): string;

  var
    P1, P2: integer;
    StartIdx, EndIdx, StartPos: integer;
    Substr: string;
    i: integer;
  begin
    if AnIndex <> -1 then { User fields }
    begin
      StartPos := Pos('<Data>', ALibstring);
      if StartPos = 0 then
        StartPos := Pos('<' + XData + '>', ALibstring);
      P1 := Pos('<' + AFieldName + '>', ALibstring, StartPos);
      if P1 = 0 then
      begin
        Comment1 := AFieldName;
        HandleILException(E_InvalidFieldName, '');
      end;
      P2 := Pos('</' + AFieldName + '>', ALibstring, StartPos);
      StartIdx := P1 + Length('<' + AFieldName + '>');
      EndIdx := P2;
      Substr := Copy(ALibstring, StartIdx, EndIdx - StartIdx);
      StartPos := 1;
      for i := 0 to AnIndex do
      begin
        P1 := Pos('<Value>', Substr, StartPos);
        if P1 = 0 then
         P1 := Pos('<' + XValue + '>', Substr, StartPos);
        if P1 = 0 then
        begin
          Result := '';
          Break;
        end;
        if i = AnIndex then
        begin
          P2 := Pos('</Value>', Substr, P1);
          if P2 = 0 then
          begin
            P2 := Pos('</' + XValue + '>', Substr, StartPos);
            StartIdx := P1 + Length('<' + XValue + '>');
          end
          else
           StartIdx := P1 + Length('<Value>');
          EndIdx := P2;
          Result := Copy(Substr, StartIdx, EndIdx - StartIdx);
          Exit;
        end;
        StartPos := P1 + 1;
      end;
    end
    else { System fields }
    begin
      P1 := Pos('<' + AFieldName + '>', ALibstring);
      P2 := Pos('</' + AFieldName + '>', ALibstring);
      StartIdx := P1 + Length('<' + AFieldName + '>');
      EndIdx := P2;
      Result := Copy(ALibstring, StartIdx, EndIdx - StartIdx);
    end;
  end;

  function CreateRecord: TILDataRecord;

  var
    Val: string;
    Fld: TILField;
    Indx: integer;
    i: integer;
  begin
    Result := TILDataRecord.Create(Self, false);
    Result.FOwnsImage := True;

    { Systemfields }
    if Pos(c_SysFieldNames[SYS_RecordState], ALibString) <> 0 then
      Result.FState := GetValue(c_SysFieldNames[SYS_RecordState], -1)
    else
      Result.FState := GetValue(XRecState, -1);

    if Pos(c_SysFieldNames[SYS_RegDate], ALibString) <> 0 then
    Result.FRegDate :=
      StrToFloat(GetValue(c_SysFieldNames[SYS_RegDate], -1), Translator.FEnFormatSettings)
    else
      StrToFloat(GetValue(XRegDate, -1), Translator.FEnFormatSettings);

    if Pos(c_SysFieldNames[SYS_ChangedDate], ALibString) <> 0 then
    Result.ChangedDate :=
      StrToFloat(GetValue(c_SysFieldNames[SYS_ChangedDate], -1), Translator.FEnFormatSettings)
    else
      StrToFloat(GetValue(XChangeDate, -1), Translator.FEnFormatSettings);

    { User fields }
    for i := 0 to Fields.Count - 1 do
    begin
      Indx := 0;
      Fld := TILField.Create(Self, Fields.Items[i].FieldName, FFields);
      Fld.FOwnerRecord := Result;
      Val := GetValue(Fields.Items[i].FieldName, Indx);
      while Val <> '' do
      begin
        if IsStringType(Fld.FieldDef.DataType) then
        begin
          Val := StringReplace(Val, '&amp;', '&', [rfReplaceAll]);
          Val := StringReplace(Val, '&lt;', '<', [rfReplaceAll]);
          Val := StringReplace(Val, '&gt;', '>', [rfReplaceAll]);
          Val := StringReplace(Val, '&quot;', '"', [rfReplaceAll]);
          Val := StringReplace(Val, '&apos;', '''', [rfReplaceAll]);
        end;
        Fld.FValues.Add(Val);
        Inc(Indx);
        Val := GetValue(Fields.Items[i].FFieldName, Indx);
      end;
      Result.Add(Fld);
    end;
  end;

  begin
    Result := 0;
    if not Connected then
      Exit;
    FXMLDoc.LoadFromFile(FileName);
    if Target = 1 then
      Dataset.BeginUpdate;
    try
      for i := 0 to FXMLDoc.DocumentElement.ChildNodes.Count - 1 do
      begin
        ANode := FXMLDoc.DocumentElement.ChildNodes[i];
        ALibstring := ANode.Xml;
        Rec := CreateRecord;
        if Target = 1 then
          Dataset.Add(Rec)
        else
          AddToTable(Rec);
        Inc(Result);
      end;
    finally
      if Target = 1 then
        Dataset.EndUpdate;
    end;
    FXMLDoc.DocumentElement.ChildNodes.Clear;
  end;

  function TILTable.ReadXMLFile(const FileName: string): integer;
  begin
    Result := DoReadXMLFile(FileName, 0);
  end;

  function TILTable.IsStringType(const DataType: TDataType): Boolean;
  begin
    Result := (DataType in [dtString, dtText, dtResource, dtLongText]);
  end;

  function TILTable.AddToDataset(MediaFiles: TStrings;
    const ReplaceDataset: Boolean = false): integer;
  begin
    Result := DoReadImages(MediaFiles, ReplaceDataset);
    if Result > 0 then
    begin
      WriteControls(Dataset.Items[0], duData);
    end;
    if TVControl <> nil then
      TVControl.BuildTree(Dataset);
  end;

  function TILTable.CreateDataRecord: TILDataRecord;
  begin
    Result := TILDataRecord.Create(Self, false);
    Result.InitFields(nil, false);
  end;

  function TILTable.CheckMandatories(Data: TILDataRecord;
    var Missing: string): Boolean;

  var
    i: integer;
  begin
    Result := True;
    Missing := '';
    for i := 0 to Data.Count - 1 do
    begin
      if (Data[i].FieldDef.Identifier or Data[i].FieldDef.Mandatory) then
      begin
        if not SameText(Data[i].FieldDef.FieldName, c_SysFieldNames[SYS_LINKID])
        and not SameText(Data[i].FieldDef.FieldName, c_SysFieldNames[SYS_LEVEL])
        and not SameText(Data[i].FieldDef.FieldName, c_SysFieldNames[SYS_RECORDID])
        and not SameText(Data[i].FieldDef.FieldName, c_SysFieldNames[SYS_SIBLINGINDEX])
        and not (Data[i].FieldDef.InitialValue <> '')
        and not (Data[i].FieldDef.LookupType = looParentField)
            { Has not yet got ist value?}
        then
        begin
          if (Data[i].Count = 0) or TMan.IsValueNull(Data[i][0]) then
          begin
            if Data.FIsDataControls then
            begin
                if Data[i].IsEmpty then
                begin
                 if not Data.OwnerDataset.FocusedRecord[Data[i].FFieldName].IsEmpty then
                   Continue;
                end;
            end;
            Missing := Data[i].FieldDef.FieldName;
            Result := false;
            Exit;
          end;
        end;
      end;
    end;
  end;

{  function TILTable.TableOf(ATableName: string): TILTable;

  var
    i: integer;
  begin
    Result := nil;
    for i := 0 to AllChildCount - 1 do
    begin
      if AnsiCompareText(ATableName, AllChildren[i].FTableName) = 0 then
      begin
        Result := AllChildren[i];
        Break;
      end;
    end;
  end;
}
  function TILTable.CompareIDS(OldData, NewData: TILDataRecord): Boolean;
  var
    i: integer;
    Indx: integer;
  begin
    Result := True;
    for i := 0 to OldData.Count - 1 do
    begin
      if OldData[i].FieldDef.Identifier or OldData[i].FieldDef.Unique then
      begin
        Indx := NewData.IndexOf(OldData[i].FieldName);
        if Indx = -1 then { Should never happen }
        begin
          Result := false;
          Exit;
        end;
        if not SameText(OldData[i].AsString, NewData[Indx].AsString) then
        begin
          Result := false;
          Exit;
        end;
      end;
    end;
  end;

  function TILTable.CompareIdents(Record1, Record2: TILDataRecord): TilField;
  var
    i: integer;
    Indx: integer;
  begin
    Result := nil;
    for i := 0 to Record1.Count - 1 do
    begin
      if Record1[i].FieldDef.Identifier then
      begin
        Indx := Record2.IndexOf(Record1[i].FieldName);
        if Indx = -1 then { Should never happen }
        begin
          Exit;
        end;
        if SameText(Record1[i].AsString, Record2[Indx].AsString) then
        begin
          Result := Record1[i];
          Exit;
        end;
      end;
    end;
  end;

  function TILTable.CompareUniques(Record1, Record2: TILDataRecord): TilField;
  var
    i: integer;
    Indx: integer;
  begin
    Result := nil;
    for i := 0 to Record1.Count - 1 do
    begin
      if Record1[i].FieldDef.Unique then
      begin
        Indx := Record2.IndexOf(Record1[i].FieldName);
        if Indx = -1 then { Should never happen }
        begin
          Exit;
        end;
        if SameText(Record1[i].AsString, Record2[Indx].AsString) then
        begin
          Result := Record1[i];
          Exit;
        end;
      end;
    end;
  end;

  function TILTable.CheckSearchExpression(Expression: TStrings): integer;

  var
    S: TSearcher;
    sl: TStringList;
  begin
    Result := -1;
    if not Connected then
      Exit;
    sl := TStringList.Create;
    sl.Assign(Expression);
    S := CreateSearcher(Self, sl);
    S.FTreeViewing := (Self is TILTreeViewTable);
    S.FChecking := True;
    try
      Result := S.Validate;
    finally
      S.Free;
      sl.Free;
    end;
  end;

  function TILTable.CheckSearchExpression: integer;

  var
    sl: TStringList;
  begin
    if not Connected then
    begin
      Result := -1;
      Exit;
    end;
    sl := TStringList.Create;
    Result := SearchCtrlsToExpression(sl, True);
    try
      if Result <> 0 then
        Exit;
      Result := CheckSearchExpression(sl);
    finally
      sl.Free;
    end;
  end;

  function TILTable.CheckSearchExpression(ARecord: TILRecord): integer;

  var
    S: TSearcher;
  begin
    Result := -1;
    if not Connected then
      Exit;
    if ARecord.Count = 0 then
    begin
      Result := 0;
      Exit;
    end;
    S := CreateSearcher(Self, ARecord);
    S.FTreeViewing := (Self is TILTreeViewTable);
    S.FChecking := True;
    try
      Result := S.Validate;
    finally
      S.Free;
    end;
  end;

  function TILTable.CheckUpdateExpression(Expression: TStrings): integer;

  var
    U: TUpdater;
    sl: TStringList;
  begin
    Result := -1;
    if not Connected then
      Exit;
    sl := TStringList.Create;
    sl.Assign(Expression);
    U := TUpdater.Create(Self, sl);
    U.FChecking := True;
    try
      Result := U.Validate;
    finally
      U.Free;
      sl.Free;
    end;
  end;

  function TILTable.CheckUpdateExpression(ARecord: TILRecord): integer;

  var
    U: TUpdater;
  begin
    Result := -1;
    if not Connected then
      Exit;
    U := TUpdater.Create(Self, ARecord);
    try
      if U.FError <> 0 then
        Result := U.FError
      else
        Result := U.Validate;
    finally
      U.Free;
    end;
  end;

  function TILTable.GetSearcher(ATable : TILTable) : TSearcher;
  begin
    if ATable is TILView then
    begin
     if TILView(ATable).SourceTable = nil then
       HandleILException(E_MissingViewTable, Tablename);
     Result := TILView(ATable).SourceTable.Searcher;
    end
    else
     Result := ATable.Searcher;
  end;

  function TILTable.CreateSearcher(ATable : TILTable; Expr: TStringList) : TSearcher;
  begin
     if ATable is TILView then
     begin
      if TILView(ATable).SourceTable = nil then
        HandleILException(E_MissingViewTable, Tablename);
      Result := TSearcher.Create(TILView(ATable).SourceTable, Expr);
     end
    else
      Result := TSearcher.Create(ATable, Expr);
  end;

  function TILTable.CreateSearcher(ATable : TILTable; ASearchRecord : TILrecord) : TSearcher;
  begin
     if ATable is TILView then
     Result := TSearcher.Create(TILView(ATable).SourceTable, ASearchRecord)
    else
      Result := TSearcher.Create(ATable, ASearchRecord);
  end;

  function TILTable.CheckUpdateExpression: integer;
  var
    U: TUpdater;
    ARecord: TILRecord;
  begin
    Result := -1;
    if not Connected then
      Exit;
    ARecord := TILRecord.Create(Self);
    if (DataHandler = nil) or (DataHandler.DataFields.Count = 0) then
      Exit;
    Result := ReadControls(ARecord, duData);
    if Result <> 0 then
    begin
      ARecord.Free;
      Exit;
    end;
    U := TUpdater.Create(Self, ARecord);
    try
      if U.FError <> 0 then
        Result := U.FError
      else
        Result := U.Validate;
    finally
      U.Free;
      ARecord.Free;
    end;
  end;

  function TILTable.Search(ASearchRecord: TILRecord): integer;

  var
    ExprStr: TStringList;
    SR: TSearcher;
  begin
    Result := 0;
    if not Connected then
      Exit;
    SR := CreateSearcher(Self, ASearchRecord);
    ExprStr := TStringList.Create;
    try
      ExprStr.Assign(SR.Expression);
      SR.Free;
      Result := Search(ExprStr);
    finally
      ExprStr.Free;
    end;
  end;

  function TILTable.Search(Expression: TStringList): integer;
  var
    ResultSet: TILDataset;
    Srch  : TSearcher;
    MH : TILMessageHandler;
    Ind : integer;
  begin
    Result := 0;
    if not Connected then
      Exit;
    Srch := GetSearcher(Self);
    Srch.Expression := Expression;
    FDatasetSearch := True;
    try
      Fields.ClearCalcFields;
      Srch.FTreeViewing := (Self is TILTreeViewTable);
      ResultSet := Srch.Search(nil, 0, True);
      Result := ResultSet.Count;
      if ResultSet.Count = 0 then
      begin
        ResultSet.Free;
        if HasViewer then
          Viewer.Invalidate;
        Ind := -1;
        MH := TMan.GetMessageHandler(Self, Ind);
        while MH <> nil do
        begin
         MH.AddCount(dboSearch, 0, False);
         MH.FSearchState := 0;
         MH := TMan.GetMessageHandler(Self, Ind);
        end;
      end
      else
      begin
         Dataset := ResultSet;
      end;
    finally
      FDatasetSearch := false;
    end;
  end;

  function TILTable.SearchDownTree(StartTable: TILTable; const StartID: integer;
    EndTable: TILTable): TTreeData;

  var
    N: TTableNode;
    Path: array [0 .. 20] of TILTable;
    Cnt: integer;
    sl: TStringList;
    PathIndx: integer;

  procedure DoSearch(PathInd, ARecId: integer);
  var
    i: integer;
    DS: TILDataset;
    S: TSearcher;
    DN: TTreeDataNode;
  begin
    sl.Clear;
    sl.Add('Select * where SYS_LinkID = ' + IntToStr(ARecId));
    S := CreateSearcher(Path[PathInd], sl);
    try
      DS := S.Search(nil, 0, False);
      DN.Dataset := DS;
      DN.Table := Path[PathInd] as TILDataTable;
      SetLength(Result, Length(Result) + 1);
      Result[High(Result)] := DN;
    finally
      S.Free;
    end;
    if PathInd > 0 then
      for i := 0 to DS.Count - 1 do
        DoSearch(PathInd - 1, DS[i].RecordIndex);
  end;

  begin
    Cnt := 0;
    N := TMan.NodeOf(EndTable);
    while N.Table <> StartTable do
    begin
      Inc(Cnt);
      Path[Cnt - 1] := N.Table;
      N := N.Parent;
    end;
    sl := TStringList.Create;
    try
      PathIndx := Cnt - 1;
      DoSearch(PathIndx, StartID);
    finally
      sl.Free;
    end;
  end;

  function TILTable.SearchUpTree(Expression: TStringList; ParentTable: TILTable;
    AChildRecord: TILDataRecord): TILDataRecord;

  { Search for an expresion in a child table to find the record in ParentTable
    that relates to the child recordIndex }
  var
    DS: TILDataset;
    i: integer;
    R: TILDataRecord;
    TBL: TILTable;
    S: TSearcher;
    FreeRecs: array of TILDataRecord;

  begin
    Result := nil;
    TBL := Self;
    R := Tbl.Records[AChildRecord['SYS_LinkID'].AsInteger];
    SetLength(FreeRecs, Length(FreeRecs) + 1);
    FreeRecs[High(FreeRecs)] := R;
    while TBL <> ParentTable do
    begin
      TBL := Tbl.MasterSource;
      if TBL <> ParentTable then
      begin
        R := Tbl.Records[R['SYS_LinkID'].AsInteger];
        SetLength(FreeRecs, Length(FreeRecs) + 1);
        FreeRecs[High(FreeRecs)] := R;
      end;
    end;
    S := CreateSearcher(ParentTable, Expression);
    S.FPrivate := True;
    DS := S.Search(nil, 0, True);
    try
      for i := 0 to DS.Count - 1 do
      begin
        if DS[i].RecordIndex = R.RecordIndex then
        begin
          Result := R;
          Break;
        end;
      end;
    finally
      S.Free;
      DS.Free;
      for i := 0 to High(FreeRecs) do
        if FreeRecs[i] <> Result then
          FreeRecs[i].Free;
    end;
  end;

  function TILTable.SearchUpLink(Expression: TStringList; ParentTable: TILTable;
    AChildRecord: TILDataRecord): TILDataRecord;

  { Search for an expresion in a child table to find the record in ParentTable
    that is linked to the path }
  var
    DS: TILDataset;
    i: integer;
  begin
    Result := nil;
    { Find the link field }
    for I := 0 to Fields.Count-1 do
    begin
      if (Fields.Items[i].LookupTable <> nil) and (Fields.Items[i].LookupType = looTableKey)
      and (Fields.Items[i].LookupTable = ParentTable) then
      begin
        DS := Fields.Items[i].LookupTable.DBView.SearchPrivate(Expression);
        try
          if DS.Count > 0 then
          begin
            Result := Fields.Items[i].LookupTable.CreateDataRecord;
            Result.Assign(DS[0]);
            Break;
          end;
        finally
          DS.Free;
        end;
      end;
    end;
  end;

  function TILTable.Exists(Expression: TStringList; var RecordIndex : integer): Boolean;
  var
    Results: TILDataset;
    SR: TSearcher;
  begin
    Result := false;
    if not Connected then
      Exit;
    if Expression.Count = 0 then
      Exit;
    SR := CreateSearcher(Self, Expression);
    SR.FPrivate := True;
    { Prevents any caulculated fields to be cleared }
    RecordIndex := -1;
    Results := SR.Search(nil, 1, True);
    try
      Result := (Results.Count > 0);
      if Result then
        RecordIndex := Results[0].RecordIndex;

    finally
      Results.Free;
      SR.Free;
    end;
  end;

  function TILTable.SearchDeleted: TILDataset;

  var
    i: integer;
    Rec: TILDataRecord;
  begin
    Result := TILDataset.Create(Self);
    Result.IsReadOnly := True;
    for i := 0 to RecordCount - 1 do
    begin
      Rec := RecordsAbs[i];
      if Rec.Deleted and not Rec.DeletedTrans and not Rec.DeletedMoved then
      begin
        Result.Add(Rec);
      end
      else
        Rec.Free;
    end;
  end;

  procedure TILTable.SetLogOp(const LogOp: TFieldLogOp);
  begin
    Fields.FieldLogOp := LogOp;
  end;

  procedure TILTable.SetLogOp(const FieldName: string;
    const LogOp: TValueLogOp);

  var
    Indx: integer;
  begin
    Indx := Fields.IndexOf(FieldName);
    if Indx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    Fields.Items[Indx].ValueLogOp := LogOp;
  end;

  procedure TILTable.SetLogOp(const FieldName: string;
    const LogOp: TFieldLogOp);

  var
    Indx: integer;
  begin
    Indx := Fields.IndexOf(FieldName);
    if Indx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    Fields.Items[Indx].FieldLogOp := LogOp;
  end;

  procedure TILTable.SetLogOp(SearchControl: TWinControl;
    const LogOp: TValueLogOp);

  var
    Indx: integer;
  begin
    Indx := Fields.IndexOfControl(SearchControl);
    if Indx <> -1 then
    begin
      SetLogOp(Fields.Items[Indx].FieldName, LogOp);
    end
    else
    begin
      Comment1 := SearchControl.Name;
      Comment2 := TableName;
      HandleILException(E_CtrlNotFound, '');
    end;
  end;

  procedure TILTable.SetLogOp(SearchControl: TWinControl;
    const LogOp: TFieldLogOp);

  var
    Indx: integer;
  begin
    Indx := Fields.IndexOfControl(SearchControl);
    if Indx <> -1 then
    begin
      SetLogOp(Fields.Items[Indx].FieldName, LogOp);
    end
    else
    begin
      Comment1 := SearchControl.Name;
      Comment2 := TableName;
      HandleILException(E_CtrlNotFound, '');
    end;
  end;

  function TILTable.TestUpdateDataset(Expression: TStringList): TILUpdateTest;

  var
    i: integer;
    TargetRec: TILDataRecord;
    Updated: Boolean;
    TR: TILTestResult;
  begin
    Result := nil;
    if not Connected then
      Exit;
    if Dataset = nil then
      Exit;
    Result := TILUpdateTest.Create;
    FCommanded := True;
    try
      for i := 0 to SelCount - 1 do
      begin
        TargetRec := Selection[i];
        TR := TestUpdate(TargetRec, Expression, Updated);
        if Updated then
          Result.Add(TR);
      end;
    finally
      FCommanded := false;
    end;
  end;

  function TILTable.TestUpdateDataset: TILUpdateTest;

  var
    i: integer;
    TargetRec: TILDataRecord;
    Updated: Boolean;
    TR: TILTestResult;
    SourceRec: TILDataRecord;
  begin
    Result := nil;
    if not Connected then
      Exit;
    if Dataset = nil then
      Exit;
    Result := TILUpdateTest.Create;
    if IsControlsEmpty(duData) then
      Exit;
    SourceRec := TILDataRecord.Create(Self, false);
    try
      ReadControls(SourceRec, duData);
      for i := 0 to SelCount - 1 do
      begin
        TargetRec := Selection[i];
        TR := TestUpdate(SourceRec, TargetRec, Updated);
        if Updated then
          Result.Add(TR);
      end;
    finally
      SourceRec.Free;
    end;
  end;

  function TILTable.UpdateDataset: integer;

  { Updates the global Dataset using the values the data controls. }
  var
    SourceRec: TILRecord;
    UPD: TUpdater;
  begin
    Result := 0;
    if not Connected then
      Exit;
    SourceRec := TILRecord.Create(Self);
    try
      ReadControls(SourceRec, duData);
      UpdateTarget := utrDataSet;
      UPD := TUpdater.Create(Self, SourceRec);
      try
        if UPD.FError <> 0 then
          HandleILException(UPD.FError, '');
        Result := UPD.Update;
      finally
        UPD.Free;
      end;
    finally
      SourceRec.Free;
    end;
    if HasViewer then
      Viewer.FReport.Redraw;
    if HasHandler then
      DataHandler.CreateSummary(nil);
     if TMan.AutoSave then
      DBWork.Save;
  end;

  function TILTable.UpdateDataset(AnUpdateRecord: TILRecord): integer;

  { Updates the records contained in Dataset, according to the values of ASourceRecord
    and the kind of UpdateAction }
  var
    UPD: TUpdater;
  begin
    Result := 0;
    if not Connected then
      Exit;
    UpdateTarget := utrDataSet;
    UPD := TUpdater.Create(Self, AnUpdateRecord);
    try
      if UPD.FError <> 0 then
        HandleILException(UPD.FError, '');
      Result := UPD.Update;
    finally
      UPD.Free;
    end;
    if HasViewer then
      Viewer.FReport.Redraw;
    if HasHandler then
      Datahandler.CreateSummary(nil);
    if TMan.AutoSave then
      DBWork.Save;
  end;

  function TILTable.ExecSQL(Expression: TStrings; const SQLTypes: TSQLTypes = []): integer;

  var
    UPD: TUpdater;
    sl: TStringList;
    Found: Boolean;
    S: string;
  begin
    Result := 0;
    RemoveComments(Expression);
    if not Connected then
      Exit;
    sl := TStringList.Create;
    try
    sl.Assign(Expression);
    TrimList(sl, false);
    Found := false;
    if SQLTypes = [] then
      Found := True
    else if ctSelect in SQLTypes then
    begin
      if PosRestrict('SELECT', sl[0], False) <> 0 then
        Found := True;
    end;
    if (ctUpdate in SQLTypes) and not Found then
    begin
      if PosRestrict('UPDATE', sl[0], False) <> 0 then
      begin
        if PosRestrict(' TABLE', sl[0], False) <> 0 then
          UpdateTarget := utrTable;
        Found := True;
      end;

    end;
    if (ctInsert in SQLTypes) and not Found then
    begin
      if PosRestrict('INSERT', sl[0], False) <> 0 then
      begin
        if PosRestrict(' TABLE', sl[0], False) <> 0 then
          UpdateTarget := utrTable;
        Found := True;
      end;

    end;
    if (ctDelete in SQLTypes) and not Found then
    begin
      if PosRestrict('DELETE', sl[0], False) <> 0 then
        Found := True;
    end;

    if not Found and (SQLTypes <> [ctSelect]) then
      HandleILException(E_SQLCommand, '');

    if SQLTypes = [] then
    begin
      if PosRestrict('UPDATE', sl[0], False) = 1 then
      begin
        if PosRestrict(' TABLE', sl[0], False) <> 0 then
          UpdateTarget := utrTable;
      end
      else if PosRestrict('INSERT', sl[0], False) = 1 then
      begin
        if PosRestrict(' TABLE', sl[0], False) <> 0 then
          UpdateTarget := utrTable;
      end
    end;

    FCommanded := True;
    if sl.Count = 0 then
     Exit;
    if (PosRestrict('SELECT', sl[0], False) = 1) or not Found then
    begin
        if not TMan.CheckUnsaved(Self) then
          Exit;
        S := sl.Text;
        S := ReplaceStr(S, #13#10, ' ');
        while Pos(' ', S) <> 0 do
          S := ReplaceStr(S, ' ', '');

        if (PosRestrict('SELECTCOUNT', S, False) <> 0) or
          (PosRestrict('SELECTMIN', S, False) <> 0) or
          (PosRestrict('SELECTMAX', S, False) <> 0) or
          (PosRestrict('SELECTAVG', S, False) <> 0) or
          (PosRestrict('SELECTSUM', S, False) <> 0) then
        begin
          ResetDataset;
        end;

        Result := Search(sl);
        Exit;
    end;
    UPD := TUpdater.Create(Self, sl);
    Upd.FRecordIndex := DBWork.FRecordIndex;
    try
        if UPD.FError <> 0 then
          HandleILException(UPD.FError, '');
        Result := UPD.Update;
    finally
        UPD.Free;
    end;
    if HasViewer then
       Viewer.FReport.Redraw;
     if HasHandler then
       DataHandler.CreateSummary(nil);
    finally
      FCommanded := false;
      sl.Free;
    end;
  end;

  function TILTable.UpdateRecord(ASourceRecord: TILRecord;
    ATargetRecord: TILDataRecord): integer;

  var
    Updater: TUpdater;
  begin
    Result := 0;
    if not Connected then
      Exit;
    if (ASourceRecord = nil) or (ATargetRecord = nil) then
      Exit;
    Updater := TUpdater.Create(Self, ASourceRecord);
    try
      if Updater.FError <> 0 then
        HandleILException(Updater.FError, '');
      Updater.Execute(ATargetRecord);
      if ATargetRecord.Changed then
      begin
        Inc(Result);
      end;
    finally
      Updater.Free;
    end;
  end;

  function TILTable.UpdateRecord(ATargetRecord: TILDataRecord;
    Expression: TStringList): integer;

  var
    Updater: TUpdater;
  begin
    Result := 0;
    if not Connected then
      Exit;
    if (ATargetRecord = nil) then
      Exit;
    Updater := TUpdater.Create(Self, Expression);
    try
      Updater.Execute(ATargetRecord);
      if ATargetRecord.Changed then
      begin
        Inc(Result);
      end;
    finally
      Updater.Free;
    end;
  end;

  procedure TILTable.UpdateImage(const AnIndex: integer; const AFileName: string);

  var
    R: TILDataRecord;
    Offs, Count : int64;
  begin
    R := Records[AnIndex];
    try
      R.UpdateImage(AFileName);
      if R.Image <> nil then
      begin
        SaveImage(R.Image, Offs, Count);
        R.FThumbnailOffset := Offs;
        R.FThumbnailSize := Count;
      end
      else
      begin
        R.FThumbnailOffset := 0;
        R.FThumbnailSize := 0;
      end;
      Records[AnIndex] := R;
    finally
      R.Free;
    end;
  end;

  procedure TILTable.DeleteImage(const AnIndex: integer);
  var
    Rec: TILDataRecord;
    S: string;
    Indx: integer;
  begin
    S := FSF_Data[AnIndex];
    Rec := XMLToRecord(S, True);
    try
      Rec.FFileName := '';
      Rec.FThumbnailOffset := -1;
      Rec.FThumbnailSize := -1;
      Indx := Rec.IndexOf(c_SysFieldNames[SYS_FileName]);
      if Indx <> -1 then
        Rec[Indx].FValues.Clear
      else
       Records[AnIndex] := Rec;
    finally
      Rec.Free;
    end;
  end;

  function TILTable.DoTestUpdate(ASourceRecord: TILRecord;
    ATargetRecord: TILDataRecord; AnExpression: TStringList;
    var Updated: Boolean): TILTestResult;

  var
    j, k: integer;
    UpdateRec, LibRec: TILDataRecord;
    FoundIndx: integer;
  I: Integer;

  function ValueItmFound(ValueItm: string; DataRec: TILDataRecord;
    DIndex: integer): integer;

  var
    i: integer;
    Fnd : Boolean;
    Dt1, Dt2 : TDateTime;
    Y1, M1, D1 : Word;
    Y2, M2, D2 : Word;
  begin
    Result := -1;
    for i := 0 to DataRec[DIndex].FValues.Count - 1 do
    begin
      if DataRec[DIndex].FieldDef.DataType = dtNumber then
       Fnd := (StrToFloat(ValueItm, Translator.FFormatSettings) =
        StrToFloat(DataRec[DIndex].FValues[i], Translator.FFormatSettings))
      else if DataRec[DIndex].FieldDef.DataType = dtDate then
      begin
       if DateSearchFormat = dsfDateTime then
        Fnd := (StrToDateTime(ValueItm, Translator.FFormatSettings) =
         StrToDateTime(DataRec[DIndex].FValues[i], Translator.FFormatSettings))
       else
       begin
        DT1 := StrToDateTime(ValueItm, Translator.FFormatSettings);
        DT2 := StrToDateTime(DataRec[DIndex].FValues[i], Translator.FFormatSettings);
        DecodeDate(DT1, Y1, M1, D1);
        DecodeDate(DT2, Y2, M2, D2);
        Fnd := (Y1 = Y2) and (M1 = M2) and (D1 = D2);
       end;
      end
      else
       Fnd := (ValueItm = DataRec[DIndex].FValues[i]);
      if Fnd then
      begin
        Result := i;
        Exit;
      end;
    end;
  end;

  begin
    Result := nil;
    if not Connected then
      Exit;
    Updated := false;
    if not FIndexing then
    for I := 0 to ATargetRecord.Count-1 do
    if ATargetRecord[i].FieldDef.DataType = dtLongText then
    begin
        if ATargetRecord[i].TextSourceChanged then
        begin
          Result := TILTestResult.Create;
          Updated := True;
          Exit;
        end;

    end;
    LibRec := TILDataRecord.Create(Self, false);
    UpdateRec := TILDataRecord.Create(Self, false);
    LibRec.AssignFields(ATargetRecord);
    UpdateRec.AssignFields(ATargetRecord);

    try
      if ASourceRecord = nil then
      begin
        if not FIndexing then
        begin
         if CheckUpdateExpression(AnExpression) <> 0 then
          Exit;
        end;
        UpdateRecord(UpdateRec, AnExpression);
      end
      else
      begin
        if not FIndexing then
        begin
         if CheckUpdateExpression(ASourceRecord) <> 0 then
            Exit;
        end;
        UpdateRecord(ASourceRecord, UpdateRec);
      end;

      { Check for adds }
      Result := TILTestResult.Create;
      for j := 0 to UpdateRec.Count - 1 do
      begin
        for k := 0 to UpdateRec[j].FValues.Count - 1 do
        begin
          if LibRec.Count = 0 then
            FoundIndx := -1
          else if (UpdateRec[j].FieldDef.DataType = dtText) and
            (k > LibRec[j].FValues.Count - 1) then
            FoundIndx := -1
          else
          begin
            FoundIndx := ValueItmFound(UpdateRec[j].FValues[k], LibRec, j);
          end;
          { Value in Updaterec not found in librec for this field }
          if FoundIndx = -1 then
          begin
            Result.Adds.Add(UpdateRec[j].FieldName + ': ' + UpdateRec[j].FValues[k]);
            Result.FRecordIndex := ATargetRecord.FRecordIndex;
            Updated := True;
          end;
        end;
      end;

      { Check for deltes }
      for j := 0 to LibRec.Count - 1 do
      begin
        for k := 0 to LibRec[j].FValues.Count - 1 do
        begin
          if (LibRec[j].FieldDef.DataType = dtText) and
            (k > UpdateRec[j].FValues.Count - 1) then
            FoundIndx := -1
          else
            FoundIndx := ValueItmFound(LibRec[j].FValues[k], UpdateRec, j);
          { Value in LibRec not found in UpdateRec for this field }
          if FoundIndx = -1 then
          begin
            Result.Deletes.Add(LibRec[j].FieldName + ': ' + LibRec.Items[j]
              .FValues[k]);
            Result.FRecordIndex := ATargetRecord.FRecordIndex;
            Updated := True;
          end;
        end;
      end;
    finally
      UpdateRec.Free;
      LibRec.Free;
    end;
  end;

  function TILTable.TestUpdate(ATargetRecord: TILDataRecord;
    Expression: TStrings; var Updated: Boolean): TILTestResult;

  { Performs a test update of Targetrecord, checking if the target will be
    changed }
  var
    sl: TStringList;
  begin
    Result := nil;
    if not Connected then
      Exit;
    sl := TStringList.Create;
    sl.Assign(Expression);
    try
      Result := DoTestUpdate(nil, ATargetRecord, sl, Updated);
    finally
      sl.Free;
    end;
  end;

  function TILTable.TestUpdate(ASourceRecord: TILRecord;
    ATargetRecord: TILDataRecord; var Updated: Boolean): TILTestResult;
  begin
    Result := nil;
    if not Connected then
      Exit;
    Result := DoTestUpdate(ASourceRecord, ATargetRecord, nil, Updated);
  end;


  procedure TILTable.ControlsKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);

  var
    Indx: integer;
    linenumber: integer;
    Msg: TMsg;
    LookupVisibl: Boolean;
    S : string;
    IsViewer : Boolean;
  begin
    if TMan = nil then
      Exit;
    if not TMan.Connected then
      Exit;
    IsViewer := false;
    LookupVisibl := FLookupControl.Visible;
    Indx := FFields.IndexOfControl(Sender as TWinControl);
    if (Indx = -1) and (FViewer <> nil) then
     begin
       if not (Sender = FViewer.FCellEditor.FEditCtrl) then
          HandleILException(0,'Internal error. Control has no field');
       Indx := FViewer.FCellEditor.Cell.FFieldDef.Index;
       IsViewer := True;
     end
     else if Indx = -1 then
        HandleILException(0,'Internal error. Control has no field');

    if (Key = vk_Return) and FLookupControl.Visible and
      (FLookupControl.ItemIndex <> -1) then
    begin
      S := FLookupControl.Items[FLookupControl.ItemIndex];
      if (Sender is TEdit) or (Sender is TLabeledEdit) then
      begin
        with Sender as TCustomEdit do
        begin
          Text := S;
        end;
      end
      else if Sender is TMemo then
        with Sender as TMemo do
        begin
          linenumber := SendMessage(TWinControl(Sender).Handle, EM_LINEFROMCHAR,
            TMemo(Sender).Selstart, 0);
          TMemo(Sender).Lines[linenumber] := S;
          if linenumber < TMemo(Sender).Lines.Count - 1 then
            Key := 0;
        end
      else if (Sender is TILSearchGrid) or (Sender is TILUpdateGrid) then
        with Sender as TILCustomGrid do
        begin
          Cells[Col, Row] := S;
        end;
      FLookupControl.Visible := false;
    end;
    try
      if not FLookupControl.Visible then
      begin
        if IsViewer then
        begin
           FViewer.FCellEditor.ProcessKey(Key, Shift);
        end
        else if (Key in [vk_Escape, vk_Return])
        and ((Sender is TEdit) or (Sender is TLabeledEdit)) then
        begin
          { No ding! }
           PeekMessage(Msg, 0, WM_CHAR, WM_CHAR, PM_REMOVE);
        end;
        Exit;
      end;

      if (Key = vk_Escape) then
      begin
        FLookupControl.Visible := false;
      end
      else if (Key = vk_Down) then
      begin
        if FLookupControl.ItemIndex = FLookupControl.Count - 1 then
          Exit
        else
          FLookupControl.ItemIndex := FLookupControl.ItemIndex + 1;
      end
      else if (Key = vk_Up) then
      begin
        if FLookupControl.ItemIndex = 0 then
          Exit
        else
          FLookupControl.ItemIndex := FLookupControl.ItemIndex - 1;
      end;
      if (Key = vk_Up) or (Key = vk_Down) then
        Key := 0;

      if (Key in [vk_Escape, vk_Return])
      and ((Sender is TEdit) or (Sender is TLabeledEdit)) then
        { No ding! }
        PeekMessage(Msg, 0, WM_CHAR, WM_CHAR, PM_REMOVE);

    finally
      { Call app keydown if assigned }
      if (Indx = -1) and (FViewer <> nil) then { RecordEdit ctrl }
      begin
        if Sender = FViewer.FCellEditor.FEditCtrl then
        begin
          FViewer.FCellEditor.FLookupVisible := LookupVisibl;
          FViewer.FCellEditor.EditKeyDown(Sender, Key, Shift);
          FViewer.FCellEditor.FLookupVisible := false;
        end;
      end;

      if Indx <> -1 then
      begin
        if Sender = Fields.Items[Indx].SearchControl then
        begin
          if Assigned(FFields.Items[Indx].FSCKeyDown) then
            FFields.Items[Indx].FSCKeyDown(Sender, Key, Shift);
        end
        else if Sender = Fields.Items[Indx].DataControl then
        begin
          if Assigned(FFields.Items[Indx].FDCKeyDown) then
            FFields.Items[Indx].FDCKeyDown(Sender, Key, Shift);
        end;
      end;
    end;
  end;

  procedure TILTable.ControlsExit(Sender: TObject);

  var
    Indx: integer;
  begin
    if Assigned(FLookupControl) and FLookupControl.Focused then
      Exit; { Click on LookupControl }
    if Assigned(FLookupControl) then
    begin
      FLookupControl.Visible := false;
    end;
    Indx := FFields.IndexOfControl(Sender as TWinControl);
    if Indx <> -1 then
    begin
      if Sender = Fields.Items[Indx].SearchControl then
      begin
        if Assigned(FFields.Items[Indx].FSCExit) then
          FFields.Items[Indx].FSCExit(Sender);
      end
      else if Sender = Fields.Items[Indx].DataControl then
      begin
        if Assigned(FFields.Items[Indx].FDCExit) then
          FFields.Items[Indx].FDCExit(Sender);
      end;
    end;
  end;

  procedure TILTable.LookUpClick(Sender: TObject);
  var
    linenumber: integer;
    s : string;
  begin
    S := FLookupControl.Items[FLookupControl.ItemIndex];
    if FLookupOwner is TMemo then
    begin
      linenumber := SendMessage(FLookupOwner.Handle, EM_LINEFROMCHAR,
        TMemo(FLookupOwner).Selstart, 0);
      TMemo(FLookupOwner).Lines[linenumber] := S;
    end
    else if FLookupOwner is TCombobox then
    begin
      TCombobox(FLookupOwner).Text := S;
      if HasViewer then
      begin
        Viewer.Report.SelectedCell.SetValue(TCombobox(FLookupOwner).Text);
        Viewer.FCellEditor.Cancel;
      end;
    end
    else if (FLookupOwner is TILSearchGrid) or (FLookupOwner is TILUpdateGrid)
    then
      with FLookupOwner as TILCustomGrid do
      begin
        Cells[Col, Row] := S
      end
    else
    begin
        TEdit(FLookupOwner).Text := S;
    end;
    FLookupControl.Visible := false;
    FLookupControl.Update;
    if HasViewer and (Viewer.Report.SelectedCell <> nil)
    and Viewer.Report.SelectedCell.Editing then
    begin
      Viewer.FCellEditor.UpdateValue;
      Viewer.FCellEditor.Cancel;
    end;
  end;

  function TILTable.IsFieldLookupSource(const ALookupType : TLookupType; AField : TILFieldDef) : Boolean;
  var
    i, j : integer;
  begin
     Result := false;
     for I := 0 to TMan.TableCount-1 do
     begin
       if TMan.Tables[i] = Self then
         Continue;
       for j := 0 to TMan.Tables[i].Fields.Count-1 do
       if SameText(AField.FieldName, TMan.Tables[i].Fields.Items[j].LookupField)
       and (ALookupType = TMan.Tables[i].Fields.Items[j].LookupType) then
       begin
         Result := True;
         Break;
       end;
       if Result then
         Break;
     end;
  end;

  function TILTable.GetLookupClients(const ALookupType : TLookupType; AField : TILFieldDef) : TList;
  var
    i, j : integer;
  begin
    Result := TList.Create;
    for I := 0 to TMan.TableCount-1 do
     begin
       if TMan.Tables[i] = Self then
         Continue;
       for j := 0 to TMan.Tables[i].Fields.Count-1 do
       if SameText(AField.FieldName, TMan.Tables[i].Fields.Items[j].LookupField)
       and (ALookupType = TMan.Tables[i].Fields.Items[j].LookupType) then
       begin
         if (TMan.Tables[i].Dataset.Count > 0) then
          Result.Add(TMan.Tables[i]);
       end;
     end;
  end;

  function TILTable.IsControlActive(AControl: TControl): Boolean;
  var
    sl: TStringList;
    S: string;
    i: integer;
    N: integer;
  begin
    Result := false;
    if (AControl is TILSearchGrid) or (AControl is TILUpdateGrid) then
      with AControl as TILCustomGrid do
      begin
        if AControl is TILUpdateGrid then
          with AControl as TILUpdateGrid do
            N := StartRow
        else
          N := 0;
        if Enabled and Visible then
          for i := N to RowCount - 1 do
          begin
            if Trim(Values[i]) <> '' then
            begin
              Result := True;
              Break;
            end;
          end;
      end
    else if (AControl is TEdit) or (AControl is TLabeledEdit) then
      with AControl as TCustomEdit do
      begin
        S := TrimBlanks(Text);
        Result := (S <> '') and Enabled and Visible;
      end
    else if AControl is TMemo then
      with AControl as TMemo do
      begin
        sl := TStringList.Create;
        try
          sl.Assign(Lines);
          TrimList(sl);
          Result := (sl.Count > 0) and Enabled and Visible;
        finally
          sl.Free;
        end;
      end
    else if AControl is TILResourceList then
      with AControl as TILResourceList do
      begin
        Result := not Empty
      end
    else if AControl is TListBox then
      with AControl as TListBox do
      begin
        Result := (Count > 0) and Enabled and Visible;;
      end
    else if AControl is TCheckbox then
      with AControl as TCheckbox do
      begin
        Result := (State <> cbGrayed) and Enabled and Visible;;
      end
    else if AControl is TRadioGroup then
      with AControl as TRadioGroup do
      begin
        Result := (ItemIndex <> -1) and Enabled and Visible;;
      end
    else if AControl is TCombobox then
      with AControl as TCombobox do
      begin
        Result := (ItemIndex <> -1) and Enabled and Visible;;
      end
    else if AControl is TCheckListBox then
      with AControl as TCheckListBox do
      begin
        Result := false;
        if Enabled and Visible then
          for i := 0 to Count - 1 do
          begin
            if not(State[i] = cbGrayed) then
            begin
              Result := True;
              Break;
            end;

          end;
      end
    else if AControl is TDateTimePicker then
      with AControl as TDateTimePicker do
      begin
        if DateTime = 0 then
          Result := false
        else
          Result := Checked and Enabled and Visible;
      end;
  end;

  function TILTable.GetRoot: TILTable;
  begin
    Result := nil;
    if Assigned(TMan) and Assigned(TMan.Root) then
      Result := TMan.Root.Table;
  end;


procedure TILTable.AlterTable(const Option: TILOperation; NewSpace: integer;
    NewIndexes, NewLookups: TStringList);
  var
    TmpLib: TFileContainer;
    S: string;
    Data: TSubFile;
    ProgCounter : integer;
    IsConnected : Boolean;

  procedure UpdateCfg;
  begin
      if Option = iloAlterTableSpace then
      begin
        Tman.FCfgFile.DeleteKey(TableName, 'TableSpace');
        Tman.FCfgFile.WriteInteger(TableName, 'TableSpace', TableSpace);
      end;
  end;

  procedure AlterRecSpace;
   var
     FS : TFileSpec;
     FSX : TFileSpecEx;
     Sub : TSubFile;
     i : integer;
     S : string;
     Indx : integer;
     Loops, Cnt : integer;
   begin
        FS.FileName := Data.FFileSpec.FileName;
        FS.FileSpace := Data.FFileSpec.FileSpace;
        FS.RecordSpace := NewSpace;
        FS.FixedStringSize := False;
        FS.Sorted := False;
        FS.Duplicates := dupAccept;
        Sub := TmpLib.Add(FS);
        Cnt := Data.Count;
        Loops := Cnt * 2;
        ProgCounter := 0;
        TMan.FProgressTicker := GetTickCount;
        for I := 0 to Cnt-1 do
        begin
          S := Data.AbsItems[i];
          Sub.Add(S);
          Inc(ProgCounter);
          TMan.Progress(Self, iloAlterRecordSize, Loops, ProgCounter);
        end;
        Data.Clear;

        FSX := Data.FFileSpec;
        FSX.RecordSpace := NewSpace;

        TMan.FLibContainer.WriteHeader(Data.Index, FSX);
        Data.FFileSpec.RecordSpace := NewSpace;

        if NewSpace > 0 then
        begin
         if Data.FIndexTable <> nil then
         begin
          Data.FIndexTable.Clear;
          Data.FIndexTable := nil;
         end;
        end
        else
        begin
           Indx := TMan.FLibContainer.IndexOf(TableName + '_Data_IndexTable');
           if Indx = -1 then
           begin
             FS.Sorted := False;
             FS.FileName := TableName + '_Data_IndexTable';
             FS.FixedStringSize := True;
             FS.Duplicates := dupIgnore;
             FS.RecordSpace := 10;
             FS.FileSpace := FS.FileSpace div 2;
             Sub := TMan.FLibContainer.Add(FS);
             Data.FIndexTable := Sub;
           end
           else
             Data.FIndexTable := TMan.FLibContainer.Files[Indx];
        end;

        for I := 0 to Cnt-1 do
        begin
          S := Sub.AbsItems[i];
          Data.Add(S);
          Inc(ProgCounter);
          TMan.Progress(Self, iloAlterRecordSize, Loops, ProgCounter);
        end;
   end;

   procedure AlterPageSize;
   var
     FSX : TFileSpecEx;
   begin
     FSX := TMan.FLibContainer.ReadHeader(TMan.FLibContainer.IndexOf(FDataName));
     FSX.RecordSpace := NewSpace;
     TMan.FLibContainer.WriteHeader(TMan.FLibContainer.IndexOf(FDataName), FSX);
     Data.FFileSpec := FSX;
   end;

   procedure AlterRecordLayout;
   var
     FSX : TFileSpecEx;
     Lay : TRecordLayout;
   begin
     Lay := TRecordLayout(NewSpace);
     FSX := TMan.FLibContainer.ReadHeader(TMan.FLibContainer.IndexOf(FDataName));
     if (Lay = rlPaged) and (FSX.RecordSpace > c_PageBase) then
       Exit
     else if (Lay = rlVariableSize) and (FSX.RecordSpace = 0) then
       Exit
     else if (Lay = rlFixedSize) and (FSX.RecordSpace > 0) then
       Exit;
     if (Lay = rlPaged) then
     begin
       FRecordlayout := rlPaged;
       if FSX.RecordSpace = 0  then
         FSX.RecordSpace := ComputeRecordSize;
       FSX.RecordSpace := FSX.RecordSpace + c_PageBase;
     end
     else if (Lay = rlVariableSize) then
     begin
       FSX.RecordSpace := 0;
     end
     else if (Lay = rlFixedSize) then
     begin
       FRecordLayout := rlFixedSize;
       if FSX.RecordSpace > c_PageBase then
         FSX.RecordSpace := FSX.RecordSpace - c_PageBase
       else if FSX.RecordSpace = 0  then
         FSX.RecordSpace := ComputeRecordSize;
     end;
     TMan.FLibContainer.WriteHeader(TMan.FLibContainer.IndexOf(FDataName), FSX);
   end;

  begin
    if csDesigning in ComponentState then
      HandleILException(0, 'This function cannot be executed design time');
    if (Self is TILView) or (Self is TILMemTable) then
      HandleILException(E_TableOperation);
    if Tman.TransactionActive then
      HandleILException(E_TransActive);
    if (Self is TILView) or (Self is TILMemTable) then
      HandleILException(E_TableOperation);
    IsConnected := TMan.Connected;
    TMan.Connected := false;
    if IsFileInUse(TMan.FLibPath) then
    begin
      HandleILException(E_ConnectionsActive, '');
    end;

    TMan.ConnectSilent;

    if iloAlterTableSpace = Option then
    begin
      if NewSpace < 10000 then
        HandleILException(E_TableSpaceTooSmall);
      Data := TMan.FLibContainer.Files[TMan.FLibContainer.IndexOf(FDataName)];
      if Data.FFileSpec.FileSpace = NewSpace then
        Exit;
      Data.FFileSpec.FileSpace := NewSpace;
      TMan.FLibContainer.WriteHeader(Data.Index, Data.FFileSpec);
      FTableSpace := NewSpace;
      UpdateCfg;
      if not IsConnected then
        TMan.Connected := false;
      Exit;
    end
    else if iloAlterIndexSpace = Option then
    begin
      if (NewSpace < 10) or (NewSpace > 122) then { 100 + 12, space of counters }
        HandleILException(E_InvalidIndexSpace);
    end
    else if iloAlterRecordFreeSpace = Option then
    begin
      if (NewSpace < 0) or (NewSpace > 255) then
      begin
        if not IsConnected then
          TMan.Connected := false;
        HandleILException(0,'Invalid free space. Must be between 0 and 255');
      end;
    end
    else if Option = iloBuildIndexes then
    begin
      RebuildIndexes;
      if not IsConnected then
        TMan.Connected := false;
      Exit;
    end;

    if iloAlterRecordSize = Option then
    { NewSpace is the numer of bytes occupied by data }
    begin
      if RecordLayout = rlPaged then
      begin
         NewSpace := NewSpace + c_PageBase;
      end;

      Data := TMan.FLibContainer.FFiles[TMan.FLibContainer.IndexOf(FDataName)];

      if (NewSpace < Data.RecordSpace) and (RecordLayout = rlFixedSize) then
      begin
         if not IsConnected then
          TMan.Connected := false;
         HandleILException(0,'Record size cannot be less than the current record size (' +
          IntToStr(Data.RecordSpace) + ')');
      end;

      if Data.RecordSpace = NewSpace then
      begin
        if not IsConnected then
          TMan.Connected := false;
        HandleILException(0,'New size and current size are alike!');
      end;

      if RecordLayout = rlPaged then
      begin
         AlterPageSize;
         if not IsConnected then
          TMan.Connected := false;
         Exit;
      end;

      try
       TmpLib := TFileContainer.Create(Tman._FTmpLibPath, 0, false);
       TmpLib.Unicode := TMan.Unicode;
       TMan.MakeBackups(1);
       try
        AlterRecSpace;
       except
         TMan.RestoreFromBackups(1);
         raise;
       end;
      finally
        S := TmpLib.FFilename;
        TmpLib.Free;
        DeleteFile(Pchar(S));
        if not IsConnected then
          TMan.Connected := false;
      end;
    end;

    if iloAlterRecordLayout = Option then
    begin
     try
      AlterRecordLayout;
     finally
       if not IsConnected then
         TMan.Connected := false;
     end;
     Exit;
    end;

    if iloAlterIndexSpace = Option then
    begin
      TMan.MakeBackups(1);
      Data := TMan.FLibContainer.FFiles[TMan.FLibContainer.IndexOf(FKeyListName)];
      if Data.FFileSpec.RecordSpace = NewSpace then
      begin
        if not IsConnected then
         TMan.Connected := false;
        Exit;
      end;
      try
        Data.FFileSpec.RecordSpace := NewSpace;
        TMan.FLibContainer.WriteHeader(TMan.FLibContainer.IndexOf(FKeyListName),
          Data.FFileSpec);
        RebuildIndexes;
        if not IsConnected then
          TMan.Connected := false;
        Exit;
      except
        on E: Exception do
        begin
          TMan.RestoreFromBackups(1);
          if not IsConnected then
           TMan.Connected := false;
          HandleILException(0,'An unexpected error occured (' + E.Message + '). Work aborted');
        end;
      end;
      if not IsConnected then
         TMan.Connected := false;
    end;

    if iloAlterRecordFreeSpace = Option then
    begin
      Data := TMan.FLibContainer.FFiles[TMan.FLibContainer.IndexOf(FDataName)];
      if Data.FFileSpec.RecordSpace = NewSpace then
      begin
       if not IsConnected then
          TMan.Connected := false;
        Exit;
      end;
      Data.FFileSpec.RecordSpace := -NewSpace;
      TMan.FLibContainer.WriteHeader(TMan.FLibContainer.IndexOf(FDataName),
        Data.FFileSpec);
      if not IsConnected then
        TMan.Connected := false;
      Exit;
    end;
    if not IsConnected then
      TMan.Connected := false;
  end;

  { TILView ---------------------------------------------------------------------}

  procedure TILView.SetSourceTable(Value : TILTable);
  var
   i : integer;
   S : string;
   N : integer;
  begin
    if (FSourceTable = Value) or (Value is TILMemTable) or (Value is TILView) then
      Exit;
    if Value = Self then
      Exit;
    FSourceTable := Value;
    if not (csLoading in ComponentState)
    and (FSourceTable <> nil) then
    begin
     S := TMan.SyncTable(Self);
    end;
    if not Assigned(FColumns) then
      FColumns := TILColumns.Create(Self, Fields);
    FColumns.AssignSource;
    if Assigned(Datahandler) and (Fields.Count > 0) then
    begin
      DataHandler.DataFields.Clear;
      if FSourceTable = nil then
        DataHandler.Table := nil
      else
       DataHandler.FillDataFields;
      FreeNotification(Datahandler);
    end;
    Dataset.IsReadOnly := True;
    if FSourceTable = nil then
    begin
     FTableName := '';
     FFields.Clear;
     FColumns.Clear;
     if HasViewer then
       Viewer.Table := nil;
    end
    else
    begin
     N := 0;
     S := 'VIEW_' + FSourceTable.TableName;
     for I := 0 to TMan.TableCount-1 do
       begin
         if (TMan.Tables[i] <> Self) and
         (TMan.Tables[i] is TILView) and (TILView(TMan.Tables[i]).SourceTable = SourceTable) then
         begin
           Inc(N);
           if CompareText(TMan.Tables[i].TableName, S) > 0 then
             S := TMan.Tables[i].TableName
         end;

       end;
     if N > 0 then
     try
       TableName := 'VIEW_' + FSourceTable.TableName + IntToStr(N)
     except
       TableName := S + IntToStr(N);
     end
     else
       TableName := 'VIEW_' + FSourceTable.TableName;

    end;
    if HasViewer then
    begin
      FreeNotification(Viewer);
      Viewer.Invalidate;
    end;
  end;

  constructor TILView.Create(AComponent : TComponent);
  begin
    inherited;
    IsReadOnly := True;
  end;

  procedure TILView.AssignSource;
  var
    i : integer;
    DS : TILDataset;
    R : TILDataRecord;
    MH : TILMessageHandler;
  begin
    if SourceTable.Dataset.Count = 0 then
    begin
      i := -1;
      MH := TMan.GetMessageHandler(SourceTable, i);
      while MH <> nil do
      begin
       MH.Reset;
       MH := TMan.GetMessageHandler(SourceTable, i);
      end;
      Exit;
    end;
    DS := TILDataset.Create(Self);
    for I := 0 to SourceTable.Dataset.Count-1 do
    begin
      R := CreateDataRecord;
      R.Assign(SourceTable.Dataset[i]);
      DS.Add(R);
    end;
    Dataset := DS;
  end;

  { TILMemTable --------------------------------------------------------------}

  constructor TILMemTable.Create(AOwner : TComponent);
  begin
    inherited;
    FTableRecords := TILDataset.Create(Self);
    FTableRecords.FOwnsRecords := True;
    FBaseTables := TStringList.Create;
    IsReadOnly := True;
  end;

  destructor TILMemTable.Destroy;
  begin
    inherited;
    FTableRecords.Free;
    FBaseTables.Free;
  end;

  procedure TILMemTable.SetBaseTables(Value: TStrings);
  var
    i : integer;
    T : TILTable;
  begin
    for I := 0 to Value.Count-1 do
    begin
      T := TMan.TableOf(Value[i]);
      if T = nil then
        HandleILException(E_InvalidTableName, Value[i])
      else if (T = Self) then
       HandleILException(E_InvalidTableName, Value[i])
    end;
    FBaseTables.Assign(Value);
  end;

  procedure TILMemTable.RecreateRecords;
  begin
    TMan.InvalidateMemTables(Self);
  end;

  function TILMemTable.RecordOf(const ARecordIndex: Integer) : TILDataRecord;
  var
    i : integer;
  begin
    Result := nil;
    for I := 0 to FTableRecords.Count-1 do
    begin
      if FTableRecords[i].RecordIndex = ARecordIndex then
      begin
        Result := FTableRecords[i];
        Break;
      end;
    end;
  end;

  { TILDataTable ----------------------------------------------------------------}


  procedure TILDataTable.AddUserCalcField(FieldName : string; const DataType : TDataType;
    ColWidth : integer; const Formula : string; const CreateSearchControl : Boolean;
     const Save : Boolean);
  var
   F : TILFieldDef;
   s : string;
   N : integer;
   i : integer;
   P : TILControlPanel;
   T : integer;
   TP : TILFieldDef;
   C : TEdit;
   L : TLabel;

   procedure DoCreate(AParent : TWinControl);
   begin
      C := TEdit.Create(Owner);
      C.Parent := AParent;
      C.Width := ColWidth;
      C.Name := 's_' + FieldName;
      C.Text := '';
      F.SearchControl := C;
      L := TLabel.Create(Owner);
      L.Parent := AParent;
      L.Caption := F.Caption;
      L.Name := 'lbl_' + FieldName;
      F.SearchLabel := L;
   end;

   procedure DoSave;
   var
     i : integer;
   begin
      for I := 1 to 20 do
      begin
        S := 'Calc_' + TableName + '_' + IntToStr(i);
        if not Tman.FCfgFile.SectionExists(S) then
        begin
          N := i;
          Break;
        end;
      end;

      Tman.FCfgFile.WriteString('Calc_' + TableName + '_' + IntToStr(i), 'FieldName', FieldName);
      Tman.FCfgFile.WriteInteger('Calc_' + TableName + '_' + IntToStr(i), 'ColumnWidth', ColWidth);
      Tman.FCfgFile.WriteInteger('Calc_' + TableName + '_' + IntToStr(i), 'DataType', Ord(DataType));
      Tman.FCfgFile.WriteString('Calc_' + TableName + '_' + IntToStr(i), 'Formula', Formula);
      if CreateSearchControl then
        Tman.FCfgFile.WriteBool('Calc_' + TableName + '_' + IntToStr(i), 'SearchControl', True);
    end;

  begin
     i :=  Fields.IndexOf(FieldName) ;
     if i <> -1 then
      HandleILException(E_FieldNameExists);
     FieldName := TrimBlanks(FieldName);

    if TMan.ActiveTable <> Self then
    begin
      Comment1 := Alias;
      HandleILException(E_CalcActiveTable);
    end;
    TMan.ClearDatasets;
    if not (DataType in [dtString, dtNumber, dtDate]) then
       HandleILException(E_CalcDatatype);
    if TrimBlanks(Formula) = '' then
      HandleILException(E_MissingFormula);
    if Pos(' ', FieldName) <> 0 then
      HandleILException(E_InvalidFieldName, FieldName);
    if TMan.UseCommandModes then
    begin
      while TMan.FWorkHistory.Count > 0 do
      { Clear history so the new field can be included }
      begin
        TWorkHistoryElement(TMan.FWorkHistory[0]).Free;
        TMan.FWorkHistory.Delete(0);
      end;
    end;

    if ColWidth < 10 then
      ColWidth := 10;

    F := Fields.Add;
    F.FFieldType := ftCalculated;
    F.FFieldName := FieldName;
    F.FDataType := DataType;
    F.FUserCalc := True;
    F.FColumnWidth := ColWidth;
    F.FFormula := Formula;
    Columns.Add(F.FieldName);
    Columns.Apply;

    if CreateSearchControl then
    begin
      P := TMan.FindPanel(Self as TILDataTable, cmSearch);
      if P <> nil then
      begin
        TMan.FPanelBuild := True;
        try
        P.FSourceBuild := True;
        DoCreate(P);
        P.FSourceBuild := false;
        if TMan.CommandMode = cmSearch then
        begin
         if P.ShowLogOpButtons then
          P.RemoveLogopButtons;
         P.CreateItems;
         if P.ShowLogOpButtons then
           P.CreateLogopButtons;
         TMan.CreateWorkFields(Self as TILDataTable, TMan.CommandMode);
         P.ShowCtrls(TMan.FWorkFields);
         TMan.UpdateWorkList;
        end;
        finally
          TMan.FPanelBuild := False;
        end;
      end
      else
      begin
        T := - Maxint;
        TP := nil;
        for I := 0 to Fields.Count-1 do
        begin
            if Fields.Items[i].SearchControl <> nil then
            begin
               T := Max(T, Fields.Items[i].SearchControl.Top);
               TP := Fields.Items[i];
            end;
        end;
        if TP <> nil then
        begin
           DoCreate(TP.SearchControl.Parent);
           if TP.SearchLabel <> nil then
             L.Left := TP.SearchLabel.Left
           else
             L.Left := TP.SearchControl.Left - TForm(Owner).Canvas.TextWidth(L.Caption) - 3;
           C.Left := TP.SearchControl.Left;
           L.Top := TP.SearchControl.Top + TP.SearchControl.Height + 3;
           C.Top := L.Top;
           if TMan.UseCommandModes then
           begin
              TMan.CreateWorkFields(Self as TILDataTable, TMan.CommandMode);
              TMan.UpdateWorkList;
           end;
        end;
      end;
    end;
    if Save then
      DoSave;
  end;

  procedure TILDataTable.DeleteUserCalcField(const FieldName : string; const AMessage : string);
  var
   I : integer;
   S : string;
  begin
   { The user must restart app to complete deltion. AMessage can tell  }
   i :=  Fields.IndexOf(FieldName) ;
    if i = -1 then
      HandleILException(E_InvalidFieldName, FieldName);
   if not Fields.Items[i].FUserCalc then
   begin
     HandleILException(E_NotCalcField, FieldName);
   end;
   for I := 1 to 20 do
   begin
     S := 'Calc_' + TableName + '_' + IntToStr(i);
     if Tman.FCfgFile.SectionExists(S) then
     begin
       if Tman.FCfgFile.ValueExists(S, FieldName) then
       begin
         Tman.FCfgFile.EraseSection(S);
         Break;
       end;
     end;
   end;
   if TrimBlanks(AMessage) <> '' then
     ShowMessage(AMessage);
  end;

  procedure TILDataTable.GetUserCalcProps(const FieldName : string;  var DataType : TDataType;
     var ColWidth : integer; var Formula : string; var Save : Boolean);
  var
   I : integer;
   S : string;
  begin
     i :=  Fields.IndexOf(FieldName) ;
     Comment1 := FieldName;
     if i = -1 then
       HandleILException(E_InvalidFieldName);
     if not Fields.Items[i].FUserCalc then
       HandleILException(E_NotUserCalcField);
     Comment1 := '';
     Save := False;
     for I := 1 to 20 do
     begin
       S := 'Calc_' + TableName + '_' + IntToStr(i);
       if Tman.FCfgFile.SectionExists(S) then
       begin
         if Tman.FCfgFile.ValueExists(S, FieldName) then
         begin
           Save := True;
           Break;
         end;
       end;
     end;
     Formula := Fields.Items[i].FFormula;
     ColWidth := Fields.Items[i].ColumnWidth;
     DataType := Fields.Items[i].DataType;
  end;

  procedure TILDataTable.EditUserCalcField(const FieldName : string; const DataType : TDataType;
      const ColWidth : integer; const Formula : string; const Save : Boolean);
  var
   I : integer;
   S : string;
   N : integer;
   SC : Boolean;
  begin
     i :=  Fields.IndexOf(FieldName) ;
     Comment1 := FieldName;
     if i = -1 then
       HandleILException(E_InvalidFieldName);
     if not Fields.Items[i].FUserCalc then
      HandleILException(E_NotUserCalcField);
     if not (DataType in [dtString, dtNumber, dtDate]) then
      HandleILException(E_CalcDataType);
     if TrimBlanks(Formula) = '' then
       HandleILException(E_MissingFormula);
     Comment1 := '';

     SC := False;
     N := -1;
     for I := 1 to 20 do
     begin
       S := 'Calc_' + TableName + '_' + IntToStr(i);
       if Tman.FCfgFile.SectionExists(S) then
       begin
         SC := Tman.FCfgFile.ReadBool(S, 'SearchControl', False);
         Tman.FCfgFile.EraseSection(S);
         N := I;
         Break;
       end;
     end;
     Fields.Items[N].FFormula := Formula;
     Fields.Items[N].FColumnWidth := ColWidth;

     if Save then
     begin
        if N = -1 then
        for I := 1 to 20 do
        begin
          S := 'Calc_' + TableName + '_' + IntToStr(i);
          if not Tman.FCfgFile.SectionExists(S) then
          begin
            N := i;
            Break;
          end;
        end;

        Tman.FCfgFile.WriteString('Calc_' + TableName + '_' + IntToStr(N), 'FieldName', FieldName);
        Tman.FCfgFile.WriteInteger('Calc_' + TableName + '_' + IntToStr(N), 'ColumnWidth', ColWidth);
        Tman.FCfgFile.WriteInteger('Calc_' + TableName + '_' + IntToStr(N), 'DataType', Ord(DataType));
        Tman.FCfgFile.WriteString('Calc_' + TableName + '_' + IntToStr(N), 'Formula', Formula);
        if SC then
          Tman.FCfgFile.WriteBool('Calc_' + TableName + '_' + IntToStr(N), 'SearchControl', True);
     end;
  end;

  procedure TILDatatable.GetChildRecords(AChildTable : TILDataTable);
  var
    i : integer;
  begin
    if AChildTable = nil then
    for I := 0 to AllChildCount-1 do
     AllChildren[i].GetDetails
    else
    begin
     if AChildTable.MasterSource = Self then
       AChildTable.GetDetails
     else if (AChildTable.MasterLinkCount > 0) then
     begin
       for i := 0 to AChildTable.MasterLinkCount-1 do
       begin
         if AChildTable.MasterLinks[i] = Self then
         begin
           FindLinkedRecords;
           Break;
         end;
       end;
     end;

    end;
  end;

  procedure TILDatatable.UpdateTableKey(const AKeyField : string; const ANewValue, AnOldValue : string);
  var
    i, j, k, l : integer;
    TBL : TILDataTable;
    Fld : TILFieldDef;
    sl : TstringList;
    s : string;
    AN, AO : string;

  {When the parent key of a TableKey link is changed }
  begin
     { Locate tables that uses the key }
    sl := TStringList.Create;
    try
     for I := 0 to TMan.TableNodeCount-1 do
     begin
        TBl := TMan.TreeTables[i];
        if TBL = Self then
          Continue;
        for J := 0 to Tbl.Fields.Count-1 do
        begin
          Fld := Tbl.Fields.Items[j];
          if (Fld.LookupType = looTableKey) and (Fld.LookupTable = Self)
          and SameText(AKeyField, Fld.LookupField)  then
          begin
            sl.Clear;
            AN := ANewValue;
            AO := AnOldValue;
            if Fld.DataType = dtString then
            begin
               AN := '''' + AN + '''';
               AO := '''' + AO + '''';
            end;
            s := 'update table set ' + Fld.FieldName + ' = ' + AN +
            ' where ' + Fld.FieldName + ' = ' + AO;
            sl.Add(s);
            Tbl.DBWork.ExecSQL(sl);

            for k := 0 to Tbl.Dataset.Count-1 do
            begin
              for l := 0 to TBL.DataSet[k][Fld.FieldName].FValues.Count-1 do
              if SameText(TBL.DataSet[k][Fld.FieldName].FValues[l], AnOldValue) then
              begin
                TBL.DataSet[k][Fld.FieldName].FValues[l] := ANewValue;
                TBL.DataSet[k][Fld.FieldName].FOldValues[l] := ANewValue;
              end;
            end;
            if Tbl.HasViewer then
              Tbl.Viewer.Invalidate;
            Break;
          end;
        end;
     end;
    finally
      sl.Free;
    end;
  end;

  procedure TILDataTable.FetchParentFields(ARecord : TILDatarecord);
  var
    i : integer;
    sl : TStrings;
  begin
    for I := 0 to ARecord.Count-1 do
    if ARecord[i].FieldDef.LookupType = looParentField then
    begin
        sl := GetLookupParentValue(ARecord[i].FieldDef.LookupTable, ARecord[i].FieldDef.LookupField, ARecord);
        if sl <> nil then
        begin
         ARecord[i].FValues.Assign(sl);
         sl.Free;
        end;
    end;
  end;

  procedure TILDataTable.UpdateParentField(ARecord: TILDataRecord);
  var
    i : integer;
    L : TList;
    F : TILField;

    procedure DoUpdate(Val : string);
    var
      i, j, k : integer;
      R : TILDataRecord;
      Fld : TILField;
    begin
     for I := 0 to L.Count-1 do
     begin
      if TILTable(L[i]).HasViewer then
      begin
        for J := 0 to TILTable(L[i]).Dataset.Count-1 do
        begin
          R := TILTable(L[i]).Dataset[J];
          for k := 0 to R.Count -1 do
          begin
            Fld := R[k];
            if (Fld.FieldDef.LookupType = looParentField)
            and SameText(Fld.FieldDef.LookupField, F.FieldDef.FieldName) then
              Fld.FValues.Text := Val;
          end;
        end;
        TILTable(L[i]).Viewer.Invalidate;
      end;
     end;
   end;

  begin
     for i := 0 to ARecord.Count -1 do
     begin
       F := Arecord[i];
       if (F.FValues.Count <> F.FOldValues.Count)
       or ((F.FValues.Count > 0) and (F.FValues[0] <> F.FOldValues[0])) then
         begin
             if IsFieldLookupSource(looParentField, F.FieldDef) then
             begin
               L := GetLookupClients(looParentField, F.FieldDef);
               try
                if F.FValues.Count = 0 then
                 DoUpdate('')
                else
                 DoUpdate(F.FValues.Text);
               finally
                L.Free;
               end;
             end;
         end;
     end;
  end;

  function TILDatatable.FindLinkedRecords: Boolean;
  { Finds all the linked records with AparentTable, AParentRecord as the base }
  var
      i, j : integer;
      Tbl : TILDatatable;
      N : integer;
    begin
       Result := false;
       if FocusedRecord = nil then
         Exit;
       for I := 0 to TMan.TableCount-1 do
       { Find the tables linked to parent table }
       if (Tman.Tables[i] <> Self) and (Tman.Tables[i] is TILDataTable) then
       begin
         Tbl := TMan.Tables[i] as TILDataTable;
         for j := 0 to Tbl.Fields.Count-1 do

         if (Tbl.Fields.Items[j].LookupType = looTableKey)
          and (Tbl.Fields.Items[j].LookupTable = Self)
          then
         begin
           Result := True;
           N := Tbl.GetLinkedRecords(FocusedRecord, Tbl.Fields.Items[j].LookupField);
           if N = 0 then
             TBL.Dataset.Clear();
         end;
       end;
    end;


  function TILDatatable.GetLinkedRecords(AParentRecord : TILDataRecord; const ALinkField : string) : integer;
  var
    i : integer;
    F : TILField;
    Indx : integer;
    sl, sl2 : TStringList;
    S : string;
    LooField : string;
    ColList : string;
    FSQL : TILSQL;
  begin
     Result := 0;
     if AParentRecord = nil then
       Exit;
     if FLocatingLinkedParent then
     begin
       FLocatingLinkedParent := false;
       Result := Dataset.Count;
       Exit;
     end;
     F := nil;
     for I := 0 to Fields.Count-1 do
     begin
       if Fields.Items[i].LookupType = looTableKey then
       begin
         if Fields.Items[i].LookupTable = AParentRecord.Table then
         begin
            if (ALinkField = '') or SameText(ALinkField, Fields.Items[i].LookupField) then
            begin
             Indx := AParentRecord.IndexOf(Fields.Items[i].LookupField);
             F := AParentRecord[Indx];
             LooField := Fields.Items[i].FieldName;
             Break;
            end;
         end;
       end;
     end;
     if F = nil then
       Exit;
     sl := TStringList.Create;
     try
       ColList := '';
       for I := 0 to Columns.Count-1 do
       begin
         ColList := ColList + Columns[i].FieldName;
         if I < Columns.Count-1 then
          ColList := ColList + ', ';
       end;

       if Collist = '' then
        CollIst := '*';
       if F.FieldDef.DataType = dtNumber then
         s := 'select ' + ColList + ' where ' + LooField + ' = ' + F.AsString
       else
         s := 'select ' + ColList + ' where ' + LooField + ' = ' + '''' + F.AsString  + '''';
       sl.Add(S);

       if Datahandler.FilterState = fsUseDacs then
       begin
        sl2 := DBView.ControlsToSQL(False);
        try
         for I := 0 to sl2.Count-1 do
         begin
          if (i = 0) and (PosRestrict('AND', sl2[i], false) = 0) then
            sl.Add('And ' + sl2[i])
          else
            sl.Add(SL2[i]);
         end;
        finally
         sl2.Free;
        end;
       end
       else if Datahandler.FilterState = fsUseFilter then
       begin
        FSQL := TILSQL.Create(Self);
        try
          FSQL.FNoCheck := True;
          FSQL.FIsFilter := True;
          FSQL.TargetTable := Self;
          FSQL.SetSQLStatement(DataHandler.Filter);
          if Assigned(Datahandler.FOnFilter) then
          begin
           Datahandler.FOnFilter(DataHandler, FSQL.Params);
           if FSQL.Params.Count > 0 then
           begin
            FSQL.SetValues;
            FDataHandler.Filter.Assign(FSQL.FValuedSQL);
           end;
          end;

          for i := 0 to Datahandler.Filter.Count-1 do
          begin
           if i = 0 then
             sl.Add('And ' + Datahandler.Filter[i])
           else
             sl.Add(Datahandler.Filter[i]);
          end;
          if FSQL.Params.Count > 0 then
            FDataHandler.Filter.Assign(FSQL.SQLStatement);
        finally
          FSQL.Free;
        end;
       end;

       Result := DBView.Search(sl);
     finally
       sl.Free;
     end;
  end;

  function TILDataTable.GetLinkedParentRecord(AChildRecord : TILDataRecord; const ALinkField : string;
    AParentTable : TILDataTable) : Boolean;
    var
    i : integer;
    F : TILField;
    Indx : integer;
    sl : TStringList;
    S : string;
    LooField : string;
    DS : TILDataset;
    NeedsRefresh : Boolean;
    ColList : string;
  begin
     Result := False;
     if not (TMan.ActiveTable = self) then
      Exit;
     F := nil;
     if AChildRecord = nil then
       Exit;
     for I := 0 to Fields.Count-1 do
     begin
       if Fields.Items[i].LookupType = looTableKey then
       begin
         if Fields.Items[i].LookupTable = AParentTable then
         begin
            if (ALinkField = '') or SameText(ALinkField, Fields.Items[i].LookupField) then
            begin
             Indx := AChildRecord.IndexOf(Fields.Items[i].FieldName);
             F := AChildRecord[Indx];
             LooField := Fields.Items[i].LookupField;
             Break;
            end;
         end;
       end;
     end;
     if F = nil then
       Exit;
     sl := TStringList.Create;
     FLocatingLinkedParent := True;
     try
       ColList := '';
       for I := 0 to AParentTable.Columns.Count-1 do
       begin
         ColList := ColList + AParentTable.Columns[i].FieldName;
         if I < AParentTable.Columns.Count-1 then
          ColList := ColList + ', ';
       end;
       if F.FieldDef.DataType = dtNumber then
         s := 'select ' + CollIst + ' where ' + LooField + ' = ' + F.AsString
       else
         s := 'select ' + ColList + ' where ' + LooField + ' = ' + '''' + F.AsString  + '''';
       sl.Add(S);
       if AParentTable.Dataset.Count = 0 then
       begin
         AParentTable.DBView.Search(sl);
       end
       else
       begin
         DS := AParentTable.DBView.SearchPrivate(sl);
         if DS.Count = 1 then
         begin
            if F.FieldDef.DataType = dtNumber then
             NeedsRefresh := not (DS[0][LooField].AsInteger = AParentTable.Dataset.FocusedRecord[LooField].AsInteger)
            else
             NeedsRefresh := not SameText(DS[0][LooField].AsString, AParentTable.Dataset.FocusedRecord[LooField].AsString);
            if not NeedsRefresh then
            begin
              DS.Free;
              FLocatingLinkedParent := false;
            end
            else
              AParentTable.Dataset := DS;
            Result := True;
         end;
       end;
     finally
       sl.Free;
     end;
  end;

  function TILDataTable.CopyTable : TILDataTable;
  begin
    Result := TILDataTable.Create(Owner);
    Result.Fields.Assign(Fields);
    TMan.RegisterFreeTable(Result);
    Result.FTableSpace := TableSpace;
    Result.FTableName := TableName;
    Result.SetNames(Result);
    Result.FColumns := TILColumns.Create(Result, Result.Fields);
    Result.OpenTable(TMan.FLibPath);
    Result.IsReadOnly := True;
  end;

  procedure TILDataTable.AddToImageTable(const TagName: string;
    SourceRecord: TILDataRecord);
  begin
    SourceRecord.AddToImageTable(TagName);
  end;

  function TILDataTable.GetFromImageTable(const TagName: string;
    TargetRecord: TILDataRecord) : Boolean;
  begin
    Result := TargetRecord.GetFromImageTable(TagName);
  end;


  { TILLookupTable ------------------------------------------------------------- }

  constructor TILLookupTable.Create(AOwner: TComponent);
  begin
    inherited;
  end;

  procedure TILLookupTable.CreateLookupField(const AFieldName: string;
    const ColumnWidth: integer);

  var
    F: TILFieldDef;
  begin

    if AFieldName = '' then
      Exit;
    F := FFields.Add;
    F.FDataType := dtString;
    F.FFieldType := ftUserData;
    F.FFieldName := AFieldName;
    F.FColumnWidth := 100;
    F.FReportField := True;
    F.FIndexed := True;
    F.FMandatory := True;
    F.FIdentifier := True;
    F.FMaxLength := 0;
    if Self is TILImageTable then
    begin
      F.FImageText := True;
    end;
    if Columns = nil then
      FColumns := TILColumns.Create(Self, Fields);
    FColumns.AssignSource;
  end;

  procedure TILLookupTable.Loaded;
  begin
    inherited;
    if Fields.Count = 0 then
      CreateLookupField(FFieldName, 100);
  end;


  function TILLookupTable.GetLookupFieldName: string;
  begin
    if Fields.Count > 0 then
      Result := Fields.Items[0].FieldName
    else
      Result := FFieldName;
  end;

  procedure TILLookupTable.SetLookupFieldName(const Value : string);
  begin
    if Value = '' then
      Exit;
    FFieldName := Value;
    if Fields.Count > 0 then
      Fields.Items[0].FFieldName := Value
    else
      CreateLookupField(FFieldName, 100);
  end;

  { TILImageTable -------------------------------------------------------------- }

  constructor TILImageTable.Create(AOwner: TComponent);
  begin
    inherited;
  end;

  procedure TILImageTable.Add(const TagName: string; SourceRecord: TILDataRecord);

  var
    DR: TILDataRecord;
    F: TILField;
  begin
    if SourceRecord.Ilib is TILLookupTable then
      HandleILException(E_ImageSourceRecord);

    if SourceRecord.IsNew then
      HandleILException(E_SaveImage);
    if (SourceRecord.ThumbnailSize = 0) and (SourceRecord.FileName = '') then
      HandleILException(E_MissingImage);
    DR := CreateDataRecord;
    DR.Clear;
    F := TILField.Create(Self, FieldName, Fields);
    DR.Add(F);
    F.FOwnerRecord := DR;
    F.AsString := TagName;

    DR.FFileName := SourceRecord.FFileName;
    DR.ThumbnailOffset := SourceRecord.ThumbnailOffset;
    DR.ThumbnailSize := SourceRecord.ThumbnailSize;
    DR.FMediaType := SourceRecord.MediaType;
    DR.FImageFetched := false;
    Dataset.Add(DR);
    Dataset.Save;
    SourceRecord.FImgTableIndex := DR.RecordIndex;
    if not SourceRecord.Changed then
    begin
      SourceRecord.FImageChanged := True;
      SourceRecord.Save;
    end;
    Inval;
  end;

  procedure TILImageTable.Add(const TagName: string; const FileName: string);
  var
    DR: TILDataRecord;
    F: TILField;
    Pic : TPicture;
  begin
    if (Trim(TagName) = '') or (Trim(FileName) = '') then
      Exit;
    if SameText(FileName, CLIPBOARD_IMAGE) then
    begin
      if Clipboard.HasFormat(CF_BITMAP) then
      begin
        Pic := TPicture.Create;
        try
          Pic.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(cf_bitmap), 0);
          Add(TagName, Pic);
        finally
          Pic.Free;
        end;
      end
      else
        HandleILException(E_MissingImage);
      Exit;
    end;

    if not FileExists(Filename) then
    begin
      Comment1 := FileName;
      HandleILException(E_FileNotExists);
    end;

    DR := CreateDataRecord;
    DR.Clear;
    F := TILField.Create(Self, FieldName, Fields);
    F.FOwnerRecord := DR;
    DR.Add(F);
    F.AsString := TagName;
    DBWork.Insert(FileName, DR);
  end;

  procedure TILImageTable.Add(const TagName: string; Image: TPicture);

  var
    DR: TILDataRecord;
    F: TILField;
  begin
    if (Trim(TagName) = '') or (Image = nil) then
      Exit;
    DR := CreateDataRecord;
    DR.Clear;
    F := TILField.Create(Self, FieldName, Fields);
    F.FOwnerRecord := DR;
    DR.Add(F);
    F.AsString := TagName;
    DBWork.Insert(Image, DR);
  end;

  procedure TILImageTable.Add(Files: TStringList);
  begin
    DBWork.Insert(Files, false);
  end;

  procedure TILImageTable.ChangeImage(Image : TPicture);
  begin
    if Dataset.Position = -1 then
      Exit;
    DBWork.UpdateImage(Dataset.Position, Image);
  end;

  procedure TILImageTable.ChangeImage(const FileName : string);
  begin
    DBWork.UpdateImage(Dataset.Position, FileName);
  end;

  function TILImageTable.ShareWith(const TagName: string;
    TargetRecord: TILDataRecord) : Boolean;

  var
    sl: TStringList;
    DS: TILDataset;
  begin
    if TMan.SharedImages = nil then
      HandleILException(E_ImgTableNotAssigned);
    Result := True;
    sl := TStringList.Create;
    sl.Add(FieldName + ' = ' + '''' + TagName + '''');
    DS := DBView.SearchPrivate(sl);
    try
      if DS.Count = 0 then
      begin
        TargetRecord.UpdateImage(nil);
        TargetRecord.Ilib.Inval;
        Result := false;
        Exit;
      end;

      { Remove olds }
      if TargetRecord.Image <> nil then
      begin
        if (TargetRecord.FOldImage <> TargetRecord.FImage) and (TargetRecord.FOldImage <> nil) then
         FreeAndNil(TargetRecord.FOldImage);
        FreeAndNil(TargetRecord.FImage);
      end;
      TargetRecord.FOldImage := nil;

      { Set ref values }
      TargetRecord.OwnsImage := True;
      TargetRecord.FFileName := DS[0].FFileName;
      TargetRecord.ThumbnailOffset := DS[0].ThumbnailOffset;
      TargetRecord.ThumbnailSize := DS[0].ThumbnailSize;
      TargetRecord.FMediaType := DS[0].MediaType;
      TargetRecord.FImageFetched := false;
      TargetRecord.FImgTableIndex := DS[0].RecordIndex;

      TargetRecord.FImageChanged := True;
      if Assigned(TargetRecord.OwnerDataset) then
        TargetRecord.OwnerDataset.FChngTracker.Add(TargetRecord);
      if Tman.AutoSave then
        TargetRecord.ILib.DBWork.Save;
      if TMan.FTranslogBuffer.Size = 0  then
        TargetRecord.Ilib.Inval;
    finally
      DS.Free;
      sl.Free;
    end;
  end;

  function TILImageTable.ShareWith(TargetRecord: TILDataRecord) : Boolean;
  begin
    Result := True;
    if TMan.SharedImages = nil then
     HandleILException(E_ImgTableNotAssigned);
    TargetRecord.OwnsImage := True;
    TargetRecord.FFileName := FocusedRecord.FFileName;
    TargetRecord.ThumbnailOffset := FocusedRecord.ThumbnailOffset;
    TargetRecord.ThumbnailSize := FocusedRecord.ThumbnailSize;
    TargetRecord.FMediaType := FocusedRecord.MediaType;
    TargetRecord.FImageFetched := false;
    TargetRecord.FImgTableIndex := FocusedRecord.RecordIndex;
    if TargetRecord.FOldImage = TargetRecord.FImage then
       TargetRecord.FOldImage := nil;
    if Assigned(TargetRecord.FOldImage) and (TargetRecord.FOldImage <> TargetRecord.FImage) then
      FreeAndNil(TargetRecord.FOldImage);
    if Assigned(TargetRecord.Image) then
      FreeAndNil(TargetRecord.FImage);
    TargetRecord.FImageChanged := True;
    if Tman.AutoSave then
        TargetRecord.ILIB.DBWork.Save;
    TargetRecord.Ilib.Inval;
  end;

  function TILImageTable.GetBitmap(const TagName : string; const Size: integer)
    : Vcl.Graphics.TBitmap;
  var
    sl: TStringList;
    DS: TILDataset;
  begin
    Result := nil;
    sl := TStringList.Create;
    sl.Add(FieldName + ' = ' + '''' + TagName + '''');
    DS := DBView.SearchPrivate(sl);
    try
      if DS.Count > 0 then
       Result := GetBitmap(DS[0].RecordIndex, Size);
    finally
      DS.Free;
      sl.Free;
    end;
  end;

  function TILImageTable.IndexOfTagName(const TagName : string) : integer;
  var
    sl: TStringList;
    DS: TILDataset;
  begin
    sl := TStringList.Create;
    sl.Add(FieldName + ' = ' + '''' + TagName + '''');
    DS := DBView.SearchPrivate(sl);
    try
      if DS.Count = 0 then
        Result := -1
      else
        Result := DS[0].RecordIndex;
    finally
      DS.Free;
      sl.Free;
    end;
  end;

  function TILImageTable.GetBitmap(const RecordIndex: integer; const Size: integer)
    : Vcl.Graphics.TBitmap;
  var
    R: TILDataRecord;
  begin
    R := Records[RecordIndex];
    Result := GetThumbNail(R.ThumbnailOffset, R.ThumbnailSize, False);
    R.Free;
    if Size > 0 then
      ScaleBitmap(Result, Size, Size);
  end;

  { TILTreeViewTable ---------------------------------------------------------- }

  constructor TILTreeViewTable.Create(AOwner: TComponent);
  begin
    inherited;
    FTreeViewParams := TILTreeViewParams.Create(Self);
    FRootSiblings := TStringList.Create;
    FRootSiblings.OwnsObjects := True;
  end;

  destructor TILTreeViewTable.Destroy;
  begin
    FTreeViewParams.Free;
    FRootSiblings.Free;
    inherited;
  end;

  procedure TILTreeViewTable.Notification(AComponent: TComponent;
    Operation: TOperation);
  begin
    inherited Notification(AComponent, Operation);
    if csDestroying in ComponentState then
      Exit;
    if Operation = opRemove then
    begin
      if AComponent = TreeViewControl then
        FTreeViewControl := nil;
    end;
  end;

  procedure TILTreeViewTable.InsertAfter(ASiblingRecord : TILDataRecord; ReadCtrls : Boolean);
  begin
      TreeViewParams.SetInsertAfter(ASiblingRecord);
      TreeViewParams.InsertMode := timSiblingAfter;
      if ReadCtrls then
        AddToDataSet
      else
        DBWork.InsertEmptyRecord;
  end;

  procedure TILTreeViewTable.InsertBefore(ASiblingRecord : TILDataRecord; ReadCtrls : Boolean);
  begin
     TreeViewParams.SetInsertBefore(ASiblingRecord);
     TreeViewParams.InsertMode := timSiblingBefore;
     if ReadCtrls then
       AddToDataSet
     else
       DBWork.InsertEmptyRecord;
  end;

  procedure TILTreeViewTable.InsertChild(AParentRecord : TILDataRecord;  ReadCtrls : Boolean);
  begin
    TreeViewParams.SetInsertChild(AParentRecord);
    TreeViewParams.InsertMode := timChild;
    if ReadCtrls then
     AddToDataSet
    else
     DBWork.InsertEmptyRecord;
  end;

  procedure TILTreeViewTable.InsertRoot(ReadCtrls : Boolean);
  begin
    TreeViewParams.InsertMode := timRoot;
    if ReadCtrls then
     AddToDataSet
    else
     DBWork.InsertEmptyRecord;
  end;

  function TILTreeViewTable.GetTVChildCount(ParentID: integer): integer;
  var
    SR: TSearcher;
    sl: TStringList;
    DS: TILDataset;
  begin
    sl := TStringList.Create;
    try
      if ParentID = -1 then
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' + TMan.ReportNull)
      else
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' +
          IntToStr(ParentID));
      SR := CreateSearcher(Self, sl);
      SR.FPrivate := True;
      DS := SR.Search;
      try
        Result := DS.Count;
      finally
        SR.Free;
        DS.Free;
      end;
    finally
      sl.Free;
    end;
  end;

  function TILTreeViewTable.GetTVChildCount(ParentRecord: TILDataRecord): integer;
  var
   i : integer;
  begin
    Result := 0;
    for I := 0 to Dataset.Count-1 do
    begin
      if Dataset[i].TVParentRecord = ParentRecord then
        Inc(Result);
    end;
  end;

  function TILTreeViewTable.Circular(const ParentID, ChildID: integer): Boolean;

  var
    R: TILDataRecord;
    ID: integer;
  begin
    Result := false;
    if ParentID = ChildID then
    begin
      Result := True;
      Exit;
    end;
    R := RecordsX[ChildID];
    while not R[c_SysFieldNames[SYS_LinkID]].IsEmpty and
      (R[c_SysFieldNames[SYS_LinkID]].AsInteger <> ParentID) do
    begin
      ID := R[c_SysFieldNames[SYS_LinkID]].AsInteger;
      FreeAndNil(R);
      if ID = ChildID then
      begin
        Result := True;
        Break;
      end;
      R := RecordsX[ID];
    end;

    if Assigned(R) then
      R.Free;
    if Result then
      Exit;

    { Check i parent has a child as parent }
    R := RecordsX[ParentID];
    while not R[c_SysFieldNames[SYS_LinkID]].IsEmpty and
      (R[c_SysFieldNames[SYS_LinkID]].AsInteger <> ParentID) do
    begin
      ID := R[c_SysFieldNames[SYS_LinkID]].AsInteger;
      FreeAndNil(R);
      if ID = ChildID then
      begin
        Result := True;
        Break;
      end;
      R := RecordsX[ID];
    end;
    if Assigned(R) then
      R.Free;
  end;

  function TILTreeViewTable.GetParentRecord(ChildID: integer): TILDataRecord;
  var
    R: TILDataRecord;
    ID: integer;
  begin
    Result := nil;
    if ChildID = -1 then
      Exit;
    R := RecordsX[ChildID];
    if R[c_SysFieldNames[SYS_LinkID]].IsEmpty then
    begin
      R.Free;
      Exit;
    end;

    if R[c_SysFieldNames[SYS_Level]].AsInteger = 0 then
    begin
      R.Free;
      Exit;
    end;
    ID := R[c_SysFieldNames[SYS_LinkID]].AsInteger;
    R.Free;
    Result := RecordsX[ID];
  end;

  function TILTreeViewTable.GetParentRecord(ChildRecord: TILDataRecord): TILDataRecord;
  var
    Indx: integer;
  begin
    Result := nil;
    if ChildRecord = nil then
      Exit;
    Result := ChildRecord.TVParentRecord;
    if Result = nil then
    begin
      Indx := -1;
      if not ChildRecord[c_SysFieldNames[SYS_LinkID]].IsEmpty then
       Indx := Dataset.IndexOf(ChildRecord[c_SysFieldNames[SYS_LinkID]].AsInteger);
      if Indx <> -1 then
        Result := Dataset[Indx];
    end;
  end;

  function TILTreeViewTable.GetParentID(ChildID: integer): Integer;
  var
    R : TILDataRecord;
  begin
    Result := -1;
    R := GetParentRecord(ChildID);
    if R = nil then
      Exit;
    Result := R.RecordIndex;
    R.Free;
  end;

  procedure TILTreeViewTable.SetTreeViewControl(const Value : TILTreeView);
  begin
    if FTreeViewControl = Value then
      Exit;
    FTreeViewControl := Value;
    if Value <> nil then
    begin
      FTreeViewControl.FTVTable := Self;
      FTreeViewControl.FreeNotification(Self);
    end;
  end;

  procedure TILTreeViewTable.SetTreeViewLabelField(const Value : string);
  var
    Indx: integer;
  begin
    if Value = FTreeViewLabelField then
      Exit;
    if Value = '' then
      Exit;
    Indx := Fields.IndexOf(Value);
    if Indx = -1 then
      HandleILException(E_InvalidFieldName, Value);
    if not(Fields.Items[Indx].DataType in [dtString, dtDate, dtNumber]) then
      HandleILException(E_TreeViewLabel);
    if Fields.Items[Indx].IsReadOnly then
      HandleILException(E_TreeViewLabel);
    if Fields.Items[Indx].IsMultiVal then
      HandleILException(E_TreeViewLabel);
    FTreeViewLabelField := Value;
  end;

  function TILTreeViewTable.GetSiblings(ParentID, Index: integer)
    : TILDataRecord;
  var
    SR: TSearcher;
    sl: TStringList;
    DS: TILDataset;
  begin
    if (Index < 0) then
      HandleILException(E_OutOfBounds, IntToStr(Index));
    sl := TStringList.Create;
    try
      if ParentID = -1 then
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' + TMan.ReportNull)
      else
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' +
          IntToStr(ParentID));
      SR := CreateSearcher(Self, sl);
      SR.FPrivate := True;
      DS := SR.Search;
      try
        if Index > (DS.Count - 1) then
          HandleILException(E_OutOfBounds, IntToStr(Index));
        Result := TILDataRecord.Create(Self, false);
        Result.Assign(DS[Index]);
      finally
        SR.Free;
        DS.Free;
      end;
    finally
      sl.Free;
    end;
  end;

  procedure TILTreeViewTable.FillSiblings(ParentRecord : TILDataRecord);
  var
    DS : TILDataset;
    i : Integer;
    sl : TStringList;
    L : TStringList;
  begin
    if ParentRecord = nil then
      L := FRootSiblings
    else
      L := ParentRecord.FSiblingList;

    if Dataset.Changed and (L.Count > 0) then
      Exit;

    L.Clear;
    if ParentRecord <> nil then
    DS := DBView.SearchPrivate('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' +
      IntToStr(ParentRecord.RecordIndex))
    else
      DS := DBView.SearchPrivate('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' + TMan.ReportNull);
    DS.OwnsRecords := false;
    sl := TStringList.Create;
    sl.Add('SYS_SiblingIndex');
    DS.Sort(sl, soAscending);
    sl.Free;
    for I := 0 to DS.Count-1 do
    begin
       L.AddObject(FillZeroes(IntToStr(DS[i].RecordID), 7), DS[i]);
    end;
    DS.Free;
  end;

  procedure TILTreeViewTable.AddToSiblingList(Index : integer; ParentRecord, Sibling : TILDataRecord);
  var
    S : string;
    R : TILDatarecord;
    L : TStringList;
  begin
      if ParentRecord = nil then
        L := FRootSiblings
      else
        L :=  ParentRecord.FSiblingList;
      R := CreateDataRecord;
      R.Assign(Sibling);
      S := FillZeroes(IntToStr(Sibling.RecordID), 7);
      if (Index = L.Count) or (Index = -1) then
        L.AddObject(S, R)
      else
        L.InsertObject(Index, S, R);
  end;

  procedure TILTreeViewTable.DeleteSiblingList(ParentRecord, Sibling : TILDataRecord);
  var
    S : string;
    Index : integer;
    L : TStringList;
  begin
     if ParentRecord = nil then
       L := FRootSiblings
     else
       L := ParentRecord.FSiblingList;
     S := FillZeroes(IntToStr(Sibling.RecordID),7);
     Index := L.IndexOf(S);
     if Index <> -1 then
       L.Delete(Index);
  end;

  procedure TILTreeViewTable.MoveSibling(NewIndex : integer; Sibling : TILDataRecord);
  var
    S : string;
    Index : integer;
    L : TStringList;
  begin
     if Sibling.TVParentRecord = nil then
       L := FRootSiblings
     else
       L := Sibling.TVParentRecord.FSiblingList;
     S := FillZeroes(IntToStr(Sibling.RecordID),7);
     Index := L.IndexOf(S);
     if Index <> -1 then
     begin
       if NewIndex = -1 then
         L.Move(Index, L.Count-1)
       else
        L.Move(Index, NewIndex);
     end;
  end;

  function TILTreeViewTable.GetTVChildren(ParentID: integer): TILDataset;
  var
    SR: TSearcher;
    sl: TStringList;
  begin
    sl := TStringList.Create;
    try
      if ParentID = -1 then
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' + TMan.ReportNull)
      else
        sl.Add('Select * where ' + c_SysFieldNames[SYS_LinkID] + ' = ' +
          IntToStr(ParentID));
      SR := CreateSearcher(Self, sl);
      SR.FPrivate := True;
      try
        Result := SR.Search;
      finally
        SR.Free;
      end;
    finally
      sl.Free;
    end;
  end;

  function TILTreeViewTable.GetTVChildren(ParentRecord: TILDataRecord): TILDataset;
  var
    i : integer;
  begin
    Result := TILDataset.Create(Self);
    Result.OwnsRecords := false;
    for i := 0 to Dataset.Count-1 do
    begin
      if Dataset[i].TVParentRecord = ParentRecord then
      begin
         Result.Add(DataSet[i]);
         Result[Result.Count-1].FOwnerDataset := Dataset;
      end;
    end;
  end;

  function TILTreeViewTable.GetLevel : integer;
  var
    PR : TILDataRecord;
  begin
     if TreeViewParams.InsertParentRecord <> nil then
     begin
        PR := TreeViewParams.InsertParentRecord;
        Result := PR[c_SysFieldNames[SYS_Level]].AsInteger + 1;
        Exit;
     end;

     if TreeViewParams.InsertParentID <> -1 then
     begin
      PR := RecordsX[TreeViewParams.InsertParentID];
      try
        Result := PR[c_SysFieldNames[SYS_Level]].AsInteger + 1;
      finally
        PR.Free;
      end;
     end
     else
     begin
        Result := 0;
     end;
  end;

  procedure TILTreeViewTable.RenumberSiblings(AParentRecord : TILDataRecord; const NewSiblingIndex : integer);
  var
    DS : TILDataSet;
    DR : TILDataRecord;
    i, j : integer;
    L : TStringList;
  begin
    if AParentRecord = nil then
      L := FRootSiblings
    else
      L := AParentRecord.FSiblingList;

    { Renumber siblings }
    DS := GetTVChildren(AParentRecord);
    try
      for i := 0 to DS.Count - 1 do
      begin
        DR := DS[i];

        for j := 0 to l.Count-1 do
        begin
          if TILDataRecord(l.Objects[j]).RecordID = DR.RecordID then
          begin
             DR[c_SysFieldNames[SYS_SiblingIndex]].AsInteger := j;
          end;
        end;
      end;
    finally
      DS.Free;
    end;
   end;

  function TILTreeViewTable.GetSiblingIndex : integer;
  begin
     if TreeViewParams.InsertSiblingIndex = -1 then
     begin
        if (TreeViewParams.InsertParentRecord  = nil) and (TreeViewParams.InsertParentId  = -1) then
           Result := FRootSiblings.Count
        else if TreeViewParams.InsertParentRecord <> nil then
          Result := TreeViewParams.InsertParentRecord.FSiblingList.Count
        else
          Result := GetTVChildCount(TreeViewParams.InsertParentID);
     end
      else
      begin
        Result := TreeViewParams.InsertSiblingIndex;
      end;
  end;

  function TILTreeViewTable.InPath(ParentRecord, ChildRecord : TILDatarecord;
    IncludeRoot: Boolean): Boolean;

  var
    Rec: TILDataRecord;
    LinkID: integer;
  begin
    Result := false;
    if IncludeRoot and (ParentRecord = ChildRecord) then
    begin
      Result := True;
      Exit;
    end;

    Rec := ChildRecord;
    while not Rec['SYS_LinkID'].IsEmpty do
    begin
      LinkID := Rec['SYS_LinkID'].AsInteger;
      if LinkID = ParentRecord.RecordIndex then
      begin
        Result := True;
        Break;
      end;
      Rec := Dataset[Dataset.IndexOf(LinkID)];
    end;
  end;

  function TILTreeViewTable.MoveNode(SourceNode : TILDataRecord; NewParent: TILDataRecord;
      SiblingIndex: integer ): integer;
  begin
      Result := MoveNode(SourceNode.RecordIndex, NewParent.RecordIndex, SiblingIndex);
  end;

  function TILTreeViewTable.MoveNode(ARecordIndex: integer;
    NewParentID: integer; NewSiblingIndex: integer) : integer;

  { Moves a record node from one parent to another and / or changes its sibling index }
  var
    DR, DR2, DR3: TILDataRecord;
    Indx : integer;
    OldParentID: integer;

    procedure Check(Prnt : TILDataRecord);
    var
        SCnt1, SCnt2 : integer;
    begin
        if Prnt = nil then
        begin
          SCnt1 := GetTVChildCount(Prnt);
          SCnt2 := GetTVChildCount(-1);
          if SCnt1 < Scnt2 then
            HandleILException(E_SiblingsIncomplete);
        end
        else if Prnt.RecordIndex <> -1 then
        begin
          SCnt1 := GetTVChildCount(Prnt);
          SCnt2 := GetTVChildCount(Prnt.RecordIndex);
          if SCnt1 < Scnt2 then
           HandleILException(E_SiblingsIncomplete);
        end;
    end;

  begin
    Result := 0;
    if NewParentID <> -1 then
     if Circular(NewParentID, ARecordIndex) then
     begin
      Result := E_Circular;
      Exit;
     end;

    Indx := Dataset.IndexOf(ARecordIndex);
    if Indx = -1 then
    begin
      Comment1 := Translator.GetTranslation(T_SOURCE, TRANSL_TERMS);
      Result := E_TVMoveRecord;
      Exit;
    end;

    DR := Dataset[Indx];

    Check(DR.TVParentRecord); {SiblingList complete?}

    if DR[c_SysFieldNames[SYS_LinkId]].IsEmpty then
      OldParentID := -1
    else
      OldParentID := DR[c_SysFieldNames[SYS_LinkId]].AsInteger;
    Indx := Dataset.IndexOf(OldParentID);

    if Indx = -1 then
    begin
      Comment1 := Translator.GetTranslation(T_SOURCE, TRANSL_TERMS);
      Result := E_TVMoveRecord;
      Exit;
    end;

    DR3 := Dataset[Indx];
    FillSiblings(DR3);
    FillSiblings(DR.TVParentRecord);

    if OldParentID <> NewParentID then
    { Set new parent and level }
    begin
      if NewParentID = -1 then
      begin
         DR[c_SysFieldNames[SYS_LinkId]].Clear;
         DR[c_SysFieldNames[SYS_Level]].AsInteger := 0;
         AddToSiblingList(NewSiblingIndex, nil, DR);
         RenumberSiblings(nil, 0);
      end
      else
      begin
         DR[c_SysFieldNames[SYS_LinkId]].AsInteger := NewParentID;
         DR2 := Dataset[Dataset.IndexOf(NewParentID)];
         DR[c_SysFieldNames[SYS_Level]].AsInteger :=
             DR2[c_SysFieldNames[SYS_Level]].AsInteger + 1;
         AddToSiblingList(NewSiblingIndex, DR2, DR);
         RenumberSiblings(DR2, 0);
      end;

      Indx := Dataset.IndexOf(ARecordIndex);
      DeleteSiblingList(DR3, Dataset[Indx]);
      RenumberSiblings(DR3, 0);
    end
    else { Just change position }
    begin
      MoveSibling(NewSiblingIndex, DR);
      RenumberSiblings(DR.TVParentRecord, 0);
    end;

    Dataset.Sort;
    if HasViewer then
      Viewer.Report.Redraw;
    if TVControl <> nil then
     TVControl.BuildTree(Dataset);
  end;

  function TILTreeViewTable.ExpandNode(ParentRecord: TILDataRecord;
    Recursive: Boolean): integer;

  var
    DS: TILDataset;
    i: integer;
    Indx: integer;
    ParentID : integer;
  begin
    if ParentRecord = nil then
     ParentID := -1
    else
     ParentID := ParentRecord.RecordIndex;
    if not Recursive then
      DS := GetTVChildren(ParentID)
    else
      DS := GetPathNodes(ParentID);
    Result := 0;
    Indx := -1;
    if DS.Count > 0 then
      Indx := Dataset.IndexOf(DS[0].RecordIndex);

    if (Indx <> -1) or (DS.Count = 0) then
    begin
      DS.Free;
      Exit;
    end;

    Result := DS.Count;
    try
      DS.OwnsRecords := false;
      for i := 0 to DS.Count - 1 do
      begin
        DS[i].FOwnerDataset := Dataset;
        Dataset.Add(DS[i]);
      end;
    finally
      DS.Free;
    end;

    if (Result > 0) and (Columns.FSortFields.Count > 0) then
    begin
      FDataset.Sort(Columns.FSortFields, FColumns.SortOrdering);
    end;
    if HasViewer and (Result > 0) then
    begin
      Viewer.CreateReport;
      Viewer.Invalidate;
    end;
  end;

  function TILTreeViewTable.CollapseNode(ParentRecord: TILDataRecord): integer;

  var
    DS: TILDataset;
    i: integer;
    Indx: integer;
    ParentID : integer;
  begin
    if ParentRecord = nil then
     ParentID := -1
    else
     ParentID := ParentRecord.RecordIndex;
    DS := GetPathNodes(ParentID);
    Result := DS.Count;
    try
      DS.OwnsRecords := True;
      Dataset.BeginUpdate;
      try
        for i := 0 to DS.Count - 1 do
        begin
          DS[i].FOwnerDataset := Dataset;
          Indx := Dataset.IndexOf(DS[i].RecordIndex);
          if Indx = -1 then
          begin
            Continue;
          end;
          Dataset.Delete(Indx);
        end;
      finally
        Dataset.EndUpdate;
      end;
    finally
      DS.Free;
    end;

    if (Result > 0) and (Columns.FSortFields.Count > 0) then
    begin
      FDataset.Sort(Columns.FSortFields, FColumns.SortOrdering);
    end;
    if HasViewer and (Result > 0) then
    begin
      Viewer.CreateReport;
      Viewer.Invalidate;
    end;
  end;

  function TILTreeViewTable.GetPathNodes(ParentID: integer): TILDataset;

  var
    DS: TILDataset;
    i: integer;
    procedure DoGetNodes(PID: integer);

  var
    DS: TILDataset;
    i: integer;
  begin
    DS := GetTVChildren(PID);
    try
      DS.OwnsRecords := false;
      for i := 0 to DS.Count - 1 do
        Result.Add(DS[i]);
      for i := 0 to DS.Count - 1 do
        DoGetNodes(DS[i].RecordIndex);
    finally
      DS.Free;
    end;
  end;
  begin
    Result := TILDataset.Create(Self);
    Result.OwnsRecords := True;
    DS := GetTVChildren(ParentID);
    try
      DS.OwnsRecords := false;
      for i := 0 to DS.Count - 1 do
        Result.Add(DS[i]);
      for i := 0 to DS.Count - 1 do
        DoGetNodes(DS[i].RecordIndex);
    finally
      DS.Free;
    end;
  end;

  function TILTreeViewTable.GetPathNodes(ParentRecord: TILDataRecord): TILDataset;

  var
    DS: TILDataset;
    i: integer;
    procedure DoGetNodes(DR: TILDataRecord);

  var
    DS: TILDataset;
    i: integer;
  begin
    DS := GetTVChildren(DR);
    try
      for i := 0 to DS.Count - 1 do
      begin
        Result.Add(DS[i]);
        DS[i].FOwnerDataset := Dataset;
      end;
      for i := 0 to DS.Count - 1 do
        DoGetNodes(DS[i]);
    finally
      DS.Free;
    end;
  end;

  begin
    Result := TILDataset.Create(Self);
    Result.OwnsRecords := false;
    DS := GetTVChildren(ParentRecord);
    try
      DS.OwnsRecords := false;
      for i := 0 to DS.Count - 1 do
      begin
        Result.Add(DS[i]);
        DS[i].FOwnerDataset := Dataset;
      end;
      for i := 0 to DS.Count - 1 do
        DoGetNodes(DS[i]);
    finally
      DS.Free;
    end;
  end;

  { TILIndexer classes ---------------------------------------------------------}

  procedure TILSaveIndex.Execute;
  var
    i : integer;
  begin
    while TMan.FSystemBusy do
    begin
      Application.ProcessMessages;
     sleep(50);
    end;

    while TMan.FMainThreadBusy do
     sleep(50);

    Tman.FSystemBusy := True;
    try
    for I := 0 to Tman.TableCount-1 do
     begin
       if Tman.Tables[i].FIndexUpdated then
       begin
         Tman.Tables[i].SaveIndex;
         Tman.Tables[i].FIndexUpdated := false;
       end;
     end;
    finally
      TMan.FSystemBusy := False;
    end;
  end;

  procedure TILLoadIndex.Execute;
  begin
    Tman.FSystemBusy := True;
    try
      TMan.LoadIndexes;
    finally
      Tman.FSystemBusy := False;
    end;
  end;


  { TILTreeViewParams ---------------------------------------------------------- }

  procedure TILTreeViewParams.SetSearchCtrlMode(const Value : TSearchCtrlMode);
  begin
    if FSearchCtrlMode = Value then
      Exit;
    ChangeCtrlMode(Value);
  end;

  procedure TILTreeViewParams.SetSortingMethod(const Value : TTreeViewSorting);
  begin
    FSortingMethod := Value;
  end;

  constructor TILTreeViewParams.Create(AnOwner: TILTreeViewTable);
  begin
    inherited Create;
    FIlib := AnOwner;
    FRootCondition := TILRecord.Create(Ilib);
    FWhereCondition := TILRecord.Create(Ilib);
    FInsertParentID := -1;
    FInsertSiblingIndex := -1;
    FSortingMethod := tsBySiblings;
    FSearchCtrlMode := scmStartWith;
    FFocusReferenced := True;
    FRecursive := True;
  end;

  procedure TILTreeViewParams.SetInsertMode(const Value : TTVInsertMode);
  begin
    FInsertMode := Value;
  end;

  function TILTreeViewParams.GetInsertParentRecord : TILDataRecord;
  begin
    Result := nil;
    if FFocusReferenced then { Always!}
    begin
      if InsertMode in[timSiblingBefore, timSiblingAfter] then
      begin
        if Ilib.DataSet.FocusedRecord <> nil then
          Result := Ilib.DataSet.FocusedRecord.GetTVParentRecord;
      end
      else if InsertMode <> timRoot then
        Result := Ilib.DataSet.FocusedRecord;
    end
    else
      Result := FInsertParentRecord;
  end;

  destructor TILTreeViewParams.Destroy;
  begin
    inherited;
    FRootCondition.Free;
    FWhereCondition.Free;
  end;

  procedure TILTreeViewParams.Assign(Source : TILTreeViewParams);
  begin
    FIlib := Source.Ilib;
    FRootCondition.Assign(Source.RootCondition);
    FWhereCondition.Assign(Source.WhereCondition);
    FInsertParentID := Source.InsertParentID;
    FInsertSiblingIndex := Source.InsertSiblingIndex;
    FSortingMethod := Source.SortingMethod;
    FSearchCtrlMode := Source.SearchCtrlMode;
    FRecursive := Source.Recursive;
  end;

  procedure TILTreeViewParams.SetInsertAfter(ASiblingRecord : TILDataRecord);
  var
    PRec : TILDataRecord;
  begin
      PRec := ILib.GetParentRecord(ASiblingRecord);
      FIlib.FillSiblings(PRec);
      FInsertParentRecord := PRec;
      if PRec = nil then
       InsertSiblingIndex := FIlib.FRootSiblings.IndexOf(FillZeroes(IntToStr(ASiblingRecord.RecordID), 7)) + 1
      else
       InsertSiblingIndex := Prec.FSiblingList.IndexOf(FillZeroes(IntToStr(ASiblingRecord.RecordID), 7)) + 1;
      InsertParentID := -1;
  end;

  procedure TILTreeViewParams.SetInsertBefore(ASiblingRecord : TILDataRecord);
  var
   PRec : TILDataRecord;
  begin
     PRec := Ilib.GetParentRecord(ASiblingRecord);
     FInsertParentRecord := PRec;
     FIlib.FillSiblings(Prec);
     if PRec = nil then
      InsertSiblingIndex := FIlib.FRootSiblings.IndexOf(FillZeroes(IntToStr(ASiblingRecord.RecordID), 7))
     else
      InsertSiblingIndex := Prec.FSiblingList.IndexOf(FillZeroes(IntToStr(ASiblingRecord.RecordID), 7));
     InsertParentID := -1;
  end;

  procedure TILTreeViewParams.SetInsertChild(AParentRecord : TILDataRecord);
  begin
    FIlib.FillSiblings(AParentRecord);
    FInsertParentRecord := AParentRecord;
    InsertSiblingIndex := -1;
  end;

  procedure TILTreeViewParams.ChangeCtrlMode(const AMode : TSearchCtrlMode);
  var
    R : TILRecord;
  begin
     if AMode = SearchCtrlMode then
       Exit;

     R := TILRecord.Create(FIlib);
     FIlib.DBView.ReadControls(R); { This fills the condtions records}
     R.Free;
     if AMode = scmWhere then
     begin
      FSearchCtrlMode := scmWhere;
      if WhereCondition.IsDataCleared(True) and (Self = FILib.TreeViewParams) then
       FIlib.DBView.ClearControls
      else if (Self = FILib.TreeViewParams) then
      begin
       FIlib.DBView.ClearControls;
       FIlib.DBView.WriteControls(WhereCondition);
      end;
     end
     else
     begin
      FSearchCtrlMode := scmStartWith;
      if RootCondition.IsDataCleared(True) and (Self = FILib.TreeViewParams) then
       FIlib.DBView.ClearControls
      else if (Self = FILib.TreeViewParams) then
      begin
        FIlib.DBView.ClearControls;
        FIlib.DBView.WriteControls(RootCondition);
      end;
     end;
  end;

  { TILColumns ------------------------------------------------------------ }

  function TILColumns.GetItems(Index: integer): TILFieldDef;

  var
    Indx: integer;
  begin
    Indx := FFieldSource.IndexOf(FOutDefs[Index]);
    if Indx = -1 then
     Indx := FFieldSource.IndexOfCaption(FOutDefs[Index]);
    Result := Ilib.Fields.Items[Indx];
  end;

  function TILColumns.GetCount: integer;
  begin
    Result := FOutDefs.Count;
  end;

  function TILColumns.GetExcludedItems(Index: integer): TILFieldDef;

  var
    i: integer;
    Ind: integer;
  begin
    Ind := -1;
    Result := nil;
    for i := 0 to FFieldSource.Count do
    begin
      if FFieldSource.Items[i].ReportField then
      begin
        if FOutDefs.IndexOf(FFieldSource.Items[i].FieldName) = -1 then
        begin
          Inc(Ind);
          if Ind = Index then
          begin
            Result := FFieldSource.Items[i];
            Break;
          end;
        end;
      end;
    end;
  end;

  function TILColumns.GetExcludedCount: integer;

  var
    i: integer;
  begin
    Result := 0;
    for i := 0 to FFieldSource.Count - 1 do
    begin
      if FFieldSource.Items[i].ReportField then
      begin
        if FOutDefs.IndexOf(FFieldSource.Items[i].FieldName) = -1 then
        begin
          Inc(Result);
        end;
      end;
    end;
  end;

  function TILColumns.GetTotalWidth: integer;

  var
    i: integer;
  begin
    Result := 0;
    if not Ilib.HasViewer then
     Exit;
    for i := 0 to Count - 1 do
    begin
      Inc(Result, Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing);
    end;
    if Ilib.FViewer.CanReportDisplayImages or (Ilib.FViewer.UpdateMarkers <> upmNone)
    or (roShowExpandButtons in ILib.FViewer.ReportOptions) then
      inc(Result, ILib.FViewer.GetRptLeftMargin);
  end;

  procedure TILColumns.AssignSource;
  var
    i: integer;

  function UseHandler : Boolean;
  var
    i : integer;
  begin
   Result := false;
   if Ilib.DataHandler = nil then
     Exit;
   if (FFieldSource.Count> 0) and
    (Ilib.DataHandler.DataFields.Count > 0) then
    begin
      for i := 0 to Ilib.DataHandler.DataFields.Count - 1 do
      if Ilib.DataHandler.DataFields.Items[i].Fieldname <> '' then
      begin
        Result := True;
        Break;
      end;
    end
  end;

  begin
    FOutDefs.Clear;
    if UseHandler then
    begin
      for i := 0 to Ilib.DataHandler.DataFields.Count - 1 do
      begin
      if not Assigned(Ilib.DataHandler.DataFields.Items[i].FieldDef) then
        Continue;
      if Ilib.DataHandler.DataFields.Items[i].ColumnVisible then
       if Ilib.DataHandler.DataFields.Items[i].FieldDef.FieldType <> ftStoredEvaluation then
        FOutDefs.Add(Ilib.DataHandler.DataFields.Items[i].FieldName);
      end;
    end
    else
    begin
      for i := 0 to FFieldSource.Count - 1 do
      begin
        if FFieldSource.Items[i].ReportField
        and (FFieldSource.Items[i].FieldType <> ftStoredEvaluation) then
          FOutDefs.Add(FFieldSource.Items[i].FieldName);
      end;

    end;
    FClickSortIndex := -1;
    FClickSortOrdering := soAscending;
    if Ilib.DataHandler = nil then
    begin
       FSortFields.Clear;
       FSortOrdering := soAscending;
    end
    else
    begin
      FSortFields.Assign(Ilib.DataHandler.SortKey);
      FSortOrdering := Ilib.DataHandler.SortOrdering;
    end;
  end;

  procedure TILColumns.Assign(Source: TILColumns);
  begin
    FOutDefs.Assign(Source.FOutDefs);
    FSortFields.Assign(Source.FSortFields);
    FSortOrdering := Source.SortOrdering;
  end;

  constructor TILColumns.Create(AnOwner: TILTable;
    FieldSource: TILFieldDefs);

  begin
    Ilib := AnOwner;
    FOutDefs := TStringList.Create;
    FSortFields := TStringList.Create;
    FFieldSource := FieldSource;
    FSortFields.OnChange := Chng;
    FClickSortIndex := -1;
    AssignSource;
    inherited Create;
  end;

  destructor TILColumns.Destroy;
  begin
    FOutDefs.Free;
    FSortFields.Free;
    inherited;
  end;

  procedure TILColumns.RestoreDefaults;
  begin
    AssignSource;
    FChanged := True;
  end;

  procedure TILColumns.Clear;
  begin
    FOutDefs.Clear;
    FChanged := True;
  end;

  procedure TILColumns.Add(const FieldName: string);

  var
    Indx: integer;
  begin
    Indx := FFieldSource.IndexOfCaption(FieldName);
    Comment1 := FieldName;
    if Indx = -1 then
      HandleILException(E_InvalidFieldName, '');
    if not FFieldSource.Items[Indx].ReportField then
      HandleILException(E_NotReportField, '');
    FOutDefs.Add(FFieldSource.Items[Indx].FieldName);
    FChanged := True;
    Comment1 := '';
  end;

  procedure TILColumns.Delete(const Index: integer);
  begin
    FOutDefs.Delete(Index);
    if Count = 0 then
      AssignSource;
    FChanged := True;
  end;

  procedure TILColumns.Move(const FromIndex, ToIndex: integer);
  begin
    FOutDefs.Move(FromIndex, ToIndex);
    FChanged := True;
  end;

  procedure TILColumns.Insert(const Index: integer; const FieldName: string);

  var
    Indx: integer;
  begin
    Indx := FOutDefs.IndexOf(FieldName);
    if Indx <> -1 then
      Exit;
    Indx := FFieldSource.IndexOfCaption(FieldName);
    if Indx = -1 then
    begin
      Comment1 := FieldName;
      HandleILException(E_InvalidFieldName, '');
    end;
    FOutDefs.Insert(Index, FFieldSource.Items[Indx].FieldName);
    FChanged := True;
  end;

  function TILColumns.IndexOf(const FieldName: string): integer;

  var
    i: integer;
    FldName: string;
    Indx: integer;
  begin
    Result := -1;
    Indx := FFieldSource.IndexOfCaption(FieldName);
    if Indx = -1 then
      Exit;
    FldName := FFieldSource.Items[Indx].FieldName;
    for i := 0 to Count - 1 do
    begin
      If SameText(FOutDefs[i], FldName) then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  procedure TILColumns.GetVisibleColumns(var First, Last: integer);

  { Returns the columns that are fully visible in report view mode }
  var
    i: integer;
    Acc: integer;
  begin
    Acc := 0;
    First := -1;
    Last := -1;
    if not ILib.HasViewer then
      Exit;
    First := Ilib.FViewer.FLeftColumn;
    for i := Ilib.FViewer.FLeftColumn to Count - 1 do
    begin
      Acc := Acc + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
      if Acc > Ilib.FViewer.ClientWidth then
      begin
        Last := i;
        Break;
      end;
      if i = Count - 1 then
        Last := i;
    end;
  end;

  procedure TILColumns.Apply;
  begin
    if FSortFields.Count > 0 then
    begin
      Ilib.Dataset.Sort(FSortFields, FSortOrdering);
    end;
    FSortChanged := false;
   { if FChanged then
    begin
      if Ilib.HasViewer then
      begin
        Ilib.Viewer.FLeftColumn := 0;
        Ilib.Viewer.FReport.Redraw;
      end;
    end;}
    FClickSortIndex := -1;
    FClickSortOrdering := soAscending;
    FChanged := false;
    if Ilib.HasViewer then
    begin
      Ilib.Viewer.FLeftColumn := 0;
      Ilib.Viewer.FReport.Redraw;
      Ilib.Viewer.UpdateScrollRange;
      Ilib.Viewer.UpdateScrollPos;
      ILib.Viewer.Invalidate;
    end;
    if ILib.DataHandler <> nil then
     if ILib.DataHandler.SaveColumns then
        Ilib.SaveColumns;
  end;

  function TILColumns.CompareSort(SortKey : TStrings; const Ordering : TSortOrdering) : Boolean;
  var
    i : integer;
  begin
     Result := True;
     if Ordering <> SortOrdering then
     begin
        Result := false;
        Exit;
     end;
     if SortFields.Count <> SortKey.Count then
     begin
        Result := false;
        Exit;
     end;
     for I := 0 to SortKey.Count-1 do
     begin
        if not SameText(Sortkey[i], SortFields[i]) then
        begin
          Result := false;
          Break;
        end;
     end;
  end;

  function TILColumns.CompareSort(SortKey : TCheckListBox; const Ordering : TSortOrdering) : Boolean;
  var
    i : integer;
    sl : TStringList;
  begin
     sl := TStringList.Create;
     try
       for I := 0 to SortKey.Count-1 do
       begin
        if SortKey.Checked[i] then
          sl.Add(Sortkey.Items[i]);
       end;
     Result := CompareSort(sl, Ordering);
     finally
       sl.Free;
     end;
  end;


  function TILColumns.CompareColumns(Columns : TStrings) : Boolean;
  var
    i : integer;
  begin
     Result := True;
     if Columns.Count <> Count then
     begin
       Result := false;
       Exit;
     end;
     for I := 0 to Columns.Count-1 do
     begin
         if not Sametext(Columns[i], Items[i].FieldName) then
         begin
           Result := false;
           Break;
         end;
     end;
  end;

  function TILColumns.CompareColumns(Columns : TChecklistBox) : Boolean;
  var
    i : integer;
    sl : TStringList;
  begin
     sl := TStringList.Create;
     try
       for I := 0 to Columns.Count-1 do
       begin
        if Columns.Checked[i] then
          sl.Add(Columns.Items[i]);
       end;
     Result := CompareColumns(sl);
     finally
       sl.Free;
     end;
  end;

  procedure TILColumns.CheckColumns(Cols: TStrings);
  var
    i : integer;
    Indx : integer;
  begin
  exit;
    for I := 0 to Cols.Count-1 do
    begin
         indx:=  Ilib.Fields.IndexOf(Cols[i]);
         if Indx = -1 then
           HandleILException(E_InvalidSortField);
         if not Ilib.Fields.Items[Indx].ReportField
         or (Ilib.Fields.Items[Indx].FieldType = ftStoredEvaluation)
         or (Ilib.Fields.Items[Indx].DataType in [dtLongText, dtText]) then
           HandleILException(E_IllegalSortField);
      end;
  end;

  procedure TILColumns.SetSortKey(SortKey : TStrings; const Ordering : TSortOrdering);
  begin
     CheckColumns(SortKey);
     SortOrdering := Ordering;
     SortFields.Assign(SortKey);
  end;

  procedure TILColumns.SetSortKey(SortKey : TCheckListBox; const Ordering : TSortOrdering);
  var
    i : integer;
    sl : TStringList;
    Indx : integer;
  begin
     sl := TStringList.Create;
     try
       for I := 0 to SortKey.Count-1 do
       begin
        if SortKey.Checked[i] then
        begin
          Indx := FFieldSource.IndexOf(SortKey.Items[i]);
          if Indx = -1 then
           Indx := FFieldSource.IndexOfCaption(SortKey.Items[i]);
          sl.Add(FFieldSource.Items[Indx].FieldName);
        end;
       end;
     SetSortKey(sl, Ordering);
     finally
       sl.Free;
     end;
  end;

  procedure TILColumns.SetColumns(Columns : TStrings);
  begin
    CheckColumns(Columns);
    FOutDefs.Assign(Columns);
  end;

  procedure TILColumns.SetColumns(Columns : TChecklistBox);
  var
    i : integer;
    sl : TStringList;
    Indx : integer;
  begin
     sl := TStringList.Create;
     try
       for I := 0 to Columns.Count-1 do
       begin
        if Columns.Checked[i] then
        begin
          Indx := FFieldSource.IndexOf(Columns.Items[i]);
          if Indx = -1 then
           Indx := FFieldSource.IndexOfCaption(Columns.Items[i]);
          sl.Add(FFieldSource.Items[Indx].FieldName);
        end;
       end;
     SetColumns(sl);
     finally
       sl.Free;
     end;
  end;


  function TILColumns.ColumnSplitterAtXY(X, Y: integer): integer;
  var
    R: TRect;
    i: integer;
    W: integer;
    P: integer;
  begin
    Result := -1;
    if not Ilib.HasViewer then
      Exit;
    R.Top := Ilib.FViewer.GetTopMargin;
    R.Bottom := Ilib.FViewer.ReportLineHeight + Ilib.FViewer.GetTopMargin;
    W := Ilib.FViewer.GetRptLeftMargin;
    P := ColumnPos(Ilib.FViewer.FLeftColumn, false);
    X := P + X;
    for i := 0 to Count - 1 do
    begin
      W := W + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
      R.Left := W - 2;
      R.Right := W + 2;
      if PtInRect(R, Point(X, Y)) then
      begin
        Result := i;
        Break;
      end;
    end;
  end;

  function TILColumns.GetColumnHeaderRect(Index: integer): TRect;

  { Returns the rect of a column header }
  var
    i: integer;
    W: integer;
    Ind: integer;
  begin
    Result.TopLeft := Point(0,0);
    Result.BottomRight := Point(0,0);
    if not Ilib.HasViewer then
      Exit;
    Ind := Ilib.FViewer.FLeftColumn - 1;
    Result.Top := Ilib.FViewer.GetTopMargin;
    Result.Bottom := Ilib.FViewer.ReportLineHeight + Ilib.FViewer.GetTopMargin;
    Result.Left := Ilib.FViewer.GetRptLeftMargin;
    W := Ilib.FViewer.GetRptLeftMargin;
    for i := Ilib.FViewer.FLeftColumn to Count - 1 do
    begin
      Inc(Ind);
      W := W + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
      Result.Right := W;
      if Ind = Index then
      begin
        Break;
      end;
      Result.Left := W;
    end;
  end;

  function TILColumns.ColumnPos(const ColumnIndex: integer; const FromLeftColumn: Boolean): integer;

  { Returns the position of ColumnIndex - absolute or relative to Left position }
  var
    i: integer;
    Start: integer;
  begin
    Result := -1;
    if not Ilib.HasViewer then
      Exit;
    if FromLeftColumn then
    begin
      if ColumnIndex < Ilib.FViewer.FLeftColumn then
      begin
        Result := -1;
        Exit;
      end;
      Start := Ilib.FViewer.FLeftColumn;
    end
    else
      Start := 0;
    Result := 0;
    for i := Start to Count - 1 do
    begin
      if i = ColumnIndex then
        Break;
      Result := Result + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
    end;
  end;

  function TILColumns.ColumnFromPos(const XPos: integer): integer;

  { Returns the column number at XPos }
  var
    i: integer;
    Acc1, Acc2: integer;
  begin
    Result := -1;
    if not Ilib.HasViewer then
      Exit;
    if XPos <= 0 then
    begin
      Result := 0;
      Exit;
    end;
    Acc1 := 0;
    Acc2 := 0;
    for i := 0 to Count - 1 do
    begin
      Acc2 := Acc2 + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
      if (XPos >= Acc1) and (XPos < Acc2) then
      begin
        Result := i;
        Break;
      end;
      Acc1 := Acc1 + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
    end;
    if Result > Ilib.Columns.TotalWidth then
      Result := Count - 1;
  end;

  function TILColumns.ColumnHeaderAtXY(X, Y: integer): integer;

  { Returns the column header at XY }
  var
    R: TRect;
    i: integer;
    W: integer;
    P: integer;
  begin
    Result := -1;
    if not Ilib.HasViewer then
      Exit;
    R.Top := Ilib.FViewer.GetTopMargin;
    R.Bottom := Ilib.FViewer.ReportLineHeight + Ilib.FViewer.GetTopMargin;
    R.Left := Ilib.FViewer.GetRptLeftMargin;
    W := Ilib.FViewer.GetRptLeftMargin;
    P := ColumnPos(Ilib.FViewer.FLeftColumn, false);
    X := P + X;
    for i := 0 to Count - 1 do
    begin
      W := W + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
      R.Right := W;
      if PtInRect(R, Point(X, Y)) then
      begin
        Result := i;
        Break;
      end;
      R.Left := W;
    end;
  end;

  function TILColumns.RectFromLeftColumn(const YPos: integer): TRect;

  { Returns the rectangel of the columns that starts with the left column.
    For the absolute rectangle see the TReportRow ItemRect }
  var
    i: integer;
    Acc: integer;
  begin
    Result.TopLeft := Point(0,0);
    Result.BottomRight := Point(0,0);
    if not Ilib.HasViewer then
      Exit;
    Result.Left := Ilib.FViewer.GetRptLeftMargin;
    Result.Top := YPos - 2;
    Result.Bottom := YPos + Ilib.FViewer.ReportLineHeight - 1;
    Acc := Ilib.FViewer.GetRptLeftMargin;
    for i := Ilib.FViewer.FLeftColumn to Count - 1 do
    begin
      Acc := Acc + Items[i].ColumnWidth + Ilib.FViewer.FColumnSpacing;
    end;
    Result.Right := Acc;
  end;

  procedure TILColumns.SetSortOrdering(const Value : TSortOrdering);
  begin
    if Value = FSortOrdering then
      Exit;
    FSortOrdering := Value;
    FSortChanged := True;
  end;

  procedure TILColumns.Chng(Sender: TObject);
  var
    i: integer;
    ErrIndx: integer;
    S: string;
  begin
    ErrIndx := -1;
    for i := 0 to FSortFields.Count - 1 do
    begin
      S := FSortFields[i];
      if Ilib.Fields.IndexOfCaption(S) = -1 then
      begin
        ErrIndx := i;
        Break;
      end;
    end;
    if ErrIndx <> -1 then
    begin
      FSortFields.Delete(ErrIndx);
    end;
    FSortChanged := True;
  end;

  { TILReplicatorItem -------------------------------------------------------------- }
  procedure TILReplicatorItem.CreateFields;

  { Ctreats the fields from the table string }
  var
    P, P2: integer;
    F, F2: integer;
    S: string;
    s2: string;
    FD: TILFieldDef;
    L : TLookupTbls;
    sl : TStringList;
  begin
    P := Pos('<Field>', MetaString);
    P2 := Pos('</Field>', MetaString);
    Inc(P2, Length('</Field>'));
    S := Copy(MetaString, P, P2 - P);

    if S = '' then
      Exit;

    Table.FFields.Clear;
    repeat
      F := Pos('<FieldName>', S);
      F2 := Pos('</FieldName>', S);
      Inc(F, Length('<FieldName>'));
      s2 := Copy(S, F, F2 - F);

      if s2 = '' then
        Break;

      FD := Table.FFields.Add;
      FD.FFieldName := s2;

      F := Pos('<FieldID>', S);
      F2 := Pos('</FieldID>', S);
      Inc(F, Length('<FieldID>'));
      s2 := Copy(S, F, F2 - F);
      if s2 = '' then
        s2 := IntToStr(FD.FFields.IndexOf(FD.FieldName));
      FD.FFieldID := StrToInt(s2);

      F := Pos('<FieldType>', S);
      F2 := Pos('</FieldType>', S);
      Inc(F, Length('<FieldType>'));
      s2 := Copy(S, F, F2 - F);
      FD.FFieldType := TFieldType(StrToInt(s2));

      F := Pos('<DataType>', S);
      F2 := Pos('</DataType>', S);
      Inc(F, Length('<DataType>'));
      s2 := Copy(S, F, F2 - F);
      FD.FDataType := TDataType(StrToInt(s2));

      F := Pos('<DateFormat>', S);
      F2 := Pos('</DateFormat>', S);
      Inc(F, Length('<DateFormat>'));
      s2 := Copy(S, F, F2 - F);
      FD.FDateFormat := s2;

      F := Pos('<Alignment>', S);
      F2 := Pos('</Alignment>', S);
      Inc(F, Length('<Alignment>'));
      s2 := Copy(S, F, F2 - F);
      if S2 = '' then
        FD.FAlignment := taLeftJustify
      else
        FD.FAlignment := TAlignment(StrToInt(s2));

      F := Pos('<Decimals>', S);
      F2 := Pos('</Decimals>', S);
      Inc(F, Length('<Decimals>'));
      s2 := Copy(S, F, F2 - F);
      FD.FDecimals := StrToInt(s2);

      F := Pos('<Identifier>', S);
      F2 := Pos('</Identifier>', S);
      Inc(F, Length('<Identifier>'));
      s2 := Copy(S, F, F2 - F);
      FD.FIdentifier := Boolean(StrToInt(s2));

      F := Pos('<Indexed>', S);
      F2 := Pos('</Indexed>', S);
      Inc(F, Length('<Indexed>'));
      s2 := Copy(S, F, F2 - F);
      FD.FIndexed := Boolean(StrToInt(s2));

      F := Pos('<IndexPriority>', S);
      F2 := Pos('</IndexPriority>', S);
      Inc(F, Length('<IndexPriority>'));
      s2 := Copy(S, F, F2 - F);
      FD.FIndexPriority := TIndexPriority(StrToInt(s2));

      F := Pos('<LookupType>', S);
      F2 := Pos('</LookupType>', S);
      Inc(F, Length('<LookupType>'));
      s2 := Copy(S, F, F2 - F);
      FD.FLookupType := TLookupType(StrToInt(s2));

      F := Pos('<LookupField>', S);
      F2 := Pos('</LookupField>', S);
      Inc(F, Length('<LookupField>'));
      s2 := Copy(S, F, F2 - F);
      FD.FLookupField := s2;

      F := Pos('<LookupSearchField>', S);
      F2 := Pos('</LookupSearchField>', S);
      Inc(F, Length('<LookupSearchField>'));
      s2 := Copy(S, F, F2 - F);
      FD.FLookupSearchField := s2;

      F := Pos('<LookupTable>', S);
      F2 := Pos('</LookupTable>', S);
      Inc(F, Length('<LookupTable>'));
      s2 := Copy(S, F, F2 - F);
      if s2 <> '' then
      begin
        SetLength(FLookupTbls, Length(FLookupTbls) + 1);
        L.TableName := s2;
        L.FieldName := FD.FieldName;
        FLookupTbls[High(FLookupTbls)] := L;
      end;

      F := Pos('<InitialValue>', S);
      F2 := Pos('</InitialValue>', S);
      Inc(F, Length('<InitialValue>'));
      s2 := Copy(S, F, F2 - F);
      FD.FInitialValue := s2;

      F := Pos('<Mandatory>', S);
      F2 := Pos('</Mandatory>', S);
      Inc(F, Length('<Mandatory>'));
      s2 := Copy(S, F, F2 - F);
      FD.FMandatory := Boolean(StrToInt(s2));

      F := Pos('<MaxLength>', S);
      F2 := Pos('</MaxLength>', S);
      Inc(F, Length('<MaxLength>'));
      s2 := Copy(S, F, F2 - F);
      FD.FMaxLength := StrToInt(s2);

      F := Pos('<MaxValues>', S);
      F2 := Pos('</MaxValues>', S);
      Inc(F, Length('<MaxValues>'));
      s2 := Copy(S, F, F2 - F);
      FD.FMaxValues := StrToInt(s2);

      F := Pos('<PickValues>', S);
      if F <> 0 then
      begin
        sl := TStringList.Create;
        try
          F2 := Pos('</PickValues>', S);
          Inc(F, Length('<PickValues>'));
          s2 := Copy(S, F, F2 - F);
          CreateListFromDelimiter('~', s2, sl);
          FD.PickValues.Assign(sl);
        finally
          sl.Free;
        end;
      end;

      F := Pos('<ReportField>', S);
      F2 := Pos('</ReportField>', S);
      Inc(F, Length('<ReportField>'));
      s2 := Copy(S, F, F2 - F);
      FD.FReportField := Boolean(StrToInt(s2));

      F := Pos('<ReportCaption>', S);
      F2 := Pos('</ReportCaption>', S);
      Inc(F, Length('<ReportCaption>'));
      s2 := Copy(S, F, F2 - F);
      FD.FReportCaption := s2;

      FD.FColumnWidth := FOwner.FOwner.DefaultColWidth(FD);

      F := Pos('<Unique>', S);
      F2 := Pos('</Unique>', S);
      Inc(F, Length('<Unique>'));
      s2 := Copy(S, F, F2 - F);
      FD.FUnique := Boolean(StrToInt(s2));

      F := Pos('<LTPageSize>', S);
      F2 := Pos('</LTPageSize>', S);
      Inc(F, Length('<LTPageSize>'));
      s2 := Copy(S, F, F2 - F);
      if s2 = '' then
        FD.FLTPageSize := ltp4KB
      else
       FD.FLTPageSize := TLTPageSize(StrToInt(s2));

      P := Pos('<Field>', MetaString, P + 1);
      P2 := Pos('</Field>', MetaString, P2 + 1);
      Inc(P2, Length('</Field>'));
      S := Copy(MetaString, P, P2 - P);
    until P = 0;
  end;

  procedure TILReplicatorItem.Load;
  begin
    CreateFields;
    if not Assigned(Table.FColumns) then
      Table.FColumns := TILColumns.Create(Table, Table.FFields)
    else
      Table.Columns.AssignSource;
  end;

  procedure TILReplicatorItem.Link;
  var
    i, j : integer;
  begin
    {Attatch lookup tables }
    for I := 0 to Table.Fields.Count-1 do
    begin
       for j  := 0 to High(FLookupTbls) do
       begin
         if SameText(FLookupTbls[j].FieldName, Table.Fields.Items[i].FieldName) then
         begin
           Table.Fields.Items[i].FLookupTable := FOwner.TableOf(FLookupTbls[j].TableName);
           if Table.Fields.Items[i].FLookupTable is TILLookupTable then
           begin
            SetLength(FOwner.FOwner.FFreeTables, Length(FOwner.FOwner.FFreeTables) + 1);
            FOwner.FOwner.FFreeTables[High(FOwner.FOwner.FFreeTables)] := Table.Fields.Items[i].FLookupTable;
           end;
         end;
       end;
    end;
  end;

  { TILReplicator ------------------------------------------------------------}

  function TILReplicator.GetCount: integer;
  begin
    Result := FTableList.Count;
  end;

  function TILReplicator.GetItems(Index: integer): TILReplicatorItem;
  begin
    Result := TILReplicatorItem(FTableList[Index]);
  end;

  procedure TILReplicator.Replicate;
  var
    S, S2: string;
    Indx: integer;
    MTbl: TILReplicatorItem;
    P, P2: integer;
    Mt : TILReplicatorItem;
    i : integer;
    TblName : string;

  function GetTableString(var Start: integer): string;

  var
    P, P2: integer;
  begin
    P := Pos('<Table>', FMetaString, Start);
    if P = 0 then
    begin
      Result := '';
      Exit;
    end;
    P2 := Pos('</Table>', FMetaString, Start);
    Inc(P2, Length('</Table>'));
    Result := Copy(FMetaString, P, P2 - P);
    Start := P2;
  end;

  begin
    Indx := 1;
    S := GetTableString(Indx);
    if S = '' then
      HandleILException(E_InvalidMetaFile);
    Importing := True;

    FOwner.FLocation := ExtractFileDir(FMetaFileName);

    try
    while S <> '' do
    begin
      FDBName := '';
      FRootTable := '';

      if FTableName <> '' then
      begin
       if FOwner.TableOf(FTableName) <> nil then
         Exit;
       P := Pos('<TableName>', S);
       P2 := Pos('</TableName>', S);
       Inc(P, Length('<TableName>'));
       if not SameText(Copy(S, P, P2 - P), FTableName) then
       begin
         S := GetTableString(Indx);
         Continue;
       end;
      end
      else
      begin
        P := Pos('<TableName>', S);
        P2 := Pos('</TableName>', S);
        Inc(P, Length('<TableName>'));
        TblName := Copy(S, P, P2 - P);
        if FOwner.TableOf(TblName) <> nil then
        begin
         S := GetTableString(Indx);
         Continue;
        end;
      end;

      MTbl := TILReplicatorItem.Create;
      MTbl.FOwner := Self;
      MTbl.FMetaString := S;
      if FTableName <> '' then
       MTbl.FTableName := FTableName
      else
       MTbl.FTableName := TblName;

      P := Pos('<ErrorState>', MTbl.FMetaString);
      P2 := Pos('</ErrorState>', MTbl.FMetaString);
      Inc(P, Length('<ErrorState>'));
      MTbl.FErrorState := Copy(MTbl.FMetaString, P, P2 - P);
      if MTbl.FErrorState = '' then
        MTbl.FErrorState := '0';

      P := Pos('<MasterSource>', MTbl.FMetaString);
      P2 := Pos('</MasterSource>', MTbl.FMetaString);
      Inc(P, Length('<MasterSource>'));
      MTbl.FMasterSource := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<TableSpace>', MTbl.FMetaString);
      P2 := Pos('</TableSpace>', MTbl.FMetaString);
      Inc(P, Length('<TableSpace>'));
      MTbl.FTableSpace := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<RecordFreeSpace>', MTbl.FMetaString);
      P2 := Pos('</RecordFreeSpace>', MTbl.FMetaString);
      Inc(P, Length('<RecordFreeSpace>'));
      MTbl.FRecordFreeSpace := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<RecordSize>', MTbl.FMetaString);
      P2 := Pos('</RecordSize>', MTbl.FMetaString);
      Inc(P, Length('<RecordSize>'));
      MTbl.FRecordSize := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<Relations>', MTbl.FMetaString);
      P2 := Pos('</Relations>', MTbl.FMetaString);
      Inc(P, Length('<Relations>'));
      MTbl.FRelations := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<LocalImageSize>', MTbl.FMetaString);
      P2 := Pos('</LocalImageSize>', MTbl.FMetaString);
      Inc(P, Length('<LocalImageSize>'));
      MTbl.FLocalImageSize := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<DateSearchFormat>', MTbl.FMetaString);
      P2 := Pos('</DateSearchFormat>', MTbl.FMetaString);
      Inc(P, Length('<DateSearchFormat>'));
      S := Copy(MTbl.FMetaString, P, P2 - P);
      MTbl.FDateSearchFormat := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<ParentFields>', MTbl.FMetaString);
      P2 := Pos('</ParentFields>', MTbl.FMetaString);
      Inc(P, Length('<ParentFields>'));
      S2 := Copy(MTbl.FMetaString, P, P2 - P);
      MTbl.FParentFields := (S2 = '1');

      P := Pos('<Shared>', MTbl.FMetaString);
      P2 := Pos('</Shared>', MTbl.FMetaString);
      Inc(P, Length('<Shared>'));
      S2 := Copy(MTbl.FMetaString, P, P2 - P);
      MTbl.FShared := (S2 = '1');

      P := Pos('<RecordLayout>', MTbl.FMetaString);
      P2 := Pos('</RecordLayout>', MTbl.FMetaString);
      Inc(P, Length('<RecordLayout>'));
      MTbl.FRecordLayout := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<TableClass>', MTbl.FMetaString);
      P2 := Pos('</TableClass>', MTbl.FMetaString);
      Inc(P, Length('<TableClass>'));
      MTbl.FTableClass := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<Alias>', MTbl.FMetaString);
      P2 := Pos('</Alias>', MTbl.FMetaString);
      Inc(P, Length('<Alias>'));
      MTbl.FAlias := Copy(MTbl.FMetaString, P, P2 - P);

      P := Pos('<FreeTable>', MTbl.FMetaString);
      P2 := Pos('</FreeTable>', MTbl.FMetaString);
      Inc(P, Length('<FreeTable>'));
      S2 := Copy(MTbl.FMetaString, P, P2 - P);
      MTbl.FFreeTable := (S2 = '1');

      P := Pos('<IsRoot>', MTbl.FMetaString);
      P2 := Pos('</IsRoot>', MTbl.FMetaString);
      Inc(P, Length('<IsRoot>'));
      S2 := Copy(MTbl.FMetaString, P, P2 - P);
      MTbl.FIsRoot := (S2 = '1');
      if MTbl.FIsRoot then
        FRootTable := FTableName;

      P := Pos('<FirstRecordID>', MTbl.FMetaString);
      P2 := Pos('</FirstRecordID>', MTbl.FMetaString);
      Inc(P, Length('<FirstRecordID>'));
      S2 := Copy(MTbl.FMetaString, P, P2 - P);
      if S2 = '' then
        MTbl.FFirstRecordID := 1
      else
        MTbl.FFirstRecordID := StrToInt(S2);

      if SameText(MTbl.FTableClass, 'TILDataTable') then
        MTbl.FTable := TILDataTable.Create(FOwner.Owner)
      else if SameText(MTbl.FTableClass, 'TILLookupTable') then
        MTbl.FTable := TILLookupTable.Create(FOwner.Owner)
      else if SameText(MTbl.FTableClass, 'TILImageTable') then
        MTbl.FTable := TILImageTable.Create(FOwner.Owner)
      else if SameText(MTbl.FTableClass, 'TILTreeViewTable') then
        MTbl.FTable := TILTreeViewTable.Create(FOwner.Owner)
      else
      begin
        Comment1 := MTbl.FTableClass;
        HandleILException(E_UnknownTableClass);
      end;
      MTbl.Table.Name := 'TBL_' + MTbl.TableName;
      MTbl.Table.FTableName := MTbl.TableName;
      MTbl.Table.FAlias := MTbl.FAlias;
      MTbl.Table.FTableSpace := StrToInt(MTbl.FTableSpace);
      MTbl.Table.ErrorState := StrToInt(MTbl.FErrorState);
      MTbl.Table.FParentFields := MTbl.FParentFields;
      MTbl.Table.FFirstRecordID := MTbl.FFirstRecordID;
      if MTbl.FRecordFreeSpace = '' then
       MTbl.Table.FRecordFreeSpace := 0
      else
       MTbl.Table.FRecordFreeSpace := StrToInt(MTbl.FRecordFreeSpace);
      if MTbl.FRecordSize = '' then
        MTbl.Table.FRecordSize := 0
      else
        MTbl.Table.FRecordSize := StrToInt(MTbl.FRecordSize);

      if MTbl.FLocalImageSize = '' then
        MTbl.Table.FImageLocalSize := ils200
      else
        MTbl.Table.FImageLocalSize := TImageLocalSize(StrToInt(MTbl.FLocalImageSize));

      if MTbl.FRelations = '' then
        MTbl.Table.FRelations := relMany
      else
        MTbl.Table.FRelations := TRelations(StrToInt(MTbl.FRelations));

      if MTbl.FDateSearchFormat = '' then
        MTbl.Table.FDateSearchFormat := dsfDate
      else
        MTbl.Table.FDateSearchFormat := TDateSearchFormat(StrToInt(MTbl.FDateSearchFormat));

      if MTbl.FRecordLayout = '' then
        MTbl.Table.FRecordLayout := rlVariableSize
      else
        MTbl.Table.FRecordLayout := TRecordLayout(StrToInt(MTbl.FRecordLayout));
      if MTbl.FIsRoot then
       FOwner.FRootTable := MTbl.Table as TILDataTable;

      if MTbl.FShared then
        FOwner.FSharedImages := Mtbl.Table as TILImageTable;

      Add(MTbl);
      if FTableName = '' then
        S := GetTableString(Indx)
      else
        Break;
    end;

    P := Pos('<DBName>', FMetaString);
    P2 := Pos('</DBName>', FMetaString);
    Inc(P, Length('<DBName>'));
    FDBName := Copy(FMetaString, P, P2 - P);

    P := Pos('<Encoding>', FMetaString);
    P2 := Pos('</Encoding>', FMetaString);
    Inc(P, Length('<Encoding>'));
    FUnicode := Copy(FMetaString, P, P2 - P);

    P := Pos('<TransHistory>', FMetaString);
    P2 := Pos('</TransHistory>', FMetaString);
    Inc(P, Length('<TransHistory>'));
    S := Copy(FMetaString, P, P2 - P);
    FTransHistory := (S = '1');

    FOwner.FDatabase := FDBName;
    if FUnicode <> '' then
    FOwner.FUnicode := StrToInt(FUnicode) = 0
    else
      FOwner.Unicode := True;

    if FRootTable <> '' then
    begin
      FOwner.FRootTable := TableOf(FRootTable) as TILDataTable;
    end;

    FOwner.FTransHistory := FTransHistory;

    { Create table chain; }
    for I := 0 to Count-1 do
    begin
       Mt := Items[i];
       if Mt.FMasterSource <> '' then
       begin
         Mt.Table.FMasterSource := TILDataTable(TableOf(Mt.FMasterSource));
         if Mt.Table.FMasterSource = nil then
          Mt.Table.FMasterSource := TILDataTable(TMan.TableOf(Mt.FMasterSource));
       end;
    end;

    FOwner.CreateNodes;

    { Create the fields }
    for i := 0 to Count - 1 do
    begin
      Mt := Items[i];
      Mt.Load;
      if Mt.Table is TILLookupTable then
      with Mt.Table as TILLookupTable do
        if Mt.Table.Fields.Count > 0 then
       FFieldName := Mt.Table.Fields.Items[0].FFieldName;
    end;

    { Link Lookup tables}
    for i := 0 to Count - 1 do
    begin
      Mt := Items[i];
      Mt.Link;
      Fowner.AddInheritedFields(Mt.Table);
    end;

    { Check free tables with no references }
    for i := 0 to Count - 1 do
    begin
      Mt := Items[i];
      if Mt.FFreeTable then
      begin
        FOwner.RegisterFreeTable(Mt.Table);
      end;
    end;

    finally
      Importing := false;
    end;

  end;

  procedure TILReplicator.Replicate(const TableName : string);
  begin
     FTableName := TableName;
     Replicate;
  end;


  constructor TILReplicator.Create(const AMetaFileName: string; TableManager: TTableManager);
  { Extracts the table strings and creates table info }
  var
    P: integer;
  begin
    if not FileExists(AMetaFileName) then
    begin
      Comment1 := AMetaFileName;
      HandleILException(E_FileNotExists);
    end;
    FMetaFileName := AMetaFileName;
    FExtDesigner := TableManager.ExtDesigner;
    FOwner := TableManager;
    FTableList := TList.Create;
    FXMLDoc := TXMLDocument.Create(FOwner);
    try
      FXMLDoc.Active := True;
      FXMLDoc.LoadFromFile(FMetaFileName);
      FMetaString := FXMLDoc.DocumentElement.Xml;
    finally
      FXMLDoc.Free;
    end;
    P := Pos('<Ilb>', FMetaString);
    if P = 0 then
      HandleILException(E_InvalidMetafile, AMetafileName);
  end;

  constructor TILReplicator.Create(const AMetaFileName: string; TableManager : TTableManager;
   const ExtDesigner : Boolean);
  begin
    FExtDesigner := True;
    Create(AMetaFileName, TMan);
  end;

  destructor TILReplicator.Destroy;
  var
    i: integer;
  begin
    for i := 0 to Count - 1 do
      TILReplicatorItem(Items[i]).Free;
    FTableList.Free;
  end;

  procedure TILReplicator.Add(ATable: TILReplicatorItem);
  begin
    FTableList.Add(ATable);
  end;

  function TILReplicator.TableOf(const TableName : string) : TILTable;
  var
  I: Integer;
  begin
    Result := nil;
    for I := 0 to Count-1 do
    begin
      if SameText(Items[i].TableName, TableName) then
      begin
        Result := Items[i].Table;
        Break;
      end;
    end;
  end;

  { TILSQLParams -------------------------------------------------------}

 function TILSQLParams.GetCount : integer;
 begin
   Result := FItems.Count;
 end;

 function TILSQLParams.GetItems(Index : integer) : TILSQLParam;
 begin
    Result := TILSQLParam(FItems[Index]);
 end;

 function TILSQLParams.Add : TILSQLParam;
 begin
     Result := TILSQLParam.Create;
     Result.FParams := Self;
     FItems.Add(Result);
 end;

 constructor TILSQLParams.Create;
 begin
    FItems := TList.Create;
    Inherited;
 end;

 destructor TILSQLParams.Destroy;
 begin
    inherited;
    Clear;
    FItems.Free;
 end;

 procedure TILSQLParams.Clear;
  var
    i : integer;
 begin
    for I := 0 to FItems.Count-1 do
    begin
      TILSQLParam(FItems[i]).Free;
    end;
    FItems.Clear;
 end;

procedure TILSQLParams.ClearValues;
var
  i : integer;
begin
  for I := 0 to Count-1 do
  begin
    Items[i].ParamValue := EmptyValue;
    Items[i].CndOperator := '';
  end;

end;

function TILSQLParams.ParamByName(const AName : string) : TILSQLParam;
var
  i : integer;
begin
    Result := nil;
    for I := 0 to Count-1 do
      if SameText(AName, Items[i].FParamName) then
      begin
        Result := Items[i];
        Break;
      end;
end;

{ TILSQLParam --------------------------------------------------------------}

function TILSQLParam.GetFieldDef : TILFieldDef;
var
  Tbl : TILTable;
  Indx : integer;
begin
   Result := nil;
   if FTableName = '' then
     Exit;
   if FFieldName = '' then
     Exit;
   TBL := TMan.TableOf(FTableName);
   if TBL = nil then
     Exit;
   Indx := TBL.Fields.IndexOf(Fieldname);
   if Indx <> -1 then
     Result := TBL.Fields.Items[Indx];
end;

procedure TILSQLParam.SetParamValue(const Value : Variant);
var
  P : integer;
  Val : Variant;
begin
   Val := Value;
   if VarIsStr(Value) then
   begin
      if (Value <> EmptyValue) and not TMan.IsValueNull(Value) then
      begin
       P := Pos('''', Value);
       if P = 0 then
         Val := '''' + Value + '''';
       end;
   end;
   FParamValue := Val;
end;

procedure TILSQLParam.SetCndOperator(const Value : string);
var
  Val : string;
begin
  Val := Trim(Value);
  if Val = FCndOperator then
    Exit;
  if Val = '' then
  begin
    FCndOperator := '';
    Exit;
  end;
  if (Value <> '>')
  and (Value <> '<')
  and (Value <> '>=')
  and (Value <> '<=')
  and (Value <> '==')
  and (Value <> '<>')
  and (Value <> '=')
  then
    HandleILException(E_InvalidCndOp);
  FCndOperator := Val;

end;

{ TILSQL-------------------------------------------------------------}

 procedure TILSQL.SetSQLStatement(const Value : TStringlist);
 begin
   TrimList(FSQLStatement, False);
   FSQLStatement.Assign(Value);
   FValuedSQL.Assign(FSQLStatement);
   if TargetTable = nil then
     HandleILException(E_MissingTableName);
   InitParams;
 end;

 procedure TILSQL.InitParams;
 begin
   if (csDesigning in Componentstate) and not FNoCheck  then
   begin
     TMan.ConnectSilent;
   end;
   try
    ParseStatement;
   finally
    if (csDesigning in Componentstate) and not FNoCheck then
    begin
     TMan.Connected := False;
    end;
   end;
 end;

 function TILSQL.ParseStatement : integer;
 var
   P, P1, P2 : integer;
   S : string;
   TName, PName : string;
   Parm : TILSQLParam;
   TempStr : TStringList;

 begin
    Result := 0;
    Params.Clear;
    TempStr := TStringList.Create;
    try
    TempStr.Assign(FSQLStatement);
    RemoveComments(TempStr);
    S := TempStr.Text;
    P := PosRestrict(':', S, False);
    while P <> 0 do
    begin
      TName := TargetTable.TableName;
      P1 := P + 1;
      P2 := P1;
      while (P2 < Length(S)) and (CharInSet(S[P2], ['_', 'a'..'z', 'A'..'Z', '0'..'9'])) do
        inc(P2);
      { Now at the end of paramname }
      PName := Copy(S, P1, P2-P1);
      if PName <> '' then
      begin
        Parm := Params.Add;
        Parm.FFieldName := '';
        Parm.FTableName := TName;
        Parm.FParamName := PName;
        Parm.ParamValue := EmptyValue;
      end;
      P := PosRestrict(':', S, False, P2);
    end;
    if not FNoCheck then
    begin
     Result := CheckStatement;
     if (Result = -1) and (csDesigning in ComponentState) then
       Result := 0;
     if Result <> 0 then
       HandleILException(Result);
     end;
    finally
      TempStr.Free;
    end;
 end;

 function TILSQL.CheckStatement : integer;
 var
   i : integer;
   Parm : TILSQLParam;
 begin
   Result := 0;
   if SQLType = ctInsert then
     Exit;
   for I := 0 to Params.Count-1 do
   begin
      Parm := Params.Items[i];
      Parm.ParamValue := EmptyValue;
   end;
   SetValues;
   Result := TargetTable.DBCommons.CanExecSQL(FValuedSQL);
   Params.ClearValues;
   FValuedSQL.Assign(FSQLStatement);
 end;

 function TILSQL.GetSQLType : TSQLType;
 begin
   Result := TMan.CommandFromSQL(SQLStateMent);
 end;

 function TILSQL.StringsToVarArray(Str : TStrings) : Variant;
 var
   i : integer;
 begin
   Result := VarArrayCreate([0, Str.Count-1], varOleStr);
   for I := 0 to Str.Count-1 do
     Result[i] := '''' + Str[i] + '''';
 end;

 procedure TILSQL.SetValues;
 var
   i, j : integer;
   Prm : TILSQLParam;
   s, s2 : string;
   TempStr : TStringList;
   P, P2 : integer;
   Cnt, DimCnt : integer;
   Indx : integer;

   procedure CheckValueField;
   var
     P : integer;
     F, T : string;
     Tbl : TILTable;
   begin
     F := '';
     T := '';
     Prm.FValueTable := nil;
     Prm.FValueField := nil;
     if not FIsFilter then
       Exit;
     P := Pos('.', Prm.ParamName);
     if P <> 0 then
     begin
       F := Copy(Prm.ParamName, 1, P-1);
       T := Copy(Prm.ParamName, P+1, Length(Prm.ParamName));
     end
     else
     begin
       T := TargetTable.TableName;
       F := Prm.ParamName;
     end;
     Tbl := TMan.TableOf(T);
     if Tbl = nil then
      Exit;
     Indx := Tbl.Fields.IndexOf(F);
     if Indx = -1 then
       Exit;
     if not TMan.IsInSearchPath(TargetTable,Tbl) then
       HandleILException(E_TableOutOfPath, Tbl.TableName);
     if TargetTable.MasterLinkCount = 0 then
     begin
       if (Tbl.MasterLinkCount > 0) or (TMan.NodeOf(Tbl).Level >  TMan.NodeOf(Tbl).Level) then
         HandleILException(E_NotAParentTable, Tbl.TableName);
     end;

     Prm.FValueTable := Tbl;
     Prm.FValueField := Tbl.Fields.Items[Indx];
   end;

 begin
   TempStr := TStringList.Create;
   try
   TempStr.Assign(FSQLStatement);
   RemoveComments(TempStr);
   FValuedSQL.Assign(TempStr);
   s := FValuedSQL.Text;
   for I := 0 to Params.Count -1 do
   begin
     Prm := Params.Items[i];
     if Prm.CndOperator <> '' then
     begin
       P := PosRestrict(':' + Prm.ParamName, S, False);
       while (P > 0) and not CharInSet(S[P], ['>', '<', '='])  do
        dec(P);
       if P <> 0 then
       begin
        P2 := P;
        while (P2 > 0) and CharInSet(S[P2], ['>', '<', '=']) do
         dec(P2);
       end
       else
         P2 := 0;
       System.Delete(S, P2+1, P-P2);
       System.Insert(Prm.CndOperator, S, P);
     end;

     CheckValueField;
     if Prm.FValueField <> nil then
     begin
       if Prm.FValueTable.FocusedRecord = nil then
         Prm.ParamValue := EmptyValue
       else if Prm.FValueTable.FocusedRecord[Prm.FValueField.FieldName].IsEmpty then
         Prm.ParamValue := EmptyValue
       else
       begin
         case Prm.FValueField.DataType of
           dtString, dtText, dtDate, dtResource, dtLongText:
            Prm.ParamValue := Prm.FValueTable.FocusedRecord[Prm.FValueField.FieldName].AsString;
           dtBoolean: Prm.ParamValue := Prm.FValueTable.FocusedRecord[Prm.FValueField.FieldName].AsBoolean;
           dtNumber: Prm.ParamValue := Prm.FValueTable.FocusedRecord[Prm.FValueField.FieldName].AsNumber;
         end;

       end;
     end;


     if VarIsArray(Prm.ParamValue) then
     begin
         s2 := '';
         DimCnt := VarArrayDimCount(Prm.ParamValue);
         Cnt := VarArrayHighBound(Prm.ParamValue, DimCnt);
         for j := 0 to Cnt do
         begin
            s2 := s2 + Prm.ParamValue[j];
            if j < Cnt then
             s2 := s2 + ', ';
         end;
         S := StringReplace(S, ':' + Prm.ParamName, s2, []);
     end
     else if VarIsEmpty(Prm.ParamValue) then
       S := StringReplace(S, ':' + Prm.ParamName, EmptyValue, [])
     else
       S := StringReplace(S, ':' + Prm.ParamName, Prm.ParamValue, []);
   end;
   FValuedSQL.Text := S;
   finally
     TempStr.Free;
   end;
 end;

function TILSQL.Execute : integer;
var
  i : integer;
  AllEmpty : Boolean;
begin
   if TargetTable = nil then
     HandleILException(E_InvalidTableName);
   AllEmpty := True;
   for I := 0 to Params.Count-1 do
   begin
       if VarIsArray(Params[i].ParamValue) then
       begin
         AllEmpty := false;
       end
       else if VarToStr(Params[i].ParamValue) <> EmptyValue then
       begin
         AllEmpty := false;
       end;
   end;
   if AllEmpty then
     HandleILException(-1);
   SetValues;
   Tman.ActiveTable := TargetTable;
   Result := TargetTable.ExecSQL(FValuedSQL);
   FValuedSQL.Assign(FSQLStatement);
end;

constructor TILSQL.Create;
begin
    inherited;
    FSQLStatement := TStringList.Create;
    FValuedSQL := TStringList.Create;
    FParams := TILSQLParams.Create;
    FParams.FOwner := Self;
    RegSQLs.Add(Self);
end;

destructor TILSQL.Destroy;
var
  Indx : integer;
begin
   FParams.Free;
   FSQLStatement.Free;
   FValuedSQL.Free;
   Indx := RegSQLs.IndexOf(Self);
   RegSQLS.Delete(Indx);
   inherited;
end;

{ TILMessageHandler ------------------------------------------------------------}

constructor TILMessageHandler.Create;
begin
  RegMbh.Add(Self);
  FFilters := TList.Create;
  Reset;
end;

destructor TILMessageHandler.Destroy;
var
  i : integer;
  T : TmessageFilter;
begin
   for I := 0 to FFilters.Count-1 do
   begin
     T := TMessageFilter(FFilters[i]);
     T.Free;
   end;
   FFilters.Free;
   inherited;
end;

procedure TILMessageHandler.AddToFilter(AMessageType : TMessageType;
 AMessageKind : TMessageOption; AMessage : integer);
var
  F : TMessageFilter;
begin
  if InFilter(AMessageType, AMessageKind, AMessage) then
    Exit;
  F := TMessageFilter.Create;
  F.MessageType := AMessageType;
  F.MessageKind := AMessageKind;
  F.MesageID := AMessage;
  FFilters.Add(F);
end;

procedure TILMessageHandler.RemoveFromFilter(AMessageType : TMessageType;
  AMessageKind : TMessageOption; AMessage : integer);
var
  I: Integer;
  F : TMessageFilter;
  Found : Integer;
begin
  Found := -1;
  F := nil;
  for I := 0 to FFilters.Count-1 do
  begin
    F := TMessageFilter(FFilters[i]);
    if (F.MessageType = AMessageType) and (F.MessageKind = AMessageKind) then
    begin
      if AMessageKind in[moError, moReady] then
      begin
        if AMessage = F.MesageID then
        begin
          Found := I;
          Break;
        end;
      end
      else
      begin
        Found := I;
        Break;
      end;
    end;
  end;

  if Found <> -1 then
  begin
    F.Free;
    FFilters.Delete(Found);
  end;
end;

function TILMessageHandler.InFilter(AMessageType : TMessageType;
  AMessageKind : TMessageOption; AMessage : integer) : Boolean;
var
  I: Integer;
  F : TMessageFilter;
begin
  Result := false;
  for I := 0 to FFilters.Count-1 do
  begin
    F := TMessageFilter(FFilters[i]);
    if (F.MessageType = AMessageType) and (F.MessageKind = AMessageKind) then
    begin
      if AMessageKind in[moError, moReady] then
      begin
        if AMessage = F.MesageID then
        begin
          Result := True;
          Break;
        end;
      end
      else
      begin
        Result := True;
        Break;
      end;
    end;
  end;
end;

function TILMessageHandler.GetTable : TILTable;
begin
  if TMan = nil then
  begin
    Result := nil;
    Exit;
  end;
  if FTable <> nil then
    Result := FTable
  else
    Result := TMan.ActiveTable;
end;

function TILMessageHandler.GetEditCount : integer;
begin
  Result := Table.Dataset.ChangeCount;
end;

function TILMessageHandler.GetAffectedTables : TList;
begin
  Result := TMan.FAffectedTables;
end;

procedure TILMessageHandler.Reset;
begin
  FUpdateCount := -1;
  FSearchCount := -1;
  FDeleteCount := -1;
  FInsertCount := -1;
  FSaveCount := -1;

  FUpdateState := -1;
  FSearchState := -1;
  FInsertState := -1;
  FSaveState := -1;

  FLastOperation := dboNone;
  DatasetChanged(dboNone);
end;

procedure TILMessageHandler.AddCount(AnOperation : TDBOperation; ACount : integer; Increment : Boolean = True);
var
  Cnt : integer;
begin
   Target := utrDataset;
   if (AnOperation <> dboSave) and
   (LastOperation = dboSave) then
     Reset;

   FSearchCount := -1;
   case AnOperation of
    dboUpdate: begin
      if not InFilter(mtEdit, moConfirm, -1) then
      begin
        if ACount = -1 then
          Cnt := Table.Dataset.GetChangeCount
        else
          Cnt := ACount;
        if InsertCount > 0 then
          dec(Cnt, InsertCount);
        if DeleteCount > 0 then
          dec(Cnt, DeleteCount);
        if not SQLMode and (Cnt = 0) then
          Cnt := -1;
        FUpdateCount := Cnt;
      end;
     end;
    dboInsert: begin
      begin
        if (FInsertCount = -1) and Increment then inc(ACount);
        if Increment then
          inc(FInsertCount, ACount)
        else
          FInsertCount := ACount;
      end;
    end;
    dboDelete:
    begin
      if (FDeleteCount = -1) and Increment then inc(ACount);
      if Increment then
         inc(FDeleteCount, ACount)
       else
         FDeleteCount := ACount;
    end;
    dboSave:
    begin
     if not InFilter(mtSave, moConfirm, -1) then
     begin
       if (FSaveCount = -1) and Increment then inc(ACount);
       if Increment then
         inc(FSaveCount, ACount)
       else
         FSaveCount := ACount;
     end;
    end;
    dboSearch: begin
     if not InFilter(mtSearch, moConfirm, -1) then
     begin
       if (FSearchCount = -1) and Increment then inc(ACount);
       if Increment then
         inc(FSearchCount, ACount)
       else
         FSearchCount := ACount;
     end;
    end;
  end;
  LastOperation := AnOperation;
  DatasetChanged(AnOperation);
end;

procedure TILMessageHandler.SQLChanged(SQLType : TSQLType; AState : integer);
begin
    if (LastOperation = dboSave) and not (AState > 0) then
      Exit;

    FInsertState := -1;
    FUpdateState := -1;
    FInsertCount := -1;
    FUpdateCount := -1;
    FDeleteCount := -1;
    FSaveCount := -1;
    FUpdateState := -1;
    FSearchState := -1;
    LastOperation := dboNone;

    if SQLType = ctSelect then
    begin
      FSearchState := AState;
      FSearchComment := Comment1;
    end
    else
    begin
      FUpdateState := AState;
      FUpdateComment := Comment1;
    end;

    if Assigned(FOnChangeMessage) then
    begin
      FOnChangeMessage(Self)
    end;
end;

procedure TILMessageHandler.ControlsChanged(Usage: TDataUsage);
var
  S : string;
  Res : integer;
begin
  if Table = nil then
    Exit;
  if SQLMode then
    Exit;
  if (Table.Dataset.Count = 0) and (Usage = duData) then
    Exit;

  if LastOperation = dboSave then
    Reset;
  if Usage = duData then
  begin
    S := '';
    if FSaveState > 0 then
    begin
     FSaveComment := Comment1;
     S := Comment1;
    end;

    Res := Table.DBWork.CanUpdateData;
    if not ExcludeMessage(mtEdit, Res) then
    begin
      FUpdateComment := Comment1;
      FUpdateState := Res;
    end;
    if Comment1 = '' then
     Comment1 := S;

    if Assigned(FOnChangeMessage) then
      FOnChangeMessage(Self);
  end
  else
  begin
    FSearchState := Table.DBView.CanSearch;
    FSearchComment := Comment1;
    if Assigned(FOnChangeMessage) then
      FOnChangeMessage(Self);
  end;
end;

function TILMessageHandler.MessageKind(AMessageType : TMessageType) : TMessageOption;
begin
  Result := moNotAvailable;
  case AMessageType of
    mtDataset: Result := moConfirm;
    mtSearch:
    begin
      if (FSearchState > 0) then
        Result := moError
      else if FSearchCount > -1 then
        Result := moConfirm;
    end;
    mtEdit:
    begin
      if TMan.AutoSave and not (LastOperation = dboSave) then
       Result := moNotAvailable
      else
      if (UpdateState > 0) then
        Result := moError
      else if ((UpdateCount > -1) or (DeleteCount > -1) or (InsertCount > 0))
      and not (LastOperation = dboSave) and not InFilter(mtEdit, moConfirm,-1) then
        Result := moConfirm
      else if UpDateState = 0 then
        Result := moReady;
    end;
    mtSave:
    begin
      if (SaveState > 0) then
        Result := moError
      else if (SaveState = 0) and (SaveCount = -1) then
        Result := moReady
      else if (SaveCount > -1) and not InFilter(mtSave, moConfirm,-1) then
      begin
        Result := moConfirm;
      end;
    end;
    mtProgress: ;
  end;
end;

function TILMessageHandler.MessageText(AMessageType : TMessageType) : string;
var
  Handled : Boolean;
begin
  Result := '';

  if Assigned(FOnComposeMessage) then
  begin
    Handled := True;
    FOnComposeMessage(Self, AMessageType, Result, Handled);
    if Handled then
      Exit;
  end;

  case AMessageType of
    mtDataset:
    begin
      if Table.Dataset.Position = -1 then
        Result := Table.Alias + ': 0 ' + Translator.GetTranslation(T_MBRecords, TRANSL_TERMS)
      else
      begin
        Result := Table.Alias + ': ' + IntToStr(Table.Dataset.Position+1) +
        ' / ' + IntToStr(Table.Dataset.Count) +  ' ' +  Translator.GetTranslation(T_MBRecords, TRANSL_TERMS);
        if Table.Datahandler.MultiSelect then
          Result := Result + ' (' + IntToStr(Table.SelCount) + ' ' +
           Translator.GetTranslation(T_MBRecordsSelected, TRANSL_TERMS) + ')';
      end;
    end;
    mtSearch: Result := SearchMessage;
    mtEdit: Result := UpdateMessage;
    mtSave: Result := SaveMessage;
    mtProgress:;
  end;
end;

function TILMessageHandler.ConfirmMessage(AMessageType : TMessageType) : string;
var
 S : string;
begin
  Result := '';
  S := '';
  if UpdateCount > -1 then
    S := S + UpdateCountMessage + ', ';
  if DeleteCount > -1 then
    S := S + DeleteCountMessage + ', ';
  if InsertCount > -1 then
    S := S + InsertCountMessage;
  if Copy(S, Length(S)-1, 2) = ', ' then
    Delete(S, Length(S)-1, 2);
  Result := S;
end;

function TILMessageHandler.ValidationState(AMessageType : TMessageType) : integer;
begin
  Result := -1;
  case AMessageType of
    mtDataset: Result := 0;
    mtSearch: Result := SearchState;
    mtEdit: Result := UpdateState;
    mtSave: Result := SaveState;
    mtProgress: Result := -1;
  end;
end;

function TILMessageHandler.ConfirmCount(AMessageType : TMessageType) : integer;
begin
  Result := -1;
  case AMessageType of
    mtDataset: Result := Table.Dataset.Count;
    mtSearch: Result := SearchCount;
    mtEdit: Result := UpdateCount;
    mtSave: Result := SaveCount;
    mtProgress: ;
  end;
end;


function TILMessageHandler.ExcludeMessage(AMessagetype : TMessageType; AMessage : integer) : Boolean;
begin
    Result := false;
    if AMessage = 0 then
    begin
      if InFilter(AMessageType, moReady, AMessage) then
        Result := True
    end
    else if AMessage > 0 then
    begin
      if InFilter(AMessageType, moError, AMessage) then
        Result := True;
    end;
end;

procedure TILMessageHandler.DatasetChanged(Operation : TDBOperation);
var
  Err : integer;
  S : string;
  Res : integer;
begin
  if Table = nil then
    Exit;

  Res := Table.DBWork.CanSave(Err);
  if not ExcludeMessage(mtSave, Res) then
     FSaveState := Res;
  if FSaveState > 0 then
  begin
    FSaveComment := Comment1;
    S := Comment1; { Prevent comment to be ovrerwritten}
  end;
  if not SQLMode then
  begin
   Res := Table.DBWork.CanUpdateData;
   if not ExcludeMessage(mtEdit, Res) then
     FUpdateState := Res;
   if FUpdateState > 0 then
     FUpdateComment := Comment1;
   if FUpdateState = -1 then
    Comment1 := S;
  end;
  if Assigned(FOnChangeMessage) then
  begin
      FOnChangeMessage(Self)
  end;
end;

function TILMessageHandler.GetSearchCountMessage : string;
begin
   Result := '';
   if SearchCount > -1 then
   begin
     Result := IntToStr(SearchCount) + ' ' +
       Translator.GetTranslation(T_MBRecordsFound, TRANSL_TERMS);
   end;
end;

function TILMessageHandler.GetUpdateCountMessage : string;
var
  S : string;
begin
   Result := '';
   if UpdateCount > -1 then
   begin
     S := IntToStr(UpdateCount);
     Result := Translator.GetTranslation(T_MBRecordsChanged, TRANSL_TERMS);
     Result := Translator.FormatStr(Result, S, '', '');
   end;
end;

function TILMessageHandler.GetDeleteCountMessage : string;
var
  S : string;
begin
   Result := '';
   if DeleteCount > -1 then
   begin
     S := IntToStr(DeleteCount);
     Result := Translator.GetTranslation(T_MBRecordsRemoved, TRANSL_TERMS);
     Result := Translator.FormatStr(Result, S, '', '');
   end;
end;

function TILMessageHandler.GetInsertCountMessage : string;
var
  S : string;
begin
   Result := '';
   if InsertCount > -1 then
   begin
     S := IntToStr(InsertCount);
     Result := Translator.GetTranslation(T_MBRecordsAdded, TRANSL_TERMS);
     Result := Translator.FormatStr(Result, S, '', '');
   end;
end;

function TILMessageHandler.GetSearchStateMessage : string;
begin
   Result := '';
   if not TMan.Connected then
   begin
     Exit;
   end;
   if FSearchState > 0 then
   begin
     Comment1 := FSearchComment;
     Result := GetILErrorText(FSearchState);
   end
   else if FSearchState = 0 then
     Result := Translator.GetTranslation(T_MBReady, TRANSL_TERMS);
end;

function TILMessageHandler.GetUpdateStateMessage : string;
begin
   Result := '';
   if not TMan.Connected then
   begin
     Exit;
   end;
   if FUpdateState > 0 then
   begin
     Comment1 := FUpdateComment;
     Result := GetILErrorText(FUpdateState);
   end
   else if FUpdateState = 0 then
   begin
     Result := Translator.GetTranslation(T_MBReady, TRANSL_TERMS);
   end;
end;

function TILMessageHandler.GetSaveStateMessage : string;
begin
   Result := '';
   if not TMan.Connected then
   begin
     Exit;
   end;
   if FSaveState > 0 then
   begin
     Result := GetILErrorText(FSaveState);
     Comment1 := FSaveComment;
   end
   else if FSaveState = 0 then
     Result := Translator.GetTranslation(T_MBReady, TRANSL_TERMS);
end;


function TILMessageHandler.GetSearchMessage : string;
begin
   Result := '';
   if SearchState > 0 then
   begin
     Result := SearchStateMessage;
   end
   else if SearchCount > -1 then
     Result := SearchCountMessage;
end;

function TILMessageHandler.GetUpdateMessage : string;
begin
   Result := '';
   if (UpdateState > 0) then
     Result := UpdateStateMessage
   else if (MessageKind(mtEdit) = moConfirm) and not (LastOperation = dboSave) then
   begin
     Result := ConfirmMessage(mtEdit);
   end
   else if UpdateState = 0 then
      Result := UpdateStateMessage;
 end;

procedure TILMessageHandler.SetLastOperation(Value : TDBOperation);
begin
    if Value = FLastOperation then
     Exit;
    FLastOperation := Value;
end;

procedure TILMessageHandler.SetSQLMode(Value : Boolean);
begin
  if FSQLMode = Value then
    Exit;
  FSqlMode := Value;
  Reset;
  if Assigned(FOnChangeMessage) then
    FOnChangeMessage(Self);
end;

function TILMessageHandler.GetSaveMessage : string;
var
  S : string;
begin
   Result := '';
   S := '';
   if (FSaveState > 0) then
     Result := SaveStateMessage
   else if SaveCount > -1 then
   begin
    if TMan.FAffectedTables.Count > 1 then
    begin
        Result := Translator.GetTranslation(T_MBRecordsAffected, TRANSL_TERMS);
        Result := Translator.FormatStr(Result, IntToStr(SaveCount), IntToStr(TMan.FAffectedTables.Count), '');
        Exit;
    end
    else if TMan.FAffectedTables.Count > 0 then
    begin
      if TMan.FAffectedTables[0] <> Table then
        S := TILTable(TMan.FAffectedTables[0]).Alias + ': ';
    end
    else
       S := IntToStr(FSaveCount) + ' ' +
      Translator.GetTranslation(T_MBRecordsNoneAffected, TRANSL_TERMS);


    if UpdateCount > -1 then
      S := S + IntToStr(UpdateCount) + ' ' +
      Translator.GetTranslation(T_MBRecordsUpdated, TRANSL_TERMS) + ', ';
    if DeleteCount > -1 then
      S := S + IntToStr(DeleteCount) + ' ' +
      Translator.GetTranslation(T_MBRecordsDeleted, TRANSL_TERMS) + ', ';
    if InsertCount > -1 then
      S := S + IntToStr(InsertCount) + ' ' +
      Translator.GetTranslation(T_MBRecordsInserted, TRANSL_TERMS);
    if Copy(S, Length(S)-1, 2) = ', ' then
      Delete(S, Length(S)-1, 2);
    Result := S;
   end
   else
   begin
     if FSaveCount > 0 then
      Result := IntToStr(FSaveCount) + ' ' +
      Translator.GetTranslation(T_MBRecordsSaved, TRANSL_TERMS)
      else
       Result := SaveStateMessage;
   end;
end;

{ TILResDialog ----------------------------------------------------------}

  constructor TILResourceDialog.Create(AOwner : TComponent);
  var
    NC, AC : TLabel;
    BtnTop : integer;
  begin
    inherited;
    FResDialog := TForm.Create(AOwner);
    FResDialog.Position := poScreenCenter;
    FResDialog.OnCloseQuery := ResOnClose;

    FResDialog.Height := 160;
    FResDialog.Width := 600;
    FResDialog.BorderIcons := [biSystemMenu];
    FResDialog.BorderStyle := bsDialog;
    FResDialog.Caption := 'Resource properties';

    NC := TLabel.Create(Self);
    NC.Parent := FResDialog;
    NC.Left := 10;
    NC.Top := 10;
    NC.Caption := 'Name';
    NC.Visible := True;

    FNameControl := TEdit.Create(Self);
    FNameControl.Parent := FResDialog;
    FNameControl.Left := 10;
    FNameControl.Top := NC.Top + NC.Height +3;
    FNameControl.Visible := True;

    AC := TLabel.Create(Self);
    AC.Parent := FResDialog;
    AC.Left := 10;
    AC.Top := FNameControl.Top + FNameControl.Height + 3;
    AC.Caption := 'Address';
    AC.Visible := True;

    FAddressControl := TEdit.Create(Self);
    FAddressControl.Parent := FResDialog;
    FAddressControl.Width := 570;
    FAddressControl.Left := 10;
    FAddressControl.Top := AC.Top + AC.Height + 3;
    FAddressControl.Visible := True;
    BtnTop := FAddressControl.Top + FAddressControl.Height + 10;

    FOK := TButton.Create(Self);
    FOK.Parent := FResDialog;
    FOK.Left := 10;
    FOK.ModalResult := mrOK;
    FOK.Top := BtnTop;
    FOK.Visible := True;
    FOK.Caption := '&OK';

    FCancel := TButton.Create(Self);
    FCancel.Parent := FResDialog;
    FCancel.Left := FOk.Left + FOK.Width +10;
    FCancel.ModalResult := mrCancel;
    FCancel.Top := BtnTop;
    FCancel.Visible := True;
    FCancel.Caption := '&Cancel';
  end;

  constructor TILResourceDialog.Create(AOwner : TComponent; ATextField : TILField);
  begin
    FTextField := ATextField;
    inherited Create(AOwner);
  end;

  function TILResourceDialog.Execute: Boolean;
  begin
       Result := false;
       if FResDialog.ShowModal = mrOk then
       begin
        Result := True;
       end;
  end;

  procedure TILResourceDialog.ResOnClose(Sender : TObject; var CanClose : Boolean);
  begin
   if FResdialog.ModalResult = mrOK then
   begin
    if Trim(FNameControl.Text) = '' then
      HandleIlException(E_MissingresName);
    if FTextField = nil then
     if Trim(FAddressControl.Text) = '' then
      HandleIlException(E_MissingResAddress);
   end;
  end;

initialization
RegTables := TList.Create;
RegPanels := TList.Create;
RegHandlers := TList.Create;
RegSQLs := TList.Create;
RegBars := TList.Create;
RegMbs := TList.Create; { Message bars}
RegMbh := TList.Create; {Message handlers}
TMan := nil;
TMan2 := nil;
HotTrackClr := RGB(209, 223, 231);
SelectedClr := RGB(229, 243, 252);
Importing := false;
begin
  try
    TStyleManager.Engine.RegisterStyleHook(TILTable, TScrollingStyleHook);
  except
  end;
end;

Translator := TTranslator.Create;

finalization
Translator.Free;
RegTables.Free;
RegPanels.Free;
RegHandlers.Free;
RegBars.Free;
RegMbs.Free;
RegMbh.Free;
RegSQLs.Free;
TMan := nil;
TMan2 := nil;

end.
